/*
Macro code to create individual macro variables from variable lists,
including indicator variables created from classification variables via
PROC GLMMOD or a data step.

The data set d must exist and the macro variables ClassVars, ContinuousVars, 
PredictorVars, and ClassPredVars must be defined prior to calling this code.

UtilityMacros.sas is assumed to already have been included.
*/

%macro MakeClassVars;

	/* %ClockStart(MakeClassVars); */

	ods exclude all;
	ods noresults;
	
	
	* close open data sets, needed for CV processing;	
	%if %symexist(_dsix_) %then %do;
		%if (&_dsix_>0) %then %let rc = %sysfunc(close(&_dsix_));
	%end;
	%if %symexist(_dsip_) %then %do;
		%if (&_dsip_>0) %then %let rc = %sysfunc(close(&_dsip_));
	%end;
	%if %symexist(_dsixl_) %then %do;
		%if (&_dsixl_>0) %then %let rc = %sysfunc(close(&_dsixl_));
	%end;
	%if %symexist(_dsipl_) %then %do;
		%if (&_dsipl_>0) %then %let rc = %sysfunc(close(&_dsipl_));
	%end;
	
	%if ^%symexist(LockContPredVars) %then %do;
		%global LockContPredVars;
		%let LockContPredVars = ;
	%end;
	%if ^%symexist(LockClassPredVars) %then %do;
		%global LockClassPredVars;
		%let LockClassPredVars = ;
	%end;
		
	%global nics nic nic1 nit nix nicp nip nir nis nixl nipl _dsix_ _dsip_ _dsixl_ _dsipl_;
	%let nics = 0;
	%let nic = 0;
	%let nic1 = 0;
	%let nit = 0;
	%let nix = 0;
	%let nicp = 0;
	%let nip = 0;
	%let nir = 0;
	%let nis = 0;
	%let nixl = 0;
	%let nipl = 0;
	%let _dsix_ = 0;
	%let _dsip_ = 0;
	%let _dsixl_ = 0;
	%let _dsipl_ = 0;
	%if ^%symexist(NameOnLabel) %then %let NameOnLabel = 1;
	
	%NObsVars(d);
	%let nir = &nobs;
	%put nir = &nir;

	/*---create continuous variable list---*/
	%if %length(&ContinuousVars) %then %do;
		data a;
			set d(obs=1);
		run;

		proc transpose data=a out=at;
			var &ContinuousVars;
		run;

		%if &syserr %then %goto cexit;

		%NObsVars(at);
		%let nit = &nobs;
		
		%do i=1 %to &nit; 
			%global _t&i;
		%end; 
		
		data _null_;
			set at end=_last;
			length ic$ 16;
			ic = "_t" || trim(left(_n_));
			call symput(ic,trim(left(_name_)));
			if (_last) then call symput("nit",trim(left(_n_)));
		run;
	
	%end;

	/*---create predictor continuous variable list---*/
	%if %length(&PredictorVars) %then %do;
		data dsix;
			set d(obs=0);
			keep &PredictorVars;
		run;
		%let _dsix_ = %sysfunc(open(dsix));
		%let nix = %sysfunc(attrn(&_dsix_,NVARS));
	%end;
	%put nix = &nix;

	/*---create lock-in predictor continuous variable list---*/
	%if %length(%trim(&LockContPredVars)) %then %do;
		%put LockContPredVars = &LockContPredVars;
		data dsixl;
			set d(obs=0);
			keep &LockContPredVars;
		run;
		%let _dsixl_ = %sysfunc(open(dsixl));
		%let nixl = %sysfunc(attrn(&_dsixl_,NVARS));
	%end;
	%put nixl = &nixl;

	/*---create lock-in class predictor variable list---*/
	%if %length(%trim(&LockClassPredVars)) %then %do;
		
		%put LockClassPredVars = &LockClassPredVars;
		%put NeedClassIndicators = &NeedClassIndicators;
		%put RecoverClassPredVars = &RecoverClassPredVars;

		%if (&NeedClassIndicators = No) or (&RecoverClassPredVars = Yes) %then %do;
		
			data dsipl;
				set d(obs=0);
				keep &LockClassPredVars;
			run;
			%let _dsipl_ = %sysfunc(open(dsipl));
			%let nipl = %sysfunc(attrn(&_dsipl_,NVARS));
			
		%end;
		%else %do;
	
			%put Creating indicator variables from lock-in class predictor variables;

			/*
			full-rank coding via proc transreg, reference-level coding is the default along with zero=last
			data d1;
				set d;
				_one = 1;
				keep _one &LockClassPredVars;
			run;

			proc transreg data=d1;
				model identity(_one) = class(&LockClassPredVars);
				output out=outlib.treg(drop=_type_ _name_ _one Intercept &LockClassPredVars) replace;
			run;
			*/

			proc transpose data=d out=dt;
				var &LockClassPredVars;
			run;

			%put nir = &nir;
			* data step processing is faster than proc glmmod;
			data opcl;
				set dt;
				length new_name $ 32 cjval $ 24;
				array c $ col1-col&nir;
				array i i1-i&nir;
				array a a1-a&nir;
				do j=1 to &nir;
					a[j]= 0;
				end;
				lenname = length(_name_);
				do j=1 to &nir;
					do k=1 to &nir;
						i[k]= 0;
					end;
					if a[j] = 0 then do;
						a[j] = 1;
						i[j] = 1;
						do k=(j+1) to &nir;
							if c[k] = c[j] then do;
								a[k] = 1;
								i[k] = 1;
							end;
						end;
						orig_name = _name_;
						cjval = trim(left(c[j]));
						len = length(cjval);
						if (lenname > 32-len-1) then oname = substr(_name_,1,32-len-1);
						else oname = _name_;
						if cjval=" " or cjval="." then cjval = "Mis";
						new_name = trim(left(oname)) || "_" || cjval;
						output;
					end;
				end;
				_ord_= _n_;
				keep _ord_ orig_name new_name i1-i&nir;
			run;
			/*data OutLib.opcl; set opcl; run;*/
			
			/*S1471036: begin*/
			proc sort data=opcl nodupkey;
				by new_name;
			run;
			
			proc sort data=opcl out=opcl(drop= _ord_);
				by _ord_;
			run;
			/*S1471036: end*/
			
			%NObsVars(opcl);
			
			proc transpose data=opcl out=dsipl(drop=_name_);
				var i1-i&nir;
				id new_name;
			run;

			* reference the data set for var names, this is faster than using long macro var lists;
			%let _dsipl_ = %sysfunc(open(dsipl));
			%let nipl = %sysfunc(attrn(&_dsipl_,NVARS));

			/*---change LockClassPredVars to refer to the newly created indicator variables---*/
			%let LockClassPredVars = ;
			%do i=1 %to &nipl; 
				%let LockClassPredVars = &LockClassPredVars %sysfunc(varname(&_dsipl_,&i));
			%end; 

			/*---merge with original data---*/
			data d(compress=yes);
				merge d dsipl;
			run;
		%end;
	%end;
	%put nipl = &nipl;

	/*---create class predictor variable list---*/
	%if %length(%trim(&ClassPredVars)) %then %do;

		%put ClassPredVars = &ClassPredVars;
		%put NeedClassIndicators = &NeedClassIndicators;

		%if (&NeedClassIndicators = No) %then %do;
		
			data dsip;
				set d(obs=0);
				keep &ClassPredVars;
			run;
			
			%if &nipl %then %do;
				data dsip;
					set dsip;
					drop %do i=1 %to &nipl; %sysfunc(varname(&_dsipl_,&i)) %end;;
				run;
			%end;
			
			%let _dsip_ = %sysfunc(open(dsip));
			%let nip = %sysfunc(attrn(&_dsip_,NVARS));
			
			/*S1486072: When data set does not have character variable, using "keep _character_" generations a system error with syerr= 4
				To prevent this error, check if there are character variables before using _character_;	*/
			%let charlist= ;
			%let i=1;
			%let cflag= 0;
			%do %while(&i <= &nip and ^&cflag);
				%if %sysfunc(vartype(&_dsip_,&i)) = C %then %do;
					%let charlist = &charlist %sysfunc(varname(&_dsip_,&i));
					%let cflag= 1;
				%end;
				%let i= %sysevalf(&i + 1);
			%end;
			%put charlist = &charlist;
			%put cflag= &cflag;
			
			%if &cflag %then %do;	
				/*S1468946 Begin: Fixing error when character class variables are used for prediction in GLMSELECT procedure.*/
				%put syserr_in= &syserr;
				data charvars;
					set dsip;
					keep _character_;
				run;
				%put syserr_out= &syserr;
	
				/*data OutLib.charvars; set charvars; run;*/				
					
				%NObsVars(charvars);
				%if &nvars %then %do;	
				
					%let _dsipc_ = %sysfunc(open(charvars));
					%let nipc = %sysfunc(attrn(&_dsipc_,NVARS));
	
					*ods select none;
					ods noresults;
					proc freq data=d nlevels;
						tables %do i=1 %to &nipc; %sysfunc(varname(&_dsipc_, &i)) %end; / noprint;
						ods output nlevels= nlevelsdat(where=(nmisslevels > 0));
					run;
					*ods select all;
					ods results;
					/*data OutLib.nlevelsdat; set nlevelsdat; run;*/
	
					%let _rcpc_ = %sysfunc(close(&_dsipc_));
	
					%NObsVars(nlevelsdat);
					%if &nobs %then %do;
						%let FlagCharVarMissing= 1;
						
						data opn;
							set nlevelsdat;
							parameter= trim(left(tablevar))||" _miss_";
							drop tablevar;
						run;
								
					%end;
				%end;
			%end;
			/*S1468946 End*/
		%end;
		%else %do;
	
			%put Creating indicator variables from class predictor variables;

			data cpv;
				set d;
				_row_ = _n_;
				keep _row_ &ClassPredVars;
			run;
			
			%if &nipl %then %do;
				data cpv;
					set cpv;
					drop %do i=1 %to &nipl; %sysfunc(varname(&_dsipl_,&i)) %end;;
				run;
			%end;
			
			/*data OutLib.cpv; set cpv; run;*/

			data cpv0;
				set cpv;
				if (0);
				drop _row_;
			run;

			%let _dsip_ = %sysfunc(open(cpv0));
			%let nip = %sysfunc(attrn(&_dsip_,NVARS));
			
			* use glmselect to create the class vars, much faster than data step code;
			ods exclude all;
			ods noresults;
			proc glmselect data=cpv outdesign(fullmodel names)=dsip namelen=200;
				class %do i=1 %to &nip; %sysfunc(varname(&_dsip_,&i)) %end; / missing;
				model _row_ = %do i=1 %to &nip; %sysfunc(varname(&_dsip_,&i)) %end; / steps=1;
				ods output parameternames=opn;
			run;
			ods exclude none;
			ods results;
			
			%let _dsip_ = %sysfunc(close(&_dsip_));
			
			/*S1468946 Begin: Fixing error when character class variables are used for prediction in GLMSELECT procedure.*/
			data charvars;
				set cpv;
				keep _character_;
			run;

			/*data OutLib.charvars; set charvars; run;*/

			%NObsVars(charvars);
			%if &nvars %then %do;

				%let _dsipc_ = %sysfunc(open(charvars));
				%let nipc = %sysfunc(attrn(&_dsipc_,NVARS));	

				*ods select none;
				ods noresults;
				proc freq data=charvars nlevels;
					tables %do i=1 %to &nipc; %sysfunc(varname(&_dsipc_, &i)) %end; / noprint;
					ods output nlevels= nlevelsdat(where=(nmisslevels > 0));
				run;
				*ods select all;
				ods results;
				/*data OutLib.nlevelsdat; set nlevelsdat; run;*/

				%let _rcpc_ = %sysfunc(close(&_dsipc_));

				%NObsVars(nlevelsdat);
				%if &nobs %then %do;
					%let FlagCharVarMissing= 1;
					proc transpose data= nlevelsdat(keep= tablevar) out=nlevelsdatt(drop= _name_ _label_ where=(0));
						var _all_;
						id tablevar;
					run;

					/*data OutLib.nlevelsdatt; set nlevelsdatt; run;*/

					%let _dsnl_ = %sysfunc(open(nlevelsdatt));
					%let nvnl = %sysfunc(attrn(&_dsnl_,NVARS));

					data dsip; 
						set dsip;
						rename 
						%do i=1 %to &nvnl;
							%sysfunc(varname(&_dsnl_,&i)) = %sysfunc(cat(%substr(%trim(%left(%sysfunc(varname(&_dsnl_,&i)))),1,30),__))
						%end;
						;
					run;
					/*data OutLib.dsip; set dsip; run;*/

					data opn;
						set opn;
						_ord_ = _n_;
					run;
					
					/*data OutLib.opn1; set opn; run;*/

					proc sql noprint;
						create table opn2 as
						select a.tablevar, b.*
						from opn as b left join nlevelsdat as a
						on b.name = a.tablevar
						order by _ord_;
					run;
					quit;
					
					/*data OutLib.opn2; set opn2; run;*/

					data opn;
						length name parameter $32;
						set opn2(rename=(name= tmpname parameter= tmpparameter));
						if missing(tablevar) then do;
							name= tmpname;
							parameter= tmpparameter;
						end;
						else do; 
							name= tmpname;
							*name= trim(left(tablevar))||"__";
							parameter= trim(left(tablevar))||" _miss_";
						end;
						keep name parameter;
					run;
					
					/*data OutLib.opn2; set opn; run;*/

					%let _dsrc_ = %sysfunc(close(&_dsnl_));
				%end;
			%end;
			/*S1468946 End*/

			data dsip;
				set dsip;
				drop Intercept _row_;
			run;
			
			/*data OutLib.dsip; set dsip; run;*/
			/*data OutLib.opn; set opn; run;*/
			
			/*
			data dsip;
				set d;
				keep &ClassPredVars;
			run;
			
			%if &nipl %then %do;
				data dsip;
					set dsip;
					drop %do i=1 %to &nipl; %sysfunc(varname(&_dsipl_,&i)) %end; ;
				run;
			%end;

			proc transpose data=dsip out=dt;
				var _all_;
			run;

			* data step processing is faster than proc glmmod;
			data opc;
				set dt;
				length new_name $ 32 cjval $ 24;
				array c $ col1-col&nir;
				array i i1-i&nir;
				array a a1-a&nir;
				do j=1 to &nir;
					a[j]= 0;
				end;
				lenname = length(_name_);
				do j=1 to &nir;
					do k=1 to &nir;
						i[k]= 0;
					end;
					if a[j] = 0 then do;
						a[j] = 1;
						i[j] = 1;
						do k=(j+1) to &nir;
							if c[k] = c[j] then do;
								a[k] = 1;
								i[k] = 1;
							end;
						end;
						orig_name = _name_;
						cjval = trim(left(c[j]));
						len = length(cjval);
						if (lenname > 32-len-1) then oname = substr(_name_,1,32-len-1);
						else oname = _name_;
						if cjval=" " or cjval="." then cjval = "Mis";
						new_name = trim(left(oname)) || "_" || cjval;
						output;
					end;
				end;
				keep orig_name new_name i1-i&nir;
			run;

			%NObsVars(opc);
			*/

			* reference the data set for var names, this is faster than using long macro var lists;
			%let _dsip_ = %sysfunc(open(dsip));
			%let nip = %sysfunc(attrn(&_dsip_,NVARS));
			
			%if ^%symexist(ClassCheck) %then %let ClassCheck = No;
			%if ^%symexist(ColumnLimit) %then %let ColumnLimit = -1;
			
			* transpose in some cases for efficiency,e.g. PCA;
			%if (&ClassCheck = Yes) and (%eval(&nix + &nip) > &ColumnLimit) %then %do;

				proc transpose data=dsip out=opc prefix=i;
					var _all_;
				run;

			%end;
			%else %do;
			
				/*
				proc transpose data=opc out=dsip(drop=_name_);
					var i1-i&nir;
					id new_name;
				run;
				*/

				/*---change ClassPredVars to refer to the newly created indicator variables---*/
				/*S1459438: Commenting out this data step because when a class variable 
				(e.g., sex) is used in the predictor reduction tab (e.g., sex=1), this variable 
				needs to be present in the data set.*/				
				/*
				data d;
					set d;
					drop &ClassPredVars;
				run;
				*/
				
				/*---special processing for colon specifications to avoid constructing a long list of individual macro variables,
					  assumes the colon specification still exactly refers to the newly constructed indicator variables---*/
				%if &ListClassColon and ^&nixl and ^&nipl %then %do;

					* create list of just the new indicator vars derived from ClassPredVars, not ListClassPredVars;
					data opd2;
						set dsip;
						drop &ListClassPredVars;
						* drop _idx &ListClassPredVars;
					run;

					proc transpose data=opd2 out=opd2t(keep=_name_);
					run;

					%NObsVars(opd2t);

					%let nis = &nobs;

					%do i=1 %to &nis; 
						%global _s&i;
					%end; 

					/*---create macro variables---*/
					data _null_;
						set opd2t;
						nm = "_s" || trim(left(put(_n_,10.0)));
						call symput(nm,trim(left(_name_)));
					run;

					%let ClassPredVars = ;

					%do i=1 %to &nis; 
						%let ClassPredVars = &ClassPredVars &&_s&i;
					%end; 

					%let ClassPredVars = &ClassPredVars &ListClassPredVars;
				%end;
				%else %do;
					%let ClassPredVars = ;

					%do i=1 %to &nip; 
						%let ClassPredVars = &ClassPredVars %sysfunc(varname(&_dsip_,&i));
					%end; 
				%end;
				
				/*data OutLib.d; set d; run;*/

				/*---merge with original data---*/
				data d(compress=yes);
					merge d dsip;
					* by _idx;
				run;

			%end;
		%end;
	%end;
	%put nip = &nip;

	
	/*---create class variable list---*/
	%if %length(&ClassVars) %then %do;
	
		%put;
		%put ClassVars = &ClassVars;
		%put;
		
		/*
		data d(compress=yes);
			set d;
			_idx = _n_;
		run;
		*/

		data a;
			set d(obs=1);
		run;

		proc transpose data=a out=at;
			var &ClassVars;
		run;
		
		/*
		ods exclude none;
		ods results;
		title "at";
		proc print data=at;
		run;
		*/

		%if &syserr %then %goto cexit;
		
		%NObsVars(at);
		%let nics = &nobs;
		%put nics = &nics;
		
		/* check for and remove duplicates */
		data at;
			set at;
			_idx = _n_;
		run;
		
		proc sort data=at out=at1 nodupkey;
			by _name_;
		run;
		
		%NObsVars(at1);
		%if (&nobs < &nics) %then %do;
			%let dup = 1;
			proc sort data=at1;
				by _idx;
			run;
			data at;
				set at1;
				drop _idx;
			run;
			%let nics = &nobs;
		%end;
		%else %let dup = 0;
			
		%do i=1 %to &nics; 
			%global _cs&i;
		%end; 
		
		data _null_;
			set at;
			length is$ 16;
			is = "_cs" || trim(left(_n_));
			call symput(is,trim(left(_name_)));
		run;
		
		%let maxcnamelen = %length(&_cs1);
		%do i=2 %to &nics;
			%let lci = %length(&&_cs&i);
			%if (&lci > &maxcnamelen) %then %do;
				%let maxcnamelen = &lci;
			%end;
		%end;
		
		%if (&dup=1) %then %do;
			%put;
			%put Removing duplicate names from ClassVars.;
			%put;
			%let ClassVars = ;
			%do i=1 %to &nics;
				%let ClassVars = &ClassVars &&_cs&i;
			%end;
			%put;
			%put ClassVars = &ClassVars;
			%put nics = &nics;
			%put;
		%end;

		/*
		%if (&nic1 <= 100) %then %do;
			%do i=1 %to &nic1;
				%put _lev&i = &&_lev&i;
			%end;
		%end;
		%else %do;
			%do i=1 %to 100;
				%put _lev&i = &&_lev&i;
			%end;
			%put ...plus %eval(&nic1 - 100) more levels.;
		%end;
		%let lenc = %eval(%length(&nic1)+3);
		*/
		
		
		/*---create indicator variables---*/
		%put;
		%put Creating indicator variables for class variables &ClassVars.;
		%put;
		
		* use hash object to create indicators;
		%do i=1 %to &nics;

			%put levelizing _cs&i = &&_cs&i;

			* save unique levels;
			data ocp1;
				set d;
				if not missing(&&_cs&i);
				keep &&_cs&i;
			run;

			proc sort data=ocp1 nodupkey;
				by &&_cs&i;
			run;

			%NObsVars(ocp1);
			%let nic&i = &nobs;
			%put nic&i = &&nic&i;
			%if ^&nobs %then %do;			
				%let exiterror=22;
				%goto cexit;
			%end;

			* save levels and compute max length for first variable;
			%if &i=1 %then %do;
				%global cprefix;
				%let cprefix = &_cs1;
		
				%do j=1 %to &nic1; 
					%global _lev&j;
				%end;
				data _null_;
					set ocp1 end=_last;
					retain maxclen 0;
					levi = "_lev" || trim(left(_n_));
					call symput(levi,trim(left(&_cs1)));
					maxclen = max(maxclen,lengthn(&_cs1));
					if _last then do;
						maxclen = maxclen + &maxcnamelen + 1; 
						call symputx("maxclen",trim(left(maxclen)));
					end;
				run;
				%put maxclen = &maxclen;
				%let lenc = &maxclen;
				%if (&lenc > 32) %then %let lenc = 32;
				%put lenc = &lenc;
			%end;
		
			data ocp1h;
				set ocp1;
				gidx = _n_;
				_name_ = "&&_cs&i";
				lenname = length(_name_);
				if lenname > 24 then do;
					_name_ = substr(_name_,1,24);
					lenname = 24;
				end;
				curri = _n_;
				* try and guarantee a unique 32 char varname, include row number to be sure;
				cjval = trim(left(dequote(&&_cs&i)));
				if cjval=" " or cjval="." then cjval = "Mis";
				len = length(cjval);
				leno = lenname + length(trim(left(curri))) + 2;
				length new_name $ 32;
				if ((len+leno) > 32) then do;
					new_name = trim(left(_name_)) || "_" || trim(left(substr(cjval,1,32-leno))) || "_" || trim(left(curri));
				end;
				else do;
					new_name = trim(left(_name_)) || "_" || trim(left(cjval)) || "_" || trim(left(curri));
				end;
				* convert to valid sas name;
				j = 0; 
				len = length(trim(left(new_name)));
				do until(j=0 or j>len);  
					j = notname(new_name,j+1);  
					if j and j<=len then do; 
						substr(new_name,j,1) = "_";  
					end; 
				end;

				* label;
				%if &NameOnLabel %then %do;
					new_label = trim(left(_name_)) || " " || cjval;
				%end;
				%else %do;
					new_label = cjval;
				%end;
				* remove all quotes from label;
				new_label = translate(new_label,'','"''');

				call symput("_cname&i._" || trim(left(_n_)),trim(left(new_name)));
				call symput("_clabel&i._" || trim(left(_n_)),trim(left(new_label)));
			run;

			data d(compress=yes);
				set d;
				if _n_ = 1 then do;
					declare hash h(dataset:"ocp1h");
					h.defineKey("&&_cs&i");
					h.defineData("gidx");
					h.defineDone();
				end;
				array iv{*} %do j=1 %to &&nic&i; &&_cname&i._&j %end; ;
				label %do j=1 %to &&nic&i; &&_cname&i._&j="&&_clabel&i._&j" %end; ;
				gidx = 0;
				if h.find()=0 then do;
					do _i=1 to &&nic&i;
						iv[_i] = 0;
					end;
					iv[gidx] = 1;
				end;
				drop _i;
			run;
						
		%end;
		
		* keep newly created indicator variables;
		data ocd1;
			set d(obs=1);
			keep 
				%do i=1 %to &nics;
					%do j=1 %to &&nic&i;
						&&_cname&i._&j
					%end;
				%end;
			;
		run;

		/*
		%do i=1 %to &nics;
			%do j=1 %to &&nic&i;
				%put _cname&i._&j = &&_cname&i._&j;
			%end;
		%end;
		
		data outlib.ocd1;
			set ocd1;
		run;
					
		data outlib.ocp1;
			set ocp1;
		run;
					
		%put ERROR: Not!;
		%let exiterror = 1;
		%goto cexit;
		*/
		
		/*
		* use transpose and data step processing to create indicators;
		proc transpose data=d out=dt(compress=yes);
			var &ClassVars;
		run;
		
		%if &syserr > 4 %then %do;
			%let exiterror = 1;
			%goto cexit;
		%end;
		
		* data step processing is much faster than proc glmmod;
		data opcv(compress=yes);
			set dt;
			length new_name $ &lenc new_label $ &maxclen;
			array c $ col1-col&nir;
			array i i1-i&nir;
			array a a1-a&nir;
			retain curri 0 _nidx 1;
			do j=1 to &nir;
				a[j] = 0;
			end;
			orig_name = _name_;
			lenname = length(_name_);
			if lenname > 24 then do;
				_name_ = substr(_name_,1,24);
				lenname = 24;
			end;
			if (_n_ = 1) then call symputx('cprefix',trim(left(_name_)));
			do j=1 to &nir;
				do k=1 to &nir;
					i[k] = 0;
				end;
				if a[j] = 0 then do;
					a[j] = 1;
					i[j] = 1;
					do k=(j+1) to &nir;
						if c[k] = c[j] then do;
							a[k] = 1;
							i[k] = 1;
						end;
					end;
					curri = curri+1;
					* try and guarantee a unique 32 char varname, include row number if value is too long to be sure;
					cjval = trim(left(c[j]));
					if cjval=" " or cjval="." then cjval = "Mis";
					len = length(cjval);
					if (len > 32-lenname-1) then do;
						new_name = trim(left(_name_)) || "_" || trim(left(substr(cjval,1,30-length(trim(left(_name_)))-length(trim(left(curri)))))) || 
							"_" || trim(left(curri));
					end;
					else do;
						new_name = trim(left(_name_)) || "_" || cjval;
					end;
					%if &NameOnLabel %then %do;
						new_label = trim(left(_name_)) || " " || dequote(cjval);
						* new_label = trim(left(_name_)) || " " || cjval;
					%end;
					%else %do;
						new_label = dequote(cjval);
					%end;
					
					* verify new_name is a valid sas name, if not, add a unique numerical suffix to avoid;
					* duplicate names when invalid characters are converted to underscores later;
					len_new = length(new_name);
					_check1 = verify(upcase(new_name),"_ABCDEFGHIJKLMNOPQRSTUVWXYZ");
					_check2 = verify(upcase(new_name),"_ABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789");
					if (_check1 = 1) or ((_check2 > 0) and (_check2 <= len_new)) then do;
						new_name = trim(left(_name_)) || "_" || trim(left(substr(cjval,1,30-length(trim(left(_name_)))-length(trim(left(curri)))))) ||
							"_" || trim(left(curri));					
						* new_name = trim(left(_name_)) || "_" || trim(left(curri)) || "_" || cjval;
					end;
					output;
				end;
			end;
			keep orig_name new_name new_label i1-i&nir;
		run;
		
		%put cprefix = &cprefix;
				
		* merge with original data;
		proc transpose data=opcv out=ocd;
			var i1-i&nir;
			id new_name;
			idlabel new_label;
		run;

		data outlib.ocp1;
			set ocp1;
		run;
		
		data outlib.opcv;
			set opcv;
		run;
		
		data outlib.ocd;
			set ocd;
		run;
		
		%put ERROR: Not!;
		%let exiterror = 1;
		%goto cexit;
		
		data d;
			merge ocd d;
			drop _name_;
		run;
			
		data ocd1;
			set ocd(obs=1);
		run;
		*/
		
		* fill in macro variables with new indicator variable names;
		proc transpose data=ocd1 out=ocd1t;
		run;

		%if ^%symexist(ProcessName) %then %let ProcessName = Unknown;
		
		data ocd1t;
			set ocd1t;
			%if ^%index(&ProcessName,Incidence) %then %do;
				if index(_name_,"_Mis") or index(_name_,"_true") then delete;
			%end;
		run;

		proc sort data=ocd1t nodupkey;
			by _name_;
		run;
		%NObsVars(ocd1t);
		
		%let nic = &nobs;
		%put nic = &nic;

		%do i=1 %to &nic; 
			%global _c&i;
		%end; 
		
		data _null_;
			set ocd1t;
			nm = "_c" || trim(left(_n_));
			call symputx(nm,_name_);
		run;
		
		/*
		%do i=1 %to &nic; 
			%put _c&i = &&_c&i;
		%end; 
		*/
		
	%end;


	ods exclude none;
	ods results;

%cexit:;

	/* %ClockStop(MakeClassVars); */
	
%mend MakeClassVars;

%MakeClassVars;