/*	
	JMP acknowledges the use of the following Third Party software:
	
	BGLR: Bayesian Generalized Linear Regression. Copyright (C) 2007-13 Gustavo de los Campos and 
	Paulino Perez Rodriguez (perpdgo@colpos.mx).
		
	BGLR is free software; you can redistribute it and/or modify it under the terms of the GNU General 
	Public License as published by the Free Software Foundation; either version 2 of the License, or 
	(at your option) any later version.
		
	BGLR is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the 
	implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public 
	License for more details.
		
	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, 
	USA."
*/

proc iml;

	/*removes objects in R*/
	start RemoveRObjs(RObjsList_);
				
		submit RObjsList_ / R ok=isOK;
			#print("&RObjsList_");
			#print(nchar("&RObjsList_"));
			
			if(nchar("&RObjsList_")) {
				if("&RObjsList_" == "NONE") {
					#print("1")
				} else if("&RObjsList_" == "ALL") {
					#print("2")
					rm(list=ls())
				} else {
					#print("3")
					rm(&RObjsList_)
				}
			}
			
			#print(ls())
			
		endsubmit;
	finish RemoveRObjs;
	store module=(RemoveRObjs);
	
	start SetRMemory(maxsize_);
		submit maxsize_ / R ok=isOK;
			#if(Sys.getenv("R_ARCH")=="/x64") {
			#}
			memory.limit(size = &maxsize_)
		endsubmit;
	finish SetRMemory;
	store module=(SetRMemory);
	
	start SaveRData(ROutPath_, fname_);
		submit ROutPath_ fname_ / R ok=isOK;
			#print(paste("&ROutPath_", "&fname_.RData", sep=""))
			save.image(file=paste("&ROutPath_", "&fname_.RData", sep=""))
			#print("saving image 2")
		endsubmit;
	finish SaveRData;
	store module=(SaveRData);

	/*loads R packages*/
	start LoadRPkgs;
		submit / R ok=isOK;
			library(BGLR)			
		endsubmit;
	finish LoadRPkgs;
	store module=(LoadRPkgs);

	/*loads R functions*/
	start LoadRFunc(RSourcePath_);
		submit RSourcePath_ / R ok=isOK;	
			*source(file=paste("&RSourcePath_", "/GenomicBayesianRegressionRFunctions.R", sep=""))						
		endsubmit;
	finish LoadRFunc;
	store module=(LoadRFunc);
		
	/*exports data from SAS to R*/
	start ExportDataToR(DataName_, DataObjName_);
		*print(DataName_);
		*print(DataObjName_);
						
		call ExportDataSetToR(DataName_, DataObjName_);
		
		submit DataObjName_ / R ;
			
			debug <- 0
			if(debug) {
				print(&DataObjName_)
				print(dim(&DataObjName_))
			}
		endsubmit;
		
	finish ExportDataToR;
	store module=(ExportDataToR);
	
	/*exports marker variable names from SAS to R*/
	start ExportMarkerVarsToR(MarkerVars_, MarkerVarsObjName_);
		*print(MarkerVars_);
		*print(MarkerVarsObjName_);
		
		i=1;
		mkvar=scan(MarkerVars_,i);
		do while(mkvar ^= ' ');
			mkvars_all=mkvars_all//mkvar;
			i= i + 1; 		
			mkvar=scan(MarkerVars_,i);
		end;
		*print(mkall);
				
		call ExportMatrixToR(mkvars_all, MarkerVarsObjName_);
		
	finish ExportMarkerVarsToR;
	store module=(ExportMarkerVarsToR);
	
	/*Fit Bayesian models*/
	start CallBGLRFunction(y_, response_type_= "gaussian", a_= "NULL", b_= "NULL", 
				ETA_= "NULL", nIter_= 1500, burnIn_= 500, thin_= 5, 
				saveAt_= "NULL", S0_= 0, df0_= 5, R2_= 0.5, weights_= "NULL",
				verbose_= "TRUE", rmExistingFiles_= "TRUE", groups_= "NULL", 
				FixedEffFlag_= 0, RandEffFlag_= 0);
				
		*print(y_);
		*print(ETA_);

		submit y_ response_type_ a_ b_ 
			ETA_ nIter_ burnIn_ thin_ 
			saveAt_ S0_ df0_ R2_ weights_ 
			verbose_ rmExistingFiles_ groups_ 
			FixedEffFlag_ RandEffFlag_/ R ok=isOK;
			
			#print("&y_")
			#print("&response_type_")
			#print("&ETA_")
			#print("&saveAt_")
			#print("&S0_")
			#print("&verbose_")
			#print("&rmExistingFiles_")
			#print(&groups_)
			#print(is.factor(&groups_))
			
			if(1){
			
				if(&S0_ == 0) {	
					S0 <- NULL			
				} else {
					S0 <- &S0_						
				}
				
				if("&saveAt_" == "NULL") {
					saveAt <- ""
				} else {
					saveAt <- "&saveAt_"
				}
				
				if("&a_" != "NULL" & "&b_" == "NULL"){
					a <- &a_
					a[is.na(a)] <- -Inf
					b <- rep(Inf, length(a))
				} else if("&a_" == "NULL" & "&b_" != "NULL"){
					b <- &b_
					b[is.na(b)] <- Inf
					a <- rep(-Inf, length(b))
				} else if("&a_" != "NULL" & "&b_" != "NULL"){
					a <- &a_
					a[is.na(a)] <- -Inf
					b <- &b_
					b[is.na(b)] <- Inf
				} else {
					a <- &a_
					b <- &b_
				}
				
				if ("&groups_" != "NULL") {
					groups <- as.factor(&groups_)
				} else {
					groups <- &groups_
				}
				
				#print(groups)
				
				fmBGLR <- BGLR(y= &y_, response_type= "&response_type_", a= a, b= b, 
						ETA= &ETA_, nIter= &nIter_, burnIn= &burnIn_, thin =&thin_, 
						saveAt= saveAt, S0= S0, df0= &df0_, R2= &R2_, weights= &weights_,
						verbose= &verbose_, rmExistingFiles= &rmExistingFiles_, groups= groups)
				
				#print(fmBGLR)
				
				beta0.obj <- cbind("Intercept", fmBGLR$mu)
				colnames(beta0.obj) <- c("Effect","Estimate")		

				betas.obj <- cbind(fmBGLR$ETA$MAR$colNames, fmBGLR$ETA$MAR$b)
				colnames(betas.obj) <- c("_Effect_","_Estimate_")	

				pred.obj <- matrix(fmBGLR$yHat, ncol=1) 
				colnames(pred.obj) <- c("pred")
				
				if (&FixedEffFlag_) {	
					fixedeff.obj <- cbind(fmBGLR$ETA$FIX$colNames, fmBGLR$ETA$FIX$b)
					colnames(fixedeff.obj) <- c("_Effect_", "_Estimate_")
				}
				
				if (&RandEffFlag_) {
					randeff.obj <- cbind(fmBGLR$ETA$RAN$colNames, fmBGLR$ETA$RAN$b)
					colnames(randeff.obj) <- c("_Effect_", "_Estimate_")				
				}
				
				if ("&response_type_"=="ordinal"){
					probs.obj <- matrix(fmBGLR$probs, ncol=dim(fmBGLR$probs)[2]) 
					colnames(probs.obj) <- paste("p_", colnames(fmBGLR$probs),sep="")
					
					#threshold.obj <- cbind(paste("Intercept", fmBGLR$levels[1:(fmBGLR$nlevels-1)], sep=""), -fmBGLR$threshold) 
					#colnames(threshold.obj) <- c("Effect", "Estimate")
					
					threshold.obj <- cbind("Intercept", fmBGLR$levels[1:(fmBGLR$nlevels-1)], -fmBGLR$threshold) 
					colnames(threshold.obj) <- c("Effect", "Response", "Estimate")
				}

			}
		endsubmit;
		
		
		call ImportDataSetFromR("beta0", "beta0.obj");
		call ImportDataSetFromR("betas", "betas.obj");
		call ImportDataSetFromR("pred", "pred.obj");
		if (response_type_= "ordinal") then do;
			call ImportDataSetFromR("probs", "probs.obj");
			call ImportDataSetFromR("threshold", "threshold.obj");
		end;
		
		if (FixedEffFlag_) then do;
			call ImportDataSetFromR("fixedeff", "fixedeff.obj");
		end;
		
		if (RandEffFlag_) then do;
			call ImportDataSetFromR("randeff", "randeff.obj");
		end;		
		
	finish CallBGLRFunction; 
	store module=(CallBGLRFunction);	

quit;