/*	
	JMP acknowledges the use of the following Third Party software:
	
	OneMap: software for genetic mapping in outcrossing species
	Copyright (C) 2007-13 Gabriel R A Margarido (mmollina@usp.br) and 
	Marcelo Mollinari (gramarga@gmail.com).
		
	OneMap is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	(at your option) any later version.
		
	OneMap is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.
		
	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, 
	USA."
*/

proc iml;

	/*removes objects in R*/
	start RemoveRObjs(RObjsList_);
				
		submit RObjsList_ / R ok=isOK;
			#print("&RObjsList_");
			#print(nchar("&RObjsList_"));
			
			if(nchar("&RObjsList_")) {
				if("&RObjsList_" == "NONE") {
					#print("1")
				} else if("&RObjsList_" == "ALL") {
					#print("2")
					rm(list=ls())
				} else {
					#print("3")
					rm(&RObjsList_)
				}
			}
			
			#print(ls())
			
		endsubmit;
	finish RemoveRObjs;
	store module=(RemoveRObjs);
	
	start SetRMemory(maxsize_);
		submit maxsize_ / R ok=isOK;
			#if(Sys.getenv("R_ARCH")=="/x64") {
			#}
			memory.limit(size = &maxsize_)
		endsubmit;
	finish SetRMemory;
	store module=(SetRMemory);
	
	start SaveRData(ROutPath_, fname_);
		submit ROutPath_ fname_ / R ok=isOK;
			#print(paste("&ROutPath_", "&fname_.RData", sep=""))
			save.image(file=paste("&ROutPath_", "&fname_.RData", sep=""))
			#print("saving image 2")
		endsubmit;
	finish SaveRData;
	store module=(SaveRData);

	/*loads R packages*/
	start LoadRPkgs;
		submit / R ok=isOK;
			library(onemap)
			
			useOneQTL <- FALSE
			if(useOneQTL) {
				library(oneqtl)
				# not be used util OneQTL gets realeased to the public.
			} else {
				# we load a dll file with compiled 'c' code, see LoadRFunc #			
			}			
		endsubmit;
	finish LoadRPkgs;
	store module=(LoadRPkgs);

	/*loads R functions*/
	start LoadRFunc(RSourcePath_);
		submit RSourcePath_ / R ok=isOK;	
			source(file=paste("&RSourcePath_", "/OutcrossRFunctions.R", sep=""))						
		endsubmit;
	finish LoadRFunc;
	store module=(LoadRFunc);
	
	/*loads R dll files*/
	start LoadRdll(RdllPath_);
		submit RdllPath_ / R ok=isOK;	

			useOneQTL <- FALSE
			if(useOneQTL) {	
				# not be used util OneQTL gets realeased to the public.
			} else {	
				
				bits <- .Machine$sizeof.pointer
				if(bits==4)
					dyn.load(paste("&RdllPath_","/hmm_out_32.dll", sep=""))
				else 
					dyn.load(paste("&RdllPath_","/hmm_out_64.dll", sep=""))
			}			
			
		endsubmit;
	finish LoadRdll;
	store module=(LoadRdll);
	
	/*Unloads R dll files*/
	start UnLoadRdll(RdllPath_);
		submit RdllPath_ / R ok=isOK;	

			useOneQTL <- FALSE
			if(useOneQTL) {	
				# not be used util OneQTL gets realeased to the public.
			} else {	
				
				bits <- .Machine$sizeof.pointer
				if(bits==4)
					dyn.unload(paste("&RdllPath_","/hmm_out_32.dll", sep=""))
				else 
					dyn.unload(paste("&RdllPath_","/hmm_out_64.dll", sep=""))
			}			
			
		endsubmit;
	finish UnLoadRdll;
	store module=(UnLoadRdll);	

	/*reads input txt raw file into R*/
	/*
	start ReadTxtData(InPathTxt_, InFileTxt_, OutCrossObjName_);
		*print(InPathTxt_);
		*print(InFileTxt_);

		submit InPathTxt_ InFileTxt_ OutCrossObjName_ / R ok=isOK;
			&OutCrossObjName_ <- oneqtl.read.outcross(dir="&InPathTxt_", file="&InFileTxt_")
		endsubmit;				
	finish ReadTxtData; 
	store module=(ReadTxtData);
	*/
		
	/*exports data from SAS to R*/
	start ExportDataToR(DataName_, DataObjName_);
		*print(DataName_);
		*print(DataObjName_);
						
		call ExportDataSetToR(DataName_, DataObjName_);
		
		/*
		submit DataObjName_ / R ;
			print(&DataObjName_)
			print(dim(&DataObjName_))
		endsubmit;
		*/
		
	finish ExportDataToR;
	store module=(ExportDataToR);
	
	/*rebuilds outcross object in R*/
	start RebuildOutcrossObj(GenoDataObjName_, AnnoDataObjName_, OutCrossObjName_, Mktype_);
		*print(GenoDataObjName_);
		*print(AnnoDataObjName_);
		*print(OutCrossObjName_);
		*print(Mktype_);
						
		submit Mktype_ GenoDataObjName_ AnnoDataObjName_ OutCrossObjName_ / R ok=isOK;											
			#print(&GenoDataObjName_)
			#print(&AnnoDataObjName_)
			&OutCrossObjName_ <- rebuild.outcross.obj(genodata= &GenoDataObjName_, annodata=&AnnoDataObjName_, mktype= "&Mktype_")	
			#print(&OutCrossObjName_$geno[1:10,])
			#print(&OutCrossObjName_$segr.type)
			#print(&OutCrossObjName_$segr.type.num)
			
			#rm(&GenoDataObjName_)
			#rm(&AnnoDataObjName_)
		endsubmit;
		
	finish RebuildOutcrossObj;
	store module=(RebuildOutcrossObj);
			
	/*estimates two-point pairwise  recombination */
	start TwoPts_RF(LODCutoff_, RCutoff_, OutCrossObjName_, TwoptsObjName_);
		*print(LODCutoff_);
		*print(RCutoff_);
		*print(OutCrossObjName_);
		*print(TwoptsObjName_);

		submit LODCutoff_ RCutoff_ OutCrossObjName_ TwoptsObjName_ / R ok=isOK;
			#print(&OutCrossObjName_)
			
			&TwoptsObjName_ <- rf_2pts(&OutCrossObjName_, LOD=&LODCutoff_, max.rf=&RCutoff_, verbose=FALSE)
			
			#save.image("C:/Users/sbrlsi/AppData/Roaming/SAS/JMPGenomics/13/JMPG/ProcessResults/simuf1.RData");
			#print(&TwoptsObjName_)
			#print("class(&TwoptsObjName_)");
			#print(class(&TwoptsObjName_));
		endsubmit;

	finish TwoPts_RF; 
	store module=(TwoPts_RF);
	
	/*estimates linkage groups based on two-point pairwise recombination */
	start Groups_LG(LODCutoff_, RCutoff_, TwoptsObjName_, LGObjName_, LGDataName_);
		*print(LODCutoff_);
		*print(RCutoff_);
		*print(LGObjName_);
		*print(TwoptsObjName_);

		submit LODCutoff_ RCutoff_ TwoptsObjName_ LGObjName_ / R ok=isOK;
			#print(&LGObjName_)
			markerseq <- make_seq(&TwoptsObjName_, "all")
			LGs <- group(markerseq, LOD=&LODCutoff_, max.rf=&RCutoff_)
			k <- LGs$n.groups + 1
			if(any(is.na(LGs$groups))) {
				for(i in 1:LGs$n.mar){
				    	if(is.na(LGs$groups[i])){
				    		LGs$groups[i] <- k
				    	    	k <- k + 1
					}
				}
			}
			&LGObjName_ <- cbind(LGs$marnames, LGs$groups)
			colnames(&LGObjName_) <- c("mkname", "LGroup")
			#print(&LGObjName_)
		endsubmit;
		
		call ImportDataSetFromR(LGDataName_, LGObjName_);

	finish Groups_LG; 
	store module=(Groups_LG);

	/*sets mapping function to print linkage groups in R*/
	start SetMapFunc(MapFunc_);
		*print(MapFunc_);

		submit MapFunc_ / R ok=isOK;
			if("&Mapfunc_" == "Haldane") {
				mapfunc <- "haldane"
			} else if("&Mapfunc_" == "Kosambi") {
				mapfunc <- "kosambi"	
			} else {
				stop("Mapping function is not valid.")
			}
			
			set_map_fun(type=mapfunc)
		endsubmit;
		
	finish SetMapFunc; 
	store module=(SetMapFunc);

	/*builds linkage groups and order markers in each group*/
	/*
	start BuildLGroupsAndOrder(LODCutoff_, RCutoff_, OrderAlgo_, OrderArg_, TwoptsObjName_, GenoDataObjName_);
		*print(LODCutoff_);
		*print(RCutoff_);
		*print(OrderAlgo_);
		*print(OrderArg_);

		submit LODCutoff_ RCutoff_ OrderAlgo_ OrderArg_ TwoptsObjName_ GenoDataObjName_ / R ok=isOK;
		
			twopts <- &TwoptsObjName_
			
			mark.all <- make_seq(twopts, "all")
			LGs <- group(mark.all, LOD=&LODCutoff_, max.rf=&RCutoff_)
			#print(LGs)
			#print(twopts, "M1", "M3")

			### storing unmapped markers ###
			unmapped.col.names <- c("LGroup", "LGLab", "mkname", "mktype", "seqnum")	
			unmapped.final <- matrix(NA, 1, length(unmapped.col.names))
			colnames(unmapped.final) <- unmapped.col.names
			unmapped.final <- unmapped.final[-1,]
			
			### ungrouped markers ###
			ungrouped.num <- which(is.na(LGs$groups))
			if(length(ungrouped.num)){
				ungrouped.nam <- colnames(get(LGs$data.name, pos = 1)$geno)[ungrouped.num]
				ungrouped.typ <- get(LGs$data.name, pos = 1)$segr.type[ungrouped.num]
				ungrouped.final <- cbind(NA, NA, ungrouped.nam, ungrouped.typ, ungrouped.num)
				unmapped.final <- rbind(unmapped.final, ungrouped.final)
			}
	
			### storing mapped markers ###
			mapped.col.names <- c("LGroup",
					"LGLab",
					"mkname",
					"mktype",
					"mktypenum",
					"seqnum",
					"seq_phases", 
					"seq_rf",
					"CM_Haldane",
					"CM_Kosambi",
					"P1_Chrom1",
					"P1_Chrom2",
					"P2_Chrom1",
					"P2_Chrom2") 

			mapped.final <- matrix(NA, 1, length(mapped.col.names))
			colnames(mapped.final) <- mapped.col.names
			mapped.final <- mapped.final[-1,]

			### start ordering markers ###
			nLGs <- LGs$n.groups
			if(nLGs > 0) {

				for(i in 1:nLGs) {
					LG.start <- make_seq(LGs, i)

					if("&OrderAlgo_" == "ser") {
						LG.tmp <- seriation(LG.start)
					} else if("&OrderAlgo_" == "rcd") {
						LG.tmp <- rcd(LG.start)
					} else if("&OrderAlgo_" == "record") {
						LG.tmp <- record(LG.start)
					} else if("&OrderAlgo_" == "ug") {
						LG.tmp <- ug(LG.start)
					} else if("&OrderAlgo_" == "compare") {
						LG.tmp <- compare(LG.start, verbose=FALSE) 
					} else if("&OrderAlgo_" == "automated") {
						LG.tmp <- order_seq(LG.start, THRES=&LODCutoff_, touchdown=TRUE) 
					} else {
						Stop("&OrderAlgo_ is not available.");
					}

					LG.final <- make_seq(LG.tmp, "&OrderArg_")

					### mapped markers ###
					mknames <- colnames(get(LG.final$data.name, pos = 1)$geno)[LG.final$seq.num]
					mktypes <- get(LG.final$data.name, pos = 1)$segr.type[LG.final$seq.num]
					mktypesnum <- get(LG.final$data.name)$segr.type.num[LG.final$seq.num]

					cm.haldane <- cumsum(c(0, haldane(LG.final$seq.rf)))
					cm.kosambi <- cumsum(c(0, kosambi(LG.final$seq.rf)))

					parents.map <- parents.map.adpt(LG.final) 
					
					LG.lab <- paste("LG",i,sep="")

					LG.final.tmp <- cbind(i,
								LG.lab,
								mknames,
								mktypes,
								mktypesnum,
								LG.final$seq.num, 
								c(NA, LG.final$seq.phases), 
								c(NA, LG.final$seq.rf),
								cm.haldane,
								cm.kosambi,
								parents.map
								)

					mapped.final <- rbind(mapped.final, LG.final.tmp)				

					### get two-point and multipoint recombination ###
					matlist <- rf.graph.table.adpt1(LG.final)

					colnames(matlist$mat.rf) <- colnames(matlist$mat)
					assign(paste("twop.rf",i,sep=""), matlist$mat)
					assign(paste("mult.rf",i,sep=""), matlist$mat.rf)

					#print(matlist$mat)
					#print(matlist$mat.rf)
					
					### above diagonal two-points and below diagonal multipoint ###
					two.mul.rf <- matlist$mat					
					for(j in 1:dim(matlist$mat)[1])
						for(k in 1:j)
							two.mul.rf[j,k] <- matlist$mat.rf[j,k]
									
					assign(paste("two.mul.rf",i,sep=""), two.mul.rf)

					### unlinked markers ###
					unlinked.num <- setdiff(LG.start$seq.num, LG.final$seq.num) 
					if(length(unlinked.num)) {
						unlinked.nam <- colnames(get(LG.start$data.name, pos = 1)$geno)[unlinked.num]
						unlinked.typ <- get(LG.start$data.name, pos = 1)$segr.type[unlinked.num]
						unlinked.final <- cbind(i, LG.lab, unlinked.nam, unlinked.typ, unlinked.num)
						unmapped.final <- rbind(unmapped.final, unlinked.final)
					}
				}
			}
						
			### subset and sort genotypic data according to linkage map ###						
			which.mk <- pmatch(as.vector(mapped.final[,"mkname"]), colnames(get(twopts$data.name)$geno))							
			subgen.final <- &GenoDataObjName_[,which.mk]
			
			#print(subgen.final)								
		endsubmit;
		
		*goto exitLinkageGroups;

		call ImportMatrixFromR(nLGs, "nLGs");
		call ImportDataSetFromR("nLGs", "nLGs");
		call ImportDataSetFromR("subgendat", "subgen.final"); 			

		do i=1 to nLGs;
			call symputx("j",i);
			call ImportDataSetFromR("twop_rf"+symget("j"), "twop.rf"+symget("j"));
			call ImportDataSetFromR("mult_rf"+symget("j"), "mult.rf"+symget("j"));
			call ImportDataSetFromR("two_mul_rf"+symget("j"), "two.mul.rf"+symget("j"));
		end;
		
		call ImportDataSetFromR("mapped_final", "mapped.final");
		call ImportDataSetFromR("unmapped_final", "unmapped.final");

		exitLinkageGroups:;
		
	finish BuildLGroupsAndOrder; 
	store module=(BuildLGroupsAndOrder);
	*/

	/*exports marker variable names from SAS to R*/
	start ExportMarkerVarsToR(MarkerVars_, MarkerVarsObjName_);
		*print(MarkerVars_);
		*print(MarkerVarsObjName_);
		
		i=1;
		mkvar=scan(MarkerVars_,i);
		do while(mkvar ^= ' ');
			mkvars_all=mkvars_all//mkvar;
			i= i + 1; 		
			mkvar=scan(MarkerVars_,i);
		end;
		*print(mkall);
				
		call ExportMatrixToR(mkvars_all, MarkerVarsObjName_);
		
	finish ExportMarkerVarsToR;
	store module=(ExportMarkerVarsToR);
		
	/*rebuilds linkage group objects in R*/
	start RebuildLGsObj(Chrom_, Mkpos_, Mklab_, Linkphase_, Mapfunc_, GenoDataObjName_, 
			AnnoDataObjName_, OutCrossObjName_, LGroupsObjName_);
		*print("RebuildLGsObj");
		*print(Chrom_);
		*print(Mkpos_);
		*print(Mklab_);
		*print(Linkphase_);
		*print(Mapfunc_);	
		*print(GenoDataObjName_);
		*print(AnnoDataObjName_);
				
		submit Chrom_ Mkpos_ Mklab_ Linkphase_ Mapfunc_ GenoDataObjName_ 
			AnnoDataObjName_ OutCrossObjName_ LGroupsObjName_/ R ok=isOK;
													
			&LGroupsObjName_ <- rebuild.LGs.obj(genodata= &GenoDataObjName_, annodata= &AnnoDataObjName_, 
						chrom="&Chrom_", mkpos="&Mkpos_", mklab= "&Mklab_", 
						linkphase="&Linkphase_", mapfunc= "&Mapfunc_", 
						outcrossobjname="&OutCrossObjName_")
						
			#print(&LGroupsObjName_)
									
		endsubmit;
	finish RebuildLGsObj;
	store module=(RebuildLGsObj);
	
	/*estimates conditional probabilities of pseudomarkers*/
	start BuildGP(Mapfunc_, LGroupsObjName_, GProbsData_);
							
		submit Mapfunc_ LGroupsObjName_ / R ok=isOK;
												
			if(toupper("&Mapfunc_") == "HALDANE") {
				mapfunc <- "haldane"
			} else if(toupper("&Mapfunc_") == "KOSAMBI") {
				mapfunc <- "kosambi"	
			} else {
				stop("Mapping function is not valid.")
			}
			
			#print(&LGroupsObjName_)
									
			LGs.obj <- &LGroupsObjName_
						
			g.probs <- c()
			for(k in LGs.obj$LGs.num){
								
				LG.tmp <- LGs.obj$LGs.map[[k]]
				
				#print(LG.tmp)												
				#print(LG.tmp$seq.num)
				#print(LG.tmp$seq.phases)
				#print(LG.tmp$seq.rf)
				#print(LG.tmp$seq.like)
				#print(LG.tmp$data.name)
				#print(LG.tmp$twopt)
				
				useOneQTL <- FALSE
				if(useOneQTL){
					## probs() from the OneQTL package ##
					probs.tmp <- probs(seq.obj=list(LG.tmp), map.function= mapfunc, step.size= 1, 
							error.prob= 0.0001, chrom.names= NULL, cond.limit= 3.5)
	
					n.pseudo.mrk <- dim(probs.tmp$chromosomes$LG.tmp$probs)[2]
					
					#print(probs.tmp$chromosomes$LG.tmp$map$dist)
													
					probs.mat <- c()
					pseudo.pos <- c()
					mk.indc <- c()
					lgnum <- c()
					for(u in 1:n.pseudo.mrk) {
						probs.mat <- cbind(probs.mat, probs.tmp$chromosomes$LG.tmp$probs[,u,1:3])	
						pseudo.pos <- c(pseudo.pos, rep(probs.tmp$chromosomes$LG.tmp$map$dist[u],(4-1)))
						if(substr(names(probs.tmp$chromosomes$LG.tmp$map$dist),1,3)[u] != "loc")
							mk.indc <- c(mk.indc, rep(1, (4-1)))
						else
							mk.indc <- c(mk.indc, rep(0, (4-1)))
						lgnum <- c(lgnum, rep(k, (4-1)))
					}	
				} else {
				
					inv.kosambi <- function(d) {
					    0.5*tanh(d/50)
					}
					inv.haldane <- function(d) {
					    0.5*(1-exp(-d/50))
					}

					if (toupper(mapfunc) == 'KOSAMBI' ) {
						map.func <- kosambi #from onemap
						inv.func <- inv.kosambi
					} else {
						inv.func <- inv.haldane
						map.func <- haldane #from onemap
					}

					numb.ind <- get(LG.tmp$data.name)$n.ind
					map.dist <- c(0, cumsum(map.func(LG.tmp$seq.rf)))
					names(map.dist) <- colnames(get(LG.tmp$data.name)$geno[,LG.tmp$seq.num])
					loc.grid <- c()
					for (i in 1:(length(map.dist) - 1)) {
					    ce <- ceiling(map.dist[i])
					    fl <- floor(map.dist[i+1])
					    if(ce < fl) {
						loc.grid.tmp <- seq(ce, fl, by=1)
						loc.grid <- c(loc.grid, loc.grid.tmp)
					    }
					}
					loc.grid <- setdiff(loc.grid, map.dist)
					names(loc.grid) <- rep("loc", length(loc.grid))
					loc.grid <- sort(c(map.dist, loc.grid))
					numb.loc <- length(loc.grid)
					loc.types <- rep(1, numb.loc)
					loc.types[pmatch(names(map.dist), names(loc.grid))] <- get(LG.tmp$data.name)$segr.type.num[LG.tmp$seq.num]
					loc.phase <- rep(1, numb.loc)
					loc.phase[pmatch(names(map.dist), names(loc.grid))] <- c(LG.tmp$seq.phases, 1)
					loc.phase <- loc.phase[-length(loc.phase)]
					loc.genos <- matrix(0, nrow= numb.ind, ncol=numb.loc)
					loc.genos[,pmatch(names(map.dist), names(loc.grid))] <- get(LG.tmp$data.name)$geno[,LG.tmp$seq.num]
					loc.rf <- inv.func(diff(loc.grid))

					listcalc <- .C("calc_genoprob_outbred",
						as.integer(numb.ind), 
						as.integer(numb.loc), 
						as.integer(loc.types),
						as.integer(loc.phase),
						as.integer(loc.genos),     
						as.double(loc.rf),     
						as.double(0.0001),
						genoprob=as.double(rep(0,4*numb.ind*numb.loc))
						) #from onemap

					probs.mat <- c()
					for(i in 1:numb.loc) {
					    probs.mat <- cbind(probs.mat,
								 listcalc$genoprob[((i-1)*numb.ind + 1):(i*numb.ind)],
								 listcalc$genoprob[(numb.ind * (1*numb.loc + i - 1) + 1):((numb.ind * (numb.loc + i - 1)) + numb.ind)],
								 listcalc$genoprob[(numb.ind * (2*numb.loc + i - 1) + 1):((numb.ind * (2*numb.loc + i - 1)) + numb.ind)]
								 )
					}

					pseudo.pos <- c()
					mk.indc <- c()
					lgnum <- c()
					z <- 1
					for(u in 1:numb.loc) {
					    pseudo.pos <- c(pseudo.pos, rep(loc.grid[u],(4-1)))
					    if(substr(names(loc.grid),1,3)[u] != "loc") {
						#mk.indc <- c(mk.indc, rep(1, (4-1)))
						mk.indc <- c(mk.indc, rep(z, (4-1)))
						z <- z + 1
					    } else {
						mk.indc <- c(mk.indc, rep(0, (4-1)))
					    }
					    lgnum <- c(lgnum, rep(k, (4-1)))
					}
				}
								
				#print(c(length(lgnum), length(pseudo.pos), length(mk.indc), dim(probs.mat)[2]))
				
				probs.mat.final <- rbind(lgnum, pseudo.pos, mk.indc, probs.mat)
				
				g.probs <- cbind(g.probs, probs.mat.final)	
				
				#print(c("k after",k))
			}
			
			colnames(g.probs) <- paste("COL", seq(1:dim(g.probs)[2]),sep="")
			
		endsubmit;
		
		call ImportDataSetFromR(GProbsData_, "g.probs");
		
	finish BuildGP;	
	store module=(BuildGP);	
	
	start GetTwopts_Analysis(TwoptsObjName_, BestRFMatrixObjName_, BestRFMatrixDataName_, 
						BestLODMatrixObjName_, BestLODMatrixDataName_,
						BestPhMatrixObjName_, BestPhMatrixDataName_,
						AllPhasesObjName_, AllPhasesDataName_);
		*print(TwoptsObjName_);
		*print(BestRFMatrixObjName_);
		*print(BestRFMatrixDataName_);
		
		submit TwoptsObjName_ BestRFMatrixObjName_ BestRFMatrixDataName_ 
			BestLODMatrixObjName_ BestLODMatrixDataName_ 
			BestPhMatrixObjName_ BestPhMatrixDataName_ 
			AllPhasesObjName_ AllPhasesDataName_ / R ok=isOK;
	
			#print("names(&TwoptsObjName_)");
			#print(names(&TwoptsObjName_));
			#print(&TwoptsObjName_);
			#print("names(&TwoptsObjName_$analysis)");
			#print(names(&TwoptsObjName_$analysis));
			#print("&TwoptsObjName_$analysis");
			#print(&TwoptsObjName_$analysis);
			
			#save.image("C:/Users/sbrlsi/AppData/Roaming/SAS/JMPGenomics/13/JMPG/ProcessResults/simuf1.RData");
			
			#print("Pairwise");
			#print(&TwoptsObjName_,c(mrk1="M1", mrk2="M2"));
			#print(&TwoptsObjName_,c(mrk1="M1", mrk2="M3"));
			#print(&TwoptsObjName_,c(mrk1="M1",mrk2="M4"));
			#print(&TwoptsObjName_,c(mrk1="M2",mrk2="M3"));
			#print(&TwoptsObjName_,c(mrk1="M2",mrk2="M4"));
			
			two.pts.mats <- get.2pts.analysis(&TwoptsObjName_) 
			
			&BestRFMatrixObjName_ <- two.pts.mats$best.rf2.pts.mat
			#print(&BestRFMatrixObjName_);
			&BestLODMatrixObjName_ <- two.pts.mats$best.lod2.pts.mat
			#print(&BestLODMatrixObjName_);
			&BestPhMatrixObjName_ <- two.pts.mats$best.ph2.pts.mat
			#print(&BestPhMatrixObjName_);
			&AllPhasesObjName_ <- two.pts.mats$all.phases
			#print(&AllPhasesObjName_);			
	
		endsubmit;
		
		call ImportDataSetFromR(BestRFMatrixDataName_, BestRFMatrixObjName_);
		call ImportDataSetFromR(BestLODMatrixDataName_, BestLODMatrixObjName_);
		call ImportDataSetFromR(BestPhMatrixDataName_, BestPhMatrixObjName_);
		call ImportDataSetFromR(AllPhasesDataName_, AllPhasesObjName_);
		
		
	finish GetTwopts_Analysis;
	store module=(GetTwopts_Analysis);
	
	
	start RebuildRF2PtsObj(PhasesObjName_, OutCrossObjName_, minLOD_, maxRF_, TwoptsObjName_);
		*print(PhasesObjName_);
		*print(OutCrossObjName_);
		*print(TwoptsObjName_);
		*print(minLOD_);
		*print(maxRF_);
		
		submit PhasesObjName_ OutCrossObjName_ TwoptsObjName_ minLOD_ maxRF_/ R ok=isOK;
	
			#print("&PhasesObjName_")
			#print(&PhasesObjName_)
			#save.image("C:/Users/sbrlsi/AppData/Roaming/SAS/JMPGenomics/13/JMPG/ProcessResults/simuf1.RData");
	
			&TwoptsObjName_ <- rebuild.2pts.rf.obj(&PhasesObjName_, &OutCrossObjName_, &minLOD_, &maxRF_)
			
			#print("&TwoptsObjName_")
			#print(&TwoptsObjName_)
			#print(&TwoptsObjName_$analysis)	
			
		endsubmit;
		
	finish RebuildRF2PtsObj;
	store module=(RebuildRF2PtsObj);

	/*orders markers on a given linkage group*/
	start OrderMarkersOneMap(LGObjName_, MarkerVarName_, TwoptsObjName_, minLOD_, maxRF_, OrderAlgo_, ForceArg_, FixedOrder_, SkeletonVar_, nShuffle_, ShuffleWSize_, ShuffleLOD_);
		/*FixedOrder= none (no marker have fixed ordering), 
			    = partial (some markers have fixed ordering)
			    = all (all markers have fixed ordering)*/
		*print(LGObjName_);
		*print(MarkerVarName_);
		*print(TwoptsobjName_);
		*print(minLOD_);
		*print(maxRF_);
		*print(OrderAlgo_);
		*print(ForceArg_);
		*print(FixedOrder_);
		*print(SkeletonVar_);
	
		submit LGObjName_ MarkerVarName_ TwoptsObjName_ minLOD_ maxRF_ OrderAlgo_ ForceArg_ FixedOrder_ SkeletonVar_ nShuffle_ ShuffleWSize_ ShuffleLOD_/ R ok=isOK;

			#save.image("C:/Users/sbrlsi/AppData/Roaming/SAS/JMPGenomics/13/JMPG/ProcessResults/simuf1.RData");
			#print(&LGObjName_)
			#print("&MarkerVarName_")
			#print("hello")
			
			debug0 <- 0
			if (debug0 == 1) {
				load("MyRData")
				&TwoptsObjName_$analysis <- twopts.obj.rec$analysis
			}
			
			debug1 <- 0
			if (debug1 == 1) {
				print("d1.indata")
				d1.indata <- read_outcross(dir="MyPath", file="MyFile")
				d1.indata$segr.type
				d1.indata$segr.type.num
				d1.rf2pts <- rf_2pts(d1.indata, max.rf=.5, LOD=2.5)
				round(d1.rf2pts$analysis[1:66,,1],5)
				set_map_fun("kosambi")
				d1.lg <- make_seq(d1.rf2pts, seq(1:20))
				marker_type(d1.lg)
				d1.lg.ord <- order_seq(d1.lg, THRES=2.5, touchdown=FALSE)
				(d1.lg.fin <- make_seq(d1.lg.ord))
				marker_type(d1.lg.fin)
			}
			
			debug2 <- 0
			if (debug2 == 1) {
				print("outcross.obj")
				outcross.obj$segr.type
				outcross.obj$segr.type.num
				twopts.obj.rec <- rf_2pts(outcross.obj, max.rf=.5, LOD=2.5)
				round(twopts.obj.rec$analysis[, ,1],10)
				round(twopts.obj.rec$analysis[, ,2],10)
				set_map_fun("kosambi")
				curr.mk.seq.rec <- make_seq(twopts.obj.rec, "all")
				print(curr.mk.seq.rec)
				marker_type(curr.mk.seq.rec)
				LG.tmp.jr.rec <- order_seq(curr.mk.seq.rec, THRES=2.5, touchdown=FALSE)
				(LG.final.jr.rec <- make_seq(LG.tmp.jr.rec))
				marker_type(LG.final.jr.rec)
				(LG.final.jr.rec.rcd <- rcd(curr.mk.seq.rec, LOD=2.5, max.rf=.35))
			}
			
			debug3 <- 0
			if (debug3 == 1) {
				print("names(TwoptsObjName_)")
				print(names(&TwoptsObjName_))
				print(get(&TwoptsObjName_$data.name)$segr.type)
				print(get(&TwoptsObjName_$data.name)$segr.type.num)
				print(round(&TwoptsObjName_$analysis[,,1],10))
				print(round(&TwoptsObjName_$analysis[,,2],10))
			}
			
			debug4 <- 0
			if (debug4 == 1) {
				&TwoptsObjName_$analysis <- twopts.obj.rec$analysis
			}
			
			debug5 <- 0
			if (debug5 == 1) {
				&TwoptsObjName_ <- twopts.obj.rec
			}
			
			runflag0 <- 1
			if (runflag0==1) {
				#curr.mk.nam <- as.vector(unlist(&LGObjName_[pmatch(toupper("&MarkerVarName_"), toupper(colnames(&LGObjName_)))]))					
				#print("curr.mk.nam")
				#print(curr.mk.nam)
				#print("marnames")
				#print(&TwoptsObjName_)
				#print(&TwoptsObjName_$marnames)
				#curr.mk.num <- pmatch(curr.mk.nam, &TwoptsObjName_$marnames) 
				#print(curr.mk.num)
				#curr.mk.seq <- make_seq(&TwoptsObjName_, curr.mk.num)
				curr.mk.seq <- make_seq(&TwoptsObjName_, "all")
				#print(curr.mk.seq)
				#print(get(curr.mk.seq$twopt)$analysis[1:10,,1])
				#is.numeric(get(curr.mk.seq$twopt)$analysis[1,1,1])
				if (toupper("&FixedOrder_") == "NONE") { 					
					# ordering markers #
					if(toupper("&OrderAlgo_") == "AUTOMATED") {
						#print("AUTOMATED")
						LG.tmp <- order_seq(curr.mk.seq, THRES=&minLOD_, touchdown=FALSE) 
						LG.final <- make_seq(LG.tmp, "&ForceArg_")
					} else if(toupper("&OrderAlgo_") == "RCD") {
						#print("RCD")
						#LG.final <- rcd(curr.mk.seq, LOD= &minLOD_, max.rf= &maxRF_)
						LG.final <- rcd(curr.mk.seq, LOD= 2.5, max.rf= 0.35)
					} else if(toupper("&OrderAlgo_") == "SERIATION") {
						#print("SERIATION")
						LG.final <- seriation(curr.mk.seq, LOD= &minLOD_, max.rf= &maxRF_)
					} else if(toupper("&OrderAlgo_") == "RECORD") {
						#print("RECORD")
						LG.final <- record(curr.mk.seq, LOD= &minLOD_, max.rf= &maxRF_)
					} else if(toupper("&OrderAlgo_") == "UG") {
						#print("UF")
						LG.final <- ug(curr.mk.seq, LOD= &minLOD_, max.rf= &maxRF_)
					} else if(toupper("&OrderAlgo_") == "COMPARE") {
						#print("COMPARE")	
						if (length(curr.mk.seq$seq.num) > 2) {
							LG.tmp <- compare(curr.mk.seq, verbose=FALSE) 
							LG.final <- make_seq(LG.tmp, arg=NULL)
						} else {
							LG.final <- map(curr.mk.seq)
						}						
					} else {
						stop("Order Algorithm &OrderAlgo_ is not available.");
					}

				} else if (toupper("&FixedOrder_") == "PARTIAL") { 					
					## ordering markers ##
					#print("PARTIAL")
					#print(curr.mk.num)
					#framednum <- curr.mk.num[!is.na(as.vector(&LGObjName_$&SkeletonVar_))]
					framednum <- curr.mk.seq$seq.num[!is.na(as.vector(unlist(&LGObjName_[pmatch(toupper("&SkeletonVar_"), toupper(colnames(&LGObjName_)))])))]					
					#print("framednum")
					#print(framednum)
					#print("curr.mk.seq$seq.num")
					#print(curr.mk.seq$seq.num)
					#print("curr.mk.seq")
					#print(curr.mk.seq)
					#save.image("C:/Users/sbrlsi/AppData/Roaming/SAS/JMPGenomics/13/JMPG/ProcessResults/simuf1.RData");
					if(length(framednum) != length(curr.mk.seq) && length(framednum) > 1) {
						LG.tmp <- order.seq.adpt(curr.mk.seq, THRES=&minLOD_, touchdown=FALSE, framed.num= framednum)							
						LG.final <- make_seq(LG.tmp, "&ForceArg_")
					} else {
						LG.final <- map(curr.mk.seq)				
					}
				} else if (toupper("&FixedOrder_") == "ALL") {
					## no need to order markers ##
					#print("FIXEDORDER")
					LG.final <- map(curr.mk.seq)
				} else {
					stop("&FixedOrder_ is wrong!")
				}
				
				if (&nShuffle_ > 0 && length(curr.mk.seq$seq.num) > 5) {
					#print("Shuffling")
					k <- 1
					maxnRip <- min(&nShuffle_, prod(1:length(curr.mk.seq$seq.num)))
					maxws <- min(&ShuffleWSize_, length(curr.mk.seq$seq.num) - 1)
					#LG.final.Debug <- LG.final
					#save.image("C:/Users/sbrlsi/Documents/Defects/S1428516/Results/Outcross.RData");
					#print(LG.final)
					#print(LG.final$seq.rf)
					#print(sum(LG.final$seq.rf))
					while(k <=  maxnRip) {
						LG.final <- ripple.seq.adpt(LG.final, ws=maxws, LOD=&ShuffleLOD_, tol= 0.01, verbose=FALSE)
						#print(k)
						#print(LG.final)
						#print(LG.final$seq.rf)
						#print(sum(LG.final$seq.rf))
						k <- k + 1
					}
				} else if (&nShuffle_ > 0 && length(curr.mk.seq$seq.num) <= 5) {
					#print("Shuffling")
					#names(curr.mk.seq)
					#print(curr.mk.seq)
					#print(class(curr.mk.seq))
					if (length(curr.mk.seq$seq.num) > 2) {
						LG.tmp <- compare(curr.mk.seq, verbose=FALSE) 
						LG.final <- make_seq(LG.tmp, arg=NULL)	
					} else {
						LG.final <- map(curr.mk.seq)
					}
				}
				
				#print(LG.final)
				#print(marker.type(LG.final))
				#save.image("C:/Users/sbrlsi/Documents/Defects/S1428463/Results/OutCross.RData");
				
				runflag1 <- 1
				if (runflag1==1) {

					### get two-point and multipoint recombination ###
					rfmats1 <- rf.graph.table.adpt1(LG.final)

					#Recombination fractions are stored in the lower triangular matrix of $mat#
					rfmats1.mat <- rfmats1$mat 

					for(j in 1:(dim(rfmats1.mat)[1] - 1))
						for(k in (j+1):dim(rfmats1.mat)[1])
							rfmats1.mat[j,k] <- NA

					### mapped markers data set ###
					mknames <- colnames(get(LG.final$data.name, pos = 1)$geno)[LG.final$seq.num]
					mktypes <- get(LG.final$data.name, pos = 1)$segr.type[LG.final$seq.num]
					mkorder <- seq(1, length(LG.final$seq.num))
					cm.haldane <- cumsum(c(0, haldane(LG.final$seq.rf)))
					cm.haldane.step <- c(0, haldane(LG.final$seq.rf))
					cm.kosambi <- cumsum(c(0, kosambi(LG.final$seq.rf)))
					cm.kosambi.step <- c(0, kosambi(LG.final$seq.rf))
					parents.map <- parents.map.adpt(LG.final) 

					mappedm.final <- data.frame(
							mkname_=mknames,
							mktype_=mktypes,
							mkorder_=mkorder,
							mknumb_=LG.final$seq.num, 
							phase_=c(NA, LG.final$seq.phases), 
							rf_=c(NA, LG.final$seq.rf),
							cMHaldane_=cm.haldane,
							cMHaldaneStep_= cm.haldane.step,
							cMKosambi_=cm.kosambi,
							cMKosambiStep_=cm.kosambi.step,
							A1P1_=parents.map[,1],
							A2P1_=parents.map[,2],
							A1P2_=parents.map[,3],
							A2P2_=parents.map[,4]
							)

					#print(mappedm.final)

					### unmapped markers data set ###
					unmapped.num <- setdiff(curr.mk.seq$seq.num, LG.final$seq.num) 
					if(length(unmapped.num)) {
						unmapped.nam <- colnames(get(curr.mk.seq$data.name, pos = 1)$geno)[unmapped.num]
						unmapped.typ <- get(curr.mk.seq$data.name, pos = 1)$segr.type[unmapped.num]
						unmappedm.final <- data.frame(mkname_=unmapped.nam,
									mktype_=unmapped.typ,
									mknumb_=unmapped.num)
					} else {
						unmappedm.final <- data.frame(mkname_=numeric(0),
									mktype_=numeric(0),
									mknumb_=numeric(0)
									)
					}
				}
			}
									
		endsubmit;
		
		*goto exitOrderMarkers;
		
		call ImportDataSetFromR("mappedm", "mappedm.final");
		call ImportDataSetFromR("rfmat1_mat", "rfmats1.mat");
		call ImportDataSetFromR("unmappedm", "unmappedm.final");
		
		exitOrderMarkers:;
		
	finish OrderMarkersOneMap; 
	store module=(OrderMarkersOneMap);

quit;
