/*
JMP Genomics Analytical Process

Name: ConfigureSettings
Author: Russ Wolfinger, SAS Institute Inc.
Support: Russ Wolfinger, SAS Institute Inc.
History:
   sasrdw 02Oct08 initial coding

Description: 
This process defines the default settings
for JMP Genomics. 
*/

/*
-----------------------------------------------------------
Copyright (C) 2008 SAS Institute, Inc. All rights reserved. 

Notice: 
The above copyright notice and this notice must appear in
any whole or partial copy of this code and any related
documentation. Permission to use, copy, and modify the
source code contained in this file is hereby granted, but
is limited to customers of SAS with a valid license for the
SAS software product with which this file was provided. 
Except as expressly set forth herein, the terms of such
license apply.
-----------------------------------------------------------
*/

*ProcessVariables;

%global StudyName SettingName DefaultInPath DefaultLibPath DefaultAnnoPath 
DefaultSDTMInPath DefaultAInPath 
DefaultSettingsPath DefaultOutPath ServerOutPath  
	ArchiveSASCode ResolveSASCode PromptTabbedReport ShowCustButtons
	OutPath ServerOutPath MacroPath
	CVSStarterFiltering CVSReportStudyFiltering CVSDialogReqLink
	CVSShowSelParamsOnly CVSPreStudyReqSet
	NarrativeTemplatePath NarrativeTemplateDefaultName
	GridMetaServer GridMetaPort GridMetaRepository 
	GridMetaUser GridMetaPass ClientGridSharePath ServerGridSharePath
	ProxyName ProxyPort DocumentationPreference
	SDDEnabled SDDURL SDDVersion
;

*ProcessBody;

%macro ConfigureSettings;

%let ProcessName=ConfigureSettings;
%let FullAPName = %str(Configure Life Sciences Settings);

%include "&MacroPath./UtilityMacros.sas" / nosource;

%if &syserr > 4 %then %goto exit;

%if %length(&SettingName) %then %let OutName = &SettingName;

%let DefaultInPath = %trim(&DefaultInPath);
%if %length(&DefaultInPath) %then %CheckEndSlash(DefaultInPath);
%let DefaultLibPath = %trim(&DefaultLibPath);
%if %length(&DefaultLibPath) %then %CheckEndSlash(DefaultLibPath);
%let DefaultAnnoPath = %trim(&DefaultAnnoPath);
%if %length(&DefaultAnnoPath) %then %CheckEndSlash(DefaultAnnoPath);
%let DefaultSDTMInPath = %trim(&DefaultSDTMInPath);
%if %length(&DefaultSDTMInPath) %then %CheckEndSlash(DefaultSDTMInPath);
%let DefaultAInPath = %trim(&DefaultAInPath);
%if %length(&DefaultAInPath) %then %CheckEndSlash(DefaultAInPath);
%let DefaultSettingsPath = %trim(&DefaultSettingsPath);
%if %length(&DefaultSettingsPath) %then %CheckEndSlash(DefaultSettingsPath);
%let DefaultOutPath = %trim(&DefaultOutPath);
%if %length(&DefaultOutPath) %then %CheckEndSlash(DefaultOutPath);

%let SavePath = &USER_HOME;

filename dfile "&SavePath.DefaultPaths.jsl";

filename jslmsg "&OutPath.sasmessage.jsl";
data _null_;
	file jslmsg %if %nrbquote(&SYSENCODING) ne wlatin1 %then encoding="utf-8";;
	put 'sasmsgwindow=New Window(locale:get("Results_Text") || ": " || locale:get("ConfigureLifeSciencesSettings_Text"),';
	put '   VListBox(TextBox(locale:get("Thefollowingfileshavebeensaved_Text")),';
run;

%let copyfiles = TabbedReportPrompt.jsl ArchiveSASCode.jsl;

%if %length(&DefaultInPath) or
	%length(&DefaultLibPath) or
	%length(&DefaultAnnoPath) or
	%length(&DefaultSDTMInPath) or
	%length(&DefaultAInPath) or
	%length(&DefaultSettingsPath) or
	%length(&DefaultOutPath) or
	%upcase(&DocumentationPreference)=%str(LOCAL VERSION OF THE USER GUIDE) or
	%upcase(&DocumentationPreference)=%str(ONLINE VERSION OF THE USER GUIDE) or 
	^%sysfunc(fexist(dfile)) %then %do;
	
	/*
	%if ^%length(&DefaultSettingsPath) %then %do;
		%let DefaultSettingsPath = &ClientOutPath.Settings\;
	%end;
	*/
	
	%if %upcase(&DocumentationPreference)=%str(LOCAL VERSION OF THE USER GUIDE) %then %do;
		%let DefaultDocPath=%nrbquote(&LS_HOME\Documentation\life_sciences_documentation\);
		%let DefaultCDocPath=%nrbquote(&LS_HOME\Documentation\clinical_documentation\);
	%end;
	
	%else %do;
		%let DefaultDocPath=%nrbquote(http://www.jmp.com/support/downloads/JMPG91_documentation/);
		%let DefaultCDocPath=%nrbquote(http://www.jmp.com/support/downloads/clinical_documentation/);
	%end;
	
 
	data _null_;
		file dfile %if %nrbquote(&SYSENCODING) ne wlatin1 %then encoding="utf-8";;	
		put "jg:_DefaultInPath_= ""&DefaultInPath"";";
		put "jg:_DefaultLibPath_ = ""&DefaultLibPath"";";
		put "jg:_DefaultAnnoPath_ = ""&DefaultAnnoPath"";";
		put "jg:_DefaultSettingsPath_ = ""&DefaultSettingsPath"";";
		put "jg:_DefaultOutPath_ = ""&DefaultOutPath"";";
		put "jg:HTMLPrefix = ""&DefaultDocPath"";";
		put "jg:HTMLPrefixC = ""&DefaultCDocPath"";";
	run;

	%if ^&syserr and ^&exiterror %then %do;
		data _null_;
			file jslmsg mod %if %nrbquote(&SYSENCODING) ne wlatin1 %then encoding="utf-8";;
			put "      TextBox("" ""),";
			put "      TextEditBox(""&&CLIENT_LSUSER_HOME.DefaultPaths.jsl"",";
			put '         <<Font Color(5), <<Set Font Size(10), <<Set Wrap(1000)),';
			put '      TextBox(" "),';
			%if %upcase(&DocumentationPreference)=%str(LOCAL VERSION OF THE USER GUIDE) %then %do;
				put "		TextBox(locale:get(""NoteInternetExplorerIsOnlySuppBrLocalUserGuide_Text""),<<FontColor(6),<< Set Wrap(530)),";
			%end;
		run;
		%PackageInsertFile(Default paths from &ProcessName, &OutPath, DefaultPaths.jsl, , text/plain, yes, no);
		%let copyfiles = &copyfiles DefaultPaths.jsl;
	%end;
%end;


%if %length(&GridMetaServer) and
	%length(&GridMetaPort) and
	%length(&GridMetaRepository) and
	%length(&GridMetaUser) and
	%length(&ClientGridSharePath) and 
	%length(&ServerGridSharePath)
	%then %do;
	
	filename gofile "&SavePath.GridOptions.sas";

	data _null_;
		file gofile %if %nrbquote(&SYSENCODING) ne wlatin1 %then encoding="utf-8";;	
		put "options metaserver=""&GridMetaServer"";";
		put "options metaport=&GridMetaPort;";
		put "options metarepository=""&GridMetaRepository"";";
		put "options metauser=""&GridMetaUser"";";
		put "options metapass=""&GridMetaPass"";";
		put "%nrbquote(%)global clientgshare servergshare;";
		put "%nrbquote(%)let clientgshare = &ClientGridSharePath;";
		put "%nrbquote(%)let servergshare = &ServerGridSharePath;";
		put "libname gshare ""&ClientGridSharePath"";";
	run;

	%if ^&syserr and ^&exiterror %then %do;
		data _null_;
			file jslmsg mod %if %nrbquote(&SYSENCODING) ne wlatin1 %then encoding="utf-8";;
			put "      TextBox("" ""),";
			put "      TextEditBox(""&CLIENT_LSUSER_HOME.GridOptions.sas"",";
			put '         <<Font Color(5), <<Set Font Size(10), <<Set Wrap(1000)),';
			put '      TextBox(" "),';
		run;
		%PackageInsertFile(Grid options from &ProcessName, &OutPath,GridOptions.sas, , text/plain, yes, no);
		%let copyfiles = &copyfiles GridOptions.sas;
	%end;
%end;

%if %length(&ProxyName) and %length(&ProxyPort) %then %do;
	filename pfile "&SavePath.ProxyOptions.sas";

	data _null_;
		file pfile %if %nrbquote(&SYSENCODING) ne wlatin1 %then encoding="utf-8";;	
		put '%let ProxyName = ' "&ProxyName;";
		put '%let ProxyPort = ' "&ProxyPort;";
	run;
	
	%if ^&syserr and ^&exiterror %then %do;
		data _null_;
			file jslmsg mod %if %nrbquote(&SYSENCODING) ne wlatin1 %then encoding="utf-8";;
			put "      TextBox("" ""),";
			put "      TextEditBox(""&CLIENT_LSUSER_HOME.ProxyOptions.sas"",";
			put '         <<Font Color(5), <<Set Font Size(10), <<Set Wrap(1000)),';
			put '      TextBox(" "),';
		run;
		%PackageInsertFile(Proxy options from &ProcessName, &OutPath, ProxyOptions.sas, , text/plain, yes, no);
		%let copyfiles = &copyfiles ProxyOptions.sas;
	%end;
%end;

/*
filename pfile "&SavePath.CVSOptions.jsl";

data _null;
    file pfile %if %nrbquote(&SYSENCODING) ne wlatin1 %then encoding="utf-8";;
    put "jg:CVSStarterFiltering = &CVSStarterFiltering;";
    put "jg:CVSReportStudyFiltering = &CVSReportStudyFiltering;";
    put "jg:CVSDialogReqLink = &CVSDialogReqLink;";
    put "jg:CVSShowSelParamsOnly = &CVSShowSelParamsOnly;";
    put "jg:CVSPreStudyReqSet = &CVSPreStudyReqSet;";
run;

%if ^&syserr and ^&exiterror %then %do;
    data _null_;
        file jslmsg mod %if %nrbquote(&SYSENCODING) ne wlatin1 %then encoding="utf-8";;
        put "      TextBox("" ""),";
        put "      TextEditBox(""&CLIENT_LSUSER_HOME.CVSOptions.jsl"",";
        put '         <<Font Color(5), <<Set Font Size(10), <<Set Wrap(1000)),';
        put '      TextBox(" "),';
    run;
    %PackageInsertFile(CVS options from &ProcessName, &OutPath, CVSOptions.jsl, , text/plain, yes, no);
    %let copyfiles = &copyfiles CVSOptions.jsl;
%end;

filename pfile "&SavePath.NarrativeTemplateOptions.jsl";

data _null_;
    file pfile %if %nrbquote(&SYSENCODING) ne wlatin1 %then encoding="utf-8";;	
    put "jg:NarrativeTemplatePath = ConvertFilePath(""&NarrativeTemplatePath"", absolute, windows);";
    put "jg:NarrativeTemplateDefaultName = ""&NarrativeTemplateDefaultName"";";
run;

%if ^&syserr and ^&exiterror %then %do;
    data _null_;
        file jslmsg mod %if %nrbquote(&SYSENCODING) ne wlatin1 %then encoding="utf-8";;
        put "      TextBox("" ""),";
        put "      TextEditBox(""&CLIENT_LSUSER_HOME.NarrativeTemplateOptions.jsl"",";
        put '         <<Font Color(5), <<Set Font Size(10), <<Set Wrap(1000)),';
        put '      TextBox(" "),';
    run;
    %PackageInsertFile(NarrativeTemplateOptions options from &ProcessName, &OutPath, NarrativeTemplateOptions.jsl, , text/plain, yes, no);
    %let copyfiles = &copyfiles NarrativeTemplateOptions.jsl;
%end;

filename pfile "&SavePath.IntegrationOptions.jsl";

data _null_;
    file pfile %if %nrbquote(&SYSENCODING) ne wlatin1 %then encoding="utf-8";;	
    put "Names Default To Here( 1 );";
    put "if (namespaceExists(""sddInt""),";
    put "    ns = namespace(""sddInt"");";
    put ",//else";
    put "    ns = new Namespace( ""sddInt"" );";
    put ");";
    put "sddInt:SDDEnabled = &SDDEnabled;";
    put "sddInt:SDDURL = ""&SDDURL"";";
    put "sddInt:SDDVersion = ""&SDDVersion"";";
run;

%if ^&syserr and ^&exiterror %then %do;
    data _null_;
        file jslmsg mod %if %nrbquote(&SYSENCODING) ne wlatin1 %then encoding="utf-8";;
        put "      TextBox("" ""),";
        put "      TextEditBox(""&CLIENT_LSUSER_HOME.IntegrationOptions.jsl"",";
        put '         <<Font Color(5), <<Set Font Size(10), <<Set Wrap(1000)),';
        put '      TextBox(" "),';
    run;
    %PackageInsertFile(Integration options from &ProcessName, &OutPath, IntegrationOptions.jsl, , text/plain, yes, no);
    %let copyfiles = &copyfiles IntegrationOptions.jsl;
%end;
*/

filename tfile "&SavePath.TabbedReportPrompt.jsl";

data _null_;
	file tfile %if %nrbquote(&SYSENCODING) ne wlatin1 %then encoding="utf-8";;	
	put "jg:_PromptTabbedReport_ = &PromptTabbedReport;";
run;

data _null_;
	file jslmsg mod %if %nrbquote(&SYSENCODING) ne wlatin1 %then encoding="utf-8";;
	put "      TextBox("" ""),";
	put "      TextEditBox(""&CLIENT_LSUSER_HOME.TabbedReportPrompt.jsl"",";
	put '         <<Font Color(5), <<Set Font Size(10), <<Set Wrap(1000)),';
	put '      TextBox(" "),';
run;
%PackageInsertFile(Prompt from tabbed report from &ProcessName, &OutPath, TabbedReportPrompt.jsl, , text/plain, yes, no);


/* This block of code has been made OBSOLETE by the "Configure ___ Settings" launch buttons on the Starters.
filename scfile "&SavePath.ShowCustButtons.jsl";

data _null_;
	file scfile;	
	%if %length(&ShowCustButtons) %then %do;
		put "jg:_showcustbuttons_ = &ShowCustButtons;";
	%end;
	%else %do;
		put "jg:_showcustbuttons_ = 0;";
	%end;	
run;

data _null_;
	file jslmsg mod;
	put "      TextBox("" ""),";
	put "      TextEditBox(""&SavePath.ShowCustButtons.sas"",";
	put '         <<Font Color(5), <<Set Font Size(10), <<Set Wrap(1000)),';
	put '      TextBox("Note: If the Show Starter Customization buttons option changed, you must close and reopen JMP.",<<FontColor(3)),';
	put '      //TextBox(" "),';					
run;
%PackageInsertFile(Show customization buttons from &ProcessName, &OutPath, ShowCustButtons.jsl, , text/plain, yes, no);
*/


/* Archive SAS code checkbox - put last so you can see the warning message on top */

filename afile "&SavePath.ArchiveSASCode.jsl";

data _null_;
	file afile %if %nrbquote(&SYSENCODING) ne wlatin1 %then encoding="utf-8";;	
	put "jg:_ArchiveSASCode_ = &ArchiveSASCode;";
	put "jg:_ResolveSASCode_ = &ResolveSASCode;";
run;

%if ^&syserr and ^&exiterror %then %do;
	data _null_;
		file jslmsg mod %if %nrbquote(&SYSENCODING) ne wlatin1 %then encoding="utf-8";;
		put "      TextBox("" ""),";
		put "      TextEditBox(""&CLIENT_LSUSER_HOME.ArchiveSASCode.jsl"",";
		put '         <<Font Color(5), <<Set Font Size(10), <<Set Wrap(1000)),';
		put '      TextBox(" "),';
		%if &ResolveSASCode = 1 %then %do;
			put '      TextBox(locale:get("WarningCompTimesIncrResolvedMacroCodeArchived_Text"),<<FontColor(3),<< Set Wrap(800)),';
			put '      TextBox(" "),';					
		%end;
		put "		TextBox(locale:get(""YoumustrestartJMPforanychangestotakeeffect_Text"")),";
		put	"		TextBox("" ""),";
		put "		ButtonBox(locale:get(""ExitJMP_Text""), Quit()),";
		put '   )';
		put ");";
		put 'if(hostis(windows),';
		put '  jg:setSavedSettingsDir(jg:jmpLSUserDir[1]||"MyDefaultSettings\"),';
		put '  jg:setSavedSettingsDir(jg:jmpLSUserDir[1]||"MyDefaultSettings/"),';
		put ');';
		put 'jg:settingssaveok(1, "ConfigureSettings", "MyDefault");';		
		put 'wait(0);';
		%if &Server ^= Local %then %do;
			%let i=1;
			%do %while(%length(%scan(&copyfiles,&i,' ')));
				%let file = %scan(&copyfiles,&i,' ');
				put "try(DeleteFile(""&CLIENT_LSUSER_HOME.&file""));";
				put "CopyFile(""&ClientOutPath.&file"",""&CLIENT_LSUSER_HOME.&file"");";
				%let i=%eval(&i+1);
			%end;
		%end;
		/*
		put "jg:InitCVS;";
		put "LoadCVSOptions(1);";
		*/
	run;
	%PackageInsertFile(Archive SAS code from &ProcessName, &OutPath, ArchiveSASCode.jsl, , text/plain, yes, no);
%end;
%else %do;
	data _null_;
		file jslmsg mod;	
		put ')); //end window and vlistbox';
	run;
%end;

%PackageInsertFile(SAS message from &ProcessName, &OutPath, sasmessage.jsl, , text/plain, yes);

%exit:;

%ExitSDSProcess(&OutPath, &ProcessName, &syserr, &exiterror, &clockrunning);

%mend ConfigureSettings;

* run macro;
%ConfigureSettings;


