/*
JMP Genomics Analytical Process

Name: Genomic BLUP
Author: Luciano Silva and Russ Wolfinger, SAS Institute Inc.
History:
	sbrlsi 07July16 initial coding
*/

/*
-----------------------------------------------------------
Copyright (C) 2013 SAS Institute, Inc. All rights reserved. 

Notice: 
The above copyright notice and this notice must appear in
any whole or partial copy of this code and any related
documentation. Permission to use, copy, and modify the
source code contained in this file is hereby granted, but
is limited to customers of SAS with a valid license for the
SAS software product with which this file was provided. 
Except as expressly set forth herein, the terms of such
license apply.
-----------------------------------------------------------
*/

*ProcessVariables;

%global StudyName InData ClassVar MarkerVars PredictorVars ClassPredVars WeightVar LabelVar IDVars ListMarkerVars
ListContPredVars ListClassPredVars LockMarkerVars LockContPredVars LockClassPredVars ListLockMarkerVars 
ListLockContPredVars ListLockClassPredVars WhereStmt OutPath ServerOutPath
PredWhere PredClassWhere DataStepStatements TransformationMethod 
StandardizationMethod NumArg ImputeZeros KMeans KM_Usage KM_Max KM_CorrRadius  
StatTest ST_Max ST_Method Nominalize ST_AsymmetricLossFitProp ST_QuantileLevel 
MultipleTestingMethod NSample NegLog10pCutoff AbsMeanDiffCutoff AbsPropDiffCutoff
RG_SelectMethod RG_Adaptive RG_StopCriterion RG_SLEntry RG_SLStay 
RG_CVPMethod RG_CVKFold RG_Seed RG_MaxStep RG_OrderInt RG_Poolwise RG_PoolSize RG_PoolMax 
RG_ModelAveraging RG_MA_NSamples RG_MA_Max RG_MaxRounds
FR_ImpVars FR_Inds FR_NImpVars FR_MaxDepth FR_LeafSize FR_Seed FR_MaxTrees FR_NInputs
Optimize Opt_Similarity Opt_Criterion Opt_Algorithm StandardizeRows 
TraitType EventTrait AsymmetricLossFitProp NPred 
IncludeDom RandSVD RandSVD_K RandSVD_Q RandSVD_seed RandSVD_error
HetVar Priors VarSelect OrderInt Kernel SASProc BLUPWhere
NumVar MaxIter VarFreq StopVal Seed 
BJ_Est CensorVar CensorValues CensorWeight BJ_Maxiter BJ_RMSE_Tol
AnnoInData AnnoWhereStmt ChromNum ChromLab MarkerLab MarkerPos MarkerPos_BP AnnoWhereStmt
TestInData 
MProcOption MClassVariables MFixedEffects /*MRandEffects*/ MModOption ProcMixedStatements /*MHetVar*/
HProcOption HClassVariables HFixedEffects /*HRandEffects*/ HModOption ProcHMixedStatements /*HHetVar*/
AsymmetricLossEvalProp CustomCosts
GenomicBLUPScript GenomicBLUP_OutData GenomicBLUP_OutDataVars
MacroPath;

*ProcessBody;

%macro GenomicBLUP;

	* initialization and argument checking;
	%let ProcessName = GenomicBLUP;
	%let FullAPName = %str(Genomic BLUP); 
	
	%let GBLUPError= 0;
	
	%let DepVar = &ClassVar;
	%let Mode = Automated;
	%let VarSelect = ;

	/* 
	The code used to require that chromosome variable be numeric and in ascending order. 
	The new code relax that requirement by allowing numeric or character chromosome variable
	with levels grouped, not necessarely sorted. In this code, a new numeric chromosome
	variable, TmpChromNum, is created such that if the check for grouping of chromosome 
	levels in the code below passes, this variable is in ascending order as needed. 
	In addition, a new variable, TmpAnnoInOrd, is created to make sure that the chromosome 
	and marker order in the input annotation data set is kept the same if sorting is ever
	needed in the code.
	*/
	
	%global ChromID InAnnoOrd;
	%let ChromID= &ChromNum;
	%let ChromNum= TmpChromNum;
	%let InAnnoOrd= TmpAnnoInOrd;
	
	%include "&MacroPath./UtilityMacros.sas" / nosource;

	%GetOrigFileName(InData,OrigInData);
	
	%if %length(&GenomicBLUP_OutData) %then %do;
		%let prefix = &GenomicBLUP_OutData;
		%let GenomicBLUP_OutData = ;
	%end;
	%else %let prefix = &OrigInData;

	%PrepOutDS(GenomicBLUP_OutData,GenomicBLUP_OutDataName,&prefix,_gb);
	%PrepOutDS(GenomicBLUP_OutDataVars,GenomicBLUP_OutDataVarsName,&prefix,_gbv);
	%PrepOutFile(GenomicBLUPScript,GenomicBLUPScriptName,&ProcessName Script for &prefix..jsl);	
	
	%if %symexist(CVing) %then %let CVFlag = 1;
	%else %let CVFlag = 0;
	%put CVFlag= &CVFlag;
	
	%if %length(&MarkerVars &ListMarkerVars)= 0 %then %do;
		%put ERROR: Marker variables must be specified in either the Marker Variables or List-Style Specification of Marker Variables parameters.;
		%let exiterror=1;
		%goto GBLUPExit;	
	%end;	
	
	**************************************************************;
	** Marker variables are assumed to be continuous variables*;
	**************************************************************;
	%global OrigPredictorVars OrigListContPredVars OrigLockContPredVars OrigListLockContPredVars OrigMarkerVars OrigListMarkerVars;
	%let OrigPredictorVars= &PredictorVars;
	%let OrigListContPredVars= &ListContPredVars;
	%let OrigLockContPredVars= &LockContPredVars;
	%let OrigListLockContPredVars= &ListLockContPredVars;
	%let OrigMarkerVars= &MarkerVars;
	%let OrigListMarkerVars= &ListMarkerVars;
	
	%let PredictorVars= &OrigPredictorVars &MarkerVars;
	%let ListContPredVars= &OrigListContPredVars &ListMarkerVars;
	%let LockContPredVars= &OrigLockContPredVars &LockMarkerVars;
	%let ListLockContPredVars= &OrigListLockContPredVars &ListLockMarkerVars;
	/*
	%let PredictorVars= &MarkerVars;
	%let ClassPredVars= ;
	%let ListContPredVars= &ListMarkerVars;
	%let ListClassPredVars= ;
	%let LockContPredVars= &LockMarkerVars;
	%let LockClassPredVars= ;
	%let ListLockContPredVars= &ListLockMarkerVars;
	%let ListLockClassPredVars= ;
	*/	
	%let PredClassWhere= ;
	%let TransformationMethod= ;
	%let Nominalize= No;
	%let FR_Inds= No;
	%let RG_OrderInt= 1;
	%let StandardizeRows= No; /*Row-wise standardization takes place after marker reduction, so it has no usage in Genomic BLUP*/
	%let BLUPWhere= ;
	%let NPred= 0;
	%let Kernel= Dot;
	%let OrderInt= 1;
	%*let BJ_Est= No;
	%*let CensorVar=;
	%*let CensorValues=;
	%*let CensorWeight=;
	%*let BJ_Maxiter=;
	%*let BJ_RMSE_Tol=;	
	
	%if &Kernel ^= Dot and &SASProc = HPMIXED %then %do;
		%put ERROR: The Kernel Function &Kernel is not available for the SAS Procedure HPMIXED. You must use the Dot Kernel Function when using PROC HPMIXED. Please, select the Dot Kernel Function in the Analysis tab.;
		%let exiterror=1;
		%goto GBLUPExit;
	%end;	
	
	%let OrigInDataFullPath= &InData;
	%let OrigTestInDataFullPath= &TestInData;
	%if %length(&AnnoInData) and ^&CVFlag %then %do;
		
		%PathName(&AnnoInData);
		%if &exiterror %then %do;
			%put ERROR: Annotation data set (map information) specification is invalid.;
			%let exiterror=1;
			%goto GBLUPExit;
		%end;
	
		%let AnnoData = &TmpName;
		libname AnnoLib "&TmpPath";	
		
		%*goto skipanno;
		
		**when annotation data set is avalaible, check its properties**;
		%global VType;
		
		*** 1. Deal with map data set - AnnoLib.&AnnoData ***;
		%if %symexist(_dsmap_) %then %do;
			%let _rc_ = %sysfunc(close(&_dsmap_));
		%end;		
		
		data nmapfile;
			set AnnoLib.&AnnoData;
			&InAnnoOrd= _n_;
			by &ChromID notsorted;
			if _n_= 1 then _count_= 0;
			if first.&ChromID and not missing(&ChromID) then do;
				_count_= _count_ + 1;
			end;
			else do;
				_count_= _count_;
			end;
			retain _count_;
			if not missing(&ChromID) then &ChromNum=_count_;
			else &ChromNum= .;
			drop _count_;
		run;
		
		%global MapFile;
		%let MapFile = nmapfile;

		%let ListMapVars = &ChromNum &ChromLab &MarkerLab &MarkerPos &MarkerPos_BP;
		%if &ChromNum = &ChromLab %then %let ChromLab =;
		** Test the type of chromNum etc. *;
		%VarAttributes(_Ds=&MapFile,_Var=&ChromNum,_Len=,_Type=VType);
		%if &VType ^= N %then %do;
			%put ERROR: Variable (&ChromNum) is not a numeric variable.;
			%let exiterror=1;
			%goto GBLUPExit;
		%end;
		%if %length(&ChromLab)^=0 %then %do;
			%VarAttributes(_Ds=&MapFile,_Var=&ChromLab,_Len=,_Type=ChLab_Type);
		%end;
		%VarAttributes(_Ds=&MapFile,_Var=&MarkerLab,_Len=,_Type=VType);
		%if &VType ^= C %then %do;
			%put ERROR: Variable (&MarkerLab) is not a character variable.;
			%let exiterror=1;
			%goto GBLUPExit;
		%end;
		%VarAttributes(_Ds=&MapFile,_Var=&MarkerPos,_Len=,_Type=VType);
		%if &VType ^= N %then %do;
			%put ERROR: Variable (&MarkerPos) is not a numeric variable.;
			%let exiterror=1;
			%goto GBLUPExit;
		%end;
		%if %length(&MarkerPos_BP)^=0 %then %do;
			%VarAttributes(_Ds=&MapFile,_Var=&MarkerPos_BP,_Len=,_Type=MkBP_Type);
		%end;

		*checking if there are markers with missing chromosome ID*;	
		data miss_chr;
			set &MapFile;
			where missing(&ChromNum);
		run;

		%NObsVars(miss_chr);
		%if &nobs %then %do;
			%if &nobs = 1 %then %do;
				%put ERROR: There is &nobs marker with missing chromosome ID.;
			%end;
			%else %do;
				%put ERROR: There are &nobs markers with missing chromosome ID.;
			%end;
			%let exiterror= 1;
			%goto GBLUPExit;
		%end;

		*checking if there are markers with missing position*;		
		data miss_pos;
			set &MapFile;
			where missing(&MarkerPos);
		run;

		%NObsVars(miss_pos);
		%if &nobs %then %do;
			%if &nobs = 1 %then %do;
				%put ERROR: There is &nobs marker with missing position.;
			%end;
			%else %do;
				%put ERROR: There are &nobs markers with missing position.;
			%end;
			%let exiterror= 1;
			%goto GBLUPExit;
		%end;

		* Check if levels of ChromID variable are grouped properly*;	
		proc sort data=&MapFile out=Tmp_Map;
			by &ChromID;
		run;
		
		%let FlagChromOutOfOrder= 0;
		data _null_;
			set Tmp_Map;
			by &ChromID;
			_lagord_= lag(&InAnnoOrd);
			if first.&ChromID then _diff_= 0;
			else _diff_= &InAnnoOrd - _lagord_;
			if _diff_ > 1 then do;
				_out_of_order_= 1;
				call symputx("FlagChromOutOfOrder", 1);
			end;
		run;
		%put FlagChromOutOfOrder= &FlagChromOutOfOrder;
		
		proc datasets nolist;
			delete Tmp_Map;
		quit;
		
		%if &FlagChromOutOfOrder %then %do;
			%put ERROR: &ChromID variable is not grouped properly in the annotation data set (map information). The levels of the &&ChromID variable must be grouped, however, they do not need to be sorted.;
			%let exiterror= 1;
			%goto GBLUPExit;
		%end;
	
		* Check if MarkerPos is in ascending order *;	
		%let FlagMarkerOutOfOrder= 0;
		data _null_;
			set &MapFile;
			by &ChromID notsorted;
			_lagpos_= lag(&MarkerPos);
			if first.&ChromID then _diff_= 0;
			else _diff_= &MarkerPos - _lagpos_;
			if _diff_ < 0 then do;
				_out_of_order_= 1;
				call symputx("FlagMarkerOutOfOrder", 1);
			end;
		run;
		%put FlagMarkerOutOfOrder= &FlagMarkerOutOfOrder;
		
		%if &FlagMarkerOutOfOrder %then %do;
			%put ERROR: &MarkerPos variable is not sorted properly in the annotation data set (map information). &MarkerPos must be in ascending order within each level of the &ChromID variable.;
			%let exiterror= 1;
			%goto GBLUPExit;
		%end;

		* Check ChromNum, MarkerPos is in ascending order or not *;	
		%let _tmp_ = 1;
		data _null_;
			set &MapFile;
			if (&ChromNum*1000000000+&MarkerPos/100)<(lag(&ChromNum)*1000000000+lag(&MarkerPos)/100) then do;
				call symput('_tmp_',"-1");
			end;
		run;
		%if &_tmp_ = -1 %then %do;
			%put ERROR: Either &ChromNum or &MarkerPos variables are not sorted properly in the annotation data set (map information). The &&ChromNum variable must be in ascending order, and &MarkerPos must be in ascending order within each level of the &ChromNum variable.;
			%let exiterror=1;
			%goto GBLUPExit;
		%end;
		
		*** 3. Combine map (MapFile) and genotype (InData) information *;
		* create libname for input data set;
		%PathName(&InData); 
		%let InDataG = %trim(&TmpName);
		libname InLibG "&TmpPath";
		%if &exiterror %then %do;
			%put ERROR: Input dataset specification is invalid.;
			%let exiterror=1;
			%goto GBLUPExit;
		%end;
			
		data MkInFile;
			set InLibG.&InDataG;
			where 0;
			keep &MarkerVars &ListMarkerVars &LockMarkerVars &ListLockMarkerVars;
		run;		
		%let _dsix_in = %sysfunc(open(MkInFile));
		%let nm_in = %sysfunc(attrn(&_dsix_in,NVARS));
		
		data _null_;
			set &MapFile end=last;
			call symputx("annomrk"||strip(_n_),&MarkerLab);
			if last then call symputx("nanno",_n_);
		run;

		%let allmatch=1;
		%put nm_in= &nm_in;
		%put nanno= &nanno;

		%*let exiterror=1;
		%*goto GBLUPExit;

		%if &nanno < &nm_in %then %do;
			%let allmatch=0;
			%let allmatch_string= There are markers in the Input SAS Data Set that are not present in the Annotation SAS Data Set.;
			%put WARNING: &allmatch_string;
			%goto subset;
		%end;

		%do i=1 %to &nm_in; 
			%let varj= %sysfunc(varname(&_dsix_in,&i));

			%if %upcase(&&annomrk&i) ne %upcase(&varj) %then %do;
				%let allmatch=0;
				%let allmatch_string= For Variable &i, Marker Variable is &Varj and Marker Label in Annotation is &&annomrk&i..;
				%put WARNING: &allmatch_string;
				%goto subset;
			%end;
		%end; 	

		%subset:;
		%if &allmatch=0 %then %do;  

			%let AlleleVars= ;
			%do i=1 %to &nm_in; 
				%let AlleleVars= &AlleleVars %sysfunc(varname(&_dsix_in,&i));
			%end;	
			
			/*S1350463: user error handling when use non marker variable as lock- in marker variables*/
			%let ErrorString1= %str(The markers in the input data set do not match the annotation data set according to the variable selected as Marker Label Variable.);
			%let ErrorString2= %str(To fix this error: Firstly, please check if variables listed in the Marker Variables or List-Style Specification of Marker Variables (General Tab), as well as in the);
			%let ErrorString3= %str(Lock-In Marker Variables or List-Style Specification of Lock-In Variables (Predictor Reduction Tab) are all marker variables.);
			%let ErrorString4= %str(If the error persists, please check that the Marker Label (Annotation Tab) exactly matches the input data columns or run the Subset and Reorder Genetic Data AP to order these data sets correctly.);

			%put ERROR: &ErrorString1 &ErrorString2 &ErrorString3 &ErrorString4;
			%put &allmatch_string;

			* create JSL file;
			%global SRButtonsScript;
			%PrepOutFile(SRButtonsScript,SRButtonsScriptName,launchAP_buttons_SR.jsl);
			filename jslbtns "&SRButtonsScript";

			options noquotelenmax;
			data _null_;
				file jslbtns lrecl=32767;
				put "//!";
				put "Names Default to Here(1);";
				put "wait(0.5);";
				put "if(host is(Windows),";
				put "	include( ConvertFilePath(""$LS_HOME/Generator/DialogHelperFunctions.jsl"", absolute, windows) ),";
				put "	include( ConvertFilePath(""$LS_HOME/Generator/DialogHelperFunctions.jsl"", absolute, POSIX) ));";
				put "	";
				put "srbuttonswindow=New Window(""Launch AP to Reorder Markers"",";
				put "		VListBox(";
				put "			TextBox(""Click on the button below to launch the Subset and Reorder Genetic Data process to reorder marker columns properly.""),";
				put "			TextBox(""Use the output data sets from that process as the new input and annotation data sets to .""),";

				put "			TextBox("" ""),";
				put "			ButtonBox(""Subset and Reorder Genetic Data"",";
				put "				if(host is(Windows),";
				put "					Include(ConvertFilePath(""$HOME/JMPG/ProcessLibrary/SubsetandReorderGeneticData.jsl"", absolute, windows)),";
				put "					Include( ConvertFilePath(""$HOME/JMPG/ProcessLibrary/SubsetandReorderGeneticData.jsl"", absolute, POSIX) ));";
				put "	";

				* Set input data set and Output Folder *;
				put "				jg:setDefaultParmValues(2, ""SubsetandReorderGeneticData"");";				
				put "				jg:setCurrentParmValue(2, ""SubsetandReorderGeneticData"", jg:getParmNum(""SubsetandReorderGeneticData"",""InData""),""%sysfunc(pathname(InLibG))\&InDataG..sas7bdat"");";
				put "				jg:SubsetandReorderGeneticData_updateDeps;";
				put "				jg:setCurrentParmValue(2, ""SubsetandReorderGeneticData"", jg:getParmNum(""SubsetandReorderGeneticData"",""ListMarkerVars""),""&AlleleVars"");";	
				put "				jg:setCurrentParmValue(2, ""SubsetandReorderGeneticData"", jg:getParmNum(""SubsetandReorderGeneticData"",""MarkerFmt""),""Numeric Genotypes"");";
				put "				jg:setCurrentParmValue(2, ""SubsetandReorderGeneticData"", jg:getParmNum(""SubsetandReorderGeneticData"",""MinPercent""),""0"");";		
				put "				jg:setCurrentParmValue(2, ""SubsetandReorderGeneticData"", jg:getParmNum(""SubsetandReorderGeneticData"",""OutPath""),""&ClientOutPath"");";					
				put "				jg:setCurrentParmValue(2, ""SubsetandReorderGeneticData"", jg:getParmNum(""SubsetandReorderGeneticData"",""AnnoInData""),""%sysfunc(pathname(AnnoLib))\&AnnoData..sas7bdat"");";					
				put "				jg:SubsetandReorderGeneticData_updateDeps;";				
				put "				jg:setCurrentParmValue(2, ""SubsetandReorderGeneticData"", jg:getParmNum(""SubsetandReorderGeneticData"",""AnnoMName""),""&MarkerLab"");";	
				put "			)";;

				put "		)";
				put ");";
			run;

			%PackageInsertFile(jsl code from &ProcessName, &OutPath, &SRButtonsScriptName, , text/plain, yes);		
			%let exiterror=1;
			%goto GBLUPExit;
		%end;

		**Removing markers by the AnnoWhereStmt**;
		%if %length(&AnnoWhereStmt) %then %do;	
			
			%if %length(&LockMarkerVars &ListLockMarkerVars) %then %do;
				proc transpose data=&MapFile out=AllMapFileT(drop = _name_ where=(0));
					var &MarkerLab;
					id &MarkerLab;
				run;
			
				data LockMarkersData;
					set AllMapFileT;
					keep &LockMarkerVars &ListLockMarkerVars;
				run;	
				
				proc transpose data=LockMarkersData out=LockMarkersDataT(rename=(_name_= _lockedmarker_));
					var _all_;
				run;
				
				proc sql UNDO_POLICY=NONE;
					create table MapFileJoin as 
					select *
					from &MapFile as a left join LockMarkersDataT as b
					on a.&MarkerLab = b._lockedmarker_
					;
				quit;
				
				data &MapFile; 
					set MapFileJoin;
					where &AnnoWhereStmt or not missing(_lockedmarker_);
					drop _lockedmarker_;
				run;
				
			%end;	
			%else %do;
				data &MapFile; 
					set &MapFile;
					where &AnnoWhereStmt;
				run;
			%end;

			/*data OutLib.p_MapFile; set &MapFile; run;*/

			%NObsVars(&MapFile);
			%if &nobs = 0 %then %do;
				%put ERROR: The statement entered in the Filter to Include Markers option from the Annotation tab removed all markers from the analysis. Please, review the statement.;
				%let exiterror= 1;
				%goto GBLUPExit;
			%end;

			proc transpose data=&MapFile out=SubMapFileT(drop = _name_ where=(0));
				var &MarkerLab;
				id &MarkerLab;
			run;
			
			%let _dsix_annoin = %sysfunc(open(SubMapFileT));
			%let nm_annoin = %sysfunc(attrn(&_dsix_annoin,NVARS));

			data 
				SubMarkersOnly(keep = %do i=1 %to &nm_annoin; %sysfunc(varname(&_dsix_annoin,&i)) %end;)
				NonMarkersOnly(drop = %do i=1 %to &nm_in; %sysfunc(varname(&_dsix_in,&i)) %end;)
				;
				set InLibG.&InDataG;
			run;

			data SubInData;
				 merge NonMarkersOnly SubMarkersOnly;
			run;
			
			%let WorkPath= %sysfunc(pathname(work));	
			%put WorkPath= &WorkPath;
			%put InDataOrig= &InData;
						
			%let InData= %str(&WorkPath.\SubInData.sas7bdat);
			%put InDataNew= &InData;		
			
			%if %length(&TestInData) %then %do;

				*data VarNamesSubInData;
				*	set SubInData;
				*	where 0;
				*run;
				
				%*let _dsix_subin = %sysfunc(open(SubInData));
				%*let nv_subin = %sysfunc(attrn(&_dsix_subin,NVARS));			
			
				%PathName(&TestInData);
				libname TInLibG "&TmpPath";
				%let TestInDataG = %trim(&TmpName);
				data SubTestMrkOnly(keep = %do i=1 %to &nm_annoin; %sysfunc(varname(&_dsix_annoin,&i)) %end;)
					SubTestNonMrkOnly(drop = /*%do i=1 %to &nm_in; %sysfunc(varname(&_dsix_in,&i)) %end;*/
							&MarkerVars &ListMarkerVars &LockMarkerVars &ListLockMarkerVars)
					;
					set TInLibG.&TestInDataG;
					/*
					keep 
						%do i=1 %to &nv_subin;
							%sysfunc(varname(&_dsix_subin,&i))
						%end;
					;
					*/
				run;
				%*let _dsix_subin = %sysfunc(close(&_dsix_subin));
				
				data SubTestInData;
					 merge SubTestNonMrkOnly SubTestMrkOnly;
				run;				

				%let TestInData= %str(&WorkPath.\SubTestInData.sas7bdat);
			%end;
			
			**S1346551: Fixing issue when using filter from the annotation data set and using list-style marker specification**;
			%let MarkerVars= ;
			%let ListMarkerVars= %sysfunc(varname(&_dsix_annoin,1))--%sysfunc(varname(&_dsix_annoin,&nm_annoin));			
			%let PredictorVars= &OrigPredictorVars &MarkerVars;
			%let ListContPredVars= &OrigListContPredVars &ListMarkerVars; 
			%let _dsix_annoin = %sysfunc(close(&_dsix_annoin));
		%end;
		
		%let _dsix_in = %sysfunc(close(&_dsix_in));
		%skipanno:;
	%end;
	
	%PathName(&InData); 
	%let trainpath=&TmpPath;
	%let InDataTmp = %trim(&TmpName);
	libname InLibTmp "&TmpPath";
	
	%if %length(&IDVars) %then %do;
		/*
		S1349962: adding a check to verify that IDVars uniquely define each row in the input data set.
		If the input data has a column termed DataType, then exclude all rows whose entry is Test. 
		This is needed because when iterating between GBLUP and Progeny Simulation, the testing set 
		is stacked in button of the input data set, and the test data set does not necessarely have IDVars.
		*/
		
		%put IDVars= &IDVars;
		
		%let _dsg_ = %sysfunc(open(InLibTmp.&InDataTmp));
		%let _datatype_= %sysfunc(varnum(&_dsg_,DataType));
		%let _rcg_ = %sysfunc(close(&_dsg_));
		
		data InDataIDVars;
			set InLibTmp.&InDataTmp(keep= &IDVars %if &_datatype_ %then DataType;);
			/*
			where not missing (%scan(&IDVars,1,' '))
				%let i=2;
				%do %while(%length(%scan(&IDVars,&i,' ')));
					and not missing (%scan(&IDVars,&i,' '))
					%let i=%eval(&i+1);
				%end;
			;		
			*/
			%if &_datatype_ %then where DataType ne "Test";;
		run;	
		
		%NObsVars(InDataIDVars);		
		%let nobs0 = &nobs;

		proc sort data=InDataIDVars nodupkey;
			by &IDVars;
		run;
		
		%NObsVars(InDataIDVars);		
		%if &nobs ne &nobs0 %then %do;
			%put ERROR: ID Variables must uniquely define each row in the input data set.  Use different variables or remove them altogether.;
			%let exiterror=1;
			%goto GBLUPExit;
		%end;	
	%end;
	
	data MkInFile;
		set InLibTmp.&InDataTmp;
		where 0;
		keep &MarkerVars &ListMarkerVars &LockMarkerVars &ListLockMarkerVars;
	run;	
	
	proc transpose data=MkInFile out= MkInFileT;
		var _all_;
	run;
	
	data MkInFileT;
		set MkInFileT;
		_ord_= _n_;
	run;
	
	/*data OutLib.gb_MkInFileT; set MkInFileT; run;*/
	
	%*let exiterror=1;
	%*goto GBLUPExit;

	%global ProcOption ClassVariables FixedEffects RandEffects ModOption ProcStatements /*HetVar*/;
	%if &SASProc = MIXED %then %do;
		%let ProcOption= &MProcOption; 
		%let ClassVariables= &MClassVariables; 
		%let FixedEffects= &MFixedEffects; 
		%*let RandEffects= &MRandEffects; 
		%let ModOption= &MModOption; 
		%let ProcStatements= &ProcMixedStatements;
		%*let HetVar= &MHetVar;
	%end;
	%else %if &SASProc = HPMIXED %then %do;
		%let ProcOption= &HProcOption; 
		%let ClassVariables= &HClassVariables; 
		%let FixedEffects= &HFixedEffects; 
		%*let RandEffects= &HRandEffects; 
		%let ModOption= &HModOption; 
		%let ProcStatements= &ProcHMixedStatements;
		%*let HetVar= &HHetVar; 
	%end;
	%else %do;
		%put ERROR: Invalid SAS PROC.;
		%let exiterror=1;
		%goto GBLUPExit;		
	%end;
	
	%put RandSVD= &RandSVD;
	%put RandSVD_error= &RandSVD_error;
	%global RandSVDFlag RandSVD_errorFlag;
	%if %upcase(&RandSVD) = NO %then %do; 
		%let RandSVDFlag= 0;
		%let RandSVD_errorFlag= 0;
		*Must set these variables to numeric values to avoid error in the IML code within the PredMacros.sas when not computing Randomized SVD*;
		%let RandSVD_K= 0; 
		%let RandSVD_Q= 0;
		%let RandSVD_seed= 0;
	%end;
	%else %do;
		%let RandSVDFlag= 1;
		%if %upcase(&RandSVD_error) = NO %then %do;
			%let RandSVD_errorFlag= 0;
		%end;
		%else %do;
			%let RandSVD_errorFlag= 1;		
		%end;
	%end;
	%put RandSVDFlag= &RandSVDFlag;
	%put RandSVD_errorFlag= &RandSVD_errorFlag;

	%if &KMeans= Yes %then %do;
		%let KM_Usage= Select One Representative Predictor from Each Cluster;
	%end;
	
	%if &Kernel = Gaussian %then %let spkernel = gau;
	%else %if &Kernel = Exponential %then %let spkernel = exp;
	%else %if &Kernel = Spherical %then %let spkernel = sph;
	%else %if &Kernel = Matern %then %let spkernel = matern;
	%else %if &Kernel = Linear %then %let spkernel = lin;
	%else %let spkernel = &Kernel;

	%if ^%length(&OrderInt) %then %let OrderInt = 1;
	%if ^%length(&NPred) %then %let NPred = 0;
	
	*These flags will be used to recover the unstandardized input data set prior to *;
	*entering the genomic blup analysis when marker variables is standardized using *;
	*the standadization methods in Predictor Reduction tab*;
	%global Flag_RecoverInData Flag_RecoverTestData;
	%let Flag_RecoverInData= 0;
	%let Flag_RecoverTestData= 0;
	
	%*let exiterror=1;
	%*goto GBLUPExit;
	
	* call PredictiveAnalysis macro;
	%include "&MacroPath./PredMacros.sas" / nosource;	
	
	%PredictiveAnalysis(
		GenomicBLUP_OutData, GenomicBLUP_OutDataName,
		GenomicBLUP_OutDataVars, GenomicBLUP_OutDataVarsName,
		GenomicBLUPScript, GenomicBLUPScriptName
	);
	
	%GBLUPExit:;
	%if &exiterror %then %do;
		%global OutName;
		%let OutName = &prefix;
		%ExitSDSProcess(&OutPath, &ProcessName, &syserr, &exiterror, &clockrunning);
	%end;

%mend GenomicBLUP;

%GenomicBLUP;
