/*
JMP Life Sciences Analytical Process

Name: PredictiveModelingReview
Author: Russ Wolfinger, SAS Institute Inc.
History:
   sasrdw 07Nov13 initial coding  

Description: 
	This process sets up a predictive modeling review.
*/

/*
-----------------------------------------------------------
Copyright (C) 2013 SAS Institute, Inc. All rights reserved. 

Notice: 
The above copyright notice and this notice must appear in
any whole or partial copy of this code and any related
documentation. Permission to use, copy, and modify the
source code contained in this file is hereby granted, but
is limited to customers of SAS with a valid license for the
SAS software product with which this file was provided. 
Except as expressly set forth herein, the terms of such
license apply.
-----------------------------------------------------------
*/

*ProcessVariables;

%global StudyName InData ClassVar PredictorVars ClassPredVars WeightVar LabelVar 
Priors CustomPriors CustomCosts AsymmetricLossEvalProp 
ListContPredVars ListClassPredVars LockContPredVars LockClassPredVars ListLockContPredVars ListLockClassPredVars
CurrentReview IndividualModelFits
DoCVMC CVMethod GroupName BlockName HOSize KFold LOut HOPct HONum CVSeed 
TestInPath FileFilter TestInDatas TS_ChartByTestSet TS_ChartByModel
DoLCMC LC_InnerNum LC_NMin LC_NMax LC_OuterNum LC_InnerLoop LC_TestSet LC_FixedTestSet LC_GroupName LC_Seed
LC_CVMethod LC_BlockName LC_HOSize LC_KFold LC_LOut LC_HOPct LC_HONum
PredictiveModelingReviewScript OutPath MacroPath;

*ProcessBody;

%macro PredictiveModelingReview;

* initialization and argument checking;
%let ProcessName = PredictiveModelingReview;
%let FullAPName = %str(Predictive Modeling Review);

%include "&MacroPath./UtilityMacros.sas" / nosource;

%if ^%length(%trim(&PredictorVars &ClassPredVars &ListContPredVars &ListClassPredVars 
	&LockContPredVars &LockClassPredVars &ListLockContPredVars &ListLockClassPredVars)) %then %do;
	%put ERROR: No predictor variables specified.;
	%let exiterror = 1;
	%goto exit;
%end;

%if (%upcase(&IndividualModelFits) = NO) and (%upcase(&DoCVMC)= NO and %upcase(&DoLCMC= NO)) %then %do;
	/*S1489241: When option to Compute Individual Model Fits is unchecked and neither CVMC nor 
	LCMC is checked there is nothing to be run in the workflow. Therefore, an error message of 
	no setting is issued. Updating the code to issue an error message when Individual Model Fits 
	is unchecked and neither CVMC nor LCMC is checked.*/
	%put ERROR: When the Compute Individual Model Fits is unchecked, either Cross Validation Model Comparison or Learning Curvers Model Comparison option must be checked.; 
	%let exiterror = 1;
	%goto exit;
%end;

%CheckInData;

/*
%if %length(&TestInDatas) %then %do;
	%let TestInDatas = %Search_and_Replace(string=&TestInDatas,search=.sas7bdat,replace=);
%end;
*/

* create JSL file;
%PrepOutFile(PredictiveModelingReviewScript, PMRScriptName,&ProcessName Script for &InData..jsl);
%global _mainjslfile_;
%let _mainjslfile_ = &PredictiveModelingReviewScript;

data _null_;
	file "&PredictiveModelingReviewScript" %if %nrbquote(&SYSENCODING) ne wlatin1 %then encoding="utf-8";;
	put "//!";
	put "Here:FullAPName = ""&FullAPName"";";
	put "Here:StudyName = ""&StudyName"";";
	put "reviewlist = jg:reviewlist;";
	put "raplist = jg:raplist;";
	put "raclist = jg:raclist;";
	put "revfolder = jg:revfolder;";
	put "rcb = jg:rcb;";
	put "lsindex = 1;";
	put "apname = ""&ProcessName"";";
	put "nrap = NItems(jg:raplist);";
	put "_slash_ = ""\"";";
	put "outpath = ""&ClientOutPath"";";
	put "jg:ReviewClass = ""PredictiveModeling"";";
	put "jg:_rv3study_ = jg:PredictiveModelingReview_StudyName;";
	%if %length(&StudyName) %then %do; 
		put "jg:PMRSettingPath = ""&ClientOutPath.PredictiveModelingReview__&StudyName..sas"";";
	%end;
	%else %do;
		put "jg:PMRSettingPath = ""&ClientOutPath.PredictiveModelingReview__&InData..sas"";";	
	%end;
	put "indata = ""&InData"";";
	put "classvar = ""&ClassVar"";";
	put "priors = ""&Priors"";";
	put "custompriors = ""&CustomPriors"";";
	put "customcosts = ""&CustomCosts"";";
	put "asymmetriclossevalprop = ""&AsymmetricLossEvalProp"";";
	put "individualmodelfits = ""&IndividualModelFits"";";
	put "docvmc = ""&DoCVMC"";";
	%if (&DoCVMC = Yes) %then %do;
		put "cvmethod = ""&CVMethod"";";
		put "cv_groupname= ""&GroupName"";";
		put "cv_blockname= ""&BlockName"";";
		put "cv_hosize = ""&HOSize"";";
		put "cv_kfold = ""&KFold"";";
		put "cv_lout = ""&LOut"";";
		put "cv_hopct = ""&HOPct"";";
		put "cv_honum = ""&HONum"";";
		put "cv_seed = ""&CVSeed"";";
	%end;
	put "testinpath = ""&TestInPath"";";
	put "testindatas = ""&TestInDatas"";";
	put "ts_chartbytestset = ""&TS_ChartByTestSet"";";
	put "ts_chartbymodel = ""&TS_ChartByModel"";";
	put "dolcmc = ""&DoLCMC"";";
	%if (&DoLCMC = Yes) %then %do;
		put "lc_innernum = ""&LC_InnerNum"";";
		put "lc_nmin = ""&LC_NMin"";";
		put "lc_nmax = ""&LC_NMax"";";
		put "lc_outernum = ""&LC_OuterNum"";";
		put "lc_innerloop = ""&LC_InnerLoop"";";
		put "lc_testset = ""&LC_TestSet"";";
		put "lc_fixedtestset = ""&LC_FixedTestSet"";";
		put "lc_groupname = ""&LC_GroupName"";";
		%if %length(%trim(&GroupName)) %then %do;
			put "cv_groupname = ""&GroupName"";";
		%end;
		%else %do;
			put "cv_groupname = ""&LC_GroupName"";";
		%end;
		%if %length(%trim(&BlockName)) %then %do;
			put "cv_blockname = ""&BlockName"";";
		%end;
		%else %do;
			put "cv_blockname = ""&LC_BlockName"";";
		%end;
		put "lc_seed = ""&LC_Seed"";";
		put "lc_cvmethod = ""&LC_CVMethod"";";
		put "lc_hosize = ""&LC_HOSize"";";
		put "lc_kfold = ""&LC_KFold"";";
		put "lc_lout = ""&LC_LOut"";";
		put "lc_hopct = ""&LC_HOPct"";";
		put "lc_honum = ""&LC_HONum"";";
	%end;	
	put "jg:RunReview;";
	put "// throw to prevent blank tabbed report";
	put "throw();";
run;

%PackageInsertFile(SAS message from &ProcessName, &OutPath, &PMRScriptName, , text/plain, yes);

%exit:;

%ExitSDSProcess(&OutPath, &ProcessName, &syserr, &exiterror, &clockrunning);


%mend PredictiveModelingReview;

%PredictiveModelingReview;
