/*
JMP Life Sciences Analytical Process

Name: TestSetModelComparison
Author: Russ Wolfinger, SAS Institute Inc.

Description: 
	This process compares a set of models using cross validation.
*/

/*
-----------------------------------------------------------
Copyright (C) 2015 SAS Institute, Inc. All rights reserved. 

Notice: 
The above copyright notice and this notice must appear in
any whole or partial copy of this code and any related
documentation. Permission to use, copy, and modify the
source code contained in this file is hereby granted, but
is limited to customers of SAS with a valid license for the
SAS software product with which this file was provided. 
Except as expressly set forth herein, the terms of such
license apply.
-----------------------------------------------------------
*/

*ProcessVariables;

%global StudyName InPath SettingsFilter Workflow TS_InPath FileFilter TS_Names
Priors CustomPriors CustomCosts AsymmetricLossEvalProp 
TS_ChartByTestSet TS_ChartByModel TS_ListAll TS_OutName TSMCScript OutPath MacroPath;

*ProcessBody;

%macro TestSetModelComparison;

* initialization and argument checking;
%let ProcessName = TestSetModelComparison;
%let FullAPName = %str(Test Set Model Comparison);

%include "&MacroPath./UtilityMacros.sas" / nosource;

* create JSL file;
%PrepOutFile(TSMCScript,TSMCScriptName,&ProcessName Setup Script.jsl);
%global _mainjslfile_;
%let _mainjslfile_ = &TSMCScript;

data _null_;
	file "&TSMCScript" %if %nrbquote(&SYSENCODING) ne wlatin1 %then encoding="utf-8";;
	put "//!";
	put "include(ConvertFilePath(""$LS_HOME"", absolute, POSIX)||""JSLFiles/Model Comparison Functions.jsl"");";
	put "fullapname = ""&FullAPName"";";	
	put "apname = ""&ProcessName"";";
	put "inpath = ""&InPath"";";
	put "workflow = words(""&Workflow"",""|"");";
	put "testinpath = ""&TS_InPath"";";
	put "testindatas = words(""&TS_Names"",""|"");";
	put "outpath = ""&ClientOutPath"";";
	put "priors = ""&Priors"";";
	put "custompriors = ""&CustomPriors"";";
	put "customcosts = ""&CustomCosts"";";
	put "asymmetriclossevalprop = ""&AsymmetricLossEvalProp"";";
	put "chartbytestset = ""&TS_ChartByTestSet"";";
	put "chartbymodel = ""&TS_ChartByModel"";";	
	put "listall = ""&TS_ListAll"";";
	put "outname = ""&TS_OutName"";";
	put "_RunTSMC;";
	put "// throw to prevent blank tabbed report";
	put "throw();";
run;

%PackageInsertFile(SAS message from &ProcessName, &OutPath, &TSMCScriptName, , text/plain, yes);

%exit:;

%ExitSDSProcess(&OutPath, &ProcessName, &syserr, &exiterror, &clockrunning);


%mend TestSetModelComparison;

%TestSetModelComparison;
