/*
Central JMP Life Sciences Macros used for the following Predictive Modeling Analytical Processes: 

All Data Types:
	Distance Scoring
	GLM Select
	Partial Least Squares
	Partition Trees
	Quant Select
	Radial Basis Machine
	Ridge Regession
	
Binary Data:
	Discriminant Analysis
	K Nearest Neighbors
	Logistic Regression

Survival Data:	
	Life Regression
	Proportional Hazards Regression (formerly Survival Predictive Modeling)
	
Molecular Marker Data:
	Genomic BLUP
	Genomic Bayesian Regression (Add-In) 
*/

%macro PredictiveAnalysis(_outdata,_outdataname,_outdatavars,_outdatavarsname,_jslfile,_jslfilename);

%if %symexist(exiterror) %then %do;
	%if &exiterror %then %goto exit;
%end;
%let exiterror = 0;
%*ClockStart(PredictiveAnalysis);
* create libname for input data set;
%PathName(&InData); 
%let trainpath=&TmpPath;
%let InData = %trim(&TmpName);
libname InLib "&TmpPath";
%if &exiterror %then %do;
	%put ERROR: Input dataset specification is invalid.;
	%goto exit;
%end;

%if %symexist(CVing) %then %let CVFlag = 1;
%else %let CVFlag = 0;
%*put CVFlag= &CVFlag;

* check for incompatible options;
%if %symexist(Mode) %then %do;
	%if &Mode = Interactive and %index(&VarSelect,Genetic) %then %do;
   	%put ERROR: Genetic Algorithm is not available in &ProcessName Interactive mode.;
		%let exiterror = 1;
	%end;
	%if &CVFlag %then %do;
		%if &Mode = Interactive %then %do;
			%put ERROR: Cross validation is not available in &ProcessName Interactive mode.;
			%let exiterror = 1;
		%end;
	%end;
%end;	

%if &ProcessName = DiscriminantAnalysis %then %do;
	%if &Metric = Regularized and &Mode = Automated %then %do;
		%put ERROR: Regularized metric is available only in &ProcessName Interactive mode.;
		%let exiterror = 1;
	%end;
%end;

%if ((&ProcessName = PartitionTrees) or (&ProcessName = VirtualTwins)) and (&Priors = Equal) %then %do; 
	%if (&Mode = Interactive) %then %do;
		%put ERROR: Equal priors are not available in Interactive Mode for Partition Trees.;
		%let exiterror = 1;
		%goto exit;
	%end;
	%if (&Server ^= Local) %then %do;
		%put ERROR: Equal priors are not available in client-server mode for Partition Trees.;
		%let exiterror = 1;
		%goto exit;		
	%end;
	
%end;

%if &exiterror %then %goto exit;

%*ClockStart(Initializing);

/*
proc catalog cat=work.sasmacr;
	contents;
run;
*/
%global ClassVars OutName SelVars;

%if %symexist(prefix) and %length(&prefix) %then %let OutName = &prefix;
%else %let OutName = &InData;

%if ^%symexist(ImputeZeros) %then %let ImputeZeros = No;
%if ^%symexist(KMeans) %then %let KMeans = No;
%if ^%symexist(StatTest) %then %let StatTest = No;
%if ^%symexist(RG_SelectMethod) %then %let RG_SelectMethod = None;
%if ^%symexist(TraitType) %then %let TraitType = ;
%if ^%symexist(NumDepAsClass) %then %let NumDepAsClass = Yes;
%if ^%symexist(ClassVars) %then %let ClassVars = ;
%if ^%symexist(ContinuousVars) %then %let ContinuousVars = ;
%if ^%symexist(GenerateHTML) %then %let GenerateHTML = No;
%if ^%symexist(TreeMethod) %then %let TreeMethod = ;
%if ^%symexist(FR_ImpVars) %then %let FR_ImpVars = No;
%if ^%symexist(FR_Inds) %then %let FR_Inds = No;
%if ^%symexist(FR_NImpVars) %then %let FR_NImpVars = ;
%if ^%symexist(FR_Impfit) %then %let FR_Impfit = No;
%if ^%symexist(FR_MaxDepth) %then %let FR_MaxDepth = ;
%if ^%symexist(FR_LeafSize) %then %let FR_LeafSize = ;
%if ^%symexist(FR_Seed) %then %let FR_Seed = ;
%if ^%symexist(FR_MaxTrees) %then %let FR_MaxTrees = ;
%if ^%symexist(FR_NInputs) %then %let FR_NInputs = ;
%if ^%symexist(FR_NInputs) %then %let FR_NInputs = ;
%if ^%symexist(Optimize) %then %let Optimize = No;
%if ^%symexist(LambdaMode) %then %let LambdaMode = ;
%if ^%symexist(ST_AsymmetricLossFitProp) %then %let ST_AsymmetricLossFitProp = 0.5;
%if ^%symexist(AsymmetricLossFitProp) %then %let AsymmetricLossFitProp = 0.5;
%if ^%symexist(AsymmetricLossEvalProp) %then %let AsymmetricLossEvalProp = 0.5;
%if ^%symexist(CustomPriors) %then %let CustomPriors = ;
%if ^%symexist(CustomCosts) %then %let CustomCosts = ;
%if ^%symexist(cv_custompriors) %then %let cv_custompriors = ;
%if ^%symexist(cv_customcosts) %then %let cv_customcosts = ;
%if ^%symexist(cv_alossprop) %then %let cv_alossprop = 0.5;
%if ^%symexist(SuccessCutoff) %then %let SuccessCutoff = 0.5;
%if ^%symexist(YReflection) %then %let YReflection = No;
%if ^%symexist(WeightVar) %then %let WeightVar = ;
%if ^%symexist(Kernel) %then %let Kernel = Dot;
%if ^%symexist(StandardizeRows) %then %let StandardizeRows = No;
%if ^%symexist(BJ_Est) %then %let BJ_Est = No;
%if ^%symexist(CensorVar) %then %let CensorVar = ;
%if ^%symexist(IDVar) %then %let IDVar = ;
%if ^%symexist(OrderInt) %then %let OrderInt = 1;
%if ^%symexist(DataStepStatements) %then %let DataStepStatements = ;
%if ^%symexist(RG_ModelAveraging) %then %let RG_ModelAveraging = No;
%if ^%symexist(FixedEffects) %then %let FixedEffects = ;
%if ^%symexist(FlagCharVarMissing) %then %let FlagCharVarMissing= 0;

* override custom priors and costs if they are specified elsewhere (e.g. CVMC);
%if %length(&cv_custompriors) %then %let CustomPriors = &cv_custompriors;
%if %length(&cv_customcosts) %then %let CustomCosts = &cv_customcosts;
%if &cv_alossprop ne 0.5 %then %let AsymmetricLossEvalProp = &cv_alossprop;
%if ^%length(%trim(&ST_AsymmetricLossFitProp)) %then %let ST_AsymmetricLossFitProp = 0.5;
%if ^%length(%trim(&AsymmetricLossFitProp)) %then %let AsymmetricLossFitProp = 0.5;
%if ^%length(%trim(&AsymmetricLossEvalProp)) %then %let AsymmetricLossEvalProp = 0.5;

ods html close;
options sysprint="";

* options for cross-validation;
%if &CVFlag %then %do;
	%let indat = &CVTrainInData;
	%if ^%length(&indat) %then %let indat = InLib.&InData;
	%let intestdat = &CVTestInData;
	%if %length(&&mdlname&WorkFlowrp) > 23 %then %let mdltrunc = %substr(&&mdlname&WorkFlowrp,1,23);
	%else %let mdltrunc = &&mdlname&WorkFlowrp;
	%let outdat = CV_&mdltrunc._preds;
	%let outdatvars = CV_&mdltrunc._vars;
	%CheckDataName(_name=outdat);
	%CheckDataName(_name=outdatvars);
	%if &ProcessName = SurvivalPredictiveModeling %then %do;
		%let outdataspm1 = CV_&mdltrunc._spm1;
		%let outdataspm2 = CV_&mdltrunc._spm2;
		%let outdataspm3 = CV_&mdltrunc._spm3;
	%end;
	options nonotes;
	
	/*
	*Luciano: Added for testing purpose.*;
	%if ^%symexist(cvc) %then %do;
		%global cvc;
		%let cvc= 1;
	%end;
	%else %do;
		%let cvc= %sysevalf(&cvc + 1);
	%end;
	%put cvc= &cvc;
	
	data OutLib.train&cvc; 
		set &indat; 
		drop 
			&PredictorVars &ClassPredVars &ListContPredVars &ListClassPredVars 
			&LockContPredVars &LockClassPredVars &ListLockContPredVars &ListLockClassPredVars
			%if &ProcessName= GenomicBLUP or &ProcessName = GenomicBayesianRegression %then &MarkerVars &ListMarkerVars &LockMarkerVars &ListLockMarkerVars;
		;
		_train= 1;
	run;
	
	data OutLib.test&cvc; 
		set &intestdat; 
		drop 
			&PredictorVars &ClassPredVars &ListContPredVars &ListClassPredVars 
			&LockContPredVars &LockClassPredVars &ListLockContPredVars &ListLockClassPredVars
			%if &ProcessName= GenomicBLUP or &ProcessName = GenomicBayesianRegression %then &MarkerVars &ListMarkerVars &LockMarkerVars &ListLockMarkerVars;
		;
		_train= 0;
	run;
	
	*data OutLib.cvdat&cvc;
	*	set train&cvc test&cvc;
	*run;
	*/
%end;
%else %do;
	%let indat = InLib.&InData;
	%let intestdat = d;
	%let outdat = &&&_OutData;
	%let outdatvars = &&&_OutDataVars;
	%include "&MacroPath./CrossValidationModelComparisonMacros.sas" / nosource;
	%include "&MacroPath./PartitionTreesMacros.sas" / nosource;
	%include "&MacroPath./BinaryResponseEffectSelectionMacros.sas" / nosource;
	options notes;
%end;
%include "&MacroPath./SurvivalMacros.sas" / nosource;
%put indat=&indat;
%put intestdat=&intestdat;
%put outdat=&outdat;
%put outdatvars=&outdatvars;

%*ClockStop(Initializing);
%*ClockStart(Translating);

/* Definitions for translated strings */

%let trAccuracy =						%nrbquote(%LocaleGet(Accuracy_Text));
%let trAnalysisSettings =	 			%nrbquote(%LocaleGet(AnalysisSettings_Text));
%let trAreaUnderROCCurve =				%nrbquote(%LocaleGet(AreaUnderROCCurve_Text));
%let trAveSquareErrorLift = 			%nrbquote(%LocaleGet(AveSquareErrorLift_Text));
%let trBestXfromForest =				%nrbquote(%LocaleGet(BestXfromForest_Text));
%let trClassPredictorIncludeFilter = 	%nrbquote(%LocaleGet(ClassPredictorIncludeFilter_Text));
%let trComputingDistancestoX =			%nrbquote(%LocaleGet(ComputingDistancestoX_Text));
%let trContPredictorIncludeFilter = 	%nrbquote(%LocaleGet(ContinuousPredictorIncludeFilter_Text));
%let trCorrelation =					%nrbquote(%LocaleGet(Correlation_Text));
%let trCustomCosts =					%nrbquote(%LocaleGet(CustomCosts_Text));
%let trDevianceLift = 					%nrbquote(%LocaleGet(DevianceLift_Text));
%let trDistanceMetric =					%nrbquote(%LocaleGet(DistanceMetric_Text));
%let trFinalSelectedVariables = 		%nrbquote(%LocaleGet(FinalSelectedVariables_Text));
%let trFisherExactTest =				%nrbquote(%LocaleGet(FisherExactTest_Text));
%let trForest =						%nrbquote(%LocaleGet(Forest_Text));
%let trHarrellCStatistic =				%nrbquote(%LocaleGet(HarrellCStatistic_Text));
%let trHarrellsCStatistic =				%nrbquote(%LocaleGet(HarrellsCStatistic_Text));
%let trKernel =							%nrbquote(%LocaleGet(Kernel_Text));
%let trKMeans =							%nrbquote(%LocaleGet(KMeans_Text));
%let trMatthewsCorrelationCoeff =		%nrbquote(%LocaleGet(MatthewsCorrelationCoefficient_Text));
%let trMaxStep =						%nrbquote(%LocaleGet(MaxStep_Text));
%let trMeanAbsoluteError =				%nrbquote(%LocaleGet(MeanAbsoluteError_Text));
%let trMeanDifferenceCutoff = 			%nrbquote(%LocaleGet(MeanDifferenceCutoff_Text));
%let trMetric =							%nrbquote(%LocaleGet(Metric_Text));
%let trModelSelectionMethod =			%nrbquote(%LocaleGet(ModelSelectionMethod_Text));
%let trMultipleTestingMethod = 			%nrbquote(%LocaleGet(MultipleTestingMethod_Text));
%let trnPredictors =					%nrbquote(%LocaleGet(nPredictors_Text));
%let trNegativePredictiveValue =		%nrbquote(%LocaleGet(NegativePredictiveValue_Text));
%let trNeglog10pvalueCutoff =			%nrbquote(%LocaleGet(Neglog10pvalueCutoff_Text));
%let trNone =							%nrbquote(%LocaleGet(None_Text));
%let trNumberofNearestNeighbors =		%nrbquote(%LocaleGet(NumberofNearestNeighbors_Text));
%let trNumberofPLSComponents =			%nrbquote(%LocaleGet(NumberofPLSComponents_Text));
%let trOptimization =					%nrbquote(%LocaleGet(Optimization_Text));
%let trOptimizedContinuousPredictors = 	%nrbquote(%LocaleGet(OptimizedContinuousPredictors_Text));
%let trplusXnotshownseeX =				%nrbquote(%LocaleGet(plusXnotshownseeX_Text));
%let trPositivePredictiveValue =		%nrbquote(%LocaleGet(PositivePredictiveValue_Text));
%let trPredictorReductionSettings = 	%nrbquote(%LocaleGet(PredictorReductionSettings_Text));
%let trPredictorTransformation =	 	%nrbquote(%LocaleGet(PredictorTransformation_Text));
%let trPredictorStandardization =	 	%nrbquote(%LocaleGet(PredictorStandardization_Text));
%let trPriors =							%nrbquote(%LocaleGet(Priors_Text));
%let trProportionDifferenceCutoff = 	%nrbquote(%LocaleGet(ProportionDifferenceCutoff_Text));
%let trReferenceSurvivalTime =		 	%nrbquote(%LocaleGet(ReferenceSurvivalTime_Text));
%let trRootMeanSquareError =			%nrbquote(%LocaleGet(RootMeanSquareError_Text));
%let trseeXfor_IDefinition =			%nrbquote(%LocaleGet(seeXfor_IDefinition_Text));
%let trSensitivity =					%nrbquote(%LocaleGet(Sensitivity_Text));
%let trSLEntry =						%nrbquote(%LocaleGet(SLEntry_Text));
%let trSLStay =							%nrbquote(%LocaleGet(SLStay_Text));
%let trSpecificity =					%nrbquote(%LocaleGet(Specificity_Text));
%let trStatFilter =					 	%nrbquote(%LocaleGet(StatFilter_Text));
%let trStopCriterion =					%nrbquote(%LocaleGet(StopCriterion_Text));
%let trSummarizationMethod =			%nrbquote(%LocaleGet(SummarizationMethod_Text));
%let trTestSetCriteria = 				%nrbquote(%LocaleGet(TestSetCriteria_Text));
%let trTopXBLUPs =						%nrbquote(%LocaleGet(TopXBLUPs_Text));
%let trTopXPredictors =					%nrbquote(%LocaleGet(TopXPredictors_Text));
%let trTopXPredictorsfromX =			%nrbquote(%LocaleGet(TopXPredictorsfromX_Text));
%let trTotalAccuracy =					%nrbquote(%LocaleGet(TotalAccuracy_Text));
%let trTrainingSetCriteria = 			%nrbquote(%LocaleGet(TrainingSetCriteria_Text));
%let trTreeModelType =					%nrbquote(%LocaleGet(TreeModelType_Text));
%let trVariableSelectionMethod =		%nrbquote(%LocaleGet(VariableSelectionMethod_Text));
%let trWeight =							%nrbquote(%LocaleGet(Weight_Text));
%let trXandX = 							%nrbquote(%LocaleGet(XandX_Text));
%let trYoudenIndex =						%nrbquote(%LocaleGet(YoudenIndex_Text));
%let trZeroOneIndex =					%nrbquote(%LocaleGet(ZeroOneIndex_Text));	

/* End of definitions for translated strings */

%*ClockStop(Translating);
%*ClockStart(Survival);

/*
S1403219: When the same predictor as continuous and class, new variables are created for each level of the class 
predictor, which is then used in the analysis. However, the original class variables names were still used to 
produce the stack data, and because the original variables are no longer in the data set used to build the stack data, 
the original variables will be created with missing on all entries. The code below generates a warning message, when 
the same variable is specified as continuous and class. In the future, we can turn this warning message into an error
message. 
*/
%if %length(%trim(%left(&ClassPredVars &ListClassPredVars &LockClassPredVars &ListLockClassPredVars))) 
	and %length(%trim(%left(&PredictorVars &ListContPredVars &ListLockContPredVars &LockContPredVars))) %then %do;
	
	data _classpred_;
		set &indat;
		where 0;
		keep &ClassPredVars &ListClassPredVars &LockClassPredVars &ListLockClassPredVars;
	run;
	
	proc transpose data= _classpred_ out=_classpredt_;
		var &ClassPredVars &ListClassPredVars &LockClassPredVars &ListLockClassPredVars;
	run;
	
	/*data OutLib._classpredt_; set _classpredt_; run;*/
	
	data _contpred_;
		set &indat;
		where 0;
		keep &PredictorVars &ListContPredVars &ListLockContPredVars &LockContPredVars;
	run;	
	
	proc transpose data= _contpred_ out=_contpredt_;
		var &PredictorVars &ListContPredVars &ListLockContPredVars &LockContPredVars;
	run;
	
	/*data OutLib._contpredt_; set _contpredt_; run;*/
	
	proc sql;
		create table _class_cont_ as
		select * 
		from _classpredt_ as l, _contpredt_ as r
		where l._name_=r._name_;
	quit;
	
	%NObsVars(_class_cont_);
	%if &nobs > 0 %then %do;
		%put WARNING: There are predictor variables specified as both continuous and class. Predictors must be specified as either continuous or class.;
		%*let exiterror= 1;
		%*goto exit;
	%end;
	
%end;

data d;
	set &indat;
	_train_ = 1;
run;
%let indat = d;
%put indat = &indat;

%if &BJ_Est = Yes and not %length(&CensorVar) %then %do;
	%put ERROR: You must specify a Censor Variable when using Buckley-James estimation.;
	%let exiterror = 1;
	%goto exit;
%end;

%if (&ProcessName= GenomicBLUP or &ProcessName = GenomicBayesianRegression) and ^&CVFlag %then %do;
	*counting number of markers in the input data set*;
	data mrkname;
		set &indat;
		where 0;
		keep &MarkerVars &ListMarkerVars &LockMarkerVars &ListLockMarkerVars;
	run;
	%nObsVars(mrkname);
	%let nmrkin= &nvars;
%end;

* Handling censor variable for survival data;
%if %length(&CensorVar) %then %do;
	data _null_;
		set InLib.&InData;
		call symputx("cvtype",vtype(&CensorVar));
		stop;
	run;
	%put cvtype = &cvtype;

	%if ^%length(%trim(&CensorValues)) %then %do;
		%if &cvtype=C %then %let CensorValues = "1";
		%else %let CensorValues = 1;
	%end;
	%else %if &cvtype=C %then %do;
		%let char1 = %bquote(%substr(&CensorValues,1,1));
		%put char1 = &char1;
		%if %bquote(&char1) ^= %str(%") %then %do;
			%put ERROR: Values for a character Censor Variable must be enclosed in double quotes.;
			%let exiterror = 1;
			%goto exit;
		%end;
	%end;

	/*
	* check to make sure at least one censoring event is found;
	%let foundone = 0;
	data _null_;
		set InLib.&InData;
		if &CensorVar in(%unquote(&CensorValues)) then do;
			call symputx('foundone',1);
			stop;
		end;
	run;

	%if ^&foundone %then %do;
		%put ERROR: The censoring variable &CensorVar has no values matching &CensorValues..;
		%let exiterror = 1;
		%goto exit;
	%end;
	*/

	* check for character type, create numeric if necessary;
	%let CensorVar_old = &CensorVar;
	%let CensorValues_old = &CensorValues;
	%if &cvtype=C %then %do;
		data d;
			set d; 
			if &CensorVar in(%unquote(&CensorValues)) then _Censor1 = 1;
			else _Censor1 = 0;
			_RowNum_ = _n_;
		run;
		%put NOTE: Numeric variable _Censor1 has been created to replace character variable &CensorVar..;
		%let indat = d;
		%let CensorVar = _Censor1;
		%let CensorValues = 1;
	%end;
	%else %do;
		data d;
			set &indat;
			_RowNum_ = _n_;
		run;
	%end;
	%if ^%symexist(ColorVar) %then %let ColorVar = &CensorVar;
	%else %if ^%length(&ColorVar) %then %let ColorVar = &CensorVar;
	%if ^%symexist(CensorWeight) %then %let CensorWeight = 1;
	%if &CensorWeight ne 1 %then %do;
		%let WeightVar0 = &WeightVar;
		%if %length(&WeightVar0) %then %do;
			%let WeightVar = _CombinedWeight;
		%end;
		%else %do;
			%let WeightVar = _CensoredWeight;
		%end;
		data d;
			set d;
			%if %length(&WeightVar0) %then %do;
				if &CensorVar in (%unquote(&CensorValues)) then do;
					&WeightVar = &WeightVar0 * &CensorWeight;
				end;
				else do;
					&WeightVar = &WeightVar0;
				end;
			%end;
			%else %do;
				if &CensorVar in (%unquote(&CensorValues)) then do;
					&WeightVar = &CensorWeight;
				end;
				else do;
					&WeightVar = 1;
				end;			
			%end;
		run;
	%end;
%end;


%if &ProcessName = SurvivalPredictiveModeling or &ProcessName = LifeRegression %then %do;

	%if &ProcessName = SurvivalPredictiveModeling %then %do;
		%let trait_type = survival; * parameter for CVMC;
	%end;
	%else %do;
		%let trait_type = quantitative; * parameter for CVMC;	
	%end;
	%let TraitType = Continuous;

	%if ^%length(&TestInData) & ^&CVFlag %then %let intestdat = ;

	%if ^%length(&IDVar) %then %do;
		%let IDVar = _RowNum_;
	%end;

	%if ^%length(&CensorVar) %then %do;
		data d;
			set &indat;
			%if &IDVar=_RowNum_ %then %do;
				_RowNum_ = _n_;
			%end;
		run;
	%end;	

	data _C_;
		set InLib.&InData(keep=&DepVar &WeightVar);
		if (&DepVar ne .) %if %length(&WeightVar) %then & (&WeightVar ne .); ;
	run;

	* check if the reference time is within the range of time to event variable;
	data _null_;
		set _C_  end=_e_;
  		retain _min_ _max_; * check range of time variable;
  		if _n_ = 1 then do;
  			_min_ = &DepVar;
  			_Max_ = _min_;
  		end;
  		else do;
  			if &DepVar < _min_ then _min_ = &DepVar;
  			else if &DepVar > _max_ then _max_ = &DepVar;
  		end;
  		if _e_ then do;
  			call symput('MinTime',_min_);
  			call symput('MaxTime',_max_);
  		end;
	run;
	
	%put The Minimum time = &MinTime;
	%put The Maximum time = &MaxTime;
	
  	%if %length(&RefTime) %then %do;
  		%let i = 1;  		
		%do %while(%length(%scan(&RefTime,&i," ")));
			%let RefTime&i = %scan(&RefTime,&i," ");
			%if &CVFlag %then %do;
				%global cv_RefTime_m&Workflowrp._r&i;
				%let cv_RefTime_m&Workflowrp._r&i = &&RefTime&i;
			%end;
  			%if (&&RefTime&i < &MinTime) | (&&RefTime&i > &MaxTime) %then %do;
  				%put ERROR: The specified Reference Time for Comparing Survival Curves, &&RefTime&i, is not within the range of the Time to Event Variable, &DepVar..;
  				%put ERROR: Please specify a reference time within %trim(%left(&MinTime)) and %trim(%left(&MaxTime)).; 
  				%let exiterror=1;
 			%end; 			
			%let i = %eval(&i+1);
  		%end;
  		%let nRefTime = %eval(&i-1);
		%if &CVFlag %then %do;
			%global cv_nRefTime_m&Workflowrp;
			%let cv_nRefTime_m&Workflowrp = &nRefTime;
		%end;
	%end;
	%if &exiterror %then %goto exit;  
	
  	%if %length(&MinRefTime) %then %do;
  		%if (&MinRefTime < &MinTime) %then %do;
  			%put ERROR: The specified Minimum Time for Area under Survival Curves, &MinRefTime, is not within the range of Time to Event Variable, &DepVar..;
  			%put ERROR: Please specify a reference time within %trim(%left(&MinTime)) and %trim(%left(&MaxTime)).; 
  			%let exiterror=1;
 		%end; 	
	%end;
	%else %do;
		%if %length(&MaxRefTime) %then %do;
			%let MinRefTime = &MinTime;
		%end;
	%end;
	%if &exiterror %then %goto exit; 
	
  	%if %length(&MaxRefTime) %then %do;
  		%if (&MaxRefTime > &MaxTime) %then %do;
  			%put ERROR: The specified Maximum Time for Area under Survival Curves, &MaxRefTime, is not within the range of Time to Event Variable, &DepVar..;
  			%put ERROR: Please specify a reference time within %trim(%left(&MinTime)) and %trim(%left(&MaxTime)).;  
  			%let exiterror=1;
 		%end; 	
	%end;
	%else %do;
		%if %length(&MinRefTime) %then %do;
			%let MaxRefTime = &MaxTime;
		%end;
	%end;
	%if &exiterror %then %goto exit; 
	
	%if &CVFlag %then %do;
		%global cv_MinRefTime_m&Workflowrp cv_MaxRefTime_m&Workflowrp;
		%let cv_MinRefTime_m&Workflowrp = &MinRefTime;
		%let cv_MaxRefTime_m&Workflowrp = &MaxRefTime;
	%end;

	%nObsVars(_C_);
	%let nAllObs_sTrain = &nObs;
	%if &nOBS < 4 %then %do;
		%let exiterror = 1;
		%if &nObs = 1 %then %do;
			%let _v = is;
			%let _n = observation;
		%end;
		%else %do;
			%let _v = are;
			%let _n = observations;
		%end;
		%put ERROR: There &_v only &nObs valid &_n in the input data set. Please check the input data set and provide more valid data.;
		%goto exit;
	%end;
%end;

%*ClockStop(Survival);
%*ClockStart(Symexist);

%if %length(&WeightVar) %then %do;
	data d;
		set d;
		if missing(&WeightVar) or &WeightVar <= 0 then delete;
	run;
	%NObsVars(d);
	%if ^&nobs %then %do;
		%put ERROR: No valid observations due either to missing or nonpositive weight values.;
		%let exiterror = 1;
		%goto exit;
	%end;
%end;

%if %index(%quote(&ST_Method),Shrunken) %then %do;
	%include "&MacroPath./MixedModelMacros.sas" / nosource;
%end;
quit;

libname OutLib "&OutPath";
/*
%sysfunc(ifc(%sysfunc(libref(OutLib)),libname OutLib "&OutPath",));
*/

%if %symexist(NewPredictorVars) %then %do;
	%let PredictorVars = &NewPredictorVars;
%end;
%if %symexist(NewClassPredVars) %then %do;
	%let ClassPredVars = &NewClassPredVars;
%end;
%if %symexist(NewListContPredVars) %then %do;
	%let ListContPredVars = &NewListContPredVars;
%end;
%if %symexist(NewListClassPredVars) %then %do;
	%let ListClassPredVars = &NewListClassPredVars;
%end;
%if ^%symexist(LockContPredVars) %then %let LockContPredVars = ;
%if ^%symexist(LockClassPredVars) %then %let LockClassPredVars = ;
%if ^%symexist(ListLockContPredVars) %then %let ListLockContPredVars = ;
%if ^%symexist(ListLockClassPredVars) %then %let ListLockClassPredVars = ;

%if (%length(&LockContPredVars) or %length(&LockClassPredVars) or 
	%length(&ListLockContPredVars) or %length(&ListLockClassPredVars)) and %index(&VarSelect,Genetic) %then %do;
	%put ERROR: Genetic Algorithm is not available with lock-in predictor variables.;
	%let exiterror = 1;
	%goto exit;
%end;

%if %symexist(DropPredVars) %then %do;
	%if %length(%trim(&DropPredVars)) %then %do;
		%let i = 1;
		%do %while(%length(%scan(&DropPredVars,&i)));
			%let dpv = %scan(&DropPredVars,&i);
			%if %index(&PredictorVars,&dpv) %then %let PredictorVars = %RemoveWord(&PredictorVars,&dpv);
			%if %index(&ClassPredVars,&dpv) %then %let ClassPredVars = %RemoveWord(&ClassPredVars,&dpv);
			%if &exiterror %then %goto exit;
			%let i = %eval(&i+1);
		%end;
		data &indat;
			set &indat;
			drop &DropPredVars;
		run;
	%end;
%end;

/*
%if ^%length(%trim(&PredictorVars &ListContPredVars &ClassPredVars &ListClassPredVars)) %then %do;
	%put ERROR: Predictor variables must be specified.;
	%let exiterror = 1;
	%goto exit;
%end;
*/

%*ClockStop(Symexist);
%*ClockStart(DepCheck);

data ind1;
   set &indat;
	if _n_ > 1 then stop;
	keep &PredictorVars &ListContPredVars &ClassPredVars &ListClassPredVars
		&LockContPredVars &LockClassPredVars &ListLockContPredVars &ListLockClassPredVars
	;
run;

* check for p_ predictor variables;
%let _tid_  = %sysfunc(open(ind1));
%let nv = %sysfunc(attrn(&_tid_,NVARS));
%let badvars = ;
%do i=1 %to &nv;
	%let v = %sysfunc(varname(&_tid_,&i));
	%*put i=&i v=&v;
	%if %length(&v) > 1 %then %do;
		%if %qupcase(%substr(&v,1,2)) = %quote(P_) %then %do;
			%if %length(&badvars) < 32735 %then %do;
				%let badvars = &badvars &v;
			%end;
		%end;
	%end;
%end;
%if %length(%trim(&badvars)) %then %do;
	%put ERROR: Please rename the following predictor variables: &badvars..  The suffix P_ is reserved for variables created as predictions.;
	%let exiterror = 1;
	%goto exit;
%end;
%let _tid_ 	= %sysfunc(close(&_tid_));


* check dependent variable;
%put DepVar = &DepVar;
%let _tid_  = %sysfunc(open(ind1));
%let _dnum_ = %sysfunc(varnum(&_tid_,&DepVar));
%let _tid_ 	= %sysfunc(close(&_tid_));
%if &_dnum_ %then %do;
	%put Dropping &DepVar from predictor variables;
	data ind1;
		set ind1;
		drop &DepVar;
	run;
 	%if %index(&PredictorVars,&DepVar) %then %let PredictorVars = %RemoveWord(&PredictorVars,&DepVar);
 	%if %index(&ClassPredVars,&DepVar) %then %let ClassPredVars = %RemoveWord(&ClassPredVars,&DepVar);
  	%if %index(&LockContPredVars,&DepVar) %then %let LockContPredVars = %RemoveWord(&LockContPredVars,&DepVar);
  	%if %index(&LockClassPredVars,&DepVar) %then %let LockClassPredVars = %RemoveWord(&LockClassPredVars,&DepVar);
	/*
	%let exiterror = 1;
	%put ERROR: The dependent variable &DepVar must not be included as a predictor variable.;
   %goto exit;
   */
%end;

%global _dtype_;
%let _tid_  = %sysfunc(open(&indat));
%let _dnum_ = %sysfunc(varnum(&_tid_,&DepVar));
%let _dtype_ = %sysfunc(vartype(&_tid_,&_dnum_));
%if %symexist(ColorVar) %then %do; 
	%if %length(&ColorVar) %then %do;
		%let _cnum_ = %sysfunc(varnum(&_tid_,&ColorVar));
		%let _ctype_ = %sysfunc(vartype(&_tid_,&_cnum_));
	%end;
%end;
%else %do;
	%let ColorVar = ;
%end;
%let _tid_ = %sysfunc(close(&_tid_));
%put _dtype_ = &_dtype_;

%if (&TraitType = Continuous) and (&_dtype_ = C) %then %do;
	%put ERROR: Continuous variable type is incompatible with character dependent variable &DepVar..;
	%put You may want to create a numeric version of &DepVar in the input data set.;
	%let exiterror = 1;
	%goto exit;
%end;

* assign default trait type;
%if ^%length(&TraitType) %then %do;
	%if &_dtype_ = C %then %do;
		%CheckBinaryChar(&indat,&DepVar);
	%end;
	%else %do;
		%CheckBinary(&indat,&DepVar);
	%end;
	%let nv = &nvalue;
	%put nvalue = &nv;
	%if &nv <= 1 %then %do;
		%if &nv = 0 %then %put ERROR: All values of &DepVar are missing.  A predictive model cannot be fit.;
		%else %put ERROR: Only one nonmissing value of &DepVar detected. A predictive model cannot be fit.;
		%let exiterror = 1;
		%goto exit;
	%end;
	* character and numeric variables with two levels are considered binary by default;
	%if &nv = 2 %then %let TraitType = Binary;
	%else %if (&_dtype_ = C) or (&ProcessName = DiscriminantAnalysis) or (&ProcessName = KNearestNeighbors) or 
		(&ProcessName = LogisticRegression) %then %let TraitType = Nominal;
	%else %let TraitType = Continuous;
%end;
%put TraitType = &TraitType;

%if (&TraitType=Continuous) or (&TraitType=Count) %then %do;
	%let NumDepAsClass = No;
%end;
%else %do;
	%let NumDepAsClass = Yes;
	%let ClassVars = &DepVar;
%end;

%*ClockStop(DepCheck);
%*ClockStart(Truth);

/* Options for Proc Glimmix, wait until SAS 9.2
%if (&ProcessName = RadialBasisMachine) %then %do;
	%if &TraitType=Binary %then %do;
		%if %length(&EventTrait)>0 %then %let DepOption = %str((event="&EventTrait"));
		%else %let DepOption = %str((event=LAST));
	%end;
	%else %if &TraitType=Nominal %then %do;
		%if %length(&RefTrait)>0 %then %let DepOption = %str((ref="&RefTrait"));
		%else %let DepOption = %str((ref=FIRST));
	%end;   	
	%else %let DepOption = ;
	%put DepOption = &DepOption;

	%let ModOption = ;
	%if &TraitType=Binary %then %let ModOption=%str(/ link=logit dist=binary);
	%else %if &TraitType=Nominal %then %let ModOption=%str(/ link=glogit dist=mult);
	%else %if &TraitType=Ordinal %then %let ModOption=%str(/ link=clogit dist=mult);
	%else %if &TraitType=Count %then %let ModOption=%str(/ link=log dist=p);

	%if ^%length(&ConvOption) %then %let ConvOption = %str(maxiter=50 absgconv=0.001); 
	%put ModOption = &ModOption;
	%put ConvOption = &ConvOption;

	%if (&TraitType=Binary) or (&TraitType=Nominal) or (&TraitType=Ordinal) %then %let GlimmixClassVars = &DepVar;
	%else %let GlimmixClassVars = ;
%end;
*/

* valid data set;
%let ValidPath = ;
%if ^%symexist(ValidInData) or &CVFlag %then %do;
	%let ValidPath = ;
	%let ValidInData = ;
%end;
%if %length(&ValidInData) %then %do;
	%PathName(&ValidInData);
	%let ValidPath=&TmpPath;
	%let ValidInData = %trim(&TmpName);
%end;

* test data set;
%let TestPath = ;
%if ^%symexist(TestInData) or &CVFlag %then %do;
	%let TestPath = ;
	%let TestInData = ;
%end;
%if %length(&TestInData) %then %do;
	%PathName(&TestInData);
	libname TInLib "&TmpPath";
	%let TestPath=&TmpPath;
	%let TestInData = %trim(&TmpName);
	%let TestSpec = Yes;
	data d_test;
		set TInLib.&TestInData end=_e_;
		_Obs = _n_;
		if _e_ then call symput('nAllObs_sTest',_n_);
	run;
	%let intestdat = d_test; 
	%put intestdat = &intestdat;

	* check if the first 5 predictors in Input Dataset exist in the Test Dataset;
	/*
	%let _iid_  = %sysfunc(open(ind1));
	%let _tVars_ = %sysfunc(attrn(&_iid_,NVARS));
	%if &_tVars_ > 5 %then %let _tVars_ = 5;
	%checkvarexist(
		%do i = 1 %to &_tVars_;
			%sysfunc(VARNAME(&_iid_,&i)) 
		%end;
		%let _iid_ 	= %sysfunc(close(&_iid_)); 
		,
		TInLib.&TestInData,
		The predictors %if %symexist(Covariates) %then (or covariates);,
		_ShowError=1,
		_DsLabel= Test Data Set
	);
	%if &exiterror %then %goto exit;
	*/

	* check if test has the dependent variable;
	%let _tid_  = %sysfunc(open(d_test));
	%let _dnum_ = %sysfunc(varnum(&_tid_,&DepVar));

	* check the existence of &CensorVar for test data;
	%if %symexist(CensorVar) %then %do;
		%if %length(&CensorVar) %then %do; 
			%let _cnum_ = %sysfunc(varnum(&_tid_,&CensorVar));
			%if ^&_cnum_ %then %do;	
				%let _tid_ 	= %sysfunc(close(&_tid_));
				%put ERROR: Censor Variable, &CensorVar, is not in the Test Data Set.  Please check the specified Test Data Set.; 
				%let exiterror = 1;
				%goto exit;
			%end;
		%end;
		%if %length(&ColorVar) %then %do;
			%let _cnum_ = %sysfunc(varnum(&_tid_,&ColorVar));
			%if ^&_cnum_ %then %do;	
				%let _tid_ 	= %sysfunc(close(&_tid_));
				%put ERROR: Group Variable, &ColorVar, is not in the Test Data Set.  Please check the specified Test Data Set.; 
				%let exiterror = 1;
				%goto exit;
			%end;
		%end;
	%end;

	%let _tid_ 	= %sysfunc(close(&_tid_));
	%put _dnum_ = &_dnum_;
	%put DepVar = &DepVar;
	* if so, create a truth variable and set dependent variable to have missing values; 
	%if &_dnum_ %then %do;
		%if %length(&DepVar) > 19 %then %let dv = %substr(&DepVar,1,19);
		%else %let dv = &DepVar;
		%let TruthVar = &dv._true;
		%put TruthVar = &TruthVar;
		data d_test;
			set d_test;
			&TruthVar = &DepVar;
			%if &_dtype_ = C %then %do;
				&DepVar = " ";
			%end;
			%else %do;
				&DepVar = .;
			%end;
			_train_ = 0; 
			%if %symexist(CensorVar) %then %do;	* for survival data;
				%if %length(&CensorVar) %then %do;
					%if &cvtype=C %then %do;
						if &CensorVar in (%unquote(&CensorValues)) then _Censor1 = 1;
						else _Censor1 = 0; 
						%let CensorVar = _Censor1;
					%end;
				%end;
				%if &IDVar=_RowNum_ %then %do;
					_RowNum_ = _n_;
				%end;
			%end; 
		run;

		data &indat;
			set &indat;
			&TruthVar = &DepVar;
		run;
	%end;
	%else %do;
		%let TruthVar = &DepVar;

		* Check &CensorVar for SMP;
		%if %symexist(CensorVar) %then %do;	
			%if %length(&CensorVar) %then %do;
				%if &cvtype=C %then %do;
					data d_test;
						set d_test;	
						if &CensorVar in (%unquote(&CensorValues)) then _Censor1 = 1;
						else _Censor1 = 0; 
					run; 
					%let CensorVar = _Censor1;
				%end;
			%end;
		%end;
	%end;
%end;
%else %do;
	%let TestSpec = No;
	%let TruthVar = &DepVar;
%end;

%*ClockStop(Truth);
%*ClockStart(Vars);

%if &ProcessName = SurvivalPredictiveModeling or &ProcessName = LifeRegression %then %do;
	%*put indat= &indat;
	%*put intestdat = &intestdat;
	* keep ColorVar short set to be applied later (S0969267);
	%if %length(&ColorVar) %then %do;
		data _ColorVar_;
			set &indat;
			
			%if &IDVar=_RowNum_ %then %do;
				_RowNum_ = _n_;
			%end;
			
			keep &IDVar &ColorVar;
		run;
		%if %length(&intestdat) %then %do;
			data _ColorVar_t;
				set &intestdat;
				
				%if &IDVar=_RowNum_ %then %do;
					_RowNum_ = _n_;
				%end;
				
				keep &IDVar &ColorVar;
			run;
		%end;
	%end;
%end;

* check list-style variables;
%if %length(&ListContPredVars) | %length(&ListClassPredVars) %then %do;
   %let ListVars = &ListContPredVars &ListClassPredVars;
	%let i = 1;
   %do %while(%length(%scan(&ListVars,&i)));
	   %CheckListVarExist(%scan(&ListVars,&i),ind1);
		%if &exiterror %then %goto exit;
	   %let i = %eval(&i+1);
	%end;
%end;
%nObsVars(ind1);
%if ^%symexist(NumVar) %then %let NumVar = &nVars;
%else %if ^%length(&NumVar) %then %let NumVar = &nVars;
%if %index(&VarSelect,Genetic) and (&NumVar > &nVars) %then %do;
	%let NumVar = &nVars;
	%put NOTE: Number of Predictor Variables to Select is reduced to &NumVar, the total number available.;
%end;
%put NumVar = &NumVar;
%if &syserr > 4 %then %goto exit;

* check variable types;
%if %length(&PredictorVars &ListContPredVars)>0 %then %do;
	%let BadList = ;

	%CheckVarType(ind1,&PredictorVars &ListContPredVars,1,BadList);
	%if %length(&BadList) %then %do;
		%let exiterror = 1;
	   %put ERROR: The following variables are of type character and misspecified as Predictor Continuous Variables: &BadList..; 
		%goto exit;
	%end;
%end;

%if (&ProcessName = DiscriminantAnalysis) or (&ProcessName = KNearestNeighbors) %then %do;
	%if (&Priors = Custom) %then %do;
		%if ^%length(&CustomPriors) %then %do;
			%put ERROR: You must specify custom prior probabilities when specifying Priors = Custom.;
			%let exiterror = 1;
			%goto exit;
		%end;
		%let Priors = %unquote(&CustomPriors);
		%put Priors = &Priors;
	%end;
%end;

* combine listvars;
%let PredictorVars = &PredictorVars &ListContPredVars;
%let ClassPredVars = &ClassPredVars &ListClassPredVars;
%let LockContPredVars = &LockContPredVars &ListLockContPredVars;
%let LockClassPredVars = &LockClassPredVars &ListLockClassPredVars;

%*ClockStop(Vars);

%*ClockStart(ClassVars);

* set up for ClassVars.sas;
%global ListContColon ListClassColon;
%if %index(&ListContPredVars,:) %then %let ListContColon = 1;
%else %let ListContColon = 0;
%if %index(&ListClassPredVars,:) %then %let ListClassColon = 1;
%else %let ListClassColon = 0;

%let NeedClassIndicators = No;
%let RecoverClassPredVars = No;

%if ^%symexist(PredWhere) %then %let PredWhere = ;
%if ^%symexist(PredClassWhere) %then %let PredClassWhere = ;
%if ^%symexist(BLUPWhere) %then %let BLUPWhere = ;
%if ^%symexist(PLSWhere) %then %let PLSWhere = ;


* APs that can handle class variables with missing values;
%if (&ProcessName=LogisticRegression) or
	 (&ProcessName=GLMSelect) or 
	 (&ProcessName=PartitionTrees) or
	 (&ProcessName=QuantSelect) or
	 (&ProcessName=SurvivalPredictiveModeling) %then %do;
	 
	%if %length(&ClassPredVars) and 
		((&KMeans=Yes) or (&StatTest=Yes) or %index(&VarSelect,Genetic) or %length(&PredClassWhere)) %then %do;
		%let NeedClassIndicators = Yes;
		%if ^%index(&VarSelect,Genetic) %then %do;
			%let RecoverClassPredVars = Yes;
		%end;
	%end;		
%end;
%else %do;
	%if %length(&ClassPredVars) or %length(&LockClassPredVars) %then %do;
		%let NeedClassIndicators = Yes;
	%end;
%end;	

%put RecoverClassPredVars = &RecoverClassPredVars;

* save original class variables for later recovery;
%if (&RecoverClassPredVars = Yes) %then %do;
	data orig_d_class;
		set &indat &intestdat;
		keep &ClassPredVars;
	run;
%end;

* if cross-validating, drop obs with missing depvar;
%if &CVFlag %then %do;
	data &indat;
		set &indat;
		if missing(&DepVar) then delete;
	run;

	%if ^%symexist(DropMissing) %then %let DropMissing = 1;

	%if &DropMissing %then %do;
		data dtest;
			set &intestdat;
			if missing(&DepVar) then delete;
			_train_ = 0;
			_Obs = _n_;
			%if &ProcessName = SurvivalPredictiveModeling or &ProcessName = LifeRegression %then %do;
				%if %length(&CensorVar) %then %do;
					%if &cvtype=C %then %do;
						if &CensorVar_old in (%unquote(&CensorValues_old)) then _Censor1 = 1;
						else _Censor1 = 0;
					%end;
				%end;
				%if &IDVar=_RowNum_ %then %do;
					_RowNum_ = _n_;
				%end;
			%end;
		run;
		%let intestdat = dtest;
		%put intestdat = &intestdat;
	%end;
%end;

* for GLM, PLS, QTS or RBM with binary dependent variable, use the second response indicator variable by default;
%if ((&ProcessName = GLMSelect) or (&ProcessName = PartialLeastSquares) or (&ProcessName = RadialBasisMachine) 
	or (&ProcessName = QuantSelect) or (&ProcessName = RidgeRegression) or (&ProcessName = GenomicBLUP)) and 
	((&_dtype_ = C) or (&NumDepAsClass = Yes)) %then %let Level2 = Yes;
%else %let Level2 = No;

%if ^%symexist(KM_Usage) %then %let KM_Usage = Representative;

* if test set has class variables that require levelization or if using the second response level or kmeans;
* temporarily combine training and test so they can be levelized together;
%if (%length(&intestdat) and &intestdat^=d) and 
	(((&NeedClassIndicators = Yes) and (%length(&LockClassPredVars) or %length(&ClassPredVars))) or (&Level2 = Yes) or %index(&KM_Usage,Means))
	%then %do;
	%put Temporarily combining training and test for levelization.;
	data d_test;
		set &intestdat;
		_train_ = 0;
	run;
	data &indat;
		set &indat d_test;
		_Obs = _n_;
	run;	
	%let separate = 1;
%end;
%else %do;
	data &indat;
		set &indat;
		_Obs = _n_;
	run;
	%let separate = 0;
%end;

%if (&TestSpec = Yes) and (&NumDepAsClass = Yes) %then %do;
	%let ClassVars = &ClassVars &TruthVar;
%end;

* levelize class variables, create macro lists and indicator variables;
* replace ClassPredVars accordingly;

/*
%if %index(&KM_Usage,Means) %then %do;
	%if %length(&TestInData) %then %let _testDs_ = d_test;
	%if &CVFlag %then %let _testDs_ = d2;
	%let _tmp_cVars = &ClassPredVars;
	proc datasets library=Work nolist;
		change d = _tmp_;
		change &_testDs_ = d;
	quit;

	%include "&MacroPath./ClassVars.sas";
	%if &syserr %then %goto exit;

	proc datasets library=Work nolist;
		change d = &_testDs_;
		change _tmp_ = d;
	quit;
	%let ClassPredVars = &_tmp_cVars;
%end;
*/

%put Beginning class levelization;
%include "&MacroPath./ClassVars.sas";
%if &syserr %then %goto exit;

%put Ending class levelization:;
%put nixl = &nixl;
%put nipl = &nipl;
%put nix = &nix;
%put nip = &nip;

%*ClockStop(ClassVars);
%*ClockStart(Predictor Reduction);

%if (&Optimize = Yes) and &nix and &TraitType ne Continuous and &TraitType ne Binary %then %do;
	%put ERROR: The dependent variable must be binary, continuous, or count in order to perform continuous predictor optimization.;
	%let exiterror = 1;
	%goto exit;
%end;

/*
%if (&nix = 0) and (&nip = 0) and (&nixl = 0) and (&nipl = 0) %then %do;
	%put ERROR: At least one predictor variable must be specified.;
	%let exiterror = 1;
	%goto exit;
%end;
*/
%global upred uclass;
%let upred = 0;
%let uclass = 0;

* remove depvar if it is in the pred var lists;
%if &nix %then %do;
	%let _dnum_ = %sysfunc(varnum(&_dsix_,&DepVar));
	%if &_dnum_ %then %do;
		%put Dropping &DepVar from predictor variables;
		%let _tid_ 	= %sysfunc(close(&_dsix_));
		data dsix;
			set dsix;
			drop &DepVar;
		run;
		%let _dsix_ = %sysfunc(open(dsix));
		%let nix = %sysfunc(attrn(&_dsix_,NVARS));
		%put nix = &nix;
		%let upred = 1;
	%end;
%end;
%if &nip %then %do;
	%let _dnum_ = %sysfunc(varnum(&_dsip_,&DepVar));
	%if &_dnum_ %then %do;
		%put Dropping &DepVar from class predictor variables;
		%let _tid_ 	= %sysfunc(close(&_dsip_));
		data dsip;
			set dsip;
			drop &DepVar;
		run;
		%let _dsip_ = %sysfunc(open(dsip));
		%let nip = %sysfunc(attrn(&_dsip_,NVARS));
		%put nip = &nip;
		%let uclass = 1;
	%end;
%end;
%if &nixl %then %do;
	%let _dnum_ = %sysfunc(varnum(&_dsixl_,&DepVar));
	%if &_dnum_ %then %do;
		%put Dropping &DepVar from lock-in predictor variables;
		%let _tid_ 	= %sysfunc(close(&_dsixl_));
		data dsixl;
			set dsixl;
			drop &DepVar;
		run;
		%let _dsixl_ = %sysfunc(open(dsixl));
		%let nixl = %sysfunc(attrn(&_dsixl_,NVARS));
		%put nixl = &nixl;
	%end;
%end;
%if &nipl %then %do;
	%let _dnum_ = %sysfunc(varnum(&_dsipl_,&DepVar));
	%if &_dnum_ %then %do;
		%put Dropping &DepVar from lock-in class predictor variables;
		%let _tid_ 	= %sysfunc(close(&_dsipl_));
		data dsipl;
			set dsipl;
			drop &DepVar;
		run;
		%let _dsipl_ = %sysfunc(open(dsipl));
		%let nipl = %sysfunc(attrn(&_dsipl_,NVARS));
		%put nipl = &nipl;
	%end;
%end;

* remove lock-in vars from pred var lists if necessary;
%if &nixl %then %do;
	%if &nix %then %do;
		%let dropv = ;
		%do i=1 %to &nixl;
			%let _dnum_ = %sysfunc(varnum(&_dsix_,%sysfunc(varname(&_dsixl_,&i))));
			%if &_dnum_ %then %do;
				%let dropv = &dropv %sysfunc(varname(&_dsixl_,&i));
			%end;
		%end;
		%if %length(&dropv) %then %do;
			%put Dropping &dropv from continuous predictor variables because of lock-in;
			%let _tid_ 	= %sysfunc(close(&_dsix_));
			data dsix;
				set dsix;
				drop &dropv;
			run;
			%let _dsix_ = %sysfunc(open(dsix));
			%let nix = %sysfunc(attrn(&_dsix_,NVARS));
			%put nix = &nix;
			%let upred = 1;
		%end;
	%end;
	%if &nip %then %do;
		%let dropv = ;
		%do i=1 %to &nixl;
			%let _dnum_ = %sysfunc(varnum(&_dsip_,%sysfunc(varname(&_dsixl_,&i))));
			%if &_dnum_ %then %do;
				%let dropv = &dropv %sysfunc(varname(&_dsixl_,&i));
			%end;
		%end;
		%if %length(&dropv) %then %do;
			%put Dropping &dropv from class predictor variables because of lock-in;
			%let _tid_ 	= %sysfunc(close(&_dsip_));
			data dsip;
				set dsip;
				drop &dropv;
			run;
			%let _dsip_ = %sysfunc(open(dsip));
			%let nip = %sysfunc(attrn(&_dsip_,NVARS));
			%put nip = &nip;
			%let uclass = 1;
		%end;
	%end;
%end;
%if &nipl %then %do;
	%if &nix %then %do;
		%let dropv = ;
		%do i=1 %to &nipl;
			%let _dnum_ = %sysfunc(varnum(&_dsix_,%sysfunc(varname(&_dsipl_,&i))));
			%if &_dnum_ %then %do;
				%let dropv = &dropv %sysfunc(varname(&_dsipl_,&i));
			%end;
		%end;
		%if %length(&dropv) %then %do;
			%put Dropping &dropv from continuous predictor variables because of lock-in;
			%let _tid_ 	= %sysfunc(close(&_dsix_));
			data dsix;
				set dsix;
				drop &dropv;
			run;
			%let _dsix_ = %sysfunc(open(dsix));
			%let nix = %sysfunc(attrn(&_dsix_,NVARS));
			%put nix = &nix;
			%let upred = 1;
		%end;
	%end;
	%if &nip %then %do;
		%let dropv = ;
		%do i=1 %to &nipl;
			%let _dnum_ = %sysfunc(varnum(&_dsip_,%sysfunc(varname(&_dsipl_,&i))));
			%if &_dnum_ %then %do;
				%let dropv = &dropv %sysfunc(varname(&_dsipl_,&i));
			%end;
		%end;
		%if %length(&dropv) %then %do;
			%put Dropping &dropv from class predictor variables because of lock-in;
			%let _tid_ 	= %sysfunc(close(&_dsip_));
			data dsip;
				set dsip;
				drop &dropv;
			run;
			%let _dsip_ = %sysfunc(open(dsip));
			%let nip = %sysfunc(attrn(&_dsip_,NVARS));
			%put nip = &nip;
			%let uclass = 1;
		%end;
	%end;
%end;

* update full predictor lists if necessary;
%put upred= &upred;
%if &upred %then %do;
	%global nix_blocks;
	%let max_blocks= 9000;
	%let nix_blocks= %sysevalf(&nix/&max_blocks,ceil);
	
	/*
	%let PredictorVars = ;
	%do i=1 %to &nix; 
		%let PredictorVars = &PredictorVars %sysfunc(varname(&_dsix_,&i)); 
	%end;
	*/
	
	/*
	%let PredictorVars = &PredictorVars &ListContPredVars;
	*/	
	
	/*S1308696,  S1308697 and S1307478: Bugs related to (1) the usage of Lock-In continuous 
	predictor variables along with multiple methods of predictor reduction; (2) the usage of 
	Mahalanobis distance in the standardization method for continuous predictor variables; 
	(3) lenght of macro variables handled by SAS macro language (exceeding the string length 
	limit that a macro variable can hold). */
	%if &nix_blocks %then %do;
		*Set this variable to 1 becasue it is used in "if" statements. It will work as flag. May not be needed*;
		%let PredictorVars = 1; 
	%end;
	%else %do;
		%let PredictorVars = ;
	%end;
	
	%let i=1;
	%let r=&nix;
	%do j=1 %to &nix_blocks;
		%global PredictorVars&j; 
		%let PredictorVars&j = ;
		%do k=1 %to %sysfunc(min(&max_blocks,&r));
			%let PredictorVars&j= &&PredictorVars&j %sysfunc(varname(&_dsix_,&i));
			%let i= %sysevalf(&i + 1);
			%let r= %sysevalf(&nix - &max_blocks);
		%end;
		%*put PredictorVars&j = &&PredictorVars&j;
	%end;
%end;

%*goto exit;

%put uclass= &uclass; 
%if &uclass %then %do;
	%let ClassPredVars = ;
	%do i=1 %to &nip; 
		%let ClassPredVars = &ClassPredVars %sysfunc(varname(&_dsip_,&i)); 
	%end;
	/*
   	%let ClassPredVars = &ClassPredVars &ListClassPredVars;
   	*/
%end;


* additional initialization;
%global nlev BPV nclasses event;
%let nlev = 0;
%let nclasses = ;
%let BPV = 0;
%let event = ;
%if ^%symexist(Priors) %then %let Priors = Proportional;

* determine number of classes and define prior weights if necessary;
%if (&_dtype_=C) or (&NumDepAsClass=Yes) %then %do;

	proc freq data=d noprint;
		tables &DepVar / out=f;
	data f;
		set f;
		if missing(&DepVar) then delete;
	run;
	
	proc sort data=f;
		by &DepVar;
	run;

	/*
	title "f";
	proc print data=f;
	run;
	*/

	%NObsVars(f);
	%let nclasses = &nobs;
	%do i=1 %to &nclasses;
		%global class&i;
	%end;
	data _null_;
		set f;
		call symputx("class"||trim(left(_n_)),trim(left(&DepVar)));
		call symputx("pct"||trim(left(_n_)),Percent);
	run;
	/*
	proc sort data=f;
		by percent;
	data _null_;
		set f nobs=nobs end=the_end;
		if _n_ = 2 then call symputx('event', trim(left(&TruthVar)));
	run;
	*/
	%put nclasses = &nclasses;	
	%do i=1 %to &nclasses;
		%global class&i;
		%put class&i = &&class&i   pct&i = &&pct&i;
	%end;
	%if (&nclasses > 1) %then %do;
		%let event = &class2;
	%end;
	%else %do;
		%let event = &class1;
	%end;
	%put event = &event;

	* for nonproportional priors, use reciprocal frequency weights to undo implicit proportional priors;
	%if (&Priors ^= Proportional) %then %do;
		data d;
			set d;
			%do i=1 %to &nclasses;
				%if (&_dtype_=C) %then %do;
					if &TruthVar = "&&class&i" then _Prior = 100/&&pct&i;
				%end;
				%else %do;
					if &TruthVar = &&class&i then _Prior = 100/&&pct&i;
				%end;
			%end;
		run;
	%end;
	
	* create macro variables containing P_ names for posterior probs;
	* truncate depvar name if it is too long;
	%let maxclen = 0;
	%do c=1 %to &nclasses;
		%let clen = %length(&&class&c);
		%if &clen > &maxclen %then %do;
			%let maxclen = &clen;
		%end;
	%end;
	%let maxdlen = %eval(32 - &maxclen - 3);
	%put maxclen = &maxclen;
	%put maxdlen = &maxdlen;
	%if maxdlen < 1 %then %do;
		%let &DepVarTrunc = ;
	%end;
	%else %do;
		%if %length(&DepVar) > &maxdlen %then %do;
			%let DepVarTrunc = %substr(&DepVar,1,&maxdlen);
		%end;
		%else %do;
			%let DepVarTrunc = &DepVar;
		%end;
	%end;

	* verify class values are valid for sas names, if not, use values 1, 2, 3...;
	data vn;
		length vname $ 32;
		one = 1;
		%do c=1 %to &nclasses;
			vname = trim(left(("p_&DepVarTrunc._&&class&c")));
			lenv = length(vname);
			_check1 = verify(upcase(vname),"_ABCDEFGHIJKLMNOPQRSTUVWXYZ");
			_check2 = verify(upcase(vname),"_ABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789");
			if (_check1 = 1) or ((_check2 > 0) and (_check2 <= lenv)) then do;
				vname = "p_&DepVarTrunc._&c";
			end;
			output;
		%end;
	run;

	/*
	ods exclude none;
	ods results;
	title "vname";
	proc print data=vn;
	run;
	*/

	proc transpose data=vn out=vnt;
		var one;
		id vname;
	run;
	proc transpose data=vnt out=vntt;
	run;
	%do c=1 %to &nclasses;
		%global p_prob&c;
	%end;
	data _null_;
		set vntt;
		call symputx("p_prob"||trim(left(_n_)),_name_);
	run;
	%do c=1 %to &nclasses;
		%put p_prob&c = &&p_prob&c;
	%end;	
	
%end;
%else %do;
	%let Priors = Proportional;
%end;
%if %length(&DepVar) > 30 %then %do;
	%let DepVarTrunc30 = %substr(&DepVar,1,30);
%end;
%else %do;
	%let DepVarTrunc30 = &DepVar;
%end;

* separate training and test that were combined above;
%if &separate %then %do;
	%put Separating training and test combined above.;
	data d_test;
		set d;
		where _train_ = 0;
	run;
	%let intestdat = d_test;
	%put intestdat = &intestdat;
	data d;
		set d;
		where _train_ = 1;
	run;
%end;


%let DepVar0 = &DepVar;
%let _dtype0_ = &_dtype_;
%let TruthVar0 = &TruthVar;


* for APs using the second indicator variable by default (including GLM, PLS, QTS or RBM with binary dependent variable);
%if (&Level2 = Yes) %then %do;
	%if (&nic1 = 1) %then %do;
		%put ERROR: Dependent variable has only one level.;
		%let exiterror = 1;
		%goto exit;
	%end;
	%else %if (&nic1 > 2) %then %do;
		%put ERROR: &nic1 levels detected for &DepVar.  Only 2 levels can be modeled.;
		%put If &DepVar is numeric, make sure missing values are coded with a period.;
		%put If &DepVar is character, make sure missing values are coded with a blank.;
		%let exiterror = 1;
		%goto exit;
	%end;
	%if %length(&EventTrait) %then %do;
		%let eidx = 0;
		%let tidx = 0;
		data _null_;
			set ocp1;
			%if &_dtype_ = C %then %do;
				if &DepVar = "&EventTrait" then
			%end;
			%else %do;
				if &DepVar = %unquote(&EventTrait) then
			%end;
				call symput('eidx',trim(left(_n_)));
		run;
		%if ^&eidx %then %do;
			%put ERROR: Event trait level &EventTrait not found for &DepVar.  Please specify a different value or leave it blank to use the second ordered level.;
			%let exiterror = 1;
			%goto exit;
		%end;	
	%end;
	%else %do;
		%let eidx = 2;
	%end;
	%let EventTrait = &&_lev&eidx;
	%let event = &EventTrait;
	%let neidx = %eval(3-&eidx);
	%let NonEventTrait = &&_lev&neidx;
	%put eidx = &eidx;
	%put EventTrait = &EventTrait;
	%put NonEventTrait = &NonEventTrait;
	* make sure levels are valid for sas variable names;
	%if %sysfunc(verify(%upcase(&EventTrait),"_ABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789")) %then %do;
		%let ETV = 2;
	%end;
	%else %do;
		%let ETV = &EventTrait;
	%end;
	%if %sysfunc(verify(%upcase(&NonEventTrait),"_ABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789")) %then %do;
		%let NTV = 1;
	%end;
	%else %do;
		%let NTV = &NonEventTrait;
	%end;
	%put ETV = &ETV;
	%put NTV = &NTV;
	%let DepVar = &&_c&eidx;
	* make sure chosen variable is not the missing value indicator;
	%let ld = %length(&DepVar);
	%if &ld > 4 %then %do;
		%let dvlast4 = %substr(&DepVar,%eval(&ld-3),4);
		%if &dvlast4 = _Mis %then %do;
			%let eidx1 = %eval(&eidx+1);
			%put eidx1 = &eidx1;
			%let DepVar = &&_c&eidx1;
		%end;
	%end;
	%put New DepVar = &DepVar;
	* redefine DepVarTrunc and DepVarTrunc30;
	%if %length(&DepVar) > &maxdlen %then %do;
		%let DepVarTrunc = %substr(&DepVar,1,&maxdlen);
	%end;
	%else %do;
		%let DepVarTrunc = &DepVar;
	%end;
	%if %length(&DepVar) > 30 %then %do;
		%let DepVarTrunc30 = %substr(&DepVar,1,30);
	%end;
	%else %do;
		%let DepVarTrunc30 = &DepVar;
	%end;
	%let ClassVar = &DepVar;
	%let _dtype_ = N;
	%let DepSwitch = Yes;
	%if (&TruthVar0 = &DepVar0) %then %do;
		%let TruthVar = &DepVar;
	%end;
	%else %do;
		%let len0 = %length(&DepVar0);
		%let len1 = %eval(%length(&DepVar) - %length(&DepVar0));
		%let TruthVar = &TruthVar0.%substr(&DepVar,%eval(&len0+1),&len1);
	%end;
	%put New TruthVar = &TruthVar;
	
	* set new DepVar to missing wherever original DepVar is missing;
	data d;
		set d;
		if missing(&DepVar0) then &DepVar = .;
	run;
	%let TraitType = Binary;		
%end;
%else %do;
	%let DepSwitch = No;
%end;



* Where Clause filter on continuous and class predictors;
%if (%length(&PredWhere) and &nix) or (%length(&PredClassWhere) and &nip) %then %do;

	%let allstats = CSS CV IQR KURTOSIS MAD MAX MEAN MEDIAN MIN NMISS SKEWNESS STD STDERR SUM USS VAR;

	* build a data set with variable names to keep;
	%if &nix %then %do;
	
		proc transpose data=d out=pw;
			var %do i=1 %to &nix; %sysfunc(varname(&_dsix_,&i)) %end;  ;
			where not missing(&DepVar);
		run;
		
		/*data OutLib.pw; set pw; run;*/

		%if %length(&PredWhere) %then %do;
			* check for stat specification;
			%let nstat = 0;
			%let upPredWhere = %upcase(&PredWhere);
			%let i=1;
			%do %while(%length(%scan(&allstats,&i,' ')));
				%let stat = %scan(&allstats,&i,' ');
				%if %index(&upPredWhere,&stat) %then %do;
					%let nstat = %eval(&nstat + 1);
					%let stat&nstat = &stat;
				%end;
				%let i=%eval(&i+1);
			%end;

			%if (&nstat > 0) %then %do;
				data pw;
					set pw;
					%do i=1 %to &nstat;
						&&stat&i = &&stat&i.(of col:);
					%end;
				run;
			%end;

			%if &syserr > 4 %then %goto exit;

			data pw;
				set pw;
				where %unquote(&PredWhere);
				keep _name_;
			run;

		%end;
		%else %do;
			data pw;
				set pw;
				keep _name_;
			run;
		%end;
		%if &syserr > 4 %then %goto exit;
	%end;
	
	* filter class predictor variables;
	%if &nip %then %do;
		
		proc transpose data=d out=pwc;
			var %do i=1 %to &nip; %sysfunc(varname(&_dsip_,&i)) %end;  ;
			where not missing(&DepVar);
		run;
		
		/*data OutLib.pwc; set pwc; run;*/
		
		%if %length(&PredClassWhere) %then %do;
			* check for stat specification;
			%let nstat = 0;
			%let upPredWhere = %upcase(&PredClassWhere);
			%let i=1;
			%do %while(%length(%scan(&allstats,&i,' ')));
				%let stat = %scan(&allstats,&i,' ');
				%if %index(&upPredWhere,&stat) %then %do;
					%let nstat = %eval(&nstat + 1);
					%let stat&nstat = &stat;
				%end;
				%let i=%eval(&i+1);
			%end;

			%if (&nstat > 0) %then %do;
				data pwc;
					set pwc;
					%do i=1 %to &nstat;
						&&stat&i = &&stat&i.(of col:);
					%end;
				run;
			%end;
			
			%if &syserr > 4 %then %goto exit;

			data pwc;
				set pwc;
				where %unquote(&PredClassWhere);
				keep _name_;
			run;
		%end;
		%else %do;
			data pwc;
				set pwc;
				keep _name_;
			run;
		%end;

		%if &syserr > 4 %then %goto exit;
		
		%if &nix %then %do;
			data pw;
				length _name_ $32;
				set pw pwc;
			run;
		%end;
		%else %do;
			proc datasets library=work;
				change pwc=pw;
			run;
		%end;
	%end;

	* remove filtered predictors from the data set;
	%ReduceData(d,pw,&intestdat,Where Clause,No);

%end;
	
%if &syserr > 4 %then %goto exit;

* Set-up for Buckley-James loop;
%let BJ_Iterating = 1;
%let BJ_OneMore = 0;
%if &BJ_Est = Yes %then %do;
	%if (&_dtype_ = C) %then %do;
		%put ERROR: The dependent variable must be numeric for Buckley-James estimation.;
		%let exiterror = 1;
		%goto exit;
	%end;
	
	%if (&TraitType = Binary) %then %do;
		%put ERROR: The dependent variable must be numeric with more than two distinct values for Buckley-James estimation.;
		%let exiterror = 1;
		%goto exit;
	%end;

	
	%put Initializing Buckley-James Estimation;
	
	%let DepVarSave = &DepVar;
	%if %length(&DepVar) > 29 %then %let DepVar = %substr(&DepVar,1,29)_BJ;
	%else %let DepVar = &DepVar._BJ;

	data d;
		length &DepVar 8;
		set d;
		&DepVar = &DepVarSave;
	run;

	%if ^%symexist(DepVarTrunc) %then %let DepVarTrunc = ;
	%if %length(&DepVarTrunc) %then %do;
		%let DepVarTruncSave = &DepVarTrunc;
		%if %length(&DepVar) > &maxdlen %then %do;
			%let DepVarTrunc = %substr(&DepVar,1,&maxdlen);
		%end;
		%else %do;
			%let DepVarTrunc = &DepVar;
		%end;
	%end;
	%if ^%symexist(DepVarTrunc30) %then %let DepVarTrunc30 = ;
	%if %length(&DepVarTrunc30) %then %do;
		%let DepVarTrunc30Save = &DepVarTrunc30;
		%if %length(&DepVar) > 30 %then %do;
			%let DepVarTrunc30 = %substr(&DepVar,1,30);
		%end;
		%else %do;
			%let DepVarTrunc30 = &DepVar;
		%end;
	%end;

	%let TruthVarSave = &TruthVar;
	%let TruthVar = &DepVar;
		
	* save a copy of the data because predictor variables may be dropped;
	* and need to be restored for each iteration;
	data dsave;
		set d;
	run;
	
	%if %length(&intestdat) and (&intestdat^=d) %then %do;
		%let intestdatsave = &intestdat;
		%put intestdatsave = &intestdatsave;
		%let intestdat = d;
	%end;
	%else %do;
		%let intestdatsave =;
	%end;
	
	* save predictor variables so they can be restored for each iteration;
	%if ^&upred %then %do;	
		%let PredictorVarsSave = &PredictorVars;
	%end; 
	%else %do;
		%let PredictorVarsSave1 = &PredictorVars1;
		%do j=2 %to &nix_blocks;
			%let PredictorVarsSave&j = &&PredictorVars&j;
		%end;
	%end;
	%let ClassPredVarsSave = &ClassPredVars;

	%let nixsave = &nix;
	%if &nix %then %do;
		data dsixsave;
			set dsix;
		run;
	%end;
	%let nipsave = &nip;
	%if &nip %then %do;
		data dsipsave;
			set dsip;
		run;
	%end;
	%let nixlsave = &nixl;
	%if &nixl %then %do;
		data dsixlsave;
			set dsixl;
		run;
	%end;
	%let niplsave = &nipl;
	%if &nipl %then %do;
		data dsiplsave;
			set dsipl;
		run;
	%end;
	%if %symexist(NPred) %then %do;
		%let NPredSave = &NPred;
	%end;
	
	* if cross-validating, disable cv processing until bj loop is finished;
	%let CVFlagSave = &CVFlag;
	%let CVFlag = 0;		
	
	%let BJ_Niter = 0;
	%if ^%length(&BJ_Maxiter) %then %let BJ_Maxiter = 10;
	%if ^%length(&BJ_RMSE_Tol) %then %let BJ_RMSE_Tol = 0.01;
	
%end;

* Buckley-James loop until convergence or maxiter is reached;
* when not doing Buckley-James this loop only gets executed once;
%do %while(&BJ_Iterating);

%if %length(&DataStepStatements) %then %do;
	data d;
		set d;
		&DataStepStatements;
	run;
	%let ts1 = d;
%end;

* Censoring weight, repeat calculation here because;
* weightvar might be set in data step statements above;
%if %length(&CensorVar) %then %do;
	%if &CensorWeight ne 1 %then %do;
		data d;
			set d;
			%if %length(&WeightVar0) %then %do;
				if &CensorVar in (%unquote(&CensorValues)) then do;
					&WeightVar = &WeightVar0 * &CensorWeight;
				end;
				else do;
					&WeightVar = &WeightVar0;
				end;
			%end;
			%else %do;
				if &CensorVar in (%unquote(&CensorValues)) then do;
					&WeightVar = &CensorWeight;
				end;
				else do;
					&WeightVar = 1;
				end;			
			%end;
		run;
	%end;
%end;


* Transformation for continuous predictors;
%if ^%symexist(TransformationMethod) %then %let TransformationMethod = ;
%if %length(&TransformationMethod) and (&nixl or &nix) %then %do;

	%put Specified Transformation Method = &TransformationMethod;

	%if &TransformationMethod = Spline %then %do;

		%if %length(&intestdat) and (&intestdat^=d) %then %do;
			%put Temporarily combining training and test for spline transformation.;
			data d_test;
				set &intestdat;
				_train_ = 0;
			run;
			data d;
				set d d_test;
				_obs = _n_;
			run;	
		%end;
		
		data d0;
			set d;
			drop 
				%do i=1 %to &nixl; %sysfunc(varname(&_dsixl_,&i)) %end; 
				%do i=1 %to &nix; %sysfunc(varname(&_dsix_,&i)) %end;
			;
		run;
		
		data d1;
			set d;
			keep _obs &DepVar _train_
				%do i=1 %to &nixl; %sysfunc(varname(&_dsixl_,&i)) %end; 
				%do i=1 %to &nix; %sysfunc(varname(&_dsix_,&i)) %end;
			;
		run;
		
		data dstack;
			set d1;
			y = &DepVar;
			if _train_ = 0 then y = .;
			length varname $ 32;
			%do i=1 %to &nixl; 
				varname = "%sysfunc(varname(&_dsixl_,&i))";
				x = %sysfunc(varname(&_dsixl_,&i));
				output;
			%end;
			%do i=1 %to &nix; 
				varname = "%sysfunc(varname(&_dsix_,&i))";
				x = %sysfunc(varname(&_dsix_,&i));
				output;
			%end;
			keep _obs varname y x _train_;
		run;
	
		proc sort data=dstack;
			by varname;
		run;

		/*
		ods exclude all;
		ods noresults;

		* use REML to estimate the smoothing spline parameter;
		* runs slowly;
		proc glimmix data=dstack;
			by varname;
			effect spl = spline(x);
			model y = ;
			random spl / type=toep(1);
			output out=og pred(blup ilink)=pred;
		run;
		
		ods exclude none;
		ods results;

		data dstack;
			set og;
			x = pred;
			keep _obs varname x;
		run;
		
		proc stdize data=dstack;
			by varname;
			var x;
		run;
		
		proc sort data=dstack;
			by _obs;
		run;
			
		proc transpose data=dstack out=d1;
			by _obs;
			var x;
			id varname;
		run;
		*/

		* to avoid interpolation and extrapolation errors, set missing ys to ymean of training data;
		proc means data=d1 noprint;
			var &DepVar;
			output out=ymean mean=ymean;
		run;

		data _null_;
			set ymean;
			call symputx("ymean",ymean);
		run;
		
		data dstack;
			set dstack;
			if missing(y) then y = &ymean;
			keep varname y x _obs;
		run;
				
		ods exclude all;
		ods noresults;
				
		proc glmselect data=dstack;
			by varname;
			effect spl = spline(x / separate split degree=3);
			model y = spl / /* selection=stepwise backward select=aicc */;
			output out=og pred;
		run;
		
		ods exclude none;
		ods results;

		data dstack;
			set og;
			x = p_y;
			keep _obs varname x;
		run;
		
		/*
		proc stdize data=dstack out=dstack;
			by varname;
			var x;
		run;
		*/
		
		proc sort data=dstack;
			by _obs;
		run;
			
		proc transpose data=dstack out=d1;
			by _obs;
			var x;
			id varname;
		run;
		
		/*
		
		* get min and max of each x variable for nonmissing ys;
		proc means data=dstack noprint;
			where not missing(y);
			by varname;
			var x;
			output out=mm min=xmin max=xmax;
		run;
		
		* mean of y variable;
		proc means data=d1 noprint;
			var &DepVar;
			output out=ymean mean=ymean;
		run;

		data _null_;
			set ymean;
			call symputx("ymean",ymean);
		run;
		
		* to avoid extrapolation errors, set y to ymean for any d values that exceed the max and min for observed ys;
		data dstack;
			merge dstack mm;
			by varname;
			if (x < xmin) or (x > xmax) then y = &ymean;
			keep varname y x _obs;
		run;
		
		ods exclude all;
		ods noresults;

		proc transreg data=dstack noprint;	
			by varname;
			model identity(y) = spline(x);
			output out=ot;
			id _obs;
		run;
		
		data dstack;
			set ot;
			keep _obs varname tx;
		run;

		proc sort data=dstack;
			by _obs;
		run;
			
		proc transpose data=dstack out=d1;
			by _obs;
			var tx;
			id varname;
		run;
		*/
		
		/*
		* only fit spline to continuous predictors with at least three levels to avoid errors and wild extrapolations;
		* separate out the variables that have at least three levels;
		proc freq data=dstack nlevels;
			where _train_ = 1;
			by varname;
			tables x / noprint;
			ods output nlevels=nx;
		run;
		
		data OutLib.nx;
			set nx;
		run;
		
		data dstack1 dstack2;
			merge dstack nx;
			by varname;
			if nlevels > 2 then output dstack1;
			else output dstack2;
		run;
		
		proc gam data=dstack1;
			by varname;
			model y = spline(x);
			id _obs _train_;
			output out=pgam p=pgam_;
			ods output itersummary=OutLib.gam_is;
		run;

		ods exclude none;
		ods results;

		data dstack1;
			set ot;
			x = tx;
			keep _obs varname x;
		run;
		
		data dstack1;
			set pgam;
			x = pgam_y;
			keep _obs varname x;
		run;
		
		proc stdize data=dstack1 method=std;
			by varname;
			var x;
		run;
		
		data dstack1;
			set dstack1;
			if missing(x) then x = 0;
		run;
		
		data dstack;
			set dstack1 dstack2;
		run;

		proc sort data=dstack;
			by _obs;
		run;
			
		proc transpose data=dstack out=d1;
			by _obs;
			var x;
			id varname;
		run;
		*/
		
		data d;
			merge d0 d1;
			by _obs;
		run;
		
		/*
		data OutLib.d;
			set d;
		run;
		*/
		
		%if %length(&intestdat) and (&intestdat^=d) %then %do;
			%put Separating training and test combined above.;
			data d_test;
				set d;
				where _train_ = 0;
			run;
			%let intestdat = d_test;
			data d;
				set d;
				where _train_ = 1;
			run;
		%end;
		
	%end;
	
%end;

%if &syserr > 4 %then %goto exit;

* Standardization for continuous predictors;
%if ^%symexist(StandardizationMethod) %then %let StandardizationMethod = ;
%if %length(&StandardizationMethod) and (&nixl or &nix) %then %do;

	%put Specified Standardization Method = &StandardizationMethod;
	
	%if &ProcessName = GenomicBLUP or &ProcessName = GenomicBayesianRegression %then %do;
		%let Flag_RecoverInData= 1;
		
		data d_nostd;
			set d;
		run;
		
		%if %length(&intestdat) and (&intestdat^=d) %then %do;
			%let Flag_RecoverTestData= 1;
			data d_test_nostd;
				set &intestdat;
			run;
		%end;
	%end;

	%if &StandardizationMethod = MAHALANOBIS %then %do;
		%NObsVars(d);
		%if &nobs < &nvars %then %let Transpose = 1;
		%else %let Transpose = 0;
		%let nn = &nobs;
		%let mahal = 1;
		%let StandardizationMethod = MEAN;
	%end;
	%else %do;
		%let mahal = 0;
		%let Transpose = 0;
	%end;
	
	%if ^&mahal or ^&Transpose %then %do;
		* substitute numerical argument if necessary;
		%if ^%length(&NumARg) %then %let NumArg = 0;
		%put NumArg = &NumArg;
		%let firstpart = %scan(&StandardizationMethod,1,'(');
		%put firstpart = &firstpart;
		%let lfp = %length(&firstpart);
		%if (%length(&StandardizationMethod) = &lfp) %then %let parenloc = 0;
		%else %let parenloc = 1;
		%put parenloc = &parenloc;
		%if (&NumArg > 0) and (&parenloc > 0) %then %do;
			%let StandardizationMethod = &firstpart.(&NumArg);
		%end;

		* standardize continuous predictors in training data;
		proc stdize data=d out=d method=&StandardizationMethod outstat=tmpstat;
			var
				%do i=1 %to &nixl; %sysfunc(varname(&_dsixl_,&i)) %end; 
				%do i=1 %to &nix; %sysfunc(varname(&_dsix_,&i)) %end; 
			;
		run;

		* apply stats to test data;
		%if %length(&intestdat) and (&intestdat^=d) /*and (&nixl or &nix)*/ %then %do;
			* replace any missing location estimators with 0;
			* and scale estimators with 1 to avoid errors in next call;
			data tmpstat;
				set tmpstat;
				array pv{*} 
					%do i=1 %to &nixl; %sysfunc(varname(&_dsixl_,&i)) %end; 
					%do i=1 %to &nix; %sysfunc(varname(&_dsix_,&i)) %end; 
				;
				if (_TYPE_ = "LOCATION") or (_TYPE_ = "SCALE") then do;
					do i=1 to dim(pv);
						if (_TYPE_ = "LOCATION") then do;
							if missing(pv[i]) then pv[i] = 0;
						end;
						else if (_TYPE_ = "SCALE") then do;
							if missing(pv[i]) then pv[i] = 1;
						end;
					end;
				end;
			run;

			proc stdize data=&intestdat out=&intestdat method=in(tmpstat);
				var 
					%do i=1 %to &nixl; %sysfunc(varname(&_dsixl_,&i)) %end; 
					%do i=1 %to &nix; %sysfunc(varname(&_dsix_,&i)) %end; 
				;
			run;
		%end;
	%end;
	%if &mahal %then %do;
		%let StandardizationMethod = MAHALANOBIS;
		%if &Transpose %then %do;
			proc transpose data=d out=ts1std;
				var &LockContPredVars %if ^&upred %then %do; &PredictorVars %end; %else %do; %do j=1 %to &nix_blocks; &&PredictorVars&j %end; %end;
					/*
					%do i=1 %to &nixl; %sysfunc(varname(&_dsixl_,&i)) %end; 
					%do i=1 %to &nix; %sysfunc(varname(&_dsix_,&i)) %end; 
					*/
				;
			run;
			%let invars = col1-col&nobs;
			%let idvar = _NAME_;
			
			* mean standardize after transpose for speed;
			data ts1std;
				set ts1std;
				array c{*} &invars;
				mc = mean(of &invars);
				do i=1 to dim(c);
					c[i] = c[i] - mc;
				end;
				drop i;
			run;
			
			* mean standardize test data using means from training data;
			%if %length(&intestdat) and &intestdat^=d %then %do;
				data t1;
					set &intestdat;
					drop &LockContPredVars %if ^&upred %then %do; &PredictorVars %end; %else %do; %do j=1 %to &nix_blocks; &&PredictorVars&j %end; %end;;
				run;
				data t2;
					set &intestdat;
					keep &LockContPredVars %if ^&upred %then %do; &PredictorVars %end; %else %do; %do j=1 %to &nix_blocks; &&PredictorVars&j %end; %end;;
				run;
				proc transpose data=t2 out=t2t;
					var &LockContPredVars %if ^&upred %then %do; &PredictorVars %end; %else %do; %do j=1 %to &nix_blocks; &&PredictorVars&j %end; %end;
						/*
						%do i=1 %to &nixl; %sysfunc(varname(&_dsixl_,&i)) %end; 
						%do i=1 %to &nix; %sysfunc(varname(&_dsix_,&i)) %end; 
						*/
					;
				run;
				%NObsVars(t2);
				%let invars1 = col1-col&nobs;
				data t2t;
					merge t2t ts1std(keep=mc);
					array c{*} &invars1;
					do i=1 to dim(c);
						c[i] = c[i] - mc;
					end;
					drop i;
				run;
				proc transpose data=t2t out=t2;
					var &invars1;
				run;
				data &intestdat;
					merge t1 t2;
				run;
			%end;
		%end;
		%else %do;
			/*
			proc stdize data=d method=mean out=ts1std(keep=
					%do i=1 %to &nixl; %sysfunc(varname(&_dsixl_,&i)) %end; 
					%do i=1 %to &nix; %sysfunc(varname(&_dsix_,&i)) %end; 
				);
				var 
					%do i=1 %to &nixl; %sysfunc(varname(&_dsixl_,&i)) %end; 
					%do i=1 %to &nix; %sysfunc(varname(&_dsix_,&i)) %end; 
				;							
			run;
			*/
			
			data ts1std;
				set d;
				_row_ = _n_;
			run;
			
			/*
			%let invars = ;
			%do i=1 %to &nixl; %let invars = &invars %sysfunc(varname(&_dsixl_,&i)); %end; 
			%do i=1 %to &nix; %let invars = &invars %sysfunc(varname(&_dsix_,&i)); %end; 
			*/
			
			%if ^&upred %then %do; 
				%let invars = &PredictorVars &LockContPredVars; 
			%end; 
			%else %do;
				*use PredictorVars&j variables directly within the %svd() macro*;
				%let invars = ; 
			%end;
			
			%let idvar = _row_;
		%end;

		/*
		data OutLib.ts1std;
			set ts1std;
		run;
		*/
		
		%svd(work,ts1std,out_v,out_s,out_u,&invars,&idvar);

		/*
		data OutLib.out_v;
			set out_v;
		run;
		data OutLib.out_s;
			set out_s;
		run;
		data OutLib.out_u;
			set out_u;
		run;
		*/

		%NObsVars(out_s);
		%let nsv = &nobs;
		%put nsv = &nsv;
		
		%if %length(&intestdat) and &intestdat^=d %then %do;
			data b1;
				set d &intestdat;
			run;
			%let b1 = b1;
			%NObsVars(d);
			%let ntrain = &nobs;
		%end;
		%else %do;
			%let b1 = d;
		%end;

		proc iml;
			use out_v;
			read all var{%do i=1 %to &nsv; prin&i %end;} into v;
			use out_s;
			read all var{singularvalue} into d;
			use out_u;
			read all var{%do i=1 %to &nsv; prin&i %end;} into u;
			* print v d u;
			use &b1;
			read all var{
				&LockContPredVars
				%do j=1 %to &nix_blocks; 
					&&PredictorVars&j
				%end;
				/*
				%do i=1 %to &nixl; %sysfunc(varname(&_dsixl_,&i)) %end; 
				%do i=1 %to &nix; %sysfunc(varname(&_dsix_,&i)) %end; 
				*/
				} into b;
			%if &Transpose %then %do;
				btrans = sqrt(&nn-1)*b*u*inv(diag(d))*t(v);
			%end;
			%else %do;
				btrans = sqrt(&nn-1)*b*v*inv(diag(d))*t(u);
			%end;
			create b1tr from btrans;
			append from btrans;
		quit;	
		
		%NObsVars(b1tr);
		
		data b1tr;
			set b1tr;
			rename %do i=1 %to &nvars; col&i=Mah&i %end; ;
		run;
		
		/*
		data OutLib.b1tr;
			set b1tr;
		run;
		*/

		/*
		proc iml;
			use ts1std;
			read all var{
				%if %index(&VarSelect,Genetic) %then %do;
					&SelVars
				%end;
				%else %do;
					%do i=1 %to &nixl; %sysfunc(varname(&_dsixl_,&i)) %end; 
					%do i=1 %to &nipl; %sysfunc(varname(&_dsipl_,&i)) %end; 
					%do i=1 %to &nix; %sysfunc(varname(&_dsix_,&i)) %end; 
					%do i=1 %to &nip; %sysfunc(varname(&_dsip_,&i)) %end; 
				%end;						
				} into x;
			* print x;
			n = nrow(x);
			c = &nm;
			xt = t(x)/sqrt(n-c);
			print xt;
			call svd(v,d,u,xt);
			print v d u;
			use b1;
			read all var{
				%if %index(&VarSelect,Genetic) %then %do;
					&SelVars
				%end;
				%else %do;
					%do i=1 %to &nixl; %sysfunc(varname(&_dsixl_,&i)) %end; 
					%do i=1 %to &nipl; %sysfunc(varname(&_dsipl_,&i)) %end; 
					%do i=1 %to &nix; %sysfunc(varname(&_dsix_,&i)) %end; 
					%do i=1 %to &nip; %sysfunc(varname(&_dsip_,&i)) %end; 
				%end;						
				} into b;
			btrans = b*v*inv(diag(d))*t(u);
			create b1tr from btrans;
			append from btrans;
		quit;	
		*/
		
		data &b1;
			set &b1;
			drop &LockContPredVars %if ^&upred %then %do; &PredictorVars %end; %else %do; %do j=1 %to &nix_blocks; &&PredictorVars&j %end; %end;;
		run;
	
		/*
		* drop with long list takes a long time so use chunks;
		%if &nixl %then %do;
			data &b1;
				set &b1;
				drop
					%do i=1 %to &nixl; %sysfunc(varname(&_dsixl_,&i)) %end; 
				;
			run;
		%end;
		%if &nix %then %do;
			%let another = 1;
			%let last = 0;
			%let nchunk = 10000;
			%let b2 = b2;
			%do %while(&another);
				%let first = %eval(&last + 1);
				%let last = %eval(&first + &nchunk - 1);
				%if &last >= &nix %then %do;
					%let last = &nix;
					%let another = 0;
				%end;
				%put first=&first last=&last;
				data &b2;
					set &b1;
					drop
						%do i=&first %to &last; %sysfunc(varname(&_dsix_,&i)) %end; 
					;
				run;
				%let temp = &b1;
				%let b1 = &b2;
				%let b2 = &temp;
			%end;
			%let b1 = &b2;
		%end;
		*/
		
		data &b1;
			merge &b1 b1tr;
		run;
		
		* separate training and test combined above;
		%if %length(&intestdat) and &intestdat^=d %then %do;
			data d &intestdat;
				set &b1;
				if _n_ <= &ntrain then output d;
				else output &intestdat;
			run;
		%end;
		
		%if (&nixl > 0) %then %do;
			%let _rc_ = %sysfunc(close(&_dsixl_));
			%let nixl = 0;
		%end;
		%if (&nix > 0) %then %do;
			%let _rc_ = %sysfunc(close(&_dsix_));
		%end;

		data dsix;
			set d;
			if (0);
			keep Mah:;
		run;

		%let _dsix_ = %sysfunc(open(dsix));
		%let nix = &nsv;

		%if ^&upred %then %do;	
			%let PredictorVars = Mah:;
		%end; 
		%else %do;
			%let PredictorVars1 = Mah:;
			%do j=2 %to &nix_blocks;
				%let PredictorVars&j = ;
			%end;
		%end;		
		%let ClassPredVars = ;
		%let LockContPredVars= ;

		/*
		data b2;
			set b1;
			drop
				%do i=1 %to &nixl; %sysfunc(varname(&_dsixl_,&i)) %end; 
				%do i=1 %to &nix; %sysfunc(varname(&_dsix_,&i)) %end;
			;
		run;
		data b1;
			merge b2 b1tr;
		run;
		%NObsVars(b1tr);
		%let nv = &nvars;
		%put nv = &nv;
		*/
	%end;	
%end;

%if (&ImputeZeros = Yes) and (&nixl or &nix) %then %do;
	data d;
		set d;
		array pv{*} 
			%do i=1 %to &nixl; %sysfunc(varname(&_dsixl_,&i)) %end; 
			%do i=1 %to &nix; %sysfunc(varname(&_dsix_,&i)) %end;
		;
		do _i=1 to dim(pv);
			if missing(pv[_i]) then pv[_i] = 0;
		end;
		drop _i;
	run;

	%if %length(&intestdat) & &intestdat^=d %then %do;
		data &intestdat;
			set &intestdat;
			array pv{*} 
				%do i=1 %to &nixl; %sysfunc(varname(&_dsixl_,&i)) %end; 
				%do i=1 %to &nix; %sysfunc(varname(&_dsix_,&i)) %end;
			;
			do _i=1 to dim(pv);
				if missing(pv[_i]) then pv[_i] = 0;
			end;
			drop _i;
		run;
	%end;
%end;

%if &syserr > 4 %then %goto exit;

ods exclude all;
ods noresults;

* Statistical Testing for Predictor Variable Reduction;
%if (&StatTest = Yes) %then %do;

	%put Statistical Testing for Predictor Reduction started: &sysdate &systime;
	
	* if depvar is continuous, nominalize it if necessary;
	%let UseDep = &DepVar;
	%let Nominalize = No;
	%if (&_dtype_ = N) and (&NumDepAsClass = No) %then %do;
		%if %index(%quote(&ST_Method),%quote(Shrunken)) or %index(%quote(&ST_Method),%quote(Unequal)) or
			%index(%quote(&ST_Method),%quote(Rank)) or %index(%quote(&ST_Method),%quote(Median)) or 
			%index(%quote(&ST_Method),%quote(Robust)) %then %do;
			%let Nominalize = Yes;
		%end;
		%if &Nominalize=Yes %then %do;
			%let ncl = 7;
			%if %length(&DepVar) > 30 %then %let UseDep = %substr(&DepVar,1,30)_&ncl;
			%else %let UseDep = &DepVar._&ncl;
			proc rank data=d out=d groups=&ncl;
				var &DepVar;
				ranks &UseDep;
			run;
			%do i=1 %to &ncl;
				%let lev&i = &i;
			%end;
		%end;
		%else %do;
			%let ncl = 1;
		%end;
	%end;
	%else %do;
		data dep;
			set d;
			keep &DepVar;
			where not missing(&DepVar);
		run;
		proc sort data=dep out=depn nodupkey;
			by &DepVar;
		run;
		data _null_;
			set depn end=last;
			call symput("lev"||trim(left(_n_)),trim(left(&DepVar)));
			if last then call symput("ncl",trim(left(_n_)));
		run;
		%if (&_dtype_ = C) %then %do;
			%if %index(%quote(&ST_Method),Correlation) %then %do;
				%let UseDep = &_c1;
				%do i=2 %to %eval(&nic-1);
					%let UseDep = &UseDep &&_c&i;
				%end;
			%end;
			%else %if ((%index(%quote(&ST_Method),Radial) or %index(%quote(&ST_Method),Quadratic))) and 
				%length(&ClassVars) %then %do;
				%let UseDep = &_c1;
			%end;
		%end;
	%end;
			
	%if &MultipleTestingMethod = None %then %let MultipleTestingMethod = ;
	%if (%eval(&nix + &nip) > &ST_Max) or (&RecoverClassPredVars = Yes) %then %do;

		%include "&MacroPath./MultipleTestingAdjustment.sas" / nosource;
		* create names of adjusted p-values computed by PROC MULTTEST;
		%AdjPValueName(method=&MultipleTestingMethod);
		
		%let sterrmsg = did not complete successfully.  If it ran out of memory, try reducing the number of predictors;
		%let sterrmsg = &sterrmsg with filtering or use a different statistical testing method.;

		* tests for continuous predictors;
		%if (&nix) %then %do;

			* chunk the data for I/O performance;
			%if %index(%quote(&ST_Method),Shrunken) %then %let chunksize = 1000000;
			%else %let chunksize = 10000;
			%let startv = 1;
			%let endv = %eval(&startv + &chunksize - 1);
			%if (&endv > &nix) %then %let endv = &nix;
			%let nclm1 = %eval(&ncl-1);
			%let morevars = 1;
			
			data opd1;
			run;
						
			%do %while(&morevars=1);

				%put Computing statistical tests for predictor vars &startv to &endv out of &nix;

				data d1;
					set d;
					%do ii=1 %to %sysfunc(countw(&UseDep));
						if missing(%scan(&UseDep,&ii,' ')) then delete;
					%end;
					/*
					keep &UseDep %do i=&startv %to &endv; %sysfunc(varname(&_dsix_,&i)) %end; 
						%if &ProcessName = SurvivalPredictiveModeling %then %do;
							&CensorVar &Covariates &WeightVar &ColorVar;
						%end;
					;
					*/
				run;
				
				%if %index(%quote(&ST_Method),Correlation) %then %do;
					%let ctype = %scan(&ST_Method,1,' ');
					proc corr data=d1 rank %if &ST_Max %then %do; best=&ST_Max %end; &ctype;
						var %do i=&startv %to &endv; %sysfunc(varname(&_dsix_,&i)) %end; ;
						with &UseDep;
						ods output &ctype.corr=corr;
					run;

					data opd1;
						set corr;
						array best{*} best:;
						array r{*} r:;
						array p{*} p:;
						do i=1 to dim(best);
							_name_ = best[i];
							Correlation = r[i];
							Difference = Correlation;
							raw_p = p[i];
							output;
						end;
						keep _name_ Correlation Difference raw_p;
					run;
				
				%end;

				%else %if (%quote(&ST_Method) = %quote(T-Test)) and &Nominalize=Yes %then %do;

					proc multtest data=d1 out=opd1;
						class &UseDep;
						test mean(%do i=&startv %to &endv; %sysfunc(varname(&_dsix_,&i)) %end;);
						%do i=2 %to &ncl;
							contrast "&&lev&i - &lev1" -1 %do j=1 %to %eval(&i-2); 0 %end; 1;
						%end;
					run;

					%if (&syserr > 4) %then %do;
						%put ERROR: PROC MULTTEST &sterrmsg;
						%goto exit;
					%end;

					* get number of nonmissing observations for computing adjusted statistics;
					%NObsVars(d1);

					data opd1;
						length _name_ $ 32;
						set opd1;
						_name_ = _var_;
						_value_ = (&ncl-1)*_value_/&nobs;
						_se_ = (&ncl-1)*_se_/&nobs;
						rename _value_=Difference _se_=StdErr; 
						label _value_="Difference" _se_="Standard Error";
						drop _var_ _test_;
					run;
				%end;

				%else %if ((%quote(&ST_Method) = %quote(T-Test)) and &Nominalize=No) or
					%index(%quote(&ST_Method),%quote(Unequal)) or %index(%quote(&ST_Method),%quote(Asymmetric))
					%then %do;
					
					data stack;
						length _name_ $ 32;
						set d1;
						%do i=&startv %to &endv; 
							_name_ = "%sysfunc(varname(&_dsix_,&i))";
							_y = %sysfunc(varname(&_dsix_,&i));
							output;
						%end; 
						keep _name_ &UseDep _y;
					run;
					proc sort data=stack;
						by _name_;
					run;	
					
					%if ^%length(&ST_AsymmetricLossFitProp) %then %let ST_AsymmetricLossFitProp = 0.5;
					
					* set up for asymmetric loss iteration loop, implementing iteratively reweighted; 
					* maximum likelihood as in Efron (1991) "Regression Percentiles" Statistica Sinica;
					%if &ST_AsymmetricLossFitProp ne 0.5 %then %do;
						%if %length(&WeightVar) %then %do;
							%let ST_WeightVar0 = &WeightVar;
						%end;
						%else %do;
							%let ST_WeightVar0 = ;
						%end;
						%let ST_WeightVar = _Asymmetric;
						data stack;
							set stack;
							_Row_ = _n_;
							&ST_WeightVar = 0.5;
							%if %length(&ST_WeightVar0) %then %do;
								&ST_WeightVar = &ST_WeightVar * &ST_WeightVar0;
							%end;
						run;
					%end;
					%let aloss_iterating = 1;
					%let aloss_niter = 1;
					%let aloss_maxiter = 30;

					* asymmetric loss loop, if loss is symmetric then this just gets executed once;
					%do %while(&aloss_iterating);

						%if &ST_AsymmetricLossFitProp ne 0.5 %then %do;
							%put Beginning statistical testing asymmetric loss iteration &aloss_niter;
						%end;
						
						/*
						S1403219: When the same predictor as continuous and class,
						new variables are created for each level of the class predictor, which is then used in the analysis. However,
						the original class variables names were still used to produce the stack data, and because the original variables
						are no longer in the data set used to build the stack data, the original variables will be created with missing
						on all entries. The code below removes by groups with missing values on all rows. 
						*/
						proc means data=stack noprint;
							var _y;
							by _name_;
							output out=mean_y(rename=(_name_=name)) mean=_mean_;
						run;
						
						data mean_y_miss;
							set mean_y;
							where missing(_mean_);
						run;
						
						%NOBsVars(mean_y_miss);
						%if &nobs > 0 %then %do;
						
							proc sql;
								create table stackednew as
								select * 
								from stack as l left join mean_y as r
								on l._name_=r.name;
							quit;
							
							data stack;
								set stackednew;
								where not missing(_mean_);
								drop name _type_ _freq_ _mean_;
							run;
							
							proc datasets nolist;
								delete stackednew;
							quit;
								
						%end;
						
						proc datasets nolist;
							delete mean_y mean_y_miss;
						quit;
			
						%ShowTime(PROC MIXED started:);
						%if &Nominalize=Yes or &_dtype_ = C %then %do;
							proc mixed data=stack;
								by _name_;
								class &UseDep;
								model _y = &UseDep 
									%if &ST_AsymmetricLossFitProp ne 0.5 %then %do;
										/ outp=preda
									%end;
								;
								%if %index(%quote(&ST_Method),%quote(Unequal)) %then %do;
									repeated / group=&UseDep;
								%end;
								%if &ST_AsymmetricLossFitProp ne 0.5 %then %do;
									weight &ST_WeightVar;
								%end;
								lsmeans &UseDep / diff;
								ods output diffs=opd1;
							run;
						%end;
						%else %do;
							proc mixed data=stack;
								by _name_;
								model &UseDep = _y
									%if &ST_AsymmetricLossFitProp ne 0.5 %then %do;
										/ outp=preda
									%end;
								;
								%if &ST_AsymmetricLossFitProp ne 0.5 %then %do;
									weight &ST_WeightVar;
								%end;
								estimate "slope" _y 1;
								ods output estimates=opd1;
							run;					
						%end;
						%ShowTime(PROC MIXED finished:);

						%if (&syserr > 4) %then %do;
							%put ERROR: PROC MIXED &sterrmsg;
							%goto exit;
						%end;

						* asymmetric loss calculations;
						%if &ST_AsymmetricLossFitProp = 0.5 %then %do;
							%let aloss_iterating = 0;
						%end;
						%else %do;
							* update asymmetric weights, if they have not changed then stop iterating;
							%let nchanged = 0;
							data preda;
								set preda end=_last;
								retain nchanged 0;
								_wprev = &ST_WeightVar;
								_resid = &UseDep - Pred;
								if _resid >= 0 then &ST_WeightVar = &ST_AsymmetricLossFitProp;
								else &ST_WeightVar = 1 - &ST_AsymmetricLossFitProp;
								%if %length(&ST_WeightVar0) %then %do;
									&ST_WeightVar = &ST_WeightVar * &ST_WeightVar0;
								%end;
								if _wprev ne &ST_WeightVar then nchanged = nchanged + 1;
								if _last then call symputx('nchanged',trim(left(nchanged)));
								keep _Row_ &ST_WeightVar;
							run;
							%put nchanged in statistical testing asymmetric iteration &aloss_niter = &nchanged;
							%if ^&nchanged %then %do;
								%let aloss_iterating = 0;
							%end;
							%else %do;
								data stack;
									set stack;
									drop &ST_WeightVar;
								run;
								data stack;
									merge stack preda;
									by _Row_;
								run;
							%end;
						%end;
						%if &aloss_niter = &aloss_maxiter %then %let aloss_iterating = 0;
						%if &aloss_iterating %then %do;
							%let aloss_niter = %eval(&aloss_niter + 1);
						%end;
					%end;

					%if &ST_AsymmetricLossFitProp ne 0.5 %then %do;
						%if &aloss_niter = &aloss_maxiter %then %do;
							%put ERROR: Statistical testing asymmetric loss calculations did not converge after &aloss_maxiter iterations.;
							%put You may want to try a fitting proportion different from &ST_AsymmetricLossFitProp..;
							%let exiterror = 1;
							%goto exit;
						%end;
						%else %do;
							%put Statistical testing asymmetric loss calculations converged in &aloss_niter iterations.;
						%end;
					%end;
					
					data opd1;
						set opd1;
						rename Estimate=Difference Probt=raw_p;
					run;
					
				%end;

				%else %if %index(%quote(&ST_Method),Quantile) %then %do;
					data stack;
						length _name_ $ 32;
						set d1;
						%do i=&startv %to &endv; 
							_name_ = "%sysfunc(varname(&_dsix_,&i))";
							_x = %sysfunc(varname(&_dsix_,&i));
							output;
						%end; 
						keep _name_ &UseDep _x;
					run;
					proc sort data=stack;
						by _name_;
					run;
					
					%ShowTime(PROC QUANTSELECT started:);
					/*
					%if ^%symexist(Seed) %then %let Seed = 12345;
					%if ^%length(&Seed) %then %let Seed = 12345;
					*/
					%if &Nominalize=Yes %then %do;
					
						proc quantselect data=stack namelen=128;
							by _name_;
							class &UseDep;
							model _x = &UseDep / quantiles=&ST_QuantileLevel selection=forward(select=sl maxstep=1 slentry=0.99);
							ods output selectionsummary=opd1;
						run;
						
						/*
						proc quantreg data=stack ci=resampling;
							by _name_;
							class &UseDep;
							model _x = &UseDep / quantile=&ST_QuantileLevel seed=&Seed;
							ods output parameterestimates=opd1;
						run;
						*/
					%end;
					%else %do;
					
						proc quantselect data=stack namelen=128;
							by _name_;
							model &UseDep = _x / quantiles=&ST_QuantileLevel selection=forward(select=sl maxstep=1 slentry=0.99);
							ods output selectionsummary=opd1;
						run;
						
						/*
						proc quantreg data=stack ci=resampling;
							by _name_;
							model &UseDep = _x / quantile=&ST_QuantileLevel seed=&Seed;
							test _x / lr;
							ods output tests=opd1;
							* ods output parameterestimates=opd1;
						run;
						*/
					%end;
					%ShowTime(PROC QUANTSELECT finished:);

					%if (&syserr > 4) %then %do;
						%put ERROR: PROC QUANTSELECT &sterrmsg;
						%goto exit;
					%end;
					
					/*
					data OutLib.opd1;
						set opd1;
					run;
					*/
					
					data opd1;
						set opd1;
						if EffectEntered="Intercept" then delete;
						rename lrpvalue=raw_p;
					run;

					/*
					data opd1;
						set opd1;
						if Parameter="Intercept" then delete;
						rename Estimate=Difference Probt=raw_p;
					run;
					*/

				%end;
				
				%else %if %index(%quote(&ST_Method),Shrunken) %then %do;
				
					* set up to call OneWay macro;
					%global ExpInData AnnoInData GroupVars BlockVars InClassVars 
						MultipleTestingMethod EBShrink LSMeansDiffSet LSMeanStdMethod; 
					
					* create tall and ed data sets from wide data;
					proc transpose data=d1 out=OutLib.tall;
						var %do i=&startv %to &endv; %sysfunc(varname(&_dsix_,&i)) %end; ;
					run;
					
					%if %length(&UseDep) > 24 %then %do;
						%let UseDep1 = %substr(&UseDep,1,24);
					%end;
					%else %do;
						%let UseDep1 = &UseDep;
					%end;

					data OutLib.ed;
						length ColumnName $ 32;
						set d1;
						&UseDep1 = &UseDep;
						Array = _n_;
						ColumnName = "COL" || trim(left(_n_));
						keep &UseDep1 Array ColumnName;
					run;
	
					* assign macro vars;	
					%let InDataSave = &InData;
					%let ClassVarsSave = &ClassVars;
					
					ods exclude all;
					ods noresults;
					
					proc contents data=OutLib.tall;
						ods output enginehost=OutLib.eng;
					run;
										
					data _null_;
						set OutLib.eng;
						if Label1="Filename" then call symputx("InData",cValue1);
					run;
										
					proc contents data=OutLib.ed;
						ods output enginehost=eng;
					run;
					
					data _null_;
						set eng;
						if Label1="Filename" then call symputx("ExpInData",cValue1);
					run;
					
					%let suffix = ;
					%let AnnoInData = ;
					%let AnnoWhereStmt = ;
					%let AnnoOnly = ;
					%let GroupVars = &UseDep1;
					%let BlockVars = ;
					%let InClassVars = ;
					%let EBShrink = yes;
					%let LSMeansDiffSet = None;
					%let OutData = opd1;
					%let OneWayLaunchJSL = 0;
					

					* call OneWay macro;
					%ShowTime(OneWay started:);
					%OneWay;
					%ShowTime(OneWay finished:);
					
					%if (&syserr > 4) %then %do;
						%put ERROR: OneWay macro &sterrmsg;
						%goto exit;
					%end;
					
					%let InData = &InDataSave;
					%let ClassVars = &ClassVarsSave;
					
					data opd1;
						set OutLib.opd1;
						rename PrF_T3_&UseDep1=raw_p;
					run;
					
					* sort by descending F statistic since very small p-values may be identical;
					proc sort data=opd1;
						by descending F_T3_&UseDep1;
					run;
			
				%end;
								
				%else %if %index(%quote(&ST_Method),Radial) %then %do;
				
					/*
					%if &_dtype_ ^= N %then %do;
						%put ERROR: Radial Basis testing is not available for character dependent variables.;
						%let exiterror = 1;
						%goto exit;
					%end;
					*/
					
					* fit of null model for likelihood ratio test;
					proc glimmix data=d1;
						model &UseDep = ;
						ods output fitstatistics=opd0;
					run;
										
					data _null_;
						set opd0;
						if Descr = "-2 Res Log Likelihood" then call symput("m2rll0",trim(left(value)));
					run;
					%put m2rll0 = &m2rll0;
					
					data stack;
						length _name_ $ 32;
						set d1;
						%do i=&startv %to &endv; 
							_name_ = "%sysfunc(varname(&_dsix_,&i))";
							_y = %sysfunc(varname(&_dsix_,&i));
							output;
						%end; 
						keep _name_ &UseDep _y &WeightVar;
					run;
					proc sort data=stack;
						by _name_;
					run;
					
					* the glimmix call with a lot of by groups can be slow;
					%ShowTime(PROC GLIMMIX started:);
					proc glimmix data=stack;
						by _name_;
						model &UseDep = ;
						random _y / type=rsmooth knotmethod=data(stack); 
						%if %length(&WeightVar) %then %do;
							weight &WeightVar;
						%end;
						ods output fitstatistics=opd1;
					run;
					%ShowTime(PROC GLIMMIX finished:);

					%if (&syserr > 4) %then %do;
						%put ERROR: PROC GLIMMIX &sterrmsg;
						%goto exit;
					%end;
					
					* likelihood ratio test;
					data opd1;
						set opd1;
						if Descr = "-2 Res Log Likelihood";
						Difference = &m2rll0 - Value;
						if (Difference <= 0) then raw_p = 1;
						* divide lrt by 2 because test is 1 df and on the boundary;
						else raw_p = (1 - probchi(Difference,1))/2;
					run;
					
				%end;
				
				%else %if %index(%quote(&ST_Method),Quadratic) %then %do;;
					data stack;
						length _name_ $ 32;
						set d1;
						%do i=&startv %to &endv; 
							_name_ = "%sysfunc(varname(&_dsix_,&i))";
							x = %sysfunc(varname(&_dsix_,&i));
							x2 = x*x;
							output;
						%end; 
						keep _name_ &UseDep x x2;
					run;
					proc sort data=stack;
						by _name_;
					run;
					
					%ShowTime(PROC REG started:);
					proc reg data=stack;
						by _name_;
						model &UseDep = x x2 ; 
						ods output parameterestimates=opd1;
					run;

					%ShowTime(PROC REG finished:);

					%if (&syserr > 4) %then %do;
						%put ERROR: PROC REG &sterrmsg;
						%goto exit;
					%end;

					data opd1;
						set opd1;
						if variable = "Intercept" then delete;
						* if variable = "x" then delete;
						rename Estimate=Difference Probt=raw_p;
					run;
					
				%end;
				
				%else %if %index(%quote(&ST_Method),Rank) or %index(%quote(&ST_Method),Median) %then %do;

					%if %index(%quote(&ST_Method),Median) %then %do;
						%let npmethod = MEDIAN;
						%let npstat = _CHMED_;
						%let nppval = P_CHMED;
					%end;
					%else %do;
						%let npmethod = WILCOXON;
						%let npstat = _KW_;
						%let nppval = P_KW;
					%end;

					proc npar1way data=d1 &npmethod;
						class &UseDep;
						/* var %do i=&startv %to &endv; %sysfunc(varname(&_dsix_,&i)) %end; ; */
						output out=opd1;
					run;

					%if (&syserr > 4) %then %do;
						%put ERROR: PROC NPAR1WAY &sterrmsg;
						%goto exit;
					%end;
					data opd1;	
						length _name_ $ 32;
						set opd1;
						Difference = &npstat;
						raw_p = &nppval;
						_name_ = _var_;
						drop _var_;
					run;					

				%end;
				
				%else %if %index(%quote(&ST_Method),Robust) %then %do;

					* PROC ROBUSTREG runs out of memory with a large number of BY groups in 9.1.3, so cannot handle large chunks;
					* This is fixed in 9.2;

					* PROC ROBUSTREG does not accept CLASS effects for S Estimation, so create indicator variables using PROC GLMMOD;
					data d1a;
						set d1;
						_one = 1;
						keep _one &UseDep;
					run;

					proc glmmod data=d1a prefix=diff outdesign=d1b;
						class &UseDep;
						model _one = &UseDep / noint;
					run;
					
					data d2;
						merge d1b d1;
					run;

					data stack;
						length _name_ $ 32;
						set d2;
						%do i=&startv %to &endv; 
							_name_ = "%sysfunc(varname(&_dsix_,&i))";
							_y = %sysfunc(varname(&_dsix_,&i));
							output;
						%end; 
						keep _name_ _y diff1-diff&nclm1;
					run;
					proc sort data=stack;
						by _name_;
					run;
					
					* call robustreg;
					%ShowTime(PROC ROBUSTREG started:);
					proc robustreg data=stack method=mm;
						by _name_;
						model _y = diff1-diff&nclm1 ;
						ods output parameterestimates=opd1;
					run;
					%ShowTime(PROC ROBUSTREG finished:);

					%if (&syserr > 4) %then %do;
						%put ERROR: PROC ROBUSTREG &sterrmsg;
						%goto exit;
					%end;

					* save only the indicator effects;
					data opd1;
						set opd1;
						where parameter in (%do i=1 %to &nclm1; "DIFF&i", %end; "dummystring");
						rename Estimate=Difference ProbChiSq=raw_p;
					run;
				%end;	
				
				%else %if %index(%quote(&ST_Method),Life) %then %do;
					data stack;
						length _name_ $ 32;
						set d1;
						%do i=&startv %to &endv; 
							_name_ = "%sysfunc(varname(&_dsix_,&i))";
							_x = %sysfunc(varname(&_dsix_,&i));
							output;
						%end; 
						keep _name_ &DepVar &CensorVar &Covariates &WeightVar _x 
							%do i=1 %to &nixl; %sysfunc(varname(&_dsixl_,&i)) %end; 
							%do i=1 %to &nipl; %sysfunc(varname(&_dsipl_,&i)) %end; 	
						;
					run;

					proc sort data=stack;
						by _name_;
					run;
				
					* options nonotes;
					%ShowTime(PROC LIFEEG started:);
					proc lifereg data=stack %if %length(&LifeRegOptions) %then &LifeRegOptions;;
						by _name_; 
						%if %length(&WeightVar) %then %do;
							Weight &WeightVar;
						%end;
						%if &nipl %then %do;
							class %do i=1 %to &nipl; %sysfunc(varname(&_dsipl_,&i)) %end; ;
						%end;
						%if %length(&CensorVar) %then %do; 
							model &DepVar*&CensorVar(&CensorValues) = 
								%do i=1 %to &nixl; %sysfunc(varname(&_dsixl_,&i)) %end; 
								%do i=1 %to &nipl; %sysfunc(varname(&_dsipl_,&i)) %end; 	
								_x
								/ distribution=&Distribution &LifeRegModelOptions; 
							;
						%end;
						%else %do;
							model &DepVar = 
								%do i=1 %to &nixl; %sysfunc(varname(&_dsixl_,&i)) %end; 
								%do i=1 %to &nipl; %sysfunc(varname(&_dsipl_,&i)) %end; 	
								_x
								/ dist=&Distribution &LifeRegModelOptions; 
							;
						%end;	
						ods output parameterestimates=opd1;
					run;
					%ShowTime(PROC LIFEREG finished:);

					%if (&syserr > 4) %then %do;
						%put ERROR: PROC LIFEREG &sterrmsg;
						%goto exit;
					%end;

					data opd1;
						set opd1;
						if Parameter = "_x";
						rename Estimate=Difference ProbChiSq=raw_p;
					run;

				%end;

				%else %if %index(%quote(&ST_Method),Cox) %then %do;
					* added in v4.1 for survival data;
					data stack;
						length _name_ $ 32;
						set d1;
						%do i=&startv %to &endv; 
							_name_ = "%sysfunc(varname(&_dsix_,&i))";
							_x = %sysfunc(varname(&_dsix_,&i));
							output;
						%end; 
						keep _name_ &DepVar &CensorVar &Covariates &WeightVar _x 
							%do i=1 %to &nixl; %sysfunc(varname(&_dsixl_,&i)) %end; 
							%do i=1 %to &nipl; %sysfunc(varname(&_dsipl_,&i)) %end; 	
						;
					run;

					proc sort data=stack;
						by _name_;
					run;
				
					* options nonotes;
					%ShowTime(PROC PHREG started:);
					proc phreg data=stack %if %length(&PhregOptions) %then &PhregOptions;;
						by _name_; 
						%if %length(&WeightVar) %then %do;
							Weight &WeightVar;
						%end;
						%if &nipl %then %do;
							class %do i=1 %to &nipl; %sysfunc(varname(&_dsipl_,&i)) %end; / missing;
						%end;
						%if %length(&CensorVar) %then %do; 
							model &DepVar*&CensorVar(&CensorValues) = 
								%do i=1 %to &nixl; %sysfunc(varname(&_dsixl_,&i)) %end; 
								%do i=1 %to &nipl; %sysfunc(varname(&_dsipl_,&i)) %end; 	
								_x
								%if %length(PhregModelOptions) %then / &PhregModelOptions; 
							;
						%end;
						%else %do;
							model &DepVar = 
								%do i=1 %to &nixl; %sysfunc(varname(&_dsixl_,&i)) %end; 
								%do i=1 %to &nipl; %sysfunc(varname(&_dsipl_,&i)) %end; 	
								_x
								%if %length(PhregModelOptions) %then / &PhregModelOptions; 
							;
						%end;	
						ods output parameterestimates=opd1;
					run;
					%ShowTime(PROC PHREG finished:);

					%if (&syserr > 4) %then %do;
						%put ERROR: PROC PHREG &sterrmsg;
						%goto exit;
					%end;

					data opd1;
						set opd1;
						if Parameter = "_x";
						rename Estimate=Difference ProbChiSq=raw_p;
					run;

				%end;
			
				%if (&startv=1) %then %do;
					data opd;
						length _name_ $ 32;
						set opd1;
					run;
				%end;
				%else %do;
					data opd;
						set opd opd1;
					run;
				%end;

				%if &endv = &nix %then %do;
					%let morevars = 0;
				%end;
				%else %do;
					%let startv = %eval(&endv+1);
					%let endv = %eval(&startv + &chunksize - 1);
					%if (&endv > &nix) %then %let endv = &nix;
				%end;
					
			%end;

		%end;
		
		* statistical tests for class predictor variables;
		%if (&nip) %then %do;

			* if depvar is continuous, discretize it;
			%let UseDep = &DepVar;
			%if (&_dtype_ = N) and (&NumDepAsClass = No) %then %do;
				%let ncl = 7;
				%if %length(&DepVar) > 30 %then %let UseDep = %substr(&DepVar,1,30)_&ncl;
				%else %let UseDep = &DepVar._&ncl;
				proc rank data=d out=d groups=&ncl;
					var &DepVar;
					ranks &UseDep;
				run;
				%do i=1 %to &ncl;
					%let lev&i = &i;
				%end;
			%end;

			* chunk the data for I/O performance;
			%let chunksize = 20000;
			%let startv = 1;
			%let endv = %eval(&startv + &chunksize - 1);
			%if (&endv > &nip) %then %let endv = &nip;
			%let nclm1 = %eval(&ncl-1);
			%let morevars = 1;
			data opd1;
			run;
						
			%do %while(&morevars=1);

				%put Computing statistical tests for class predictor vars &startv to &endv out of &nip;

				/*
				data d1;
					set d;
					keep &UseDep &ClassPredVars %do i=&startv %to &endv; %sysfunc(varname(&_dsip_,&i)) %end;  
						%if &ProcessName = SurvivalPredictiveModeling %then %do;
							&CensorVar &Covariates &ColorVar
						%end;
					;
				run;
				*/
				
				%if ^%index(%quote(&ST_Method),Cox) and ^%index(%quote(&ST_Method),Life) %then %do;
					* Fisher exact test for class predictors;
					proc multtest data=d out=opd1;
						class &UseDep;
						test fisher(%do i=&startv %to &endv; %sysfunc(varname(&_dsip_,&i)) %end;);
						%do i=2 %to &ncl;
							contrast "&&lev&i - &lev1" -1 %do j=1 %to %eval(&i-2); 0 %end; 1;
						%end;
					run;

					%if (&syserr > 4) %then %do;
						%put ERROR: PROC MULTTEST &sterrmsg;
						%goto exit;
					%end;	

					data opd1;
						length _name_ $ 32;
						set opd1;
						_name_ = _var_;
						if (_xval_ ne .) and (_yval_ ne .) and (_mval_ > 0) and (_nval_ > 0) then 
							PropDiff = _xval_/_mval_ - _yval_/_nval_;
						else PropDiff = .;
						label PropDiff="Proportion Diff";
						drop _var_ _test_;
					run;
				%end;
				%else %if %index(%quote(&ST_Method),Cox) %then %do; 	
					* added in v4.1 for survival data;
					data stack;
						length _name_ $ 32;
						set d;
						%do i=&startv %to &endv; 
							_name_ = "%sysfunc(varname(&_dsip_,&i))";
							_x = %sysfunc(varname(&_dsip_,&i));
							output;
						%end; 
						keep _name_ &DepVar &CensorVar &Covariates _x
							%do i=1 %to &nixl; %sysfunc(varname(&_dsixl_,&i)) %end; 
							%do i=1 %to &nipl; %sysfunc(varname(&_dsipl_,&i)) %end; 
						;
					run;

					proc sort data=stack;
						by _name_;
					run;
				
					* options nonotes;
					%ShowTime(PROC PHREG started:);
					proc phreg data=stack %if %length(&PhregOptions) %then &PhregOptions;;
						by _name_; 
						class 
							%do i=1 %to &nipl; %sysfunc(varname(&_dsipl_,&i)) %end; 	
							_x / missing;
						;	
						%if %length(&WeightVar) %then %do;
							Weight &WeightVar;
						%end;
						%if %length(&CensorVar) %then %do; 
							model &DepVar*&CensorVar(&CensorValues) = 
								%do i=1 %to &nixl; %sysfunc(varname(&_dsixl_,&i)) %end; 
								%do i=1 %to &nipl; %sysfunc(varname(&_dsipl_,&i)) %end; 	
								_x
								 / &PhregModelOptions; 
							;
						%end;
						%else %do;
							model &DepVar = 
								%do i=1 %to &nixl; %sysfunc(varname(&_dsixl_,&i)) %end; 
								%do i=1 %to &nipl; %sysfunc(varname(&_dsipl_,&i)) %end; 	
								_x
								/ &PhregModelOptions; 
							;
						%end;	
						ods output parameterestimates=opd1;
					run;
					%ShowTime(PROC PHREG finished:);

					%if (&syserr > 4) %then %do;
						%put ERROR: PROC PHREG &sterrmsg;
						%goto exit;
					%end;

					data opd1;
						set opd1;
						if Parameter = "_x";
						rename Estimate=Difference;
					run;

					%if (&syserr > 4) %then %do;
						%put ERROR: PROC PHREG &sterrmsg;
						%goto exit;
					%end;
				%end;

				%else %if %index(%quote(&ST_Method),Life) %then %do; 	
					* added in v4.1 for survival data;
					data stack;
						length _name_ $ 32;
						set d1;
						%do i=&startv %to &endv; 
							_name_ = "%sysfunc(varname(&_dsip_,&i))";
							_x = %sysfunc(varname(&_dsip_,&i));
							output;
						%end; 
						keep _name_ &DepVar &CensorVar &Covariates _x
							%do i=1 %to &nixl; %sysfunc(varname(&_dsixl_,&i)) %end; 
							%do i=1 %to &nipl; %sysfunc(varname(&_dsipl_,&i)) %end; 
						;
					run;

					proc sort data=stack;
						by _name_;
					run;
				
					* options nonotes;
					%ShowTime(PROC LIFEREG started:);
					proc lifereg data=stack %if %length(&LifeRegOptions) %then &LifeRegOptions;;
						by _name_; 
						class 
							%do i=1 %to &nipl; %sysfunc(varname(&_dsipl_,&i)) %end; 	
							_x ;
						;	
						%if %length(&WeightVar) %then %do;
							Weight &WeightVar;
						%end;
						%if %length(&CensorVar) %then %do; 
							model &DepVar*&CensorVar(&CensorValues) = 
								%do i=1 %to &nixl; %sysfunc(varname(&_dsixl_,&i)) %end; 
								%do i=1 %to &nipl; %sysfunc(varname(&_dsipl_,&i)) %end; 	
								_x
								/ dist=&Distribution &LifeRegModelOptions;
							;
						%end;
						%else %do;
							model &DepVar = 
								%do i=1 %to &nixl; %sysfunc(varname(&_dsixl_,&i)) %end; 
								%do i=1 %to &nipl; %sysfunc(varname(&_dsipl_,&i)) %end; 	
								_x
								/ dist=&Distribution &LifeRegModelOptions;
							;
						%end;	
						ods output parameterestimates=opd1;
					run;
					%ShowTime(PROC LIFEREG finished:);

					%if (&syserr > 4) %then %do;
						%put ERROR: PROC LIFEREG &sterrmsg;
						%goto exit;
					%end;

					data opd1;
						set opd1;
						if Parameter = "_x";
						rename Estimate=Difference;
					run;

					%if (&syserr > 4) %then %do;
						%put ERROR: PROC LIFEREG &sterrmsg;
						%goto exit;
					%end;
				%end;

				%if (&startv=1) and ^(&nix) %then %do;
					data opd;
						length _name_ $ 32;
						set opd1;
					run;
				%end;
				%else %do;
					data opd;
						set opd opd1;
					run;
				%end;

				%if &endv = &nip %then %do;
					%let morevars = 0;
				%end;
				%else %do;
					%let startv = %eval(&endv+1);
					%let endv = %eval(&startv + &chunksize - 1);
					%if (&endv > &nip) %then %let endv = &nip;
				%end;
				
			%end;
		%end;
				
		* set missing p-values to 1 so they are at the end of the sort;
		data opd;
			set opd;
			if missing(raw_p) then raw_p = 1;
		run;
		
		* p-value adjustment calculations in proc multtest;
		%if %length(&MultipleTestingMethod) %then %do;
			%MultipleTestingAdjustment(data=opd,method=&MultipleTestingMethod,out=opd);
		%end;

		proc sort data=opd;
			by &mtpvar %if &mtpvar ^= Raw_p %then %do; Raw_p %end; ;
		run;

		data opd;
			set opd;
			if (raw_p < 1e-300) then do;
				NegLog10p = 300; 
			end;
			else do;
				NegLog10p = -log10(raw_p);
			end;
			if (&mtpvar < 1e-300) then do;
				NegLog10Adjp = max(300,&NegLog10pCutoff+1);
			end;
			else do;
				NegLog10Adjp = -log10(&mtpvar);
			end;
		run;
				
		/*
		%if &CVFlag %then %do;
			%if %symexist(kval) %then %do;
				%let suffix = _sts_&iter._&kval;
			%end;
			%else %do;
				%let suffix = _sts_&iter;
			%end;
		%end;
		*/
		%if &CVFlag %then %do;
			%if %symexist(kval) %then %do;
				%if %symexist(iter) %then %do;
					%let suffix = _sts_&iter._&kval;
				%end;
				%else %do;
					%let suffix = _sts_&kval;
				%end;
			%end;
			%else %do;
				%if %symexist(iter) %then %do;
					%let suffix = _sts_&iter;
				%end;
				%else %do;
					%let suffix = _sts;
				%end;
			%end;
		%end;
		%else %do;
			%let suffix = _sts;
		%end;
		%let b32 = %eval(32 - %length(&suffix));
		%if %length(&OutName) > &b32 %then %let inname = %substr(&OutName,1,&b32);
		%else %let inname = &OutName;		
		%let stout = %quote(&inname.%trim(%left(&suffix)));
		%put Statistical testing output data set name is &stout;
		
		data OutLib.%unquote(&stout) stat_test;
			set opd;
		run;
				
		* apply filters;
		data _null_;
			if (&NegLog10pCutoff < 0) or (&AbsMeanDiffCutoff < 0) or (&AbsPropDiffCutoff < 0) then do;
				call symput("exiterror",1);
			end;
		run;
		%if &exiterror %then %do;
			%put ERROR: Cutoffs must be nonnegative.;
			%goto exit;
		%end;
		
		%put NegLog10pCutoff = &NegLog10pCutoff;
		%put AbsMeanDiffCutoff = &AbsMeanDiffCutoff;
		%put AbsPropDiffCutoff = &AbsPropDiffCutoff;

		%if (&NegLog10pCutoff > 0) or (&AbsMeanDiffCutoff > 0) or (&AbsPropDiffCutoff > 0) %then %do;
			data stat_test;
				set stat_test;
				%if (&NegLog10pCutoff > 0) %then %do;
					if (NegLog10Adjp >= &NegLog10pCutoff);
				%end;
				%if &ProcessName ^= SurvivalPredictiveModeling %then %do;
					%if &nix and (&AbsMeanDiffCutoff > 0) %then %do;
						if (not missing(Difference) and (abs(Difference) >= &AbsMeanDiffCutoff));
					%end;
					%if &nip and (&AbsPropDiffCutoff > 0) %then %do;
						if (not missing(PropDiff) and (abs(PropDiff) >= &AbsPropDiffCutoff));
					%end;
				%end;	
			run;
		%end;
		
		%NObsVars(stat_test);
		
		%if (&nobs > &ST_Max) or (&RecoverClassPredVars = Yes) %then %do;
			* balance number of sig results among class diffs;
			%if (&ncl > 2) and (%quote(&ST_Method) = %quote(T-Test)) and (&Nominalize = Yes) %then %do;
				%if &ST_Max = 0 %then %let np = %eval(&nix+&nip);
				%else %let np = &ST_Max;
				data _null_;
					call symput("npc",trim(left(ceil(&np/(&ncl-1)))));
				run;
				%put npc = &npc;
				%do i=2 %to &ncl;
					%let conname = &&lev&i - &lev1;
					%if %length(&conname) > 21 %then %let conname = %substr(&conname,1,21);
					data tfa1;
						set stat_test;
						if _contrast_ = "&conname";
					run;
					data tfa1;
						set tfa1(obs=&npc);
					run;
					%if &i=2 %then %do;
						data tfa;
							set tfa1;
						run;
					%end;
					%else %do;
						proc append base=tfa data=tfa1;
						run;
					%end;
				%end;
				data stat_test;
					set tfa;
				run;
			%end;
			data stat_test;
				set stat_test %if &ST_Max %then %do; (obs=&ST_Max) %end; ;
			run;
		%end;
		
		%if (&KMeans=Yes) or (&FR_ImpVars = Yes) %then %let recov = No;
		%else %let recov = &RecoverClassPredVars;
		
		%put recov = &recov;
		
		%ReduceData(d,stat_test,&intestdat,Statistical testing,&recov); 
					
		%if &exiterror %then %goto exit;
	%end;
	%else %do;
		%put Number of predictors is less than the specified maximum, so statistical test filtering is not performed.;
		%let StatTest = No;
	%end;
	
	%put Statistical Testing for Predictor Reduction finished: &sysdate &systime;
	
	/*
	data OutLib.dreduced;
		set d;
	run;
	%put nix = &nix;
	%put nip = &nip;
	*/

	%if ^&CVFlag %then %do;
		ods exclude none;
		ods results;
		%if (&StatTest = Yes) %then %do;
			%if (&nix) %then %do;
				%if (&nip) %then %do;
					%let tstring = %nrbquote(%LocaleFill(%nrbquote(&trXandX), %nrbquote(%LocaleEZX(%quote(&ST_Method), EIA=1)), %nrbquote(&trFisherExactTest)));
				%end;
				%else %do;
					%let tstring = %nrbquote(%LocaleEZX(%quote(&ST_Method), EIA=1));
				%end;
			%end;
			%else %do;
				%let tstring = &trFisherExactTest;
			%end;
			proc sort data=stat_test;
				by descending NegLog10Adjp;
			run;
			%NObsVars(stat_test);
			%let trTitle = %nrbquote(%LocaleFill(%nrbquote(&trTopXPredictorsfromX), &nobs, %nrbquote(&tstring)));
			title "&trTitle";
			proc print data=stat_test;
			run;
			title;
		%end;
	%end;
%end; 

* KMeans Predictor Variable Reduction;
%if (&KMeans = Yes) %then %do;

	%let _tid_   = %sysfunc(open(d));
	%let _nVars_ = %sysfunc(attrn(&_tid_, NVARS));
	%let _nObs_  = %sysfunc(attrn(&_tid_, NOBS));
	%let _tid_ 	 = %sysfunc(close(&_tid_));
	%let _nCluster_ = &KM_Max;

	/*
	%let _nCluster_ = %eval(&KM_Multiple*&_nObs_);
	%if &_nCluster_ > &KM_Max %then %let _nCluster_ = &KM_Max;
	*/

	* call fastclus to compute the k-means;
	%if %eval(&nix + &nip) > &_nCluster_ or 
		((&RecoverClassPredVars = Yes) and %index(&KM_Usage,Representative)) %then %do;
		%if %index(&KM_Usage,Representative) %then %do; 
			%let outseed = ;
		%end;
		%else %do;
			%let outseed = seed;
		%end;
		/* Fix for defect S0758603 */
		data dsipx;
			merge 
			%if &nix %then %do;
				dsix
			%end;
			%if &nip %then %do;
				dsip
			%end;
			;
		run;
		%fastclus(datapath=,
			dsname=d,
			where_clause=,
			quantitative_vars=
				/*
				%do i=1 %to &nix; %sysfunc(varname(&_dsix_,&i)) %end; 
				%do i=1 %to &nip; %sysfunc(varname(&_dsip_,&i)) %end; 
				*/
			,
			qualitative_vars=,
			list_style_quantitative_vars=,
			list_style_qualitative_vars=,
			quantitative_vars_dsname=dsipx,
			maxclusters=&_nCluster_,
			corr_radius=&KM_CorrRadius,
			metadata_variables_dsname=,
			seed_dsname=&outseed
		);

		%if %length(&InData) > 28 %then %let inname = %substr(&InData,1,28);
		%else %let inname = &InData;
		data OutLib.&inname._kmc;
			set clus;
			drop col:;
		run;
		
		%put KM_Usage= &KM_Usage;

		%if %index(&KM_Usage,Representative) %then %do; 
			data postclus;
				set postclus;
				rename name=_name_;
			run;

			%if (&FR_ImpVars = Yes) %then %let recov = No;
			%else %let recov = &RecoverClassPredVars;
			%ReduceData(d,postclus,&intestdat,K-Means,&recov); 
			%if &exiterror %then %goto exit;
		%end;
		%else %do;
			data OutLib.&inname._kms;
				set seed;
				drop col:;
			run;
		
			* replace predictors with kmeans;
			data km;
				set seed;
				keep cluster col:;
			run;
			
			proc transpose data=km out=kmt prefix=KMean;
				var col:;
				id cluster;
			run;
			
			data d;
				merge d kmt;
				drop 
					%do i=1 %to &nix; %sysfunc(varname(&_dsix_,&i)) %end; 
					%do i=1 %to &nip; %sysfunc(varname(&_dsip_,&i)) %end; 
				;
			run;
								
			* compute the same kmeans for the test data set;
			%if %length(&intestdat) and (&intestdat ^= d) %then %do;
				%let _tid_ = %sysfunc(open(&intestdat));
				%let vnum = %sysfunc(varnum(&_tid_, _NAME_));
				%let _tid_ = %sysfunc(close(&_tid_));
				proc transpose data=&intestdat %if &vnum %then %do; (drop=_name_) %end; out=trans name=NAME;
					var 
						%do i=1 %to &nix; %sysfunc(varname(&_dsix_,&i)) %end; 
						%do i=1 %to &nip; %sysfunc(varname(&_dsip_,&i)) %end; 
					;
				run;
			
				proc sort data=trans;
					by NAME;
				run;
				
				proc sort data=OutLib.&inname._kmc out=tm;
					by NAME;
				run;
								
				data tm;
					merge tm trans;
					by NAME;
				run;
				
				proc sort data=tm;
					by cluster;
				run;
				
				%NObsVars(&intestdat);

				proc means data=tm noprint;
					by cluster;
					var col1-col&nobs;
					output out=km mean=col1-col&nobs;
				run;
				
				proc transpose data=km out=kmt prefix=KMean;
					var col1-col&nobs;
					id cluster;
				run;
				
				data &intestdat;
					merge &intestdat kmt;
					drop 
						%do i=1 %to &nix; %sysfunc(varname(&_dsix_,&i)) %end; 
						%do i=1 %to &nip; %sysfunc(varname(&_dsip_,&i)) %end; 
					;
				run;
			%end;

			%if (&nix > 0) %then %do;
				%let _rc_ = %sysfunc(close(&_dsix_));
			%end;
			
			data dsix;
				set kmt;
				if (0);
				keep KMean:;
			run;

			%let _dsix_ = %sysfunc(open(dsix));
			%NObsVars(seed);
			%let nix = &nobs;

			%if (&nip > 0) %then %do;
				%let _rc_ = %sysfunc(close(&_dsip_));
				%let nip = 0;
			%end;
			
			%if ^&upred %then %do;	
				%let PredictorVars = KMean:;
			%end; 
			%else %do;
				%let PredictorVars1 = KMean:;
				%do j=2 %to &nix_blocks;
					%let PredictorVars&j = ;
				%end;
			%end;					
			%let ClassPredVars = ;
			
		%end;		
	%end;
	%else %do;
		%put K-means clustering is not performed because the total number of predictors (%eval(&nix + &nip));
		%put is not greater than the specified number of clusters (&_nCluster_).;
	%end;
%end;


* Use forest to filter predictors and create interaction indicators;
%if (&FR_ImpVars = Yes) or (&FR_Inds = Yes) %then %do;

	%put Predictors from Forest started: &sysdate &systime;

	%if (&TraitType=Continuous)	%then
		%let trait_type = quantitative;
	%else 
		%let trait_type = qualitative;

	%if ^&CVFlag and (&BJ_Est = No or &BJ_OneMore) %then %do;
		%let plib = %scan(&outdat,1,'.');
		%let pdat = %scan(&outdat,2,'.');
		%if %length(&pdat) > 28 %then %do;
			%let pdat = %substr(&pdat,1,28);
		%end;
		%let path = &plib..&pdat._pth;
		%let impdat = &plib..&pdat._ift;
		%if (&TestSpec=Yes) %then %do;
			data b2;
				set d d_test;
			run;
			%let ts2 = b2;
			%put ts2 = &ts2;
		%end;
	%end;
	%else %do;
		%let path = path;
		%let impdat = impdat;
	%end;
	
	/*
	%if (%eval(&nixl + &nix + &nipl + &nip) <= &FR_NImpVars) and (&FR_ImpVars = Yes) %then %do; 
		%put Number of predictors is less than the specified maximum, so forest filtering is not performed.;
		%let FR_ImpVars = No;
	%end;
	*/

	* SET FOREST PROC OPTIONS;
	%if ^%length(&FR_MaxDepth) %then %let FR_MaxDepth = 6;
	%if ^%length(&FR_LeafSize) %then %let FR_LeafSize = 5;
	%if ^%length(&FR_Seed) %then %let FR_Seed = 12345;
	%if ^%length(&FR_MaxTrees) %then %let FR_MaxTrees = 50;
	%if ^%length(&FR_NInputs) %then %let FR_NInputs = 100;
	%if (&FR_Inds = Yes) %then 
		%let ForestOptions = maxdepth=2 leafsize=&FR_LeafSize seed=&FR_Seed maxtrees=&FR_MaxTrees ninputs=&FR_NInputs;
	%else 
		%let ForestOptions = maxdepth=&FR_MaxDepth leafsize=&FR_LeafSize seed=&FR_Seed maxtrees=&FR_MaxTrees ninputs=&FR_NInputs;

	proc geneselect method=forest data=d &ForestOptions;
		%if &nix %then %do;
	    	var %do i=1 %to &nix; %sysfunc(varname(&_dsix_,&i)) %end; / vartype=quantitative;
		%end;
		%if &nip %then %do;
			var %do i=1 %to &nip; %sysfunc(varname(&_dsip_,&i)) %end; / vartype=qualitative;
		%end;
		trait &DepVar / vartype=&trait_type;
		%if %length(&WeightVar) %then %do;
			freq &WeightVar;
		%end;
		%if (&FR_Inds = Yes) %then %do;
	 		%str(score data=d out=scrts1 dummy;) * CREATE INTERACTION INDICATORS;  
			%str(save path=&path;) * INTERACTION INDICATORS DEFINITIONS;
			%if %length(&intestdat) > 0 %then %do;
				%str(score data=&intestdat role=test out=scrts2 dummy;)
			%end;
		%end;
		%if (&FR_ImpVars = Yes) %then %do;
	 		importance outfit=impdat nvars=&FR_NImpVars; * PREDICTOR REDUCTION;
		%end;
	quit;

	* RETRIEVE FILTERED QUANTITATIVE VS QUALITATIVE PREDICTORS;
	%if (&FR_ImpVars = Yes) %then %do;
		data impdat;
			set impdat;
			if _input1_ = "" then delete;
		run;
		%nObsVars(impdat);
		%if &nObs %then %do;

			* RETRIEVE QUANTITATIVE VS QUALITATIVE PREDICTORS;
			%if &nix %then %do;
				data p;
					set d;
					keep %do i=1 %to &nix; %sysfunc(varname(&_dsix_,&i)) %end; ;
				run;
				proc transpose data=p out=pt(keep=_NAME_); 
				run;
				data pt;
					length vartype $32;
					set pt;
					rename _NAME_ = _input1_;
					label _NAME_ = "Input 1";
					vartype = "quantitative";
				run;
			%end;
			%if &nip %then %do;
				data cp;
					set d;
					keep %do i=1 %to &nip; %sysfunc(varname(&_dsip_,&i)) %end; ;
				run;
				proc transpose data=cp out=cpt(keep=_NAME_);
				run;
				data cpt;
					length vartype $32;
					set cpt;
					rename _NAME_ = _input1_;
					label _NAME_ = "Input 1";
					vartype = "qualitative";
				run;							
			%end;
			data preds;
				set 
				%if &nix %then %do;
					pt
				%end;
				%if &nip %then %do;
					cpt
				%end;
				;
			run;
			data &impdat;
				set impdat;
			run;
			proc sort data=impdat;
				by _input1_;
			run;
			proc sort data=preds;
				by _input1_;
			run;
			data ipreds cpreds;
				merge impdat preds;
				keep _input1_;
				by _input1_;
				if _ASE_ = . or _input1_ = "" then delete;
				if vartype = "quantitative" then output ipreds;
				else output cpreds;
			run; 
			
			%let FRClassPredVars =; 
			%NObsVars(cpreds);
			%if &nObs %then %do;
				%let _cpid_ = %sysfunc(open(cpreds));
				%do c=1 %to &nObs;
					%let rc = %sysfunc(fetchobs(&_cpid_, &c));
					%let input = %sysfunc(getvarc(&_cpid_, 1));
					%let FRClassPredVars=&FRClassPredVars &input;	
				%end;
				%let _cpid_ = %sysfunc(close(&_cpid_));
			%end;
		
			%let FRPredictorVars =; 
			%NObsVars(ipreds);
			%if &nObs %then %do;
				%let _ipid_ = %sysfunc(open(ipreds));
				%do i=1 %to &nObs;
					%let rc = %sysfunc(fetchobs(&_ipid_, &i));
					%let input = %sysfunc(getvarc(&_ipid_, 1));
					%let FRPredictorVars=&FRPredictorVars &input;	
				%end;
				%let _ipid_ = %sysfunc(close(&_ipid_));
			%end;
							
		%end;
		data impdat;
			set impdat;
			rename _input1_ = _name_;
		run;
			
		* redefine data sets containing predictor variable columns;
		/*
		%if %length(&FRPredictorVars) %then %do;
			%let _rc_ = %sysfunc(close(&_dsix_));
			data dsix;
				set d;
				keep &FRPredictorVars;
			run;
			%let _dsix_ = %sysfunc(open(dsix));
			%let nix = %sysfunc(attrn(&_dsix_,NVARS));
		%end;

		%if %length(&FRClassPredVars) %then %do;
			%let _rc_ = %sysfunc(close(&_dsip_));
			data dsip;
				set d;
				keep &FRClassPredVars;
			run;
			%let _dsip_ = %sysfunc(open(dsip));
			%let nip = %sysfunc(attrn(&_dsip_,NVARS));
		%end;
		*/

		%let recov = &RecoverClassPredVars;
		%ReduceData(d,impdat,&intestdat,Forest,&recov); 
		%if &exiterror %then %goto exit;
		/*
		%if %length(&FRPredictorVars) %then %do;
			%let PredictorVars = &FRPredictorVars;
		%end;
		%if %length(&FRClassPredVars) %then %do;
			%let ClassPredVars = &FRClassPredVars;			
		%end;
		*/
	%end;

	%if (&FR_Inds = Yes) %then %do;

		%NObsVars(&path);
		%let _nObspath_ = &nObs;
		%if &_nObspath_ %then %do;

			* RETRIEVE INTERACTION INDICATORS DEFINITIONS;
			data path;                                                                                                                                                  
			   set &path;                                                                                                   
			   X1 = "_Ix"||trim(left(LEAF));                                                                                                                  
			run;                                                                                                                                                
			                                                                                                                                                  
			proc sort data=path;                                                                                                                                
			    by X1;                                                                                                                                        
			run;                                                                                                                                                
	                                                                                                                                                                      
			data friidef;                                                                                                 
			      set path;                                                                                                                               
			      retain prev_varname prev_variable;                                                                                                      
			      keep X1 INDICATOR_DEFINITION LEAF;                                                                                                           
			      if _n_ = 1 then do;                                                                                                                     
			         prev_varname = VAR_NAME;                                                                                                             
			         prev_variable = VARIABLE;                                                                                                            
			      end;                                                                                                                                    
			      if VAR_NAME = "" then do;                                                                                                               
			         VAR_NAME = prev_varname;                                                                                                             
			         VARIABLE = prev_variable;                                                                                                            
			      end;                                                                                                                                    
			      else do;                                                                                                                                
			        prev_varname = VAR_NAME;                                                                                                              
			        prev_variable = VARIABLE;                                                                                                             
			      end;                                                                                                                                    
			      if NUMERIC_VALUE = . then delete;                                                                                                       
			      INDICATOR_DEFINITION = "( "||trim(left(VARIABLE))||" "||                                                                                
			                                   trim(left(RELATION))||" "||                                                                                
			                                   trim(left(CHARACTER_VALUE))||" )";                                                                         
			                                                                                                                                              
			run;   

			proc sort data=friidef;
				by LEAF;
			run; 
			                                                                                                                                              
			data friidef;                                                                                                                                   
			      set friidef end=end;   
				  length def $128;
			      keep prev_x1 def;                                                                                                                       
			      rename prev_x1 = X1;                                                                                                                    
			      label prev_x1 = "Predictor 1";                                                                                                          
			      rename def = INDICATOR_DEFINITION;                                                                                                      
			      label def = "Indicator Definition";                                                                                                     
			      retain prev_x1 def;                                                                                                                     
			      if _n_ = 1 then do;                                                                                                                     
			         prev_x1 = X1;                                                                                                                        
			         def = trim(left(INDICATOR_DEFINITION));                                                                                              
			      end;                                                                                                                                    
			      else do;                                                                                                                                
			         if X1 = prev_x1 then do;                                                                                                             
			             def = trim(left(def))||" * "||trim(left(INDICATOR_DEFINITION));                                                                  
			         end;                                                                                                                                 
			         else do;                                                                                                                             
			             output;                                                                                                                          
			             def = trim(left(INDICATOR_DEFINITION));                                                                                          
			             prev_X1 = X1;                                                                                                                    
			         end;                                                                                                                                 
			      end;                                                                                                                                    
			      if end ne 0 then                                                                                                                        
			         output;                                                                                                                              
			run;   

			%NObsVars(friidef);
			%let _nvii_ = &nObs;
			%put Adding &_nvii_ interaction indicators from forest;
			%if &_nvii_ %then %do;
				* redefine training and test sets;
				data scrts1;
					set scrts1;
					keep _I:;
					rename %do i=1 %to &_nvii_; _I&i = _Ix&i %end; ;
				run;
				data d;
					merge d scrts1;
				run;

				%if %length(&intestdat) > 0 %then %do;
					data scrts2;
						set scrts2;
						keep _I:;
						rename %do i=1 %to &_nvii_; _I&i = _Ix&i %end; ;
					run;
					data &intestdat;
						merge &intestdat scrts2;
					run;
				%end;

				* redefine data set containing predictor variable columns;
				data dsix1;
					set d;
					if (0);
					keep %do i=1 %to &nix; %sysfunc(varname(&_dsix_,&i)) %end; _Ix:;
				run;
				%let _rc_ = %sysfunc(close(&_dsix_));
				data dsix;
					set dsix1;
				run;
				%let _dsix_ = %sysfunc(open(dsix));
				%let nix = %sysfunc(attrn(&_dsix_,NVARS));
				%put updated nix = &nix after adding forest interaction indicators;
			%end;

		%end;
		
	%end;

	%put Predictors from Forest finished: &sysdate &systime;

	%if ^&CVFlag %then %do;
		ods exclude none;
		ods results;
		%if (&FR_ImpVars = Yes) %then %do;
			%let trTitle = %nrbquote(%LocaleFill(%nrbquote(&trTopXPredictorsfromX), &FR_NImpVars, %nrbquote(&trForest)));
			title "&trTitle";
			proc print data=impdat;
			run;
			title;
		%end;
	%end;
%end;


* Use regression to filter predictors;
%if &RG_SelectMethod ^= None %then %do;

	%put Predictors from regression started: &sysdate &systime;
	
	%if (&_dtype_ = C) or (&NumDepAsClass = Yes) %then %do;
		%let UseDep = &_c2;
	%end;
	%else %do;
		%let UseDep = &DepVar;
	%end;
	%put UseDep for regression = &UseDep;
	
	* save current variable lists;
	%if &nix %then %do;
		data dsix0;
			set dsix;
		run;
	%end;
	%if &nip %then %do;
		data dsip0;
			set dsip;
		run;
	%end;
	%if ^&CVFlag %then %do;
		ods exclude none;
		ods results;
	%end;	
	%let lockin = %eval(&nixl+&nipl);
	%let nteams = 1;
	%let another = 1;
	
	%if ^%length(&RG_Seed) %then %let RG_Seed = 2147483;
	%if ^%length(&RG_MaxStep) %then %let RG_MaxStep = 0;
	%if ^%length(&RG_Poolwise) %then %let RG_Poolwise = No;
	%do %while(&another);

		%put selecting regression team &nteams;
		
		/*data OutLib.d; set d; run;*/
		
		%if &RG_Poolwise = Yes %then %do;

			%if &nipl or &nip %then %do;
				%put ERROR: Pool-wise selection is not available for class predictor variables.;
				%let exiterror = 1;
			%end;

			data d1;
				set d;
				if not missing(&UseDep);
				keep &UseDep;
			run;
			
			%NObsVars(d1);
			
			%* let RG_PoolSize = 150;
			%* let RG_PoolMax = 150;
			%put pool size = &RG_PoolSize;
			%put pool max predictors = &RG_PoolMax;
						
			proc transpose data=d(obs=1) out=vnames;
				var %do i=1 %to &nix; %sysfunc(varname(&_dsix_,&i)) %end; ;
			run;
			
			%NObsVars(vnames);
			%let npv = &nobs;
			%put npv = &npv;
			%let r = 1;
			%let seed = &RG_Seed;
			%* let seed = 78430;
			%* let seed = 127639;
			%* let seed = 2147483;
			%* Let seed = 145890000;
			
			%let PoolMax = &RG_PoolMax;
			%if &npv <= &PoolMax %then %let PoolMax = %eval(&npv - 1);
			
			%do %while(&npv > &PoolMax);
			
				data _null_;
					call symputx("npools",ceil(&npv/&RG_PoolSize));
				run;
							
				* assign variables to pools randomly;
				data vnames;
					set vnames;
					pool = ceil(&npools * ranuni(&seed));
					keep _name_ pool;
				run;

				* loop over each pool;
				%do p=1 %to &npools;

					%put round &r pool &p of &npools;

					proc transpose data=vnames out=pool(drop=_name_);
						where pool = &p;
						var pool;
					run;
					%let _dsixp_ = %sysfunc(open(pool));
					%let nixp = %sysfunc(attrn(&_dsixp_,NVARS));
					%put nixp = &nixp;

					title "Round &r  Pool &p";
					proc glmselect data=d
						%if %length(&RG_Seed) %then %do;
							seed=&RG_Seed
						%end;
						namelen=128
						;
						%if &nipl or &nip %then %do;
							class 
								%do i=1 %to &nipl; %sysfunc(varname(&_dsipl_,&i)) %end;
								%do i=1 %to &nip; %sysfunc(varname(&_dsip_,&i)) %end; 
								/ missing
							;
						%end;

						model &UseDep =
							/*
							%do i=1 %to &nixl; 
								%sysfunc(varname(&_dsixl_,&i)) 
								%if &RG_OrderInt > 1 %then %do;
									%do j = 2 %to &RG_OrderInt; | %sysfunc(varname(&_dsixl_,&i)) %end;
									%if ((&i < &nixl) or (&nix > 0)) %then %do; | %end;
								%end;
							%end; 
							*/
							%do i=1 %to &nix; 
								%sysfunc(varname(&_dsix_,&i)) 
								%if &RG_OrderInt > 1 %then %do;
									%do j = 2 %to &RG_OrderInt; | %sysfunc(varname(&_dsix_,&i)) %end;
									%if ((&i < &nix) or (&nixp > 0)) %then %do; | %end;
								%end;
							%end; 							
							%do i=1 %to &nixp; 
								%sysfunc(varname(&_dsixp_,&i)) 
								%if &RG_OrderInt > 1 %then %do;
									%do j = 2 %to &RG_OrderInt; | %sysfunc(varname(&_dsixp_,&i)) %end;
									%if (&i < &nixp) %then %do; | %end;
								%end;
							%end;
							%if &RG_OrderInt > 1 %then %do;
								@ &RG_OrderInt
							%end;
							/ 
								selection=&RG_SelectMethod 
								%if &RG_SelectMethod ^= None %then %do;
									(
										%if &RG_StopCriterion = None %then %do;
											%if (&RG_SelectMethod^=LAR) and (&RG_SelectMethod^=LASSO) and (&RG_SelectMethod^=ElasticNet) %then %do;
												select=SL 
												%if (&RG_SelectMethod=Forward) or (&RG_SelectMethod=Stepwise) %then %do;
													slentry=0.99
												%end;
												%if (&RG_SelectMethod=Backward) or (&RG_SelectMethod=Stepwise) %then %do;
													slstay=0.99
												%end;
											%end;
											/* stop=%eval(&RG_MaxStep+1) */ %if &RG_MaxStep > 0 %then %do; maxstep=&RG_MaxStep %end;
										%end;
										%else %do;
											stop=&RG_StopCriterion %if &RG_MaxStep > 0 %then %do; maxstep=&RG_MaxStep %end;
											%if (&RG_SelectMethod^=LAR) and (&RG_SelectMethod^=LASSO) and (&RG_SelectMethod^=ElasticNet) %then %do;
												select=&RG_StopCriterion
											%end;
											%else %do;
												%if (&RG_Adaptive = Yes) %then %do;
													ADAPTIVE
												%end;
												%if (&RG_StopCriterion=PRESS) %then %do;
													LSCOEFFS
												%end;
											%end;
											%if (&RG_StopCriterion=SL) %then %do;
												%if (&RG_SelectMethod^=Backward) %then %do;
													slentry=&RG_SLEntry
												%end;
												%if (&RG_SelectMethod^=Forward) %then %do;
													slstay=&RG_SLStay
												%end;
												/*
												%if (&RG_SelectMethod=Forward) or (&RG_SelectMethod=Stepwise) %then %do;
													slentry=&RG_SLEntry
												%end;
												%if (&RG_SelectMethod=Backward) or (&RG_SelectMethod=Stepwise) %then %do;
													slstay=&RG_SLStay
												%end;
												*/
											%end;
											%if %length(&lockin) %then %do;
												%if (&RG_SelectMethod=Forward) or (&RG_SelectMethod=Backward) or (&RG_SelectMethod=Stepwise) %then %do;
													include=&lockin
												%end;												
											%end;
										%end;
									) 
								%if (&RG_StopCriterion=CV) %then %do;
									cvmethod=&RG_CVPMethod ( &RG_CVKFold )
								%end;
							%end;
						;
						%if %length(&WeightVar) %then %do;
							weight &WeightVar;
						%end;
						%else %if (&Priors^=Proportional) %then %do;
							weight _Prior;
						%end;
						ods output selectedeffects=sep parameterestimates=pep;
					run;
					title;

					%let _dsixp_ = %sysfunc(close(&_dsixp_));

					%if &p = 1 %then %do;
						data se;
							length effects $ 32767;
							set sep;
						run;
						data pe;
							length parameter $ 32;
							set pep;
						run;
					%end;
					%else %do;
						data se;
							set se sep;
						run;
						data pe;
							set pe pep;
						run;
					%end;

				%end;
				
				/*
				data OutLib.pe;
					set pe;
				run;
				*/
				
				data vnames;
					set pe;
					_name_ = parameter;
					if _name_ ne "Intercept" then output;
					keep _name_;
				run;
				
				%NObsVars(vnames);
				%let npv = &nobs;
				%put npv = &npv;
				%let seed = %eval(&seed + 7);
				%if &r >= 10 %then %do;
					%put Stopping after 10 rounds.;
					%let PoolMax = &npv;
				%end;
				%else %do;
					%let r = %eval(&r + 1);
				%end;
				
			%end;
		%end;
		%else %do;

			proc glmselect data=d
				%if %length(&RG_Seed) %then %do;
					seed=&RG_Seed
				%end;
				namelen=128
				;
				%if &nipl or &nip %then %do;
					class 
						%do i=1 %to &nipl; %sysfunc(varname(&_dsipl_,&i)) %end;
						%do i=1 %to &nip; %sysfunc(varname(&_dsip_,&i)) %end; 
						/ missing
					;
				%end;

				model &UseDep = 
					/*
					%do i=1 %to &nixl; 
						%sysfunc(varname(&_dsixl_,&i)) 
						%if &RG_OrderInt > 1 %then %do;
							%do j = 2 %to &RG_OrderInt; | %sysfunc(varname(&_dsixl_,&i)) %end;
							%if ((&i < &nixl) or (&nipl > 0) or (&nix > 0) or (&nip > 0)) %then %do; | %end;
						%end;
					%end; 
					%do i=1 %to &nipl; 
						%sysfunc(varname(&_dsipl_,&i)) 
						%if &RG_OrderInt > 1 and ((&i < &nipl) or (&nix > 0) or (&nip > 0)) %then %do; | %end;
					%end;
					*/
					%do i=1 %to &nix; 
						%sysfunc(varname(&_dsix_,&i)) 
						%if &RG_OrderInt > 1 %then %do;
							%do j = 2 %to &RG_OrderInt; | %sysfunc(varname(&_dsix_,&i)) %end;
							%if ((&i < &nix) or (&nip > 0)) %then %do; | %end;
						%end;
					%end; 
					%do i=1 %to &nip; 
						%sysfunc(varname(&_dsip_,&i)) 
						%if &RG_OrderInt > 1 and (&i < &nip) %then %do; | %end;
					%end; 
					%if &RG_OrderInt > 1 %then %do;
						@ &RG_OrderInt
					%end;

					/ 
						selection=&RG_SelectMethod 
						%if &RG_SelectMethod ^= None %then %do;
							(
								%if &RG_StopCriterion = None %then %do;
									%if (&RG_SelectMethod^=LAR) and (&RG_SelectMethod^=LASSO) and (&RG_SelectMethod^=ElasticNet) %then %do;
										select=SL 
										%if (&RG_SelectMethod=Forward) or (&RG_SelectMethod=Stepwise) %then %do;
											slentry=0.99
										%end;
										%if (&RG_SelectMethod=Backward) or (&RG_SelectMethod=Stepwise) %then %do;
											slstay=0.99
										%end;
									%end;
									/* stop=%eval(&RG_MaxStep+1) */ %if &RG_MaxStep > 0 %then %do; maxstep=&RG_MaxStep %end;
								%end;
								%else %do;
									stop=&RG_StopCriterion %if &RG_MaxStep > 0 %then %do; maxstep=&RG_MaxStep %end;
									%if (&RG_SelectMethod^=LAR) and (&RG_SelectMethod^=LASSO) and (&RG_SelectMethod^=ElasticNet) %then %do;
										select=&RG_StopCriterion
									%end;
									%else %do;
										%if (&RG_Adaptive = Yes) %then %do;
											ADAPTIVE
										%end;
										%if (&RG_StopCriterion=PRESS) %then %do;
											LSCOEFFS
										%end;
									%end;
									%if (&RG_StopCriterion=SL) %then %do;
										slentry=&RG_SLEntry
										slstay=&RG_SLStay
										/*
										%if (&RG_SelectMethod=Forward) or (&RG_SelectMethod=Stepwise) %then %do;
											slentry=&RG_SLEntry
										%end;
										%if (&RG_SelectMethod=Backward) or (&RG_SelectMethod=Stepwise) %then %do;
											slstay=&RG_SLStay
										%end;
										*/
									%end;
									%if (&RG_SelectMethod=Forward) or (&RG_SelectMethod=Backward) or (&RG_SelectMethod=Stepwise) %then %do;
										include=&lockin
									%end;
								%end;
							) 
						%if (&RG_StopCriterion=CV) %then %do;
							cvmethod=&RG_CVPMethod ( &RG_CVKFold )
						%end;
					%end;
				;
				%if %length(&WeightVar) %then %do;
					weight &WeightVar;
				%end;
				%else %if (&Priors^=Proportional) %then %do;
					weight _Prior;
				%end;
				%if &RG_ModelAveraging = Yes %then %do;
					modelaverage nsamples=&RG_MA_NSamples;
					ods output effectselectpct=se(rename=(effect=effects)) avgparmest=pe;
				%end;
				%else %do;
					ods output selectedeffects=se parameterestimates=pe;
				%end;
			run;
		%end;
		
		%if &RG_ModelAveraging = Yes %then %do;
			proc sort data=se;
				by descending selectionpct;
			run;
			
			/*
			data OutLib.ma_se;
				set se;
			run;
			*/

			%if &RG_MA_Max %then %do;
				data se;
					set se(obs=&RG_MA_Max);
				run;
			%end;
		%end;

		%if &syserr > 4 %then %do;
			%let exiterror = 1;
			%goto exit;
		%end;

		data ts2vars;
			length name $ 32;
			set se;
			i = 1;
			do while(i);
				name = scan(effects,i,' ');
				if lengthn(name)=0 then i = 0;
				else do;
					if name ne "Intercept" then output;
					i = i + 1;
				end;
			end;
			drop i label effects;
		run;
		%let havets2v = 1;

		* for LAR, LASSO, and ElasticNet, class vars are split, so adjust names here;
		%if (&nipl or &nip) and ((&RG_SelectMethod=LAR) or (&RG_SelectMethod=LASSO) or (&RG_SelectMethod=ElasticNet)) %then %do;
			%let ts2 = d;
			%UpdateSelVars;
			%UnsplitSelVars;
		%end;

		data ts2vars;
			length Team 8;
			set ts2vars;
			Team = &nteams;
		run;

		%if &nteams = 1 %then %do;
			data ts2vars_all;
				set ts2vars;
			run;
		%end;
		%else %do;
			data ts2vars_all;
				set ts2vars_all ts2vars;
			run;
		%end;
		
		%NObsVars(ts2vars);

		%if &nobs = 0 or &nteams = &RG_MaxRounds %then %do;
			%let another = 0;
		%end;
		%else %do;
			* reduce variable lists;
			%if &nix %then %do;
				%let _dsix_ = %sysfunc(close(&_dsix_));
				
				data dsix;
					set dsix;
					if (0);
				run;

				proc transpose data=dsix out=dsixt;
				run;

				proc sql;
					create table dsixt1 as
					select * 
					from dsixt as l left join ts2vars as r
					on l._name_=r.name;
				quit;
				
				/*
				data OutLib.dsixt1;
					set dsixt1;
				run;
				*/
				
				data dsixt1;
					set dsixt1;
					if not missing(name) then delete;
				run;
				
				proc transpose data=dsixt1 out=dsix(drop=_name_);
				run;
				
				%let _dsix_ = %sysfunc(open(dsix));
			%end;
			%let nteams = %eval(&nteams + 1);
		%end;

	%end;
	
	%if ^&CVFlag %then %do;
		ods exclude all;
		ods noresults;
	%end;	
	
	data ts2vars_all;
		set ts2vars_all;
		rename name = _name_;
	run;
	
	/*
	data OutLib.regvars_all;
		set ts2vars_all;
	run;
	*/

	* restore variable lists;
	%if &nix %then %do;
		%let _dsix_ = %sysfunc(close(&_dsix_));
		data dsix;
			set dsix0;
		run;
		%let _dsix_ = %sysfunc(open(dsix));
	%end;
	%if &nip %then %do;
		%let _dsip_ = %sysfunc(close(&_dsip_));
		data dsip;
			set dsip0;
		run;
		%let _dsip_ = %sysfunc(open(dsip));
	%end;

	* redefine data sets containing predictor variable columns;
	%if (&FR_ImpVars = Yes) %then %let recov = No;
	%else %let recov = &RecoverClassPredVars;
	%ReduceData(d,ts2vars_all,&intestdat,regression,&recov); 
	%if &exiterror %then %goto exit;
	
%end;


* USE PROC OPTMODEL TO FIND AN OPTIMAL SET;
%if (&Optimize = Yes) and &nix %then %do;
	
	%put Continuous predictor optimization started: &sysdate &systime;
	%put nix to optimize= &nix;
	
	* if DepVar is character use 0-1 variable;
	%if (&_dtype_ = C) %then %do;
		%let UseDep = &_c1;
	%end;
	%else %do;
		%let UseDep = &DepVar;
	%end;
	
	%put UseDep = &UseDep;
	data d1;
		set d;
		if missing(&UseDep) then delete;
		keep &UseDep %do i=1 %to &nix; %sysfunc(varname(&_dsix_,&i)) %end; ;
	run;

	proc corr data=d1 out=corr %scan(&Opt_Similarity,1,' ') noprint;
		var &UseDep %do i=1 %to &nix; %sysfunc(varname(&_dsix_,&i)) %end; ;
	run;
		
	data corr;
		set corr;
		drop _name_ _type_;
		if _n_ <= 3 then delete;
		array c{*} &UseDep %do i=1 %to &nix; %sysfunc(varname(&_dsix_,&i)) %end;;
		do i=1 to dim(c);
			if missing(c[i]) then c[i] = 0;
			else c[i] = abs(c[i]);
		end;
		drop i;
	run;
	
	data upper_triangular(drop=j);
		set corr;
		array c{*} %do i=1 %to &nix; %sysfunc(varname(&_dsix_,&i)) %end;;
		do j = 1 to dim(c);
			if _n_ > j then c[j] = .;
		end;
	run;
	
	* transpose to get predictor variable names;
	proc transpose data=upper_triangular(obs=0 drop=&UseDep) out=trans;
	run;

	options mstored sasmstore=orlib;
	%if %length(%sysfunc(pathname(orlib))) = 0 %then %do;
		libname orlib "&MacroPath.";
	%end;
	
	%FeatureOptimization;
	
	%if %index(&Opt_Algorithm,Greedy) %then %do;
		data optc;
			set optc;
			rename feature_name=_name_;
		run;
	%end;
	%else %do;
		data optc;
			set opta;
			if x=1;
			rename feature_name=_name_;
		run;
	%end;
	
	%if &nixl or &nipl or &nip %then %do;
		data opt;
			length _name_ $32;
			%do i=1 %to &nixl; _name_="%sysfunc(varname(&_dsixl_,&i))"; output; %end;;		
			%do i=1 %to &nipl; _name_="%sysfunc(varname(&_dsipl_,&i))"; output; %end;;		
			%do i=1 %to &nip; _name_="%sysfunc(varname(&_dsip_,&i))"; output; %end;;		
		run;

		data opt;
			set optc opt;
		run;
	%end;
	%else %do;
		data opt;
			set optc;
		run;
	%end;

	%ReduceData(d,opt,&intestdat,Optimize,&RecoverClassPredVars); 
	%if &exiterror %then %goto exit;

	%put Continuous predictor optimization finished: &sysdate &systime;
	
	%if ^&CVFlag %then %do;
		ods exclude none;
		ods results;
		%if &Optimize=Yes and &nix %then %do;
			title "&trOptimizedContinuousPredictors";
			proc print data=optc;
			run;
			title;
		%end;
	%end;

%end;

%let npv = %eval(&nixl+&nipl+&nix+&nip);
%put npv = &npv;

* standardize predictors row-wise so that sscp becomes corr matrix;
%if (&StandardizeRows = Yes) and (%eval(&nixl + &nix) > 1) and
	%length(%trim(&LockContPredVars &PredictorVars)) %then %do;
	data d;
		set d;
		array pv{*} 
			%do i=1 %to &nixl; %sysfunc(varname(&_dsixl_,&i)) %end; 
			%do i=1 %to &nix; %sysfunc(varname(&_dsix_,&i)) %end; 
			/* &LockContPredVars &LockClassPredVars %if ^&upred %then %do; &PredictorVars %end; %else %do; %do j=1 %to &nix_blocks; &&PredictorVars&j %end; %end; &ClassPredVars */
		;
		_mean_ = mean(of pv{*});
		_std_ = std(of pv{*});
		if _std_ < 1e-6 then _std_ = 1;
		do _i_=1 to dim(pv);
			pv[_i_] = (pv[_i_]-_mean_)/_std_;
		end;
		drop _i_;
	run;
	%if %length(&intestdat) and &intestdat^=d %then %do;
		data &intestdat;
			set &intestdat;
			array pv{*} 
				%do i=1 %to &nixl; %sysfunc(varname(&_dsixl_,&i)) %end; 
				%do i=1 %to &nix; %sysfunc(varname(&_dsix_,&i)) %end; 
				/* &LockContPredVars &LockClassPredVars %if ^&upred %then %do; &PredictorVars %end; %else %do; %do j=1 %to &nix_blocks; &&PredictorVars&j %end; %end; &ClassPredVars */
			;
			_mean_ = mean(of pv{*});
			_std_ = std(of pv{*});
			if _std_ < 1e-6 then _std_ = 1;
			do _i_=1 to dim(pv);
				pv[_i_] = (pv[_i_]-_mean_)/_std_;
			end;
			drop _i_;
		run;
	%end;
%end;


%if (&GenerateHTML = Yes) and ^&CVFlag %then %do;
	%if &ProcessName = SurvivalPredictiveModeling %then %do;
		%let ProcName = SPM;
	%end;
	%else %if &ProcessName = LifeRegression %then %do;
		%let ProcName = LifeReg;
	%end;
	%else %do;
		%let ProcName = &ProcessName;
	%end;
	%PrepHTML(&ProcName._Body, &ProcName._Frame, &ProcName._Contents, 
		&ProcName..body.html, &ProcName..frame.html, &ProcName..contents.html);
	ods listing gpath="&OutPath";
	ods html 
		body = "&&&ProcName._Body" (url="&bodyfile")
		frame = "&&&ProcName._Frame"  (url="&framefile")
		contents = "&&&ProcName._Contents" (url="&contentsfile")
		gpath = "&OutPath"
	;
	ods graphics on %if (&ProcessName = GLMSelect) or (&ProcessName = QuantSelect) %then /imagename="&&&_OutDataName";;
	filename plotout "&OutPath";
	goptions reset=all device=png gsfname=plotout;
%end;

%if &CVFlag %then %do;
	%put CV_ListAll = &CV_ListAll;
	%if &CV_ListAll = Yes %then %do;
		ods exclude none;
		ods results;
	%end;
%end;
%else %do;
	ods exclude none;
	ods results;
%end;


%*ClockStop(Predictor Reduction);
%*ClockStart(Fitting);
	
*************************************
* JMP Analyses;
%if (&Mode = Interactive) %then %do;
   data &outdat;
      set d;
   run;
   /*
 	%if (&ProcessName = PartialLeastSquares) %then %do;
 		%if %length(&ClassPredVars) %then %do;
 			%let i = 1;
			%do %while(%length(%scan(&ClassPredVars,&i)));
				%let plsV&i = %scan(&ClassPredVars,&i,' ');
				%let i=%eval(&i+1);
			%end;
			%let plsVn= %eval(&i-1);
 	 	%end; 
 	%end; 
 	*/
 	
 	/*
	%CheckLabel(&outdat,DepVar);
	%if %length(&LabelVar) %then %do;
		%CheckLabel(&outdat,LabelVar);
	%end;
	*/
	%global _mainjslfile_ ;
	%let _mainjslfile_ = &&&_JSLFile;
	* put JSL statements;

	data _null_;
		file "&&&_JSLFile"  encoding="utf-8";
		put "//!";
		put 'Names Default to Here(1);';
		put "Here:DataList = {};";
		put "Here:TabButtonList = {};";
		put "Here:TabCodeList = {};";
		put "Here:TabAfterCodeList = {};";
		put "Here:TabOpenList = {};";
		put "Here:data=open(""&ClientOutPath.&&&_OutDataName...sas7bdat"",UseLabelsForVarNames(false),invisible);";
		put " ";
		put "Here:data<<Color By Column(:&DepVar);";
		%if %length(&LabelVar) %then %do;
			put "%str(column(%"&LabelVar%") << label(1);)";
		%end;
		
		%if (&ProcessName ne PartialLeastSquares) %then %do;
			%if (&_dtype_=C) or (&NumDepAsClass=Yes) %then %do;
				put "%str(column(%"&DepVar%") << Modeling Type(%"Nominal%");)";
			%end;
			%do i=1 %to &nipl;
				%let v = %sysfunc(varname(&_dsipl_,&i));
				put "%str(column(%"&v%") << Modeling Type(%"Nominal%");)";
			%end;
			%do i=1 %to &nip;
				%let v = %sysfunc(varname(&_dsip_,&i));
				put "%str(column(%"&v%") << Modeling Type(%"Nominal%");)";
			%end;
		%end;
		put 'Here:vin=Function({}, {default local},';		
		put "   If(IsEmpty(Here:data),";
		put "      Here:data=open(""&ClientOutPath.&&&_OutDataName...sas7bdat"",invisible);";
		put "   );";
		put "   Current Data Table(Here:data);";
		put "   Vlistbox(";	
		%if (&ProcessName = DiscriminantAnalysis) %then %do;
			put "      Discriminant(X( :&DepVar), DiscriminantMethod(&Metric), Y(";		
		%end;
		%else %if (&ProcessName = LogisticRegression) %then %do;
			put "      Fit Model(Y( :&DepVar), " @@;
			%if %length(&WeightVar) %then %do;
				put "Weight(:&WeightVar), " @@;
			%end;
			put "Keep dialog open(1), Effects(";
		%end;
		%else %if ((&ProcessName = PartitionTrees) or (&ProcessName = VirtualTwins)) %then %do;
 			put "      Partition(Y( :&DepVar), " @@;
			%if %length(&WeightVar) %then %do;
				put "Weight(:&WeightVar), " @@;
			%end;
 			put "Color Points(1), X(";
		%end;
		%else %if (&ProcessName = PartialLeastSquares) %then %do;
			put "      PLS(Y( :&DepVar), X("; 	
			/*
			%let _dsip2_ = %sysfunc(open(d));
			put "      pls(Y( ";
				%do i=1 %to &nic; 
					%if ^%index(&&_c&i,_Mis) and ^%index(&&_c&i,_true) %then %do;
						%let v = %sysfunc(varname(&_dsip2_,%sysfunc(varnum(&_dsip2_,&&_c&i))));
						put " Name(""&v"")" @@;
					   put ":&&_c&i, " ;
					   put ' column("&&_c&i")<<label(1); ' ;

						%if &nit or (&i<&nic) %then %do;
							put "," @@;
						%end;
						%else %if ^&nit %then %do;
							put " ";
						%end;
					%end;
				%end;
				%do i=1 %to &nit; 
					put " Name(""&&_t&i"")" @@;
					%if (&i<&nit) %then %do;
						put "," @@;
					%end;
					%else %do;
						put " ";
					%end;
				%end; 
				put " ), X(";
			*/
		%end;
		
		%do i=1 %to &nixl;
			%let v = %sysfunc(varname(&_dsixl_,&i));
			put " Name(""&v"")" @@;
			%if (&i<&nixl) or (&nipl>0) or (&nix>0) or (&nip>0) %then %do;
				put "," @@;
			%end;
			%else %do;
				put " ";
			%end;
		%end;

		%do i=1 %to &nipl;
			%let v = %sysfunc(varname(&_dsipl_,&i));
			put " Name(""&v"")" @@;
			%if (&i<&nipl) or (&nix>0) or (&nip>0) %then %do;
				put "," @@;
			%end;
			%else %do;
				put " ";
			%end;
		%end;
		
		%do i=1 %to &nix;
			%let v = %sysfunc(varname(&_dsix_,&i));
			put " Name(""&v"")" @@;
			%if (&i<&nix) or (&nip>0) %then %do;
				put "," @@;
			%end;
			%else %do;
				put " ";
			%end;
		%end;
				
		%do i=1 %to &nip;
			%let v = %sysfunc(varname(&_dsip_,&i));
			put " Name(""&v"")" @@;
			%if (&i<&nip) %then %do;
				put "," @@;
			%end;
			%else %do;
				put " ";
			%end;
		%end;
		
		put ")" @@;
		
		%if &VarSelect = Stepwise %then %do;
			%if (&ProcessName = LogisticRegression) %then %do;
				put "   , Personality(Stepwise), Run Model";
				put "   );";
  				put '   fmw = Window("Fit Model");';
  				put "	  fmw<<CloseWindow;";
			%end;
			%else %do;
				put "   , Stepwise Variable Selection(1)";
				put "   );";
			%end;
		%end;
		%else %do;
			put ");";
		%end;
		put ');';	
		put ");";
		put " ";
		put "InsertInto(DataList,{{""Here:data""}});";
		put "InsertInto(TabButtonList, Expr(locale:get(""Results_Text"")));";
		put "InsertInto(TabCodeList,expr(Here:vin()));";
		put "InsertInto(TabAfterCodeList, "" "");";
		put "InsertInto(TabOpenList,1);";
	run;

	%PackageInsertFile(jsl code from &ProcessName, &OutPath, &&&_JSLFileName, &&&_OutDataName...sas7bdat, text/plain, yes);
%end;
%else %do;
	
	%let cvl=;
	
	%if %index(&VarSelect,Genetic) %then %do;
		%include "&MacroPath./GAMacros.sas" / nosource;
	%end;
	
	%let SomeSel = 0;	
	
  	%let SelVars = ;
  	%let ASet = ;
 	
  	* set up training and test sets;
	%let ts1 = d;
	%let ts2 = &intestdat;
	%put ts1 = &ts1;
	%put ts2 = &ts2;
	
	/*
	data OutLib.ts1;
		set &ts1;
	run;
	data OutLib.ts2;
		set &ts2;
	run;
	*/

  	* drop certain variables to avoid naming conflicts with new calculations;
	%let droplist1 = ;
	%let _dsid_ = %sysfunc(open(&ts1));
	%if &_dsid_ %then %do;
		%let nvars = %sysfunc(attrn(&_dsid_,NVARS));
		%do i=1 %to &nvars;
			%let vari = %upcase(%sysfunc(varname(&_dsid_,&i)));
			/* %put vari = &vari; */
			%if (%quote(&vari)=PRED) %then %let droplist1 = &droplist1 PRED;
			%if (%quote(&vari)=PREDICTED_CLASS) %then %let droplist1 = &droplist1 PREDICTED_CLASS;
			%if (%quote(&vari)=PROB_ACTUAL) %then %let droplist1 = &droplist1 PROB_ACTUAL;
			%if (%quote(&vari)=CORRECT) %then %let droplist1 = &droplist1 CORRECT;
			%if (%quote(&vari)=CORRECT_PRED) %then %let droplist1 = &droplist1 CORRECT_PRED;
			%if (%quote(&vari)=PREDICTED_VALUE) %then %let droplist1 = &droplist1 PREDICTED_VALUE;
			%if %length(&vari) > 1 %then %do;
				%let first2 = %upcase(%substr(&vari,1,2));
				%if %quote(&first2) = %quote(P_) %then %let droplist1 = &droplist1 &vari;
			%end;
		%end;
	   %let _rc_ = %sysfunc(close(&_dsid_));
	%end;
	%if %length(&droplist1) or (&ProcessName= GenomicBLUP or &ProcessName = GenomicBayesianRegression) %then %do;
		%let sim_exist= 0;
		%let _sim_exist= 0;
		%let p_id_exist= 0;
		%let _p_id_exist= 0;
		%let _cross_exist= 0;
		%let droplist1G= ;
		%if &ProcessName= GenomicBLUP or &ProcessName = GenomicBayesianRegression %then %do;
			%let _dsg_= %sysfunc(open(&ts1));
			%let sim_exist= %sysfunc(varnum(&_dsg_,sim));
			%put sim_exist= &sim_exist;
			%let _sim_exist= %sysfunc(varnum(&_dsg_,_sim_));
			%put _sim_exist= &_sim_exist;
			%let p_id_exist= %sysfunc(varnum(&_dsg_,p_id));
			%put p_id_exist= &p_id_exist;
			%let _p_id_exist= %sysfunc(varnum(&_dsg_,_p_id_));
			%put _p_id_exist= &_p_id_exist;
			%let _p= 0;
			%let other_p_id= 1;
			%do %while(&other_p_id);
				%let other_p_id= %sysfunc(varnum(&_dsg_,_p_id%eval(&_p+1)));
				%put &_p &other_p_id;
				%let _p = %eval(&_p + 1);
			%end;
			%let _p= %eval(&_p - 1);
			%put _p= &_p;
			
			%let _cross_exist= %sysfunc(varnum(&_dsg_,_cross_));
			%put _cross_exist= &_cross_exist;
			%let _c= 0;
			%let other_cross_= 1;
			%do %while(&other_cross_);
				%let other_cross_= %sysfunc(varnum(&_dsg_,_cross_%eval(&_c+1)));
				%put &_c &other_cross_;
				%let _c = %eval(&_c + 1);
			%end;
			%let _c= %eval(&_c - 1);
			%put _c= &_c;	
			
			%let _rcg_ = %sysfunc(close(&_dsg_));	

			%if &p_id_exist or &_p_id_exist %then %let droplist1G= &droplist1G _m_id_: _p_id_:;
			%if &_cross_exist %then %let droplist1G= &droplist1G _cross_:;
			%if &sim_exist %then %let droplist1G= &droplist1G sim;
			%if &_sim_exist %then %let droplist1G= &droplist1G _sim_;			
		%end;
				
		data &ts1;
			%if &ProcessName= GenomicBLUP or &ProcessName = GenomicBayesianRegression %then %do;
				length 
				%*if &_cross_exist %then %do; 
					_tmp_cross_ $ 256
				%*end;
				%if (&p_id_exist) or (&_p_id_exist) %then %do;
					_tmp_m_id_ _tmp_p_id_ $ 256
				%end;
				%if &sim_exist or &_sim_exist %then %do; 
					_tmp_sim_ 8
				%end;				
				;
			%end;
			set &ts1 
				/*
				%if (&ProcessName= GenomicBLUP or &ProcessName = GenomicBayesianRegression) and &p_id_exist %then %do;
					(
					%if (&ProcessName= GenomicBLUP or &ProcessName = GenomicBayesianRegression) and &_p_id_exist %then %do;
						drop= _p_id _m_id
					%end;
					rename=(p_id= _p_id m_id= _m_id)
					)
				%end;
				*/
				%if &ProcessName= GenomicBLUP or &ProcessName = GenomicBayesianRegression %then %do;
					%if (&_cross_exist) or (&p_id_exist) or (&_p_id_exist) %then %do;
						(rename= (
							%if &_cross_exist %then %do;
								_cross_= _cross_%eval(&_c+1)
							%end;
							%if &p_id_exist %then %do;
								p_id= _p_id_%eval(&_p+1)
								m_id= _m_id_%eval(&_p+1)
							%end;
							%else %if &_p_id_exist %then %do;
								_p_id_= _p_id_%eval(&_p+1)
								_m_id_= _m_id_%eval(&_p+1)
							%end;
							)
						)
					%end;
				%end;
			;
			%if &ProcessName= GenomicBLUP or &ProcessName = GenomicBayesianRegression %then %do;
				%if &_cross_exist %then %do;
					if not missing(_cross_1) then _tmp_cross_= _cross_1; 
					%do i=2 %to &_c %by 1; 
						else if not missing(_cross_%eval(&i+1)) then _tmp_cross_= _cross_%eval(&i+1); 
					%end;
					else 
						%if %length(&IDVars) %then %do;
							_tmp_cross_= trim(left(%scan(&IDVars,1,' '))) 
							%let i=2;
							/*
							%do %while(%length(%scan(&IDVars,&i,' ')));
								|| " " || trim(left(%scan(&IDVars,&i,' ')))
								%let i=%eval(&i+1);
							%end;
							*/
							%do %while(%length(%scan(&IDVars,&i,' ')));
								|| "-" || trim(left(%scan(&IDVars,&i,' ')))
								%let i=%eval(&i+1);
							%end;
							;
						%end;
						%else %do;
							_tmp_cross_= input(trim(left(_n_)), $256.);
						%end;
				%end;
				%else %do;
					%if %length(&IDVars) %then %do;
						_tmp_cross_= trim(left(%scan(&IDVars,1,' '))) 
						%let i=2;
						/*
						%do %while(%length(%scan(&IDVars,&i,' ')));
							|| " " || trim(left(%scan(&IDVars,&i,' ')))
							%let i=%eval(&i+1);
						%end;
						*/
						%do %while(%length(%scan(&IDVars,&i,' ')));
							|| "-" || trim(left(%scan(&IDVars,&i,' ')))
							%let i=%eval(&i+1);
						%end;
						;
					%end;
					%else %do;
						_tmp_cross_= input(trim(left(_n_)), $256.);
					%end;				
				%end;
				%if &p_id_exist %then %do;
					if not missing(_p_id_%eval(&_p+1)) then _tmp_p_id_= _p_id_%eval(&_p+1);
					if not missing(_m_id_%eval(&_p+1)) then _tmp_m_id_= _m_id_%eval(&_p+1);
				%end;	
				%else %if &_p_id_exist %then %do;
					if not missing(_p_id_1) then _tmp_p_id_= _p_id_1;
					%do i=2 %to &_p %by 1; 
						else if not missing(_p_id_%eval(&i+1)) then _tmp_p_id_= _p_id_%eval(&i+1); 
					%end;
					if not missing(_m_id_1) then _tmp_m_id_= _m_id_1;
					%do i=2 %to &_p %by 1; 
						else if not missing(_m_id_%eval(&i+1)) then _tmp_m_id_= _p_id_%eval(&i+1); 
					%end;				
				%end;
				%if &sim_exist %then %do;
					_tmp_sim_= sim;
				%end;
				%else %if &_sim_exist %then %do;
					_tmp_sim_= _sim_;
				%end;
				;
			%end;
			%if %length(&droplist1 &droplist1G) %then %do; 
				drop &droplist1 &droplist1G;
			%end;
		run;
		
		%if %length(&droplist1 &droplist1G) %then %do;
			%put WARNING: The following variables in the training data have been dropped naming conflicts in output results: &droplist1 &droplist1G;
		%end;
		
		%if &ProcessName= GenomicBLUP or &ProcessName = GenomicBayesianRegression %then %do; 
			%*if (&_cross_exist) or (&p_id_exist) or (&_p_id_exist) %then %do;
				proc datasets nolist;
					modify &ts1;
					rename 
						/*%if &_cross_exist %then*/ _tmp_cross_ = _cross_/*;*/
						%if &p_id_exist or &_p_id_exist %then _tmp_p_id_= _p_id_ _tmp_m_id_= _m_id_;
						%if &sim_exist or &_sim_exist %then _tmp_sim_= _sim_; 
					;
				quit;
			%*end;
		%end;
		
		/*data OutLib.ts1; set &ts1; run;*/
		
	%end;
	
	%if (&ts1 ^= &ts2) %then %do;
		%let droplist2 = ;
		%let _dsid_ = %sysfunc(open(&ts2));
		%if &_dsid_ %then %do;
			%let nvars = %sysfunc(attrn(&_dsid_,NVARS));
			%do i=1 %to &nvars;
				%let vari = %upcase(%sysfunc(varname(&_dsid_,&i)));
				/* %put vari = &vari; */
				%if (%quote(&vari)=PRED) %then %let droplist2 = &droplist2 PRED;
				%if (%quote(&vari)=PREDICTED_CLASS) %then %let droplist2 = &droplist2 PREDICTED_CLASS;
				%if (%quote(&vari)=PROB_ACTUAL) %then %let droplist2 = &droplist2 PROB_ACTUAL;
				%if (%quote(&vari)=CORRECT) %then %let droplist2 = &droplist2 CORRECT;
				%if (%quote(&vari)=CORRECT_PRED) %then %let droplist2 = &droplist2 CORRECT_PRED;
				%if (%quote(&vari)=PREDICTED_VALUE) %then %let droplist2 = &droplist2 PREDICTED_VALUE;
				%if %length(&vari) > 1 %then %do;
					%let first2 = %upcase(%substr(&vari,1,2));
					%if %quote(&first2) = %quote(P_) %then %let droplist2 = &droplist2 &vari;
				%end;
			%end;
			%let _rc_ = %sysfunc(close(&_dsid_));
		%end;
		%if %length(&droplist2) or (&ProcessName= GenomicBLUP or &ProcessName = GenomicBayesianRegression) %then %do;
			%let sim_exist= 0;
			%let _sim_exist= 0;
			%let p_id_exist= 0;
			%let _p_id_exist= 0;
			%let _cross_exist= 0;
			%let droplist2G= ;
			%if &ProcessName= GenomicBLUP or &ProcessName = GenomicBayesianRegression %then %do;
				%let _dsg_= %sysfunc(open(&ts2));
				%let sim_exist= %sysfunc(varnum(&_dsg_,sim));
				%put sim_exist= &sim_exist;
				%let _sim_exist= %sysfunc(varnum(&_dsg_,_sim_));
				%put _sim_exist= &_sim_exist;
				%let p_id_exist= %sysfunc(varnum(&_dsg_,p_id));
				%put p_id_exist= &p_id_exist;
				%let _p_id_exist= %sysfunc(varnum(&_dsg_,_p_id_));
				%put _p_id_exist= &_p_id_exist;
				%let _p= 0;
				%let other_p_id= 1;
				%do %while(&other_p_id);
					%let other_p_id= %sysfunc(varnum(&_dsg_,_p_id%eval(&_p+1)));
					%put &_p &other_p_id;
					%let _p = %eval(&_p + 1);
				%end;
				%let _p= %eval(&_p - 1);
				%put _p= &_p;
				
				%let _cross_exist= %sysfunc(varnum(&_dsg_,_cross_));
				%put _cross_exist= &_cross_exist;
				%let _c= 0;
				%let other_cross_= 1;
				%do %while(&other_cross_);
					%let other_cross_= %sysfunc(varnum(&_dsg_,_cross_%eval(&_c+1)));
					%put &_c &other_cross_;
					%let _c = %eval(&_c + 1);
				%end;
				%let _c= %eval(&_c - 1);
				%put _c= &_c;			
	
				%let _rcg_ = %sysfunc(close(&_dsg_));	
				
				%if &p_id_exist or &_p_id_exist %then %let droplist2G= &droplist2G _m_id_: _p_id_:;
				%if &_cross_exist %then %let droplist2G= &droplist2G _cross_:;
				%if &sim_exist %then %let droplist2G= &droplist2G sim; 
				%if &_sim_exist %then %let droplist2G= &droplist2G _sim_;				
			%end;
			
			data &ts2;
				%if &ProcessName= GenomicBLUP or &ProcessName = GenomicBayesianRegression %then %do;
					length 
					%*if &_cross_exist %then %do; 
						_tmp_cross_ $ 256
					%*end;
					%if (&p_id_exist) or (&_p_id_exist) %then %do;
						_tmp_m_id_ _tmp_p_id_ $ 256
					%end;
					%if &sim_exist or &_sim_exist %then %do;
						_tmp_sim_ 8;
					%end;
					;
				%end;
				set &ts2 
					/*
					%if (&ProcessName= GenomicBLUP or &ProcessName = GenomicBayesianRegression) and &p_id_exist %then %do;
						(
						%if (&ProcessName= GenomicBLUP or &ProcessName = GenomicBayesianRegression) and &_p_id_exist %then %do;
							drop= _p_id _m_id
						%end;
						rename=(p_id= _p_id m_id= _m_id)
						)
					%end;
					*/
					%if &ProcessName= GenomicBLUP or &ProcessName = GenomicBayesianRegression %then %do;
						%if (&_cross_exist) or (&p_id_exist) %then %do;
							(rename= (
								%if &_cross_exist %then %do;
									_cross_= _cross_%eval(&_c+1)
								%end;
								%if &p_id_exist %then %do;
									p_id= _p_id_%eval(&_p+1)
									m_id= _m_id_%eval(&_p+1)
								%end;
								%else %if &_p_id_exist %then %do;
									_p_id_= _p_id_%eval(&_p+1)
									_m_id_= _m_id_%eval(&_p+1)									
								%end;
								)
							)
						%end;
					%end;
				;
				%if &ProcessName= GenomicBLUP or &ProcessName = GenomicBayesianRegression %then %do;
					%if &_cross_exist %then %do;
						if not missing(_cross_1) then _tmp_cross_= _cross_1; 
						%do i=2 %to &_c %by 1; 
							else if not missing(_cross_%eval(&i+1)) then _tmp_cross_= _cross_%eval(&i+1); 
						%end;
						else _tmp_cross_= input(trim(left(_n_)), $256.);
					%end;
					%else %do;
						_tmp_cross_= input(trim(left(_n_)), $256.);
					%end;
					%if &p_id_exist %then %do;
						if not missing(_p_id_%eval(&_p+1)) then _tmp_p_id_= _p_id_%eval(&_p+1);
						if not missing(_m_id_%eval(&_p+1)) then _tmp_m_id_= _m_id_%eval(&_p+1);
					%end;	
					%else %if &_p_id_exist %then %do;
						if not missing(_p_id_1) then _tmp_p_id_= _p_id_1;
						%do i=2 %to &_p %by 1; 
							else if not missing(_p_id_%eval(&i+1)) then _tmp_p_id_= _p_id_%eval(&i+1); 
						%end;
						if not missing(_m_id_1) then _tmp_m_id_= _m_id_1;
						%do i=2 %to &_p %by 1; 
							else if not missing(_m_id_%eval(&i+1)) then _tmp_m_id_= _p_id_%eval(&i+1); 
						%end;				
					%end;
					%if &sim_exist %then %do;
						_tmp_sim_= sim;
					%end;
					%else %if &_sim_exist %then %do;
						_tmp_sim_= _sim_;
					%end;
				; 
				%end;
				%if %length(&droplist2 &droplist2G) %then %do; 
					drop &droplist2 &droplist2G;
				%end;
			run;
			
			%if %length(&droplist2 &droplist2G) %then %do;
				%put WARNING:  The following variables in the test data have been dropped to avoid naming conflicts in output results: &droplist2 &droplistG2;
			%end;
			
			%if &ProcessName= GenomicBLUP or &ProcessName = GenomicBayesianRegression %then %do; 
				%*if (&_cross_exist) or (&p_id_exist) or (&_p_id_exist) %then %do;
					proc datasets nolist;
						modify &ts2;
						rename 
							/*%if &_cross_exist %then*/ _tmp_cross_ = _cross_/*;*/
							%if &p_id_exist or &_p_id_exist %then _tmp_p_id_= _p_id_ _tmp_m_id_= _m_id_;
							%if &sim_exist or &_sim_exist %then _tmp_sim_= _sim_; 
						;
					quit;
				%*end;
			%end;		

			/*data OutLib.ts2; set &ts2; run;*/		
		%end;
	%end;
	
	%if &CVFlag %then %do;
		%let ts2 = &intestdat;
		%if &CV_ListAll = No %then %do;
			ods exclude all;
			ods noresults;
		%end;
		%let droplist = ;
		%let _dsid_ = %sysfunc(open(&ts2));
		%if &_dsid_ %then %do;
			%let nvars = %sysfunc(attrn(&_dsid_,NVARS));
			%do i=1 %to &nvars;
				%let vari = %upcase(%sysfunc(varname(&_dsid_,&i)));
				%if (&vari=PRED) %then %let droplist = &droplist PRED;
				%if (&vari=PROB_ACTUAL) %then %let droplist = &droplist PROB_ACTUAL;
				%if (&vari=CORRECT) %then %let droplist = &droplist CORRECT;
				%if (&vari=CORRECT_PRED) %then %let droplist = &droplist CORRECT_PRED;
				/* %put i=&i vari=&vari varis2=%substr(&vari,1,2); */
				%if %length(&vari) > 1 %then %do;
					%if %qsubstr(&vari,1,2) = %quote(P_) %then %let droplist = &droplist &vari;
				%end;
			%end;
			%let _rc_ = %sysfunc(close(&_dsid_));
		%end;

		%if %length(&droplist) or (&ProcessName= GenomicBLUP or &ProcessName = GenomicBayesianRegression) %then %do;
			%let sim_exist= 0;
			%let _sim_exist= 0;
			%let p_id_exist= 0;
			%let _p_id_exist= 0;
			%let _cross_exist= 0;
			%let droplistG= ;
			%if &ProcessName= GenomicBLUP or &ProcessName = GenomicBayesianRegression %then %do;
				%let _dsg_= %sysfunc(open(&ts2));
				%let sim_exist= %sysfunc(varnum(&_dsg_,sim));
				%put sim_exist= &sim_exist;
				%let _sim_exist= %sysfunc(varnum(&_dsg_,_sim_));
				%put _sim_exist= &_sim_exist;
				%let p_id_exist= %sysfunc(varnum(&_dsg_,p_id));
				%put p_id_exist= &p_id_exist;
				%let _p_id_exist= %sysfunc(varnum(&_dsg_,_p_id_));
				%put _p_id_exist= &_p_id_exist;
				%let _p= 0;
				%let other_p_id= 1;
				%do %while(&other_p_id);
					%let other_p_id= %sysfunc(varnum(&_dsg_,_p_id%eval(&_p+1)));
					%put &_p &other_p_id;
					%let _p = %eval(&_p + 1);
				%end;
				%let _p= %eval(&_p - 1);
				%put _p= &_p;
				
				%let _cross_exist= %sysfunc(varnum(&_dsg_,_cross_));
				%put _cross_exist= &_cross_exist;
				%let _c= 0;
				%let other_cross_= 1;
				%do %while(&other_cross_);
					%let other_cross_= %sysfunc(varnum(&_dsg_,_cross_%eval(&_c+1)));
					%put &_c &other_cross_;
					%let _c = %eval(&_c + 1);
				%end;
				%let _c= %eval(&_c - 1);
				%put _c= &_c;			
	
				%let _rcg_ = %sysfunc(close(&_dsg_));	
				
				%if &p_id_exist or &_p_id_exist %then %let droplistG= &droplistG _m_id_: _p_id_:;
				%if &_cross_exist %then %let droplistG= &droplistG _cross_:;
				%if &sim_exist %then %let droplistG= &droplistG sim; 
				%if &_sim_exist %then %let droplistG= &droplistG _sim_;				
			%end;
			
			data &ts2;
				%if &ProcessName= GenomicBLUP or &ProcessName = GenomicBayesianRegression %then %do;
					length 
					%*if &_cross_exist %then %do; 
						_tmp_cross_ $ 256
					%*end;
					%if (&p_id_exist) or (&_p_id_exist) %then %do;
						_tmp_m_id_ _tmp_p_id_ $ 256
					%end;
					%if &sim_exist or &_sim_exist %then %do;
						_tmp_sim_ 8;
					%end;
					;
				%end;
				set &ts2 
					/*
					%if (&ProcessName= GenomicBLUP or &ProcessName = GenomicBayesianRegression) and &p_id_exist %then %do;
						(
						%if (&ProcessName= GenomicBLUP or &ProcessName = GenomicBayesianRegression) and &_p_id_exist %then %do;
							drop= _p_id _m_id
						%end;
						rename=(p_id= _p_id m_id= _m_id)
						)
					%end;
					*/
					%if &ProcessName= GenomicBLUP or &ProcessName = GenomicBayesianRegression %then %do;
						%if (&_cross_exist) or (&p_id_exist) %then %do;
							(rename= (
								%if &_cross_exist %then %do;
									_cross_= _cross_%eval(&_c+1)
								%end;
								%if &p_id_exist %then %do;
									p_id= _p_id_%eval(&_p+1)
									m_id= _m_id_%eval(&_p+1)
								%end;
								%else %if &_p_id_exist %then %do;
									_p_id_= _p_id_%eval(&_p+1)
									_m_id_= _m_id_%eval(&_p+1)									
								%end;
								)
							)
						%end;
					%end;
				;
				%if &ProcessName= GenomicBLUP or &ProcessName = GenomicBayesianRegression %then %do;
					%if &_cross_exist %then %do;
						if not missing(_cross_1) then _tmp_cross_= _cross_1; 
						%do i=2 %to &_c %by 1; 
							else if not missing(_cross_%eval(&i+1)) then _tmp_cross_= _cross_%eval(&i+1); 
						%end;
						else _tmp_cross_= input(trim(left(_n_)), $256.);
					%end;
					%else %do;
						_tmp_cross_= input(trim(left(_n_)), $256.);
					%end;
					%if &p_id_exist %then %do;
						if not missing(_p_id_%eval(&_p+1)) then _tmp_p_id_= _p_id_%eval(&_p+1);
						if not missing(_m_id_%eval(&_p+1)) then _tmp_m_id_= _m_id_%eval(&_p+1);
					%end;	
					%else %if &_p_id_exist %then %do;
						if not missing(_p_id_1) then _tmp_p_id_= _p_id_1;
						%do i=2 %to &_p %by 1; 
							else if not missing(_p_id_%eval(&i+1)) then _tmp_p_id_= _p_id_%eval(&i+1); 
						%end;
						if not missing(_m_id_1) then _tmp_m_id_= _m_id_1;
						%do i=2 %to &_p %by 1; 
							else if not missing(_m_id_%eval(&i+1)) then _tmp_m_id_= _p_id_%eval(&i+1); 
						%end;				
					%end;
					%if &sim_exist %then %do;
						_tmp_sim_= sim;
					%end;
					%else %if &_sim_exist %then %do;
						_tmp_sim_= _sim_;
					%end;
				; 
				%end;
				%if %length(&droplist &droplistG) %then %do;
					drop &droplist &droplistG;
				%end;
			run;
			
			%if &ProcessName= GenomicBLUP or &ProcessName = GenomicBayesianRegression %then %do; 
				%*if (&_cross_exist) or (&p_id_exist) or (&_p_id_exist) %then %do;
					proc datasets nolist;
						modify &ts2;
						rename 
							/*%if &_cross_exist %then*/ _tmp_cross_ = _cross_/*;*/
							%if &p_id_exist or &_p_id_exist %then _tmp_p_id_= _p_id_ _tmp_m_id_= _m_id_;
							%if &sim_exist or &_sim_exist %then _tmp_sim_= _sim_; 
						;
					quit;
				%*end;
			%end;			
		%end;
	%end;

	* stepwise variable selection;
	%let reduce = 0;
	%if (%index(&VarSelect,Stepwise)|%index(&VarSelect,Forward)|
		%index(&VarSelect,Backward)|%index(&VarSelect,Score)) and (&nix|&nip) %then %do; 
		* v4.1 updates:
		* add in the F|B|Score selections for utilizing the corresponding selection for PROC PHREG;
		* may update for Proc StepDisc and PROC LOGISTIC;
		%put nixl=&nixl;
		%put nipl=&nipl;
		%put nix=&nix;
		%put nip=&nip;

		data sum;
		run; 

		%if (&ProcessName = SurvivalPredictiveModeling) %then %do;
			data ParEst;
			run;
		%end;

		%if (&ProcessName = LogisticRegression) %then %do;

			%let lockin = %eval(&nixl+&nipl);
			%if &lockin or (&OrderInt > 1) or (&UseHP = No) %then %let whichproc = logistic;
			%else %let whichproc = hplogistic;
		
			proc &whichproc data=&ts1 &LogisticOptions namelen=128;
				%if &nipl | &nip %then %do;
					class 
						%do i=1 %to &nipl; %sysfunc(varname(&_dsipl_,&i)) %end; 
						%do i=1 %to &nip; %sysfunc(varname(&_dsip_,&i)) %end; 
						/ missing
					;
				%end;
				model &DepVar &LogisticResponseOptions =
					%do i=1 %to &nixl; 
						%sysfunc(varname(&_dsixl_,&i)) 
						%if &OrderInt > 1 %then %do;
							%do j = 2 %to &OrderInt; | %sysfunc(varname(&_dsixl_,&i)) %end;
							%if ((&i < &nixl) or (&nipl > 0) or (&nix > 0) or (&nip > 0)) %then %do; | %end;
						%end;
					%end; 
					%do i=1 %to &nipl; 
						%sysfunc(varname(&_dsipl_,&i)) 
						%if &OrderInt > 1 and ((&i < &nipl) or (&nix > 0) or (&nip > 0)) %then %do; | %end;
					%end; 
					%do i=1 %to &nix; 
						%sysfunc(varname(&_dsix_,&i)) 
						%if &OrderInt > 1 %then %do;
							%do j = 2 %to &OrderInt; | %sysfunc(varname(&_dsix_,&i)) %end;
							%if ((&i < &nix) or (&nip > 0)) %then %do; | %end;
						%end;
					%end; 
					%do i=1 %to &nip; 
						%sysfunc(varname(&_dsip_,&i)) 
						%if &OrderInt > 1 and (&i < &nip) %then %do; | %end;
					%end; 
					%if &OrderInt > 1 %then %do;
						@ &OrderInt
					%end;
					/
					%if &whichproc = logistic %then %do;
						selection=stepwise slentry=&SLEntry slstay=&SLStay %if &MaxStep > 0 %then %do; maxstep=&MaxStep %end; include=%eval(&nixl+&nipl)
					%end;
					&LogisticModelOptions
				;
				;
				%if &whichproc = hplogistic %then %do;
					selection method=stepwise(slentry=&SLEntry slstay=&SLStay %if &MaxStep > 0 %then %do; maxstep=&MaxStep %end;);
				%end;

				%if %length(&WeightVar) %then %do;
					weight &WeightVar;
				%end;
				&LogisticStatements;
				%if &whichproc = logistic %then %do;
					ods output modelbuildingsummary=sum;
				%end;
				%else %do;
					ods output selectionsummary=sum;
				%end;
			run;
			
			%if (&syserr>4 | &exiterror) %then %goto exit;
			%let effentered = effectentered;
			%let effremoved = effectremoved;
			data sum;
				set sum;
				if trim(left(&effentered)) = "Intercept" then delete;
			run;
			/*
			data OutLib.sum;
				set sum;
			run;
			*/
		%end;
		%else %if (&ProcessName = SurvivalPredictiveModeling) %then %do; 

			* set up baseline data;
			%if %length(&ColorVar) %then %do;
			
				%put ColorVar=&ColorVar IDVar=&IDVar;
				/*
				data OutLib.colorvar;
					set _ColorVar_;
				run;
				data OutLib.ts1;
					set &ts1;
				run;
				*/
				
				data _ts1c_;
					if 0 then set _ColorVar_;
					if _n_ = 1 then do;
						declare hash h(dataset: "work._ColorVar_", hashexp: 6);
						h.defineKey("&IDVar");
						h.defineData("&ColorVar");
						h.defineDone();
					end;
					set &ts1;
					rc = h.find();
					if (rc = 0) then output;
					drop rc;
				run;
				%let ts1 = _ts1c_;
				%if %length(&TestInData) | &CVFlag %then %do;
					data _ts2c_;
						if 0 then set _ColorVar_t;
						if _n_ = 1 then do;
							declare hash h(dataset: "work._ColorVar_t", hashexp: 6);
							h.defineKey("&IDVar");
							h.defineData("&ColorVar");
							h.defineDone();
						end;
						set &ts2;
						rc = h.find();
						if (rc = 0) then output;
						drop rc;
					run;
					%let ts2 = _ts2c_;
				%end;
			%end;
							
			%if &CVFlag %then %do;
				%let _CovData = &ts2;
			%end;
			%else %do;
				%if %length(&TestInData) %then %do;
					data _Covariates;
						set &ts1 &ts2;
					run;
					%let _CovData = _Covariates;
				%end;
				%else %do; 
					%let _CovData = &ts1;
				%end;
			%end;
			/*
			%if %length(&TestInData) | &CVFlag %then %do;
				data _Covariates;
					%if %length(&TestInData) %then %do;
						set &ts1 &intestdat;
					%end;
					%else %do;
						set &ts1 &ts2;
					%end;
				run;
			%end;
			%else %do; 
				data _Covariates;
					set &ts1;
				run;
			%end;
			%let _CovData = _Covariates;
			*/
			* delete rows with missing covariates;
			/*
			data &_CovData;
				set &_CovData;
				if nmiss(of 
					%if %length(&WeightVar) %then %do;
						&WeightVar 
					%end;
					%if %length(&CensorVar) %then %do;
						&CensorVar 
					%end;
					&Covariates 
					%do i=1 %to &nix; %sysfunc(varname(&_dsix_,&i)) %end; 
					%do i=1 %to &nip; %sysfunc(varname(&_dsip_,&i)) %end; 
					) then delete;
			run;	
			*/

			%if %index(&VarSelect,Stepwise)|%index(&VarSelect,Forward)|%index(&VarSelect,Backward) %then %do;

				%put phreg ts1 = &ts1;
				
				/*
				data OutLib.ts1;
					set &ts1;
				run;
				data OutLib.covdata;
					set &_CovData;
				run;
				*/
			
				proc phreg data=&ts1 namelen=128 %if %length(&PhregOptions) %then &PhregOptions;;
					%if &nipl | &nip %then %do;
						class 
							%do i=1 %to &nipl; %sysfunc(varname(&_dsipl_,&i)) %end; 
							%do i=1 %to &nip; %sysfunc(varname(&_dsip_,&i)) %end; / missing
						;
					%end;	
					%if %length(&WeightVar) %then %do;
						weight &WeightVar;
					%end;
					%if %length(&CensorVar) %then %do;
						model &DepVar*&CensorVar(&CensorValues) =	 
					%end;
					%else %do; 
						model &DepVar = 
					%end;	
						%do i=1 %to &nixl; 
							%sysfunc(varname(&_dsixl_,&i)) 
							%if &OrderInt > 1 %then %do;
								%do j = 2 %to &OrderInt; | %sysfunc(varname(&_dsixl_,&i)) %end;
								%if ((&i < &nixl) or (&nipl > 0) or (&nix > 0) or (&nip > 0)) %then %do; | %end;
							%end;
						%end; 
						%do i=1 %to &nipl; 
							%sysfunc(varname(&_dsipl_,&i)) 
							%if &OrderInt > 1 and ((&i < &nipl) or (&nix > 0) or (&nip > 0)) %then %do; | %end;
						%end; 
						%do i=1 %to &nix; 
							%sysfunc(varname(&_dsix_,&i)) 
							%if &OrderInt > 1 %then %do;
								%do j = 2 %to &OrderInt; | %sysfunc(varname(&_dsix_,&i)) %end;
								%if ((&i < &nix) or (&nip > 0)) %then %do; | %end;
							%end;
						%end; 
						%do i=1 %to &nip; 
							%sysfunc(varname(&_dsip_,&i)) 
							%if &OrderInt > 1 and (&i < &nip) %then %do; | %end;
						%end; 
						%if &OrderInt > 1 %then %do;
							@ &OrderInt
						%end;
						%if %index(&VarSelect,Stepwise) %then %do;
							/ selection=stepwise slentry=&SLEntry slstay=&SLStay	%if &MaxStep > 0 %then %do; maxstep=&MaxStep %end; include=%eval(&nixl+&nipl) 
							%if %length(PhregModelOptions) %then &PhregModelOptions; 
						%end;
						%else %if %index(&VarSelect,Forward) %then %do;	
							/ selection=forward slentry=&SLEntry include=%eval(&nixl+&nipl)	 
							%if %length(PhregModelOptions) %then &PhregModelOptions; 
						%end;	
						%else %if %index(&VarSelect,Backward) %then %do; 
							/ selection=backward slstay=&SLStay	include=%eval(&nixl+&nipl)  
							%if %length(PhregModelOptions) %then &PhregModelOptions; 
						%end;
					; * end of Model statement;
					
					baseline covariates=&_CovData 
						out=SurFunc(keep=_obs _train_ &DepVar &CensorVar &IDVar &ColorVar &WeightVar Survival StdErrSurvival %if &CVFlag %then %do; __id %end;)
						survival=_all_ / rowid=&IDVar
					;
					/*
					%if ^&CVFlag %then %do;
						assess 
							var=(
								&Covariates
								%do i=1 %to &nix; %sysfunc(varname(&_dsix_,&i)) %end; 
								%do i=1 %to &nip; %sysfunc(varname(&_dsip_,&i)) %end; 
							) ph / crpanel resample seed=19
						;
					%end;
					*/
					ods output ModelBuildingSummary=Sum parameterestimates=ParEst;
				run; 
			%end;
			%else %if %index(&VarSelect,Score) %then %do;
				* 1. Select effect by Score option first (without fitting model);
				* 2. Call PROC PHREG second time to fit model with the effects selected in step 1.;
				%if %eval(&nix+&nip) < &nScorePredictors %then %do;
					%let nTmp = %trim(%left(%eval(&nix+&nip)));
					%put WARNING: The specified Number of Predictors Included in Model, %trim(%left(&nScorePredictors)), is larger than;
					%put the total predictors, &nTmp, left after Predictor Reduction.;
					%put WARNING: The Number of Predictors Included in Model for Score Model Selection will be replaced by &nTmp.;
					%let nScorePredictors = %eval(&nix+&nip);
				%end;
				* check how many combination of factor selections for Score approach;
				data _null_;
					format n2 8.2;
					x = %eval(&nix+&nip);
					y = &nScorePredictors;
					n = (lperm(x,y)-lperm(y))/log(10);
					n1 = floor(n);
					n2 = 10**(n-n1); 
					if n>17 then do;
						put 'WARNING: There are ' n2 @;
						put +(-1) 'e+' n1 'variable selections to be screened to search for the best selection.';
						put 'WARNING: It can take a long time to search for the best score among these selections.';
					end;
					else do;
						put 'There are ' n2 @;
						put +(-1) 'e+' n1 'variable selections to be screened.';
					end;
				run;

				proc phreg data=&ts1 %if %length(&PhregOptions) %then &PhregOptions;; 
					%if &nipl | &nip %then %do;
						class 
							%do i=1 %to &nipl; %sysfunc(varname(&_dsipl_,&i)) %end; 
							%do i=1 %to &nip; %sysfunc(varname(&_dsip_,&i)) %end; / missing
						;
					%end;
					%if %length(&WeightVar) %then %do;
						weight &WeightVar;
					%end;
					%if %length(&CensorVar) %then %do;
						model &DepVar*&CensorVar(&CensorValues) =
					%end;
					%else %do; 
						model &DepVar = 
					%end;	
						%do i=1 %to &nixl; %sysfunc(varname(&_dsixl_,&i)) %end; 
						%do i=1 %to &nipl; %sysfunc(varname(&_dsipl_,&i)) %end; 
						%do i=1 %to &nix; %sysfunc(varname(&_dsix_,&i)) %end; 
						%do i=1 %to &nip; %sysfunc(varname(&_dsip_,&i)) %end; 
							/ selection=Score best=1 start=&nScorePredictors stop=&nScorePredictors include=%eval(&nixl+&nipl)
							%if %length(PhregModelOptions) %then &PhregModelOptions;
					;
					ods output BestSubsets=BestSet;
				run;

				data _null_;
					set BestSet end=_e_;
					if _e_ then call symput("ScoreEffects",VariablesInModel);
				run;

				* call PROC PHREG second time to fit scored model;
				proc phreg data=&ts1 %if %length(&PhregOptions) %then &PhregOptions;; 
					%if &nipl | &nip %then %do;
						class 
							%do i=1 %to &nipl; %sysfunc(varname(&_dsipl_,&i)) %end; 
							%do i=1 %to &nip; %sysfunc(varname(&_dsip_,&i)) %end; / missing
						;
					%end;	
					%if %length(&WeightVar) %then %do;
						weight &WeightVar;
					%end;
					%if %length(&CensorVar) %then %do;
						model &DepVar*&CensorVar(&CensorValues) = &ScoreEffects
							%if %length(PhregModelOptions) %then /&PhregModelOptions; ;
					%end;
					%else %do; 
						model &DepVar = &ScoreEffects
							%if %length(PhregModelOptions) %then /&PhregModelOptions; ;
					%end;	
				
					baseline covariates=&_CovData 
						out=SurFunc(keep=_obs _train_ &DepVar &CensorVar &IDVar &ColorVar &WeightVar Survival StdErrSurvival)
						survival=_all_ / rowid=Id
					;
					
					ods output parameterestimates=ParEst;
				run;
			%end;	
			%put syserr = &syserr;
			%if (&syserr>4 | &exiterror) %then %goto exit;
			%let effentered = effectentered;
			%let effremoved = effectremoved;
		%end;
		%else %do;
			proc stepdisc data=&ts1 slentry=&SLEntry slstay=&SLStay %if &MaxStep > 0 %then %do; maxstep=&MaxStep %end; include=%eval(&nixl+&nipl) short;
				class &DepVar;	
				var    	 
					%do i=1 %to &nixl; %sysfunc(varname(&_dsixl_,&i)) %end; 
					%do i=1 %to &nipl; %sysfunc(varname(&_dsipl_,&i)) %end; 
					%do i=1 %to &nix; %sysfunc(varname(&_dsix_,&i)) %end; 
					%do i=1 %to &nip; %sysfunc(varname(&_dsip_,&i)) %end; 
				;
				ods output summary=sum;
			run;
			%if (&syserr>4 | &exiterror) %then %goto exit;
			%let effentered = entered;
			%let effremoved = removed;
		%end;

		%if (&ProcessName ne SurvivalPredictiveModeling) %then %do;
			%NObsVars(sum);
		%end;
		%else %do;
			%NObsVars(ParEst);
		%end;
		
		%put nobs stepwise = &nobs;

		%if (&nvars and &nobs) or &nixl or &nipl %then %do;
			%if (&ProcessName = SurvivalPredictiveModeling) %then %do;
				* extract selected variables into macro variables (including covariates);
				%let SelVars = ;
				* avoid duplicate record for class variable;
				data PE;	
					set ParEst(keep=Parameter);
					row = _n_;
				run;
				proc sort data=PE nodupkey;
					by Parameter;
				run;
				proc sort data=PE;
					by row;
				run;
				data _null_;
					set PE end=_e_;
					name = "sVar"||trim(left(_n_));
					call symput(name,Parameter);
					if _e_ then call symput("NumVar",_n_);
				run; 
				* build single macro variable with entered variables;
				%do i = 1 %to &NumVar;
					* covariate included;
					%let SelVars = &SelVars &&sVar&i; 
				%end;
			%end;
			%else %if %index(&VarSelect,Stepwise) %then %do;
				%let nsteps = 0;
				* extract selected variables into macro variables;
				data sum;
					set sum end=end;
					en = "e" || trim(left(_n_));
					rn = "r" || trim(left(_n_));
					call symput(en,trim(left(&effentered)));
					call symput(rn,trim(left(&effremoved)));
					if end=1 then call symput('nsteps',trim(left(_n_)));
				run;

				* build single macro variable with entered variables;
				%do i=1 %to &nixl;
					%let SelVars = &Selvars %sysfunc(varname(&_dsixl_,&i));
				%end;
				%do i=1 %to &nipl;
					%let SelVars = &Selvars %sysfunc(varname(&_dsipl_,&i));
				%end;
				
				%put nsteps = &nsteps;

				%let NumVar = 0;
				%do s=1 %to &nsteps;
					%put e&s = &&e&s;
					%put r&s = &&r&s;
					%if %length(%qtrim(&&e&s)) %then %do;
						%put entering &&e&s;
						%let SelVars = &SelVars &&e&s;
						%let NumVar = %eval(&NumVar + 1);
					%end;
					%if (%length(%qtrim(&&r&s)) and %qtrim(&&r&s) ne %quote(.)) %then %do;
						/*
						%let sm1 = %eval(&s-1);
						%if (%quote(&&r&s) = %quote(&&e&s)) or ((&s > 1) and (%quote(&&e&s) = ) and (%quote(&&r&s) = %quote(&&e&sm1))) %then %do;
							%put not removing &&r&s because it was just entered;
						%end;
						%else %do;
						*/
							%put removing &&r&s;
							%let lr = %length(%trim(&&r&s));
							%let idx = %index(&SelVars,&&r&s);
							%let idx2 = %eval(&idx+&lr+1);
							%if (&idx = 1) %then %do;
								* removing the first variable in the string;
								%let SelVars = %substr(&SelVars,&idx2);
							%end;
							%else %if (&idx2 > %length(%trim(&SelVars))) %then %do;
								* removing the last variable in the string;
								%let SelVars = %substr(&SelVars,1,%eval(&idx-1));
							%end;
							%else %do;
								* removing variable from the middle of the string;
								%let SelVars = %substr(&SelVars,1,%eval(&idx-1)) %substr(&SelVars,&idx2);
							%end;
							%let NumVar = %eval(&NumVar - 1);
						/*
						%end;
						*/
					%end;
				%end;
			%end;	
		%end;
		%let reduce = 1;
	%end;	
	%else %if %index(&VarSelect,Genetic) %then %do;
		* skip this part for Survival data;
		%let ClassKeep = &DepVar;
		%if (&ProcessName = PartialLeastSquares) %then %do;
			%do i=1 %to &nic; 
				%let ClassKeep = &ClassKeep &&_c&i;
			%end; 
			%do i=1 %to &nit; 
				%let ClassKeep = &ClassKeep &&_t&i;
			%end; 		
		%end;
		
		%SelectVarsGA(
			Data=&ts1,
			Class=&ClassKeep,
			Vars=
				%do i=1 %to &nix; %sysfunc(varname(&_dsix_,&i)) %end;  
				%do i=1 %to &nip; %sysfunc(varname(&_dsip_,&i)) %end; 
			,
			WeightVar=&WeightVar,
			GroupSize=&NumVar,
			MaxIter=&MaxIter,
			StopValue=&StopVal
		);
						
		%if &exiterror = 1 %then %goto exit;
						
		data vipd;
			set VariableImportance;
		run;
		data bpd;
			set BestPredictorRuns(obs=10);
		run;

		* use the first set from BestPredictorRuns as the selected variables;
		data bpd1;
			set BestPredictorRuns(obs=1);
			%do s=1 %to &NumVar;
				call symput("e&s",var&s);
			%end;
		run;

		/*---use the top &NumVar important variables as the selected variables---*/
		/*
		data vsel;
			set VariableImportance(obs=&NumVar);
			en = "e" || trim(left(_n_));
			call symput(en,trim(left(variable)));
		run;
		*/

		%do s=1 %to &NumVar;
			%let SelVars = &SelVars &&e&s;
		%end;		
		%let reduce = 1;
	%end;
	%else %do;
		* without variable selection;
		%if ^&upred %then %do; 
			%let SelVars = &LockContPredVars &LockClassPredVars &PredictorVars &ClassPredVars;   
		%end; 
		%else %do;
			%let SelVars = &LockContPredVars &LockClassPredVars &ClassPredVars;
			%do j=1 %to &nix_blocks; 
				%let SelVars = &SelVars &&PredictorVars&j;
			%end;
		%end;
	%end;
	* end of variable selection;
	
	* if no variables selected, create a dummy intercept as the only predictor;
	%*put SelVars = &SelVars;
	%let SelVars = %trim(&SelVars);
	%if ^%length(&SelVars) %then %do;
	
		%put WARNING: No variables selected.  An intercept-only model is assumed.;
		
		%if ^%index(&PredictorVars &ClassPredVars,Intercept) %then %do;
			%put adding Intercept;
			data &ts1;
				set &ts1;
				Intercept = 1;
				%if %length(%trim(&PredictorVars &ClassPredVars)) %then %do;
					drop %if ^&upred %then %do; &PredictorVars %end; %else %do; %do j=1 %to &nix_blocks; &&PredictorVars&j %end; %end; &ClassPredVars;
				%end;
			run;
			
			%if &ts2^=&ts1 %then %do;
				data &ts2;
					set &ts2;
					Intercept = 1;
					%if %length(%trim(&PredictorVars &ClassPredVars)) %then %do;
						drop %if ^&upred %then %do; &PredictorVars %end; %else %do; %do j=1 %to &nix_blocks; &&PredictorVars&j %end; %end; &ClassPredVars;
					%end;
				run;
			%end;
		%end;
		%let SelVars = Intercept;

		%if %symexist(_dsix_) %then %do;
			%put _dsix_ = &_dsix_;
			%if (&_dsix_) %then %do;
				%let _dsix_ = %sysfunc(close(&_dsix_));
			%end;
		%end;
		%if %symexist(_dsip_) %then %do;
			%if (&_dsip_) %then %do;
				%let _rc_ = %sysfunc(close(&_dsip_));
			%end;
		%end;
		%let nip = 0;
		%let nix = 1;
		data dsix;
			Intercept = 1;
		run;
		%let _dsix_ = %sysfunc(open(dsix));
		%put _dsix_=&_dsix_ varname=%sysfunc(varname(&_dsix_,1));
	%end;
	%else %do;
		%put reduce = &reduce;
		%if %index(&SelVars,*) %then %do;
			%let KeepVars = %Search_and_Replace(string=&SelVars,search=%str(*),replace=%str( ));
		%end;
		%else %do;
			%let KeepVars = &SelVars;
		%end;
		%if &reduce %then %do;
			data v;
				set d(obs=0);
				keep &KeepVars;
			run;
			proc transpose data=v out=vt1;
				var _numeric_;
			run;
			proc transpose data=v out=vt2;
				var _character_;
			run;
			data vt;
				length _name_ $ 32;
				set vt1 vt2;
			run;
			/*
			data OutLib.vt;
				set vt;
			run;
			*/
			%ReduceData(d,vt,&intestdat,Selection,&RecoverClassPredVars);
		%end;
	%end;
	
	
	******************************************************************;
	* main computing block, with sections for each predictive method *;
	******************************************************************;
	
	%if (&BJ_Est = Yes) %then %do;
		title2 "Buckley-James Iteration &BJ_Niter";
	%end;
	
	%if %length(&SelVars) %then %do;
		%let SomeSel = 1;
		%let havets2v = 0;
		%put SelVars = &SelVars;
		
		%if ^%length(&TruthVar) %then %let TruthVar = &DepVar;
		%if %length(%trim(&WeightVar)) %then %let ASet = &trWeight = &WeightVar;

		* set up for asymmetric loss iteration loop, implementing iteratively reweighted; 
		* maximum likelihood as in Efron (1991) "Regression Percentiles" Statistica Sinica;
		%if &AsymmetricLossFitProp ne 0.5 %then %do;
			%if %length(&ASet) %then %let ASet = &ASet, Asymmetric Loss Fitting Proportion = &AsymmetricLossFitProp;
			%else %let ASet = Asymmetric Loss Fitting Proportion = &AsymmetricLossFitProp;
			%if %length(&WeightVar) %then %do;
				%let WeightVar0 = &WeightVar;
			%end;
			%else %do;
				%let WeightVar0 = ;
			%end;
			%let WeightVar = _Asymmetric;
			data &ts1;
				set &ts1;
				_Row_ = _n_;
				&WeightVar = 0.5;
				%if %length(&WeightVar0) %then %do;
					&WeightVar = &WeightVar * &WeightVar0;
				%end;
			run;
		%end;
		%let aloss_iterating = 1;
		%let aloss_niter = 1;
		%let aloss_maxiter = 30;
		
		* asymmetric loss loop, if loss is symmetric then this just gets executed once;
		%do %while(&aloss_iterating);

			data ts2o;
				if 0;
			run;
			data ts2vars;
				if 0;
			run;
		
			%if &AsymmetricLossFitProp ne 0.5 %then %do;
				%put Beginning asymmetric loss iteration &aloss_niter;
				title3 "Asymmetric Loss Iteration &aloss_niter";
			%end;

			* fit models;
			%if (&ProcessName = LogisticRegression) %then %do;

				* string for analysis settings;
				%if %length(&ASet) %then %do;
					%let ASet = &ASet, &trVariableSelectionMethod = %LocaleEZX(%nrbquote(&VarSelect), EIA=1);
				%end;
				%else %do;
					%let ASet = &trVariableSelectionMethod = %LocaleEZX(%nrbquote(&VarSelect), EIA=1);
				%end;	
				%if (&VarSelect = Stepwise) %then %do;
					%let ASet = &Aset, &trSLEntry = &SLEntry, &trSLStay = &SLStay;
				%end;

				%if (&VarSelect = Penalized) %then %do;

					%if ^&CVFlag %then %do;
						%let plib = %scan(&outdat,1,'.');
						%let pdat = %scan(&outdat,2,'.');
						%if %length(&pdat) > 28 %then %do;
							%let pdat = %substr(&pdat,1,28);
						%end;
					%end;

					%if (&_dtype_=C) or (&NumDepAsClass=Yes) %then %let trait_type = qualitative;
					%else %let trait_type = quantitative;

					%if &CVFlag %then %do;
						%let savep = NO;
						%let podat = ;
						%let pfit = ;
					%end;
					%else %do;
						%let savep = YES;
						%let podat = &plib..&pdat._mod;
						%let pfit = &plib..&pdat._fit;
						%if %length(&ts2) and &ts2 ne &ts1 %then %do;
							data &ts2;
								set &ts1 &ts2;
							run;
						%end;
					%end;

					%if (&FR_Inds = Yes) %then 
						%let GeneSelectOptions = &GeneSelectOptions nway=1;
					%else %do;
						%if &OrderInt > 0 %then 
							%let GeneSelectOptions = &GeneSelectOptions nway=&OrderInt;
						%else
							%let GeneSelectOptions = &GeneSelectOptions nway=2;
					%end;

					%if ^&CVFlag %then %do;
						%let GeneSelectOptions = &GeneSelectOptions plrdropfit=yes;
					%end;

					%geneselect(
						train_dsname=&ts1,
						test_dsname=&ts2,
						where_clause=,
						inmodel_dsname=,
						method=plr,
						trait=&DepVar,
						trait_type=&trait_type,
						quantitative_vars=_dataset_,
						qualitative_vars=_dataset_,
						n_quantitative_lists=0,
						n_qualitative_lists=0,
						n_vars=,
						n2way=,
						weight=&WeightVar,
						save_model=,
						model_output=,
						save_parms=&savep,
						parms_output=&podat,
						save_fit=&pfit,
						save_importance=,
						importance=ts2i,							
						importance_var=,
						importance_out=,
						importance_outfit=,
						save_score=YES,
						score_out=ts2o,
						score_outfit=,
						convert_to_cvscore=YES,
						proc_options=&GeneSelectOptions,
						priors=&Priors
					);

					/*
					data OutLib.ptrain;
						set ptrain;
					run;

					data OutLib.parms;
						set &podat;
					run;

					data OutLib.ts2i;
						set ts2i;
					run;
					*/

					%put pfit=&pfit;

					%if &CVFlag %then %do;
						%if (&FR_Inds = Yes) %then %do;
							* IF CVMC, REMOVE _Ixx INTERACTION INDICATOR FROM THE SAVE IMPORTANCE= DATA;
							%NObsVars(&path);
							%let _nObspath_ = &nObs;
							%if &_nObspath_ %then %do;
								proc sort data=ts2i;
									by NAME;
								run;
								data indicators;
									set friidef;
									rename X1 = NAME;
									TYPE = "Indicator"; 
								run;
								proc sort data=indicators;
									by NAME;
								run;
								data ts2i;
									merge ts2i indicators;
									by NAME;
									drop TYPE;
									if TYPE="Indicator" then delete;
								run;
							%end;	
						%end;
					%end;
					%else %do;
						%if %length(&pfit) > 0 and &pfit ne "" %then %do;
							%NObsVars(&pfit);
							%if &nObs %then %do;
								%let _pfitid_ = %sysfunc(open(&pfit));
								%let x2num = %sysfunc(varnum(&_pfitid_, X2));
								%if &x2num %then %do;
									%let x2type = %sysfunc(vartype(&_pfitid_, &x2num));
								%end;
								%let _pfitid_ = %sysfunc(close(&_pfitid_));
								data &pfit;                                                                                                                              
									set &pfit;  
									length Predictor $64;
									drop base_ase base_deviance;                                                                                                            
									retain base_ase base_deviance 0; 
									%if &x2num %then %do;
										if
										%if (&x2type=C) %then %do;  
											trim(left(X2))="" then do;
										%end;
										%else %do;
											X2=. then do;
										%end;
											Predictor=trim(left(X1));
										end;
										else do;
											Predictor=trim(left(X1))||" * "||trim(left(X2));	
										end; 
									%end;
									%else %do;
										Predictor=trim(left(X1));
									%end; 
									if X1="Full Model" then do;                                                                                                             
										base_ase = ASE;                                                                                                                      
										base_deviance = DEVIANCE;                                                                                                            
									end;                                                                                                                                    
									ASE_LIFT = (ASE - BASE_ASE)/BASE_ASE;                                                                                                   
									DEVIANCE_LIFT = (DEVIANCE - BASE_DEVIANCE)/BASE_DEVIANCE;                                                                               
									label ASE_LIFT = "&trAveSquareErrorLift";                                                                                               
									label DEVIANCE_LIFT = "&trDevianceLift";     
									label Predictor="&trFinalSelectedVariables";
								run;       
							%end;
						%end;

						%if (&FR_Inds = Yes) %then %do;

							proc sort data=&pfit;                                                                                                                     
								by X1;                                                                                                                                  
							run;    

							proc sort data=friidef;
								by X1;
							run;

							data &pfit;                                                                                                                             
								merge &pfit friidef;                                                                                                                         
								by X1;             
								if ASE = . then delete;  
								if trim(left(INDICATOR_DEFINITION)) ne "" then do;
									Predictor = INDICATOR_DEFINITION;
								end;
							run; 

							%NObsVars(&pfit);
							%if &nObs %then %do;
								%let _pfitid_ = %sysfunc(open(&pfit));
								%let x2num = %sysfunc(varnum(&_pfitid_, X2));
								%let idefnum = %sysfunc(varnum(&_pfitid_, INDICATOR_DEFINITION));
								%let _pfitid_ = %sysfunc(close(&_pfitid_));
								%if &x2num or &idefnum %then %do;
									proc freq data=&pfit noprint;
										%if &x2num %then %do;
											tables X2/out=x2frq;
										%end;
										%if &idefnum %then %do;
											tables INDICATOR_DEFINITION/out=idfrq;
										%end;
									run;  
								%end;
								%if &x2num %then %do;
									%NObsVars(x2frq);
									%if &nObs = 1 %then %do;
										data _null_;
											set x2frq;
											if X2="" or X2=. then do;
												call symput("dropx2", 1);
											end;
										run;	
										%if &dropx2=1 %then %do;
											data &pfit;
												set &pfit;
												drop X2;
											run;
										%end;
									%end;
								%end;
								%if &idefnum %then %do;
									%NObsVars(idfrq);
									%if &nObs = 1 %then %do;
										%let dropid = 0;
										data _null_;
											set idfrq;
											if INDICATOR_DEFINITION="" then do;
												call symput("dropid", 1);
											end;
										run;	
										%if &dropid=1 %then %do;
											data &pfit;
												set &pfit;
												drop INDICATOR_DEFINITION;
											run;
										%end;
									%end;
								%end;
							%end;
						%end;
					%end;

					%if &syserr > 4 %then %do;
						%let exiterror = 1;
						%goto exit;
					%end;
					%if ^&CVFlag %then %do;
						title "Penalized Logistic Regression Model";
						proc print data=&podat;
						run;
						title;
						proc sort data=ts2o;
							by _Obs;
						run;
					%end;

					/*
					data OutLib.ts2o;
						set ts2o;
					run;
					*/

					* insert underscore in p_ var names;
					data p1;
						set ts2o(obs=1);
						keep p_:;
					run;
					proc transpose data=p1 out=p1t;
						var p_:;
					run;
					data _null_;
						set p1t;
						call symput("pgprob"||trim(left(_n_)),trim(left(_name_)));
					run;
					%let tlen = %eval(%length(&DepVarTrunc30)+3);
					%put tlen=&tlen;
					data ts2o;
						set ts2o;
						rename 
							%do c=1 %to &nclasses; 
								%let lastpart = %substr(&&pgprob&c,&tlen);
								%let newname = P_&DepVarTrunc30._&lastpart;
								&&pgprob&c = &newname
							%end; 
						;
						* drop I_&DepVarTrunc30 to avoid type conflicts while merging results with other APs.;
						drop I_&DepVarTrunc30;
					run;
					%if (&TestSpec=Yes) %then %do;
						data ts2o;
							set ts2o;
							array pp %do c=1 %to &nclasses; &&p_prob&c %end; ;
							_cidx = .;
							%do c=1 %to &nclasses;
								%if (&_dtype_=N) %then %do;
									if trim(left(&TruthVar)) = &&class&c then _cidx = &c; 
								%end;
								%else %do;
									if trim(left(&TruthVar)) = "&&class&c" then _cidx = &c; 						
								%end;
							%end;
							if not missing(_cidx) then actualp = pp[_cidx];
							else actualp = .;
						run;
						%if ^&CVFlag %then %do;
							data &ts1;
								set ts2o;
								where _train_ = 1;
							run;
							data &ts2;
								set ts2o;
								where _train_ = 0;
							run;
						%end;
					%end;

					%Rename_P_;

					%let into = U_&DepVarTrunc30;
					%let pred = P_&DepVarTrunc30;

					data ts2vars;
						set ts2i;
						if importance > 0;
					run;

					%let havets2v = 1;
					%UpdateSelVars;
				%end;
				%else %do;

					%let CompleteSeparation = 0;
					
					proc logistic data=&ts1 &LogisticOptions namelen=128;	
						%if &nipl | &nip %then %do;
							class 
								%do i=1 %to &nipl; %sysfunc(varname(&_dsipl_,&i)) %end; 
								%do i=1 %to &nip; %sysfunc(varname(&_dsip_,&i)) %end; 
								/ missing param=glm
							;
						%end;
						model &DepVar &LogisticResponseOptions = 
							%if %index(&VarSelect,Stepwise) or %index(&VarSelect,Genetic) %then %do;
								&SelVars
							%end;
							%else %do;
								%do i=1 %to &nixl; %sysfunc(varname(&_dsixl_,&i)) %end; 
								%do i=1 %to &nipl; %sysfunc(varname(&_dsipl_,&i)) %end; 
								%do i=1 %to &nix; %sysfunc(varname(&_dsix_,&i)) %end; 
								%do i=1 %to &nip; %sysfunc(varname(&_dsip_,&i)) %end; 
							%end;
							/ &LogisticModelOptions;
						%if %length(&WeightVar) %then %do;
							weight &WeightVar;
						%end;
						%else %if (&Priors^=Proportional) %then %do;
							weight _Prior;
						%end;
						&LogisticStatements;
						output predprobs=i out=ts1o(drop=
							%do i=1 %to &nixl; %sysfunc(varname(&_dsixl_,&i)) %end; 
							%do i=1 %to &nipl; %sysfunc(varname(&_dsipl_,&i)) %end; 
							%do i=1 %to &nix; %sysfunc(varname(&_dsix_,&i)) %end; 
							%do i=1 %to &nip; %sysfunc(varname(&_dsip_,&i)) %end; 
						);
						score data=&ts2 out=ts2o(drop=
							%do i=1 %to &nixl; %sysfunc(varname(&_dsixl_,&i)) %end; 
							%do i=1 %to &nipl; %sysfunc(varname(&_dsipl_,&i)) %end; 
							%do i=1 %to &nix; %sysfunc(varname(&_dsix_,&i)) %end; 
							%do i=1 %to &nip; %sysfunc(varname(&_dsip_,&i)) %end; 
						) fitstat;
						ods output responseprofile=_namedat convergencestatus=status parameterestimates=pe;;
					run;

					/*
					data OutLib.ts1o;
						set ts1o;
					run;
					data OutLib.ts2o;
						set ts2o;
					run;
					*/

					%if &syserr > 4 %then %do;
						%let exiterror = 1;
						%goto exit;
					%end;

					/*
					data OutLib.pe;
						set pe;
					run;
					*/

					%if ^&CVFlag %then %do;
						* create parameter with class variable values concatenated;
						%let _did_ = %sysfunc(open(pe));
						%let ncv = 0;
						%do %while(%sysfunc(varnum(&_did_,ClassVal&ncv)));
							%let ncv = %eval(&ncv+1);
						%end;
						%let ncv = %eval(&ncv-1);
						%let _did_ = %sysfunc(close(&_did_));
						%put ncv = &ncv;
						data pe;
							set pe;
							%if &ncv >= 0 %then %do;
								length Parameter $ 64;
								Parameter = trim(left(Variable));
								%do i=0 %to &ncv;
									if not missing(ClassVal&i) and lengthn(trim(left(ClassVal&i))) then do;
										Parameter = catx(" ",Parameter,ClassVal&i);
									end;
								%end;
							%end;
							%else %do;
								rename Variable=Parameter;
							%end;
						run;
						
						%ScoringCodeMacroVars;
						
						%let suffix= _lr_pe;
						%let b32 = %eval(32 - %length(&suffix));
						%if %length(&OutName) > &b32 %then %let inname = %substr(&OutName,1,&b32);
						%else %let inname = &OutName;		
						%let peout = %quote(&inname.%trim(%left(&suffix)));
						%put peout = &peout;

						data OutLib.%unquote(&peout);
							set pe1;
						run;
						
					%end;

					* check for complete separation;
					data status;
						set status;
						if length(reason) >= 19 then do;
							if substr(reason,1,19) = "Complete separation" then do;
								call symputx("CompleteSeparation",1);
							end;
						end;
					run;
					%put CompleteSeparation = &CompleteSeparation;
					title "status";
					proc print data=status;
					run;
					title;

					* if there is a test set specified, rename the _IP: variables in ts1o to _P: so they agree with those in ts2o;
					%if (&TestSpec = Yes) %then %do;
						data t1;
							set ts1o(obs=1);
							keep ip_:;
						run;
						%let _dsid_ = %sysfunc(open(t1));
						%let nvars = %sysfunc(attrn(&_dsid_,NVARS));
						data ts1o;
							set ts1o;
							rename 
								%do i=1 %to &nvars;
									%let ipname = %sysfunc(varname(&_dsid_,&i));
									%let pname = %substr(&ipname,2,%eval(%length(&ipname)-1));
									&ipname = &pname
								%end;
								_INTO_ = I_&DepVarTrunc30;
							;
						run;
						%let _rc_ = %sysfunc(close(&_dsid_));
						/*
						title "ts1o after rename";
						proc print data=ts1o;
						run;
						*/
					%end;

					/*data OutLib._namedat; set _namedat; run;*/
					
					* create variables for computing cv ase;
					data _namedat;
						set _namedat end=_last;
						cn = "cvalue" || trim(left(_n_));
						call symput(cn,trim(left(Outcome)));
						if (_last) then call symput('ncl',trim(left(_n_)));
					run;

					%put ncl = &ncl;
					%global cvartype;
					%VarAttributes(ts2o,&TruthVar,_Len=,_Type=cvartype);
					data ts2o;
						set %if (&TestSpec = Yes) %then %do; ts1o %end; ts2o;
						array pp p_:;
						_cidx = .;
						%do c=1 %to &ncl;
							%if (&cvartype=N) %then %do;
								if trim(left(&TruthVar)) = &&cvalue&c then _cidx = &c; 
							%end;
							%else %do;
								if trim(left(&TruthVar)) = "&&cvalue&c" then _cidx = &c; 						
							%end;
						%end;
						if not missing(_cidx) then actualp = pp[_cidx];			
						else actualp = .;
					run;
					
					/*data OutLib.ts2o2; set ts2o; run;*/

					* for complete separation, set training posterior probabilities;
					%if (&CompleteSeparation) %then %do;
						data ts2o;
							set ts2o;
							array pp p_:;
							if not missing(_cidx) and _train_ = 1 then do;
								do _c=1 to &ncl;
									if _cidx = _c then pp[_c] = 0.9999;
									else pp[_c] = 0.0001;
								end;
								actualp = pp[_cidx];
								F_&DepVarTrunc30 = &DepVar;
								I_&DepVarTrunc30 = &DepVar;
								drop _c;
							end;
						run;
					%end;

					%Rename_P_;
					
					/*data OutLib.ts2o3; set ts2o; run;*/					

					%let into = I_&DepVarTrunc30;
					%let pred = P_&DepVarTrunc30;
				%end;
			%end;
			%else %if ((&ProcessName = PartitionTrees) or (&ProcessName = VirtualTwins)) %then %do;

				* string for analysis settings;
				%if &aloss_niter = 1 %then %do;
					%if %length(&ASet) %then %do;
						%let ASet = &ASet, &trTreeModelType = %LocaleEZX(%nrbquote(&ModelType), EIA=1);
					%end;
					%else %do;
						%let ASet = &trTreeModelType = %LocaleEZX(%nrbquote(&ModelType), EIA=1);
					%end;
				%end;

				%if (&_dtype_=C) or (&NumDepAsClass=Yes) %then %let trait_type = qualitative;
				%else %let trait_type = quantitative;

				%if &CVFlag or (&BJ_Est = Yes and ^&BJ_OneMore) %then %do;
					%let savem = NO;
					%let modat = ;
					%let impdat = ;
					%let saveimp = NO;
					%let N_ImpVars = ;
				%end;
				%else %do;
					%let savem = YES;
					%if %index(&outdat,.) %then %do;
						%let mlib = %scan(&outdat,1,'.');
						%let mdat = %scan(&outdat,2,'.');
					%end;
					%else %do;
						%let mlib = work;
						%let mdat = &outdat;
					%end;
					%if %length(&mdat) > 28 %then %do;
						%let mdat = %substr(&mdat,1,28);
					%end;
					%let modat = &mlib..&mdat._mod;
					%let N_ImpVars = ;
					%let impdat = ;
					%let saveimp = NO;

					%if %upcase(&ModelType)=FOREST %then %do;
						%if (&Impfit=Yes) %then %do;
							%let impdat = &mlib..&mdat._ift;
							%let saveimp = YES;
						%end;
						%let N_ImpVars = &NImpVars;
					%end;
				%end;

				%if ^&CVFlag and (&BJ_Est = No or &BJ_OneMore) %then %do;
					%if (&TestSpec=Yes) %then %do;
						data b2;
							set d d_test;
						run;
						%let ts2 = b2;
					%end;
					%else 
						%let ts2 = ;
				%end;

				%put TreeMethod=&TreeMethod;
				%put ModelType=&ModelType;

				* remove labels of predictors so tree displays sas names;
				%if %upcase(&TreeMethod)=TREE and ^&CVFlag and (&BJ_Est = No or &BJ_OneMore) %then %do;
					proc datasets nolist library=work;
						modify &ts1;
						label 
							%do i=1 %to &nixl; %sysfunc(varname(&_dsixl_,&i))= %end;  
							%do i=1 %to &nix; %sysfunc(varname(&_dsix_,&i))= %end; 
							%do i=1 %to &nipl; %sysfunc(varname(&_dsipl_,&i))= %end;  
							%do i=1 %to &nip; %sysfunc(varname(&_dsip_,&i))= %end; 
						;
					quit;
					%if %length(&ts2) %then %do;
						proc datasets nolist library=work;
							modify &ts2;
							label 
								%do i=1 %to &nixl; %sysfunc(varname(&_dsixl_,&i))= %end;  
								%do i=1 %to &nix; %sysfunc(varname(&_dsix_,&i))= %end; 
								%do i=1 %to &nipl; %sysfunc(varname(&_dsipl_,&i))= %end;  
								%do i=1 %to &nip; %sysfunc(varname(&_dsip_,&i))= %end; 
							;
						quit;
					%end;
				%end;
				
				%put ts1 = &ts1;
				%put ts2 = &ts2;
				%put savem = &savem;
				%put modat = &modat;
				%put saveimp = &saveimp;
				%put impdat = &impdat;
				
				/*data OutLib.ts1; set &ts1; run;*/
				/*data OutLib.ts2; set &ts2; run;*/

				%geneselect(
					train_dsname=&ts1,
					test_dsname=&ts2,
					where_clause=,
					inmodel_dsname=,
					method=&TreeMethod,
					trait=&DepVar,
					trait_type=&trait_type,
					quantitative_vars=_dataset_,
					qualitative_vars=_dataset_,
					n_quantitative_lists=0,
					n_qualitative_lists=0,
					n_vars=&N_ImpVars,
					n2way=,
					weight=&WeightVar,
					save_model=&savem,
					model_output=&modat,
					save_importance=&saveimp,
					importance=ts2i,							
					importance_var=,
					importance_out=,
					importance_outfit=&impdat,
					save_score=YES,
					score_out=ts2o,
					score_outfit=,
					convert_to_cvscore=YES,
					proc_options=&GeneSelectOptions,
					priors=&Priors
				);

				/*
				data OutLib.ts2o;
					set ts2o;
				run;
				*/

				/*
				%if &AsymmetricLossFitProp ne 0.5 %then %do;
					data ts1a;
						set ts2o;
						keep _Row_ &DepVar P_&DepVarTrunc30 &WeightVar &WeightVar0;
					run;
				%end;
				*/

				%if %upcase(&TreeMethod)=FOREST %then %do;
					%if (&Impfit=Yes) %then %do;
						%nObsVars(&impdat);
						%if &nObs %then %do;
							data &impdat;
								set &impdat;
								if _input1_ = "" then delete;
							run;
						%end;
					%end;
				%end;

				%if %upcase(&TreeMethod)=TREE and ^&CVFlag and (&BJ_Est = No or &BJ_OneMore) %then %do;

					%let mdat = %scan(&outdat,2,'.');
					%if %length(&mdat) > 28 %then %do;
						%let mdat = %substr(&mdat,1,28);
					%end;
					%let inmodel_dsname = &mdat._mod;
					%let model_output = &mdat._omd;
					%let nodestats_output = &mdat._nds;
					%let importance_output = &mdat._imp;
					%let rules_output = &mdat._rul;
					%let cmprules_output = &mdat._enr;
					%let sequence_output = &mdat._seq;
					%let score_train_output = &mdat._stn;
					/* %let partialdep_output = &mdat._ptd; */

					%if %length(&ValidInData) > 0 %then %do;
						%let score_valid_output = &mdat._svd;
					%end;
					%else %do;
						%let score_valid_output = ;
					%end;

					%if %length(&TestInData) > 0 %then %do;
						%let score_test_output = &mdat._stt;
					%end;
					%else %do;
						%let score_test_output = ;
					%end;

					proc datasets library=Outlib nolist;
						delete &nodestats_output
								&importance_output
								&rules_output
								&sequence_output
								&score_train_output
								&score_valid_output
								&score_test_output
								/* &partialdep_output */
						;
					quit;

					/* 
					%NObsVars(ts2i);
					%if &nObs and &nVars %then %do;
						%let _impid_ = %sysfunc(open(ts2i));
						%let namenum = %sysfunc(varnum(&_impid_, NAME)); 
						%let rc = %sysfunc(fetchobs(&_impid_, 1));
						%let var1st = %sysfunc(getvarc(&_impid_, &namenum));
						%let _impid_ = %sysfunc(close(&_impid_));
					%end;


					%put partialdep_varname=&var1st;
					%put partialdep_output=&partialdep_output;
					*/
					 %tree(
						  TrainPath=&WorkPath,
						  TestPath=&TestPath,
						  ValidPath=&ValidPath,
						  train_dsname=&ts1, 
						  test_dsname=&TestInData,
						  valid_dsname=&ValidInData,
						  outputpath=&OutPath,
						  where_clause=,
						  inmodel_dsname=&inmodel_dsname, 
						  trait=&DepVar,
						  trait_type=&trait_type,					 
						  freq=,
						  quantitative_vars=, 
						  qualitative_vars=, 
						  n_quantitative_lists=,
						  n_qualitative_lists=,
						  subtree=,
						  assess=,
						  model_output=&model_output,
						  nodestats_output=&nodestats_output,
						  importance_output=&importance_output,
						  similarity_output=,
						  rules_output=&rules_output,
						  cmprules_output=&cmprules_output,
						  summary_output=,
						  sequence_output=&sequence_output,
						  score_train_output=&score_train_output,
						  score_valid_output=&score_valid_output,
						  score_test_output=&score_test_output,
						  score_train_outfit=,
						  score_valid_outfit=,
						  score_test_outfit=,
						  partialdep_varname=/*&var1st*/,
						  partialdep_output=/*&partialdep_output*/,
						  proc_options=,
						  branch_nodes=,
						  priors=proportional
					);

					%put nleaves=&nleaves;

					%if &missing=DISTRIBUTE %then %do;

						%score_branches(TrainPath=&WorkPath,
										TestPath=&TestPath,
										ValidPath=&ValidPath,
										train_dsname=&ts1, 
										test_dsname=&TestInData,
										valid_dsname=&ValidInData,
										outputpath=&OutPath,
										inmodel_dsname=&inmodel_dsname, 
										trait=&DepVar,
										trait_type=&trait_type,					 
										nodestats_output=&nodestats_output,
										score_train_output=&score_train_output,
										score_valid_output=&score_valid_output,
										score_test_output=&score_test_output,
										score_train_outfit=,
										score_valid_outfit=,
										score_test_outfit=,
										parent_node=1
										);
					%end;

					* create data set for display using diagram() in jsl;
					data parents0;
						length parentnode 8 parenttext $ 64;
						parentnode = 0;
						parenttext = "All Samples";
					run;

					data parents1;
						set OutLib.&nodestats_output;
						length parentnode 8 parenttext $ 64;
						if missing(belowtext) then delete;
						parentnode = node;
						parenttext = belowtext;
						keep parentnode parenttext;
					run;

					data parents1;
						set parents0 parents1;
					run;

					data nodes1;
						set OutLib.&nodestats_output;
						if missing(parent) then parent = 0;
						if lengthn(trim(left(abovetext))) and substr(abovetext,1,1) not in (">","<") then abovetext = cat("= ",abovetext); 
						* remove extra spaces;
						abovetext = compbl(abovetext);
					run;
					
					/*
					data OutLib.nodes1;
						set nodes1;
					run;
					*/

					proc sql; 
						create table nodes2
							as select * 
							from nodes1 as l left join parents1 as r
							on l.parent = r.parentnode;
					quit;

					data nodes2;
						set nodes2;
						length ChildRule $ 128;
						ChildRule = cat(trim(left(parenttext))," ",trim(left(abovetext))); 
					run;

					data parents0;
						length parentnode 8 ParentRule $ 128;
						parentnode = 0;
						ParentRule = "All Samples";
					run;

					data parents1;
						set nodes2;
						if missing(belowtext) then delete;
						parentnode = node;
						ParentRule = ChildRule;
						keep parentnode ParentRule;
					run;	

					data parents2;
						set parents0 parents1;
					run;

					proc sql; 
						create table nodestats
							as select * 
							from nodes2 as l left join parents2 as r
							on l.parent = r.parentnode;
					quit;

					%if &trait_type = qualitative %then %do;
						%do c=1 %to &nclasses;
							%SASName(&&class&c);
							%if &under1 %then %let cname&c = %substr(&sasname,2);
							%else %let cname&c = &sasname;
							%put cname&c = &&cname&c;
						%end;
					%end;

					data OutLib.&nodestats_output;
						set nodestats;
						length NodeLabel $ 128 NodeLabel1 NodeLabel2 NodeLab2 $ 64 StatNames StatValues $ 512;
						NodeLabel = cat("N=",trim(left(N)));
						NodeLabel1 = cat("N = ",trim(left(N)));
						%if &trait_type = qualitative %then %do;
							StatNames = '"N"' %do c=1 %to &nclasses; || "," || '"' || "Prob(&DepVar.=&&class&c)" || '"' %end; ;
							StatValues = trim(left(N)) %do c=1 %to &nclasses; || "," || trim(left(put(P_&DepVar&&cname&c,4.2))) %end; ;
							/*
							%do c=2 %to &nclasses;
								NodeLabel = cat(trim(left(NodeLabel)),"                   Prob(&DepVar.=&&class&c)=",trim(left(put(P_&DepVar&&cname&c,4.2))));
								%if &c = 2 %then %do;
									NodeLabel2 = cat("Prob(&DepVar.=&&class&c) = ",trim(left(put(P_&DepVar&&cname&c,4.2))));
									NodeLab2 = "Prob(&DepVar.=&&class&c)";
									NodeVal2 = P_&DepVar&&cname&c;
									NodeFormat2 = "4,2";
								%end;
							%end;
							*/
						%end;
						%else %do;
							StatNames = '"N"' || "," || '"' || "Mean(&DepVar)" || '"';
							StatValues = trim(left(N)) || "," || trim(left(put(P_&DepVar,best6.)));
							/*
							NodeLabel = cat(trim(left(NodeLabel)),"                   Mean(&DepVar)=",trim(left(put(P_&DepVar,best6.))));
							NodeLabel2 = cat("Mean(&DepVar) = ",trim(left(put(P_&DepVar,best6.))));
							NodeLab2 = "Mean(&DepVar)";
							NodeVal2 = P_&DepVar;
							NodeFormat2 = '"best",6';
							*/
						%end;
						drop parentnode parenttext;
					run;

					* get min and max values for plotting;
					%if &trait_type = quantitative %then %do;
						proc means data=OutLib.&nodestats_output noprint;
							var P_&DepVar;
							output out=mm min=min max=max;
						run;

						data _null_;
							set mm;
							call symputx("tpmin",trim(left(min)));
							call symputx("tpmax",trim(left(max)));
						run;
					%end;

					* create variable importance list for value ordering;
					%NObsVars(OutLib.&importance_output);
					%if &nobs >= 100 %then %do;
						data OutLib.&importance_output;
							set OutLib.&importance_output(obs=100);
						run;
					%end;

					data _null_;
						set OutLib.&importance_output end=_last;
						call symputx("vip"||trim(left(_n_)),trim(left(name)));
						if _last then call symputx("nvip",trim(left(_n_)));
					run;

					%put nvip = &nvip;
					%let viplist = %str("")&&vip&nvip%str("");
					%do i = %eval(&nvip-1) %to 1 %by -1;
						%let viplist = &viplist , %str("")&&vip&i%str("");
					%end;	

				%end;

				%if &syserr > 4 %then %do;
					%let exiterror = 1;
					%goto exit;
				%end;
				%if ^&CVFlag %then %do;
					/* comment this out to prevent output listing to be created which 
						triggers populating results output tab
					title "Partition Tree Model";
					proc print data=&modat;
					run; */
					proc sort data=ts2o;
						by _Obs;
					run;
				%end;
				%if (&trait_type=quantitative) %then %do;
					data ts2o;
						set ts2o;
						drop R_&DepVar;
					run;
				%end;
				%else %do;

					* drop I_&DepVarTrunc30 to avoid type conflicts while merging results with other APs.;
					data ts2o;
						set ts2o;
						drop I_&DepVarTrunc30;
					run;

					%Rename_P_;

					* compute actualp for testspec;
					%if (&TestSpec=Yes) %then %do;
						data ts2o;
							set ts2o;
							array pp %do c=1 %to &nclasses; &&p_prob&c %end; ;
							_cidx = .;
							%do c=1 %to &nclasses;
								%if (&_dtype_=N) %then %do;
									if trim(left(&TruthVar)) = &&class&c then _cidx = &c; 
								%end;
								%else %do;
									if trim(left(&TruthVar)) = "&&class&c" then _cidx = &c; 						
								%end;
							%end;
							if not missing(_cidx) then actualp = pp[_cidx];
							else actualp = .;
						run;

						%if ^&CVFlag %then %do;
							data &ts1;
								set ts2o;
								where _train_ = 1;
							run;
							data &ts2;
								set ts2o;
								where _train_ = 0;
							run;
						%end;
					%end;
				%end;

				%let into = U_&DepVarTrunc30;
				%let pred = P_&DepVarTrunc30;

				data ts2vars;
					set ts2i;
					if importance > 0;
				run;
				%let havets2v = 1;
				%UpdateSelVars;

			%end;
			%else %if (&ProcessName = DiscriminantAnalysis) or (&ProcessName = KNearestNeighbors) %then %do;

				* analysis settings output string;
				%if (&ProcessName = DiscriminantAnalysis) %then %do;
					%if %length(&ASet) %then %do;
						%let ASet = &ASet, &trMetric = %LocaleEZX(%nrbquote(&Metric), EIA=1), &trVariableSelectionMethod = %LocaleEZX(%nrbquote(&VarSelect), EIA=1);
					%end;
					%else %do;
						%let ASet = &trMetric = %LocaleEZX(%nrbquote(&Metric), EIA=1), &trVariableSelectionMethod = %LocaleEZX(%nrbquote(&VarSelect), EIA=1);
					%end;
					%if (&VarSelect = Stepwise) %then %do;
						%let ASet = &Aset, &trSLEntry = &SLEntry, &trSLStay = &SLStay;
					%end;
				%end;
				%else %if (&ProcessName = KNearestNeighbors) %then %do;
					%let ASet = &trNumberofNearestNeighbors = &NNN, &trDistanceMetric = %LocaleEZX(%nrbquote(&Distance), EIA=1);
					%let ASet = &ASet, &trVariableSelectionMethod = %LocaleEZX(%nrbquote(&VarSelect), EIA=1);
				%end;

				* number of canonical components;
				%if %length(&NumVar) %then %do;
					data _null_;
						ncan = min(5,&NumVar);
						/*
						%if %index(&VarSelect,Genetic) %then %do;
							ncan = max(2,min(&NumVar,%eval(&nlev-1)));
						%end;
						%else %do;
							ncan = 2;
						%end;
						*/
						call symput("ncan",trim(left(ncan)));
					run;
				%end;
				%else %do;
					%let ncan = 2;
				%end;
				%put ncan = &ncan;

				/*
				data OutLib.ts1;
					set &ts1;
				run;
				*/

				proc discrim data=&ts1 
					out=ts1o(drop=
						%do i=1 %to &nixl; %sysfunc(varname(&_dsixl_,&i)) %end; 
						%do i=1 %to &nipl; %sysfunc(varname(&_dsipl_,&i)) %end; 
						%do i=1 %to &nix; %sysfunc(varname(&_dsix_,&i)) %end; 
						%do i=1 %to &nip; %sysfunc(varname(&_dsip_,&i)) %end; 
					) 
					testdata=&ts2 testout=ts2o(drop=
						%do i=1 %to &nixl; %sysfunc(varname(&_dsixl_,&i)) %end; 
						%do i=1 %to &nipl; %sysfunc(varname(&_dsipl_,&i)) %end; 
						%do i=1 %to &nix; %sysfunc(varname(&_dsix_,&i)) %end; 
						%do i=1 %to &nip; %sysfunc(varname(&_dsip_,&i)) %end; 
					)
					&DiscrimOptions crossvalidate canonical ncan=&ncan manova;
					class &DepVar;
					var
						%do i=1 %to &nixl; %sysfunc(varname(&_dsixl_,&i)) %end; 
						%do i=1 %to &nipl; %sysfunc(varname(&_dsipl_,&i)) %end; 
						%do i=1 %to &nix; %sysfunc(varname(&_dsix_,&i)) %end; 
						%do i=1 %to &nip; %sysfunc(varname(&_dsip_,&i)) %end; 
					;
					priors &Priors;
					ods output levels=_namedat;
				run;
				%if &syserr > 4 %then %do;
					%let exiterror = 1;
					%goto exit;
				%end;

				* create variables for computing cv ase;
				data _namedat;
					set _namedat end=_last;
					cn = "cvalue" || trim(left(_n_));
					call symputx(cn,trim(left(&DepVar)));
					if (_last) then call symput('ncl',trim(left(_n_)));
				run;
				/*
				proc print;
				run;
				*/

				/*
				get variable names straight from the data set in case they have changed
				from the original levels.  assume the post_prob variables are at the end 
				of the data set, except the very last one, which should be _INTO_
				*/

				%let _dsid_ = %sysfunc(open(ts2o));
				%let nvars = %sysfunc(attrn(&_dsid_,NVARS));
				%do c=1 %to &nclasses;
					%let cn&c = %sysfunc(varname(&_dsid_,%eval(&nvars - &nclasses - 1 + &c)));
					%put cn&c = &&cn&c;
				%end;
				%global cvartype;
				%let vnum = %sysfunc(varnum(&_dsid_,&TruthVar));
				%let cvartype = %sysfunc(vartype(&_dsid_,&vnum));
				%let _dsid_ = %sysfunc(close(&_dsid_));

				data ts2o;
					set %if (&TestSpec = Yes) %then %do; ts1o %end; ts2o;
					_cidx = .;
					%do c=1 %to &nclasses;
						&&p_prob&c = &&cn&c;
						%if (&cvartype=N) %then %do;
							if trim(left(&TruthVar)) = &&cvalue&c then _cidx = &c; 
						%end;
						%else %do;
							if trim(left(&TruthVar)) = "&&cvalue&c" then _cidx = &c; 						
						%end;
					%end;
					%if (&cvartype=N) %then %do;
						if missing(_INTO_) then _INTO_ = &cvalue1; 
					%end;
					%else %do;
						if missing(_INTO_) then _INTO_ = "&cvalue1"; 						
					%end;
				run;

				/*
				ods exclude none;
				ods results;
				proc contents data=ts2o;
				run;
				*/

				data ts2o;
					set ts2o;
					array _p p_:;
					if not missing(_cidx) then actualp = _p[_cidx];
					else actualp = .;
				run;

				%let into = _INTO_;

			%end;

			%else %if (&ProcessName = DistanceScoring) %then %do;

				* analysis settings output string;
				%if &Mahalanobis %then %do;
					%if &TraitType = Continuous %then %do;
						%put ERROR: Mahalanobis distance is currently not available with continuous dependent variables.;
						%let exiterror = 1;
						%goto exit;
					%end;
					%let dm = MAHALANOBIS;
				%end;
				%else %do;
					%let dm = &DistMetric;
				%end;
				%if &TraitType = Continuous %then %do;
					%let trComputingDistancestoCT = ;
				%end;
				%else %do;
					%let trComputingDistancestoCT = , %nrbquote(%LocaleFill(%nrbquote(&trComputingDistancestoX), %nrbquote(%LocaleEZX(%nrbquote(&CenTime), EIA=1))));
				%end;
				%if %length(&ASet) %then %do;
					%let ASet = &ASet, &trDistanceMetric = %LocaleEZX(%nrbquote(&dm), EIA=1)&trComputingDistancestoCT;
				%end;
				%else %do;
					%let ASet = &trDistanceMetric = %LocaleEZX(%nrbquote(&dm), EIA=1)&trComputingDistancestoCT;
				%end;
				%let ASet = &ASet, &trSummarizationMethod = %LocaleEZX(%nrbquote(&CenMethod), EIA=1), &trKernel = %LocaleEZX(%nrbquote(&PostFunction), EIA=1);
				%if (&CenTime = Each Observation) or (&TraitType = Continuous) %then %do;
					%let ASet = &ASet, &trNumberofNearestNeighbors = &NNN;
				%end;
				
				%let trainds = &ts1;

				* compute class centroids if necessary;
				%if ((&TraitType=Binary) or (&TraitType=Nominal)) and %index(&CenTime,Centroids) %then %do;

					* ignore K for this case;
					%let NNN = 0;
					/*
					* limit to 32767 vars for now, see DataSummary.sas for a possible way around this, or its likely much faster to transpose;
					%if ^%index(&VarSelect,Genetic) %then %do;
						%let numpred = %eval(&nix + &nip);
						%if (&numpred > 32767) %then %do;
							%put ERROR: The number of predictors &numpred exceeds the maximum of 32767 for Distance Scoring.;
							%put Pleae use Predictor Reduction or try a different model.;
							%let exiterror = 1;
							%goto exit;
						%end;
					%end;
					*/
					proc sort data=&ts1 out=ts1s;
						%if (&DepVar = &TruthVar) %then %do;
							by &DepVar;
						%end;
						%else %do;
							by &DepVar &TruthVar;
						%end;
						where not missing(&DepVar);
					run;
					/*
					data OutLib.ts1;
						set &ts1;
					run;
					data OutLib.ts1s;
						set ts1s;
					run;
					*/
					proc means data=ts1s noprint;
						%if (&DepVar = &TruthVar) %then %do;
							by &DepVar;
						%end;
						%else %do;
							by &DepVar &TruthVar;
						%end;
						var
							%if %index(&VarSelect,Genetic) %then %do;
								&SelVars
							%end;
							%else %do;
								%do i=1 %to &nixl; %sysfunc(varname(&_dsixl_,&i)) %end; 
								%do i=1 %to &nipl; %sysfunc(varname(&_dsipl_,&i)) %end; 
								%do i=1 %to &nix; %sysfunc(varname(&_dsix_,&i)) %end; 
								%do i=1 %to &nip; %sysfunc(varname(&_dsip_,&i)) %end; 
							%end;
						;
						output out=ts1m 
							&CenMethod.(
								%if %index(&VarSelect,Genetic) %then %do;
									&SelVars
								%end;
								%else %do;
									%do i=1 %to &nixl; %sysfunc(varname(&_dsixl_,&i)) %end; 
									%do i=1 %to &nipl; %sysfunc(varname(&_dsipl_,&i)) %end; 
									%do i=1 %to &nix; %sysfunc(varname(&_dsix_,&i)) %end; 
									%do i=1 %to &nip; %sysfunc(varname(&_dsip_,&i)) %end; 
								%end;
							) =
							%if %index(&VarSelect,Genetic) %then %do;
								&SelVars
							%end;
							%else %do;
								%do i=1 %to &nixl; %sysfunc(varname(&_dsixl_,&i)) %end; 
								%do i=1 %to &nipl; %sysfunc(varname(&_dsipl_,&i)) %end; 
								%do i=1 %to &nix; %sysfunc(varname(&_dsix_,&i)) %end; 
								%do i=1 %to &nip; %sysfunc(varname(&_dsip_,&i)) %end; 
							%end;						
						;
					run;
					/*
					data OutLib.ts1m;
						set ts1m;
					run;
					*/
					%NObsVars(ts1m);
					%let nm = &nobs;
					%let trainds = ts1m;
					data priors;
						set ts1m;
						%if (&Priors = Proportional) %then %do;
							prior = _freq_;
						%end;
						%else %do;
							prior = 1/&nm;
						%end;
						keep &DepVar prior;
					run;
				%end;

				%if ^&CVFlag and (&TestSpec = Yes) %then %do;
										
					data b2;
						set d d_test;
					run;
					%let ts2 = b2;
				%end;
				
				%put ts2 = &ts2;
				%put trainds = &trainds;

				data one;
					one = 1;
				run;

				data b1;
					set &ts2 &trainds;
				run;

				/*
				title "b1";
				proc print data=b1(obs=10);
					var &DepVar &TruthVar
						%if %index(&VarSelect,Genetic) %then %do;
							&SelVars
						%end;
						%else %do;
							%do i=1 %to &nix; %sysfunc(varname(&_dsix_,&i)) %end;
							%do i=1 %to &nip; %sysfunc(varname(&_dsip_,&i)) %end;
						%end;
					;
				run;
				*/

				%NObsVars(&ts2);
				%let ntest = &nobs;
				%put ntest = &ntest;

				%if (&ntest > 0) %then %do;

					* for Mahalanobis, transform rows using inverse root of covariance matrix computed via SVD;
					%*put Mahalanobis= &Mahalanobis;
					%if &Mahalanobis %then %do;
						%*put TraitType= &TraitType;
						%*put CenTime= &CenTime;
						%if ((&TraitType=Binary) or (&TraitType=Nominal)) and %index(&CenTime,Centroids) %then %do;

							proc stdize data=ts1s method=&CenMethod out=ts1std(keep=
									%if %index(&VarSelect,Genetic) %then %do;
										&SelVars
									%end;
									%else %do;
										%do i=1 %to &nixl; %sysfunc(varname(&_dsixl_,&i)) %end; 
										%do i=1 %to &nipl; %sysfunc(varname(&_dsipl_,&i)) %end; 
										%do i=1 %to &nix; %sysfunc(varname(&_dsix_,&i)) %end; 
										%do i=1 %to &nip; %sysfunc(varname(&_dsip_,&i)) %end; 
									%end;						
								);
								by &DepVar;
								var 
									%if %index(&VarSelect,Genetic) %then %do;
										&SelVars
									%end;
									%else %do;
										%do i=1 %to &nixl; %sysfunc(varname(&_dsixl_,&i)) %end; 
										%do i=1 %to &nipl; %sysfunc(varname(&_dsipl_,&i)) %end; 
										%do i=1 %to &nix; %sysfunc(varname(&_dsix_,&i)) %end; 
										%do i=1 %to &nip; %sysfunc(varname(&_dsip_,&i)) %end; 
									%end;						
								;							
							run;

							/*
							data OutLib.ts1std;
								set ts1std;
							run;
							*/

							%NObsVars(ts1std);

							%if &nobs < &nvars %then %let Transpose = 1;
							%else %let Transpose = 0;
							%let nn = &nobs;

							%if &Transpose %then %do;
								proc transpose data=ts1std out=ts1std;
								run;
								%let invars = col1-col&nobs;
								%let idvar = _NAME_;
							%end;
							%else %do;
								data ts1std;
									set ts1std;
									_row_ = _n_;
								run;
								%if %index(&VarSelect,Genetic) %then %do;
									%let invars = &SelVars;
								%end;
								%else %do;
									%let invars = ;
									%do i=1 %to &nixl; %let invars = &invars %sysfunc(varname(&_dsixl_,&i)); %end; 
									%do i=1 %to &nipl; %let invars = &invars %sysfunc(varname(&_dsipl_,&i)); %end; 
									%do i=1 %to &nix; %let invars = &invars %sysfunc(varname(&_dsix_,&i)); %end; 
									%do i=1 %to &nip; %let invars = &invars %sysfunc(varname(&_dsip_,&i)); %end; 
								%end;						
								%let idvar = _row_;
							%end;
							%*put invars= &invars;

							%svd(work,ts1std,out_v,out_s,out_u,&invars,&idvar);
							
							/*
							data OutLib.out_v;
								set out_v;
							run;
							data OutLib.out_s;
								set out_s;
							run;
							data OutLib.out_u;
								set out_u;
							run;
							*/

							%NObsVars(out_s);
							%let nsv = &nobs;
							%put nsv = &nsv;

							proc iml;
								use out_v;
								read all var{%do i=1 %to &nsv; prin&i %end;} into v;
								use out_s;
								read all var{singularvalue} into d;
								use out_u;
								read all var{%do i=1 %to &nsv; prin&i %end;} into u;
								* print v d u;
								use b1;
								read all var{
									%if %index(&VarSelect,Genetic) %then %do;
										&SelVars
									%end;
									%else %do;
										%do i=1 %to &nixl; %sysfunc(varname(&_dsixl_,&i)) %end; 
										%do i=1 %to &nipl; %sysfunc(varname(&_dsipl_,&i)) %end; 
										%do i=1 %to &nix; %sysfunc(varname(&_dsix_,&i)) %end; 
										%do i=1 %to &nip; %sysfunc(varname(&_dsip_,&i)) %end; 
									%end;						
									} into b;
								%if &Transpose %then %do;
									btrans = sqrt(&nn-&nm)*b*u*inv(diag(d))*t(v);
								%end;
								%else %do;
									btrans = sqrt(&nn-&nm)*b*v*inv(diag(d))*t(u);
								%end;
								create b1tr from btrans;
								append from btrans;
							quit;	

							/*
							data OutLib.b1tr;
								set b1tr;
							run;
							*/

							/*
							proc iml;
								use ts1std;
								read all var{
									%if %index(&VarSelect,Genetic) %then %do;
										&SelVars
									%end;
									%else %do;
										%do i=1 %to &nixl; %sysfunc(varname(&_dsixl_,&i)) %end; 
										%do i=1 %to &nipl; %sysfunc(varname(&_dsipl_,&i)) %end; 
										%do i=1 %to &nix; %sysfunc(varname(&_dsix_,&i)) %end; 
										%do i=1 %to &nip; %sysfunc(varname(&_dsip_,&i)) %end; 
									%end;						
									} into x;
								* print x;
								n = nrow(x);
								c = &nm;
								xt = t(x)/sqrt(n-c);
								print xt;
								call svd(v,d,u,xt);
								print v d u;
								use b1;
								read all var{
									%if %index(&VarSelect,Genetic) %then %do;
										&SelVars
									%end;
									%else %do;
										%do i=1 %to &nixl; %sysfunc(varname(&_dsixl_,&i)) %end; 
										%do i=1 %to &nipl; %sysfunc(varname(&_dsipl_,&i)) %end; 
										%do i=1 %to &nix; %sysfunc(varname(&_dsix_,&i)) %end; 
										%do i=1 %to &nip; %sysfunc(varname(&_dsip_,&i)) %end; 
									%end;						
									} into b;
								btrans = b*v*inv(diag(d))*t(u);
								create b1tr from btrans;
								append from btrans;
							quit;	
							*/

							data b2;
								set b1;
								drop
									%if %index(&VarSelect,Genetic) %then %do;
										&SelVars
									%end;
									%else %do;
										%do i=1 %to &nixl; %sysfunc(varname(&_dsixl_,&i)) %end; 
										%do i=1 %to &nipl; %sysfunc(varname(&_dsipl_,&i)) %end; 
										%do i=1 %to &nix; %sysfunc(varname(&_dsix_,&i)) %end;
										%do i=1 %to &nip; %sysfunc(varname(&_dsip_,&i)) %end;
									%end;
								;
							run;
							data b1;
								merge b2 b1tr;
							run;
							%NObsVars(b1tr);
							%let nv = &nvars;
							%put nv = &nv;
						%end;
					%end;

					%put VarLevel = &VarLevel;
					
					/*data OutLib.b1; set b1; run;*/

					* compute distances;
					%NObsVars(b1);
					
					%if (&DistMetric = EUCLID) and (&TraitType ne Continuous) and (&NNN = 0) and 
						%index(&CenTime,Centroids) and (&nobs > &nvars) %then %do;
					
						***S1425306: remove rows with missing predictors***;
						data b1_nomisspred;
							set b1;
							%if %length(%trim(&LockContPredVars &LockClassPredVars &PredictorVars &ClassPredVars)) %then %do;
								array pv{*} &LockContPredVars &LockClassPredVars %if ^&upred %then %do; &PredictorVars %end; %else %do; %do j=1 %to &nix_blocks; &&PredictorVars&j %end; %end; &ClassPredVars;
								do _i_=1 to dim(pv);
									if missing(pv[_i_]) then do;
										delete;
										leave;
									end;
								end;
							%end;
							;
						run;
						
						/*data OutLib.b1_nomisspred; set b1_nomisspred; run;*/
						
						%NobsVars(b1_nomisspred);						
						%if (&nobs > &nvars) %then %do;
							proc datasets nolist;
								delete b1;
								change b1_nomisspred= b1;
							quit;
							
							%let b1_nobs= &nobs;
							%let b1_nvars= &nvars;
						
							*recalculating ntest*;
							data ts2_nomisspred;
								set &ts2;
								%if %length(%trim(&LockContPredVars &LockClassPredVars &PredictorVars &ClassPredVars)) %then %do;
									array pv{*} &LockContPredVars &LockClassPredVars %if ^&upred %then %do; &PredictorVars %end; %else %do; %do j=1 %to &nix_blocks; &&PredictorVars&j %end; %end; &ClassPredVars;
									do _i_=1 to dim(pv);
										if missing(pv[_i_]) then do;
											delete;
											leave;
										end;
									end;
								%end;
								;
							run;
							
							%NObsVars(ts2_nomisspred);
							%let ntest = &nobs;
							%put ntest = &ntest;
							
							%let nobs= &b1_nobs;
							%let nvars= &b1_nvars;							
															
						%end;
   					%end;
					
					* special case for speed for tall data sets with DLDA;
					%if (&DistMetric = EUCLID) and (&TraitType ne Continuous) and (&NNN = 0) and 
						%index(&CenTime,Centroids) and (&nobs > &nvars) %then %do;
					
						%let nc = %eval(&nobs - &ntest);
						%put nc = &nc;
						%let ndist = &nobs;
						%put ndist = &ndist;
					
						* compute transposed distrec directly;
						proc iml;
							use b1;
							read all var{
								%if %index(&VarSelect,Genetic) %then %do;
									&SelVars
								%end;
								%else %do;
									%do i=1 %to &nixl; %sysfunc(varname(&_dsixl_,&i)) %end; 
									%do i=1 %to &nipl; %sysfunc(varname(&_dsipl_,&i)) %end; 
									%do i=1 %to &nix; %sysfunc(varname(&_dsix_,&i)) %end; 
									%do i=1 %to &nip; %sysfunc(varname(&_dsip_,&i)) %end; 
								%end;						
								} into b;
							centroids = b[%eval(&ntest+1):&ndist,];
							drt = j(&ndist,&nc);
							one = j(&ndist,1,1);
							do j=1 to &nc;
								diff = b - one * centroids[j,];
								do i=1 to &ndist;
									drt[i,j] = sqrt(diff[i,]*t(diff[i,]));
								end;
							end;
							create drt from drt;
							append from drt;
						quit;	
					
						%let gotdistrec = 0;
						
					%end;
					%else %do;
					
						proc distance data=b1 method=&DistMetric out=dist
							prefix=dist shape=SQR &ProcDistanceOptions ;
							var &VarLevel.(
								%if &Mahalanobis and (((&TraitType=Binary) or (&TraitType=Nominal)) and %index(&CenTime,Centroids)) %then %do;
									col1-col&nv
								%end;
								%else %if %index(&VarSelect,Genetic) %then %do;
									&SelVars
								%end;
								%else %do;
									%do i=1 %to &nixl; %sysfunc(varname(&_dsixl_,&i)) %end; 
									%do i=1 %to &nipl; %sysfunc(varname(&_dsipl_,&i)) %end; 
									%do i=1 %to &nix; %sysfunc(varname(&_dsix_,&i)) %end;
									%do i=1 %to &nip; %sysfunc(varname(&_dsip_,&i)) %end;
								%end;
							);
							%if (&DepVar = &TruthVar) %then %do;
								copy &DepVar;
							%end;
							%else %do;
								copy &DepVar &TruthVar;
							%end;
						run;

						%if &syserr>4 %then %do;
							%let exiterror = 1;
							%goto exit;
						%end;

						data d1;
							set dist;
							if (0);
							keep dist:;
						run;
						%NObsVars(d1);
						%let ndist = &NVars;
						%put ndist = &ndist;

						* extract rectangle of distances between training and test+training;
						data distrec;
							set dist;
							if _n_ > &ntest and not missing(&DepVar);
							keep &DepVar &TruthVar dist1-dist&ndist;
						run;
						
						%let gotdistrec = 1;
						
						/*
						data OutLib.distrec;
							set distrec;
						run;
						*/

						* replace zeros with missings to remove self-selfs so they are not picked up in stat;
						* perhaps this should not be done when zero distances are legit among different obs;
						/*
						data distrec;
							set distrec;
							array dist{*} dist1-dist&ndist;
							do i=1 to &ndist;
								if dist[i] < 1e-6 then dist[i] = .;
							end;
							drop i;
						run;
						*/
					%end;

					* k-nearest neighbors, keep k smallest distances, set others to missing or zero;
					%if (&NNN > 0) %then %do;

						/*
						data distrec;
							set distrec;
							if not missing(&DepVar);
						run;
						*/
						
						%NObsVars(distrec);
						%let ntrain = &nobs;
						%put ntrain = &ntrain;

						%if (&ntrain <= &NNN) %then %do;
							%put The number of training observations (&ntrain) is less than specified number of neighbors (&NNN).  All points are used.;
						%end;
						%else %do;
							%put Using the smallest &NNN out of &ntrain distances.;

							title "Distances Before &NNN Nearest Neighbors";
							title2 "First 10 Observations";
							proc print data=distrec(obs=10);
							run;
							title;
							title2;
							
							data dr1;
								set distrec;
								keep &DepVar &TruthVar;
							run;

							proc transpose data=distrec out=drt prefix=d;
								var dist1-dist&ndist;
							run;

							data drt;
								set drt;
								array d{*} d1-d&ntrain;
								do k=1 to &NNN;
									minidx = 0;
									do i=1 to &ntrain;
										if not missing(d[i]) and (d[i] > 0) then do;
											minidx = i;
											i = &ntrain + 1;
										end;
									end;
									if (minidx > 0) then do;
										do i=(minidx+1) to &ntrain;
											if not missing(d[i]) and (d[i] > 0) and (d[i] < d[minidx]) then minidx = i;
										end;
										* use temporary sign reversal to flag keepers, assumes all distances are nonnegative;
										d[minidx] = -d[minidx];
									end;
								end;
								do i=1 to &ntrain;
									if not missing(d[i]) then do;
										if (d[i] < 0) then d[i] = -d[i];
										* if (d[i] < 0) then d[i] = 1;
										else do;
											%if (&Priors = Proportional) %then %do;
												d[i] = .;
											%end;
											%else %do;
												d[i] = 0;
											%end;
										end;
									end;
								end;
							run;

							proc transpose data=drt out=drtt;
								var d1-d&ntrain;
							run;

							data distrec;
								merge dr1 drtt;
							run;

							title "Distances After &NNN Nearest Neighbors";
							title2 "First 10 Observations";
							proc print data=distrec(obs=10);
							run;
							title;
							title2;
							
							/*
							data OutLib.distrec_after;
								set distrec;
							run;
							*/
						%end;
					%end;

					* KNN summaries;
					%if ((&TraitType=Binary) or (&TraitType=Nominal)) and ^%index(&CenTime,Centroids) %then %do;
						proc sort data=distrec;
							%if (&DepVar = &TruthVar) %then %do;
								by &DepVar;
							%end;
							%else %do;
								by &DepVar &TruthVar;
							%end;
							where not missing(&DepVar);
						run;
						%if (&Priors = Proportional) %then %do;
							%let mmethod = sum;
						%end;
						%else %do;
							%let mmethod = mean;
						%end;
						proc means data=distrec noprint;
							%if (&DepVar = &TruthVar) %then %do;
								by &DepVar;
							%end;
							%else %do;
								by &DepVar &TruthVar;
							%end;
							var dist1-dist&ndist;
							output out=drm &mmethod.(%do i=1 %to &ndist; dist&i %end;)=%do i=1 %to &ndist; dist&i %end; ;
						run;
						/*
						data OutLib.drm;
							set drm;
						run;
						*/
						data distrec;
							set drm;
						run;
						%let dvlist = dist1-dist&ndist;
						%if (&Priors = Proportional) %then %do;
							data distrec;
								set distrec;
								array d{*} dist1-dist&ndist;
								do i=1 to dim(d);
									if missing(d[i]) then d[i] = 0;
								end;
							run;
						%end;
						%NObsVars(drm);
						data priors;
							set drm;
							%if (&Priors = Proportional) %then %do;
								prior = _freq_;
							%end;
							%else %do;
								prior = 1/&nobs;
							%end;
							keep &DepVar prior;
						run;
					%end;

					* initialize output data set;
					data ts2o;
						set &ts2;
						drop 
							%do i=1 %to &nixl; %sysfunc(varname(&_dsixl_,&i)) %end; 
							%do i=1 %to &nipl; %sysfunc(varname(&_dsipl_,&i)) %end; 
							%do i=1 %to &nix; %sysfunc(varname(&_dsix_,&i)) %end; 
							%do i=1 %to &nip; %sysfunc(varname(&_dsip_,&i)) %end; 
						;
					run;

					/*
					data ts2os;
						set &ts2;
						keep
							%if %index(&VarSelect,Genetic) %then %do;
								&SelVars
							%end;
							%else %do;
								%do i=1 %to &nixl; %sysfunc(varname(&_dsixl_,&i)) %end; 
								%do i=1 %to &nipl; %sysfunc(varname(&_dsipl_,&i)) %end; 
								%do i=1 %to &nix; %sysfunc(varname(&_dsix_,&i)) %end;
								%do i=1 %to &nip; %sysfunc(varname(&_dsip_,&i)) %end;
							%end;
						;
					run;

					data ts2o;
						merge ts2o ts2os;
					run;
					*/

					* for continuous traits, compute averages weighted by inverse distances;
					%if (&TraitType=Continuous) %then %do;

						%NObsVars(distrec);
						%let ntrain = &nobs;
						
						proc iml;
							use distrec;
							read all var {%do j = 1 %to &ndist; dist&j %end;} into distrec;
							read all var {&DepVar} into y;
							
							* function to compute predictions and evaluate root mean square error;
							start rmsef(x) global(y,distrec,pred);
								pred = j(&ndist,1,0);
								* print x;
								do i=1 to &ndist;
									nomissidx = loc(distrec[,i] ^= . & distrec[,i] ^= 0);
									if nrow(nomissidx) = 0 then nomissidx = (1:nrow(distrec));
									di = distrec[nomissidx,i];
									%if &PostFunction = Gaussian %then %do;
										wi = exp(-0.5*di#di/x[1]/x[1]);
									%end;
									%else %if &PostFunction = Exponential %then %do;
										wi = exp(-di/x[1]);
									%end;
									%else %if &PostFunction = Cauchy %then %do;
										wi = 1/(1+di#di/x[1]/x[1]);
									%end;
									%else %if &PostFunction = Reciprocal %then %do;
										wi = 1/(1+di/x[1]);
									%end;
									%else %if &PostFunction = Uniform %then %do;
										wi = j(nrow(di),1,1);
										do j=1 to nrow(di); wi[j] = 1; end;
									%end;
									sumw = wi[+];
									* print i sumw;
									if sumw > 1e-6 then do;
										pred[i] = (wi#y[nomissidx])[+]/sumw;
									end;
									else do;
										pred[i] = mean(y[nomissidx]);
									end;
								end;
								resid = y - pred[(nrow(pred)-&ntrain+1):nrow(pred)];
								* print y pred resid;
								rmse = sqrt(resid`*resid/nrow(resid));
								return(rmse);
							finish rmsef;							
							
							* xopt = {1};
							* by default set the scale to the median of all distances;
							* xopt = mean(colvec(distrec));
							xopt = max(colvec(distrec))/2;
							
							* find optimial scaling by minimizing rmse;
							%if &PostFunction ^= Uniform %then %do;
								x0 = xopt;
								optn = {0 5};
								blc = {0.000001,1000000};
								CALL NLPDD(rc, xopt, "rmsef", x0, optn, blc);
							%end;
							
							rmse1 = rmsef(xopt);
							pred1 = pred[1:&ntest];
							create wm_all from pred1;
							append from pred1;
						quit;

						/*
						* compute inverse distances, which are used as weights below;
						data distrec;
							set distrec;
							array dist{*} dist1-dist&ndist;
							array idist{*} idist1-idist&ndist;
							do t=1 to &ndist;
								if missing(dist[t]) then idist[t] = .;
								else do; 
									%if (&PostFunction = Gaussian) %then %do;
										idist[t] = exp(-0.5*dist[t]*dist[t]);
									%end;
									%else %do;
										if dist[t] = 0 then do;
											idist[t] = 100;
										end;
										else do;
											idist[t] = 1/dist[t];
										end;
									%end;
								end;
							end;
							drop t;
						run;

						%if (&PostFunction = Gaussian) %then %do;
							title "Gaussian Distances";
						%end;
						%else %do;
							title "Reciprocal Distances";
						%end;
						title2 "First 10 Observations";
						proc print data=distrec;
						run;
						title;
						title2;

						%do t=1 %to &ndist;
							%put computing prediction as mean using inverse distance weight &t of &ndist;
							proc means data=distrec &CenMethod noprint;
								var &DepVar;
								weight idist&t;
								output out=wm(keep=pred) &CenMethod.(&DepVar)=pred;
							run;
							%if (&t=1) %then %do;
								data wm_all;
									set wm;
								run;
							%end;
							%else %do;
								proc append base=wm_all data=wm;
								run;
							%end;
						%end;
						*/
						
						data ts2o;
							merge ts2o wm_all(rename=(col1=pred));
						run;
					%end;
					%else %do;

						* merge prior probabilities;
						%if (&Priors = Proportional) %then %do;
							proc stdize data=priors method=sum out=priors;
								var prior;
							run;
							title "Distance Scoring Prior Probabilities";
							proc print data=priors;
							run;
							title;
							%if &gotdistrec %then %do;
								data distrec;
									merge distrec priors;
									by &DepVar;
								run;
							%end;
						%end;

						* compute posterior probabilities using Gaussian kernel as in PROC DISCRIM;
						* weight by prior probabilities;
						%if %index(&CenTime,Centroids) %then %do;
							%if &gotdistrec %then %do;
								%NObsVars(distrec);
								%let nc = &nobs;
							%end;
							%if (&Priors = Equal) %then %do;
								%do i=1 %to &nc;
									%let pratio&i = 1;
									%* put pratio&i = &&pratio&i;
								%end;
							%end;
							%else %do;
								proc transpose data=priors out=pr;
									var prior;
								run;
								data pr;
									set pr;
									%do i=1 %to &nc;
										call symputx("pratio&i",col&i/col1);
									%end;
								run;
							%end;

							%if &gotdistrec %then %do;
								proc transpose data=distrec out=drt;
									var dist1-dist&ndist;
								run;
							%end;

							data drt;
								set drt;
								array cca{*} col1-col&nc;
								array ppa{*} ppr1-ppr&nc;
								array gra{*} gratio1-gratio&nc;
								grsum = 0;
								if col1 > 1e-6 then sf1 = col1;
								else sf1 = 1;
								%do i=1 %to &nc;
									%if (&PostFunction = Gaussian) %then %do;
										gratio&i = &&pratio&i * exp(-0.5*(col&i*col&i/sf1/sf1));
									%end;
									%else %if (&PostFunction = Exponential) %then %do;
										gratio&i = &&pratio&i * exp(-(col&i/sf1));
									%end;
									%else %if (&PostFunction = Cauchy) %then %do;
										gratio&i = &&pratio&i / (1+col&i*col&i/sf1/sf1);
									%end;
									%else %if (&PostFunction = Reciprocal) %then %do;
										gratio&i = &&pratio&i / (1+col&i/col1);
									%end;
									%else %if (&PostFunction = Uniform) %then %do;
										if missing(col&i) or col&i = 0 then uc = 0;
										else uc = 1;
										gratio&i = &&pratio&i * uc;
									%end;
									grsum = grsum + gratio&i;
								%end;
								* handle overflows;
								if missing(grsum) then do;
									mini = 0;
									mind = 1e100;
									do i=1 to &nc;
										if not missing(cca[i]) and cca[i] < mind then do;
											mini = i;
											mind = cca[i];
										end;
									end;
									do i=1 to &nc;
										if i = mini then ppa[i] = 1;
										else ppa[i] = 0;
									end;									
								end;
								else do;
									do i=1 to &nc;
										ppa[i] = gra[i]/grsum;
									end;
								end;
							run;
							
							/*
							data OutLib.drt;
								set drt;
							run;
							*/

							proc transpose data=drt prefix=gdist out=drtt(drop=_name_);
								var ppr1-ppr&nc;
							run;

							%if &gotdistrec %then %do;
								data distrec;
									merge distrec drtt;
								run;
							%end;
							%else %do;
								data distrec;
									merge priors drtt;
								run;
							%end;

							/*
							* use min distance as a scale factor to help avoid overflows;
							data _null_;
								set distrec(obs=1);
								md = min(of dist1-dist&ndist);
								call symputx("d1sq",md*md);
							run;
							%put d1sq = &d1sq;
							data distrec;
								set distrec;
								array dist{*} dist1-dist&ndist;
								array gdist{*} gdist1-gdist&ndist;
								do t=1 to &ndist;
									if missing(dist[t]) then dist[t] = 1;
									%if (&Priors = Equal) %then %do;
										gdist[t] = exp(-0.5*(dist[t]*dist[t]-&d1sq));
									%end;
									%else %do;
										gdist[t] = prior*exp(-0.5*(dist[t]*dist[t]-&d1sq));
									%end;
									* if gdist[t] < 1e-8 then gdist[t] = 1e-8;
								end;
								drop t;
							run;
							*/

							/*
							data OutLib.distrec;
								set distrec;
							run;
							*/

							%let dvlist = gdist1-gdist&ndist;
						%end;

						* divide posterior probabilities by their sum so they sum to one;
						proc stdize data=distrec method=sum out=pp;
							var &dvlist;
						run;

						%put syserr = &syserr;
						%if &syserr >= 4 %then %do;
							%put ERROR: Unable to standardize posterior probabilities.  The computed distances may be too large.;
							%put;
							%put If so, it should help to standardize or rescale the predictor variables or use a different kernel.;
							%let exiterror = 1;
							%goto exit;
						%end;
						
						/*data OutLib.ds_pp1; set pp; run;*/

						* make sure values will result in valid sas names;
						data pp;
							set pp end=_last;
							length _depid $ 24;
							retain maxlendep 0;
							_depid = trim(left(&DepVar));
							lendep = min(length(_depid),24);
							maxlendep = max(maxlendep,lendep);
							do i=1 to lendep;
								_check2 = verify(upcase(substr(_depid,i,1)),"_ABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789");
								if _check2 then do;
									substr(_depid,i,1) = "_";
								end;
							end;
							if _last then call symputx("maxlendep",trim(left(maxlendep)));
						run;

						/*data OutLib.ds_pp2; set pp; run;*/

						/*
						ods exclude none;
						ods results;
						title "pp";
						proc print data=pp;
							var _depid lendep _check2;
						run;
						title;
						*/
						%put maxlendep = &maxlendep;
						%let lenleft = %eval(32-&maxlendep-3);
						%if %length(&cprefix) > &lenleft %then %let cpref = %substr(&cprefix,1,&lenleft);
						%else %let cpref = &cprefix;

						* transpose;
						proc transpose data=pp prefix=p_&cpref._ out=ppt(drop=_name_);
							var &dvlist;
							id _depid;
						run;
						
						/*data OutLib.ds_ppt; set ppt; run;*/

						/*
						title "Distance Scoring Posterior Probabilities";
						proc print data=ppt;
						run;
						title;
						*/
						
						/*data OutLib.ds_ts2o1; set ts2o; run;*/

						* levels of depvar;
						data ts2o;
							set ts2o;
							%if &_dtype_ = C %then %do;
								%do i=1 %to &nic1; _lev&i = "&&_lev&i"; %end;
							%end;
							%else %do;
								%do i=1 %to &nic1; _lev&i = &&_lev&i; %end;
							%end;
						run;
						
						/*data OutLib.ds_ts2o2; set ts2o; run;*/

						/*
						proc print data=ts2o;
						run;
						*/

						%put DS nic1 = &nic1;

						* get names of posterior probabilities;
						data ppn;
							set ppt(obs=1);
							keep p_&cpref._:;
						run;
						proc transpose data=ppn out=ppnt;
							var p_&cpref._:;
						run;
						proc sort data=ppnt;
							by _name_;
						run;
						
						/*data OutLib.ds_ppnt; set ppnt; run;*/
						
						data _null_;
							set ppnt;
							call symputx("pp"||trim(left(_n_)),trim(left(_name_)));
						run;

						%put cpref = &cpref;
						%do i=1 %to &nic1; 
							%put pp&i = &&pp&i;
						%end; 

						* merge and compute predicted classes;
						data ts2o;
							merge ts2o ppt;
							%if &CVFlag %then %do;
								if missing(&DepVar) then delete;
							%end;
							array pp %do i=1 %to &nic1; &&pp&i %end; ;
							array lev _lev1-_lev&nic1;
							maxp = 0;
							maxpi = 0;
							levi = .;
							do i=1 to dim(pp);
								if (pp[i] > maxp) then do;
									maxp = pp[i];
									maxpi = i;
								end;
								if (lev[i] = &TruthVar) then levi = i;
							end;
							I_&DepVarTrunc30 = lev[maxpi];
							if not missing(levi) then actualp = pp[levi];
							else actualp = .;
							drop _lev: i maxp maxpi levi;
						run;
						
						/*data OutLib.ds_ts2o3; set ts2o; run;*/

						%Rename_P_;

					%end;
				%end;

				%let into = I_&DepVarTrunc30;
				%let pred = pred;

			%end;
			%else %if (&ProcessName = GLMSelect) %then %do;

				* analysis settings output string;
				%if &aloss_niter = 1 %then %do;
					%if %length(&ASet) %then %do;
						%let ASet = &Aset, &trModelSelectionMethod = %LocaleEZX(%nrbquote(&SelectMethod), EIA=1);
					%end;
					%else %do;
						%let ASet = &trModelSelectionMethod = %LocaleEZX(%nrbquote(&SelectMethod), EIA=1);
					%end;
					%if %length(%trim(&StopCriterion)) %then %do;
						%let ASet = &ASet, &trStopCriterion = %LocaleEZX(%nrbquote(&StopCriterion), EIA=1);
					%end;
					%if (&StopCriterion = SL) %then %do;
						%let ASet = &ASet, &trSLEntry = &SLEntry, &trSLStay = &SLStay;
					%end;
					%if (&VarSelect = Genetic Algorithm) %then %do;
						%let ASet = &ASet, %LocaleEZX(%nrbquote(&VarSelect), EIA=1);
					%end;
				%end;

				/*
				data OutLib.ts1o;
					set &ts1;
				run;
				*/
				%let lockin = %eval(&nixl+&nipl);
				%if &lockin or (&StopCriterion=ADJRSQ) or (&StopCriterion=CV) or (&StopCriterion=CP) or (&StopCriterion=PRESS) %then %do;
					%let whichproc = glmselect;
				%end;
				%else %do;
					%let whichproc = hpreg;
				%end;
				* hpreg not working well in all cases so default back to glmselect;
				%let whichproc = glmselect;

				/*
				%let whichproc = hpgenselect;
				%let GLMSelectModelOptions = &GLMSelectModelOptions distribution=normal link=logit;
				*/
				/*
				%let GLMSelectModelOptions = &GLMSelectModelOptions distribuion=multinomial link=cumlogit;
				*/
				
				/*data OutLib.glm_&ts1; set &ts1; run;*/

				proc &whichproc data=&ts1 
					%if %length(&Seed) %then %do;
						seed=&Seed
					%end;
					&GLMSelectOptions namelen=128
					%if &whichproc = glmselect and ^&CVFlag and &SelectMethod ^= None %then %do;
						plots=all
					%end;
					;
				
					%if &nipl or &nip %then %do;
						class 
							%do i=1 %to &nipl; %sysfunc(varname(&_dsipl_,&i)) %end; 
							%do i=1 %to &nip; %sysfunc(varname(&_dsip_,&i)) %end; 
							/ missing
						;
					%end;

					/*
					effect spl=spline(
						%if %index(&VarSelect,Genetic) %then %do;
							&SelVars
						%end;
						%else %do;
							%do i=1 %to &nixl; %sysfunc(varname(&_dsixl_,&i)) %end; 
							%do i=1 %to &nipl; %sysfunc(varname(&_dsipl_,&i)) %end; 
							%do i=1 %to &nix; %sysfunc(varname(&_dsix_,&i)) %end; 
							%do i=1 %to &nip; %sysfunc(varname(&_dsip_,&i)) %end; 					
						%end;
						/ separate split degree=3
					);
					*/

					model &DepVar = 
						%if %index(&VarSelect,Genetic) %then %do;
							&SelVars
						%end;
						%else %do;
							/*
							spl
							*/

							%do i=1 %to &nixl; 
								%sysfunc(varname(&_dsixl_,&i)) 
								%if &OrderInt > 1 %then %do;
									%do j = 2 %to &OrderInt; | %sysfunc(varname(&_dsixl_,&i)) %end;
									%if ((&i < &nixl) or (&nipl > 0) or (&nix > 0) or (&nip > 0)) %then %do; | %end;
								%end;
							%end; 
							%do i=1 %to &nipl; 
								%sysfunc(varname(&_dsipl_,&i)) 
								%if &OrderInt > 1 and ((&i < &nipl) or (&nix > 0) or (&nip > 0)) %then %do; | %end;
							%end; 
							%do i=1 %to &nix; 
								%sysfunc(varname(&_dsix_,&i)) 
								%if &OrderInt > 1 %then %do;
									%do j = 2 %to &OrderInt; | %sysfunc(varname(&_dsix_,&i)) %end;
									%if ((&i < &nix) or (&nip > 0)) %then %do; | %end;
								%end;
							%end; 
							%do i=1 %to &nip; 
								%sysfunc(varname(&_dsip_,&i)) 
								%if &OrderInt > 1 and (&i < &nip) %then %do; | %end;
							%end; 
							%if &OrderInt > 1 %then %do;
								@ &OrderInt
							%end;

						%end;
						/ 
						%if &whichproc = glmselect %then %do;
							selection=&SelectMethod 
							%if &SelectMethod ^= None %then %do;
								(
									%if &StopCriterion = None %then %do;
										%if (&SelectMethod^=LAR) and (&SelectMethod^=LASSO) and (&SelectMethod^=ElasticNet) %then %do;
											select=SL 
											%if (&SelectMethod=Forward) or (&SelectMethod=Stepwise) %then %do;
												slentry=0.99
											%end;
											%if (&SelectMethod=Backward) or (&SelectMethod=Stepwise) %then %do;
												slstay=0.99
											%end;
										%end;
										%if &MaxStep > 0 %then %do; maxstep=&MaxStep %end;
									%end;
									%else %do;
										stop=&StopCriterion %if &MaxStep > 0 %then %do; maxstep=&MaxStep %end;
										%if (&SelectMethod^=LAR) and (&SelectMethod^=LASSO) and (&SelectMethod^=ElasticNet) %then %do;
											select=&StopCriterion
										%end;
										%else %do;
											%if (&Adaptive = Yes) %then %do;
												ADAPTIVE
											%end;
											%if (&StopCriterion=PRESS) %then %do;
												LSCOEFFS
											%end;
											%else %if (&StopCriterion=L1) %then %do;
												L1=&L1RegParm
											%end;
											%if (&SelectMethod=ElasticNet) and (&L2RegParm > 0) %then %do;
												L2=&L2RegParm
											%end;	
										%end;
										%if (&StopCriterion=SL) %then %do;
											%if (&SelectMethod^=Backward) %then %do;
												slentry=&SLEntry
											%end;
											%if (&SelectMethod^=Forward) %then %do;
												slstay=&SLStay
											%end;
											/*
											%if (&SelectMethod=Forward) or (&SelectMethod=Stepwise) or (&SelectMethod=LAR) or 
												(&SelectMethod=LASSO) or (&SelectMethod=ElasticNet) %then %do;
												slentry=&SLEntry
											%end;
											%if (&SelectMethod=Backward) or (&SelectMethod=Stepwise) or (&SelectMethod=LAR) or 
												(&SelectMethod=LASSO) or (&SelectMethod=ElasticNet) %then %do;
												slstay=&SLStay
											%end;
											*/
										%end;
										%if (&SelectMethod=Forward) or (&SelectMethod=Backward) or (&SelectMethod=Stepwise) %then %do;
											include=&lockin
										%end;
									%end;
								) 
							%end;
							%if (&StopCriterion=CV) and ^%index(%upcase(&GLMSelectModelOptions),CVMETHOD) %then %do;
								cvmethod=&CVPMethod ( &CVKFold )
							%end;
						%end;
						&GLMSelectModelOptions
					;
					%if &whichproc ^= glmselect %then %do;
						selection method=&SelectMethod 
						%if &SelectMethod ^= None %then %do;
							(
								stop=&StopCriterion %if &MaxStep > 0 %then %do; maxstep=&MaxStep %end;
								%if (&SelectMethod^=LAR) and (&SelectMethod^=LASSO) and (&SelectMethod^=ElasticNet) %then %do;
									select=&StopCriterion
								%end;
								%else %do;
									%if (&Adaptive = Yes) %then %do;
										ADAPTIVE
									%end;
								%end;
								%if (&StopCriterion=SL) %then %do;
									%if (&SelectMethod=Forward) or (&SelectMethod=Stepwise) %then %do;
										slentry=&SLEntry
									%end;
									%if (&SelectMethod=Backward) or (&SelectMethod=Stepwise) %then %do;
										slstay=&SLStay
									%end;
								%end;
							) 
						%end;
						;
					%end;
					%if %length(&WeightVar) %then %do;
						weight &WeightVar;
					%end;
					%else %if (&Priors^=Proportional) %then %do;
						weight _Prior;
					%end;
					/*
					%if %length(&FreqVar) %then %do;
						freq &FreqVar;
					%end;
					*/
					%if (&TestSpec = Yes) or (&AsymmetricLossFitProp ne 0.5) %then %do;
						output out=ts1o(drop=
							%do i=1 %to &nix; %sysfunc(varname(&_dsix_,&i)) %end; 
							%do i=1 %to &nip; %sysfunc(varname(&_dsip_,&i)) %end; 
						) pred;
					%end;
					%if &whichproc = glmselect %then %do;
						score data=&ts2 out=ts2o(drop=
							%do i=1 %to &nix; %sysfunc(varname(&_dsix_,&i)) %end; 
							%do i=1 %to &nip; %sysfunc(varname(&_dsip_,&i)) %end; 
						) pred;
					%end;
					%else %do;
						code file="&OutPath.&whichproc._score.sas";
					%end;
					ods output selectedeffects=se parameterestimates=pe;
				run;
				
				/*
				data OutLib.se;
					set se;
				run;
				*/

				%if &syserr > 4 %then %do;
					%let exiterror = 1;
					%goto exit;
				%end;
				
				/*
				%if &CVFlag %then %do;
					data OutLib.ts2o_&iter._&k;
						set ts2o;
					run;
				%end;
				*/
				
				/*
				data OutLib.glm_ts2o;
					set ts2o;
				run;
				data OutLib.glm_pe;
					set pe;
				run;
				*/
				
				%if ^&CVFlag %then %do;
					* remove underscore after classvar parameter name to match format from other procs;
					/*S1471036: commenting out this if statement*/
					/*
					%if &nip or &nipl %then %do;
						data pe;
							set pe;
							%do i=1 %to &nipl; 
								%let cn = %upcase(%sysfunc(varname(&_dsipl_,&i)));
								%let lcn = %length(&cn);
								if upcase(substr(parameter,1,&lcn)) = "&cn" then substr(parameter,%eval(&lcn+1),1) = " ";
							%end; 
							%do i=1 %to &nip; 
								%let cn = %upcase(%sysfunc(varname(&_dsip_,&i)));
								%let lcn = %length(&cn);
								if upcase(substr(parameter,1,&lcn)) = "&cn" then substr(parameter,%eval(&lcn+1),1) = " ";
							%end;
						run;
						
						data OutLib.glm_pe01; set pe; run;
						
					%end;
					*/
					
					*****************************;
					*S1301121: avoid var name trunction from Proc GlmSelect;
					/*S1470991: correcting the parameter variable so it captures 
					all words from the levels of those variables that have more 
					than one word. For instance, a variable Smoking with levels 
					"Never Used" and "Not Available".*/
					%if &whichproc = glmselect %then %do; 
						data pe;
							set pe(rename= (parameter=parold));
							if scan(parold,2," ") ^= "" then do;
								*parameter= trim(left(effect))||" "||trim(left(scan(parold,2," ")));
								_vlen_ = length(trim(left(scan(parold,1," "))));
								parameter= trim(left(effect))||" "||trim(left(substr(parold,_vlen_+1)));
							end;
							else do;
								parameter= parold;
							end;
							drop parold;
						run;
					%end;
					*****************************;
					
					/*data OutLib.glm_pe02; set pe; run;*/

					%ScoringCodeMacroVars;
					
					%let suffix= _glm_pe;
					%let b32 = %eval(32 - %length(&suffix));
					%if %length(&OutName) > &b32 %then %let inname = %substr(&OutName,1,&b32);
					%else %let inname = &OutName;		
					%let peout = %quote(&inname.%trim(%left(&suffix)));
					%put peout = &peout;

					data OutLib.%unquote(&peout);
						set pe1;
					run;					
						
				%end;

				%if &whichproc ^= glmselect %then %do;
					data ts2o;
						set &ts2;
						%include "&OutPath.&whichproc._score.sas";
						drop
							%do i=1 %to &nix; %sysfunc(varname(&_dsix_,&i)) %end; 
							%do i=1 %to &nip; %sysfunc(varname(&_dsip_,&i)) %end;
						;
					run;
					/*
					data OutLib.ts2o;
						set ts2o;
					run;
					*/
				%end;


				%if &SelectMethod ^= None %then %do;

					/*
					data ts2vars;
						length name $ 32;
						set se;
						i = 1;
						do while(i);
							name = scan(effects,i,' ');
							if lengthn(name)=0 then i = 0;
							else do;
								if name ne "Intercept" then output;
								i = i + 1;
							end;
						end;
						drop i label effects;
					run;
					*/
					data ts2vars;
						set pe;
						if effect = "Intercept" then delete;
						keep effect;
						rename effect = name;
					run;
					
					%let havets2v = 1;
					%UpdateSelVars;

					* for LAR, LASSO, and ElasticNet, class vars are split, so adjust names here;
					%if (&nipl or &nip) and ((&SelectMethod=LAR) or (&SelectMethod=LASSO) or (&SelectMethod=ElasticNet)) %then %do;
						%UnsplitSelVars;
					%end;
				%end;

				%if &AsymmetricLossFitProp ne 0.5 %then %do;
					data ts1a;
						set ts1o;
						keep _Row_ &DepVar p_&DepVar &WeightVar &WeightVar0;
						rename p_&DepVar=pred;
					run;
				%end;

				%if (&TestSpec = Yes) %then %do; 
					data ts2o;
						set ts1o ts2o;
					run;
				%end;

				data ts2o;
					set ts2o;
					rename p_&DepVar=pred;
				run;
				/*
				proc print data=ts2o;
				run;
				*/
				* create variables for computing cv ase;
				%if (&TraitType=Binary) %then %do;							

					data ts2o;
						set ts2o;
						if missing(pred) then do;
							p_&DepVarTrunc._0 = .;
							p_&DepVarTrunc._1 = .;
							_INTO_ = .;
						end;
						else do;
							p_&DepVarTrunc._0 = 1 - pred;
							if p_&DepVarTrunc._0 < 0 then p_&DepVarTrunc._0 = 0;
							else if p_&DepVarTrunc._0 > 1 then p_&DepVarTrunc._0 = 1;
							p_&DepVarTrunc._1 = 1 - p_&DepVarTrunc._0;
							_INTO_ = (p_&DepVarTrunc._1 > 0.5);
						end;
						%if &_dtype_ = C %then %do;
							if &TruthVar = " " then
						%end;
						%else %do;
							if &TruthVar = . then
						%end;
							actualp = .;
						else if (&TruthVar=1) then actualp = p_&DepVarTrunc._1;
						else actualp = p_&DepVarTrunc._0;
					run;

					/*
					ods exclude none;
					ods results;
					title "glmselect binary ts2o";
					proc print data=ts2o;
					run;
					title;
					*/

					/*
					%let ncl = 2;
					%let pprob1 = p_&DepVarTrunc._0;
					%let pprob2 = p_&DepVarTrunc._1;
					*/
				%end;

				%let into = _INTO_;
				%let pred = pred;
			%end;
			%else %if (&ProcessName = QuantSelect) %then %do;

				* analysis settings output string;
				%if %length(&ASet) %then %do;
					%let ASet = &Aset, Quantile Level = &QuantileLevel, &trModelSelectionMethod = %LocaleEZX(%nrbquote(&SelectMethod), EIA=1), &trStopCriterion = %LocaleEZX(%nrbquote(&StopCriterion), EIA=1);
				%end;
				%else %do;
					%let ASet = Quantile Level = &QuantileLevel, &trModelSelectionMethod = %LocaleEZX(%nrbquote(&SelectMethod), EIA=1), &trStopCriterion = %LocaleEZX(%nrbquote(&StopCriterion), EIA=1);
				%end;
				%if (&StopCriterion = SL) %then %do;
					%let ASet = &ASet, &trSLEntry = &SLEntry, &trSLStay = &SLStay;
				%end;
				/*
				%if (&VarSelect = Genetic Algorithm) %then %do;
					%let ASet = &ASet, %LocaleEZX(%nrbquote(&VarSelect), EIA=1);
				%end;
				*/
				
				data pe;
					if (0);
				run;

				/*S1471114: using namelen to fix this truncation of variable names*/
				proc quantselect data=&ts1
					%if %length(&Seed) %then %do;
						seed=&Seed
					%end;
					&QuantSelectOptions namelen=128 
					/*
					%if (&TestSpec = Yes) %then %do;
						testdata=&ts2
					%end;
					*/
					%if ^&CVFlag and &SelectMethod ^= None %then %do;
						plots=all
					%end;
					;
					%if &nipl or &nip %then %do;
						class 
							%do i=1 %to &nipl; %sysfunc(varname(&_dsipl_,&i)) %end; 
							%do i=1 %to &nip; %sysfunc(varname(&_dsip_,&i)) %end; 
							/ missing
						;
					%end;
					model &DepVar = 
						%if %index(&VarSelect,Genetic) %then %do;
							&SelVars
						%end;
						%else %do;
							%do i=1 %to &nixl; 
								%sysfunc(varname(&_dsixl_,&i)) 
								%if &OrderInt > 1 %then %do;
									%do j = 2 %to &OrderInt; | %sysfunc(varname(&_dsixl_,&i)) %end;
									%if ((&i < &nixl) or (&nipl > 0) or (&nix > 0) or (&nip > 0)) %then %do; | %end;
								%end;
							%end; 
							%do i=1 %to &nipl; 
								%sysfunc(varname(&_dsipl_,&i)) 
								%if &OrderInt > 1 and ((&i < &nipl) or (&nix > 0) or (&nip > 0)) %then %do; | %end;
							%end; 
							%do i=1 %to &nix; 
								%sysfunc(varname(&_dsix_,&i)) 
								%if &OrderInt > 1 %then %do;
									%do j = 2 %to &OrderInt; | %sysfunc(varname(&_dsix_,&i)) %end;
									%if ((&i < &nix) or (&nip > 0)) %then %do; | %end;
								%end;
							%end; 
							%do i=1 %to &nip; 
								%sysfunc(varname(&_dsip_,&i)) 
								%if &OrderInt > 1 and (&i < &nip) %then %do; | %end;
							%end; 
							%if &OrderInt > 1 %then %do;
								@ &OrderInt
							%end;
						%end;
						/ selection=&SelectMethod 
						%if &SelectMethod ^= None %then %do;
							(
								%if &StopCriterion = None %then %do;
									%if (&SelectMethod^=LAR) and (&SelectMethod^=LASSO) and (&SelectMethod^=ElasticNet) %then %do;
										select=SL 
										%if (&SelectMethod=Forward) or (&SelectMethod=Stepwise) %then %do;
											slentry=0.99
										%end;
										%if (&SelectMethod=Backward) or (&SelectMethod=Stepwise) %then %do;
											slstay=0.99
										%end;
									%end;
									%if &MaxStep > 0 %then %do; maxstep=&MaxStep %end;
								%end;
								%else %do;
									%if &StopCriterion ^= SL %then %do;
										stop=&StopCriterion 
									%end;
									%if &MaxStep > 0 %then %do; maxstep=&MaxStep %end;
									%if (&SelectMethod^=LASSO) and (&SelectMethod^=LAR) and (&SelectMethod^=ElasticNet) %then %do;
										select=&StopCriterion
									%end;
									/*
									%else %do;
										%if (&StopCriterion=PRESS) %then %do;
											LSCOEFFS
										%end;
									%end;
									*/
									%if (&StopCriterion=SL) %then %do;
										%if (&SelectMethod=Forward) or (&SelectMethod=Stepwise) %then %do;
											slentry=&SLEntry
										%end;
										%if (&SelectMethod=Backward) or (&SelectMethod=Stepwise) %then %do;
											slstay=&SLStay
										%end;
									%end;
									%if (&SelectMethod=Forward) or (&SelectMethod=Backward) or (&SelectMethod=Stepwise) %then %do;
										include=%eval(&nixl+&nipl)
									%end;
								%end;
							) 
						%end;
						/*
						%if (&StopCriterion=CV) and ^%index(%upcase(&QuantSelectModelOptions),CVMETHOD) %then %do;
							cvmethod=&CVPMethod ( &CVKFold )
						%end;
						*/
						&QuantSelectModelOptions
					;
					%if %length(&WeightVar) %then %do;
						weight &WeightVar;
					%end;
					%else %if (&Priors^=Proportional) %then %do;
						weight _Prior;
					%end;
					/*
					%if %length(&FreqVar) %then %do;
						freq &FreqVar;
					%end;
					*/
					output out=ts1o(drop=
						%do i=1 %to &nixl; %sysfunc(varname(&_dsixl_,&i)) %end; 
						%do i=1 %to &nipl; %sysfunc(varname(&_dsipl_,&i)) %end; 
						%do i=1 %to &nix; %sysfunc(varname(&_dsix_,&i)) %end; 
						%do i=1 %to &nip; %sysfunc(varname(&_dsip_,&i)) %end; 
					) pred=pred;
					/*
					score data=&ts2 out=ts2o(drop=
						%do i=1 %to &nix; %sysfunc(varname(&_dsix_,&i)) %end; 
						%do i=1 %to &nip; %sysfunc(varname(&_dsip_,&i)) %end; 
					) pred;
					*/
					ods output selectedeffects=se ParameterEstimates=pe;
				run;

				%NObsVars(pe);
				%if &nobs = 0 %then %do;
					%put ERROR: PROC QUANTSELECT did not converge or produce predictions.;
					%put If cross-validating, you may have constant data.  If not, please ;
					%put inspect the SAS Output and try specifying convergence options or a different model.;
					%let exiterror = 1;
					/* %goto exit; */
				%end;	
				%else %do;

					* calculate prediction for &ts2 dataset, which is
					*    1. the test dataset if it exists
					*    2. the training dataset if no test data
					*    3. the holdout set when &CVing=1;
															
					%ScoringCodeMacroVars;
					
					%let suffix= _qs_pe;
					%let b32 = %eval(32 - %length(&suffix));
					%if %length(&OutName) > &b32 %then %let inname = %substr(&OutName,1,&b32);
					%else %let inname = &OutName;		
					%let peout = %quote(&inname.%trim(%left(&suffix)));
					%put peout = &peout;

					data OutLib.%unquote(&peout);
						set pe1;
					run;

					*****************************;
					* avoid var name trunction from Proc QuantSelect in Stat v13.1;
					ods exclude all;
					ods noresults;
					proc contents data=&ts1 out=tmp1;
					run;
					ods exclude none;
					ods results;
					
					data tmp1;
						set tmp1;
						if length(Name) >= 20;
					run;

					%do i = 1 %to &nPar;
						%if %length(&&ParName_&i) = 20 %then %do;
							%*put ParName_&i = &&ParName_&i;
							data _null_;
								set tmp1;
								if Name =: "&&ParName_&i" then do;
									call symput("ParName_&i",name);
									stop;
								end;
							run;
							%*put ParName_&i = &&ParName_&i;
						%end;
					%end;
					*****************************;

					data ts2o;
						set &ts2;
						if &IntFlag then pred = &Intercept;
						else pred = 0; 
						%do i = 1 %to &nPar;
						%*put i=&i, cFlag=&&ClassFlag_&i, ParName=&&ParName_&i, ParVal=&&ParVal_&i;
							%if (&&ClassFlag_&i) %then %do;
								%if (&&ParType_&i=N) %then %do;
									if (&&ParName_&i=&&Level_&i) then do;
										pred = pred +  &&ParVal_&i;
									end;
								%end;
								%else %do;
									if (&&ParName_&i="&&Level_&i") then do;
										pred = pred +  &&ParVal_&i;
									end;
								%end;
							%end;
							%else %do;
								pred = pred +  &&ParVal_&i * &&ParName_&i;
							%end;
						%end;
						drop
							%do i=1 %to &nixl; %sysfunc(varname(&_dsixl_,&i)) %end; 
							%do i=1 %to &nipl; %sysfunc(varname(&_dsipl_,&i)) %end; 
							%do i=1 %to &nix; %sysfunc(varname(&_dsix_,&i)) %end; 
							%do i=1 %to &nip; %sysfunc(varname(&_dsip_,&i)) %end; 
						;
					run;

					%if &syserr > 4 %then %do;
						%let exiterror = 1;
						%goto exit;
					%end;

					%if &SelectMethod ^= None %then %do;
						data ts2vars;
							length name $ 32;
							set se;
							i = 1;
							do while(i);
								name = scan(effects,i,' ');
								if lengthn(name)=0 then i = 0;
								else do;
									if name ne "Intercept" then output;
									i = i + 1;
								end;
							end;
							drop i label effects;
						run;
						%let havets2v = 1;
						%UpdateSelVars;

						* for LASSO, class vars are split, so adjust names here;
						%if (&nipl or &nip) and ((&SelectMethod=LASSO) or (&SelectMethod=LAR) or (&SelectMethod=ElasticNet))
							%then %do;
							%UnsplitSelVars;
						%end;
					%end;

					%if (&TestSpec = Yes) %then %do;
						* combine training and test dataset;
						data ts2o;
							set  ts1o ts2o;
						run;
					%end;

					data ts2o;
						set ts2o;
						*rename p_&DepVar=pred;
					run;
					/*
					proc print data=ts2o;
					run;
					*/
					* create variables for computing cv ase;
					%if (&TraitType=Binary) %then %do;
						data ts2o;
							set ts2o;
							if missing(pred) then do;
								p_&DepVarTrunc._0 = .;
								p_&DepVarTrunc._1 = .;
								_INTO_ = .;
							end;
							else do;
								p_&DepVarTrunc._0 = 1 - pred;
								if p_&DepVarTrunc._0 < 0 then p_&DepVarTrunc._0 = 0;
								else if p_&DepVarTrunc._0 > 1 then p_&DepVarTrunc._0 = 1;
								p_&DepVarTrunc._1 = 1 - p_&DepVarTrunc._0;
								_INTO_ = (p_&DepVarTrunc._1 > 0.5);
							end;
							%if &_dtype_ = C %then %do;
								if &TruthVar = " " then
							%end;
							%else %do;
								if &TruthVar = . then
							%end;
								actualp = .;
							else if (&TruthVar=1) then actualp = p_&DepVarTrunc._1;
							else actualp = p_&DepVarTrunc._0;
						run;

						/*
						ods exclude none;
						ods results;
						title "quantselect binary ts2o";
						proc print data=ts2o;
						run;
						title;
						*/

						/*
						%let ncl = 2;
						%let pprob1 = p_&DepVarTrunc._0;
						%let pprob2 = p_&DepVarTrunc._1;
						*/
					%end;
				%end;
				%let into = _INTO_;
				%let pred = pred;
			%end;
			%else %if (&ProcessName = PartialLeastSquares) %then %do;

				%if ^%index(&VarSelect,Genetic) and %eval(&nix+&nip) > &PLSLimit %then %do;
					%put ERROR: Partial Least Squares cannot handle more than &PLSLimit total predictor columns.  Please specify fewer predictors.;
					%let exiterror = 1;
					%goto exit;
				%end;

				* using missing values for the dependent variable to create predictions;
				%if &CVFlag or (&TestSpec=Yes) %then %do;
					data &ts1;
						set &ts1;
						_y0 = &DepVar;
						_y = &DepVar;
					run;
					data ts1o;
						set &ts1;
					run;
					data &ts2;
						set &ts2;
						_y0 = &DepVar;
						%if (&_dtype_=C) %then %do;
							_y = " ";
						%end;
						%else %do;
							_y = .;
						%end;
					run;
					data &ts1;
						set &ts1 &ts2;
					run;
					%let UseDep = _y;
				%end;
				%else %do;
					%let UseDep = &DepVar;
				%end;

				%if %sysfunc(exist(xw)) %then %do;
					proc datasets library=work nolist;
						delete xw;
					quit;
				%end;

				%if (&YReflection = Yes) %then %do;
					data &ts1;
						set &ts1;
						&DepVarTrunc._R = 1 - &UseDep;
						_y_std = &UseDep;
						_y_std_r = &DepVarTrunc._R;
					run;
				%end;
				%else %do;
					data &ts1;
						set &ts1;
						_y_std = &UseDep;
					run;			
				%end;

				* standardize dependent variable;
				proc stdize data=&ts1 method=std out=&ts1 outstat=stat;
					var _y_std %if (&YReflection = Yes) %then %do; _y_sts_r %end; ;
				run;

				data _null_;
					set stat;
					if _type_ = "LOCATION" then call symputx('location',_y_std);
					if _type_ = "SCALE" then call symputx('scale',_y_std);
					%if (&YReflection = Yes) %then %do;
						if _type_ = "LOCATION" then call symputx('location_r',_y_std_r);
						if _type_ = "SCALE" then call symputx('scale_r',_y_std_r);
					%end;
				run;	

				proc pls data=&ts1 nfac=&NumberOfComponents &ProcPLSOptions details
						/* missing=avg missing=em */;

					/*
					effect spl=spline(
						%if %index(&VarSelect,Genetic) %then %do;
							&SelVars
						%end;
						%else %do;
							%do i=1 %to &nixl; %sysfunc(varname(&_dsixl_,&i)) %end; 
							%do i=1 %to &nipl; %sysfunc(varname(&_dsipl_,&i)) %end; 
							%do i=1 %to &nix; %sysfunc(varname(&_dsix_,&i)) %end; 
							%do i=1 %to &nip; %sysfunc(varname(&_dsip_,&i)) %end; 					
						%end;
						/ separate
					);
					model _y_std %if (&YReflection = Yes) %then %do; _y_std_r %end; =  
						spl / solution;
					;
					*/

					model _y_std %if (&YReflection = Yes) %then %do; _y_std_r %end; =  
						%if %index(&VarSelect,Genetic) %then %do;
							&SelVars
						%end;
						%else %do;
							%do i=1 %to &nixl; %sysfunc(varname(&_dsixl_,&i)) %end; 
							%do i=1 %to &nipl; %sysfunc(varname(&_dsipl_,&i)) %end; 
							%do i=1 %to &nix; %sysfunc(varname(&_dsix_,&i)) %end; 
							%do i=1 %to &nip; %sysfunc(varname(&_dsip_,&i)) %end; 
						%end;
						/ solution;
					;

					output 
						out=ts2o(drop=
							%do i=1 %to &nixl; %sysfunc(varname(&_dsixl_,&i)) %end; 
							%do i=1 %to &nipl; %sysfunc(varname(&_dsipl_,&i)) %end; 
							%do i=1 %to &nix; %sysfunc(varname(&_dsix_,&i)) %end; 
							%do i=1 %to &nip; %sysfunc(varname(&_dsip_,&i)) %end; 
						)
						predicted=pred %if (&YReflection = Yes) %then %do; &DepVarTrunc._RP %end; xscore=pls h=leverage tsquare=tsquare
					;
					ods output XWeights=xw PercentVariation=pctvar ParameterEstimates=solution;
					ods exclude XLoadings XWeights ParameterEstimates;
				run;

				%if ^%sysfunc(exist(xw)) %then %do;
					%put WARNING: No PLS components selected.;
					data ts2vars;	
						length name $ 32;
						name = "Intercept";
					run;
					%let NumberOfComponents = 1;
				%end;
				%else %do;

					%NObsVars(xw);
					%let NumberOfComponents = &nobs;
					/*
					data OutLib.xw;
						set xw;
					run;
					*/
					%let dropvars = NumberOfFactors;
					%if ^%index(%upcase(&ProcPLSOptions),PCR) %then %do;
						%let dropvars = &dropvars InnerRegCoef;
					%end;
					proc transpose data=xw(drop=&dropvars) out=ts2vars prefix=pls;
						/*
						var 
							%if %index(&VarSelect,Genetic) %then %do;
								&SelVars
							%end;
							%else %do;
								%do i=1 %to &nixl; %sysfunc(varname(&_dsixl_,&i)) %end; 
								%do i=1 %to &nipl; %sysfunc(varname(&_dsipl_,&i)) %end; 
								%do i=1 %to &nix; %sysfunc(varname(&_dsix_,&i)) %end;
								%do i=1 %to &nip; %sysfunc(varname(&_dsip_,&i)) %end; 
							%end;
						;
						*/
					run;
					
					/*data OutLib.ts2vars; set ts2vars; run;*/

					* compute variable importance as described in PROC PLS doc;
					data _null_;
						set pctvar;
						call symput("cyv"||trim(left(_n_)),trim(left(CurrentYVariation)));
					run;

					data ts2v;
						length name $ 32;
						set ts2vars;
						name = _name_;
						tsquare = uss(of pls1-pls&NumberOfComponents);
						_ScoreType = "Col";
						drop _name_;
					run;

					proc transpose data=pctvar out=pctvary prefix=cyv;
						var CurrentYVariation;
					run;

					proc sql; 
						create table ts2vars 
							as select *, 
							%do i=1 %to &NumberOfComponents;
								pls&i/sqrt(uss(pls&i)) as plsnorm&i,
							%end;
							tsquare/max(tsquare) as tsquare_range
							from ts2v left join pctvary(drop=_name_ _label_) on 1; 

					%NObsVars(ts2vars);

					data ts2vars;
						set ts2vars;
						VIP = 0; 
						%do i=1 %to &NumberOfComponents;
							VIP = VIP + cyv&i * (plsnorm&i**2);
						%end;
						VIP = sqrt(VIP * &nobs / sum(of cyv:)); 
					run;

					data solution; 
						set solution; 
						if (RowName = 'Intercept') then delete; 
						drop RowName; 
					run; 

					data ts2vars;
						merge ts2vars solution;
					run;
					
					%if %length(&PLSWhere) %then %do;
						data ts2vars;
							set ts2vars;
							where %unquote(&PLSWhere);
						run;

						%if ^&NPred %then %let NPred = 999999999999;
					%end;


					/*
					data OutLib.ts2vars;
						set ts2vars;
					run;
					*/

					%if &NPred %then %do;

						%if &nixl or &nipl %then %do;
							data ts2v_lock;
								set ts2vars(obs=%eval(&nixl+&nipl));
							run;
							data ts2vars;
								set ts2vars(firstobs=%eval(&nixl+&nipl+1));
							run;
						%end;

						proc sort data=ts2vars;
							by descending vip;
						run;

						%NObsVars(ts2vars);
						%if &nobs < &NPred %then %let NPred = &nobs;

						data ts2vars;
							set ts2vars(obs=&NPred);
						run;

						%if &nixl or &nipl %then %do;
							data ts2vars;
								set ts2v_lock ts2vars;
							run;
						%end;

						proc transpose data=ts2vars out=_dsib_(drop=_name_);
							var vip;
							id name;
						run;

						%let _dsib_ = %sysfunc(open(_dsib_));

						proc pls data=&ts1 nfac=&NumberOfComponents &ProcPLSOptions details 
								/* missing=avg missing=em */;
							model _y_std %if (&YReflection = Yes) %then %do; _y_std_r %end; =  
								%if %index(&VarSelect,Genetic) %then %do;
									&SelVars
								%end;
								%else %do;
									%do i=1 %to &nixl; %sysfunc(varname(&_dsixl_,&i)) %end; 
									%do i=1 %to &nipl; %sysfunc(varname(&_dsipl_,&i)) %end; 
									%do i=1 %to &NPred; %sysfunc(varname(&_dsib_,&i)) %end; 
								%end;
								/ solution;
							;
							output 
								out=ts2o(drop=
									%do i=1 %to &nixl; %sysfunc(varname(&_dsixl_,&i)) %end; 
									%do i=1 %to &nipl; %sysfunc(varname(&_dsipl_,&i)) %end; 
									%do i=1 %to &nix; %sysfunc(varname(&_dsix_,&i)) %end; 
									%do i=1 %to &nip; %sysfunc(varname(&_dsip_,&i)) %end; 
								)
								predicted=pred %if (&YReflection = Yes) %then %do; pred_r %end; xscore=pls h=leverage tsquare=tsquare
							;
							ods output XWeights=xw PercentVariation=pctvar ParameterEstimates=solution;
							ods exclude XLoadings XWeights ParameterEstimates;
						run;

						%if ^%sysfunc(exist(xw)) %then %do;
							%put WARNING: No PLS components selected.;
							data ts2vars;	
								length name $ 32;
								name = "Intercept";
							run;
							%let NumberOfComponents = 1;
						%end;
						%else %do;

							%NObsVars(xw);
							%let NumberOfComponents = &nobs;
							proc transpose data=xw out=ts2vars prefix=pls;
								var 
									%if %index(&VarSelect,Genetic) %then %do;
										&SelVars
									%end;
									%else %do;
										%do i=1 %to &nixl; %sysfunc(varname(&_dsixl_,&i)) %end; 
										%do i=1 %to &nipl; %sysfunc(varname(&_dsipl_,&i)) %end; 
										%do i=1 %to &NPred; %sysfunc(varname(&_dsib_,&i)) %end; 
									%end;
								;
							run;

							* compute variable importance again;
							data _null_;
								set pctvar;
								call symput("cyv"||trim(left(_n_)),trim(left(CurrentYVariation)));
							run;

							data ts2v;
								length name $ 32;
								set ts2vars;
								name = _name_;
								tsquare = uss(of pls1-pls&NumberOfComponents);
								_ScoreType = "Col";
								drop _name_;
							run;

							proc transpose data=pctvar out=pctvary prefix=cyv;
								var CurrentYVariation;
							run;

							proc sql; 
								create table ts2vars 
									as select *, 
									%do i=1 %to &NumberOfComponents;
										pls&i/sqrt(uss(pls&i)) as plsnorm&i,
									%end;
									tsquare/max(tsquare) as tsquare_range
									from ts2v left join pctvary(drop=_name_ _label_) on 1; 

							%NObsVars(ts2vars);

							data ts2vars;
								set ts2vars;
								VIP = 0; 
								%do i=1 %to &NumberOfComponents;
									VIP = VIP + cyv&i * (plsnorm&i**2);
								%end;
								VIP = sqrt(VIP * &nobs / sum(of cyv:)); 
							run;

							data solution; 
								set solution; 
								if (RowName = 'Intercept') then delete; 
								drop RowName; 
							run; 

							data ts2vars;
								merge ts2vars solution;
								if missing(vip) then delete;
							run;

							proc sort data=ts2vars;
								by descending vip;
							run;

						%end;

						%let _dsib_ = %sysfunc(close(&_dsib_));

					%end;
					%else %do;
						proc sort data=ts2vars;
							by descending vip;
						run;
						
						/*data OutLib.ts2vars2; set ts2vars; run;*/

						%if &CVFlag %then %do;
							data ts2vars;
								set ts2vars;
								if VIP > 0.999;
								keep name;
							run;
						%end;
					%end;
				%end;

				%let havets2v = 1;

				%put location = &location;
				%put scale = &scale;
				/*
				data OutLib.xw;
					set xw;
				run;
				data OutLib.solution;
					set solution;
				run;
				*/

				* string for analysis settings;
				%if &aloss_niter = 1 %then %do;
					%if %length(&ASet) %then %do;
						%let ASet = &ASet, &trNumberofPLSComponents = &NumberOfComponents;
					%end;
					%else %do;
						%let ASet = &trNumberofPLSComponents = &NumberOfComponents;
					%end;
					* string for analysis settings;
					%if &NPred %then %do;
						%let trTopNPredPredictors = %nrbquote(%LocaleFill(%nrbquote(&trTopXPredictors), &NPred));
						%let ASet = &ASet, &trTopNPredPredictors &PLSWhere;
					%end;
					%if (&VarSelect = Genetic Algorithm) %then %do;
						%let ASet = &ASet, %LocaleEZX(%nrbquote(&VarSelect), EIA=1);
					%end;
				%end;

				* generate scoring code;			
				%if ^&CVFlag %then %do;
					data pe;
						length Parameter $ 32 Estimate 8;
						Parameter = "Intercept";
						Estimate = &location;
					run;

					%if %sysfunc(exist(xw)) %then %do;
						proc transpose data=xw(drop=&dropvars) out=xwt;
							/*
							var 
								%if %index(&VarSelect,Genetic) %then %do;
									&SelVars
								%end;
								%else %if ^&NPred %then %do;
									%do i=1 %to &nixl; %sysfunc(varname(&_dsixl_,&i)) %end; 
									%do i=1 %to &nipl; %sysfunc(varname(&_dsipl_,&i)) %end; 
									%do i=1 %to &nix; %sysfunc(varname(&_dsix_,&i)) %end;
									%do i=1 %to &nip; %sysfunc(varname(&_dsip_,&i)) %end; 
								%end;
								%else %do;
									%do i=1 %to &nixl; %sysfunc(varname(&_dsixl_,&i)) %end; 
									%do i=1 %to &nipl; %sysfunc(varname(&_dsipl_,&i)) %end; 
									%do i=1 %to &NPred; %sysfunc(varname(&_dsib_,&i)) %end; 
								%end;
							;
							*/
						run;

						data pe1;
							length Parameter $ 32 Estimate 8;
							merge xwt solution;
							Parameter = _name_;
							Estimate = _y_std * &scale;
							keep Parameter Estimate;
						run;
						data pe;
							set pe pe1;
						run;
					%end;

					%ScoringCodeMacroVars;
					
					%let suffix= _pls_pe;
					%let b32 = %eval(32 - %length(&suffix));
					%if %length(&OutName) > &b32 %then %let inname = %substr(&OutName,1,&b32);
					%else %let inname = &OutName;		
					%let peout = %quote(&inname.%trim(%left(&suffix)));
					%put peout = %unquote(&peout);

					data OutLib.&peout;
						set pe1;
					run;
				%end;

				/*
				%if &AsymmetricLossFitProp ne 0.5 %then %do;
					data ts1a;
						set ts2o;
						pred = pred*&scale + &location;
						keep _Row_ &DepVar P_&DepVarTrunc30 &WeightVar &WeightVar0;
					run;
				%end;
				*/

				* update predictions;
				data ts2o;
					length _ScoreType $ 3 _Label $ 32;
					set ts2o;
					%if &CVFlag %then %do;
						%if &_dtype_ = C %then %do;
							where &UseDep = " ";
						%end;
						%else %do;
							where &UseDep = .;
						%end;
						&DepVar = _y0;
						drop _y0 _y;
					%end;
					pred = pred*&scale + &location;
					/*
					if pred < 0 then pred = 0;
					else if pred > 1 then pred = 1;
					*/
					/*
					pred = exp(7.5*(pred-0.5))/(1+exp(7.5*(pred-0.5)));
					pred = probnorm(4*(pred-0.5));
					*/
					%if (&YReflection = Yes) %then %do;
						&DepVarTrunc._RP = pred_r*&scale_r + &location_r;
						pred1 = pred;
						if &DepVarTrunc._RP < 0 then &DepVarTrunc._RP = 0;
						else if &DepVarTrunc._RP > 1 then &DepVarTrunc._RP = 1;
						/*
						&DepVarTrunc._RP = exp(7.5*(&DepVarTrunc._RP-0.5))/(1+exp(7.5*(&DepVarTrunc._RP-0.5)));
						&DepVarTrunc._RP = probnorm(4*(&DepVarTrunc._RP-0.5));
						*/

						pred_tot = pred+&DepVarTrunc._RP;
						if pred_tot > 0 then pred = pred/pred_tot;
						else pred = .;
						/*
						pred = (pred + 1 - &DepVarTrunc._RP)/2;
						*/
					%end;
					_ScoreType = "Row";
					%if %length(&ColorVar) %then %do;
						_Color = trim(left(&ColorVar));
					%end;
					%else %do;
						_Color = "Row";
					%end;
					%if %length(&LabelVar) %then %do;
						_Label = trim(left(&LabelVar));
					%end;
					%else %do;
						_Label = "Row Score";
					%end;	
					label _Color=' ' _Label=' ';
				run;


				/*
				data OutLib.ts2oplsb;
					set ts2o;
				run;
				*/

				* create variables for computing cv ase;
				%if (&TraitType=Binary) %then %do;						
					data ts2o;
						set ts2o;
						if missing(pred) then do;
							p_&DepVarTrunc._0 = .;
							p_&DepVarTrunc._1 = .;
							_INTO_ = .;
						end;
						else do;
							/*
							pred = exp(pred-0.5)/(1+exp(pred-0.5));
							*/
							p_&DepVarTrunc._0 = 1 - pred;
							if p_&DepVarTrunc._0 < 0 then p_&DepVarTrunc._0 = 0;
							else if p_&DepVarTrunc._0 > 1 then p_&DepVarTrunc._0 = 1;
							p_&DepVarTrunc._1 = 1 - p_&DepVarTrunc._0;
							_INTO_ = (p_&DepVarTrunc._1 > 0.5);
						end;
						%if &_dtype_ = C %then %do;
							if &TruthVar = " " then
						%end;
						%else %do;
							if &TruthVar = . then
						%end;
							actualp = .;
						else if (&TruthVar=1) then actualp = p_&DepVarTrunc._1;
						else actualp = p_&DepVarTrunc._0;
					run;

				%end;

				%let into = _INTO_;
				%let pred = pred;

			%end;
			%else %if (&ProcessName = RadialBasisMachine) %then %do;

				* string for analysis settings;
				%if &aloss_niter = 1 %then %do;
					%if &NPred %then %do;
						%if %length(&ASet) %then %do;
							%let trTopNPredBLUPs = %nrbquote(%LocaleFill(%nrbquote(&trTopXBLUPs), &NPred));
							%let ASet = &ASet, &trTopNPredBLUPs;
						%end;
						%else %do;
							%let trTopNPredBLUPs = %nrbquote(%LocaleFill(%nrbquote(&trTopXBLUPs), &NPred));
							%let ASet = &trTopNPredBLUPs;
						%end;
					%end;
					%if (&VarSelect = Genetic Algorithm) %then %do;
						%if %length(&ASet) %then %do;
							%let ASet = &ASet, %LocaleEZX(%nrbquote(&VarSelect), EIA=1);
						%end;
						%else %do;
							%let ASet = %LocaleEZX(%nrbquote(&VarSelect), EIA=1);
						%end;
					%end;
				%end;

				* remove rows with missing predictors;
				data d_nomisspred;
					set &ts1;
					if not missing(1) 
						%do i=1 %to &nixl; and not missing(%sysfunc(varname(&_dsixl_,&i))) %end; 
						%do i=1 %to &nipl; and not missing(%sysfunc(varname(&_dsipl_,&i))) %end; 
						%do i=1 %to &nix; and not missing(%sysfunc(varname(&_dsix_,&i))) %end; 
						%do i=1 %to &nip; and not missing(%sysfunc(varname(&_dsip_,&i))) %end; 
						%if %length(&WeightVar) %then %do;
							and not missing(&WeightVar) and &WeightVar > 0
						%end;
					;
				run;
				%let ts1 = d_nomisspred;

				* using missing values for the dependent variable to create predictions;
				%if &CVFlag or (&TestSpec=Yes) %then %do;
					data &ts1;
						set &ts1;
						_y0 = &DepVar;
						_y = &DepVar;
					run;
					data &ts2;
						set &ts2;
						_y0 = &DepVar;
						%if (&_dtype_=C) %then %do;
							_y = " ";
						%end;
						%else %do;
							_y = .;
						%end;
					run;
					data &ts1;
						set &ts1 &ts2;
					run;
					%let UseDep = _y;
				%end;
				%else %do;
					%let UseDep = &DepVar;
				%end;

				* rescale predictors for stability;
				/*
				data &ts1;
					set &ts1;
					if _n_ = 1 then do;
						uss1 = uss(of
							%do i=1 %to &nixl; %sysfunc(varname(&_dsixl_,&i)) %end; 
							%do i=1 %to &nipl; %sysfunc(varname(&_dsipl_,&i)) %end; 
							%do i=1 %to &nix; %sysfunc(varname(&_dsix_,&i)) %end; 
							%do i=1 %to &nip; %sysfunc(varname(&_dsip_,&i)) %end; 
						);
						sqrtuss1 = sqrt(uss1);
						call symputx("uss1",uss1);
						call symputx("sqrtuss1",sqrtuss1);
						retain sqrtuss1;
					end;
					%do i=1 %to &nixl; %sysfunc(varname(&_dsixl_,&i)) = %sysfunc(varname(&_dsixl_,&i))/sqrtuss1; %end; 
					%do i=1 %to &nipl; %sysfunc(varname(&_dsipl_,&i)) = %sysfunc(varname(&_dsipl_,&i))/sqrtuss1; %end; 
					%do i=1 %to &nix; %sysfunc(varname(&_dsix_,&i)) = %sysfunc(varname(&_dsix_,&i))/sqrtuss1; %end; 
					%do i=1 %to &nip; %sysfunc(varname(&_dsip_,&i)) = %sysfunc(varname(&_dsip_,&i))/sqrtuss1; %end; 
					drop uss1 sqrtuss1;
				run;

				%put uss1 = &uss1;
				%put sqrtuss1 = &sqrtuss1;
				*/
				%let uss1 = 1;

				* use only the training data for knots;
				%if &CVFlag or (&TestSpec=Yes) %then %do;
					data ts1o;
						set &ts1;
						where not missing(_y);
					run;
					%let kdata= ts1o;
				%end;
				%else %do;
					%let kdata = &ts1;
				%end;
				
				/*
				data OutLib.ts1;
					set &ts1;
				run;
				*/

				proc glimmix data=&ts1 &ProcOption;
					%if %length(&ClassVariables) %then %do;
						class &ClassVariables;
					%end;
					model &UseDep.&DepOption = &FixedEffects / &ModOption solution;
					%if ^%index(%quote(&SelVars),Intercept) %then %do;
						random 
							%if %index(&VarSelect,Genetic) %then %do;
								&SelVars
							%end;
							%else %do;
								%do i=1 %to &nixl; %sysfunc(varname(&_dsixl_,&i)) %end; 
								%do i=1 %to &nipl; %sysfunc(varname(&_dsipl_,&i)) %end; 
								%do i=1 %to &nix; %sysfunc(varname(&_dsix_,&i)) %end; 
								%do i=1 %to &nip; %sysfunc(varname(&_dsip_,&i)) %end; 
							%end;
							/ type=rsmooth knotmethod=data(&kdata) 
							/* vi option takes way longer so use v and compute vi in iml */
							%if ^&CVFlag or &NPred %then %do;
								v
							%end;
							/*
							_obs_ / type=sp(gau)(
								%do i=1 %to &nixl; %sysfunc(varname(&_dsixl_,&i)) %end; 
								%do i=1 %to &nipl; %sysfunc(varname(&_dsipl_,&i)) %end; 
								%do i=1 %to &nix; %sysfunc(varname(&_dsix_,&i)) %end; 
								%do i=1 %to &nip; %sysfunc(varname(&_dsip_,&i)) %end; 
							)
							*/
							%if (&TraitType=Nominal) or (&TraitType=Ordinal) %then %do;
								group=&UseDep
							%end;	
						;
					%end;
					%if %length(&WeightVar) %then %do;
						weight &WeightVar;
					%end;
					%else %if (&Priors^=Proportional) %then %do;
						weight _Prior;
					%end;
					/* random _residual_ / group=&UseDep; */
					&ProcGlimmixStatements;
					output out=ts2o(drop=
						%do i=1 %to &nixl; %sysfunc(varname(&_dsixl_,&i)) %end; 
						%do i=1 %to &nipl; %sysfunc(varname(&_dsipl_,&i)) %end; 
						%do i=1 %to &nix; %sysfunc(varname(&_dsix_,&i)) %end; 
						%do i=1 %to &nip; %sysfunc(varname(&_dsip_,&i)) %end; 
						) 
						pred(blup ilink)=pred					
						pred(noblup ilink)=xb
					;
					ods output parameterestimates=pe;
					%if ^&CVFlag or &NPred %then %do;
						ods output v=v covparms=cp;
						ods exclude v;
					%end;
					%if (&TraitType=Ordinal) %then %do;
						ods output responseprofile=rp;
					%end;
				run;

				/*
				data OutLib.ts2o0;
					set ts2o;
				run;
				*/
				/*
				data OutLib.v;
					set v;
				run;
				*/

				%put syserr = &syserr;
				%if &syserr > 4 %then %do;
					%let exiterror = 1;
					%goto exit;
				%end;

				* compute BLUPs directly if necessary;
				%if (^&CVFlag or &NPred) and ^%index(%quote(&SelVars),Intercept) %then %do;
					/*
					data ts2o_miss;
						set ts2o;
						if missing(&UseDep);
					run;
					*/
					data ts2o_nomiss;
						set ts2o;
						if not missing(&UseDep);
						resid = &UseDep - xb;
					run;
					data d_nomiss;
						set &ts1;
						if not missing(&UseDep); 				
					run;
					/*
					data _null_;
						set pe;
						call symputx("Formula_Intercept",trim(left(Estimate)));
					run;
					%put Formula_Intercept = &Formula_Intercept;
					*/
					%NObsVars(v);
					%let n = &nobs;
					%let p = %eval(&nixl + &nipl + &nix + &nip);
					proc iml;
						use cp;
						read all var {estimate} into cp;
						* print cp;
						use v;
						read all var {%do i = 1 %to &n; col&i %end;} into v;
						use ts2o_nomiss;
						read all var {resid} into resid;
						use d_nomiss;
						read all var {						
							%do i=1 %to &nixl; %sysfunc(varname(&_dsixl_,&i)) %end; 
							%do i=1 %to &nipl; %sysfunc(varname(&_dsipl_,&i)) %end; 
							%do i=1 %to &nix; %sysfunc(varname(&_dsix_,&i)) %end; 
							%do i=1 %to &nip; %sysfunc(varname(&_dsip_,&i)) %end; 
							} into z;
						vi = inv(v);
						u = cp[1]*z`*vi*resid/&uss1;
						create blup from u;
						append from u;
						/*
						%if &NPred %then %do;
							create blup from u;
							append from u;
						%end;
						%else %do;
							use ts2o;
							read all var {xb} into xb;
							use &ts1;
							read all var {						
								%do i=1 %to &nixl; %sysfunc(varname(&_dsixl_,&i)) %end; 
								%do i=1 %to &nipl; %sysfunc(varname(&_dsipl_,&i)) %end; 
								%do i=1 %to &nix; %sysfunc(varname(&_dsix_,&i)) %end; 
								%do i=1 %to &nip; %sysfunc(varname(&_dsip_,&i)) %end; 
								} into z;
							pred = xb + z*u;
							create pred from pred;
							append from pred;
						%end;

						%put syserr = &syserr;
						%if &syserr > 4 %then %do;
							%let exiterror = 1;
							%goto exit;
						%end;
						*/
					quit;

					/*
					data OutLib.blup_rbm;
						set blup;
					run;
					*/

					%if &NPred %then %do;
						data names;
							length _name_ $32;
							%do i=1 %to &nixl; _name_ = "%sysfunc(varname(&_dsixl_,&i))"; output; %end; 
							%do i=1 %to &nipl; _name_ = "%sysfunc(varname(&_dsipl_,&i))"; output;  %end; 
							%do i=1 %to &nix; _name_ = "%sysfunc(varname(&_dsix_,&i))"; output;  %end; 
							%do i=1 %to &nip; _name_ = "%sysfunc(varname(&_dsip_,&i))"; output;  %end; 
						run;

						data blup;
							merge names blup;
							blup = col1;
							abs_blup = abs(col1);
							drop col1;
						run;

						%if &nixl or &nipl %then %do;
							data blup_lock;
								set blup(obs=%eval(&nixl+&nipl));
							run;
							data blup;
								set blup(firstobs=%eval(&nixl+&nipl+1));
							run;
						%end;

						proc sort data=blup;
							by descending abs_blup;
						run;

						%NObsVars(blup);
						%if &NPred > &nobs %then %do;
							%put Reducing number of predictors from &NPred to &nobs because there are no more.;
							%let NPred = &nobs;
						%end;

						data ts2vars;
							set blup(obs=&NPred);
						run;

						%if &nixl or &nipl %then %do;
							data ts2vars;
								set blup_lock ts2vars;
							run;
						%end;

						proc transpose data=ts2vars out=_dsib_(drop=_name_);
							var abs_blup;
						run;

						%let _dsib_ = %sysfunc(open(_dsib_));

						data ts2vars;
							set ts2vars;
							rename _name_ = name;
						run;
						%let havets2v = 1;

						* rescale predictors again using only reduced set;
						/*
						data &ts1;
							set &ts1;
							if _n_ = 1 then do;
								uss1 = uss(of
									%do i=1 %to &nixl; %sysfunc(varname(&_dsixl_,&i)) %end; 
									%do i=1 %to &nipl; %sysfunc(varname(&_dsipl_,&i)) %end; 
									%do i=1 %to &NPred; %sysfunc(varname(&_dsib_,&i)) %end; 
								);
								call symputx("uss1",uss1);
								sqrtuss1 = sqrt(uss1);
								retain sqrtuss1;
							end;
							%do i=1 %to &nixl; %sysfunc(varname(&_dsixl_,&i)) = %sysfunc(varname(&_dsixl_,&i))/sqrtuss1; %end; 
							%do i=1 %to &nipl; %sysfunc(varname(&_dsipl_,&i)) = %sysfunc(varname(&_dsipl_,&i))/sqrtuss1; %end; 
							%do i=1 %to &NPred; %sysfunc(varname(&_dsib_,&i)) = %sysfunc(varname(&_dsib_,&i))/sqrtuss1; %end; 
						run;
						*/

						%put uss1 reduced = &uss1;

						/*
						data OutLib.ts1r;
							set &ts1;
						run;
						*/

						proc glimmix data=&ts1 &ProcOption;
							/* class _obs_; */
							model &UseDep.&DepOption = &FixedEffects / &ModOption solution;
							random 
								%do i=1 %to &nixl; %sysfunc(varname(&_dsixl_,&i)) %end; 
								%do i=1 %to &nipl; %sysfunc(varname(&_dsipl_,&i)) %end; 
								%do i=1 %to &NPred; %sysfunc(varname(&_dsib_,&i)) %end; 
								/ type=rsmooth knotmethod=data(&kdata) 
								%if ^&CVFlag %then %do;
									v
								%end;
								%if (&TraitType=Nominal) or (&TraitType=Ordinal) %then %do;
									group=&UseDep
								%end;	
							;
							%if %length(&WeightVar) %then %do;
								weight &WeightVar;
							%end;
							%else %if (&Priors^=Proportional) %then %do;
								weight _Prior;
							%end;
							&ProcGlimmixStatements;
							output out=ts2o(drop=
								%do i=1 %to &nixl; %sysfunc(varname(&_dsixl_,&i)) %end; 
								%do i=1 %to &nipl; %sysfunc(varname(&_dsipl_,&i)) %end; 
								%do i=1 %to &nix; %sysfunc(varname(&_dsix_,&i)) %end; 
								%do i=1 %to &nip; %sysfunc(varname(&_dsip_,&i)) %end; 
								) 
								pred(blup ilink)=pred
								pred(noblup ilink)=xb
							;
							ods output parameterestimates=pe;
							%if ^&CVFlag %then %do;
								ods output v=v covparms=cp1;
								ods exclude v;
							%end;
							%if (&TraitType=Ordinal) %then %do;
								ods output responseprofile=rp;
							%end;
						run;

						%put syserr = &syserr;
						%if &syserr > 4 %then %do;
							%let exiterror = 1;
							%goto exit;
						%end;

						* compute BLUPs again directly if necessary;
						%if ^&CVFlag %then %do;
							/*
							data ts2o_miss;
								set ts2o;
								if missing(&UseDep);
							run;
							*/
							data ts2o_nomiss;
								set ts2o;
								if not missing(&UseDep);
								resid = &UseDep - xb;
							run;
							data d_nomiss;
								set &ts1;
								if not missing(&UseDep);
							run;
							/*
							data _null_;
								set pe;
								call symputx("Formula_Intercept",trim(left(Estimate)));
							run;
							%put Reduced Formula_Intercept = &Formula_Intercept;
							*/
							%NObsVars(v);
							%let n = &nobs;
							%let p = &NPred;
							proc iml;
								use cp1;
								read all var {estimate} into cp;
								* print cp;
								use v;
								read all var {%do i = 1 %to &n; col&i %end;} into v;
								use ts2o_nomiss;
								read all var {resid} into resid;
								use d_nomiss;
								read all var {						
									%do i=1 %to &nixl; %sysfunc(varname(&_dsixl_,&i)) %end; 
									%do i=1 %to &nipl; %sysfunc(varname(&_dsipl_,&i)) %end; 
									%do i=1 %to &NPred; %sysfunc(varname(&_dsib_,&i)) %end;								
								} into z;
								vi = inv(v);
								u = cp[1]*z`*vi*resid/&uss1;

								create blup from u;
								append from u;

								/*
								use ts2o;
								read all var {pred} into xb;
								use &ts1;
								read all var {						
									%do i=1 %to &nixl; %sysfunc(varname(&_dsixl_,&i)) %end; 
									%do i=1 %to &nipl; %sysfunc(varname(&_dsipl_,&i)) %end; 
									%do i=1 %to &NPred; %sysfunc(varname(&_dsib_,&i)) %end;								
								} into z;
								pred = xb + z*u;

								create pred from pred;
								append from pred;
								*/

								%put syserr = &syserr;
								%if &syserr > 4 %then %do;
									%let exiterror = 1;
									%goto exit;
								%end;
							quit;

							/*
							data ts2o;
								merge ts2o pred;
								pred = col1;
								resid = &UseDep - col1;
								drop col1;
							run;
							*/

							/*
							data ts2o;
								set ts2o ts2o_miss;
							run;
							*/

							data names;
								length _name_ $32;
								%do i=1 %to &nixl; _name_ = "%sysfunc(varname(&_dsixl_,&i))"; output; %end; 
								%do i=1 %to &nipl; _name_ = "%sysfunc(varname(&_dsipl_,&i))"; output;  %end; 
								%do i=1 %to &NPred; _name_ = "%sysfunc(varname(&_dsib_,&i))"; output; %end;								
							run;

							data blup;
								merge names blup;
								blup = col1;
								abs_blup = abs(col1);
								drop col1;
							run;

							proc sort data=blup;
								by descending abs_blup;
							run;

							/*
							data OutLib.blup_reduced_rbm;
								set blup;
							run;
							*/

							data ts2vars;
								set blup;
								rename _name_ = name;
							run;

						%end;

						%let _dsib_ = %sysfunc(close(&_dsib_));
					%end;
					%else %do;
						/*
						data ts2o;
							merge ts2o pred;
							pred = col1;
							resid = &UseDep - col1;
							drop col1;
						run;

						data OutLib.ts2o1;
							set ts2o;
						run;
						*/
						/*
						data ts2o;
							set ts2o ts2o_miss;
						run;
						*/
					%end;
				%end;

				%NObsVars(ts2o);
				%if &nobs = 0 %then %do;
					%put ERROR: PROC GLIMMIX did not converge or produce predictions.;
					%put If cross-validating, you may have constant data.  If not, please ;
					%put inspect the SAS Output and try specifying convergence options or a different model.;
					%let exiterror = 1;
					/* %goto exit; */
				%end;	
				%else %do;
					%put RBM TraitType = &TraitType;
					%if (&TraitType=Nominal) or (&TraitType=Ordinal) %then %do;

						%if (&TraitType=Ordinal) %then %do;

							%NObsVars(rp);
							%let nlev = &nobs;

							* load levels into macro variables;
							data _null_;
								set rp;
								call symput('lev_'||trim(left(_n_)),trim(left(&DepVar)));
							run;

						%end;

						data ts2o;
							set ts2o;
							keep _Obs _Level_ pred;
						run;

						proc transpose data=ts2o out=justpred(drop=_name_ _label_) prefix=p_;
							by _Obs;
							var pred;
							id _level_;
						run;

						%if (&TraitType=Nominal) %then %do;
							proc transpose data=ts2o out=justlev(drop=_name_ _label_) prefix=lev_;
								by _Obs;
								var _level_;
							run;
						%end;
						%else %if (&TraitType=Ordinal) %then %do;
							* for binary and ordinal, convert cumulative probabilities to class probabilities and fill in last level;
							* transpose twice to get rid of special characters and ensure valid SAS names;
							proc transpose data=rp out=rpt prefix=lev_;
								var &DepVar;
							run;
							proc transpose data=rpt out=rptt;
								var lev_:;
							run;

							* load sasified levels into macro variables;
							data _null_;
								set rp;
								call symput('levsas_'||trim(left(_n_)),trim(left(&DepVar)));
							run;					

							data justpred;
								set justpred;
								array pp p_:;
								p_&&levsas_&nlev = 1 - pp[dim(pp)];
								do i=dim(pp) to 2 by - 1;
									pp[i] = pp[i] - pp[i-1];
								end;
							run;

							%NObsVars(&ts1);

							data justlev;
								set rpt;
								do _Obs = 1 to &nobs;
									output;
								end;
								drop _name_;
							run;
						%end;

						data ts2o;
							merge &ts1 justpred justlev;
							by _Obs;
						run;
					%end;

					%put RBM TruthVar = &TruthVar;

					%if &AsymmetricLossFitProp ne 0.5 %then %do;
						data ts1a;
							set ts2o;
							&DepVar = &UseDep;
							keep _Row_ &DepVar pred &WeightVar &WeightVar0;
						run;
					%end;
					
					/*data OutLib.rbm_ts2o1; set ts2o; run;*/

					* create variables for computing cv ase;
					data ts2o;
						set ts2o;
						%if &CVFlag %then %do;
							%if &_dtype_ = C %then %do;
								where &UseDep = " ";
							%end;
							%else %do;
								where &UseDep = .;
							%end;
							&DepVar = _y0;
							drop _y0 _y;
						%end;
						%if (&TraitType=Binary) %then %do;
							if missing(pred) then do;
								p_&DepVarTrunc._0 = .;
								p_&DepVarTrunc._1 = .;
								_INTO_ = .;
							end;
							else do;
								p_&DepVarTrunc._0 = 1 - pred;
								if p_&DepVarTrunc._0 < 0 then p_&DepVarTrunc._0 = 0;
								else if p_&DepVarTrunc._0 > 1 then p_&DepVarTrunc._0 = 1;
								p_&DepVarTrunc._1 = 1 - p_&DepVarTrunc._0;
								_INTO_ = (p_&DepVarTrunc._1 > 0.5);
							end;
							%if &_dtype_ = C %then %do;
								if &TruthVar = " " then
							%end;
							%else %do;
								if &TruthVar = . then
							%end;
								actualp = .;
							else if (&TruthVar=1) then actualp = p_&DepVarTrunc._1;
							else actualp = p_&DepVarTrunc._0;
							/*
							%let ncl = 2;
							%let pprob1 = p_&DepVarTrunc._0;
							%let pprob2 = p_&DepVarTrunc._1;
							*/
						%end;
						%else %if (&TraitType=Nominal) or (&TraitType=Ordinal) %then %do;
							array pp p_:;
							array lev lev_:;
							maxp = 0;
							maxpi = 0;
							levi = .;
							do i=1 to dim(pp);
								if (pp[i] > maxp) then do;
									maxp = pp[i];
									maxpi = i;
								end;
								if (lev[i] = &TruthVar) then levi = i;
							end;
							_INTO_ = lev[maxpi];
							if not missing(levi) then actualp = pp[levi];
							else actualp = .;
							* drop lev_: i maxp maxpi levi;
						%end;
					run;
					
					/*data OutLib.rbm_ts2o2; set ts2o; run;*/

				%end;
				%let into = _INTO_;
				%let pred = pred;
			%end;
			%else %if (&ProcessName = RidgeRegression) %then %do;
				
				%*goto exit;				

				* string for analysis settings;
				%if %length(&ASet) %then %do;
					%let ASet = &ASet, Kernel = &Kernel;
				%end;
				%else %do;
					%let ASet = Kernel = &Kernel;				
				%end;
				%if &aloss_niter=1 and &NPred %then %do;
					%if %length(&ASet) %then %do;
						%let trTopNPredBLUPs = %nrbquote(%LocaleFill(%nrbquote(&trTopXBLUPs), &NPred));
						%let ASet = &ASet, &trTopNPredBLUPs &BLUPWhere;
					%end;
					%else %do;
						%let trTopNPredBLUPs = %nrbquote(%LocaleFill(%nrbquote(&trTopXBLUPs), &NPred));
						%let ASet = &trTopNPredBLUPs &BLUPWhere;
					%end;
				%end;
				
				/*data OutLib.rr_ts1; set &ts1; run;*/
				
				* remove rows with missing predictors;
				/*
				%put LockContPredVarsRR= &LockContPredVars;
				%put LockClassPredVarsRR= &LockClassPredVars;
				%put ClassPredVarsRR= &ClassPredVars;
				%if ^&upred %then %do; 
					%put PredictorVarsRR= &PredictorVars;
				%end; 
				%else %do; 
					%do j=1 %to &nix_blocks; 
						%put PredictorVarsRR&j= &&PredictorVars&j; 
					%end; 
				%end;
				*/
				
				data d_nomisspred;
					set &ts1;
					%if %length(%trim(&LockContPredVars &LockClassPredVars &PredictorVars &ClassPredVars)) %then %do;
						array pv{*} &LockContPredVars &LockClassPredVars %if ^&upred %then %do; &PredictorVars %end; %else %do; %do j=1 %to &nix_blocks; &&PredictorVars&j %end; %end; &ClassPredVars;
						do _i_=1 to dim(pv);
							if missing(pv[_i_]) then do;
								delete;
								leave;
							end;
						end;
					%end;
					%if %length(&WeightVar) %then %do;
						if missing(&WeightVar) or &WeightVar = 0 then delete;
					%end;
					;
				run;

				/*
				data d_nomisspred;
					set &ts1;
					if not missing(1) 
						%do i=1 %to &nixl; and not missing(%sysfunc(varname(&_dsixl_,&i))) %end; 
						%do i=1 %to &nipl; and not missing(%sysfunc(varname(&_dsipl_,&i))) %end; 
						%do i=1 %to &nix; and not missing(%sysfunc(varname(&_dsix_,&i))) %end; 
						%do i=1 %to &nip; and not missing(%sysfunc(varname(&_dsip_,&i))) %end; 
						%if %length(&WeightVar) %then %do;
							and not missing(&WeightVar) and &WeightVar > 0
						%end;
					;
				run;
				*/
				
				%NobsVars(d_nomisspred);
				%if &nobs = 0 %then %do;
      					%put ERROR: There are no observations in the analysis data set after removing predictors with missing values.;
      					%let exiterror=1;
      					%goto exit;
   				%end;				
				
				%let ts1 = d_nomisspred;
				%put ts1 = &ts1;

				%if ^&CVFlag and (&TestSpec = Yes) and (&Kernel^=Dot) %then %do;
					data b2;
						set d d_test;
					run;
					%let ts2 = b2;
				%end;

				* using missing values for the dependent variable to create predictions;
				%if &CVFlag or (&TestSpec=Yes) or (&Kernel^=Dot) %then %do;
					data &ts1;
						set &ts1;
						_y0 = &DepVar;
						_y = &DepVar;
					run;
					data ts1o;
						set &ts1;
					run;
					data &ts2;
						set &ts2;
						_y0 = &DepVar;
						%if (&_dtype_=C) %then %do;
							_y = " ";
						%end;
						%else %do;
							_y = .;
						%end;
					run;
					data &ts1;
						set &ts1 &ts2;
						_obs = _n_;
						_one = 1;
						_name_ = "";
						drop _name_;
					run;
					%let UseDep = _y;
				%end;
				%else %do;
					%let UseDep = &DepVar;
					data &ts1;
						set &ts1;
						_obs = _n_;
						_one = 1;
						_name_ = "";
						drop _name_;
					run;
				%end;

				data d_nomiss;
					set &ts1;
					if not missing(&UseDep);
				run;

				%NObsVars(d_nomiss);
				%let nnmiss = &nobs;
				%put nnmiss = &nnmiss;

				%let uss1 = 1;
				/*
				%let uss1 = &npv;
				*/
				%put uss1 = &uss1;
				%let ts1prev = &ts1;

				/*
				%let nnmiss = 0;
				*/

				* set up to use repeated statement with ldata if there are more predictors than nonmissing observations;
				%if (&Kernel = Dot and (&npv >= &nnmiss)) or (&Kernel = Dot1) or (&Kernel = Linear2) %then %do;
					%put setting up to use repeated statement;
					
					/*data OutLib.rr_ts1; set &ts1; run;*/

					* compute sscp matrix for use with type=lin(1);
					proc transpose data=&ts1 out=dt;
						var &LockContPredVars &LockClassPredVars %if ^&upred %then %do; &PredictorVars %end; %else %do; %do j=1 %to &nix_blocks; &&PredictorVars&j %end; %end; &ClassPredVars
							/*
							%do i=1 %to &nixl; %sysfunc(varname(&_dsixl_,&i)) %end; 
							%do i=1 %to &nipl; %sysfunc(varname(&_dsipl_,&i)) %end; 
							%do i=1 %to &nix; %sysfunc(varname(&_dsix_,&i)) %end; 
							%do i=1 %to &nip; %sysfunc(varname(&_dsip_,&i)) %end; 
							*/
						;
					run;
					
					
					/*data OutLib.dt; set dt; run;*/
					
						
					/*  use svd decomposition approximation
					data dt1;
						set dt;
						drop _name_;
					run;
					
					data OutLib.dt1;
						set dt1;
					run;
					
					%svd(work,dt1,out_v,out_s,out_u,col:,_name_);

					data OutLib.out_v;
						set out_v;
					run;
					data OutLib.out_s;
						set out_s;
					run;
					data OutLib.out_u;
						set out_u;
					run;
					
					%let na = 100;
					
					proc iml;
						use OutLib.out_s;
						read all var {singularvalue} into s;
						sdiag = diag(s);
						print (sdiag[1:5,1:5]);
						use OutLib.out_v;
						read all var _num_ into v;
						print (v[1:5,1:5]);
						aidx = (1:&na);
						covmata = v[,aidx]*sdiag[aidx,aidx]*sdiag[aidx,aidx]*v[,aidx]`;
						create covmata from covmata;
						append from covmata;
						sqrtz = v*sdiag;
						create sqrtz from sqrtz;
						append from sqrtz;
					run;
					quit;
					
					data covmat;
						set covmata;
						parm = 1;
						row = _n_;
					run;
					
					data OutLib.sqrtz;
						set sqrtz;
					run;
					*/

					%if &Kernel = Dot or &Kernel = Dot1 %then %do;
						proc corr data=dt sscp out=sscp noprint;
							var col:;
						run;

						data covmat;
							set sscp;
							where _type_="SSCP" and _name_ ne "Intercept";
							* if _type_="SSCP" and _name_ = "COL1" then call symputx("uss1",col1);
							keep col:;
						run;

						data covmat;
							set covmat;
							array col col:;
							do i=1 to dim(col);
								col[i] = col[i]/&uss1;
							end;
							parm = 1;
							row = _n_;
							drop i;
						run;
						
						%if &Kernel = Dot1 %then %do;
							data covmat1;
								set covmat;
								array col col:;
								do i=1 to dim(col);
									col[i] = 1;
								end;
								parm = 2;
								drop i;
							run;
							
							data covmat;
								set covmat covmat1;
							run;
							
							%let ldn = 2;
						%end;
						%else %do;
							%let ldn = 1;
						%end;
					%end;
					%else %if &Kernel = Linear2 %then %do;
						proc corr data=dt outp=corr noprint;
							var col:;
						run;

						data covmat;
							set corr;
							where _type_="CORR";
							keep col:;
						run;

						* convert to dcorr distance matrix;
						* could have alternatively called PROC DISTANCE;
						data covmat;
							set covmat;
							array col col:;
							do i=1 to dim(col);
								col[i] = 1 - col[i];
							end;
							parm = 1;
							row = _n_;
							drop i;
						run;
						
						data covmat1;
							set covmat;
							array col col:;
							do i=1 to dim(col);
								col[i] = 1;
							end;
							parm = 2;
							drop i;
						run;
						
						data covmat;
							set covmat covmat1;
						run;
						
						%let ldn = 2;
					%end;
					
					/*
					data covmat2;
						set covmat;
						array col col:;
						do i=1 to dim(col);
							col[i] = 1;
						end;
						parm = 2;
						row = _n_;
						drop i;
					run;
					
					data covmat;
						set covmat covmat2;
					run;
					*/ 

					/*
					libname g "\\epi\projects\SusanShao\Genomics Selection";
					data covmat;
						set g.pheno_snp_rename_ibd;
						parm = 1;
						row = _n_;
						drop i;
						keep col: parm row;
					run;
					%let uss1 = 1;
					*/

					/*
					data OutLib.covmat;
						set covmat;
					run;
					*/
					
					proc transpose data=d_nomiss out=dt_nomiss;
						var &LockContPredVars &LockClassPredVars %if ^&upred %then %do; &PredictorVars %end; %else %do; %do j=1 %to &nix_blocks; &&PredictorVars&j %end; %end; &ClassPredVars;
							/*
							%do i=1 %to &nixl; %sysfunc(varname(&_dsixl_,&i)) %end; 
							%do i=1 %to &nipl; %sysfunc(varname(&_dsipl_,&i)) %end; 
							%do i=1 %to &nix; %sysfunc(varname(&_dsix_,&i)) %end; 
							%do i=1 %to &nip; %sysfunc(varname(&_dsip_,&i)) %end; 
							*/
						;
					run;

					data d_drop;
						set &ts1;
						drop &LockContPredVars &LockClassPredVars %if ^&upred %then %do; &PredictorVars %end; %else %do; %do j=1 %to &nix_blocks; &&PredictorVars&j %end; %end; &ClassPredVars;
						/*
						drop
							%do i=1 %to &nixl; %sysfunc(varname(&_dsixl_,&i)) %end; 
							%do i=1 %to &nipl; %sysfunc(varname(&_dsipl_,&i)) %end; 
							%do i=1 %to &nix; %sysfunc(varname(&_dsix_,&i)) %end; 
							%do i=1 %to &nip; %sysfunc(varname(&_dsip_,&i)) %end;
						;
						*/
					run;

					%let ts1 = d_drop;	

					/*
					data OutLib.d_drop;
						set d_drop;
					run;
					*/
				%end;
				%else %do;
					/* 
					* rescale predictors by sqrt(uss1);
					data &ts1;
						set &ts1;
						array pv{*} 
							%do i=1 %to &nixl; %sysfunc(varname(&_dsixl_,&i)) %end; 
							%do i=1 %to &nipl; %sysfunc(varname(&_dsipl_,&i)) %end; 
							%do i=1 %to &nix; %sysfunc(varname(&_dsix_,&i)) %end; 
							%do i=1 %to &nip; %sysfunc(varname(&_dsip_,&i)) %end; 
						;
						do _i_=1 to dim(pv);
							pv[_i_] = pv[_i_]/sqrt(&uss1);
						end;
						drop _i_;
					run;
					*/
				%end;

				%let DataDistribution = Gaussian;
				%if &DataDistribution = Binary %then %do;
					%let WhichProc = GLIMMIX;
				%end;
				%else %do;
					%let WhichProc = MIXED;
				%end;

				/*
				%let WhichProc = GLIMMIX;
				%let DataDistribution = Multinomial;
				%let ModOption = %str(link=cumlogit);
				*/
				/*
				%let ModOption = %str(link=cumlogit);
				*/

				%put ProcOption = &ProcOption;
				%put ClassVariables = &ClassVariables;
				%put UseDep = &UseDep;
				%put FixedEffects = &FixedEffects;
				%put ModOption = &ModOption;
				%put OrderInt = &OrderInt;
				%put HetVar = &HetVar;
				%put ProcMixedStatements = &ProcMixedStatements;
				%put WeightVar = &WeightVar;
				%put uss1 = &uss1;
				%put nnmiss = &nnmiss;
				%if &Kernel = Dot %then %do;
					%if (&npv < &nnmiss) %then %do;
						%put npv is less than nnmiss, so use random statement;
					%end;
					%else %do;
						%put npv is greater than or equal to nnmiss, so use repeated statement;
					%end;
				%end;
				
				/*data OutLib.rr_ts1; set &ts1; run;*/
				
				%*put nixlrr= &nixl niplrr= &nipl nixrr= &nix niprr= &nip;

				proc &WhichProc data=&ts1 &ProcOption namelen=128;
					class &ClassVariables 
						%if &Kernel ^= Dot or ((&npv >= &nnmiss) and (&OrderInt = 1)) %then %do; _obs %end; 
					;
					%if %index(&WhichProc,MIXED) %then %do;
						model &UseDep = &FixedEffects / &ModOption solution
							outp=ts2o
							%if (&npv < &nnmiss) or &Kernel ^= Dot %then %do;
								(drop=&LockContPredVars &LockClassPredVars 
									%if ^&upred %then %do; 
										&PredictorVars 
									%end; 
									%else %do; 
										%do j=1 %to &nix_blocks; 
											&&PredictorVars&j 
										%end; 
									%end; 
									&ClassPredVars
									/*
									%do i=1 %to &nixl; %sysfunc(varname(&_dsixl_,&i)) %end; 
									%do i=1 %to &nipl; %sysfunc(varname(&_dsipl_,&i)) %end; 
									%do i=1 %to &nix; %sysfunc(varname(&_dsix_,&i)) %end; 
									%do i=1 %to &nip; %sysfunc(varname(&_dsip_,&i)) %end; 
									*/
								)
							%end;
						;
					%end;
					%else %do;
						model &UseDep(event=last) = &FixedEffects / &ModOption
							dist=&DataDistribution
						;
					%end;
					%if ^%index(%quote(&SelVars),Intercept) %then %do;
						%if &Kernel = Dot and ((&npv < &nnmiss) or (&OrderInt > 1)) %then %do;
							random 
								%do i=1 %to &nixl; 
									%sysfunc(varname(&_dsixl_,&i)) 
									%if &OrderInt > 1 %then %do;
										%do j = 2 %to &OrderInt; | %sysfunc(varname(&_dsixl_,&i)) %end;
										%if ((&i < &nixl) or (&nipl > 0) or (&nix > 0) or (&nip > 0)) %then %do; | %end;
									%end;
								%end; 
								%do i=1 %to &nipl; 
									%sysfunc(varname(&_dsipl_,&i)) 
									%if &OrderInt > 1 and ((&i < &nipl) or (&nix > 0) or (&nip > 0)) %then %do; | %end;
								%end; 
								%do i=1 %to &nix; 
									%sysfunc(varname(&_dsix_,&i)) 
									%if &OrderInt > 1 %then %do;
										%do j = 2 %to &OrderInt; | %sysfunc(varname(&_dsix_,&i)) %end;
										%if ((&i < &nix) or (&nip > 0)) %then %do; | %end;
									%end;
								%end; 
								%do i=1 %to &nip; 
									%sysfunc(varname(&_dsip_,&i)) 
									%if &OrderInt > 1 and (&i < &nip) %then %do; | %end;
								%end; 
								%if &OrderInt > 1 %then %do;
									@ &OrderInt
								%end;
								/ %if (&HetVar^=Yes) %then %do; type=toep(1) %end; s;
							ods output solutionr=blup;
							ods exclude solutionr;
							%if (&WhichProc = GLIMMIX) %then %do;
								output out=ts2o(drop=&LockContPredVars &LockClassPredVars %if ^&upred %then %do; &PredictorVars %end; %else %do; %do j=1 %to &nix_blocks; &&PredictorVars&j %end; %end; &ClassPredVars
									/*
									%do i=1 %to &nixl; %sysfunc(varname(&_dsixl_,&i)) %end; 
									%do i=1 %to &nipl; %sysfunc(varname(&_dsipl_,&i)) %end; 
									%do i=1 %to &nix; %sysfunc(varname(&_dsix_,&i)) %end; 
									%do i=1 %to &nip; %sysfunc(varname(&_dsip_,&i)) %end; 
									*/
									) pred(blup ilink)=pred residual(blup ilink)=resid;
							%end;
						%end;
						%else %do;
							%if %index(&WhichProc,MIXED) %then %do;
								* repeated statement can be faster than random for wide data but does not generate;
								* predictions on training data, so output inv(r) matrix and use iml below;
								%if &Kernel = Dot and (^&CVFlag or &NPred) %then %do;
									repeated _obs / type=lin(1) ldata=covmat subject=_one local ri;
									ods output invr=ri solutionf=sf covparms=cp;
									ods exclude invr;
								%end;
								%else %do;
									%if &Kernel = Dot or &Kernel = Dot1 or &Kernel = Linear2 %then %do;
										repeated _obs / type=lin(&ldn) ldata=covmat subject=_one local;
									%end;
									%else %do;
										repeated _obs / type=sp(&spkernel)(  
											%do i=1 %to &nixl; %sysfunc(varname(&_dsixl_,&i)) %end; 
											%do i=1 %to &nipl; %sysfunc(varname(&_dsipl_,&i)) %end; 
											%do i=1 %to &nix; %sysfunc(varname(&_dsix_,&i)) %end; 
											%do i=1 %to &nip; %sysfunc(varname(&_dsip_,&i)) %end; 
										) subject=_one local;									
									%end;
								%end;
							%end;
							%else %do;
								%if &Kernel = Dot and ^&CVFlag or &NPred %then %do;
									random _obs / type=lin(1) ldata=covmat subject=_one vi;
									ods output invv=ri covparms=cp;
									ods exclude invv;
								%end;
								%else %do;
									%if &Kernel = Dot or &Kernel = Dot1 or &Kernel = Linear2 %then %do;
										random _obs / type=lin(&ldn) ldata=covmat subject=_one local;
									%end;
									%else %do;
										random _obs / type=sp(&spkernel)(  
											%do i=1 %to &nixl; %sysfunc(varname(&_dsixl_,&i)) %end; 
											%do i=1 %to &nipl; %sysfunc(varname(&_dsipl_,&i)) %end; 
											%do i=1 %to &nix; %sysfunc(varname(&_dsix_,&i)) %end; 
											%do i=1 %to &nip; %sysfunc(varname(&_dsip_,&i)) %end; 
										) subject=_one local;									
									%end;
								%end;
								output out=ts2o pred(blup ilink)=pred residual(blup ilink)=resid pred(noblup ilink)=pred0 residual(noblup ilink)=resid0;
							%end;
							/*
							%if ^%index(%qupcase(&ProcMixedStatements),PARMS) and (&HetVar^=Yes) %then %do;
								parms / lowerb=1e-6,1e-6;
							%end;
							*/
						%end;
					%end;
					%if %length(&WeightVar) %then %do;
						weight &WeightVar;
					%end;
					%else %if (&Priors^=Proportional) %then %do;
						weight _Prior;
					%end;
					&ProcMixedStatements;
					ods output covparms=cp solutionf=sf;
					ods exclude classlevels;
				run;

				%put syserr = &syserr;
				%if &syserr > 4 %then %do;
					%let exiterror = 1;
					%goto exit;
				%end;

				%let ts1 = &ts1prev;

				%NObsVars(ts2o);
				%if ^&nobs %then %do;
					%put ERROR: PROC &WhichProc did not converge when fitting a model to response variable &DepVar..  You may want to add a PARMS statement or change the model.;
					%let exiterror = 1;
					%goto exit;
				%end;
				
				/*data OutLib.rr_ts2o0; set ts2o; run;*/
				/*data OutLib.rr_sf; set sf; run;*/
				
				%put UseDep = &UseDep;

				* change PROC GLIMMIX output data to match that of PROC MIXED;
				%if &WhichProc = GLIMMIX and (&npv >= &nnmiss) %then %do;
					data ts2o;
						set ts2o;
						if not missing(&UseDep) then do;
							pred = pred0;
							resid = resid0;
						end;
						* drop pred0 resid0;
					run;
				%end;

				* BLUP processing;
				%if &Kernel = Dot and (^&CVFlag or &NPred or %length(&BLUPWhere)) %then %do;
					/*
					data ts2o_miss;
						set ts2o;
						if missing(&UseDep);
					run;
					*/
					data ts2o_nomiss;
						set ts2o;
						if not missing(&UseDep);
						drop stderrpred df alpha lower upper;
					run;
					data _null_;
						set sf;
						where Effect = "Intercept";
						call symputx("Formula_Intercept",trim(left(Estimate)));
					run;
					%put Formula_Intercept = &Formula_Intercept;

					%if ^%index(%quote(&SelVars),Intercept) %then %do;

						* compute BLUPs directly if necessary;
						%if (&npv >= &nnmiss) %then %do;
							%NObsVars(ri);
							%let n = &nobs;
							data cp1;
								set cp;
								if covparm="LIN(1)" and subject="_one";
							run;
							proc iml;
								use cp1;
								read all var {estimate} into cp;
								* print cp;
								use ri;
								read all var {%do i = 1 %to &n; col&i %end;} into vi;
								use ts2o_nomiss;
								read all var {resid} into resid;
								use dt_nomiss;
								read all var _num_ into zt;

								u = cp[1]*zt*vi*resid;

								/*
								use d_nomiss;
								read all var {						
									%do i=1 %to &nixl; %sysfunc(varname(&_dsixl_,&i)) %end; 
									%do i=1 %to &nipl; %sysfunc(varname(&_dsipl_,&i)) %end; 
									%do i=1 %to &nix; %sysfunc(varname(&_dsix_,&i)) %end; 
									%do i=1 %to &nip; %sysfunc(varname(&_dsip_,&i)) %end; 
									} into z;
								u = cp[1]*z`*vi*resid;
								*/

								* u = cp[1]*z`*vi*resid/&uss1;
								create blup from u;
								append from u;
								
								%if ^&NPred %then %do;
									use ts2o;
									read all var {pred} into pm;
									use dt;
									read all var _num_ into zt;
									pred = pm + zt`*u;
									/*
									use &ts1;
									read all var {	
										%do i=1 %to &nixl; %sysfunc(varname(&_dsixl_,&i)) %end; 
										%do i=1 %to &nipl; %sysfunc(varname(&_dsipl_,&i)) %end; 
										%do i=1 %to &nix; %sysfunc(varname(&_dsix_,&i)) %end; 
										%do i=1 %to &nip; %sysfunc(varname(&_dsip_,&i)) %end; 
										} into z;
									pred = &Formula_Intercept + z*u;
									*/
									create pred from pred;
									append from pred;
								%end;

								%put syserr = &syserr;
								%if &syserr > 4 %then %do;
									%let exiterror = 1;
									%goto exit;
								%end;
							quit;

							data names;
								set dt;
								keep _name_;
							run;

							/*
							data names;
								length _name_ $32;
								%do i=1 %to &nixl; _name_ = "%sysfunc(varname(&_dsixl_,&i))"; output; %end; 
								%do i=1 %to &nipl; _name_ = "%sysfunc(varname(&_dsipl_,&i))"; output;  %end; 
								%do i=1 %to &nix; _name_ = "%sysfunc(varname(&_dsix_,&i))"; output;  %end; 
								%do i=1 %to &nip; _name_ = "%sysfunc(varname(&_dsip_,&i))"; output;  %end; 
							run;
							*/

							data blup;
								merge names blup;
								blup = col1;
								abs_blup = abs(col1);
								drop col1;
							run;
						%end;
						%else %do;
							data blup;
								set blup;
								blup = estimate;
								abs_blup = abs(estimate);
								rename estimate=blup effect=_name_;
							run;
						%end;
						
						%if %length(&BLUPWhere) %then %do;
							data blup;
								set blup;
								where %unquote(&BLUPWhere);
							run;
						
							%if ^&NPred %then %let NPred = 999999999999;
						%end;
						
						%let suffix =_rr_blup;
						%let b32 = %eval(32 - %length(&suffix));
						%if %length(&OutName) > &b32 %then %let inname = %substr(&OutName,1,&b32);
						%else %let inname = &OutName;		
						%let bout = %quote(&inname.%trim(%left(&suffix)));
						%put bout = &bout;

						data OutLib.%unquote(&bout);
							set blup;
						run;

						%if &NPred %then %do;

							%if &nixl or &nipl %then %do;
								data blup_lock;
									set blup(obs=%eval(&nixl+&nipl));
								run;
								data blup;
									set blup(firstobs=%eval(&nixl+&nipl+1));
								run;
							%end;

							proc sort data=blup;
								by descending abs_blup;
							run;

							/*
							proc sort data=blup;
								by probt;
							run;
							*/


							/*
							data ts2vars;
								set blup;
							run;

							%if &OrderInt > 1 %then %do;
								data ts2v;
									set ts2vars;
									i = 1;
									name = _name_;
									do while(lengthn(scan(name,i,"*")));
										_name_ = scan(name,i,"*");
										output;
										i = i+1;
									end;
									drop i name;
								run;
								proc sort data=ts2v nodupkey;
									by _name_;
								run;
								%let ts2v = ts2v;
							%end;
							%else %do;
								%let ts2v = ts2vars;
							%end;
							
							%NObsVars(&ts2v);
							%if &NPred > &nobs %then %do;
								%put Reducing number of predictors from &NPred to &nobs because there are no more.;
								%let NPred = &nobs;
							%end;
							%else %do;
								data &ts2v;
									set &ts2v(obs=&NPred);
								run;
							%end;
							*/
							
							%macro SASName2(inname,outname,unique=1,compbl=0);
									length name name2 $ 32;
									name = compress(trim(left(&inname)));
									len = lengthn(name);
									j=1;
									do until(j=0);
										j=notname(name,j+1);
										if j>0 then substr(name,j,1)=' ';
									end;
									%if &compbl %then %do;
										name = translate(compbl(name),'_','');
									%end;
									%else %do;
										name = translate(name,'_','');		
									%end;
									name = substr(name,1,len);
									if substr(name,1,1) in ('0','1','2','3','4','5','6','7','8','9') then do;
										name2='_'||name;
									end;
									else do;
									   name2=name;
									end;
									%if &unique %then %do;
										if length(name2) = 32 then do;
											name2 = substr(name2,1,24) || trim(left(_n_));
										end;
									%end;
									&outname = trim(left(name2));
							%mend;
							
							%NObsVars(blup);
							%if &nobs < &NPred %then %let NPred = &nobs;

							data blup;
								set blup(obs=&NPred);
								_label_ = _name_;
								%SASName2(_name_,_name_);
							run;
							
							/*
							data OutLib.blup2;
								set blup;
							run;
							*/
							
							proc transpose data=blup out=_dsib_(drop=_name_);
								var abs_blup;
							run;

							/*
							data OutLib.dsib;
								set _dsib_;
							run;
							*/

							%let _dsib_ = %sysfunc(open(_dsib_));

							%if &nixl or &nipl %then %do;
								data blup_lock;
									set blup_lock;
									rename _name_ = _label_;
								run;
							%end;

							data ts2vars;
								%if &nixl or &nipl %then %do;
									set blup_lock blup;
								%end;
								%else %do;
									set blup;
								%end;
								* keep _label_;
								rename _label_ = name;
							run;
							%let havets2v = 1;

							* re-standardize predictors row-wise so that sscp becomes corr matrix;
							%if (&StandardizeRows = Yes) and (&nixl or &nipl or &NPred) and (&OrderInt = 1) %then %do;
								data &ts1;
									set &ts1;
									array pv{*} 
										%do i=1 %to &nixl; %sysfunc(varname(&_dsixl_,&i)) %end; 
										%do i=1 %to &nipl; %sysfunc(varname(&_dsipl_,&i)) %end; 
										%do i=1 %to &NPred; %sysfunc(varname(&_dsib_,&i)) %end; 
									;
									_mean_ = mean(of pv{*});
									_std_ = std(of pv{*});
									do _i_=1 to dim(pv);
										pv[_i_] = (pv[_i_]-_mean_)/_std_;
									end;
									drop _i_;
								run;
							%end;

							%let uss1full = &uss1;
							%let npv = %eval(&nixl+&nipl+&NPred);
							%put npv reduced = &npv;
							%let uss1 = 1;
							/*
							%let uss1 = &npv;
							%let npv = 10000000;
							*/
							%put uss1 reduced = &uss1;

							/*
							data OutLib.ts1;
								set &ts1;
								keep &DepVar
									%do i=1 %to &nixl; %sysfunc(varname(&_dsixl_,&i)) %end; 
									%do i=1 %to &nipl; %sysfunc(varname(&_dsipl_,&i)) %end; 
									%do i=1 %to &NPred; %sysfunc(varname(&_dsib_,&i)) %end; 
								;
							run;
							*/

							* set up to use repeated statement below if necessary;
							%if (&npv >= &nnmiss) and (&OrderInt = 1) %then %do;

								* compute sscp matrix from reduced blup set for use with type=lin(1);
								proc transpose data=&ts1(keep=
										%do i=1 %to &nixl; %sysfunc(varname(&_dsixl_,&i)) %end; 
										%do i=1 %to &nipl; %sysfunc(varname(&_dsipl_,&i)) %end; 
										%do i=1 %to &NPred; %sysfunc(varname(&_dsib_,&i)) %end; 
									) out=dt;
									var 
										%do i=1 %to &nixl; %sysfunc(varname(&_dsixl_,&i)) %end; 
										%do i=1 %to &nipl; %sysfunc(varname(&_dsipl_,&i)) %end; 
										%do i=1 %to &NPred; %sysfunc(varname(&_dsib_,&i)) %end;
									;
								run;

								proc corr data=dt sscp out=sscp noprint;
									var col:;
								run;

								data covmat;
									set sscp;
									where _type_="SSCP" and _name_ ne "Intercept";
									* if _type_="SSCP" and _name_ = "COL1" then call symputx("uss1",col1);
									keep col:;
								run;

								data covmat;
									set covmat;
									array col col:;
									do i=1 to dim(col);
										col[i] = col[i]/&uss1;
									end;
									parm = 1;
									row = _n_;
									drop i;
								run;	

								%let ts1 = d_drop;

								/*
								data OutLib.covmat;
									set covmat;
								run;
								*/
							%end;
							%else %do;
								/*
								data &ts1;
									set &ts1;
									array pv{*} 
										%do i=1 %to &nixl; %sysfunc(varname(&_dsixl_,&i)) %end; 
										%do i=1 %to &nipl; %sysfunc(varname(&_dsipl_,&i)) %end; 
										%do i=1 %to &NPred; %sysfunc(varname(&_dsib_,&i)) %end; 
									;
									do _i_=1 to dim(pv);
										pv[_i_] = pv[_i_]/sqrt(&uss1/&uss1full);
									end;
									drop _i_;
								run;
								*/
							%end;

							%if ^%index(%qupcase(&ProcMixedStatements),PARMS) and (&HetVar^=Yes) %then %do;
								data cp;
									set cp;
									if covparm = "Residual" and Estimate < 1.1e-6 then Estimate = 1.1e-6;
								run;
								/*
								data OutLib.cp;
									set cp;
								run;
								*/
							%end;
							
							/*
							%put NPred = &NPred;
							%do i=1 %to &NPred; 
								%put blup &i = %sysfunc(varlabel(&_dsib_,&i));
							%end;
							*/

							* second proc mixed run using reduced predictor set;
							proc &WhichProc data=&ts1 &ProcOption namelen=128;						
								class &ClassVariables %if (&npv >= &nnmiss) and (&OrderInt = 1) %then %do; _obs %end; ;
								model &UseDep = &FixedEffects / &ModOption solution
									outp=ts2o
									%if (&npv < &nnmiss) %then %do;
										(drop=&LockContPredVars &LockClassPredVars 
											%if ^&upred %then %do; 
												&PredictorVars 
											%end; 
											%else %do; 
												%do j=1 %to &nix_blocks; 
													&&PredictorVars&j 
												%end; 
											%end; 
											&ClassPredVars
										)
											/*
											(drop = %do i=1 %to &nixl; %sysfunc(varname(&_dsixl_,&i)) %end; 
											%do i=1 %to &nipl; %sysfunc(varname(&_dsipl_,&i)) %end; 
											%do i=1 %to &nix; %sysfunc(varname(&_dsix_,&i)) %end; 
											%do i=1 %to &nip; %sysfunc(varname(&_dsip_,&i)) %end;) 
											*/
										
									%end;
								;
								%if (&npv < &nnmiss) or (&OrderInt > 1) %then %do;
									random 
										%do i=1 %to &nixl; 
											%sysfunc(varname(&_dsixl_,&i)) 
											%if &OrderInt > 1 %then %do;
												%do j = 2 %to &OrderInt; | %sysfunc(varname(&_dsixl_,&i)) %end;
												%if ((&i < &nixl) or (&nipl > 0)) %then %do; | %end;
											%end;
										%end; 
										%do i=1 %to &nipl; 
											%sysfunc(varname(&_dsipl_,&i)) 
											%if &OrderInt > 1 and (&i < &nipl) %then %do; | %end;
										%end; 
										%if (&nixl or &nipl) and (&OrderInt > 1) %then %do;
											@ &OrderInt
										%end;
										%do i=1 %to &NPred; 
											%sysfunc(varlabel(&_dsib_,&i)) 											
										%end; 
										/ %if (&HetVar^=Yes) %then %do; type=toep(1) %end; s;
									ods output solutionf=sf solutionr=blup;
									ods exclude solutionr;
								%end;
								%else %do;
									* repeated statement can be faster than random but does not generate predictions on training data, so use iml below;
									%if ^&CVFlag %then %do;
										repeated _obs / type=lin(1) ldata=covmat subject=_one local ri;
										/*
										repeated _obs / subject=_one ri type=sp(mathsw)(
											%do i=1 %to &nixl; %sysfunc(varname(&_dsixl_,&i)) %end; 
											%do i=1 %to &nipl; %sysfunc(varname(&_dsipl_,&i)) %end; 
											%do i=1 %to &NPred; %sysfunc(varname(&_dsib_,&i)) %end;								
										);
										*/
										ods output solutionf=sf invr=ri covparms=cp1;
										ods exclude invr;
									%end;
									%else %do;
										repeated _obs / type=lin(1) ldata=covmat subject=_one local;
										/*
										repeated _obs / subject=_one type=sp(mathsw)(
											%do i=1 %to &nixl; %sysfunc(varname(&_dsixl_,&i)) %end; 
											%do i=1 %to &nipl; %sysfunc(varname(&_dsipl_,&i)) %end; 
											%do i=1 %to &NPred; %sysfunc(varname(&_dsib_,&i)) %end;								
										);
										*/
									%end;
									%if ^%index(%qupcase(&ProcMixedStatements),PARMS) and (&HetVar^=Yes) %then %do;
										parms / lowerb=.,1e-6 pdata=cp;
									%end;
								%end;
								%if %length(&WeightVar) %then %do;
									weight &WeightVar;
								%end;
								%else %if (&Priors^=Proportional) %then %do;
									weight _Prior;
								%end;
								&ProcMixedStatements;
								ods exclude classlevels;
							run;

							%put syserr = &syserr;
							%if &syserr > 4 %then %do;
								%let exiterror = 1;
								%goto exit;
							%end;
							%let ts1 = &ts1prev;

							* compute BLUPs again directly if necessary;
							%if ^&CVFlag %then %do;
								data ts2o_nomiss;
									set ts2o;
									if not missing(&UseDep);
									drop stderrpred df alpha lower upper;
								run;
								data d_nomiss;
									set &ts1;
									if not missing(&UseDep);
								run;

								data _null_;
									set sf;
									where Effect = "Intercept";
									call symputx("Formula_Intercept",trim(left(Estimate)));
								run;
								%put Reduced Formula_Intercept = &Formula_Intercept;

								%if (&npv >= &nnmiss) %then %do;
									%NObsVars(ri);
									%let n = &nobs;
									data cp2;
										set cp1;
										if covparm="LIN(1)" and subject="_one";
									run;
									proc iml;
										use cp2;
										read all var {estimate} into cp;
										* print cp;
										use ri;
										read all var {%do i = 1 %to &n; col&i %end;} into vi;
										use ts2o_nomiss;
										read all var {resid} into resid;
										use d_nomiss;
										read all var {						
											%do i=1 %to &nixl; %sysfunc(varname(&_dsixl_,&i)) %end; 
											%do i=1 %to &nipl; %sysfunc(varname(&_dsipl_,&i)) %end; 
											%do i=1 %to &NPred; %sysfunc(varname(&_dsib_,&i)) %end;								
										} into z;
										u = cp[1]*z`*vi*resid/&uss1;
										use &ts1;
										read all var {						
											%do i=1 %to &nixl; %sysfunc(varname(&_dsixl_,&i)) %end; 
											%do i=1 %to &nipl; %sysfunc(varname(&_dsipl_,&i)) %end; 
											%do i=1 %to &NPred; %sysfunc(varname(&_dsib_,&i)) %end;								
										} into z;
										use ts2o;
										read all var {pred} into pm;
										
										pred = pm + z*u;

										create blup from u;
										append from u;

										create pred from pred;
										append from pred;

										%put syserr = &syserr;
										%if &syserr > 4 %then %do;
											%let exiterror = 1;
											%goto exit;
										%end;
									quit;

									data ts2o;
										merge ts2o pred;
										if not missing(&UseDep) then do;
											pred = col1;
											resid = &UseDep - col1;
										end;
										drop col1;
									run;

									/*
									data ts2o;
										set ts2o ts2o_miss;
									run;
									*/

									data names;
										length _name_ $32;
										%do i=1 %to &nixl; _name_ = "%sysfunc(varname(&_dsixl_,&i))"; output; %end; 
										%do i=1 %to &nipl; _name_ = "%sysfunc(varname(&_dsipl_,&i))"; output;  %end; 
										%do i=1 %to &NPred; _name_ = "%sysfunc(varname(&_dsib_,&i))"; output; %end;								
									run;

									data blup;
										merge names blup;
										blup = col1;
										abs_blup = abs(col1);
										drop col1;
									run;
								%end;
								%else %do;
									data blup;
										set blup;
										abs_blup = abs(estimate);
										rename estimate=blup effect=_name_;
									run;
								%end;

								proc sort data=blup;
									by descending abs_blup;
								run;

								/*
								data OutLib.blup_reduced_rr;
									set blup;
								run;
								*/

								data ts2vars;
									set blup;
									* keep method _name_;
									rename _name_ = name;
								run;
							%end;

							%let _dsib_ = %sysfunc(close(&_dsib_));
						%end;
						%else %do;
							%if (&npv >= &nnmiss) %then %do;
								data ts2o;
									merge ts2o pred;
									if not missing(&UseDep) then do;
										pred = col1;
										resid = &UseDep - col1;
									end;
									drop col1;
								run;
							%end;
							/*
							data ts2o;
								set ts2o ts2o_miss;
							run;
							*/
						%end;
					%end;
				%end;

				/*data OutLib.rr_ts2o1; set ts2o; run;*/

				%NObsVars(ts2o);
				%if &nobs = 0 %then %do;
					%put ERROR: PROC MIXED did not converge or produce predictions.;
					%put If cross-validating, you may have constant data.  If not, please ;
					%put inspect the SAS Output and try specifying convergence options or a different model.;
					%let exiterror = 1;
					/* %goto exit; */
				%end;	
				%else %do;
				
					/*data OutLib.rr_sf; set sf; run;*/
					
					* S1452712: including fixed effect variables from the Options tab*;
					%let ClassOrFixedFlag = 0;
					%if %length(&ClassVariables &FixedEffects) %then %do;
						
						%let ncla= 0;
						%if %length(&ClassVariables) %then %do;
							data classvarnames;
								set sf;
								keep &ClassVariables;
								where 0;
							run;
	
							%NObsVars(classvarnames);
							%let ncla= &nvars;
						%end;
						%put ncla= &ncla;
						
						%let nfix= 0;
						%if %length(&FixedEffects) %then %do;
							data fixedvarnames;
								set &ts1;
								keep &FixedEffects;
								where 0;
							run;
	
							%NObsVars(fixedvarnames);
							%let nfix= &nvars;
						%end;
						%put nfix= &nfix;						
				
						%if &ncla and &nfix %then %do;
							%let ClassOrFixedFlag = 1;
							%let dsidc = %sysfunc(open(classvarnames));
							%let reff = %sysfunc(varname(&dsidc,1));
							
							/*S1465688: making length for name equal to 32*/
							data sf2;
								set sf;
								length Parameter name $32 /*Estimate 8*/;
								parameter = effect;
								name= effect;
								%do i=1 %to &ncla; 
									if not missing(&reff) then do;
										parameter= trim(left(effect))||" "||trim(left(%sysfunc(varname(&dsidc,&i))));
										name= trim(left(effect))||"_"||trim(left(%sysfunc(varname(&dsidc,&i))));
									end;
								%end; 
							run;
							/*data OutLib.rr_sf2; set sf2; run;*/

							%let dsidc = %sysfunc(close(&dsidc));
						%end;
						%else %if ^&ncla and &nfix %then %do;
							%let ClassOrFixedFlag = 1;
							
							data sf2;
								set sf;
								length Parameter $ 32 /*Estimate 8*/;
								parameter = effect;
								name = effect;
							run;	
							
							/*data OutLib.rr_sf2; set sf2; run;*/
						%end;
					%end;
						
					%if &Kernel = Dot and ^&CVFlag %then %do;
					
						%if &ClassOrFixedFlag %then %do;
							data pe;
								set sf2;
								drop name;
							run;
						%end;					
						%else %do;
							data pe;
								length Parameter $ 32 Estimate 8;
								Parameter = "Intercept";
								Estimate = &Formula_Intercept;
							run;
						%end;
					
						%*put SelVars= &SelVars;
						%if ^%index(%quote(&SelVars),Intercept) %then %do;
						
							/*data OutLib.rr_blup; set blup; run;*/
						
							data pe1;
								length Parameter $ 32 Estimate 8;
								set blup;
								Parameter = _name_;
								Estimate = blup;
								keep Parameter Estimate;
							run;
							data pe;
								set pe pe1;
							run;
						%end;
						
						/*data OutLib.rr_pe2; set pe; run;*/
						
						%ScoringCodeMacroVars;
						
						%let suffix= _rr_pe;
						%let b32 = %eval(32 - %length(&suffix));
						%if %length(&OutName) > &b32 %then %let inname = %substr(&OutName,1,&b32);
						%else %let inname = &OutName;		
						%let peout = %quote(&inname.%trim(%left(&suffix)));
						%put peout = &peout;

						data OutLib.%unquote(&peout);
							set pe1;
						run;
							
					%end;
					%if ^&CVFlag and (&TestSpec = Yes) and (&Kernel^=Dot) %then %do;
						data &ts1;
							set ts1o;
						run;
						data &ts2;
							set d_test;
						run;
					%end;

					%put RR TraitType = &TraitType;				
					%put RR TruthVar = &TruthVar;
					
					%if &AsymmetricLossFitProp ne 0.5 %then %do;
						data ts1a;
							set ts2o;
							&DepVar = &UseDep;
							keep _Row_ &DepVar pred &WeightVar &WeightVar0;
						run;
					%end;

					* create variables for computing cv ase;
					data ts2o;
						set ts2o;
						%if &CVFlag or (&Kernel^=Dot) %then %do;
							%if &_dtype_ = C %then %do;
								where &UseDep = " ";
							%end;
							%else %do;
								where &UseDep = .;
							%end;
							&DepVar = _y0;
							drop _y0 _y;
						%end;
						%if (&TraitType=Binary) %then %do;
							if missing(pred) then do;
								p_&DepVarTrunc._0 = .;
								p_&DepVarTrunc._1 = .;
								_INTO_ = .;
							end;
							else do;
								p_&DepVarTrunc._0 = 1 - pred;
								if p_&DepVarTrunc._0 < 0 then p_&DepVarTrunc._0 = 0;
								else if p_&DepVarTrunc._0 > 1 then p_&DepVarTrunc._0 = 1;
								p_&DepVarTrunc._1 = 1 - p_&DepVarTrunc._0;
								_INTO_ = (p_&DepVarTrunc._1 > 0.5);
							end;
							%if &_dtype_ = C %then %do;
								if &TruthVar = " " then
							%end;
							%else %do;
								if &TruthVar = . then
							%end;
								actualp = .;
							else if (&TruthVar=1) then actualp = p_&DepVarTrunc._1;
							else actualp = p_&DepVarTrunc._0;
							/*
							%let ncl = 2;
							%let pprob1 = p_&DepVarTrunc._0;
							%let pprob2 = p_&DepVarTrunc._1;
							*/
						%end;
					run;	
				%end;
				
				/*data OutLib.rr_ts2o2; set ts2o; run;*/

				%let into = _INTO_;
				%let pred = pred;
			%end;
			%else %if &ProcessName = LifeRegression %then %do;

				* string for analysis settings;
				%if %length(&ASet) %then %do;
					%let ASet = &ASet, Distribution = &Distribution;
				%end;
				%else %do;
					%let ASet = Distribution = &Distribution;
				%end;
				%if %length(%trim(&ColorVar)) %then %do;
					data _ts1c_;
						if 0 then set _ColorVar_;
						if _n_ = 1 then do;
							declare hash h(dataset: "work._ColorVar_", hashexp: 6);
							h.defineKey("&IDVar");
							h.defineData("&ColorVar");
							h.defineDone();
						end;
						set &ts1;
						rc = h.find();
						if (rc = 0) then output;
						drop rc;
					run;
					%let ts1 = _ts1c_;
					%if %length(&TestInData) | &CVFlag %then %do;
						data _ts2c_;
							if 0 then set _ColorVar_t;
							if _n_ = 1 then do;
								declare hash h(dataset: "work._ColorVar_t", hashexp: 6);
								h.defineKey("&IDVar");
								h.defineData("&ColorVar");
								h.defineDone();
							end;
							set &ts2;
							rc = h.find();
							if (rc = 0) then output;
							drop rc;
						run;
						%let ts2 = _ts2c_;
					%end;
				%end;

				* using missing values for the dependent variable to create predictions;
				%if &CVFlag or (&TestSpec=Yes) %then %do;
					data &ts1;
						set &ts1;
						_y0 = &DepVar;
						_y = &DepVar;
					run;
					data &ts2;
						set &ts2;
						_y0 = &DepVar;
						%if (&_dtype_=C) %then %do;
							_y = " ";
						%end;
						%else %do;
							_y = .;
						%end;
					run;
					data &ts1;
						set &ts1 &ts2;
					run;
					%let UseDep = _y;
				%end;
				%else %do;
					%let UseDep = &DepVar;
				%end;

				proc lifereg data=&ts1 %if %length(&LifeRegOptions) %then &LifeRegOptions; ;
					%if &nipl | &nip %then %do;
						class 
							%do i=1 %to &nipl; %sysfunc(varname(&_dsipl_,&i)) %end; 
							%do i=1 %to &nip; %sysfunc(varname(&_dsip_,&i)) %end; 
						; 
					%end;	
					%if %length(&WeightVar) %then %do;
						weight &WeightVar;
					%end;
					%if %length(&CensorVar) %then %do;
						model &UseDep * &CensorVar(&CensorValues) =	
					%end;
					%else %do;
						model &UseDep = 					
					%end;
						%if ^%length(&ListContPredVars) & ^%length(&ListClassPredVars) %then %do;
							&SelVars	
						%end;
						%else %do;
							%do i=1 %to &nixl; 
								%sysfunc(varname(&_dsixl_,&i)) 
								%if &OrderInt > 1 %then %do;
									%do j = 2 %to &OrderInt; | %sysfunc(varname(&_dsixl_,&i)) %end;
									%if ((&i < &nixl) or (&nipl > 0) or (&nix > 0) or (&nip > 0)) %then %do; | %end;
								%end;
							%end; 
							%do i=1 %to &nipl; 
								%sysfunc(varname(&_dsipl_,&i)) 
								%if &OrderInt > 1 and ((&i < &nipl) or (&nix > 0) or (&nip > 0)) %then %do; | %end;
							%end; 
							%do i=1 %to &nix; 
								%sysfunc(varname(&_dsix_,&i)) 
								%if &OrderInt > 1 %then %do;
									%do j = 2 %to &OrderInt; | %sysfunc(varname(&_dsix_,&i)) %end;
									%if ((&i < &nix) or (&nip > 0)) %then %do; | %end;
								%end;
							%end; 
							%do i=1 %to &nip; 
								%sysfunc(varname(&_dsip_,&i)) 
								%if &OrderInt > 1 and (&i < &nip) %then %do; | %end;
							%end; 
							%if &OrderInt > 1 %then %do;
								@ &OrderInt
							%end;
						%end;
						/ dist=&Distribution &LifeRegModelOptions; 
					;
					output out=ts2o predicted=pred;
				run;
				
				%if &CVFlag %then %do;
					data ts2o;
						set ts2o;
						%if &_dtype_ = C %then %do;
							where &UseDep = " ";
						%end;
						%else %do;
							where &UseDep = .;
						%end;
						&DepVar = _y0;
						drop _y0 _y;
					run;
					
					/*
					data OutLib.ts2o_&iter._&k;
						set ts2o;
					run;
					*/
				%end;
				%let pred = pred;

			%end;

			%else %if &ProcessName = SurvivalPredictiveModeling %then %do;

				* string for analysis settings;
				%if %length(&ASet) %then %do;
					%let ASet = &ASet, &trVariableSelectionMethod = %LocaleEZX(%nrbquote(&VarSelect), EIA=1);
				%end;
				%else %do;
					%let ASet = &trVariableSelectionMethod = %LocaleEZX(%nrbquote(&VarSelect), EIA=1);
				%end;
				%if (&VarSelect = Stepwise) %then %do;	
					%let ASet = &Aset, &trSLEntry = &SLEntry, &trSLStay = &SLStay;
				%end;	
				%if &MaxStep > 0 %then %do; 
					%let ASet = &Aset, &trMaxStep = &MaxStep;
				%end;
				%else %if (&VarSelect = Forward) %then %do; 
					%let ASet = &Aset, &trSLEntry = &SLEntry;
				%end;	
				%else %if (&VarSelect = Backward) %then %do;	
					%let ASet = &Aset, &trSLStay = &SLStay;
				%end;	
				%else %if (&VarSelect = Score) %then %do;	
					%let ASet = &Aset, &trnPredictors = &nScorePredictors;
				%end;
				%else %do; 	
					* for &VarSelect = None;				 
					%if %length(&ColorVar) %then %do;
						data _ts1c_;
							if 0 then set _ColorVar_;
							if _n_ = 1 then do;
								declare hash h(dataset: "work._ColorVar_", hashexp: 6);
								h.defineKey("&IDVar");
								h.defineData("&ColorVar");
								h.defineDone();
							end;
							set &ts1;
							rc = h.find();
							if (rc = 0) then output;
							drop rc;
						run;
						%let ts1 = _ts1c_;
						%if %length(&TestInData) | &CVFlag %then %do;
							data _ts2c_;
								if 0 then set _ColorVar_t;
								if _n_ = 1 then do;
									declare hash h(dataset: "work._ColorVar_t", hashexp: 6);
									h.defineKey("&IDVar");
									h.defineData("&ColorVar");
									h.defineDone();
								end;
								set &ts2;
								rc = h.find();
								if (rc = 0) then output;
								drop rc;
							run;
							%let ts2 = _ts2c_;
						%end;
					%end;

					%if &CVFlag %then %do;
						%let _CovData = &ts2;
					%end;
					%else %do;
						%if %length(&TestInData) %then %do;
							data _Covariates;
								set &ts1 &ts2;
							run;
							%let _CovData = _Covariates;
						%end;
						%else %do; 
							%let _CovData = &ts1;
						%end;
					%end;
					
					proc phreg data=&ts1 %if %length(&PhregOptions) %then &PhregOptions;;
						%if &nipl | &nip %then %do;
							class 
								%do i=1 %to &nipl; %sysfunc(varname(&_dsipl_,&i)) %end; 
								%do i=1 %to &nip; %sysfunc(varname(&_dsip_,&i)) %end; / missing
							; * specifying all class predictors in class statement to avoid error;
						%end;	
						%if %length(&WeightVar) %then %do;
							weight &WeightVar;
						%end;
						%if %length(&CensorVar) %then %do;
							model &DepVar*&CensorVar(&CensorValues) =	
						%end;
						%else %do;
							model &DepVar = 					
						%end;
						%* proc phreg cannot accept list style variable expression;
						%if ^%length(&ListContPredVars) & ^%length(&ListClassPredVars) %then %do;
							&SelVars	
						%end;
						%else %do;
							%do i=1 %to &nixl; 
								%sysfunc(varname(&_dsixl_,&i)) 
								%if &OrderInt > 1 %then %do;
									%do j = 2 %to &OrderInt; | %sysfunc(varname(&_dsixl_,&i)) %end;
									%if ((&i < &nixl) or (&nipl > 0) or (&nix > 0) or (&nip > 0)) %then %do; | %end;
								%end;
							%end; 
							%do i=1 %to &nipl; 
								%sysfunc(varname(&_dsipl_,&i)) 
								%if &OrderInt > 1 and ((&i < &nipl) or (&nix > 0) or (&nip > 0)) %then %do; | %end;
							%end; 
							%do i=1 %to &nix; 
								%sysfunc(varname(&_dsix_,&i)) 
								%if &OrderInt > 1 %then %do;
									%do j = 2 %to &OrderInt; | %sysfunc(varname(&_dsix_,&i)) %end;
									%if ((&i < &nix) or (&nip > 0)) %then %do; | %end;
								%end;
							%end; 
							%do i=1 %to &nip; 
								%sysfunc(varname(&_dsip_,&i)) 
								%if &OrderInt > 1 and (&i < &nip) %then %do; | %end;
							%end; 
							%if &OrderInt > 1 %then %do;
								@ &OrderInt
							%end;
						%end;
						%if %length(PhregModelOptions) %then / &PhregModelOptions ; 
						;
						baseline covariates=&_CovData 
							out=SurFunc(keep=_obs _train_ &DepVar &CensorVar &IDVar &ColorVar &WeightVar Survival StdErrSurvival %if &CVFlag %then %do; __id %end;)
							survival=_all_ / rowid=&IDVar
						;
						output out=OutLib.ts2o atrisk=atrisk cif=cif ld=ld lmax=lmax loglogs=loglogs logsurv=p_logsurv resdev=resdev
							resmart=resmart stdxbeta=stdxbeta survival=p_survival xbeta=xbeta;
						ods output parameterestimates=ParEst;
					run;
				%end;

				%nObsVars(SurFunc);
				%if ^&nobs %then %do;
					%put ERROR: There is no valid fitting model found.;
					%put ERROR: This may be caused by not enough valid data in the input data set.; 
					%let exiterror = 1;
					%goto exit;
				%end;

				%if %upcase(&DepVar) = SURVIVAL %then %let SurvivalVar = Survival2;
				%else %let SurvivalVar = Survival;

				/*
				options notes;
				*/

				* output selected predictors;
				%if %quote(&SelVars) ne %quote(Intercept) %then %do;
					data spmv;
						set ParEst(keep= Parameter);
						order = _n_;
						rename Parameter = Name;
					run;

					proc sort data=spmv nodupkey;
						by Name;
					run;

					proc sort data=spmv out=&outdatvars(drop=order);
						by order;
					run;

					data _null_;
						set &outdatvars end=_e_;
						vname="_v"||trim(left(_n_));
						call symput(vname,name);
						if _e_ then call symput('_nsv',_n_);
					run;
					%let SelVarShow = &_v1;
					%do i = 2 %to &_nsv;
						%let SelVarShow = &SelVarShow &&_v&i;
					%end;
				%end;
				%else %do; 
					data &outdatvars;
						name = "Intercept";
						output;
					run;
					%let SelVarShow = Intercept;
				%end;	

				* Extract selected predictors;
				%let i = 1;
				%let sVarList = ;
				%do %while(%length(%qscan(&SelVarShow,&i)));
					%if %qscan(&SelVarShow,&i) ne %quote(Intercept) %then %do;
						%let sVarList = &sVarList %qscan(&SelVarShow,&i);
					%end;
					%let i = %eval(&i+1);
				%end;

				* keep original time var;
				data _TimeVar;
					set &_CovData;
					%if %length(&TestInData) %then %do;
						&DepVar = &DepVar._true;
					%end;
					keep _Obs %if %length(&TestInData) | &CVFlag %then _train_; &DepVar;
					rename &DepVar = Org_&DepVar;
					label &DepVar = "Original &DepVar";
				run;
				* merge original &TimeVar back to SurFunc;
				data surfunc;
					if 0 then set _TimeVar;
					if _n_ = 1 then do;
						declare hash h(dataset: "_TimeVar", hashexp: 6);
						%if %length(&TestInData) | &CVFlag %then %do;
							h.defineKey('_Obs','_Train_');
						%end;
						%else %do;
							h.defineKey('_Obs');
						%end;
						h.defineData("Org_&DepVar");
						h.defineDone();
					end;
					set surfunc;
					if (h.find() = 0) then output;
				run;

				proc sort data=SurFunc;
					by %if %length(&TestInData) | &CVFlag %then descending _train_; _obs;
				run;

				* keep a copy of stack data;
				%if &CVFlag %then %do;
					data &outdat;
					* This will be the major output for cvmc. The corresponding AUC and Harrells C will be;
					* recalculated in the cvmc routines;
				%end;
				%else %do;
					data OutLib.Surfunc_Stack;
				%end;
					set SurFunc;
					by %if %length(&TestInData) | &CVFlag %then descending _train_; _obs;
					retain _TimeOrder_;
					if first._obs then _TimeOrder_=1;
					else _TimeOrder_+1;
					if _train_ then _DataType_ = "Training";
					else _DataType_ = "Test";
					label _DataType_ = "DataType";
					keep _obs _train_ _TimeOrder_ _DataType_ &DepVar Org_&DepVar &CensorVar &IDVar &SurvivalVar StdErrSurvival %if &CVFlag %then %do; __id %end; ;
				run; 

				* Extract values at reference time;
				%if ^%length(&RefTime) %then %do;
					proc means data=SurFunc noprint;
						var &DepVar;
						output out=mt median=mt;
					run;
					data _null_;
						set mt;
						call symput("RefTime1",trim(left(put(mt,6.1))));
					run;
					%let nRefTime = 1;
					%let RefTime = &RefTime1;
					%if &CVFlag %then %do;
						%global cv_nRefTime_m&Workflowrp;
						%let cv_nRefTime_m&Workflowrp = &nRefTime;
					%end;
				%end;
				
/*	
	%macro km_roc(ds,v_time,v_censor,cval,v_x,timepoint,outds);
	*** for caluclating time-dependent AUC;
%*put ds=&ds,v_time=&v_time,v_censor=&v_censor,cval=&cval;				
				proc sort data=&ds(keep=&v_time &v_censor &v_x) out=sortds;
					by &v_x;
				run;
				
				data _null_;
					set sortds end=last;
					if last then call symput('total_n',_n_);
				run;
				
				proc univariate data=sortds noprint;
					var &v_x;
					output out=dum p5=p5 p95=p95;
				run;
				
				data _null_;
					set dum;
					call symput('p5',p5);
					call symput('p95',p95);
				run;
				
				data &outds;
				run;
				
				* get Skm(t);
				ods graphics off;
				proc lifetest data=&ds(keep= &v_time &v_censor &v_x) method=KM
					outsurv=dum noprint;
					time &v_time*&v_censor(&cval);
				run;
				
				proc sort data=dum(where=(&v_time<=&timepoint)) out=surv_all;
					by descending &v_time;
				run;
				
				data _null_;
					set surv_all(obs=1);
					call symput('Skm_',survival);
				run;
				
				* choose 20 different cutoff values and get the corresponding sensivity and specificity;
				%do i=1 %to 20;
	%*put i=&i;			
					%put i=&i of 20;
					data sortds;
						set sortds;
						if &v_x > &p5 + (&i-1)*(&p95-&p5)/19 then riskc=1;
						else riskc=0;
					run;
					
					data dum;
						set sortds(where=(riskc=0));
					run;
					
					* get accumulated distribution function Fx(c);
					data _null_;
						set dum end=last;
						if last then call symput('Fxc',_n_/&total_n);
					run;
					
					* get KM survival function by x > or < c subgroup;
					proc lifetest noprint data=sortds method=KM outsurv=surv_by;
						time &v_time*&v_censor(&cval);
						strata riskc;
					run;
					
					proc sort data=surv_by(where=(&v_time<=&timepoint)) out=dummy;
						by stratum descending &v_time;
					run;
					
					* store Skm(t|x<c) in Skm_c1, Skm(t|x>c)in Skm_c2;
					data _null_;
						set dummy;
						by stratum;
						if first.stratum and stratum=1 then
						call symput('Skm_c1',survival);
						if first.stratum and stratum=2 then
						call symput('Skm_c2',survival);
					run;
					
					* calculate sensivity and 1-specificity;
					data roc1;
						%if %symexist(Skm_c2) %then %do;
							sens = (1-&Skm_c2)*(1-&Fxc)/(1-&Skm_);
						%end;
						%else %do;
							sens = 0
						%end;
						%if %symexist(Skm_c1) %then %do;
							spec = &Skm_c1*&Fxc/(&Skm_);
						%end;
						%else %do;
							spec = 0;
						%end;
						x = 1-spec;
					run;
					
					data &outds;
						set &outds roc1;
					run;
				%end;
				
				ods graphics on;
				
				data dum;
					x=0; sens=0;
				run;
				
				data &outds;
					set &outds dum;
					if x>. and sens>.;
				run;
				
				proc sort data=&outds;
					by x;
				run;
				
				* prepare to calculate area under the curve;
				data &outds;
					set &outds end=last;
					xprev=lag(x);
					yprev=lag(sens);
					output;
					if last then do;
						xprev=x;
						yprev=sens;
						x=1;
						sens=1;
						output;
					end;
				run;
				
				data &outds._area;
					retain area 0;
					set &outds(firstobs=2) end=last;
					area = area + (sens+yprev)*(x-xprev)/2;
					var = "&v_x";
					if last then output;
				run;
				
				data &outds(rename=(x=x_&outds sens=y_&outds));
					set &outds;
					if sens > .;
					keep x sens;
				run;
				
%mend;
%*km_roc(h1,Org_&DepVar,censor,1,rs1,366,out366);				
*/				
				%macro RefSurEst(RefTime,OutData);						
					data &OutData;
						set SurFunc;
						by %if %length(&TestInData) | &CVFlag %then descending _train_; _obs;
						retain _top_ _find_;
						if first._obs then do;
							_top_ = 1; 
							_find_ = 0;
						end;
						/*** add character index for numerical color var; 
						* this will cause error when Color var is set to Censor var;
						%if %length(&ColorVar) %then %do;
							%if &_cType_ = N %then %do;
								_ColorVar_ = trim(left(&ColorVar));
							%end;
						%end;*/
						if _train_ then _DataType_ = "Training";
						else _DataType_ = "Test";
						if _top_ and &DepVar >= &RefTime then do;
							RefSurEst = &SurvivalVar;
							output;
							_top_ = 0;
							_find_ = 1;
						end; 
						else do;
							* set the reference survival time to be the largest time in the survival function;
							if last._obs and ^_find_ then do;
								RefSurEst = &SurvivalVar;
								output;
							end;
						end; 
						label RefSurEst="Reference Survival Estimate at %trim(%left(&RefTime))" 
							_DataType_ = "DataType";
						drop _top_ &SurvivalVar &DepVar;	
						/*%if %length(&ColorVar) %then %do;
							%if &_cType_ = N %then %do;
								drop &ColorVar;
								rename _ColorVar_ = &ColorVar;
							%end;
						%end;*/
					run;
										
					%nobsvars(&OutData);
					%if ^&nobs %then %do;
						/*data OutLib.surfunc; set surfunc; run;*/
						%let exiterror = 1;
						%put ERROR: no valid Reference Survival Estimate at %trim(%left(&RefTime));
					%end;					
				%mend;
				
				%if &exiterror %then %goto exit;
				
				

				%do j = 1 %to &nRefTime;
					%RefSurEst(&&RefTime&j,RefSur&j);
					data RefSur&j;
						set RefSur&j;
						rename RefSurEst = RefSurEst&j;
					run;
					
	%*put j=&j, RefTime = &&RefTime&j;
					*time-dependent-AUC;
					data rAUC&j;
						set RefSur&j;
						RiskScore&j = - RefSurEst&j;
						drop RefSurEst&j;
					run;
					%km_roc(rAUC&j,Org_&DepVar,&CensorVar,&CensorValues,RiskScore&j,&&RefTime&j,out_iAUC);
					data _null_;
						set out_iAUC_area;
						call symput("RefTime_iAUC_&j",area);
					run;

/*

				data &SPM_OutData3;
					set SurFunc;
					by %if %length(&TestInData) | &CVFlag %then descending _train_; _obs;
					retain _top_;
					if first._obs then _top_ = 1; 
					*** add character index for numerical color var;
					%if %length(&ColorVar) %then %do;
						%if &_cType_ = N %then %do;
							_ColorVar_ = trim(left(&ColorVar));
						%end;
					%end;
					if _train_ then _DataType_ = "Training";
					else _DataType_ = "Test";
					if _top_ and &DepVar >= &RefTime1 then do;
						output;
						_top_ = 0;
					end; 
					label &DepVar="&trReferenceSurvivalTime" &SurvivalVar="Reference Survival Estimate" 
						_DataType_ = "DataType";
					drop _top_;	
					%if %length(&ColorVar) %then %do;
						%if &_cType_ = N %then %do;
							drop &ColorVar;
							rename _ColorVar_ = &ColorVar;
						%end;
					%end;
				run; 
				
	*/			
					
				%end;
				
				data &SPM_OutData3;
					merge %do j = 1 %to &nRefTime; RefSur&j %end; ;
					by %if %length(&TestInData) | &CVFlag %then descending _train_; _obs;
				run;
					
					
					
					
					
					
					
					
					
					
				data AUC; 
					set surfunc; 
					by %if %length(&TestInData) | &CVFlag %then descending _train_; _obs;
					retain _AUC_ _MedSurTime_ _mIndex_;
					_T1_ = lag(&DepVar);
					_T2_ = &DepVar; 
					/* This part calculates the AUC with in specified time range;
					if _T2_ < &MinRefTime | _T2_ > &MaxRefTime then do;
						_dT_ = 0;
					end;
					else do; *z1=lag(&DepVar);*z2=&DepVar;
						_T1_ = max(_T1_,&MinRefTime);
						_T2_ = min(_T2_,&MaxRefTime);
						_dT_ = _T2_ - _T1_;
					end;
					*/
					_dT_ = _T2_ - _T1_;
					
					_PreSurvival_= lag(&SurvivalVar);
					_Area_ = _dT_*_PreSurvival_;
					if first._obs then do;
						_mIndex_ = 0;
						_MedSurTime_ = .;
						_AUC_ = 0;
					end;
					else do;
						if ^_mIndex_ then do;
							if &SurvivalVar <= 0.5 | last._obs then do;
								* all survival rate above 0.5 then assign the maximum time to be med-time;
								_MedSurTime_ = &DepVar;
								_mIndex_ = 1;
							end;
						end; 
						_AUC_ = _AUC_ + _Area_;
					end;
					if last._obs;
					label _MedSurTime_="Median Survival Time" _AUC_="Area Under Survival Curve";
					keep %if %length(&TestInData) | &CVFlag %then _train_; _obs _MedSurTime_ _AUC_;
				run;

				options notes;
				/*
				data OutLib.auc;
					set auc;
				run;
				*/

				* data with selected predictors;
				data SVars;
					set SurFunc;
					by %if %length(&TestInData) | &CVFlag %then descending _train_; _obs;
					if first._Obs;
					keep %if %length(&TestInData) | &CVFlag %then _train_; _obs;
				run;

				data &SPM_OutData3;
					merge &SPM_OutData3 AUC SVars;
					by %if %length(&TestInData) | &CVFlag %then descending _train_; _obs;
				run;
				
				
				%if %length(&MinRefTime) & %length(&MaxRefTime) %then %do;
					* time-dependent AUC;
					%*put MinRefTime=&MinRefTime, MaxRefTime=&MaxRefTime;
					%let nIter = 20;
					%let Ave_iAUC = 0;
					%do j = 1 %to &nIter;
						%let refT = %sysevalf(&MinRefTime + (&j-1)*(&MaxRefTime-&MinRefTime)/(&nIter-1));
						%*put j=&j, refT=&refT;
						%RefSurEst(&refT,iAUC&j);
						data iAUC&j;
							set iAUC&j;
							rename RefSurEst = RefSurEst&j;
						run;
						
						data iAUC&j;
							set iAUC&j;
							iRiskScore&j = - RefSurEst&j;
							drop RefSurEst&j;
						run;
						%km_roc(iAUC&j,Org_&DepVar,&CensorVar,&CensorValues,iRiskScore&j,&refT,out_iAUC);
						data _null_;
							set out_iAUC_area;
							call symput('iAUC',area);
						run;
						%let Ave_iAUC = %sysevalf(&Ave_iAUC + &iAUC/&nIter);
					%end;
				%end;
				%else %do;
					%let nIter = 0;
				%end;
				
				
				%if ^&CVFlag %then %do;
					data OutLib.Time_Dep_AUC;
						%do i = 1 %to &nRefTime;
							Ref_Time = &&RefTime&i;
							AUC = &&RefTime_iAUC_&i;
							output;
						%end;
						%if %length(&MinRefTime) & %length(&MaxRefTime) %then %do;
							Lower_Time = &MinRefTime;
							Upper_Time = &MaxRefTime;
							AUC = &Ave_iAUC;
							Ref_Time = .;
							output;
						%end;
					run;
				%end;
				%else %do;
					* keep risk score data;
					data Time_Dep_AUC_&iter._&k;
						merge 
							%do j = 1 %to &nRefTime; rAUC&j %end;
							%do j = 1 %to &nIter; iAUC&j %end;; 
							* the first set with Riskscore for RefTime;
							* the second set with Riskscore for Ranged Time;
						by descending _train_ _Obs;
						ITER = &iter;
						K = &k;
					run;
				%end;
		
				* collect all Time_Dep_AUC for the same iteration when CV.;
				%if &CVFlag %then %do;
					%if &k = &kfold %then %do;
						data OutLib.Time_Dep_AUC_m&Workflowrp._i&iter;
							set %do i = 1 %to &kfold; Time_Dep_AUC_&iter._&i %end;;
						run;
					%end;
				%end;
								
				
				
				
				
				

				* merge back the dependent variable;
				%if %length(&DepVar) < 31 %then %let MedSurVar = M_&DepVar;
				%else %let MedSurVar = M_%substr(&DepVar,1,30);
				%if ^%length(&TestInData) & ^&CVFlag %then %do;
					proc sort data=&SPM_OutData3;
						by &IDVar;
					run;

					proc sort data=&_CovData(keep=&DepVar &IDVar) out=_tSet_;
						by &IDVar;
					run;

					data &SPM_OutData3;
						merge _tSet_ &SPM_OutData3;*(rename=(&DepVar=&MedSurVar));
						by &IDVar;
						*if M_&DepVar ne .;
					run;

					data _H1_;
						set &SPM_OutData3(keep=Org_&DepVar &CensorVar _AUC_
							%do i = 1 %to &nRefTime;
								RefSurEst&i
							%end;
						);
					run;;
				%end;
				%else %do;
					proc sort data=&SPM_OutData3;
						by descending _train_ &IDVar;
					run;

					proc sort data=&_CovData(keep=&TruthVar &IDVar _train_) out=_tSet_(rename=(&TruthVar=&DepVar));
						by descending _train_ &IDVar;
					run;

					data &SPM_OutData3;
						merge _tSet_ &SPM_OutData3;*(rename=(&DepVar=&MedSurVar));
						by descending _train_ &IDVar;
						*if M_&DepVar ne .;
					run;

					data _H1_ _H2_;
						set &SPM_OutData3(keep=Org_&DepVar &CensorVar _AUC_ _DataType_
							%do i = 1 %to &nRefTime;
								RefSurEst&i
							%end;
						);
						if upcase(_DataType_) = "TRAINING" then output _H1_;
						else output _H2_;
					run;

					%if &CVFlag %then %do;
						data &outdataspm3;
							set &SPM_OutData3;
						run;
					%end;

				%end;

				* toggle censor variable for proc harrell if necessary;
				%global CVarH;
				%if %length(&CensorVar) %then %do;
					%if &cvtype=N and %length(&CensorValues) and %quote(&CensorValues) ne %quote(1) %then %do;
						data _H1_;
							set _H1_;
							if &CensorVar in (&CensorValues) then _Censor1 = 1;
							else _Censor1 = 0;
						run;
						%if %length(&TestInData) | &CVFlag %then %do;
							data _H2_;
								set _H2_;
								if &CensorVar in (&CensorValues) then _Censor1 = 1;
								else _Censor1 = 0;
							run;
							%if &CVFlag %then %let cdat = &outdat; 
							%else %let cdat = OutLib.Surfunc_Stack; 
							data &cdat;
								set &cdat;
								if &CensorVar in (&CensorValues) then _Censor1 = 1;
								else _Censor1 = 0;
							run;						
						%end;
						%let CVarH = _Censor1;
					%end;
					%else %do;
						%let CVarH = &CensorVar;
					%end;
				%end;

				data OutLib.h1;
					set _h1_;
				run;

				* Harrell C-statistic using _AUC_ as the predictive value;
				%NObsVars(_h1_);
				%if &nobs %then %do;
					proc harrell data=_H1_ out=Harrells_C_AUC;
						time Org_&DepVar;
						%if %length(&CensorVar) %then %do;
							censor &CVarH;
						%end;
						predict _AUC_;
					run;

					data _null_;
						set Harrells_C_AUC;
						call symputx('Harrells_C',Harrell);
						call symputx("Harrells_C2",trim(left(round(Harrell,0.01))));* for displaying;
					run;

					%put Harrells_C of AUC: &Harrells_C;
					
					data OutLib.Harells_C;
						set Harrells_C_AUC;
						_TYPE_ = 'AUC';
						_VALUE_ = .;
					run;
					
					%do i = 1 %to &nRefTime;
						proc harrell data=_H1_ out=Harrells_C_Ref&i;
							time Org_&DepVar;
							%if %length(&CensorVar) %then %do;
								censor &CVarH;
							%end;
							predict RefSurEst&i;
						run;

						data _null_;
							set Harrells_C_Ref&i;
							call symputx("Harrells_C_ref&i",Harrell);
							call symputx("Harrells_C2_ref&i",trim(left(round(Harrell,0.01))));* for displaying;
						run;
						
						data Ref&i;
							set Harrells_C_Ref&i;
							_TYPE_ = 'REF';
							_VALUE_ = &&RefTime&i;
						run;
						
						data OutLib.Harells_C;
							set OutLib.Harells_C Ref&i;
						run;
					%end;
					
					
					
					
				%end;
				%else %do;
					%let Harrells_C = .;
					%let Harrells_C2 = .;
				%end;

				%if %length(&TestInData) | &CVFlag %then %do;
					data _H22_;
						set _H2_;
						if not missing(&DepVar) and not missing(_AUC_);
					run;
					%NObsVars(_H22_);
					%if &nobs %then %do;
						proc harrell data=_H2_ out=Harrells_C_2;
							time Org_&DepVar;
							%if %length(&CensorVar) %then %do;
								censor &CVarH;
							%end;
							predict _AUC_;
						run;

						data _null_;
							set Harrells_C_2;
							call symput('Harrells_C_2',Harrell);
							call symput("Harrells_C2_2",trim(left(round(Harrell,0.01))));* for displaying;
						run;
					%end;
					%else %do;
						%let Harrells_C_2 = .;
						%let Harrells_C2_2 = .;
					%end;
					
					
					%do i = 1 %to &nRefTime;
						data _H22_&i;
							set _H2_;
							if not missing(&DepVar) and not missing(RefSurEst&i);
						run;
						%NObsVars(_H22_&i);
						%if &nobs %then %do;
							proc harrell data=_H2_ out=Harrells_C_2;
								time Org_&DepVar;
								%if %length(&CensorVar) %then %do;
									censor &CVarH;
								%end;
								predict RefSurEst&i;
							run;
	
							data _null_;
								set Harrells_C_2;
								call symput('Harrells_C_2_ref&i',Harrell);
								call symput("Harrells_C2_2_ref&i",trim(left(round(Harrell,0.01))));* for displaying;
							run;
						%end;
						
					%end;
					
					
					
				%end;

				proc sort data=%if &CVFlag %then &outdat; 
									%else OutLib.Surfunc_Stack; out=SurFunc;
					by %if %length(&TestInData) | &CVFlag %then _DataType_; _TimeOrder_ &DepVar;
				run;

				* set output data for estimated survival function;
				%let nBin = 50;
				%if %length(&TestInData) | &CVFlag %then %do;
					data SurFunc_train SurFunc_test;
						set SurFunc;
						if _DataType_ = "Training" then output SurFunc_Train;
						else output SurFunc_Test;
					run;

					proc transpose data=SurFunc_train out=&SPM_OutData(drop= _Name_ _Label_ _TimeOrder_);
						by _TimeOrder_ &DepVar;
						var &SurvivalVar;
						id &IDVar; 
						idlabel &IDVar;
					run; 

					proc transpose data=SurFunc_test out=&SPM_OutData2(drop= _Name_ _Label_ _TimeOrder_);
						by _TimeOrder_ &DepVar;
						var &SurvivalVar;
						id &IDVar;
						idlabel &IDVar;
					run;

					%if &CVFlag %then %do;
						data &outdataspm1;
							set &SPM_OutData;
						run;
						data &outdataspm2;
							set &SPM_OutData2;
						run;
					%end;

					data t1_DsToMerge t2_DsToMerge;
						set &SPM_OutData3;
						if _DataType_ = "Training" then output t1_DsToMerge;
						else output t2_DsToMerge;
					run;

					%if ^&CVFlag %then %do;
						%SurFunc_Tall_to_Wide(_inDs=&SPM_OutData,_outDs=Surfunc_wide1,_idVar=&IDVar,
							_nBin=&nBin,_DsToMerge=t1_DsToMerge,
							_time_vListPrefix=wide_vList,_time_lListPrefix=wide_lList);
					%end;

					%SurFunc_Tall_to_Wide(_inDs=&SPM_OutData2,_outDs=Surfunc_wide2,_idVar=&IDVar,
						_nBin=&nBin,_DsToMerge=t2_DsToMerge);

					data &SPM_OutData3;
						set %if ^&CVFlag %then %do; Surfunc_wide1 %end; Surfunc_wide2;
					run;

				%end;
				%else %do;
					proc transpose data=SurFunc out=&SPM_OutData(drop= _Name_ _Label_ _TimeOrder_);
						by _TimeOrder_ &DepVar;
						var &SurvivalVar;
						id &IDVar; 
						idlabel &IDVar;
					run; 

					%SurFunc_Tall_to_Wide(_inDs=&SPM_OutData,_outDs=Surfunc_wide,_idVar=&IDVar,
						_nBin=&nBin,_DsToMerge=&SPM_OutData3,
						_time_vListPrefix=wide_vList,_time_lListPrefix=wide_lList);

					data &SPM_OutData3;
						set Surfunc_wide;
					run;
				%end;

				%goto PRS;
			%end;
			%else %if (&ProcessName = GenomicBLUP) or (&ProcessName = GenomicBayesianRegression) %then %do;
			
				%if (&ProcessName = GenomicBLUP) %then %do;
					* string for analysis settings;
					%if %length(&ASet) %then %do;
						%let ASet = &ASet, Kernel = &Kernel;
					%end;
					%else %do;
						%let ASet = Kernel = &Kernel;				
					%end;
					%if &aloss_niter=1 and &NPred %then %do;
						%if %length(&ASet) %then %do;
							%let trTopNPredBLUPs = %nrbquote(%LocaleFill(%nrbquote(&trTopXBLUPs), &NPred));
							%let ASet = &ASet, &trTopNPredBLUPs &BLUPWhere;
						%end;
						%else %do;
							%let trTopNPredBLUPs = %nrbquote(%LocaleFill(%nrbquote(&trTopXBLUPs), &NPred));
							%let ASet = &trTopNPredBLUPs &BLUPWhere;
						%end;
					%end;
				%end;	
				%else %do;
					* string for analysis settings;
					%if %upcase(&BayesModel) = BAYESIANLASSO %then %do;
						%if %length(&ASet) %then %do;
							 %let ASet = &ASet, Fitted Model = Bayesian LASSO;
						%end;
						%else %do;	
							%let ASet = Fitted Model = Bayesian LASSO;				
						%end;
					%end;
					%else %do;
						%if %length(&ASet) %then %do;
							 %let ASet = &ASet, Fitted Model = &BayesModel;
						%end;
						%else %do;	
							%let ASet = Fitted Model = &BayesModel;				
						%end;
					%end;					
				%end;
			
				************************************************************************************************;
				*nixl= holds number of continuous predictor variables locked-in plus locked-in marker variables*;
				*nipl= holds number of class predictor variables locked-in*;
				*nix= holds number of continuous predictor variables plus marker variables*; 
				*nip= holds number of class predictor variables*; 
				*************************************************************************************************;			
				
				/*data OutLib.gb_ts10_&ts1; set &ts1; run;*/
				
				/*
				%put MarkerVarsGB= &MarkerVars;
				%put ListMarkerVarsGB= &ListMarkerVars;
				%put LockMarkerVarsGB= &LockMarkerVars;
				%put ListLockMarkerVarsGB= &ListLockMarkerVars;
				
				%put LockContPredVarsGB= &LockContPredVars;
				%put LockClassPredVarsGB= &LockClassPredVars;
				%put ClassPredVarsGB= &ClassPredVars;	
				%if ^&upred %then %do; 
					%put PredictorVarsGB= &PredictorVars; 
				%end; 
				%else %do; 
					%do j=1 %to &nix_blocks; 
						%put PredictorVarsGB&j= &&PredictorVars&j; 
					%end; 
				%end;				
				*/
				
				*creating a data set of marker variable*;	
				%if %symexist(_dsimk_) %then %do;
					%let _rc_ = %sysfunc(close(&_dsimk_));
				%end;				
				
				/*
				data dsimk;
					set &ts1;
					where 0;
					keep &MarkerVars &ListMarkerVars &LockMarkerVars &ListLockMarkerVars;
				run;	
				*/
				
				data ts1vars;
					set &ts1;
					where 0;
				run;
				
				proc transpose data=ts1vars out=ts1varsT(drop=_label_);
					var _all_;
				run;
				
				/*data OutLib.gb_ts1varsT; set ts1varsT; run;*/
				
				proc sql UNDO_POLICY=NONE;
					create table dsimks(drop= _ord_) as
					select * 
					from MkInFileT as a inner join ts1varsT as b
					on a._name_=b._name_
					order by _ord_;
				quit;
				
				/*data OutLib.gb_dsimks; set dsimks; run;*/
				
				proc transpose data=dsimks out=dsimk(drop=_name_ _label_ where=(0));
					var _name_;
				run;
				
				/*data OutLib.gb_dsimk; set dsimk; run;*/
				
				%*let exiterror= 1;
				%*goto exit;
								
				%let _dsimk_ = %sysfunc(open(dsimk));
				%let nimk = %sysfunc(attrn(&_dsimk_,NVARS));
				%*let _rc_ = %sysfunc(close(&_dsimk_));
				%*put nimk = &nimk;
				
				%if &nimk = 0 %then %do; 
					%if (&ProcessName = GenomicBLUP) %then %do;
						%put ERROR: No marker variables selected. Some markers must be selected in order to proceed with the Genomic BLUP analysis. Please, redefine the criteria used in the Predictor Reduction tab.; 
					%end;
					%else %do;
						%put ERROR: No marker variables selected. Some markers must be selected in order to proceed with the Genomic Bayesian Regression analysis. Please, redefine the criteria used in the Predictor Reduction tab.;
					%end;
					
					%let exiterror= 1;
					%goto exit;
				%end;	
				
				%put nimk = &nimk;
				%*put nmrkin = &nmrkin;
				%if ^&CVFlag and %length(&AnnoInData) %then %do;
				
					*data mapfile;
					*	set AnnoLib.&AnnoData;
					*run;				

					%if &nimk < &nmrkin or %length(&AnnoWhereStmt) %then %do;
						/*
						proc transpose data=dsimk out=dsimkt;
							var 
								%do i=1 %to &nimk; 
									%sysfunc(varname(&_dsimk_,&i)) 
								%end;
							;
						run;
						*/

						/* 
						The code used to require that chromosome variable be numeric and in ascending order. 
						The new code relax that requirement by allowing numeric or character chromosome variable
						with levels grouped, not necessarely sorted. In this code, a new numeric chromosome
						variable, TmpChromNum, is created such that if the check for grouping of chromosome 
						levels in the code below passes, this variable is in ascending order as needed. 
						In addition, a new variable, TmpAnnoInOrd, is created to make sure that the chromosome 
						and marker order in the input annotation data set is kept the same if sorting is ever
						needed in the code 
						*/
	
						proc sql UNDO_POLICY=NONE;
							create table submapfile as
							select * 
							from &MapFile as a right join /*dsimkt*/ dsimks as b
							on a.&MarkerLab=b._name_
							/*order by &ChromNum, &MarkerPos*/
							order by &InAnnoOrd
							;
						quit;

						*proc datasets nolist;
						*	delete mapfile;
						*run;

						%global AnnoOutData;
						%let AnnoOutData= ;
						%PrepOutDS(AnnoOutData,AnnoOutDataName,&&&_outdataname,_ann);

						data OutLib.&AnnoOutDataName;
							set submapfile;
							drop _name_ _label_ &InAnnoOrd &ChromNum;
						run;

						%let mapdat= submapfile;
					
					%end;
					%else %do;
						%let mapdat= &MapFile;
					%end;
					
					proc transpose data= &mapdat out=MapTrans(drop= _NAME_);
						var &MarkerLab;
						id &MarkerLab;
					run;
					
					/*data OutLib.mapdat; set &mapdat; run;*/
					
					%let _dsmap_ = %sysfunc(open(MapTrans));
					%let nmrkmap = %sysfunc(attrn(&_dsmap_,NVARS));
					%if &AsymmetricLossFitProp ne 0.5 %then %do;
						%let _rc_ = %sysfunc(close(&_dsmap_));
					%end;
				%end;

				*creating a data of non-marker variable*;
				%let niov= 0;
				%*put OrigPred= &OrigPredictorVars &OrigListContPredVars &OrigLockContPredVars &OrigListLockContPredVars; 
				%if %length(%trim(&OrigPredictorVars &OrigListContPredVars &OrigLockContPredVars &OrigListLockContPredVars)) %then %do;
				
					%if %symexist(_dsiov_) %then %do;
						%let _rc_ = %sysfunc(close(&_dsiov_));
					%end;
								
					data dsiov;
						set &ts1;
						where 0;
						*This keep statement assumes markers cannot be class variables*;
						keep 
							&OrigPredictorVars &OrigListContPredVars &OrigLockContPredVars &OrigListLockContPredVars 
							/*&ClassPredVars &ListClassPredVars &LockClassPredVars &ListLockClassPredVars*/
							%do i=1 %to &nipl; %sysfunc(varname(&_dsipl_,&i)) %end;
							%do i=1 %to &nip; %sysfunc(varname(&_dsip_,&i)) %end;
						;
					run;	
				
					%let _dsiov_ = %sysfunc(open(dsiov));
					%let niov = %sysfunc(attrn(&_dsiov_,NVARS));
					%*let _rc_ = %sysfunc(close(&_dsiov_));
				%end;
				%*put niov = &niov;
				
				%if %length(&StandardizationMethod) and &Flag_RecoverInData %then %do;
					data &ts1;
						merge 
						&ts1(drop= %do i=1 %to &nimk; %sysfunc(varname(&_dsimk_,&i)) %end; ) 
						d_nostd(keep= %do i=1 %to &nimk; %sysfunc(varname(&_dsimk_,&i)) %end; );
					run;
					
					/*data OutLib.ts11_&ts1; set &ts1; run;*/
				%end;
				
				%*put intestdat = &intestdat;
				%*put TestSpec = &TestSpec;
				
				/*
				%if %length(&ts2) %then %do;
					data OutLib.ts20; set &ts2; run;
				%end;
				*/
				
				%if %length(&StandardizationMethod) and &Flag_RecoverTestData %then %do;
					data &ts2;
						merge 
						&ts2(drop= %do i=1 %to &nimk; %sysfunc(varname(&_dsimk_,&i)) %end; ) 
						d_test_nostd(keep= %do i=1 %to &nimk; %sysfunc(varname(&_dsimk_,&i)) %end; );
					run;	
					
					/*data OutLib.ts21; set &ts2; run;*/			
				%end;
				
				%if (&ProcessName = GenomicBayesianRegression) %then %do;
					* remove rows with missing predictors;				
					data d_nomisspred;
						set &ts1;
						%if %length(%trim(%left(&LockContPredVars &LockClassPredVars &PredictorVars &ClassPredVars))) %then %do;
							array pv{*} &LockContPredVars &LockClassPredVars 
							%if ^&upred %then %do; 
								&PredictorVars 
							%end; 
							%else %do; 
								%do j=1 %to &nix_blocks; 
									&&PredictorVars&j 
								%end; 
							%end; 
							&ClassPredVars;
							do _i_=1 to dim(pv);
								if missing(pv[_i_]) then do;
									delete;
									leave;
								end;
							end;
						%end;
						%if %length(&WeightVar) %then %do;
							if missing(&WeightVar) or &WeightVar = 0 then delete;
						%end;
						;
						_rowindex_= _n_;
					run;
					
					%NObsVars(&ts1);
					%let n_in= &nobs;
					
					%NObsVars(d_nomisspred);
					%let n_out= &nobs;
					
					
				%end;
				%else %if (&ProcessName = GenomicBLUP) %then %do;				
					**S1382957: allow analysis of markers with missing values**;
					data d_nomisspred;
						set &ts1;
						%if &niov %then %do;
							*removing rows with at least one missing value for predictor variables*;
							array pv{*} %do i=1 %to &niov; %sysfunc(varname(&_dsiov_,&i)) %end;;
							do _i_=1 to dim(pv);
								if missing(pv[_i_]) then do;
									delete;
									leave;
								end;
							end;
						%end;
						*removing rows with missing values for all markers*;
						_sum_= sum(of %do i=1 %to &nimk; %sysfunc(varname(&_dsimk_,&i)) %end;);
						if missing(_sum_) then delete;
					run;
				%end;
				
				/*data OutLib.d_nomisspred; set d_nomisspred; run;*/
				
				%*goto exit;
				
				/*
				data d_nomisspred;
					set &ts1;
					if not missing(1) 
						%do i=1 %to &nixl; and not missing(%sysfunc(varname(&_dsixl_,&i))) %end; 
						%do i=1 %to &nipl; and not missing(%sysfunc(varname(&_dsipl_,&i))) %end; 
						%do i=1 %to &nix; and not missing(%sysfunc(varname(&_dsix_,&i))) %end; 
						%do i=1 %to &nip; and not missing(%sysfunc(varname(&_dsip_,&i))) %end; 
						%if %length(&WeightVar) %then %do;
							and not missing(&WeightVar) and &WeightVar > 0
						%end;
					;
				run;
				*/
				
				%let ts1orig= &ts1;
				%let ts1 = d_nomisspred;
				%put ts1 = &ts1;
				
				%*put CVFlag = &CVFlag;
				%*put ts2 = &ts2;
				%*put TestSpec = &TestSpec;

				%if ^&CVFlag and (&TestSpec = Yes) and (&Kernel^=Dot) %then %do;
					data b2;
						set d d_test;
					run;
					%let ts2 = b2;
				%end;

				* using missing values for the dependent variable to create predictions;
				%if &CVFlag or (&TestSpec=Yes) or (&Kernel^=Dot) %then %do;
					data &ts1;
						set &ts1;
						_y0 = &DepVar;
						_y = &DepVar;
					run;
					data ts1o;
						set &ts1;
					run;
					data &ts2;
						set &ts2;
						_y0 = &DepVar;
						%if (&_dtype_=C) %then %do;
							_y = " ";
						%end;
						%else %do;
							_y = .;
						%end;
					run;
					data &ts1;
						set &ts1 &ts2;
						_obs = _n_;
						_one = 1;
						_name_ = "";
						drop _name_;
					run;
					%let UseDep = _y;
				%end;
				%else %do;
					%let UseDep = &DepVar;
					data &ts1;
						set &ts1;
						_obs = _n_;
						_one = 1;
						_name_ = "";
						drop _name_;
					run;
				%end;

				data d_nomiss;
					set &ts1;
					if not missing(&UseDep);
				run;

				%NObsVars(d_nomiss);
				%let nnmiss = &nobs;
				%put nnmiss = &nnmiss;

				%let uss1 = 1;
				/*
				%let uss1 = &npv;
				*/
				%put uss1 = &uss1;
				%let ts1prev = &ts1;
				
				/*data OutLib.ts12; set &ts1; run;*/
				
				**start: Building additive and dominance relationship matrices**; 
				proc means data=&ts1 noprint;
					var 
						/*
						%do i=1 %to &nixl; %sysfunc(varname(&_dsixl_,&i)) %end; 
						%do i=1 %to &nix; %sysfunc(varname(&_dsix_,&i)) %end;
						*/
						%do i=1 %to &nimk; %sysfunc(varname(&_dsimk_,&i)) %end;
					;
					output out=mean_mrk 
						mean= 
						/*
						%do i=1 %to &nixl; %sysfunc(varname(&_dsixl_,&i)) %end; 
						%do i=1 %to &nix; %sysfunc(varname(&_dsix_,&i)) %end; 
						*/
						%do i=1 %to &nimk; %sysfunc(varname(&_dsimk_,&i)) %end;
					;
				run;
				
				/*data OutLib.mean_mrk01; set mean_mrk; run;*/
				
				proc transpose data=mean_mrk out=mean_mrk;
					var 
						/*
						%do i=1 %to &nixl; %sysfunc(varname(&_dsixl_,&i)) %end; 
						%do i=1 %to &nix; %sysfunc(varname(&_dsix_,&i)) %end; 
						*/
						%do i=1 %to &nimk; %sysfunc(varname(&_dsimk_,&i)) %end;
					;
				run;
				
				/*data OutLib.mean_mrk02; set mean_mrk; run;*/

				%let SwitchGeno= 0;
				data MAF_Data; 
					set mean_mrk(rename=(col1=mean_mk));
					tmp=0;
					SwitchG= 0;
					MinorAF= mean_mk/2; /*minor allele frequency assuming diploid*/
					MajorAF= 1 - MinorAF; /*major allele frequency assuming diploid*/
					if MinorAF > MajorAF then do;
						tmp= MajorAF;
						MajorAF= MinorAF;
						MinorAF= tmp;
						SwitchG= 1;
						call symputx("SwitchGeno", 1);
					end;
					drop tmp;
				run;
				
				/*data OutLib.MAF_Data; set MAF_Data; run;*/
				/*data OutLib.ts13; set &ts1; run;*/
								
				%*goto exit;
				
				%if (&ProcessName = GenomicBLUP) %then %do;
				
					%NObsVars(&ts1);
					%let nind= &nobs;

					%if &RandSVDFlag %then %do;
						%if %sysevalf(&RandSVD_K >= &nind) %then %do;
							%let RandSVDFlag= 0;
							%let RandSVD_errorFlag= 0;
						%end;
					%end;

					***S1382957: the IML code has been updated to allow analysis of markers with missing values, much the same way as the Genomic Heritability process**;
					%let PrintFrag= 0;
					%let AUnBiased= No;
					proc iml;
						start rsvd(_M, _O, _k, _q, _n, _seed) global(lambda, U);

							*1. Draw an n  k Gaussian random matrix.*;
							if IsEmpty(_O) then do;       
								_O = normal(j(_n, _k, _seed));
							end;
							%if &PrintFrag %then %do;
								print _O;
							%end;					

							*2. Form the m  k sample matrix Y = MO.*;
							Y = _M*_O;
							Free _O;
							*print Y;

							if _q then do;
								*2.1. subspace iterations Y= (MM`)^(q)MO.*;
								do i=1 to _q;
									CALL GSORTH(Q,R,lindep,Y);
									Z= _M`*Q;
									Free Q R Y;

									CALL GSORTH(Q,R,lindep,Z);
									Y= _M*Q;
									Free Q R Z;
								end;
							end;
							%if &PrintFrag %then %do;
								print Y;
							%end;

							*3. Form an m  k orthonormal matrix Q such that Y = QR.*;
							CALL GSORTH(Q,R,lindep,Y);
							%if &PrintFrag %then %do;
								print Q;
							%end;
							Free Y R;

							*4. Form the k  n matrix B = Q`M.*;
							B= Q`*_M;
							%if &PrintFrag %then %do;
								print B;
							%end;
							*create B_Mat from B;
							*append from B;

							*5. Compute the SVD of the small matrix B: B = UhatDiag(d)Vhat`*;
							ncb= ncol(B);
							nrb= nrow(B);
							if ncb > nrb then do;
								*either transpose B or select just non-zero columns from Uhat and lambda*;
								B= B`; 
								call svd(Vhat,lambda,Uhat,B);
							end;
							else do;
								call svd(Uhat,lambda,Vhat,B);
							end;
							Free B Vhat; 
							*create lambda_mat from lambda;
							*append from lambda;
							%if &PrintFrag %then %do;
								print Uhat lambda Vhat;
							%end;

							*6. Form the matrix U = QUhat*;
							U= Q*Uhat;
							free Q Uhat;
							%if &PrintFrag %then %do;
								print U;
							%end;
							*create U_Mat from U;
							*append from U;
						finish rsvd;

						do;
						**input marker data**;
						use &ts1;
						read /*next 20*/ all 
							var {
								/*
								%do i=1 %to &nixl; %sysfunc(varname(&_dsixl_,&i)) %end; 
								%do i=1 %to &nix; %sysfunc(varname(&_dsix_,&i)) %end;
								*/
								%do i=1 %to &nimk; %sysfunc(varname(&_dsimk_,&i)) %end;
							} /*("&FirstMrk":"&LastMrk")*/ into M;
						close &ts1;
						nind= nrow(M);
						nmrk= ncol(M);

						**input allele frequency data**;
						use MAF_Data;
						read /*next 10*/ all var {MinorAF MajorAF} into af;
						close MAF_Data;

						**Switch 0 and 2 codes if frequency of 2 is greater than frequency of 0. The code 2 is the reference minor allele**;
						%if &SwitchGeno %then %do;
							use MAF_Data;
							read /*next 10*/ all var {SwitchG} into sg;
							close MAF_Data;
							loc1= loc(sg = 1);
							*print loc1;
							T= M[,loc1];
							T[loc(T= 2)]= -9;
							T[loc(T= 0)]=  2;
							T[loc(T=-9)]=  0;
							M[,loc1]= T;
							*print M;
							free T;

							*create OutLib.gb_M_data from M;
							*append from M;						
						%end;

						**compute additive matrix**;
						lab= "_ADD1":"_ADD&nind";
						W= M - 2*shape(af[,1], nind, nmrk);
						/*
						For the data of Technow2015, VanRaden method 1 is producting additive matrix with tiny values, 
						and in turn the variance component is getting to big. On the other hand, VanRaden method 2 
						produces additive matrix with values that seem more within the expected range, and the 
						variance component is well estimated.
						*/
						*W_STD= (W#(1/sqrt(2*sum(af[,1]#af[,2])))); /*VanRaden (J Dairy Sci. 441423, 2008) method 1 and Azevedo et al. (BMC Genetics 16:105, 2015)*/ 
						W_STD= (W#(1/(sqrt(2*af[,1]#af[,2])))`); /*Equation 2 - centered additive relationship matrix from Yang et al. (Nat Genet. 2010;42:5659) and VanRaden method 2*/

						free %if %upcase(&IncludeDom) ^= YES %then M af; W;

						*Scaling each entry of the A the matrix by the number of non-missing markers*;
						missm= loc(W_STD =.);
						if ncol(missm) then do;
							W_STD0 = W_STD;
							W_STD0[ missm ] = 0;   /* replace missing values with 0 */
							non0= W_STD;
							non0[ loc(W_STD ^= .) ] = 1;   /* replace non missing values with 1 */
							non0[ missm ] = 0;   /* replace missing values with 0 */
							free W_STD;

							*create OutLib.gb_W_STD0 from W_STD0;
							*append from W_STD0;

							*create OutLib.gb_non0 from non0;
							*append from non0;			

							Nnon0= non0*non0`;	
							free non0;
							WWt= (W_STD0*W_STD0`);
							free W_STD0;

							%if %upcase(&AUnBiased) = YES %then %do;
								*DiagWWt= (M##2 - (1 + 2*af[,1]`)#M + 2*((af[,1]##2)`))*J(nmrk, 1, 1);  
								Z= (M##2 - (1 + 2*af[,1]`)#M + 2*((af[,1]##2)`));
								Z[missm]= 0;
								DiagWWt= Z*J(nmrk, 1, 1);
								*J(nmrk, 1, 1);  
								*create OutLib.gb_DiagWWt from DiagWWt;
								*append from DiagWWt;
								diagIdx = do(1, nind*nind, nind+1);
								WWt[diagIdx]= vecdiag(Nnon0) + DiagWWt;
								free DiagWWt diagIdx Z;
							%end;		

							*A = (WWt);
							*A = (WWt)#(1/Nnon0);
						end;
						else do;
							Nnon0= J(nind, nind, nmrk);
							WWt= (W_STD*W_STD`);
							free W_STD;

							%if %upcase(&AUnBiased) = YES %then %do;
								*b1= (1 + 2*af[,1]`)#M ;
								*create OutLib.gb_b1 from b1;
								*append from b1;					
								*b2= 2*((af[,1]##2)`);
								*create OutLib.gb_b2 from b2;
								*append from b2;		
								*b3= (M##2 - (1 + 2*af[,1]`)#M + 2*((af[,1]##2)`));
								*create OutLib.gb_b3 from b3;
								*append from b3;						
								DiagWWt= (M##2 - (1 + 2*af[,1]`)#M + 2*((af[,1]##2)`))*J(nmrk, 1, 1);  
								*DiagA= 1 + (M##2 - (1 + 2*af[,1]`)#M + 2*((af[,1]##2)`))*J(nmrk, 1, 1);  
								*create OutLib.gb_DiagA from DiagA;
								*append from DiagA;
								diagIdx = do(1, nind*nind, nind+1);
								WWt[diagIdx]= vecdiag(Nnon0) + DiagWWt;
								*A[diagIdx]= DiagA;
								free DiagWWt diagIdx;
							%end;	

							*A = (WWt);
							*A = (WWt)#(1/Nnon0);
						end;

						ZeroCount= loc(Nnon0 = 0);
						if ncol(ZeroCount) then do;
							call symputx("FlagZeroCount", 1);  
							goto TheEnd;
						end;
						else do;
							call symputx("FlagZeroCount", 0); 
						end;					

						A = (WWt)#(1/Nnon0);
						free WWt Nnon0;

						*create OutLib.gb_A_Mat from A (|colname=lab|);
						*append from A;	

						**input k, q and seed parameters for the randomized SVD**;
						RandSVDFlag= &RandSVDFlag;
						k= &RandSVD_K;
						q= &RandSVD_Q;
						seed= &RandSVD_seed;
						FrobFlag= &RandSVD_errorFlag;

						**compute square root of addtive matrix**;
						n= nrow(A);
						nlim= n/1.5;
						if RandSVDFlag & k <= nlim & k > 0 then do;
							O= {};
							run rsvd(A, O, k, q, n, seed);
							if ^FrobFlag then do;
								free A;
							end;
						end;
						else do;
							call svd(U,lambda,V,A);
							if ^FrobFlag then do;
								free A V;
							end;
							else do;
								free V;
							end;
						end;

						if (RandSVDFlag & k <= nlim & k > 0) | ^RandSVDFlag then do;
							UL=U#sqrt(lambda)`;
							*UL=U*diag(sqrt(lambda));
						end;
						else if (RandSVDFlag & k > nlim) then do;
							UL=U[,1:k]#sqrt(lambda[1:k])`;
							*UL=U[,1:k]*diag(sqrt(lambda[1:k]));
						end;

						*print lambda UL;
						free U lambda;
						create A_SVD_RT from UL (|colname=lab|);
						append from UL;

						if RandSVDFlag & FrobFlag then do;
							*Rebuild the input matrix from the square root matrix U = Udiag(d)U`*;
							A_re= UL*UL`;
							free UL;
							%if &PrintFrag %then %do;
								print A_re;
							%end;
							*create A_SVD from A_re;
							*append from A_re; 

							*Computing the Frobenius measure*;
							Frob= 100*norm(A - A_re, "Frobenius")/norm(A, "Frobenius");
							free A A_re;					
							Frob= k||q||Frob;
							FrobLab= {k q Frob_Percent};
							create Frob_A from Frob (|colname= FrobLab|);
							append from Frob;
						end;
						else do;
							free UL;
						end;

						**compute dominance matrix**;
						%if %upcase(&IncludeDom) = YES %then %do;
							lab= "_DOM1":"_DOM&nind";
							M= round(M);
							S= 2*(-(M=2)#(af[,2]##2)` + (M=1)#(af[,1]#af[,2])` - (M=0)#(af[,1]##2)`);
							free M;
							D= (S*S`)/(4*(sum((af[,1]#af[,2])##2)));
							free af S;

							*create OutLib.gb_D_Mat from D(|colname=lab|);
							*append from D;	

							**compute square root of dominance matrix**;
							n= nrow(D);
							nlim= n/1.5;
							if RandSVDFlag & k <= nlim & k > 0 then do;
								O= {};
								run rsvd(D, O, k, q, n, seed);
								if ^FrobFlag then do;
									free D;
								end;
							end;
							else do;
								call svd(U,lambda,V,D);
								if ^FrobFlag then do;
									free D V;
								end;
								else do;
									free V;
								end;
							end;

							if (RandSVDFlag & k <= nlim & k > 0) | (^RandSVDFlag) then do;
								UL=U#sqrt(lambda)`;
								*UL=U*diag(sqrt(lambda));
							end;
							else if (RandSVDFlag & k > nlim) then do;
								UL=U[,1:k]#sqrt(lambda[1:k])`;
								*UL=U[,1:k]*diag(sqrt(lambda[1:k]));
							end;
							*print U lambda;
							free U lambda;
							create D_SVD_RT from UL (|colname=lab|);
							append from UL;

							if RandSVDFlag & FrobFlag then do;
								*Rebuild the input matrix from the square root matrix U = Udiag(d)U`*;
								D_re= UL*UL`;
								free UL;
								%if &PrintFrag %then %do;
									print D_re;
								%end;
								*create D_SVD from D_re;
								*append from D_re; 

								*Computing the Frobenius measure*;
								Frob= 100*norm(D - D_re, "Frobenius")/norm(D, "Frobenius");
								free D D_re;					
								Frob= k||q||Frob;
								FrobLab= {k q Frob_Percent};
								create Frob_D from Frob (|colname= FrobLab|);
								append from Frob;
							end;
							else do;
								free UL;
							end;						
						%end;

						TheEnd:
						end;
					quit;
					**finish: Building additive and dominance relationship matrices**; 

					%if &syserr > 4 %then %do;
						%let exiterror=1;
						%goto exit;
					%end;

					%if &FlagZeroCount %then %do;
						%put ERROR: Due to excessive missing marker genotypes the additive relationship matrix could not be computed.;
						%let exiterror= 1;
						%goto exit;
					%end;

					%*goto exit;

					/*data OutLib.A_SVD_RT; set A_SVD_RT; run;*/

					%let nlim= %sysevalf(&nind/1.5);
					%put nlim= &nlim;
					%if %upcase(&RandSVD) = YES and ^&RandSVDFlag %then %do;
						%let String1= The target rank (k= &RandSVD_K) of the low-rank SVD is not smaller than the number of observations (n= &nind) with non-missing values values for the dependent variable. Therefore, neither randomized nor truncated low-rank SVD was computed, instead, a full-rank SVD was computed.;
						%put WARNING: &String1; 					
					%end;
					%else %if &RandSVDFlag and %sysevalf(&RandSVD_K > &nlim) %then %do;
						%let String1= Truncated SVD was computed instead of randomized SVD because the target rank (k= &RandSVD_K) of the low-rank randomzed SVD  is not smaller than the number of observations (n= &nind) with non-missing values values for the dependent variable divided by 1.5 (i.e., k > n/1.5).;
						%put WARNING: &String1; 	
					%end;

					%if &RandSVDFlag %then %do;
						%let SVD_SETTING_TEXT= %nrbquote(k= )&RandSVD_K;
						%if %sysevalf(&RandSVD_K <= &nlim) %then %do;
							%let SVD_SETTING_TEXT= &SVD_SETTING_TEXT %nrbquote(and q= )&RandSVD_Q;
						%end;
					%end;

					%if &RandSVDFlag and &RandSVD_errorFlag %then %do;
						%let SVD_ERROR_TEXT= ;
						data _null_ /*OutLib.Frob_A*/;
							set Frob_A;
							call symputx("RSVD_AError", trim(left(Frob_Percent)));
						run;
						%let SVD_ERROR_TEXT= Additive matrix= %sysfunc(round(&RSVD_AError,.11))%nrbquote(%); 
						%put SVD_ERROR_TEXT= &SVD_ERROR_TEXT;
					%end;				

					%if %upcase(&IncludeDom) = YES %then %do;
						/*data OutLib.D_SVD_RT; set D_SVD_RT; run;*/
						%if &RandSVDFlag and &RandSVD_errorFlag %then %do;
							data _null_ /*OutLib.Frob_D*/;
								set Frob_D;
								call symputx("RSVD_DError", trim(left(Frob_Percent)));
							run;
							%let SVD_ERROR_TEXT= &SVD_ERROR_TEXT.%nrbquote(, Dominance matrix=) %sysfunc(round(&RSVD_DError,.11))%nrbquote(%); 
							%put SVD_ERROR_TEXT= &SVD_ERROR_TEXT;
						%end;
					%end;
					
					/*data OutLib.gb_ts1; set &ts1; run;*/ 

					data &ts1;
						merge 
							&ts1(/**drop=&LockContPredVars &LockClassPredVars %if ^&upred %then %do; &PredictorVars %end; %else %do; %do j=1 %to &nix_blocks; &&PredictorVars&j %end; %end; &ClassPredVars**/) 
							A_SVD_RT
							%if %upcase(&IncludeDom) = YES %then %do;
								D_SVD_RT
							%end;
						;
					run;

					%let WhichProc= &SASProc;

					%*put WhichProc= &WhichProc;
					%*put ProcOption = &ProcOption;
					%*put ClassVariables = &ClassVariables;
					%*put UseDep = &UseDep;
					%*put FixedEffects = &FixedEffects;
					%*put ModOption = &ModOption;
					%*put OrderInt = &OrderInt;
					%*put HetVar = &HetVar;
					%*put ProcStatements = &ProcStatements;
					%*put WeightVar = &WeightVar;
					%*put uss1 = &uss1;
					%*put nnmiss = &nnmiss;

					/*data OutLib.gb_ts14; set &ts1; run;*/

					proc &WhichProc data=&ts1 &ProcOption namelen=128;
						class &ClassVariables 
							%if &Kernel ^= Dot /**or ((&npv >= &nnmiss) and (&OrderInt = 1))**/ %then %do; _obs %end; 
						;
						%if %upcase(&WhichProc) = MIXED %then %do;
							model &UseDep = &FixedEffects / &ModOption solution
								outp=ts2o
								%if /**(&npv < &nnmiss) or**/ &Kernel ^= Dot %then %do;
									(drop=&LockContPredVars &LockClassPredVars 
									%if ^&upred %then %do; 
										&PredictorVars 
									%end; 
									%else %do; 
										%do j=1 %to &nix_blocks; 
											&&PredictorVars&j 
										%end; 
									%end; 
									&ClassPredVars
										/*
										%do i=1 %to &nixl; %sysfunc(varname(&_dsixl_,&i)) %end; 
										%do i=1 %to &nipl; %sysfunc(varname(&_dsipl_,&i)) %end; 
										%do i=1 %to &nix; %sysfunc(varname(&_dsix_,&i)) %end; 
										%do i=1 %to &nip; %sysfunc(varname(&_dsip_,&i)) %end; 
										*/
									)
								%end;
							;
						%end;
						%else %if %upcase(&WhichProc) = HPMIXED %then %do;
							model &UseDep = &FixedEffects / &ModOption solution;
						%end;
						%else %if %upcase(&WhichProc) = GLIMMIX %then %do;
							model &UseDep(event=last) = &FixedEffects / &ModOption dist=&DataDistribution;
						%end;
						%else %do;
							%put ERROR: Invalid SAS PROC.;
							%let exiterror= 1;
							%goto exit;					
						%end;
						/*
						%if %length(&RandEffects) %then %do;
							random &RandEffects / %if (&HetVar^=Yes) %then %do; type=toep(1) %end; s;
						%end;
						*/
						%if &niov %then %do;
							random %do i=1 %to &niov; %sysfunc(varname(&_dsiov_,&i)) %end; 
								/ %if (&HetVar^=Yes) %then %do; type=toep(1) %end; s;
						%end;	
						%if ^%index(%quote(&SelVars),Intercept) %then %do;
							%if &Kernel = Dot /**and ((&npv < &nnmiss) or (&OrderInt > 1))**/ %then %do;
								random _ADD: / type=toep(1) s;
								%if %upcase(&IncludeDom) = YES %then %do;
									random _DOM: / type=toep(1) s;
								%end;
								ods output solutionr=blup;
								ods exclude solutionr;
								%if %upcase(&WhichProc) = HPMIXED %then %do;
									output out=ts2o(drop=&LockContPredVars &LockClassPredVars %if ^&upred %then %do; &PredictorVars %end; %else %do; %do j=1 %to &nix_blocks; &&PredictorVars&j %end; %end; &ClassPredVars
										) pred(blup)=pred residual(blup)=resid;							
								%end;
								%else %if %upcase(&WhichProc) = GLIMMIX %then %do;
									output out=ts2o(drop=&LockContPredVars &LockClassPredVars %if ^&upred %then %do; &PredictorVars %end; %else %do; %do j=1 %to &nix_blocks; &&PredictorVars&j %end; %end; &ClassPredVars
										/*
										%do i=1 %to &nixl; %sysfunc(varname(&_dsixl_,&i)) %end; 
										%do i=1 %to &nipl; %sysfunc(varname(&_dsipl_,&i)) %end; 
										%do i=1 %to &nix; %sysfunc(varname(&_dsix_,&i)) %end; 
										%do i=1 %to &nip; %sysfunc(varname(&_dsip_,&i)) %end; 
										*/
										) pred(blup ilink)=pred residual(blup ilink)=resid;
								%end;
							%end;
						%end;
						%if %length(&WeightVar) %then %do;
							weight &WeightVar;
						%end;
						%else %if (&Priors^=Proportional) %then %do;
							weight _Prior;
						%end;
						&ProcStatements;
						ods output covparms=cp 
							%if %upcase(&WhichProc) = HPMIXED %then %do; 
								ParameterEstimates=sf
							%end;
							%else %do; 
								solutionf=sf 
							%end;
						;
						ods exclude classlevels;
					run;						

					%put syserr = &syserr;
					%if &syserr > 4 %then %do;
						%let exiterror = 1;
						%goto exit;
					%end;

					%let ts1 = &ts1prev;

					%NObsVars(ts2o);
					%if ^&nobs %then %do;
						%put ERROR: PROC &WhichProc did not converge when fitting a model to response variable &DepVar..  You may want to add a PARMS statement or change the model.;
						%let exiterror = 1;
						%goto exit;
					%end;

					/*data OutLib.ts2o0; set ts2o; run;*/

					%put UseDep = &UseDep;					

					* BLUP processing;	
					%if &Kernel = Dot and (^&CVFlag or &NPred or %length(&BLUPWhere)) %then %do;

						/*data OutLib.ts2o; set ts2o; run;*/

						data ts2o_nomiss;
							set ts2o;
							if not missing(&UseDep);
							%if %upcase(&WhichProc) ^= HPMIXED %then %do; 
								drop stderrpred df alpha lower upper;
							%end;
						run;
						data _null_;
							set sf;
							where Effect = "Intercept";
							call symputx("Formula_Intercept",trim(left(Estimate)));
						run;
						%put Formula_Intercept = &Formula_Intercept;

						%if ^%index(%quote(&SelVars),Intercept) %then %do;

							data blup;
								set blup;
								blup = estimate;
								abs_blup = abs(estimate);
								rename estimate=blup effect=_name_;
							run;

							/* Luciano
							%if %length(&BLUPWhere) %then %do;
								data blup;
									set blup;
									where %unquote(&BLUPWhere);
								run;

								%if ^&NPred %then %let NPred = 999999999999;
							%end;
							*/

							%let suffix = _gb_blup;
							%let b32 = %eval(32 - %length(&suffix));
							%if %length(&OutName) > &b32 %then %let inname = %substr(&OutName,1,&b32);
							%else %let inname = &OutName;		
							%let gbout = %quote(&inname.%trim(%left(&suffix)));
							%put gbout = &gbout;

							data OutLib.%unquote(&gbout);
								set blup;
							run;

							*NPred = 0 for the GenomicBLUP *;
							%goto skipNpred0;

							%if &NPred %then %do;

								%if &nixl or &nipl %then %do;
									data blup_lock;
										set blup(obs=%eval(&nixl+&nipl));
									run;
									data blup;
										set blup(firstobs=%eval(&nixl+&nipl+1));
									run;
								%end;

								proc sort data=blup;
									by descending abs_blup;
								run;

								%macro SASName2(inname,outname,unique=1,compbl=0);
										length name name2 $ 32;
										name = compress(trim(left(&inname)));
										len = lengthn(name);
										j=1;
										do until(j=0);
											j=notname(name,j+1);
											if j>0 then substr(name,j,1)=' ';
										end;
										%if &compbl %then %do;
											name = translate(compbl(name),'_','');
										%end;
										%else %do;
											name = translate(name,'_','');		
										%end;
										name = substr(name,1,len);
										if substr(name,1,1) in ('0','1','2','3','4','5','6','7','8','9') then do;
											name2='_'||name;
										end;
										else do;
										   name2=name;
										end;
										%if &unique %then %do;
											if length(name2) = 32 then do;
												name2 = substr(name2,1,24) || trim(left(_n_));
											end;
										%end;
										&outname = trim(left(name2));
								%mend;

								%NObsVars(blup);
								%if &nobs < &NPred %then %let NPred = &nobs;

								data blup;
									set blup(obs=&NPred);
									_label_ = _name_;
									%SASName2(_name_,_name_);
								run;

								/*
								data OutLib.blup2;
									set blup;
								run;
								*/

								proc transpose data=blup out=_dsib_(drop=_name_);
									var abs_blup;
								run;

								/*
								data OutLib.dsib;
									set _dsib_;
								run;
								*/

								%let _dsib_ = %sysfunc(open(_dsib_));

								%if &nixl or &nipl %then %do;
									data blup_lock;
										set blup_lock;
										rename _name_ = _label_;
									run;
								%end;

								data ts2vars;
									%if &nixl or &nipl %then %do;
										set blup_lock blup;
									%end;
									%else %do;
										set blup;
									%end;
									* keep _label_;
									rename _label_ = name;
								run;
								%let havets2v = 1;

								* re-standardize predictors row-wise so that sscp becomes corr matrix;
								%if (&StandardizeRows = Yes) and (&nixl or &nipl or &NPred) and (&OrderInt = 1) %then %do;
									data &ts1;
										set &ts1;
										array pv{*} 
											%do i=1 %to &nixl; %sysfunc(varname(&_dsixl_,&i)) %end; 
											%do i=1 %to &nipl; %sysfunc(varname(&_dsipl_,&i)) %end; 
											%do i=1 %to &NPred; %sysfunc(varname(&_dsib_,&i)) %end; 
										;
										_mean_ = mean(of pv{*});
										_std_ = std(of pv{*});
										do _i_=1 to dim(pv);
											pv[_i_] = (pv[_i_]-_mean_)/_std_;
										end;
										drop _i_;
									run;
								%end;

								%let uss1full = &uss1;
								%let npv = %eval(&nixl+&nipl+&NPred);
								%put npv reduced = &npv;
								%let uss1 = 1;
								/*
								%let uss1 = &npv;
								%let npv = 10000000;
								*/
								%put uss1 reduced = &uss1;

								/*
								data OutLib.ts1;
									set &ts1;
									keep &DepVar
										%do i=1 %to &nixl; %sysfunc(varname(&_dsixl_,&i)) %end; 
										%do i=1 %to &nipl; %sysfunc(varname(&_dsipl_,&i)) %end; 
										%do i=1 %to &NPred; %sysfunc(varname(&_dsib_,&i)) %end; 
									;
								run;
								*/

								%if ^%index(%qupcase(&ProcStatements),PARMS) and (&HetVar^=Yes) %then %do;
									data cp;
										set cp;
										if covparm = "Residual" and Estimate < 1.1e-6 then Estimate = 1.1e-6;
									run;
									/*
									data OutLib.cp;
										set cp;
									run;
									*/
								%end;

								/*
								%put NPred = &NPred;
								%do i=1 %to &NPred; 
									%put blup &i = %sysfunc(varlabel(&_dsib_,&i));
								%end;
								*/

								* second proc mixed run using reduced predictor set;
								proc &WhichProc data=&ts1 &ProcOption namelen=128;						
									class &ClassVariables %if (&npv >= &nnmiss) and (&OrderInt = 1) %then %do; _obs %end; ;
									model &UseDep = &FixedEffects / &ModOption solution
										outp=ts2o
										%if (&npv < &nnmiss) %then %do;
											(drop=&LockContPredVars &LockClassPredVars 
												%if ^&upred %then %do; 
													&PredictorVars 
												%end; 
												%else %do; 
													%do j=1 %to &nix_blocks; 
														&&PredictorVars&j 
													%end; 
												%end; 
												&ClassPredVars
												)
												/*
												(drop = %do i=1 %to &nixl; %sysfunc(varname(&_dsixl_,&i)) %end; 
												%do i=1 %to &nipl; %sysfunc(varname(&_dsipl_,&i)) %end; 
												%do i=1 %to &nix; %sysfunc(varname(&_dsix_,&i)) %end; 
												%do i=1 %to &nip; %sysfunc(varname(&_dsip_,&i)) %end;) 
												*/

										%end;
									;
									%*if (&npv < &nnmiss) or (&OrderInt > 1) %then %do;
										random 
											%do i=1 %to &nixl; 
												%sysfunc(varname(&_dsixl_,&i)) 
												%if &OrderInt > 1 %then %do;
													%do j = 2 %to &OrderInt; | %sysfunc(varname(&_dsixl_,&i)) %end;
													%if ((&i < &nixl) or (&nipl > 0)) %then %do; | %end;
												%end;
											%end; 
											%do i=1 %to &nipl; 
												%sysfunc(varname(&_dsipl_,&i)) 
												%if &OrderInt > 1 and (&i < &nipl) %then %do; | %end;
											%end; 
											%if (&nixl or &nipl) and (&OrderInt > 1) %then %do;
												@ &OrderInt
											%end;
											%do i=1 %to &NPred; 
												%sysfunc(varlabel(&_dsib_,&i)) 											
											%end; 
											/ %if (&HetVar^=Yes) %then %do; type=toep(1) %end; s;
										ods output solutionf=sf solutionr=blup;
										ods exclude solutionr;
									%*end;
									%if %length(&WeightVar) %then %do;
										weight &WeightVar;
									%end;
									%else %if (&Priors^=Proportional) %then %do;
										weight _Prior;
									%end;
									&ProcStatements;
									ods exclude classlevels;
								run;

								%put syserr = &syserr;
								%if &syserr > 4 %then %do;
									%let exiterror = 1;
									%goto exit;
								%end;
								%let ts1 = &ts1prev;

								* compute BLUPs again directly if necessary;


								%let _dsib_ = %sysfunc(close(&_dsib_));
							%end;
							%else %do;
								%if (&npv >= &nnmiss) %then %do;
									data ts2o;
										merge ts2o pred;
										if not missing(&UseDep) then do;
											pred = col1;
											resid = &UseDep - col1;
										end;
										drop col1;
									run;
								%end;
							%end;

							%skipNPred0:;
						%end;
					%end;

					/*data OutLib.ts2o1; set ts2o; run;*/

					%NObsVars(ts2o);
					%if &nobs = 0 %then %do;
						%put ERROR: PROC MIXED did not converge or produce predictions.;
						%put If cross-validating, you may have constant data.  If not, please ;
						%put inspect the SAS Output and try specifying convergence options or a different model.;
						%let exiterror = 1;
						/* %goto exit; */
					%end;	
					%else %do;

						* S1452712: including fixed effect variables from the Options tab*;
						%let ClassOrFixedFlag = 0;
						%if %length(&ClassVariables &FixedEffects) %then %do;

							%let ncla= 0;
							%if %length(&ClassVariables) %then %do;
								data classvarnames;
									set sf;
									keep &ClassVariables;
									where 0;
								run;

								%NObsVars(classvarnames);
								%let ncla= &nvars;
							%end;
							%put ncla= &ncla;

							%let nfix= 0;
							%if %length(&FixedEffects) %then %do;
								data fixedvarnames;
									set &ts1;
									keep &FixedEffects;
									where 0;
								run;

								%NObsVars(fixedvarnames);
								%let nfix= &nvars;
							%end;
							%put nfix= &nfix;						

							%if &ncla and &nfix %then %do;
								%let ClassOrFixedFlag = 1;
								%let dsidc = %sysfunc(open(classvarnames));
								%let reff = %sysfunc(varname(&dsidc,1));

								/*S1465688: making length for name equal to 32*/
								data sf2;
									set sf;
									length Parameter name $32 /*Estimate 8*/;
									parameter = effect;
									name= effect;
									%do i=1 %to &ncla; 
										if not missing(&reff) then do;
											parameter= trim(left(effect))||" "||trim(left(%sysfunc(varname(&dsidc,&i))));
											name= trim(left(effect))||"_"||trim(left(%sysfunc(varname(&dsidc,&i))));
										end;
									%end; 
								run;
								/*data OutLib.sf2; set sf2; run;*/

								%let dsidc = %sysfunc(close(&dsidc));
							%end;
							%else %if ^&ncla and &nfix %then %do;
								%let ClassOrFixedFlag = 1;

								data sf2;
									set sf;
									length Parameter $ 32 /*Estimate 8*/;
									parameter = effect;
									name = effect;
								run;	

								/*data OutLib.sf2; set sf2; run;*/
							%end;
						%end;					
					
						%if &Kernel = Dot and ^&CVFlag %then %do;
						
							%if &ClassOrFixedFlag %then %do;
								data pe;
									set sf2;
									drop name;
								run;
							%end;					
							%else %do;						
								data pe;
									length Parameter $ 32 Estimate 8;
									Parameter = "Intercept";
									Estimate = &Formula_Intercept;
								run;
							%end;
							
							%if ^%index(%quote(&SelVars),Intercept) %then %do;
								data pe1;
									length Parameter $ 32 Estimate 8;
									set blup;
									Parameter = _name_;
									Estimate = blup;
									keep Parameter Estimate;
								run;
								data pe;
									set pe pe1;
								run;
							%end;

							%ScoringCodeMacroVars;

							%let suffix= _gb_pe;
							%let b32 = %eval(32 - %length(&suffix));
							%if %length(&OutName) > &b32 %then %let inname = %substr(&OutName,1,&b32);
							%else %let inname = &OutName;		
							%let peout = %quote(&inname.%trim(%left(&suffix)));
							%put peout = &peout;

							data OutLib.%unquote(&peout);
								set pe1;
							run;

							*source of variance is allocated respecting the order in which they appear in the PROC MIXED or HPMIXED*;
							data source_name;
								length source $50;
								%if &niov %then %do;
									%if (&HetVar^=Yes) %then %do;
										source= "Continuous Predictors";
										output;
									%end;
									%else %do;
										%do i=1 %to &niov; 
											source= "%sysfunc(varname(&_dsiov_,&i))"; 
											output;
										%end; 
									%end;
								%end;
								source = "Additive";
								output;
								%if %upcase(&IncludeDom) = YES %then %do;
									source= "Dominance";
									output;
								%end;
								source = "Residual";
								output;
							run;						

							%let suffix= _gb_cp;
							%let b32 = %eval(32 - %length(&suffix));
							%if %length(&OutName) > &b32 %then %let inname = %substr(&OutName,1,&b32);
							%else %let inname = &OutName;		
							%let cpout = %quote(&inname.%trim(%left(&suffix)));
							%put cpout = &cpout;

							data OutLib.%unquote(&cpout);
								merge source_name cp;
								label source = "Source of Variance";
							run;						

						%end;
						%if ^&CVFlag and (&TestSpec = Yes) and (&Kernel^=Dot) %then %do;
							data &ts1;
								set ts1o;
							run;
							data &ts2;
								set d_test;
							run;
						%end;

						%put GB TraitType = &TraitType;				
						%put GB TruthVar = &TruthVar;
						%put AsymmetricLossFitProp= &AsymmetricLossFitProp;

						%if &AsymmetricLossFitProp ne 0.5 %then %do;
							data ts1a;
								set ts2o;
								&DepVar = &UseDep;
								keep _Row_ &DepVar pred &WeightVar &WeightVar0;
							run;
						%end;

						* create variables for computing cv ase;
						data ts2o;
							set ts2o;
							%if &CVFlag or (&Kernel^=Dot) %then %do;
								%if &_dtype_ = C %then %do;
									where &UseDep = " ";
								%end;
								%else %do;
									where &UseDep = .;
								%end;
								&DepVar = _y0;
								drop _y0 _y;
							%end;
							%if (&TraitType=Binary) %then %do;
								if missing(pred) then do;
									p_&DepVarTrunc._0 = .;
									p_&DepVarTrunc._1 = .;
									_INTO_ = .;
								end;
								else do;
									p_&DepVarTrunc._0 = 1 - pred;
									if p_&DepVarTrunc._0 < 0 then p_&DepVarTrunc._0 = 0;
									else if p_&DepVarTrunc._0 > 1 then p_&DepVarTrunc._0 = 1;
									p_&DepVarTrunc._1 = 1 - p_&DepVarTrunc._0;
									_INTO_ = (p_&DepVarTrunc._1 > 0.5);
								end;
								%if &_dtype_ = C %then %do;
									if &TruthVar = " " then
								%end;
								%else %do;
									if &TruthVar = . then
								%end;
									actualp = .;
								else if (&TruthVar=1) then actualp = p_&DepVarTrunc._1;
								else actualp = p_&DepVarTrunc._0;
								/*
								%let ncl = 2;
								%let pprob1 = p_&DepVarTrunc._0;
								%let pprob2 = p_&DepVarTrunc._1;
								*/
							%end;
						run;	
					%end;

					%let into = _INTO_;
					%let pred = pred;

					%if ^&CVFlag %then %do;
						*This data set name will be used in the follow up Progeny Simulation AP*;
						%let suffix = _gb_sl;
						%let b32 = %eval(32 - %length(&suffix));
						%if %length(&OutName) > &b32 %then %let inname = %substr(&OutName,1,&b32);
						%else %let inname = &OutName;		
						%let slout = %quote(&inname.%trim(%left(&suffix)));
						%put slout = &slout;
					%end;

					/*%let _rc_ = %sysfunc(close(&_dsimk_));*/
					%if &niov %then %let _rc_ = %sysfunc(close(&_dsiov_));
					
				%end; /*End of GenomicBlup*/
				%else %if (&ProcessName = GenomicBayesianRegression) %then %do;	
									
					/*data OutLib.gbr_ts1; set &ts1; run;*/

					**genotype data*;
					data genodat;
						set &ts1;
						keep %do i=1 %to &nimk; %sysfunc(varname(&_dsimk_,&i)) %end;;
					run;
					
					/**S1462562: To address the issue with variables starting with 
					underscore used in R, the prefix X is added to the variable's 
					name starting with underscore because this is the standard in 
					SAS 9.4 M5 **/					
					data tmpt_rand; 
						length _newvar_ _oldvar_ $32;
						set dsimks;
						if find(trim(left(_name_)),"_")=1 then do;
							_newvar_= "X"||_name_;
							_oldvar_= _name_;
							_unflag_= 1;
						end;
						else do;
							_newvar_= _name_;
							_oldvar_= _name_;
							_unflag_= 0;
						end;
						drop _name_;
					run;
					
					**phenotype data*;
					%let OrigUseDep = &UseDep;
					%let UseDep= %trim(%left(&UseDep));
					%let UnderscorePrefixFlag = %sysfunc(find(&UseDep,_));
					%if &UnderscorePrefixFlag = 1 %then %do;
						/**S1462562: To address the issue with variables starting with 
						underscore used in R, the prefix X is added to the variable's 
						name starting with underscore because this is the standard in 
						SAS 9.4 M5 **/					
						%let DepForR_= X&UseDep;
					%end;
					%else %do;
						%let DepForR_= &UseDep;
					%end;
					%put DepForR_= &DepForR_;					
					
					data phenodat;
						set &ts1(keep= &UseDep);
						%if &UnderscorePrefixFlag = 1 %then %do;
							&DepForR_= &UseDep;
							keep &DepForR_;
						%end;
					run;
					
					/*data OutLib.gbr_phenodat; set phenodat; run;*/
					
					%if %length(&GroupsVar) %then %do;
					
						data groupdat;
							set &ts1;
							keep &GroupsVar;
						run;
					
					%end;

					%if %length(&WeightVar) %then %do;
					
						data weightdat;
							set &ts1;
							keep &WeightVar;
						run;
					%end;
					
					%if %length(&CensorLBVar) %then %do;
						
						data censorLBdat;
							set &ts1;
							keep &CensorLBVar;
						run;
					
					%end;
					
					%if %length(&CensorUBVar) %then %do;
						
						data censorUBdat;
							set &ts1;
							keep &CensorUBVar;
						run;
					
					%end;
																			
					%global ETAPar;
					%let ETAPar= ;
					
					%if %upcase(&BayesModel) = BAYESA %then %do; 
						%let ETAPar= %str(MAR=list(X = &GenoOjbName, model = 'BayesA'));
					%end;
					%else %if %upcase(&BayesModel) = BAYESB %then %do; 
						%let ETAPar= %str(MAR=list(X = &GenoOjbName, model = 'BayesB'));
					%end;
					%else %if %upcase(&BayesModel) = BAYESC %then %do; 
						%let ETAPar= %str(MAR=list(X = &GenoOjbName, model = 'BayesC'));
					%end;					
					%else %if %upcase(&BayesModel) = BAYESIANLASSO %then %do; 
						%let ETAPar= %str(MAR=list(X = &GenoOjbName, model = 'BL'));
					%end;					
					%else %if %upcase(&BayesModel) = BRR %then %do; 
						%let ETAPar= %str(MAR=list(X = &GenoOjbName, model = 'BRR'));
					%end;
					
					%if %length(&PedVars) %then %do;
						
						%if (&n_in = &n_out) %then %do;
							data peddat;
								set &ts1;
								keep &PedVars;
							run;
						%end;
						%else %do;
							/*S1465671: Deleting columns in the pedigree data corresponding 
							to deleted rows to maintain the pedigree data matrix square.*/
							
							proc transpose data= &ts1(keep= &PedVars) out=pedt;
							run;

							data pedt;
								set pedt;
								_row_= _n_;
							run;

							proc sql UNDO_POLICY= NONE;
								create table pedt(drop= _rowindex_ _row_) as
								select *
								from d_nomisspred(keep=_rowindex_) as a left join pedt as b
								on a._rowindex_ = b._row_
								order by _row_;
							quit;

							proc transpose data=pedt out=peddat(drop=_name_);
								id _name_;
							run;
						%end;
						
						/*data OutLib.gbr_peddat; set peddat; run;*/

						%let ETAPar= %str(&ETAPar, PED=list(K = &PedOjbName, model = 'RKHS'));
					%end;
					
					%let RandEffVarsFlag= 0;
					%if &niov %then %do;
						%let RandEffVarsFlag= 1;
					
						data predictorsdat;
							set &ts1;
							keep %do i=1 %to &niov; %sysfunc(varname(&_dsiov_,&i)) %end;; 
							
						run;
						
						/**S1462562: To address the issue with variables starting with 
						underscore used in R, the prefix X is added to the variable's 
						name starting with underscore because this is the standard in 
						SAS 9.4 M5 **/

						proc transpose data=predictorsdat(where= (0)) out=tmp0t;
						run;

						data tmp0t;
							length _newvar_ _oldvar_ $32;
							set tmp0t;
							if find(trim(left(_name_)),"_")=1 then do;
								_newvar_= "X"||_name_;
								_oldvar_= _name_;
								_unflag_= 1;
							end;
							else do;
								_newvar_= _name_;
								_oldvar_= _name_;
								_unflag_= 0;
							end;
							drop _name_;
						run;

						proc transpose data=tmp0t out=tmp0tt(drop=_name_);
							var _newvar_;
							id _newvar_;
						run;
						
						%let _tmp0tt_ = %sysfunc(open(tmp0tt));						
						
						%let RandEffectsModel= %sysfunc(varname(&_tmp0tt_,1));
						%do i=2 %to &niov; 
							%let RandEffectsModel= &RandEffectsModel %str(+) %sysfunc(varname(&_tmp0tt_,&i)); 
						%end;
						%let _rct_ = %sysfunc(close(&_tmp0tt_));
						
						%let ETAPar= %str(&ETAPar, RAN=list(~ &RandEffectsModel, data= &RandEffOjbName, model='BRR'));
						
						data tmpt_rand;
							set tmpt_rand tmp0t;
						run;
						
					%end;
					
					data tmpt_fixed;
						delete;
					run;
			
					%let FixedEffVarsFlag = 0;
					%if %length(&FixedEffects) %then %do;
						%let FixedEffVarsFlag = 1;
						
						data fixedeffdat;
							set &ts1;
							keep &FixedEffects; 
						run;						
						
						%let ncla= 0;
						%if %length(&ClassVariables) %then %do;
							data classvarnames;
								set &ts1;
								keep &ClassVariables;
								where 0;
							run;
	
							%NObsVars(classvarnames);
							%let ncla= &nvars;
						%end;
						%put ncla= &ncla;
						
						%*let nfix= 0;
						%*if %length(&FixedEffects) %then %do;
							data fixedvarnames;
								set &ts1;
								keep &FixedEffects;
								where 0;
							run;
	
							%NObsVars(fixedvarnames);
							%let nfix= &nvars;
						%*end;
						%put nfix= &nfix;						
				
						%if &ncla and &nfix %then %do;
						
							proc transpose data=classvarnames out=classvarnamest(rename=(_name_=classvar));
							run;

							proc transpose data=fixedvarnames out=fixedvarnamest(rename=(_name_=fixedvar));
							run;
							
							proc sql UNDO_POLICY=NONE;
								create table LeftJoinFixedVars as 
								select *
								from fixedvarnamest as a left join classvarnamest as b
								on a.fixedvar = b.classvar
								;
							quit;

							data FixedEffectNonClassVars FixedEffectClassVars;
								set LeftJoinFixedVars;
								if missing (classvar) then output FixedEffectNonClassVars;
								else output FixedEffectClassVars;
								label fixedvar= ;
								label classvar= ;
							run;
							
							/*data OutLib.gbr_FixedEffectNonClassVars; set FixedEffectNonClassVars; run;*/
							/*data OutLib.gbr_FixedEffectClassVars; set FixedEffectClassVars; run;*/

							/*
							proc transpose data=FixedEffectNonClassVars(keep=fixedvar) out= FixedEffectNonClassVarst(drop=_name_ where=(0));
								var fixedvar;
								id fixedvar;
							run;
							
							data OutLib.gbr_FixedEffectNonClassVarst; set FixedEffectNonClassVarst; run;
							
							%let dsidf1 = %sysfunc(open(FixedEffectNonClassVarst));
							%let nvarf1= %sysfunc(attrn(&dsidf1,NVARS));	
							*/
														
							/**S1462562: To address the issue with variables starting with 
							underscore used in R, the prefix X is added to the variable's 
							name starting with underscore because this is the standard in 
							SAS 9.4 M5 **/
							
							**Fixed effect non-class variables**;
							data tmp1t;
								length _newvar_ _oldvar_ $32;
								set FixedEffectNonClassVars(keep=fixedvar rename=(fixedvar=_name_));
								if find(trim(left(_name_)),"_")=1 then do;
									_newvar_= "X"||_name_;
									_oldvar_= _name_;
									_unflag_= 1;
								end;
								else do;
									_newvar_= _name_;
									_oldvar_= _name_;
									_unflag_= 0;
								end;
								drop _name_;
							run;
	
							proc transpose data=tmp1t out=tmp1tt(drop=_name_);
								var _newvar_;
								id _newvar_;
							run;

							%let dsidf1 = %sysfunc(open(tmp1tt));
							%let nvarf1= %sysfunc(attrn(&dsidf1,NVARS));	


							**Fixed effect class variables**;
							/*
							proc transpose data=FixedEffectClassVars(keep=classvar) out= FixedEffectClassVarst(drop=_name_ where=(0));
								var classvar;
								id classvar;
							run;	
							
							data OutLib.gbr_FixedEffectClassVarst; set FixedEffectClassVarst; run;

							%let dsidf2 = %sysfunc(open(FixedEffectClassVarst));
							%let nvarf2= %sysfunc(attrn(&dsidf2,NVARS));
							*/

							data tmp2t;
								length _newvar_ _oldvar_ $32;
								set FixedEffectClassVars(keep=classvar rename=(classvar=_name_));
								if find(trim(left(_name_)),"_")=1 then do;
									_newvar_= "X"||_name_;
									_oldvar_= _name_;
									_unflag_= 1;
								end;
								else do;
									_newvar_= _name_;
									_oldvar_= _name_;
									_unflag_= 0;
								end;
								drop _name_;
							run;
	
							proc transpose data=tmp2t out=tmp2tt(drop=_name_);
								var _newvar_;
								id _newvar_;
							run;

							%let dsidf2 = %sysfunc(open(tmp2tt));
							%let nvarf2= %sysfunc(attrn(&dsidf2,NVARS));

						%end;
						%else %if ^&ncla and &nfix %then %do;
						
							/*data OutLib.gbr_fixedvarnames; set fixedvarnames; run;*/
							
							/*
							%let dsidf1 = %sysfunc(open(fixedvarnames));	
							%let nvarf1= %sysfunc(attrn(&dsidf1,NVARS));							
							*/
							
							/**S1462562: To address the issue with variables starting with 
							underscore used in R, the prefix X is added to the variable's 
							name starting with underscore because this is the standard in 
							SAS 9.4 M5 **/
							
							**Fixed effect non-class variables**;
							proc transpose data=fixedvarnames out=fixedvarnamest(rename=(_name_=fixedvar));
							run;							
														
							data tmp1t;
								length _newvar_ _oldvar_ $32;
								set fixedvarnamest(keep=fixedvar rename=(fixedvar=_name_));
								if find(trim(left(_name_)),"_")=1 then do;
									_newvar_= "X"||_name_;
									_oldvar_= _name_;
									_unflag_= 1;
								end;
								else do;
									_newvar_= _name_;
									_oldvar_= _name_;
									_unflag_= 0;
								end;
								drop _name_;
							run;
	
							proc transpose data=tmp1t out=tmp1tt(drop=_name_);
								var _newvar_;
								id _newvar_;
							run;

							%let dsidf1 = %sysfunc(open(tmp1tt));
							%let nvarf1= %sysfunc(attrn(&dsidf1,NVARS));							

							data tmp2t;
								set _null_;
							run;

							data tmp2tt;
								set _null_;
							run;
							%let dsidf2= %sysfunc(open(tmp2tt));
							%let nvarf2= %sysfunc(attrn(&dsidf2,NVARS));

						%end;
						%put nvarf1= &nvarf1;
						%put nvarf2= &nvarf2;
						
						%*goto exit;
						
						%let FixedEffectsModel= ;
						%if &nvarf1 %then %do;
							%let FixedEffectsModel= %sysfunc(varname(&dsidf1,1));
							%do i=2 %to &nvarf1; 
								%let FixedEffectsModel= &FixedEffectsModel %str(+) %sysfunc(varname(&dsidf1,&i)); 
							%end;
							
							data tmpt_fixed;
								set tmpt_fixed tmp1t;
							run;
							
						%end;

						%if &nvarf2 %then %do; 
							%if &nvarf1 %then %do;
								%let FixedEffectsModel= &FixedEffectsModel %str(+) %nrbquote(factor( %sysfunc(varname(&dsidf2,1)) ) );
							%end;
							%else %do;
								%let FixedEffectsModel= %nrbquote(factor( %sysfunc(varname(&dsidf2,1)) ) );
							%end;
							%do i=2 %to &nvarf2; 
								%let FixedEffectsModel= &FixedEffectsModel %str(+) %nrbquote(factor( %sysfunc(varname(&dsidf2,&i)) ) ); 
							%end;
							
							data tmpt_fixed;
								set tmpt_fixed tmp2t;
							run;							
						%end;
						
						%let dsidf1 = %sysfunc(close(&dsidf1));
						%let dsidf2 = %sysfunc(close(&dsidf2));
						%put FixedEffectsModel= &FixedEffectsModel;
						
						%let ETAPar= %str(&ETAPar, FIX=list(~ &FixedEffectsModel, data= &FixEffOjbName, model='FIXED'));
						
					%end;
					
					%if %length(&ETAPar) %then %do;						
						%let ETAPar= %str(list(&ETAPar));
					%end;
					%else %do;
						%let ETAPar= NULL;
					%end;
					%put ETAPar= &ETAPar;	
					
					/*data OutLib.gbr_tmpt_rand; set tmpt_rand; run;*/
					/*data OutLib.gbr_tmpt_fixed; set tmpt_fixed; run;*/
				
				       	options nonotes;
					ods _all_ close;
					*ods html body="&Outpath.&OutName..html";
					proc iml; 
						/*NOTES:
						(1) Must keep module calls in the order they have been written here because
							the output from one module may be used in the next.
						(2) R objects created by a module can be accessed by any other module as far as 
							modules calls are keept within the same block defined by "Proc iml" and "quit".
						This fact can be used to avoid having to exchange matrices or datasets between R and IML.
						*/	
						
						****loading modules*;
						load module=(RemoveRObjs);
						load module=(SaveRData);
						load module=(SetRMemory);
						load module=(LoadRPkgs);
						load module=(LoadRFunc);
						load module=(ExportDataToR);
						load module=(CallBGLRFunction);

						%if %upcase(&IncreaseRMemory)= YES %then %do;
							*sets R max memory size*;
							run SetRMemory(&RMemorySize);
						%end;

						****loads R packages*;
						run LoadRPkgs;
						if ^isOK then do;
							call symputx("exiterror", 1);
							abort;
						end;

						****loads R functions*;
						*run LoadRFunc("&RSourcePath");
						*if ^isOK then do;
						*	call symputx("exiterror", 1);
						*	abort;
						*end;

						****exports genotypic data from SAS to R*;
						run ExportDataToR("genodat", "&GenoOjbName");
						
						****exports phenotypic data from SAS to R*;
						run ExportDataToR("phenodat", "&PhenoOjbName");
						
						%if %length(&GroupsVar) %then %do;
							****exports group variable data from SAS to R*;
							run ExportDataToR("groupdat", "&GroupOjbName");							
						%end;
						
						%if %length(&WeightVar) %then %do;
							****exports weight variable data from SAS to R*;
							run ExportDataToR("weightdat", "&WeightOjbName");							
						%end;	

						%if %length(&CensorLBVar) %then %do;
							****exports right censoring variable data from SAS to R*;
							run ExportDataToR("censorLBdat", "&CensorLBOjbName");							
						%end;

						%if %length(&CensorUBVar) %then %do;
							****exports left censoring variable data from SAS to R*;
							run ExportDataToR("censorUBdat", "&CensorUBOjbName");							
						%end;
						
						%if %length(&PedVars) %then %do;
							****exports pedigree relationship matrix data from SAS to R*;
							run ExportDataToR("peddat", "&PedOjbName");							
						%end;						

						%if &niov %then %do;
							****exports predictors data from SAS to R*;
							run ExportDataToR("predictorsdat", "&RandEffOjbName");							
						%end;
						
						%if %length(&FixedEffects) %then %do;
							****exports pedigree relationship matrix data from SAS to R*;
							run ExportDataToR("fixedeffdat", "&FixEffOjbName");							
						%end;							
						
						****saves R console image*;
						/*
						run SaveRData("&ROutPath", "&OutName");
						if ^isOK then do;
							call symputx("exiterror", 1);
							abort;
						end;
						*/
						
						run CallBGLRFunction("&PhenoOjbName$&DepForR_", "&ReponseTypePar", "&CensorLBPar", "&CensorUBPar", 
									"&ETAPar", &NumbIter, &NumbBurn, &ThinRate, 
									"&SaveAtPar", &S0Par, &DF0Par, &R2Par, "&WeightPar",
									"&VerbosePar", "&RmExistingFilesPar", "&GroupPar", 
									&FixedEffVarsFlag, &RandEffVarsFlag);

						if ^isOK then do;
							call symputx("exiterror", 1);
							abort;
						end;
						
						****saves R console image*;
						/*
						run SaveRData("&ROutPath", "&OutName");
						if ^isOK then do;
							call symputx("exiterror", 1);
							abort;
						end;
						*/						
					
					quit;
					
					options notes;
					
					%if &TraitType = Binary or &TraitType = Ordinal %then %do;
						/*
						data OutLib.gbr_pred; set pred; run;
						data OutLib.gbr_beta0; set beta0; run;
						data OutLib.gbr_betas; set betas; run;	
						data OutLib.gbr_probs; set probs; run;
						data OutLib.gbr_threshold; set threshold; run;
						*/
						
						%if &TraitType = Binary %then %do;
							data _null_;
								set threshold;
								call symputx("Formula_Intercept",trim(left(Estimate)));
							run;	
							%let Formula_Intercept= &Formula_Intercept;
						%end;
						%else %do;
							data _null_;
								set threshold end=_eof_;
								call symputx("Formula_Level"||trim(left(_n_)),trim(left(Response))); 
								call symputx("Formula_Intercept"||trim(left(_n_)),trim(left(Estimate)));
								if _eof_ then call symputx("nIntercept",trim(left(_n_))); 
							run;	
							%let Formula_Intercept= &Formula_Intercept1;
						%end;
						
					%end;
					%else %do;
						data _null_;
							set beta0;
							where Effect = "Intercept";
							call symputx("Formula_Intercept",trim(left(Estimate)));
						run;					
					%end;
					%put Formula_Intercept = &Formula_Intercept;					
					
					%let UseDep= &OrigUseDep;
										
					data ts2o;
						merge &ts1(drop= &LockContPredVars &LockClassPredVars 
							%if ^&upred %then %do; 
								&PredictorVars 
							%end; 
							%else %do; 
								%do j=1 %to &nix_blocks; 
									&&PredictorVars&j 
								%end; 
							%end; 
							&ClassPredVars
						) pred;
					run;
					
					%*goto exitGBR;
					
					%if &TraitType ^= Ordinal %then %do;
						data intercept;
							length Parameter $ 32 Estimate 8;
							Parameter = "Intercept";
							Estimate = &Formula_Intercept;
						run;
					%end;
					%else %do;
						data intercept;
							length Parameter $32 Response Estimate 8;
							%do i=1 %to &nIntercept;
								Parameter = "Intercept";
								Response = &&Formula_Level&i;
								Estimate = &&Formula_Intercept&i;
								output;
							%end;
						run;
					%end;
					
					* S1452712: including fixed effect variables from the Options tab*;
					%let ClassOrFixedFlag = 0;
					%if &FixedEffVarsFlag %then %do;
						
						%let ClassOrFixedFlag = 1;
						
						/*data OutLib.gbr_fixedeff; set fixedeff; run;*/
						
						%*goto exit;
												
						data fixedeff;
							set fixedeff;
							length _Rname_ $32;
								_Rname_= trim(left(tranwrd(tranwrd(tranwrd(_Effect_, "factor", ""),"(",""),")","")));
								if find(_Rname_,"") then do;
									_Rname1_= scan(_Rname_,1);
									_Rname2_= scan(_Rname_,2);
								end;
								else do;
									_Rname1_= _Rname_;
									_Rname2_= "";									
								end;
							_ord_= _n_;
						run;	
						
						proc sql UNDO_POLICY=NONE;
							create table fixedeff_final as
							select *
							from fixedeff as a left join tmpt_fixed as b
							on a._Rname1_ = b._newvar_
							order by _ord_;
						quit;	

						data sf2;
							set intercept fixedeff_final(in=a);
							length name $32;
							if a then do;
								Parameter= trim(left(_oldvar_))||""||trim(left(_Rname2_));
								Estimate= _Estimate_;
								if not missing(_Rname2_) then name= trim(left(_oldvar_))||"_"||trim(left(_Rname2_));
								else  name= trim(left(_oldvar_));
							end;
							else do;
								name= "Intercept";
							end;
							_ord_= _n_;
							keep Parameter name Estimate %if &TraitType = Ordinal %then Response;;
						run;	
												
						/*
						data sf2;
							set intercept fixedeff(in=a);
							length _Rname_ name $32 ;
							if a then do;
								parameter= trim(left(tranwrd(tranwrd(tranwrd(_Effect_, "factor", ""),"(",""),")","")));
								_Rname_= trim(left(tranwrd(tranwrd(tranwrd(_Effect_, "factor", ""),"(",""),")","")));
								name= trim(left(tranwrd(tranwrd(tranwrd(_Effect_, "factor", ""),"(",""),")","_")));
								Estimate= _Estimate_;
							end;
							else do;
								name= "Intercept";
							end;
							_ord_= _n_;
							drop _Effect_ _Estimate_;
						run;	
						*/							
				
					%end;
						
					%*if &Kernel = Dot and ^&CVFlag %then %do;
					
						%if &ClassOrFixedFlag %then %do;
										
							data pe;
								set sf2;
								drop name;
							run;
							
						%end;					
						%else %do;
							data pe;
								set intercept;
								/*
								length Parameter $ 32 Estimate 8;
								Parameter = "Intercept";
								Estimate = &Formula_Intercept;
								*/
							run;
						%end;
						
						/*data OutLib.gbr_pe; set pe; run;*/
					
						%*put SelVars= &SelVars;
						%if ^%index(%quote(&SelVars),Intercept) %then %do;
						
							/*data OutLib.gbr_betas2; set betas; run;*/

							data pe1;
								length Parameter $ 32 Estimate 8;
								set betas;
								Parameter = _Effect_;
								Estimate = _Estimate_;
								keep Parameter Estimate;
								
							run;
							
							%if &RandEffVarsFlag %then %do;
							
								/*data OutLib.gbr_randeff; set randeff; run;*/	
							
								data randeff;
									length Parameter $ 32 Estimate 8;
									set randeff;
									Parameter = _Effect_;
									Estimate = _Estimate_;
									keep Parameter Estimate;
								run;
								
								data pe1;
									set randeff pe1;
								run;
								
							%end;							
							/*%else %do;
								data pe1;
									*length Parameter $ 32 Estimate 8;
									set betas;
									*Parameter = _Effect_;
									*Estimate = _Estimate_;
									*keep Parameter Estimate;
								run;
							%end;
							*/
							
							/*data OutLib.gbr_pe1; set pe1; run;*/
								
							/**S1462562: To address the issue with variables starting with 
							underscore used in R, the prefix X is added to the variable's 
							name starting with underscore because this is the standard in 
							SAS 9.4 M5 **/	
						
							data pe1;
								set pe1;
								_ord_= _n_;
								rename Parameter=_Rname_; 
							run;	
							
							proc sql UNDO_POLICY=NONE;
								create table pe1_final as
								select *
								from pe1 as a left join tmpt_rand as b
								on a._Rname_ = b._newvar_
								order by _ord_;
							quit;
							
							proc datasets nolist;
								delete pe1;
							quit;	
						
							data pe1;
								length Parameter $32;
								set pe1_final;
								if _Rname_ = "Intercept" then Parameter= _Rname_;
								else Parameter= _oldvar_;
								keep Parameter Estimate;
							run;
							
							data pe;
								set pe pe1;
							run;
							
						%end;
						
						/**S1462562: To address the issue with variables starting with 
						underscore used in R, the prefix X is added to the variable's 
						name starting with underscore because this is the standard in 
						SAS 9.4 M5 **/	
						
						/*
						data pe;
							set pe;
							_ord_= _n_;
							rename Parameter=_Rname_; 
						run;	
						
						proc sql UNDO_POLICY=NONE;
							create table pe_final as
							select *
							from pe as a left join tmpt_rand as b
							on a._Rname_ = b._newvar_
							order by _ord_;
						quit;
						
						proc datasets nolist;
							delete pe;
						quit;	
						
						data pe;
							length Parameter $32;
							set pe_final;
							if _Rname_ = "Intercept" then Parameter= _Rname_;
							else Parameter= _oldvar_;
						run;
						*/
						
						/*data OutLib.gbr_pe2; set pe; run;*/
						
						%ScoringCodeMacroVars;
						
						%let suffix= _gbr_pe;
						%let b32 = %eval(32 - %length(&suffix));
						%if %length(&OutName) > &b32 %then %let inname = %substr(&OutName,1,&b32);
						%else %let inname = &OutName;		
						%let peout = %quote(&inname.%trim(%left(&suffix)));
						%put peout = &peout;

						data OutLib.%unquote(&peout);
							set pe1;
						run;
							
					%*end;
					
					%*goto exitGBR;
					
					/*
					%if ^&CVFlag and (&TestSpec = Yes) and (&Kernel^=Dot) %then %do;
						data &ts1;
							set ts1o;
						run;
						data &ts2;
							set d_test;
						run;
					%end;
					*/
					
					/*data OutLib.gbr_ts2o1; set ts2o; run;*/

					%put GBR TraitType = &TraitType;				
					%put GBR TruthVar = &TruthVar;
										
					%if (&TraitType=Binary) or (&TraitType=Nominal) or (&TraitType=Ordinal) %then %do;
					
						* make sure values will result in valid sas names;
						data probs;
							set probs;
							rename 
								%do i=1 %to &nic1;
									p_&&_lev&i= p_&DepVarTrunc._&&_lev&i		
								%end;
							;
						run;

						* levels of depvar;
						data ts2o;
							set ts2o;
							%if &_dtype_ = C %then %do;
								%do i=1 %to &nic1; _lev&i = "&&_lev&i"; %end;
							%end;
							%else %do;
								%do i=1 %to &nic1; _lev&i = &&_lev&i; %end;
							%end;
						run;
						
						/*data OutLib.gbr_ts2o3; set ts2o; run;*/

						%put DS nic1 = &nic1;
						
						data ppn;
							set probs;
							where 0;
							keep p_:;
						run;
						
						proc transpose data=ppn out=ppnt;
							var p_:;
						run;
						
						/*data OutLib.gbr_ppnt; set ppnt; run;*/
						
						data _null_;
							set ppnt;
							call symputx("pp"||trim(left(_n_)),trim(left(_name_)));
						run;

						%do i=1 %to &nic1; 
							%put pp&i = &&pp&i;
						%end; 						
						
						/*data OutLib.gbr_ts2o4a; set ts2o; run;*/
						/*data OutLib.gbr_probs1; set probs; run;*/
					
						* merge and compute predicted classes;
						data ts2o;
							*merge ts2o ppt;
							merge ts2o probs;
							%if &CVFlag %then %do;
								if missing(&DepVar) then delete;
							%end;
							array pp %do i=1 %to &nic1; &&pp&i %end; ;
							array lev _lev1-_lev&nic1;
							maxp = 0;
							maxpi = 0;
							levi = .;
							do i=1 to dim(pp);
								if (pp[i] > maxp) then do;
									maxp = pp[i];
									maxpi = i;
								end;
								if (lev[i] = &TruthVar) then levi = i;
							end;
							I_&DepVarTrunc30 = lev[maxpi];
							if not missing(levi) then actualp = pp[levi];
							else actualp = .;
							drop /*_lev:*/ i maxp maxpi levi;
						run;
							
					%end;
					
					/*data OutLib.gbr_ts2o4; set ts2o; run;*/

					* create variables for computing cv ase;
					data ts2o;
						set ts2o;
						%if &CVFlag /*or (&Kernel^=Dot)*/ %then %do;
							%if &_dtype_ = C %then %do;
								where &UseDep = " ";
							%end;
							%else %do;
								where &UseDep = .;
							%end;
							&DepVar = _y0;
							drop _y0 _y;
						%end;
						/*
						%if (&TraitType=Binary) %then %do;
							if missing(pred) then do;
								p_&DepVarTrunc._0 = .;
								p_&DepVarTrunc._1 = .;
								_INTO_ = .;
							end;
							else do;
								p_&DepVarTrunc._0 = 1 - pred;
								if p_&DepVarTrunc._0 < 0 then p_&DepVarTrunc._0 = 0;
								else if p_&DepVarTrunc._0 > 1 then p_&DepVarTrunc._0 = 1;
								p_&DepVarTrunc._1 = 1 - p_&DepVarTrunc._0;
								_INTO_ = (p_&DepVarTrunc._1 > 0.5);
							end;
							%if &_dtype_ = C %then %do;
								if &TruthVar = " " then
							%end;
							%else %do;
								if &TruthVar = . then
							%end;
							actualp = .;
							else if (&TruthVar=1) then actualp = p_&DepVarTrunc._1;
							else actualp = p_&DepVarTrunc._0;
						%end;
						%else */ %if (&TraitType=Binary) or (&TraitType=Nominal) or (&TraitType=Ordinal) %then %do;
							array pp p_:;
							array lev _lev:;
							maxp = 0;
							maxpi = 0;
							levi = .;
							do i=1 to dim(pp);
								if (pp[i] > maxp) then do;
									maxp = pp[i];
									maxpi = i;
								end;
								if (lev[i] = &TruthVar) then levi = i;
							end;
							_INTO_ = lev[maxpi];
							if not missing(levi) then actualp = pp[levi];
							else actualp = .;
							* drop lev_: i maxp maxpi levi;
						%end;
					run;	
					
					/*data OutLib.gbr_ts2o5; set ts2o; run;*/
					
					%if (&TraitType=Binary) or (&TraitType=Nominal) or (&TraitType=Ordinal) %then %do;					
						%let into = I_&DepVarTrunc30;
						%let pred = P_&DepVarTrunc30;	
					%end;
					%else %do;
						%let into = _INTO_;
						%let pred = pred;					
					%end;
					
					%if ^&CVFlag %then %do;
						*This data set name will be used in the follow up Progeny Simulation AP*;
						%let suffix = _gbr_sl;
						%let b32 = %eval(32 - %length(&suffix));
						%if %length(&OutName) > &b32 %then %let inname = %substr(&OutName,1,&b32);
						%else %let inname = &OutName;		
						%let slout = %quote(&inname.%trim(%left(&suffix)));
						%put slout = &slout;
					%end;	

					/*%let _rc_ = %sysfunc(close(&_dsimk_));*/
					%if &niov %then %let _rc_ = %sysfunc(close(&_dsiov_));	
					
					%exitGBR:;

				%end; /*End of GenomicBayesianRegression*/
			%end; /*End of GenomicBlup and GenomicBayesianRegression*/			
			%else %do;
				%put ERROR: &ProcessName not implemented in PredictiveAnalysis;
				%let exiterror = 1;
				%goto exit;
			%end;
			
			* asymmetric loss calculations;
			%if &AsymmetricLossFitProp = 0.5 %then %do;
				%let aloss_iterating = 0;
			%end;
			%else %do;
				* update asymmetric weights, if they have not changed then stop iterating;
				%let nchanged = 0;
				data ts1a;
					set ts1a end=_last;
					retain nchanged 0;
					_wprev = &WeightVar;
					_resid = &DepVar - &pred;
					if _resid >= 0 then &WeightVar = &AsymmetricLossFitProp;
					else &WeightVar = 1 - &AsymmetricLossFitProp;
					%if %length(&WeightVar0) %then %do;
						&WeightVar = &WeightVar * &WeightVar0;
					%end;
					if _wprev ne &WeightVar then nchanged = nchanged + 1;
					if _last then call symputx('nchanged',trim(left(nchanged)));
					keep _Row_ &WeightVar;
				run;
				%put nchanged in asymmetric iteration &aloss_niter = &nchanged;
				%if ^&nchanged %then %do;
					%let aloss_iterating = 0;
				%end;
				%else %do;
					data &ts1;
						set &ts1;
						drop &WeightVar;
					run;
					data &ts1;
						merge &ts1 ts1a;
						by _Row_;
					run;
				%end;
			%end;
			%if &aloss_niter = &aloss_maxiter %then %let aloss_iterating = 0;
			%if &aloss_iterating %then %do;
				%let aloss_niter = %eval(&aloss_niter + 1);
			%end;
		%end;

		%if &AsymmetricLossFitProp ne 0.5 %then %do;
			title3;
			%if &aloss_niter = &aloss_maxiter %then %do;
				%put ERROR: Asymmetric loss calculations did not converge after &aloss_maxiter iterations.;
				%put You may want to try a fitting proportion different from &AsymmetricLossFitProp..;
				%let exiterror = 1;
				%goto exit;
			%end;
			%else %do;
				%put Asymmetric loss calculations converged in &aloss_niter iterations.;
			%end;
		%end;
		
	%end;
%end;

* Buckley-James loop update;
%if &BJ_Est = Yes %then %do;

	* first time through save depvar, predictions, residuals, and delta;
	%if &BJ_Niter=0 %then %do;
		data ts2obj;
			set ts2o(where=(not missing(&DepVarSave)));
			pbj&BJ_Niter = &pred;
			rbj = &DepVarSave - &pred;
			if &CensorVar in(%unquote(&CensorValues)) then dbj = 0;
			else dbj = 1;
			keep &DepVarSave pbj&BJ_Niter rbj dbj;
		run;
		
		/*
		data OutLib.ts2obj0;
			set ts2obj;
		run;
		*/
		
	%end;
	%else %do;
		* save predictions from every iteration to deal with potential oscillation;
		data ts2o1;
			set ts2o(where=(not missing(&DepVarSave)));
			keep &pred;
		run;
		
		data ts2obj;
			merge ts2obj ts2o1; 
			pbj&BJ_Niter = &pred;
			rbj = &DepVarSave - &pred;
			array pbj{*} pbj0-pbj&BJ_Niter;
			array se{*} se1-se&BJ_Niter;
			do i=1 to &BJ_Niter;
				se[i] = (&pred-pbj[i])**2;
			end;
			keep &DepVarSave rbj dbj pbj0-pbj&BJ_Niter se1-se&BJ_Niter;
		run;
		
		%if &BJ_Niter=1 %then %do;
			data OutLib.ts2obj1;
				set ts2obj;
			run;
		%end;
		
		* compute normalized rmse with all previous predictions;
		proc means data=ts2obj noprint;
			var &DepVarSave se1-se&BJ_Niter;
			output out=mse mean=md mse1-mse&BJ_Niter;
		run;
		
		%if &BJ_Niter=1 %then %do;
			data OutLib.mse1;
				set mse;
			run;
		%end;
		
		* check for convergence;
		data _null_;
			set mse;
			array mse{*} mse1-mse&BJ_Niter;
			rmse = 1e6;
			do i=1 to &BJ_Niter;
				rmse = min(rmse,sqrt(mse[i])/max(1e-4,abs(md)));
			end;
			call symputx("rmse_bj",trim(left(rmse)));
			if (rmse <= &BJ_RMSE_Tol) or (&BJ_Niter = &BJ_Maxiter) or (&BJ_OneMore = 1) then call symputx("BJ_Iterating",0);
		run;
		%put;
		%put Buckley-James Iteration=&BJ_Niter RMSE=&rmse_bj;
		%put;
	%end;

	
	* in some cases do one more iteration after convergence;
	%if ^&BJ_Iterating and ^&BJ_OneMore %then %do;
		* turn cvflag back on;
		%if &CVFlagSave %then %do;
			%if ^&CVFlag %then %do;
				%let CVFlag = 1;
				%let BJ_OneMore = 1;
			%end;
		%end;
		* restore test set;
		%if %length(&intestdatsave) %then %do;
			%let intestdat = &intestdatsave;
			%put intestdatsave = &intestdatsave;
			data &intestdat;
				length &DepVar 8;
				set &intestdat;
				&DepVar = &DepVarSave;
			run;
			%let BJ_OneMore = 1;
			
			data OutLib.dtest1more;
				set &intestdat;
			run;
		%end;
		%if (&ProcessName = PartitionTrees) %then %do;
			%if %upcase(&TreeMethod) = TREE %then %let BJ_OneMore = 1;
		%end;
		%if &BJ_OneMore %then %do;
			%put Doing one more Buckley-James iteration;
			%let BJ_Iterating = 1;
		%end;
	%end;
	

	* if not done, compute updated dependent variable;
	%if &BJ_Iterating %then %do;
		
		* modified code from Sam Glasson PhD thesis;
		proc iml;

			*************************************************************
			* Module for calculation of the product-limit estimator
			* Input:  time = sorted vector of event times (columnvector)
			*         status = vector of censor indicators (columnvector)
			* Output: res = n*2 matrix Column1=FHat Column2=SHat
			*************************************************************;
			start ple(time,status);
				n=nrow(time);
				nalive=n;
				nfail=0;
				redist=1/n;

				* Initialise fhat and shat;
				fhat=j(nalive,1,0);
				shat=j(nalive,1,0);

				* Calculate the fhats;
				do i=1 to n-1;
					nalive=nalive-1;
					if status[i]= 0 then do;
						redist=redist+redist/nalive;
					end;
					else do;
						fhat[i]=redist;
					end;
				end;

				fhat[n]=redist;

				* Calculate the shats, taking into account ties;
				do j=1 to n-1;
					k = 0;
					another = 1;
					do while(another);
						jk1 = j + k + 1;
						if jk1 > n then another = 0;
						else if status[jk1]^=1 | time[jk1]^=time[j] then another = 0;
						if another then k = k+1;
					end;
									
					/*  original code, bug when j+k+1 > n
					do while(status[j+k+1]=1 & time[j+k+1]=time[j]);
						k=k+1;
					end;
					*/
					
					do l=0 to k;
						if j=1 then shat[l+1]=1-fhat[1]*(k+1);
						else shat[j+l]=shat[j-1]-fhat[j]*(k+1);
					end;
					j=j+k;
				end;
				shat[n]=0;
				res=fhat||shat;
				return(res);
			finish;

			***********************************************************
			* Module for calculation of the Weights matrix
			* Input:  time=sorted vector of event times (columnvector)
			*         status=vector of censor indicators (columnvector)
			* Output: res = n*n weights matrix
			***********************************************************;
			start weight(time,status);
				plest=ple(time,status);
				w=diag(status);
				n=nrow(status);
				w[n,n]=1;
				do i=1 to n;
					if w[i,i]=0 then do j=(i+1) to n;
						if w[j,j]=1 then w[i,j]=plest[j,1]/plest[i,2];
					end;
				end;
				return(w);
			finish;

			************************************************
			* Main Buckley-James calculations
			************************************************;

			* Import data;
			use ts2obj;
			read all var{pbj&BJ_Niter rbj dbj} into c;
			* print c;
			n=nrow(c);
			* print n;
			* Extract relevant matrices;
			pred=c[,1];
			resid=c[,2];
			delta=c[,3];
			order=(1:n)`;
			* print pred resid delta;

			* Sort according to residuals and censoring;
			predtmp=pred;
			residtmp=resid;
			dlttmp=delta;
			ordrtmp=order;
			srtresid=resid-delta*1E-6;	
			rs = rank(srtresid[,1]);
			* print predtmp rs;
			pred[rs,]=predtmp;
			resid[rs,]=residtmp;
			delta[rs,]=dlttmp;
			order[rs,]=ordrtmp;
			* print srtresid pred resid delta order;

			* Calculate weight matrix;
			w = weight(resid,delta);
			* print w;

			* Construct renovated values;
			ystar = pred+w*resid;
			* print ystar;

			outbj = ystar || order;

			create outbj from outbj;
			append from outbj;
		quit;

		* merge updated depvar into data;
		proc sort data=outbj;
			by col2;
		run;
		
		data d_nomiss;
			set dsave(where=(not missing(&DepVarSave)));
		run;

		data d;
			merge d_nomiss(drop=&DepVar) outbj;
			rename col1=&DepVar;
		run;

		%if &BJ_Niter=0 %then %do;
			data outbj0;
				set outbj;
			run;
			data d0;
				set d;
			run;
		%end;
		
		data d_miss;
			set dsave(where=(missing(&DepVarSave)));
		run;
		
		%NObsVars(d_miss);
		
		%if &nobs %then %do;
			data d;
				set d d_miss;
			run;
			
			proc sort data=d;
				by _RowNum_;
			run;
		%end;
		
		* reset predictor variables;
		%if ^&upred %then %do; 
			%let PredictorVars = &PredictorVarsSave;
		%end; 
		%else %do; 
		        %let PredictorVars1 = &&PredictorVarsSave1;
			%do j=2 %to &nix_blocks; 
				%let PredictorVars&j = &&PredictorVarsSave&j;
			%end;	
		%end;
		%let ClassPredVars = &ClassPredVarsSave;
		
		%if &nixsave %then %do;
			%let _tid_ 	= %sysfunc(close(&_dsix_));
			data dsix;
				set dsixsave;
			run;
			%let _dsix_ = %sysfunc(open(dsix));
			%let nix = &nixsave;
		%end;
		%if &nipsave %then %do;
			%let _tid_ 	= %sysfunc(close(&_dsip_));
			data dsip;
				set dsipsave;
			run;
			%let _dsip_ = %sysfunc(open(dsip));
			%let nip = &nipsave;
		%end;
		%if &nixlsave %then %do;
			%let _tid_ 	= %sysfunc(close(&_dsixl_));
			data dsixl;
				set dsixlsave;
			run;
			%let _dsixl_ = %sysfunc(open(dsixl));
			%let nixl = &nixlsave;
		%end;
		%if &niplsave %then %do;
			%let _tid_ 	= %sysfunc(close(&_dsipl_));
			data dsipl;
				set dsiplsave;
			run;
			%let _dsipl_ = %sysfunc(open(dsipl));
			%let nipl = &niplsave;
		%end;
		%if %symexist(NPred) %then %do;
			%let NPred = &NPredSave;
		%end;
		
		%let BJ_Niter = %eval(&BJ_Niter+1);
	%end;
%end;
%else %do;
	%let BJ_Iterating = 0;
%end;
%end;
* end of Buckley-James loop;

%if (&BJ_Est = Yes) %then %do;
	%if &BJ_OneMore %then %let BJ_Niter = %eval(&BJ_Niter-1);
	%if &BJ_Niter < &BJ_Maxiter %then %do;
		%put Buckley-James estimation converged in &BJ_Niter iterations.;
	%end;
	%else %do;
		%put WARNING: Buckley-James estimation did not converge in &BJ_Maxiter iterations.;
		%put Using results from the last iteration.;
	%end;
	%let DepVarBJ = &DepVar;
	%let TruthVarBJ = &TruthVar;
	%let DepVar = &DepVarSave;
	%let TruthVar = &TruthVarSave;
	title2;
%end;


************************************;
* final processing for all methods *;
************************************;

%if &mode ne Interactive %then %do;
		
	%put _dtype_ = &_dtype_;
	%put NumDepAsClass = &NumDepAsClass;
	
	/*data OutLib.ts2o; set ts2o; run;*/

	* binary or nominal trait;
	%if (&_dtype_=C) or (&NumDepAsClass=Yes) %then %do;

		* make sure vartype of &into matches that of &DepVar;
		%let _itype_ = ;
		%VarAttributes(ts2o,&into,_Type=_itype_);
		%put _itype_ = &_itype_;
		%if %length(&_itype_) and (&_itype_ ^= &_dtype_) %then %do;
			data ts2o;
				set ts2o;
				%if (&_dtype_ = N) %then %do;
					_newi = 0 + &into;
				%end;
				%else %do;
					_newi = trim(left(&into));
				%end;
				drop &into;
			run;
			data ts2o;
				set ts2o;
				rename _newi = &into;
			run;
		%end;

		%if &ProcessName ^= GenomicBayesianRegression %then %do;
			%if %length(&ASet) %then %let ASet = &ASet, &trPriors = %LocaleEZX(%nrbquote(&Priors), EIA=1);
			%else %let ASet = &trPriors = %LocaleEZX(%nrbquote(&Priors), EIA=1);
		%end;

		%put into = &into;
		%put SuccessCutoff = &SuccessCutoff;
		%put nclasses = &nclasses;

		* if success cutoff is not 0.5, adjust into values;
		%if (&SuccessCutoff^=0.5) and &nclasses=2 %then %do;

			* get the two values;
			data check;
				set ts2o;
				keep &TruthVar &DepVar &into p_:;
			run;

			proc sort data=check nodupkey;
				by &TruthVar &DepVar &into;
			run;

			%do c=1 %to &nclasses;
				%if &&class&c = &event %then %let eidx1 = &c;
			%end;
			%put eidx1 = &eidx1;

			%let ivalue0 = &class1;
			%let ivalue1 = &class2;
			data _null_;
				set check;
				retain got0 got1 0;
				array pp[*] p_:;
				if &DepVar = &into then do;
					if pp[&eidx1] >= 0.5 then do;
						call symputx('ivalue1',&into);
						got1 = 1;
					end;
					else do;
						call symputx('ivalue0',&into);
						got0 = 1;
					end;						
				end;
				else do;
					if not got0 and pp[&eidx1] >= 0.5 then do;
						call symputx('ivalue0',&DepVar);
					end;
					if not got1 and pp[&eidx1] < 0.5 then do;
						call symputx('ivalue1',&DepVar);
					end;						
				end;
			run;
			%put ivalue0 = &ivalue0;
			%put ivalue1 = &ivalue1;

			data ts2o;
				set ts2o;
				array pp[*] p_:;
				if pp[&eidx1] >= &SuccessCutoff then do;
					%if (&_dtype_ = N) %then %do;
						&into = 0 + &ivalue1;
					%end;
					%else %do;
						&into = "&ivalue1";
					%end;
				end;
				else do;
					%if (&_dtype_ = N) %then %do;
						&into = 0 + &ivalue0;
					%end;
					%else %do;
						&into = "&ivalue0";
					%end;
				end;
			run;

		%end;

		* construct _INTO_ and Predicted_Class variable;
		data ts2o;
			set ts2o;
			%if &into ^= _INTO_ %then %do;
				_INTO_ = &into;
			%end;
			%if &_dtype_ = C %then %do;
				Predicted_Class = &into;
			%end;
			%else %do;
				Predicted_Class = 0 + &into;
			%end;
			* drop &into;
		run;

		* switch back to original dependent variable if necessary;
		%put DepSwitch = &DepSwitch;
		%if (&DepSwitch = Yes) %then %do;

			/*
			ods exclude none;
			ods results;
			title "ts2o before depswitch";
			proc print data=ts2o;
			run;
			title;
			*/

			data ts2o;
				set ts2o;
				_order_ = _n_;
				rename Predicted_Class = Pred_Class01;
				drop &into;
			run;

			/*
			data ocp1s;
				set ocp1;
				%if &eidx = 2 %then %do;
					Pred_Class01 = _n_ - 1;
				%end;
				%else %do;
					Pred_Class01 = 2 - _n_;
				%end;
				%if &_dtype0_ = N %then %do;
					Predicted_Class = 0 + &DepVar0;
				%end;
				%else %do;
					Predicted_Class = &DepVar0;
				%end;
				&into = Predicted_Class;
				keep Pred_Class01 Predicted_Class &into;
			run;
			*/

			data dclass;
				set d;
				keep &DepVar0 &DepVar;
				if missing(&DepVar0) then delete;
			run;

			proc sort data=dclass nodupkey;
				by &DepVar0;
			run;

			data dclass;
				set dclass;
				rename &DepVar=Pred_Class01 &DepVar0=Predicted_Class;
				&into = &DepVar0;
			run;

			/*
			data OutLib.ts2o;
				set ts2o;
			run;

			data OutLib.dclass;
				set dclass;
			run;
			*/

			/*
			title "dclass";
			proc print data=dclass;
			run;
			title;
			*/

			proc sql; 
				create table ts2o1 
					as select * 
					from ts2o as l left join
						dclass as r
					on l.Pred_Class01 = r.Pred_Class01
					order by _order_;
			quit;

			data ts2o;
				set ts2o1(drop=_order_);
				label Pred_Class01=' ' Predicted_Class=' ';
			run;

			/*
			data OutLib.ts2o;
				set ts2o;
			run;
			*/

			%let DepVarOld = &DepVar;
			%let _dtypeold_ = &_dtype_;				
			%let DepVar = &DepVar0;
			%let _dtype_ = &_dtype0_;
			%put DepVar=&DepVar DepVarOld=&DepVarOld _dtype_=&_dtype_ _dtypeold_=&_dtypeold_;

			%let TruthVarOld = &TruthVar;
			%let TruthVar = &TruthVar0;
			%put TruthVar=&TruthVar TruthVarOld=&TruthVarOld;

			%let switchp = 0;
			data _null_;
				set dclass;
				if Pred_Class01 = 1 then call symput('switchp',1);
				stop;
			run;
			%if (&switchp) %then %do;
				data ts2o;
					set ts2o;
					rename p_&DepVarTrunc._0=_temp0 p_&DepVarTrunc._1=_temp1;
				run;
				data ts2o;
					set ts2o;
					rename _temp0=p_&DepVarTrunc._1 _temp1=p_&DepVarTrunc._0;
				run;
			%end;

			%Rename_P_;

		%end;
		
		/*data OutLib.ts2o_b; set ts2o; run;*/

		* if prediction is missing, impute the baseline prediction to avoid missing rmse;
		%let missingp = 0;
		data ts2o;
			set ts2o;
			if missing(Predicted_Class) then do;
				_missp_ = 1;
				call symput("missingp","1");
			end;
			else do;
				_missp_ = 0;
			end;
		run;		
		
		%if (&missingp) %then %do;
			%put;
			%put Imputing baseline predictions for missing predicted values.;
			%put;

			/*
			data OutLib.ts2obi;
				set ts2o;
			run;
			*/

			* compute baseline prediction value;
			data one;
				%if (&_dtype_=C) %then %do;
					&DepVar = " ";
					Predicted_Value = " ";
				%end;
				%else %do;
					&DepVar = .;
					Predicted_Value = .;
				%end;
				output;
			run;

			%do c=1 %to &nclasses;
				%global actualp&c;
			%end;

			%basescor(
				train_dsname=&ts1,
				test_dsname=one,
				trait=&DepVar,
				cv_custompriors=&custompriors
			);

			/*
			data OutLib.baseline_preds;
				set baseline_preds;
			run;
			*/

			data ts2o1 ts2o2;
				set ts2o;
				_sortindex_ = _n_;
				if _missp_ then output ts2o2;
				else output ts2o1;
			run;

			/*
			ods exclude none;
			ods results;
			title "ts2o2";
			proc print data=ts2o2;
			run;
			title;

			title "actualp";
			proc print data=actualp;
			run;
			title;
			*/

			%put BPV = &BPV;
			%put DepVar = &DepVar;

			/*
			data OutLib.ts2o2_i;
				set ts2o2;
			run;
			data OutLib.actualp_i;
				set actualp;
			run;
			*/

			* merge with actualp created in macro basescor;
			proc transpose data=actualp prefix=p_&DepVarTrunc._ out=apt;
				var actualp;
				id &DepVar;
			run;

			data apt;
				set apt;
				%if &_dtype_ = C %then %do;
					Predicted_Class = trim(left("&BPV"));
				%end;
				%else %do;
					Predicted_Class = &BPV;
				%end;
				_one_ = 1;
				drop _name_;
			run;
			
			/*data OutLib.apt; set apt; run;  */ 

			data ts2o2;
				set ts2o2;
				_one_ = 1;
				drop p_: Predicted_Class;
			run;
			
			/*data OutLib.ts2o2a; set ts2o2; run;*/

			* merge and compute actualp for observations that have a nonmissing response;
			data ts2o2;
				merge ts2o2 apt;
				by _one_;
				array pp p_:;
				if not missing(&DepVar._true) then do;
					_cidx = .;
					%do c=1 %to &nclasses;
						%if (&_dtype_=N) %then %do;
							if trim(left(&TruthVar)) = &&class&c then _cidx = &c; 
						%end;
						%else %do;
							if trim(left(&TruthVar)) = "&&class&c" then _cidx = &c; 						
						%end;
					%end;
					if not missing(_cidx) then actualp = pp[_cidx];
					else actualp = .;
				end;
			run;

			/*
			data OutLib.ts2o2;
				set ts2o2;
			run;
			*/

			/*
			proc sql; 
				create table ts2o3 
					as select * 
					from ts2o2(drop=actualp) as l, actualp as r
					where l.&DepVar=r.&DepVar;
			quit;

			%do c=1 %to &nclasses;
				%put actualp&c = &&actualp&c;
			%end;

			data ts2o3;
				set ts2o3;
				array pp{*} p_:;
				%do c=1 %to &nclasses;
					pp[&c] = &&actualp&c;
				%end;
				&pred = pp[2];
				%if &_dtype_ = C %then %do;
					Predicted_Class = trim(left("&BPV"));
				%end;
				%else %do;
					Predicted_Class = &BPV;
				%end;
			run;

			data OutLib.ts2o3;
				set ts2o3;
			run;

			data ts2o2;
				merge ts2o2 actualp;
				by &DepVar;
				%if &_dtype_ = C %then %do;
					Predicted_Class = trim(left("&BPV"));
				%end;
				%else %do;
					Predicted_Class = &BPV;
				%end;
			run;
			*/

			/*
			proc print data=ts2o3;
			run;
			*/

			data ts2o;
				set ts2o1 ts2o2;
			run;

			proc sort data=ts2o;
				by _sortindex_;
			run;

			data ts2o;
				set ts2o;
				drop _sortindex_ _one_;
			run;
		%end;

		%if %length(%trim(&CustomCosts)) %then %do;
			%ApplyCosts(data=ts2o,costfunction=&CustomCosts,intovar=Predicted_Class);
			%let ASet = &ASet, &trCustomCosts = &CustomCosts;
		%end;

		* construct Correct and Correct_Pred variables;
		data ts2o;
			set ts2o;
			length Correct 8 Correct_Pred $ 3;
			%if &_dtype_ = C %then %do;
				if &TruthVar = " " or Predicted_Class = " " then do;
			%end;
			%else %do;
				if &TruthVar = . or Predicted_Class = . then do;
			%end;
				Correct = .;
				Correct_Pred = "   ";
				actualp = .;
			end;
			else if &TruthVar = Predicted_Class then do;
				Correct = 1;
				Correct_Pred = "Yes";
			end;
			else do;
				Correct = 0;
				Correct_Pred = "No";
			end;
			%if (&ProcessName = DiscriminantAnalysis) or (&ProcessName = KNearestNeighbors) %then %do;
				%do c=1 %to &ncan;
					if Can&c = . then Can&c = 0;
				%end;
			%end;
			%if ^&CVFlag %then %do;
				rename actualp = Prob_Actual;
			%end;
		run;

		/*
		data OutLib.ts2oai;
			set ts2o;
		run;
		*/


	%end;
	%else %do;

		* continuous trait;
		data ts2o;
			set ts2o;
			Predicted_Value = &pred;
		run;

		/*
		data OutLib.ts2o2;
			set ts2o;
		run;
		*/

		* if not in CVMC, compute baseline prediction value;
		%if ^&CVFlag or %symexist(TSMC) or %symexist(LCMC) %then %do;
			%let missingp = 0;
			data _null_;
				set ts2o;
				if missing(Predicted_Value) then do;
					call symputx("missingp","1");
					stop;
				end;
			run;
			%put missingp = &missingp;
			%if (&missingp) %then %do;
				data one;
					&DepVar = .;
					Predicted_Value = .;
					output;
				run;
				%basescorint(train_dsname=&ts1,
							 test_dsname=one,
							 trait=&DepVar);
			%end;
		%end;
		%put BPV = &BPV;

		%if &AsymmetricLossEvalProp ne 0.5 %then %do;
			%let ALoss_Weight = Asymmetric;
		%end;
		%else %do;
			%let ALoss_Weight = ;
		%end;

		data ts2o;
			set ts2o;
			* if prediction is missing, use the baseline prediction to avoid missing rmse;
			if missing(Predicted_Value) then Predicted_Value = &BPV;
			if Predicted_Value > 1e8 then Predicted_Value = 1e8;
			Residual = &TruthVar - Predicted_Value;
			%if (&BJ_Est = Yes) %then %do;
				Residual_BJ = &TruthVarBJ - Predicted_Value;
			%end;
			AbsResidual = abs(Residual);
			%if &AsymmetricLossEvalProp ne 0.5 %then %do;
				if missing(Residual) then &ALoss_Weight = .;
				else if Residual >= 0 then &ALoss_Weight = 2 * &AsymmetricLossEvalProp;
				else &ALoss_Weight = 2 * (1 - &AsymmetricLossEvalProp);
			%end;
			* drop &pred;
		run;
	%end;

	/*
	ods exclude none;
	ods results;
	title "&ProcessName";
	proc print data=ts2o;
	run;
	title;
	ods exclude all;
	ods noresults;
	*/		

	/* for discrim or knn with nk > 1, check if the first canonical score means are positively correlated with 
		the first canonical score means from k=1.  if not, multiply can1 by -1.  this is so colored
		points will cluster appropriately in the plot of the canonical scores. */
	/* need to move this block to cross_validate, disable for now */
	/*
	%if ((&ProcessName = DiscriminantAnalysis) or (&ProcessName = KNearestNeighbors)) and 
		(&nk > 1) %then %do;
		data ts2oc1;
			set ts2o;
			keep &DepVar can1;
		run;
		proc sort data=ts2oc1;
			by &DepVar;
		run;
		proc means data=ts2oc1 noprint;
			by &DepVar;
			var can1;
			output out=c1m mean=can1mean;
		run;

		%let negcorr = 0;
		%if (&k=1) %then %do;
			data c1m1;
				set c1m;
				rename can1mean=can1mean1;
			run;
		%end;
		%else %do;
			data c1m2;
				merge c1m1 c1m;
				by &DepVar;
			run;
			proc corr data=c1m2 out=corrm1 noprint;
				var can1mean1 can1mean;
			run;
			proc print data=corrm1;
			run;
			data _null_;
				set corrm1;
				if _type_ = "CORR" and _name_ = "can1mean";
				if can1mean1 < 0 then call symput("negcorr","1");
			run;
		%end;

		%if (&negcorr=1) %then %do;
			%put Reversing the sign of the first canonical score for iteration &k of &nk..;
			data ts2o;
				set ts2o;
				can1 = -can1;
			run;
		%end;
	%end;
	*/

	* if not cross-validating, add selected variables to output data set;
	* assuming sort order of &ts1 and &ts2 has not changed;
	%if ^&CVFlag and %length(%trim(&SelVars)) %then %do;	

		ods exclude none;
		ods results;

		/*
		title "&ts2";
		proc print data=&ts2;
		run;
		title;
		*/

		%put TestSpec = &TestSpec;
		%put ts1 = &ts1;
		%put ts2 = &ts2;

		* capture where statements;
		%let wherestmt = ;
		%if (&ProcessName = RadialBasisMachine) %then %do;
			%let stmts = &ProcGlimmixStatements;
		%end;
		%else %do;
			%let stmts = ;
		%end;
		%let ls = %length(&stmts);
		%if &ls %then %do;
			%let i1 = %index(&stmts,where);
			%if &i1 %then %do;	
				%let s1 = %substr(&stmts,&i1,%eval(&ls-&i1+1));
				%let i2 = %index(&s1,%str(;));
				%let wherestmt = %substr(&s1,1,&i2);
				%put wherestmt = &wherestmt;
			%end;
		%end;

		data sv;
			set 
			&ts1
			%if (&TestSpec = Yes) %then %do; 
				&ts2 
			%end; 
			;
			%if %length(&wherestmt) %then %do;
				&wherestmt;
			%end;
			keep 	
				%let i=1;
				%do %while(%length(%scan(&SelVars,&i,'*')));
					%scan(&SelVars,&i,'*')
					%let i=%eval(&i+1);
				%end;
			;
		run;

		data ts2sum;
			merge ts2o sv;
			if missing(_obs) then delete;
		run;
	%end;
	%else %do;
		data ts2sum;
			set ts2o;
		run;
	%end;

	* data set of selected variable names;
	%if ^%length(%trim(&SelVars)) %then %let SelVars = None;
	%*put SelVars= &SelVars;
	%put havets2v = &havets2v;
	%if ^&havets2v %then %do;
		* expand SelVars if it contains list-style specs;
		%if %index(&SelVars,:) or %index(&SelVars,-) %then %do;
			/*
			%put nixl=;
			%do i=1 %to &nixl; 
				%put "%sysfunc(varname(&_dsixl_,&i))";
			%end; 
			%put nipl=;
			%do i=1 %to &nipl; 
				%put "%sysfunc(varname(&_dsipl_,&i))";
			%end;
			%put nix=;
			%do i=1 %to &nix; 
				%put "%sysfunc(varname(&_dsix_,&i))";
			%end; 
			%put nip=;
			%do i=1 %to &nip; 
				%put "%sysfunc(varname(&_dsip_,&i))";
			%end; 		
			*/
			
			data ts2vars;
				length name $ 32;
				%do i=1 %to &nixl; 
					name = "%sysfunc(varname(&_dsixl_,&i))";
					output;
				%end; 
				%do i=1 %to &nipl; 
					name = "%sysfunc(varname(&_dsipl_,&i))";
					output;
				%end; 
				%do i=1 %to &nix; 
					name = "%sysfunc(varname(&_dsix_,&i))";
					output;
				%end; 
				%do i=1 %to &nip; 
					name = "%sysfunc(varname(&_dsip_,&i))";
					output;
				%end; 
			run;
		%end;
		%else %do;
			data ts2vars;
				length name $ 32;
				%let i=1;
				%do %while(%length(%scan(&SelVars,&i,' ')));
					name = "%scan(&SelVars,&i,' ')";
					output;
					%let i=%eval(&i+1);
				%end;
			run;
		%end;

		/*
		proc print data=ts2vars;
		run;
		*/		
	%end;
			
	%if (&SomeSel = 0) %then %do;
		%put WARNING: No variables selected.  An intercept-only model is assumed.  You may want to try relaxing;
		%put the selection criteria or using a different model.;
		%let SelVars = None;
	%end;
	
	%*if %eval(&nixl + &nipl) > 0 %then %do;
		* S1350469 and S1452708: remove variables repeated twice in the Final Selected variables*;
		
		data ts2vars;
			set ts2vars;
			_ord_ = _n_;
		run;
		
		proc sort data=ts2vars out=ts2vars nodupkey;
			by name;
		run;
		
		proc sort data=ts2vars out=ts2vars(drop=_ord_);
			by _ord_;
		run;
	%*end;

	/*data OutLib.ts2vars_f1; set ts2vars; run;*/
	/*data OutLib.sf2_f1; set sf2; run;*/
	
	* S1452712: including fixed effect variables from the Options tab*;
	%if &ProcessName= RidgeRegression or &ProcessName= GenomicBLUP or &ProcessName= GenomicBayesianRegression %then %do;
		%if &ClassOrFixedFlag %then %do;
			data ts2vars;
				set sf2(keep=name where=(name ^= "Intercept")) ts2vars;
			run;
		%end;	
	%end; 
	
	/*data OutLib.ts2vars_f12; set ts2vars; run;*/
	
	* build list of first 10 variables to show in the output;
	%let onlyint = 0;
	%if ^%index(&SelVars,*) %then %do;
		%if (%quote(&SelVars) = %quote(None)) or (%quote(&SelVars) = %quote(Intercept)) %then %do;
			%let onlyint = 1;
		%end;
	%end;
		
	%if (&onlyint) %then %do;
		%let SelVarShow = Intercept;
	%end;
	%else %do;

		/* does not work right with interactions
		%if (&ProcessName = LogisticRegression) and
			(&VarSelect = Penalized) %then %do;
			data OutLib.&pfit;
				set &pfit;
			run;
			data pfit;                                                                                                                              
			    set &pfit;                                                                                                             
			    keep X1 Predictor;                                                                                                                  
			    rename X1=NAME;                                                                                                                     
			    label X1="NAME";                                                                                                                    
			    if X1="Full Model" then delete;                                                                                                     
			run;                                                                                                                                    
			proc sort data=pfit;                                                                                                                    
			    by NAME;                                                                                                                            
			run;                                                                                                                                    
			proc sort data=ts2vars;                                                                                                                 
			    by NAME;                                                                                                                            
			run;                                                                                                                                    
			data ts2vars;                                                                                                                           
			   merge pfit ts2vars;                                                                                                                  
			   by NAME;                                                                                                                             
			run;                                                                                                                                    
			proc sort data=ts2vars;                                                                                                                 
				by descending IMPORTANCE;                                                                                                               
			run;  
		%end;
		*/

		%if (&FR_Inds = Yes) %then %do;

    			data ts2vars;                                                                                                                                                        
				set ts2vars;                                                                                                                                                  
				X1 = NAME;                                                                                                                                                   
            		run;     
		    	proc sort data=ts2vars;                                                                                                                                           
               			by X1;  
            		run; 
			proc sort data=friidef;
				by X1;
			run;
             		data ts2vars;                                                                                                                                                          
               			merge ts2vars friidef;                                                                                                                                              
               			by X1;                                                                                                                                                         
               			drop X1;                                                                                                                                                       
               			if name ="" then delete;                                                                                                                                       
            		run;  
			
			%let _varsid_ = %sysfunc(open(ts2vars));
			%let idefnum = %sysfunc(varnum(&_varsid_, INDICATOR_DEFINITION));
			%let _varsid_ = %sysfunc(close(&_varsid_));
			%if &idefnum %then %do;
				proc freq data=ts2vars noprint;
					tables INDICATOR_DEFINITION/out=idfrq;
				run;
				%NObsVars(idfrq);
				%if &nObs = 1 %then %do;
					%let dropid = 0;
					data _null_;
						set ts2vars;
						if INDICATOR_DEFINITION="" then do;
							call symput("dropid", 1);
						end;
					run;	
					%if &dropid=1 %then %do;
						data ts2vars;
							set ts2vars;
							drop INDICATOR_DEFINITION;
						run;
						%let idefnum = 0;
					%end;
				%end;
			%end;
			%let selvarname=NAME;
 		%end;
		%else %do; 
			%let selvarname=name;
		%end;
		%put selvarname = &selvarname;
		%NObsVars(ts2vars);
		%if (&nobs > 7) %then %do;
			%let nlv = 7;
			data ts2vars7;
				set ts2vars;
				if _n_ > 7 then stop;
			run;
			%let ts2vds = ts2vars7;
			%let ending = %nrbquote(%LocaleFill(%nrbquote(&trplusXnotshownseeX), %eval(&nobs - &nlv), &&&_OutDataVarsName...sas7bdat));
		%end;
		%else %do;
			%let nlv = &nobs;
			%let ts2vds = ts2vars;
			%let ending = ;
			%if (&FR_Inds = Yes) %then %do;
				%if &idefnum > 0 %then 
					%let ending =, %nrbquote(%LocaleFill(%nrbquote(&trseeXfor_IDefinition), &&&_OutDataVarsName...sas7bdat));
			%end;
		%end;
		data _null_;
			set &ts2vds;
			call symput('sv'||trim(left(_n_)),&selvarname);
		run;
		%let SelVarShow = ;
		%do i=1 %to &nlv;
			/*
			%put sv=&&sv&i;
			*/
			%let SelVarShow = &SelVarShow &&sv&i;
		%end;
		%let SelVarShow = &SelVarShow &ending;
	%end;
	
	%put Selected Variables = &SelVarShow;
	
	%*ClockStop(Fitting);
	%*ClockStart(Performance);
	
	/*
	title2 "Selected Variables = &SelVarShow";
	%if (&_dtype_=C) or (&NumDepAsClass=Yes) %then %do;
		proc freq data=ts2sum;
			tables &DepVar * Predicted_Class;
		run;
	%end;
	*/
	
	%put creating outdat=&outdat;
	%put creating outdatvars=&outdatvars;
	
	%if (&ProcessName= GenomicBLUP or &ProcessName = GenomicBayesianRegression) and ^&CVFlag %then %do;
 
		data outmarkerdat; 	
			retain 
				%if %length(&AnnoInData) %then %do;
					%do i=1 %to &nimk; 
						%sysfunc(varname(&_dsmap_,&i))
					%end;
				%end;
				%else %do;
					%do i=1 %to &nimk; 
						%sysfunc(varname(&_dsimk_,&i))
					%end;
				%end;
			;
			set ts2sum;
			keep 
				%do i=1 %to &nimk; 
					%sysfunc(varname(&_dsimk_,&i))
				%end;
			;
		run;
		
		/*data OutLib.outmarkerdat; set outmarkerdat; run;*/
		
		%if %length(&AnnoInData) %then %do;
			%let ListSelMarkerVars= %sysfunc(varname(&_dsmap_,1))%str(--)%sysfunc(varname(&_dsmap_,&nimk));
		%end;
		%else %do;
			%let ListSelMarkerVars= %sysfunc(varname(&_dsimk_,1))%str(--)%sysfunc(varname(&_dsimk_,&nimk));
		%end;
	%end;
	
	/*data OutLib.ts2sum; set ts2sum; run;*/ 
	
	data &outdat;
		%if (&ProcessName= GenomicBLUP or &ProcessName = GenomicBayesianRegression) and ^&CVFlag %then %do;
			merge ts2sum(drop=  
					%do i=1 %to &nimk; 
						%sysfunc(varname(&_dsimk_,&i))
					%end;
				)
			 outmarkerdat; 
		%end;
		%else %do;
			set ts2sum; 
		%end;
		%if &ProcessName= GenomicBLUP %then %do;
			drop 
				_ADD: _DOM: _train_ _obs gidx _i_ _y0 _y _one
				%if (&_dtype_=C) or (&NumDepAsClass=Yes) %then _INTO_ _missp_;
			
			; 
		%end;
	run;
	
	%if %symexist(_dsimk_) %then %do;
		%let _rc_ = %sysfunc(close(&_dsimk_));
	%end;

	%if %symexist(_dsmap_) %then %do;
		%let _rc_ = %sysfunc(close(&_dsmap_));
	%end;

	/*data OutLib.ts2vars_f2; set ts2vars; run;*/
	
	* remove label;
	data &outdatvars;
		set ts2vars;
		label name=' ';
	run;	
	
	%global TrainSC TestSC;	
	* if not cross-validating, create JSL for output display;
	%if &CVFlag %then %do;
		* title "Cross Validation Model Comparison";
		* options nonotes;
	%end;
	%else %do;
		* check for missing dependent variable values and create indicator;
		%if (&TestSpec = Yes) %then %do;
			%let dsid_o = %sysfunc(open(&outdat));
			%let FlagDataType = 0;
			%if %sysfunc(varnum(&dsid_o,DataType)) %then %do;
				%let FlagDataType = 1;
			%end;
			%let rc = %sysfunc(close(&dsid_o));
		
			%put Test set is assumed to have missing values for &DepVar in outdat=&outdat;
			data &outdat;
				set &outdat %if &FlagDataType %then %do; (drop= DataType) %end;;
				length DataType $ 8;
				if missing(&DepVar) then DataType = "Test";
				else DataType = "Training";
			run;
		%end;

		*******************************;
		* compute evaluation criteria *;
		*******************************;

		* compute RMSE and AUC for training and test data;
		%global rmse mae mann;
		%let rmse = ;
		%let mann = ;

		data training;
			set &outdat;
			%if (&TestSpec = Yes) %then %do;
				where DataType = "Training";
			%end;
		run;
		
		%if (&TestSpec = Yes) %then %do;
			data test;
				set &outdat;
				where DataType = "Test";
			run;
		%end;

		%if %length(&&&_outdataName) > 29 %then %let _psoutname = %substr(&&&_outdataName,1,29); 
		%else %let _psoutname = &&&_outdataName;
		
		* binary or nominal trait;
		%if (&_dtype_=C) or (&NumDepAsClass=Yes) %then %do;

			%NObsVars(training);
			%if ^&nobs and ^&CVFlag %then %do;
				%put ERROR: Insufficient training data to continue calculations.  You may have too few observations or too many missing predictor values.;
				%let exiterror = 1;
				%goto exit;
			%end;

			* RMSE and MAE;
			%compute_ase_nominal(
				data=training, 
				output=train_ase, 
				actual=Prob_Actual, 
				nclasses=&nclasses, 
				mdlname=training,
				prevalences=&Priors,
				cv_custompriors=&CustomPriors,
				trait=&TruthVar
			); 
			data _null_;
				set train_ase;
				rmse = sqrt(ase);
				call symput('rmse',put(rmse,6.4));
				call symput('mae',put(mae,6.4));
			run;
			%let TrainSC = &trRootMeanSquareError = &rmse, &trMeanAbsoluteError = &mae;
			
			* AUC;
			%if &nclasses=2 %then %do;
			
				%put event=&event  TruthVar=&TruthVar  etype=&_dtype_;
				
				data training;
					set training;
					if missing(&TruthVar) then delete;
				run;
												
			   	%locate_pevent_isevent(
			   		data=training,
					trait=&TruthVar,
					event=&event,
					etype=&_dtype_,
					actual=Prob_Actual
				);

				/*
				ods exclude none;
				ods results;
				title "training after locate_pevent";
				proc print data=training;
				run;
				title;
				*/
				
				/*
				data OutLib.training_afterlp;
					set training;
				run;
				*/
																			
				%compute_mann(
					data=training,
					pevent=_pevent,
					isevent=_isevent
				);

				%global _outroc;
				%let _outroc = ;
				%PrepOutDS(_outroc,_outrocname,&&&_outdataname,_roc);

				%compute_roc(
					data=training,
					pevent=_pevent,
					isevent=_isevent,
					outdata=&_outroc;
				);

				data _null_;
					mann = &mann;
					call symput('mann',put(mann,6.4));
				run;
				%put train mann = &mann;
				%let TrainSC = &TrainSC, &trAreaUnderROCCurve = &mann;
			%end;
			
			%put Train Accuracy Trait = &DepVar;
						
			* Accuracy;
			%compute_acc_nominal(
				data=training,
				output=train_acc,
				trait=&DepVar,
				correct=Correct,
				mdlname=training,
				prevalences=&Priors,
				cv_custompriors=&CustomPriors
			);
			
			proc transpose data=train_acc out=acct;
				var Accuracy_:;
			run;
			
			%NObsVars(acct);
			%let nlev = &nobs;
			
			data acct;
				set acct;
				call symput('acc_name'||trim(left(_n_)),trim(left(_name_)));
			run;
				
			data _null_;
				set train_acc;
				call symput('acc',put(accuracy,6.4));
				%do i=1 %to &nlev;
					call symput("acc&i",put(&&acc_name&i,6.4));
				%end;
			run;
				
			%let TrainSC = &TrainSC, &trAccuracy = &acc;
			%if &nlev=2 and &acc_name1=Accuracy_0 and &acc_name2=Accuracy_1 %then %do;
				%let acc_name1 = Specificity;
				%let acc_name2 = Sensitivity;
			%end;
			%do i=1 %to &nlev;
				%let TrainSC = &TrainSC, &&acc_name&i = &&acc&i;
			%end;
			
			/* save RMSE, ACC, AUC to a table */
			data tmp_training;
				length MDLSC $1000;
				_Data_ = "training";
				_Model_ = "&&&_outdataName";
				Accuracy = &acc;
				%if &acc_name1=Specificity %then %do;
					length Specificity 6 Sensitivity 6;
					Specificity = &acc1;
					Sensitivity = &acc2;
					label Specificity = "&trSpecificity" Sensitivity = "&trSensitivity";
				%end;
				%if &nclasses=2 %then %do;
					AUC = &mann;
				%end;
				RMSE = &rmse;
				MDLSC = "&TrainSC";
			run;

			data OutLib.&_psoutname._ps;
				set tmp_training;
			run;
			
			%if (&TestSpec = Yes) %then %do;
							
				%compute_ase_nominal(
					data=test, 
					output=test_ase, 
					actual=Prob_Actual, 
					nclasses=&nclasses, 
					mdlname=test,
					prevalences=&Priors,
					cv_custompriors=&CustomPriors,
					trait=&TruthVar
				); 
				
				data _null_;
					set test_ase;
					rmse = sqrt(ase);
					call symput('rmse',put(rmse,6.4));
					call symput('mae',put(mae,6.4));
				run;
				%let TestSC = &trRootMeanSquareError = &rmse, &trMeanAbsoluteError = &mae;
				
				%if &nclasses=2 %then %do;
				
					%locate_pevent_isevent(
						data=test,
						trait=&TruthVar,
						event=&event,
						etype=&_dtype_,
						actual=Prob_Actual
					);
					
					/*
					data OutLib.test;
						set test;
					run;
					*/
																								
					%compute_mann(
						data=test,
						pevent=_pevent,
						isevent=_isevent
					);

					%compute_roc(
						data=test,
						pevent=_pevent,
						isevent=_isevent,
						outdata=tmp;
					);
					
					data &_outroc;
						set &_outroc;
						DataType = "Training";
					run;
					
					data tmp;
						set tmp;
						DataType = "Test";
					run;
					
					data &_outroc;
						set &_outroc tmp;
					run;					

					data _null_;
						mann = &mann;
						call symput('mann',put(mann,6.4));
					run;
					%put test mann = &mann;
					%let TestSC = &TestSC, &trAreaUnderROCCurve = &mann;
				%end;
								
				%put Test Accuracy Trait = &TruthVar;

				* Accuracy;
				%compute_acc_nominal(
					data=test,
					output=test_acc,
					trait=&TruthVar,
					correct=Correct,
					mdlname=test,
					prevalences=&Priors,
					cv_custompriors=&CustomPriors
				);

				proc transpose data=test_acc out=acct;
					var Accuracy_:;
				run;

				%NObsVars(acct);
				%let nlev = &nobs;

				data acct;
					set acct;
					call symput('acc_name'||trim(left(_n_)),trim(left(_name_)));
				run;

				data _null_;
					set test_acc;
					call symput('acc',put(accuracy,6.4));
					%do i=1 %to &nlev;
						call symput("acc&i",put(&&acc_name&i,6.4));
					%end;
				run;

				%let TestSC = &TestSC, &trAccuracy = &acc;
				%do i=1 %to &nlev;
					%if &&acc_name&i = Accuracy_0 %then %let acc_name&i = Specificity;
					%else %if &&acc_name&i = Accuracy_1 %then %let acc_name&i = Sensitivity;
					%let TestSC = &TestSC, &&acc_name&i = &&acc&i;
				%end;
				
				/* save RMSE, ACC, AUC to a table */
				%if %length(&InData) > 28 %then %let inname = %substr(&InData,1,28); 
				%else %let inname = &InData;
				data tmp_test;
					length MDLSC $1000;
					_Data_ = "test";
					_Model_ = "&&&_outdataName";
					Accuracy = &acc;
					length Specificity 6 Sensitivity 6;
					%do i=1 %to &nlev;
						if &&acc_name&i = Specificity then  Specificity = &&acc&i;
						else if &&acc_name&i = Sensitivity then Sensitivity = &&acc&i;
					%end;
					label Specificity = "&trSpecificity" Sensitivity = "&trSensitivity";
					%if &nclasses=2 %then %do;
						AUC = &mann;
					%end;
					RMSE = &rmse;
					MDLSC = "&TestSC";
				run;
				data OutLib.&_psoutname._ps;
					set OutLib.&_psoutname._ps tmp_test;
				run;
			%end;
		%end;
		%else %do;
			data train1;
				set training;
				if missing(&TruthVar) then delete;
			run;

			* Harrell C-statistic;
			proc harrell data=train1 out=train1_hc;
				time &TruthVar;
				%if %length(&CensorVar) %then %do;
					censor &CensorVar;
				%end;
				predict Predicted_Value;
				%if &AsymmetricLossEvalProp ne 0.5 %then %do;
					freq &ALoss_Weight;
				%end;
			run;

			data _null_;
				set train1_hc;
				call symput('Harrell_C',put(Harrell,6.4));
			run;
			
			* for censored data, compute RMSE and correlation only on uncensored values;
			%if %length(&CensorVar) %then %do;
				data train1;
					set train1;
					if &CensorVar in(%unquote(&CensorValues)) then delete; 
				run;
			%end;
			
			%compute_ase_interval(
				data=train1, 
				output=train_ase, 
				y=&TruthVar, 
				predict=Predicted_Value, 
				mdlname=training,
				aweight=&ALoss_Weight
			);
			
			data _null_;
				set train_ase;
				rmse = sqrt(ase);
				call symput('rmse',put(rmse,6.4));
				call symput('mae',put(mae,6.4));
			run;
			
			proc corr data=train1 out=cout noprint;
				var &TruthVar Predicted_Value;
				%if &AsymmetricLossEvalProp ne 0.5 %then %do;
					weight &ALoss_Weight;
				%end;
			run;
			
			data _null_;
				set cout end=_last;
				if _last then call symput('corr',put(&TruthVar,6.4));
			run;

			%if &AsymmetricLossEvalProp ne 0.5 %then %do;
				%let TrainSC = Asymmetric Loss Evaluation Proportion = &AsymmetricLossEvalProp,%str( );
			%end;
			%else %do;
				%let TrainSC =;
			%end;
			%let TrainSC = &TrainSC.&trRootMeanSquareError = &rmse, &trMeanAbsoluteError = &mae, &trHarrellCStatistic = &Harrell_C, &trCorrelation = &corr;

			data train_ps;
				length MDLSC $1000;
				_Data_ = "training";
				_Model_ = "&_OutDataName";
				RMSE = &rmse;
				MAE = &mae;
				Harrell_C = &Harrell_C;
				Correlation = &corr;
				MDLSC = "&trainSC";
			run;
			
			data OutLib.&_psoutname._ps;
				set train_ps;
			run;

			%if (&TestSpec = Yes) %then %do;

				data test1;
					set test;
					if not missing(&TruthVar) and not missing(Predicted_Value);
				run;
				%NObsVars(test1);
				%if &nobs %then %do;
					* Harrell C-statistic;
					proc harrell data=test1 out=test_hc;
						time &TruthVar;
						%if %length(&CensorVar) %then %do;
							censor &CensorVar;
						%end;
						predict Predicted_Value;
						%if &AsymmetricLossEvalProp ne 0.5 %then %do;
							freq &ALoss_Weight;
						%end;
					run;

					data _null_;
						set test_hc;
						call symput('Harrell_C',put(Harrell,6.4));
					run;

					* for censored data, compute RMSE and correlation only on uncensored values;
					%if %length(&CensorVar) %then %do;
						data test1;
							set test1;
							if &CensorVar in(%unquote(&CensorValues)) then delete; 
						run;
					%end;

					%compute_ase_interval(
						data=test1, 
						output=test_ase, 
						y=&TruthVar, 
						predict=Predicted_Value, 
						mdlname=test,
						aweight=&ALoss_Weight
					); 
					
					data _null_;
						set test_ase;
						rmse = sqrt(ase);
						call symput('rmse',put(rmse,6.4));
						call symput('mae',put(mae,6.4));
					run;
					
					proc corr data=test1 out=cout noprint;
						var &TruthVar Predicted_Value;
						%if &AsymmetricLossEvalProp ne 0.5 %then %do;
							weight &ALoss_Weight;
						%end;
					run;
					data _null_;
						set cout end=_last;
						if _last then call symput('corr',put(&TruthVar,6.4));
					run;
				%end;
				%else %do;
					%let Harrell_C = .;
					%let rmse = .;
					%let mae = .;
					%let corr = .;
				%end;
				%if &AsymmetricLossEvalProp ne 0.5 %then %do;
					%let TestSC = Asymmetric Loss Evaluation Proportion = &AsymmetricLossEvalProp,%str( );
				%end;
				%else %do;
					%let TestSC =;
				%end;
				%let TestSC = &TestSC.&trRootMeanSquareError = &rmse, &trMeanAbsoluteError = &mae, &trHarrellCStatistic = &Harrell_C, &trCorrelation = &corr;
				data test_ps;
					length MDLSC $1000;
					_Data_ = "test";
					_Model_ = "&_OutDataName";
					RMSE = &rmse;
					MAE = &mae;
					Harrell_C = &Harrell_C;
					Correlation = &corr;
					MDLSC = "&TestSC";
				run;

				data OutLib.&_psoutname._ps;
					set OutLib.&_psoutname._ps test_ps;
				run;
			%end;
		%end;

		%PRS:;		

		* create predictor reduction settings string;
		%let PRS = ;
		%if %length(&PredWhere) %then %do;
			%let PRS = &PRS &trContPredictorIncludeFilter = &PredWhere,;
		%end;
		%if %length(&PredClassWhere) %then %do;
			%let PRS = &PRS &trClassPredictorIncludeFilter = &PredClassWhere,;
		%end;
		
		* if PRS contains double quotes, add \! in front of them for jsl.;
		%let len = %length(%bquote(&PRS));
		%if &len %then %do;
			%put Current PRS = &PRS;
			%let i = 1;
			%do %while(&i <= &len);
				%let chari = %qsubstr(%nrbquote(&PRS),&i,1);
				/* %put chari = &chari;*/
				%if %str(%") = %bquote(&chari) %then %do;
					%let part1 = %qsubstr(%nrbquote(&PRS),1,%eval(&i-1));
					%let part2 = %qsubstr(%nrbquote(&PRS),&i,%eval(&len-&i+1));
					%if %length(%bquote(&part1)) = %eval(&i-1) %then %do;
						%let PRS = &part1.\!&part2;
					%end;
					%else %do;
						%let PRS = &part1. \!&part2;
					%end;
					/* %put New PRS = &PRS; */
					%let i=%eval(&i+2);
					%let len = %eval(&len+2);
				%end;
				%let i=%eval(&i+1);
			%end;
			%put New PRS = &PRS;
		%end;
		
		
		%if %length(&TransformationMethod) %then %do;
			%let PRS = &PRS  &trPredictorTransformation = %LocaleEZX(%nrbquote(&TransformationMethod), EIA=1),;
		%end;
		%if %length(&StandardizationMethod) %then %do;
			%let PRS = &PRS  &trPredictorStandardization = %LocaleEZX(%nrbquote(&StandardizationMethod), EIA=1),;
		%end;
		%if &StatTest = Yes %then %do;
			%if (&nix) %then %do; 
				%let STMeth = &ST_Method;
				%if (&nip) %then %let STMeth = %nrbquote(%LocaleFill(%nrbquote(&trXandX), %nrbquote(%LocaleEZX(%nrbquote(&STMeth), EIA=1)), %nrbquote(&trFisherExactTest)));
			%end;
			%else %do;
				%let STMeth = &trFisherExactTest;
			%end;
			%let MTMeth = &MultipleTestingMethod;
			%if ^%length(&MTMeth) %then %let MTMeth = &trNone;
			%let PRS = &PRS Stat Test = %LocaleEZX(%nrbquote(&STMeth), EIA=1),;
			%if &Nominalize = Yes %then %do;
				%let PRS = &PRS &DepVar is Nominalized,;
			%end;
			%if &ST_AsymmetricLossFitProp ne 0.5 %then %do;
				%let PRS = &PRS Asymmetric Loss Fitting Proportion = &ST_AsymmetricLossFitProp,;
			%end;
			%if %index(%quote(&ST_Method),%quote(Quantile)) %then %do;
				%let PRS = &PRS Quantile Level = &ST_QuantileLevel,;
			%end;
			%let PRS = &PRS &trMultipleTestingMethod = %LocaleEZX(%nrbquote(&MTMeth), EIA=1),;
			%if &NegLog10pCutoff > 0 %then %do;
				%let PRS = &PRS &trNeglog10pvalueCutoff = &NegLog10pCutoff,;
			%end;
			%if &AbsMeanDiffCutoff > 0 %then %do;
				%let PRS = &PRS &trMeanDifferenceCutoff = &AbsMeanDiffCutoff,;
			%end;
			%if &AbsPropDiffCutoff > 0 %then %do;
				%let PRS = &PRS &trProportionDifferenceCutoff = &AbsPropDiffCutoff,;
			%end;
		%end;
		%if &KMeans=Yes %then %do;
			%let PRS = &PRS &trKMeans = &KM_Max,;
		%end;
		%if &RG_SelectMethod ^= None %then %do;
			%let PRS = &PRS Regression Model Selection Method = &&RG_SelectMethod,;
		%end;
		%if &FR_ImpVars = Yes %then %do;
			%let trBestFRNIVfromForest = %nrbquote(%LocaleFill(%nrbquote(&trBestXfromForest), %nrbquote(&FR_NImpVars)));
			%let PRS = &PRS Max &FR_NImpVars from Forest,;
		%end;
		%if &Optimize = Yes %then %do;
			%let PRS = &PRS &trOptimization = %LocaleEZX(%nrbquote(&Opt_Algorithm), EIA=1),;
		%end;
		%if &StandardizeRows = Yes %then %do;
			%let PRS = &PRS Standardize Predictors Row-Wise,;
		%end;
		%if ^%length(&PRS) %then %do;
			%let PRS = &trNone;
		%end;
		%else %do;
			%let len = %eval(%length(&PRS)-1);
			%let PRS = %substr(%nrbquote(&PRS),1,&len);
		%end;
		%put PRS = &PRS;
		%if &ProcessName = SurvivalPredictiveModeling %then %goto SurvivalJSL;
	
		* Save DepVar name to OrgDepVar;
		%let traitVar = &DepVar;
		%let traitLabel = &DepVar;

		* convert names to labels;
		/*
		%CheckLabel(&outdat,DepVar);
		%let traitLabel=&DepVar;

		%if %length(&LabelVar) %then %do;
			%CheckLabel(&outdat,LabelVar);
		%end;

		%CheckLabel(OutLib.&&&_OutDataName,TruthVar);
		%if %symexist(ColorVar) %then %do;
			%if %length(&ColorVar) %then %do;
				%CheckLabel(OutLib.&&&_OutDataName,ColorVar);
			%end;
		%end;
		%if %length(&LabelVar) %then %do;
			%CheckLabel(OutLib.&&&_OutDataName,LabelVar);
		%end;
		*/
		
		* output scoring code for certain aps;
		%if ^&CVFlag /* and (&BJ_Est = No or &BJ_OneMore = 1) */ and &OrderInt = 1 %then %do;
			data _null_;
				%if (&ProcessName = GLMSelect) %then %do;
					file "&OutPath.&OutName._glm_score.sas"  encoding="utf-8";
					%ScoringCodeSAS;	
				%end;
				%else %if (&ProcessName = LogisticRegression and &VarSelect ^= Penalized) %then %do;
					file "&OutPath.&OutName._lgr_score.sas"  encoding="utf-8";
					%ScoringCodeSAS;	
				%end;
				%else %if (&ProcessName = PartialLeastSquares) %then %do;
					file "&OutPath.&OutName._pls_score.sas"  encoding="utf-8";
					%ScoringCodeSAS;	
				%end;
				%else %if (&ProcessName = QuantSelect) %then %do;
					file "&OutPath.&OutName._qs_score.sas"  encoding="utf-8";
					%ScoringCodeSAS;
				%end;
				%else %if (&ProcessName = RidgeRegression) and (&Kernel = Dot) %then %do;
					file "&OutPath.&OutName._rr_score.sas"  encoding="utf-8";
					%ScoringCodeSAS;
				%end;
				%else %if (&ProcessName = GenomicBayesianRegression) and (&TraitType ^= Ordinal) %then %do;
					file "&OutPath.&OutName._gbr_score.sas"  encoding="utf-8";
					%ScoringCodeSAS;
				%end;
			run;
			data _null_;
				%if (&ProcessName = GLMSelect) %then %do;
					file "&OutPath.&OutName._glm_formula.jsl"  encoding="utf-8";
					%ScoringCodeJSL;	
				%end;
				%else %if (&ProcessName = LogisticRegression  and &VarSelect ^= Penalized) %then %do;
					file "&OutPath.&OutName._lgr_formula.jsl"  encoding="utf-8";
					%ScoringCodeJSL;
				%end;
				%else %if (&ProcessName = PartialLeastSquares) %then %do;
					file "&OutPath.&OutName._pls_formula.jsl"  encoding="utf-8";
					%ScoringCodeJSL;	
				%end;
				%else %if (&ProcessName = QuantSelect) %then %do;
					file "&OutPath.&OutName._qs_formula.jsl"  encoding="utf-8";
					%ScoringCodeJSL;
				%end;
				%else %if (&ProcessName = RidgeRegression) and (&Kernel = Dot) %then %do;
					file "&OutPath.&OutName._rr_formula.jsl"  encoding="utf-8";
					%ScoringCodeJSL;
				%end;
				%else %if (&ProcessName = GenomicBayesianRegression) and (&TraitType ^= Ordinal) %then %do;
					file "&OutPath.&OutName._gbr_formula.jsl"  encoding="utf-8";
					%ScoringCodeJSL;
				%end;
			run;
		%end;
		
		%global _mainjslfile_;	
		%let _mainjslfile_ = &&&_JSLFile;

		* output JSL code;
		data _null_;
			file "&&&_JSLFile"  encoding="utf-8";
			put "//!";
			put 'Names Default to Here(1);';
			put "Here:DataList = {};";
			put "Here:TabButtonList = {};";
			put "Here:TabCodeList = {};";
			put "Here:TabAfterCodeList = {};";
			put "Here:TabOpenList = {};";

			%if ((&ProcessName = PartitionTrees) or (&ProcessName = VirtualTwins)) and %upcase(&TreeMethod)=TREE and ^&CVFlag %then %do;
				put "Here:PRS = ""&PRS"";";
				put "Here:ASet = ""&ASet"";";
				put "Here:_OutDataName = ""&&&_OutDataName"";";
				put "Here:_OutDataVarsName = ""&&&_OutDataVarsName"";";
				%let _OutPSName = &_psoutname._ps;
				put "Here:_OutPSName = ""&_OutPSName"";";
				%if &trait_type = qualitative %then %do;
					%if &nclasses = 2 %then %do;
						put "Here:_outrocname = ""&_outrocname"";";
					%end;
					%else %do;
						put "Here:_outrocname = "" "";";
					%end;
					%if &_dtype_ = C %then %do;
						put "Here:event = ""&event"";";
					%end;
					%else %do;
						put "Here:event = &event;";
					%end;
					put "Here:nclasses = &nclasses;";
					put "Here:prevalences = ""&Priors"";";
				%end;
				%else %do;
					put "Here:_outrocname = "" "";";
					put "Here:event = .;";
				 	put "Here:nclasses = .;";
					put "Here:prevalences = "" "";";
				%end;
				put "Here:TrainPath=""&TrainPath"";";
				put "Here:TestPath=""&TestPath"";";
				put "Here:ValidPath=""&ValidPath"";";
				put "Here:train_dsname=""&InData"";"; 
				put "Here:valid_dsname=""&ValidInData"";";
				put "Here:test_dsname=""&TestInData"";";
				put "Here:outputpath=""&ClientOutPath"";";
				put "Here:where_clause="" "";";
				put "Here:inmodel_dsname=""&inmodel_dsname"";";
				put "Here:trait=""&&traitVar"";";
				put "Here:traitlabel=""&&traitLabel"";";
				put "Here:trait_type=""&trait_type"";";
				put "Here:subtree="" "";";
				put "Here:assess="" "";";
				put "Here:model_output=""&model_output"";";
				put "Here:nodestats_output=""&nodestats_output"";";
				put "Here:importance_output=""&importance_output"";";
				put "Here:similarity_output="" "";";
				put "Here:rules_output=""&rules_output"";";	
				put "Here:cmprules_output=""&cmprules_output"";";
				put "Here:summary_output="" "";";
				put "Here:sequence_output=""&sequence_output"";";
				put "Here:score_train_output=""&score_train_output"";";
				put "Here:score_valid_output=""&score_valid_output"";";
				put "Here:score_test_output=""&score_test_output"";";
				put "Here:score_train_outfit="" "";";
				put "Here:score_valid_outfit="" "";";
				put "Here:score_test_outfit="" "";";
				put "Here:priors=""&priors"";";
				%if &trait_type = quantitative %then %do;
					put "Here:tpmin=&tpmin;";
					put "Here:tpmax=&tpmax;";
				%end;
				%if &missing= DISTRIBUTE %then %do;
				    put "Here:missing=""DISTRIBUTE"";";
				%end;
				%else %do;
				    put "Here:missing="" "";";
				%end;
				%if &nleaves > 1 %then %do;
					put "Here:valid_assess_column = &valid_assess_column;";
				%end;

				* UPDATE MODEL CRITERIA IN MODEL RESULTS TAB;
				put "Here:GetModelCriteria = Expr(";
				put "   If(IsEmpty(Here:ps_dt),";
				put "      Here:ps_dt=open(""&ClientOutPath.&_OutPSName..sas7bdat"",invisible);";
				put "	   CurrentDataTable(ps_dt);";
				put "	   Here:mdlsc_list = :MDLSC << Get Values;";
				put "	   Here:nsc = nitems(here:mdlsc_list);";
				put "	);";
				put ");";
				/*
				put "	   if( !IsEmpty(tb5b), tb5b << Set Text(here:mdlsc_list[1]));";
				put "	   if( nsc > 1,";
				put "		   if( !IsEmpty(tb4b), tb4b << Set Text(here:mdlsc_list[2]));";
				put "		);";
				*/
				* OPEN VARIABLE IMPORTANCE TABLE;
				put "Here:OpenImpDsExpr = Expr(";
				put "   if(IsEmpty(Here:varimp_dt),";
				put "		Here:varimp_dt=open(""&ClientOutPath.&importance_output..sas7bdat"",invisible);";
				%if %upcase(&TreeMethod)=TREE and ^&CVFlag %then %do;
					put "    Column(""NAME"") << Set Property(""Value Ordering"",{&viplist});";
				%end;
				put ");";
				put '	Here:where_imp_expr = ":RELATIVE IMPORTANCE > 0";';
				put '	if( score_valid_output != "",';
				put '		where_imp_expr = where_imp_expr || " & :VALIDATION RELATIVE IMPORTANCE > 0";';
				put "	);";
				put "	CurrentDataTable(varimp_dt);";
				put "	if( IsEmpty(subset_imp_dt) != 1, ";
				put "		Close(subset_imp_dt, NoSave);";
				put "	);";
				put '	Here:subset_imp_expr = "subrows = varimp_dt<<Get Rows Where("||where_imp_expr||");';
				put "if(nrow(subrows),";
				put "	Here:subset_imp_dt = varimp_dt<<Subset(Rows(subrows),";
				put '   		Output Table Name(\!"Variable Importance\!"),invisible);';
				put '		);";';
				put "eval(parse(subset_imp_expr));";
				put ");";

				* UPDATE FINAL SELECTED VARIABLES LIST IN MODEL RESULTS TAB;
				put "Here:GetFinalSelectedVars = Expr(";
				put "	OpenImpDsExpr;";
				%if &nleaves > 1 %then %do;
					put "	currentDataTable(subset_imp_dt);";
					put "	summarize(Here:vars_list=by(NAME));";
					put "	nvars=nitems(vars_list);";
					put "	Here:remains = 0;";
					put "	if( nvars > 10,";
					put	"		Here:remains = nvars - 10;";
					put "		endlist = 10;";
					put "			,";
					put "		endlist = nvars;";
					put "	);";
					put "	for(v=1, v<=endlist, v++,";
					put "		Here:var=vars_list[v];";
					put "		if( v==1,";
					put "			Here:selvars = var;";
					put "			,";
					put '			selvars = selvars||" "||var;';
					put "		);";
					put "	);";
					put "	if( remains > 0,";
					put '		selvars = selvars ||" "|| locale:fill(locale:get("plusXnotshownseeVariableImportanceTable_Text"), {char(eval(remains))});';
					put "	);";
				%end;
				%else %do;
					put '	selvars = locale:get("None_Text");';
				%end;
				/* put "	if( !IsEmpty(tb3b), tb3b << Set Text(selvars));"; */
				put ");";
			%end;

			* main results;
			put "Here:vb=Function({},{default local},";

			%if ((&ProcessName = PartitionTrees) or (&ProcessName = VirtualTwins)) and %upcase(&TreeMethod)=TREE and ^&CVFlag %then %do;
				put "GetModelCriteria;";
				put "GetFinalSelectedVars;";
			%end;

			put "   If(IsEmpty(Here:pred_data),";
			put "      Here:pred_data=open(""&ClientOutPath.&&&_OutDataName...sas7bdat"",UseLabelsForVarNames(false),invisible);";
			%if %length(&LabelVar) %then %do;
				put "      %str(column(%"&LabelVar%") << label(1);)";
			%end;
			put "      Here:pred_data << Select Where(1);";
			put "      Here:pred_data << Markers(8);";
			put "      Here:pred_data << Clear Select;";
			%if (&_dtype_=C) or (&NumDepAsClass=Yes) %then %do;
				put "      Here:pred_data<<Color By Column(Name(""&TruthVar""));";
				put '      for each row( Selected(Rowstate())=(:Correct_Pred=="No"));';
				put '      Here:pred_data<<Markers(2);';
				put '      Here:pred_data<<Clear Select;';
				put "      Column(""&TruthVar"") << Modeling Type(" '"Nominal");';
				put '      :Correct << Modeling Type("Nominal");';
				put ' ';
			%end;
			%if %symexist(ColorVar) %then %do;
				%if %length(&ColorVar) %then %do;
					put "      Here:pred_data<<Color By Column(:&ColorVar);";
				%end;
			%end;
			
			* add formula column for certain aps;
			%if &OrderInt = 1 %then %do;
				%if (&ProcessName = GLMSelect) %then %do;
					put "      CurrentDataTable(Here:pred_data);";
					put "      include(""&ClientOutPath.&OutName._glm_formula.jsl"");";
				%end;
				%else %if (&ProcessName = LogisticRegression and &VarSelect ^= Penalized) %then %do;
					put "      CurrentDataTable(Here:pred_data);";
					put "      include(""&ClientOutPath.&OutName._lgr_formula.jsl"");";
				%end;
				%else %if (&ProcessName = PartialLeastSquares) %then %do;
					put "      CurrentDataTable(Here:pred_data);";
					put "      include(""&ClientOutPath.&OutName._pls_formula.jsl"");";
				%end;
				%else %if (&ProcessName = QuantSelect) %then %do;
					put "      CurrentDataTable(Here:pred_data);";
					put "      include(""&ClientOutPath.&OutName._qs_formula.jsl"");";
				%end;
				%else %if (&ProcessName = RidgeRegression) and &Kernel = Dot %then %do;
					put "      CurrentDataTable(Here:pred_data);";
					put "      include(""&ClientOutPath.&OutName._rr_formula.jsl"");";
				%end;
			%end;
						
			put "   );";
			put "   If(IsEmpty(Here:vars_data),";
			put "      Here:vars_data=open(""&ClientOutPath.&&&_OutDataVarsName...sas7bdat"",invisible);";
			put "      Here:vars_data << Select Where(1);";
			put "      Here:vars_data << Markers(8);";
			put "      Here:vars_data << Colors(4);";
			put "      Here:vars_data << Clear Select;";
			put "      column(""name"") << label(1);";
			put "   );";
			put "   Current Data Table(Here:pred_data);";
			put "   VListBox(";
			%let CrossVal = None;
			put "   LineUpBox(NCol(2),Spacing(3),";
			put "      Here:tb1a = TextBox("" &trPredictorReductionSettings: ""),";
			put "      Here:tb1b = TextBox(""%nrbquote(&PRS)""),";
			put "      Here:tb1b << SetWrap(1000);";
			%if %length(&ASet) %then %do;
				put "      Here:tb2a = TextBox("" &trAnalysisSettings: ""),";
				put "      Here:tb2b = TextBox(""\[%nrbquote(&ASet)]\""),";
				put "      Here:tb2b << SetWrap(1000);";
			%end;
			put "      Here:tb3a = TextBox("" &trFinalSelectedVariables: ""),";
			%if ((&ProcessName = PartitionTrees) or (&ProcessName = VirtualTwins)) and %upcase(&TreeMethod)=TREE and ^&CVFlag %then %do;
				put "      Here:tb3b = TextBox(here:selvars),";
			%end;
			%else %do;
				put "      Here:tb3b = TextBox(""&SelVarShow""),";
			%end;
			put "      Here:tb3b << SetWrap(1000);";
			%if (&TestSpec = Yes) %then %do;
				put "      Here:tb4a = TextBox("" &trTestSetCriteria: ""),";
				%if ((&ProcessName = PartitionTrees) or (&ProcessName = VirtualTwins)) and %upcase(&TreeMethod)=TREE and ^&CVFlag %then %do;
					put "      Here:tb4b = TextBox(here:mdlsc_list[2]),";
				%end;
				%else %do;
					put "      Here:tb4b = TextBox(""&TestSC""),";
				%end;
				put "      Here:tb4b << SetWrap(1000);";
			%end;
			put "      Here:tb5a = TextBox("" &trTrainingSetCriteria: ""),";
			%if ((&ProcessName = PartitionTrees) or (&ProcessName = VirtualTwins)) and %upcase(&TreeMethod)=TREE and ^&CVFlag %then %do;
				put "      Here:tb5b = TextBox(here:mdlsc_list[1]),";
			%end;
			%else %do;
				put "      Here:tb5b = TextBox(""&TrainSC""),";
			%end;
			put "      Here:tb5b << SetWrap(1000);";
			%if (&ProcessName = GenomicBLUP) %then %do;
				%if (&RandSVDFlag) %then %do;
					%if %sysevalf(&RandSVD_K <= &nlim) %then %do;
						put "      Here:tb6a = TextBox("" Randomized SVD Setting: ""),";
					%end;
					%else %do;
						put "      Here:tb6a = TextBox("" Truncated SVD Setting: ""),";
					%end;
					put "      Here:tb6b = TextBox(""&SVD_SETTING_TEXT""),";
					%if (&RandSVD_errorFlag) %then %do;
						%if %sysevalf(&RandSVD_K <= &nlim) %then %do;
							put "      Here:tb7a = TextBox("" Randomized SVD Frobenius Measure: ""),";
							put "      Here:tb7b = TextBox(""&SVD_ERROR_TEXT""),";		
						%end;
						%else %do;
							put "      Here:tb7a = TextBox("" Truncated SVD Frobenius Measure: ""),";
							put "      Here:tb7b = TextBox(""&SVD_ERROR_TEXT""),";	
						%end;
					%end;
				%end;
			%end;
			put "   ),";
			put "   HListBox(";
			%if (&_dtype_=C) or (&NumDepAsClass=Yes) %then %do;
				put "      VListBox(Distribution(";
				%if (&TestSpec = Yes) %then %do;
					put "         By(:DataType),";
				%end;
				put "         Nominal Distribution(Column(Name(""&TruthVar""))),";
				put "         Continuous Distribution(Column(:Prob_Actual),More Moments(1)),";
				%if %length(%trim(&CustomCosts)) %then %do;
					put "         Continuous Distribution(Column(:Expected_Cost),More Moments(1)),";
				%end;
				put "         Nominal Distribution(Column(:Correct_Pred)),";
				%if (&ProcessName = GenomicBLUP) and ^&CVFlag %then %do;
					put "         Nominal Distribution(Column(:Predicted_Class))";
				%end;
				put "      )),";
				put "      VListBox(Contingency(";
				put "         Y( :Predicted_Class),";
				%if (&TestSpec = Yes) %then %do;
					put "         By(:DataType),";
				%end;
				put "         X(Name(""&TruthVar""))";
				put "      ))";
				put '   ))';
				put ');';	
	
				* roc;
				%if &nclasses=2 %then %do;
						put "Here:rocvb=Function({},{default local},";
						put "   If(IsEmpty(Here:roc_data),";
						put "      Here:roc_data=open(""&ClientOutPath.&_outrocname..sas7bdat"",invisible);";
						put "   );";
						put "   CurrentDataTable(Here:roc_data);";
						put "   HListBox(";
						put "      VListBox(Here:roc1=Overlay Plot(";
						put "         X(:Name(""1 - &trSpecificity"")),";
						put "      	  Y(:&trSensitivity),";
						%if (&TestSpec = Yes) %then %do;
							put "         By(:DataType),";
						%end;
						put "         Connect Thru Missing(1),";
						put "      	  SendToReport(";
						put "    //        Dispatch({}, ""Overlay Plot"", OutlineBox, Set Title(""ROC Curve"")),";
						put "            Dispatch({}, ""106"", ScaleBox, {Min(-0.01),Max(1.02),Inc(0.2), Show Major Grid(1), Show Minor Grid(1)}),";
						put "            Dispatch({}, ""101"", ScaleBox, {Min(-0.01),Max(1.02),Inc(0.2), Show Major Grid(1), Show Minor Grid(1)}),";
						put "		        Dispatch({}, ""Overlay Plot"", Framebox, ";
						put "               Add Graphics Script(PenColor(1);LineStyle(1);Line({-0.01,-0.01},{1.02,1.02}))";
						put "            )";						
						put "         )";
						put "      )),";
						put "      VListBox(";
						put "       CurrentDataTable(Here:roc_data);";
						put "			Here:roc2=Overlay Plot(";
						put "         X(:P_Event),";
						put "         Y(";
						put "            :&trSpecificity,";
						put "      	     :&trSensitivity,";
						put "      	     :&trTotalAccuracy,";
						put "      	     :&trPositivePredictiveValue,";
						put "      	     :&trNegativePredictiveValue,";
						put "      	     :&trMatthewsCorrelationCoeff,";
						put "      	     :&trYoudenIndex,";
						put "      	     :Name(""&trZeroOneIndex"")";
						put "         	),";
						%if (&TestSpec = Yes) %then %do;
							put "         By(:DataType),";
						%end;
						put "         Connect Points(1), Separate Axes(1),";
						put "      	  SendToReport(";
						put "    //        Dispatch({}, ""Overlay Plot"", OutlineBox, Set Title(""ROC Statistics"")),";
						put "            Dispatch({}, ""102"", ScaleBox, {Min(-0.01),Max(1.02),Inc(0.2), Show Major Grid(1), Show Minor Grid(1)}),";
						put "            Dispatch({}, ""101"", ScaleBox, {Min(-0.01),Max(1.02),Inc(0.2), Show Major Grid(1), Show Minor Grid(1)}),";
						put "		        Dispatch({}, ""Overlay Plot"")";
						put "        )";
						put "      ))";
						put "   ));";
				%end;
			%end;
			%else %do;
				put "      vlbd = VListBox(Distribution(";
				%if (&TestSpec = Yes) %then %do;
					put "         By(:DataType),";
				%end;
				put "         Continuous Distribution(Column(Name(""&TruthVar"")), More Moments(1)),";
				%if (&BJ_Est = Yes) %then %do;
					put "         Continuous Distribution(Column(Name(""&TruthVarBJ"")), More Moments(1)),";
				%end;
				put "         Continuous Distribution(Column(:Predicted_Value), More Moments(1)),";
				put "         Continuous Distribution(Column(:Residual), More Moments(1)),";
				%if (&BJ_Est = Yes) %then %do;
					put "         Continuous Distribution(Column(:Residual_BJ), More Moments(1))";
				%end;
				put "      )),";
				%if (&BJ_Est = Yes) %then %do;
					put "      vlbd[outlinebox(""Residual"")] << Close;";
					put "      vlbd[outlinebox(""Residual_BJ"")] << Close;";
					%if (&TestSpec = Yes) %then %do;
						put "      vlbd[outlinebox(""Residual"")] << Close;";
						put "      vlbd[outlinebox(""Residual_BJ"")] << Close;";
					%end;
				%end;
				put "      VListBox(";
				put "         HListBox(";
				put "            Bivariate(Y(Name(""&TruthVar"")), X( :Predicted_Value), ";
				%if (&TestSpec = Yes) %then %do;
					put "               By(:DataType),";
				%end;
				put "               Fit Special(Intercept(0),Slope(1),{Report(0), Line Color(""Red"")}),";
				put "            ),";
				%if (&BJ_Est = Yes) %then %do;
					put "            Bivariate(Y(Name(""&TruthVarBJ"")), X( :Predicted_Value), ";
					%if (&TestSpec = Yes) %then %do;
						put "               By(:DataType),";
					%end;
					put "               Fit Special(Intercept(0),Slope(1),{Report(0), Line Color(""Red"")}),";
					put "            ),";
				%end;
				put "         ),";
				put "         HListBox(";
				put "            Bivariate(Y( :Residual), X( :Predicted_Value),";
				%if (&TestSpec = Yes) %then %do;
					put "               By(:DataType),";
				%end;
				put "               Fit Spline( 1, {Line Color( ""Red"" ), Line Style( Smooth )} )";
				put "            ),";
				%if (&BJ_Est = Yes) %then %do;
					put "            Bivariate(Y( :Residual_BJ), X( :Predicted_Value), ";
					%if (&TestSpec = Yes) %then %do;
						put "               By(:DataType),";
					%end;
					put "               Fit Spline( 1, {Line Color( ""Red"" ), Line Style( Smooth )} )";
					put "            ),";
				%end;
				put "         )";
				put "      )";				
				put '   ))';
				put ');';
			%end;
			
			%if ((&_dtype_=C) or (&NumDepAsClass=Yes)) and &nclasses=2 %then %do;
				%if (&TestSpec = Yes) %then %do;
					put "Here:vb_after=Function({},{default local}," ;
					put "vbevi=EvalInsert(""\[";	
					put "Here:roc1rep = (Here:roc1<<report);";

					put "if(IsList(roc1rep),";
					put "	 Here:roc1rep[1][framebox(1)]<<Add Graphics Script(PenColor(1);LineStyle(1);Line({-0.01,-0.01},{1.02,1.02}));";
					put "	 Here:roc1rep[2][framebox(1)]<<Add Graphics Script(PenColor(1);LineStyle(1);Line({-0.01,-0.01},{1.02,1.02}));";
					put "  Here:roc1rep[1][outlinebox(1)]<<SetTitle(""ROC Curve DataType=Test"");";
					put "  Here:roc1rep[2][outlinebox(1)]<<SetTitle(""ROC Curve DataType=Training"");";
					put "  , // else only one plot";
					put "	 Here:roc1rep[framebox(1)]<<Add Graphics Script(PenColor(1);LineStyle(1);Line({-0.01,-0.01},{1.02,1.02}));";
					put "  Here:roc1rep[outlinebox(1)]<<SetTitle(""ROC Curve DataType=Training"");";
					put ");";
					
					put "Here:roc2rep = (roc2<<report);";
					put "if(IsList(roc2rep),";
					put "  Here:roc2rep[1][outlinebox(1)]<<SetTitle(""ROC Statistics DataType=Test"");";
					put "  Here:roc2rep[2][outlinebox(1)]<<SetTitle(""ROC Statistics DataType=Training"");";
					put "  , // else only one plot";
					put "	 Here:roc2rep[framebox(1)]<<Add Graphics Script(PenColor(1);LineStyle(1);Line({-0.01,-0.01},{1.02,1.02}));";
					put "  Here:roc2rep[outlinebox(1)]<<SetTitle(""ROC Statistics DataType=Training"");";
					put ");";
					put ']\"); ';
					put "Eval(Parse(vbevi));";	
					put "); //end func";
				%end;
			%end;
			
			* insert tabs;
			put "InsertInto(DataList,{{""Here:pred_data"",""Here:vars_data""}});";
			put "InsertInto(TabButtonList, Expr(locale:get(""ModelResults_Text"")));";
			put "InsertInto(TabCodeList,expr(Here:vb()));";
			put "InsertInto(TabAfterCodeList, "" "");";
			put "InsertInto(TabOpenList,1);";

			%if ((&_dtype_=C) or (&NumDepAsClass=Yes)) and &nclasses=2 %then %do;		
				put "InsertInto(DataList,{{""Here:roc_data""}});";
				put "InsertInto(TabButtonList, Expr(locale:get(""ROC_Text"")));";
				put "InsertInto(TabCodeList,expr(Here:rocvb()));";
				%if (&TestSpec = Yes) %then %do;
					put "InsertInto(TabAfterCodeList, expr(Here:vb_after()));";
				%end;
				%else %do ;
					put "InsertInto(TabAfterCodeList, "" "");";
				%end;
				put "InsertInto(TabOpenList,1);";
			%end;
			
			%if (&ProcessName = DiscriminantAnalysis) or (&ProcessName = KNearestNeighbors) %then %do;
				%if (&ncan >= 2) %then %do;
					put "Here:cs=Function({},{default local},";
					put "   If(IsEmpty(Here:pred_data),";
					put "      Here:pred_data=open(""&ClientOutPath.&&&_OutDataName...sas7bdat"",invisible);";
					put "   );";
					put "   CurrentDataTable(Here:pred_data);";
					put '   HListBox(';
					%if (&ncan=2) %then %do;
						put '      VListBox(Bivariate(Y( :Can2), X( :Can1), /* Density Ellipse(0.95, {Line Color("Red")}), */ '; 
						%if (&TestSpec = Yes) %then %do;
							put "         By(:DataType),";
						%end;
						put '         SendToReport(Dispatch({}, "Bivar Plot", FrameBox, {Marker Size(4), ';
						put '         Marker Drawing Mode(Outlined)}))))';
					%end;
					%else %do;
						put "      VListBox(Multivariate(Y( ";
						%do c = 1 %to %eval(&ncan-1);
							put "         :Can&c , " ;
						%end;
						put "         :Can&ncan),";
						%if (&TestSpec = Yes) %then %do;
							put "         By(:DataType),";
						%end;
						put "         Scatterplot Matrix(Density Ellipses(0), Show Correlations(0), Horizontal(1))";
						put "      ))";
					%end;
					put "   )";
					put ");";
					put " ";
					put "InsertInto(DataList,{{""Here:pred_data""}});";
					put "InsertInto(TabButtonList, Expr(locale:get(""CanonicalScores_Text"")));";
					put "InsertInto(TabCodeList,expr(Here:cs()));";
					put "InsertInto(TabAfterCodeList, "" "");";
					put "InsertInto(TabOpenList,1);";
				%end;
			%end;
			
			%if (&ProcessName = LogisticRegression) and (&VarSelect = Penalized) and &CVFlag %then %do;
				put "Here:vi=Function({},{default local},";
				put "   If(IsEmpty(Here:fit_data),";
				put "      Here:fit_data=open(""&ClientOutPath.&pdat._fit.sas7bdat"",invisible);";
				put "      Here:baserow=fit_data<<Get Rows Where(:Predictor 1==""Full Model"");";
				put "      Here:fit_data<<delete rows(baserow);";
				put "   );";
				put "   Current Data Table(Here:fit_data);";
				put "	Here:nvars=nrows(fit_data);";
				put "   if( nvars == 0,";
				put "	    TextBox(locale:get(""Nopredictorwasselectedbythemodel_Text""));";
				put "	    ,";
				put '       VListBox(Chart(';
				put "          X( :&trFinalSelectedVariables),";
   			    	put "          Y( :&trAveSquareErrorLift ),";
				put '          Horizontal( 1 ),';
				put '          Bar Chart( 1 ),';
				put '          SendToReport(';
				put "             Dispatch( {}, ""Chart"", OutlineBox, {Set Title( locale:get(""VariableImportance_Text"") )} ),";
				put '          )';
				put '       ));';
				put '    );';
				put ');';
				put "InsertInto(DataList,{{""Here:fit_data""}});";
				put "InsertInto(TabButtonList, Expr(locale:get(""VariableImportance_Text"")));";
				put "InsertInto(TabCodeList,expr(Here:vi()));";
				put "InsertInto(TabAfterCodeList, "" "");";
				put "InsertInto(TabOpenList,1);";
			%end;
			
			%if ((&ProcessName = PartitionTrees) or (&ProcessName = VirtualTwins)) and %upcase(&TreeMethod)=TREE and ^&CVFlag %then %do;
				put "InsertInto(DataList,{{""Here:ps_dt"",""Here:varimp_dt"",""Here:subset_imp_dt""}});";
				put "if(Host Is(Windows),";
				put "jslpath=ConvertFilePath(jg:LSpath[&LS_Index], absolute, windows)||""JSLFiles\"";";
				put "macropath=ConvertFilePath(jg:LSpath[&LS_Index], absolute, windows)||""MacroLib\"",";
				put "jslpath=ConvertFilePath(jg:LSpath[&LS_Index], absolute, POSIX)||""JSLFiles/"";";
				put "macropath=ConvertFilePath(jg:LSpath[&LS_Index], absolute, POSIX)||""MacroLib/"");";
				%if &nleaves > 1 %then %do;
					put "include(jslpath||""Partition Trees Viewer.jsl"");";
				%end;
			%end;
						
			%if (&ProcessName = PartialLeastSquares) %then %do;
				put " ";
				put "Here:vrs=Function({},{default local},";
				put "   If(IsEmpty(Here:pred_data),";
				put "      Here:pred_data=open(""&ClientOutPath.&&&_OutDataName...sas7bdat"",invisible);";
				put "   );"; 
				put "   CurrentDataTable(Here:pred_data);";
				put '   HListBox(';
				%if (&NumberOfComponents=2) %then %do;
					put '      VListBox(Bivariate(Y( :pls2), X( :pls1),'; 
					%if (&TestSpec = Yes) %then %do;
						put "         By(:DataType),";
					%end;
					put '         SendToReport(Dispatch({}, "Bivar Plot", FrameBox, {Marker Size(4), ';
					put '         Marker Drawing Mode(Outlined)}))))';
				%end;
				%else %if (&NumberOfComponents>2) %then %do;
					put "      , VListBox(Multivariate(Y( ";
					%do c = 1 %to %eval(&NumberOfComponents-1);
						put "         :pls&c , " ;
					%end;
					put "         :pls&NumberOfComponents),";
					%if (&TestSpec = Yes) %then %do;
						put "         By(:DataType),";
					%end;
					put "         Scatterplot Matrix(Density Ellipses(0), Show Correlations(0), Horizontal(1))";
					put "      ))";
					%if (&Include3D = Yes) %then %do;
						put "      , VListBox(Scatterplot 3D(";
						put "         Y(:pls1,:pls2,:pls3),";
						%if (&TestSpec = Yes) %then %do;
							put "         By(:DataType),";
						%end;
						put "         Frame3D(";
						put "            Set Marker Transparency( 0.6 ),";
						put "            Set Marker Quality( 1 ),";
						put "            Set Marker Scale( 2.5 )";
						put "         )";
						put "      ))";
					%end;
				%end;
				put "   )";
				put ");";
				put " ";
				put "InsertInto(DataList,{{""Here:pred_data""}});";
				put "InsertInto(TabButtonList, Expr(locale:get(""RowScores_Text"")));";
				put "InsertInto(TabCodeList,expr(Here:vrs()));";
				put "InsertInto(TabAfterCodeList, "" "");";
				put "InsertInto(TabOpenList,1);";
				%if (&PlotColScores=Yes) and (&NumberOfComponents>=2) %then %do;
					put " ";
					put "Here:vwp=Function({},{default local},";
					put "   If(IsEmpty(Here:vars_data),";
					put "      Here:vars_data=open(""&ClientOutPath.&&&_OutDataVarsName...sas7bdat"",invisible);";
					put "      Here:vars_data << Select Where(1);";
					put "      Here:vars_data << Markers(8);";
					put "      Here:vars_data << Colors(4);";
					put "      Here:vars_data << Clear Select;";
					put '      column("name") << label(1);';
					put "   );";
					put "   Current Data Table(Here:vars_data);";
					put '   HListBox(';
					%if (&NumberOfComponents=2) %then %do;
						put '      VListBox(Bivariate(Y( :pls2), X( :pls1),'; 
						put '         SendToReport(Dispatch({}, "Bivar Plot", FrameBox, {Marker Size(4), ';
						put '         Marker Drawing Mode(Outlined)}))))';
					%end;
					%else %do;
						put "      Multivariate(Y( ";
						%do c = 1 %to %eval(&NumberOfComponents-1);
							put "         :pls&c , " ;
						%end;
						put "         :pls&NumberOfComponents),";

						put "         Scatterplot Matrix(Density Ellipses(0), Show Correlations(0), Horizontal(1))";
						put "      )";
						%if (&Include3D = Yes) and (&NumberOfComponents>=3)  %then %do;
							put "      , Scatterplot 3D(";
							put "         Y(:pls1,:pls2,:pls3),";
							put "         Frame3D(";
							put "            Set Marker Transparency( 0.6 ),";
							put "            Set Marker Quality( 1 ),";
							put "            Set Marker Scale( 2.5 )";
							put "         )";
							put "      )";
						%end;
					%end;
					put "   )";
					put ");";
					put " ";
					put "InsertInto(DataList,{{""Here:vars_data""}});";
					put "InsertInto(TabButtonList, Expr(locale:get(""VariableWeights_Text"")));";
					put "InsertInto(TabCodeList,expr(Here:vwp()));";
					put "InsertInto(TabAfterCodeList, "" "");";
					put "InsertInto(TabOpenList,1);";
				%end;
			%end;	
			
			%if (&ProcessName = GenomicBLUP) and ^&CVFlag %then %do;

				/*Ordered predicted value by cross*/			
				put "Here:progheatmap = Function({},{default local}," ;
				put "   If(IsEmpty(Here:pred_data),";
				put "      Here:pred_data=open(""&ClientOutPath.&&&_OutDataName...sas7bdat"",UseLabelsForVarNames(false),invisible);";
				%if %length(&LabelVar) %then %do;
					put "      %str(column(%"&LabelVar%") << label(1);)";
				%end;
				put "      Here:pred_data << Select Where(1);";
				put "      Here:pred_data << Markers(8);";
				put "      Here:pred_data << Clear Select;";
				%if (&_dtype_=C) or (&NumDepAsClass=Yes) %then %do;
					put "      Here:pred_data<<Color By Column(Name(""&TruthVar""));";
					put '      for each row( Selected(Rowstate())=(:Correct_Pred=="No"));';
					put '      Here:pred_data<<Markers(2);';
					put '      Here:pred_data<<Clear Select;';
					put "      Column(""&TruthVar"") << Modeling Type(" '"Nominal");';
					put '      :Correct << Modeling Type("Nominal");';
					put ' ';
				%end;
				%if %symexist(ColorVar) %then %do;
					%if %length(&ColorVar) %then %do;
						put "      Here:pred_data<<Color By Column(:&ColorVar);";
					%end;
				%end;	
				put "   );";
				put "   Current Data Table(Here:pred_data);";
				put " Here:hlb = HListBox(";
				put "		gb_heat= Graph Builder(";
				put "			Show Control Panel( 0 ),";
				put "			Variables(";
				put "				X(";
				put "				:_Cross_,";
				%if (&TraitType = Continuous) %then %do;
					put "				Order By( Column(""Predicted_Value""), Ascending, Order Statistic( ""Mean"" ) )";
				%end;
				%else %do;
					put "				Order By( Column(""Pred""), Ascending, Order Statistic( ""Mean"" ) )";
				%end;
				put "			),";
				%if (&TraitType = Continuous) %then %do;
					put "			Y( Column(""Predicted_Value"") ),";
				%end;
				%else %do;
					put "			Y( Column(""Pred"") ),";
				%end;
				put "		),";
				put "		Elements( Heatmap( X, Y, Legend( 4 ) ) ),";
				put "		SendToReport(";
				put "			Dispatch(";
				put "				{},";
				put "				""Graph Builder"",";
				put "				OutlineBox,";
				%if (&TraitType = Continuous) %then %do;
					put "			{Set Title( ""Graph Builder = &DepVar Predicted_Value"" )}";
				%end;
				%else %do;
					put "			{Set Title( ""Graph Builder = &DepVar Pred"" )}";
				%end;
				put "			),";
				put "			Dispatch(";
				put "				{},";
				put "				""graph title"",";
				put "				TextEditBox,";
				%if (&TraitType = Continuous) %then %do;
					put "				{Set Text( ""Cross ID ordered by Predicted_Value (Ascending)"" )}";
				%end;
				%else %do;
					put "				{Set Text( ""Cross ID ordered by Pred (Ascending)"" )}";
				%end;
				put "			),";
				put "			Dispatch(";
				put "				{},";
				put "				""400"",";
				put "				ScaleBox,";
				put "				{Legend Model(";
				put "					4,";
				put "					Properties(";
				put "						0,";
				put "						{gradient(";
				put "							{Color Theme( ""Spectral"" ), Scale Values( [0 .] ),";
				put "							Range Type( ""Exact Data Range"" )}";
				put "						)}";
				put "					)";
				put "				)}";
				put "			)";
				put "		)";
				put "	),";
				put "	);";
				put "	);";
				put "InsertInto(DataList,{{""Here:pred_data""}});";
				put "InsertInto(TabButtonList, ""Simulated Trait Ordered"");";
				put "InsertInto(TabCodeList,expr(Here:progheatmap()));";
				put "InsertInto(TabAfterCodeList, "" "");";
				put "InsertInto(TabOpenList,1);";			
			
				/*Progeny Simulation Follow-up*/
				put "ProgenySimulationExpr = Expr(";
				put "	gbdata = current data table();";
				put "	nsel = NRows(gbdata<<GetSelectedRows);";
				put "	if(nsel==0,";
				/*
				put "		Dialog(""No rows are selected in the currently active JMP table.  Make sure "" ||";
				put "			""a table with cross evaluations is active and then select points in associated graphs or rows "" ||";
				put "			""from the table.""),";
				*/
				put "		New Window(""JMP Genomics Message"", <<Modal,";
				put "   	   Text Box(""No rows are selected in the JMP table from the Model Results tab. "" ||";
				put "			""Make sure a table with predicted values and marker genotypes is active "" ||";
				put "			""and then select points in associated graphs or rows "" ||";
				put "			""from the table associated with the Model Results tab."", <<set wrap(600)),";	
				put "		),";
				put "		lstColNames = gbdata << Get Column Names(string);";
				put "		correct_table= 0;";
				put "		for(i=1,i<=length(lstColNames),i++,";
				put "			if(lowercase(lstColNames[i])==""predicted_class"" | lowercase(lstColNames[i])==""predicted_value"", ";
				put "				correct_table= 1;";
				put "				i=length(lstColNames) + 1;";
				put "				,";
				put "			);";
				put "		);";
				put "   	if(correct_table==0,";
				put "			tabname= gbdata << Get Name();";
				put "			New Window( <<Modal,";
				put "				Eval(Parse(EvalInsert(""\[";
				put "					Text Box(""The current active JMP table ^tabname^ is not correct."", <<set wrap(600))";
				put "				]\"" ";
				put "				)))";
				put "			);";				
				put "			,";
				put "			gbdata << Clear Column Selection();";
				/*put "			selcross = gbdata << Subset(output table name(""selected_lines""));";*/
				put "			selcross = gbdata << Subset(output table name(""&slout""));";
				/*put "			selcross << Delete Columns({:_train_, :_obs});";*/
				/*put "			selcross << Save(""&ClientOutPath.selected_lines.sas7bdat"",SAS);";*/
				put "			selcross << Save(""&ClientOutPath.&slout..sas7bdat"",SAS);";
				put "      		if(host is(Windows),";
				put "      			Include(ConvertFilePath(""$HOME/JMPG/ProcessLibrary/ProgenySimulation.jsl"", absolute, windows)),";
				put "      			Include( ConvertFilePath(""$HOME/JMPG/ProcessLibrary/ProgenySimulation.jsl"", absolute, POSIX))";
				put "      		);";
				put "	   		jg:setCurrentParmValue(1,""ProgenySimulation"", jg:getParmNum(""ProgenySimulation"",""StudyName""),""&StudyName"");";
				/*put "      		jg:setCurrentParmValue(1,""ProgenySimulation"", jg:getParmNum(""ProgenySimulation"",""InData""),""&InData0"");";*/
				/*put "      		jg:setCurrentParmValue(1,""ProgenySimulation"", jg:getParmNum(""ProgenySimulation"",""InData""),""&ClientOutPath.selected_lines.sas7bdat"");";*/
				put "      		jg:setCurrentParmValue(1,""ProgenySimulation"", jg:getParmNum(""ProgenySimulation"",""InData""),""&ClientOutPath.&slout..sas7bdat"");";				
				/*put "      		jg:setCurrentParmValue(1,""ProgenySimulation"", jg:getParmNum(""ProgenySimulation"",""CInData""),""&ClientOutPath.selected_lines.sas7bdat"");";*/
				put "      		jg:ProgenySimulation_updateDeps;";	
				/*put "      		jg:setCurrentParmValue(1,""ProgenySimulation"", jg:getParmNum(""ProgenySimulation"",""IDVars""),""&IDVars"");";*/
				/*put "      		jg:setCurrentParmValue(1,""ProgenySimulation"", jg:getParmNum(""ProgenySimulation"",""SexVar""),""&SexVar"");";*/	
				/*put "      		jg:setCurrentParmValue(1,""ProgenySimulation"", jg:getParmNum(""ProgenySimulation"",""Maternal""),""&Maternal"");";*/
				/*put "      		jg:setCurrentParmValue(1,""ProgenySimulation"", jg:getParmNum(""ProgenySimulation"",""MatingType""),""&MatingType"");";*/	
				/*put "      		jg:setCurrentParmValue(1,""ProgenySimulation"", jg:getParmNum(""ProgenySimulation"",""TesterVar""),""&TesterVar"");";*/	
				put "      		jg:ProgenySimulation_updateDeps;";	
				/*put "    		jg:setCurrentParmValue(1,""ProgenySimulation"", jg:getParmNum(""ProgenySimulation"",""WhereStmt""),"""");";*/	
				/*put "      		jg:setCurrentParmValue(1,""ProgenySimulation"", jg:getParmNum(""ProgenySimulation"",""AlleleVars""),""&SelMarkerVars"");";*/	
				put "      		jg:setCurrentParmValue(1,""ProgenySimulation"", jg:getParmNum(""ProgenySimulation"",""ListMarkerVars""),""&ListSelMarkerVars"");";
				/*put "      		jg:setCurrentParmValue(1,""ProgenySimulation"", jg:getParmNum(""ProgenySimulation"",""ScoringCode""),""&ScoringCode"");";*/		
				put "      		jg:setCurrentParmValue(1,""ProgenySimulation"", jg:getParmNum(""ProgenySimulation"",""OutPath""),""&ClientOutPath"");";
				put "      		jg:setCurrentParmValue(1,""ProgenySimulation"", jg:getParmNum(""ProgenySimulation"",""GenomicBLUPOutData""),""&ClientOutPath.&&&_outdataname...sas7bdat"");";
				put "      		jg:ProgenySimulation_updateDeps;";
				%if %length(&AnnoInData) %then %do;
					%if &nimk < &nmrkin or %length(&AnnoWhereStmt) %then %do;
						put "      		jg:setCurrentParmValue(1,""ProgenySimulation"", jg:getParmNum(""ProgenySimulation"",""AnnoInData""),""&ClientOutPath.&AnnoOutDataName..sas7bdat"");";				
					%end;
					%else %do;
						put "      		jg:setCurrentParmValue(1,""ProgenySimulation"", jg:getParmNum(""ProgenySimulation"",""AnnoInData""),""&AnnoInData"");";
					%end;
					put "      		jg:setCurrentParmValue(1,""ProgenySimulation"", jg:getParmNum(""ProgenySimulation"",""ChromNum""),""&ChromID"");";	
					put "      		jg:setCurrentParmValue(1,""ProgenySimulation"", jg:getParmNum(""ProgenySimulation"",""ChromLab""),""&ChromLab"");";	
					put "      		jg:setCurrentParmValue(1,""ProgenySimulation"", jg:getParmNum(""ProgenySimulation"",""MarkerLab""),""&MarkerLab"");";		
					put "      		jg:setCurrentParmValue(1,""ProgenySimulation"", jg:getParmNum(""ProgenySimulation"",""MarkerPos""),""&MarkerPos"");";
					put "      		jg:setCurrentParmValue(1,""ProgenySimulation"", jg:getParmNum(""ProgenySimulation"",""MarkerPos_BP""),""&MarkerPos_BP"");";
					put "      		jg:ProgenySimulation_updateDeps;";
				%end;
				/*put "      		jg:runProcess( 1,""ProgenySimulation"" );";*/
				put "   	);";
				put "   )";
				put ");";
				put "Here:ActionButtonDesc = {};";
				put "Here:ActionButtonList={{}};";
				put "Here:ActionCodeList ={{}};";
				put "InsertInto(Here:ActionButtonDesc,{expr(""Select crosses then click"")});";
				put "InsertInto(ActionButtonList[1],{expr(""Simulate Progeny"")});";
				put "InsertInto(ActionCodeList[1],expr(ProgenySimulationExpr));";			
			%end;
		run;
		
		%goto End_JSL;

		%SurvivalJSL:;
		* output JSL code for Survival Modeling;
		* this part shall be revised if applying any accessment;

		%if %length(&ColorVar) %then %do;
			* extract varable values for &ColorVar for coloring points in one-way plot for median survival;
			proc sort data=&SPM_OutData3(keep=&ColorVar &IDVar _DataType_) out=cmap;
				by &ColorVar;
			run;

			proc sort data=cmap out=cmap2 nodupkey; 
				by &ColorVar;
			run; 
				
			data _null_;
				set cmap2 end=_e_; 
				retain _ColorIndex_ 0;
				name = "_vColorVar"||trim(left(_n_));
				call symput(name,trim(left(&ColorVar)));
				_ColorIndex_ = _ColorIndex_ + 1;	
				if _colorIndex_ > 36 then _ColorIndex_ = _ColorIndex_ - 36;
				name = "_iColorVar"||trim(left(_n_)); 
				call symput(name,trim(left(_ColorIndex_)));
				if _e_ then call symput("_nColor",_n_);
			run; 

			* Set color index for each IDVar to be applied in JSL;
			data cmap;
				set cmap;
				by &colorVar;
				retain _ColorIndex_ 0;
				if _DataType_ ne "";
				if first.&ColorVar then _ColorIndex_ = _ColorIndex_ + 1;
				if _colorIndex_ > 36 then _ColorIndex_ = _ColorIndex_ - 36;
			run; 
		%end;
		%else %do;
			data cmap;
				set &SPM_OutData3(keep=&ColorVar &IDVar _DataType_);
				if _DataType_ ne "";
				_colorIndex_ = 1;
			run; 

			%let _nColor = 1;
		%end;	

		%if %length(&TestInData) %then %do;
			data cmap cmap_test;
				set cmap;
				if _DataType_ = "Training" then output cmap;
				else if _DataType_ = "Test" then output cmap_test;
			run; 

			data _null_;
				set cmap_test end=_e_;
				%if %length(&ColorVar) %then %do;
					by &ColorVar;
					retain _nColor_ 0;
					if first.&ColorVar then _nColor_+1;
					name = "_CVar_te"||trim(left(_n_));
					call symput(name,&ColorVar);
				%end;
				name = "_IDVar_te"||trim(left(_n_));
				call symput(name,&IDVar);
				name = "_cIDVar_te"||trim(left(_n_));
				call symput(name,_ColorIndex_);
				if _e_ then do;
					%if %length(&ColorVar) %then %do;
						call symput("_nColor_te",_nColor_);
					%end;
					call symput("_nTest",_n_);
				end;
			run;
			%if ^%length(&ColorVar) %then %let _nColor_te = 1;
		%end;

		data _null_;
			set cmap end=_e_;	
			%if %length(&ColorVar) %then %do;
				by &ColorVar;
				retain _nColor_ 0;
				if first.&ColorVar then _nColor_+1;	
				name = "_CVar_te"||trim(left(_n_));
				call symput(name,&ColorVar);
			%end;
			name = "_IDVar_tr"||trim(left(_n_));
			call symput(name,&IDVar);
			name = "_cIDVar_tr"||trim(left(_n_));
			call symput(name,_ColorIndex_);
			if _e_ then do;				
				%if %length(&ColorVar) %then %do;
					call symput("_nColor_tr",_nColor_);
				%end;
				call symput("_nTrain",_n_);
			end;
		run; 
		%if ^%length(&ColorVar) %then %let _nColor_tr = 1;

		%if %length(&TestInData) %then %do;
			%let _nSample1 = &_nTest;
			%let _DataTable1 = %scan(&SPM_OutData2,2,.).sas7bdat;
			%let _IDName = _IDVar_te; 
			%let _cIDName = _cIDVar_te; 
			%let _DataType1 = Test;
		%end;
		%else %do; 
			%let _nSample1 = &_nTrain;
			%let _DataTable1 = %scan(&SPM_OutData,2,.).sas7bdat; 
			%let _IDName = _IDVar_tr; 
			%let _cIDName = _cIDVar_tr;
			%let _DataType1 = Training;
		%end;

		* check missing results;
		%let nsResults1 = 0;
		data _null_;
			set &SPM_OutData3(keep=_train_ &IDVar _DataType_);
			by descending _train_;
			retain _c_;
			if first._train_ then _c_=0;
			_c_+1;
			if last._train_ then do;
				if _train_ = 1 then call symput("nsResults1",_c_);
				else if _train_ = 0 then call symput("nsResults0",_c_);
			end;
		run;
		%let nmissingResults1 = %eval(&nsResults1-&nAllObs_sTrain);
		%put nmissingResults1 = &nmissingResults1;
		%if %length(&TestInData) %then %let nmissingResults0 = %eval(&nsResults0-&nAllObs_sTest);
		%else %let nmissingResults0 = 0;
		%global _mainjslfile_;
		%let _mainjslfile_ = %quote(&&&_JSLFile);
		
		%let IDVarLabel = &IDVar;
		%if %length(&IDVarLabel) %then %do;
			%CheckLabel(&SPM_OutData3,IDVarLabel);
		%end;
		%let ColorVarLabel = &ColorVar;
		%if %length(&ColorVarLabel) %then %do;
			%CheckLabel(&SPM_OutData3,ColorVarLabel);
		%end;

		data _null_;
			file "&&&_JSLFile"  encoding="utf-8";
			put "//!"; 			
			put 'Names Default to Here(1);';
			put "Here:DataList = {};";
			put "Here:TabButtonList = {};";
			put "Here:TabCodeList = {};";
			put "Here:TabAfterCodeList = {};";
			put "Here:TabOpenList = {};";
			put " ";
			
   			* Set 36 colors (may be refined);
			put '//color list to be applied for groupped coloring';
   			put 'Here:ColorList = {3,5,6,7,8,4,9,10,11,12,13,14,' /
				'   51,53,54,55,56,52,57,58,59,60,61,62,' /
				'   67,69,70,71,72,68,73,74,75,76,77,78};';
			put " ";
			
			
			put 'Here:Survival=Function({}, {default local},';
			put "   If(IsEmpty(Here:vars_data),";
			put "      Here:vars_data=open(""&ClientOutPath.&&&_OutDataVarsName...sas7bdat"",invisible);"; 
			put "   );";
			put '   current data table(Here:vars_data);';
			put '   vListBox('; 
			put '      LineUpBox(NCol(2),Spacing(3),';
			put "         Here:tb1a = TextBox("" &trPredictorReductionSettings: ""),";
			put '         Here:tb1b = TextBox(" ' "%nrbquote(&PRS)" '"),';
			put '         Here:tb1b << SetWrap(1000);';
			%if %length(&ASet) %then %do;
				put "         Here:tb2a = TextBox("" &trAnalysisSettings: ""),";
				put "         Here:tb2b = TextBox("" \[%nrbquote(&ASet)]\ ""),";
				put '         Here:tb2b << SetWrap(1000);';
			%end;
			put "         Here:tb3a = TextBox("" &trFinalSelectedVariables: ""),";
			put '         Here:tb3b = TextBox(" ' "&SelVarShow" '"),';
			put '         Here:tb3b << SetWrap(1000);';
			
			%if (&TestSpec = Yes) %then %do;
				put "         tb5a = TextBox("" &trTestSetCriteria: ""),";
				put '         tb5b = TextBox(" ' "&trHarrellsCStatistic = &Harrells_C2_2" '"),';
				put '         tb5b << SetWrap(1000);';
			%end;
			
			put "         Here:tb4a = TextBox("" &trTrainingSetCriteria: ""),";
			put '         Here:tb4b = TextBox(" ' "&trHarrellsCStatistic = &Harrells_C2" '"),';
			put '         Here:tb4b << SetWrap(1000);';
			
			* put "         Here:tb5a = TextBox("" &trReferenceSurvivalTime: ""),";
			put "         Here:tb5a = TextBox("" Reference Survival Times: ""),";
			put '         Here:tb5b = TextBox(" ' "&RefTime" '"),';
			put '         Here:tb5b << SetWrap(1000);';
			%if &nmissingResults1 %then %do;
				put '         Here:tb7a = TextBox(" " || locale:get("Warning_Text") || ": "),';
				%if ^&nmissingResults0 %then %do;
					%if &nmissingResults1 = 1 %then %do;
						put '         Here:tb7b = TextBox(" " || locale:get("Oneobservationismissing_Text") || " "),';
					%end;
					%else %do;
						put "         Here:tb7b = TextBox("" "" || locale:fill(locale:get(""Xobservationsaremissing_Text""), {""%trim(&nmissingResults1)""}) || "" ""),";
					%end;
				%end;
				%else %do;
					put "         Here:tb7b = TextBox("" "" || locale:fill(locale:get(""XandXobservationsinTrainingTestDataSetsMissing_Text""),
						{""%trim(&nmissingResults1)"", ""%trim(&nmissingResults0)""}) || "" ""),";
				%end;
				put '         Here:tb7b << SetWrap(1000);';
			%end;
			put '      ),';
			%if &nmissingResults1 %then %do;
				put '      Here:tb8=TextBox(locale:get("MissingResultsDueEitherToMissInvValsMsg_Text")),';
				put '      Here:tb8 << SetWrap(1000);';
				put '      textbox(" "),';
			%end;

			put '      hListBox(';
			*****  one plot for suvival function   ********;
			put "         If(IsEmpty(Here:Sur_data_1),";
			put "            Here:Sur_data_1=open(""&ClientOutPath.%scan(&SPM_OutData3,2,.).sas7bdat"",UseLabelsForVarNames(true),invisible);";
			%if %length(&IDVarLabel) %then %do;
				put '            Column("' "&IDVarLabel" '") << Label(1);';
			%end;
			put "         );";						
			put "			  current data table(Here:Sur_data_1);";
			put '         vListBox(';
			put '            Here:SurFunc1 = Parallel Plot(';
			put '               Scale Uniformly( 1 ),';
			put '               Center at zero( 0 ),';
			put '               Y( ';
			%do i = 1 %to &nBin;
				put '                  :Name("' "%trim(%left(&&wide_lList&i))" '"), ';
			%end;
			put '                  :Name("' "%trim(%left(&&wide_lList&i))" '")';
			put '               ),';

			%if ^%length(&TestInData) %then %do;
				put '               SendToReport(';
				put '                  Dispatch(';
				put '                     {},';
				put '                     "Parallel Plot",';
				put '                     OutlineBox,';
				put "                     {Set Title( ""Survival Curves: DataType=&_DataType1"" )}";
				put '                  ),';
				put '                  Dispatch( {}, "Parallel Coord", FrameBox, {Frame Size( 450, 225 )} )';
				put '               )';
			%end;
			%else %do;
				put "               By( :DataType ),";
				put "               SendToByGroup(";
				put '                  {:DataType == "Test"},';
				put '                  SendToReport(';
				put '                     Dispatch(';
				put '                        {},';
				put '                        "Parallel Plot DataType=Test",';
				put '                        OutlineBox,';
				put '                        {Set Title( "Survival Plot: DataType=Test" )}';
				put '                     ),';
				put '                     Dispatch(';
				put '                        {"Parallel Plot DataType=Test"},';
				put '                        "Parallel Coord",';
				put '                        FrameBox,';
				put '                       {Frame Size( 450, 220 )}';
				put '                     )';
				put '                  )';
				put '               ),';
				put "               SendToByGroup(";
				put '                  {:DataType == "Training"},';
				put '                  SendToReport(';
				put '                     Dispatch(';
				put '                        {},';
				put '                        "Parallel Plot DataType=Training",';
				put '                        OutlineBox,';
				put '                        {Set Title( "Survival Plot: DataType=Training" )}';
				put '                     ),';
				put '                     Dispatch(';
				put '                        {"Parallel Plot DataType=Training"},';
				put '                        "Parallel Coord",';
				put '                        FrameBox,';
				put '                       {Frame Size( 450, 220 )}';
				put '                     )';
				put '                  )';
				put '               )';
			%end;
  			put '            )'; *end of Parallel Plot;
			put '         ),'; * end of vListBox for Parallel Plot;

			*****  one plot of distribution of reference survival estimates   *****;
			%if %length(&ColorVar) %then %do;
				* set color state for rows;
				put '         for each row(';
 				put '            if(' "char(:Name( ""&ColorVarLabel"" )) == " '"' "&_vColorVar1" '",';
  				put '               ColorOf( row state() ) = Here:ColorList[ 1 ];';
				%do i = 2 %to &_nColor;
					put '            ,';
  					put "               char(:Name( ""&ColorVarLabel"" )) == " '"' "&&_vColorVar&i" '",'; 
  					put "               ColorOf( row state() ) = Here:ColorList[ &&_iColorVar&i ];"; 
				%end;
				put '            )';
  				put '         );';
				* create One-Way plot for grouped reference survival estimates;
				put '         vListBox(';
											
			%do r = 1 %to &nRefTime; 
				* repeat this part for each reference time point;
  				put "            Here:RefSur&r = Oneway(";
  				put "               Y( :Reference Survival Estimate at &&RefTime&r),";
  				put "               X( :Name( ""&ColorVarLabel"" ) ),";
  				put '               With Best( 1 ),';
  				put '               Box Plots( 1 ),';
  				put '               Mean Diamonds( 1 ),';
  				put '               Comparison Circles( 1 ),';
  				put '               X Axis Proportional( 0 ),';
  				put '               Points Jittered( 1 ),';
				%if ^%length(&TestInData) %then %do;
  					put '               SendToReport(';
  					put '                  Dispatch(';
  					put '                     {},';
  					put '                     "Oneway Analysis of Reference Survival Estimate at ' "&&RefTime&r" ' By ' "&ColorVarLabel" '",';
  					put '                     OutlineBox,';
  					put "                     Set Title( ""Reference Survival Estimate at &DepVar=&&RefTime&r By &ColorVar.: DataType=&_DataType1"" )"; 
  					put '                  ),';
  					put '                  Dispatch( {}, "Means Comparisons", OutlineBox, Close( 1 ) )';
  					put '               )';
				%end;
				%else %do;
					put "               By( :DataType ),";
					put '               SendToByGroup(';
					put '                  {:DataType == "Test"},';
  					put '                  SendToReport(';
  					put '                     Dispatch(';
  					put '                        {},';
  					put '                        "Oneway Analysis of Reference Survival Estimate at ' "&&RefTime&r" ' By ' "&ColorVarLabel" ' DataType=Test",';
  					put '                        OutlineBox,';
  					put "                        Set Title( ""Reference Survival Estimate at &DepVar=&&RefTime&r By &ColorVar.: DataType=Test"" )"; 
  					put '                     ),';
  					put '                     Dispatch( {}, "Means Comparisons", OutlineBox, Close( 1 ) )';
  					put '                  )';
					put '               ),';
					put '               SendToByGroup(';
					put '                  {:DataType == "Training"},';
  					put '                  SendToReport(';
  					put '                     Dispatch(';
  					put '                        {},';
  					put '                        "Oneway Analysis of Reference Survival Estimate at ' "&&RefTime&r" ' By ' "&ColorVarLabel" ' DataType=Training",';
  					put '                        OutlineBox,';
  					put "                        Set Title( ""Reference Survival Estimate at &DepVar=&&RefTime&r By &ColorVar.: DataType=Training"" )"; 
  					put '                     ),';
  					put '                     Dispatch( {}, "Means Comparisons", OutlineBox, Close( 1 ) )';
  					put '                  )';
					put '               ),';
				%end;
				%if &r = &nRefTime %then %do;
  					put '            ),';
  				%end;
  				%else %do;
  					put '            );';  				
  				%end;  				
  			%end; 				
  				
  				put '         ),';
			%end;
			%else %do; 
				* create Distribution for referal survival estimates;
				%if ^%length(&TestInData) %then %do;
					%do r = 1 %to &nRefTime; 
						%if &r = &nRefTime %then %do;
  							put "         Distribution( Continuous Distribution( Column( :Reference Survival Estimate at &&RefTime&r) ) );";
  						%end;
  						%else %do;
  							put "         Distribution( Continuous Distribution( Column( :Reference Survival Estimate at &&RefTime&r) ) ),";
  						%end;
  					%end;
				%end;
				%else %do;
					%do r = 1 %to &nRefTime; 
						%if &r = &nRefTime %then %do;
  							put '         Distribution( Continuous Distribution( Column( :Reference Survival Estimate ) ), By( :DataType ) );';
  						%end;
  						%else %do;
  							put '         Distribution( Continuous Distribution( Column( :Reference Survival Estimate ) ), By( :DataType ) ),';
  						%end;
  					%end;
				%end;
			%end;
			put '      );'; * end of vListBox for Reference Survival Estimate; 

			put '   );'; * end of vListBox;
			put ');';
			
			%if %length(&TestInData) %then %do;
				put "InsertInto(DataList,{{"":Here:vars_data"",""Here:Sur_data_1"",""Here:Sur_data_2"",""Here:RefSur_data_1"",""Here:RefSur_data_2""}});";
			%end;
			%else %do;
				put "InsertInto(DataList,{{"":Here:vars_data"",""Here:Sur_data_1""}});";
			%end;
			put "InsertInto(TabButtonList, Expr(locale:get(""Results_Text"")));";
			put "InsertInto(TabCodeList,expr(Here:Survival()));";
			put "InsertInto(TabAfterCodeList, "" "");";
			put "InsertInto(TabOpenList,1);";
			/*
			%if ((&ProcessName = PartitionTrees) or (&ProcessName = VirtualTwins)) and %upcase(&TreeMethod)=TREE and ^&CVFlag %then %do;
				data _null_;
					file "&&&_JSLFile" mod;
					put "include(jslpath||""\Partition Trees Viewer Selection.jsl"");";
				run;
			%end;
			*/
		run;

		/* Old code apply overlay plot for drawing survival function		
		data _null_;
			file "&&&_JSLFile";
			put "//!"; 			
			put 'Names Default to Here(1);';
			put "Here:DataList = {};";
			put "Here:TabButtonList = {};";
			put "Here:TabCodeList = {};";
			put "Here:TabAfterCodeList = {};";
			put "Here:TabOpenList = {};";
			put " ";
			
   		* Set 36 colors (may be refined);
			put '//color list to be applied for groupped coloring';
   		put 'Here:ColorList = {3,5,6,7,8,4,9,10,11,12,13,14,' /
				'   51,53,54,55,56,52,57,58,59,60,61,62,' /
				'   67,69,70,71,72,68,73,74,75,76,77,78};';
			put " ";
			
			
			put 'Here:Survival=Function({}, {default local},';
			put "   If(IsEmpty(Here:vars_data),";
			put "      Here:vars_data=open(""&ClientOutPath.&&&_OutDataVarsName...sas7bdat"",invisible);"; 
			put "   );";
			put '   current data table(Here:vars_data);';
			put '   vListBox('; 
			put '      LineUpBox(NCol(2),Spacing(3),';
			put "         Here:tb1a = TextBox("" &trPredictorReductionSettings: ""),";
			put '         Here:tb1b = TextBox(" ' "%nrbquote(&PRS)" '"),';
			put '         Here:tb1b << SetWrap(1000);';
			%if %length(&ASet) %then %do;
				put "         Here:tb2a = TextBox("" &trAnalysisSettings: ""),";
				put "         Here:tb2b = TextBox(""\[%nrbquote(&ASet)]\""),";
				put '         Here:tb2b << SetWrap(1000);';
			%end;
			put "         Here:tb3a = TextBox("" &trFinalSelectedVariables: ""),";
			put '         Here:tb3b = TextBox(" ' "&SelVarShow" '"),';
			put '         Here:tb3b << SetWrap(1000);';
			
			%if (&TestSpec = Yes) %then %do;
				put "         tb5a = TextBox("" &trTestSetCriteria: ""),";
				put '         tb5b = TextBox(" ' "&trHarrellsCStatistic = &Harrells_C2_2" '"),';
				put '         tb5b << SetWrap(1000);';
			%end;
			
			put "         Here:tb4a = TextBox("" &trTrainingSetCriteria: ""),";
			put '         Here:tb4b = TextBox(" ' "&trHarrellsCStatistic = &Harrells_C2" '"),';
			put '         Here:tb4b << SetWrap(1000);';
			%if &nmissingResults1 %then %do;
				put '         Here:tb7a = TextBox(" " || locale:get("Warning_Text") || ": "),';
				%if ^&nmissingResults0 %then %do;
					put "         Here:tb7b = TextBox("" "" || locale:fill(locale:get(""Xobservationsintrainingdatasetarenotused_Text""), {""%trim(&nmissingResults1)""}) || "" ""),";
				%end;
				%else %do;
					put "         Here:tb7b = TextBox("" "" || locale:fill(locale:get(""XandXobservationsTrainingTestSetsNotUsed_Text""),
						{""%trim(&nmissingResults1)"", ""%trim(&nmissingResults0)""}) || "" ""),";
				%end;
				put '      	Here:tb7b << SetWrap(1000);';
			%end;
			put ' 	  ),';
			%if &nmissingResults1 %then %do;
				put '      Here:tb8=TextBox(locale:get("MissingResultsDueEitherToMissInvValsMsg_Text")),';
				put '      Here:tb8 << SetWrap(1000);';
				put '      textbox(" "),';
			%end;

			put '      hListBox(';
			*****  one plot for suvival function   ********;
			put "         If(IsEmpty(Here:Sur_data_1),";
			put "            Here:Sur_data_1=open(""&ClientOutPath.&_DataTable1"",invisible);";
			put "         );";
						
			put "			  current data table(Here:Sur_data_1);";
			put '         Here:SurFunc1 = Overlay Plot(';
			put "            X( :&DepVar ),";
			put '            Y( ';
			%do i = 1 %to %eval(&_nSample1-1);
				put '               :Name("' "&&&_IDName.&i" '"), ';
			%end;
			put '               :Name("' "&&&_IDName.%trim(%left(&_nSample1))" '")';
			put '            ),';
			put '            Overlay Axis << {';
			put '               Min( -0.02 ), Max( 1.02 ), Minor Ticks( 1 ),';
			put '               Add Ref Line( 0.25, Dotted, "Black" ),';	
			put '               Add Ref Line( 0.5, Dotted, "Black" ),';
			put '               Add Ref Line( 0.75, Dotted, "Black" )';
			put '            },';
			put '            Separate Axes( 1 ),';
			put '            X Axis << {';
			put '               Min( -1 ),';
			put "               Add Ref Line( &RefTime, Dotted, ""Black"" )";
			put '            },';
			put '            Step(1),';
			put '            Show Points(0),'; 
			%if %length(&ColorVar) %then %do;
				* handle grouped colors here!!!; * may need to refine this part for test dataset!!!;
				%do i = 1 %to &_nSample1;
					put '            :Name("' "&&&_IDName.&i" '")' "( Connect Color( ColorList[ %trim(%left(&&&_cIDName.&i)) ] ) ),";
				%end;
			%end;
			put '            SendToReport(';
  			put '               Dispatch(';
  			put '                  {},';
  			put '                  "Overlay Plot",';
  			put '                  OutlineBox,';
  			put '                  Set Title( "Estimated Survival Function: DataType=' "&_DataType1" '" )'; 
  			put '               ),';
  			put '               Dispatch( {}, "", AxisBox( 2 ), Add Axis Label( "Survival Rate" ) ),';
  			put '               Dispatch( {}, "Overlay Plot", FrameBox, Frame Size( 400, 250 ) ),';
			put '               Dispatch( {}, "", LegendBox, {Set Wrap( 5 )} )';
  			put '            )';
  			put '         ),'; *end of overlay plot;
			*****  one plot of distribution of reference survival estimates   *****;
			put "         If(IsEmpty(Here:RefSur_data),";
			put "            Here:RefSur_data=open(""&ClientOutPath.%scan(&SPM_OutData3,2,.).sas7bdat"",invisible);"; 
			put "         );";

			%if %length(&TestInData) %then %do;
				put "         If(IsEmpty(Here:RefSur_data_1),";
				put '            Here:RefSur_data_1 = Here:RefSur_data<<Select Where(:DataType=="Test")';
  				put '               <<Subset(output table name("ReferenceSurvival_Test"), Selected rows,invisible);'; 
  				put '            Here:RefSur_data_1<<Minimize Window;';	
				put '         );';
				put '         current data table(Here:RefSur_data_1);';
			%end;
			%else %do;
				put '         current data table(Here:RefSur_data);';
			%end;
			%if %length(&ColorVar) %then %do;
				* set color state for rows;
				put '         for each row(';
 				put '            if(' ":&ColorVar == " '"' "&_vColorVar1" '",';
  				put '               ColorOf( row state() ) = ColorList[ 1 ];';
				%do i = 2 %to &_nColor;
					put '            ,';
  					put "               :&ColorVar == " '"' "&&_vColorVar&i" '",'; 
  					put "               ColorOf( row state() ) = ColorList[ &&_iColorVar&i ];"; 
				%end;
				put '            )';
  				put '         );';
				* create One-Way plot for grouped reference survival estimates;
				put '         vListBox(';
  				put '            Here:RefSur1 = Oneway(';
  				put '               Y( :Reference Survival Estimate ),';
  				put "               X( :&ColorVar ),";
  				put '               With Best( 1 ),';
  				put '               Box Plots( 1 ),';
  				put '               Mean Diamonds( 1 ),';
  				put '               Comparison Circles( 1 ),';
  				put '               Points Jittered( 1 ),';
  				put '               SendToReport(';
  				put '                  Dispatch(';
  				put '                     {},';
  				put '                     "Oneway Analysis of Reference Survival Estimate By ' "&ColorVar" '",';
  				put '                     OutlineBox,';
  				put "                     Set Title( ""Reference Survival Estimate at &DepVar=&RefTime By &ColorVar.: DataType=&_DataType1"" )"; 
  				put '                  ),';
  				put '                  Dispatch( {}, "Means Comparisons", OutlineBox, Close( 1 ) )';
  				put '               )';
  				put '            );';
  				put '         ),';
			%end;
			%else %do; 
				* create Distribution for referal survival estimates;
  				put '         Distribution( Continuous Distribution( Column( :Reference Survival Estimate ) ) );';
			%end;

			%if ^%length(&TestInData) %then %do;
				put '   );';
			%end;
			%else %do; 
				put '   ),';
				* plot another pair of plots (for Training data set);
				put '     hListBox(';
				*****  one plot for suvival function   ********;
				put "      If(IsEmpty(Here:Sur_data_2),";
				put "         Here:Sur_data_2=open(""&ClientOutPath.%scan(&SPM_OutData,2,.).sas7bdat"",invisible);";
				put "      );";
      				put '      current data table(Sur_data_2);';
				put '      Here:SurFunc2 = Overlay Plot(';
				put "         X( :&DepVar ),";
				put '         Y( ';
				%do i = 1 %to %eval(&_nTrain-1);
					put '            :Name("' "&&_IDVar_tr&i" '"),';
				%end;
				put '            :Name("' "&&_IDVar_tr%trim(%left(&_nTrain))" '")';
				put '         ),';
				put '         Overlay Axis << {';
				put '            Min( -0.02 ), Max( 1.02 ), Minor Ticks( 1 ),';
				put '            Add Ref Line( 0.25, Dotted, "Black" ),';	
				put '            Add Ref Line( 0.5, Dotted, "Black" ),';
				put '            Add Ref Line( 0.75, Dotted, "Black" )';
				put '         },';
				put '         Separate Axes( 1 ),';
				put '         X Axis << {';
				put '            Min( -1 ),';
				put "            Add Ref Line( &RefTime, Dotted, ""Black"" )";
				put '         },';
				put '         Step(1),';
				put '         Show Points(0),';
				%if %length(&ColorVar) %then %do;
					* handle grouped colors here!!!; * may need to refine this part for test dataset!!!;
					%do i = 1 %to &_nTrain;
						put '         :Name("' "&&_IDVar_tr&i" '")' "( Connect Color( ColorList[ %trim(%left(&&_cIDVar_tr&i)) ] ) ),";
					%end;
				%end;
				put '         SendToReport(';
				put '            Dispatch(';
				put '               {},';
				put '               "Overlay Plot",';
				put '               OutlineBox,';
				put '               Set Title( "Estimated Survival Function: DataType=Training" )'; 
				put '            ),';
				put '            Dispatch( {}, "", AxisBox( 2 ), Add Axis Label( "Survival Rate" ) ),';
				put '            Dispatch( {}, "Overlay Plot", FrameBox, Frame Size( 400, 250 ) ),';
				put '            Dispatch( {}, "", LegendBox, {Set Wrap( 5 )} )';
				put '         )';
				put '      ),';

				*****  one plot of distribution of reference survival estimates   *****;
				put "      If(IsEmpty(Here:RefSur_data_2),";
				put '         Here:RefSur_data_2 = Here:RefSur_data<<Select Where(:DataType=="Training")';
				put '            <<Subset(output table name("ReferenceSurvival_Training"), Selected rows,invisible);';
				put '         Here:RefSur_data_2<<Minimize Window;';	
				put '         Here:RefSur_data<<clear select;';
				put '      );'; 
      				put '      current data table(Here:RefSur_data_2);';
				%if %length(&ColorVar) %then %do;
					* set color state for rows;
					put '      for each row(';
					put '         if(' ":&ColorVar == " '"' "&_vColorVar1" '",';
					put '            ColorOf( row state() ) = ColorList[ 1 ];';
					%do i = 2 %to &_nColor;
						put '         ,';
						put "            :&ColorVar == " '"' "&&_vColorVar&i" '",'; 
						put "            ColorOf( row state() ) = ColorList[ &&_iColorVar&i ];";        
					%end;
					put '         )';
					put '      );';
					* create One-Way plot for grouped median survival estimates;
					put '      Here:RefSur2 = Oneway(';
					put '         Y( :Reference Survival Estimate ),';
					put "         X( :&ColorVar ),";
					put '         With Best( 1 ),';
					put '         Box Plots( 1 ),';
					put '         Mean Diamonds( 1 ),';
					put '         Comparison Circles( 1 ),';
					put '         Points Jittered( 1 ),';
					put '         SendToReport(';
					put '            Dispatch(';
					put '               {},';
					put '               "Oneway Analysis of Reference Survival Estimate By ' "&ColorVar" '",';
					put '               OutlineBox,';
					put "               Set Title( ""Reference Survival Estimate at &DepVar=&RefTime By &ColorVar.: DataType=Training"" )"; 
					put '            ),';
					put '            Dispatch( {}, "Means Comparisons", OutlineBox, Close( 1 ) )';
					put '         )';
					put '      );';
				%end;
				%else %do; 
					* create Distribution for median survival estimates;
					put '      Distribution( Continuous Distribution( Column( :Reference Survival Estimate ) ) );';
				%end;	
				put '      )';

			%end;
			put '   );'; * end of vListBox;
			put ');';
			
			%if %length(&TestInData) %then %do;
				put "InsertInto(DataList,{{"":Here:vars_data"",""Here:Sur_data_1"",""Here:Sur_data_2"",""Here:RefSur_data"",";
				put " ""Here:RefSur_data_1"",""Here:RefSur_data_2""}});";
			%end;
			%else %do;
				put "InsertInto(DataList,{{"":Here:vars_data"",""Here:Sur_data_1"",""Here:RefSur_data""}});";
			%end;
			put "InsertInto(TabButtonList, Expr(locale:get(""Results_Text"")));";
			put "InsertInto(TabCodeList,expr(Here:Survival()));";
			put "InsertInto(TabAfterCodeList, "" "");";
			put "InsertInto(TabOpenList,1);";	
			%if ((&ProcessName = PartitionTrees) or (&ProcessName = VirtualTwins)) and %upcase(&TreeMethod)=TREE and ^&CVFlag %then %do;
				data _null_;
					file "&&&_JSLFile" mod;
					put "include(jslpath||""\Partition Trees Viewer Selection.jsl"");";
				run;
			%end;
			*/

		%End_JSL:;
		%if ((&_dtype_=C) or (&NumDepAsClass=Yes)) and (&nclasses = 2) %then %do;
			%let depds = &&&_OutDataName...sas7bdat,&&&_OutDataVarsName...sas7bdat,&_outrocname..sas7bdat;
		%end;
		%else %do;
			%let depds = &&&_OutDataName...sas7bdat,&&&_OutDataVarsName...sas7bdat;
		%end;

		%if &ProcessName = SurvivalPredictiveModeling %then %do;
			%let depds = &&&_OutDataName...sas7bdat,%scan(&SPM_OutData3,2,.).sas7bdat,%scan(&SPM_OutData,2,.).sas7bdat,&_DataTable1,&&&_OutDataVarsName...sas7bdat;
		%end;
		
		%if (&ProcessName = LogisticRegression) and (&VarSelect = Penalized) and ^&CVFlag %then %do;		
			%let depds = &depds, &pdat._fit.sas7bdat;
		%end;
		
		%if ((&ProcessName = PartitionTrees) or (&ProcessName = VirtualTwins)) and %upcase(&TreeMethod)=TREE and ^&CVFlag %then %do;
			%let depds = &depds, &model_output..sas7bdat, &importance_output..sas7bdat, &nodestats_output..sas7bdat, &_psoutname._ps.sas7bdat,
				&rules_output..sas7bdat, &cmprules_output..sas7bdat, &sequence_output..sas7bdat, &score_train_output..sas7bdat,
				&score_test_output..sas7bdat;
		%end;
		
		%if (&ProcessName = GenomicBLUP and &Kernel = Dot and ^&CVFlag) %then %do;
			%let depds = &depds, &cpout..sas7bdat, &gbout..sas7bdat;
			%if %length(&AnnoInData) and &nimk < &nmrkin %then %do;
				%let depds = &depds, &AnnoOutDataName..sas7bdat;
			%end;
			
			%PackageInsertDataset(Variance components data from &ProcessName, outlib, %unquote(&cpout));
			%PackageInsertDataset(BLUP data from &ProcessName, outlib, %unquote(&gbout));
		%end;
		
		%if ^&CVFlag %then %do;
			%PackageInsertFile(jsl code from &ProcessName, &OutPath, &&&_JSLFileName, "&depds", text/plain, yes);
			%if (&ProcessName = GLMSelect) %then %do;
				%PackageInsertFile(Scoring code from &ProcessName, &OutPath, &OutName._glm_score.sas, "", text/plain, yes, no);
				%PackageInsertFile(Formula code from &ProcessName, &OutPath, &OutName._glm_formula.jsl, "", text/plain, yes, no);
			%end;
			%else %if (&ProcessName = LogisticRegression and &VarSelect ^= Penalized) %then %do;
				%PackageInsertFile(Scoring code from &ProcessName, &OutPath, &OutName._lgr_score.sas, "", text/plain, yes, no);
				%PackageInsertFile(Formula code from &ProcessName, &OutPath, &OutName._lgr_formula.jsl, "", text/plain, yes, no);
			%end;
			%else %if (&ProcessName = PartialLeastSquares) %then %do;
				%PackageInsertFile(Scoring code from &ProcessName, &OutPath, &OutName._pls_score.sas, "", text/plain, yes, no);
				%PackageInsertFile(Formula code from &ProcessName, &OutPath, &OutName._pls_formula.jsl, "", text/plain, yes, no);
			%end;
			%else %if (&ProcessName = QuantSelect) %then %do;
				%PackageInsertFile(Scoring code from &ProcessName, &OutPath, &OutName._qs_score.sas, "", text/plain, yes, no);
				%PackageInsertFile(Formula code from &ProcessName, &OutPath, &OutName._qs_formula.jsl, "", text/plain, yes, no);
			%end;
			%else %if (&ProcessName = RidgeRegression) and &Kernel = Dot %then %do;
				%PackageInsertFile(Scoring code from &ProcessName, &OutPath, &OutName._rr_score.sas, "", text/plain, yes, no);
				%PackageInsertFile(Formula code from &ProcessName, &OutPath, &OutName._rr_formula.jsl, "", text/plain, yes, no);
			%end;
		%end;
	%end;

	%if %index(&VarSelect,Genetic) and ^&CVFlag %then %do;
		%let vipd = ;
	   	%AddSuffixToDataName(_vip,&OutName,vipd);
		proc sort data=vipd;
			by variable;
		run;
		proc means data=vipd noprint;
			by variable;
			var importance;
			output out=vipdm mean=Importance;
		run;
		data vipdm;
			set vipdm;
			rename variable = _var;
			drop _type_ _freq_;
		run;
		data vipdm;
			set vipdm;
			rename _var = Variable;
		run;
		proc sort data=vipdm;
			by descending importance;
		run;
		data OutLib.&vipd;
			set vipdm;
		run;
		%let bpd = ;
	   	%AddSuffixToDataName(_dbp,&OutName,bpd);
		data OutLib.&bpd;
			set bpd;
		run;
		/*
		%PackageInsertDataset(Variable importance data from &ProcessName, outlib, &vipd);
		%PackageInsertDataset(Best predictor data from &ProcessName, outlib, &bpd);
		*/
	%end;
%end;

%if (&GenerateHTML=Yes) and ^&CVFlag %then %do;
	ods html close;
	ods graphics off;

	%PackageInsertHtml(html from &ProcessName, &OutPath, &ProcessName..body.html,
		&ProcessName..frame.html, &ProcessName..contents.html);

	%if (&ProcessName = GLMSelect) | (&ProcessName = %quote(QuantSelect)) %then %do;
		* find the last created png file;
		%FileFilter(&OutPath,PNGFile,_Prefix="&&&_OutDataName",_Ext="png");
		%NObsVars(PNGFile);
		%if &nobs >= 5 %then %do;
			data PNGFile;
				set PNGFile;
				len1 = %length(&&&_OutDataName);
				len2 = length(filename);
				len3 = len2-len1-4;
				if len3 > 0 then num = substr(filename,len1+1,len2-len1-4)+0;
				else num = 0;
			run;
			/*
			data OutLib.pngfile;
				set pngfile;
			run;
			*/
			proc sort data=PNGFile;
				by num;
			run;
			data _null_;
				set PNGFile end=_e_;
				if _e_ then call symput("nPNG",num);
			run;
			%let PNG1 = %eval(&nPNG-4);
			%let PNG2 = %eval(&nPNG-2);
			%let PNG3 = &nPNG;		
			%PackageInsertFile(graph1 from &ProcessName, &OutPath, &&&_OutDataName..&PNG1..png, ,image/png, yes,no);
			%PackageInsertFile(graph2 from &ProcessName, &OutPath, &&&_OutDataName..&PNG2..png, ,image/png, yes,no);
			%PackageInsertFile(graph3 from &ProcessName, &OutPath, &&&_OutDataName..&PNG3..png, ,image/png, yes,no);
		%end;
	%end;
%end;

/*
%if ((&ProcessName = PartitionTrees) or (&ProcessName = VirtualTwins)) and %upcase(&TreeMethod)=TREE and ^&CVFlag %then %do;
	%global jsl_after_tabbedreport;
	%let jsl_after_tabbedreport = %quote(&CLIENT_LS_HOME.JSLFiles\Partition Trees Viewer Selection.jsl);
%end;
*/

%*ClockStop(Performance);

%exit:;
%if %symexist(_dsiov_) %then %do;
	%let _rc_ = %sysfunc(close(&_dsiov_));
%end;

%if %symexist(_dsimk_) %then %do;
	%let _rc_ = %sysfunc(close(&_dsimk_));
%end;

%if %symexist(_dsmap_) %then %do;
	%let _rc_ = %sysfunc(close(&_dsmap_));
%end;

%if %symexist(OrigProcessName) %then %let ProcessName = %quote(&OrigProcessName);
%ExitSDSProcess(&OutPath, &ProcessName, &syserr, &exiterror, &clockrunning);

%*ClockStop(PredictiveAnalysis);

quit;

%mend PredictiveAnalysis;


* create macro variables for scoring code assuming a linear model;
* must create pe and &ts2 data sets before calling this;
%macro ScoringCodeMacroVars;
	
	/*S1470994 and S1471114 - Begin: Fixing issue for the jsl formula and score files when character 
	class variables have missing values and the missing values have an estimated effect obtained in 
	the PROC GLMSELECT.
	The previous fix had SQL code, which surprisingly causes error of variable IntFlag not found.
	*/
	
	/*data OutLib.pe; set pe; run;*/
		
	data pe;
		set pe;
		_ord_ = _n_;
	run;
		
	data par2;
		length parameter $150;
		set pe(rename=(parameter=partmp)) end=_eof_;
		retain _c_;
		if _n_ = 1 then _c_=0;
		if scan(partmp,2," ") ^= "" then do;
			parameter= scan(partmp,1," ");
			_c_=_c_ + 1;
			output;
		end;
		if _eof_ then call symputx("nobsin", _c_);
		drop _ord_ _c_ partmp;
	run;
	%put nobsin= &nobsin;
	
	%if &nobsin %then %do;
	
		proc sort data= par2 out=par2_nodup nodupkey;
			by parameter;
		run;
		
		data _null_;
			set par2_nodup end=_eof_;
			if _eof_ then call symputx("nobsout", _n_);
		run;
		%put nobsout= &nobsout;
		
		%if &nobsout < &nobsin %then %do;	
	
			data pe_tmp;
				if 0 then set par2_nodup;
				if _n_ = 1 then do;
					declare hash h(dataset: "work.par2_nodup", hashexp: 6);
					h.defineKey('parameter');
					h.defineDone();
				end;
				set pe;
				if (h.find()=0) then _flag_= 1;
				else _flag_= 0;
			run;
	
			data pe;
				length Parameter $150;
				set pe_tmp(rename=(parameter=par_orig));
				if _flag_ then do;
					parameter= trim(left(par_orig))||" "||"_miss_";
				end;
				else do;
					parameter= par_orig;
				end;
				drop _ord_ _flag_ par_orig ;
			run;
			
			/*data OutLib.pe_; set pe; run;*/
		%end;
	%end;
	/*S1470994 and S1471114 - End*/
	
	/*data OutLib.pe12; set pe; run;*/
	
	*data OutLib.pe1;
	data pe1;
		set pe end=_e_;
		retain _IntFlag_ 0;
		_ds_ = open("&ts2");
		if _n_=1 then do;
			if parameter="Intercept" then do;
				call symput("IntFlag",1);
				call symput("Intercept",estimate);
				_IntFlag_ = 1;
			end;
			else do;
				call symput("IntFlag",0);
			end;
		end;
		if _IntFlag_ then _i_ = _n_ - 1;
		else _i_ = _n_;
		_name_ = "ParVal_"||trim(left(_i_));
		call symput(_name_,trim(estimate));
		_name_ = "ParName_"||trim(left(_i_));
		_cflag_ = "ClassFlag_"||trim(left(_i_));
		if scan(parameter,2," ") ^= "" then do;
			* selected variable is from a class variable;
			call symputx(_cflag_,1);
			call symputx(_name_,scan(parameter,1," "));
			_vlevel_ = "Level_"||trim(left(_i_));
			_vlen_ = length(scan(parameter,1," "));
			call symputx(_vlevel_,substr(parameter,_vlen_+1));
			_vnum_ = varnum(_ds_,scan(parameter,1," "));
			_vtype_ = "ParType_"||trim(left(_i_));
			call symputx(_vtype_,vartype(_ds_,_vnum_));
		end;
		else do;
			call symputx(_cflag_,0);
			call symputx(_name_,trim(parameter));
		end;
		if _e_ then call symput("nPar",_i_);
		_rc_ = close(_ds_);
	run;
	%put IntFlag= &IntFlag;
	
	/*data OutLib.pe13; set pe1; run;*/
	
	/*
	%do i=1 %to &nPar;
		%put i=&i ParName_&i=&&ParName_&i ClassFlag_&i=&&ClassFlag_&i ParVal_&i=&&ParVal_&i;
		%if &&ClassFlag_&i %then %do;
			%put Level_&i=&&Level_&i ParType_&i=&&ParType_&i;
		%end;
	%end;
	*/
%mend;

%macro ScoringCodeSAS;
	%if &IntFlag %then %let Formula_Intercept = &Intercept;
	%else %let Formula_Intercept = 0;
	put "&TruthVar._Score = &Formula_Intercept";
	%do i = 1 %to &nPar;
		%*put ClassFlag_&i= &&ClassFlag_&i;
		%if (&&ClassFlag_&i) %then %do;
			/*S1399789: When Level_i= Not there will be a SAS error because Not is a SAS reserved value. 
			The value of Level_i must be quoted to avoid such error, which can be done with the nrbquote function*/
			/*S1470994: Fixing issue for the jsl formula and score files when character class variables have 
	 		missing values and the missing values have an estimated effect obtained in the PROC GLMSELECT*/
			%*put Level_&i= &&Level_&i;
			%if %nrbquote(&&Level_&i) = %nrbquote(.) or ^%length(%trim(%nrbquote(&&Level_&i))) or %nrbquote(&&Level_&i) = %nrbquote(_miss_) %then %do;
				put "+ &&ParVal_&i * missing(&&ParName_&i)";			
			%end;		
			%else %if (&&ParType_&i=N) %then %do;
				put "+ &&ParVal_&i * (&&ParName_&i=&&Level_&i)";
			%end;
			%else %do;
				put "+ &&ParVal_&i * (&&ParName_&i=""&&Level_&i"")";
			%end;
		%end;
		%else %do;
			put "+ &&ParVal_&i * &&ParName_&i";
		%end;
	%end;
	put ";";
	%if &ProcessName = LogisticRegression %then %do;
		%if &LinkFunction = CLOGLOG %then %do;
			put "&TruthVar._Score = 1 - Exp(-Exp(&TruthVar._Score));";		
		%end;
		%else %if &LinkFunction = PROBIT %then %do;
			put "&TruthVar._Score = probnorm(&TruthVar._Score);";
		%end;
		%else %do;
			put "call logistic(&TruthVar._Score);";
		%end;
	%end;
	%if &ProcessName = GenomicBayesianRegression and &TraitType = Binary %then %do;
		put "&TruthVar._Score = 1 - probnorm(&TruthVar._Score);";
	%end;	
%mend;

%macro ScoringCodeJSL;
	put "CurrentDataTable() << New Column(""&TruthVar Prediction Formula"", << Formula(";
	%if &ProcessName = LogisticRegression %then %do;
		%if &LinkFunction = CLOGLOG %then %do;
			put "1 - Exp(-Exp(";		
		%end;
		%else %if &LinkFunction = PROBIT %then %do;
			put "Normal Distribution(";
		%end;
		%else %do;
			put "Logist(";
		%end;
	%end;	
	%if &IntFlag %then %let Formula_Intercept = &Intercept;
	%else %let Formula_Intercept = 0;
	put "&Formula_Intercept";
	%do i = 1 %to &nPar;
		%*put ClassFlag_&i= &&ClassFlag_&i;
		%if (&&ClassFlag_&i) %then %do;
			/*S1399789: When Level_i= Not there will be a SAS error because Not is a SAS reserved value. 
			The value of Level_i must be quoted to avoid such error, which can be done with the nrbquote function*/
			/*S1470994: Fixing issue for the jsl formula and score files when character class variables have 
	 		missing values and the missing values have an estimated effect obtained in the PROC GLMSELECT*/			
			%*put Level_&i= &&Level_&i;
			%if %nrbquote(&&Level_&i) = %nrbquote(.) or ^%length(%trim(%nrbquote(&&Level_&i))) or %nrbquote(&&Level_&i) = %nrbquote(_miss_) %then %do;
				put "+ &&ParVal_&i * IsMissing(:&&ParName_&i)";			
			%end;
			%else %if (&&ParType_&i=N) %then %do;
				put "+ &&ParVal_&i * (!IsMissing(:&&ParName_&i) %str(&&) :&&ParName_&i==&&Level_&i)";
			%end;
			%else %do;
				put "+ &&ParVal_&i * (!IsMissing(:&&ParName_&i) %str(&&):&&ParName_&i==""&&Level_&i"")";
			%end;
		%end;
		%else %do;
			put "+ &&ParVal_&i * :&&ParName_&i";
		%end;
	%end;
	%if &ProcessName = LogisticRegression %then %do;
		%if &LinkFunction = CLOGLOG %then %do;
			put "))";		
		%end;
		%else %do;
			put ")";
		%end;
	%end;	
	put "));";
%mend;


* Prediction Macro for Genetic Algorithm;
%macro SelectVarsGA_Predict(TrainingSet=);

/* run predictor on by-grouped data set ByTrainingSet */
ods exclude all;
ods noresults;

%if (&ProcessName = LogisticRegression) %then %do;
	proc logistic data=ByTrainingSet &LogisticOptions namelen=128;
		by byval;
		model &Class &LogisticResponseOptions = var1-var&NumVar / &LogisticModelOptions;
		%if %length(&WeightVar) %then %do;
			weight &WeightVar;
		%end;
		&LogisticStatements
		output out=pp predprobs=i;
		ods output /* type3=t3 */ responseprofile=rp;
	run;

	%if &syserr > 4 %then %let exiterror = 1;

	/*---use average of chi-square p-values as objective---*/
	/*
	proc means data=t3;
		by byval;
		var ProbChiSq;
		output out=bo mean=objective;
	run;
	*/

	data levels1;
		set rp;
		if byval > 1 then stop;
	run;

	/*
	data _null_;
		set levels1 end=_last;
		nc = trim(left(_n_));
		call symput("CLev"||nc,trim(left(Outcome)));
		call symput("CName"||nc,"IP_"||trim(left(OutCome)));
		if (_last) then call symput("nlev",nc);
	run;
	*/
	
	data _namedat;
		set levels1 end=_last;
		cn = "cvalue" || trim(left(_n_));
		call symput(cn,trim(left(Outcome)));
		if (_last) then call symput('ncl',trim(left(_n_)));
	run;

	/*---use ase as objective---*/
	%put ncl = &ncl;
	%global cvartype;
	%VarAttributes(pp,&Class,_Len=,_Type=cvartype);
	
	data pp;
		set pp;
		array pp ip_:;
		_cidx = 0;
		%do c=1 %to &ncl;
			%if (&cvartype=N) %then %do;
				if trim(left(&Class)) = &&cvalue&c then _cidx = &c; 
			%end;
			%else %do;
				if trim(left(&Class)) = "&&cvalue&c" then _cidx = &c; 						
			%end;
		%end;
		if _cidx = 0 then SqErr = 1;
		else do;
			SqErr = 0;
			do i=1 to &ncl;
				if i=_cidx then ei = (1 - pp[i])**2;
				else ei = pp[i]**2;
				SqErr = SqErr + ei;
			end;
		end;		
	run;

	/*---use simple error rate as objective---*/
	/*
	data pp;
		set pp;
		if &DepVar = _INTO_ then SqErr = 0;
		else SqErr = 1;
	run;
	*/

	proc means data=pp;
		by byval;
		var SqErr;
		output out=bo mean=objective;
	run;
%end;
%else %if (&ProcessName = DiscriminantAnalysis) or (&ProcessName = KNearestNeighbors) %then %do;	
	%if (&LOOCV = Yes) %then %do;
		proc discrim data=ByTrainingSet &DiscrimOptions crossvalidate;
			by byval;
			class &Class;
			priors &Priors;
			ods output ErrorCrossVal=ErrorRates;
		run;
		
		%if &syserr > 4 %then %let exiterror = 1;

		data bo;
			/* separate out by-values, total error rate */
			set ErrorRates (where = (type = 'Rate'));
			objective = total;
		run;
	%end;
	%else %do;
		* disable multivariate F for now;
		%let MultiFObj = 0;
		
		proc discrim data=ByTrainingSet out=pp &DiscrimOptions 
			%if (&MultiFObj) %then %do; manova %end; ;
			by byval;
			class &Class;
			priors &Priors;
			ods output levels=levels 
				%if (&MultiFObj) %then %do; multstat=manova %end; ;
		run;
		
		%if &syserr > 4 %then %let exiterror = 1;

		data levels1;
			set levels;
			if byval > 1 then stop;
		run;
		
		data _namedat;
			set levels1 end=_last;
			nc = trim(left(_n_));
			call symput("cvalue"||nc,&Class);
			call symput("cname"||nc,VarName);
			if (_last) then call symput("ncl",nc);
		run;
		
		%put ncl = &ncl;
		%global cvartype;
		%VarAttributes(pp,&Class,_Len=,_Type=cvartype);
		
		%if (&MultiFObj) %then %do;
			/*---use multivariate F-test p-value as objective---*/
			data bo;
				set manova;
				if statistic="Hotelling-Lawley Trace";
				objective = ProbF;
				keep byval objective;
			run;
		%end;
		%else %do;
			/*---use ase as objective---*/
			data pp;
				set pp;
				array pp{&ncl} %do c=1 %to &ncl; &&cname&c %end; ;
				_cidx = 0;
				%do c=1 %to &ncl;
					%if (&cvartype=N) %then %do;
						if trim(left(&Class)) = &&cvalue&c then _cidx = &c; 
					%end;
					%else %do;
						if trim(left(&Class)) = "&&cvalue&c" then _cidx = &c; 						
					%end;
				%end;
				if _cidx = 0 then SqErr = 1;
				else do;
					SqErr = 0;
					do i=1 to &ncl;
						if i=_cidx then ei = (1 - pp[i])**2;
						else ei = pp[i]**2;
						SqErr = SqErr + ei;
					end;
				end;		
			run;
			
			/*
			data pp;
				set pp;
				array pprob{&nlev} %do _lev=1 %to &nlev; &&CName&_lev %end; ;
				cindex = 0;
				%do _lev=1 %to &nlev;
					 if _INTO_ = "&&CLev&_lev" then cindex = &_lev;
				%end;
				if (cindex = 0) then SqErr = 1;
				else SqErr = 2*(1 - pprob[cindex])**2;
			run;
			*/
			
			/*---use simple error rate as objective---*/
			/*
			data pp;
				set pp;
				if &DepVar = _INTO_ then SqErr = 0;
				else SqErr = 1;
			run;
			*/
				
			proc means data=pp;
				by byval;
				var SqErr;
				output out=bo mean=objective;
			run;
		%end;
	%end;
%end;
%else %if (&ProcessName = GLMSelect) %then %do;

	proc glmselect data=ByTrainingSet
		%if %length(&Seed) %then %do;
			seed=&Seed
		%end;
		&GLMSelectOptions namelen=128;
		by byval;
		model &DepVar = var1-var&NumVar
			/ selection=&SelectMethod 
			%if &SelectMethod ^= None %then %do;
				(
					stop=&StopCriterion %if &MaxStep > 0 %then %do; maxstep=&MaxStep %end;
					%if (&SelectMethod^=LAR) and (&SelectMethod^=LASSO) and (&SelectMethod^=ElasticNet) %then %do;
						select=&StopCriterion
					%end;
					%else %do;
						%if (&Adaptive = Yes) %then %do;
							ADAPTIVE
						%end;
						%if (&StopCriterion=PRESS) %then %do;
							LSCOEFFS
						%end;
					%end;
					%if (&StopCriterion=SL) %then %do;
						%if (&StopCriterion=Forward) or (&SelectMethod=Stepwise) %then %do;
							slentry=&SLEntry
						%end;
						%if (&SelectMethod=Backward) or (&SelectMethod=Stepwise) %then %do;
							slstay=&SLStay
						%end;
					%end;
				) 
			%end;
			%if (&StopCriterion=CV) %then %do;
				%if (&CVMethod = Index) %then %do;
					cvmethod=&CVPMethod ( &CVIndexVar )
				%end;
				%else %do;
					cvmethod=&CVPMethod ( &CVKFold )
				%end;
			%end;
			&GLMSelectModelOptions
		;
		%if %length(&WeightVar) %then %do;
			weight &WeightVar;
		%end;
		/*
		%if %length(&FreqVar) %then %do;
			freq &FreqVar;
		%end;
		*/
		output out=pp pred;
	run;
			
	/*---use ase as objective---*/
	data pp;
		set pp;
		SqErr = (&DepVar - p_&DepVarTrunc30)**2;
	run;

	proc means data=pp;
		by byval;
		var SqErr;
		output out=bo mean=objective;
	run;
%end;

%else %if (&ProcessName = RadialBasisMachine) %then %do;

	proc glimmix data=ByTrainingSet &ProcOption;
		by byval;
		%if %length(&GlimmixClassVars) %then %do;
			class &GlimmixClassVars;
		%end;
		model &DepVar.&DepOption = &FixedEffects &ModOption;
		random var1-var&NumVar
			/ type=rsmooth knotmethod=data(ByTrainingSet) 
			%if (&TraitType=Nominal) or (&TraitType=Ordinal) %then %do;
				group=&DepVar
			%end;	
		;
		%if %length(&WeightVar) %then %do;
			weight &WeightVar;
		%end;
		&ProcGlimmixStatements;
		output out=pp pred(blup ilink)=pred;
	run;

	/*---use ase as objective---*/
	data pp;
		set pp;
		%if (&TraitType=Binary) %then %do;
			p_&DepVarTrunc._0 = 1 - pred;
			if p_&DepVarTrunc._0 < 0 then p_&DepVarTrunc._0 = 0;
			else if p_&DepVarTrunc._0 > 1 then p_&DepVarTrunc._0 = 1;
			p_&DepVarTrunc._1 = 1 - p_&DepVarTrunc._0;
			_INTO_ = (p_&DepVarTrunc._1 > 0.5);
			if (&DepVar=1) then actualp = p_&DepVarTrunc._1;
			else actualp = p_&DepVarTrunc._0;
			SqErr = 2*(1 - actualp)**2;
		%end;
		%else %do;
			SqErr = (&DepVar - pred)**2;
		%end;
	run;

	proc means data=pp;
		by byval;
		var SqErr;
		output out=bo mean=objective;
	run;
%end;

%else %if (&ProcessName = PartialLeastSquares) %then %do;

	* standardize the dependent variable before calling proc pls;
	proc stdize data=ByTrainingSet method=std out=ByTrainingSet;
		by byval;
		var &DepVar;
	run;

	proc pls data=ByTrainingSet nfac=&NumberOfComponents &ProcPLSOptions nocenter noscale details 
			missing=avg /* missing=em */;
		by byval;
		model &DepVar = var1-var&NumVar;
		output out=pp predicted=pred;
	run;
	
	/*---use ase as objective---*/
	data pp;
		set pp;
		%if (&TraitType=Binary) %then %do;
			p_&DepVarTrunc._0 = 1 - pred;
			if p_&DepVarTrunc._0 < 0 then p_&DepVarTrunc._0 = 0;
			else if p_&DepVarTrunc._0 > 1 then p_&DepVarTrunc._0 = 1;
			p_&DepVarTrunc._1 = 1 - p_&DepVarTrunc._0;
			_INTO_ = (p_&DepVarTrunc._1 > 0.5);
			if (&DepVar=1) then actualp = p_&DepVarTrunc._1;
			else actualp = p_&DepVarTrunc._0;
			SqErr = 2*(1 - actualp)**2;
		%end;
		%else %do;
			SqErr = (&DepVar - pred)**2;
		%end;
	run;

	proc means data=pp;
		by byval;
		var SqErr;
		output out=bo mean=objective;
	run;
	
	/*
	data OutLib.pp;
		set pp;
	run;
	
	data OutLib.bo;
		set bo;
	run;
	*/
	
	/*
	* use negative correlation as objective to minimize;
	* make sure to set early stopping threshold to -1 on the dialog;
	proc corr data=pp noprint outp=cp;
		by byval;
		var &DepVar;
		with pred;
	run;
	
	data bo;
		set cp;
		where _type_ = "CORR";
		objective = -&DepVar;
		keep byval objective;
		rename &DepVar = objective;
	run;
	*/
	
%end;
%else %do;
	%put ERROR: Genetic Algorithm not available for &ProcessName.;
	%let exiterror = 1;
%end;

data ByObjective;
	set bo;
	%if (&exiterror) %then %do;
		objective = .;
	%end;
	keep byval objective;
run;

proc sort data=bo;
	by objective;
run;	

%if ^&CVFlag %then %do;
	ods exclude none;
	ods results;
%end;

/*
title2 "Genetic Algorithm Variable Selection for &ProcessName";
title3 "Generation &iteration Top Ten";
proc print data=bo(obs=10);
run;

title2;
title3;
proc iml;
	print ' ';
quit;
*/

%mend SelectVarsGA_Predict;



* check variable type;
%macro CheckVarType(Data,Vars,Type,MisList);
proc contents data=&Data(Keep=&Vars) out=tmp noprint;
run; 

%if &type = 1 %then %let MisType = 2;
%else MisType = 1;
%let nMis = 0;
data tmp;
   set tmp;
	if Type = &MisType;
	retain c 0;
	c = c + 1;
run;

* just save first 100 to avoid huge list;
data tmp;
	set tmp(obs=100);
run;

data _null_;
   set tmp end=_e_;
   if _e_ then call symput('nMis',c);
run;

%if &nMis > 0 %then %do;
   data _Null_;
	   set tmp;
		MisVars = 'Name'||trim(left(_n_));
      call symput(MisVars,Name);
	run;
	%do i = 1 %to &nMis;
	   %let &MisList = &&&MisList &&Name&i;
	   %put %length(&&&MisList);
	   %if %length(&&&MisList) >= 65534 %then %let i = %eval(&nMis+1);
	%end;
%end;
%mend CheckVarType;


%macro UpdateSelVars;
	%let nsv = 0;
	data _null_;
		set ts2vars end=_last;
		call symput("selvar"||trim(left(_n_)),trim(left(name)));
		if _last then call symput("nsv",trim(left(_n_)));
	run;
	%let SelVars = ;
	%do i=1 %to &nsv;
		%let SelVars = &SelVars &&selvar&i;
	%end;
	%put Updated SelVars = &SelVars;
%mend UpdateSelVars;

%macro UnsplitSelVars;
	%let _dsid_ = %sysfunc(open(&ts2));
	%let j=1;
	%let svnew = ;
	%do %while(%length(%scan(&SelVars,&j,' ')));
		%let evj = %scan(&SelVars,&j,' ');
		%let firstone = 1;
		%let i = 1;
		* handle interaction effects;
		%do %while(%length(%scan(&evj,&i,'*')));
			%let svi = %scan(&evj,&i,'*');
			%let lensvi = %length(&svi);
			%put svi=&svi lensvi=&lensvi;
			%if %substr(&svi,&lensvi,1) = . %then %do;
				%let svi = %substr(&svi,1,%eval(&lensvi-1));
			%end;
			%SASName(&svi);
			%if %quote(%upcase(&svi)) = %quote(%upcase(&sasname)) %then %do;
				%let vnum = %upcase(%sysfunc(varnum(&_dsid_,&svi)));
			%end;
			%else %let vnum = 0;
			%put sasname=&sasname vnum=&vnum;
			* if variable not found delete part of name after last underscore;
			%if ^&vnum %then %do;
				%let another=1;
				%let svij = &svi;
				%let svi = ;
				%do %while(&another);
					%let iu = %index(&svij,_);
					%if &iu %then %do;
						%if %length(&svi) %then %let svi = &svi._;
						%let svi = &svi.%substr(&svij,1,%eval(&iu-1));
						%let len = %length(&svij);
						%if (&len > &iu) %then %do;
							%let svij = %substr(&svij,%eval(&iu+1),%eval(&len-&iu));
						%end;
						%else %let another = 0;
					%end;
					%else %let another = 0;
				%end;
				%put trimmed svi = &svi;
			%end;
			%if %length(%trim(&svi)) %then %do;
				%if &firstone %then %do;
					%let evjnew = &svi;
					%let firstone = 0;
				%end;
				%else %do;
					%let evjnew = &evjnew.*&svi;
				%end;
			%end;
			%let i=%eval(&i+1);
		%end;	
		%let svnew = &svnew &evjnew;
		%let j=%eval(&j+1);
	%end;
	%let _rc_ = %sysfunc(close(&_dsid_));
	
	* remove duplicates;
	%if %length(&svnew) %then %do;
		data temp;
			length name $ 32;
			%let i=1;
			%do %while(%length(%scan(&svnew,&i,' ')));
				i = &i;
				name = "%scan(&svnew,&i,' ')";
				output;
				%let i=%eval(&i+1);
			%end;
		run;
		proc sort data=temp nodupkey out=temp;
			by name;
		run;
		proc sort data=temp out=temp;
			by i;
		run;
		data _null_;
			set temp end=_last;
			call symputx("sv"||compress(_n_),name);
			if _last then call symputx("nsv",compress(_n_));
		run;
	%end;
	%else %do;
		%let nsv = 0;
	%end;
	%let SelVars =;
	%do i=1 %to &nsv;
		%let SelVars = &SelVars &&sv&i;
	%end;
	%put Unsplit SelVars = &SelVars;

	data ts2vars;
		length name $ 32;
		%let i=1;
		%do %while(%length(%scan(&SelVars,&i,' ')));
			name = "%scan(&SelVars,&i,' ')";
			output;
			%let i=%eval(&i+1);
		%end;
	run;
%mend;



* Reduce the training and test data sets and redefine predictor macro variables;
/*
arguments:
d = training data set
ds = data set containing variable _name_ with names of predictor variables to keep
d_test = test data set
filter = label to use for warning message
recover = Yes or No, recover original class variables from the selected indicator variables created from them
*/
%macro ReduceData(d,ds,d_test,filter,recover);
	%put Entering ReduceData;
	
	/*data OutLib.ds; set &ds; run;*/
	
	%if %length(&d_test) %then %do;
		%if &d_test = &d %then %do;
			%let d_test = ;
		%end;
	%end;
	%NObsVars(&ds);	
	%if (&nobs = 0) %then %do;
		%put WARNING: &filter filter produced no predictors.  An intercept-only model is assumed.;
				
		%if %length(%trim(%left(&FixedEffects))) %then %do;
			/*S1467543: Adding back fixed effects variables when variable 
			reduction methods are applied to filter out predictor variables*/		
			data d_fix;
				set &d;
				keep &FixedEffects;
			run;
		%end;		
		
		**S1376104: Intecept variable is created and deleted in the same data step. To fix this, the intercept variable is created in an extra data step.**;
		data &d;
			set &d;
			*Intercept = 1;
			drop %if ^&upred %then %do; &PredictorVars %end; %else %do; %do j=1 %to &nix_blocks; &&PredictorVars&j %end; %end; &ClassPredVars
				/*
				%do i=1 %to &nix; %sysfunc(varname(&_dsix_,&i)) %end; 
				%do i=1 %to &nip; %sysfunc(varname(&_dsip_,&i)) %end; 
				*/
			;
		run;
	
		data &d;
			*set &d;
			merge &d %if %length(%trim(%left(&FixedEffects))) %then d_fix;;
			Intercept = 1;
		run;
		
		%if %length(&d_test) %then %do;
					
			%if %length(%trim(%left(&FixedEffects))) %then %do;
				/*S1467543: Adding back fixed effects variables when variable 
				reduction methods are applied to filter out predictor variables*/			
				data d_test_fix;
					set &d_test;
					keep &FixedEffects;
				run;				
			%end;

			data &d_test;
				set &d_test;
				*Intercept = 1;
				drop %if ^&upred %then %do; &PredictorVars %end; %else %do; %do j=1 %to &nix_blocks; &&PredictorVars&j %end; %end; &ClassPredVars
					/*
					%do i=1 %to &nix; %sysfunc(varname(&_dsix_,&i)) %end; 
					%do i=1 %to &nip; %sysfunc(varname(&_dsip_,&i)) %end; 
					*/
				;
			run;
			
			data &d_test;
				*set &d_test;
				merge &d_test %if %length(%trim(%left(&FixedEffects))) %then d_test_fix;;
				Intercept = 1;
			run;
			
		%end;

		%if ^&upred %then %do;	
			%let PredictorVars = Intercept;
		%end; 
		%else %do;
			%let PredictorVars = Intercept;
			%let PredictorVars1 = Intercept;
			%do j=2 %to &nix_blocks;
				%let PredictorVars&j = ;
			%end;
		%end;					
		%let ClassPredVars = ;
		
		%if %symexist(_dsix_) %then %do;
			%if (&_dsix_) %then %do;
				%let _rc_ = %sysfunc(close(&_dsix_));
			%end;
		%end;
		%let nix = 1;
		data dsix;
			Intercept = 1;
		run;
		%let _dsix_ = %sysfunc(open(dsix));
		%if %symexist(_dsip_) %then %do;
			%if (&_dsip_) %then %do;
				%let _rc_ = %sysfunc(close(&_dsip_));
			%end;
		%end;
		%let nip = 0;
		%put nix = &nix;
		%put nip = &nip;
	%end;
	%else %do;
		* add back vars _train_,_RowNum_,_Obs which might filter out for the case of applying user specified "_:" list style var;
		%if &PredictorVars=%nrbquote(_:) | &ClassPredVars=%nrbquote(_:) %then %do;
			%let _idx1_ = 0;
			%let _idx2_ = 0;
			%let _idx3_ = 0;
			%CheckVarExist(_train_,&d,,_ShowError=0);
			%if &exiterror %then %let exiterror = 0;
			%else %let _idx1_ = 1;
			%CheckVarExist(_RowNum_,&d,,_ShowError=0);
			%if &exiterror %then %let exiterror = 0;
			%else %let _idx2_ = 1;
			%CheckVarExist(_Obs,&d,,_ShowError=0);
			%if &exiterror %then %let exiterror = 0;
			%else %let _idx3_ = 1;

			data _VarsToKeep_;
				length _name_ $32;
				%if &_idx1_ %then %do;
					_name_ = "_train_";
					output;
				%end;
				%if &_idx2_ %then %do;
					_name_ = "_RowNum_";
					output;
				%end;
				%if &_idx3_ %then %do;
					_name_ = "_Obs";
					output;
				%end;
			run;
		
		  	data names;
				set &ds _VarsToKeep_;
				keep _name_;
			run;
		%end;
		%else %do;
		   	data names;
				set &ds;
				keep _name_;
			run;
		%end;
		
		/*
		%if &nixl %then %do;
			data np;
				length _name_ $32;
				%do i=1 %to &nixl; 
					_name_ = "%sysfunc(varname(&_dsixl_,&i))";
					output;
				%end;
			run;
			data names;
				set names np;
			run;
		%end;
		%if &nipl %then %do;
			data np;
				length _name_ $32;
				%do i=1 %to &nipl; 
					_name_ = "%sysfunc(varname(&_dsipl_,&i))";
					output;
				%end;
			run;
			data names;
				set names np;
			run;
		%end;
		*/
		
		proc sort data=names nodupkey;
			by _name_;
		run;
		
		data names;
			set names;
			_in_ = 1;
		run; 
	
		/*data OutLib.names; set names; run;*/
		
		/*
		%put PredictorVars = &PredictorVars;
		%put ListContPredVars = &ListContPredVars;
		*/
		
		%*put nipin= &nip;
		
		%if %length(&PredictorVars) %then %do;
			%if ^&upred %then %do;
				%ReduceLists(&d,names,&d_test,PredictorVars,ListContPredVars,&ListContColon,x,continuous,&recover);
			%end;
			%else %do;
				**S1312203: Fixing issue when colon style is used in the variable list and declaring lock-in variables.**;
				%ReduceLists(&d,names,&d_test,PredictorVars,ListContPredVars,0,x,continuous,&recover);
			%end;
		%end;	
		%if %length(&ClassPredVars) %then %do;
			%ReduceLists2(&d,names,&d_test,ClassPredVars,ListClassPredVars,&ListClassColon,p,class,&recover);
		%end;
		
		%*put nipout= &nip;
	%end;
	%put Exiting ReduceData;
%mend;

/*
		%if (&nobs > &nix) %then %do;
			%do i=%eval(&nix+1) %to &nobs;
				%global _x&i;
			%end;
		%end;		
		%let nix = &nobs;
		
		%put Reducing number of predictors to &nix.;

		proc sort data=&ds;
			by _name_;
		run;

		data _null_;
			set &ds;
			length ix$ 16;
			ix = "_x" || trim(left(_n_));
			call symput(ix,trim(left(_name_)));
		run;

		* update PredictorVars, take advantage of colon specification when possible;
		%let PredictorVars = ;
		%if (%length(&ListContPredVars) and &ListContColon) or 
			(%length(&ListClassPredVars) and &ListClassColon) %then %do;
			data d2;
				set &d;
				keep &PredictorVars &ClassPredVars;
			run;
			
			* create list of just the pred vars not in colon lists;
			data d2;
				set d2;
				drop
					%if (&ListContColon) %then %do; &ListContPredVars %end;
					%if (&ListClassColon) %then %do; &ListClassPredVars %end;
				;
			run;

			proc transpose data=d2 out=d2t(keep=_name_);
			run;

			%NObsVars(d2t);
			%global niv;
			%let niv = &nobs;

			%do i=1 %to &niv; 
				%global _v&i;
			%end; 
			
			data _null_;
				set d2t;
				nm = "_v" || trim(left(put(_n_,10.0)));
				call symput(nm,trim(left(_name_)));
			run;
						
			%do i=1 %to &niv; 
				%let PredictorVars = &PredictorVars &&_v&i;
			%end; 
			
			%if (&ListContColon) %then %do; 
				%let PredictorVars = &PredictorVars &ListContPredVars;
			%end;
			%if (&ListClassColon) %then %do; 
				%let PredictorVars = &PredictorVars &ListClassPredVars;
			%end;
		%end;
		%else %do;
			%do i=1 %to &nix;
				%let PredictorVars = &PredictorVars %sysfunc(varname(&_dsix_,&i));
			%end;
		%end;
		
		%let nip = 0;
		%let ClassPredVars = ;

		data d2;
			set &d;
			keep %do i=1 %to &nix; %sysfunc(varname(&_dsix_,&i)) %end; ;
		run;

		data &d;
			merge d1 d2;
		run;
	%end;
%mend;
*/

%macro ReduceLists(d,ds,d_test,vars,listvars,colonlist,v,type,recover);
	* d: input dataset to be reduced;
	* ds: dataset with variables to be kept;
	* d_test: input test dataset;
	* vars: original variable list;
	* listvars: orignal list style vars;
	* colonlist: indicator of specifying list style variable; 
	* v: has value x=continuous or p=class; 
	* type: continuous or class
	* recover: recover class variables;
	
	%if %length(%trim(%left(&FixedEffects))) %then %do;
		data d_fix;
			set &d;
			keep &FixedEffects;
		run;
		
		%if %length(&d_test) %then %do;	
			data d_test_fix;
				set &d_test;
				keep &FixedEffects;
			run;
		%end;
		
	%end;

	%if ^&upred %then %do; 
		%put Reducing list &&&vars;
	%end; 
	%else %do; 
		%put Reducing list;
		%do j=1 %to &nix_blocks; 
			%put &&PredictorVars&j; 
		%end;	
	%end;
	
	/*
	%put ReduceLists:;
	%put d=&d;
	%put ds=&ds;
	%put d_test=&d_test;
	%put vars=&&&vars;
	%put listvars=&&&listvars;
	%put colonlist=&colonlist;
	%put v=&v;
	%put type=&type;
	%put recover=&recover;
	*/
	
	* temporarily combine training and test;
	%if %length(&d_test) %then %do;	
		%let d0 = &d;
		data d1;
			set &d;
			_train_ = 1;
		run;
		data d2;
			set &d_test;
			_train_ = 0;
		run;

		data dboth;
			set d1 d2;
		run;
		%let d = dboth;
	%end;		
	
	* save all other variables;
	* drop and keep can be slow for wide data;
	/*
	data d1(drop= 
			%do i=1 %to &&ni&v; %sysfunc(varname(&&_dsi&v._,&i)) %end;
		);
		set &d;
	run;
	*/

	%if ^&upred %then %do; 
		data d1(drop=&&&vars);
			set &d;
		run;
	%end; 
	%else %do; 
		data d1(drop= %do j=1 %to &nix_blocks; &&PredictorVars&j %end;);
			set &d;
		run;
	%end;

	/* sql is even slower
	proc sql;
		alter table d1 drop 
			%sysfunc(varname(&&_dsi&v._,1)) %do i=2 %to &&ni&v; ,%sysfunc(varname(&&_dsi&v._,&i)) %end;
		;
	run;
	*/
	 

	* transpose vars to tall and join with ds by _name_ to reduce variables;
	/*
	proc sql;
		create table d2 as 
		select %sysfunc(varname(&&_dsi&v._,1)) %do i=2 %to &&ni&v; ,%sysfunc(varname(&&_dsi&v._,&i)) %end; from &d;
	quit;
	*/
	
	/*
	data d2(keep=
		   %do i=1 %to &&ni&v; %sysfunc(varname(&&_dsi&v._,&i)) %end;
		);
		set &d;
	run;
	*/

	/*
	* create macro variables;
	data _null_;
		set &ds end=_last;
		nm = "_&v" || trim(left(put(_n_,10.0)));
		call symputx(nm,trim(left(_name_)));
		if _last then call symputx(nv,trim(left(_n_))); 
	run;

	%let newvars = ;
	%do i=1 %to &nv; 
		%let newvars = &newvars &&_&v&i;
	%end;
	*/
	
	%if ^&upred %then %do; 
		data d2(keep=&&&vars /*&LockContPredVars*/);
			set &d;
		run;
	%end; 
	%else %do; 		
		data d2(keep= %do j=1 %to &nix_blocks; &&PredictorVars&j %end; /*&LockContPredVars*/);
			set &d;
		run;		
	%end;
	
	/*
	proc datasets lib=work;
	run;
	*/

	%NObsVars(d2);
	
	* perform reduction differently for tall and wide for speed;
	%if &nobs > &nvars %then %do;
		proc transpose data=&ds out=ds1(drop=_name_);
		run;
		
		/*
		data OutLib.ds1;
			set ds1;
		run;
		*/
		
		%let _ds = %sysfunc(open(ds1));
		%let nv = %sysfunc(attrn(&_ds,NVARS)); 
		
		data d2;
			set d2;
			keep %do _i=1 %to &nv; %sysfunc(varname(&_ds,&_i)) %end; /*&LockContPredVars*/;
		run;
		
		/*data OutLib.d2; set d2; run;*/
		
		%let _ds = %sysfunc(close(&_ds));

		proc transpose data=d2 out=d3;
		run;
	
	%end;
	%else %do;
		proc transpose data=d2 out=d2t;
		run;

		proc sql;
			create table d3 as
			select * 
			from &ds as l inner join d2t as r
			on l._name_=r._name_;
		quit;
	%end;

	/*
	proc sort data=d2t;
		by _name_;
	run;
	
	data d2t;
		set d2t;
		_inlist_ = 1;
	run;
	
	* assumes &ds is already sorted and has variables _name_ and _in_=1;
	data d3;
		merge &ds d2t;
		by _name_;
		if (_in_=1) and (_in_ = _inlist_);
		drop _in_ _inlist_;
	run;
	*/
		
	* recover original class variables from the selected indicator variables created from them;
	%if (&type=class) and (&recover=Yes) %then %do;
	
		%put Replacing indicator variables with original class variables;
		
		data d3;
			set d3;
			rename _name_ = new_name;
			_keep = 1;
			keep _name_ _keep;
		run;
						
		* the data set opc links indicator names to original class names;	
		/*
		data d4;
			set opc;
			_obs_ = _n_;
			keep _obs_ orig_name new_name;
		run;
				
		* sas-ify the names with two transposes;
		proc transpose data=d4 out=d4t;
			var _obs_;
			id new_name;
		run;
		
		proc transpose data=d4t out=d4tt(rename=(_name_=new_name));
		run;
				
		data d4a;
			set d4;
			keep _obs_ orig_name;
		run;
		
		data d4;
			merge d4tt d4a;
			by _obs_;
			drop _obs_;
		run;
		*/
		
		data d4;
			set opn;
			length orig_name $ 64;
			if name="Intercept" then delete;
			rename name = new_name;
			orig_name = scan(parameter,1);
			drop parameter;
		run;
				
		proc sort data=d4;
			by new_name;
		run;

		* merge and save list of class var names;
		data d5;
			merge d3 d4;
			by new_name;
			if _keep = 1;
			keep orig_name _keep;
		run;
				
		proc sort data=d5 nodupkey;
			by orig_name;
		run;

		data d5;
			set d5;
			rename orig_name = _name_;
		run;

		* orig_d_class was saved previously, transpose and sort it now;
		proc transpose data=orig_d_class out=odct;
			var _all_;
		run;
		* the above transpose will convert numerical variables to character when there are mixed types of variables; 
		
		proc sort data=odct;
			by _name_;
		run;
		
		* replace d3 with original class variable data;
		data d3;
			merge d5 odct;
			by _name_;
			if _keep = 1;
		run;
		
		data d3;
			set d3;
			drop _keep;
		run;

		proc sort data=d3 nodupkey;
			by _name_;
		run;
		
		%NObsVars(d3);
		
		* ensure that the if condition below is satisfied so data are replaced;
		%let ni&v = %eval(&nobs+1);
	%end;
	%else %do;
		proc sort data=d3 nodupkey;
			by _name_;
		run;
		
		%NObsVars(d3);
	%end;
	
	%if (&nobs < &&ni&v) %then %do;
		
		%put Reducing number of &type predictors to &nobs.;
	
		%let ni&v = &nobs;
	
		/*
		data _null_;
			set d3;
			length iv$ 16;
			iv = "_&v" || trim(left(_n_));
			call symput(iv,trim(left(_name_)));
		run;
		*/
		
		proc transpose data=d3 out=d3t;
			var col:;
		run;
			
		* update var lists, take advantage of colon specification if available;
		%if ^&upred %then %do; 
			%let &&vars = ; 
		%end; 
		%else %do; 
			%do j=1 %to &nix_blocks; 
				%let PredictorVars&j=; 
			%end;	
		%end;		

		%put colonlist = &colonlist;
		%if &colonlist %then %do;
			* create list of just the vars without colons;
			%let listvarsc = ;
			%let i = 1;
			%do %while(%length(%scan(&&&listvars,&i)));
				%let term = %scan(&&&listvars,&i);
				%if %index(&term,:) %then %do;
					%let listvarsc = &listvarsc &term;
				%end;
				%let i=%eval(&i+1);
			%end;
			%put listvarsc = &listvarsc;
			
			data d3t1;
				set d3t;
				drop _name_ _label_ &listvarsc;
			run;

			%NObsVars(d3t1);

			%if (&nvars > 0) %then %do;
				proc transpose data=d3t1 out=d3t1t(keep=_name_);
					var _all_;
				run;

				%NObsVars(d3t1t);
				%let niv = &nobs;

				* create macro variables;
				data _null_;
					set d3t1t;
					nm = "_v" || trim(left(put(_n_,10.0)));
					call symput(nm,trim(left(_name_)));
				run;

				%if ^&upred %then %do; 
					%do i=1 %to &niv; 
						%let &&vars = &&&vars &&_v&i;
					%end;
				%end; 
				%else %do; 
					%put ERROR: When using Loci-In marker variables, colon list is replaced by explicit referring to variable names.;
				%end;
			%end;

			data d3t2;
				set d3t;
				keep &listvarsc;
			run;

			%NObsVars(d3t2);
			
			%if (&nvars > 0) %then %do;
				%if ^&upred %then %do; 
					%let &&vars = &&&vars &listvarsc; 
				%end; 
				%else %do; 
					%put ERROR: When using Loci-In marker variables, colon list is replaced by explicit referring to variable names.;	
				%end;			
			%end;
		%end;
		%else %do;
			* create macro variables;
			data _null_;
				set d3;
				nm = "_&v" || trim(left(put(_n_,10.0)));
				call symput(nm,trim(left(_name_)));
			run;

			%if ^&upred %then %do; 
				%do i=1 %to &&ni&v; 
					%let &&vars = &&&vars &&_&v&i; 
				%end;
			%end; 
			%else %do; 
				**S1312203: Fixing issue when colon style is used in the variable list and declaring lock-in variables.**;
				%let nVarPerBlock= %sysevalf(&&ni&v/&nix_blocks);
				%put nVarPerBlock= &nVarPerBlock;
				%let fl= %sysfunc(floor(&nVarPerBlock));
				%let ce= %sysfunc(ceil(&nVarPerBlock));

				%let l= &fl;
				%let k= 1;				
				%do j=1 %to &nix_blocks;
					%if %sysevalf(&j= &nix_blocks) and (&ce > &fl) %then %do;
						%let l= %sysevalf(&l + 1);
					%end;
					%let PredictorVars&j=;
					%do i=&k %to &l; 
						%let PredictorVars&j= &&PredictorVars&j &&_&v&i; 
					%end;		
					%let k= %sysevalf(&l + 1);
					%let l= %sysevalf(&l + &fl);
				%end;
			%end;			
		%end;	
 	
		* update data sets, the dsi&v data set must contain the reduced variables in order for %sysfunc(varname()) refs to work;
		%let _rc_ = %sysfunc(close(&&_dsi&v._));
		%if (&&ni&v > 0) %then %do;
			data dsi&v;
				set d3t;
				drop _name_;
				/* keep statements like this can be very slow for long interleaved lists */
				/*
				set &d;
				keep %do i=1 %to &&ni&v; &&_&v&i %end; ;
				*/
			run;	
			
			/*data OutLib.d1; set d1; run;*/
			/*data OutLib.dsi&v; set dsi&v; run;*/
			
			data &d;
				merge d1 dsi&v;
			run;
			/*
			%if %length(&LockContPredVars) %then %do;
				data dsi&v;
					set dsi&v;
					drop &LockContPredVars;
				run;
			%end;
			*/
			%let _dsi&v._ = %sysfunc(open(dsi&v));
			/*%let nix= %sysfunc(attrn(&&_dsi&v._, NVARS));*/
		%end;
		%else %do;
			data &d;
				set d1;
			run;
		%end;
	%end;
	%else %do;
		%put Keeping all &&ni&v &type predictors;
	%end;
	
	%if ^&upred %then %do; 
		%put &&vars = &&&vars; 
	%end; 
	%else %do;
		%do j=1 %to &nix_blocks; 
			%put PredictorVars&j= &&PredictorVars&j; 
		%end;	
	%end;	
	
	%if %length(&d_test) %then %do;	
		data &d0;
			set &d;
			if _train_ = 1;
		run;
		data &d_test;
			set &d;
			if _train_ = 0;
		run;
		/*
		%if (&&ni&v > 0) %then %do;
			data orig_d_test;
				%if (&type=class) and (&recover=Yes) %then %do;
					set orig_d_test_class;
				%end;
				%else %do;
					set &d_test;
				%end;
				keep %do i=1 %to &&ni&v; %sysfunc(varname(&&_dsi&v._,&i))  %end; ;
			run;

			data &d_test;
				merge d1_test orig_d_test;
			run;
		%end;
		%else %do;
			data &d_test;
				set d1_test;
			run;
		%end;
		*/
	%end;

	%if %length(%trim(%left(&FixedEffects))) %then %do;

		/*S1467543: Adding back fixed effects variables when variable 
		reduction methods are applied to filter out predictor variables*/
	
		%if %length(&d_test) %then %do;	
			data &d0;
				merge &d0 d_fix;
			run;
			
			data &d_test;
				merge &d_test d_test_fix;
			run;
		%end;
		%else %do;
			data &d;
				merge &d d_fix;
			run;		
		%end;	
	%end;

%exit:;
%mend;

* Sept 13. 2009 by TC;
* modified from %ReduceLists which produces error with mixed types of Class variables;
%macro ReduceLists2(d,ds,d_test,vars,listvars,colonlist,v,type,recover);
	* d: input dataset to be reduced;
	* ds: dataset with variables to be kept;
	* d_test: input test dataset;
	* vars: original variable list;
	* listvars: orignal list style vars;
	* colonlist: indicator of specifying list style variable; 
	* type: indicator of continuous (x) or class (p) variable; 
	* recover: recover class variables;
	* note: ds can have multi-type of variables (to be kept) under _name_ variable;

	%put Reducing list2 &&&vars;
	/*
	%put Check Macro Variables:;
	%put d=&d;
	%put ds=&ds;
	%put d_test=&d_test;
	%put vars=&&&vars;
	%put listvars=&&&listvars;
	%put colonlist=&colonlist;
	%put v=&v;
	%put type=&type;
	%put recover=&recover;

	ods exclude none;
	ods results;
	title "variables to be kept";
	proc print data=&ds;
	run;
	*/

	* temporarily combine training and test;
	%if %length(&d_test) %then %do;	
		%let d0 = &d;
		data d1;
			set &d;
			_train_ = 1;
		run;
		data d2;
			set &d_test;
			_train_ = 0;
		run;

		data dboth;
			set d1 d2;
		run;
		%let d = dboth;
	%end;		
	
	* set up variable lists;
	/* 1. variable other than &&&vars;
	data d1;
		set &d;
		if _n_ > 1 then stop; 
		drop &&&vars;
	run;
	*/
	* 2. variables to drop;
	/*
	data d2;
		set &d; 
		if _n_ > 1 then stop;
		keep %do i=1 %to &&ni&v; %sysfunc(varname(&&_dsi&v._,&i)) %end; ;
		* keep &&&vars;
	run;

	proc transpose data=d2 out=d2t;
		var %do i=1 %to &&ni&v; %sysfunc(varname(&&_dsi&v._,&i)) %end; ;
		* var &&&vars;
	run;
	*/
	
	data d2t;
		length _name_ $ 32;
		%do i=1 %to &&ni&v; 
			_name_ = "%sysfunc(varname(&&_dsi&v._,&i))";
			output;
		%end; 
	run;
	
	proc sort data=d2t;
		by _name_;
	run; 
	
	/*data OutLib.d2t; set d2t; run;*/
	
	* assumes &ds is already sorted and has variables _name_ and _in_=1;
	data d3d;
		length _name_ $ 32;
		merge &ds d2t(in=_inlist_);
		by _name_;
		if (_in_ ne 1) and (_inlist_); 
		* rename _name_ = new_name;
		_one_ = 1;
		keep _one_ _name_;
	run; 
		
	data d3k; 
		length _name_ $ 32;
		merge &ds d2t(in=_inlist_);
		by _name_;
		if _in_ and (_inlist_);
		rename _name_ = new_name;
		keep _name_;
	run; 
	
	* recover original class variables from the selected indicator variables created from them;
	%if (&type=class) and (&recover=Yes) %then %do;
	
		%put Replacing indicator variables with original class variables;
	
		* the data set opc links indicator names to original class names;	
		/*
		data d4;
			set opc;
			_obs_ = _n_;
			keep _obs_ orig_name new_name;
		run; 

		* sas-ify the names with two transposes;
		proc transpose data=d4 out=d4t;
			var _obs_;
			id new_name;
		run;
		
		proc transpose data=d4t out=d4tt(rename=(_name_=new_name));
		run;
		
		data d4;
			set d4;
			drop new_name;
		run;
		
		data d4tt;
			set d4tt;
			keep new_name;
		run;
		
		data d4;
			merge d4 d4tt;
		run;
		*/
				
		data d4;
			set opn;
			length orig_name $ 64;
			if name="Intercept" then delete;
			rename name = new_name;
			orig_name = scan(parameter,1);
			drop parameter;
		run;
		
		/*data OutLib.d4; set d4; run;*/
		/*data OutLib.d3k; set d3k; run;*/
			
		* extract variables to keep;
		data d4k;
			if 0 then set d3k;
			if _n_ = 1 then do;
				declare hash h(dataset: "work.d3k", hashexp: 6);
				h.defineKey('new_name');
				h.defineDone();
			end;
			set d4;
			_one_ = 1;
			if (h.find()=0) then output;
		run;
		
		/*data OutLib.d4k; set d4k; run;*/
		
		proc sort data=d4k(keep=orig_name _one_) nodupkey;
			by orig_name;
		run;
						
		%NObsVars(d4k); 
		%let nrVars = &nobs;	
		%if &nrVars = &&ni&v %then %do;	
			%put Keeping all &&ni&v &type predictors;
			%*goto _KeepAll; 
		%end;

		/* extract variables to drop;
		data d4d;
			if 0 then set d3d;
			if _n_ = 1 then do;
				declare hash h(dataset: "work.d3d", hashexp: 6);
				h.defineKey('new_name');
				h.defineDone();
			end;
			set d4;
			if (h.find()=0) then output;
		run;

		proc sort data=d4d(keep=orig_name) nodupkey;
			by orig_name;
		run;

		data d4d2;
			merge d4d(in=d4d) d4k(in=d4k);
			by orig_name;
			if d4d & ^d4k;
		run;
		*/

		/*
		%let _ds = %sysfunc(open(d4k));
		%let ndv = %sysfunc(attrn(&_ds,NOBS)); 
		%let rc = %sysfunc(fetchobs(&_ds,1));
		* update input dataset;

		data &d;
			* orig_d_class was saved previously;
  			merge &d (drop=&&&Vars)
				orig_d_class(keep=%do i = 1 %to &ndv; %sysfunc(getvarc(&_ds,1)) %let rc = %sysfunc(fetch(&_ds)); %end;);
		run; 
		*/
		
		proc transpose data=d4k out=d4kt(drop=_name_);
			var _one_;
			id orig_name;
		run;
		
		/*data OutLib.d4kt; set d4kt; run;*/
				
		%let _ds = %sysfunc(open(d4kt));
		%let ndv = %sysfunc(attrn(&_ds,NVARS)); 

		* update input dataset;
		data &d;
			* orig_d_class was saved previously;
			merge &d (drop=&&&Vars)
				orig_d_class(keep=%do i=1 %to &ndv; %sysfunc(varname(&_ds,&i)) %end;);
		run; 
		
		/*data OutLib.d; set &d; run;*/
		
		%let rc = %sysfunc(close(&_ds));
	%end;
	%else %do;
		%NObsVars(d3k);  
		%let nrVars = &nobs;
		%if &nrVars = &&ni&v %then %do;	
			%put Keeping all &&ni&v &type predictors;
			%goto _KeepAll; 
		%end;

		/*
		%let _ds = %sysfunc(open(d3d));
		%let ndv = %sysfunc(attrn(&_ds,NOBS)); 
		%let rc = %sysfunc(fetchobs(&_ds,1));
		* update input dataset;
		data &d;
			set &d;
			drop 
				%do i = 1 %to &ndv;
					%sysfunc(getvarc(&_ds,1))	
					%let rc = %sysfunc(fetch(&_ds));
				%end;
			;
		run;
		%let rc = %sysfunc(close(&_ds));
		*/
		
		data dc;
			set &d;
			keep _character_;
		run;
		
		data d5;
			set &d;
			_name_ = "";
			drop _name_;
		run;

		proc transpose data=d5 out=d5t;
			var _numeric_;
		run;
		
		data d5t;
			set d5t;
			_row_ = _n_;
		run;
		
		proc sort data=d5t;
			by _name_;
		run;
		
		proc sort data=d3d;
			by _name_;
		run;
		
		data d6;
			merge d5t d3d;
			by _name_;
			if not missing(_one_) then delete;
		run;
		
		proc sort data=d6;
			by _row_;
		run;
		
		data d6;
			set d6;
			drop _row_ _one_;
		run;
		
		proc transpose data=d6 out=&d;
		run;
		
		data &d;
			merge dc &d;
		run;
		
		/*
		data OutLib.outnew;
			set &d;
		run;
		*/
		
		/* this way works but the drop statement is very slow for long lists
		
		proc transpose data=d3d out=d3dt(drop=_name_);
			var _one_;
		run;
		
		%let _ds = %sysfunc(open(d3dt));
		%let ndv = %sysfunc(attrn(&_ds,NVARS)); 
		
		* drop variables from input dataset;
		data &d;
			set &d;
			drop %do i=1 %to &ndv; %sysfunc(varname(&_ds,&i)) %end; ;
		run;
		%let rc = %sysfunc(close(&_ds));
		
		data OutLib.outold;
			set &d;
		run;
		*/
		
	%end;
	
	%if (&nrVars <= &&ni&v) %then %do;

		%put Reducing number of &type predictors to &nrVars.;

		%let ni&v = &nrVars;

		%if ^&nrVars %then %do;
			* no predictor was kept!;
			%let &vars = ;
			%let _rc_ = %sysfunc(close(&&_dsi&v._));
			data dsi&v;
				if (0);
			run;
		%end;
		%else %do;
			%if (&type=class) and (&recover=Yes) %then %do;
				%let _ds = %sysfunc(open(d4k));
			%end;
			%else %do; 
				%let _ds = %sysfunc(open(d3k));
			%end;
			%let ndv = %sysfunc(attrn(&_ds,NOBS)); 
			%let rc = %sysfunc(fetchobs(&_ds,1)); 

			* update var lists;
			%let &vars = ;
			%do i = 1 %to &ndv;
				%let &vars = &&&vars %sysfunc(getvarc(&_ds,1));	
				%let rc = %sysfunc(fetch(&_ds));
			%end;	
			%let rc = %sysfunc(close(&_ds));

			* update data sets, the dsi&v data set must contain the reduced variables in order for %sysfunc(varname()) refs to work;
			%let _rc_ = %sysfunc(close(&&_dsi&v._));
			data dsi&v;
				set &d;
				keep &&&Vars;
			run;
			%let _dsi&v._ = %sysfunc(open(dsi&v));
		%end;
	%end;

	%put vars=&vars = &&&vars;

	%_KeepAll:;

	%if %length(&d_test) %then %do;	
		data &d0;
			set &d;
			if _train_ = 1;
		run;
		data &d_test;
			set &d;
			if _train_ = 0;
		run;
	%end;
%exit:;
%mend;


* Rename P_ variables in ts2o data set so they are consistent;
%macro Rename_P_;
	data temp;
		set ts2o;
		keep p_:;
		if _n_ > 1 then stop;
	run;
	proc transpose data=temp out=tempt;
		var p_:;
	run;
	/*
	data tempt;
		if (index(_name_,"_")) then class = scan(_name_,-1,"_");
		else class = _name_;
	run;
	proc sort data=tempt;
		by class;
	run;
	*/
	proc sort data=tempt;
		by _name_;
	run;
	%NObsVars(tempt);
	%if &nobs ne &nclasses %then %do;
		%put p_ variables not renamed because some levels have been dropped.;
	%end;
	%else %do;
		data _null_;
			set tempt;
			call symputx("cpp"||trim(left(_n_)),_name_);
		run;
		%do c=1 %to &nclasses;
			%put renaming &&cpp&c to &&p_prob&c;
		%end;
		data ts2o;
			set ts2o;
			rename %do c=1 %to &nclasses; &&cpp&c = &&p_prob&c %end; ;
		run;
	%end;
%mend;


/* Transpose Tall survival data set into Wide data set and insert bins for plotting survival curves
	in Parallel plot. */ 
%macro SurFunc_Tall_to_Wide(_inDs=,_outDs=,_idVar=,_nBin=50,_DsToMerge=,
	_time_vListPrefix=,_time_lListPrefix=);
	* _inDs*: indput tall sorted (by &_tVar) survival data. Assuming the first column is the time-to-event
		variable and the rest columns containing survival (step) function with the columnname to be array id.
		also, assume the data was sorted by the first column.;
	* _outDs: output wide survival data on fixed-width time points (50 bins);
	* _idVar*: variable name to be the id column of each row as the first column in the _outDs;
	* _nBin*: number of bins between smallest and largest time.;
		*Note: only the time points set by the bins will be keep in &_OutDs. This will make the survival
			curves similar but not exact the Cox model estimated curves. ;
	* _DsToMerge: Data containing experimental factors and predictors to be merged after transpose. 
		Assume that this data set contains &_idVar variable to be used as merge key variable and
		was sprted by &_idVar;
	* _time_vListPrefix: optional macro name to extract the list of variable names associated to time 
		(bins) in the transposed data for applying to create parallel plot in JMP;
		*Note: there are total &_nBin + 1 time points here;
	* _time_lListPrefix: optional macro name to extract the list of variable labels associated to time 
		(bins) in the transposed data for applying to create parallel plot in JMP;
	%if ^%length(&_outDs) %then %let _outDs = &_inDs;
/*
	%put Starting transpose Tall survival dataset into Wide format.;
	%put _inDs = &_inDs;
	%put _outDs = &_outDs;
	%put _idVar = &_idVar;
	%put _nBin = &_nBin;
	%put _DsToMerge = &_DsToMerge;
	%put _time_vListPrefix = &_time_vListPrefix;
	%put _time_lListPrefix = &_time_lListPrefix;
*/
	* extract variable names;
	proc contents data=&_inDs out=cont noprint;
	run;

	data _null_;
		set cont end=_e_;
		if VARNUM = 1 then call symput('_tVar',NAME);
		else do;
			vname = "_Var" || trim(left(VARNUM-1));
			call symput(vname,NAME);
		end;
		if _e_ then call symput('_nVar_',_n_-1);
	run;

	%put _tVar = &_tVar;
	%put _inDs = &_inDs;
	
	/*
	data OutLib.inds;
		set &_inDs;
	run;
	*/
	
	* find the min and max time in the data for setting bins;
	data _null_;
		set &_inDs(keep=&_tVar) end=_e_; * assume the data set is sorted by &_tVar;
		retain minFlag 0;
		if minFlag = 0 and &_tVar > 0 then do;
			call symputx ('tMin',&_tVar); 
			minFlag = 1;
		end;
		if _e_ then call symputx ('tMax',&_tVar); 
	run;
	
	* insert bins of equal width;
	%let increment = %sysevalf((&tMax-&tMin)/&_nBin);
	%put tMin=&tMin, tMax=&tMax, Increment=&increment;
	%let _d1Flag_ = 0;
	data tmp;
		do i = 0 to &_nBin;
			&_tVar = &tMin + i*&increment;
			output;
		end;
		drop i;
	run;

	data Sur2;
		merge tmp(in=_in1_) &_inDs(in=_in2_);
		by &_tVar;
		array _tmp_{&_nVar_} _temporary_ (&_nVar_*1);;
		retain _tmp_;
		%if &_d1Flag_ %then %do;
			if &_tVar = 0 then delete; * set starting time to be non-zero;
		%end;
		%do i = 1 %to &_nVar_;
			if _in2_ then do;
				_tmp_[&i] = &&_Var&i;
			end;
			else do;
				&&_Var&i = _tmp_[&i];
			end;
		%end;
		* set label to be applied as variable label after transpose in the next step;
		if &_tVar*100 = floor(&_tVar*100) then _idLabel_ = &_tVar;
		else _idLabel_ = round(&_tVar*100)/100;
		if ^_in1_ then delete; *comment out this line to include all the original time points;
		keep &_tVar %do i = 1 %to &_nVar_; &&_Var&i %end; _idLabel_;
	run;
	
	/*
	data OutLib.tmp;
		set tmp;
	run;
	data OutLib.sur2;
		set sur2;
	run;
	*/

	proc transpose data=Sur2 out=&_outDs(rename=(_LABEL_=&_idVar) drop=_NAME_); 
		var %do i = 1 %to &_nVar_; &&_Var&i %end;;
		id &_tVar;
		idlabel _idLabel_;
	run;

	%if %length(&_DsToMerge) %then %do;
		%let _d_ = %sysfunc(open(&_DsToMerge));
		%let _vNum_ = %sysfunc(VARNUM(&_d_,&_idVar));
		%if ^&_vNum_ %then %do;
			%put ERROR: Variable &_idVar is not in data set &_DsToMerge. It will not be merged into output data set.;
			%let rc = %sysfunc(close(&_d_));
			%goto exit;
		%end;
		%let _vTyp_ = %sysfunc(VARTYPE(&_d_,&_vNum_));
		%let _vLen_ = %sysfunc(VARLEN(&_d_,&_vNum_));
		%let _vFmt_ = %sysfunc(VARFMT(&_d_,&_vNum_));
		%let rc = %sysfunc(close(&_d_));
	%end;
	%else %do;
		%let _vNum_ = %sysfunc(VARNUM(&_d_,&_idVar));
		%let _d_ = %sysfunc(open(&_outDs));
		%let _vTyp_ = %sysfunc(VARTYPE(&_d_,&_vNum_));
		%let rc = %sysfunc(close(&_d_));
	%end;

	data &_outDs;
		set &_outDs;
		%if %upcase(&_idVar) = _ROWNUM_ | &_vTyp_ = N %then %do;
			_tmp_ = &_idVar + 0;
			drop &_idVar;
			rename _tmp_ = &_idVar;
		%end;
		label &_idVar = "&_idVar";
	run;

	%if %length(&_time_vListPrefix) | %length(&_time_lListPrefix) %then %do;
		data tmp2;
			set &_outDs;
			if _n_ > 1 then stop;
		run;
		proc transpose data=tmp2 out=tmp3;
		run;

		data _null;
			set tmp3;
			%if %length(&_time_vListPrefix) %then %do;
				%do i = 1 %to %eval(&_nBin + 1);
					%global &_time_vListPrefix.&i;
				%end;
				name_v = trim("&_time_vListPrefix") || trim(left(_n_));
				call symput(name_v,_NAME_);
			%end;
			%if %length(&_time_lListPrefix) %then %do;
				%do i = 1 %to %eval(&_nBin + 1);
					%global &_time_lListPrefix.&i;
				%end;
				name_l = trim("&_time_lListPrefix") || trim(left(_n_));
				call symput(name_l,_LABEL_);
			%end;
		run;
	%end;

	proc sort data=&_outDs;
		by &_idVar;
	run;

	%if %length(&_DsToMerge) %then %do;
		data &_outDs;
			Format &_idVar &_vFmt_; * set &_idVar to be in 1st column;
			merge &_DsToMerge &_outDs(in=_in_);
			by &_idVar;
			if _in_;
		run;
	%end;
%exit:
%mend SurFunc_Tall_to_Wide;


/* averaging survival rate across multiple runs for each single individual 
	to be applied for drawing survival plots after CVMC */
%macro Surfunc_Average(_inDs=,_outDs=,_tVar=);
	* _inDs*: Input (stacked) tall data set with individual survival curve listed as column.
		Assume the leading columns are non-survival columns with the last non-survival column to be &_tVar
		and rest of columns are columns of survival curves. Also, assume Data is sorted by this order
		(Iter,I,J,K,&_tVar). Therefore, within each run the data shall be sorted by &_tVar; 
	* _outDs: Ouput data set with average survival curves across all runs. Variable 
		&_tVar is carried into this data set.;
	* _tVar*: Reference time variable which is x-cordinate in survival plot.;

%if ^%length(_outDs) %then %let _outDs = &_inDs;
* extract column name for each individual survival curve;
proc contents data=&_inDs out=tmp noprint;
run;

* extract the position of &_tVar;
data _null_;
	set tmp(keep=Name VARNUM LABEL);
	if trim(left(NAME)) = trim(left("&_tVar")) then do;
		call symput('_tVarPosition_',VARNUM);
		stop;
	end;
run;

%let _JFlag_ = 0;
data _null_;
	set tmp(keep=Name VARNUM LABEL) end=_e_;
	_C_ = VARNUM-&_tVarPosition_;
	if _C_>0 then do;
		_Var_ = "_sVar_" || trim(left(_C_));
		call symput(_Var_,NAME);
		_Label_ = "_sLabel_" || trim(left(_C_));
		call symput(_Label_,LABEL);
	end;
	else do;
		* check if column "J" exists;
		if trim(left(upcase(NAME))) = "J" then call symput('_JFlag_',1);
	end; 
	if _e_ then call symput("_nSur_",_n_-&_tVarPosition_);
run;

* Slice data by each run, then merge all subsets for averaging;
proc sort data=&_inDs(keep=Iter %if &_JFlag_ %then J; K) out=tmp nodupkey;
	by Iter %if &_JFlag_ %then J; K;
run;

data _null_;
	set tmp end=_e_;
	name_i = "_I_"||trim(left(_n_));
	call symput(name_i,Iter);
	%if &_JFlag_ %then %do;
		name_j = "_J_"||trim(left(_n_));
		call symput(name_j,J);
	%end;
	name_k = "_K_"||trim(left(_n_));
	call symput(name_k,K);
	if _e_ then call symput('_nRun_',_n_);
run;

* assume &_inDs was sorted;
%let _nAllSur_ = %eval(&_nRun_ * &_nSur_);
* Setp1: Slicing;
data %do i = 1 %to &_nRun_;  
		Slice_&i (rename=(%do n=1 %to &_nSur_; &&_sVar_&n=_&n._&i %end;))
	%end;;
	set &_inDs;
	%do i = 1 %to &_nRun_;
		if (Iter=&&_I_&i %if &_JFlag_ %then & J=&&_J_&i; & K=&&_K_&i) then output Slice_&i;
	%end;
	drop ITER %if &_JFlag_ %then J; K;
run;

* Step 2: Merging & Averaging;
data &_outDs;
	merge %do n = 1 %to &_nRun_; Slice_&n %end;;
	by &_tVar;
	array _tmp_ {&_nAllSur_} _temporary_;
	array _Sur_ {&_nAllSur_} %do i = 1 %to &_nRun_; %do j = 1 %to &_nSur_; _&j._&i %end; %end;;
	retain _tmp_;
	do i = 1 to &_nAllSur_;
		if _Sur_(i) ne . then _tmp_(i) = _Sur_(i);
		else _Sur_(i) = _tmp_(i);
	end;
	%do s = 1 %to &_nSur_;
		&&_sVar_&s = mean(of _&s._:);
	%end;
	keep &_tVar %do s = 1 %to &_nSur_; &&_sVar_&s %end;;
	label %do s = 1 %to &_nSur_; &&_sVar_&s = "%trim(%left(&&_sLabel_&s))" %end;;
run;
%mend Surfunc_Average;


* macro to compute svd using PROC PRINCOMP;
* modified from http://bazaarwocky.wordpress.com/2011/06/22/a-macro-for-svd/;
%macro svd(input_lib,input_dsn,output_V,output_S,output_U,input_vars,ID_var);

	%put Computing SVD on &input_lib..&input_dsn;
	%local blank para EV USCORE n pos dsid nobs shownote showsource;
	%let shownote=%sysfunc(getoption(NOTES));
	%let showsource=%sysfunc(getoption(SOURCE));
	%let EV = EIGENVAL;
	%let USCORE = USCORE;
	%let dsid = %sysfunc(open(&input_dsn));
	%let nobs = %sysfunc(attrn(&dsid, NOBS));
	%put nobs = &nobs;
	%let dsid = %sysfunc(close(&dsid));

	data &input_dsn._adj; 
		set &input_dsn;
		keep &ID_var
			%if ^&upred or %length(&input_vars) %then %do;
				/*Have to use or clause because the call to svd in the DistanceScoring AP does not use PredictorVars*/
				&input_vars 
			%end; 
			%else %do; 
				&LockContPredVars
				%do j=1 %to &nix_blocks; 
					&&PredictorVars&j 
				%end;	
			%end;
		;
	run;

	proc sql noprint;
		select count(*) into : n
		from dictionary.columns
		/*
		where LIBNAME = upcase("&input_lib")
		and MEMNAME = upcase("&input_dsn")
		*/
		where LIBNAME = upcase("WORK")
		and MEMNAME = upcase("&input_dsn._adj")
		and name ne "&ID_var";
	quit;

	%let n = %sysevalf(&n);
	%put n = &n;

	proc sql noprint;
		select name into : var1 - : var&n
		from dictionary.columns
		/*
		where LIBNAME = upcase("&input_lib")
		and MEMNAME = upcase("&input_dsn")
		*/
		where LIBNAME = upcase("WORK")
		and MEMNAME = upcase("&input_dsn._adj")
		and name ne "&ID_var";
	quit;

	/*
	%do i = 1 %to &n;
		%put var&i = &&var&i;
	%end;
	*/

	/*
	%macro loop;
		data &input_dsn._adj; 
			set &input_dsn._adj;
			%do i = 1 %to &n;
				&&var&i = &&var&i * &nobs;
			%end;
		run;
	%mend loop;

	%loop
	*/

	/*
	Let X = USV'
	Then X'X=VS2V'
	SV-Decompose (1/n)*X'X
	*/

	proc princomp data=&input_dsn._adj
		out=&output_U(keep=&ID_Var Prin:)  std
		outstat=&input_dsn._stat(where=(_type_ in ("&USCORE", "&EV"))) noint cov noprint vardef=n;
		var 
			%if ^&upred or %length(&input_vars) %then %do; 
				/*Have to use or clause because the call to svd in the DistanceScoring AP does not use PredictorVars*/
				&input_vars 
			%end; 
			%else %do; 
				&LockContPredVars
				%do j=1 %to &nix_blocks; 
					&&PredictorVars&j 
				%end;	
			%end;
		;
	run;

	data &output_S;
		set &input_dsn._stat;
		format Number 7.0;
		format EigenValue SingularValue Proportion Cumulative 7.4;
		keep Number EigenValue SingularValue Proportion Cumulative;
		where _type_="&EV";
		array _X{*} 
			%if ^&upred or %length(&input_vars) %then %do; 
				/*Have to use or clause because the call to svd in the DistanceScoring AP does not use PredictorVars*/
				&input_vars 
			%end; 
			%else %do;
				&LockContPredVars
				%do j=1 %to &nix_blocks; 
					&&PredictorVars&j 
				%end;	
			%end;
		;
		Total = sum(of %if ^&upred or %length(&input_vars) %then %do; &input_vars %end; %else %do; &LockContPredVars %do j=1 %to &nix_blocks; &&PredictorVars&j %end; %end;)/&nobs;
		Cumulative = 0;
		do Number=1 to dim(_X);
			EigenValue = _X[number]*&nobs;
			SingularValue = sqrt(EigenValue);
			Proportion = _X[Number]/(&nobs*Total);
			Cumulative = Cumulative+Proportion;
			output;
		end;
	run;

	proc transpose data=&input_dsn._stat(where=(_TYPE_="&USCORE"))
		out=&output_V(rename=(_NAME_=variable)) name=_NAME_;
		var 
			%if ^&upred or %length(&input_vars) %then %do; 
				/*Have to use or clause because the call to svd in the DistanceScoring AP does not use PredictorVars*/
				&input_vars 
			%end; 
			%else %do; 
				&LockContPredVars
				%do j=1 %to &nix_blocks; 
					&&PredictorVars&j 
				%end;	
			%end;
		;
		id _NAME_;
		/*
		format  
			%if ^&upred or %length(&input_vars) %then %do; 
				&input_vars 
			%end; 
			%else %do; 
				&LockContPredVars
				%do j=1 %to &nix_blocks; 
					&&PredictorVars&j 
				%end;	
			%end;		
		8.6;
		*/
	run;

	* remove zero components;
	%let firstzero = 0;
	data _null_;
		set &Output_S;
		if SingularValue < 1e-8 then do;
			call symputx("firstzero",compress(_n_));
			stop;
		end;
	run;
	%put firstzero = &firstzero;
	%if &firstzero %then %do;
		%let nm1 = %eval(&firstzero-1);
		data &Output_S;
			set &Output_S(obs=&nm1);
		run;
		data &Output_V;
			set &Output_V;
			drop Prin&firstzero-Prin&n;
		run;
		data &Output_U;
			set &Output_U;
			drop Prin&firstzero-Prin&n;
		run;
		%let n = &nm1;
	%end;

	/*
	data OutLib.stat;
		set &input_dsn._stat;
	run;

	data OutLib.output_v;
		set &output_v;
	run;
	*/

	/*
	normalize vector in output_U;
	*/

	data &output_U;
		set &output_U  nobs=ntotal;
		array _denum{1} _temporary_;
		array _A{*} Prin1-Prin&n;
		if _n_=1 then _denum[1]=sqrt(ntotal);
		do _j=1 to dim(_A);
			_A[_j]=_A[_j]/_denum[1];
		end;
		keep &ID_var Prin1-Prin&n;
	run;

	/*
	normalize vector in output_V
	*/

	/*
	data OutLib.out_v0;
		set &output_v;
	run;
	*/

	options nonotes;
	%do i = 1 %to &n;
		proc sql noprint;
			select sqrt(uss(Prin&i)) into : denom&i
			from &output_v;
		quit;
		proc sql noprint;
			update &output_V
			set Prin&i = Prin&i/&&denom&i;
		quit;
	%end;

	/*
	proc sql noprint;
		select name into : var1 - : var&n
		from dictionary.columns
		where LIBNAME = upcase("&input_lib")
		and MEMNAME = upcase("&OUTPUT_V")
		and name ne 'variable';
	quit;

	options nonotes;
	%do i = 1 %to &n;
		proc sql noprint;
			select sqrt(uss(&&var&i)) into : denom&i
			from &output_v;
		quit;
		proc sql noprint;
			update &output_V
			set &&var&i = &&var&i/&&denom&i;
		quit;
	%end;
	*/

	options &shownote &showsource;
	%put Ending SVD;

%mend svd;

