/*
JMP Genomics Analytical Process

Name: Linkage Map Order for Outcross
Author: Kelci Miclaus and Luciano Silva

Description:

*/

/*
-----------------------------------------------------------
Copyright (C) 2004 SAS Institute, Inc. All rights reserved. 

Notice: 
The above copyright notice and this notice must appear in
any whole or partial copy of this code and any related
documentation. Permission to use, copy, and modify the
source code contained in this file is hereby granted, but
is limited to customers of SAS with a valid license for the
SAS software product with which this file was provided. 
Except as expressly set forth herein, the terms of such
license apply.
-----------------------------------------------------------
*/

*ProcessVariables;

%global StudyName InData IDVar LGroupVar SkOrderVar ColorVar KeepVars MarkerType WhereStmt
	LinkPhaseInData OutPath ServerOutPath
	OrderInData AnnoMName OrderGrpVar OSkOrderVar OrderVar
	SNPIndata AlleleVars LabelVar ListMarkerVars DoCellPlots
	ForceMarkersOneMap minLODOneMap maxRFOneMap BreakGrps BreakGrpsVar BreakGrpsGD BreakGrpsR UseStoppingRules OptimalityGap MaxTimeStep
	MapFunc Algorithm Threshold ShuffleOrder nShuffle ShuffleWSize ShuffleLOD OutDataPrefix LinkageMapOrderScript MacroPath
	UseMultipoint;

/*Luciano*/
%global MarkerFmt GenoDelim GenoMcode MDSSqRoot UseWeights;
%let MarkerFmt= Outcross Nondelimited Genotypes;
%let GenoDelim= ;
%let GenoMcode= ;
%let MDSSqRoot= ;
%let UseWeights= ;
/*Luciano*/

*ProcessBody;
%macro LinkageMapOrderOutcross;

	**For now, UseMultipoint option will be always YES, but in the future it might change*;
	%let UseMultipoint= Yes;

	%global FullAPName;
	%let FullAPName=%str(Linkage Map Order for Outcross);

	/*Luciano*/
	%let GenoDataObjName= genodata.obj; 
   	%let AnnoDataObjName= annodata.obj;
	%let OutCrossObjName= outcross.obj;
	%let TwoptsObjName= twopts.obj;
	%let PhasesObjName= phases.obj;
	%let CurrLGObjName= lgdata.obj;
	        
	%include "&MacroPath./OutcrossIMLModules.sas" / nosource;
        
	%let replacethis=\;
	%let replacewiththis=/;
	%let RSourcePath= %sysfunc(tranwrd(&MacroPath,&replacethis,&replacewiththis));
	%put RSourcePath= &RSourcePath;
	%let ROutPath= %sysfunc(tranwrd(&OutPath,&replacethis,&replacewiththis));
	%put ROutPath= &ROutPath;
	/*Luciano*/

	%global exiterror; 
	%let ProcessName = LinkageMapOrderOutcross; /*Luciano*/
	/*kemicl development check, change to 1 to output interim data sets*/
	%let debug = 0;
		
	%include "&MacroPath./UtilityMacros.sas" / nosource;

	%GetOrigFileName(InData,OrigInData);
	
	%PathName(&InData);
	%if &exiterror %then %do;
	   %put ERROR: The Input Data Set specification is invalid.;
	   %goto exit;
	%end;
	%let InData = &TmpName;
	libname InLib "&TmpPath";
	libname OutLib "&OutPath";

	%if %length(&OrderInData) %then %do;
		%PathName(&OrderInData);
		%if &exiterror %then %do;
			%put ERROR: The Order Data Set specification is invalid.;
			%let exiterror=1;
			%goto exit;
		%end;
		%let OrderData = &TmpName;
		libname OrderLib "&TmpPath";
	%end;

	%if %length(&SNPInData) %then %do;
		%PathName(&SNPInData);
		%if &exiterror %then %do;
			%put ERROR: The Genotype Data Set specification is invalid.;
			%let exiterror=1;
			%goto exit;
		%end;
		%let SNPData = &TmpName;
		libname SNPLib "&TmpPath";		
	%end;
	
	/*Luciano*/
	%if %length(&LinkPhaseInData) %then %do;
		%PathName(&LinkPhaseInData);
		%if &exiterror %then %do;
			%put ERROR: The Linkage Phase Data Set specification is invalid.;
			%let exiterror=1;
			%goto exit;
		%end;
		%let LinkPhaseData = &TmpName;
		libname LinLib "&TmpPath";		
	%end;	
	/*Luciano*/
	
	%put Algorithm= &Algorithm;
	/* error checking for skeleton order specifications from dialog */
	%if ^%index(%upcase(&MarkerFmt),OUTCROSS) %then %do;
		%if (%length(&SkOrderVar) or %length(&OSkOrderVar)) and (%nrbquote(%upcase(&Algorithm)) ^= MAP ORDER OPTIMIZATION) %then %do;
			%put ERROR: The Map Order Optimization Algorithm must be used when a Framework Order Variable is specified.;
			%put ERROR: Please choose this marker ordering algorithm on the Options Tab.;
			%let exiterror =1;
			%goto exit;
		%end;
	%end;
	%else %do;
		%if (%length(&OrderVar) = 0 and (%length(&SkOrderVar) or %length(&OSkOrderVar)) 
			and (^%index(%nrbquote(%upcase(&Algorithm)),AUTOMATED) and %nrbquote(%upcase(&Algorithm)) ^= MAP ORDER OPTIMIZATION))
			%then %do;
				%put ERROR: The Order Algorithm must be either Automated or Map Order Optimization when a Framework Order Variable is specified. Please choose this marker ordering algorithm on the Analysis Tab.;
				%let exiterror =1;
				%goto exit;
		%end;
		
		%if (%nrbquote(%upcase(&Algorithm))= AUTOMATED or %nrbquote(%upcase(&Algorithm))= RCD 
			or %nrbquote(%upcase(&Algorithm))=SERIATION or %nrbquote(%upcase(&Algorithm))= RECORD 
			or %nrbquote(%upcase(&Algorithm))= UG) %then %do;
			%if %length(&minLODOneMap) = 0 %then %do;
				%put ERROR: The Mininum LOD Threshold parameter in the Analysis tab must be provided. Please, return to the Analysis tab and provide a valid value for this parameter.;
				%let exiterror =1;
				%goto exit;
			%end;
			%let maxRFOneMap= 0.0; /*used solely to avoid problem with submit R*/
		%end;
		%else %if (%nrbquote(%upcase(&Algorithm))= RCD or %nrbquote(%upcase(&Algorithm))=SERIATION 
			or %nrbquote(%upcase(&Algorithm))= RECORD or %nrbquote(%upcase(&Algorithm))= UG) %then %do;
			%if %length(&maxRFOneMap) = 0 %then %do;
				%put ERROR: The Maximum Recombination Fraction Threshold parameter in the Analysis tab must be provided. Please, return to the Analysis tab and provide a valid value for this parameter.;
				%let exiterror =1;
				%goto exit;			
			%end;
		%end;
		%else %do; /*COMPARE*/
			%let minLODOneMap= 0.0; /*used solely to avoid problem with submit R*/
			%let maxRFOneMap= 0.0; /*used solely to avoid problem with submit R*/
		%end;
		
		%if ((%index(%nrbquote(%upcase(&Algorithm)),MULTIDIMENSIONAL) 
			or %nrbquote(%upcase(&Algorithm)) = MAP ORDER OPTIMIZATION
			or %index(%nrbquote(%upcase(&Algorithm)),ACCELERATED)) 
			and %upcase(&ShuffleOrder) ^= YES) %then %do;
			%put WARNING: The Shuffle Markers parameter in the Analysis tab is not checked. Please, return to the Analysis tab and check this option.;
			%*let exiterror =1;
			%*goto exit;			
		%end;
		
		%put nShuffle= &nShuffle;
		%if ((%length(&SkOrderVar) or %length(&OSkOrderVar) or %length(&OrderVar)) and %upcase(&ShuffleOrder) = YES) or %upcase(&ShuffleOrder) ^= YES %then %do;
			%let nShuffle= 0;
			%let ShuffleWSize= 0;
			%let ShuffleLOD= 0;
		%end;
		%put nShuffle= &nShuffle;
		
		%if %upcase(&ShuffleOrder) = YES and %upcase(&UseMultipoint) ^= YES %then %do;
			%put ERROR: The option to Use Multipoint Recombinatioin Estimation must be checked when the option to Shuffle Marker is checked. Please, return to the Analysis Tab and check the option Use Multipoint Recombinatioin Estimation.;
			%let exiterror = 1;
			%goto exit;
		%end;
	%end;

	%if %length(&OSkOrderVar) and ^%length(&OrderGrpVar) %then %do;
		%put ERROR:  The Linkage Group Variable and Framework Order Variable must be specified from the same data set.;
		%put ERROR:  Please specify both of these variables either from the Input Data Set, or the Order Data set.;
		%let exiterror = 1;
		%goto exit;
	%end;

	%if ^%length(&OutDataPrefix) %then %let OutDataPrefix = &OrigInData;
	%if %length(&OutDataPrefix)>27 %then %let OutDataPrefix = %substr(&OutDataPrefix,1,27);
	%let OutName = &OutDataPrefix;
	
	%if %length(&LGroupVar &OrderGrpVar)=0 %then %do;
		 %put ERROR:A Linkage Group variable must be specified either on the General tab or on the Order Data Set Tab for this process to run.;
		 %let exiterror=1;
		 %goto exit;
	%end;
	/*
	%if %length(&DistVars &ListDistVars)=0 %then %do;
		%put ERROR: Recombination Rate variables must be specified in either the Recombination Rate Variables or List-Style Specification of Recombination Rate Variables parameters.;
		%let exiterror=1;
		%goto exit;
	%end;
	*/

	%if %length(&SkOrderVar) or %length(&OSkOrderVar) %then %do;
		%let Threshold = 1;
	%end;

	%if %length(&SNPInData) %then %do;
		%if %index(%Upcase(&MarkerFmt), NONDELIMITED)>0 %then %do;
			%let code0 = %scan(&GenoMCode,1,' ')%scan(&GenoMCode,1,' ');
			%let code11 = %scan(&GenoMCode,1,' ')%scan(&GenoMCode,2,' ');
			%let code12 = %scan(&GenoMCode,2,' ')%scan(&GenoMCode,1,' ');
			%let code2 =  %scan(&GenoMCode,2,' ')%scan(&GenoMCode,2,' ');			
		%end;
		%else %if %index(%Upcase(&MarkerFmt), HOMOZYGOUS)>0 %then %do;
			%let code0 = %scan(&GenoMCode,1,' ');
			%let code11 = %scan(&GenoMCode,1,' ')%scan(&GenoMCode,2,' ');  /* AB coded */
			%let code12 = H;  /* H coded*/
			%let code2 =  %scan(&GenoMCode,2,' ');
		%end;
		%else %if %Upcase(&MarkerFmt) = GENOTYPES %then %do; /* delimited */
			%let code0 = %scan(&GenoMCode,1,' ')&GenoDelim%scan(&GenoMCode,1,' ');
			%let code11 = %scan(&GenoMCode,1,' ')&GenoDelim%scan(&GenoMCode,2,' ');
			%let code12 =  %scan(&GenoMCode,2,' ')&GenoDelim%scan(&GenoMCode,1,' ');
			%let code2 =  %scan(&GenoMCode,2,' ')&GenoDelim%scan(&GenoMCode,2,' ');
		%end;
		%else %do;
			%let code0 = 0; 
			%let code11 = 1;
			%let code12 = 1;
			%let code2 = 2;
		%end;
		%put Genotype Codes: &code0, &code11, &code12, &code2;
		
		/*Kemicl set marker format to numeric for launching subset reorder process */
		%if %index(&MarkerFmt,Homozygous) %then %let MarkerFmt = Numeric Genotypes;
	%end;
	
	/*Luciano*/	
	%if ^%index(%nrbquote(%upcase(&Algorithm)),AUTOMATED) and %upcase(&ForceMarkersOneMap)= YES %then %do;
		%put ERROR: The option to force all markers into linkage groups is not available when the Order Algorithm is not Automated.;
		%let exiterror = 1;
		%goto exit;
	%end;
	/*Luciano*/

	/*set up vars for breaking further linkage groups */
	%if %index(%upcase(&BreakGrpsVar),GENETIC) %then %do;
		%let BreakGrpsVar = cM_Step;
		%let BreakGrpsValue = &BreakGrpsGD;
		%let BGimlprefix = cm;
	%end;
	%else %do;
		%let BreakGrpsVar = RecombinationF;
		%let BreakGrpsValue = &BreakGrpsR;
		%let BGimlprefix = rf;
	%end;
	
	%GLOBAL LinkageMapOrder_OutData LinkageMapOrder_OutDataRes LinkageMapOrder_OutDataEx;
	%let LinkageMapOrder_OutData = ;
	%let LinkageMapOrder_OutDataRes = ;
	%let LinkageMapOrder_OutDataEx = ;
	/* Prep output data linkage map data set and residuals dataset */
	%PrepOutDS(LinkageMapOrder_OutData,LinkageMapOrder_OutDataName,&OutDataPrefix,_lmo);
	%PrepOutDS(LinkageMapOrder_OutDataRes,LinkageMapOrder_OutDataResName,&OutDataPrefix,_lmr);
	%PrepOutDS(LinkageMapOrder_OutDataEx,LinkageMapOrder_OutDataExName,&OutDataPrefix,_eml);
	
	%if %sysfunc(exist(OutLib.&LinkageMapOrder_OutDataExName)) %then %do;
		proc datasets library=OutLib nolist;
			delete &LinkageMapOrder_OutDataExName;
		quit;
	%end;
	
	/*Check if the SAS/OR component is installed if Algorithm is MOO
	%if %index(%nrbquote(%upcase(&Algorithm)),MULTIDIMENSIONAL)=0 and  %length(&OrderVar)=0 %then %do; 
		data _null_;
			x = sysprod('or');
			call symput('CheckOR',trim(left(x)));
		run;
		%if &CheckOR %then %do;
			%put SAS/OR is licensed, Map Order Optimization will be used.;
		%end;
		%else %do;
			%put ERROR: The SAS/OR component is not licensed and must be purchased separately from JMP Genomics.  Map Order Optimization Cannot Be Used.; 
			%let exiterror =1;
			%goto exit;
		%end;
	%end;
	 */
	 
	/* check that the recombination matrix is square */
	/*
	data sqrchk; set InLib.&InData;
		keep &DistVars &ListDistVars;
	run;
	
	%nobsvars(sqrchk);
	%if &nobs ne &nvars %then %do;
		%put ERROR: The matrix of recombination fractions is not a square matrix of values.;
		%put ERROR: There are &nobs marker rows and &nvars marker columns, which should be equal values. Please check the input data.;
		%let exiterror =1;
		%goto exit;
	%end;
	
	* Check first 100 ID variable names to match to Column names *;
	* Get RF variable names to put in a list to check column with values matching variable names *;
	data mnames(keep= &DistVars &ListDistVars); 
		set InLib.&InData;
		if _n_ = 1 then output mnames;
	run;
	
	%let _tid_ = %sysfunc(open(mnames));
	%let _nVars_ = %sysfunc(attrn(&_tid_,NVARS));
	
	%let allmatch = 1;
	
	data markerlist;
		length _IDVAR_ $32; 	
		%do i = 1 %to %sysfunc(min(100,&_nVars_));
			_IDVAR_ = "%sysfunc(varname(&_tid_,&i))";
			output;
		%end;
	run;
	data _null_; 
		set InLib.&InData(keep = &IDVar); set markerlist;
		if _n_ <= 100;
		if upcase(strip(&IDVar)) ne upcase(strip(_IDVAR_)) then do;
			call symputx("allmatch",0);
		end;
	run;
	
	%if &allmatch = 0 %then %do;
		%put ERROR: The values of the Marker ID variable specified does not match the values of the Recombination Fraction column names.;
		%put ERROR: An ID Variable called _IDVAR_ will be created and used that assumes the rows are in the same order as the columns.;
		*Create IDvar from recomb freq variable names to merge into data *;
		data markerlist;
			length _IDVAR_ $32; 	
			%do i = 1 %to &_nVars_;
				_IDVAR_ = "%sysfunc(varname(&_tid_,&i))";
				output;
			%end;
		run;
		%let IDVar = _IDVAR_;
	%end;
	* Close data set after getting variable names *;
	 %let rc = %sysfunc(close(&_tid_));
	*/
		
	/*Get length of IDVar to use in the proc append statement*/
	data _tmp_; 
		set InLib.&InData(keep = &IDVar); 
		where 0;
	run;
	%let _tid_ = %sysfunc(open(_tmp_));
	%let _IDvarLength_ = %sysfunc(varlen(&_tid_,1));
	%let _IDvarType_= %sysfunc(vartype(&_tid_,1));
	%let _tid_ = %sysfunc(close(&_tid_));
	
	%put _IDvarLength_= &_IDvarLength_;
	%put _IDvarType_= &_IDvarType_;
	
	/*Luciano*/
	%if %index(%upcase(&MarkerFmt),OUTCROSS) %then %do;
	
		%let allmatch = 1;
		
		*Get names of markers in the IDVar from InData*;
		proc transpose data=InLib.&InData out=indatamnames(drop=_NAME_);
			var &IDVar;
			id &IDVar;
		run;

		%let idvarsmnames= ;
		%let _tid_ = %sysfunc(open(indatamnames));
		%let _n_IDVars_ = %sysfunc(attrn(&_tid_,NVARS));
		%do i=1 %to &_n_IDVars_;
			%let idvarsmnames= &idvarsmnames %sysfunc(varname(&_tid_, &i)); 
		%end;
		%let rc = %sysfunc(close(&_tid_));
		
		*Get number of matching marker names between SNPData and InData sets*;
		data snpmnames gendat;
			set SNPLib.&SNPData;
				if _n_= 1 then output snpmnames;
				output gendat;
				keep &idvarsmnames;
		run;
		%let _tid_= %sysfunc(open(snpmnames));
		%let _n_SNPVars_= %sysfunc(attrn(&_tid_, nvars));
		%let rc = %sysfunc(close(&_tid_));	
		
		%if %sysevalf(&_n_SNPVars_ ~= &_n_IDVars_) %then %do;
			%put ERROR: The values of the Marker ID variable specified does not match marker variable names in the Genotype Data Set;
			%put Please check that the marker variable names in the Genotype Data Set exactly matches the name of markers specified by the Marker ID variable from the Input Data Set columns.;		
			%exiterror=1;
			%goto exit;
		%end;
		
		/*
		* Check first 100 ID variable names to match to Column names in the SNPInData*;	
		* Get Marker variable names to put in a list to check column with values matching variable names *;		
		data mnames gendat;
			set SNPLib.&SNPData;
				if _n_= 1 then output mnames;
				output gendat;
				keep &AlleleVars &ListMarkerVars;
		run;

		%let _tid_ = %sysfunc(open(mnames));
		%let _nVars_ = %sysfunc(attrn(&_tid_,NVARS));

		%let allmatch = 1;

		data markerlist;
			length _IDVAR_ $32; 	
			%do i = 1 %to %sysfunc(min(100,&_nVars_));
				_IDVAR_ = "%sysfunc(varname(&_tid_,&i))";
				output;
			%end;
		run;
		data _null_; 
			set InLib.&InData(keep = &IDVar); set markerlist;
			if _n_ <= 100;
			if upcase(strip(&IDVar)) ne upcase(strip(_IDVAR_)) then do;
				call symputx("allmatch",0);
			end;
		run;

		%if &allmatch = 0 %then %do;
			%put ERROR: The values of the Marker ID variable specified does not match the genotype data set according to the variable selected as Marker Genotype Variable.;
			%put Please check that the Marker Genotype exactly matches the input data columns or run the Subset and Reorder Genetic Data AP to order these data sets correctly.;

			* create JSL file;
			%global SRButtonsScript;
			%PrepOutFile(SRButtonsScript,SRButtonsScriptName,launchAP_buttons_SR.jsl);
			filename jslbtns "&SRButtonsScript";

			options noquotelenmax;
			data _null_;
				file jslbtns lrecl=32767;
				put "//!";
				put "Names Default to Here(1);";
				put "wait(0.5);";
				put "if(host is(Windows),";
				put "	include( ConvertFilePath(""$LS_HOME/Generator/DialogHelperFunctions.jsl"", absolute, windows) ),";
				put "	include( ConvertFilePath(""$LS_HOME/Generator/DialogHelperFunctions.jsl"", absolute, POSIX) ));";
				put "	";
				put "srbuttonswindow=New Window(""Launch AP to Reorder Markers"",";
				put "		VListBox(";
				put "			TextBox(""Click on the button below to launch the Subset and Reorder Genetic Data process to reorder marker columns properly.""),";
				put "			TextBox(""Use the output data sets from that process as the new input and annotation data sets to .""),";

				put "			TextBox("" ""),";
				put "			ButtonBox(""Subset and Reorder Genetic Data"",";
				put "				if(host is(Windows),";
				put "					Include(ConvertFilePath(""$HOME/JMPG/ProcessLibrary/SubsetandReorderGeneticData.jsl"", absolute, windows)),";
				put "					Include(ConvertFilePath(""$HOME/JMPG/ProcessLibrary/SubsetandReorderGeneticData.jsl"", absolute, POSIX) ));";
				put "	";

				* Set input data set and Output Folder *;
				put "				jg:setDefaultParmValues(2, ""SubsetandReorderGeneticData"");";				
				put "				jg:setCurrentParmValue(2, ""SubsetandReorderGeneticData"", jg:getParmNum(""SubsetandReorderGeneticData"",""InData""),""%sysfunc(pathname(SNPLib))\&SNPData..sas7bdat"");";
				put "				jg:SubsetandReorderGeneticData_updateDeps;";
				put "				jg:setCurrentParmValue(2, ""SubsetandReorderGeneticData"", jg:getParmNum(""SubsetandReorderGeneticData"",""ListMarkerVars""),""&AlleleVars"");";	
				put "				jg:setCurrentParmValue(2, ""SubsetandReorderGeneticData"", jg:getParmNum(""SubsetandReorderGeneticData"",""MarkerFmt""),""Outcross Nondelimited Genotypes"");";
				put "				jg:setCurrentParmValue(2, ""SubsetandReorderGeneticData"", jg:getParmNum(""SubsetandReorderGeneticData"",""MinPercent""),""0"");";		
				put "				jg:setCurrentParmValue(2, ""SubsetandReorderGeneticData"", jg:getParmNum(""SubsetandReorderGeneticData"",""OutPath""),""&ClientOutPath"");";					
				put "				jg:setCurrentParmValue(2, ""SubsetandReorderGeneticData"", jg:getParmNum(""SubsetandReorderGeneticData"",""AnnoInData""),""%sysfunc(pathname(InLib))\&InData..sas7bdat"");";					
				put "				jg:SubsetandReorderGeneticData_updateDeps;";				
				put "				jg:setCurrentParmValue(2, ""SubsetandReorderGeneticData"", jg:getParmNum(""SubsetandReorderGeneticData"",""AnnoMName""),""&IDVar"");";	
				put "			)";;

				put "		)";
				put ");";
			run;

			%PackageInsertFile(jsl code from &ProcessName, &OutPath, &SRButtonsScriptName, , text/plain, yes);		

			* Close data set after getting variable names *;
			%let rc = %sysfunc(close(&_tid_));

			%let exiterror=1;
			%goto exit;

		%end;

		* Close data set after getting variable names *;
		%let rc = %sysfunc(close(&_tid_));
		*/

		/* Check that the Linkage Phase data set has the required columns names*/
		data colnames; set LinLib.&LinkPhaseData;
			where 0;
			keep Marker1 Marker2 rf1 rf2 rf3 rf4 lod1 lod2 lod3 lod4;
		run;

		%let _tid_ = %sysfunc(open(colnames));
		%let _nColVars_ = %sysfunc(attrn(&_tid_,NVARS));
		%let rc = %sysfunc(close(&_tid_));

		%if &_nColVars_ < 10 %then %do;
			%put ERROR: The Linkage Phase data set does not have the required variables: Marker1, Marker2, rf1, rf2, rf3, and rf4. Please check the input data.;
			%let exiterror = 1;
			%goto exit;
		%end;
		
		* Check that the Linkage Phase and InData data sets match by marker names of IDVar from InData*;					
		data phasedata;
			set LinLib.&LinkPhaseData(keep= Marker1 Marker2 rf1 rf2 rf3 rf4 lod1 lod2 lod3 lod4);
			_n = 1;
		run;
		
		data pdmnames;
			set phasedata(in=orig1) phasedata(in=orig2);
			if orig1= 1 then _mkname_= marker1;
			if orig2= 1 then _mkname_= marker2;
			keep _mkname_;
		run;

		proc sort data=pdmnames nodupkey;
			by _mkname_;
		run;
		
		data pdmnames;
			set pdmnames;
			_seq_= _n_;
		run;
	
		proc sort data=InLib.&InData out=indataidvar(keep=&IDVar);
			by &IDVar;
		run;
		
		data indataidvar;
			set indataidvar;
			_seq_= _n_;
		run;
		
		%if &debug %then %do;
			data outlib.pdmnames;
				set pdmnames;
			run;
			
			data outlib.indataidvar;
				set indataidvar;
			run;
		%end;
		
		%let allmatchPDInData= 1;
		data _null_;
			merge pdmnames indataidvar;
			by _seq_;
			if _mkname_ ~= &IDVar then
				call symputx("allmatchPDInData", 0);
		run;
		
		%if %sysevalf(&allmatchPDInData < 1) %then %do;
			%put ERROR: The values of the Marker ID variable specified does not match marker variable names in the Linkage Phase Data Set;
			%put Please check that the marker variable names in the Linkage Phase Data Set exactly matches the name of markers specified by the Marker ID variable from the Input Data Set columns.;		
			%exiterror=1;
			%goto exit;		
		
		%end;
		
		data mnames_mtypes;
			set InLib.&InData;
			keep &IDVar &MarkerType;
		run;

		data names_merge; 
			set mnames_mtypes;
			row = _n_;
		run;

		proc sql;
			create table stackedmk as
			select f1._MarkerName_1, f1.MarkerType1_, f1.row1, f2._MarkerName_2, f2.MarkerType2_, f2.row2
			from names_merge(rename = (&IDVar=_MarkerName_1 &MarkerType=MarkerType1_ row=row1 )) as f1 
			cross join names_merge(rename = (&IDVar=_MarkerName_2 &MarkerType=MarkerType2_ row=row2)) as f2
			where f1.row1 < f2.row2
			order by f2.row2, f1.row1
		;
		quit;
		
		/*data outlib.stackedmk; set stackedmk;run;*/

		proc sql;
			create table phasedata_tmp as
			select a2._MarkerName_1, a2.MarkerType1_, a2._MarkerName_2, a2.MarkerType2_, a1.*
			from phasedata as a1, stackedmk as a2  
			where a1.Marker1=a2._MarkerName_1 and a1.Marker2=a2._MarkerName_2
			order by a2.row2, a2.row1
			;
		quit;
		
		%nobsvars(InLib.&InData);
		%let ncomb = %sysevalf(&nobs/2 * (&nobs - 1));
			
		%nobsvars(phasedata_tmp);
		%let ncombph = &nobs;
		%put ncombph = &ncombph;
		%if %sysevalf(&ncombph ~= &ncomb) %then %do;
			%let flagphase= 1;
			
			proc datasets nolist;
				delete phasedata_tmp;
			quit;
			
			proc sql;
				create table phasedata_tmp(rename=(MarkerType2_old=MarkerType1_ MarkerType1_old=MarkerType2_)) as
				select a2._MarkerName_1, a2.MarkerType1_old, a2._MarkerName_2, a2.MarkerType2_old, a1.*
				from phasedata as a1, stackedmk(rename= (MarkerType1_=MarkerType1_old MarkerType2_=MarkerType2_old)) as a2  
				where a1.Marker1=a2._MarkerName_2 and a1.Marker2=a2._MarkerName_1
				order by a2.row2, a2.row1
				;
			quit;	
			
			%nobsvars(phasedata_tmp);
			%let ncombph = &nobs;
			%put ncombph = &ncombph;
			%if %sysevalf(&ncombph ~= &ncomb) %then %do;
				%put ERROR: The Linkage Phase data set is incomplete. There are some missing pairwise marker combination. It was expected &ncomb unique pairwise marker combinations, however, it was found only &ncombph of them. Note that two arbitrary marker pairs, for instance, (m1,m2) and (m2,m1) represent only one unique marker pair and are counted as one pair. Please check the input data set.;
				%let exiterror= 1;
				%goto exit;	
			%end;
		%end;
			
		/*data outlib.phase_tmp;set phasedata_tmp;run;*/

		* Check that the Linkage Phase data set has no missing values*;
		%let phaseanymissing = 0;
		data phasedata_tmp(drop=k _MarkerName_1 _MarkerName_2);
			set phasedata_tmp;
			select (trim(left(strip(MarkerType1_))));
				when('A.1','A.2','A.3','A.4') MType1Num_= 1;
				when("B1.5") MType1Num_= 2;
				when("B2.6") MType1Num_= 3;
				when("B3.7") MType1Num_= 4;
				when("C.8") MType1Num_= 5;
				when("D1.9","D1.10","D1.11","D1.12","D1.13") MType1Num_= 6;
				when("D2.14","D2.15","D2.16","D2.17","D2.18") MType1Num_= 7;				
				otherwise MType1Num_= .;
			end;
			select (trim(left(strip(MarkerType2_))));
				when("A.1","A.2","A.3","A.4") MType2Num_= 1;
				when("B1.5") MType2Num_= 2;
				when("B2.6") MType2Num_= 3;
				when("B3.7") MType2Num_= 4;
				when("C.8") MType2Num_= 5;
				when("D1.9","D1.10","D1.11","D1.12","D1.13") MType2Num_= 6;
				when("D2.14","D2.15","D2.16","D2.17","D2.18") MType2Num_= 7;				
				otherwise MType2Num_= .;
			end;
			array r{*} rf1-rf4;
			array lod{*} lod1-lod4;
			do k=1 to dim(r);
				if MType1Num_= 6 and MType2Num_= 7 or MType1Num_= 7 and MType2Num_= 6 then do; 
					r{k} = .; ** 0.25 will be assigned within the R code for usage in the multipoint estimation*;
					lod{k} = .; ** 0.00 will be assigned within the R code for usage in the multipoint estimation*;
					/*
					r{k} = .25; **this value will be used in the multipoint estimation*;
					lod{k} = 0; **this value will be used in the multipoint estimation*;
					if missing(r{k}) or missing(lod{k}) then
						call symputx('phaseanymissing', 1, 'L');
					*/
				end;
				else do;
					if missing(r{k}) or missing(lod{k}) then
						call symputx('phaseanymissing', 1, 'L');
				end;
			end;					
		run;
		
		%if &phaseanymissing %then %do;
			%put ERROR: The Linkage Phase data set has missing values. Please check the input data set.;
			%let exiterror= 1;
			%goto exit;
		%end;
			
		proc datasets nolist;
			delete phasedata;
			change phasedata_tmp= phasedata;
		quit;
			
		%if %nrbquote(%upcase(&Algorithm)) = AUTOMATED %then
			%let maxRFOneMap= .5;		
	
	%end;
	/*Luciano*/
		
	%if %length(&OrderVar &OrderGrpVar &OSkOrderVar)>0 %then %do;
		/* Use Order var and or Order group var from Order data set */
		/* Also use Skeleton order varo and Order Group var if both specified on Order data set*/
	
		data indat; set InLib.&InData;
			_Ord_ = _n_;
			%if %length(&OrderVar) %then drop &OrderVar;;
			%if %length(&OrderGrpVar) %then drop &OrderGrpVar;;
			%if %length(&OSkOrdervar) %then drop &OSkOrderVar;;
		run;
		proc sql UNDO_POLICY=NONE; 
			create table indat as 
				select *
				from indat as d1 join OrderLib.&OrderData(keep = &OrderVar &OrderGrpVar &OSkOrderVar &AnnoMName) as d2
				on d1.&IDVar = d2.&AnnoMName;
		quit;
		/* kemicl - get number of markers in indata and  after merge in Order data */
		%nobsvars(InLib.&InData);
		%let nm_in = &nobs;
		%nobsvars(indat);
		%let nm_out = &nobs;
		/* If nobs doesn't match throw error and exit */
		%if &nm_in ne &nm_out %then %do;
			%put ERROR: Merging in the Order Data Set resulted in only &nm_out markers when the Input Data Set had &nm_in markers.;
			%put ERROR: Please check that the values of the Marker ID Merge Variable in the Order Data Set exactly matches the values of the Marker ID Variable in the Input Data Set.;
			%let exiterror = 1;
			%goto exit;
		%end;	
		
		proc sort data=indat out=indat(drop = _ord_);
			by _ord_;
		run;
	
		%if &syserr>4 %then %do;
			%put ERROR: Merging in the Order Data Set produced errors.  Please check that the Marker Merge Variable matches the Input Data Set ID Variable.;
			%let exiterror = 1;
			%goto exit;
		%end;
		%let indat = indat;
		/* use Order Group Var instead of Linkage Group var put on input data set */
		%if %length(&OrderGrpVar) %then %let LGroupVar = &OrderGrpVar;
		/* use Skeleton Order Var from Order data set instead of any SkOrderVar specified on input */
		%if %length(&OSkOrderVar) %then %let SkOrderVar = &OSkOrderVar;
	%end;
	%else %do;
		/*Luciano*/
		*needed because missing recombintion values must be replaced only in the working file*;
		data indat;
			set InLib.&InData;
		run;
		/*Luciano*/
		
		%let indat = indat;
	%end;
		
	%*goto exitcheck;
	
	*begin: Luciano, added a general where statement to filter out observations from the Input Data Set*;
	%put WhereStmt= &WhereStmt;
	%if %length(&WhereStmt) %then %do;
		/*
		%if %upcase(&AddDroppedMarkers)= YES %then %do;

			data Orig_Indat;
				set &indat;
				*_order_= _n_;
			run;

		%end;
		%put WhereStmt= &WhereStmt;
		*/

		*subseting indat*;
		data KeepIndat;
			retain _countKeep_ _countDrop_ 0;
			set &indat end=_eof_;
			if &WhereStmt then do;
				output KeepIndat;
				_countDrop_ + 0;
				_countKeep_ + 1;
				call symputx("KeepMK"||trim(left(_countKeep_)), trim(left(&IDVar)));
			end;
			else do;
				_countKeep_ + 0;
				_countDrop_ + 1;
				call symputx("DropMK"||trim(left(_countDrop_)), trim(left(&IDVar)));		
			end;
			if _eof_ then do;
				call symputx("TotKeepMK", trim(left(_countKeep_)));
				call symputx("TotDropMK", trim(left(_countDrop_)));
			end;
			drop _countKeep_ _countDrop_;
		run;
		%put DropMK1= &DropMK1;
		%put TotDropMK= &TotDropMK;
		%if &debug %then %do;
			data outlib.Keepindat;set Keepindat;run;
		%end;

		%NObsVars(&indat);
		%let nobs_in= &nobs;
		%NObsVars(KeepIndat);
		%let nobs_out= &nobs;

		%if &nobs_in = &nobs_out %then %do;
			%put ERROR: No observation was removed from the Input Data Set when the Where Statement specified in the General Tab was applyied to this data set. Please return to the General Tab and change the Where Statement that is being specified.; 
			%let exiterror=1;
			%goto exit;
		%end;

		%if &nobs_out < 2 %then %do;
			%put ERROR: &nobs observations left in the Input Data Set after applying the Where Statement specified in the General Tab. At least 2 observations must remain in the Input Data Set after applying the Where Statement.; 
			%let exiterror=1;
			%goto exit;
		%end;

		*dropping unwanted columns*;
		data KeepIndat;
			set KeepIndat;
			drop 
			%do i=1 %to &TotDropMK;
				&&DropMK&i
			%end;
			;
		run;
		%let indat= KeepIndat;

		%*let OrigListDistVars= &ListDistVars;
		%*let OrigDistVars= &DistVars;

		%let ListDistVars= ;
		%let DistVars= ;
		%do i=1 %to &TotKeepMK;
			%let ListDistVars= &ListDistVars &&KeepMK&i;
		%end;		
		
		/*
		%if %upcase(&AddDroppedMarkers)= YES %then %do;
			*checking if clusters are nested withing linkage groups*;
			proc sort data=&indat out=LGByCluster nodupkey;
				by &LGroupVar &ClusterVar;
			run;

			%let notnestedlg= 0;
			%let notnestedcl= ;
			data _null_;
				set LGByCluster;
				retain clus_;
				if &ClusterVar= clus_ then do;
					call symputx("notnestedlg", trim(left(1)));
					call symputx("notnestedcl", trim(left(&ClusterVar)));
				end;
				clus_ = &ClusterVar;
			run;

			%if &notnestedlg %then %do;
				%put ERROR: Cluster &notnestedcl appears in more than one linkage group.;
				%let exiterror=1;
				%goto exit;
			%end;
		%end;
		*/
		
		data gendat;
			set gendat;
			keep &ListDistVars;
		run;
		
		data mnames_mtypes;
			set &indat;
			keep &IDVar &MarkerType;
		run;

		data names_merge; 
			set mnames_mtypes;
			row = _n_;
		run;

		proc sql;
			create table stackedmk as
			select f1._MarkerName_1, f1.MarkerType1_, f1.row1, f2._MarkerName_2, f2.MarkerType2_, f2.row2
			from names_merge(rename = (&IDVar=_MarkerName_1 &MarkerType=MarkerType1_ row=row1 )) as f1 
			cross join names_merge(rename = (&IDVar=_MarkerName_2 &MarkerType=MarkerType2_ row=row2)) as f2
			where f1.row1 < f2.row2
			order by f2.row2, f1.row1
		;
		quit;

		proc sql;
			create table phasedata_tmp(drop= _MarkerName_1 _MarkerName_2) as
			select a2._MarkerName_1, a2.MarkerType1_, a2._MarkerName_2, a2.MarkerType2_, a1.*
			from phasedata(drop=MarkerType1_ MarkerType2_) as a1, stackedmk as a2  
			where a1.Marker1=a2._MarkerName_1 and a1.Marker2=a2._MarkerName_2
			order by a2.row2, a2.row1
			;
		quit;
		
		proc datasets nolist;
			delete phasedata;
			change phasedata_tmp= phasedata;
		run;
		
	%end;
	*end: Luciano, added a general where statement to filter out observations from the Input Data Set*;
	
	%if %index(%upcase(&MarkerFmt),OUTCROSS) %then %do;
			
		proc freq data=&indat noprint;
			table &LGroupVar / out=fr;
		run;

		proc sort data= fr;
			by descending count;
		run;
	
		data _null_;
			set fr (obs=1);
			call symputx("maxMKbyLG", trim(left(count)));
		run;
	
		/*S1070935; linkage Map order outcross: error when use compare order algorithm */
		%if %sysevalf(%index(%nrbquote(%upcase(&Algorithm)),COMPARE) and &maxMKbyLG > 10) %then %do;
			%global LinkageMapOrder_ScriptWarn;
		    	
			%PrepOutFile(LinkageMapOrder_ScriptWarn,LinkageMapOrder_ScriptWarnName,saswarning.jsl);
			
			data _null_;
				file "&LinkageMapOrder_ScriptWarn" lrecl=32767 ;
				put '//!' ;
				put "Names Default to Here(1);" ;
				put "new Window(""WARNING running &FullAPName.""," ;
				put "vlb=VListBox(" ; 
				put "textBox(""The following warning ocurred in process &FullAPName:""),";
				put "errtxt = TextEditBox(""\!N""||""";
				put "There are more than 10 markers to be ordered by the Compare algorithm. For m markers, the Compare algorithm compares all m!/2 possible orders.";
				put "Therefore, this process may take a very long time to be finished. Please, choose another Order Algorithm in the Analysis tab."; 
				put """||""\!N\!N"",WhiteBoxStyle(0)),";
				put "errtxt << Font Color(3)," ; 
				put "errtxt << Set Wrap(1000)," ;
				/*
				put "TextBox(""Please refer to the SAS log displayed in the JMP log window for further details.\!N\!N""),";
				*/
				put ")" ;
				put ");";
			run;
		     
			%PackageInsertFile(jsl code from &ProcessName, &OutPath, &LinkageMapOrder_ScriptWarnName, no, no, yes);
			%let exiterror=0;
			%goto exit;
		%end;
	%end;
	
	/* if a skeleton order var exists, create a data set for subsetting for each linkage group to get any anchored marker positions*/
	/* in Linkage Group loop, will subset and check if any markers in curr lgroup have skeleton positions to use ordinally in TSP */
	%if %length(&SkOrderVar) %then %do;
		data skeleton; 
			set &indat;
			where not missing(&SkOrderVar);
			keep &IDVar &LGroupVar &SkOrderVar; 
		run;
	%end;
	
	%if %length(&SkOrderVar) or %length(&OrderVar) %then %do;
		data tmp; 
			set &indat;
			where 0;
		run;
		
		%let dsid = %sysfunc(open(tmp));
		%if %length(&SkOrderVar) %then %do;
			%let SkPos=%sysfunc(varnum(&dsid,&SkOrderVar));
			%let SkType = %sysfunc(vartype(&dsid, &SkPos));
			%if &SkType = C %then %do;
				%put ERROR: The Framework Order Variable must be numeric.;
				%let exiterror = 1;
				%goto exit;
			%end;
		%end;
		%if %length(&OrderVar) %then %do;
			%let OPos = %sysfunc(varnum(&dsid,&OrderVar));
			%Let OType = %sysfunc(vartype(&dsid, &OPos));
			%if &OType = C %then %do;
				%put ERROR: The Marker Order Variable must be numeric.;
				%let exiterror = 1;
				%goto exit;
			%end;		
		%end;
		%let rc = %sysfunc(close(&dsid));
	%end;
	
	/* check type and Length of Linkage Group Variable and replace missing with -99 if numeric and MISSING if character */
	data tmp; set &indat;
		where 0;
		keep &LGroupVar;
	run;
	
	%let dsid = %sysfunc(open(tmp));
	%let LGType = %sysfunc(vartype(&dsid, 1));
	%let LGLen = %sysfunc(varlen(&dsid,1));
	%let rc = %sysfunc(close(&dsid));
	
	/*Luciano*/
	%if %length(&MarkerType) %then %do;
		data tmp; set &indat;
			where 0;
			keep &MarkerType;
		run;
		%let dsid = %sysfunc(open(tmp));
		%let MkTypeType = %sysfunc(vartype(&dsid, 1));
		%let MkTypeLen = %sysfunc(varlen(&dsid,1));
		%let rc = %sysfunc(close(&dsid));
	%end;
	/*Luciano*/

	%if &LGType = C %then %do;
		%let LGLen = %sysfunc(max(&LGLen,7));
		%let LGLen = %eval(&LGLen +3);
	%end;
	
	data &indat; 
		%if &LGType = C %then %do;
	 		length &LGroupVar $&LGLen;
	 	%end;
	 	%if %length(&MarkerType) %then %do;
	 		length &&MarkerType $&MkTypeLen;
	 	%end;
		%if &allmatch = 0 %then %do; merge MarkerList &indat; %end; %else %do; set &indat; %end;
	 	%if &LGType = C %then %do;
			if missing(&LGroupVar) then &LGroupVar = "MISSING";
		%end;
		%else %do;
			if missing(&LGroupVar) then &LGroupVar = -99;
		%end;
	run;
	
	*data indat(keep =&LGroupVar &IDVar &DistVars &ListDistVars)  copydat(keep = &LGroupVar &IDVar &KeepVars &SkOrderVar &ColorVar orig_order &OrderVar &AnnoMName);
	data &indat(keep =&LGroupVar &IDVar  
			%if %index(%upcase(&MarkerFmt),OUTCROSS) %then %do;
				&MarkerType
				%if %length(&SkOrderVar) %then &SkOrderVar;
				%if %length(&OrderVar) %then &OrderVar;
			%end;
			%else %do;
				 &DistVars &ListDistVars
			%end;
			)  
		copydat(keep = &LGroupVar &IDVar &KeepVars &SkOrderVar &ColorVar orig_order &OrderVar &AnnoMName
			%if %length(&MarkerType) %then &MarkerType;) ; /*Luciano*/
		set &indat; 
		orig_order = _n_;
	run;
	
	%if %length(&SkOrderVar) %then %do;
		data skeleton;
			%if &LGType = C %then %do;
				 length &LGroupVar $&LGLen;
			%end;
			set skeleton;
		run;
	%end;

	%let runold= 0;
	%if &runold %then %do;
		/*Get Levels of the Linkage Group Var (in case it's been recoded) */
		/* Loop over linkage groups and create data sets for each */

		%let nGrps = ;
		%*GetLevels(_Ds =indat,_Var=&LGroupVar,_LPrefix = LGrp,_nLvls=nGrps,_vType= LGType);
		%GetLevels(_Ds =&indat,_Var=&LGroupVar,_LPrefix = LGrp,_nLvls=nGrps,_vType= LGType);
		%put There are &nGrps Linkage Groups for ordering;
		%put The linkage group variable is of type &LGType;
		%if &syserr>4 %then %goto exit;
	%end;
	
	/* Set up stored macro library if using Map Order Optimization*/ 
	%put Algorithm= &Algorithm;
	%if %index(%nrbquote(%upcase(&Algorithm)),OPTIMIZATION)>0 %then %do;
		options mstored sasmstore=orlib;
		%if %length(%sysfunc(pathname(orlib))) = 0 %then %do;
			libname orlib "&MacroPath.";
		%end;
	 	/* define macros for solving for optimal order in PROC OPTMODEL macros */
		%Global exiterror relobjgap1 relobjgap2 maxtime;
		%let relobjgap1 = ;
		%let relobjgap2 = ;
		%let maxtime = ;
		%if %upcase(&UseStoppingRules) = YES %then %do;
			/* convert time to seconds */
			%let MaxTimeStep = %sysevalf(&MaxTimeStep*60);
			%let OptGap1 = %sysevalf(&OptimalityGap*2);
			%let relobjgap1 = relobjgap=&OptGap1;
			%let relobjgap2 = relobjgap=&OptimalityGap;
			%let maxtime = maxtime=&MaxTimeStep;
			%put The Algorithm will solve the marker order within &OptimalityGap of optimality with max time of &MaxTimeStep seconds per MILP solve.;
		%end;
		%macro postSolve;
			if _solution_status_ = 'INFEASIBLE' then do;
				call symputx('exiterror', 1);
				put "Problem is infeasible.  Exiting...";
				leave;
			end;
		%mend postSolve;
	
		%macro findConnectedComponents;
			/* find connected components of support graph */
			EDGES1 = {<i,j> in EDGES: round(x[i,j].sol) = 1};
			EDGES1 = EDGES1 union {setof {<i,j> in EDGES1} <j,i>};
			VERTICES1 = INITVERTICES;
			CLOSURE = EDGES1;
			for {i in INITVERTICES} component[i] = 0;
			for {i in VERTICES1} do;
				NEIGHBORS = slice(<i,*>,CLOSURE);
				CLOSURE = CLOSURE union (NEIGHBORS cross NEIGHBORS);
			end;
			numcomp = 0;
			do while (card(VERTICES1) > 0);
				numcomp = numcomp + 1;
				for {i in VERTICES1} do;
					NEIGHBORS = slice(<i,*>,CLOSURE);
					for {j in NEIGHBORS} component[j] = numcomp;
					VERTICES1 = VERTICES1 diff NEIGHBORS;
					leave;
				end;
			end;
			put numcomp=;
			put;
			if numcomp > 1 then do;
				numsubtour = numsubtour + numcomp;
				for {comp in 1..numcomp} do;
					SUBTOUR[numsubtour-numcomp+comp]
					= {i in VERTICES: component[i] = comp};
				end;
			end;
		%mend findConnectedComponents;
		
		/* rowGenerationLoop is used when no skeleton marker exists */
		%macro rowGenerationLoop(simple=);
			 %if &simple = 1 %then %do;
				  /* generate constraints dynamically as needed: simple version */
				  do until (numcomp = 1);
					iter = iter + 1;
					put iter=;
						/* solve with current set of constraints */
						put "Solving to optimality...";
						solve with milp %if %upcase(&UseStoppingRules) = YES %then  / &relobjgap2 &maxtime;;
					%postSolve;
						%findConnectedComponents;
						/* can use optimal objective as global lower bound on all subsequent solves 
						lower_bound = obj.sol;
						put lower_bound=;
						*/
				  end;
			 %end;
			 %else %do;
				  /* generate constraints dynamically as needed: fancy version */
				  do until (numcomp = 1);
					iter = iter + 1;
					put iter=;
						/* solve with current set of constraints */
						solve with milp %if %upcase(&UseStoppingRules) = YES %then / &relobjgap1 &maxtime;;
					%postSolve;
						%findConnectedComponents;
						/* put lower_bound=; */
	
						if (numcomp = 1) then do;
							iter = iter + 1;
							put iter=;
							 /* if solution has no violations, then solve to optimality (starting with feasible solution) */
							 put "Solving to optimality...";
							 solve with milp / primalin %if %upcase(&UseStoppingRules) = YES %then &relobjgap2 &maxtime;;
							%postSolve;
							 %findConnectedComponents;
							 /* can use optimal objective as global lower bound on all subsequent solves 
							 lower_bound = obj.sol;
						  	 put lower_bound=;
							 */
						end;
				  end;
			 %end;
		%mend rowGenerationLoop;
	
		/* Macros used when skeleton marker orders exist for constrained TSP */
		%macro computeOrder;
			/* traverse arcs to determine order */
			SUPPORT = {<i,j> in EDGES: round(x[i,j].sol) = 1};
			OPTIMAL_SUPPORT = SUPPORT;
			curr_vert = 'DUMMY';
			for {curr_order in 1..card(VERTICES)-1} do;
				for {<(curr_vert),j> in SUPPORT} do;
					order[j] = curr_order;
					unrank[curr_order] = j;
					curr_vert = j;
					SUPPORT = SUPPORT diff {<curr_vert,j>};
				end;
			end;
		%mend computeOrder;
	
		%macro findBadPaths;
			/* find paths that violate precedence constraints */
			violates_precedence = 0;
			put (card(VIOLATED_PAIRS))=;
			for {<i,j> in VIOLATED_PAIRS} do;
				violates_precedence = 1;
				PATH = {};
				order_this = order[j];
				do while (order_this < order[i]);
					PATH = PATH union {<unrank[order_this],unrank[order_this+1]>};
					order_this = order_this + 1;
				end;
				numbadpath = numbadpath + 1;
				BADPATH[numbadpath] = PATH;
			end;
		%mend findBadPaths;
	
		%macro SrowGenerationLoop(simple=);
			 %if &simple = 1 %then %do;
				  /* generate constraints dynamically as needed: simple version */
				  do until (violates_precedence = 0);
					  do until (numcomp = 1);
						feasible = 1;
						iter = iter + 1;
						put iter=;
						/* solve with current set of constraints */
						put "Solving to optimality...";
						solve with milp %if %upcase(&UseStoppingRules) = YES %then  / &relobjgap2 &maxtime;;
						%postSolve;
						%findConnectedComponents;
						/* can use optimal objective as global lower bound on all subsequent solves 
						lower_bound = obj.sol;
						put lower_bound=;
						*/
					  end;
					if feasible = 1 then do;
						%computeOrder;
						%findBadPaths;
					end;
					else leave;
				  end;
			 %end;
			 %else %do;
				  /* generate constraints dynamically as needed: fancy version */
				  do until (violates_precedence = 0);
					do until (numcomp = 1);
						feasible = 1;
						iter = iter + 1;
						put iter=;
						/* solve with current set of constraints */
						solve with milp %if %upcase(&UseStoppingRules) = YES %then / &relobjgap1 &maxtime;;
						%postSolve;
						%findConnectedComponents;
						/* put lower_bound=; */
					end;
					if feasible = 1 then do;
						%computeOrder;
						%findBadPaths;
					end;
					else leave;
	
					if (numcomp = 1 and violates_precedence = 0) then do;
						feasible = 1;
						iter = iter + 1;
						put iter=;
						 /* if solution has no violations, then solve to optimality (starting with feasible solution) */
						 put "Solving to optimality...";
						 solve with milp / primalin %if %upcase(&UseStoppingRules) = YES %then &relobjgap2 &maxtime;;
						%postSolve;
						 %findConnectedComponents;
						 /* can use optimal objective as global lower bound on all subsequent solves 
						 lower_bound = obj.sol;
						 put lower_bound=;
						 */
						%computeOrder;
						%findBadPaths;
					end;
				end;
			 %end;
		%mend SrowGenerationLoop;
	%end;
	
	%let UseMapDistance = NO;
	/* use this macrovar to see if a group had an order created, if 1 then dim data set exists otherwise it was a singleton group*/
	%global Singletons sflag allsflag OrderCreated BreakMaxCheck;
	%let BreakMaxCheck = 0;
	%let EverBroke = 0;  /*update this if a group was ever broken so I know to resort lmo data */
	/* create a macrovariable  storing the linkage groups and markers that have only one marker */
	%let Singletons = ; 
	%let sflag = 0;
	%let allsflag = 1;
	
	/*Luciano*/
	%if (%nrbquote(%upcase(&Algorithm))= AUTOMATED or %nrbquote(%upcase(&Algorithm))= RCD 
		or %nrbquote(%upcase(&Algorithm))=SERIATION or %nrbquote(%upcase(&Algorithm))= RECORD 
		or %nrbquote(%upcase(&Algorithm))=UG or %nrbquote(%upcase(&Algorithm))= COMPARE) 
		or %length(&OrderVar) %then %do;
		%let UseMultipoint = YES;
	%end;
	
	%global sas_opt; 
	%let sas_opt = 0;
	%if %index(%upcase(&MarkerFmt),OUTCROSS)  
		and (%index(%nrbquote(%upcase(&Algorithm)),MULTIDIMENSIONAL) 
		or %nrbquote(%upcase(&Algorithm)) = MAP ORDER OPTIMIZATION
		or %index(%nrbquote(%upcase(&Algorithm)),ACCELERATED)) %then %do;
		
		%let sas_opt = 1;
		%let OrigOrderVar= &OrderVar;
	%end;
	
	%global umpflag IDVar_R SkletonVar_R; 
	%let umpflag = 0;
	%let IDVar_R= MarkerName_;
	%let SkletonVar_R= SkletonVar_;
	
	%put Algorithm= &Algorithm;
	%put sas_opt= &sas_opt;
	%put UseMultipoint= &UseMultipoint;
	/*Luciano*/
	
	%*goto exit;

	%macro OrderMarkers(_Group,iter,OrigGrp=);
		
		%if &debug %then %do;
			data outlib._lg; set _lg; run;
		%end;
		
		data lgdata;
			set _lg;
			where &LGroupVar = &_Group;
			*keep &LGroupVar &IDVar %do j=1 %to &LGtot; &&MVar&j %end; ;
			keep &LGroupVar &IDVar 
				%if %index(%upcase(&MarkerFmt),OUTCROSS) %then %do;
					&MarkerType
					%if %length(&OrderVar) %then &OrderVar;
					%if %length(&SkOrderVar) %then &SkOrderVar; 
				%end;
				%else %do;
					%do j=1 %to &LGtot; &&MVar&j %end;
				%end;
				;/*Luciano*/
		run;
		%put Number of markers in Group &_Group = &LGTot;
		
		%if &debug %then %do;
			data outlib.lgdata; set lgdata; run;
		%end;
		
		%nobsvars(lgdata);
		%put nobs = &nobs;
		%let nvars = %eval(&nvars -2);
		%put nvars = &nvars;
			
		%if %index(%upcase(&MarkerFmt),OUTCROSS) = 0 %then %do;
			/* check that the matrix is symmetric for the current linkage group */
			%if &nobs ne &nvars %then %do;
				%put ERROR: The matrix of recombination fractions for &LGroupVar = &_Group is not a square matrix of values.;
				%put ERROR: There are &nobs marker rows and &nvars marker columns, which should be equal values. Please check the input data.;
				%let exiterror =1;
				%goto exito;
			%end;
		%end;
		%else %do;
			%if %index(%nrbquote(%upcase(&Algorithm)),MULTIDIMENSIONAL) 
				or %nrbquote(%upcase(&Algorithm)) = MAP ORDER OPTIMIZATION
				or %index(%nrbquote(%upcase(&Algorithm)),ACCELERATED) %then %do;
								
				data lgnames_merge; 
					set lgdata(keep= &IDVar &MarkerType);
					row = _n_;
				run;
												
				proc sql;
					create table lgstackedmk as
					select f1._MarkerName_1, f1.MarkerType1_, f1.row1, f2._MarkerName_2, f2.MarkerType2_, f2.row2
					from lgnames_merge(rename = (&IDVar=_MarkerName_1 &MarkerType=MarkerType1_ row=row1 )) as f1 
					cross join lgnames_merge(rename = (&IDVar=_MarkerName_2 &MarkerType=MarkerType2_ row=row2)) as f2
					where f1.row1 < f2.row2
					order by f2.row2, f1.row1
				;
				quit;

			       *phasesdata must be formed by stacking rows of the lower diagonal matrix without the diagonal elements (row-wise stacking)*;
				proc sql;
					create table subphasedata(rename=(_MarkerName_1=Marker1 _MarkerName_2=Marker2) drop= row2 row1) as
					select a2._MarkerName_1, a2.MarkerType1_, a2._MarkerName_2, a2.MarkerType2_, a2.row2, a2.row1, a1.*
					from phasedata(rename=(Marker1=Marker1_old Marker2=Marker2_old) drop=MarkerType1_ MarkerType2_) as a1, lgstackedmk as a2  
					where a1.Marker1_old=a2._MarkerName_1 and a1.Marker2_old=a2._MarkerName_2
					or a1.Marker1_old=a2._MarkerName_2 and a1.Marker2_old=a2._MarkerName_1
					order by a2.row2, a2.row1
					;
				quit;
				
				/*data outlib.phasedata; set phasedata; run;*/
				/*data outlib.subphasedata; set subphasedata; run;*/

				proc iml;
					use subphasedata;
					read all var {rf1 rf2 rf3 rf4} into rfvec;	
					nr= nrow(rfvec);
					*print(nr);
					nm= &nobs;
					minrfmat= J(nm,nm,0);
					k=1;
					do i=2 to nm;
						do j=1 to i-1;
							if ^missing(rfvec[k,1]) then do;
								minrfmat[i,j]=min(rfvec[k,1:4]);
							end;
							else do;
								minrfmat[i,j]=.90;
							end;
							minrfmat[j,i]= minrfmat[i,j];
							k= k + 1;	
						end;
					end;
					*print(k);
					*print(minrfmat);
			
					create rfdata from minrfmat;
					append from minrfmat;
				quit;
				
				data rfdata;
					set rfdata;
					rename 
						%do j=1 %to &LGtot; 
							col&j= &&MVar&j
						%end;
					;
				run;
				
				data origlgdata;
					set lgdata;
				run;
				
				data lgdata;
					merge lgdata rfdata;
				run;
				
				/*data outlib.lgdatarf; set lgdata; run;*/
			%end;
		%end;

		%if &nobs <2 %then %do;
			%put Note: A linkage group must contain at least 2 markers to compute order and distances ;
			%put Note: There is only one marker in the linkage group, &LGroupVar = &_Group... Please add this marker to another group if you want it included in analysis. ;
			data lgdata;
				Length &IDVar $&_IDvarLength_ %if %length(&MarkerType) %then &MarkerType $&MkTypeLen;;
				set currcopydat;
				*keep &LGroupVar &IDVar &ColorVar &KeepVars Original_Group;
				keep &LGroupVar &IDVar &ColorVar &KeepVars Original_Group orig_order %if %length(&MarkerType) %then &MarkerType; ; /*Luciano*/
				%if %length(&OrigGrp) %then %do;
					%if %UPCASE(&LGType) = C %then %do;
						length Original_Group $&LGLen;
					%end;
					Original_Group = &OrigGrp;
				%end;
			run;
	
			%if &sflag=0 %then %do;
				%let Singletons = "&_Group";
				*data Outlib.&LinkageMapOrder_OutDataExName; 
				data singletons_all;
					set lgdata;
				run;
			%end;
			%else %do;
				%let Singletons = &Singletons "&_Group";
				/*proc append base=Outlib.&LinkageMapOrder_OutDataExName data=lgdata force; run;*/
				proc append base=singletons_all data=lgdata; run;
			%end;
			%let sflag = 1;
			%put Singleton List = &Singletons;
			%let OrderCreated = 0;
		%end;
		%else %do;
			%let OrderCreated = 1;
			%let allsflag = 0;
			%let nobscheck = &nobs;
	
			%if %index(%nrbquote(%upcase(&Algorithm)),MULTIDIMENSIONAL) 
				or %nrbquote(%upcase(&Algorithm)) = MAP ORDER OPTIMIZATION
				or %index(%nrbquote(%upcase(&Algorithm)),ACCELERATED) %then %do; 
				%if %length(&OrderVar) or %index(%nrbquote(%upcase(&Algorithm)),MULTIDIMENSIONAL)=0 %then %do;					
					/* kemicl change to PROC SQL to avoid data OOM 
					create a data set of names for output when MDS isn't run to get stacked data 
					*/
					
					data namesformerge; set _lg(keep = &IDVar);
						row = _n_;
					run;
					
					proc sql;
						create table stackedmlist as
						select f1.&IDVar, f1.row1, f2._COL_, f2.row2
						from namesformerge(rename = (row=row1)) as f1 
						cross join namesformerge(rename = (&IDVar=_COL_ row=row2)) as f2
						where f1.row1 <= f2.row2
						order by f2.row2, f1.row1
						;
					quit;

					data stackedmlist; set stackedmlist;
						rename &IDVar = _ROW_;
						drop row1 row2;
					run;			
					
				%end;

				%if ^%length(&OrderVar) %then %do;
					%if %index(%nrbquote(%upcase(&Algorithm)),MULTIDIMENSIONAL)>0 %then %do; 
						/* no order is specified, calculate from MDS */

						/*Never enters the if condition because UseMapDistance=NO*/
						%if %Upcase(&UseMapDistance) = YES %then %do;
							data lgdata;	 set lgdata;
								array d{*} %do j=1 %to &LGtot; &&MVar&j %end; ;
								do i = 1 to &LGtot;
									if not missing(d{i}) then do;
										%if %Upcase(&MapFunc) = HALDANE %then %do;
											d{i} = min(200, 100*(-(1/2)*log(1-2*min(d{i},0.49))));
										%end;
										%else %do;
											d{i} = min(200, 100*((1/4)*log((1+2*min(d{i},0.49))/(1-2*min(d{i},0.49)))));
										%end;
									end;
								end;
								drop i;
							run;
						%end;

						/*Never enters the if condition because UseWeights=NO in the xml file*/					
						%if %Upcase(&UseWeights) = YES %then %do;
							data lgdata;	 set lgdata;
								array d{*}  %do j=1 %to &LGtot; &&MVar&j %end;;
								array weights{&LGtot};
								do i = 1 to &LGtot;
									weights{i} = 1-d{i};
								end;
								drop i;
							run;
						%end;

						%put MDS order processing for Linkage Group = &_Group;
						options DKROCOND=nowarn;

						/*Never enters the if condition because MDSSqRoot=NO in the xml file*/										
						proc mds data=lgdata dim=1 formula = 2 %if %upcase(&MDSSqRoot)= YES %then fit = 0.5;
								OCONFIG pfinal pfit 
								out=Moutdim(drop=_name_ _matrix_ _type_)
								outres=Moutres outfit=mf;
							var %do j=1 %to &LGtot; &&MVar&j %end; ;
							%if %Upcase(&UseWeights) = YES %then %do;
								weight weights:;
							%end;
							id &IDVar;
						run;

						%if &syserr>4 %then %goto exito;

						data moutres; set Moutres;
							*&LGroupVar = &_Group;
							rename Data = Recombination;
							label FitDist = "MDS Distance" _ROW_ = ' ' _COL_= ' ' Data = "Recombination";
							drop _MATRIX_ _DIMENS_ TranData Distance ;
						run;

						data mf; set mf;
							%if &LGType = C %then %do;
								length &LGroupVar $&LGLen;
							%end;
							&LGroupVar = &_Group;
							if missing(_MATRIX_) then delete;
						run;

						/* merge back in orig_order and copyvars */
						proc sql; 
							create table dims as 
								select *
								from currcopydat d1, Moutdim d2
								where d1.&IDVar = d2.&IDVar;
						quit;

						proc sort data = dims out=dims(drop = Dim1 _DIMENS_);
							by Dim1;
						run;

						data dims; set dims;
							Init_Map_Order= _n_;
							%if %UPCASE(&LGType) = C %then %do;
								NumLinkageGroup = &iter;
							%end;
						run;

						/* data set with a column of marker orders, ordered by the orignal RF matrix, ready for IML manipulation */
						proc sort data = dims out=_ord_1(keep = Init_Map_Order);
							by orig_order;
						run;

						%put &nobscheck;
						%if &nobscheck > 2 %then %do;
							proc iml;
								start CostFunction(r) global(recomb,s,iter); 
									nc = ncol(r);
									cost_r =0; 
									do i = 1 to nc-1; 
									  cost_r = cost_r + recomb[loc(r=i),loc(r=i+1)]; 
									end; 
									s = r;
									if recomb[loc(s=2),loc(s=3)]>recomb[loc(s=1),loc(s=3)] then do;
										xi = loc(s=1);
										xj = loc(s=2);
										sxi =s[loc(s=1)];
										sxj = s[loc(s=2)];
										s[xi] = sxj;
										s[xj] =  sxi;
									end;
									/* local optimization  */ 
									do i = iter+1 to nc-(iter+1);
										tmp12 = recomb[loc(s=i-iter),loc(s=i)];
										tmp34 = recomb[loc(s=i+iter),loc(s=i+(iter+1))];
										tmp13 = recomb[loc(s=i-iter),loc(s=i+iter)];
										tmp24 = recomb[loc(s=i),loc(s=i+(iter+1))];
										if tmp12 +tmp34 > tmp13+tmp24 then do;
											xi = loc(s=i);
											xj = loc(s=i+iter);
											sxi =s[loc(s=i)];
											sxj = s[loc(s=i+iter)];
											s[xi] = sxj;
											s[xj] =  sxi;

										end;
									end;

									if recomb[loc(s=nc-1),loc(s=nc)]>recomb[loc(s=nc-2),loc(s=nc)] then do;
										xi = loc(s=nc-1);
										xj = loc(s=nc);
										sxi =s[loc(s=nc-1)];
										sxj = s[loc(s=nc)];
										s[xi] = sxj;
										s[xj] =  sxi;
									end;

									/* compute objective function */ 
									cost_s =0; 
									do i = 1 to nc-1; 
									  cost_s = cost_s + recomb[loc(s=i),loc(s=i+1)]; 
									end; 
									if cost_s < cost_r then
										cost = cost_s;
									else do;
										cost = cost_r;
										s = r;
									end;
									 /* if uniform(1234)<=p then 
										r = s; 
										*/
									return (cost); 
								finish; 

								use lgdata;
								read all var{ %do j=1 %to &LGtot; &&MVar&j %end;} into recomb;
								use _ord_1;
								read all into ord;
								nloci = ncol(recomb);

								/* run cost function with MDS order */
								init_o = T(ord);
								*print init_o;
								initcost =0; 
								do i = 1 to nloci-1; 
								  initcost = initcost + recomb[loc(init_o=i),loc(init_o=i+1)]; 
								end; 
								iter = 1;
								newcost = CostFunction(init_o);
								print "Local Optimization of MDS Solution";
								print "Initial Cost"  initcost  "		Local Optimized Cost" newcost;
								newo = s;
								maxiter = min(10, floor(nloci/4));
								do iter = 2 to maxiter; 
									oldcost = newcost;
									newcost = CostFunction(newo);
									if oldcost <= newcost then i=maxiter;
									else do;
										newo = s;
										print "Previous Cost" oldcost "New Cost" newcost;
									end;
								end; 				

								bestmembert = T(newo);
								create ordermat from bestMembert;
								append from bestMembert;
							quit;

							data _ord_; set ordermat(rename= (col1 = Map_Order)); run;

							data tord; merge _ord_1 ordermat(rename= (col1 = Map_Order)); run;	

							proc sort data=tord;
								by Init_Map_Order;
							run;

							data dims; merge dims tord; by Init_Map_Order;run;

							proc sort data=dims; by Map_Order; run;
						%end;
						%else %do;
							data dims; set dims;
								Map_Order = Init_Map_Order;
							run;

							proc sort data = dims out=_ord_(keep = Map_Order);
								by Map_Order;
							run;
						%end;
					%end;
					%else %do;
						%let foundcatmacro=%sysfunc(fileexist(&MacroPath.\sasmacr.sas7bcat));
						%put foundcatmacro= &foundcatmacro;

						%if &foundcatmacro = 0 %then %do;
							%put ERROR: The stored macro catalog "sasmacr.sas7bcat" was not found in "&MacroPath.".;
							%put This file should be copied from;
							%put "&MacroPath.\Stored32" or;
							%put "&MacroPath.\Stored64" depending on your OS.;
							%let exiterror=1;
							%goto exito;
						%end;

						/* use PROC OPTmodel to perform Map Order Optimization using TSP algorithms*/
						/* constraint on maximum recomb rate between markers placed next to each other */
						%put recombination constraint threshold = &Threshold;
						%if %length(&SkOrderVar) %then %do;
							%nobsvars(skdata_lg);

							%if &nobs > 1 %then %do;
								%put At least two markers in linkage group &_Group have skeleton positions.  Constrained TSP will be used.;
								/* constrained TSP macro to honor ordinality of skeleton markers */
								%ClockStart(Linkage Group OPTMODEL PROCESSING);
								%SkeletonLMO(indata=lgdata, skeletondata=skdata_lg, SPosVar=&SkOrderVar);
								%ClockStop(Linkage Group OPTMODEL PROCESSING);
							%end;
							%else %do;
								%put Markers in linkage group &_Group have no skeleton positions.  General TSP will be used.;
								/* no skeleton marker constraints for current linkage group- general TSP solver */
								/* iterative solution using the subtour formulation from SAS PROC OPTMODEL*/
								%ClockStart(Linkage Group OPTMODEL PROCESSING);
								%LMOOptModel(indata=lgdata);
								%ClockStop(Linkage Group OPTMODEL PROCESSING);
							%end;
						%end;
						%else %do;
							/* no skeleton marker constraints - general TSP solver */
							/* iterative solution using the subtour formulation from SAS PROC OPTMODEL*/
							%if %nrbquote(%upcase(&Algorithm)) = MAP ORDER OPTIMIZATION %then %do;
								%put Algorithm= OPTMODEL;
								%ClockStart(Linkage Group OPTMODEL PROCESSING);
								%LMOOptModel(indata=lgdata);
								%ClockStop(Linkage Group OPTMODEL PROCESSING);

							%end;
							%else %if %index(%nrbquote(%upcase(&Algorithm)),ACCELERATED)=1 %then %do;
								%put Algorithm= OPTNET;
								%ClockStart(Linkage Group OPTNET PROCESSING);
								%LMOOptNet(indata=lgdata);
								%ClockStop(Linkage Group OPTNET PROCESSING);
								%put _OROPTNET_macrovar=&_OROPTNET_macrovar;

								%let bstatus= %index(&_OROPTNET_macrovar, STATUS=);
								%let btsp= %index(&_OROPTNET_macrovar, TSP=);
								%let status= %substr(&_OROPTNET_macrovar, &bstatus + 7, &btsp - &bstatus - 7);
								%put &status;

								%if %upcase(&status)=OUT_OF_MEMORY or %upcase(&status)=ERROR %then %do;
									%let exiterror=1;
								%end;
							%end;
						%end;

						%if &exiterror %then %do;
							%if %nrbquote(%upcase(&Algorithm)) = MAP ORDER OPTIMIZATION %then %do;
								%put ERROR: Map Order Optimization is infeasible for Linkage Group &_Group..;
								%put You may need to increase the Nearby Marker Recombination Constraint on;
								%put the Optimization Options Tab or further split this linkage group.;
							%end;
							%else %if %index(%nrbquote(%upcase(&Algorithm)),ACCELERATED)=1 %then %do;
								%put ERROR: Accelerated Map Order Optimization is infeasible for Linkage Group &_Group..;
								%put You may need to increase the Nearby Marker Recombination Constraint on;
								%put the Optimization Options Tab or further split this linkage group.;	
							%end;
							%goto exito;
						%end;

						data dims; set currcopydat;
						run;

						data ordermat;
							set solData2; 
							keep Order;
						run;

						data dims; merge dims ordermat(rename= (Order = Opt_Order)); run;

						proc sort data=dims;
							by Opt_order;
						run;

						data dims; set dims;
							Map_Order= _n_;
							%if %UPCASE(&LGType) = C %then %do;
								NumLinkageGroup = &iter;
							%end;
							drop Opt_Order; /*Luciano*/
						run;
						
						/* data set with a column of marker orders, ordered by the orignal RF matrix, ready for IML manipulation */
						proc sort data = dims out=_ord_(keep = Map_Order);
							by orig_order;
						run;
					%end;
				%end;
				%else %do;
					/* Use Order from Order data set */
					data dims;
						set currcopydat;
					run;
					proc sort data = dims;
						by &OrderVar;
					run;
					data dims; set dims;
						Map_Order= _n_;
						%if %UPCASE(&LGType) = C %then %do;
							NumLinkageGroup = &iter;
						%end;
					run;
					/* data set with a column of marker orders, ordered by the orignal RF matrix, ready for IML manipulation */
					proc sort data = dims out=_ord_(keep = Map_Order);
						by orig_order;
					run;				
				%end;	
			
				proc iml;
					use lgdata;
					read all var{ %do j=1 %to &LGtot; &&MVar&j %end;} into r;
					use _ord_;
					read all into ord;
					nm= nrow(ord);
					cmstep = J(nm, 1, .);
					cmstep[1,1] = 0;
					rfstep = J(nm,1,.);
					cmtot = J(nm, 1, .);
					cmmax = J(1,1,.);
					rfmax = J(1,1,.);
					do i = 1 to nm-1;
						do j = i to nm;
							if j-i = 1 then do;
								/* add in option to output recombination frequency between ordered markers */
								rfstep[i+1] = r[loc(ord=i),loc(ord=j)];
								%if %Upcase(&UseMapDistance) = YES %then %do;
									cmstep[i+1] = r[loc(ord=i),loc(ord=j)];
								%end;
								%else %do;
									if r[loc(ord=i),loc(ord=j)] = . then do;
										/* the recomb freq is missing, something is wrong */
										cmstep[i+1] = .;
									end;
									else if r[loc(ord=i),loc(ord=j)] >= 0.5 then do;
										cmstep[i+1] = 200;
									end;
									else do;
										%if %Upcase(&MapFunc) = HALDANE %then %do;
											/* Haldane's Function */
											cmstep[i+1] = 100*(-(1/2)*log(1-2*r[loc(ord=i),loc(ord=j)]));
										%end;
										%else %do; /* Kosambi */
											cmstep[i+1] = 100*((1/4)*log((1+2*r[loc(ord=i),loc(ord=j)])/(1-2*r[loc(ord=i),loc(ord=j)])));
										%end;
									end;
									/* cap at 200 */
									if cmstep[i+1] > 200 then cmstep[i+1] = 200;
								%end;
							end;
						end;
						cmtot[i,1] = sum(cmstep[1:i]);
					end;
					cmtot[nm,1] = sum(cmstep[1:nm]);
					%if %upcase(&UseMultipoint) = NO %then %do;
						%if %upcase(&BreakGrps) = YES %then %do;
							&BGimlprefix.max = max(&BGimlprefix.step[1:nm]);
							if &BGimlprefix.max > &BreakGrpsValue then call symput("BreakMaxCheck",trim(left(char(1))));
						%end;
					%end;
					
					%if %length(&OrderVar) or %index(%nrbquote(%upcase(&Algorithm)),MULTIDIMENSIONAL)=0 %then %do;
						rstack = symsqr(r);

						create rstack from rstack;
						append from rstack;
					%end;

					create cmmat from cmtot;
					append from cmtot;
					create cmstep from cmstep;
					append from cmstep;
					create rfstep from rfstep;
					append from rfstep;
				quit;
				
				%put BreakMaxCheck = &BreakMaxCheck..  A value of 1 means Linkage Group &_Group will be broken.;
				%if &syserr>4 %then %do;
					%put ERROR: The genetic distances could not be calculated.;
					%let exiterror=1;
					%goto exito;
				%end;		
				
				data dims; 
					merge dims cmmat(rename =(col1 = cM_&MapFunc)) cmstep(rename =(col1 =cM_Step)) rfstep(rename=(col1=RecombinationF));
					cM_&MapFunc = round(cM_&MapFunc,0.01);
					cM_Step = round(cM_Step,0.01);
					RecombinationF = round(RecombinationF, 0.001);
					%if %length(&OrigGrp) %then %do;
						%if %UPCASE(&LGType) = C %then %do;
							length Original_Group $&LGLen;
						%end;
						Original_Group = &OrigGrp;
					%end;
				run;
			
				%if &debug %then %do;
					data outlib.dims&iter; set dims; run;
				%end;				
			%end;
			
			%if %index(%upcase(&MarkerFmt),OUTCROSS) and &sas_opt and %upcase(&UseMultipoint) = YES %then %do;
			
				/*data outlib.dims; set dims; run;*/
								
				%let sasLGcM= 0;
				data _null_;
					set dims end=_eof_;
					if _eof_ then call symputx("sasLGcM", trim(left(cM_&MapFunc)));
				run;
					
				%let OrderVar= maporder_;
				data lgdata;
					set dims(rename= (Map_Order= &OrderVar));
					keep &LGroupVar &IDVar &MarkerType
						%if %length(&OrderVar) %then &OrderVar;
						%if %length(&SkOrderVar) %then &SkOrderVar;
					;
				run;
				
				/*data outlib.lgdata; set lgdata; run;*/

				data subphasedata;
					set subphasedata(keep= Marker1 Marker2 MarkerType1_ MarkerType2_ rf1 rf2 rf3 rf4 lod1 lod2 lod3 lod4);
				run;
				
				%*let exiterror=1;
				%*goto exito;
			%end;
				
			/*
			%if (%nrbquote(%upcase(&Algorithm))= AUTOMATED or %nrbquote(%upcase(&Algorithm))= RCD 
				or %nrbquote(%upcase(&Algorithm))=SERIATION or %nrbquote(%upcase(&Algorithm))= RECORD 
				or %nrbquote(%upcase(&Algorithm))= UG or %nrbquote(%upcase(&Algorithm))= COMPARE or &sas_opt) %then %do;
				*/ 
			%if %upcase(&UseMultipoint) = YES %then %do;
				%put UseMultipoint= &UseMultipoint;
			
				%let fixedorder= none;
				%let SkletonVar_tmp= &SkletonVar_R;
				%let SkletonVar_R= null;
				%if %length(&OrderVar) %then %do;
					%let fixedorder= all;
					proc sort data=lgdata;
						by &OrderVar;
					run;
				%end;
				%else %if %length(&SkOrderVar) %then %do;
					/*data outlib.skdata_lg&iter; set skdata_lg;run;*/
					%nobsvars(skdata_lg);
					%if &nobs > 1 %then %do; 
						%let fixedorder= partial;
						%let SkletonVar_R= &SkletonVar_tmp; 
						proc sort data=lgdata;
							by &SkOrderVar;
							*by &SkletonVar_R;
						run;
					%end;
				%end;
				
				%global forcearg;
				%if %upcase(&ForceMarkersOneMap)= NO %then
					%let forcearg= safe;
				%else
					%let forcearg= force;
					
				*data sets lggendat, lgannodat, and lgphasedata must be ordered according to the lgdata*;
				data lgdata(drop= row 
						rename=(&IDVar= &IDVar_R 
						%if %length(&SkOrderVar) %then &SkOrderVar=&SkletonVar_R;
						)
					) 
					lgannodat(keep= &IDVar &MarkerType) 
					lgnames_merge(keep= &IDVar &MarkerType row); 
					set lgdata end=_e_;
					row = _n_;
					call symput("MVar_tmp"||trim(left(_n_)), trim(left(&IDVar)));
					if _e_ then call symput("LGtot_tmp",trim(left(_n_)));
				run;
			
				data lggendat;
					retain 
					%do k=1 %to &LGtot_tmp;
						&&MVar_tmp&k
					%end;
					;
					set gendat;
					keep 
					%do k=1 %to &LGtot_tmp;
						&&MVar_tmp&k
					%end;
					;
				run;
								
				proc sql;
					create table lgstackedmk as
					select f1._MarkerName_1, f1.MarkerType1_, f1.row1, f2._MarkerName_2, f2.MarkerType2_, f2.row2
					from lgnames_merge(rename = (&IDVar=_MarkerName_1 &MarkerType=MarkerType1_ row=row1 )) as f1 
					cross join lgnames_merge(rename = (&IDVar=_MarkerName_2 &MarkerType=MarkerType2_ row=row2)) as f2
					where f1.row1 < f2.row2
					order by f2.row2, f1.row1
				;
				quit;
				
				/*data outlib.phasedata; set phasedata; run;*/
				/*data outlib.lgstackedmk; set lgstackedmk; run;*/

			       *phasesdata must be formed by stacking rows of the lower diagonal matrix without the diagonal elements (row-wise stacking)*;
				proc sql;
					create table lgphasedata(rename=(_MarkerName_1=Marker1 _MarkerName_2=Marker2) drop= row2 row1) as
					select a2._MarkerName_1, a2.MarkerType1_, a2._MarkerName_2, a2.MarkerType2_, a2.row2, a2.row1, a1.*
					from 
					%if &sas_opt = 0 %then %do;
						phasedata(rename=(Marker1=Marker1_old Marker2=Marker2_old) drop=MarkerType1_ MarkerType2_) 
					%end;
					%else %do;
						subphasedata(rename=(Marker1=Marker1_old Marker2=Marker2_old) drop=MarkerType1_ MarkerType2_) 
					%end;
					as a1, lgstackedmk as a2  
					where a1.Marker1_old=a2._MarkerName_1 and a1.Marker2_old=a2._MarkerName_2
					or a1.Marker1_old=a2._MarkerName_2 and a1.Marker2_old=a2._MarkerName_1
					order by a2.row2, a2.row1
					;
				quit;
				
				%if &debug %then %do;
					data outlib.lgdata&_Group;
						set lgdata;
					run;
					
					data outlib.lgannodat&_Group;
						set lgannodat;
					run;

					data outlib.lggendat&_Group;
						set lggendat;
					run;
					
					data outlib.lgphasedata&_Group;
						set lgphasedata;
					run;
				%end;
				
				%*let exiterror=1;
				%*goto exito;
						
			        /*begin marker ordering - OneMap*/
				options nonotes;
				ods _all_ close;
				*ods html body="&Outpath.&OutName..html";
				proc iml; 
					/*NOTES:
					(1) Must keep module calls in the order they have been written here because
						the output from one module may be used in the next.
					(2) R objects created by a module can be accessed by any other module as far as 
						modules calls are keept within the same block defined by "Proc iml"	and "quit".
					This fact can be used to avoid having to exchange matrices or datasets between 
						R and IML.
					*/

					/*loading modules*/
					load module=(RemoveRObjs);
					load module=(SaveRData);
					load module=(LoadRPkgs);
					load module=(LoadRFunc);
					load module=(ExportDataToR);
					load module=(RebuildOutcrossObj);
					load module=(RebuildRF2PtsObj);
					load module=(OrderMarkersOneMap);
					
					/*loads R packages*/
					run LoadRPkgs;
					if ^isOK then do;
						call symputx("exiterror", 1);
						abort;
					end;
					
					/*loads R functions*/
					run LoadRFunc("&RSourcePath");
					if ^isOK then do;
						call symputx("exiterror", 1);
						abort;
					end;
					
					/*exports genotypic data from SAS to R*/
					run ExportDataToR("lggendat", "&GenoDataObjName");

					/*exports annotation data from SAS to R*/
					run ExportDataToR("lgannodat", "&AnnoDataObjName");

					/*rebuilds outcross object in R*/
					run RebuildOutcrossObj("&GenoDataObjName", "&AnnoDataObjName", "&OutCrossObjName", "&MarkerType");
					if ^isOK then do;
						call symputx("exiterror", 1);
						abort;
					end;

					/*exports linkage phase data from SAS to R*/
					run ExportDataToR("lgphasedata", "&PhasesObjName");
				
					/*estimates pairwise recombinations*/
					run RebuildRF2PtsObj("&PhasesObjName", "&OutCrossObjName", &minLODOneMap, &maxRFOneMap, "&TwoptsObjName");								
					if ^isOK then do;
						call symputx("exiterror", 1);
						abort;
					end;

					%*let exiterror=1;
					%*goto exito;
					
					/*removes objects in R*/
					run RemoveRObjs("&GenoDataObjName, &AnnoDataObjName");
					if ^isOK then do;
						call symputx("exiterror", 1);
						abort;
					end;

					/*exports linkage group data from SAS to R*/
					run ExportDataToR("lgdata", "&CurrLGObjName");
					
					%*let exiterror=1;
					%*goto exito;
						
					/*ordering markers*/
					run OrderMarkersOneMap("&CurrLGObjName", "&IDVar_R", "&TwoptsObjName", &minLODOneMap, &maxRFOneMap, 
								"&Algorithm", "&forcearg", "&fixedorder", "&SkletonVar_R", 
								&nShuffle, &ShuffleWSize, &ShuffleLOD);			
					if ^isOK then do;
						call symputx("exiterror", 1);
						abort;
					end;
					
					/*saves R console image*/
					/*
					run SaveRData("&ROutPath", "&Outname");
					if ^isOK then do;
						call symputx("exiterror", 1);
						abort;
					end;
					*/
					
					%*let exiterror=1;
					%*goto exito;
					
					use rfmat1_mat;
					read all into Rf;
					rstack = symsqr(Rf);	
					create rstack from rstack;
					append from rstack;
					close rfmat1_mat;

				quit; 
				options notes;
				/*end marker ordering - OneMap*/
				
				*Must reassing OrderVar to its original value if sas_opt = 1*;
				%if &sas_opt %then %do;				
					%let OrderVar= &OrigOrderVar;
				%end;
				
				%*let exiterror=1;
				%*goto exito;
				
				*collect unmapped markers*;
				*unmappedm is created within R software*;
				%nobsvars(unmappedm);
				%if &nobs > 0 %then %do;

					*data unmappedm_tmp(keep= &IDVar &MarkerType Original_Group Orig_Order);
					data unmappedm_tmp(keep= &IDVar &MarkerType Original_Group);
						length &IDVar $&_IDvarLength_ &MarkerType $&MkTypeLen; 
						set unmappedm;
						%if %length(&OrigGrp) %then %do;
							%if %UPCASE(&LGType) = C %then %do;
								length Original_Group $&LGLen;
							%end;
							Original_Group = &OrigGrp;
						%end;
						&IDVar= mkname_; 
						&MarkerType= mktype_; 
						Orig_Order= mknumb_;
					run;

					%if &umpflag=0 %then %do;
						data all_unmappedm; 
							set unmappedm_tmp;
						run;
					%end;
					%else %do;
						proc append base=all_unmappedm data=unmappedm_tmp; run;
					%end;
					%let umpflag = 1;

					/*				
					data outlib.unmappedm_tmp_&iter; set unmappedm_tmp; run;
					data outlib.all_unmappedm_&iter; set all_unmappedm; run;
					*/				
				%end;

				*collect mapped markers*;
				*mappedm is created within R software*;
				%if &debug %then %do;
					data outlib.mappedm0&_Group; set mappedm; run;
				%end;
				
				proc sql UNDO_POLICY=NONE; 
					create table mappedm as 
					select *
					from mappedm as d1 join currcopydat(keep = &LGroupVar &IDVar &ColorVar &KeepVars &OrderVar &SkOrderVar orig_order) as d2
					on d1.mkname_ = d2.&IDVar;
				quit;
				
				%if &debug %then %do;
					data outlib.mappedm1&_Group; set mappedm; run;
				%end;

				data dims(drop= mknumb_);
					length &IDVar $&_IDvarLength_ &MarkerType $&MkTypeLen;
					set mappedm;	
					&LGroupVar= &_Group;
					%if %length(&OrigGrp) %then %do;
						%if %UPCASE(&LGType) = C %then %do;
							length Original_Group $&LGLen;
						%end;
						Original_Group = &OrigGrp;
					%end;
					&MarkerType= mktype_;
					&IDVar= mkname_; 
					rename %if %index(%upcase(&MapFunc),HALDANE) %then cMHaldane_= cM_&MapFunc cMHaldaneStep_= cM_Step; 
						%else cMKosambi_= cM_&MapFunc cMKosambiStep_= cM_Step;
						rf_= RecombinationF
						mkorder_= Map_Order
						/*mknumb_= Orig_Order*/
						phase_= Linkage_Phase
						;
					drop %if %index(%upcase(&MapFunc),HALDANE) %then cMKosambi_ cMKosambiStep_; 
						%else cMHaldane_ cMHaldaneStep_; 
					mktype_ mkname_;
					*label A1P1_= "Allele 1 of Parent 1";
					*label A2P1_= "Allele 2 of Parent 1";
					*label A1P2_= "Allele 1 of Parent 2";
					*label A2P2_= "Allele 2 of Parent 2";				
				run;
				
				proc sort data=dims;
					by Map_Order;
				run;
				
				%if &debug %then %do;
					data outlib.dims&_Group; set dims; run;
				%end;	
				
				%if &sas_opt %then %do;
				
					%let OneMapLGcM= 0;
					data _null_;
						set dims end=_eof_;
						if _eof_ then call symputx("OneMapLGcM", trim(left(cM_&MapFunc)));
					run;	
					
					%if &sasLGcM < &OneMapLGcM %then %do;
						%put WARNING: The Linkage Group &_Group returned from OneMap multipoint function has length larger than the one produced by SAS procedures &Algorithm..;
					%end;
				%end;

				data lgmnames_;
					set rfmat1_mat;
					where 0;
				run;	
				
				%*let exiterror=1;
				%*goto exito;

				%let _tid_ = %sysfunc(open(lgmnames_));
				%let _nVars_ = %sysfunc(attrn(&_tid_,NVARS));				
				data stackedmlist;
					length _ROW_ $32 _COL_ $32;
					%do k = 1 %to &_nVars_;
						%do m = 1 %to &k;
							_ROW_ = "%sysfunc(varname(&_tid_,&m))";
							_COL_ = "%sysfunc(varname(&_tid_,&k))";
							output;
						%end;
					%end;
				run;	
				%let _tid_ = %sysfunc(close(&_tid_));

				%if %upcase(&BreakGrps) = YES %then %do;
					proc means data=dims noprint;
						var &BreakGrpsVar;
						output out=mxbreak max=mxvar_;
					run;						

					data _null_;
						set mxbreak;	
						if _n_ = 1 and mxvar_ > &BreakGrpsValue then call symputx("BreakMaxCheck","1");
					run;

					%put BreakMaxCheck = &BreakMaxCheck..  A value of 1 means Linkage Group &_Group will be broken.;
					%if &syserr>4 %then %do;
						%put ERROR: The genetic distances could not be calculated.;
						%let exiterror=1;
						%goto exito;
					%end;

					proc datasets nolist;
						delete mxbreak;
					quit;

				%end;
			
				%*let exiterror=1;
				%*goto exito;

				proc datasets nolist;
					delete mappedm unmappedm;
				quit;				
			%end; /*end UseMultipoint*/						
		%end; /*nobs >= 2 */
		%exito:;
		%if &exiterror %then quit;
		
	%mend OrderMarkers;
	
	%let AppendStarted = 0;
	/* macro for appending the dims data out of ordering into a complete data set for lmo and lmr;*/
	%macro BuildLMOData(first=);
		/* need descending for computing reverse order */		
		proc sort data=dims;	
			by &LGroupVar descending Map_Order;
		run;
		data dims; set dims;
			retain _RMO_ _cMR_ ;
			by &LGroupVar;
			_cMStepR_ = lag1(CM_step);
			_rfR_ = lag1(RecombinationF);
			if first.&LGroupVar then do;
				_cMR_ = 0;
				_cMStepR_ = 0;
				_rfR_ = .;
				_RMO_ = 1;
			end;
			else do;
				_RMO_ = _RMO_ +1;
				_cMR_ = _cMR_ + _cMStepR_;
			end;
		run;
	
		proc sort data=dims;	
			by &LGroupVar Map_Order;
		run;
		
		/* create stacked recombination matrix output for interactive plots */
		%if %length(&OrderVar) or %index(%nrbquote(%upcase(&Algorithm)),MULTIDIMENSIONAL)=0 %then %do;
			data moutres;
				merge stackedmlist rstack(rename = (col1 = Recombination));
				label Recombination = "Recombination";
				if _ROW_ = _COL_ then delete;
			run;
		%end;
	
		%if &first %then %do;
			data outdims; 
				%if &_IDVarType_ = C %then length &IDVar $ &_IDVarLength_;;
				set dims; 
			run;
			
			data outres; set Moutres; run;
			%if ^%length(&OrderVar) and %index(%nrbquote(%upcase(&Algorithm)),MULTIDIMENSIONAL)>0 %then %do;   /* fit data doesn't exist when order var spec'd */
				data outfit; set mf; run; 
			%end;
		%end;
		%else %do;
	
			proc append base=outdims data=dims FORCE;run;
	
			proc append base=Outres data=Moutres;run;
	
			%if ^%length(&OrderVar) and %index(%nrbquote(%upcase(&Algorithm)),MULTIDIMENSIONAL)>0  %then %do; 
				proc append base=outfit data=mf;run; 
			%end;
		%end;
	
		%if ^%length(&OrderVar) and %nrbquote(%Upcase(&Algorithm))=MDS  %then %do;
			proc datasets library=work nolist;
				delete Moutdim Moutres mf;
			quit;
		%end;
		%else %do;
			proc datasets library=work nolist;
				delete Moutdim Moutres ;
			quit;
		%end;
		/* this macr has been run*/
		%let AppendStarted = 1;
	%mend BuildLMOData;
		
	%if &runold %then %do;
		/*make macro initialized to original number of groups, but if they get broke update to have new number of groups */
		%let bnGrps = &nGrps;

		/* loop over groups to do marker ordering and calculate genetic distance */
		%do i=1 %to &nGrps;
			%put Processing Linkage Group &&LGrp&i;
			%let OLGrp&i = &&LGrp&i;
			%if %UPCASE(&LGType) = C %then %do;
				%let LGrp&i = "&&LGrp&i";
			%end;
			
			/*data outlib.indat; set indat;run;*/

			/* make data set of linkage group and list of the variables to keep for this linkage group for MDS processing data set 	*/
			data _lg; set indat end=_e_;
				where &LGroupVar = &&LGrp&i;  
				call symput("MVar"||trim(left(_n_)), trim(left(&IDVar)));
				if _e_ then call symput("LGtot",trim(left(_n_)));
			run;
			%put LGtot= &LGtot;

			/* get current copy data for linkage group merging.  need to break up so that can handle and change it for new groups*/
			data currcopydat;
				set copydat;
				where &LGroupVar = &&LGrp&i; 
			run;
			/* changed copydat reference in macro to currcopydat, break this up for new groups before recall of macro*/

			%if %length(&SkOrderVar) %then %do;
				data skdata_lg; set skeleton;
					where &LGroupVar = &&LGrp&i;
				run;
			%end;

			 /*call macro defined above to order markers or detect singletons; */
			%OrderMarkers(&&LGrp&i, &i, OrigGrp = &&LGrp&i);
			%if &exiterror = 1 %then %goto exit;

			%if &OrderCreated = 1 %then %do;
				/* Kemicl check if too large of distance, if so, split groups and rerun order */
				%if &BreakMaxCheck %then %do;
					%let EverBroke = 1;
					%let OrigGrpValue = &&LGrp&i;

					data newgps;
						*set dims(Keep = &IDVar &LGroupVar &BreakGrpsVar) end=last;
						set dims(Keep = &IDVar &LGroupVar &BreakGrpsVar %if %length(&MarkerType) %then &MarkerType;) end=last; /*Luciano*/
						Original_Group = &LGroupVar;
						if _n_ = 1 then do;
							%if %UPCASE(&LGType) = C %then %do;
								newLGroup = 1;
								NewNumLGrp = &bngrps;
							%end;
							%else %do;
								newLGroup = &bngrps;
							%end;
						end;
						retain NewLGroup;
						%if %UPCASE(&LGType) = C %then retain NewNumLGrp;;
						if &BreakGrpsVar > &BreakGrpsValue then do;
							NewLGroup+1;
							%if %UPCASE(&LGType) = C %then NewNumLGrp+1;;
						end;
						%if %UPCASE(&LGType) = C %then %do;
							&LGroupVar = trim(left(&LGroupVar))||"_"||trim(left(NewLGroup));
						%end;
						%else %do;
							if newLGroup ne &bngrps then &LGroupVar = newLGroup;
						%end;
						%if %UPCASE(&LGType) = C %then %do;
							if last then  call symput("bnGrps",trim(left(NewNumLGrp)));
						%end;
						%else %do;
							if last then  call symput("bnGrps",trim(left(NewLGroup)));
						%end;					
						*keep &IDVar &LGroupVar NewNumLGrp NewLGroup;
						keep &IDVar &LGroupVar NewNumLGrp NewLGroup %if %length(&MarkerType) %then &MarkerType; ;/*Luciano*/
					run;

					%if &debug %then %do;
						data outlib.newgps; set newgps; run;
					%end;

					/* create macro variables for new groups to loop through for ordering from current original group*/
					%let NnewGrps = ;
					%GetLevels(_Ds =newgps,_Var=&LGroupVar,_LPrefix = NLGrp,_nLvls=NnewGrps,_vType= nLGType);
					%put New Number of Groups = &BnGrps;
					%put There are &NnewGrps new Linkage Groups for ordering broken from &&LGrp&i ;
					
					/* get values of the new linkage group numbers for the iteration count */
					%if %UPCASE(&LGType) = C %then %do;
						%let newitervar = NewNumLGrp;
						%let tmpdropvars = NewNumLGrp NewLGroup;
					%end;
					%else %do;
						%let newitervar = NewLGroup;
						%let tmpdropvars = NewLGroup;
					%end;

					proc sort data=newgps out=getiter nodupkey;
						by &newitervar;
					run;

					data _null_; 
						set getiter;
						call symput("niter"||trim(left(_n_)),trim(left(&newitervar)));
					run;

					data newlgdata;
						if 0 then set newgps(drop = &tmpdropvars);
						if _N_ = 1 then do;
							declare hash h(dataset:"newgps");
							h.defineKey("&IDVar");
							h.defineData("&IDVar","&LGroupVar");
							h.defineDone();
						end;
						set _lg(drop = &LGroupVar);
						if h.find() = 0;
					run;

					data newcopydat;
						if 0 then set newgps(drop = &tmpdropvars);
						if _N_ = 1 then do;
							declare hash h(dataset:"newgps");
							h.defineKey("&IDVar");
							h.defineData("&IDVar","&LGroupVar");
							h.defineDone();
						end;
						set currcopydat(drop = &LGroupVar);
						if h.find() = 0;
					run;

					%if %length(&SkOrderVar) %then %do;
						data newskdat;
							if 0 then set newgps(drop = &tmpdropvars);
							if _N_ = 1 then do;
								declare hash h(dataset:"newgps");
								h.defineKey("&IDVar");
								h.defineData("&IDVar","&LGroupVar");
								h.defineDone();
							end;
							set skdata_lg(drop = &LGroupVar);
							if h.find() = 0;
						run;
					%end;

					/* reset breakmax back to 0 so it doesn't keep looping through */
					%let BreakMaxCheck = 0;

					/*reloop through OrderMacro with new smaller groups here*/
					%do ng = 1 %to &NnewGrps;
						%if %UPCASE(&LGTYPE) = C %then %let NLGrp&ng = "&&NLGrp&ng";

						data _lg; set newlgdata end=_e_;
							where &LGroupVar = &&NLGrp&ng;
							call symput("MVar"||trim(left(_n_)), trim(left(&IDVar)));
							if _e_ then call symput("LGtot",trim(left(_n_)));
						run;

						%if &debug %then %do;
							data outlib._newlgdata&i; set newlgdata; run;
							data outlib._lg&i._&ng; set _lg; run;
						%end;

						data currcopydat; 
							set newcopydat;
							where &LGroupVar = &&NLGrp&ng;
						run;

						%if %length(&SkOrderVar) %then %do;
							data skdata_lg; 
								set newskdat;
								where &LGroupVar = &&NLGrp&ng;
							run;
						%end;

						 /*call macro defined above to order markers or detect singletons; */
						%if &ng = 1 %then %let newi = &i; 
						%else %let newi = &&niter&ng;
						%OrderMarkers(&&NLGrp&ng, &newi, OrigGrp = &&LGrp&i);
						%if &exiterror = 1 %then %goto exit;						
						%if &OrderCreated = 1 %then %do;
							%if &AppendStarted = 0 %then %let fcheck = 1;
							%else %let fcheck = 0;
							%BuildLMOData(first=&fcheck);
						%end;	
					%end;
					/*Once that done, at end after dim data appends, recreate macros for each lgroup value and final number of groups for jsl */					
				%end; /* end for breaking groups and recreating order and dims data */
				%else %do;
					%if &AppendStarted = 0 %then %let fcheck = 1;
					%else %let fcheck = 0;
					%BuildLMOData(first = &fcheck);
				%end;
			%end;  /* end loop for if a marker belongs in a linkage group */
		%end;   /* end loop over linkage groups */
	%end;
	%else %do;
	
		%macro LoopOverLGs_Recursion(nGrps_=, LGrpPrefix_=, InDat_=, CopyDat_=, Skeleton_=);
		
			%put ;
			%put Processing recursion number &nRecu on &nGrps_ linkage groups;
			
			%if &nRecu > &maxrecu %then %do;
				%if &sflag %then %do;
					data Outlib.&LinkageMapOrder_OutDataExName;
						set singletons_all;
					run;
				%end;
				%if &umpflag %then %do;

					%let noutdims= 0;
					%if %sysfunc(exist(outdims)) %then %do;
						proc sort data=outdims out=tmp_outdims nodupkey;
							by &LGroupVar;
						run;
						%NObsVars(tmp_outdims);
						%let noutdims= &nobs;
					%end;

					%let noutex= 0;
					%if %sysfunc(exist(OutLib.&LinkageMapOrder_OutDataExName)) %then %do;
						%let _dsid_ = %sysfunc(open(OutLib.&LinkageMapOrder_OutDataExName));
						%let noutex = %sysfunc(attrn(&_dsid_,NOBS));
						%let _rc_ = %sysfunc(close(&_dsid_));
					%end;

					data all_unmappedm;
						set all_unmappedm;
						newLGroup= &noutdims + &noutex + _n_;
						%if %UPCASE(&LGType) = C %then %do;	
							&LGroupVar = trim(left("S"))||"_"||trim(left(newLGroup));
						%end;
						%else %do;
							&LGroupVar = newLGroup;
						%end;
						_ordunm_= _n_;
					run;

					proc sql UNDO_POLICY=NONE;
						create table all_unmappedm (drop= _idvar_ _ordunm_) as
						select a.*, b.*
						from all_unmappedm as a left join copydat(keep= &IDVar &ColorVar &KeepVars orig_order rename=(&IDVar= _idvar_)) as b 
						on a.&IDVar=b._idvar_
						order by _ordunm_;
					quit;

					%if %sysfunc(exist(OutLib.&LinkageMapOrder_OutDataExName)) %then %do;
						data OutLib.&LinkageMapOrder_OutDataExName;
							set OutLib.&LinkageMapOrder_OutDataExName
							    all_unmappedm (keep= &IDVar &ColorVar &KeepVars Original_Group orig_order &&LGroupVar %if %length(&MarkerType) %then &MarkerType; );
						run;
					%end;
					%else %do;
						data OutLib.&LinkageMapOrder_OutDataExName;
							set all_unmappedm (keep= &IDVar &ColorVar &KeepVars Original_Group orig_order &&LGroupVar %if %length(&MarkerType) %then &MarkerType; );
						run;		
					%end;				
				%end;
			
				%put ERROR: The number of recursion calls &nRecu exceeded the maximum allowed of &maxrecu .;
				%let exiterror= 1;
				%goto exitL;
			%end;
			
			%if &debugrec1 %then %do;
				data outlib.InDat_&nRecu; set &InDat_; run;
			%end;
			%if &debugrec %then %do;
				data outlib.Copydat_&nRecu; set &CopyDat_; run;
				%if %length(&SkOrderVar) %then %do;
					data outlib.Skeleton_&nRecu; set &Skeleton_; run;
				%end;
			%end;
			
			%local i;
			%let i= 1;
			*loop over groups to do marker ordering and calculate genetic distance*;
			%*do i=1 %to &nGrps_;
				%put Processing Linkage Group &&&LGrpPrefix_&i;
				
				%*global OLGrp&i;
				%*let OLGrp&i = &&&LGrpPrefix_&i;
				%if %UPCASE(&LGType) = C %then %do;
					%let &LGrpPrefix_&i = "&&&LGrpPrefix_&i";
				%end;

				%if &debugrec1 %then %do;
					data outlib.InDat_&nRecu._&&&LGrpPrefix_&i; set &InDat_; run;
				%end;
				%if &debugrec %then %do;
					data outlib.Copydat_&nRecu._&&&LGrpPrefix_&i; set &CopyDat_; run;
					%if %length(&SkOrderVar) %then %do;
						data outlib.Skeleton_&nRecu._&&&LGrpPrefix_&i; set &Skeleton_; run;
					%end;
				%end;

				*make data set of linkage group and list of the variables to keep for this linkage group for MDS processing data set*;	
				%let niter_=;
				data _lg; 
					set &InDat_ end=_e_;
					where &LGroupVar = &&&LGrpPrefix_&i;  
					call symput("MVar"||trim(left(_n_)), trim(left(&IDVar)));
					if _e_ then do;
						call symput("LGtot",trim(left(_n_)));
						call symput("OrigLGroup",trim(left(_OrigLGroupVar_))); 
						%if %UPCASE(&LGType) = N %then 
							call symput("niter_",trim(left(&LGroupVar)));;
					end;
				run;
				%put LGtot= &LGtot;

				%if %UPCASE(&LGType) = C %then %do;
					%let OrigLGroup = "&OrigLGroup";
				%end; 

				*get current copy data for linkage group merging.  need to break up so that can handle and change it for new groups*;
				data currcopydat;
					set &Copydat_;
					where &LGroupVar = &&&LGrpPrefix_&i; 
				run;
				*changed copydat reference in macro to currcopydat, break this up for new groups before recall of macro*;

				%if %length(&SkOrderVar) %then %do;
					data skdata_lg; 
						set &Skeleton_;
						where &LGroupVar = &&&LGrpPrefix_&i;
					run;
				%end;

				*call macro defined above to order markers or detect singletons*;
				%*OrderMarkers(&&&LGrpPrefix_&i, &i, OrigGrp = &OrigLGroup);
				%OrderMarkers(&&&LGrpPrefix_&i, &niter_, OrigGrp = &OrigLGroup);
				%if &exiterror = 1 %then %goto exitL;

				%if &OrderCreated = 1 %then %do;
					*Kemicl check if too large of distance, if so, split groups and rerun order*;
					%if &BreakMaxCheck %then %do;

						%let EverBroke = 1;
						%let OrigGrpValue = &&&LGrpPrefix_&i;

						data newgps;
							*set dims(Keep = &IDVar &LGroupVar &BreakGrpsVar) end=last;
							set dims(Keep = &IDVar &LGroupVar &BreakGrpsVar %if %length(&MarkerType) %then &MarkerType;) end=last; 
							Original_Group = &LGroupVar;
							if _n_ = 1 then do;
								%if %UPCASE(&LGType) = C %then %do;
									newLGroup = 1;
									NewNumLGrp = &bngrps;
								%end;
								%else %do;
									newLGroup = &bngrps;
								%end;
							end;
							retain NewLGroup;
							%if %UPCASE(&LGType) = C %then retain NewNumLGrp;;
							if &BreakGrpsVar > &BreakGrpsValue then do;
								NewLGroup+1;
								%if %UPCASE(&LGType) = C %then NewNumLGrp+1;;
							end;
							%if %UPCASE(&LGType) = C %then %do;
								&LGroupVar = trim(left(&LGroupVar))||"_"||trim(left(NewLGroup));
							%end;
							%else %do;
								if newLGroup ne &bngrps then &LGroupVar = newLGroup;
							%end;
							%if %UPCASE(&LGType) = C %then %do;
								if last then  call symput("bnGrps",trim(left(NewNumLGrp)));
							%end;
							%else %do;
								if last then  call symput("bnGrps",trim(left(NewLGroup)));
							%end;					
							*keep &IDVar &LGroupVar NewNumLGrp NewLGroup;
							keep &IDVar &LGroupVar %if %UPCASE(&LGType) = C %then NewNumLGrp; NewLGroup %if %length(&MarkerType) %then &MarkerType; ;
						run;
						
						%if &debugrec1 %then %do;
							data outlib.dims&nRecu._&&&LGrpPrefix_&i; set dims;run;
							data outlib.newgps&nRecu._&&&LGrpPrefix_&i; set newgps; run;
						%end;

						*create macro variables for new groups to loop through for ordering from current original group*;
						%let NnewGrps = ;
						%GetLevels(_Ds =newgps,_Var=&LGroupVar,_LPrefix = NLGrp,_nLvls=NnewGrps,_vType= nLGType);
						%put New Number of Groups in the recursive call &nRecu = &BnGrps;
						%put There are &NnewGrps new Linkage Groups for ordering broken from &&&LGrpPrefix_&i in the recursive call &nRecu .;

						*get values of the new linkage group numbers for the iteration count*;
						%if %UPCASE(&LGType) = C %then %do;
							%let newitervar = NewNumLGrp;
							%let tmpdropvars = NewNumLGrp NewLGroup;
						%end;
						%else %do;
							%let newitervar = NewLGroup;
							%let tmpdropvars = NewLGroup;
						%end;

						proc sort data=newgps out=getiter nodupkey;
							by &newitervar;
						run;

						data _null_; 
							set getiter;
							call symput("niter"||trim(left(_n_)),trim(left(&newitervar)));
						run;
						
						%if &debugrec %then %do;
							data outlib.getiter&nRecu._&&&LGrpPrefix_&i; set getiter; run;
						%end;

						data newlgdata;
							if 0 then set newgps(drop = &tmpdropvars);
							if _N_ = 1 then do;
								declare hash h(dataset:"newgps");
								h.defineKey("&IDVar");
								h.defineData("&IDVar","&LGroupVar");
								h.defineDone();
							end;
							set _lg(drop = &LGroupVar);
							if h.find() = 0;
						run;
						
						%if &debugrec1 %then %do;
							data outlib.newlgdata&nRecu._&&&LGrpPrefix_&i; set newlgdata; run;
						%end;

						data newcopydat;
							if 0 then set newgps(drop = &tmpdropvars);
							if _N_ = 1 then do;
								declare hash h(dataset:"newgps");
								h.defineKey("&IDVar");
								h.defineData("&IDVar","&LGroupVar");
								h.defineDone();
							end;
							set currcopydat(drop = &LGroupVar);
							if h.find() = 0;
						run;
						
						%if &debugrec %then %do;
							data outlib.newcopydat&nRecu._&&&LGrpPrefix_&i; set newcopydat; run;
						%end;

						%if %length(&SkOrderVar) %then %do;
							data newskdat;
								if 0 then set newgps(drop = &tmpdropvars %if %length(&MarkerType) %then &MarkerType;);
								if _N_ = 1 then do;
									declare hash h(dataset:"newgps");
									h.defineKey("&IDVar");
									h.defineData("&IDVar","&LGroupVar");
									h.defineDone();
								end;
								set skdata_lg(drop = &LGroupVar);
								if h.find() = 0;
							run;
							
							%if &debugrec %then %do;
								data outlib.newskdat&nRecu._&&&LGrpPrefix_&i; set newskdat; run;
							%end;
						%end;	

						*collecting unanalyzed data to feedback into the recursion*;
						data NewInDat_; 
							set &InDat_;
							where &LGroupVar ^= &&&LGrpPrefix_&i;  
						run;
						
						%if &debugrec %then %do;
							data outlib.NewInDat_&nRecu._&&&LGrpPrefix_&i; set NewInDat_; run;
						%end;

						data NewCopydat_;
							set &Copydat_;
							where &LGroupVar ^= &&&LGrpPrefix_&i; 
						run;
						
						%if &debugrec %then %do;
							data outlib.NewCopydat_&nRecu._&&&LGrpPrefix_&i; set NewCopydat_; run;
						%end;

						%if %length(&SkOrderVar) %then %do;
							data NewSkeleton_; 
								set &Skeleton_;
								where &LGroupVar ^= &&&LGrpPrefix_&i;
							run;
							
							%if &debugrec %then %do;
								data outlib.NewSkeleton_&nRecu._&&&LGrpPrefix_&i; set NewSkeleton_; run;
							%end;
						%end;

						*append new data sets to unanalyzed data to feedback into recursion*;
						%NObsVars(NewInDat_);
						%if &nobs %then %do;
							proc append base=NewInDat_ data= newlgdata;
						%end;
						%else %do;
							data NewInDat_;
								set newlgdata;
							run;
						%end;

						%NObsVars(NewCopydat_);
						%if &nobs %then %do;
							proc append base=NewCopydat_ data= newcopydat;
						%end;
						%else %do;
							data NewCopydat_;
								set newcopydat;
							run;
						%end;

						%if %length(&SkOrderVar) %then %do;
							%NObsVars(NewSkeleton_);
							%if &nobs %then %do;
								proc append base=NewSkeleton_ data= newskdat;
							%end;
							%else %do;
								data NewSkeleton_;
									set newskdat;
								run;
							%end;
						%end;	
					%end; /*end for breaking groups and recreating order and dims data */
					%else %do;
						%if &AppendStarted = 0 %then %let fcheck = 1;
						%else %let fcheck = 0;
						%BuildLMOData(first = &fcheck);				
					%end;
				%end; /*end loop for if a marker belongs in a linkage group */	
				
				%if &BreakMaxCheck %then %do;
					data &InDat_;
						set NewInDat_;
					run;
					
					data &Copydat_;
						set NewCopydat_;
					run;
					
					%if %length(&SkOrderVar) %then %do;
						data &Skeleton_;
							set NewSkeleton_;
						run;
					%end;
					
					*reset breakmax back to 0 so it does not keep looping through*;	
					%let BreakMaxCheck = 0;
				%end;
				%else %do;
				
					%if &debugrec %then %do;
						data outlib.&InDat_.in&nRecu._&&&LGrpPrefix_&i; set &InDat_; run;
					%end;

					data &InDat_;
						set &InDat_;
						where &LGroupVar ^= &&&LGrpPrefix_&i;
					run;

					%if &debugrec1 %then %do;
						data outlib.&InDat_.out&nRecu._&&&LGrpPrefix_&i; set &InDat_; run;
					%end;

					data &Copydat_;
						set &Copydat_;
						where &LGroupVar ^= &&&LGrpPrefix_&i; 
					run;

					%if &debugrec %then %do;
						data outlib.&Copydat_.out&nRecu._&&&LGrpPrefix_&i; set &Copydat_; run;
					%end;

					%if %length(&SkOrderVar) %then %do;
						data &Skeleton_; 
							set &Skeleton_;
							where &LGroupVar ^= &&&LGrpPrefix_&i;
						run;

						%if &debugrec %then %do;
							data outlib.&Skeleton_.out&nRecu._&&&LGrpPrefix_&i; set &Skeleton_; run;
						%end;
					%end;
				%end;
			
				%nobsvars(&InDat_);
				%if &debugrec1 %then %do;
					%put i= &i;
					%put nGrps_= &nGrps_;
					%put nobs= &nobs;
				%end;
				
				%if &nobs %then %do;
					*Get Levels of the Linkage Group Var for the new data (in case it is been recoded) *;
					%let NewnGrps = ;
					%let NewLGType= ;
					%GetLevels(_Ds = &InDat_, _Var=&LGroupVar, _LPrefix = NewLGrpPrefix, _nLvls=NewnGrps, _vType= NewLGType);
					%put There are &NewnGrps Linkage Groups for ordering in the recursive call &nRecu .;
					%put The linkage group variable is of type &NewLGType in the recursive call &nRecu .;
					%if &syserr>4 %then %goto exitL;
					
				%end;
				%else %do;
					%goto exitL;
				%end;
				
				*increment number of recusion calls*;
				%put nRecu= &nRecu;
				%let nRecu= %sysevalf(&nRecu + 1);
				
				*Recursive call*;
				%*LoopOverLGs_Recursion(nGrps_= &NewnGrps, LGrpPrefix_= NewLGrpPrefix, InDat_= &InDat_, CopyDat_= &Copydat_, Skeleton_= &Skeleton_); 
				%LoopOverLGs_Recursion(nGrps_= 1, LGrpPrefix_= NewLGrpPrefix, InDat_= &InDat_, CopyDat_= &Copydat_, Skeleton_= &Skeleton_); 
				
			%*end; /*end loop over linkage groups */
			%exitL:;
		%mend LoopOverLGs_Recursion;
		
		/*Get Levels of the Linkage Group Var (in case it's been recoded) */
		/* Loop over linkage groups and create data sets for each */
		%let nGrps = ;
		%*GetLevels(_Ds =indat,_Var=&LGroupVar,_LPrefix = LGrp,_nLvls=nGrps,_vType= LGType);
		%GetLevels(_Ds =&indat,_Var=&LGroupVar,_LPrefix = LGrp,_nLvls=nGrps,_vType= LGType);
		%put There are &nGrps Linkage Groups for ordering;
		%put The linkage group variable is of type &LGType;
		%if &syserr>4 %then %goto exit;
		
		/*make macro initialized to original number of groups, but if they get broke update to have new number of groups */
		%let bnGrps = &nGrps;
		
		*maximum number or recursion calls*;
		%nobsvars(&indat);
		%let nobsindat= &nobs;
		%global maxrecu;
		%let maxrecu= %sysevalf(&nobsindat*&nobsindat + 5);

		*create temporary data sets*;
		data indat_tmp;
			set &indat;
			_OrigLGroupVar_= &LGroupVar;
		run;
		
		data copydat_tmp;
			set copydat;
		run;
		
		%if %length(&SkOrderVar) %then %do;
			data skeleton_tmp;
				set skeleton;
			run;
		%end;
		
		*sbrlsi development check, change to 1 to output recursion data sets*;
		%global debugrec debugrec1;
		%let debugrec= 0;
		%let debugrec1= 0;
		
		%if &debugrec1 %then %do;
			data outlib.gendat; set gendat; run;
		%end;
		
		*counts number of recursion calls*;
		%global nRecu;
		%let nRecu= 0; 	
		
		*ordering markers using a recursion macro function*;
		%*LoopOverLGs_Recursion(nGrps_=&nGrps, LGrpPrefix_=LGrp, InDat_=indat, CopyDat_=copydat, Skeleton_=skeleton);
		%LoopOverLGs_Recursion(nGrps_= 1, LGrpPrefix_=LGrp, InDat_=indat_tmp, CopyDat_=copydat_tmp, Skeleton_=skeleton_tmp);
		%put nRecuTotal= &nRecu;
		%if &exiterror= 1 %then %goto exit;
			
	%end;	
	
	%*goto exit;
	
	%let nsig1= 0;
	%if &sflag %then %do;
		%nobsvars(singletons_all);
		%let nsig1= &nobs;
		data Outlib.&LinkageMapOrder_OutDataExName;
			set singletons_all;
		run;
	%end;
	
	%let nsig2= 0;
	%if &umpflag %then %do;
		%nobsvars(all_unmappedm);
		%let nsig2= &nobs;
	%end;
	
	%if %sysevalf(&nsig1 + &nsig2)= &nobsindat %then %do;
		%put ERROR: No linkage groups contain more than 1 marker.  All markers have been excluded from analysis.;
		%let exiterror =1;
		%goto exit;
	%end;
	
	%if &allsflag %then %do;
		%put ERROR: No linkage groups contain more than 1 marker.  All markers have been excluded from analysis.;
		%put ERROR: This process cannot be run without linkage groups containing two or more markers.;
		%let exiterror =1;
		%goto exit;
	%end;
	
	%if %sysfunc(exist(outdims))=0 %then %do;
		%put ERROR: No linkage groups contain more than 1 marker after breaking groups using the specified threshold.  All markers have been excluded from analysis.;
		%put ERROR: Please increase the value for which to break linkage groups on the Options Tab.;
		%let exiterror =1;
		%goto exit;
	%end;
	
	%let nGrps = ;
	%GetLevels(_Ds =outdims,_Var=&LGroupVar,_LPrefix = LGrp,_nLvls=nGrps,_vType= LGType);
	%put nGrpsFinal= &nGrps;
	%do ii=1 %to &nGrps;
		%put Processing Linkage Group &&LGrp&ii;
		%let OLGrp&ii = &&LGrp&ii;
		%if %UPCASE(&LGType) = C %then %do;
			%let LGrp&ii = "&&LGrp&ii";
		%end;
	%end;

	/*Luciano*/
	%put umpflag= &umpflag;
	%if &umpflag %then %do;
		%let EverBroke = 1;
		
		/*data OutLib.outdims; set outdims; run;*/
		
		%let noutdims= 0;
		%if %sysfunc(exist(outdims)) %then %do;
			proc sort data=outdims out=tmp_outdims(keep= &LGroupVar) nodupkey;
				by &LGroupVar;
			run;
			%NObsVars(tmp_outdims);
			%let noutdims= &nobs;
		%end;
		
		%let noutex= 0;
		%if %sysfunc(exist(OutLib.&LinkageMapOrder_OutDataExName)) %then %do;
   			%let _dsid_ = %sysfunc(open(OutLib.&LinkageMapOrder_OutDataExName));
      			%let noutex = %sysfunc(attrn(&_dsid_,NOBS));
      			%let _rc_ = %sysfunc(close(&_dsid_));
      			
			proc sort data=OutLib.&LinkageMapOrder_OutDataExName out=tmp_ExName(keep= &LGroupVar) nodupkey;
				by &LGroupVar;
			run;        			
		%end;
		
		%if %eval(&noutdims + &noutex) > 0 %then %do;
			%if &noutdims and &noutex %then %do;
				data tmp_data;
					set tmp_outdims;
				run;
				
				proc append base=tmp_data data=tmp_ExName; run;
				
			%end;
			%else %if &noutdims > 0 and &noutex = 0 %then %do;
				data tmp_data;
					set tmp_outdims;
				run;	
			%end;
			%else %if &noutdims = 0 and &noutex > 0 %then %do;
				data tmp_data;
					set tmp_ExName;	
				run;	
			%end;
			
			proc sort data=tmp_data out=tmp_datas(keep= &LGroupVar) nodupkey;
				by &LGroupVar;
			run; 	
			
			%if %UPCASE(&LGType) = N %then %do;
				data _null_;
					set tmp_data end= _eof_;	
					if _eof_ then call symputx("NumbLG", &LGroupVar);
				run;
			%end;
			%else %do;
				%let NumbLG= &noutdims + &noutex;
			%end;
				
		%end;
		
		data all_unmappedm;
			set all_unmappedm;
			newLGroup= &NumbLG + _n_;
			%if %UPCASE(&LGType) = C %then %do;	
				&LGroupVar = trim(left("S"))||"_"||trim(left(newLGroup));
			%end;
			%else %do;
				&LGroupVar = newLGroup;
			%end;
			_ordunm_= _n_;
		run;
		
		/*
		data all_unmappedm;
			set all_unmappedm;
			newLGroup= &noutdims + &noutex + _n_;
			%if %UPCASE(&LGType) = C %then %do;	
				&LGroupVar = trim(left("S"))||"_"||trim(left(newLGroup));
			%end;
			%else %do;
				&LGroupVar = newLGroup;
			%end;
			_ordunm_= _n_;
		run;
		*/
	
		proc sql UNDO_POLICY=NONE;
			create table all_unmappedm (drop= _idvar_ _ordunm_) as
			select a.*, b.*
			from all_unmappedm as a left join copydat(keep= &IDVar &ColorVar &KeepVars orig_order rename=(&IDVar= _idvar_)) as b 
			on a.&IDVar=b._idvar_
			order by _ordunm_;
		quit;
		
		/*data outlib.all_unmappedm2; set all_unmappedm; run;*/
	
		%if %sysfunc(exist(OutLib.&LinkageMapOrder_OutDataExName)) %then %do;
			data OutLib.&LinkageMapOrder_OutDataExName;
				set OutLib.&LinkageMapOrder_OutDataExName
				    all_unmappedm (keep= &IDVar &ColorVar &KeepVars Original_Group orig_order &&LGroupVar %if %length(&MarkerType) %then &MarkerType; );
			run;
		%end;
		%else %do;
			data OutLib.&LinkageMapOrder_OutDataExName;
				set all_unmappedm (keep= &IDVar &ColorVar &KeepVars Original_Group orig_order &&LGroupVar %if %length(&MarkerType) %then &MarkerType; );
			run;		
		%end;
		
		*reset singletons to account for unmapped markers*;
		%let nGrpsS=;
		%let LGTypeS=;
		%GetLevels(_Ds =Outlib.&LinkageMapOrder_OutDataExName,_Var=&LGroupVar,_LPrefix = LGrpS,_nLvls=nGrpsS,_vType= LGTypeS);
		%let Singletons= ;
		%do i=1 %to &nGrpsS;
			%let Singletons = &Singletons "&&LGrpS&i";
		%end;
		%put Singletons= &Singletons;
		%let sflag=1;
	%end;
	/*Luciano*/
	
	/* kemicl added so we don't have label issues for main data */
	options nolabel;
	
	%if ^%length(&OrderVar) and %index(%nrbquote(%upcase(&Algorithm)),MULTIDIMENSIONAL)=0 %then %let OrderVar = Map_Order;
	
	%if ^%length(&OrderVar) and %index(%nrbquote(%upcase(&Algorithm)),MULTIDIMENSIONAL)>0  %then %do;
		/* sum up absolute value of residuals to color by as indication of poorly fitted markers */
		data absres; set outres;
			keep _ROW_ _COL_ Residual;
			Residual = abs(Residual);
		run;
	
		proc sort data=absres;
			by _ROW_;
		run;
	
		proc means data = absres sum noprint;
			by _ROW_;
			var Residual;
			output out = rowsums(keep = _ROW_ srow) sum=srow;
		run;
	
		proc sort data=absres;
			by _COL_;
		run;
	
		proc means data = absres sum noprint;
			by _COL_;
			var Residual;
			output out = colsums(keep = _COL_ scol) sum=scol;
		run;
	
		proc sql UNDO_POLICY=NONE;
			create table rsums as 
				select *
				from rowsums as d1 full join colsums as d2
				on d1._ROW_ = d2._COL_;
		quit;
	
		data rsums;
			set rsums;
			if missing(srow) then srow = 0;
			if missing(scol) then scol = 0;
			Sum_AbsResidual = srow + scol;
			if missing(_ROW_) then _ROW_ = _COL_;
			drop _COL_ scol srow;
		run;
	
		proc sql UNDO_POLICY=NONE;
			create table Outlib.&LinkageMapOrder_OutDataName(drop=_ROW_ Init_Map_Order) as 
				select *
				from outdims d1, rsums d2
				where d1.&IDVar = d2._ROW_;
		quit;
		
		/* for plots in JMP to color by if no colorvar*/
			%if ^%length(&ColorVar) %then %do;
				%let ColorVar = Sum_AbsResidual;
			%end;
	%end;
	%else %do;
		/*data Outlib.&LinkageMapOrder_OutDataName; set outdims; drop Opt_Order; run;*/
		data Outlib.&LinkageMapOrder_OutDataName; set outdims; run;
		
		%if ^%length(&ColorVar) %then %do;
			%if &EverBroke %then %let ColorVar = Original_Group;
			%else %let ColorVar = &LGroupVar;
		%end;
		/* %CheckLabel(outdims, ColorLabel); */
	%end;
	
	%if ^%length(&OrderVar) and %index(%nrbquote(%upcase(&Algorithm)),MULTIDIMENSIONAL)>0  %then %do;
	/* columns to keep when order was was specified and output is from MDS */
		%let keepcols1 = _ROW_, _COL_, Recombination, FitData, FitDist, Weight, Residual, Locus1, O1;
		%let keepcols2 = Marker1, _COL_, Recombination, FitData, FitDist, Weight, Residual, &LGroupVar, Locus1, Locus2, O1, O2;
	%end;
	%else %do;
		%let keepcols1 = _ROW_, _COL_, Recombination, Locus1, O1;
		%let keepcols2 = Marker1, _COL_, Recombination, &LGroupVar, Locus1, Locus2, O1, O2;
	%end;
		
	proc sql UNDO_POLICY=NONE;
		create table res1(rename = (_ROW_ = Marker1)) as 
			select &keepcols1
			from outres as d1, outdims (rename = (cm_&MapFunc = Locus1 Map_Order = O1)) as d2 
			where d1._Row_ = d2.&IDVar;
			
		create table res2(rename = (_COL_ = Marker2)) as 
			select &keepcols2
			from res1 as d1, outdims (rename = (cm_&MapFunc = Locus2 Map_Order = O2)) as d2 
			where d1._COL_ = d2.&IDVar;
		
		/*Luciano*/
		create table res3 as
			select a.*, b.MarkerType1
			from res2 as a, outdims(rename=(&MarkerType=MarkerType1)) as b
			where a.Marker1 = b.&IDVar;
		
		create table res as
			select a.*, b.MarkerType2
			from res3 as a, outdims(rename=(&MarkerType=MarkerType2)) as b
			where a.Marker2 = b.&IDVar;			
		/*Luciano*/
	quit;
	
	data matchmark; 
		set outdims(rename = (&IDVar = Marker1 cm_&MapFunc = Locus1 Map_Order = O1 %if %length(&MarkerType) %then &MarkerType= MarkerType1;)); 
		set outdims(rename = (&IDVar = Marker2 cm_&MapFunc = Locus2 Map_Order = O2 %if %length(&MarkerType) %then &MarkerType= MarkerType2;));
		Recombination = 0;
		*keep Marker1 Marker2 Locus1 Locus2 Recombination &LGroupVar O1 O2;
		keep Marker1 Marker2 Locus1 Locus2 Recombination &LGroupVar O1 O2 %if %length(&MarkerType) %then MarkerType1 MarkerType2; ;/*Luciano*/
	run;

	%*goto exitcheck;
	
	data res;
		%if %length(&MarkerType) %then 
			length Marker1 Marker2 $50 MarkerType1 MarkerType2 $32; ;/*Luciano*/
		set res matchmark;
		if O1 > O2 then do;
			tmp = O1;
			O1 = O2;
			O2 = tmp;
			tmp = Locus1;
			Locus1 = Locus2;
			Locus2 = tmp;
			tmp1 = Marker1;
			Marker1 = Marker2;
			Marker2 = tmp1;
			/*Luciano*/
			%if %length(&MarkerType) %then %do;
				tmp1 = MarkerType1;
				MarkerType1= MarkerType2;
				MarkerType2= tmp1;
			%end;
			/*Luciano*/
		end;
		drop tmp tmp1;
	run;
	
	proc sort data=res out=Outlib.&LinkageMapOrder_OutDataResName;
		by &LGroupVar O1 O2;
	run;

	/* resort data if breakgroups was possible */
	%if &EverBroke %then %do;
		proc sort data=Outlib.&LinkageMapOrder_OutDataName;
			by &LGroupVar Map_Order;
		run;
		
		%GetLevels(_Ds =Outlib.&LinkageMapOrder_OutDataName,_Var=&LGroupVar,_LPrefix = LGrp,_nLvls=nGrps,_vType= LGType);
		%do i=1 %to &nGrps;
			%put Processing Linkage Group &&LGrp&i;
			%let OLGrp&i = &&LGrp&i;
			%if %UPCASE(&LGType) = C %then %do;
				%let LGrp&i = "&&LGrp&i";
			%end;
		%end;
	%end;
	
	%if ^%length(&OrderVar) and %index(%nrbquote(%upcase(&Algorithm)),MULTIDIMENSIONAL)>0 %then %do;
		/* Get the badness of fit and fit correlation into macro variables */
		data _null_; set outfit;
			CRITER = Round(CRITER,0.001);
			FitCorr = Round(FitCorr,0.001);
			call symput("BoF"||trim(left(_n_)),trim(left(CRITER)));
			call symput("Corr"||trim(left(_n_)), trim(left(FitCORR)));
		run;
	%end;
	
	/* get maxpos and minpos to switch axes with */
	proc means data = outdims noprint;
		var cM_&MapFunc;
		output out = mp max = maxpos;
	run;
	
	data _null_;
		set mp;
		call symput("maxpos",trim(left(maxpos+11)));
	run;
	%let minpos = -11;
	
	%if &sflag %then %do;
		%PackageInsertDataset(Excluded Markers from &ProcessName, OutLib, &LinkageMapOrder_OutDataExName,no);
	%end;
	
	data Outlib.&LinkageMapOrder_OutDataName;
		set Outlib.&LinkageMapOrder_OutDataName;
		label A1P1_= "Allele 1 of Parent 1";
		label A2P1_= "Allele 2 of Parent 1";
		label A1P2_= "Allele 1 of Parent 2";
		label A2P2_= "Allele 2 of Parent 2";
	run;
	
	/* create macro vars for the maxposition of each linkage group*/
	/* create new lgrp macros and number of lgrps if option to break linkage groups enabled*/
	data checkmax;
		set Outlib.&LinkageMapOrder_OutDataName(keep=&LGroupVar cm_&MapFunc);
		by &LGroupVar;
		if last.&LGroupVar;
	run;
	
	data _null_;
		set checkmax;
		call symput("LGMax"||trim(left(_n_)),trim(left(cm_&MapFunc)));
	run;
	
	/* Create all the macro variables of the markers to use in cell plots */
	/* also use lgtot for reversal of maps */
	%do i=1 %to &nGrps;
		%if %index(&Singletons,"&&LGrp&i")= 0 %then %do;
			data _null_;
				set Outlib.&LinkageMapOrder_OutDataName(keep=&IDVar &LGroupVar) end=_e_;
				where &LGroupVar = &&LGrp&i;
				%if %length(&SNPInData) and %upcase(&DoCellPlots) = YES %then %do;
					call symput("MVar&i"||trim(left(_n_)), trim(left(&IDVar)));
				%end;
				if _e_ then call symput("LGtot&i",trim(left(_n_)));
			run;
			/* check list 
				%if %index(&Singletons,"&&LGrp&i")= 0 %then %do;
					%do m=1 %to &&LGtot&i;
					%put &&&MVar&i&m;
					%end;
				%end;
			*/
		%end;
	%end;
	
	%*goto exitcheck;
	
	* create JSL file;
	%PrepOutFile(LinkageMapOrderScript,LinkageMapOrderScriptName,&ProcessName Script for &OutDataPrefix..jsl);
	
	%*global _mainjslfile_ FullAPName;
	%global _mainjslfile_;
	%let _mainjslfile_ = &LinkageMapOrderScript;
	%*let FullAPName=%str(Linkage Map Order for Outcross);
	
	data _null_;
	   	file "&LinkageMapOrderScript" lrecl=32767;
		put "//!";
		put "Names Default to Here(1);";
		put "Here:DataList = {};";
		put "Here:TabButtonList = {};";
		put "Here:TabCodeList = {};";
		put "Here:TabAfterCodeList = {};";
		put "Here:TabOpenList = {};";
		put "vfsize = 650;";
		put "hfsize = 300;";
		put "OutPath = ""&OutPath"";";
		put "ClientOutPath = ""&ClientOutPath"";";
		put "OutDataName = ""&LinkageMapOrder_OutDataName"";";
		put "OutDataResName = ""&LinkageMapOrder_OutDataResName"";";
		put "GroupVar = ""&LGroupVar"";";
		put "LGType = ""&LGtype"";";
		put " if(host is(Windows),";
		put "   include( ConvertFilePath(""$GENOMICS_HOME/JSLFiles/Linkage Group Expressions.jsl"", absolute, windows));";
		put " ,";
		put "   include( ConvertFilePath(""$GENOMICS_HOME/JSLFiles/Linkage Group Expressions.jsl"", absolute, POSIX));";	
		put " );";	
		put "Here:GB_results = Expr(";
		put " Here:gb = Graph Builder(";
		put "		Size(625, 525),";
		put "		Show Control Panel(0),";
		put "		Auto Stretching( 0 ),";
		put "		Variables(";
		put "			X(:Name(""&LGroupVar"")),";
		put "			Y(:Name(""cm_&MapFunc"")),";
		put " 		Color(:Name(""&ColorVar""))";
		put "		),";
		put "		Elements(Points(X,Y,Color,Legend(4), Jitter( 0 ))),";
		put "		SendToReport(";
		put "			Dispatch({}, ""graph title"", TextEditBox,";
		put "				{Set Text(""Genetic Distances For Linkage Groups"")}";
		put "			),";
		put "			Dispatch({}, ""cm_&MapFunc"", ScaleBox, {Min( &maxpos), Max( &minpos), Rotated Labels(""Automatic"")}),";
		put "			Dispatch({}, ""Graph Builder"", FrameBox, {Marker Size(3)})";
		put "		)";
		put "	);";
		put ");";
		put "Here:DF_results = Expr(";
		put "	Here:df = odat << Data Filter(";
		put "		Add Filter(";
		put "			Columns(";
		%if &EverBroke %then %do;
			put "			:Name(""Original_Group""), ";
		%end;
		put "			:Name(""cM_Step""), :Name(""RecombinationF""))";
		put "		)";
		put " );";
		put ");";
		
		put "Here:GBfunc = Function({},{default local},";
		put "	If(IsEmpty(Here:odat),";
		put " 	Here:odat = open(""&ClientOutPath.&LinkageMapOrder_OutDataName..sas7bdat"",invisible);";
		put "		Column(""&LGroupVar"")<<Set Modeling Type(Nominal);";
		%if &EverBroke %then %do;
			put "		Column(""Original_Group"")<<Set Modeling Type(Nominal);";
		%end;
		%if %length(&SkOrderVar) %then %do;
			put "		Here:odat << Select Where(!Is Missing(:Name(""&SkOrderVar"")[]));";
			put "		Here:odat << Markers(11);";
		%end;
		put "		Column(""&IDVar"")<<Label(1);";
		put " ,"; /* else */
		put "		Current Data Table(Here:odat);";
		put "	);";
		put "	VListBox(";
		%if &sflag %then %do;
			put "		TextBox(""  NOTE: Some Linkage Groups contained only one marker. These markers are excluded from the analysis and output map.    A data set of the excluded markers can be found under the Output Data tab."", <<Set Wrap(715)),";
		%end;
		%if %length(&SkOrderVar) %then %do;
				put "		TextBox(""  Framework Markers are denoted with a star."", <<Set Wrap(415)),";
		%end;
		put " Here:LO_HB = HListBox(";
		put "		Here:GB_results, Here:DF_results;";
		put " ));"; /* Hlistbox and VlistBox */
		
		put ");"; /* Close Function */
		
		put "InsertInto(DataList,{{""Here:odat""}});";
		put "InsertInto(TabButtonList, ""Linkage Order Results"");";
		put "InsertInto(TabCodeList, expr(Here:GBfunc()));";
		put "InsertInto(TabAfterCodeList, "" "");";
		put "InsertInto(TabOpenList,1);";
		%if ^%length(&OrderVar) and %index(%nrbquote(%upcase(&Algorithm)),MULTIDIMENSIONAL)>0  %then %do;
			put "Here:Rfunc = Function({},{default local},";
			put "	If(IsEmpty(Here:rdat),";
			put " 	Here:rdat = open(""&ClientOutPath.&LinkageMapOrder_OutDataResName..sas7bdat"",invisible);";
			put "		Column(""&LGroupVar"")<<Set Modeling Type(Nominal);";
			put "		Column(""Marker1"")<<Label(1);";
			put "		Column(""Marker2"")<<Label(1);";
			put " ,"; 
			put "		Current Data Table(Here:rdat);";
			put "	);";
			put "	VListBox(";
			%do i = 1 %to &nGrps;
				%if %index(&Singletons,"&&LGrp&i")= 0 %then %do; 
					put "  OutlineBox(""Linkage Group &&OLGrp&i"",";
					put "  	TextBox(""Recombination Fraction and Fitted Distance Correlation = &&Corr&i"", <<Set Wrap(600)),";
					put "   	TextBox(""MDS Model Badness of Fit (lower is better) = &&BoF&i"", <<Set Wrap(600)),";
					put "  	biv&i = Bivariate(";
					put "			Y(:Name(""Residual"")), X(:Name(""FITDIST"")),";
					%if %upcase(&LGType) = C %then %do;
						put "			Where (:Name(""&LGroupVar"") == ""&&OLGrp&i""),";
					%end;
					%else %do;
						put "			Where (:Name(""&LGroupVar"") == &&LGrp&i),";
					%end;				
					put "			Fit Spline(1), Histogram Borders(1),";
					put "			SendToReport(";
					put "				Dispatch({}, ""Bivar Plot"", FrameBox, {Frame Size(375,225)}),";
					put "				Dispatch( {}, ""Smoothing Spline Fit"", OutlineBox, {Close( 1 )} )";
					put "			)";
					put "	 	)";
					put "  ),";
				%end;
			%end;
			put "));";
	
			put "InsertInto(DataList,{{""Here:rdat""}});";
			put "InsertInto(TabButtonList, ""MDS Residual Plots"");";
			put "InsertInto(TabCodeList, expr(Here:Rfunc()));";
			put "InsertInto(TabAfterCodeList, "" "");";
			put "InsertInto(TabOpenList,0);";	
			put "Here:TabsSectionFlag = 1;";
		%end;
		/* Kemicl added tab to show cell plots Jan192012*/
		%if %upcase(&DoCellPlots) = YES %then %do;
			put "Here:CPfunc = Function({},{default local},";
			put "	If(IsEmpty(Here:sdat),";
			put " 	Here:sdat = open(""&SNPInData"",invisible);";
			/*Force to use SAS names so markers match*/
			put "		cols = Here:sdat  << GetColumnNames();";
			put "		for(i=1,i<=NItems(cols),i++,";
			put "			ci = Column(cols[i]);";
			put "			sasnm = char(ci << GetProperty(""SAS Name""));";
			put "			if(length(sasnm),ci << SetName(sasnm))";
			put "		);";
			%do i=1 %to &nGrps;
				%if %index(&Singletons,"&&LGrp&i")= 0 %then %do;
					%do m=1 %to &&LGtot&i;
						put "  Column(""&&&MVar&i&m"")<<Set Modeling Type(""Ordinal"");";		 
						put "  Column(""&&&MVar&i&m"")<<Set Property(""Value Colors"",{""&code2""=21,""&code11""=1, ""&code12"" = 1, ""&code0""=19});"; 
					%end;
				%end;
			%end;
			%if %length(&LabelVar) %then %do;
				put "		Column(""&LabelVar"")<<Label(1);";
			%end;
			put " ,"; 
			put "		Current Data Table(Here:sdat);";
			put "	);";
			put " HListBox(";
			put "VListBox(";
			put "            LineUpBox(NCol(2),";
			put "               Text Box( "" Vertical Frame Size "" ),";
			put "         		  vframeSizeSlider = SliderBox(num(200),num(1500),vfsize,";
			%do i=1 %to &nGrps;
				%if %index(&Singletons,"&&LGrp&i")= 0 %then %do;
				put "	try((Here:cp&i<<report)[CellPlotBox(1)] << FrameSize(hfsize,vfsize);); ";
				%end;
			%end;
			put "					  ),";
			put "               Text Box( "" Horizontal Frame Size "" ),";
			put "         		  hframeSizeSlider = SliderBox(num(100),num(800),hfsize,";
			%do i=1 %to &nGrps;
				%if %index(&Singletons,"&&LGrp&i")= 0 %then %do;
				put "								try((Here:cp&i<<report)[CellPlotBox(1)] << FrameSize(hfsize,vfsize); );";
				%end;
			%end;
			put "					  ),";
			put "            ),";	
			%if ^%index(%upcase(&MarkerFmt),OUTCROSS) %then %do;
				put "panelbox(""Legend"",";
				put "   t1=Textbox(""Homozygous for Parental Line 1""),";
				put "   t1<<Font Color(3),";
				put "   t1<<Bullet Point(1);";
				put "   t2=Textbox(""Heterozygous""),";
				put "   t2<<Font Color(1),";
				put "   t2<<Bullet Point(1);";
				put "   t3=Textbox(""Homozygous for Parental Line 2""),";
				put "   t3<<Font Color(5),";
				put "   t3<<Bullet Point(1);";
				put "   t4=Textbox(""x  Missing value""),";
				put ")";
			%end;
			put "),";
			%do i=1 %to &nGrps;
				%if %index(&Singletons,"&&LGrp&i")= 0 %then %do;
					put "		Here:cp&i =  Cell Plot( ScaleUniformly(1), Center at Zero(0),";
					put "			Y(";
					%do m=1 %to &&LGtot&i;
						put "			:Name(""&&&MVar&i&m""),";
					%end;
					put "			),";
					%if %length(&LabelVar) %then %do;
					put "			Label(:Name(""&LabelVar"")),";
					%end;
					put "			SendToReport(";
					put "				Dispatch({},""Cell Plot"",OutlineBox,";
					put "					{Set Title(""Genotype Color Plot for Linkage Group &&OLGrp&i"")}";
					put "				),";
					%if &&LGtot&i > 50 %then %do;
						put "				Dispatch( {}, """", CellPlotBox, {Frame Size(300, 650), No separator lines(1)})";
					%end;
					%else %do;
						put "				Dispatch( {}, """", CellPlotBox, {Frame Size(300, 650)})";
					%end;
					put "			)";
					put "		);";
				%end;
			%end;
			put " ));";
			
			put "InsertInto(DataList,{{""Here:sdat""}});";
			put "InsertInto(TabButtonList, ""Genotype Color Plots"");";
			put "InsertInto(TabCodeList, expr(Here:CPfunc()));";
			put "InsertInto(TabAfterCodeList, "" "");";
			put "InsertInto(TabOpenList,1);";
		%end;
		put "InsertInto(DataList,{{"""" }});";
		%let tmpcount = 1;
		%*put nGrps= &nGrps;
		%*put Singletons= &Singletons;
		%do i = 1 %to &nGrps;
			%*put LGrp&i= &&LGrp&i;
			%if %index(&Singletons,"&&LGrp&i")= 0 %then %do;
				put "ldhvp&i = Function({},{default local},";
				put "	If(IsEmpty(Here:rdat),";
				put " 	Here:rdat = open(""&ClientOutPath.&LinkageMapOrder_OutDataResName..sas7bdat"",invisible);";
				put "		Column(""&LGroupVar"")<<Set Modeling Type(Nominal);";
				put "		Column(""Marker1"")<<Label(1);";
				put "		Column(""Marker2"")<<Label(1);";
				put " ,"; 
				put "		Current Data Table(Here:rdat);";
				put "	);";
				put "	Here:rdat << Clear Column Selection();";
				put "	Here:rdat << Clear Select;";
				put " If(IsEmpty(Here:data&i),";
					%if %upcase(&LGType) = C %then %do;
						put "	Here:data&i = Subset(Where(:Name(""&LGroupVar"") == ""&&OLGrp&i""), Output Table Name(OutDataResName||""_&&OLGrp&i""));";
					%end;
					%else %do;
							put "	Here:data&i = Subset(Where(:Name(""&LGroupVar"") == &&LGrp&i), Output Table Name(OutDataResName||""_&&LGrp&i""));";
					%end;
				put "	,";
				put "		Current Data Table(Here:data&i);";
				put ");";
				put "Here:dt = Today();";
				put "Here:_ag_=""for Linkage Group &&OLGrp&i"";";
				put "maxrows = 100000;";
				put "if(nrows(Here:data&i)<=maxrows,";
				put " Here:ld_i=1;";
				put " jg:theme_index = 8;";
				put " Here:MeasureVar = ""Recombination"";";
				put " Here:BlocksDefined = 0;";		
				put " Here:currchr = ""&i"";";	
				put "	Here:currby = ""1"";";
				put " Here:ExtraTitle = """";";
				put " Eval(Parse(Eval(ldexprexpr)));";
	
				put ", //else too big";
				put " errwin=New Window(""JMP Genomics Message"",";
				put "  TextBox(""You cannot view the Linkage Order and Recombination plot for more than ""||char(maxrows)||"" pairs.""));";
				put ' errwin << SetWindowIcon("$LS_HOME/Documentation/Icons/JMPGenomicsApp.ico");';				
				put ");";
				put "); //end func";
				%if &i = 1 %then %do;
					put "InsertInto(DataList,{{""Here:rdat""}});";
				%end;
				put "InsertInto(DataList,{{""Here:data&i"" }}) ;";
				put "_ag_buttonname=""Linkage Group &&OLGrp&i"";";
				%if &tmpcount < 2 %then %do;
					put "if(isEmpty(Here:ActionButtonDesc) | NItems(Here:ActionButtonDesc)==0,";
					put "   Here:ActionButtonList = {{""""}};";
					put "   Here:ActionCodeList = {{""""}};";
					put "   Here:ActionButtonDesc = {0};";	
					put "   LArryIndex=1;";
					put ", //else";
					put "   LArryIndex=NItems(Here:ActionButtonList)+1;";
					put "   InsertInto(Here:ActionButtonList,{{""""}});";
					put "   InsertInto(Here:ActionCodeList,{{""""}});";
					put "   InsertInto(Here:ActionButtonDesc,0);";					
					put ");";
					put "Here:ActionButtonDesc[LArryIndex]=""Launch Interactive Order and Recombination Plots"";";
				%end;
				%if &tmpcount>1 %then %do;
					%if &tmpcount>2 %then %do;
						put "InsertInto(Here:ActionButtonList[LArryIndex],0);";
						put "InsertInto(Here:ActionCodeList[LArryIndex],{{""""}});";				
					%end;
					put "Here:ActionButtonList[LArryIndex][&tmpcount]= _ag_buttonname;";
					put "Here:ActionCodeList[LArryIndex][&tmpcount]=expr(ldhvp&i());";
				%end;
				%else %do;
					put "Here:ActionButtonList[LArryIndex][1]= _ag_buttonname;";
					put "Here:ActionCodeList[LArryIndex][1]=expr(ldhvp&i());";				
				%end;
				%let tmpcount = %eval(&tmpcount+1);
			%end;
		%end;
		
		/* kemicl Jan242012 added work to make interactive triangle plots reorder for reversal*/
		put "Here:RevGroups = Function({},{default local},";
			%if &nGrps > 1 %then %do;
				put " aglist = { ""_All_"" " @;
			%end;
			%else %do;
				put "	aglist = { " @;
			%end;
				%do jj=1 %to &nGrps;
					%if %index(&Singletons,"&&LGrp&jj")= 0 %then %do; 
						put ", ""&&OLGrp&jj"" " @;
					%end;
				%end;
				put "};";
				/* create a list of grps and max gds to pass to sas*/
				put " maxdlist = {};";
				put "	nmarklist = {};";
				put " grplist = {};";
				%let maxi = 1;
				%do jj=1 %to &nGrps;
					%if %index(&Singletons,"&&LGrp&jj")= 0 %then %do;
						put " InsertInto(maxdlist, &&&&LGMax&&maxi);";
						put "	InsertInto(nmarklist, &&LGtot&jj);";
						%if &LGType = C %then %do;
							put "	InsertInto(grplist, ""&&OLGrp&jj"");";
						%end;
						%else %do;
							put "	InsertInto(grplist, &&OLGrp&jj);";
						%end;	
						%let maxi = %eval(&maxi + 1);
					%end;	
				%end;
				/* put "Show(maxdlist, grplist);"; */
				put " dlgg = New Window(""Select the Linkage Groups for Order Reversal"",<<modal,";
				put "   vlistbox(";
				put "    cglistb=listBox(aglist),";
				put "    buttonbox(""OK"")));";
				put " cglist=cglistb<<Get Selected;";
				put "	selindex = cglistb<<Get Selected Indices;";
				put "	try(Current Data Table(Here:odat), Throw(""The Linkage Map Order results data set is not open.""));";
				put "	//show(cglist); show(current data table());";
				put " if(nitems(cglist)>0, ";
				put " 	try(close(Here:rdat,nosave););";
				/* change values in open map data */
				put "		if(Contains(cglist,""_All_""),";
				/* create selected groups and maxes to ready for SAS to flip orders for interactive plots */
				put "			selgrps = grplist;";
				put "			selmax = maxdlist;";
				put "			selnmark = nmarklist;";
				put "			numsel = nitems(selgrps);";
				put "			for each row(";
				put "				cms_tmp = :cM_Step; cm_tmp = :cM_&MapFunc; ord_tmp = :Map_Order; rf_tmp = :RecombinationF;";
				put "				:cM_Step = :_cMStepR_; :cM_&MapFunc = :_cMR_; :Map_Order = :_RMO_; :RecombinationF = :_rfR_;";
				put "				:_cMStepR_ = cms_tmp; :_cMR_ = cm_tmp; :_RMO_ = ord_tmp; :_rfR_ = rf_tmp;";
				put "			);";
				put "		,";  /*else for if contains all */
				%if &nGrps > 1 %then %do;
					put "			selindex = selindex -1;";
				%end;
				put "			selgrps = grplist[selindex];";
				put "			selmax = maxdlist[selindex];";
				put "			selnmark = nmarklist[selindex];";
				put "			numsel = nitems(cglist);";
				put "			for each row(";
				put "				if(Contains(cglist,Char(:&LGroupVar)),";
				put "					cms_tmp = :cM_Step; cm_tmp = :cM_&MapFunc; ord_tmp = :Map_Order; rf_tmp = :RecombinationF;";
				put "					:cM_Step = :_cMStepR_; :cM_&MapFunc = :_cMR_; :Map_Order = :_RMO_; :RecombinationF = :_rfR_;";
				put "					:_cMStepR_ = cms_tmp; :_cMR_ = cm_tmp; :_RMO_ = ord_tmp; :_rfR_ = rf_tmp;";
				put "				);";
				put "			);";			
				put "		); "; /* close All list contain */
				/* remove gb so I can resort the data table and save it with new orders*/
				put "	try(Here:gb<<Delete);";
				put " try(Here:Here:df<<Delete);";
				put " wait(0.5);";
				put "	Here:odat << Sort(By( :&LGroupVar, :Map_Order ),Order( Ascending, Ascending ), Replace Table(0));";
				put " Here:odat<< save(""&ClientOutPath.&LinkageMapOrder_OutDataName..sas7bdat"");";
				put "	try(Here:LO_HB<<Prepend(HListBox(Eval(Here:GB_results)), Horizontal));";
				/* processing for converting SAS lmr data to new orders for interactive plots */
				/* put "Show(selgrps, selmax);"; */
				put "	Here:FlipOrderExpr;";
				%if ^%length(&OrderVar) and %index(%nrbquote(%upcase(&Algorithm)),MULTIDIMENSIONAL)>0  %then %do;
					put "		gettabsel = trtb << Get Selected();";
					put " 	mdstab = trtb<<Index of Tab(""MDS Residual Plots"");";
					put "		if(mdstab>0,";
					put "			trtb<<Delete(mdstab);";
					put "			trtb<<Append(""MDS Residual Plots"", Eval(expr(Here:Rfunc())));";
					put "			trtb << Set Selected(gettabsel);";
					put "		);";
				%end;
				put "	);"; /* Close If loop for if groups were selected */
		put ");";		
		
		put "InsertInto(Here:ActionButtonDesc ,""Click to Select Linkage Groups for Marker Order Reversal"");";	
		put "InsertInto(Here:ActionButtonList,{{""Reverse Linkage Group Marker Orders""}});";
		put "InsertInto(Here:ActionCodeList,{{Here:RevGroups}});";
		
		put "if(host is(Windows),";
		put "	include( ConvertFilePath(""$LS_HOME/Generator/DialogHelperFunctions.jsl"", absolute, windows) ),";
		put "	include( ConvertFilePath(""$LS_HOME/Generator/DialogHelperFunctions.jsl"", absolute, POSIX) ));";
		put "	";
		
		put "srbuttonswindow=Function({},{default local},";
		put "		VListBox(";
		put "			TextBox(""Click the button below to launch the Linkage Map Viewer process to draw the full linkage map.""),";
		put "			ButtonBox(""Linkage Map Viewer"",";
		put "				if(host is(Windows),";
		put "					Include(ConvertFilePath(""$HOME/JMPG/ProcessLibrary/LinkageMapViewer.jsl"", absolute, windows)),";
		put "					Include( ConvertFilePath(""$HOME/JMPG/ProcessLibrary/LinkageMapViewer.jsl"", absolute, POSIX) ));";
		put "	";
		/* Set input data set and Output Folder */
		put "				jg:setDefaultParmValues(2, ""LinkageMapViewer"");";				
		put "				jg:LinkageMapViewer_updateDeps;";
		put "				jg:setCurrentParmValue(2, ""LinkageMapViewer"", jg:getParmNum(""LinkageMapViewer"",""InData""),""&ClientOutPath.&LinkageMapOrder_OutDataName..sas7bdat"");";		
		put "				jg:setCurrentParmValue(2, ""LinkageMapViewer"", jg:getParmNum(""LinkageMapViewer"",""OutPath""),""&ClientOutPath"");";										
		put "				jg:LinkageMapViewer_updateDeps;";				
		put "				jg:setCurrentParmValue(2, ""LinkageMapViewer"", jg:getParmNum(""LinkageMapViewer"",""ChrVar""),""&LGroupVar"");";	
		put "				jg:setCurrentParmValue(2, ""LinkageMapViewer"", jg:getParmNum(""LinkageMapViewer"",""Label""),""&IDVar"");";
		put "				jg:setCurrentParmValue(2, ""LinkageMapViewer"", jg:getParmNum(""LinkageMapViewer"",""PosVar""),""cM_&MapFunc"");";
		/*
		put "				jg:setCurrentParmValue(2, ""LinkageMapViewer"", jg:getParmNum(""LinkageMapViewer"",""ColorVar""),""&ColorVar"");";
		put "				jg:setCurrentParmValue(2, ""LinkageMapViewer"", jg:getParmNum(""LinkageMapViewer"",""ColorType""),""Continuous"");";
		*/
		put "				jg:LinkageMapViewer_updateDeps;";
		/*put "				jg:setCurrentParmValue(2, ""LinkageMapViewer"", jg:getParmNum(""LinkageMapViewer"",""Theme""),""Blue to Gray to Red"");"; */
		put "				Wait(.5);";
		/* put "				jg:runProcess( 2, ""LinkageMapViewer"" );"; */
		put "			),";
		put "			TextBox("" ""),";	
		put "			TextBox(""Click the button below to launch the Subset and Reorder Genetic Data process.""),";
		put "			TextBox(""The genotype marker data must be reordered to match the linkage map order for QTL analysis.""),";
		put "			ButtonBox(""Subset and Reorder Genetic Data"",";
		put "				if(host is(Windows),";
		put "					Include(ConvertFilePath(""$HOME/JMPG/ProcessLibrary/SubsetandReorderGeneticData.jsl"", absolute, windows)),";
		put "					Include( ConvertFilePath(""$HOME/JMPG/ProcessLibrary/SubsetandReorderGeneticData.jsl"", absolute, POSIX) ));";
		put "	";
		/* Set input data set and Output Folder */
		put "				jg:setDefaultParmValues(2, ""SubsetandReorderGeneticData"");";	
		%if %length(&SNPInData) %then %do;
			put "				jg:setCurrentParmValue(2, ""SubsetandReorderGeneticData"", jg:getParmNum(""SubsetandReorderGeneticData"",""InData""),""&SNPInData"");";
			put "				jg:SubsetandReorderGeneticData_updateDeps;";
			%if %length(&AlleleVars) le 32767 %then %do;		
				put "				jg:setCurrentParmValue(2, ""SubsetandReorderGeneticData"", jg:getParmNum(""SubsetandReorderGeneticData"",""AlleleVars""),""&AlleleVars"");";
			%end;
			put "				jg:setCurrentParmValue(2, ""SubsetandReorderGeneticData"", jg:getParmNum(""SubsetandReorderGeneticData"",""ListMarkerVars""),""&ListMarkerVars"");";
			put "				jg:setCurrentParmValue(2, ""SubsetandReorderGeneticData"", jg:getParmNum(""SubsetandReorderGeneticData"",""MarkerFmt""),""&MarkerFmt"");";
			put "				jg:SubsetandReorderGeneticData_updateDeps;";
			put "				jg:setCurrentParmValue(2, ""SubsetandReorderGeneticData"", jg:getParmNum(""SubsetandReorderGeneticData"",""GenoDelim""),""&GenoDelim"");";
		%end;
		put "				jg:SubsetandReorderGeneticData_updateDeps;";
		put "				jg:setCurrentParmValue(2, ""SubsetandReorderGeneticData"", jg:getParmNum(""SubsetandReorderGeneticData"",""MinPercent""),""0"");";		
		put "				jg:setCurrentParmValue(2, ""SubsetandReorderGeneticData"", jg:getParmNum(""SubsetandReorderGeneticData"",""OutPath""),""&ClientOutPath"");";					
		put "				jg:setCurrentParmValue(2, ""SubsetandReorderGeneticData"", jg:getParmNum(""SubsetandReorderGeneticData"",""AnnoInData""),""&ClientOutPath.&LinkageMapOrder_OutDataName..sas7bdat"");";					
		put "				jg:SubsetandReorderGeneticData_updateDeps;";				
		put "				jg:setCurrentParmValue(2, ""SubsetandReorderGeneticData"", jg:getParmNum(""SubsetandReorderGeneticData"",""AnnoMName""),""&IDVar"");";	
		put "				jg:setCurrentParmValue(2, ""SubsetandReorderGeneticData"", jg:getParmNum(""SubsetandReorderGeneticData"",""ReorderVars""),""&LGroupVar Map_Order"");";	
		
		put "				jg:SubsetandReorderGeneticData_updateDeps;";	
		put "			)";	
		put "		)";
		put ");";
		
		put "Here:LaunchCode = srbuttonswindow();";
	
	run;
	
	%PackageInsertFile(JSL Code from &ProcessName, &OutPath, &LinkageMapOrderScriptName, "&LinkageMapOrder_OutDataName..sas7bdat,&LinkageMapOrder_OutDataResName..sas7bdat", yes);
		
	/* for now just launch file 
	%PackageInsertDataset(Output data from &ProcessName, OutLib, &LinkageMapOrder_OutDataName);
	*/
	
	%exit:;
	%ExitSDSProcess(&OutPath, &ProcessName, &syserr, &exiterror, &clockrunning);

	%if %SYMEXIST(_tid_) %then
		%let rc = %sysfunc(close(&_tid_));
	%exitcheck:; *---Luciano used it while testing code ---*;
	
%mend;

%LinkageMapOrderOutcross;	