/*
JMP Genomics Macros

Name: UtilityMacros
Author: Tzu-Ming Chu, SAS Institute Inc.
Support: Tzu-Ming Chu, SAS Institute Inc.
History:

	tzuchu 28Aug02 initial coding
	sasrdw 27Jun03 upgraded publish macros
	sasrdw 05Nov04 added LabelTable
	tzuchu 12Jen05 added CheckFileType
	tzuchu 31Jan05 added CheckLengthForMerging
	tzuchu 07Feb05 added Merge2sets
	sasrdw 18Apr05 added Stack
	tzuchu 19Apr05 added Stack_to_Tall
	tzuchu 19Apr05 added CheckInData
	tzuchu 30Jun05 added VarAttributes
	tzuchu 03Oct05 added CheckJSL
	tzuchu 03Oct05 added AddSuffixToDataName
	gemann 06Oct05 added Process Time Tracker calls
	wehass 23Nov05 added PrepHTML 
   tzuchu 04Apr06 added GetVars & GetLevels
   sasrdw 20Apr06 added OutDataConvert
   tzuchu 24May06 added CheckListVarExist
   sasrdw 09Sep06 changed Desktop to SASMode
	tzuchu 24Oct06 upgraded Naming
	tzuchu 31Oct06 added TrimStringVariabe 
	tzuchu 01Nov06 added CopySettingFile
	tzuchu 20Nov06 added FileFilter
	sasrdw 31Mar07 added CheckBinary
	sasrdw 31Mar07 added ExpandVars
	tzuchu 17Apr07 added RemoveData
	sasrdw 05May07 moved GroupMacroVars and Sweep from OneWay to this file
	sasrdw 22Jun07 added ShowTime
	tzuchu 15Oct07 added GetFileSize
	tzuchu 18Oct07 added GetLineLoc
	sasrdw 21Nov07 added RemoveWord
	tzuchu 11Jun08 added GetFileFormat
	stmart 08Aug08 added CountVars
	tzuchu 19Oct08 added Journal & SliceData
	tzuchu 02Dec08 added MergeByChunk
	tzuchu 26Apr10 added AddCommaBetweenVars
	saslii 10Nov10 added RemoveDir
	saslii 08Dec10 added ConvertTransportFile
	saslii 09Dec10 added CheckandCreateDir
	tzuchu 20Feb12 upgraded CheckInData
	tzuchu 20Feb12 added CheckInData2
	saslii 07Mar12 added LaunchBatFile
	tzuchu 11Feb13 added RemoveGlobalMacros;
	tzuchu 03Mar13 added CleanUpWorkLibrary;	
	tzuchu 11Apr13 added Update_ProcessInfo;
	tzuchu 12Apr13 added Setup_LogLib;
	tzuchu 16Apr13 added Study_Product
	tzuchu 26Aug13 added ResultMeta
	jocrom 02Oct13 added LocaleInit
	jocrom 02Oct13 added LocaleGet
	jocrom 02Oct13 added LocaleFill
	jocrom 04Oct13 added LocaleReverseGet
	jocrom 17Oct13 added LocaleEZX
	jocrom 28Oct13 added LocaleAreEquiv
	jocrom 29Oct13 added LocaleWrap
	jocrom 06Mar14 added LocaleGetAll
	jocrom 30Jun14 added LocaleSDSEZX

Description: UtilityMacros contains a collection
of useful SAS macros that can be called from
analytical processes.
*/

/*
-----------------------------------------------------------
Copyright (C) 2004 SAS Institute, Inc. All rights reserved.

Notice:
The above copyright notice and this notice must appear in
any whole or partial copy of this code and any related
documentation. Permission to use, copy, and modify the
source code contained in this file is hereby granted, but
is limited to customers of SAS with a valid license for the
SAS software product with which this file was provided.
Except as expressly set forth herein, the terms of such
license apply.
-----------------------------------------------------------



*/
options nospool nosymbolgen nosyntaxcheck MVARSIZE=MAX MSYMTABMAX=MAX NOOVP;
options formchar='|----|+|---+=|-/\<>*';
options label;

*Set macrovariable to define JMP Genomics/Clinical Version;

* check for zero length argument;
%macro CheckForBlank(_arg,_label);
   %if %length(%nrbquote(&_arg)) = 0 %then %do;
      %put ERROR: &_label specification is required.;
      %let exiterror = 1;
   %end;
%mend CheckForBlank;


* check input data sets;
%macro CheckInData(_ExpFlag=0,_CheckDesign=1);
%ClockStart(CheckInData);

%if ^%sysfunc(fileexist(&InData)) %then %do;
	%put ERROR: Physical file does not exist, &InData.; 
	%let exiterror = 1;
	%goto exit;
%end;

%put InData = &InData;
%CheckForBlank(&InData,Input dataset);
%if &SASMode=SPS or &SASMode=DMS %then %do;
   %if %sysfunc(fileexist(&InData.*)) = 0 %then %do; 
      %let exiterror = 1;
      %put ERROR: Specified InData, &InData, does not exist.;
   %end;
%end;

%PathName(&InData);

%if &exiterror %then %do;
   %put ERROR: Input dataset specification, &InData, is invalid.;
   %goto exit;
%end;
%global InPath InData;
%let InPath = &TmpPath;
%let InData = &TmpName;
%put InPath = &InPath;
%put InData = &InData;
libname InLib "&TmpPath";

* check ExpInData; 
%if &_ExpFlag %then %do;
   %put ExpInData = &ExpInData;
   %CheckForBlank(&ExpInData,Experimental Design Data Set);
   %if &SASMode ne SDD %then %do;
   	%if %sysfunc(fileexist(&ExpInData.*)) = 0 %then %do; 
      	%let exiterror = 1;
      	%put ERROR: Specified Experimental Design Data Set, &ExpInData, does not exist.;
			%goto exit;
   	%end;
   %end;
   %PathName(&ExpInData);
   %if &exiterror %then %do;
      %put ERROR: Experimental Design Data Set specification, &ExpInData, is invalid.;
      %goto exit;
   %end;
   %let ExpInData = &TmpName;
   %global ExpInPath;
   %let ExpInPath = &TmpPath;
   libname ExpInLib "&TmpPath";
   %put ExpInPath = &ExpInPath;
   %put ExpInData = &ExpInData;
   
   %let _nVarInMacroVar_ = 2000; * set the maximum number of varaibles in macro varibles below;

	%if &_CheckDesign %then %do;
		%CheckVarExist(ColumnName,ExpInLib.&ExpInData,Variable);
		%if &exiterror %then %do;
			%goto exit;
		%end;

		data OutLib.&ExpInData;
			set ExpInLib.&ExpInData;
		run;

		%if %symexist (WhereEddmt) %then %do;
			%if %length(&WhereEddmt) %then %do;
				* check name for ExpInData;
				%global OutEtdSub;
				%if %length(&ExpInData) < 29 %then %let etdname = &ExpInData;
				%else %let etdname = %substr(&ExpInData,1,28);
				%PrepOutDS(OutEtdSub,OutEtdSubName,&etdname,_sub);
				
				data OutLib.&OutEtdSubName;
					set OutLib.&ExpInData;
				 where %unquote(&WhereEddmt);
				 run;
				 %let ExpInData=&OutEtdSubName;
			%end;
		%end;

		* Check design file;
		%let _tid_ = %sysfunc(open(OutLib.&ExpInData));
		%let _nVars_ = %sysfunc(attrn(&_tid_,NVARS)); 
		%let _nObs_ = %sysfunc(attrn(&_tid_,NOBS));
		%let _ColNameFlag_ = 0;
		%let _ArrayFlag_ = 0;
		%global _ChannelFlag_ _FileFlag_; * indicator for Channel variable;
		%global ArrayAlias ChannelAlias FileAlias;
		%let _ChannelFlag_ = 0;
		%let _FileFlag_ = 0;
		%if ^%symexist(MeasureVar) %then %let MeasureVar = Measure;
		%do _i_ = 1 %to &_nVars_; 
			%global _Exp_Var&_i_._ _Exp_VarType&_i_._;
			%let _Exp_Var&_i_._ = %sysfunc(varname(&_tid_,&_i_));
			%let _Exp_VarType&_i_._ = %sysfunc(vartype(&_tid_,&_i_));
			%if %upcase(&&_Exp_Var&_i_._) = COLUMNNAME %then %do;
				%let _ColNameFlag_ = 1;
				%let _ColNameNum_ = &_i_;
				%do _j_ = 1 %to &_nObs_;
					%let _rc_=%sysfunc(fetchobs(&_tid_,&_j_));
					%global _ColName_&_j_._;
					%let _ColName_&_j_._ = %sysfunc(getvarc(&_tid_,%sysfunc(varnum(&_tid_,ColumnName))));
				%end;
			%end;
			%if %upcase(&&_Exp_Var&_i_._) = ARRAY |
				 %upcase(&&_Exp_Var&_i_._) = CHIP |
				 %upcase(&&_Exp_Var&_i_._) = SPECTRUM %then %do ;
				%let _ArrayFlag_ = 1;
				%let ArrayAlias = &&_Exp_Var&_i_._;
			%end;
			%if %upcase(&&_Exp_Var&_i_._) = CHANNEL |
				 %upcase(&&_Exp_Var&_i_._) = DYE | 
				 %upcase(&&_Exp_Var&_i_._) = %upcase(&MeasureVar) %then %do;
				%let _ChannelFlag_ = 1;
				%let ChannelAlias = &&_Exp_Var&_i_._;
			%end;
			%if %upcase(&&_Exp_Var&_i_._) = FILE |
				 %upcase(&&_Exp_Var&_i_._) = FILENAME %then %do;
				%let _FileFlag_ = 1;
				%let _FileNum_ = &_i_;
				%let FileAlias = &&_Exp_Var&_i_._;
				%do _j_ = 1 %to &_nObs_;
					%let _rc_=%sysfunc(fetchobs(&_tid_,&_j_));
					%global _FileName_&_j_._; 
					%let _FileName_&_j_._ = %sysfunc(getvarc(&_tid_,%sysfunc(varnum(&_tid_,&FileAlias))));
					%*put &_j_: &&_FileName_&_j_._;
				%end;
			%end;
		%end;
		%let _tid_ = %sysfunc(close(&_tid_));

		%if ^(&_ArrayFlag_) %then %do;
			%put ERROR: A variable named Array, Chip, or Spectrum was not found in the Experimental Design Data Set.;
			%put Please choose a different SAS data set that has one of these variables.;
			%let exiterror = 1;
		%end;
		%if ^(&_ChannelFlag_) %then %do;
			%put NOTE: A variable named Channel, Dye, or &MeasureVar is not in the Experimental Design Data Set.  Single channel data is assumed.;;
		%end;
		%if ^(&_ColNameFlag_) %then %do;
			%put ERROR: A variable named ColumnName (indicating column names in the Input Data) was not found in the Experimental Design Data Set.;
			%put Please choose a different SAS data set that has this variable.;
			%let exiterror = 1;
		%end;
		%if &exiterror %then %goto exit;

		%global _nFactor_; * number of vars before File column and used in %Stack;
		%if &_FileFlag_ %then %let _nFactor_ = %eval(&_FileNum_-1);
		%else %let _nFactor_ = &_nVars_;

		/* _factor macro vars no longer used below 
		%let _tid_ = %sysfunc(open(OutLib.&ExpInData));
			%do _j_ = 1 %to &_nObs_;
				%do _i_ = 1 %to &_nFactor_; 
					%global _factor&_i_._&_j_._; * Factor associated with row in ExpInData;
					%let _rc_=%sysfunc(fetchobs(&_tid_,&_j_));
					%if &&_Exp_VarType&_i_._ = C %then %let _factor&_i_._&_j_._ =
						%sysfunc(getvarc(&_tid_,%sysfunc(varnum(&_tid_,&&_Exp_Var&_i_._))));
					%else %let _factor&_i_._&_j_._ =
						%sysfunc(getvarn(&_tid_,%sysfunc(varnum(&_tid_,&&_Exp_Var&_i_._))));
				%end;
			%end;
		%let _tid_ = %sysfunc(close(&_tid_));
		*/

		* check uniqueness;
		proc sort data=OutLib.&ExpInData out=t nodupkey;
			by &ArrayAlias %if &_ChannelFlag_ %then &ChannelAlias;;
		run;

		%let _tid_ = %sysfunc(open(t));
		%let _nObs2_ = %sysfunc(attrn(&_tid_,NOBS));
		%let _tid_ = %sysfunc(close(&_tid_));
		%if &_nObs_ ne &_nObs2_ %then %do;
			%if &_ChannelFlag_ %then %do;
				%put ERROR: The combination of variables &ArrayAlias and &ChannelAlias must be unique for each row in the Experimental Design Data Set.;
			%end;
			%else %if %eval(&_nObs_/&_nObs2_)=%sysevalf(&_nObs_/&_nObs2_) %then %do; 
				%put ERROR: The variable &ArrayAlias is not unique for each row in the Experimental Design Data Set.;  
				%put This may due to multiple channels in the data, but a variable named Channel, Dye, or &MeasureVar is not present.;
			%end;
			%else %do;
				%put ERROR: The variable &ArrayAlias is not unique for each row in the Experimental Design Data Set.;
			%end;
			%let exiterror = 1;
		%end;

		proc sort data=OutLib.&ExpInData out=ed nodupkey;
			by ColumnName;
		run;

		%let _tid_ = %sysfunc(open(ed));
		%let _nObs2_ = %sysfunc(attrn(&_tid_,NOBS));
		%let _tid_ = %sysfunc(close(&_tid_));
		%if &_nObs_ ne &_nObs2_ %then %do;
			%put ERROR: The variable ColumnName is not unique in &ExpInData.;
			%let exiterror = 1; 
		%end;
		%if &exiterror %then %goto exit;

		* check variables; 
		%Naming(_Dset=ed,_AllVars=ColumnName);

		%global _nTallCol_ _KeepVar_; * macros used in %Stack and %Stack_to_Tall;
		%let _KeepVar_ = ; 
		%let _tid_ = %sysfunc(open(InLib.&InData));
		%let _nVars2_ = %sysfunc(attrn(&_tid_,NVARS)); 
		%do _i_ = 1 %to &_nVars2_;
			%let _In_Var&_i_._ = %sysfunc(varname(&_tid_,&_i_));
		%end;
		%let _tid_ = %sysfunc(close(&_tid_));
		%let _NonMatchVar_ = ;
		
		* check mismatch and make var list;	
		data VarName;
			length ColumnName $32;
			%do _i_ = 1 %to &_nVars2_;
				ColumnName = upcase("&&_In_Var&_i_._");
				output;
			%end;
		run;
		
		proc sort data=VarName;
			by ColumnName;
		run;
		
		data VarName_exp;
			length ColumnName $32;
			set ed(keep=ColumnName);
			ColumnName = upcase(ColumnName);
		run;
		
		proc sort data=VarName_exp;
			by ColumnName;
		run;

		data Ex_indata Ex_exp;
			merge VarName(in=_ids_) VarName_exp(in=_exp_);
			by ColumnName;
			if _ids_ & ^_exp_ then output Ex_Indata;
			if ^_Ids_ & _exp_ then output Ex_exp;
		run;
		
		%let _tid_ = %sysfunc(open(Ex_exp));
		%let _nVarsExp_ = %sysfunc(attrn(&_tid_,NOBS)); 
		%let _NonMatchVar_ = ;
		%do _i_ = 1 %to &_nVarsExp_;
			%let _rc_ = %sysfunc(fetchobs(&_tid_,&_i_));
			%let _NonMatchVar_ = &_NonMatchVar_ %sysfunc(getvarc(&_tid_,1));
		%end;
		%let _tid_ = %sysfunc(close(&_tid_));
		%if %length(&_NonMatchVar_) %then %do;
			%put ERROR: The following values of ColumnName in the Experimental Design Data Set do not match with the variable names in the Input Data Set.;
			%put Mismatching columns: &_NonMatchVar_.;
			%put;
			%put ColumnName values must exactly match the names of the variables in the Input Data set.;
			%put They must be valid SAS names, that is, begin with a letter or underscore, contain no special;
			%put characters, and be no longer than 32 characters.;
			%put Genomics > Experimental Design > Create ColumnName can be used to create valid names.;
			%put;
			%let exiterror = 1;
			%goto exit;
		%end;
		
		* make var list of vars in Input but not a ColumnName in ExpInput;
		%let _tid_ = %sysfunc(open(Ex_indata));
		%let _nVarsIds_ = %sysfunc(attrn(&_tid_,NOBS)); 
		%let _KeepVar_ = ;
		%do _i_ = 1 %to &_nVarsIds_;
			%let _rc_ = %sysfunc(fetchobs(&_tid_,&_i_));
			%let _KeepVar_ = &_KeepVar_ %sysfunc(getvarc(&_tid_,1));
		%end;
		%let _tid_ = %sysfunc(close(&_tid_));
		%put KeepVar = &_KeepVar_;
	
		* set the largest total number of variables in a macro to be 2000 to avoid the length limitation of a macro var;
		%do _i_ = 1 %to &_nObs_;
			%let _j_ = %sysfunc(ceil(&_i_/&_nVarInMacroVar_));
			%if %sysfunc(mod(&_i_,&_nVarInMacroVar_)) = 1 %then %do;
				%global _AllCol_&_j_._;
				%let _AllCol_&_j_._ = &&_ColName_&_i_._;
			%end;
			%else %do;
				%let _AllCol_&_j_._ = &&_AllCol_&_j_._ &&_ColName_&_i_._;
			%end;
			%*put i=&_i_, j=&_j_, _AllCol_&_j_=&&_AllCol_&_j_._;
			%*put ERROR: i=&_i_, colname=&&_ColName_&_i_._;
		%end;
		%let _nTallCol_ = &_nObs_;
		%global _nAllColChunk_;
		%let _nAllColChunk_ = &_j_;
		
	/*	
		%do _k_ = 1 %to &_j_;
			%*if %eval(%length(&_AllCol_)+%length(&&_AllCol_&_k_._)) < 65530 %then %do;
				%let _AllCol_ = &_AllCol_ &&_AllCol_&_k_._;
			%*end;
		%end;
	
	%put _AllCOl_ = %upcase(&_AllCol_);
			%let exiterror = 1;
			%goto exit;	
		/*
		%do _i_ = 1 %to &_nObs_;
			%let _MissFlag_ = 1;
			%do _j_ = 1 %to &_nVars2_;
				%if %qupcase(&&_In_Var&_j_._) = %qupcase(&&_ColName_&_i_._) %then %do;
					%let _MissFlag_ = 0;
					%let _j_ = &_nVars2_;
				%end; 
			%end;
			%let _AllCol_ = &_AllCol_ &&_ColName_&_i_._;
			%if &_MissFlag_ %then %let _NonMatchVar_ = &_NonMatchVar_ &&_ColName_&_i_._;
		%end;
		%if %length(&_NonMatchVar_) %then %do;
			%put ERROR: The following values of ColumnName in the Experimental Design Data Set do not match with the variable names in the Input Data Set.;
			%put Mismatching columns: &_NonMatchVar_.;
			%put;
			%put ColumnName values must exactly match the names of the variables in the Input Data set.;
			%put They must be valid SAS names, that is, begin with a letter or underscore, contain no special;
			%put characters, and be no longer than 32 characters.;
			%put Genomics > Experimental Design > Create ColumnName can be used to create valid names.;
			%put;
			%let exiterror = 1;
			%goto exit;
		%end;
		* make var list of vars in Input but not a ColumnName in ExpInput;
		%do _j_ = 1 %to &_nVars2_;
			%let _MissFlag_ = 1;
			%do _i_ = 1 %to &_nObs_;
				%if %upcase(&&_In_Var&_j_._) = %upcase(&&_ColName_&_i_._) %then %do;
					%let _MissFlag_ = 0;
					%let _i_ = &_nObs_;
				%end; 
			%end;
			%if &_MissFlag_ %then %let _KeepVar_ = &_KeepVar_ &&_In_Var&_j_._;
		%end;
		%let _nTallCol_ = &_nObs_;
		*/
	%end;
	%else %do;
		* assuming design file is valid, so just create the needed macro variables;
		%let _tid_ = %sysfunc(open(ExpInLib.&ExpInData));
		%let _nVars_ = %sysfunc(attrn(&_tid_,NVARS)); 
		%let _nObs_ = %sysfunc(attrn(&_tid_,NOBS));
		%let _tid_ = %sysfunc(close(&_tid_));
		%global _nTallCol_ _KeepVar_; * macros used in %Stack and %Stack_to_Tall;

		%let _nTallCol_ = &_nObs_;

		data a;
			set ExpInLib.&ExpInData;
			one = 1;
			keep one ColumnName;
		run;

		proc transpose data=a out=at(drop=_name_);
			var one;
			id ColumnName;
		run;

		%let _tid_ = %sysfunc(open(at));
		%do i=1 %to &_nTallCol_;
			%global _ColName_&i._;
			%let _ColName_&i._ = %sysfunc(varname(&_tid_,&i));
			%let j = %sysfunc(ceil(&i/&_nVarInMacroVar_));
			%if %sysfunc(mod(&i,&_nVarInMacroVar_)) = 1 %then %do;
				%global _AllCol_&j._;
				%let _AllCol_&j._ = &&_ColName_&i._;
			%end;
			%else %do;
				%let _AllCol_&j._ = &&_AllCol_&j._ &&_ColName_&i._ ;
			%end;
		%end;
		%let _tid_ = %sysfunc(close(&_tid_));
		%global _nAllColChunk_;
		%let _nAllColChunk_ = &j;

		%if ^%symexist(AlreadyStacked) %then %let AlreadyStacked = 0;
		data i;
			set InLib.&InData;
			%if ^&AlreadyStacked %then %do;
				drop %do i = 1 %to &_nAllColChunk_; &&_AllCol_&i._ %end;;
			%end;
		run;

		%let _tid_ = %sysfunc(open(i));
		%let _nv_ = %sysfunc(attrn(&_tid_,NVARS)); 
		%do i=1 %to &_nv_;
			%let _KeepVar_ = &_KeepVar_ %sysfunc(varname(&_tid_,&i));
		%end;
		%let _tid_ = %sysfunc(close(&_tid_));

	%end;

	%put _nTallCol_=&_nObs_;
	%do i = 1 %to &_nAllColChunk_;
		%put _AllCol_&i._=&&_AllCol_&i._;
	%end;;
	%put _KeepVar_= &_KeepVar_;

%end;
%exit:;
%if (&syserr | &exiterror) %then %put Macro CheckInData exited due to errors.;
%ClockStop(CheckInData);
%mend CheckInData;

* extract column info direct from input data set when 
* 1. based on a user specified variable list
* 2. based on a user specified list-style variable
* 3. based on _numeric_ type if items 1 and 2 above are not applicable.;
%macro CheckInData2(_InputData,_Vars=,_VarListStyle=);
	%ClockStart(CheckInData2);
	%let _nVarInMacroVar_ = 2000; * the maximum nubmer of vars in a list;
	%if %length(_vars) | %length(&_VarListStyle) %then %do;
		%let _keepVars_ = &_Vars &_VarListStyle;
	%end;
	%else %do;
		%let _KeepVars_ = _numeric_;
	%end;
	
	data tmp;
		set &_InputData(obs=0);
		keep &_KeepVars_;
	run;
	
	%global _nTallCol_;
	%let _dsid_ = %sysfunc(open(tmp));
	%let _nTallCol_ = %sysfunc(attrn(&_dsid_,NVARS));
	%do _i_ = 1 %to &_nTallCol_;
		%global _ColName_&_i_._;
		%let _ColName_&_i_._ = %sysfunc(varname(&_dsid_,&_i_));
		%let _j_ = %sysfunc(ceil(&_i_/&_nVarInMacroVar_));
		%if %sysfunc(mod(&_i_,&_nVarInMacroVar_)) = 1 %then %do;
			%global _AllCol_&_j_._;
			%let _AllCol_&_j_._ = &&_ColName_&_i_._;
		%end;
		%else %do;
			%let _AllCol_&_j_._ = &&_AllCol_&_j_._ &&_ColName_&_i_._;
		%end;
	%end;
	%let _rc_ = %sysfunc(close(&_dsid_));
	%global _nAllColChunk_;
	%let _nAllColChunk_ = %eval(&_j_);
	%ClockStop(CheckInData2);
	/*
	%do i = 1 %to &_nAllColChunk_;
		%put i=&i, &&_AllCol_&i._;
	%end;
	*/
%mend CheckInData2;


%macro ConvertFilepaths(_listOfMacros);
   %put _listOfMacros= &_listOfMacros;
   %if &SASMode=SDD %then %do;
      %let _i_ = 1;
      %do %while(%length(%scan(&_listOfMacros,&_i_))); 
         %let _currentparm_ = %scan(&_listOfMacros,&_i_);
			* screen out empty optional InData;
			%if %length(&&&_currentparm_) %then %do;
	         %if %index(&&&_currentparm_,.)=0 %then %ConvertInputFile(&_currentparm_);
	         %else %ConvertInputDataset(&_currentparm_);
            %put &_currentparm_ = &&&_currentparm_;
			%end;
	      %let _i_ = %eval(&_i_+1);
      %end;
   %end;
%mend ConvertFilepaths;

%macro ConvertInputFile(_var);
   data _null_;
      v=pathname("&&&_var");
      call symput("&_var",v);
   run;
%mend ConvertInputFile;

%macro ConvertInputDataset(_var);
   data _null_;
      v="&&&_var";
      path=trim(pathname(scan(v,1,'.'))) || "&_slash_" || scan(v,2,'.') || ".sas7bdat";
      call symput("&_var",path);
   run;
%mend ConvertInputDataset;

%macro GetOrigFileName(_var, _origvar);
   %global &_origvar;
   %if &SASMode=SDD %then %do;
      data _null_;
  	      set &SDDPARMS;
	      if id="%upcase(&_var)" and valtype="filename";
	      name=substr(value,1,index(value,".")-1);
	      call symput("&_origvar",trim(left(name)));
		run;
   %end;
   %else %do;
   	%let _varq = %nrbquote(&&&_var);
      %PathName(&_varq); 
      data _null_;
         name="&TmpName";
         call symput("&_origvar",name);
		run;
   %end;
%mend GetOrigFileName;


* For ODS HTML statement;
%macro PrepHTML(_htmlbody,_htmlframe,_htmlcontents,_defaulthtmlbodyname,_defaulthtmlframename,_defaulthtmlcontentsname);
	%global bodyfile framefile contentsfile;
	%if &SASMode ne SDD %then %do;
		%if %length(&_defaulthtmlbodyname) %then %do;
			%let &_htmlbody=&_defaulthtmlbodyname;
		%end;
		%if %length(&_defaulthtmlframename) %then %do;
			%let &_htmlframe=&_defaulthtmlframename;
		%end;
		%if %length(&_defaulthtmlcontentsname) %then %do;
			%let &_htmlcontents=&_defaulthtmlcontentsname;
		%end;
		%let bodyfile=&&&_htmlbody;
		%let &_htmlbody=&OutPath.&&&_htmlbody;
		%let framefile=&&&_htmlframe;
		%let &_htmlframe=&OutPath.&&&_htmlframe;
		%let contentsfile=&&&_htmlcontents;
		%let &_htmlcontents=&OutPath.&&&_htmlcontents;
	%end;
	%else %do;
		options mlogic macrogen;
		data _null_;
			bodyrevfilepath = Reverse(strip("&&&_htmlbody"));
			call symput("bodyfile",Reverse( substr(bodyrevfilepath, 1, indexc(bodyrevfilepath,'/ \')-1) ));
			framerevfilepath = Reverse(strip("&&&_htmlframe"));
			call symput("framefile",Reverse( substr(framerevfilepath, 1, indexc(framerevfilepath,'/ \')-1) ));
			contentsrevfilepath = Reverse(strip("&&&_htmlcontents"));
			call symput("contentsfile",Reverse( substr(contentsrevfilepath, 1, indexc(contentsrevfilepath,'/ \')-1) ));
		run;
	%end;
%mend; 

* Other output file types;
%macro PrepOutFile(_fullpath,_file,_defaultname);
	%global &_file;
	* check .jsl extension;
	%if %upcase(&_fullpath)=JSLFILE | %index(%upcase(&_fullpath),SCRIPT) %then %do;
	   %if %length(&&&_fullpath)>3 %then %do;
		   %if %upcase(%substr(&&&_fullpath,%length(&&&_fullpath)-3)) ne .JSL %then
			   %let &_fullpath = %trim(&&&_fullpath).jsl;
		%end;
		%else %do;
         %if %length(&&&_fullpath) %then %let &_fullpath = %trim(&&&_fullpath).jsl;
		%end;
	%end;
	%if &SASMode ne SDD %then %do;
		%let &_file=&&&_fullpath;
		%if ^%length(&&&_file) %then %let &_file=&_defaultname;
		%if ^%length(&&&_file) %then %let exiterror = 1;
		%let &_fullpath=&OutPath.&&&_file;
	%end;
	%else %do;
		%let &_file=%sysfunc(scan(&&&_fullpath,-2)).%sysfunc(scan(&&&_fullpath,2,"."));
	%end;
	%if &LS_Index = 3 %then %do;
		%let ext = %sysfunc(scan(&&&_file,-1,"."));
		%if %length(&ext) = %length(&&&_file) %then %do;
			%let &_file = %trim(&&&_file)_&_nThePro_;
		%end;
		%else %do;
			%let &_file = %substr(&&&_file,1,%length(&&&_file)-%length(&ext)-1)_&_nThePro_..&ext;
		%end;
		%let &_fullpath=&OutPath.&&&_file;
	%end;
%mend;

* Output data sets;
%macro PrepOutDS(_dataset,_dsname,_defaultname,_suffix);
	%global &_dsname;
	%if &LS_Index = 3 %then %do;
		%let _suffix = &_suffix._&_nThePro_;
	%end;
	%if &SASMode ne SDD %then %do;	
		%*let &_dsname=%scan(&&&_dataset,1);
		%let &_dsname=&&&_dataset;
		%if ^%length(&&&_dsname) %then %do;
			%let &_dsname=&_defaultname;
			%if %length(&_suffix) %then %AddSuffixToDataName(&_suffix,&_defaultname,&_dsname);
		%end;
		%CheckDataName(_name=&_dsname,_default=&_defaultname);
		%if ^%length(&&&_dsname) %then %let exiterror = 1;
		%let &_dataset=%trim(OutLib.&&&_dsname);
	%end;
	%else %do;
		%let &_dsname=%scan(&&&_dataset,2);
	%end;
%mend;

* check if there is a &_slash_ in the end of a path provided;
%macro CheckEndSlash(_path,_add=1);
	%let path1 = %nrbquote(%trim(&&&_path));
	 /* %put path1 = &path1   slash = &_slash_; */
	%let len1 = %length(&path1);
	%let lastchar = %qsubstr(&path1,&len1,1);
	 /* %put len1 = &len1  lastchar = &lastchar; */
   %if &_add %then %do;
      %if &lastchar ne %quote(&_slash_) and &lastchar ne %quote(&_oslash_) %then 
         %let &_path = %nrbquote(&path1.&_slash_);
   %end;
   %else %do;
      %if &lastchar = %quote(&_slash_) %then %do;
         %let &_path = %qsubstr(&path1,1,%eval(&len1-1));
		%end;
   %end;
%mend CheckEndSlash;


* check for existence of variables;
%macro CheckVarExist(_Vars,_Dset,_Label,_ShowError=1,_DsLabel=);
   %let _error_ = 0;
   %let _count_ = 0;
   %let _i_ = 1; 
   %do %while(%length(%scan(%quote(&_Vars),&_i_)));
      %let _found_ = 0;
      %let _ds_ = %sysfunc(open(&_Dset));
      %let numlist = 0 1 2 3 4 5 6 7 8 9;
		%if ^%index(&numlist, %substr(%scan(%quote(&_Vars),&_i_), 1, 1)) %then %do;
      	%let _found_ = %sysfunc(varnum(&_ds_,%scan(%quote(&_Vars),&_i_)));
      %end;
      %let _rc_=%sysfunc(close(&_ds_));
      %if ^&_found_ %then %do;
         %let exiterror = 1;
         %let _error_ = 1;
         %if &_count_ %then %let _BadVars_ = &_BadVars_, %scan(%quote(&_Vars),&_i_);
         %else %let _BadVars_ = %scan(%quote(&_Vars),&_i_);
         %let _count_ = %eval(&_count_+1);
      %end;
      %let _i_ = %eval(&_i_+1);
   %end;
   %if &_ShowError %then %let _MsgLab_ = ERROR;
   %else %let _MsgLab_ = NOTE;
   %if &_count_>1 %then %let _d_ = do;
   %else %let _d_ = does;
   %if &_error_ %then %do;
	   %let _DL_ = &_Dset data set;
	   %if %length(&_DsLabel) %then %let _DL_ = &_DsLabel;
      %put &_MsgLab_: &_Label &_BadVars_ &_d_ not exist in the &_DL_..;
   %end;
%mend CheckVarExist;


*Check for existence of list-style variables;
%macro CheckListVarExist(_ListVars,_Dset,_ShowError=1);
   data _tmp;
	   set &_Dset;
		if _n_>1 then stop;
		keep &_ListVars;
	run;
	%nObsVars(_tmp);
	%if ^&nVars %then %do;
	   %let exiterror = 1;
		%if &_ShowError %then %do;
			%put [MS_S_0249];
			%put ERROR: No variable of the List-Style variable &_ListVars exists.;
		%end;
	%end;
%mend CheckListVarExist;

* Check if variable &old in dataset &Dset has label.;
* If label exists then replace &_old (or asign &_new) by label.;
%macro CheckLabel(_Dset,_Old,_New=,_Flag=);
	%if %sysfunc(notname(&&&_old)) %then %do;
		%put Variable &&&_old is not a valid SAS variable name.;
	%end;
	%else %do;
		%let _ds_ = %sysfunc(open(&_dset));
		%if &_ds_ %then %do;
			%if %sysfunc(varnum(&_ds_,&&&_old)) %then %do;
				%let _l_ = %qsysfunc(varlabel(&_ds_,%sysfunc(varnum(&_ds_,&&&_old))));
				%if %length(&_l_) %then %do;
					%if %length(&_Flag) %then %let &_Flag = 1;
					%if %length(&_new) %then %let &_new = &_l_;
					%else %let &_old = &_l_;
				%end;
				%else %if %length(&_new) %then %let &_new = &&&_old;
			%end;
			%else %do;
				%put [MS_S_0230];
				%put ERROR: Variable &&&_old not found in dataset &_dset.;
			%end;
			%let _rc_ = %sysfunc(close(&_ds_));
		%end;
		%else %do;
			%put [MS_S_0231];
			%put ERROR: Open for data set &_dset failed when checking label status - %sysfunc(sysmsg());
		%end;
	%end;
%mend CheckLabel;


* Extract Vars for a Dataset;
%macro GetVars(_Ds,_vList,_nVars=);
   %let _dsid_ = %sysfunc(open(&_ds));
   %if &_dsid_ %then %do;
      %let _nVars_ = %sysfunc(attrn(&_dsid_,NVARS));
		%if %length(&_nVars) %then %let &_nvars = &_nVars_;
		%do _i_ = 1 %to &_nVars_;
		   %if &_i_ = 1 %then %let &_vList = %sysfunc(varname(&_dsid_,&_i_));
			%else %let &_vList = &&&_vList %sysfunc(varname(&_dsid_,&_i_));
		%end;
      %let _rc_ = %sysfunc(close(&_dsid_));
   %end;
   %else %do;
      %put [MS_S_0250];
      %put ERROR: Open for data set &_ds_ failed - %sysfunc(sysmsg());
      %let &_vList = ;
      %if %length(&_nVars) %then %let &_nvars = 0;
      %let exiterror = 1;
   %end;
%mend GetVars;


* Add comma between elements in a list;
%macro AddCommaBetweenVars(_inListName,_outListName=,_flagQuote=0);
	%if ^%length(&_outListName) %then %let _outListName = &_inListName;
	%if &_flagQuote %then %let &_outListName = "%scan(&&&_inListName,1)";
	%else %let &_outListName = %scan(&&&_inListName,1);
	%let i = 2;
	%do %while(%length(%scan(&&&_inListName,&i)));
		%if &_flagQuote %then %let &_outListName = &&&_outListName, "%scan(&&&_inListName,&i)";
		%else %let &_outListName = &&&_outListName, %scan(&&&_inListName,&i);
		%let i = %eval(&i+1);
	%end;
%mend AddCommaBetweenVars;


* Extract levels for a variable;
%macro GetLevels(_Ds,_Var,_LList,_LPrefix=,_nLvls=,_vType=,_dropmiss=0);
   proc sort data=&_Ds%if &_dropmiss=1 %then (where=(missing(&_Var)=0)); out=tmp(keep=&_Var) nodupkey;
	   by &_Var;
	run;

	data _null_;
	   set tmp end=_e_;
		name = "Lvl"||trim(left(_n_));
		call symput(name,&_Var);
		if _e_ then do;
         %if %length(&_vType) %then call symputx("&_nLvls",_n_);;
			%if %length(&_vType) %then call symputx("&_vType",vtype(&_Var));;
		end;
	run; 
   %put Number of Levels= &&&_nlvls;
	%put Variable Type= &&&_vtype;
	%do _i_ = 1 %to &&&_nLvls;	
		%if %length(&_LList) %then %do;
	   	%let &_LList = &&&_LList %left(%nrbquote(%trim(%nrbquote(&&Lvl&_i_))));
	   %end;
	   %if %length(&_LPrefix) %then %do;
	   	%global &_LPrefix.&_i_;
	   	%let &_LPrefix.&_i_ = %left(%nrbquote(%trim(%nrbquote(&&Lvl&_i_))));
	   %end;
	%end;
	%put List of Levels= &&&_llist;
%mend GetLevels;

* Get size(s) of a file (and free space) specified in _InFile;
* Specify a folder to get total size of the folder;
* Specify _FreeSize to extract free space of the drive;
* Specify _OutData to extract individual file and size if _inFile is a path;
%macro GetFileSize(_InFile,_FileSize,_FreeSize=,_OutData=_filesize,_ErrStopFlag=1);
	%global &_FileSize;
	%let &_FileSize = ;
	%if %length(&_FreeSize) %then %global &_FreeSize;

	%if %symexist(USER_HOME) %then %do;
		filename BatFile "&USER_HOME.ProcessResults&_Slash_.tmp101507.bat";	
		*filename JslFile "&USER_HOME.ProcessResults&_Slash_.tmp101507.jsl";
		filename TxtFile "&USER_HOME.ProcessResults&_Slash_.tmp101507.txt";
		filename TxtFile2 "&USER_HOME.ProcessResults&_Slash_.tmp101508.txt";
		%let rc = %sysfunc(fDelete(BatFile));
		%let rc = %sysfunc(fDelete(TxtFile));
		%let rc = %sysfunc(fDelete(TxtFile2));
		* preparing .bat file;
		data _null_;
			file BatFile;
			put 'dir "' @;
			put "&_InFile" @;
			put '" > "' @;
			put "&USER_HOME.ProcessResults&_Slash_.tmp101507.txt" @;
		run; 
		/* preparing .jsl file;
		data _null_;
			file JslFile;
			put '//!';
			put 'open("' @;
			put "&USER_HOME.ProcessResults&_Slash_.tmp101507.bat" @;
			put '")';
		run;
		*/
		* launch .bat to save file size info to a .txt file;
		%LaunchFile(&USER_HOME.ProcessResults&_Slash_.tmp101507.bat);

		* wait 5 seconds for tmp101507.txt to be created completely;
		data _null_;
			format t1 t2 time11.2;
			t1=time();
			t2=time();
			do while(t2-t1<5);
				t2=time();
			end;
		run;

		data &_OutData;
			infile TxtFile MISSOVER TRUNCOVER;
			length x $500.;
			if _n_ = 1 then input #5;
			input x $ 1-500;
		run;

		data _null_;
			set &_OutData end=_e_;
			if _e_ then call symput('nRow',_n_);
		run;

		data &_OutData;
			set &_OutData;
			length file $200.;
			if ^index(x,'<DIR>');
			if _n_ = %eval(&nRow-1) then do;
				size1 = scan(x,3,' ');
				size = compress(translate(size1,'',',.'))+0;
				file = 'Total Size';
				call symputx("&_FileSize",size);
			end;
			else if _n_ = &nRow then do;
				size1 = scan(x,3,' ');
				size = compress(translate(size1,'',',.'))+0;
				file = 'Free Disk Size';
				%if %length(&_FreeSize) %then call symputx("&_FreeSize",size);;
			end;
			else do;
				t=scan(x,3,' ');
				if t='AM' | t='PM' then do;
					size1 = scan(x,4,' ');
					size=compress(translate(size1,'',',.'))+0;
					loc = 5;
				end;
				else do;
					size1 = scan(x,3,' ');
					size=compress(translate(size1,'',',.'))+0;
					loc = 4;
				end;
				f=scan(x,loc,' ');
				y=indexw(x,f);
				file = substr(x,y);
			end;
			drop x loc f y t;
		run;

		%if %length(&&&_FileSize) = 0 %then %do;
			%let &_FileSize = -1;
			%put File Size: &&&_FileSize;
			%if &_ErrStopFlag %then %do;
				%put ERROR: The specified _InFile, &_InFile, does not exist.;
				%put ERROR: Macro GetFileSize exited due to errors.;
				%let exiterror = 1;
				%goto exit;
			%end;
		%end;
		%else %do;
			%put File Size: &&&_FileSize;
		%end;
		%if %length(&_FreeSize) %then %put Free Disk Size: &&&_FreeSize;
		%if &syserr>4 %then %let exiterror = 1;
	%end;
	%exit:;
%mend GetFileSize;

%macro GetDataSetSize(ds,_FileSize,lib=work);
	%global &_FileSize;
	ods listing close;
	ods output members=_m_;
	proc datasets library=&lib;
	quit;
	ods listing;
	
	%let &_FileSize=0;
	data _null_;
		set _m_;
		if memtype="DATA";
		if strip(name)=upcase("&ds") then call symputx("&_FileSize",FileSize);
	run;
	
	%if &&&_FileSize=0 %then %do;
		%put ERROR: Data set &ds does not exist in the library specified.;
		%let exiterror=1;
	%end;
	
%mend;

* Check JSL names;
%macro CheckJSL(_Data=);
   %global JSLFileName;
   %if ^%length(&_Data) %then %let _Data = &OutData;
	%if %length(&JSLFile) %then %let JSLFileName = &JSLFile..jsl;
	%else %let JSLFileName = &ProcessName Script for &_Data..jsl;
	filename jslfile "&OutPath.&JSLFileName";
%mend CheckJSL;

* Generate Journal script;
* _JRNFileName: name of the output JRN file name;
* _Title: Main title;
* _SubTitleList: List of button names separated by "|";
* _JSLList: List of corresponding jsl file (w extension) separated by "|".
	Put one jsl file here if it will be applied for all buttons. If the following
	two optional parameters are applied, data table reference outds&i (with
	a sequence number as suffix) shall be applied in the jsl file. If the two
	optional parameters are not applied, the jsl file shall handle the desire
	open function(s).;

* optional:;
* _nWithinDs: number of data set to be open corresponding to each script.
	Assume all jsl files applies the same number of datasets. The corresponding 
	dataset to each jsl can also be define in each individual jsl file for more
	complicate case;
* _SubDataList: List of sub-dataset (w/o extension) associated to &_SubTitleList;
%macro Journal(_JRNFileName,_Title,_SubTitleList,_JSLList,_nWithinDs=0,_SubDataList=);
	filename JRNFile "&_JRNFileName";
	data _null_;
		file JRNFile encoding="utf-8";
		put '//!';
		put 'List Box(';
		put '   vertical,';
		put '   near,';
		put '   sub(';	
		put '      Outline Box( Open(1),';
		put '         title("' "&_Title" '"),';
		put '         sub(';
		put '            Button Box(title("' "%scan(&_SubTitleList,1,'|')" '"), '; 
		put '               Script Text("'; 
		%let _d_ = 0;
		%do _n_ = 1 %to &_nWithinDs;
			%if ^%length(&_SubDataList) %then %do;
				%put ERROR: The optional parameter _SubDataList is required when applying optional parameter _nWithDs with a non-zero value.;
				%goto exit;
			%end;
			* This is optional.  The open function(s) can be put in the corresponding .jsl file;
			%let _d_ = %eval(&_d_+1);
			put "                  jg:outds&_n_ = \!"'"' "%scan(&_SubDataList,&_d_)" '.sas7bdat\!";';
		%end;
		put "                  jg:subTitle = \!"'"' "%scan(&_SubTitleList,1,'| ')" '\!";';
		put '                  include(\!".\' "%scan(&_JSLList,1,'|')" '\!", <<new context, << \!"Names Default To Here\!");';
 		put '               ")';
		put '            )'@;
		%let _i_ = 2;
		%do %while(%length(%scan(&_SubTitleList,&_i_,'|')));
			put ',';	
			put '            Button Box(title("' "%scan(&_SubTitleList,&_i_,'|')" '"), '; 
			put '               Script Text("';  
			%do _n_ = 1 %to &_nWithinDs; 	
				* This is optional.  The open function(s) can be put in the corresponding .jsl file;
				%let _d_ = %eval(&_d_+1);
				put "                  jg:outds&_n_ = \!"'"' "%scan(&_SubDataList,&_d_)" '.sas7bdat\!";';
			%end;
			put "                  jg:subTitle = \!"'"' "%scan(&_SubTitleList,&_i_,'| ')" '\!";';
			%if %index(&_JSLList,|) %then %do;
				* Multiple JSL;
				put '                  include(\!".\' "%scan(&_JSLList,&_i_,'|')" '\!", <<new context, << \!"Names Default To Here\!");';
			%end;
			%else %do; 
				* Single JSL;
				put '                  include(\!".\' "&_JSLList" '\!", <<new context, << \!"Names Default To Here\!");';
			%end;
 			put '               ")';
			put '            )'@;
			%let _i_ = %eval(&_i_+1);
		%end;
		put ;
		put '         )';
		put '      )';
		put '   )';
		put ');';
	run;
	%exit:;
%mend Journal;

* Slice &_Data into a series of sub set based on _ByVars;
* Inputs:
	1. _Data: Data to be subset.
	2. _ByVars:	By Variables.
	3. _SubDataPrefix: Desire prefix for output sub-datasets.
	4. _OutPath: Output folder for sliced subsets.

  Outputs:
	1. _LevelData: Dataset with combinations of ByVars-levels.
	2. _SubDataList: reference of macro name with a list of sub-dataset names (w/o library); 
%macro SliceData(_Data,_ByVars,_LevelData,_OutPath,_SubDataList=,_SubDataPrefix=);
	Proc sort data=&_Data(keep=&_ByVars) out=&_LevelData nodupkey;
		by &_ByVars;
	run;

	%let _v_ = 1;
	%do %while(%length(%scan(&_ByVars,&_v_)));
		%let byvar&_v_ = %scan(&_ByVars,&_v_);
		%let _v_ = %eval(&_v_+1);
	%end;
	%let _nByVars_ = %eval(&_v_-1);

	data _null_;
		set &_LevelData end=_e_;
		%do _v_ = 1 %to &_nByVars_;
			name&_v_ = "bvLevel&_v_._" || trim(left(_n_));
			call symput(name&_v_,&&byvar&_v_);
		%end;
		if _e_ then do;
			call symput("_nLevel_",_n_);
			* Check var type;
			%do _v_ = 1 %to &_nByVars_;
				if vtype(&&byvar&_v_)="N" then call symput("bvtype&_v_",1);
				else call symput("bvtype&_v_",0);
			%end;
		end;
	run;

	libname _OP_ "&_OutPath";
	%if %length(&_SubDataPrefix) %then %let _dName_ = &_SubDataPrefix;
	%else %let _dName_ = %scan(&_Data,-1,'.'); 
	%global &_SubDataList; 
	%let &_SubDataList = ;
	%let _sdList_ = ;
	%do _l_ = 1 %to &_nLevel_;	
		%let _sfx_ = %trim(%left(&_l_));  
		%let OutData&_l_ = ;
		%AddSuffixToDataName(&_sfx_,&_dName_,OutData&_l_);	
		%let &_SubDataList = &&&_SubDataList &&OutData&_l_; 
		%let _sdList_ = &_sdList_ _OP_.&&OutData&_l_;
	%end;

	* adjust for string value to be used in subsetting;
	%do _v_ = 1 %to &_nByVars_;
		%if &&bvtype&_v_=0 %then %do;
			%do _l_ = 1 %to &_nLevel_;
				%let bvLevel&_v_._&_l_ = "&&bvLevel&_v_._&_l_"; 
			%end;
		%end;
	%end;

	data &_sdList_;
		set &_Data;
		%do _l_ = 1 %to &_nLevel_;
			if (&byvar1 = &&bvLevel1_&_l_) 
				%do _v_ = 2 %to &_nByVars_;
					& (&&byvar&_v_ = &&bvLevel&_v_._&_l_)
				%end;
			then output _OP_.&&OutData&_l_;
		%end;
	run;
		
	%global _NByGps_;
	%let _NByGps_ = &_nLevel_;	
%mend SliceData;

%macro AddSuffixToDataName(_Suffix,_OldName,_NewName,_SDS=0);
	%if (&SASMode ne SDD) or (&_SDS=1) %then %do;
   	%if %length(&_OldName) <= %eval(32-%length(&_Suffix)) %then 
	   	%let &_NewName = %trim(%left(&_OldName))&_Suffix;
		%else %let &_NewName = %trim(%left(%substr(&_OldName,1,32-%length(&_Suffix))))&_Suffix;
		* trim the ending string of original name;
 	%end;
%mend AddSuffixToDataName;

%macro SliceBigData(_Data,_ByVars,_LevelData,_OutPath);
	Proc sort data=&_Data(keep=&_ByVars) out=&_LevelData nodupkey;
		by &_ByVars;
	run;

	%let _v_ = 1;
	%do %while(%length(%scan(&_ByVars,&_v_)));
		%let byvar&_v_ = %scan(&_ByVars,&_v_);
		%let _v_ = %eval(&_v_+1);
	%end;
	%let _nByVars_ = %eval(&_v_-1);

	data _null_;
		set &_LevelData end=_e_;
		%do _v_ = 1 %to &_nByVars_;
			name&_v_ = "bvLevel&_v_._" || trim(left(_n_));
			call symput(name&_v_,&&byvar&_v_);
		%end;
		if _e_ then do;
			call symput("_nLevel_",_n_);
			* Check var type;
			%do _v_ = 1 %to &_nByVars_;
				if vtype(&&byvar&_v_)="N" then call symput("bvtype&_v_",1);
				else call symput("bvtype&_v_",0);
			%end;
		end;
	run;

	libname _OP_ "&_OutPath";
	%let _dName_ = %scan(&_Data,-1,'.'); 

	%let _sdList_ = ;
	%do _l_ = 1 %to &_nLevel_;	
		%let _sfx_ = %trim(%left(&_l_));  
		%let OutData&_l_ = ;
		%AddSuffixToDataName(&_sfx_,&_dName_,OutData&_l_);	
		
		%*let _sdList_ = &_sdList_ _OP_.&&OutData&_l_;
	%end;

	* adjust for string value to be used in subsetting;
	%do _v_ = 1 %to &_nByVars_;
		%if &&bvtype&_v_=0 %then %do;
			%do _l_ = 1 %to &_nLevel_;
				%let bvLevel&_v_._&_l_ = "&&bvLevel&_v_._&_l_"; 
			%end;
		%end;
	%end;

	%let chuncksize =5000;
	
	%let nChuncks = %sysfunc(ceil(&_nLevel_/&chuncksize));

	%put _nLevel_=&_nLevel_ nChuncks=&nChuncks;
	%do _p_ = 1 %to &nChuncks;
		%let _nstotal_ = %eval(&_p_ * &chuncksize);
		%let startNumber = %eval((&_p_ -1)*&chuncksize +1);
		%*put _nstotal_=&_nstotal_ startNumber=&startNumber;

		%if &_nstotal_> &_nLevel_ %then %let _nstotal_=&_nLevel_;

		data %do _m_ = &startNumber %to &_nstotal_; _OP_.&&OutData&_m_ %end;;
			set &_Data;
			%do _l_ = &startNumber %to &_nstotal_;
				if (&byvar1 = &&bvLevel1_&_l_) 
					%do _v_ = 2 %to &_nByVars_;
						& (&&byvar&_v_ = &&bvLevel&_v_._&_l_)
					%end;
				then output _OP_.&&OutData&_l_;
			%end;
		run;
	%end;
		
	%global _NByGps_;
	%let _NByGps_ = &_nLevel_;	
%mend SliceBigData;

* check return code for a function;
%macro Checkrc(_function,_rc=);
   %if &_rc = 0 %then %put &_function succeeded.;
   %else %put ERROR: &_function failed - %sysfunc(sysmsg());
%mend Checkrc;

* check annotation parameters;
%macro CheckAnno(_CheckIndata=1,_CheckAccession=1,_CheckLabel=1,_CheckVarList=);
	%global AnnoData AnnoPath;
   %if %length(&AnnoInData) %then %do;
      %if &SASMode ne SDD %then %do;
      	%if %sysfunc(fileexist(&AnnoInData)) = 0 %then %do;
      	   %let exiterror = 1;
      	   %put ERROR: Specified AnnoInData does not exist.;
      	   %goto caexit;
      	%end;
      %end;
      %PathName(&AnnoInData);
      %if &exiterror %then %do;
         %put ERROR: Annotation data set specification is invalid.;
         %goto caexit;
      %end;
      %let AnnoData = &TmpName;
      %let AnnoPath = &TmpPath;
      libname AnnoLib "&AnnoPath";
      %CheckForBlank(&AnnoMergeVars,AnnoMergeVars);
      %if (&_checkindata=1) %then %do;
         %if %length(&BaseMergeVars) | %length(&AnnoMergeVars) %then %do;
            %if ^%length(&BaseMergeVars) %then %let BaseMergeVars = &AnnoMergeVars;
            %else %if ^%length(&AnnoMergeVars) %then %let AnnoMergeVars = &BaseMergeVars;
            %CheckVarExist(&BaseMergeVars,InLib.&InData,Base Merge Variables);
            %CheckVarExist(&AnnoMergeVars,AnnoLib.&AnnoData,Anno Merge Variables);
            %if &exiterror %then %goto caexit;
         %end;
      %end;
      %else %do;
         %CheckVarExist(&AnnoMergeVars,AnnoLib.&AnnoData,AnnoMergeVars);
      %end;
      %if (&_CheckAccession) %then %do;
         %if %length(&AnnoAccession) %then %do;
            %CheckVarExist(&AnnoAccession,AnnoLib.&AnnoData,AnnoAccession);
         %end;
         %else %put NOTE: Functions using annotation accession numbers are not included in this analysis.;
      %end;
      %if (&_CheckLabel) %then %do;
         %if %length(&AnnoLabel) %then %do;
            %CheckVarExist(&AnnoLabel,AnnoLib.&AnnoData,AnnoLabel);
         %end;
         %else %put NOTE: Annotation labels are not included in this analysis.;
      %end;
      %if %length(&_CheckVarList) %then %do;
         %let _i_ = 1;
         %let _VarNotFound_ = 0;
         %do %while(%length(%scan(&BaseMergeVars,&_i_)));
           %if ^%index(%upcase(&&&_CheckVarList),%upcase(%trim(%left(%scan(&BaseMergeVars,&_i_))))) %then %do;
               %let _VarNotFound_ = 1;
           %end;
           %let _i_ = %eval(&_i_+1);
         %end;
         %if (&_VarNotFound_) %then %do;
            %put ERROR: BaseMergeVars (or AnnoMergeVars) must be contained in &_CheckVarList in order to combine AnnoInData.;
            %let exiterror = 1;
         %end;
         * may need to add code for hadling descending case;
      %end;
      %if &exiterror %then %goto caexit;
   %end;
   %else %do;
      %put NOTE: Annotation information is not included in this analysis.;
      %let AnnoMergeVars = ;
      %let AnnoAccession = ;
      %let AnnoLabel = ;
      %let AnnoData = ;
      %let AnnoPath = ;
   %end;
   %caexit:;
%mend CheckAnno;

%macro OnlyAnno;
	%global indat;
	
	* check compatibility of varlists;
	%let _i_ = 1;
	%do %while(%length(%scan(&AnnoMergeVars,&_i_)));
		%let _aType&_i_ =;
		%VarAttributes(AnnoLib.&AnnoData,%scan(&AnnoMergeVars,&_i_),_type=_aType&_i_);
		%put Type of anno var %scan(&AnnoMergeVars,&_i_) =  &&_aType&_i_;
		%let _i_ = %eval(&_i_+1);
	%end;
	%let _j_ = 1;
	%do %while(%length(%scan(&BaseMergeVars,&_j_)));
		%let _bType&_j_ =;
		%VarAttributes(InDataAll,%scan(&BaseMergeVars,&_j_),_type=_bType&_j_);
		%put Type of base var %scan(&BaseMergeVars,&_j_) =  &&_bType&_j_;
		%let _j_ = %eval(&_j_+1);
	%end;
	%let _i_ = %eval(&_i_-1); 
	%let _j_ = %eval(&_j_-1); 
	%if &_i_ ^= &_j_ %then %do;
		%put ERROR: The number of merge key variables is not the same for the Input Data Set and Annotation Data Set.;
		%put You must specify the same number of merge key variables in order to merge annotation data.;
		%let exiterror = 1;
		%goto exit;
	%end;
	%do _k_ = 1 %to &_i_;
		%if &&_aType&_k_ ^= &&_bType&_k_ %then %do;
	
				%if &&_bType&_k_=C %then %do;
					%let BaseMergeTmp =%scan(&BaseMergeVars,&_k_);
					data InDataAll;
						set InDataAll(rename=(&BaseMergeTmp=cCol2));
						&BaseMergeTmp=cCol2+0;
						drop cCol2;
					run;
				%end;	
				%if &&_aType&_k_=C %then %do;
					%let AnnoMergeTmp =%scan(&AnnoMergeVars,&_k_);
					data AnnoLib.&AnnoData;
							set AnnoLib.&AnnoData;
							cCol = 0+ &AnnoMergeTmp;
							drop &AnnoMergeTmp;
					run;
					data AnnoLib.&AnnoData;
							set AnnoLib.&AnnoData;
							rename cCol =&AnnoMergeTmp;
					run;
					
				%end;	
			 %end;
		/*	 %else %do;
				 %put ERROR: Your merge key should use the same types, either numeric or character.;
				 %let exiterror =1;
			 %end;*/
		
	%end;
/*	%if &exiterror = 1 %then %goto exit;*/

	* use hash key to subset;
	%if %upcase(&BaseMergeVars) = %upcase(&AnnoMergeVars) %then %do;
		%let adat = AnnoLib.&AnnoData;
	%end;
	%else %do;
		data adat;
			set AnnoLib.&AnnoData;
			%let i=1;
			%do %while(%length(%scan(&BaseMergeVars,&i)));
				%scan(&BaseMergeVars,&i)=%scan(&AnnoMergeVars,&i);
				%let i = %eval(&i+1);
			%end;
			keep &BaseMergeVars;
		run;
		%let adat = adat;
	%end;
	data InDataAll;
		if _N_ = 1 then do;
			declare hash h(dataset:"&adat");
			%let i = 1;
			%do %while(%length(%scan(&BaseMergeVars,&i)));
				h.defineKey("%scan(&BaseMergeVars,&i)");
				%let i = %eval(&i+1);
			%end;
			h.defineDone();
		end;
		set InDataAll;
		if h.find()=0 then output;
	run;
	%let indat = InDataAll;
	
	* save and replace InLib and InData;
	%let InDataNew = ;
	%PrepOutDS(InDataNew,InDataNewName,&InData,_ann);
	data &InDataNew;
		set InDataAll;
	run;
	%let InData = &InDataNewName;
	libname InLib "&OutPath";
%exit:;
%mend;

* merge anno data;
%macro MergeAnno(_BaseData,_OutData=,_AnnoKeepVars=,_NoMissKeys=,LComFlag=0);
   /* v1.3
   keep descending order if &AnnoMergeVar is one of ByVars;
   %let MergeVar = &AnnoMergeVar;
   %let a = 1;
   %let flag = 0;
   %do %while(%length(%scan(&ByVars,&a)));
      %if %upcase(%scan(&ByVars,&a)) = %upcase(&AnnoMergeVar) %then %let flag = &a;
      %let a = %eval(&a+1);
   %end;
   %if (&flag > 1) %then %do;
      %if %upcase(%scan(&ByVars,%eval(&flag-1)))=DESCENDING %then
       %let AnnoMergeVar = descending &AnnoMergeVar;
   %end; */
   %if %length(&_OutData) %then %let _oDs_ = &_OutData;
   %else %let _oDs_ = &_BaseData;
	
	%if %symexist(AnnoKeepVars) %then %do;
		%if ^%index(%upcase(&ProcessName),BASIC) & %length(&_AnnoKeepVars) %then %do;
			%let _AnnoKeepVars = &AnnoMergeVars &AnnoAccession &AnnoChrVar
				&AnnoPosVar &AnnoKeepVars &AnnoLabel &AnnoGeneSymbol &AnnoDescription
				&AnnoGeneId &AnnoSnpId;
		%end;
	%end;

	* check merging keys;
	%let _i_ = 1;
	%do %while(%length(%scan(&AnnoMergeVars,&_i_)));
		%let _aType&_i_ =;
		%VarAttributes(AnnoLib.&AnnoData,%scan(&AnnoMergeVars,&_i_),_type=_aType&_i_);
		%let _i_ = %eval(&_i_+1);
	%end;
	%let _j_ = 1;
	%do %while(%length(%scan(&BaseMergeVars,&_j_)));
		%let _bType&_j_ =;
		%VarAttributes(&_oDs_,%scan(&BaseMergeVars,&_j_),_type=_bType&_j_);
		%let _j_ = %eval(&_j_+1);
	%end;
	%let _i_ = %eval(&_i_-1); 
	%let _j_ = %eval(&_j_-1); 
	%if &_i_ ^= &_j_ %then %do;
		%let exiterror = 1;
		%put ERROR: The number of merge key variables is not consistent for the Input Data Set and Annotation Data Set.;
		%goto exit;
	%end;
	/*
	%else %do; *checking var types;
		%let _c_ = 0;
		%let _n_ = 0;
		%do _k_ = 1 %to &_i_; 
			%if &&_aType&_k_ ne &&_bType&_k_ %then %do;
				%if %upcase(&&_aType&_k_) = C %then %do; *switch to character type;
					%let _c_ = %eval(&_c_+1);
					%let _cVar&_c_ = %scan(&BaseMergeVars,&_k_);
				%end;
				%else %do; *switch to numerical type;
					%let _n_ = %eval(&_n_+1);
					%let _nVar&_n_ = %scan(&BaseMergeVars,&_k_);
				%end;
			%end;
		%end;

		%if &_c_ %then %do; *set length for string type;
			proc means data=&_oDs_ max noprint;
				var 
					%do _i_ = 1 %to &_c_;
						&&_cVar&_i_
					%end;
				;
				output out=tmp;
			run;

			data _null_;
				set tmp;
				if _STAT_ = "MAX";
				%do _i_ = 1 %to &_c_;
					call symput("_cLen&_i_",length(trim(left(put(&&_cVar&_i_,32.)))));
				%end;
			run;
		%end;

		%if &_c_ | &_n_ %then %do;
			data &_oDs_;
				set &_oDs_;
				%do _i_ = 1 %to &_c_;
					_NewcVar_&_i_ = trim(left(put(&&_cVar&_i_,&&_cLen&_i_...)));
				%end;
				%do _j_ = 1 %to &_n_;
					_NewnVar_&_j_ = input(&&_nVar&_i_,8.);
				%end;
				drop
					%do _i_ = 1 %to &_c_;
						&&_cVar&_i_
					%end;
					%do _j_ = 1 %to &_n_;
						&&_nVar&_i_
					%end;
				;
				rename 
					%do _i_ = 1 %to &_c_;
						_NewcVar_&_i_ = &&_cVar&_i_
					%end;
					%do _j_ = 1 %to &_n_;
						_NewnVar_&_j_  = &&_nVar&_i_
					%end;
				;
			run;
		%end; 
	%end;
	*/

	* drop out variables which are common in the main data set to avoid mismatch var types;
	* get var names from anno dataset; 
	%let _AnnoVarList_ = ; 
	%let _nAnnoVar_ = ;
	%GetVars(AnnoLib.&AnnoData,_AnnoVarList_,_nVars=_nAnnoVar_);
	%let _dropAnnoVar_ = ;
	%do _na_ = 1 %to &_nAnnoVar_;
		%let _TmpVar_ = %scan(&_AnnoVarList_,&_na_);
		%let _cFlag_ = 1;
		%if ^%index(%upcase(&AnnoMergeVars),%upcase(&_TmpVar_)) %then %do;
			%if %length(&_AnnoKeepVars) %then %do;
				%if ^%index(%upcase(&_AnnoKeepVars),%upcase(&_TmpVar_)) %then %do;
					%let _cFlag_ = 0;
				%end;
			%end;
			%if &_cFlag_ %then %do;
				%CheckVarExist(&_TmpVar_,&_BaseData,_ShowError=0);
				%if ^&exiterror %then %do;
					* variable contained in both anno and main data set;
					%let _dropAnnoVar_ = &_dropAnnoVar_ &_TmpVar_;
				%end;
				%else %do;
					* &_tmpVar is not contained in main data set and reset the error flag;
					%let exiterror = 0;
				%end;
			%end;
		%end;
	%end;
	%let _AnnoDs_ = AnnoLib.&AnnoData;
	%if %length(&_AnnoKeepVars) | %length(&_dropAnnoVar_) %then %do;
		data Anno;
			set AnnoLib.&AnnoData;
			%if %length(&_AnnoKeepVars) %then %do;
				keep &AnnoMergeVars &_AnnoKeepVars;
			%end;
			%if %length(&_dropAnnoVar_) %then %do;
				drop &_dropAnnoVar_;
			%end;
		run;
		%let _AnnoDs_ = Anno;
	%end;
	/*
	%TrimStringVariabe(_Ds=&_AnnoDs_,_oDs=Anno,_MaxLen=256);
	*/
	%if &LComFlag %then %do;
   	%Merge2sets(&_AnnoDs_,&_BaseData,&_oDs_,&AnnoMergeVars,&BaseMergeVars,
			_SortFlag1=1,_Subset1=1,_Subset2=1,_NoMissVars=&_NoMissKeys);
	%end;
	%else %do;
   	%Merge2sets(&_AnnoDs_,&_BaseData,&_oDs_,&AnnoMergeVars,&BaseMergeVars,
			_SortFlag1=1,_Subset2=1,_NoMissVars=&_NoMissKeys);
	%end;
%exit:;
%mend MergeAnno;

/*
separate all under-merge datasets by _ChunkSize.  Merge within each chunk. 
Then, merge all chunk. This is useful for avoiding memory problem when merging 
hugh datasets.
_InDs : input data set with listing datasets under variable "Data" and listing
	variables to keep under optional varialbe "Keep"
_OutDs : output dataset 
_ChunkSize : chunk size
_ByVar : by variables for merging
_DropByVar : Indicator to drop _ByVar
_RemoveInDs : Indicator to remove all individaul datasets;
*/
%macro MergeByChunk(_InDs,_OutDs,_ChunkSize,_ByVar=,_DropByVar=1,_RemoveInDs=1);
%CheckVarExist(keep,&_InDs,,_ShowError=0);
%if &exiterror %then %do;
	%let _kFlag_ = 0;
	%let exiterror = 0;
%end;
%else %do; 
	%let _kFlag_ = 1;
%end;

data _null;
	set &_InDs end=_e_;
	name = "_Ds_"||trim(left(_n_));
	call symput(name,Data);	
	%if &_kFlag_ %then %do;
		name = "_kVar_"||trim(left(_n_));
		call symput(name,Keep);
	%end;
	if _e_ then call symput("_nSet_",_n_);
run;
%let _iChunk_ = 1;
%let _dsInChunk_ = 0;
%let _dList_ = ;
%do _i_ = 1 %to &_nSet_;
	%let _dsInChunk_ = %eval(&_dsInChunk_+1);
	%if &_dsInChunk_ > &_ChunkSize %then %do;
		%let _iChunk_ = %eval(&_iChunk_+1);	
		%let _dsInChunk_ = 1; *reset;
		%let _dList_ = ;
	%end;
	%let _dList_ = &_dList_ &&_Ds_&_i_;
	%if %length(&_ByVar) %then %do;
		proc sort data=&&_Ds_&_i_;
			by &_ByVar;
		run;
	%end;
	%if (&_dsInChunk_=&_ChunkSize) | (&_i_=&_nSet_) %then %do;
		data Chunk&_iChunk_;
			%if %length(&_ByVar) %then %do;
				merge 
					%do _j_ = 1 %to &_dsInChunk_;
						%let _k_ = %eval((&_iChunk_-1)*&_ChunkSize+&_j_);
						&&_Ds_&_k_ %if &_kFlag_ %then (Keep=&&_kVar_&_k_ &_ByVar);
					%end;
				;
				by &_ByVar;
			%end;
			%else %do;
				%do _j_ = 1 %to &_dsInChunk_;
					%let _k_ = %eval((&_iChunk_-1)*&_ChunkSize+&_j_);
					set &&_Ds_&_k_ %if &_kFlag_ %then (Keep=&&_kVar_&_k_);;
				%end;
			%end;
		run;
		%if &_RemoveInDs %then %RemoveData(&_dList_);
	%end;
%end;	

data &_OutDs;
	%if %length(&_ByVar) %then %do;
		merge
			%do _i_ = 1 %to &_iChunk_;
				Chunk&_i_
			%end;
		;
		by &_ByVar;
		%if &_DropByVar %then drop &_ByVar;; 
	%end;
	%else %do;
		%do _i_ = 1 %to &_iChunk_;
			set Chunk&_i_;
		%end;
	%end;	
run;
%do _i_ = 1 %to &_iChunk_;
	%RemoveData(Chunk&_i_);
%end;
%mend MergeByChunk;


%macro TrimStringVariabe(_Ds=,_oDs=,_MaxLen=256);
	%let _did_ = %sysfunc(open(&_Ds));
	%let _nVars_ = %sysfunc(attrn(&_did_,NVARS));
	%let _f_ = 0;
	%let _LenStmt_ = Length;
	%do _i_ = 1 %to &_nVars_;
		%let _Name_ = %sysfunc(varname(&_did_,&_i_));
		%let _Type_ = %sysfunc(vartype(&_did_,&_i_));
		%let _Len_ = %sysfunc(varlen(&_did_,&_i_));
		%if &_Type_ = C %then %do;
			%if &_Len_ > &_MaxLen %then %do; 
				%let _f_ = 1;
				%let _Len_ = &_MaxLen;
			%end;
			%let _Dollar_ = $;
		%end; 
		%else %do;
			%let _Dollar_ = ;
		%end;
		%*let _LenStmt_ = &_LenStmt_ &_Name_ &_Dollar_ &_Len_;
		%let _LenStmtS_&_i_ =  &_Name_ &_Dollar_ &_Len_;
	%end;
	%let _rc_ = %sysfunc(close(&_did_));
	%if &_f_ %then %put Variable(s) with length longer than &_MaxLen in data set &_ds were trimmed.;
	%if %length(&_oDs) %then %do;
		data &_oDs;
			%* &_LenStmt_;
			Length %do _i_ = 1 %to &_nVars_; &&_LenStmtS_&_i_ %end;;
			set &_Ds;
		run;
	%end;
	%else %if &_f_ %then %do;
		data &_Ds;;
			%* &_LenStmt_;
			Length %do _i_ = 1 %to &_nVars_; &&_LenStmtS_&_i_ %end;;
			set &_Ds;
		run;
	%end;
%mend TrimStringVariabe;

* Check if user provided data name is a valid SAS data set name;
%macro CheckDataName(_name=,_default=,_SDS=0);
	%if (&SASMode ne SDD) or (&_SDS=1) %then %do;
		%if ^%length(&&&_name) %then %let &_name = %trim(%left(&_default));
		data _null_; 
			name = trim(left(lowcase("&&&_name")));
			j=0;
			do until(j=0);
				j=notname(name,j+1);
				if j>0 then substr(name,j,1)=' ';
			end;
			name = translate(compbl(name),'_','');
			if substr(name,1,1) in ('0','1','2','3','4','5','6','7','8','9') then
				name2='_'||name;
			else name2=name;
			if length(name2) < 32 then call symput("&_name",trim(name2));
			else call symput("&_name",trim(substr(name2,1,min(length(name2),32))));
			/*
			%if %symexist(OutName) %then %do;
				else call symput("&_name",trim(substr(name2,1,min(length(name2),32))));
			%end;
			%else %do;
				else do;
					name3=substr(name2,length(name2)-31);
					if substr(name3,1,1) in ('0','1','2','3','4','5','6','7','8','9') then
						call symput("&_name",'_'||trim(substr(name3,2)));
					else call symput("&_name",trim(name3));
				end;
			%end;
			*/
		run;
	%end;
%mend CheckDataName;

* convert name to a valid SAS variable name;
%macro SASName(_name,compbl=0);
	%global sasname under1;
	data _null_; 
		name = trim(left(lowcase("&_name")));
		j=1;
		do until(j=0);
			j=notname(name,j+1);
			if j>0 then substr(name,j,1)=' ';
		end;
		%if &compbl %then %do;
			name = translate(compbl(name),'_','');
		%end;
		%else %do;
			name = translate(name,'_','');		
		%end;
		if substr(name,1,1) in ('0','1','2','3','4','5','6','7','8','9') then do;
			name2='_'||name;
			call symputx("under1",trim(1));
		end;
		else do;
		   name2=name;
			call symputx("under1",trim(0));		
		end;
		if length(name2) < 32 then call symput("sasname",trim(name2));
		else call symput("sasname",trim(substr(name2,1,min(length(name2),32))));
	run;
%mend SASName;

%macro CheckByVars(_OldList=,_NewList=,_nVar=tmp);
   %if %length(&_OldList) %then %do;
      %let &_NewList = ;
      %let &_nVar = 0;
      %let _i_ = 1;
      %do %while(%length(%scan(&_OldList,&_i_,' ')));
         %if %upcase(%scan(&_OldList,&_i_,' '))^=DESCENDING %then %do;
            %let &_NewList = &&&_NewList %scan(&_OldList,&_i_,' ');
            %let &_nVar = %eval(&&&_nVar+1);
         %end;
         %let _i_ = %eval(&_i_+1);
      %end;
   %end;
%mend CheckByVars;


%macro CheckLengthForMerging(_Ds1,_Ds2,_KeyVars);
   * This macro only handles two data sets.;
   %let _i_ = 1;
   %let _mod1_ = ;
   %let _mod2_ = ;
	%let _ctVar1_ = ;
	%let _ctVar2_ = ;
	%let _nctVar1_ = 0;
	%let _nctVar2_ = 0;
   %do %while(%length(%scan(&_KeyVars,&_i_)));
      %let _tmp&_i_._ = 0;
      %let _vName_ = %scan(&_KeyVars,&_i_);
      %let _d1_ = %sysfunc(open(&_Ds1));
      %let _vNum1_ = %sysfunc(varnum(&_d1_,&_vName_));
      %let _vType1_ = %sysfunc(vartype(&_d1_,&_vNum1_));
      %let _vLen1_ = %sysfunc(varlen(&_d1_,&_vNum1_));
      %let _rc_=%sysfunc(close(&_d1_));
      %let _d2_ = %sysfunc(open(&_Ds2));
      %let _vNum2_ = %sysfunc(varnum(&_d2_,&_vName_));
      %let _vType2_ = %sysfunc(vartype(&_d2_,&_vNum2_));
      %let _vLen2_ = %sysfunc(varlen(&_d2_,&_vNum2_));
      %let _rc_=%sysfunc(close(&_d2_));
		%let _1convert&_i_._ = 0;
		%let _2convert&_i_._ = 0;
      %if (&_vType1_ ne &_vType2_) %then %do;
         %put WARNING: Variable &_vName_ has mixed types in both data sets &_Ds1 and &_Ds2.;
         %put Type1 = &_vType1_  Type2 = &_vType2_;
			%put The character string variable will be converted into numerical variable before merging.;
         %if (&_vType1_ = C) %then %do;
				%let _vType1_ = N; * convert to numerical with length 8.;
				%let _vLen1_ = 8;
				%if &_vLen2_ < 8 %then %do;
					%let _vLen2_ = 8;
					%let _2convert&_i_._ = 1;
				%end;
				%let _1convert&_i_._ = 1;
				%let _ctVar1_ = &_ctVar1_ &_vName_;
				%let _nctVar1_ = %eval(&_nctVar1_+1);
			%end;
			%else %do;
				%let _vType2_ = N; * convert to numerical with length 8.;
				%let _vLen2_ = 8;
				%if &_vLen1_ < 8 %then %do;
					%let _vLen1_ = 8;
					%let _1convert&_i_._ = 1;
				%end;
				%let _2convert&_i_._ = 1;
				%let _ctVar2_ = &_ctVar2_ &_vName_;
				%let _nctVar2_ = %eval(&_nctVar2_+1);
			%end;
      %end;
      %if (&_vType1_ = C) %then %let _symbol_ = $;
      %else %let _symbol_ = ;
      %if (&_vLen1_ > &_vLen2_) | (&&_2convert&_i_._) %then %do;
			%if &&_2convert&_i_._ %then %do;
				%if &_vLen1_ > &_vLen2_ %then %let _mod2_ = &_mod2_ _ctName&_i_ &_vLen1_..;
				%else %let _mod2_ = &_mod2_ _ctName&_i_ &_vLen2_..;
			%end;
			%else %do;
				%let _mod2_ = &_mod2_ &_vName_ &_symbol_.&_vLen1_..;
			%end;
		%end;
      %if (&_vLen1_ < &_vLen2_) | (&&_1convert&_i_._) %then %do;
			%if &&_1convert&_i_._ %then %do;
				%if &_vLen1_ > &_vLen2_ %then %let _mod1_ = &_mod1_ _ctName&_i_ &_vLen1_..;
				%else %let _mod1_ = &_mod1_ _ctName&_i_ &_vLen2_..;
			%end;
			%else %do;	
				%let _mod1_ = &_mod1_ &_vName_ &_symbol_.&_vLen2_..;
			%end;
		%end;
%*put i=&_i_ l1=&_vLen1_ l2=&_vLen2_;
%*put i=&_i_ c1=&&_1convert&_i_._ c2=&&_2convert&_i_._;
      %let _i_ = %eval(&_i_+1); 
   %end;
%*put Mod1= &_mod1_;
%*put Mod2= &_mod2_;
   %if %length(&_mod1_) %then %do;
      data &_ds1;
      	length &_mod1_;
      	set &_ds1;
			%do _j_ = 1 %to %eval(&_i_-1);
		  		%if &&_1convert&_j_._ %then %do;
					_ctName&_j_ = %scan(&_KeyVars,&_j_) + 0;;
					drop %scan(&_KeyVars,&_j_);
					rename _ctName&_j_ = %scan(&_KeyVars,&_j_);
				%end;
			%end;
   	run;
		* check if truly wrong var type;
		%if (&_nctVar1_) %then %do;
	  		data _null_;
				set &_ds1(keep=&_ctVar1_) end=_e_;
				%do _k_ = 1 %to &_nctVar1_;
					retain _nMiss&_k_ 0;
					if %scan(&_ctVar1_,&_k_) = . then _nMiss&_k_ = _nMiss&_k_ + 1;
				%end;
				if (_n_=2000) | _e_ then do;
					%do _k_ = 1 %to &_nctVar1_;
						if _nMiss&_k_ / _n_ > .95 then call symput("_wType&_k_._",&_k_);
						else call symput("_wType&_k_._",0);
					%end; 
					stop;
				end;
			run;
			%do _k_ = 1 %to &_nctVar1_;
				%if &&_wType&_k_._ %then %do;
					%put ERROR: Failed to convert variable %scan(&_ctVar1_,&_k_) into numeric variable in data set &_DS1..;
					%let exiterror = 1;
					%goto exit;
				%end;
			%end;
		%end;
   %end;

   %if %length(&_mod2_) %then %do;
      data &_ds2;
        length &_mod2_;
      	set &_ds2;
			%do _j_ = 1 %to %eval(&_i_-1);
		  		%if &&_2convert&_j_._ %then %do;
					_ctName&_j_ = %scan(&_KeyVars,&_j_) + 0;;
					drop %scan(&_KeyVars,&_j_);
					rename _ctName&_j_ = %scan(&_KeyVars,&_j_);
				%end;
			%end;
   	run;
		* check if truly wrong var type;
		%if (&_nctVar2_) %then %do;
	  		data _null_;
				set &_ds2(keep=&_ctVar2_) end=_e_;
				%do _k_ = 1 %to &_nctVar2_;
					retain _nMiss&_k_ 0;
					if %scan(&_ctVar2_,&_k_) = . then _nMiss&_k_ = _nMiss&_k_ + 1;
				%end;
				if (_n_=2000) | _e_ then do;
					%do _k_ = 1 %to &_nctVar2_;
						if _nMiss&_k_ / _n_ > .95 then call symput("_wType&_k_._",&_k_);
						else call symput("_wType&_k_._",0);
					%end; 
					stop;
				end;
			run;
			%do _k_ = 1 %to &_nctVar2_;
				%if &&_wType&_k_._ %then %do;
					%put ERROR: Failed to convert variable %scan(&_ctVar2_,&_k_) into numerical variable in data set &_DS2..;
					%let exiterror = 1;
					%goto exit;
				%end;
			%end;
		%end;
   %end;
%exit:;
%mend CheckLengthForMerging;


* extract attributes (length and type) of variable;
%macro VarAttributes(_Ds,_Var,_Len=,_Type=);
	%let dsid=%sysfunc(open(&_Ds));
	%if &dsid %then %do;
		%let lv = %length(&_Var);
		%if &lv > 0 and &lv <= 32 %then %do;
			%let vnum = %sysfunc(varnum(&dsid,&_Var));
			%if &vnum %then %do;
				%if %length(&_Len) %then %let &_Len=%sysfunc(varlen(&dsid,&vnum));
				%if %length(&_Type) %then %let &_Type=%sysfunc(vartype(&dsid,&vnum));
			%end;
		%end;
		%let rc=%sysfunc(close(&dsid));
	%end;
%mend VarAttributes;

* remove datasets;
%macro RemoveData(_dList,_Lib=);
	%if %length(&_dList) %then %do;
		%if ^%length(&_Lib) %then %let _Lib = Work;
		proc datasets library=&_Lib nolist; 
			delete &_dList; 
		quit;
	%end;
%mend RemoveData;

* Check Excel file and OS system;
%macro CheckExcel(_Type,_ErrMsge,_OS=WIN);
   %if (%length(&_Type)>3) & (&sysscp ^= &_OS) %then %do;
      %put ERROR: Excel type of &_ErrMsge can not be imported by Unix server.;
     %let exiterror = 1;
   %end;
%mend CheckExcel;

* Check file type;
%macro CheckFileType(_File,_FileType,_ErrorFlag=1);
   %let _len_ = %length(&_File);
   %if %substr(&_File,%eval(&_len_-3),1) = . %then
      %let _exten_ = %upcase(%substr(&_File,%eval(&_len_-3)));
   %else %if %substr(&_File,%eval(&_len_-4),1) = . %then
      %let _exten_ = %upcase(%substr(&_File,%eval(&_len_-4)));
   %else %let _exten_ = %upcase(%substr(&_File,%eval(&_len_-8)));
   
   %if (&_exten_ = .XLS) %then %let &_FileType = XLS;
   %else %if (&_exten_ = .XLSX) %then %let &_FileType = EXCEL2007;
   %else %if (&_exten_ = .CSV) %then %let &_FileType = CSV;
   %else %if (&_exten_ = .TXT) %then %let &_FileType = TAB;
   %*else %if (&_exten_ = .DAT) %then %let &_FileType = TAB;
   %else %if (&_exten_ = .SAS7BDAT) %then %let &_FileType = SAS;
   %else %do;
	   %if &_ErrorFlag %then %do;
         %let exiterror = 1;
         %put ERROR: Specified raw file extension must be .csv, .sas7bdat, .txt, or .xls.;
         %put ERROR: Please specify the data file type.;
		%end;
		%else %do;
         %if %length(&FileType) = 0 %then %do;
            %put ERROR: File Type specification is needed.;
            %let exiterror = 1;
         %end;
		%end;
   %end;
   %if %sysfunc(fileexist(&_File)) = 0 %then %do;
      %let exiterror = 1;
      %put ERROR: &_File does not exist.;
   %end;
%mend CheckFileType;

* extract the phisical starting location of the # of line (_Line) in a text file (_File);
%macro GetLineLoc(_File,_line,_StartLoc);
	%*global &_StartLoc;
	* read file with binary format and count the number of carriage return;
	data _null_;
   	infile "&_File" RECFM=N delimiter='0A'x col=columnpt DSD lrecl=32767;
		length dummy $32767; 
		retain StartLoc 1 nrows 1;
		do while(nrows < &_Line);
			PreLoc = StartLoc;
			input dummy $@;
			StartLoc = columnpt;
			*put nrows= PreLoc= StartLoc= ;
			if PreLoc = StartLoc then nrows = nrows + 1;
			else input +(-1) @; * back 1 for trapping cr;
		end;	
		call symputx("&_StartLoc",StartLoc); * location of the last carriage return before &VarNameRow;
		stop;
	run;
%mend GetLineLoc;

*Extract file format (Win or Unit) for &_File_ and put value in &_FileFormat;
%macro GetFileFormat(_File,_FileFormat);
	%let _tmp_ =;
	%GetLineLoc(&_File,2,_tmp_);
	data _null_; t=&_tmp_;
  		infile "&_File" RECFM=N DSD;
  		input +(%eval(&_tmp_-3)) x $5. @;
  		if index(x,'0D'x) then do;
  			put "This is Win file!";
  			call symput("&_FileFormat",'Win');
  		end;
  		else do;
  			put "This is Unix file!";
  			call symput("&_FileFormat",'Unit');
  		end; output;
  		stop;
  	run;
%mend GetFileFormat;

/*
Merge two data sets with specified merging keys.
When applying multiple merging keys, each component
of the same rank in Keys1 and Keys2 will be treated
as correponding pair. The variable names will be
based on the names in Keys1 when applying different
names for Keys1 and Keys2.;
*/
%macro Merge2sets(_DS1,_DS2,_MergedData,_Keys1,_Keys2,_SubSet1=0,_SubSet2=0,
		_NoMissVars=,_DropVars=,_KeepVars1=,_KeepVars2=,_SortFlag1=0,
		_SortFlag2=0,_CaseFlag=0);
	%put Start to merge &_DS1 with &_DS2;
	
	* set up libname and memname for first data set;
   %let _DotTmp_ = %index(&_DS1,.);
   %if &_DotTmp_ %then %do;
      %let _LibTmp_ = %substr(&_DS1,1,%eval(&_DotTmp_-1));
      %let _MemTmp_ = %substr(&_DS1,%eval(&_DotTmp_+1));
   %end;
   %else %do;
      %let _LibTmp_ = Work;
      %let _MemTmp_ = &_DS1;
   %end;
   
   %let _path1_ = %sysfunc(pathname(&_LibTmp_));
   %let _path2_ = %sysfunc(pathname(Work));
   %checkendslash(_path1_);
   %checkendslash(_path2_);
   /*
   %put _path1_= &_path1_;
   %put _path2_= &_path2_;
   */
	* make a working copy of the first data set to avoid modification of the original one;
	%if &_path1_ ^= &_path2_ %then %do;
		%removeData(&_MemTmp_);
		proc datasets library=&_LibTmp_ nolist;
			copy out=Work;
			select &_MemTmp_;
		quit;
		%let _LibTmp_ = Work;
	%end;
	%let _dsb_ = &_LibTmp_..&_MemTmp_;

	* set up libname and memname for second data set;
   %let _DotTmp_ = %index(&_DS2,.);
   %if &_DotTmp_ %then %do;
      %let _LibTmp_ = %substr(&_DS2,1,%eval(&_DotTmp_-1));
      %let _MemTmp_ = %substr(&_DS2,%eval(&_DotTmp_+1));
   %end;
   %else %do;
      %let _LibTmp_ = Work;
      %let _MemTmp_ = &_DS2;
   %end;
   %let _path1_ = %sysfunc(pathname(&_LibTmp_));
   %let _path2_ = %sysfunc(pathname(Work));
   %checkendslash(_path1_);
   %checkendslash(_path2_);

	* make a working copy of the second data set to avoid modification of the original one;
	%if &_path1_ ^= &_path2_ %then %do;
		%removeData(&_MemTmp_);
		proc datasets library=&_LibTmp_ nolist;
			copy out=Work;
			select &_MemTmp_;
		quit;
		%let _LibTmp_ = Work;
	%end;

	%let _dsm_ = &_LibTmp_..&_MemTmp_;

	* make sure the number of key variables is the same;
   %let _i_ = 1;
	%do %while(%length(%scan(&_Keys1,&_i_)));
		%let _i_ = %eval(&_i_+1);
	%end;
	%let _nKeys_ = %eval(&_i_-1); 

   %let _i_ = 1;
	%do %while(%length(%scan(&_Keys2,&_i_)));
		%let _i_ = %eval(&_i_+1);
	%end;
	%if &_nKeys_ ^= %eval(&_i_-1) %then %do;
		%put ERROR: The number of merge key variables must be the same for both data sets.;
		%let exiterror = 1;
		%goto exit;
	%end;

	* if merge key variable names differ, create new columns in second data set with matching names;
	%if %upcase(&_Keys1) ^= %upcase(&_Keys2) %then %do;
	
		data &_dsm_(compress=yes);
			set &_dsm_;
			%do _i_ = 1 %to &_nKeys_;
				%let _bkvi_ = %upcase(%scan(&_Keys1,&_i_));
				%let _mkvi_ = %upcase(%scan(&_Keys2,&_i_));
				%if &_bkvi_ ne &_mkvi_ %then %do;
					&_bkvi_ = &_mkvi_;
				%end;
			%end; 
		run;
		
		/*	
		proc datasets library=&_LibTmp_ nolist;
			modify &_MemTmp_;
				rename
				%do _i_ = 1 %to &_nKeys_;
					%let _bkvi_ = %scan(&_Keys1,&_i_);
					%let _mkvi_ = %scan(&_Keys2,&_i_);
					%if &_bkvi_ ne &_mkvi_ %then %do;
						&_mkvi_ = &_bkvi_
					%end;
				%end; ;
		quit;
		*/
	%end;

   %CheckLengthForMerging(&_dsb_,&_dsm_,&_Keys1);
	%if &exiterror %then %goto exit;

	%if ^&_CaseFlag %then %do;
		* This may take time to adjust case insensitive for very large or wide data set!!;
		data _dsb_(compress=yes);
			set &_dsb_;
			%do _i_ = 1 %to &_nKeys_;
				%if &_i_ = 1 %then %let _newKeys_ = _key_&_i_._;
				%else %let _newKeys_ = &_newKeys_ _key_&_i_._;
				_key_&_i_._ = upcase(trim(left(%scan(&_Keys1,&_i_))));
			%end;
		run;
		data _dsm_(compress=yes);
			set &_dsm_;
			%do _i_ = 1 %to &_nKeys_;
				%if &_i_ = 1 %then %let _newKeys_ = _key_&_i_._;
				%else %let _newKeys_ = &_newKeys_ _key_&_i_._;
				_key_&_i_._ = upcase(trim(left(%scan(&_Keys1,&_i_))));
			%end;
		run;
		%let _Keys1 = &_newKeys_;
		%let _dsb_ = _dsb_;
		%let _dsm_ = _dsm_;
	%end;

   proc sort data=&_dsb_(compress=yes %if %length(&_KeepVars1) %then Keep=&_Keys1 &_KeepVars1; );
      by &_Keys1;
   run;

   proc sort data=&_dsm_(compress=yes %if %length(&_KeepVars2) %then Keep=&_Keys1 &_KeepVars2; );
      by &_Keys1;
   run;

   data &_mergedData(compress=yes);
      merge &_dsb_(in=_select1_) &_dsm_(in=_select2_);
      by &_Keys1;
      %if (&_Subset1) & (&_Subset2) %then if _select1_ & _select2_;
      %else %if (&_Subset1) %then if _select1_;
      %else %if (&_Subset2) %then if _select2_; ;
      %if %length(&_NoMissVars) %then %do;
         %let _i_ = 1;
         %do %while(%length(%scan(&_NoMissVars,&_i_)));
            if %scan(&_NoMissVars,&_i_) ne .;
            %let _i_ = %eval(&_i_+1);
         %end;
      %end;
      %if %length(&_DropVars) %then drop &_DropVars;;
		%if ^&_CaseFlag %then drop &_Keys1;;
   run;
%exit:;
%mend Merge2Sets;


* start timing;
%macro ClockStart(_ProcessName,_Initiate=0);

	%let clockrunning = 1;
	data tmp_clock;
		format Process $32. Section $32. StartingDay date7. StartingTime time11.2
			TotalTime time11.2 Status $10. Product $13. Version 4.1  Notes $100. User $32. 
			ProcessNumber 3. ProcessNumber2 3.;
		if "&ProcessName"="&_ProcessName" then do;
			Process = "&ProcessName";
			Section = "";
		end;
		else do;
			Process = ""; 
			Section = "&_ProcessName";
		end;
		StartingDay = round( date() );
		StartingTime = round( time(),.01 );
		TotalTime = .;
		Status = "Processing";
		Product = "&LS_Product";
		Version = %scan(&jmplsversion,-1,' ');
		Notes = " ";
		User = "&sysuserid";
		put "&_ProcessName started";
*		put "&_ProcessName started on "  StartingDay date7. " " StartingTime time11.2 ".";
	run;
	
	%if &_Initiate = 1 %then %do;

		%if ^%symexist(UseGrid) %then %let UseGrid = 0;
		%if (&SASMode ne SDD) and &UseGrid = 0 %then %do; * turn off this Project Log for SDS and Grid;
			%if %symexist(Study_Home) %then %do;
				* with user specified Study;
				%let ProjectID = &StudyName;
			%end;
			%else %do;
				%let ProjectID = LastRun;
			%end;
			filename ProjLog "%sysfunc(pathname(_LogLib_))&_slash_.StudyLog.txt";
			/*
			%if (&Server = Local) %then %do;
				%if %symexist(Study_Home) %then %do;
					filename ProjLog "%quote(&Study_Home)StudyLog.txt";			
				%end;
				%else %do;
					filename ProjLog "&OutPath.StudyLog.txt";
				%end;
			%end;
			%else %if %symexist(metapath) %then %do;				
				%if %symexist(Study_Home) %then %do;
					%let study_name = %sysfunc(transtrn(&ProjectID, %str( ), _));
					filename ProjLog "&metapath.Metadata&_slash_.&study_name.&_slash_.StudyLog.txt";
				%end;
				%else %do;
					* keep studylog in server outpath;
					filename ProjLog "&ServerOutPath.StudyLog.txt";
				%end;
			%end;
			*/
			/*
			%if %sysfunc(fexist(ProjLog)) %then %do;
				%let NewProjFlag = 0; 
			%end;
			%else %do;
				%let NewProjFlag = 1; 
			%end;
			*/
			%*if (&Server = Local) or %symexist(metapath) %then %do;
				%*if (&Server = Local) %then %do;		
					* Write out records to Project Log;
					%let _nProj_ = 1;
					%let _nThePro_ = 001;
					%if ^%sysfunc(fexist(ProjLog)) | ^%sysfunc(exist(_LogLib_._processinfo)) %then %do;
						* initiate Project Log file; 
						data _null_;
							file ProjLog;
							put "Study Name = &ProjectID";
						run;
					%end;
					%else %do;
						data _null_;
							set _LogLib_._processinfo;
							if _n_ = 1 then call symputx('_nProj_',ProcessNumber+1);
							if Process = "&ProcessName" then do;
								call symputx('_nThePro_',ProcessNumber2+1);
								stop;
							end;
						run;
						%if &_nThePro_ ^= . %then %do;
							%if &_nThePro_<10 %then %let _nThePro_ = 00%trim(%left(%eval(&_nThePro_+0)));
							%else %if &_nThePro_<100 %then %let _nThePro_ = 0%trim(%left(&_nThePro_)); 
							%else %let _nThePro_ = %trim(%left(&_nThePro_)); 
						%end;
					%end;
					/*					
					data _tmpc_;
						infile ProjLog;
						retain p 0;
						length x $15;
						input x $;
						if substr(x,1,8) = '[Process' then p+1;
					run;

					data _null_;
						set _tmpc_ end=_end_;
						if _end_ then call symputx('_nProj_',trim(left(p+1)));
					run;
					*/
					data _null_;
						file ProjLog MOD;
						put " ";
						put "[Process &_nProj_]";
						put "   ProcessName= &ProcessName (# &_nThePro_)";
						* check input;
						%if %symexist(InData) %then put "   InData= &InData";
						%else %if %symexist(DataInPath) %then put "   DataInPath= &DataInPath";;
						%if %symexist(ExpInData) %then put "   ExpInData= &ExpInData";
						%else %if %symexist(ExpInFile) %then put "   ExpInFile= &ExpInFile";;
					run;
				%*end;
			%*end;			
		%end;
	   
		data _pinfo;
			set tmp_clock;
			ProcessNumber = &_nProj_ + 0;
			ProcessNumber2 = &_nThePro_ + 0;
		run;
		/*
		proc print;
		run;
		*/
	%end;
	%else %do;
		data _pinfo;
			set _pinfo tmp_clock;
		run;
	%end;

%mend ClockStart;


* stop timing;
%macro ClockStop(_ProcessName,_end=0,_Notes=" ",_Section=" ");
   data tmp_clock;
      set _pinfo;
      Obs = _n_;
      if TotalTime = .;
   run;

   data tmp_clock;
      set tmp_clock end=eof;
      if eof then call symput('_Current_',Obs);
   run;

	%if %symexist(_current_) %then %do;
		data _pinfo;
			set _pinfo;
			if _n_ = &_Current_ then do;
				Day = round( date() );
				Time = round( time(),.01 );
				if StartingDay = Day then TotalTime = round(Time-StartingTime,.01);
				else TotalTime = (Day-StartingDay)*86400 + round(Time-StartingTime,.01);
				Status = "Processed";
				Notes = &_Notes;
				*ProcessNumber = &_nProj_;
				*ProcessNumber2 = &_nThePro_+0;
				if lengthn(&_Section) then Section = &_Section;
				%if &syserr > 4 or &exiterror = 1 %then %do;
					Notes = Notes || ' ERROR(s)';
				%end;
				put "&_ProcessName finished on "  Day date7. " " Time time11.2 ".";
				put "The total time for &_ProcessName was " Totaltime time11.2 ".";
			end;
			drop Day Time;
		run;
	%end;
	%if &_end %then %do;
		%if (&ProcessName=ManageStudies) %then %do;
			%if (%quote(&Task)=Rename Study) %then %do;
				%let NewStudyName_ = %sysfunc(translate(&NewStudyName,%str(_),%str( )));
				%let _LogLib_ = NewMeta;
			%end;
			%else %if (%quote(&Task)=Remove Study) %then %do;
				%let _LogLib_ = NotesLib;
			%end;
			%else %do;
				%let _LogLib_ = _LogLib_;
			%end;				
		%end;
		%else %if (&ProcessName=ManageGenomicsStudy) %then %do;
			%let _LogLib_=OutLib;
		%end;
		%else %do;
			%let _LogLib_ = _LogLib_;
		%end;
		%if %sysfunc(exist(&_LogLib_.._processinfo)) %then %do;
			data &_LogLib_.._processinfo;
				set _pinfo &_LogLib_.._processinfo;
			run;
		%end;
		%else %do;
			data &_LogLib_.._processinfo;
				set _pinfo;
			run;
		%end;
		/*
		%if %symexist(Study_Home)=0 %then %do;
			%if %sysfunc(exist(OutLib._processinfo)) %then %do;
				data OutLib._processinfo;
					set _pinfo OutLib._processinfo;
				run;
			%end;
			%else %do;
				data OutLib._processinfo;
					set _pinfo;
				run;
			%end;
		%end;
		%else %do;
			%*if &Server = Local & (&ProcessName ne ManageStudies) & (&ProcessName ne ManageGenomicsStudy) %then %do;
			%if &Server = Local %then %do;
				%if %symexist(Study_Home) %then %do;
					libname SHome "&Study_Home";
					%if %sysfunc(exist(SHome._processinfo)) %then %do;
						data SHome._processinfo;
							set _pinfo SHome._processinfo;
						run;
					%end;
					%else %do;
						data SHome._processinfo;
							set _pinfo;
						run;
					%end;	
				%end;
				%else %do;
					%if %sysfunc(exist(OutLib._processinfo)) %then %do;
						data OutLib._processinfo;
							set _pinfo OutLib._processinfo;
						run;
					%end;
					%else %do;
						data OutLib._processinfo;
							set _pinfo;
						run;
					%end;		
				%end;
			%end;
			%else %do;
				%let study_name = %sysfunc(transtrn(&StudyName, %str( ), _));
				libname S_mlib "&metapath.Metadata&_slash_.&study_name";
				%if %sysfunc(exist(S_metalib._processinfo)) %then %do;
					data S_mlib._processinfo;
						set _pinfo OutLib._processinfo;
					run;
				%end;
				%else %do;
					data S_mlib._processinfo;
						set _pinfo;
					run;
				%end;		
			%end;
		%end;*/
	%end;
%mend ClockStop;

* write current day and time to log with prefixing message;
%macro ShowTime(msg);
	%global currday currtime;
	data _null_;
		CurrDay = round( date() );
		CurrTime = round( time(),.01 );
		call symput("currday",trim(left(put(CurrDay,date7.))));
		call symput("currtime",trim(left(put(CurrTime,time11.2))));
	run;
	%put &msg &currday &currtime;
%mend;

* make a sorted table of variable labels;
%macro LabelTable(_data=,_drop=,_out=,_label=);
   data tmp1;
      set &_data(obs=1);
      %if %length(&_drop) %then %do;
         drop &_drop;
      %end;
   run;

   proc transpose data=tmp1 out=&_out;
   run;

   data &_out;
      set &_out;
      &_label = _label_;
      keep &_label;
   run;

  /* proc sort data=&_out;
      by &_label;
   run;
   */
%mend LabelTable;

* make a variable list using proc contents;
%macro MakeVarList(_data=,_nvar=,_prefix=,_typeflag=1,_labelflag=0,_MaxlengthOfVar=,_MaxlengthOfLab=);
   proc contents data=&_data out=tmp(keep=name type label varnum) noprint; run;
   %if ^%length(&_prefix) %then %let _prefix=var;
   %else %do;
   data tmp;
      set tmp;
      if substr(NAME,1,%length(&_prefix)) = "&_prefix";
   run;
   %end;
   %global &_nvar;
   %let &_nvar = 0;
   data _null_;
      set tmp nobs=eof;
      call symput("&_nvar",trim(left(eof)));
   run;
   %if (&&&_nvar = ) %then %let &_nvar = 0;
   %let _ni_ = &&&_nvar;
	%if ^%length(&_MaxlengthOfVar) %then %let _MaxlengthOfVar = _MaxVlength;
	%global &_MaxlengthOfVar;
	%if &_labelflag %then %do;
		%if ^%length(&_MaxlengthOfLab) %then %let _MaxlengthOfLab = _MaxLlength;
		%global &_MaxlengthOfLab;
	%end;
   %if (&_ni_ = 0) %then %do;
      %put NOTE: MakeVarList(_data=&_data,_prefix=&_prefix) has zero elements.;
		%let &_MaxlengthOfVar = 0;
   %end;
   %else %do;
      proc sort data=tmp;
        by varnum;
      run;
      %do _i_ = 1 %to &_ni_;
         data _null_;
            set tmp;
            if _n_ = &_i_;
            call symput('_tmp_',name);
				call symput('_type_',type);
				call symput('_label_',label);
         run;
         %global &_prefix.&_i_;
         %let &_prefix.&_i_ = &_tmp_;
			%if &_i_ = 1 %then %let &_MaxlengthOfVar = %length(&_tmp_);
			%else %do;
				%if %length(&_prefix.&_i_) > &&&_MaxlengthOfVar %then 
					%let &_MaxlengthOfVar = %length(&_prefix.&_i_);
			%end;
			%if &_typeflag %then %do;
				%global &_prefix._type_&_i_;
				%let &_prefix._type_&_i_ = &_type_;
			%end;
			%if &_labelflag %then %do;
				%global &_prefix._label_&_i_;
				%let &_prefix._label_&_i_ = &_label_;
				%if &_i_ = 1 %then %let &_MaxlengthOfLab = %length(&_label_);
				%else %do;
					%if %length(&_label_) > &&&_MaxlengthOfLab %then 
						%let &_MaxlengthOfLab = %length(&_label_);
				%end;
			%end;
     %end;
   %end;
%mend MakeVarList;

* determine number of observations and variables;
%macro NObsVars(_ds); 
   %*global nvars nobs nlobs;
	%if ^%symexist(nvars) %then %global nvars; 
	%if ^%symexist(nobs) %then %global nobs;
	%if ^%symexist(nlobs) %then %global nlobs;
   %let _dset_ = &_ds;
   %let _dsid_ = %sysfunc(open(&_dset_));
   %if &_dsid_ %then %do;
      %let nobs = %sysfunc(attrn(&_dsid_,NOBS));
      %let nlobs = %sysfunc(attrn(&_dsid_,NLOBS));
      %let nvars = %sysfunc(attrn(&_dsid_,NVARS));
      %let _rc_ = %sysfunc(close(&_dsid_));
   %end;
   %else %do;
      %put ERROR: Open for data set &_dset_ failed - %sysfunc(sysmsg());
      %let nobs = 0;
      %let nlobs = 0;
      %let nvars = 0;
      %let exiterror = 1;
   %end;
%mend NObsVars;


* Filter files under path, _Path, by extension name, _Extension, or prefix, _Prefix (case sensitive)
* and keep the filtered files in a data set, _OutData;
%macro FileFilter(_Path,_OutData,_Prefix="",_Ext="",_CaseFlag=1);
%CheckEndSlash(_Path);

data &_OutData;
	rc = filename("mydir","&_Path");
  	did = dopen("mydir");
	if did = 0 then do;
		put "ERROR: The path &_Path does not exist.";
		call symput("exiterror",1);
		rc = dclose(did);
		stop;
	end;
	do i = 1 to dnum(did);
		filename = dread(did,i);
		* screen out subfolder;
		mem = "&_Path"||trim(left(filename));
		rc = filename("mydir2",mem);
		did2 = dopen("mydir2");
		rc = dclose(did2);
		if did2 = 0 then do;
			out = 1;
			if &_Prefix ^= "" then do;
				%if &_CaseFlag %then %do;
					if filename ^=: &_Prefix then out = 0;
				%end;
				%else %do;
					if upcase(filename) ^=: upcase(&_Prefix) then out = 0;
				%end;
			end;
			if out & &_Ext ^= "" then do; 
				if upcase(trim(scan(filename,-1,'.'))) ^= upcase(&_Ext) then out = 0;
			end;
		end;
		else do;
			out = 0;
		end;
		if out then output;
	end;
	rc = dclose(did);
	keep filename;
run;
%mend FileFilter;


/** Use the javaobj to launch each file **/
%macro LaunchFile(_absoluteFilePath0);
	%let _absoluteFilePath = %nrbquote(&_absoluteFilePath0);
	%if &SASMode ne SDD %then %do;
		%if &sysscp = WIN %then %do;
			data _null_/picklist='jmpsasgenom/genomics.txt';
				length e 8;
				put "file is &_absoluteFilePath";
				declare javaobj test ("com/sas/autolaunch/LaunchFile","&_absoluteFilePath"); 
				rc = test.ExceptionCheck(e);
				if (e) then
					put 'Problems Launching file';
			run;
		%end;
		%else %do;
			/** if extension = .jsl, then Include the jsl file in sasclean.jsl **/
			%let lenf = %length(&_absoluteFilePath);
			%if (&lenf > 4) %then %let last3 = %substr(&_absoluteFilePath,%eval(&lenf-2));
			%else %let last3 = ;
			%if (%upcase(&last3) = JSL) %then %do;
				%let AllJSLIncludedFiles=&AllJSLIncludedFiles.%str(Include(%"&_absoluteFilePath%", <<new context, << %"Names Default To Here%"););
			%end;
			/** if extension != .jsl, then add as web call to sasclean.jsl to let the MIME type in the browser handle the file. **/
			%else %do;
				%let AllJSLIncludedFiles=&AllJSLIncludedFiles.%str(Web(%"file://&_absoluteFilePath%"););
				/** There may be some circumstances where launching and external program is desirable.
					LaunchExternalProgram(args,&_absolutefilepath);
				**/
			%end;
		%end;
	%end;
%mend LaunchFile;

/** Use the javaobj to launch batch file **/
%macro LaunchBatFile(_absoluteFilePath0);
	%let _absoluteFilePath = %nrbquote(&_absoluteFilePath0);
	%if &SASMode ne SDD %then %do;
		%if &sysscp = WIN %then %do;
			data _null_/picklist='jmpsasgenom/genomics.txt';
				length e 8;
				put "file is &_absoluteFilePath";
				declare javaobj test ("LaunchBatFile","&_absoluteFilePath"); 
				rc = test.ExceptionCheck(e);
				if (e) then
					put 'Problems Launching file';
			run;
		%end;
		%else %do;
			/** if extension = .jsl, then Include the jsl file in sasclean.jsl **/
			%let lenf = %length(&_absoluteFilePath);
			%if (&lenf > 4) %then %let last3 = %substr(&_absoluteFilePath,%eval(&lenf-2));
			%else %let last3 = ;
			%if (%upcase(&last3) = JSL) %then %do;
				%let AllJSLIncludedFiles=&AllJSLIncludedFiles.%str(Include(%"&_absoluteFilePath%", <<new context, << %"Names Default To Here%"););
			%end;
			/** if extension != .jsl, then add as web call to sasclean.jsl to let the MIME type in the browser handle the file. **/
			%else %do;
				%let AllJSLIncludedFiles=&AllJSLIncludedFiles.%str(Web(%"file://&_absoluteFilePath%"););
				/** There may be some circumstances where launching and external program is desirable.
					LaunchExternalProgram(args,&_absolutefilepath);
				**/
			%end;
		%end;
	%end;
%mend LaunchBatFile;

%macro PrintTimeFor(_macroname);
	%put &_macroname is curtime=%sysfunc(time(),time.) ;
%mend PrintTimeFor;


/** Use the javaobj to launch each file **/
%macro LaunchExternalProgram(_executablepath, _commandargs, _absoluteFilePath, _workingDirectory);
	%if (&SASMode ne SDD) %then %do;
		data _null_/picklist='jmpsasgenom/genomics.txt';			
			length e 8;
			%put;
			%put LaunchExternalProgram: executable path = &_executablepath;
			%put;
			%put LaunchExternalProgram: commandargs = &_commandargs;
			%put;
			%put LaunchExternalProgram: path = &_absoluteFilePath;
			%put;
			%put LaunchExternalProgram: path = &_workingDirectory;
			%put;
			declare javaobj test ("com/sas/autolaunch/LaunchExternalProgram","&_executablepath", "&_commandargs","&_absoluteFilePath", "&_workingDirectory"); 
			rc = test.ExceptionCheck(e);
			if (rc) then put "ERROR: LaunchExternalProgram returned exception code.";
		run;
	%end;
%mend LaunchExternalProgram;

%macro RenameFile(_absoluteFilePath, _newName);
	%if (&SASMode ne SDD) %then %do;
		data _null_;
			put "file is &_absoluteFilePath";
			put "new file name is &_newName";
			rc=rename("&_absoluteFilePath", "&_newName", 'file');
			if (rc) then
				put 'Problems renaming file';
		run;
	%end;
%mend RenameFile;

* Remove / delete file; 
%macro RemoveFile(_absoluteFilePath);
	%if (&SASMode ne SDD) %then %do;
		%if %sysfunc(fileexist(&_absoluteFilePath)) %then %do;
			filename FCHK "&_absoluteFilePath";
			%let _frc_=%sysfunc(fdelete(FCHK));
		%end;
	%end;
%mend RemoveFile;

* remove a directory;
%macro RemoveDir(dir);
   %if &Server = Local %then %do;
		options noxwait;
	%end;
	%local fileref ;
	%let rc = %sysfunc(filename(fileref,&dir)) ;
	%if %sysfunc(fexist(&fileref)) %then %do;
		%put Removing directory &dir....;  
		/*%if &Server = Local %then %do;
			%sysexec rmdir /Q /S "&dir" ;
			%let rc = &sysrc;
		%end;
		%else %do;*/
			*Unix: need to delete files in the directory before deleting the dicetory;
			%FileFilter(&dir,filelist);
			data OutLib.filelist;
				set filelist;
			run;
			data filelist;
				set filelist;
				_F_ = 'sasInFile'||trim(left(_n_));
				call symput(_F_,trim(filename));;
			run;
			%NObsVars(filelist);
			%let nFile = &nObs;
			%put nFile=&nFile;
			%do j=1 %to &nFile;
				%let oldfileName =&dir.&&sasInFile&j;
				%put oldfileName=&oldfileName ;
				filename testdir "&oldfileName";
				%let rc = %sysfunc(fdelete(testdir));
				%put %sysfunc(sysmsg());
				%*let rc = %sysfunc(fdelete(d));
			%end;			
			filename testdir "&dir";
			%let rc = %sysfunc(fdelete(testdir));
			%put %sysfunc(sysmsg());
			
		/*	libname di "&dir";
			proc datasets library=di kill;
			run;
			filename d "&dir";
			%let rc = %sysfunc(fdelete(d));
			%put %sysfunc(sysmsg());*/
		%*end;
		%put rc = &rc;
		%if &rc eq 0 %then %do;
			%put The directory &dir and its sub-directories have been deleted.;
		%end;
		%else %do;
			%put Unable to delete folder &dir..  Another application may have it locked or you may not have appropriate authorization.;
		%end;
	%end;
	%else %put The directory &dir does not exist;	
%mend RemoveDir ; 

*remove a pc directory using a batch file via Java;
/*
%macro RemoveDir(dir);
	%local fileref ;
	%let rc = %sysfunc(filename(fileref,&dir));
	%if %sysfunc(fexist(&fileref)) %then %do;
		%put ;
		%put Trying to remove directory &dir;
		filename BatFile "&USER_HOME.ProcessResults&_Slash_.tmp101107.bat";
		%let rc = %sysfunc(fDelete(BatFile));
		* preparing .bat file;
		data _null_;
			file BatFile;
			put 'rmdir /Q/S ' @;
			put "&dir" @;
		run; 
		* launch .bat to save file size info to a .txt file;
		%let lf = %LaunchFile(&USER_HOME.ProcessResults&_Slash_.tmp101107.bat);
		* determine if it really got deleted;
		%let rc = %sysfunc(filename(fileref,&dir));
		%if %sysfunc(fexist(&fileref)) %then %do;
			%put ERROR: Unable to delete folder &dir..  Another application may have it locked or you may not have appropriate authorization.;
			%let rc = 2;
		%end;
		%else %do;
			%put Remove successful.;
			%let rc = 0;
		%end;
	%end;
	%else %do;
		%put The directory &dir does not exist; 
		%let rc = 1;
	%end;
%mend RemoveDir;
*/

*remove a directory using SAS code;
%macro RemoveDir2(dir);
	%put RemoveDir2: &dir;
	/*
	%let rc = 0;
	libname rd "&dir";
	proc datasets lib=rd kill;
	run;
	*/
	filename d "&dir";
	%let rc = %sysfunc(fdelete(d));
	%put rc=&rc;
	%put %sysfunc(sysmsg());
	&rc
%mend;

*create a directory;
%macro CheckandCreateDir(dir);
	options noxwait;
	%local rc fileref;
	%let rc = %sysfunc(filename(fileref,&dir)) ;
	%if %sysfunc(fexist(&fileref)) %then
		%put The directory "&dir" already exists ;
	%else %do;
		%put Creating directory &dir...;
		filename BatFile "&USER_HOME.ProcessResults&_Slash_.tmp101108.bat";
		%let rc = %sysfunc(fDelete(BatFile));
		* preparing .bat file;
		data _null_;
			file BatFile;
			put 'mkdir "' @;
			put "&dir" @;
			put '" ' @;
		run; 
		* launch .bat to save file size info to a .txt file;
		%LaunchFile(&USER_HOME.ProcessResults&_Slash_.tmp101108.bat);		
		%*sysexec mkdir"&dir";
		%*if &sysrc eq 0 %then %put The directory &dir has been created. ;
		%*else %put There was a problem while creating the directory &dir;
	%end;
%mend;

* convert sas transport files;
%macro ConvertTransportFile(_Path_,_OutPath_);
	* define libnames;
	
	libname OutPLib "&_OutPath_";
	%let _workpath_=%sysfunc(pathname(work));
	
	* Transport .cpt files from the input folder;
	%FileFilter(&_Path_,CPT,_ext="cpt");
	%NObsVars(CPT);
	%let nCPT = &nObs;
	
	* Transport .xpt files from the input folder;
	%FileFilter(&_Path_,XPT,_ext="xpt");
	%NObsVars(XPT);
	%let nXPT = &nObs;
	
	* Transport .cpt files from the input folder;
	%if &nCPT > 0 %then %do;
		data _null_;
			set CPT;
			_F_ = 'CPTInFile'||trim(left(_n_));
			call symput(_F_,"&_Path_"||trim(filename));
		run;	
		%do i = 1 %to &nCPT;
			%put CPT file =&&CPTInFile&i;
			proc cimport infile="&&CPTInFile&i" library=OutPLib;
			run;
			%if &syserr %then %do;
				%put [MS_S_0251];
				%put ERROR: Error(s) occur during transporting cpt files.;
				%goto exit;
			%end;
		%end;
	%end;
	
	* Transport .xpt files from the input folder;
	%if &nXPT > 0 %then %do;
		data _null_;
			set XPT;
			_F_ = 'XPTInFile'||trim(left(_n_));
			call symput(_F_,"&_Path_"||trim(filename));
		run;
		%do i = 1 %to &nXPT;
			options compress=Yes ;
			libname xportin xport "&&XPTInFile&i";
			proc copy in=xportin out=OutPLib memtype = data;
			run;
			%if &syserr %then %do;
				%put [MS_S_0252];
				%put ERROR: Error(s) occur during transporting xpt files.;
				%goto exit;
			%end;
		%end;
	%end;
	
%exit:;
%mend;

* Packaging macros;
%macro PackageBegin(_ProcessName);
   %global ProcessID ProcessNameValue rcp OriginalProcessName _pkn;
   %let ProcessID = 0;
   %let _description_ = &_ProcessName Results Package;
   %let ProcessNameValue = Project=&_ProcessName;
   %let OriginalProcessName = &_ProcessName;
   %let rcp = 0;
   %let _pkn = 0;
   %if &SASMode=SPS %then %do;
      %syscall package_begin(ProcessID,_description_,ProcessNameValue,rcp);
      %Checkrc(Package_Begin,_rc=&rcp);
   %end;
%mend PackageBegin;

%macro PackageInsertDataset(_description,_libname,_memname,_clientdownload,_showmessage,_launchsasds,_type=wide);
	%put Calling PackageInsertDataset on &_libname..&_memname; 
	
	/* _type = (blank) - don't add to study data set, tall, wide, EDDS, other */
	
	/* force clientdownload for all data sets in package */
	%let _clientdownload=yes;
	
	/* For now, force clientdownload when running a workflow, in case client path of output data set used */
	/* as input data set for subsequent AP                                                                */
	/* %if (&WorkFlowrp ne -1) %then %let _clientdownload=yes; */
	
	%if (&SASMode ne SDD) %then %do;
		%if ^%length(&_clientdownload) %then %let _clientdownload=yes;
		%if ^%length(&_showmessage) %then %let _showmessage=yes;
		%if ^%length(&_launchsasds) %then %let _launchsasds=no;
		%if %upcase(&_clientdownload)=YES %then %do;
			%let nOut = %eval(&nOut+1);
			%global OutFile&nOut;
			%let OutFile&nOut = %lowcase(&_memname).sas7bdat;
			* update _ProcessInfo2 (result metadta);
			%ResultMeta(&&OutFile&nOut);
		%end;
		
		%if %symexist(StudyName) %then %do;
			%if %length(&_type) and %length(&StudyName) %then %do;
				%let nStudyOut = %eval(&nStudyOut+1);
				%global StudyFile&nStudyOut StudyType&nStudyOut;
				%let StudyFile&nStudyOut = %lowcase(&_memname);
				%let StudyType&nStudyOut = %upcase(&_type);
			%end;
		%end;
		
		* compute quantities for sas message window;
		%if %upcase(&_showmessage)=YES %then %do;
			%let _pkn = %eval(&_pkn+1);
			%global pkds&_pkn pkds_client&_pkn pkds_nvars&_pkn pkds_nobs&_pkn pkds_fs&_pkn pkds_desc&_pkn pkds_memname&_pkn;

			%let pkds_memname&_pkn = &_memname;
			ods listing close;
			proc datasets lib=&_libname;
				contents data=&_libname..&_memname;
				ods output enginehost=eh;
			run;
			quit;
			ods listing;

			data _null_;
				set eh;
				if label1="Filename" then
				call symput("_pds",trim(left(cvalue1)));
			run;
			%let pkds&_pkn = %nrbquote(&_pds);
			%let pkds_desc&_pkn = %quote(&_description);
			%let pkds_client&_pkn = &ClientOutPath.&_memname..sas7bdat;

			%GetDataSetSize(&_memname,pkds_fs&_pkn,lib=&_libname);

			%Nobsvars(&_libname..&_memname);
			%let pkds_nvars&_pkn = &nvars;
			%let pkds_nobs&_pkn = &nobs;

		%end;
		%if &_launchsasds=yes %then %do;
			%if &sysscp = WIN %then %do;
				/* %LaunchFile(&OutPath.&_memname..sas7bdat); */
			%end;
		%end;
		%if &SASMode=SPS %then %do;
			%let _nameValue_ = "dirname=%sysfunc(pathname(&_libname)) -- clientdownload=&_clientdownload";
			%syscall insert_dataset(ProcessID,_libname,_memname,_description,_nameValue_,rcp);
			%Checkrc(Insert_Dataset,_rc=&rcp);
		%end;
	%end;

%mend PackageInsertDataset;

%macro PackageInsertHtml(_description,_outpath,_bodyFile,_frameFile,_contentsFile,_clientdownload,_jmpwindow);
	%put Calling PackageInsertHtml; 

	%if %length(&_bodyFile) > 0 %then %do;
		filename b "&OutPath.&_bodyFile";
		%let nOut = %eval(&nOut+1);
		%global OutFile&nOut;
		%let OutFile&nOut = &_bodyFile;
		* update _ProcessInfo2 (result metadta);
		%ResultMeta(&&OutFile&nOut);
	%end;
	%if %length(&_contentsFile) > 0 %then %do;
		filename c "&OutPath.&_contentsFile";
		%let nOut = %eval(&nOut+1);
		%global OutFile&nOut;
		%let OutFile&nOut = &_contentsFile;
		* update _ProcessInfo2 (result metadta);
		%ResultMeta(&&OutFile&nOut);
	%end;
	%if %length(&_frameFile) > 0 %then %do;
		filename f "&OutPath.&_frameFile";
		%let nOut = %eval(&nOut+1);
		%global OutFile&nOut;
		%let OutFile&nOut = &_frameFile;
		* update _ProcessInfo2 (result metadta);
		%ResultMeta(&&OutFile&nOut);
	%end;

	%let _body_ = ;
	%let _bodyUrl_ = ;
	%if %length(&_bodyFile) > 0 %then %do;
		%let _body_ = fileref:b;
		%let _bodyUrl_ = &_bodyFile;
	%end;

	%let _contents_ = ;
	%let _contentsUrl_ = ;
	%if %length(&_contentsFile) > 0 %then %do;
		%let _contents_ = fileref:c;
		%let _contentsUrl_ = &_contentsFile;
	%end;

	%let _frame_ = ;
	%let _frameUrl_ = ;
	%if %length(&_frameFile) > 0 %then %do;
		%let _frame_ = fileref:f;
		%let _frameUrl_ = &_frameFile;
	%end;
	%else %do;
		%let _frame_ = fileref:b;
		%let _frameUrl_ = &_bodyFile;
		%let _body_ = ;
		%let _bodyUrl_ = ;
	%end;

	%let _page_ = ;
	%let _pageUrl_ = ;
	%if ^%length(&_clientdownload) %then %let _clientdownload = yes;
	%let _nameValue_ = "dirname=&outpath -- clientdownload=&_clientdownload";

	%if %length(&_jmpwindow)=0 %then %let _jmpwindow = yes;

	%if (&SASMode ne SDD) & (&ProcessName ne DataContents) %then %do;
		%if (&WorkFlowrp ne -1) %then
			%let AllJSLIncludedFiles=&AllJSLIncludedFiles.%str(open(\!%"&ClientOutPath.&_frameFile\!%", JMP window););
		%else
			%if &_jmpwindow ne yes %then %do;
				%let AllJSLIncludedFiles=&AllJSLIncludedFiles.%str(open(%"&ClientOutPath.&_frameFile%"););
			%end;
		%else %do;	
			%let AllJSLIncludedFiles=&AllJSLIncludedFiles.%str(Web(%"file://&ClientOutPath.&_frameFile%", JMP window););
		%end;
	%end;
	%if &SASMode=SPS %then %do;
		%syscall insert_html(ProcessID,
			_body_,_bodyUrl_,
			_frame_,_frameUrl_,
			_contents_,_contentsUrl_,
			_page_,_pageUrl_,
			_description,_nameValue_, rcp);
		%Checkrc(Insert_Html,_rc=&rcp);
	%end;

%mend PackageInsertHtml;

/* in the macro below:

dependson is a list of files that the inserted file depends on.  this list must be
delimited by commas and contain no spaces

mimetype indicates the type of file, possible values are as follows, default is
text/plain:

application/msword
application/xls
application/octet-stream
application/binary
application/pdf
application/postscript
application/zip
audio/basic
image/jpeg
image/gif
image/tiff
model/vrml
text/html
text/plain
text/richtext
text/xcmdlog
video/mpeg
video/quicktime
*/
%macro PackageInsertFile(_description,_filepath,_filename,_dependson,_mimetype,_clientdownload,_launch);
	%put Calling PackageInsertFile on &_filepath.&_filename; 
	filename _pif "&_filepath.&_filename" lrecl=32767;
	/*
   %let _filenm_ = filename: &_filepath.&_filename;
   */
   %let _filenm_ = fileref: _pif;
   %let _filetype_ = binary;
   %if ^%length(&_mimetype) %then %let _mimetype = text/plain;
   %if ^%length(&_clientdownload) %then %let _clientdownload = yes;
   %if ^%length(&_launch) %then %let _launch = yes;
   
   /* convert JSL output file format */
   %let _lenf = %length(&_filename);
   %if (&_lenf > 4) %then %let _last3 = %substr(&_filename,%eval(&_lenf-2));
   %else %let _last3 = ;
   %if ((%upcase(&_last3) = JSL) or (%upcase(&_last3) = JRN)) and (&OutDataFormat ^= sas7bdat) %then %do;
   
   	* check format;
   	%if (&OutDataFormat = xls) %then %let _db = EXCEL;
   	%else %if (&OutDataFormat = txt) or (&OutDataFormat = dat) %then %let _db = TAB;
   	%else %do;	
   		%let OutDataFormat = jmp;
   		%let _db = JMP;
   	%end;
   	
    	* make copy of original jsl file that works with sas7bdat files;
   	%let prefix = %substr(&_filename,1,%eval(&_lenf-4));
		
		filename orig "&_filepath.&_filename";
		filename orig_sas "&_filepath.&prefix._sas7bdat.jsl";
		
   	data a1;
   		infile orig missover sharebuffers length=len;
   		file orig_sas;
   		length inline outline $32767;
   		input @;
   		input @1 inline $varying32767. len;
   		outline = inline;
   		lenx = length(outline);
   		put @1 outline $varying32767. lenx;
   	run;
   	
   	/*
   	proc print;
   	run;
   	*/
   	
		* loop to find all dependency data sets and create macro variables;
		%let _ndep = 0;
		%put dependson = &_dependson;
		%if %length(&_dependson) %then %do;
			%let _i=1;
			%do %while(%length(%scan(&_dependson,&_i,",")));
				%let _fn = %trim(%scan(&_dependson,&_i,","));
				%let _lfn = %length(&_fn);
				%if (&_lfn > 8) %then %let _last8 = %substr(&_fn,%eval(&_lfn-7));
				%else %let _last8 = &_fn;
				/* %put _i=&_i _fn=&_fn _lfn=&_lfn _last8=&_last8; */
				%if (&_lfn > 1) and (%upcase(&_last8) = SAS7BDAT) %then %do;
					%let _ndep = %eval(&_ndep+1);
					%let _ds&_ndep = %substr(&_fn,1,%eval(&_lfn-9));
				%end;
				%let _i=%eval(&_i+1);
			%end;
		%end;
	
   	* convert the data sets;
   	libname out1 "&_filepath";
   	
  		%do _d=1 %to &_ndep;
  		
   		%let _ds = &&_ds&_d;
   		%put;
   		%put Converting dependency data set &_d of &_ndep, name=&_ds;
   		%if (&OutDataFormat = txt) or (&OutDataFormat = dat) %then %do;
	   		filename outf "&_filepath.temp.&OutDataFormat" lrecl=32767;
	   		filename outf2 "&_filepath.&_ds..&OutDataFormat" lrecl=32767;
	   	%end;
	   	%else %do;
	   		filename outf "&_filepath.&_ds..&OutDataFormat" lrecl=32767;
			%end;	   		
	   	
	   	options label missing='.';
	   	
	   	/*
	   	proc contents data=out1.&ds;
	   	run;
   		*/
 
 			/*
 			the proc export label does not work all the way
 			in SAS 9.2, e.g. with labels longer than 32 characters.
 			*/

   		proc export data=out1.&_ds outfile=outf dbms=&_db replace;
   		run;
   		  	
			* if using text tables, change names to labels in first row;
			%if (&OutDataFormat = txt) or (&OutDataFormat = dat) %then %do;
				* create name-label pairs of macro variables;
				%let _tid_ = %sysfunc(open(out1.&_ds));
				%let _nVars_&_d = %sysfunc(attrn(&_tid_,NVARS)); 
				%let _nLabels_&_d = 0;
				%do _i_ = 1 %to &&_nVars_&_d;
					%let vname = %sysfunc(varname(&_tid_,&_i_));
					%let vlabel = %sysfunc(varlabel(&_tid_,&_i_));
					%if %length(&vlabel) %then %do;
						%let _nLabels_&_d = %eval(&&_nLabels_&_d + 1);
						%let _vname_&_d._&&_nLabels_&_d = &vname;
						%let _vlabel_&_d._&&_nLabels_&_d = &vlabel;						
					%end;
				%end;
				%let rc = %sysfunc(close(&_tid_));
				
				/*
				%put _nLabels_&_d = &&_nLabels_&_d;
				%do _i_=1 %to &&_nLabels_&_d;
					%put _vname_&_d._&_i_ = &&_vname_&_d._&_i_;
					%put _vlabel_&_d._&_i_ = &&_vlabel_&_d._&_i_;
				%end;
				*/

				* make substitutions in the first row of the text file, which contains the headers;
				data _null_;
					infile outf missover sharebuffers length=len;
					file outf2;
					length inline outline $32767;
					input @;
					input @1 inline $varying32767. len;

					outline = inline;
					if _n_ = 1 then do;
						* loop backwards so more complex substitutions do not get counterfeited by simpler ones;
						%do _i_=&&_nLabels_&_d %to 1 %by -1;
							outline = tranwrd(outline,"&&_vname_&_d._&_i_","&&_vlabel_&_d._&_i_");
						%end;
					end;

					leno = length(outline);
					put @1 outline $varying32767. leno;
				run;
			%end;
		%end;
			
		* if using text or jmp tables, determine name-label pairs and save them in macro variables;
		/*
		%if (&OutDataFormat = txt) or (&OutDataFormat = dat) or (&OutDataFormat = jmp) %then %do;
			%do _d_=1 %to &_ndep;
				%let _ds_ = &&_ds&_d_;
				%let _tid_ = %sysfunc(open(out1.&_ds_));
				%let _nVars_&_d_ = %sysfunc(attrn(&_tid_,NVARS)); 
				%let _nLabels_&_d_ = 0;
				%do _i_ = 1 %to &&_nVars_&_d_;
					%let vname = %sysfunc(varname(&_tid_,&_i_));
					%let vlabel = %sysfunc(varlabel(&_tid_,&_i_));
					%if %length(&vlabel) %then %do;
						%let _nLabels_&_d_ = %eval(&&_nLabels_&_d_ + 1);
						%let _vname_&_d_._&&_nLabels_&_d_ = &vname;
						%let _vlabel_&_d_._&&_nLabels_&_d_ = &vlabel;
					%end;
				%end;
				%let rc = %sysfunc(close(&_tid_));
			%end;
		%end;
		*/

   	* update _dependson spec;
   	%if %length(&_dependson) %then %do;
   		%put _dependson old = &_dependson;
   		%let _dependson = %Search_and_Replace(string=&_dependson,search=sas7bdat,replace=&OutDataFormat);
   		%put _dependson new = &_dependson;
   	%end;
   	
		* make substitutions in the JSL file with the new data format when found,;
		data _null_;
			infile orig_sas missover sharebuffers length=len;
			file orig;
			length inline outline $32767;
			input @;
			input @1 inline $varying32767. len;

			idx = index(inline,"sas7bdat");
			if (idx > 0) then do;
				outline = substr(inline,1,(idx-1)) || "&OutDataFormat" || substr(inline,(idx+8)); 
			end;
			else outline = inline;

			lenx = length(outline);
			put @1 outline $varying32767. lenx;

			* if using text or jmp tables, add JSL code to change names to labels;
			/*
			%if (&OutDataFormat = txt) or (&OutDataFormat = dat) or (&OutDataFormat = jmp) %then %do;
				* make sure this is an open statement and not just an assignment;
				if (idx > 0) and index(inline,"open") then do;
					* determine the data set name;
					firstpart = substr(inline,1,(idx-2));
					lenfp = length(firstpart);
					do j=lenfp to 1 by -1;
						charj = lowcase(substr(firstpart,j,1));
						if charj ^in ('_','0','1','2','3','4','5','6','7','8','9',
							'a','b','c','d','e','f','g','h','i','j',
							'k','l','m','n','o','p','q','r','s','t',
							'u','v','w','x','y','z') then do;
							start = j+1;
							j = 0;
						end;
					end;
					
					dsname = substr(firstpart,start);
					dsidx = 0;
					%do _d_=1 %to &_ndep;
						%let _ds_ = &&_ds&_d_;
						if upcase(dsname) = upcase("&_ds_") then do;
							%do _i_=1 %to &&_nLabels_&_d_;
								put "Column(""&&_vname_&_d_._&_i_"") << SetName(""&&_vlabel_&_d_._&_i_"");";
							%end;
						end; 
					%end;
					
				end;
			%end;
			*/
		run;
   %end;
   
   
   %if %length(&_dependson) %then %do;
		%let _i=1;
		%do %while(%length(%scan(&_dependson,&_i,",")));
			%let _fn&_i = %trim(%scan(&_dependson,&_i,","));
			* update _ProcessInfo2 (result metadta);
			%ResultMeta(&&_fn&_i);
			%let _i=%eval(&_i+1);
		%end;
   %end;

	%if &SASMode ne SDD %then %do;
		%if %upcase(&_launch)=YES %then %do;
			%put Server=&Server _filepath=&_filepath OutPath=&OutPath ClientOutPath=&ClientOutPath;
			%if (&Server ^= Local) and %quote(&_filepath) = %quote(&OutPath) %then %do;
				%let _filepath = &ClientOutPath;
			%end;
			%if (%upcase(&_last3) = JSL) %then %do;
				%if (&WorkFlowrp ne -1) %then
					%let AllJSLIncludedFiles=&AllJSLIncludedFiles.%str( Include(\!%"&_filepath.&_filename\!%", <<new context, << \!%"Names Default To Here\!%"););
				%else
					%let AllJSLIncludedFiles=&AllJSLIncludedFiles.%str( Include(%"&_filepath.&_filename%", <<new context, << %"Names Default To Here%"););
			%end;
			%else %do;
				%if &sysscp = WIN %then %do;
					%if (&WorkFlowrp = -1) %then %let AllJSLIncludedFiles=&AllJSLIncludedFiles.%str( Open("&_filepath.&_filename", force refresh););
					%else %let AllJSLIncludedFiles=&AllJSLIncludedFiles.%str( Web(\!%"file://&_filepath.&_filename\!%"););;
				%end;
				%else %do;
					%if (%upcase(&_last3) = JMP) %then %do;
						%if (&WorkFlowrp ne -1) %then
							%let AllJSLIncludedFiles=&AllJSLIncludedFiles.%str( Open(\!%"&_filepath.&_filename\!%"););
						%else
							%let AllJSLIncludedFiles=&AllJSLIncludedFiles.%str( Open(%"&_filepath.&_filename%"););
					%end;
					%else %do;
						%if (&WorkFlowrp ne -1) %then
							%let AllJSLIncludedFiles=&AllJSLIncludedFiles.%str( Web(\!%"file://&_filepath.&_filename\!%"););
						%else 
							%let AllJSLIncludedFiles=&AllJSLIncludedFiles.%str( Web(%"file://&_filepath.&_filename%"););
					%end;
				%end;
			%end;
			%if (&Server ^= Local) and %quote(&_filepath) = %quote(&ClientOutPath) %then %do;
				%let _filepath = &OutPath;
			%end;
		%end;
		%let nOut = %eval(&nOut+1);
		%global OutFile&nOut;
		%let OutFile&nOut = &_filename;
		* update _ProcessInfo2 (result metadta);
		%ResultMeta(&&OutFile&nOut);
	%end;
	%if &SASMode=SPS or &Server^=Local %then %do;
	   * loop through all dependency files and insert them into the package;
	   libname _outd1 "&_filepath";
   	%if %length(&_dependson) %then %do;
   		%let _i=1;
   		%do %while(%length(%scan(&_dependson,&_i,",")));
   			%let _fn = %trim(%left(%scan(&_dependson,&_i,",")));
				%let _lfn = %length(&_fn);
				%if (&_lfn > 8) %then %let _last8 = %substr(&_fn,%eval(&_lfn-7));
				%else %let _last8 = &_fn;
				%if (&_lfn > 1) %then %do;
					%if (%upcase(&_last8) = SAS7BDAT) %then %do;
						%if &_lfn > 9 %then %do;
							%let _memname = %trim(%substr(&_fn,1,%eval(&_lfn-9)));
							%let _desci_ = &_memname;
							%if &SASMode=SPS %then %do;
								%put Inserting SAS data set &_memname into package.;
								%PackageInsertDataset(&_desci_, _outd1, &_memname, yes);
							%end;
							%else %do;
								%let nOut = %eval(&nOut+1);
								%global OutFile&nOut;
								%let OutFile&nOut = &_memname..sas7bdat;
							%end;
						%end;
					%end;
					%else %do;
						%let _desci_ = &_fn;
						filename _outf&_i "&_filepath.&_fn" lrecl=32767;
						%let _filenmi_ = fileref: _outf&_i;
						%let _NameValue_ = ;
						%if &SASMode=SPS %then %do;
							%put Inserting file &_fn into package.;
							%syscall insert_file(ProcessID,_filenmi_,_filetype_,_mimetype,_desci_,_NameValue_,rcp);
							%Checkrc(Insert_File &_filenmi_,_rc=&rcp);
						%end;
						%else %do;
							%let nOut = %eval(&nOut+1);
							%global OutFile&nOut;
							%let OutFile&nOut = &_fn;				
						%end;
					%end;
				%end;
				%let _i=%eval(&_i+1);
   		%end;
	   %end;

		%if &SASMode=SPS %then %do;
			%let _NameValue_ = dirname=&_filepath -- TextFileName=&_filename -- clientdownload=&_clientdownload -- DependsOn=&_dependson;
			%syscall insert_file(ProcessID,_filenm_,_filetype_,_mimetype,_description,_NameValue_,rcp);
			%Checkrc(Insert_File &_filenm_,_rc=&rcp);
		%end;
	%end;

%InFileExit:;
%mend PackageInsertFile;

* search and replace in a macro string;
%macro Search_and_Replace(string=,search=,replace=);
	%let newstr = %nrbquote(&string);
	%let lens = %length(&search);
	/* %put input string = &newstr  search = &search   replace = &replace; */
	
   %do %while(%index(&newstr,&search));
   	%let len = %length(&newstr);
   	%let idx = %index(&newstr,&search);
   	%let len2 = %eval(&idx+&lens);
   	/* %put len=&len len2=&len2  idx=&idx; */
   	%if &len2 <= &len %then %do;
   		%let secondpart = %nrbquote(%substr(&newstr,&len2));
   	%end;
   	%else %do;
   		%let secondpart = ;
   	%end;
   	%if (&idx = 1) %then %do;
   		%let newstr = &replace.&secondpart;
   	%end;
   	%else %do;
   		%let firstpart = %nrbquote(%substr(&newstr,1,%eval(&idx-1)));
   		/* %put firstpart = &firstpart   secondpart = &secondpart; */
   		%let newstr = &firstpart.&replace.&secondpart ;
   	%end;
   	/* %put newstr = %unquote(&newstr); */
	%end;
	&newstr
%mend Search_and_Replace;


%macro PackagePublish(_path,_pubType);

   %if &SASMode=SPS %then %do;
   	%put Publishing the package.;
      %if (&_pubType = to_archive) %then %do;
         %let _properties = archive_path, archive_name;
	      %let _name = &ProcessName..spk;
         %syscall package_publish(ProcessID,_pubType,rcp,_properties,_path,_name);
      %end;
      %else %do;
         %let _pubType = to_requester;
         %let _properties = archive_path;
         %syscall package_publish(ProcessID,_pubType,rcp,_properties,_path);
      %end;
      
      %Checkrc(Package_Publish,_rc=&rcp);
 
     	%global _archive_fullpath;
     	%let _archive_fullpath = &_path.&_name;
     	%put _archive_fullpath = &_archive_fullpath;
   %end;
%mend PackagePublish;


%macro PackageEnd;
	%if %symexist(_mainjslfile_)=0 %then %do;
		%global _mainjslfile_;
		%let _mainjslfile_=;
	%end;

	%if &SASMode ne SDD %then %do;

   	* assign setting name;
   	%global fName;
   	%let fName =;
   	%if %symexist(StudyName)=0 %then %let StudyName=; 
   	
   	%if &Workflowrp ne -1 or %index(%lowcase(&processname),workflow) %then %do;
			%if %length(&StudyName) & &LS_Index ^= 3 %then %do; 
				%let fName = &StudyName; 
				%put fName from StudyName = &StudyName;
			%end;
		%end;
   	%if %lowcase(&processname)=standardsafetyreports or %lowcase(&processname)=assigndefaultdatasets %then %do;
			%if %length(&StudyName) %then %do; 
				%let fName = &StudyName; 
				%put fName from StudyName = &StudyName;
			%end;
		%end;
		%if ^%length(&fName) and %symexist(OutName) %then %do;
			%if %length(&OutName) %then %do;
				%let fName = &OutName; 
				%put fName from OutName = &OutName;
			%end;
		%end;
		%if ^%length(&fName) and %symexist(OutData) %then %do;
			%if %length(&OutData) %then %do;
				%if %index(&OutData,.) %then %do;
					%let fName = %scan(&OutData,2);
				%end;
				%else %do;
					%let fName = &OutData;
				%end;
				%put fName from OutData = &fName;
			%end;
		%end;
		%if ^%length(&fName) and %symexist(InData) %then %do;
			%if %length(&InData) %then %do; 
				%if %index(&InData,\) or %index(&InData,/) %then %do;
					%PathName(&InData);
					%let InData = &TmpName;
				%end;
				%let fName = &InData; 
				%put fName from InData = &fName;
			%end;
		%end; 
		%if ^%length(&fName) and %symexist(ExpInData) %then %do; 
			%if %length(&ExpInData) %then %do;
				%let fName = &ExpInData; 
				%put fName from ExpInData = &fName;
			%end;
		%end; 
		%if ^%length(&fName) and %length(&StudyName) %then %do; 
			%let fName = &StudyName; 
			%put fName from StudyName = &StudyName;
		%end;
		%if ^%length(&fName) %then %do;
			%let fName = LastRun;
			%put fName = LastRun;
		%end;
		%if &LS_Index = 3 %then %do;
			%let _tmp_ = %quote(%scan(&fName,-1,"_"));
			%if &_tmp_ ^= &_nThePro_ %then %let fName = &fName._&_nThePro_;
			%put fName with 3 digit suffix = &fName;
		%end;
		%let sasmessagefile = ;

		%if &_pkn>0 %then %do;
			%if (&WorkFlowrp ne -1)=0 %then %let sasmessagefile = &OutPath.sasmessage_&pkds_memname1..jsl;
			%else %let sasmessagefile = &OutPath.sasmessage_&_SettingsName_..jsl; * avoid replacement when appying same input datasets;
			filename jslmsg "&sasmessagefile";
			data _null_;
				%if %length(&_mainjslfile_) %then %do;
					file "&_mainjslfile_" mod %if %nrbquote(&SYSENCODING) ne wlatin1 %then encoding="utf-8";;
				%end;
				%else %do;
					file jslmsg %if %nrbquote(&SYSENCODING) ne wlatin1 %then encoding="utf-8";;
					put "//!";
					put "Names Default to Here(1);";
				%end;

				put "jg:_sasmessage_=1;";

				%if %length(&_mainjslfile_)=0 %then %do;
					put "Here:DataList = {};";
				%end;
				put "if(host is(Windows),";
				put "	include( ConvertFilePath(jg:LSpath[1]||""/Generator/DialogHelperFunctions.jsl"", absolute, windows) ),";
				put "	include( ConvertFilePath(jg:LSpath[1]||""/Generator/DialogHelperFunctions.jsl"", absolute, POSIX) ));";
				
				put "Here:sasmsgwindow=Function({},{default local},";
				put "STPN = locale:get(""&ProcessName._Label"");";
				put "Substitute Into(STPN, ""_Label"", """");";
				put "Vlistbox(   Here:_tb = TextBox(locale:fill(locale:get(""ProcessXhasgenerated_Text""),{STPN})),";
				/*
				put " ""has generated the SAS data set(s) shown below.  "" || ";
				put " ""You can view them using SAS labels by checking the box below, "" || ";
				put " ""otherwise SAS names are used.  The checkbox is ignored if only viewing a subset.""),";
				*/
				put "   TextBox("" ""),";
				put "  // Here:uselabelcb = Checkbox(""Use SAS labels for JMP column names"",<<Set(1,1)),";
				put "   Here:_tb << Set Wrap(400);";
				put "   TextBox("" ""),";
				%do _n_ = 1 %to &_pkn;
					%let dsn = %nrbquote(&&pkds_client&_n_);
					put "origdsdesc = ""&&pkds_desc&_n_"";";
					put "transdsdesc = origdsdesc;";
					put "Substitute Into(transdsdesc, ""_Label"", """");";
					put "   HListBox(";
					put "   Here:tbd=TextBox(transdsdesc),"; 
					%if %length(&_mainjslfile_) %then %do;
						put "   Here:tbd << Set Wrap(350),";
					%end;
					%else %do;
						put "   Here:tbd << Set Wrap(660),";
					%end;
					put "			ButtonBox(locale:get(""Open_Text""),";
					put "          jg:clientoutpath = ""&ClientOutPath"";";
					put "				//if(!isEmpty(Here:uselabelcb),";
					put "				Here:uselabels = 1;";
					put "				//);";
					put "				if(hostis(windows),";
					put "					jg:OpenWithSASNames(""&dsn"", 0, ""Here:smwdata&_n_"");";
					put "					if(!IsEmpty(Here:DataList), InsertInto(DataList,{{""Here:smwdata&_n_""}}));,";
					put "					SASDataSetTextOpen(""&dsn"",jg:jmpLSDir[&LS_Index]||""ProcessResults/"",1)";
					put "			)	),";
					put "			ButtonBox(locale:get(""ViewSubset_Text""),";
					put "				Include(jg:jmpLSUserDir[1]||""ProcessLibrary/CheckDataContents.jsl"");";
					put "				jg:CheckDataContents_InData << SetText(""&dsn"");";
					put "				jg:CheckDataContents_OutPath << SetText(""&ClientOutPath"");";
					put "				jg:runProcess(1,""CheckDataContents"")";
					put "			)";
					put "   ),";					
					put '   Here:_teb = TextEditBox("' @;
					put "&dsn" @;
					put '"),';
					put "      Here:_teb << Font Color(5),";
					%if %length(&_mainjslfile_) %then %do;
						put "   Here:_teb << Set Wrap(1000),";
					%end;
					%else %do;
						put "   Here:_teb << Set Wrap(1000),";
					%end;
					%if %upcase(&ProcessName) ne ANOVA & %upcase(&ProcessName) ne ONEWAYANOVA & %upcase(&ProcessName) ne MIXEDMODELANALYSIS %then %do;
						put "   TextBox(""   "" || locale:fill(locale:get(""Xobs_Text""),{""&&pkds_nobs&_n_""}) || "", "" || locale:fill(locale:get(""Xvariables_Text""),{""&&pkds_nvars&_n_""})),";
						put "   TextBox("" ""),";
					%end;
				%end;
				put "   TextBox("" "")";
				put "));";

				put "//sasmsgwindow << Set Window Icon(""$LS_HOME\Documentation\Icons\SASLogo.bmp"");";

			run;
			
			%let old_sasmessagefile = &sasmessagefile;
			
			%if (&Server ^= Local) %then %do;
				%if (&WorkFlowrp ne -1)=0 %then %let sasmessagefile = &ClientOutPath.sasmessage_&pkds_memname1..jsl;
				%else %let sasmessagefile = &ClientOutPath.sasmessage_&_SettingsName_..jsl;
				%if %length(&_mainjslfile_)=0 %then %do;
					%let nOut = %eval(&nOut+1);
					%global OutFile&nOut;
					%if (&WorkFlowrp ne -1)=0 %then %let OutFile&nOut = sasmessage_&pkds_memname1..jsl;
					%else %let OutFile&nOut = sasmessage_&_SettingsName_..jsl;
				%end;
			%end;

			%if %length(&_mainjslfile_)=0 %then %do;
				%if (&WorkFlowrp ne -1) %then 
					%let AllJSLIncludedFiles=&AllJSLIncludedFiles.%str( Include(\!%"&sasmessagefile\!%", <<new context, << \!%"Names Default To Here\!%"););
				%else %let AllJSLIncludedFiles=&AllJSLIncludedFiles.%str( Include(%"&sasmessagefile%", <<new context, << %"Names Default To Here%"););
				%let _mainjslfile_ = %quote(&old_sasmessagefile);
			%end;

  		%end; /* end any output SAS data sets */
  		
		%if %length(&_mainjslfile_) %then %do;
			
			data _null_;
				file "&_mainjslfile_" mod %if %nrbquote(&SYSENCODING) ne wlatin1 %then encoding="utf-8";;
				put "Here:APName = ""&processname"";";
				%if %symexist(FullAPName) %then %do;
					put "Here:FullAPName = ""&FullAPName"";";
				%end;
				%else %do;
					put "Here:FullAPName = ""&processname"";";
				%end;
				%if %symexist(StudyName) %then %do;
					put "Here:StudyName = ""&StudyName"";";
					put "jg:StudyName = ""&StudyName"";";
				%end;
				%else %do;
					put "Here:StudyName = """";";
				%end;				
				
				%if %index(&ProcessName,Bivariate) %then %do;
					put 'if (processname =="BivariateOneWayANOVAAverages",';
					put 'Here:APName = "BivariateOneWayANOVAAverages";';
					put 'Here:FullAPName = "Bivariate OneWay ANOVA on Averages",';
					put '//else';
					put 'Here:APName = "BivariateOneWayANOVADifferences";';
					put 'Here:FullAPName = "Bivariate OneWay ANOVA on Differences");';
					%let ProcessName = BivariateOneWayANOVA; 
				%end;
				
				%*if (&WorkFlowrp ne -1)=0 %then %do;
					put "Here:_oapwsf_ = 1;";
					put "Here:_OpenAPwithSetting_=Function({},{default local},";
					%if &processname eq SaveAsSASDataSet %then %do;
						put " include( jg:jmpLSUserdir[&LS_Index]||""ProcessLibrary/"" || ""SaveAsSASDataSetWrapper.JSL"", << Allow Include File Recursion);";
					%end;
					%else %do;
					/*
						%if (&Server ^= Local) %then %do;
							put " include( ""&CLIENT_USER_HOME.ProcessLibrary/"" || ""&processname"" || "".jsl"", << Allow Include File Recursion );";
						%end;
						%else %do;
					*/
							put " include( jg:jmpLSUserDir[&LS_Index]||""ProcessLibrary/"" || ""&processname"" || "".jsl"", << Allow Include File Recursion);";
						%*end;		
					%end;
					put " jg:setDefaultParmValues(&LS_Index,""&processname"");";
										
					%if (&WorkFlowrp ne -1)=0 %then %do;
						put " jg:runWithSettings(&LS_Index, ""&processname"", ""&ClientOutPath.&ProcessName.__&fName..sas"", 0 );";	
					%end;
					%else %do;
						put " jg:runWithSettings(&LS_Index, ""&processname"", ""&ClientOutPath.&_SettingsName_..sas"", 0 );";	
					%end;
					put ");";
				%*end;
				%if (&WorkFlowrp ne -1)=0 or %symexist(CVing) %then %do;
					put "try(Here:thissaslst = LoadTextFile(""&ClientOutPath.&OriginalProcessName.__&fname..lst""));";
				%end;				
				%else %do;
					put "try(Here:thissaslst = LoadTextFile(""&ClientOutPath.&OriginalProcessName._&WorkFlowRP..lst""));";
				%end;	
				%if ^%symexist(OpenSASOutputTab) %then %let OpenSASOutputTab = 1;
				put "Here:OpenSASOutputTab = &OpenSASOutputTab;";				
				put "Here:LSIndex = &LS_Index;";
				put "_OutputFolder = ""&ClientOutPath"";";
				put "Here:nwcounter=1;";
				put "Here:TabWinList = {};";
				
				put "include(jg:jmpLSDir[1]||""\JSLFiles\TabbedReport.jsl"");";
			run;
			
			%if %symexist(jsl_after_tabbedreport) %then %do;
				%if %length(&jsl_after_tabbedreport) %then %do;
					data _null_;
						file "&_mainjslfile_" mod;
						put "include(""&jsl_after_tabbedreport"");";
					run;
				%end;
			%end;
  		
  		%end;
		
		* update &_LogLib_.._processinfo2;
		%if (&ProcessName=ManageStudies) %then %do;
			%if (%quote(&Task)=Rename Study) %then %do;
				%let NewStudyName_ = %sysfunc(translate(&NewStudyName,%str(_),%str( )));
				%let _LogLib_ = NewMeta;
			%end;
			%else %if (%quote(&Task)=Remove Study) %then %do;
				%let _LogLib_ = NotesLib;
			%end;
			%else %do;
				%let _LogLib_ = _LogLib_;
			%end;				
		%end;
		%else %if (&ProcessName=ManageGenomicsStudy) %then %do;
			%let _LogLib_=OutLib;
		%end;
		%else %do;
			%let _LogLib_ = _LogLib_;
		%end;
		%if %length(&_MainJslFile_) %then %let _jFile_ = %scan(&_MainJslFile_,-1,&_slash_);
		%else %let _jFile_ = %scan(&sasmessagefile,-1,&_slash_);
		* update _ProcessInfo2 (result metadta);
		%ResultMeta(&_jFile_);
		
		proc sort data=_processinfo2 nodupkey;
			by File;
		run;
		data _processinfo2;
			set _processinfo2;
			format OutPath $256.;
			fileNumber = _n_;
			JslToOpen = 0;
			if upcase(scan(File,-1,'.')) = 'JSL' then do;
				if upcase(File) = upcase("&_jFile_") then do;
					JslToOpen = 1;
				end;
			end;
		run;
		%if %sysfunc(exist(&_LogLib_.._processinfo2)) %then %do;
			data &_LogLib_.._processinfo2;
				set _processinfo2 &_LogLib_.._processinfo2;
				if _n_ > 100000 then stop;
			run;
		%end;
		%else %do;
			data &_LogLib_.._processinfo2;
				set _processinfo2;
			run;				
		%end;
		%if &WorkFlowrp ne -1 %then %do;
			* keep information for building journal;
			%if &WorkFlowrp = 1 %then %do;			
				data _JL_pinfo2;
					set _processinfo2;
					WorkFlowrp = 1;
				run;
			%end;
			%else %do;		
				data _JL_pinfo2;
					set _JL_pinfo2 _processinfo2;
					if WorkFlowrp=. then WorkFlowrp = &WorkFlowrp;
				run;
			%end;
		%end;		
		
 		* write to project log;
 		%if ^%symexist(_StudyLogEndFlag_) %then %do;
 			%let LogEndding = 1;
 		%end;
 		%else %do;
 			%if &_nProj_ ^= &_PreEndingNum_ %then %let LogEndding = 1;
 			%else %let LogEndding = 0;
 		%end;
 		%if &LogEndding %then %do;
			%if (^%symexist(CVing) or 
				&ProcessName = CrossValidationModelComparison or 
				&ProcessName = TestSetModelComparison or
				&ProcessName = LearningCurveModelComparison )
				and %length(%sysfunc(pathname(projlog))) %then %do;
				%global _JslToOpen_;
				%if %length(&_MainJslFile_) %then %let _JslToOpen_ = &_MainJslFile_;
				%else %let _JslToOpen_ = &sasmessagefile;
			%*if (&Server = Local) & (&ProcessName ne ManageStudies) & (&processname ne ManageGenomicsStudy) %then %do;
				%if (&Server = Local) %then %do;
					%let _OutData_ = &&pkds&_i_;
					%let _OutJSL_ = &_JslToOpen_;
					%let _SASFilePath_ = &OutPath;
				%end;
				%else %do;
					%let _outdata_ = &ClientOutPath%scan(&&pkds&_i_,-1,&_slash_);
					%let _outjsl_ = &ClientOutPath%scan(&_JslToOpen_,-1,&_slash_);
					%let _SASFilePath_ = &ClientOutPath;
				%end;
				%if (&ProcessName=ManageStudies) %then %do;
					%if (%quote(&Task)=Rename Study) %then %do;
						filename ProjLog "%sysfunc(pathname(NewMeta))&_slash_.StudyLog.txt";
					%end;
					%if (%quote(&Task)=Remove Study) %then %do;
						filename ProjLog "%sysfunc(pathname(NotesLib))&_slash_.StudyLog.txt";
					%end;
				%end;	
				%else %if (&ProcessName=ManageGenomicsStudy) %then %do;
					filename ProjLog "%sysfunc(pathname(OutLib))&_slash_.StudyLog.txt";
				%end;
				data _null_;
					file ProjLog MOD %if %nrbquote(&SYSENCODING) ne wlatin1 %then encoding="utf-8";;
					%if &_pkn>0 %then %do;
						* write output data sets into Project Log;
						put " ";
						%if &_pkn>1 %then %let plural=s;
						%else %let plural=;
						put "   &_pkn Output Data Set&plural";
						%do _i_ = 1 %to &_pkn;
							put "   %trim(%left(&_i_)). &&pkds_desc&_i_:";
							put "      &_outdata_ ";
						%end; 
					%end;
					put " ";
					put "   JSLFile= &_outjsl_";
					put "   SASFile= &_SASFilePath_.&ProcessName.__&fName..sas";
					put "[End of Process &_nProj_] ";
				run;	
				%global _StudyLogEndFlag_ _PreEndingNum_;
				%let _StudyLogEndFlag_ = 1;
				%let _PreEndingNum_ = &_nProj_;
			%*end;		
			%end;
		%end;
	
		%global newSASname;
		%let newSASname=&OutPath.&ProcessName;

		%if &Server = Local %then %do;

			%if (&WorkFlowrp ne -1) & (&LS_Product^=Clinical) %then %do;
				%if %symexist(_SettingFullPath_) %then %do;
					%let fullName = %scan(&_SettingFullPath_,-1,"&_slash_");
					%let fName = %substr(&fullName,%length(&ProcessName)+3,
						%length(&fullName)-%length(&ProcessName)-6); 
				%end;
			%end;	

			filename OFCHK "&OutPath.&ProcessName..sas";
				
			%if %sysfunc(fexist(OFCHK)) %then %do;
				filename FCHK "&OutPath.&ProcessName.__&fName..sas";

				%let _frc_ = 0;
				%if %sysfunc(fexist(FCHK)) %then %do;
					%let _frc_=%sysfunc(fdelete(FCHK));
				%end;			

				%if (&_frc_) %then %do;
					%let _frc_=%sysfunc(fdelete(OFCHK));
					%put WARNING: The settings file, &OutPath.&ProcessName.__&fName, is read-only and can not be updated.;
					%put WARNING: The corresponding JSL file information can not be added into the seetings file.;
				%end;
				%else %do;
					%RenameFile(&OutPath.&ProcessName..sas,&OutPath.&ProcessName.__&fName..sas);
					%let newSASname=&OutPath.&ProcessName.__&fName;
					data _null_;
						file FCHK MOD %if %nrbquote(&SYSENCODING) ne wlatin1 %then encoding="utf-8";;
						put " ";
						put " ";
						put " ";
						put "/* parameters of corresponding jsl file: ";
						put "_ProcessNumber_= &_nProj_;";
						put "_JSLFile_= &_JslToOpen_;";
						/*
						%if (&Server = Local) %then %do;
							put "_JSLFile_= &_JslToOpen_;";
						%end;
						%else %do;
							%let _JSL_ = &ClientOutPath%scan(&_JslToOpen_,-1,&_slash_);
							put "_JSLFile_= &_JSL_;";
						%end;
						*/
						put "*/";
					run;
				%end;
			%end;

		%end;
		/* else have to do rename in sasclean */
  		
  %end;

   %if &SASMode=SPS & (&_ProcessName ne BivariateOneWayANOVAAverages) %then %do;
      %syscall package_end(ProcessID,rcp);
      %Checkrc(Package_End,_rc=&rcp);
      quit;
   %end;
 
%mend PackageEnd;

%macro ExitAndRetrieveLogLstFromJMP(_outpath, _processname);
	/** This code will cleanup the sas session. It is necessary for the ASYNC call to IOM in JMP.  **/
	%put NOTE: Calling ExitAndRetrieve;
	%put _outpath=&_outpath;
	%put _processname=&_processname;
	%let stillcving = 0;
	%if %symexist(CVing) %then %do;
		%put CVing = &CVing;
		%if (&CVing = 1) %then %let stillcving = 1;
	%end;
	%if (&WorkFlowrp ne -1) %then %do;
		%put Workflowrp = &Workflowrp;
	%end;
	/*---delete kill file---*/
	/* %RemoveFile(&killfile); */
	
	%if %symexist(killfile) %then %do;
		%put deleting &killfile;
		%if %symexist(gshare) %then %do;
			%if %sysfunc(fileexist(&gshare.&killfile)) %then %do;
				filename FCHK "&gshare.&killfile";
				%let _frc_=%sysfunc(fdelete(FCHK));
			%end;
		%end;
		%else %do;
			%if %sysfunc(fileexist(&_outpath.&killfile)) %then %do;
				filename FCHK "&_outpath.&killfile";
				%let _frc_=%sysfunc(fdelete(FCHK));
			%end;
		%end;
	%end;
	%else %do;
		%let killfile = %str( );
	%end;
	%if ^%symexist(sasmessagefile) %then %let sasmessagefile = ;
	

	/*---create sasclean.jsl, disabled for now within each CV iteration---*/
	%if (&stillcving = 0) %then %do;
		options obs=max;
		/*
		%if %symexist(CVing) & (&CVing=1) %then %do;
			filename sasclean "&OutPath.sasclean&Workflowrp..jsl";
		%end;
		*/
		filename sasclean "&OutPath.sasclean.jsl";
		
		/* if running on server, switch outpath values */
		%if (&Server ^= Local) %then %do;
			%let ServerOutPath = &OutPath;
			%let OutPath = &ClientOutPath;
		%end;
		
		%if %symexist(CVing) %then %do;
			%let OriginalProcessName = &_ProcessName;
		%end;
		
		data _null_;
			file sasclean %if %nrbquote(&SYSENCODING) ne wlatin1 %then encoding="utf-8";;
			put '//!'; 
			put "Names Default to Here(1);";
			put 'wait(0.5);';

			put "saslog = jg:saslog;";
			put "saslst = jg:saslst;";         

			/* substitute grid signon errors with warnings */
			put 'jg:saslog=substitute(jg:saslog,"ERROR: A communication subsystem","WARNING: A communication subsystem");';
			put 'jg:saslog=substitute(jg:saslog,"ERROR: Could not start grid job","WARNING: Could not start grid job");';
			put 'jg:saslog=substitute(jg:saslog,"ERROR: Remote signon to","WARNING: Remote signon to");';
			put 'jg:saslog=substitute(jg:saslog,"ERROR: A link to","WARNING: A link to");';

			%if (&WorkFlowrp ne -1) and (^%symexist(CVing) | &_Run_From_WFB_) %then %do;
				put 'try(SaveTextFile("'"&ClientOutPath.&OriginalProcessName._&WorkFlowrp..log"'", jg:saslog),';
			%end;
			%else %do;
				put 'try(SaveTextFile("'"&ClientOutPath.&OriginalProcessName..log"'", jg:saslog),';
			%end;
			put "   throw(""ERROR: Unable to save log file to &ClientOutPath..  You may not have write permission to this folder."");";
			put ');';
			%if &processname ne AEDistribution %then %do;
				put 'if(length(saslst)>0,';
				/*
				put 'SaveTextFile('"&OriginalProcessName"'_outp||"'"&OriginalProcessName"'"||".lst", saslst);';
				*/
				%if (&WorkFlowrp ne -1) and (^%symexist(CVing) | &_Run_From_WFB_) %then %do;
					put "  SaveTextFile(""&ClientOutPath.&OriginalProcessName._&WorkFlowrp..lst"", saslst);";
					put ', //else';
					put " if(File Exists(""&ClientOutPath.&OriginalProcessName._&WorkFlowrp..lst""),";
					put "  Delete File(""&ClientOutPath.&OriginalProcessName._&WorkFlowrp..lst"")";
					put " )";
				%end;
				%else %do;
					%if ^%symexist(fname) %then %let fname=;
					put '  //sasout=NewWindow("SAS Output",ScriptBox(saslst),<<SetWindowIcon("SASReport"));';
					put '  //sasout<<SizeWindow(800,600);';
					put '  SaveTextFile("'"&ClientOutPath.&OriginalProcessName.__&fname..lst"'", saslst);';
					put ', //else';
					put " if(File Exists(""&ClientOutPath.&OriginalProcessName.__&fname..lst""),";
					put "  Delete File(""&ClientOutPath.&OriginalProcessName.__&fname..lst"")";
					put " )";				
				%end;
				put ');';
			%end;
			/*
				put 'jg:saslog = LoadTextFile("'"&ClientOutPath.&OriginalProcessName..log"'");';	
			*/
			%if %symexist(NewStudy) %then %do;
				%if &NewStudy %then %do;
					put 'sasmsgwindow=New Window(locale:fill(locale:get("XMessage_Text"), {locale:get("JMPLifeSciences_Text")}),showmenu(0),showtoolbars(0),';
					put '   VListBox(';
					put "      TextBox("" ""),";
					put "      TextBox(locale:fill(locale:get(""ThestudyXhasbeensuccessfullyadded_Text""), {""&StudyName""}) || ""  ""),";
					put "   )";

					put ");";

					/* Update Current Study List on Genomics Starter */
					put "smd = open(jg:jmpLSUserDir[1]||""metadata_study.sas7bdat"");";
					put "jg:StudyType = Associative Array(:StudyName,:Type);";

					put "for each row(Selected(Rowstate())=(:Type==jg:LSprod[2]));";
					put "subdt = smd<<Subset(Output table name(""_metadata_study""));";	
					put "jg:study2list = Insert(Column(""StudyName"")<<GetAsMatrix,"" "",1);";
					put "smd << Close Window;";
					put "setstudy = ""&StudyName"";";
					put "subdt << Close Window;";

					put "jg:_rb2study_ << Set Items(jg:study2list);";

					put "if(!isEmpty(setstudy),jg:_rb2study_ << Set(Loc(jg:study2list,setstudy));";	
					/* Due to defect S0767103, need to manually run combobox script when setting via JSL */
					put "  jg:_curr2study_ = jg:study2list[jg:_rb2study_ << Get];";
					put "  SaveTextFile(jg:jmpLSUserDir[2]||""_CurrStudy_""||jg:sasserver||""_.jsl"",""jg:_curr2study_ = \!""""||jg:_curr2study_||""\!"";"");";	
					put " );";

					/* Create folder in user's metadata for StudyDataSets.jmp */
					put "//Create Directory(""&Study_Home"");";
					put "sdst = New Table(""StudyDataSets"", invisible);";
					put "sdst << New Column(""Path"",character);";
					put "sdst << New Column(""Name"",character);";
					put "sdst << New Column(""Type"",character);";
					put "sdst << New Column(""Date Created"",character, Formula(if(FileExists(:Path || :Name || "".sas7bdat""),MDYHMS(CreationDate(:Path || :Name || "".sas7bdat"")))));";
					put "sdst << New Column(""Date Modified"",character, Formula(if(FileExists(:Path || :Name || "".sas7bdat""),MDYHMS(LastModificationDate(:Path || :Name || "".sas7bdat"")))));";	

					put "Close(sdst, Save(""&Study_Home.StudyDataSets.jmp""));";

					put "sdst = New Table(""StudySettings"", invisible);";
					put "sdst << New Column(""Path"",character);";
					put "sdst << New Column(""Name"",character);";
					put "sdst << New Column(""Date Created"",character, Formula(if(FileExists(:Path || :Name || "".sas""),MDYHMS(CreationDate(:Path || :Name || "".sas"")))));";
					put "sdst << New Column(""Date Modified"",character, Formula(if(FileExists(:Path || :Name || "".sas""),MDYHMS(LastModificationDate(:Path || :Name || "".sas"")))));";	
					put "Close(sdst, Save(""&Study_Home.StudySettings.jmp""));";
				
					%let NewStudy = 0;
				%end;
			%end;
			
			%if &nStudyOut > 0 %then %do;
			
				put "try(_sds = open(""&Study_Home.StudyDataSets.jmp"", invisible);";
				put "_sds << Suppress formula eval(1);";
				put "_sds << Add Rows( " @;
				%do i=1 %to &nStudyOut;
					put "{:Path=""&OutPath"",:Name=""&&StudyFile&i"",:Type=""Genomics""},"@;
				%end;
				put ");";
				put "_sds << Suppress formula eval(0);";
				put "Column(""Date Created"") << EvalFormula;";
				put "Column(""Date Modified"") << EvalFormula;";				
				put "_sds << Sort(By(:Data Modified), Order(Descending), Replace table);";
				/* Drop dups */
				put " _sdsnew = _sds << Join( With( _sds ),";
				put " By Matching Columns( :Path=:Path,:Name=:Name), Drop multiples(1,1), Include nonmatches(0,0) ";
				put ");";
				put "Close(_sds,No save);";
				put "Close(_sdsnew,Save(""&Study_Home.StudyDataSets.jmp""));";
				put "); //end try";
			%end;
			
			%if %symexist(Study_Home) %then %do;
				put "try(_sds = open(""&Study_Home.StudySettings.jmp"", invisible);";
				put "_sds << Suppress formula eval(1);";
				put "_sds << Add Rows( " @;
				put "{:Path=""&OutPath"",:Name=""&ProcessName.__&fName""},"@;
				put ");";
				put "_sds << Suppress formula eval(0);";
				put "Column(""Date Created"") << EvalFormula;";
				put "Column(""Date Modified"") << EvalFormula;";				
				put "_sds << Sort(By(:Data Modified), Order(Descending), Replace table);";
				/* Drop dups */
				put " _sdsnew = _sds << Join( With( _sds ),";
				put " By Matching Columns( :Path=:Path,:Name=:Name), Drop multiples(1,1), Include nonmatches(0,0) ";
				put ");";
				put "Close(_sds,No save);";
				put "Close(_sdsnew,Save(""&Study_Home.StudySettings.jmp""));";
				put "); //end try";			
			
			%End;			
			
			* if running on server, copy all output files to client;
			%if (&Server ^= Local) and (&nOut>0 or (&WorkFlowrp ne -1)=0) %then %do;
				%put nOut = &nOut;
				put 'if(MetaIsConnected() | jg:sasserver!="Local",';
				%do i=1 %to &nOut;
					put "   rc = jg:SASGetFile2(";
					put "      ""&ServerOutPath.&&OutFile&i"",";
					put "      ""&ClientOutPath.&&OutFile&i""";
					put "   );";
					put "   if(!rc,";
					put "      rc = jg:SASGetFile2(";
					put "         ""&ServerOutPath.%lowcase(&&OutFile&i)"",";
					put "         ""&ClientOutPath.%lowcase(&&OutFile&i)""";
					put "      );";
					put "   );";
					put "   if(!rc,";
					put "      NewWindow(locale:get(""ServerGetFileError_Text""),showmenu(0),showtoolbars(0),";
					put "         TextBox(locale:get(""ErrorUnableToGetOutputFile_Text"") || ""\!N\!N""),";
					put "         HListBox(";
					put "            TextBox(""   ""),";
					put "            TextEditBox(""&ServerOutPath.&&OutFile&i"",<<Font Color(5),<<Set Wrap(700),<<SetNLines(2)),";
					put "         ),";
					put "         TextBox(""\!N"" || locale:get(""fromtheserverandsaveto_Text"") || ""\!N\!N""),";
					put "         HListBox(";
					put "            TextBox(""   ""),";
					put "            TextEditBox(""&ClientOutPath.&&OutFile&i"",<<Font Color(5),<<Set Wrap(700),<<SetNLines(2)),";
					put "         ),";
					put "         TextBox(""\!N"" || locale:get(""ontheclientThefilemayexistorbeinuseorlocked_Text"")),";
					put "         jg:messageHelp(""ErrorUnableToGetOutputFile_Text, fromtheserverandsaveto_Text, ontheclientThefilemayexistorbeinuseorlocked_Text"")";
					put "      )";
					put "   );";
					put "   if(munger(Uppercase(""&&OutFile&i""),1,Uppercase(""Body.html"")),";
					put "      hfile=loadtextfile(""&ClientOutpath.&&OutFile&i"");";
					put "      hfile=substitute(hfile,""&ServerOutPath"",""&ClientOutpath"");";
					put "      SaveTextFile(""&ClientOutpath.&&OutFile&i"",hfile);";
					put "   );";
				%end;
				%if (&WorkFlowrp ne -1) & (&LS_Product^=Clinical) %then %do;
					%if %index(&_SettingFullPath_,&_slash_) %then %let fullName = %scan(&_SettingFullPath_,-1,"&_slash_");
					%else %let fullName = %scan(&_SettingFullPath_,-1,"\");
					%let fName = %substr(&fullName,%length(&ProcessName)+3,%length(&fullName)-%length(&ProcessName)-6);
				%end;	
				%*if (&WorkFlowrp ne -1)=0 %then %do;
					%*if %length(&_MainJslFile_) %then %let _JslToOpen_ = &_MainJslFile_;
					%*else %let _JslToOpen_ = &sasmessagefile;
					%let _jslName_ = %scan(&_JslToOpen_,-1,"&_slash_");
					%let _JslToOpen_ = &ClientOutPath.&_jslName_;
					%if %upcase(&ProcessName) = CROSSVALIDATIONMODELCOMPARISON %then %do;
					put "   _ssf = LoadTextFile(""&ClientOutPath.CrossValidationModelComparison__LastRun.sas"");";
					%end;
					%else %do;
					put "   _ssf = LoadTextFile(""&ClientOutPath.&ProcessName..sas"");";
					%end;
					put '   _jb_ = EvalInsert("\[';
					put "      ";
					put "      ";
					put "      ";
					put "/* parameters of corresponding jsl file: ";
					put "_ProcessNumber_= &_nProj_;";
					put "_JSLFile_= &_JslToOpen_;";
					put "*/";
					put '   ]\");';
					put "   _ssf = _ssf || _jb_;";
					put "   SaveTextFile(""&ClientOutPath.&ProcessName.__&fName..sas"",_ssf);";
					
					%let newSASname=&ClientOutPath.&ProcessName.__&fName;
				%*end;
				put ');';
			%end;
			
			* remove process from running process window;
			put 'somerunning = NItems(jg:runningAPs);';
			put 'for(j=2,j<=somerunning, j=j+3,';
			put '   if(jg:runningAPs[j]=="'"&killfile"'",';
			put '      i = (j+1)/3;';
			put '     if(!isempty(jg:runningApsListBox),';
			put '        jg:runningApsListBox << SetSelected(i,1);';
			put '        jg:runningApsListBox << RemoveSelected();';
			put '    );';
			put '      RemoveFrom(jg:runningAPs,j-1);';
			put '      RemoveFrom(jg:runningAPs,j-1);';
			put '      RemoveFrom(jg:runningAPs,j-1);';
			put '   )';
			put ');';
			put 'nrunning = NItems(jg:runningAPs);';
			/*
			put 'show(nrunning);';
			put 'show(jg:runningprocessesDialog);';
			*/
			put 'if (nrunning==0 & somerunning>0 & !isempty(jg:runningprocessesDialog),jg:runningprocessesDialog << CloseWindow);';
			/*
			put 'wait(0.5);';
			put "if(MetaIsConnected() | jg:sasserver==""Local"",SASDisconnect());";
			*/

			/*
			put 'endlogcheck = munger(jg:saslog,1,"***PACMAN***");';
			put 'if (endlogcheck==0, endlogcheck = length(jg:saslog));';
			put 'jg:saslog = substr(jg:saslog,1,endlogcheck);';	
			*/

			put 'jg:SASErr = munger(jg:saslog,1,"\!NERROR");';
			put 'if(jg:SASErr>0,';
			%if (&WorkFlowrp ne -1) and (^%symexist(CVing) | &_Run_From_WFB_) %then %do;
				put '   SaveTextFile("'"&ClientOutPath.SAS Log for &OriginalProcessName._&Workflowrp..jsl"'", jg:saslog);';			
			%end;
			%else %do;
				put '   SaveTextFile("'"&ClientOutPath.SAS Log for &OriginalProcessName..jsl"'", jg:saslog);';
			%end;
			put '   enderr = munger(jg:saslog,jg:SASErr,"\!NNOTE");';
			put '   errmsg = substr(jg:saslog,jg:SASErr,enderr-jg:SASErr);';
			put '   toppart = substr(jg:saslog,1,jg:SASErr);';
			put '   botpart = substr(jg:saslog,enderr,length(jg:saslog));';
			put '   ew = New Window(locale:fill(locale:get("SASErrorRunningX_Text"), {"'"&ProcessName"'"}), showmenu(0),showtoolbars(0),';
			put '      vlb=VListBox(';
			put '         TextBox(locale:fill(locale:get("ThefollowingerroroccurredinprocessX_Text"), {"'"&ProcessName"'"})),';
			put '         errtxt = TextEditBox(jg:stripMessageHelpTags(errmsg)||"\!N\!N",WhiteBoxStyle(0)),';
			put '         errtxt << Font Color(3),';
			put '         errtxt << Set Wrap(1000),';
			put "	      jg:messageHelp(jg:saslog),";
			put '         TextBox(locale:get("PleaseReferToSASLog_Text") || "\!N\!N"),';
			put '         TextBox(" "),';
			put '         ButtonBox(';
			put '            locale:get("ViewSASLog_Text"),';		
			%if (&WorkFlowrp ne -1) and (^%symexist(CVing) | &_Run_From_WFB_) %then %do;
				put '            try(';
				put '               vsl << BringWindowToFront();';
				put '            ,';
				put '               vsl=open("'"&ClientOutPath.SAS Log for &OriginalProcessName._&Workflowrp..jsl"'", force refresh);';
				put '            )';
			%end;
			%else %do;
				put '            try(';
				put '               vsl << BringWindowToFront();';
				put '            ,';
				put '               vsl=open("'"&ClientOutPath.SAS Log for &OriginalProcessName..jsl"'", force refresh);';
				put '            )';
			%end;
			put '         )';
			put '      ),';
			put '      If(and(!isempty(orig_apname),length(orig_apname)!=0),';			
			put '        TextBox("WARNING: The parameter from "||orig_apname||" will not be loaded. \!N\!N\!N\!N",<<Font Color(3)),';
			put '      );';
			put '   );';	
			/*
			%if (&WorkFlowrp ne -1) and (^%symexist(CVing) | &_Run_From_WFB_) %then %do;
				put '   try(open("'"&ClientOutPath.SAS Log for &OriginalProcessName._&Workflowrp..jsl"'", force refresh));';			
			%end;
			%else %do;
				put '   try(open("'"&ClientOutPath.SAS Log for &OriginalProcessName..jsl"'", force refresh));';
			%end;
			*/
			put '   ew << BringWindowToFront();';
			put "   ew << Set Window Icon(jg:jmpLSdir[1]||""\Documentation\Icons\SASLogo.bmp"");";
			put '   orig_apname="";';			
			put ');';
						
			/*
			%let i1 = (&WorkFlowrp ne -1);
			%let i2 = %symexist(CVing);
			%put i1=&i1 i2=&i2;
			*/
			%if (&WorkFlowrp ne -1) and (^%symexist(CVing) | &_Run_From_WFB_) %then %do;
				%put entered if cond;
				%if %upcase(&OriginalProcessName) = ADDSTUDYLOCAL or %upcase(&OriginalProcessName) = MANAGESTUDIES %then %do;
					put 'jg:studyname="'"&StudyNameUnder"'";';
				%end;
				put "sasclean&WorkFlowrp=EvalInsert("'"\[';
				put '//start';
				put '   saslog=LoadTextFile(\!"'"&ClientOutPath.&OriginalProcessName._&WorkFlowrp..log"'\!");';
				put '   endlogcheck = munger(saslog,1,\!"***PACMAN***\!");';
				put '   if (endlogcheck==0, endlogcheck = length(saslog));';
				put '   saslogcheck = substr(saslog,1,endlogcheck);';	
				put '   jg:SASErr = munger(saslogcheck,1,\!"\!NERROR\!");';
				put '   if(jg:SASErr>0,';
				put '      // log=NewWindow(\!"SAS Log\!",ScriptBox(saslog));';
				put '      enderr = munger(saslogcheck,jg:SASErr,\!"\!NNOTE\!");';
				put '      errmsg = substr(saslogcheck,jg:SASErr,enderr-jg:SASErr);';
				put '      toppart = substr(saslogcheck,1,jg:SASErr);';
				put '      botpart = substr(saslogcheck,enderr,length(saslogcheck));';
				put "      New Window(locale:fill(locale:get(\!""SASErrorRunningX_Text\!""), {\!""&ProcessName\!""}), showmenu(0),showtoolbars(0),";
				put '         vlb=VListBox(';
				put "         TextBox(locale:fill(locale:get(\!""ThefollowingerroroccurredinprocessX_Text\!""), {\!""&ProcessName\!""})),";
				put '         errtxt = TextBox(jg:stripMessageHelpTags(errmsg)||\!"\!N\!N\!"),';
				put '         errtxt << Font Color(3),';
				put '         errtxt << Set Wrap(1000),';
				put "	      jg:messageHelp(saslogcheck),";
				put "         TextBox(locale:get(\!""PleaseReferToSASLogDispJMPLogWind_Text\!"") || \!""\!N\!N\!""),";
				/*
				put '         // TextBox(locale:get(\!"PleaseReferToSASLog_Text\!") || \!"\!N\!N\!N\!N\!N\!"),';
				put '         // ButtonBox(locale:get(\!"ViewJMPLog_Text\!"),Window(\!"Log\!") << BringWindowToFront;);';
				*/
				put '         ButtonBox(locale:get(\!"ViewJMPLog_Text\!"),Main Menu(\!"View:Log\!"));';
				put '         )';
				put '      );';
				put '      show(saslog);';
				put '   );';
				put '   saslst=\!"\!";';
				put '   try(saslst=LoadTextFile(\!"'"&ClientOutPath.&OriginalProcessName._&WorkFlowrp..lst"'\!")';
				put '   );';
				%let w=1;
				%do %while(%length(%qscan(&AllJSLIncludedFiles,&w,%str(;))));
					%let word=%qscan(&AllJSLIncludedFiles,&w,%str(;));
					put " %qleft(&word;)";
					%let w=%eval(&w+1);
				%end;
				put '//end';
				put ']\");';
				%if &_Run_From_WFB_ %then %do;
					put 'here:_JRNFlagGlobal_ = 1;';
				%end;
				put 'jg:_PostProcess();';
			%end;
			%else %do;
				%let w=1;
				%do %while(%length(%qscan(&AllJSLIncludedFiles,&w,%str(;))));
					%let word=%qscan(&AllJSLIncludedFiles,&w,%str(;));
					%put word = &word;
					put "%qleft(&word;)";
					%let w=%eval(&w+1);
				%end;
			%end;
			/*
			put 'wait(1);';
			put 'Main Menu("View:Log");'; 
			put '  ';
			*/
			%if (&WorkFlowrp ne -1)=0 and %index(&ProcessName,Workflow)=0 and %symexist(USER_HOME) %then %do;
				%if &Server=Local %then %do;
					put "try(include(""&USER_HOME.ArchiveSASCode.jsl""));";
				%end;
				%else %do;
					put "try(include(""&CLIENT_USER_HOME.ArchiveSASCode.jsl""));";
				%end;
				put "if(!isEmpty(jg:_ArchiveSASCode_),";
				put "  if(jg:_ArchiveSAScode_==1,";
				put "    sascode = LoadTextFile(""&newSASname..sas"");";
				put '    incloc=Munger(sascode,1,"%include \!"");';
				put "    incend=Munger(sascode,incloc,"";"");";
				put "    while(incloc>0,";
				put "	     incfileline = Substr(sascode,incloc,incend);";
				put '      quote1= Munger(incfileline,1,"\!"")+1;';
				put '      quote2 = Munger(incfileline,quote1,"\!"");';
				put "      incfileS = Substr(incfileline,quote1,quote2-quote1);";
				put "      //show(incfileS);";
				put "      incfileJ = Munger(incfileS,1,""&LS_HOME."",""$LS_HOME/"");";
				put '      incfileJ = Munger(incfileJ,1,"&MacroPath./","$LS_HOME/MacroLib/");';
				put '      incfileJ = Munger(incfileJ,1,"&MacroPath.\","$LS_HOME/MacroLib/");';		
				put '      incfileJ = Munger(incfileJ,1,"&CLINICAL_HOME.\","$CLINICAL_HOME/");';
				%if %symexist(SInPath) %then %do;
					put "      incfileJ = Munger(incfileJ,1,""%nrstr(&SInPath.)"",""&SInPath."");";
				%end;
				put "  	  //show(incfileJ);";
				put "      sascode = Munger(sascode,1,incloc-1)||""/**************************************\!N""||";
				put "      ""Code %include'd from ""||incfileS||""\!N***************************************/\!N""||";
				put "      LoadTextFile(incfileJ)||""/***********************************\!N""||";
				put "      ""End of Code %include'd from ""||incfileS||""\!N************************************/\!N""||Munger(sascode,incend+1,-1);";
				put '      incloc = Munger(sascode,1,"%include \!"");';
				put "      incend=Munger(sascode,incloc,"";"");";
				put "    );";
				put "    //show(incfile);";
				put "    SaveTextFile(""&newSASname._Archive.sas"",sascode);";	
				put ") );  ";
			%end;
			%if %symexist(StudyName) %then %do;
				put "jg:study = ""&StudyName"";";
			%end;
			put "newsasname = ""&newSASname..sas"";";
			put "if(!isEmpty(jg:_ArchiveSASCode_),if(jg:_ArchiveSAScode_==1,";
			put "   newsasnamearchive = ""&newSASname._Archive.sas"";";
			put "));";
		run;
		/*
		%LaunchSASClean;			
		*/
	%end;
	
	/*
	%if %symexist(USER_HOME) %then %do;
		data _null_;
			filename workpath "&USER_HOME.workpath.txt";
			file workpath;
			put;
		run;
	%end;
	*/
	
	%if (&Server ^= Local) %then %do;
		%if %upcase(&_ProcessName) = BIVARIATEONEWAYANOVAAVERAGES %then %do;
			%let ClientOutPath = &OutPath;
			%let OutPath = &ServerOutPath;
		%end;
	%end;
	
%mend ExitAndRetrieveLogLstFromJMP;


%macro LaunchSASClean;
	%if &sysscp = WIN %then %do;
		%LaunchFile(&OutPath.sasclean.jsl);
	%end;
	%else %do;
		data a;
			length instring $ 256;
			infile "&LS_HOME.GenomicsInstallLog.txt" truncover;
			input instring $1-256;
			if substr(instring,1,3) = "JMP" then call symput("JMP_HOME",trim(left(substr(instring,5))));
			if substr(instring,1,3) = "SAS" then call symput("SAS_HOME",trim(left(substr(instring,5))));
		run;

		%CheckEndSlash(JMP_HOME);
		%CheckEndSlash(SAS_HOME);

		%put JMP_HOME = &JMP_HOME;
		%put SAS_HOME = &SAS_HOME;

		%LaunchExternalProgram(&JMP_HOME.jmp, --singleton, &OutPath.sasclean.jsl, &OutPath);

	%end;
%mend LaunchSASClean;


* determine path and filename from a full reference;
%macro PathName(_ref0);
   %global TmpPath TmpName TmpName2;
   %let _ref = %nrbquote(&_ref0);
   %put PathName: _ref = &_ref; 
   
   %let lp = %length(&_ref);
   %let x1 = 0;
   %let x2 = 0;
   %do _i_=1 %to &lp;
   	%let ri = %qsubstr(&_ref,&_i_,1);
   	%if &ri = %quote(/) or &ri = %quote(\) %then %let x1 = &_i_;
   	%if &ri = %quote(.) %then %let x2 = &_i_;
   %end;
   %if &x2 = 0 | &x1 > &x2 %then %let x2 = %eval(&lp+1);
   %let TmpPath = %qsubstr(&_ref,1,&x1);
   %let TmpName = %qsubstr(&_ref,%eval(&x1+1),%eval(&x2-&x1-1));
   %let TmpName2 = %qsubstr(&_ref,%eval(&x1+1));
   /*
   data _null_;
      l = length("&_ref");
      if (l<2) then do;
         error "ERROR: Length of path reference is less than 2.";
         call symput("exiterror",1);
         stop;
      end;
      flag = 0;
      do i = 1 to l;
         if substr("&_ref",i,1) = "&_slash_" then x1 = i;
         if substr("&_ref",i,1) = '.' then do;
            x2 = i;
            flag = 1;
         end;
      end;
      if flag = 0 then x2 = l + 1;
      call symput('TmpPath0',trim(left(substr("&_ref",1,x1))));
      call symput('TmpName0',trim(left(substr("&_ref",x1+1,x2-x1-1))));
      call symput('TmpName02',trim(left(substr("&_ref",x1+1))));
   run;
   %put PathName: TmpPath0=&TmpPath0 TmpName0=&TmpName0 TmpName02=&TmpName02;
   %let TmpPath = %trim(%left(%nrbquote(&TmpPath0)));
   %let TmpName = %trim(%left(%nrbquote(&TmpName0)));   
   %let TmpName2 = %trim(%left(%nrbquote(&TmpName02)));
   */
   %put PathName: TmpPath=&TmpPath TmpName=&TmpName TmpName2=&TmpName2;
   %if (%length(&TmpPath)=0) or (%length(&TmpName)=0) %then %let exiterror = 1;
%mend PathName;


* Check for naming convention;
* 1. Check leading letter;
* 2. Replace non-alphabetical charactor to '_';
* 3. UniqueRow to be indicator for checking uniqueness of rows;
* 4. DupFlag keep the list of flags indicating within column duplication;
* 5. Specify SingleVar as the macro containing string for naming checking
	  if no Dset provided;  
%macro Naming(_Dset=,_AllVars=,_UniqueRow=0,_DupFlag=,_SingleVar=);
%let _String1_ = '_','A','B','C','D','E','F','G','H','I',
           		  'J','K','L','M','N','O','P','Q','R','S',
					  'T','U','V','W','X','Y','Z';
%let _String2_ = '0','1','2','3','4','5','6','7','8','9';
%if %length(&_Dset) %then %do;
	%let _i_ = 1;
	%do %while(%length(%scan(&_AllVars,&_i_)));
		%let _Name&_i_._ = %trim(%left(%scan(&_AllVars,&_i_)));
		%let _i_ = %eval(&_i_+1);
	%end;
	%let _nName_ = %eval(&_i_-1);
   data &_Dset;
      set &_Dset;
		array _cName_ {&_nName_} $ &_AllVars;
      array _tmp_ {&_nName_} $32 ;
		%do _j_ = 1 %to &_nName_;
			_tmp_(&_j_) = _cName_(&_j_);
         if upcase(substr(_tmp_(&_j_),1,1)) not in (&_String1_) then _tmp_(&_j_) = '_'||trim(left(_tmp_(&_j_)));
			do i = 2 to length(_tmp_(&_j_));
         	if upcase(substr(_tmp_(&_j_),i,1)) not in (&_String1_,&_String2_) then
					substr(_tmp_(&_j_),i,1)='_';
			end;
			%if ^&_UniqueRow %then %do;
				drop &&_Name&_j_._;
				rename _tmp_&_j_=&&_Name&_j_._;
			%end;
		%end;
		%if &_UniqueRow %then _order_ = _n_;;
		drop i;
   run;

	%if &_UniqueRow %then %do;
		%do _i_= 1 %to &_nName_;
			proc sort data=&_Dset;
				by _tmp_&_i_;
			run;

			data &_Dset;
				length &&_Name&_i_._ $32;
				set &_Dset end=_e_;
				by _tmp_&_i_;
				retain _dFlag_ 0 _ctmp_;
				_Label&_i_._ = trim(left(&&_Name&_i_._));
				%if &_i_ = 1 %then %do;
					vLabel = trim(left(&&_Name&_i_._));
				%end;
				if first._tmp_&_i_ then do;
					_ctmp_=trim(left(_tmp_&_i_));
					&&_Name&_i_._ = _tmp_&_i_;
				end;
				else do;
					_dFlag_ = 1;
					_Len1_ = length(_ctmp_);
					_Len2_ = length(trim(left(_order_)));
					if (_Len1_+_Len2_<=32) then &&_Name&_i_._ = trim(left(_ctmp_)) || trim(left(_order_));
					else &&_Name&_i_._ = substr(_ctmp_,1,_Len1_-(_Len1_+_Len2_-32))||trim(left(_order_));
				end;
				%if %length(&_DupFlag) %then %do;
					if _e_ then call symput("&_DupFlag","&&&_DupFlag"||' '||trim(left(_dFlag_)));
				%end;
				drop _ctmp_ _Len1_ _Len2_;
			run;
		%end;

		proc sort data=&_Dset out=&_Dset(drop=_order_);
			by _order_;
		run;
	%end;
%end;
%else %if %length(&_SingleVar) %then %do;
	data _null_;
		length name $32.;
		name = "&&&_SingleVar";
		if upcase(substr(name,1,1)) not in (&_String1_) then name = '_'||name;
		do i = 2 to length(name);
         if upcase(substr(name,i,1)) not in (&_String1_,&_String2_) then
				substr(name,i,1)='_';
		end;
		call symput("&_SingleVar",name);
	run;
%end;
%mend Naming;
/*
* Check for naming convention;
* 1. check leading letter;
* 2. call proc transpose twice to get valid SAS variable name; 
%macro Naming(_Dset=,_AllVars=);
   %let _i_ = 1;
   %do %while(%length(%scan(&_AllVars,&_i_)));
	   %put Check naming on variable number &_i_ - %scan(&_AllVars,&_i_);
      %let _Name&_i_._ = %scan(&_AllVars,&_i_);
      data tmp0;
         set &_Dset;
         length tmp $32;
         NCL1 = substr(trim(left(&&_Name&_i_._)),1,1);
         if upcase(NCL1) in ('_','A','B','C','D','E','F','G','H','I',
                             'J','K','L','M','N','O','P','Q','R','S',
                             'T','U','V','W','X','Y','Z') then NCflag = 1;
         if NCflag ne 1 then tmp = '_'||trim(left(&&_Name&_i_._));
         else tmp = trim(left(&&_Name&_i_._));
         order = _n_;
         drop NCL1 NCFlag &&_Name&_i_._;
         rename tmp=&&_Name&_i_._;
      run;

		* modify length;
		data a; 
		   set tmp0 end=_e;
			retain _len_;
			if _n_ = 1 then _len_ = length(&&_Name&_i_._);
			else _len_ = max(length(&&_Name&_i_._),_len_);
			if _e then call symput('_NameLen_',_len_);
		run;

		data tmp0;
		   set tmp0;
			length _NewName_ $ &_NameLen_;
			_NewName_ = &&_Name&_i_._;
			drop &&_Name&_i_._;
			rename _NewName_ = &&_Name&_i_._;
		run;

      proc sort data=tmp0;
         by &&_Name&_i_._;
      run;

      proc sort data=tmp0(keep=&&_Name&_i_._) out=tmp1 nodupkey;
         by &&_Name&_i_._;
      run;

      proc transpose data=tmp1 out=tmp2(drop=_name_);
         var &&_Name&_i_._;
         id &&_Name&_i_._;
      run;

      proc contents data=tmp2 out=tmp3(keep=NAME) noprint;
      run;

      %let found = 0;
      data _null_;
        set tmp3;
        if upcase(NAME) = "_LABEL_" then do;
           call symput('found',1);
           stop;
        end;
      run;

      %if &found %then %do;
         data tmp2;
            set tmp2;
            drop _Label_;
         run;
      %end;

      proc transpose data=tmp2 out=tmp4(rename=(col1=&&_Name&_i_._));
         var _all_;
      run;

      proc sort data=tmp4;
         by &&_Name&_i_._;
      run;

      data tmp5;
         merge tmp0 tmp4;
         by &&_Name&_i_._;
      run;

     proc sort data=tmp5;
        by order;
     run;

     data &_Dset;
        set tmp5;
        &&_Name&_i_._ = _name_;
        drop order _name_;
     run;
     %let _i_ = %eval(&_i_+1);
   %end;
   %put End of Checking Names in Data Set - &_Dset;
%mend Naming;
*/

%macro ImportScanRows(_Flag=,_ScanRows=);
%let _workpath_=%sysfunc(pathname(work));

%if &_ScanRows > 20 & &_Flag %then %do;
   proc registry export="&_workpath_.&_slash_.registry.txt";
   run;

   filename guessrow "&_workpath_.&_slash_.guessingrows.txt";
   data _null_;
      file guessrow;
     put '[\PRODUCTS\BASE\EFI]';
     put "%str(%"GuessingRows%"=int:&ScanRows)";
   run;

   proc registry import=guessrow;
   run;

   %if %sysfunc(fexist(guessrow)) %then %let _frc_=%sysfunc(fdelete(guessrow));
%end;

filename registry "&_workpath_.&_slash_.registry.txt";
%if %sysfunc(fexist(registry)) & &_Flag=0 %then %do;
   proc registry import=registry;
   run;
   %let _frc_=%sysfunc(fdelete(registry));
%end;
%mend ImportScanRows;


/*---create a stacked data set from a combination of a tall data set and an 
	  experimental design file---*/
%macro Stack(_In=InLib.&InData,_InED=OutLib.&ExpInData,_Out=,_Response=Response,_EDrop=,_IKeep=,_WhereStmt=,_FirstNObs=0);
%ClockStart(Stack);

* stack indata;
proc sort data=&_InED out=ed;
	by ColumnName;
run;


* get length of ColumnName;
%let _edid_ = %sysfunc(open(ed));
%let _cnum_ = %sysfunc(varnum(&_edid_,ColumnName)); 
%let _clen_ = %sysfunc(varlen(&_edid_,&_cnum_));
%let _rc_ = %sysfunc(close(&_edid_));

* check for variables to keep initially to avoid building huge stack;
%let _IKeep0 = ;
%if %length(&_IKeep) %then %do;
	%let _IKeep0 = &_IKeep;
	%put _IKeep0 = &_IKeep0;
%end;

* stack indata;
data &_Out;
	set &_In %if %length(&_WhereStmt) %then (where=(&whereStmt));;
	array _col[*] %do i = 1 %to &_nAllColChunk_; &&_AllCol_&i._ %end;;
	length ColumnName $ &_clen_;
	_rownum = _n_;
	%if &_FirstNObs > 0 %then %do;
		if _n_ <= &FirstNObs;
	%end;
	%do _c_=1 %to &_nTallCol_;
		ColumnName = "&&_ColName_&_c_._";
		&_response = _col[&_c_];
		output;
	%end;
	
	%if %length(&_IKeep0) %then %do;
		keep _rownum ColumnName &_response &_IKeep0;
	%end;
	%else %do;
		drop %do i = 1 %to &_nAllColChunk_; &&_AllCol_&i._ %end;;
	%end;
run;

proc sort data=&_Out;
	by ColumnName;
run;

* match merge experimental design data;
%put _IKeep = &_IKeep;
data &_Out;
	merge ed &_Out;
	by ColumnName;
	%if %length(&_IKeep) %then %do;
		keep &_response &_IKeep;
	%end;
	%if %length(&_EDrop) %then %do; drop &_EDrop; %end;
run;

/*
data &_Out;
	set InLib.&InData;
	array _col[*] &_AllCol_;
   _rownum = _n_;
	%do _c_=1 %to &_nTallCol_;
		&_response = _col[&_c_];
		%do _f_ = 1 %to &_nFactor_;
		   %if %upcase(&&_Exp_Var&_f_._) ne COLUMNNAME %then %do;
		      %if &&_Exp_VarType&_f_._ = N %then &&_Exp_Var&_f_._ = &&_factor&_f_._&_c_._;
			   %else &&_Exp_Var&_f_._ = "&&_factor&_f_._&_c_._";;
			%end;
		%end;
		output;
	%end;
	drop &_allcol_ &_EDrop;
run;
*/


%ClockStop(Stack);
%mend Stack;


/*---create a tall data set from a stacked data set ---*/
%macro Stack_to_Tall(_InData=,_InTallData=InLib.&InData,_Out=,_Responses=,
	_Prefix=,_VarLists=,_LabLists=);

%ClockStart(Stack_to_Tall,_initiate=0);
/*
proc sort data=&_InData;
   by _rownum;
run;

proc transpose data=&_InData out=outd(drop=_name_ _label_)
	%if %length(&_Prefix) %then %do; prefix=&_Prefix %end; ;
	by _rownum;
	var &_Responses;
	id ColumnName;
run;

%put InData = &InData;
data &_Out;
	set InLib.&InData;
	_rownum = _n_;
	drop &_allcol_;
run;

data &_Out;
	merge &_Out outd;
	by _rownum;
	drop _rownum;
run;

*/
* This macro shall be run after %Stack; 
* Multiple responses can be applied to _Responses= and _Prefix.;
* _VarLists= shall contain name (reference) of macros containing variables
  to be applied.  The order shall be corresponding to the order in ColumnName
  in Design file.;
* _LabLists= shall contain name (reference) of macros containing labels
  to be applied.  Labels with in macro shall be separated by ||.;

proc sort data=&_InData;
   by _rownum ColumnName;
run;

%let _r_ = 1;
%let _MergeData_ = ;
%let _VarList_ = ;
%let _LabList_ = ;
%let dFlag = 1;
%*put _Responses=&_Responses;
%do %while(%length(%scan(&_Responses,&_r_)));
   %let _Resp_= %scan(&_Responses,&_r_);
	%if %length(&_VarLists) %then %do;
	   %let _tmp_ = %scan(&_VarLists,&_r_); 
	   %let _VarList_ = &&&_tmp_;
	%end;
	%if %length(&_LabLists) %then %do;
	   %let _tmp_ = %scan(&_LabLists,&_r_); 
	   %let _LabList_ = &&&_tmp_;
	%end;
   %let _tid_ = %sysfunc(open(&_Indata));
   %let _lab_ = %sysfunc(varlabel(&_tid_,%sysfunc(varnum(&_tid_,&_Resp_))));
   %let _tid_ = %sysfunc(close(&_tid_));
   %if %length(&_lab_) %then %let _dlab_ = _label_;
   %else %let _dlab_ = ;
%*put _InData=&_InData _VarList_=&_VarList_ _LabList_=&_LabList_ _nTallCol_=&_nTallCol_;
*option notes;
   proc transpose data=&_InData out=outd&_r_(drop=_name_ &_dlab_)
	   %if %length(%scan(&_Prefix,&_r_)) %then %do; prefix=%scan(&_Prefix,&_r_) %end; ;
	   by _rownum;
	   var &_Resp_;
      id ColumnName;
   run;

   * renaming; 
   %if %length(&_VarList_) | %length(&_LabList_) %then %do;
      proc datasets library=work nolist;
         modify outd&_r_;
         %do _i_ = 1 %to &_nTallCol_;
            %let _InVar_ = %scan(&_Prefix,&_r_)&&_ColName_&_i_._;
				%if %length(&_LabList_) %then %do;
				   Label &_InVar_ = %scan(&_LabList_,&_i_,'||');
				%end;
		      %if %length(&_VarList_) %then %do;
               %let _OutVar_ = %scan(&_VarList_,&_i_);
               %if %upcase(&_OutVar_) ne %upcase(&_InVar_) %then %do;
                  rename &_InVar_ = &_OutVar_;
               %end;
            %end;
         %end; 
      quit;
   %end;
	%let _MergeData_ = &_MergeData_ outd&_r_;
	%let _r_ = %eval(&_r_+1);
%end;

%put InTallData = &_InTallData;
data &_Out;
	set &_InTallData;
	_rownum = _n_;
	keep &_KeepVar_ _rownum;
run;

data &_Out;
	merge &_Out &_MergeData_;
	by _rownum;
	drop _rownum;
run;

%ClockStop(Stack_to_Tall);
%mend Stack_to_Tall;


* copy a file from one location to one or more others via SAS file library refs;
* the refs must be created previously via filename statements;
%macro CopyFile(_InFileRef,_OutFileRefs,_File);
	data _null_;
		infile &_InFileRef.("&_File");
		input;
		%let i=1;
		%do %while(%length(%scan(&_OutFileRefs,&i,' ')));
			file %scan(&_OutFileRefs,&i,' ')("&_File");
			put _infile_;
			%let i=%eval(&i+1);
		%end;
	run;
%mend CopyFile;


* PipeLine Macro; 
* Copy setting file (_File) from _Inpath to _Outpath; 
%macro CopySettingFile(_InPath,_OutPath,_File,_StackFlag=1);
filename _cFile "&_OutPath.&_File..sas";
%let _mLen_ = 32767; 
%if &_StackFlag = 1 %then %do;
	data _null_;
		infile "&_InPath.&_File..sas" truncover lrecl=&_mLen_;
		length x $&_mLen_;
		input x 1-&_mLen_;
		if upcase(substr(left(x),1,12)) = '%LET OUTPATH' then x = '%let OutPath=%nrbquote('||"&_OutPath"||');';
		file _cFile lrecl=&_mLen_;
		put x;
	run; 
%end;
%else %do;
	%let d = %sysfunc(fdelete(_cFile));
%end;
%mend CopySettingFile;


%macro ExitSDSProcess(_outpath, _processname, _syserr, _exiterror, _clockrunning);
	%put syserr = &_syserr;
	%put exiterror = &_exiterror;
	%if (&_syserr = 3 | &_syserr > 4 | &_exiterror) %then %put ERROR: &_processname exited due to errors.;
	%if (&_clockrunning) %then %do;
		%let clockend = 1;
		%let notes = " ";
		%let section = " ";
		%if %symexist(CVing) %then %do;
			%if &ProcessName ^= CrossValidationModelComparison and 
				&ProcessName ^= TestSetModelComparison and
				&ProcessName ^= LearningCurveModelComparison and
				&ProcessName ^= DepVarType
				%then %do;
				%let clockend = 0;
				%if %symexist(CurrentSetting) %then %do;
					%let notes = "&CurrentSetting";
				%end;
				%if %symexist(iter) %then %do;
					%let section = "&iter";
				%end;
			%end;
		%end;
		%ClockStop(&_processname,_end=&clockend,_notes=&notes,_section=&section);
	%end;
	%if &SASMode ne SDD %then %do;
		%*PackagePublish(&OutPath,to_archive);
		%PackageEnd;		
		%ExitAndRetrieveLogLstFromJMP(&_outpath, &_processname);
	%end;
	%if ^%symexist(CVing) or 
		&ProcessName = CrossValidationModelComparison or
		&ProcessName = TestSetModelComparison or
		&ProcessName = LearningCurveModelComparison %then %do;
		%RemoveGlobalMacros;
	%end;
%mend ExitSDSProcess;


* Remove Global SAS macros;
%macro RemoveGlobalMacros(_RemoveVars=,_KeepVars=);

	* Clean up Work Library first;
	%CleanUpWorkLibrary;

	%let _KeepVars = &_KeepVars "AFLG" "FSPEC" "AFFVALUE" "WORKFLOWRP" "EXITERROR" "_RUN_FROM_WFB_" "AFFNUM";
	%global _JL_StudyName;
	%let _JL_StudyName = &StudyName; * keep study name for used for Workflow.;
	data MacroVars(keep=name);
		set sashelp.vmacro;
  		where scope="GLOBAL" & Name Ne: 'SYS' & Name ne: "_JL_" & Name ne: "_JG_" & Name ne: "_JC_" & Name ne: "FM_ERROR";
	run; 
	data _null_;
		set MacroVars;
		%if %length(&_RemoveVars)%then %do;
			if upcase(name) in (%upcase(&_RemoveVars)) & symexist(name) then _dFlag_ = 1;
		%end;
		%else %do;
			_dFlag_ = 1;
		%end;
		%if %length(&_KeepVars)%then %do;
			if upcase(name) in (%upcase(&_KeepVars)) then _dFlag_ = 0;
		%end;
		if _dFlag_ then call symdel(name,"NOWARN");
	run;
	
	%do i = 1 %to 5;
		%let rc = %sysfunc(close(&i));
	%end;
	
	ods listing;
	option notes label; 
%mend RemoveGlobalMacros;


* clean up datasets in work library;
%macro CleanUpWorkLibrary;	
	data _Packaged_;
		length memname $32;
		%do i = 1 %to &nOut;
			%put OutFile&i = &&OutFile&i;
			%if %scan(&&OutFile&i,-1,.) = sas7bdat %then %do;
				memname = upcase("%scan(&&OutFile&i,1,.)");
				output;
			%end;
		%end;
		/*
		%do i = 1 %to &nStudyOut;
			%put StudyFile&i = &&StudyFile&i;
			memname = upcase("&&StudyFile&i");
			output;
		%end;
		*/
	run;
	
	data _EndOfWork_;
		set sashelp.vmember;
		if upcase(libname) = "WORK" & memtype = "DATA";
		keep memname;
	run;
	
	proc sort data=_EndOfWork_;
		by memname;
	run;
	
	%NObsVars(_Packaged_);
	%if &nobs & &nvars %then %do;
		* skip the data to be packaged;
		proc sort data=_Packaged_ nodupkey;
			by memname;
		run;
	
		data _EndOfWork_;
			merge _EndOfWork_ _Packaged_(in=_P_);
			by memname;
			if _P_ then delete;
		run;	
	%end;
		
	proc sort data=_BeginOfWork_;
		by memname;
	run;

	data _Extra_;
		merge _BeginOfWork_(in=_B_) _EndOfWork_(in=_E_);
		by memname;
		if (^_B_ & _E_) & (memname ne: "_JL_" & memname ne: "_JG_" & memname ne: "_JC_" & memname ne: upcase("&InDataNew"));
		* reserve prefixes _JL_, _JC_, and _JG_ for storing information using in cross APs process; 
	run;
	
	%let _nData_ = 0;
	data _null_;
		set _Extra_ end=_e_;
		name = "_DataToRemove_"||trim(left(_n_));
		call symput(name,memname);
		if _e_ then call symput("_nData_",_n_);
	run;
	
	%if &ProcessName = BamInputEngine or &ProcessName = SamInputEngine %then %do;
		%let _DataToRemoveList_=;
		%do i = 1 %to &_nData_;
			%if %length(&_DataToRemoveList_) < 65500 %then %do;
				%let _DataToRemoveList_=&_DataToRemoveList_ %unquote(&&_DataToRemove_&i);
			%end;
			%else %do;
				%RemoveData(&_DataToRemoveList_);
				%let _DataToRemoveList_=%unquote(&&_DataToRemove_&i);
			%end;
		%end;
		%RemoveData(&_DataToRemoveList_);
	%end;
	%else %do;
		%RemoveData(
			%do i = 1 %to &_nData_;
				%unquote(&&_DataToRemove_&i) 
			%end;
		);
	%end;
%mend CleanUpWorkLibrary;


* for a numeric variable, check if it is binary;
* returns: 0=all missing, 1=a single value, 2=two values, 3=three or more values;
%macro CheckBinary(ds,v,nv);
	%global nvalue;
	data tmp;
		set &ds end=_last;
		retain _v1 . _v2 . _v3 .;
		if not missing(&v) then do;
			if missing(_v1) then _v1 = &v;
			else if missing(_v2) then do;
				if &v ne _v1 then _v2 = &v;
			end;
			else if missing(_v3) and (&v ne _v1) and (&v ne _v2) then do;
				_v3 = &v;
				output;
				stop;
			end;
		end;
		if _last then output;
		keep _v1-_v3;
	run;
	data tmp;
		set tmp;
		nv = 3 - nmiss(of _v1-_v3);
		call symput("nvalue",trim(left(nv)));
	run;
%mend;

* for a character variable, check if it is binary;
* returns: 0=all missing, 1=a single value, 2=two values, 3=three or more values;
%macro CheckBinaryChar(ds,v,_len=8);
	%global nvalue cval1 cval2;
	data tmp;
		length _v1 _v2 _v3 $&_len;
		set &ds end=_last;
		retain _v1  _v2  _v3 '';
		if not missing(&v) then do;
			if missing(_v1) then _v1 = &v;
			else if missing(_v2) then do;
				if &v ne _v1 then _v2 = &v;
			end;
			else if missing(_v3) and (&v ne _v1) and (&v ne _v2) then do;
				_v3 = &v;
				output;
				stop;
			end;
		end;
		if _last then output;
		keep _v1-_v3;
	run;
	data tmp;
		set tmp;
		nv = 3 - missing( _v1) - missing(_v2) - missing(_v3);
		call symput("nvalue",trim(left(nv)));
		call symput("cval1",strip(_v1));
		call symput("cval2",strip(_v2));		
	run;
%mend;

* create a list of variables separated by spaces, used for list-style specifications of variables;
* warning: for long lists the macro length limit of 65324 can be exceeded;
%macro ExpandVars(ds=,varspec=);

	data a;
		set &ds(obs=1);
	run;

	proc transpose data=a out=at;
		var &&&varspec;
	run;

	%if &syserr %then %do;
		%let exiterror = 1;
		%goto evexit;
	%end;

	%NObsVars(at);
	%let nvars = &nobs;

	%do i=1 %to &nvars; 
		%global _ev&i;
	%end; 

	data _null_;
		set at;
		length is$ 16;
		is = "_ev" || trim(left(_n_));
		call symput(is,trim(left(_name_)));
	run;

	%let &&varspec = ;
	%do i=1 %to &nvars;
		%let &&varspec = &&&varspec &&_ev&i;
	%end;

	%if &syserr %then %do;
		%let exiterror =1 ;
		%goto evexit;
	%end;

	%evexit:;

%mend;

/*---macro to create various macro variables corresponding to a set of grouping variables---*/
%macro GroupMacroVars(data=,vars=,prefix=,maxnamelen=);

	%global &prefix.Effect &prefix.Name &prefix.Con &prefix.Num;

	proc sort data=&data;
		by &vars;
	run;
	%let gvn = %scan(&vars,-1," ");
	%let i = 1;
	%do %while(%length(%scan(&vars,&i)));
		%global &prefix.Var&i;
		%let &prefix.Var&i = %scan(&vars,&i);
		%if &i = 1 %then %do;
			%let &prefix.Effect = &&&prefix.Var&i;
			%let &prefix.Name = &&&prefix.Var&i;
			%let &prefix.Con = trim(left(&&&prefix.Var&i));
		%end;
		%else %do;
			%let &prefix.Effect = &&&prefix.Effect.*&&&prefix.Var&i;
			%let &prefix.Name = &&&prefix.Name._&&&prefix.Var&i;
			%let &prefix.Con = &&&prefix.Con || " " || trim(left(&&&prefix.Var&i));
		%end;
		%let i = %eval(&i+1);
	%end;

	%if ^%length(&maxnamelen) %then %let maxnamelen = 32;
	%let namelen = %length(&&&prefix.Name);
	%if (&namelen > &maxnamelen) %then %do;
		%let &prefix.Name = %substr(&&&prefix.Name,1,&maxnamelen);
	%end;
	
	%put &prefix.Effect = &&&prefix.Effect;
	%put &prefix.Name = &&&prefix.Name;
	%put &prefix.Con = &&&prefix.Con;
	
	data _gvm1;
		set &data;
	run;

	data &data;
		set &data end=_last;
		retain &prefix.Num 0;
		by &vars;
		if (first.&gvn) then do;
			&prefix.Num = &prefix.Num + 1;
		end;
		if (_last) then do;
			call symput("&prefix.Num",trim(left(&prefix.Num)));		
		end;
	run;

	%put &prefix.Num = &&&prefix.Num;

	%do i=1 %to &&&prefix.Num;
		%global &prefix.ID&i &prefix.Num&i;
	%end;

	data _gmv2;
		set _gvm1 end=_last;
		retain &prefix.Num 0 nr 0 n&prefix.Name "&prefix.Num00000";
		nr = nr + 1;
		by &vars;
		if (first.&gvn) then do;
			&prefix.Num = &prefix.Num + 1;
			nr = 1;
			&prefix.ID = "&prefix.ID" || trim(left(&prefix.Num));
			call symput(&prefix.ID,&&&prefix.Con);
			n&prefix.Name = "&prefix.Num" || trim(left(&prefix.Num));
		end;
		if (last.&gvn) then call symput(n&prefix.Name,trim(left(nr)));
	run;

	%do i=1 %to &&&prefix.Num;
		%global &prefix.ID&i;
		%put &prefix.ID&i = &&&prefix.ID&i   &prefix.Num&i = &&&prefix.Num&i;
	%end;

	%do i=1 %to &&&prefix.Num;
		%do k=1 %to &&&prefix.Num&i;
			%global &prefix.&i.R&k;
		%end;
	%end;

	data _gmv3;
		set _gvm1 end=_last;
		retain &prefix.Num 0 nr 0 n&prefix.Name "&prefix.Num100";
		nr = nr + 1;
		by &vars;
		if (first.&gvn) then do;
			&prefix.Num = &prefix.Num + 1;
			nr = 1;
		end;
		cname = "&prefix" || trim(left(&prefix.Num)) || "R" || trim(left(nr));
		call symput(cname,trim(left(ColumnName)));
	run;

	%do i=1 %to &&&prefix.Num;
		%do k=1 %to &&&prefix.Num&i;
			%let mvar = &&&prefix.&i.R&k;
			/* %put i=&i k=&k varname=&mvar; */
		%end;
	%end;

	%if &syserr %then %do;
		%put ERROR: Unable to complete grouping calculations for prefix=&prefix.;
		%let exiterror = 1;
	%end;
%mend;


/*---macro to sweep a symmetric matrix within a data step, matrix is stored as an array in lower triangular form---*/
%macro Sweep(mat,start,finish,dim,work);
	
	* use zero indexing and add 1 when indexing into the matrix or work vector;
	%let dimm1 = %eval(&dim-1);
	
	do k=%eval(&start-1) to %eval(&finish-1);

		step = 0;
		pivotOff=k*(k+3)/2;

		pivot=&mat[pivotOff+1];

		pivot=1/pivot;
		mpivot=-pivot;
		&mat[pivotOff+1]=pivot;

		step = 1;
		pivotRowOff=k*(k+1)/2;
		do j=0 to (k-1);
			val=&mat[pivotRowOff+j+1];
			&work[j+1] = val;
			val = val*pivot;
			&mat[pivotRowOff+j+1] = -val;
		end;

		step = 2;
		&work[k+1]=0;
		colOff=pivotOff+k+1;

		do j=(k+1) to &dimm1;
			val=&mat[colOff+1];
			&work[j+1] = val;
			val = val*pivot;
			&mat[colOff+1] = val;
			colOff = colOff + j + 1;
		end;

		step = 3;
		rowOff=0;

		do i=0 to (k-1);
			fac=&mat[pivotRowOff+i+1];
			do j=0 to i;
				&mat[rowOff+j+1] = &mat[rowOff+j+1] - fac * &work[j+1];
			end;
			rowOff = rowOff + i + 1;
		end;

		step = 4;
		rowOff = rowOff + k + 1;
		colOff = pivotOff + k + 1;

		do i=(k+1) to &dimm1;
			fac = &mat[colOff+1];
			do j=0 to i;
				&mat[rowOff+j+1] = &mat[rowOff+j+1] - fac * &work[j+1];
			end;
			rowOff = rowOff + i + 1;
			colOff = colOff + i + 1;
		end;
	end;
	drop k step pivotOff pivot mpivot pivotRowOff val colOff rowOff fac;
%mend;

* remove a word from a blank-delimited string;
%macro RemoveWord(sntnc,wrd);
	 %let sentence=%str( )%nrbquote(&sntnc); 
	 %if &sentence^=%str( ) %then %do;
		 %let word=%str( )%nrbquote(&wrd);
		 %let answer=;
		 %let i=%index(&sentence,&word);
		 %if &i and &word^=%str( ) %then %do;
			 %if &i>1 %then %let answer=%qsubstr(&sentence,1,&i-1);
			 %let j=%eval(&i+%index(%qsubstr(&sentence,&i+1),%str( )));
			 %if &j>&i %then
			 %let answer=&answer%qsubstr(&sentence,&j);
		 %end;
		 %else %let answer=&sentence;
		 %unquote(&answer)
	 %end;
%mend RemoveWord;

%macro countvars(string);
/**************************************************************/
/* A %DO %WHILE will allow you to loop through the string and */
/* assign each 'word' into a separate macro variable.         */
/*                                                            */
/* Note:  The delimiter of the string is the 3rd argument to  */
/* the %QSCAN function.  In this sample the delimiter is a    */
/* a space.  If the delimiter was a comma, the syntax would   */
/* be:                                                        */
/*                                                            */
/*     %let var&count=%qscan(&string,&count,%str(,));         */
/**************************************************************/


  /* Intialize COUNT */
  %let count=1;

  /* Assign the first value in the string to the macro variable var1*/
  %let var&count=%qscan(&string,&count,%str( ));

  /* This loop will continue to loop as long as VAR&COUNT has a value */
  %do %while(&&var&count ne);

    /* Increment the value of COUNT for each iteration of the DO loop */
    %let count=%eval(&count+1);

    /* Create a different macro variable for each 'word' in the string */
    %let var&count=%qscan(&string,&count,%str( ));
  %end;

  /* Capture the final number of 'words' in the string */
  %if &count<22 %then %do;
		%let count=%eval(&count-1); 
	%end;
	%else %do;
		%let count = 20; * set maximum count allowed to 20;
		%put WARNING: There are more than 20 variables listed and only the first 20 ones will be kept.;
	%end;

  /* DO loop to write the values of the newly created variables to the log */
  %do x=1 %to &count;
  %end;

%mend;

%macro AddDatatoStudy(_memname,_type=wide);
	%if %symexist(StudyName) %then %do;
		%if %length(&_type) and %length(&StudyName) %then %do;
			%let nStudyOut = %eval(&nStudyOut+1);
			%global StudyFile&nStudyOut StudyType&nStudyOut;
			%let StudyFile&nStudyOut = %lowcase(&_memname);	
			%let StudyType&nStudyOut = %upcase(&_type);
		%end;
	%end;
%mend;

%macro CheckNewStudyName;

	%let numlist = 0 1 2 3 4 5 6 7 8 9;
	%if %index(&numlist, %substr(&NewStudyName, 1, 1)) %then %do;
		%put [MS_S_0204];
		%put ERROR: Study Name must not begin with a number.;
		%let exiterror = 1;
	%end;
	%else %do;
		%let symlist = %nrstr(< > * | ? / \ , : = ; $ ^);
		%do _i_ = 1 %to 17;
			%if &_i_ < 14 %then %let temp = %scan(&symlist,&_i_,' ');
			%else %if &_i_ = 14 %then %let temp = %nrstr(&);
			%else %if &_i_ = 15 %then %let temp = %nrstr(%%);
			%else %if &_i_ = 16 %then %let temp = %nrstr(%');
			%else %if &_i_ = 17 %then %let temp = %nrstr(%");
			%if %index(&NewStudyName, %nrbquote(&temp)) %then %do;
				%put [MS_S_0205];
				%put ERROR: Study Name must not contain the symbol &temp.;
				%let exiterror = 1;
				%let _i_=18;
			%end;
		%end;
	%end;
	
	* check if the new study name is already in the metadata table;
	%if ^&exiterror %then %do;
		%if &Server = Local %then %do;
			libname Userlib "&LSUSER_HOME";	
		%end;
		%else %do;
			libname Userlib "&Metapath";		
		%end;

		%let tid = %sysfunc(open(Userlib.metadata_study));
		%if &tid %then %do;
			%let nObs = %sysfunc(attrn(&tid,NOBS));
			%let nVars = %sysfunc(attrn(&tid,NVARS));

			%do _i_ = 1 %to &nVars;
				%let vname = %sysfunc(varname(&tid,&_i_)); 
				%if %qupcase(&vname)=STUDYNAME %then %do;
					%do _j_=1 %to &nObs;
						%global _stName_&_j_ ;
						%let _rc_=%sysfunc(fetchobs(&tid,&_j_));
						%let _stName_&_j_= %sysfunc(getvarc(&tid,%sysfunc(varnum(&tid,StudyName))));			
						/* %put study name =&&_stName_&_j_; */ 
						%if %qupcase(&NewStudyName)=%qupcase(&&_stName_&_j_) or %qupcase(%sysfunc(transtrn(&NewStudyName, %str( ), _))) = %qupcase(%sysfunc(transtrn(&&_stName_&_j_, %str( ), _))) %then %do;
							%put [MS_S_0253];
							%put ERROR: Study Name &NewStudyName already exists. Please choose another name or delete this study.;
							%let exiterror = 1;
							%let _i_=&nVars;
							%let _j_=&nObs;
						%end;
					%end;
				%end;
			%end;
			%let tid = %sysfunc(close(&tid));
		%end;
	%end;
%mend;

%macro AddStudyName;
	%let studynameunder = %sysfunc(transtrn(&StudyName, %str( ), _));

	data newstudy;
		length StudyName $100 SDTMFolder $200 ADaMFolder $200 OutputFolder $200 Server Type InitialDate LastUpdatedDate $25 Source $8 CDIStudyName $64 SDTMLibraryName $64 ADAMLibraryName $64;
		%let d=%sysfunc(today(),date9.);
		%let t=%sysfunc(time(),time16.);
		%let c=&d &t;
		StudyName="&StudyName";
		SDTMFolder="%quote(&InPath)";
		ADaMFolder="%quote(&AInPath)";
		%if &ProcessName = AddStudy %then %do;
			OutputFolder="&OutPath";
		%end;
		%else %do;
			OutputFolder="$HOME&_slash_.JMPG&_slash_.Output&_slash_.&studynameunder&_slash_";
		%end;
		Server = "&Server";
		Type = "Genomics";
		InitialDate="&c";
		LastUpdatedDate="&c";
		%if %symexist(TransportCheck) %then %do;
			%if &TransportCheck=Yes %then %do;
				SASTransport =1;
			%end;
			%else %do;
				SASTransport =0;
			%end;
		%end;
		%if %symexist(SnpComp) %then %do;
			%if &SnpComp=Yes %then %do;
				EnableSnapshot =1;
			%end;
			%else %do;
				EnableSnapshot =1;
			%end;
		%end;
		%if %symexist(CDIStudy) %then %do;
			Source = "CDI";
			CDIStudyName = "&cdistudyname";
			SDTMLibraryName = "";
			ADAMLibraryName = "";				
		%end;
		%else %if %symexist(metalibstudy) %then %do;
			Source = "Metadata";
			SDTMLibraryName = "&metalibsname";
			ADAMLibraryName = "&metalibaname";
		%end;
		%else %do;
			Source = "Folders";
			CDIStudyName = "";
			SDTMLibraryName = "";
			ADAMLibraryName = "";			
		%end;
	run;	

	%if &Server = Local %then %do;
		libname Userlib "&LSUSER_HOME";	
	%end;
	%else %do;
		libname Userlib "&Metapath";		
	%end;
	%let tid = %sysfunc(open(Userlib.metadata_study));
	%if &tid %then %do;
		%let tid = %sysfunc(close(&tid));
		data Userlib.metadata_study;
			set newstudy Userlib.metadata_study ;
			if missing(Source) then Source = "Folders";
		run;
	%end;
	%else %do;
		data Userlib.metadata_study;
			set newstudy; 
		run;
	%end;
	
	%CreateUserStudies;
%mend;

%macro CreateUserStudies;
	/* Update for permissions assuming data sets from metadata query still exist */
	%if &Server ne Local %then %do;
		data folders; set Userlib.metadata_study;
			where Source = "Folders";
		run;

		%if ^%sysfunc(exist(studies)) %then %do;
			%include "&MacroPath./CDIMacros.sas" / nosource;
		%end;
		
		%if ^%sysfunc(exist(metalibraries)) %then %do;
			%include "&MacroPath./GetMetaDataLibraries.sas" / nosource;
		%end;	
		
		proc sql noprint;
			create table cdistudies as
				select d2.*
				from studies(rename=(StudyName =cdis))as d1 join userlib.metadata_study as d2
				on d1.cdis = d2.cdistudyname;

			create table sdtmlibs as
				select d2.*
				from metalibraries as d1 join userlib.metadata_study as d2
				on d1.LibraryName = d2.SDTMLibraryName;

			create table adamlibs as
				select d2.*
				from metalibraries as d1 join userlib.metadata_study as d2
				on d1.LibraryName = d2.ADAMLibraryName;
		quit;

		data metalibs;
			set sdtmlibs adamlibs;
		run;

		proc sort data=metalibs nodupkey;
			by StudyName;
		run;

		data userstudies;
			set cdistudies folders metalibs;
		run;	
	%end;
%mend;


* update _processinfo if it was created in the version prior to v6.1;
%macro Update_ProcessInfo;
%if %sysfunc(exist(_LogLib_._processinfo)) %then %do;
	%CheckVarExist(ProcessNumber,_LogLib_._processinfo,,_ShowError=0);
	%if &exiterror %then %do;
		%let exiterror = 0;
		data _info_;
			set _LogLib_._processinfo;
			_c_ = _n_;
			if section = "";
		run;
		proc sort data=_info_;
			by decending _c_;
		run;
		data _info_;
			set _info_;
			ProcessNumber = _n_;
		run;
		proc sort data=_info_;
			by Process ProcessNumber;
		run;
		data _info_;
			set _info_;
			by Process ProcessNumber;
			retain ProcessNumber2;
			if first.Process then ProcessNumber2 = 1;
			else ProcessNumber2+1;
		run;
		proce sort data=_info_;
			by _c_;
		run;
		data _LogLib_._processinfo;
			set _LogLib_._processinfo;
			_c_ = _n_;
		run;
		data _LogLib_._processinfo;
			merge _info_ _LogLib_._processinfo;
			by _c_;
			drop _c_;
		run;
	%end;
%end;
%mend;

* Check which product (JMPC or JMPG) does the study belong to;
%macro Study_Product(_StudyName);
	%if (&Server = Local) %then %do;
		libname tmp "&LSUSER_HOME";
	%end;
	%else %do;
		libname tmp "&metapath";
	%end;
	%global S_Prod;
	data _null_;
		set tmp.metadata_study;
		if StudyName = "&_StudyName" then call symputx('S_Prod',Type);
	run;
	%*put S_Prod= &S_Prod;
%mend;



%macro MAKEDIR(dir = , varname =, parent = ) ; 
   %local rc fileref;
   %let rc = %sysfunc(filename(fileref,&parent.&dir)); 
   %if %sysfunc(fexist(&fileref)) %then %do;
      %put NOTE: The directory "&parent.&dir" exists;
   %end;
   %else %do; 
      %let rc = %sysfunc(dcreate(&dir, &parent));
      %if %length(&rc) %then %put NOTE: The directory &varname (&parent.&dir) has been created; 
      %else %do;
      	%put [MS_S_0202];
      	%put ERROR: Unable to create directory &parent.&dir;
      	%let CSTERROR = 1;
      %end;
   %end; 
   %if &Server ne Local %then %do;
		%global &&varname;
		%let &&varname = &parent.&dir;
		libname &&varname "&parent.&dir";
   %end;
   %else %if &Server = Local %then %do;
		%global &varname;
		%let &varname = &parent.&dir;
		libname &varname "&parent.&dir";
   %end;   
   %let rc=%sysfunc(filename(fileref)); 
%mend MAKEDIR;


* Setup libname where contain _ProcessInfo.sas7bdat and StudyLog.txt;
%macro Setup_LogLib;
	%if ^%symexist(StudyName) %then %do;
		libname _LogLib_ "&OutPath";
		%goto exit;
	%end;
	%if %length(%trim(&StudyName)) %then %do;
		%let StudyName_ = %sysfunc(translate(&StudyName,%str(_),%str( )));
		%if (&Server = Local) %then %do;
			%Study_Product(&StudyName);
			%let _tmp_ = %substr(&LSUSER_HOME,1,%length(&LSUSER_HOME)-6);
			%if (%quote(&S_Prod) = Genomics) or &ProcessName=AddStudy  %then %do;
				* JMPG;
				%let _mpath_ = &_tmp_.JMPG/;
			%end;
			%else %if (%quote(&S_Prod) = Clinical) %then %do;
				* JMPC;
				%let _mpath_ = &_tmp_.JMPC/;
			%end;	
			%else %do;
				%let _mpath_ = &_tmp_.JMPG/;
			%end;
		%end;
	%end;
	%if (&Server = Local) %then %do;
		%if %length(%trim(&StudyName)) %then %do;
			%if &ProcessName=AddStudyLocal or &ProcessName=AddStudy or &ProcessName=CombineStudies %then %do;		
				%let newdir = %sysfunc(dcreate(&StudyName_,&_mpath_.MetaData));
			%end;	
			Libname _LogLib_ "&_mpath_.MetaData&_slash_.&StudyName_";
		/*
			%if ^(&ProcessName=ManageStudies) %then %do;
				* 1. Running on Local with specified Study;
				Libname _LogLib_ "&metapath.MetaData&_slash_.&StudyName_";
			%end;
			%else %do;
				%if ^(%quote(&Task)=Remove Study) %then %do;
					Libname _LogLib_ "&metapath.MetaData&_slash_.&StudyName_";
					%if (%quote(&Task)=Rename Study) %then %do;
						%let NewStudyName_ = %sysfunc(translate(&NewStudyName,%str(_),%str( )));
						Libname _NewLib_ "&metapath.MetaData&_slash_.&NewStudyName_";
					%end;
				%end;
				%else %do;
					Libname _LogLib_ "&metapath.Notes&_slash_.&StudyName_";
					libname _tmp_ "&metapath.MetaData&_slash_.&StudyName_";
					data _logLib_._processinfo;
						set _tmp_._processinfo;
					run;				
					filename inf "%sysfunc(pathname(_tmp_))&_slash_.StudyLog.txt";
					filename outf "%sysfunc(pathname(_LogLib_))&_slash_.StudyLog.txt";
					data _null_;
						infile inf;
						file outf;
						input;
						put _infile_;
					run;
				%end;
			%end;
		*/
		%end;
		%else %do;
			* 2. Running on Local without specified Study;
			libname _LogLib_ "&OutPath";
		%end;
	%end;
	%else %do;
		%if %length(%trim(&StudyName)) %then %do;
			%MAKEDIR(dir=MetaData, varname = MD, parent = &Metapath);
			* 3. Running on Server with specified Study;
			%if &ProcessName=AddStudyLocal OR &ProcessName=AddStudy %then %do;	
				%let newdir = %sysfunc(dcreate(&StudyName_,&metapath.Metadata));
			%end;		
			libname _LogLib_ "&metapath.Metadata&_slash_.&StudyName_";			
			/*
			%if ^(&ProcessName=ManageStudies) %then %do;
				* 3. Running on Server with specified Study;
				libname _LogLib_ "&metapath.Metadata&_slash_.&StudyName_";
			%end;
			%else %do;
				%if ^(%quote(&Task)=Remove Study) %then %do;
					libname _LogLib_ "&metapath.Metadata&_slash_.&StudyName_";
					%if (%quote(&Task)=Rename Study) %then %do;
						%let NewStudyName_ = %sysfunc(translate(&NewStudyName,%str(_),%str( )));
						Libname _NewLib_ "&metapath.Metadata&_slash_.&NewStudyName_";
					%end;
				%end;
				%else %do;
					* 3.1 Copy _ProcessInfo and StudyLog to "Note" folder before metadata was removed;
					Libname _LogLib_ "&metapath.Notes&_slash_.&StudyName_";
					libname _tmp_ "&metapath.Metadata&_slash_.&StudyName_";
					data _logLib_._processinfo;
						set _tmp_._processinfo;
					run;
				
					filename inf "&metapath.Metadata&_slash_.&StudyName_.&_slash_.StudyLog.txt";
					filename outf "%sysfunc(pathname(_LogLib_))&_slash_.StudyLog.txt";
					data _null_;
						infile inf;
						file outf;
						input;
						put _infile_;
					run;
				%end;
			%end;
			*/
		%end;
		%else %do;
			* 1. Running on Server without specified Study;
			libname _LogLib_ "&ServerOutPath";
		%end;
	%end;
%exit:; 
%mend;	

* Create/Append output file (&_File) information into _ProcessInfo2;
* This is applied during packaging (%PackageInsertDataset,%PackageInsertHTML,
* %PackageInsertFile, and %PackageEnd with processing one file at a time;
%macro ResultMeta(_File,_pName=&ProcessName,_pNum=&_nProj_,_pNum2=&_nThePro_,_OPath=&OutPath);
	%if ^%sysfunc(exist(_ProcessInfo2)) %then %do;
		%let _nFile_ = 1;
		data _ProcessInfo2;
			format Process $32. File $64. OutPath $264.; 
			Process = "&_pName";
			ProcessNumber = &_pNum + 0;
			ProcessNumber2 = &_pNum2 + 0;
			File = "&_File";
			FileNumber = 1;
			OutPath = "&_OPath";
			output;
		run;
	%end;
	%else %do;
		data _null_;
			set _ProcessInfo2 end = _e_;
			if _e_ then call symput('_nFile_',_n_);
		run;
		data p2;
			format Process $32. File $64.; 
			Process = "&_pName";
			ProcessNumber = &_pNum + 0;
			ProcessNumber2 = &_pNum2 + 0;
			File = "&_File";
			FileNumber = %eval(&_nFile_+1);
			OutPath = "&_OPath";
			output;
		run;
		data _ProcessInfo2;
			set _ProcessInfo2 p2;
		run;
	%end;
%mend;


* Initialize SAS-side JMPLS translation functions by reading translation source file into a SAS Data Set;
%macro LocaleInit();
	%if ^%sysfunc(exist(_JL_TransLookup)) or ^%symexist(_JL_TransLookupIndex) or ^%symexist(_JL_TransLookupIndexKL) or
	 ^%symexist(_JL_TransLookupLangs) %then %do;
		%global _JL_TransLookupIndex _JL_TransLookupIndexKL _JL_TransLookupLangs;
		%local TransSourceFile TrashLog _pathslash_ _workpath_ TransSourceFileLoc TrashLogLoc;
		%local _tuicount_ _numRows_ patt_tui patt_src patt_att patt_tue max_TUI max_STRSB max_STRDB;
		%let TransSourceFile = %str(Common.xlf);
		%let TrashLog = %str(Trash.log);
		%if &sysscp = WIN %then 
			%let _pathslash_ = \;
		%else
			%let _pathslash_ = /;
		%let _workpath_ = %sysfunc(pathname(work));
		%let TrashLogLoc = &_workpath_.&_pathslash_.&TrashLog;
		%if ^%symexist(CLIENT_LSUSER_HOME) %then %do;
			%put %str(WARN)%str(ING: CLIENT_LSUSER_HOME has never been defined, so &TransSourceFile cannot be located.);
			%let TransSourceFileLoc = &TransSourceFile;
		%end;
		%else
			%let TransSourceFileLoc = &CLIENT_LSUSER_HOME&TransSourceFile;
		%if %symexist(Server) %then %do;
			%if (&Server ^= Local) %then %do;
				%let TransSourceFileLoc = &_workpath_.&_pathslash_.&TransSourceFile;
			%end;
		%end;
		%else %do;
			%put %str(WARN)%str(ING: Server variable has never been defined, so assuming local mode.);
		%end;
		%if ^%sysfunc(fileexist(&TransSourceFileLoc)) %then
			%put %str(WARN)%str(ING: Cannot find &TransSourceFileLoc, so the _JL_TransLookup data set cannot be created.);
		%else %do;
			%let _JL_TransLookupLangs = %str(tui~en/*~de~es~fr~it*/~ja~zh);
			%let patt_tui = %nrstr(^\s*?<trans-unit id=%"\s*(.*?)\s*%" restype=%")%nrstr(x-misc%">\s*?$);
			%let patt_src = %nrstr(^\s*?<source>\s*(.*?)\s*<\/source>\s*?$);
			%let patt_att = %nrstr(^\s*?<alt-trans alttranstype=%")%nrstr(reference%" xml:lang=%"([\w-]{2,5})%">);
			%let patt_att = &patt_att%nrstr(<target state=%")%nrstr(translated%">\s*(.*?)\s*<\/target><\/alt-trans>\s*?$);
			%let patt_tue = %nrstr(^\s*?<\/trans-unit>\s*?$);
			%let _tuicount_ = 0;
			%let max_TUI = 0;
			%let max_STRSB = 0;
			proc printto log="&TrashLogLoc"; run;
			data _null_;
				infile "&TransSourceFileLoc" lrecl=1024 dlm = '0D0A'x truncover encoding = "utf-8";
				length WholeLine $2048;
				input WholeLine $;
				if prxmatch("/&patt_tui/", WholeLine) then do;
					call symputx('_tuicount_',symget('_tuicount_') + 1);
					call symputx('max_TUI', max(length(prxchange("s/&patt_tui/$1/", 1, WholeLine)), symget('max_TUI')));
				end;
				else if prxmatch("/&patt_src/", WholeLine) then
					call symputx('max_STRSB', max(length(prxchange("s/&patt_src/$1/", 1, WholeLine)), symget('max_STRSB')));
			run;
			%let max_STRSB = %eval(&max_STRSB + 50);
			%let max_STRDB = %eval(&max_STRSB * 2);
			%let _JL_TransLookupIndex =;
			%let _JL_TransLookupIndexKL = %sysfunc(min(5, %sysevalf((32767 / (&_tuicount_ + 1)) - 1, floor)));
			%if &_JL_TransLookupIndexKL < 1 %then %do;
				proc printto; run;
				%put %str(WARN)%str(ING: There are more trans units (&_tuicount_) than can be indexed (%sysevalf(32767 / 2 - 1, floor)).);
				proc printto log="&TrashLogLoc"; run;
				%let _JL_TransLookupIndexKL = 1;
			%end;
			data _JL_TransLookup;
				infile "&TransSourceFileLoc" lrecl=1024 dlm = '0D0A'x truncover encoding = "utf-8";
				length WholeLine $%eval(&max_STRDB + %length(&patt_att)) tui $&max_TUI lang $5;
				length source $&max_STRSB /*tr_de $&max_STRSB tr_es $&max_STRSB tr_fr $&max_STRSB tr_it $&max_STRSB*/;
				length tr_ja $&max_STRDB tr_zhCN $&max_STRDB tstr $&max_STRDB;
				input WholeLine $;
				WholeLine = tranwrd(WholeLine, '&lt;', '<');
				if prxmatch("/&patt_tui/", WholeLine) then do;
					tui = prxchange("s/&patt_tui/$1/", 1, WholeLine);
					tui = prxchange('s/\W//', -1, tui);
					call symputx('_JL_TransLookupIndex',catx("~",symget('_JL_TransLookupIndex'),substr(trim(tui),1,symget('_JL_TransLookupIndexKL'))));
				end;
				else if prxmatch("/&patt_src/", WholeLine) then
					source = prxchange("s/&patt_src/$1/", 1, WholeLine);
				else if prxmatch("/&patt_att/", WholeLine) then do;
					lang = prxchange("s/&patt_att/$1/", 1, WholeLine);
					tstr = prxchange("s/&patt_att/$2/", 1, WholeLine);
					select (lang);
						/*when ('de') tr_de = tstr;
						when ('es') tr_es = tstr;
						when ('fr') tr_fr = tstr;
						when ('it') tr_it = tstr;*/
						when ('ja') tr_ja = tstr;
						when ('zh-CN') tr_zhCN = tstr;
						otherwise;
					end;
				end;
				else if prxmatch("/&patt_tue/", WholeLine) then do;
					call missing(tui, source/*, tr_de, tr_es, tr_fr, tr_it*/, tr_ja, tr_zhCN, lang, tstr);
				end;
				if lang ^= 'zh-CN' then delete;
				retain _all_;
				drop WholeLine lang tstr;
			run;
			proc printto; run;
			%if &_tuicount_ < 2500 %then %put %str(WARN)%str(ING: &TransSourceFile contains only &_tuicount_ trans units.);
		%end;
	%end;
%mend;


* Retrieve a translated string in the current language, by providing a trans-unit id;
* Returns the translated string;
%macro LocaleGet(_itui, _language=%sysfunc(getoption(locale)), _allquote=%str(d), _allcase=%str(u));
	%local _lang ts tid nObs nVars startRow _i_ _j_ _rc_ _tui_ tempLang _tuicol _langcol tstemp quotechar;
	%let _lang = %lowcase(%substr(&_language,1,2));
	%*put LocaleGet language is &_lang;
	%let ts = &_itui;
	%*Open translation lookup table;
	%let tid = %sysfunc(open(_JL_TransLookup));
	%if &tid %then %do;
		%*Get table dimensions;
		%let nObs = %sysfunc(attrn(&tid,NOBS));
		%let nVars = %sysfunc(attrn(&tid,NVARS));
		%if &nObs < 1 %then
			%put %str(WARN)%str(ING: _JL_TransLookup contains no trans units.);
		%else %if &nVars < 2 %then
			%put %str(WARN)%str(ING: _JL_TransLookup contains no languages.);
		%*Table meets minimum dimensions;
		%else %do;
			%*Create language column index if it does not already exist;
			%if ^%symexist(_JL_TransLookupLangs) %then %do;
				%put %str(WARN)%str(ING: _JL_TransLookupLangs did not exist. Re-creating it.);
				%global _JL_TransLookupLangs;
				%do _i_ = 1 %to &nVars;
					%let tempLang = %sysfunc(varname(&tid,&_i_));
					%if %upcase(&tempLang) = TUI %then %let tempLang = tui;
					%else %if %upcase(&tempLang) = SOURCE %then %let tempLang = en;
					%else %if %upcase(%substr(&tempLang, 1, 3)) = TR_ %then %let tempLang = %lowcase(%substr(&tempLang, 4, 2));
					%else %let tempLang = %str();
					%if %length(&tempLang) %then %do;
						%if ^%length(&_JL_TransLookupLangs) %then %let _JL_TransLookupLangs = &tempLang;
						%else %let _JL_TransLookupLangs = &_JL_TransLookupLangs%str(~)&tempLang;
					%end;
				%end;
			%end;
			%*Utilize language column index;
			%let _tuicol = %sysfunc(findw(~&_JL_TransLookupLangs~,tui,~,E));
			%let _langcol = %sysfunc(findw(~&_JL_TransLookupLangs~,&_lang,~,E));
			%if ^&_langcol %then %let _langcol = %sysfunc(findw(~&_JL_TransLookupLangs~,en,~,E));
			%if &_tuicol < 1 %then
				%put %str(WARN)%str(ING: Trans unit id column was not found in _JL_TransLookup.);
			%else %if &_langcol < 1 %then
				%put %str(WARN)%str(ING: Language column was not found in _JL_TransLookup.);
			%else %if &_tuicol > &nVars %then
				%put %str(WARN)%str(ING: Index for trans unit id column exceeds total columns in _JL_TransLookup.);
			%else %if &_langcol > &nVars %then
				%put %str(WARN)%str(ING: Index for language column exceeds total columns in _JL_TransLookup.);
			%*Required columns are present, and language index is valid;
			%else %do;
				%*If a row index has been initialized, use it to quickly start at approximate row;
				%if %symexist(_JL_TransLookupIndex) and %symexist(_JL_TransLookupIndexKL) %then %do;
					%*Prevent tuis shorter than lookup index key length from trying to use the index;
					%if %length(&_itui) < &_JL_TransLookupIndexKL %then %do;
						%put %str(WARN)%str(ING: Trans unit id (&_itui) length (%length(&_itui)) failed index minimum check (&_JL_TransLookupIndexKL).);
						%let startRow = 1;
					%end;
					%else %do;
						%let startRow = %sysfunc(findw(~&_JL_TransLookupIndex~,%qsubstr(&_itui,1,&_JL_TransLookupIndexKL),~,E));
						%*If tui could not be found in the index;
						%if &startRow = 0 %then %do;
							%*No point in searching for it -- go to end of table;
							%if &_JL_TransLookupIndexKL > 1 %then %let startRow = %eval(&nObs + 1);
							%*Unless the lookup index key length is indicative of a possibly full index, in which case a slow search can still be performed;
							%else %let startRow = 1;
						%end;
					%end;
				%end;
				%*No row index found, so must start at top of table;
				%else %do;
					%put %str(WARN)%str(ING: _JL_TransLookupIndex vars do not exist. Translation lookups will be slow. TUI supplied: )&_itui%str(.);
					%let startRow = 1;
				%end;
				%*For every row;
				%do _i_ = &startRow %to &nObs;
					%let _rc_ = %sysfunc(fetchobs(&tid,&_i_));
					%let _tui_ = %qtrim(%sysfunc(getvarc(&tid,&_tuicol)));
					%*If tui at this row matches requested tui, grab the string(s) in requested language(s) and go to end of table;
					%if %nrbquote(&_tui_) = %nrbquote(&_itui) %then %do;
						%*If only one language requested;
						%if &_lang ^= al %then %do;
							%let ts = %qktrim(%qsysfunc(getvarc(&tid,&_langcol)));
						%end;
						%*If ALL languages requested;
						%else %do;
							%let ts = ;
							%do _j_ = 2 %to %sysfunc(countw(&_JL_TransLookupLangs,~));
								%if &_j_ > &nVars %then
									%put %str(WARN)%str(ING: Index for language column exceeds total columns in _JL_TransLookup.);
								%else %do;
									%let tstemp = %qktrim(%qsysfunc(getvarc(&tid,&_j_)));
									%*Apply case modification;
									%if %upcase(&_allcase) = %str(U) %then %let tstemp = %qkupcase(&tstemp);
									%else %if %upcase(&_allcase) = %str(L) %then %let tstemp = %qklowcas(&tstemp);
									%*Apply quote modification;
									%if %upcase(&_allquote) = %str(D) %then %let quotechar = %str(%");
									%else %if %upcase(&_allquote) = %str(S) %then %let quotechar = %str(%');
									%else %let quotechar = %str();
									%*Add unique translations to list;
									%if ^%sysfunc(findw(&ts,&quotechar&tstemp&quotechar,%str(,),E Q)) %then %do;
										%if %klength(&ts) %then %let ts = &ts%str(,);
										%let ts = &ts&quotechar&tstemp&quotechar;
									%end;
								%end;
							%end;
						%end;
						%let _i_ = &nObs;
					%end;
				%end;
				%*Close translation lookup table;
				%let tid = %sysfunc(close(&tid));
			%end;
		%end;
	%end;
	%else %do;
		%*put %str(ERR)%str(OR: _JL_TransLookup data set does not exist. TUI supplied: )&_itui%str(.);
		%put %str(WARN)%str(ING: _JL_TransLookup data set does not exist. TUI supplied: )&_itui%str(.);
	%end;
	&ts
%mend;


* Retrieve all unique translations of a string, by providing a trans-unit id;
* Returns comma separated, optionally quoted, optionally cased, unique translations of a string across all languages;
* This function is useful within SAS data step IN operator for finding a match across any language;
* Specify _quote =  d to generate values surrounded by double quotes (default option);
*                   s to generate values surrounded by single quotes;
*                   n to generate values not surrounded by quotes;
* Specify _case =   u to generate uppercase values (default option);
*                   l to generate lowercase values;
*                   o to generate original values;
* CAUTION: By default, all values returned will be UPPERCASE, so be sure to compare against an UPPERCASE value;
*          By default, all values returned will be surrounded by DOUBLE QUOTES. If values inside contain;
*           double quotes themselves, or resolvable macro tokens, you should use the single quotes option instead;
%macro LocaleGetAll(_itui, _quote = %str(d), _case = %str(u));
	%LocaleGet(&_itui, _language=all, _allquote=&_quote, _allcase=&_case)
%mend;


* Retrieve trans-unit id(s), by providing a string in any language;
* Returns one or more trans-unit ids;
* Specify _retMult = 1 to return multiple trans-unit ids;
* Specify CI = 1 to retrieve based on a case-insensitive search;
%macro LocaleReverseGet(_string, _retMult = 0, CI=0);
	%local _rettui tid nObs nVars _i_ _rc_ _tui_ _j_ vname _ccs_;
	%if &CI = 1 %then %let _string = %qkupcase(&_string);
	%let _rettui =;
	%*Open translation lookup table;
	%let tid = %sysfunc(open(_JL_TransLookup));
	%if &tid %then %do;
		%*Get table dimensions and initialize foundtui flag;
		%let nObs = %sysfunc(attrn(&tid,NOBS));
		%let nVars = %sysfunc(attrn(&tid,NVARS));
		%*For every row, get the whole row;
		%do _i_ = 1 %to &nObs;
			%let _rc_ = %sysfunc(fetchobs(&tid,&_i_));
			%let _tui_ =;
			%*For every column;
			%do _j_ = 1 %to &nVars;
				%*Get the column name;
				%let vname = %sysfunc(varname(&tid,&_j_));
				%*If we found the trans-unit id column; 
				%if %qupcase(&vname) = %qupcase(tui) %then
					%let _tui_ = %qtrim(%sysfunc(getvarc(&tid,&_j_)));
				%*Otherwise, check to see if the cell content matches the input string; 
				%else %do;
					%let _ccs_ = %qktrim(%qsysfunc(getvarc(&tid,&_j_)));
					%if &CI = 1 %then %let _ccs_ = %qkupcase(&_ccs_);
					%if %nrbquote(&_ccs_) = %nrbquote(&_string) %then %do;
						%*If caller wants only one tui, save it and go to end of table;
						%if &_retMult = 0 %then %do;
							%let _rettui = &_tui_;
							%let _j_ = &nVars;
							%let _i_ = &nObs;
						%end;
						%*If caller wants all tuis, append to comma-separated string and go to end of row;
						%else %do;
							%if %nrbquote(&_rettui) = %nrbquote() %then %do;
							%end;
							%else %do;
								%let _rettui = &_rettui,;
							%end;
							%let _rettui = &_rettui&_tui_;
							%let _j_ = &nVars;
						%end;
					%end;
				%end;
			%end;
		%end;
		%let tid = %sysfunc(close(&tid));
	%end;
	%else %do;
		%*put %str(ERR)%str(OR: _JL_TransLookup data set does not exist. String supplied: )&_string%str(.);
		%put %str(WARN)%str(ING: _JL_TransLookup data set does not exist. String supplied: )&_string%str(.);
	%end;
	&_rettui
%mend;


* Exchange a string in any language with its translation in the current language;
*	To exchange using a case-insensitive key, specify the CI=1 option;
* Returns the string in current language;
* If multiple trans-unit-ids found for the input string, only the first is used;
*   Therefore, only one translation is returned;
* If no trans-unit ids found for input string, the input string is returned unaltered;
*   Therefore, this function always returns a non-empty string unless an empty string is given;
* If the input is known to be English (EIA=1 = English Input Assumption), this function can work more quickly;
*	By default, this function does not make any assumption about input language;
%macro LocaleEZX(_string, EIA=0, CI=0);
	%local _string _preretstring _retstring _stringtui EIA;
	%let _retstring = %nrbquote(&_string);
	%*If English Input Assumption is true, we can speed things up;
	%if &EIA = 1 %then %do;
		%*With English locale also true, there is nothing for this macro function to do;
		%if %upcase(%substr(%sysfunc(getoption(locale)),1,2)) = EN %then;
		%*Otherwise, use a trans-unit-id-forming heuristic to obtain the translation more quickly;
		%else %do;
			%*Replace {{^n}} with X, remove spaces and specials, and add _Text;
			%let _stringtui = %qsysfunc(prxchange(s/\{\{\^\d+\}\}/X/, -1, &_string));
			%let _stringtui = %sysfunc(prxchange(s/\W//, -1, &_stringtui));
			%let _stringtui = &_stringtui._Text;
			%*Obtain the translation for this guessed trans-unit-id;
			%let _preretstring = %nrbquote(%LocaleGet(&_stringtui));
			%*If trans-unit-id and return string differ, it indicates success, so this is our return string;
			%if %nrbquote(&_preretstring) ^= %nrbquote(&_stringtui) %then %let _retstring = %nrbquote(&_preretstring);
			%*Otherwise, there is something unusual about the trans-unit id (we have not guessed it), so we need to use the slow approach;
			%else %do;
				%let _stringtui = %nrbquote(%LocaleReverseGet(%nrbquote(&_string), CI=&CI));
				%if %nrbquote(&_stringtui) ne %str() %then %let _retstring = %nrbquote(%LocaleGet(&_stringtui));
			%end;
		%end;
	%end;
	%*Otherwise, there is no way to guess the trans-unit id based on non-English characters, so we need to use the slow approach;
	%else %do;
		%let _stringtui = %nrbquote(%LocaleReverseGet(%nrbquote(&_string), CI=&CI));
		%if %nrbquote(&_stringtui) ne %str() %then %let _retstring = %nrbquote(%LocaleGet(&_stringtui));
	%end;
	&_retstring
%mend;


* Determine if two strings are equivalent translations;
* Accepts exactly two strings;
* Returns 1 if the two strings are identical or map to the same trans-unit id;
* Otherwise, returns 0;
%macro LocaleAreEquiv(_string1, _string2);
	%local _string1 _string2 _rc _s1LRG _s2LRG _tuic1 _tuic2 i j;
	%let _rc = 0;
	%*If the two strings match exactly, they are equivalent;
	%if %nrbquote(&_string1) = %nrbquote(&_string2) %then %let _rc = 1;
	%*Only do this if the two strings dont match exactly;
	%else %do;
		%*Obtain the full list of trans-unit ids containing string 1;
		%let _s1LRG = %nrbquote(%LocaleReverseGet(&_string1, _retMult = 1));
		%let _tuic1 = %sysfunc(countw(&_s1LRG, ','));
		%*Only do this if string 1 was found in a trans-unit;
		%if &_tuic1 > 0 %then %do;
			%*Obtain the full list of trans-unit ids containing string 2;
			%let _s2LRG = %nrbquote(%LocaleReverseGet(&_string2, _retMult = 1));
			%let _tuic2 = %sysfunc(countw(&_s2LRG, ','));
			%*Only do this if string 2 was found in a trans-unit;
			%if &_tuic2 > 0 %then %do;
				%*Iterate through trans-unit ids in both lists;
				%do i = 1 %to &_tuic1;
					%do j = 1 %to &_tuic2;
						%*If the same trans-unit id can be found in both lists,;
						%* the strings are equivalent;
						%if %scan(&_s1LRG, &i, ',') = %scan(&_s2LRG, &j, ',') %then %do;
							%let _rc = 1;
							%*Go to the end of both loops;
							%let j = %eval(&_tuic2 + 1);
							%let i = %eval(&_tuic1 + 1);
						%end;
					%end;
				%end;
			%end;
		%end;
	%end;
	&_rc			
%mend;


* Fill a translated string containing placeholders (the first parameter) with filling items (subsequent parameters);
* Returns the filled string;
* CAUTION: If the filling items are empty strings or contain commas, you must use the fn= keyword parameters;
*	with each filling item wrapped in a quoting function;
%macro LocaleFill(s, f1=, f2=, f3=, f4=, f5=, f6=, f7=, f8=, f9=, f10=, f11=, f12=, f13=, f14=, f15=)/parmbuff;
	%local s stf iiba patt htf sysbuff safemode hct hci f1 f2 f3 f4 f5 f6 f7 f8 f9 f10 f11 f12 f13 f14 f15;
	%let safemode = 0;
	%let stf = %nrbquote(&s);
	%*Determine the highest-valued placeholder index (limited by safemode fn= parms) that must be filled;
	%let hct = 0;
	%let hci = 1;
	%do %while (%symexist(f&hci));
		%if %kindex(&stf, %str({{^)%eval(&hci)%str(}})) %then %do;
			%let hct = &hci;
		%end;
		%let hci = %eval(&hci + 1);
	%end;
	%*Remove parentheses from parameter buffer, while guarding against the zero-or-one-parameter case;
	%if %klength(&syspbuff) < 3 or ^%kindex(&syspbuff, %str(,)) %then %let sysbuff = %str();
	%else %let sysbuff = %qksubstr(&syspbuff, 2, %klength(&syspbuff) - 2);
	%*Remove the very first parameter (string-to-fill) from parameter buffer;
	%if %eval(%klength(&sysbuff)) > %eval(%klength(&stf) + 1) %then
		%let sysbuff = %qksubstr(&sysbuff, %eval(%klength(&stf) + 2));
	%else %let sysbuff = %str();
	%*Get the first filling-item from explicit args or parameter buffer;
	%let iiba = 1;
	%let param_&iiba = %kscan(&sysbuff,&iiba,%str(,));
	%*If first filling-item param started with f1 =, turn on safemode;
	%let patt = %nrstr(/^\s*f1\s*=/);
	%if %sysfunc(prxmatch(&patt, &&param_&iiba)) %then %do;
		%let safemode = 1;
		%let param_&iiba = %nrbquote(&&f&iiba);
	%end;
	%*For every filling-item found in explicit args or parameter buffer;
    %do %while (&iiba <= &hct);
		%*Define the hole-to-fill as {{^n}};
		%let htf = %str({{^)%eval(&iiba)%str(}});
		%*Set string-to-fill equal to itself but with one hole filled;
		%let stf = %nrbquote(%sysfunc(tranwrd(&stf, &htf, &&param_&iiba)));
		%*Get the next filling-item from explicit args or parameter buffer;
		%let iiba = %eval(&iiba+1);
		%if &safemode %then %do;
			%if %symexist(f&iiba) %then %let param_&iiba = %nrbquote(&&f&iiba);
			%else %let param_&iiba = %str();
		%end;
		%else %let param_&iiba = %kscan(&sysbuff,&iiba,%str(,));
	%end;
	&stf
%mend;


*Create ods pdf text or log lines from a long message, desired width, and line prefix;
*Wraps lines naturally (on spaces) whenever possible;
*You must do all of the initial setup (options, orientation, escapechar, etc);
* prior to running this macro function;
*Does not return a value;
%macro LocaleWrap(_string, _width = 70, _lineprefix = %nrstr(), _format = %str(PDF));
	%local _string _width _lineprefix _format SL PL lines startpos stoppos origstoppos currline;
	%let SL = %klength(&_string);
	%let PL = %klength(&_lineprefix);
	%if %eval(&_width + &PL) > 260 %then %do;
		%put %str(WARN)%str(ING: The sum of the width and line prefix exceeds 260 characters.);
	%end;
	%let lines = 1;
	%let startpos = 1;
	%let stoppos = 1;
	%*Unless the end of the input string has been reached;
	%do %while(&startpos <= &SL & &stoppos <= &SL);
		%*Bring the stop pointer up to the next width cutoff, if possible;
		%do %while(&stoppos < &SL & &stoppos < %eval(&startpos + &_width - 1));
			%let stoppos = %eval(&stoppos + 1);
		%end;
		%let origstoppos = &stoppos;
		%*Take backward steps of size 1 through the string until a space can be found;
		%do %while(%eval(&stoppos > &startpos) & %eval(&stoppos ne &SL) & %eval(%qksubstr(&_string, &stoppos, 1) ne %str( )));
			%let stoppos = %eval(&stoppos - 1);
		%end;
		%*A space could never be found, so we have no choice but to do a rough chop;
		%if (&stoppos = &startpos) %then %let stoppos = &origstoppos;
		%*Write the line out in the desired output format;
		%let currline = %nrbquote(&_lineprefix%qksubstr(&_string, &startpos, %eval(&stoppos - &startpos + 1)));
		%if %upcase(&_format) = %str(PDF) %then %do;
			ods pdf text = "&currline";
		%end;
		%else %do;
			%put &currline;
		%end;
		%*If more of the input string remains, increment line counter and prepare for next round;
		%if (&stoppos < &SL) %then %let lines = %eval(&lines + 1);
		%let startpos = %eval(&stoppos + 1);
		%let stoppos = &startpos;
	%end;
%mend;


* Translate a SAS Data Set;
* Given a SAS Data Set, translate specified column headers and column data, and write to output SAS Data Set;
* Parameters:;
*	_indata: Full path to input data set;
*	_outdata: Full path to output data set;
*	_colspecmeth:;
*		P for column position (e.g., 4) -- specify -1 for ALL;
*		N for column name (e.g., USUBJID) -- specify __ALL__ for ALL;
*		(Not yet supported:) L for column label (e.g., Unique Subject Identifier) -- specify __ALL__ for ALL;
*	_colheadtranslist: List (~ delimited) of column headers to translate, specified according to _colspecmeth;
*	_coldatatranslist: List (~ delimited) of column whose rows should be translated, specified according to;
*		_colspecmeth;
*	EIA: English Input Assumption (0 = no, 1 = yes);
*	CI: Case Insensitive (0 = no, 1 = yes);
*	_language: Forces the new language to be something other than the current system locale;
*;
* !!! WARNING: This macro function has barely been tested, abandoned for its JSL counterpart which can more;
*	easily produce SAS Data Sets in ALL of our supported languages while running in English locale. In;
*	contrast, this is not possible while running this macro function in the standard non-Unicode English SAS. !!!;
%macro LocaleSDSEZX(_indata, _outdata, _colspecmeth=%str(P), _colheadtranslist=%str(-1),
		_coldatatranslist=%str(-1), EIA=1, CI=0, _locale=%sysfunc(getoption(locale)));
	%local _indata _outdata _colspecmeth _colheadtranslist _coldatatranslist EIA CI _language tid;
	
	%*Open input data set;
	%let tid = %sysfunc(open(&_indata, , , F));
	%if &tid %then %do;
		%*Get table dimensions;
		%let nObs = %sysfunc(attrn(&tid,NOBS));
		%let nVars = %sysfunc(attrn(&tid,NVARS));
		%*put nObs = &nObs;
		%*put nVars = &nVars;
		%*Convert input lists to positions;
		%*Convert -1 and __ALL__ cases -- e.g., -1 --> 1,2,3,4,5;
		
		%do _i_ = 1 %to 2;
			%if %eval(&_i_) = 1 %then 
				%let _ColList = _colheadtranslist;
			%else %if %eval(&_i_) = 2 %then
				%let _ColList = _coldatatranslist;
			%if %index(&&&_ColList, %str(-1)) or %index(&&&_ColList, %str(__ALL__)) %then %do;
				%let &_ColList = %str();
				%do _j_ = 1 %to &nVars;
					%let &_ColList = &&&_ColList%str(&_j_);
					%if %eval(&_j_) < &nVars %then %do;
						%let &_ColList = &&&_ColList%str(~);
					%end;
				%end;			
			%end;
		%end;
		%*put _colheadtranslist = &_colheadtranslist;
		%*put _coldatatranslist = &_coldatatranslist;
		
		%*Next, convert non-position lists to position lists -- e.g., USUBJID,AGE --> 4,7;
		%if &_colspecmeth = %str(N) %then %do;
			%do _i_ = 1 %to 2;
				%if %eval(&_i_) = 1 %then 
					%let _ColList = _colheadtranslist;
				%else %if %eval(&_i_) = 2 %then
					%let _ColList = _coldatatranslist;
				%let templist = %str();
				%do _j_ = 1 %to %sysfunc(countw(&&&_ColList,~));
					%let templist = &templist%sysfunc(varnum(&tid, %scan(&&&_ColList, &_j_, ~)));
					%if %eval(&_j_) < %sysfunc(countw(&&&_ColList,~)) %then %do;
						%let templist = &templist%str(~);
					%end;
				%end;
				%let &_ColList = &templist;
			%end;
		%end;
		%*put _colheadtranslist = &_colheadtranslist;
		%*put _coldatatranslist = &_coldatatranslist;
		
		%let localechange = 0;
		%if &_locale ^= %sysfunc(getoption(locale)) %then %do;
			%let localechange = 1;			
			%let origlocale = %sysfunc(getoption(locale));
			options locale=&_locale;
		%end;
		
		%*Create output data set;
		
		data _NULL_;
			dsid = open("&_indata", , , 'F');
		run;
		
		data "&_outdata";
			set "&_indata";
			
			%*Prepare translated column labels for output data set;
			%do _i_ = 1 %to %sysfunc(countw(&_colheadtranslist,~));
				%let labelstatement = %str(label );
				%let curvarnum = %scan(&_colheadtranslist, &_i_, ~);
				%let curvarname = %sysfunc(varname(&tid, &curvarnum));
				%let curvarlabel = %sysfunc(varlabel(&tid, &curvarnum));
				%let curtransvarlabel = %LocaleEZX(&curvarlabel, EIA=&EIA, CI=&CI);
				%let labelstatement = &labelstatement%str(&curvarname = "&curtransvarlabel";);
				&labelstatement;
			%end;

			%*Prepare translated column data for output data set;
			%do _i_ = 1 %to %sysfunc(countw(&_coldatatranslist,~));
				%let assignstatement = %str();
				%let curvarnum = %scan(&_colheadtranslist, &_i_, ~);
				%let curvarname = %sysfunc(varname(&tid, &curvarnum));
				call symputx("TranslateThis", &curvarname);
				&curvarname = resolve('%LocaleEZX(&TranslateThis, EIA=&EIA, CI=&CI)');
			%end;

			%*Close input data set;
			%let tid = %sysfunc(close(&tid));
			
		run;
		
		%if &localechange %then %do;
			options locale=&origlocale;
		%end;
	%end;
	%else %do;
		%put %str(WARN)%str(ING: Could not open )%str(&_indata)%str( for input.);
	%end;
%mend;


* add double quotes around a string if they are not present;
%macro AddQuotes(string);
	%let s = &string;
	%*put string = &s;
	%let firstchar = %qsubstr(&s,1,1);
	%*put firstchar = &firstchar;
	%if %nrbquote(&firstchar) ne %str(%") %then %let s = %str(%")&s;
	%let lastchar = %qsubstr(&s,%length(&s),1);
	%*put lastchar = &lastchar;
	%if %nrbquote(&lastchar) ne %str(%") %then %let s = &s.%str(%");
	%*put string = &s;
	&s
%mend;
	
	
* Initialization macro called by all APs;
%macro UtilInit;
	* extract information of datasets in work library;
	data _BeginOfWork_;
		set sashelp.vmember;
		if upcase(libname) = "WORK" & memtype = "DATA";
		keep memname;
	run;

	options validvarname=v7;
	%global SASMode clockrunning _slash_ _oslash_ exiterror OutDataFormat AllJSLIncludedFiles upperLrecl NumLength;
	%let AllJSLIncludedFiles =;
	%let jsl_after_tabbedreport =;
	%let NumLength = 3;
	%let exiterror = 0;
	
	%global LS_Index JMPLSVersion;
	
	%if ^%symexist(LS_Product) %then %let LS_Product = %str(Life Sciences);
	%if &LS_Product=Genomics %then %do;
		%let LS_Index=2;
		%let JMPLSversion = %str(JMP Genomics 9.1);
	%end;
	%else %if &LS_Product=Clinical %then %do;
		%let LS_Index=3;	
		%let JMPLSversion = %str(JMP Clinical 7.0);		
	%end;
	%else %do;
		%let LS_Index=1;
		%let JMPLSversion = %str(JMP Life Sciences 4.91);
	%end;
	
	/* If running on a server, switch values for OutPath */
	%if ^%symexist(Server) %then %do;
		%global Server;
		%let Server = Local;
	%end;
	%global ClientOutPath ServerOutPath;
	%let ClientOutPath = &OutPath;
	%if (&Server ^= Local) %then %do;
		%let OutPath = &ServerOutPath;
	%end;
	%else %do;
		%let ServerOutPath = &OutPath;
	%end;
	%put Server = &Server;
	%put OutPath = &Outpath;
	%put ClientOutPath = &ClientOutPath;
	%put ServerOutPath = &ServerOutPath;
		
	/* Delete sasclean jsl file before new process really gets going. */
	%RemoveFile( &OutPath.sasclean.jsl );

	/*  Determine what OS we are on, so that we know which slash to look for */
	%put ;
	%put Running &JMPLSversion with SAS Version &sysver;
	%put Initializing &ProcessName;
	%put SYSSCP = &sysscp;
	
	%if &sysscp = WIN %then %do;
		%let _slash_ = \;
		%let _oslash_ = /;
	%end;
	%else %do;
		%let _slash_ = /;
		%let _oslash_ = \;
	%end;

	/**
		To provide a mechanism for killing the sas process.
		write out a temporary text file containing
		the process id, so that it may be read and terminated
		at a later time. (e.g. using taskkill /PID &SYSJOBID on Windows
		or kill -9 &SYSJOBID on linux)
	**/
	/*  no longer needed, now using JSL KillSession*/
	%if %symexist(KILLFILE) %then %do;
		data _null_;
			filename thefile "&OutPath.&KILLFILE";
			file thefile;
			%if &sysscp = WIN %then %do;
				put "taskkill /F /PID &SYSJOBID";
			%end;
			%else %do;
				put "kill -9 &SYSJOBID";
			%end;
		run;
	%end;
	%put SYSJOBID = &SYSJOBID;
	
	%global workpath;
	data _null_;
		call symput('workpath',pathname("work"));
	run;
	%put WORKPATH = &workpath;
	
	%if %symexist(GHOME) %then %do;
		%put Setting LS_HOME to GHOME = &GHOME..;
		%let LS_HOME = &GHOME;
	%end;

	/*
	%put LS_HOME= &LS_HOME;
	%put USER_HOME= &USER_HOME;
	%put LSUSER_HOME= &LSUSER_HOME;
	%put Starter_Index = &Starter_Index;
	
	%if %symexist(USER_HOME) %then %do;
		data _null_;
			filename workpath "&USER_HOME.workpath.txt";
			file workpath;
			a=pathname("work");
			put a;
		run;
	%end;
	*/
	
	/* Determine what SAS engine is running, valid values are DMS, SDD, SPS, and Batch */
	%if %symexist(_SDDPRC_)=1 %then %do;
		%let SASMode = SDD;
		%if %symexist(SDDPARMS) %then %do; 
			%let _tid_ = %sysfunc(open(&SDDPARMS));
			%let _nObs_ = %sysfunc(attrn(&_tid_,NOBS)); %put NOBS = &_nOBS_;
			%let listOfMacros = ;
			%do _j_ = 1 %to &_nObs_; 
				%let _rc_=%sysfunc(fetchobs(&_tid_,&_j_));
				%if (%sysfunc(getvarc(&_tid_,%sysfunc(varnum(&_tid_,parmtype)))) = INTABLE |
					  %sysfunc(getvarc(&_tid_,%sysfunc(varnum(&_tid_,parmtype)))) = INFILE |
					  %sysfunc(getvarc(&_tid_,%sysfunc(varnum(&_tid_,parmtype)))) = OUTFILE) &
					  %sysfunc(getvarc(&_tid_,%sysfunc(varnum(&_tid_,valtype)))) = filename %then %do;
					%let listOfMacros = &listOfMacros %sysfunc(getvarc(&_tid_,%sysfunc(varnum(&_tid_,id))));
				%end;
			%end;
			%let _tid_ = %sysfunc(close(&_tid_));
			%put Start converting file paths;
			%ConvertFilepaths(&listOfMacros);
		%end;
	%end;
	%else %do; 
		%if %symexist(_METAUSER) %then %do;
			/*
			%let SASMode = SPS;
			*/
			%let SASMode = WSS;
		%end;
		%else %if %nrquote(&SYSPROCESSNAME)=DMS Process %then %do; 
			%let SASMode = DMS;
		%end;
		%else %do;
			%let SASMode = Batch;
		%end;
		%PackageBegin(&ProcessName);	
	%end;
	%put SASMode = &SASMode;

	/* Load translation data set if it hasn't already been loaded. */
	%LocaleInit();
	
	/* specify output data format, valid values are sas7bdat and txt.
		JMP on Linux and Mac does not read sas7bdat files, jmp and xls formats are possible
		with SAS 9.2 via the new label option in proc export */
	%if &SASMode=Batch or &SASMode=DMS or &SASMode=WSS %then %do;
		%if ^%symexist(CLIENT_HOST) %then %let CLIENT_HOST = Windows;
		%if %index(&CLIENT_HOST,Win) %then %do;;
			%let OutDataFormat = sas7bdat;
		%end;
		%else %do;
			%let OutDataFormat = dat;
		%end;
		/*
		%if &sysscp = WIN %then %let OutDataFormat = sas7bdat;
		%else %let OutDataFormat = txt;
		*/
	%end;
	%else %if &SASMode=SPS %then %do;  
	
	 	/**** Don't see this defined anywhere - always Windows for now ****/
	 	%let _CLIENT = Windows;
	 	
		%if %index(&_CLIENT,Mac) %then %do;;
			%let OutDataFormat = dat;
		%end;
		%else %if %index(&_CLIENT,Win) %then %do;;
			%let OutDataFormat = sas7bdat;
		%end;
		%else %do;
			%let OutDataFormat = txt;
		%end;
	%end;
	%else %do;
		%let OutDataFormat = txt;
	%end;
	/* %let OutDataFormat = jmp; */
	%put OutDataFormat = &OutDataFormat;

	%global nOut nStudyOut;
	%let nOut = 0;
	%let nStudyOut = 0;
		
	%if %symexist(NewStudyName) %then %do; 
		%if %length(&NewStudyName) and (&processname ne ManageStudies) & (&processname ne ManageGenomicsStudy) %then %do;
			%include "&MacroPath./CSTMacros.sas" / nosource;
			%CHKSTUDYNAME;
			%if &exiterror %then %goto exit;
			%if %length(&NewStudyName) > 32 %then %do;
				%put [MS_S_0254];
				%put ERROR: The length of the study name &NewStudyName exceeds 32 characters. Please specify a study name that has no more than 32 characters.;
				%let exiterror=1;
				%goto exit;
			%end;
			* Create Study_Home folder;
			%global StudyNameUnder;
			%let StudyNameUnder = %sysfunc(translate(&NewStudyName,%str(_),%str( )));
			%if &Server=Local %then %do;
				%if &ProcessName = DataImportEngine %then %do;
					%if &Starter_Index=2 %then %let USER_HOME = %substr(&USER_HOME,1,%length(&USER_HOME)-6)%str(JMPG\);
					%else %let USER_HOME = %substr(&USER_HOME,1,%length(&USER_HOME)-6)%str(JMPG\);
				%end;
				%let mpath = %quote(&USER_HOME);
			%end;
			%else %do;
				%let mpath = &Metapath;
			%end;
			%let rc = %sysfunc(filename(fileref,&mpath.Metadata)) ; 
			%if ^%sysfunc(fexist(&fileref)) %then %do;
				%let newdir = %sysfunc(dcreate(Metadata,&mpath));
				%if ^%length(&newdir) %then %do;
					%put [MS_S_0255];
					%put ERROR: Metadata folder for study &NewStudyName could not be created. You may not have appropriate permissions to the path &mpath..;
					%let StudyName=;
					%let exiterror=1;
					%goto exit;
				%end;
			%end;
			%let mpath = &mpath.Metadata/;
			%let rc = %sysfunc(filename(fileref,&mpath.&StudyNameUnder)) ; 
			%if ^%sysfunc(fexist(&fileref)) %then %do;
				%let newdir = %sysfunc(dcreate(&StudyNameUnder,&mpath));
				%if ^%length(&newdir) %then %do;
					%put [MS_S_0255];
					%put ERROR: Metadata folder for study &NewStudyName could not be created. You may not have appropriate permissions to the path &mpath..;
					%let StudyName=;
					%let exiterror=1;
					%goto exit;
				%end;
			%end;
			%let StudyName = &NewStudyName;
			%let NewStudyName=;
			%global Study_Home;
			%let Study_Home = &mpath.&StudyNameUnder./;			

			* check if the new study name or output folder is already in the meta data table;
			libname Userlib "&Metapath";			

			%let tid = %sysfunc(open(Userlib.metadata_study));
			%if &tid %then %do;
				%let nObs = %sysfunc(attrn(&tid,NOBS));
				%let nVars = %sysfunc(attrn(&tid,NVARS));
				%put nObs =&nObs nVars=&nVars;

				%do i = 1 %to &nVars;
					%let vname = %sysfunc(varname(&tid,&i)); 
					%if %upcase(&vname) = STUDYNAME %then %do;
						%do _j_=1 %to &nObs;
							%global _stName_&_j_;
							%let _rc_=%sysfunc(fetchobs(&tid,&_j_));
							%let _stName_&_j_= %sysfunc(getvarc(&tid,%sysfunc(varnum(&tid,StudyName))));
							%put study name = &&_stName_&_j_;
							%if %str(%qupcase(&StudyName))=%str(%qupcase(&&_stName_&_j_)) %then %do;
								%put [MS_S_0256];
								%put ERROR: Study &StudyName already exists. Please choose a different study name.;
								%let exiterror = 1;
								%goto exit;
							%end;
						%end;
					%end;
					%if %upcase(&vname) = OUTPUTFOLDER %then %do;
						%do _j_=1 %to &nObs;
							%global _stName_&_j_;
							%let _rc_=%sysfunc(fetchobs(&tid,&_j_));
							%let _stName_&_j_= %sysfunc(getvarc(&tid,%sysfunc(varnum(&tid,OutputFolder))));
							%put OutPath = &&_stName_&_j_;
							%if %quote(%upcase(&OutPath))=%quote(%upcase(&&_stName_&_j_)) %then %do;
								%put [MS_S_0257];
								%put ERROR: Output Folder &OutPath is already in use by another study. Please choose a different Output Folder.;
								%let exiterror = 1;
								%goto exit;
							%end;
						%end;
					%end;
				%end;
				%let tid = %sysfunc(close(&tid));
			%end;

			* add the study name to the metadata table;
			%if &ProcessName ne AddStudyLocal & &ProcessName ne AddStudy %then %do;
				%AddStudyName;
				%global NewStudy;
				%let NewStudy = 1;		
			%end;
		%end;
	%end;

	%global _nProj_; * the sequential process running number of current process assigned in %ClockStart;
	%global _nThePro_; * the 3-digits sequential same process running number of current process assigned in %ClockStart; 

	%Setup_LogLib; 
	%Update_ProcessInfo;	* update _processinfo if it was created in the version prior to v6.1;
	%if &exiterror %then %goto exit;
	* Maximum logical record length: 1M for Win and 1G for Unix;
	%if &sysscp=WIN %then %let upperLrecl=1048575; 
	%else %let upperLrecl=1073741823; 
	
	%if %symexist(WhereStmt) %then %let WhereStmt=%unquote(&WhereStmt);
	%if %symexist(AnnoWhereStmt) %then %let AnnoWhereStmt=%unquote(&AnnoWhereStmt);
	%if %symexist(WhereClause) %then %let WhereClause=%unquote(&WhereClause);

	%if %symexist(DataInPath) %then %CheckEndSlash(DataInPath);
	%*put Outpath before slash = &Outpath;
   %CheckEndSlash(OutPath);
   %put OutPath = &OutPath;
	libname outlib "&OutPath";

	/* Start the tracking time */
	%let clockinit = 1;
	%if %symexist(CVing) %then %do;
		%if &ProcessName ^= CrossValidationModelComparison and &ProcessName ^= TestSetModelComparison 
			and &ProcessName ^= LearningCurveModelComparison and &ProcessName ^= DepVarType
			%then %let clockinit = 0;
	%end;
	%ClockStart(&ProcessName,_Initiate=&clockinit);
	%if &SASMode=SDD %then %do;
	 	options label;
	%end;
	/* Check workflow parameters */
	%if ^%symexist(Workflowrp) %then %do;
		%global Workflowrp;
		%let Workflowrp = -1;
	%end;
	%if ^%symexist(_Run_From_WFB_) %then %do;
		%global _Run_From_WFB_;
		%let _Run_From_WFB_ = 0;
	%end;
	
	%if ^%symexist(InData) %then %goto exit;

	%if %symexist(WorkFlowWhereStmt) or %symexist(WorkFlowTestNRows) %then %do;
		%PathName(&InData);
		%if &exiterror %then %do;
		%put [MS_S_0071];
   		%put ERROR: Input dataset specification is invalid.;
   		%goto exit;
		%end;
		libname InLib "&TmpPath";
		%let TestName = ;
		%AddSuffixToDataName(_wtd,&TmpName,TestName);
		%let whereFlag = 0; 
		%if %symexist(WorkFlowWhereStmt) %then %do;
			%if %length(&WorkFlowWhereStmt) %then %do;
				%let whereFlag = 1;
				data Outlib.&TestName;
					set InLib.&TmpName;
					where &WorkFlowWhereStmt;
				run; 
				%let InData = %sysfunc(pathname(Outlib))&_slash_.&TestName..sas7bdat;
			%end;
		%end;
		%if %symexist(WorkFlowTestNRows) %then %do; 
			%if %length(&WorkFlowTestNRows) %then %do;
				data Outlib.&TestName;
					%if &whereFlag %then set Outlib.&TestName;
					%else set InLib.&TmpName;;
					if _n_ > &WorkFlowTestNRows then stop;
				run; 
				%let InData = %sysfunc(pathname(Outlib))&_slash_.&TestName..sas7bdat;
			%end;
		%end;
	%end;

%exit:;
%mend UtilInit;

%UtilInit;
