/*
JMP Genomics Analytical Process

Name: Manage Study for Genomics
Author: Li Li, SAS Institute Inc.
Support: Li Li, SAS Institute Inc.
History:
   sasrdw 10Oct11 initial coding

Description: 
This process defines the default settings
for JMP Genomics. 
*/

/*
-----------------------------------------------------------
Copyright (C) 2008 SAS Institute, Inc. All rights reserved. 

Notice: 
The above copyright notice and this notice must appear in
any whole or partial copy of this code and any related
documentation. Permission to use, copy, and modify the
source code contained in this file is hereby granted, but
is limited to customers of SAS with a valid license for the
SAS software product with which this file was provided. 
Except as expressly set forth herein, the terms of such
license apply.
-----------------------------------------------------------
*/

*ProcessVariables;

*options mprint mlogic symbolgen;

%global StudyName DSInPathW WInFiles DSInPathT TInFiles DSInPathA AInFiles DSInPathE EInFiles ServerOutPath  
	 NewStudyName FileFilter DeleteStudy OutPath ServerOutPath MacroPath _slash_ CombineStudyName CombStudyName1
	 ;

*ProcessBody;

%macro ManageGenomicsStudy;

%let ProcessName=ManageGenomicsStudy;
%let FullAPName = %str(Manage Genomics Studies);

%global Study_Home;
%let Study_Home = %quote(&USER_HOME.Metadata/%sysfunc(translate(&StudyName,%str(_),%str( )))/);
%include "&MacroPath./UtilityMacros.sas" / nosource;
%include "&MacroPath./CSTMacros.sas" / nosource;

%if &syserr > 4 %then %goto exit;

filename jslmsg "&OutPath.sasmessage.jsl";

%if &sysscp = WIN %then %do;
	%let _slash_ = \;
%end;
%else %do;
	%let _slash_ = /;
%end;

%if %length(&StudyName) %then %let OutName = &StudyName;

%CHKSTUDYNAME;

%if ^%length(&NewStudyName) & ^%length(&WInFiles) & ^%length(&CombineStudyName)
	& ^%length(&TInFiles) & ^%length(&AInFiles) & ^%length(&EInFiles) & (&DeleteStudy=No | %index(&DeleteStudy,Data)) %then %do;
	
	%if  %index(&DeleteStudy,Data) & (^%length(&WInFiles) & ^%length(&TInFiles) & ^%length(&AInFiles) & ^%length(&EInFiles)) %then %do;
		%put ERROR: Please specify data set from the study in order to delete data sets.;
	%end;
	%else %do;
		%put ERROR: No actions performed.;
	%end;
	%let exiterror = 1;
	%goto exit;
%end;

%put StudyName = &StudyName;

%let SavePath = &USER_HOME;
%let metaData = metadata_study;
%let metaHome = &USER_HOME.Metadata;

*check if the new study name is already in the meta data table;
libname Userlib "&LSUSER_HOME";

%let tid = %sysfunc(open(Userlib.&metaData));
%let nObs = %sysfunc(attrn(&tid,NOBS));
%let nVars = %sysfunc(attrn(&tid,NVARS));
%put nObs =&nObs nVars=&nVars;

%global _sdtm_ _adam_ _medDra_ _outF_;

*check if the new study name already exists;
%let foundit = 0;
%do i = 1 %to &nVars;
	%let vname = %sysfunc(varname(&tid,&i)); 
	%if %upcase(&vname) =STUDYNAME %then %do;
		%do _j_=1 %to &nObs;
			%global _stName_&_j_ ;
			%let _rc_=%sysfunc(fetchobs(&tid,&_j_));
			%let _stName_&_j_= %sysfunc(getvarc(&tid,%sysfunc(varnum(&tid,StudyName))));			
			%put study name =&&_stName_&_j_;
			%if %str(%qupcase(&NewStudyName))=%str(%qupcase(&&_stName_&_j_)) %then %do;
				%put ERROR: The Study Name &NewStudyName already exists. Please choose a different study name.;
				%goto exit;
			%end;
			
			%if %str(%qupcase(&StudyName))=%str(%qupcase(&&_stName_&_j_)) %then %do;
				%let _outF_= %sysfunc(getvarc(&tid,%sysfunc(varnum(&tid,OutputFolder))));
				%let foundit = 1;
			%end;
		%end;
	%end;
%end;
%let tid = %sysfunc(close(&tid));

%if ^&foundit %then %do;
	%put ERROR: StudyName &StudyName not found in the metadata table list.;
	%put You can use Add Study Local to add a study.;
	%let exiterror = 1;
	%goto exit;
%end;

%let _outF_ = %Replace_Dollars(&_outF_);

*rename the study name to the metadata table or add new data sets to the specified study;
%if ^%index(&DeleteStudy,Delete) %then %do;
	%if %length(%trim(&NewStudyName &WInFiles &TInFiles &AInFiles &EInFiles)) %then %do;

		* rename all of the settings and output result files which are related the old study by new studyname;
		%if %length(&NewStudyName) %then %do;
		/*	%put _outF_=&_outF_;
			%FileFilter(&_outF_,studylist);

			%let dot =.;
			%let StudyName_dot = &StudyName&dot;
			data studylist;
				set studylist;
				idx = index(filename,"&StudyName_dot");
				if (idx > 0) then do;
					newName = tranwrd(filename,"&StudyName", "&NewStudyName"); 
				end;
				else delete;
			run;	

			data studylist;
				set studylist;
				_F_ = 'sasInFile'||trim(left(_n_));
				_NF_ = 'NewFileName'||trim(left(_n_));
				call symput(_F_,trim(filename));
				call symput(_NF_,trim(newName));
			run;
			%NObsVars(studylist);
			%let nStudy = &nObs;

			%CheckEndSlash(_outF_);
			%do j=1 %to &nStudy;
				%let oldfileName =&_outF_.&&sasInFile&j;
				%let newName=&_outF_.&&NewFileName&j;
				%RenameFile(&oldfileName,&newName);
			%end;
		*/
      	*** If studyname, wide datasets, tall datasets, annotation dataset, or experimental datasets change then update StudyDataSets.jmp;		
		   %let maxrc = 0;
	      %let studynameunder = %sysfunc(transtrn(&studyname, %str( ), _));
	      %let studynameunder2 = %sysfunc(transtrn(&NewStudyName, %str( ), _));
	      %let oldstudydir=&USER_HOME.Metadata&_slash_.&studynameunder&_slash_;
	      %let newstudydir=&USER_HOME.Metadata&_slash_.&studynameunder2&_slash_;
	      %put oldstudydir=&oldstudydir newstudydir=&newstudydir;
	      *rename the old study name directory folder by the new study name directory;
	      data _null_;
				rc= rename("&oldstudydir","&newstudydir",'file');
				if (rc) then put 'problems renaming folders';
			run;
      	%let rc = %sysfunc(filename(fileref,&USER_HOME.Metadata&_slash_.&studynameunder2&_slash_)) ; 
			%if &rc %then %let maxrc = &rc; 
				
			%if &maxrc %then %do;
				%let exiterror = 1;
				%goto exit;
			%end;
			%let Study_Home = %quote(&USER_HOME.Metadata/%sysfunc(translate(&NewStudyName,%str(_),%str( )))/);
		%end;

		* update new study name & new output path in meta_study table;
		%if %length(%trim(&NewStudyName)) %then %do;
			data Userlib.&metaData;
				length StudyName $32 SDTMFolder ADAMFolder OutputFolder $200 InitialDate LastUpdatedDate $25;
				set Userlib.&metaData;
				%let d=%sysfunc(today(),date9.);
				%let t=%sysfunc(time(),time16.);
				%let c=&d &t;
				if StudyName="&StudyName" then do;
					%if %length(&NewStudyName) %then %do;
						StudyName="&NewStudyName";
					%end;
					LastUpdatedDate="&c";
				end;
			run;
			%if %length(%trim(&NewStudyName)) %then %let StudyName = &NewStudyName;
		%end;
	
		data _null_;
			file jslmsg;
			put 'sasmsgwindow=New Window("JMP Genomics Message",showmenu(0),showtoolbars(0),';
			put '   VListBox(';
			put "      TextBox("" ""),";
			%if %length(&NewStudyName) %then %do;
				put "TextBox("" The Study &StudyName and its related files have been successfully updated. ""),";
			%end;

			/* update folder in user's metadata for StudyDataSets.jmp */
			%if %length(%trim(&WInFiles &TInFiles &AInFiles &EInFiles)) %then %do;
				%put Study_Home=&Study_Home;
				put "sdst =  open(""&Study_Home.StudyDataSets.jmp"",invisible);";
				put "sdst << Suppress formula eval(1);";
				put "sdst << Add Rows( " @;
				%if %length(&WInFiles) %then %do;
					%let w=1;
					%do %while(%length(%scan(&WInFiles,&w,%str(|))));
						put "{:Path=""&DSInPathW"",:Name=""%scan(%scan(&WInFiles,&w,%str(|)),1,%str(.))"",:Type=""WIDE""},"@;
						%let w = %eval(&w+1);
					%end;
				%end;
				%if %length(&TInFiles) %then %do;
					%let w=1;
					%do %while(%length(%scan(&TInFiles,&w,%str(|))));
						put "{:Path=""&DSInPathT"",:Name=""%scan(%scan(&TInFiles,&w,%str(|)),1,%str(.))"",:Type=""TALL""},"@;
						%let w = %eval(&w+1);
					%end;
				%end;	
				%if %length(&AInFiles) %then %do;
					%let w=1;
					%do %while(%length(%scan(&AInFiles,&w,%str(|))));
						put "{:Path=""&DSInPathA"",:Name=""%scan(%scan(&AInFiles,&w,%str(|)),1,%str(.))"",:Type=""ANNO""},"@;
						%let w = %eval(&w+1);			
					%end;
				%end;	
				%if %length(&EInFiles) %then %do;
					%let w=1;
					%do %while(%length(%scan(&EInFiles,&w,%str(|))));
						put "{:Path=""&DSInPathE"",:Name=""%scan(%scan(&EInFiles,&w,%str(|)),1,%str(.))"",:Type=""EDDS""},"@;
						%let w = %eval(&w+1);
					%end;
				%end;	
				put ");";
				put "sdst << Suppress formula eval(0);";
				put "Column(""Date Created"") << EvalFormula;";
				put "Column(""Date Modified"") << EvalFormula;";				
				put "sdst << Sort(By(:Data Modified), Order(Descending), Replace table);";	
				put "Close(sdst,Save(""&Study_Home.StudyDataSets.jmp""));";
			%end;
			
			%if %length(&WInFiles) %then %do;
				put "TextBox("" The &StudyName wide data sets has been successfully added. ""),";
				put "      TextBox("" ""),";
			%end;
			%if %length(&TInFiles) %then %do;
				put "TextBox("" The &StudyName tall data sets has been successfully added. ""),";
				put "      TextBox("" ""),";
			%end;
			%if %length(&AInFiles) %then %do;
				put "TextBox("" The &StudyName annotation data sets has been successfully added. ""),";
				put "      TextBox("" ""),";
			%end;
			%if %length(&EInFiles) %then %do;
				put "TextBox("" The &StudyName experimental data sets has been successfully added. ""),";
			%end;
			put ')); //end window and vlistbox';
			put "jg:_defaultexists_2 = 0;";
			put 'jg:ManageGenomicsStudy_mainDialog <<close window;';
			
			/* Update Current Study List on Genomics Starter */
			%if %length(&NewStudyName) %then %do;
				put "smd = open(jg:jmpLSUserDir[1]||""metadata_study.sas7bdat"");";
				put "jg:StudyType = Associative Array(:StudyName,:Type);";
				
				put "for each row(Selected(Rowstate())=(:Type==jg:LSprod[2]));";
				put "subdt = smd<<Subset(Output table name(""_metadata_study""));";
				put "jg:study2list = Column(""StudyName"")<<GetAsMatrix;";
				put "smd << Close Window;";
				put "setstudy = ""&NewStudyName"";";
				put "subdt << Close Window;";

				put "jg:_rb2study_ << Delete;";
				put "jg:scshlb2 << Append(";
				put " jg:_rb2study_ = ComboBox(jg:study2list, <<set script(";
				put "  jg:updateStudyStuff(2);";
				put ")));";
				put "if(!isEmpty(setstudy),jg:_rb2study_ << Set(Loc(jg:study2list,setstudy));";	
				/* Due to defect S0767103, need to manually run combobox script when setting via JSL */
				put "  jg:updateStudyStuff(2);";
				put " );";
				
			%end;
			
			%*put 'jg:ManageGenomicsStudy_updateDeps;';
			
		run;
	%end;
	
	%* combine two studies together;
	%* update combined study name in meta_study table;
	%if %length(%trim(&CombineStudyName)) %then %do;
		data Userlib.&metaData;
			length StudyName $32 SDTMFolder ADAMFolder OutputFolder $200 InitialDate LastUpdatedDate $25;
			set Userlib.&metaData end=_last_;
			%let d=%sysfunc(today(),date9.);
			%let t=%sysfunc(time(),time16.);
			%let c=&d &t;
			if _last_ then do;
				output;
				StudyName="&CombineStudyName";
				OutputFolder="%quote(&OutPath)";
				Server = "Local";
				Type = "Genomics";
				InitialDate="&c";
				LastUpdatedDate="&c";
			end;
			output;
		run;

		%let Study_Home2 = %quote(&USER_HOME.Metadata/%sysfunc(translate(&CombStudyName1,%str(_),%str( )))/);
		%let CombinedS_Home = %quote(&USER_HOME.Metadata/%sysfunc(translate(&CombineStudyName,%str(_),%str( )))/);
		
		%CheckandCreateDir(&CombinedS_Home);
		filename StudyLog "%quote(&CombinedS_Home)StudyLog.txt";	

		/*%if ^%sysfunc(fexist(StudyLog)) %then %do;
			data _null_;
				file StudyLog;
				put "Study Name = &CombineStudyName";
			run;
		%end;*/
		
		data _null_;
			file jslmsg;
			put 'sasmsgwindow=New Window("JMP Genomics Message",showmenu(0),showtoolbars(0),';
			put '   VListBox(';
			put "      TextBox("" ""),";
			%if %length(&CombineStudyName) %then %do;
				put "TextBox("" The studies &StudyName and &CombStudyName1 have been successfully combined to a new study &CombineStudyName. ""),";
			%end;

			/* update folder in user's metadata for StudyDataSets.jmp */

			%put Study_Home=&Study_Home;
			put "//Create Directory(""&CombinedS_Home"");";
			put "sdst =  open(""&Study_Home.StudyDataSets.jmp"",invisible);";
			put "sdst2 =  open(""&Study_Home2.StudyDataSets.jmp"",invisible);";
			put "both=sdst<<Concatenate(Data table(sdst2),output table name(""&CombinedS_Home.StudyDataSets.jmp""),invisible);";
			put "both << Sort(By(:Data Modified), Order(Descending), Replace table);";	
			put "Close(both, Save(""&CombinedS_Home.StudyDataSets.jmp""));";
			put "sdst2<<Close Window;";
			put "sdst<<Close Window;";
			
			put "sdst =  open(""&Study_Home.StudySettings.jmp"",invisible);";
			put "sdst2 =  open(""&Study_Home2.StudySettings.jmp"",invisible);";
			put "both=sdst<<Concatenate(Data Table(sdst2),output table name(""&CombinedS_Home.StudySettings.jmp""),invisible);";
			put "both << Sort(By(:Data Modified), Order(Descending), Replace table);";	
			put "Close(both, Save(""&CombinedS_Home.StudySettings.jmp""));";
			put "sdst2<<Close Window;";
			put "sdst<<Close Window;";

			put "try(spst =open(""&Study_Home._processinfo.sas7bdat"",invisible);";
			put "spst2 = open(""&Study_Home2._processinfo.sas7bdat"",invisible)";
			put "); //end try";
			put "if(!isempty(spst),";
			put "		if (!isempty(spst2),";
			put "			both=sdst<<Concatenate(Data Table(spst2),invisible);";
			put "			both << Sort(By(:Data Modified), Order(Descending), Replace table);";	
			put "			Close(both, Save(""&CombinedS_Home._processinfo.sas7bdat""));";
			put "			spst2<<Close Window;";
			put "			spst<<Close Window;";
			put "		,";
			put "		close (spst,save(""&CombinedS_Home._processinfo.sas7bdat""));";
			put "		);";
			put "		,";
			put "		try(spst2 = open(""&Study_Home2._processinfo.sas7bdat"",invisible));";
			put "		if (!isempty(spst2),";
			put "			close (spst2,save(""&CombinedS_Home._processinfo.sas7bdat""))";	
			put "		)"; 
			put ");";			

			put "try(sdst =  open(""&Study_Home.StudyLog.txt"",invisible));";
			put "try(sdst2 =  open(""&Study_Home2.StudyLog.txt"",invisible));";
			put "if(!isempty(spst),";
			put "		if (!isempty(spst2),";
			put "both=sdst<<Concatenate(Data Table(sdst2),output table name(""&CombinedS_Home.StudyLog.txt""),invisible);";
			put "both << Sort(By(:Data Modified), Order(Descending), Replace table);";	
			put "Close(both, Save(""&CombinedS_Home.StudyLog.txt""));";
			put "sdst2<<Close Window;";
			put "sdst<<Close Window;";
			put "		,";
			put "		close (spst,save(""&CombinedS_Home.StudyLog.txt""));";
			put "		);";
			put "		,";
			put "		try(spst2 = open(""&Study_Home2.StudyLog.txt"",invisible));";
			put "		if (!isempty(spst2),";
			put "			close (spst2,save(""&CombinedS_Home.StudyLog.txt""))";	
			put "		)"; 
			put ");";
			
			put ')); //end window and vlistbox';
			put "jg:_defaultexists_2 = 0;";
			put 'jg:ManageGenomicsStudy_mainDialog <<close window;';
			
			/* Update Current Study List on Genomics Starter */
			%if %length(&CombineStudyName) %then %do;
				put "smd = open(jg:jmpLSUserDir[1]||""metadata_study.sas7bdat"");";
				put "jg:StudyType = Associative Array(:StudyName,:Type);";
				
				put "for each row(Selected(Rowstate())=(:Type==jg:LSprod[2]));";
				put "subdt = smd<<Subset(Output table name(""_metadata_study""));";
				put "jg:study2list = Column(""StudyName"")<<GetAsMatrix;";
				put "smd << Close Window;";
				put "setstudy = ""&CombineStudyName"";";
				put "subdt << Close Window;";

				put "jg:_rb2study_ << Delete;";
				put "jg:scshlb2 << Append(";
				put " jg:_rb2study_ = ComboBox(jg:study2list, <<set script(";
				put "  jg:updateStudyStuff(2);";
				put ")));";
				put "if(!isEmpty(setstudy),jg:_rb2study_ << Set(Loc(jg:study2list,setstudy));";	
				/* Due to defect S0767103, need to manually run combobox script when setting via JSL */
				put "  jg:updateStudyStuff(2);";
				put " );";
				
			%end;
			
			%*put 'jg:ManageGenomicsStudy_updateDeps;';
			
		run;
	%end;
%end;
%else %do;

	* delete folders;
	%let maxrc = 0;
	%if (%scan(&DeleteStudy,2,' ')= Output) %then %do;
      %let studynameunder = %sysfunc(transtrn(&studyname, %str( ), _));
      %RemoveDir(&USER_HOME.Metadata&_slash_.&studynameunder&_slash_);
      %let rc = %sysfunc(filename(fileref,&USER_HOME.Metadata&_slash_.&studynameunder&_slash_)) ; 
		%if &rc %then %let maxrc = &rc; 
	%end;
	%else %if (%scan(&DeleteStudy,2,' ')= Everything) %then %do;
		%if %length(&_sdtm_) %then %do;
			%RemoveDir(&_sdtm_);
			%let rc = %sysfunc(filename(fileref,&_sdtm_)) ; 
			%if &rc %then %let maxrc = &rc;
		%end;
		%if %length(&_adam_) %then %do;
			%RemoveDir(&_adam_);
			%let rc = %sysfunc(filename(fileref,&_adam_)) ; 
			%if &rc %then %let maxrc = &rc;
		%end;
		%if %length(&_medDra_) %then %do;
			%RemoveDir(&_medDra_);
			%let rc = %sysfunc(filename(fileref,&_medDra_)) ; 
			%if &rc %then %let maxrc = &rc;
		%end;
      
      %let studynameunder = %sysfunc(transtrn(&studyname, %str( ), _));
      %RemoveDir(&USER_HOME.Metadata&_slash_.&studynameunder&_slash_);
      %let rc = %sysfunc(filename(fileref,&USER_HOME.Metadata&_slash_.&studynameunder&_slash_)) ; 
		%if &rc %then %let maxrc = &rc; 
	%end;
	%else %if (%scan(&DeleteStudy,2,' ')= Study) %then %do;
      %let studynameunder = %sysfunc(transtrn(&studyname, %str( ), _));
      %RemoveDir(&USER_HOME.Metadata&_slash_.&studynameunder&_slash_);
      %let rc = %sysfunc(filename(fileref,&USER_HOME.Metadata&_slash_.&studynameunder&_slash_)) ; 
		%if &rc %then %let maxrc = &rc; 
   %end;
	
	%if &maxrc %then %do;
		%let exiterror = 1;
		%goto exit;
	%end;

	* delete the study name from metadata table;
	%if ^%index(&DeleteStudy,Data) %then %do;
		data Userlib.&metaData;
			set Userlib.&metaData;
			if StudyName="&StudyName" then delete;
		run;
	%end;
	
	data _null_;
		file jslmsg;
		put 'sasmsgwindow=New Window("JMP Genomics Message",';
		put '   VListBox(';
		put "      TextBox("" ""),";
		%if (%scan(&DeleteStudy,2,' ')= Everything) %then %do;
			put "TextBox("" All study &StudyName folders have been successfully deleted. ""),";
		%end;
		%else %if (%scan(&DeleteStudy,2,' ')= Data) %then %do;
			/* delete dataset in user's metadata for StudyDataSets.jmp */
			%if %length(%trim(&WInFiles &TInFiles &AInFiles &EInFiles)) %then %do;
				put "sdst =  open(""&Study_Home.StudyDataSets.jmp"",invisible);";
				put "studynamePath = Column(""Path"")<<GetAsMatrix;";
				put "studynameList = Column(""Name"")<<GetAsMatrix;";
				put "studynameType = Column(""Type"")<<GetAsMatrix;";
				put "mylist ={};";
				%if %length(&WInFiles) %then %do;
					%let w=1;
					%do %while(%length(%scan(&WInFiles,&w,%str(|))));
						put "if(isList(studynameList),";
						put "for(w=1,w<=nitems(studynameList),w++, ";
						put "	if(studynameList[w]== ""%scan(%scan(&WInFiles,&w,%str(|)),1,%str(.))"",";
						put "		InsertInto(mylist,w); ";
						put "))),";
						%let w = %eval(&w+1);
					%end;
				%end;
				%if %length(&TInFiles) %then %do;
					%let w=1;
					%do %while(%length(%scan(&TInFiles,&w,%str(|))));
						put "if(isList(studynameList),";
						put "for(w=1,w<=nitems(studynameList),w++, ";
						put "	if(studynameList[w]== ""%scan(%scan(&TInFiles,&w,%str(|)),1,%str(.))"",";
						put "		InsertInto(mylist,w); ";						
						put "))),";
						%let w = %eval(&w+1);
					%end;
				%end;	
				%if %length(&AInFiles) %then %do;
					%let w=1;
					%do %while(%length(%scan(&AInFiles,&w,%str(|))));
						put "if(isList(studynameList),";
						put "for(w=1,w<=nitems(studynameList),w++, ";
						put "	if(studynameList[w]== ""%scan(%scan(&AInFiles,&w,%str(|)),1,%str(.))"",";
						put "		InsertInto(mylist,w); ";
						put "))),";
						%let w = %eval(&w+1);			
					%end;
				%end;	
				%if %length(&EInFiles) %then %do;
					%let w=1;
					%do %while(%length(%scan(&EInFiles,&w,%str(|))));
						put "if(isList(studynameList),";
						put "for(w=1,w<=nitems(studynameList),w++, ";
						put "	if(studynameList[w]== ""%scan(%scan(&EInFiles,&w,%str(|)),1,%str(.))"",";
						put "		InsertInto(mylist,w); ";
						put "))),";
						%let w = %eval(&w+1);
					%end;
				%end;	
				put "sdst <<Delete Rows(mylist);";				
				put "sdst << Sort(By(:Data Modified), Order(Descending), Replace table);";	
				put "Close(sdst, Save(""&Study_Home.StudyDataSets.jmp""));";
			%end;
			%if %length(&WInFiles) %then %do;
				put "TextBox("" The &StudyName wide data sets has been successfully deleted. ""),";
				put "      TextBox("" ""),";
			%end;
			%if %length(&TInFiles) %then %do;
				put "TextBox("" The &StudyName tall data sets has been successfully deleted. ""),";
				put "      TextBox("" ""),";
			%end;
			%if %length(&AInFiles) %then %do;
				put "TextBox("" The &StudyName annotation data sets has been successfully deleted. ""),";
				put "      TextBox("" ""),";
			%end;
			%if %length(&EInFiles) %then %do;
				put "TextBox("" The &StudyName experimental data sets has been successfully deleted. ""),";
			%end;
		%end;
		%else put "TextBox("" The study &StudyName has been deleted from the study list. ""),";
		put ')); //end window and vlistbox';
		put "jg:_defaultexists_2 = 0;";
		put 'jg:ManageGenomicsStudy_mainDialog <<close window;';
		
		/* Update Current Study List on Genomics Starter */
		put "smd = open(jg:jmpLSUserDir[1]||""metadata_study.sas7bdat"");";
		put "jg:StudyType = Associative Array(:StudyName,:Type);";

		put "for each row(Selected(Rowstate())=(:Type==jg:LSprod[2]));";
		put "subdt = smd<<Subset(Output table name(""_metadata_study""));";           
		put "jg:study2list = Insert(Column(""StudyName"")<<GetAsMatrix,"" "",1);";
		put "smd << Close Window;";
		put "setstudy = ""&StudyName"";";
		put "subdt << Close Window;";

		put "jg:_rb2study_ << Set Items(jg:study2list);";

		put "if(!isEmpty(setstudy),jg:_rb2study_ << Set(Loc(jg:study2list,setstudy));";  
		/* Due to defect S0767103, need to manually run combobox script when setting via JSL */
		put "  jg:updateStudyStuff(2);";
		put " );";						
	run;
%end;

%PackageInsertFile(SAS message from &ProcessName, &OutPath, sasmessage.jsl, , text/plain, yes);

%exit:;

%ExitSDSProcess(&OutPath, &ProcessName, &syserr, &exiterror, &clockrunning);

%mend ManageGenomicsStudy;

%macro Replace_Dollars(dstring);
	%let lh = %eval(%index(&USER_HOME,JMPG)-1);
	%let dstring = %Search_and_Replace(string=&dstring,search=$HOME,replace=%substr(&USER_HOME,1,&lh));
	%let lh = %eval(%index(&GENOMICS_HOME,Genomics)-1);
	%let dstring = %Search_and_Replace(string=&dstring,search=$JMP_HOME,replace=%substr(&GENOMICS_HOME,1,&lh));
	&dstring
%mend;

* run macro;
%ManageGenomicsStudy;