/*
JMP Genomics Analytical Process

Name: Multiple SNP-Trait Association
Author: Wendy Czika


*/

/*
-----------------------------------------------------------
Copyright (C) 2010 SAS Institute, Inc. All rights reserved. 

Notice: 
The above copyright notice and this notice must appear in
any whole or partial copy of this code and any related
documentation. Permission to use, copy, and modify the
source code contained in this file is hereby granted, but
is limited to customers of SAS with a valid license for the
SAS software product with which this file was provided. 
Except as expressly set forth herein, the terms of such
license apply.
-----------------------------------------------------------
*/

*ProcessVariables;

%global StudyName InData AlleleVars ByVars WhereStmt ListMarkerVars ListTraitVars TraitVar TraitType EventTrait RefTrait CensorVar CensorValues ClassVars 
FixedEff RandEff RandOpts ParmsStmt IntEff StrataVars  AnnoInData AnnoLabel AnnoMName AnnoGroup AnnoGroup2 AnnoLoc AnnoWeight AnnoAccession AnnoWhereStmt  
MarkerFmt GenoDelim HotellingT2 MultModel PCARegModel KernelMachine SKAT VarWts BetaAlpha BetaBeta Kernel ExcludeSingles DDFM GMethod EstMethod AccProportion NumberofComponents Listing
Conv MTC  PValAdj Tau Bandwidth Alpha OnlySig OutPath ServerOutPath OutName MacroPath MSNPTraitAssoc_OutData MSNPTraitAssocScript MSNPTraitAssoc_PET MSNPTraitAssoc_CPT MSNPTraitAssoc_PETp MSNPTraitAssoc_CPTp ;
 
*ProcessBody;

%macro MultipleSNPAssociation;

*ods html close; /*Luciano*/

%include "&MacroPath./GenMacros.sas" / nosource;

%global FullAPName;
%let FullAPName=%str(Multiple SNP-Trait Association);

%GenInit(MultipleSNPTraitAssociation,23);
%if &exiterror=1 %then %goto exit;

%if %upcase(&HotellingT2) ne YES and %upcase(&MultModel)=NO and %upcase(&KernelMachine)=NO and %upcase(&PCARegModel)=NO and %upcase(&SKAT)=NO %then %do;
	%put ERROR: You must select an analysis to perform on the Model Variables tab.;
	%let exiterror=1;
	%goto exit;
%end;

%if %length(&ListMarkerVars &AlleleVars)=0 %then %do;
 %put ERROR: Marker variables must be specified in either the Marker Variables or List-Style Specification of Marker Variables parameters.;
 %let exiterror=1;
 %goto exit;
%end;

%if %length(&ListMarkerVars &AlleleVars)=0 %then %do;
 %put ERROR: Marker variables must be specified in either the Marker Variables or List-Style Specification of Marker Variables parameters.;
 %let exiterror=1;
 %goto exit;
%end;

%if %length(&ListTraitVars &TraitVar)=0 %then %do;
 %put ERROR: One or more trait variables must be specified in either the Trait Variables or List-Style Specification of Trait Variables parameters.;
 %let exiterror=1;
 %goto exit;
%end;

%let AlleleVars = &AlleleVars &ListMarkerVars;

%let TraitVar = &TraitVar &ListTraitVars;

%PrepOutDS(MSNPTraitAssoc_OutData,MSNPTraitAssoc_OutDataName,&OutName._mst);
%PrepOutFile(MSNPTraitAssocScript,MSNPTraitAssocScriptName,&ProcessName Script for &OutName..jsl);
%PrepOutDS(MSNPTraitAssoc_PET,MSNPTraitAssoc_PETName,&OutName._mpe);
%PrepOutDS(MSNPTraitAssoc_PETp,MSNPTraitAssoc_PETNamep,&OutName._mpep);

%if %length(&RandEff) %then %do;
	%PrepOutDS(MSNPTraitAssoc_CPT,MSNPTraitAssoc_CPTName,&OutName._mcp);
	%PrepOutDS(MSNPTraitAssoc_CPTp,MSNPTraitAssoc_CPTNamep,&OutName._mcpp);	
%end;

%global ReorderingVars;
%if %length(&AnnoGroup)=0 %then %let ReorderingVars = _group_ &AnnoGroup2 &AnnoLoc;
%let ReorderingVars = &AnnoGroup &AnnoGroup2 &AnnoLoc;

%PrepMapandBy;
run;
%if &exiterror=1 %then %goto exit;

%if &nummark<2 %then %do;
	%put ERROR: Allele or genotype variables for 2 or more markers must be specified.;
	%let exiterror = 1;
	%goto exit;
%end;

%let OrigClassVars = &ClassVars;

%if %upcase(&HotellingT2) = YES %then %let HT=1;
%else %let HT=0;
%if %upcase(&KernelMachine) = YES %then %let KM=1;
%else %let KM=0;
%if %upcase(&MultModel) = YES %then %let LR=1;
%else %let LR=0;
%if %upcase(&PCARegModel) = YES %then %let PCR=1;
%else %let PCR=0;

%if &LR or &PCR %then %do;

	%let WhichProc=MIXED;
	%let Stat=F; 
	%let TTable=Contrasts;
	%let TStmt=contrast;
	%let usestrata=0;
	%if %length(&DDFM)=0 %then %let DDFM=residual;
	%let ModOption=%str(/ htype=3 ddfm=&DDFM s); 
	%let ClassOpt=;
	%let ProcOption= ;
	%let ParmEst = SolutionF;
	%let EffName = Effect;
	%let DropVar = Label;
	/* Need a separate check for OR when using GLIMMIX since not creating OR data set */
	%let AnyOdds=0;
	%if %upcase(&TraitType)=COUNT %then %do;
		%let WhichProc=GLIMMIX; 
		%let ModOption=%str(/ link=log dist=p htype=3 ddfm=&DDFM s);
		%let ParmEst = ParameterEstimates;
		%let DropVar = Statement;
	%end;
	%else %if %upcase(&TraitType)=BINARY or %upcase(&TraitType)=NOMINAL or %upcase(&TraitType)=ORDINAL %then %do;
		%let ParmEst = ParameterEstimates;
		%if %length(&RandEff)>0 %then %do;
			%let WhichProc=GLIMMIX;
			%let ProcOption=%str(Method=&GMethod);		
			%if %upcase(&EstMethod)=YES %then %let ProcOption=%quote(&ProcOption EMPIRICAL);
			
			%if %upcase(&TraitType)=BINARY %then %let ModOption=%str(/ link=logit dist=binary htype=3 ddfm=&DDFM s);
			%else %if %upcase(&TraitType)=NOMINAL %then %do;
				%let ModOption=%str(/ link=glogit dist=mult htype=3 ddfm=&DDFM s);
				%let ClassVars = &OrigClassVars &TraitVar;
			%end;
			%else %if %upcase(&TraitType)=ORDINAL %then %let ModOption=%str(/ link=clogit dist=mult htype=3 ddfm=&DDFM s);
			%let DropVar = Statement;
		%end;
		%else %do;
			%let WhichProc=LOGISTIC;
			%let Stat=ChiSq;
			%let TTable=ContrastTest;
			%let TStmt=contrast;
			%let Otable=OddsRatiosWald;
			%let usestrata=1;
			%if %upcase(&TraitType)=NOMINAL %then %let ModOption=%str(/ link=glogit);
			%else %if %length(StrataVars)>0 %then %let ModOption=/ GCONV=1E-6; 
			%else %let ModOption =;
			%let ClassOpt = %str(/ cprefix=8);
			%let EffName = Variable;
			%let DropVar = Contrast;
		%end;
	%end; 
	%else %if %upcase(&TraitType)=SURVIVAL %then %do;
		%let WhichProc=PHREG;
		%let Stat=ChiSq;
		%let TStmt=test;
		%let TTable=TestStmts;
		%let ModOption =;
		%let usestrata=1;
		%let ParmEst = ParameterEstimates;  
		%let EffName = Parameter;
		%let DropVar = Status Label;
	%end;
	%if &usestrata=0 %then %let StrataVars=;
%end;
%if &PCR %then %do;
	%if &NumberofComponents le 0 %then %do;
		%put ERROR: The Maximum Number of Principal Components must be positive.;
		%let exiterror=1;
		%goto exit;
	%end;
%end;

%if %upcase(&SKAT)=YES %then %do;
	%let usestratas=0;
	%if %upcase(&TraitType)=BINARY %then %do;
		%let WhichProcS = LOGISTIC;
		%if %length(&StrataVars) and %length(&FixedEff) %then %let usestratas=1;
	%end;
	%else %if %upcase(&TraitType)=CONTINUOUS %then %do;
		%let WhichProcS = GLM;
	%end;
	%else %do;
		%put ERROR: Only continuous or binary traits can be used for the SKAT.;
		%let exiterror=1;
		%goto exit;
	%end;
%end;

%let ClassVarsKM= &OrigClassVars;

%if &KM %then %do;
	%let ModOptionKM=;
	%if %upcase(&TraitType)=COUNT %then %let ModOptionKM = %str(/ link=log dist=p);
	%else %if %upcase(&TraitType)=ORDINAL %then %let ModOptionKM=%str(/ link=clogit dist=mult);
	%else %if %upcase(&TraitType)=BINARY %then %let ModOptionKM=%str(/ link=logit dist=binary);
	%else %if %upcase(&TraitType)=NOMINAL %then %do;
		%let ModOptionKM=%str(/ link=glogit dist=mult);
		%let ClassVarsKM = &OrigClassVars &TraitVar;
	%end;
	%let DropVar = Label Note Objective;
%end;

data at;
	set &ds(obs=1);
	keep &AlleleVars;
run;
	
proc transpose data=at out=at;
	var &AlleleVars;
run;
%if &syserr>4 %then %goto exit;

%NObsVars(at);
%let Navars = &nobs;

data _null_;
	set at;
	length is$ 16;
	is = "_av" || strip(_n_);
	call symput(is,strip(_name_));
run;

data a1;
	set &ds(obs=1);
run;

proc transpose data=a1 out=at;
	var &TraitVar;
run;

%if &syserr %then %goto exit;

%NObsVars(at);
%global Ntrait;
%let Ntrait = &nobs;

%do i=1 %to &Ntrait; 
	%global _tv&i;
%end; 

proc sort data=at;
	by _name_;
run;

data _null_;
	set at;
	length is$ 16;
	is = "_tv" || strip(_n_);
	call symput(is,strip(_name_));
run;

%let FixedEffVars = ;
%let i = 1;
%do %while(%length(%scan(&FixedEff,&i,'*| ')));
	%let FixedEffVars = &FixedEffVars %scan(&FixedEff,&i,'*| ');
	%let i=%eval(&i+1);
%end;

%put Covariates in Model = &FixedEffVars;

%let RandEffVars = ;
%let i = 1;
%do %while(%length(%scan(&RandEff,&i,'*| ')));
	%let RandEffVars = &RandEffVars %scan(&RandEff,&i,'*| ');
	%let i=%eval(&i+1);
%end;

%let RandOpts=%lowcase(&RandOpts);

%if &i>1 %then %do;
	%if %qlowcase(&RandEffVars)=%str(intercept) %then %do;
		%let RandEffVars=;
		/* Setting i=3 just to stay out of IF below */
		%let i=3;
	%end;
	/* If just one random effect variable, use alternative random statement */
	%if &i=2 and %length(&RandOpts)=0 and %length(&ParmsStmt)=0 and %index(&RandEffVars,%str(-))=0 %then %do;

		proc sort data=&ds out=dssort;
			by &ByVars &RandEffVars;
		run;
		%if %index(%upcase(&ClassVars),%upcase(&RandEffVars))>0 %then %do;
			%let NumRandEff = %trim(%substr(&RandEffVars,1,28)_num);
			data numrandeff;
				set dssort;
				by &ByVars &RandEffVars;
				if first.&RandEffVars then &NumRandEff+1;
			run;

			%let ClassVars = %sysfunc(tranwrd(%upcase(&ClassVars),%upcase(&RandEffVars),%str()));
			%let ClassVarsKM = %sysfunc(tranwrd(%upcase(&ClassVarsKM),%upcase(&RandEffVars),%str()));
			
			%let RandEffVars = &NumRandEff;
			%let ds = numrandeff;
		%end;
		%else %let ds=dssort;		
		%let RandEff = %str(intercept / subject=&RandEffVars);
	%end;
	%else %do;
		%let foundsub=0;
		%let RandEff = %str(&RandEff / &RandOpts); %* Need slash for nominal trait to do group= option;
		/* Need to see if there's a SUBJECT= variable that needs to be kept */
		%let subind=%index(&RandOpts,sub=)+4;
		%if &subind=4 %then %do;
			%let subind=%index(&RandOpts,subject=)+8;
			%if &subind>8 %then %let foundsub=1;
		%end;
		%else %let foundsub=1;
		%if &foundsub=1 %then %let RandEffVars = &RandEffVars %scan(%substr(&RandOpts,&subind),1);
	%end;
%end;

%let OtherVars = %quote(&ByVarsOnly &TraitVar &ClassVars &FixedEffVars &RandEffVars &StrataVars &CensorVar) ;

%if &syserr %then %goto exit;

%if &Listing=Yes %then %do;
	ods listing;
%end;
%else %do;
	ods listing close;
%end;

data mapdata(keep=&AnnoGroup2._Index &AnnoGroup2._Size) othervars(drop=&AnnoGroup2._Index &AnnoGroup2._Size); 
	set mapdata;
	by &AnnoGroup &AnnoGroup2;
	if first.&AnnoGroup2 or missing(&AnnoGroup2) then do;
		_gp+1;
		_count=1;
		call symput("firstingroup"||strip(_gp),strip(_n_));
		output othervars;
	end;
	else 	_count+1;
	if last.&AnnoGroup2 or missing(&AnnoGroup2) then do;
		call symput("HGNum"||strip(_gp),strip(_count));
		call symput("lastingroup"||strip(_gp),strip(_n_));
		call symputx('NumGps2',_gp);
		&AnnoGroup2._Index = _gp;
		&AnnoGroup2._Size = _count;
		output mapdata;
	end;
	%if %length(&AnnoWeight) %then %do;
		call symput("Weightg"||strip(_gp)||"n"||strip(_count),strip(&AnnoWeight));
	%end;
	drop _gp _count;
run;

data mapdata;
	merge othervars mapdata;
run;

%let winno=1;   
%let maxhaplen=3;
%let mhaplen=2;

options label;

%let anygood=0;

%let n5thousand = %sysfunc(ceil(&NumGps2/5000));

%if %length(&ACCProportion)=0 %then %let ACCProportion=0;
%let prevNofC = &NumberofComponents;
	    
%let firstn=1;
%let lastn=%sysfunc(min(&NumGps2,5000));
%let anyanynonsingles=0; /* across all sets */

%do n=1 %to &n5thousand;
	
	%if %upcase(&ExcludeSingles)=YES %then %do;
		%let anynonsingles=0;
		%do i = &firstn %to &lastn;
			%if &&HGNum&i>1 %then %let anynonsingles=1;
		%end;
		%if &anynonsingles=0 %then %goto nextset;
		%else %let anyanynonsingles=1;
	%end;
	
	*begin: S1080263 Luciano*;
	/*
	data 
		%do i = &firstn %to &lastn; 
			%if %upcase(&ExcludeSingles)=NO or &&HGNum&i>1 %then %do; 
				%let forskat = &i;
				gene&i(keep=&OtherVars &&&&_av&&firstingroup&i -- &&&&_av&&lastingroup&i) 
			%end; 
		%end;
		;
		set &ds;
		%if %length(&WhereStmt) %then %do;
		  where &WhereStmt;
		%end;
	run;
	*/
	*end: S1080263 Luciano*;
	
	%nextset:
	
	%let firstn= %eval(&firstn + 5000);
	%let lastn = %sysfunc(min(%eval(&lastn+5000),&NumGps2));
%end;

%if %upcase(&ExcludeSingles)=YES and &anyanynonsingles=0 %then %do;
	%put ERROR: There are no multiple-SNP genes to analyze.;
	%let exiterror=1;
	%goto exit;
%end;

data pat_all;	
	length Trait $32;
	delete;
run;	

data ta_all;	
	length Trait $32;
	delete;
run;

data pat_allp;	
	length Trait $32;
	delete;
run;	

data ta_allp;	
	length Trait $32;
	delete;
run;

data ct_all;	
	length Trait $32;
	delete;
run;

data skat_all;
	length Trait $32;
	delete;
run;

data cp_all;	
	length Trait $32;
	delete;
run;

data cp_allp;	
	length Trait $32;
	delete;
run;

data ms_all;
	length Trait $32;
	delete;
run;

%put WhichProcS= &WhichProcS; /*Luciano*/
%*put geneforskat= gene&forskat; /*Luciano*/
%put Ntrait= &Ntrait; /*Luciano*/
%put FixedEffVars= &FixedEffVars; /*Luciano*/

%if %upcase(&SKAT)=YES %then %do;
/* Get Residuals from model of covariates only, use gene1 (doesn't matter which since not including genotypes) */

	*begin: S1080263 Luciano*;
	%let forskat= forskat;
	data gene&forskat(keep=&OtherVars);
		set &ds;
		%if %length(&WhereStmt) %then %do;
			where &WhereStmt;
		%end;
	run;
	*end: S1080263 Luciano*;
	
	%do t=1 %to &Ntrait; 
		%let TraitVar = &&_tv&t;
		
		%if &WhichProcS = GLM %then %do;
			/* Need to use Logistic instead of GLMMOD to get non-singular X when class vars */
			proc logistic data=gene&forskat outdesignonly outdesign=xmat;
				%if %length(&ClassVars) %then %do;
					class  &ClassVars;
				%end;
				model &&_tv&t =  &FixedEff;
			run;

			ods output overallanova=oa;
		%end;
		
		title "Sequence Kernel Association Test - Null Model";
		
		%if &WhichProcS=LOGISTIC %then %do;
			%if %length(&EventTrait)>0 %then %let TraitVarm=%str(&TraitVar(event="%sysfunc(dequote(&EventTrait))"));
			%else %let TraitVarm=%str(&TraitVar(event=LAST));			
			ods output responseprofile=rp;
		%end;
		%else %let TraitVarm = &TraitVar;
		
		proc &WhichProcS data=gene&forskat %if &WhichProcS=LOGISTIC %then outdesign=xmat;;

			%if %length(&ClassVars) %then %do;
				class  &ClassVars;
			%end;
			model &TraitVarm =  &FixedEff;
			%if &usestratas %then %do;
				strata &StrataVars;
			%end;			
			%if &WhichProcS = GLM %then %do;
				output out=outr&t r=resid;
			%end;
			%else %do;
				output out=outr&t pred=pred;
			%end;
		run;
		
		%if &WhichProcS = GLM %then %do;
			data _null_;
				set oa;
				if source="Error" then call symputx("MSE&t",MS);
			run;
			
		%end;
		%else %do;
		
			/* Determine event level for calculating resids */
			data _null_;
				set rp;
				if OrderedValue=2 then call symput("event",strip(Outcome));
			run;
		
			data outr&t;
				set outr&t;	
				if strip(&&_tv&t)="&event" then resid = 1 - pred;
				else resid = 0 - pred;
			run;
		%end;
		
		/*beging fixing bug S1013210*/		
		data xmat;
			set xmat(drop= &&_tv&t);
			%if %length(&FixedEffVars) %then %do;
				array a{*} _ALL_;
				do _i=1 to dim(a);
					if missing(a[_i]) then delete;
				end;
				drop _i;
			%end;
		run;
		/*end fixing bug S1013210*/
		
		data outr&t; 
			set outr&t;
			%if &WhichProcS =  GLM %then %do;
				where not missing(resid);
			%end;
			%else %do;
				where not missing(pred) or not missing(resid);
			%end;	
		run;
		
		%Nobsvars(outr&t);
		
		%if &nobs = 0 %then %do;
			%put ERROR: There was an error fitting the null model for the SKAT.  The SKAT will not be performed.;
			%if %length(&FixedEff) %then %do;
				%put ERROR: This error may be due to missing values in covariates specified as fixed effects.;
			%end;
			%let SKAT = No;
			%goto skatexit;
		%end;	
		
		proc iml;
			%if &WhichProcS =  GLM %then %do;
				use outr&t var{resid};
				read all var{resid};		
				/* Divide Q by MSE to make this 1 */
				
				Vmat = I(&nobs)*&&MSE&t;

			%end;
			%else %do;
				use outr&t var{pred resid};
				read all var{pred resid};

				Vmat = diag(pred#(1-pred));
			%end;

			use xmat var _all_;
			read all into xmat;
			P0s = Vmat - Vmat*Xmat*inv(t(Xmat)*Vmat*Xmat)*t(Xmat)*Vmat;
			store P0s resid;
			
		quit;

	%end;
	%skatexit:;
%end;

%do i = 1 %to &NumGps2;

	%if %upcase(&ExcludeSingles)=NO or &&HGNum&i>1 %then %do;
	
		*begin: S1080263 Luciano*;
		data gene&i(keep=&OtherVars &&&&_av&&firstingroup&i -- &&&&_av&&lastingroup&i);
			set &ds;
			%if %length(&WhereStmt) %then %do;
			 	where &WhereStmt;
			%end;
		run;
		*end: S1080263 Luciano*;
		
		%if &&HGNum&i > 1 %then %let ModelSNPs = %quote(&&&&_av&&firstingroup&i -- &&&&_av&&lastingroup&i);
		%else %let ModelSNPs = %quote(&&&&_av&&firstingroup&i);

		%do t=1 %to &NTrait;
			%let TraitVar = &&_tv&t;
			%if %upcase(&TraitType)=BINARY %then %do;
				%if %length(&EventTrait)>0 %then %let TraitVarm=%str(&TraitVar(event="%sysfunc(dequote(&EventTrait))"));
				%else %let TraitVarm=%str(&TraitVar(event=LAST));
			%end;
			%else %if %upcase(&TraitType)=NOMINAL %then %do;
				%if %length(&RefTrait)>0 %then %let TraitVarm=%str(&TraitVar(ref="%sysfunc(dequote(&RefTrait))"));
				%else %let TraitVarm=%str(&TraitVar(ref=FIRST));
			%end;  
			%else %if %upcase(&TraitType)=SURVIVAL %then %do;
				%let TraitVarm=%quote(&TraitVar*&CensorVar(&CensorValues)); 
			%end;
			%else %let TraitVarm=&TraitVar;

			data ta;			                     
				delete;
			run;

			data cp;
				delete;
			run;

			data ms;
				delete;
			run;
			
			data ct;
				delete;
			run;

			%if &HT %then %do;
			/* Hotelling's T2 test */
				title "Hotelling's T-squared Test";
				ods output multstat=ms;
				proc glm data=gene&i;
					%if %length(&ByVars) %then %do; 
					  by &ByVars;
					%end;
					class &TraitVar;
					model &ModelSNPs = &TraitVar / nouni;
					manova h=&Traitvar;
				run;
				quit;
				%if &syserr <= 4 and %sysfunc(exist(ms)) %then %let anygood=1;
				%else %goto nextLRrun;			

				data ms_all;
					length Trait $32;
					set ms_all(in=orig) ms;
					if orig=0 then do;
						Trait = "&TraitVar";
						&Annogroup2._Index = &i;
					end;
				run;			

			%end;

			%nextLRrun:

			%if &LR %then %do;
			/*
				%if %index(&WhichProc,PHREG)=0 %then %let teststmt=%quote('genotypes' &&&&_av&&firstingroup&i 1);
				%else %let teststmt=%quote(&&&&_av&&firstingroup&i);

				%if (&&HgNum&i>1) %then %do;
					%do k=%eval(&&firstingroup&i+1) %to &&lastingroup&i;
						%let kindexed = %eval(&k - &&firstingroup&i);
						%if %index(&WhichProc,PHREG)>0 %then %let teststmt=&teststmt, &&_av&k;
						%else %if %quote(&WhichProc)=%str(GLIMMIX) %then %let testtmt=&teststmt, %quote("genotypes&kindexed" &&_av&k 1); 
						%else %let teststmt=%str(&teststmt, &&_av&k 1);
					%end;
				%end;	
				*/
				options notes;
				
				title "Multiple-Locus Regression Model";
				ods output &TTable=ta &ParmEst = pat %if %length(&RandEff) %then CovParms=cp;	;
				proc &WhichProc data=gene&i &ProcOption ;

					%if %length(&ClassVars) %then %do;
						class  &ClassVars &ClassOpt;
					%end;
					model &TraitVarm =  &FixedEff &ModelSNPs &ModOption;

					%if %length(&RandEff) %then %do;
						random &RandEff %if %upcase(&TraitType)=NOMINAL %then group=&TraitVar;;
						%if %length(&ParmsStmt) %then %do;
							parms &ParmsStmt;
						%end;
					%end;

					%if %length(&StrataVars)>0 %then %do;
						strata &StrataVars;
					%end;
					&TStmt 

					%if %index(&WhichProc,PHREG)=0 %then %do;
						"genotypes" &&&&_av&&firstingroup&i 1
					%end;
					%else %do;
						&&&&_av&&firstingroup&i
					%end;				
					%if (&&HgNum&i>1) %then %do;
						%do k=%eval(&&firstingroup&i+1) %to &&lastingroup&i;
							%let kindexed = %eval(&k - &&firstingroup&i + 1);
							%if %index(&WhichProc,PHREG)>0 %then %do;
								, &&_av&k
							%end;
							%else %do;
								, &&_av&k 1
							%end;
						%end;
					%end;					
					;
					%if %length(&ByVars) %then %do; 
						by &ByVars;
					%end;
				run;
				%if  %sysfunc(exist(pat)) or %sysfunc(exist(ta)) %then %let anygood=1;
				%else %goto nextPCRrun;

				%if %sysfunc(exist(pat)) %then %do;

					data pat_all;	
						length Trait $32;
						set pat_all(in=orig) pat;
						if orig=0 then do;
							Trait="&TraitVar";
							&Annogroup2._Index = &i;
						end;
					run;	
					
					proc datasets nolist;
						delete pat;
					quit;					
				%end;					

				%if %length(&RandEff) %then %do;
					%if %sysfunc(exist(cp)) %then %do;
				
						data cp_all;	
							length Trait $32;
							set cp_all(in=orig) cp;
							if orig=0 then do;
								Trait="&TraitVar";
								&Annogroup2._Index = &i;
							end;
						run;	
						
						proc datasets nolist;
							delete cp;
						quit;					
					%end;							
				%end;

				%if %sysfunc(exist(ta)) %then %do;

					data ta_all;	
						length Trait $32;
						set ta_all(in=orig) ta;
						if orig=0 then do;
							Trait="&TraitVar";
							&Annogroup2._Index = &i;
						end;
					run;	

					proc datasets nolist;
						delete ta;
					quit;					
				%end;					
			%end; /* end LR */
			
			%nextPCRrun:

			%if &PCR %then %do;
			
				%if &t=1 %then %do;
			
					proc datasets nolist;
						delete pca eigen;
					quit;

					options notes;

					ods output eigenvalues=eigen;
					ods exclude simplestatistics corr eigenvectors;
					proc princomp data=gene&i out=pca n=&prevNofC;
						var &ModelSNPs;
					run;
					%if %sysfunc(exist(eigen))=0 or %sysfunc(exist(pca))=0 %then %goto nextKMrun;

					/* Determine number of components to use based on ACCProportion */
					%let NumberofComponents = &prevNofC;

					%if &ACCProportion>0 %then %do;

						data _null_;
							set eigen end=last; 
							if Cumulative >= &AccProportion then do;
								call symputx('NumberofComponents',_n_);
								stop;
							end; 
							if last then call symputx('NumberofComponents',_n_);
						run;		
					%end;		
					%else %do;
						%Nobsvars(eigen);
						%if &nobs < &NumberofComponents %then %let NumberofComponents = &nobs;
					%end;
				%end;
				
				title "PCA Regression Model";
				ods output &TTable=tap &ParmEst = patp %if %length(&RandEff) %then CovParms=cpp;	;
				proc &WhichProc data=pca &ProcOption ;

					%if %length(&ClassVars) %then %do;
						class  &ClassVars &ClassOpt;
					%end;
					model &TraitVarm =  &FixedEff Prin1-Prin&NumberofComponents &ModOption;

					%if %length(&RandEff) %then %do;
						random &RandEff %if %upcase(&TraitType)=NOMINAL %then group=&TraitVar;;
						%if %length(&ParmsStmt) %then %do;
							parms &ParmsStmt;
						%end;
					%end;

					%if %length(&StrataVars)>0 %then %do;
						strata &StrataVars;
					%end;
					&TStmt 

					%if %index(&WhichProc,PHREG)=0 %then %do;
						"genotypes" Prin1 1
					%end;
					%else %do;
						Prin1
					%end;				
					%if (&NumberofComponents>1) %then %do;
						%do k=2 %to &NumberofComponents;
							%if %index(&WhichProc,PHREG)>0 %then %do;
								, Prin&k
							%end;
							%else %do;
								, Prin&k 1
							%end;
						%end;
					%end;					
					;
					%if %length(&ByVars) %then %do; 
						by &ByVars;
					%end;
				run;
				%if &syserr <= 4 or %sysfunc(exist(patp)) or %sysfunc(exist(tap)) %then %let anygood=1;
				%else %goto nextKMrun;

				%if %sysfunc(exist(patp)) %then %do;
					data pat_allp;	
						length Trait $32;
						set pat_allp(in=orig) patp;
						if orig=0 then do;
							Trait="&TraitVar";
							&Annogroup2._Index = &i;
						end;
					run;	
					
					proc datasets nolist;
						delete patp;
					quit;					
				%end;

				%if %length(&RandEff) %then %do;
					%if %sysfunc(exist(cpp)) %then %do;
				
						data cp_allp;	
							length Trait $32;
							set cp_allp(in=orig) cpp;
							if orig=0 then do;
								Trait="&TraitVar";
								&Annogroup2._Index = &i;
							end;
						run;	
						
						proc datasets nolist;
							delete cpp;
						quit;
					%end;
				%end;

				%if %sysfunc(exist(tap)) %then %do;

					data ta_allp;	
						length Trait $32;
						set ta_allp(in=orig) tap;
						if orig=0 then do;
							Trait="&TraitVar";
							&Annogroup2._Index = &i;
						end;
					run;	
					
					proc datasets nolist;
						delete tap;
					quit;					
				%end;					
			%end; /* end PCR */			
			
			%nextKMrun:
			
			%if &KM %then %do;
			
				title "Radial Basis Machine (Kernel Method)";
				ods output covtests=ct;
				proc glimmix data=gene&i METHOD=&GMethod ;

					%if %length(&ClassVarsKM) %then %do;
						class  &ClassVarsKM ;
					%end;
					model &TraitVarm =  &FixedEff /*&ModelSNPs*/ &ModOptionKM ;

					random &ModelSNPs / type=rsmooth knotmethod=data(gene&i) %if %upcase(&TraitType)=NOMINAL %then group=&TraitVar;;
					covtest indep;

					%if %length(&ByVars) %then %do; 
						by &ByVars;
					%end;
				run;		
				
				%if &syserr <= 4 and %sysfunc(exist(ct)) %then %let anygood=1;
				%else %goto nextSKATrun;						
				
				data ct_all;
					length Trait $32;
					set ct_all(in=orig) ct;
					if orig=0 then do;
						Trait="&TraitVar";
						&Annogroup2._Index = &i;
					end;
				run;

			%end;
			
			%nextSKATrun:
			
			%if %upcase(&SKAT)=YES %then %do;
			
				title "Sequence Kernel Association Test";
				
				data gonly(keep=&ModelSNPS) %if %length(&ByVars) %then byonly(keep=&ByVars);;
					set gene&i;
					%if %length(&FixedEffVars) %then %do;
						%let j=1;
						%do %while(%length(%scan(&FixedEffVars,&j)));
							if missing(%scan(&FixedEffVars,&j)) then delete;
							%let j=%eval(&j+1);
						%end;
					%end;					
				run;

				proc iml;
					use gonly var _all_;
					read all var _all_ into Zall;
					load P0s resid;
					
					/* Don't accommodate By Vars for now, but keep this code anyway */
					%if %length(&ByVars) %then %do;
						use byonly var _all_;
						read all var _all_ into ByV;
						ur = uniqueby(ByV,1:ncol(ByV),1:nrow(ByV));
					%end;
					%else %do;
						ur = {1,1};
						ur[2]=nrow(Zall)+1;
					%end;
					
					/* Calculate MAF to use for weights and/or imputing missing values */
					Zmiss = (Zall<0);

					maf = t(Zall[+,]) / (2*(nrow(Zall)-t(Zmiss[+,])));
					maf = maf >< (1-maf);					
					
					/* unweighted for now, these will need to be sqrt(w) */
					weights = J(&&HGNum&i,1,1);
					%if %length(&AnnoWeight) %then %do;
						%do m=1 %to &&HGNum&i;
							weights[&m] = &&Weightg&i.n&m;
							if weights[&m]=. then weights[&m]=0;
						%end;
					%end;
					%else %if %index(&VarWts,Beta) %then %do;
						/* Beta distribution at MAF */
						weights = (maf##%sysevalf(&BetaAlpha-1))#((1-maf)##%sysevalf(&BetaBeta-1));
						if any(weights<0) then do;
							do j=1 to nrow(weights);
								if weights[j]=. then weights[j]=0;
							end;
						end;
					%end;
					if sum(weights)=0 then do;
						ProbQMat = .;
						create skat from ProbQMat;
						append from ProbQMat;
						abort;
					end;
					else do;
						weightss = weights##2;
						W = diag(weightss);

						if any(Zmiss) then do;
							impute = J(nrow(Zall),1,0);

							do j=1 to ncol(Zmiss);
								if any(Zmiss[,j]) then do;
									call randgen(impute, 'BINOMIAL', maf[j], 2);

									Zall[,j] = Zall[,j] <> (Zmiss[,j] # impute);

								end;
							end;
						end;	
						free Zmiss impute maf;

						nr= nrow(Zall);
						%if %upcase(&Kernel)=LINEAR %then %do;
							K = Zall*W*t(Zall);
						%end;
						%else %if %upcase(&Kernel)=QUADRATIC %then %do;
							K = (1 + Zall*W*t(Zall))##2;
						%end;
						%else %do; /* IBS */
							ibsi = 2*sum(weightss); /* for diagonal */
							K = ibsi*I(nr);
							do s=2 to nr;
								do t=1 to (s-1);
									ibs = 2 - abs(Zall[s,]-Zall[t,]);
									K[s,t]= ibs*weightss;
									K[t,s] = K[s,t];
								end;
							end;
						%end;
						free Zall;

						W1 = K*P0s;

						*	print lambda;

						*lambdas = sum(lambda);
						*check = trace(w1);
						*print lambdas check;

						*Qsqrt = t(resid)*Z;
						Q = t(resid)*K*resid;
						*print q;

						w2 = w1**2;
						w3 = w2*w1;
						w4 = w3*w1;

						c1 = trace(w1);
						c2 = trace(w2);
						c3 = trace(w3);
						c4 = trace(w4);
						s1 = c3/(c2**(1.5));
						s2 = c4/(c2**2);

						s1s = s1**2;

						if s1s > s2 then do;
							a = 1/(s1 - sqrt(s1s-s2));
							delta = s1*a**3 - a**2;
							L = a**2 - 2*delta;
						end;
						else do;
							delta=0;
							a = 1/s1;
							L = a**2;
						end;

						mux = L + delta;
						sigmax = sqrt(2)*a;

						tstar = (Q - c1) / sqrt(2*c2);
						chiarg = tstar * sigmax + mux;
						if chiarg >= 0 then	ProbQ = 1 - probchi(chiarg, L, delta);
						else ProbQ = 1;

						ProbQMat = ProbQ;

						create skat from ProbQMat;
						append from ProbQMat;
					end;
					
				quit;				
				
				%if &syserr <= 4 and %sysfunc(exist(skat)) %then %let anygood=1;
				%else %goto nexttrait;						
				
				data skat_all;
					length Trait $32;
					set skat_all(in=orig) skat;
					if orig=0 then do;
						Trait="&TraitVar";
						&Annogroup2._Index = &i;
					end;
				run;
				
				proc datasets nolist;
					delete skat;
				quit;					
			%end;
			
			%nexttrait:;

		%end; /* end trait loop */
		%nextwindow:;
		proc datasets nolist;
			delete gene&i;
		quit;
	%end; /*end if */
%end;

%if &Listing=Yes %then %do;
	ods listing close;
%end;

%if &anygood=0 %then %do;
 %put ERROR: There were no successful runs.  The process will exit without creating any results.  See the SAS log for more details.;
 %let exiterror = 1;
 %goto exit;
%end;

%let Stats=;
%if &LR %then %do;
	%Nobsvars(ta_all);
	%if &nobs=0 %then %do;	
		%let LR = 0;
	%end;
	%else %do;
		data ta_all;                                          
			merge mapdata ta_all;
			by &Annogroup2._Index ;
			drop &DropVar _excluded _order_ _oldorder_;
			label Prob&Stat = " ";
			%if %upcase(&ExcludeSingles)=YES %then %do;
				if &AnnoGroup2._Size > 1;
			%end;
			rename Prob&Stat=ProbMultiLocusReg;
		run; 
		%let Stats = MultiLocusReg;
	%end;
%end;
%if &PCR %then %do;
	%Nobsvars(ta_allp);
	%if &nobs=0 %then %do;	
		%let PCR = 0;
	%end;
	%else %do;
		data ta_all;                                          
			merge %if &LR %then ta_all; %else mapdata; ta_allp(rename=(Prob&Stat = ProbPCAReg));
			by &Annogroup2._Index ;
			drop &DropVar _excluded _order_ _oldorder_;
			label ProbPCAReg = " ";
			%if %upcase(&ExcludeSingles)=YES %then %do;
				if &AnnoGroup2._Size > 1;
			%end;
		run; 
		%let Stats = &Stats PCAReg;
	%end;
%end;
%if &KM %then %do;

	%Nobsvars(ct_all);
	%if &nobs=0 %then %let KM = 0;
	%else %do;
		data ta_all;                                          
			merge %if &LR or &PCR %then ta_all; %else mapdata; ct_all(rename=(ProbChiSq = ProbKM ChiSq = ChiSqKM DF=DF_KM));
			by &Annogroup2._Index ;
			drop &DropVar _excluded _order_ _oldorder_;
			label ProbKM = " ";
			%if %upcase(&ExcludeSingles)=YES %then %do;
				if &AnnoGroup2._Size > 1;
			%end;
		run; 

		%let Stats = &Stats KM;
	%end;

%end;

%if &HT %then %do;

	%Nobsvars(ms_all);
	%if &nobs=0 %then %let HT = 0;
	%else %do;
		data ta_all;
			merge %if &LR or &KM or &PCR %then ta_all; %else mapdata; ms_all(where=(Statistic="Hotelling-Lawley Trace") drop=Hypothesis Error PValue);
			by &Annogroup2._Index ;
			rename FValue = F_HT2 NumDF = NumDF_HT2 DenDF = DenDF_HT2 ProbF = ProbHT2 Value = HotLawleyTrace_Value;
			drop Statistic;
			label ProbF = " " NumDF=" " DenDF=" " FValue=" ";
			%if %upcase(&ExcludeSingles)=YES %then %do;
				if &AnnoGroup2._Size > 1;
			%end;		
		run;

		%let Stats = &Stats HT2;
	%end;
%end;

%if %upcase(&SKAT)=YES %then %do;

	%Nobsvars(skat_all);
	%if &nobs=0 %then %let SKAT = No;
	%else %do;
		data ta_all;
			merge %if &LR or &KM or &PCR or &HT %then ta_all; %else mapdata; skat_all;
			by &Annogroup2._Index ;
			rename Col1 = ProbSKAT;

			%if %upcase(&ExcludeSingles)=YES %then %do;
				if &AnnoGroup2._Size > 1;
			%end;		
		run;

		%let Stats = &Stats SKAT;
	%end;
%end;

%if &NTrait>1 %then %do;
%if %length(&ByVars)>0 %then %do;
 %let ByVars = Trait &ByVars;
 %let ByVarsOnly = Trait &ByVarsOnly;
 %let JBy = %str(Trait,&JBy);
%end;
%else %do;
 %let ByVars = Trait;
 %let ByVarsOnly = Trait;
 %let JBy=Trait;
%end;
%let Nbyval = %eval(&Nbyval*&Ntrait);
%end;

proc sort data=ta_all out=&MSNPTraitAssoc_OutData;
 by &ByVars &AnnoGroup &AnnoGroup2 &AnnoLoc;
run;

%if &LR %then %do;
	data &MSNPTraitAssoc_PET;
		merge mapdata pat_all;
		by &Annogroup2._Index;
		drop _excluded _oldorder_ _order_;
	run;	
	%PackageInsertDataset(Parameter estimate data for Multiple-Locus Reg from &ProcessName, OutLib, &MSNPTraitAssoc_PETName, _type=other);

	%if %length(&RandEff) %then %do;
		data &MSNPTraitAssoc_CPT;
			merge mapdata cp_all;
			by &Annogroup2._Index;
			drop _excluded _oldorder_ _order_;
		run;

		%PackageInsertDataset(Covariance parameter data for Multiple-Locus Reg from &ProcessName, OutLib, &MSNPTraitAssoc_CPTName, _type=other);
	%end;
%end;

%if &PCR %then %do;
	data &MSNPTraitAssoc_PETp;
		merge mapdata pat_allp;
		by &Annogroup2._Index;
		drop _excluded _oldorder_ _order_;
	run;	
	%PackageInsertDataset(Parameter estimate data for PCA Reg from &ProcessName, OutLib, &MSNPTraitAssoc_PETNamep, _type=other);

	%if %length(&RandEff) %then %do;
		data &MSNPTraitAssoc_CPTp;
			merge mapdata cp_allp;
			by &Annogroup2._Index;
			drop _excluded _oldorder_ _order_;
		run;

		%PackageInsertDataset(Covariance parameter data for PCA Reg from &ProcessName, OutLib, &MSNPTraitAssoc_CPTNamep, _type=other);
	%end;
%end;

%if %upcase(&ExcludeSingles)=YES %then %do;

	/* Need to modify what anno groups occur in output data set */
	proc sort data=mapdata(where=(&AnnoGroup2._Size > 1)) out=mapgp nodupkey;
	 by &AnnoGroup;
	run;

	/************************
	Retrieve number of groups in
	MAP ds.  Get type of Group var
	for plotting.  Find which markers
	variables belong to which map
	group.
	**************************/
	%let dsid=%sysfunc(open(mapgp));
	%let NumGps=%sysfunc(attrn(&dsid,NOBS));
	%let ret=%sysfunc(close(&dsid));

	data _null_;
	 set mapgp end=last_one;
	 retain maxlenG maxlenL 0;
	 lenG = lengthn(strip(&AnnoGroup));
	 lenL = lengthn(strip(&AnnoLabel));
	 if (lenG > maxlenG) then maxlenG = lenG;
	 if (lenL > maxlenL) then maxlenL = lenL;
	 %if &GpType=C %then %do;
		call symput('Groupn'||left(_n_),trimn(&AnnoGroup));
		 _sasag=trimn(&AnnoGroup);
		 j=0;
		 do until(j=0);  
				j=notname(trimn(_sasag));  
				if j > 0 then substr(_sasag,j,1)="_";  
		 end;
		call symput('SGroupn'||left(_n_),lowcase(trimn(_sasag)));
	 %end;
	 %else %do;
	  call symput('Groupn'||left(_n_),trimn(left(&AnnoGroup)));
	  if &AnnoGroup ne . then call symput('SGroupn'||left(_n_),trimn(left(&AnnoGroup)));
	  else call symput('SGroupn'||left(_n_),"_");
	 %end;
	 if last_one then do;
		call symputx("maxlenAG",strip(maxlenG));
		call symputx("maxlenAL",strip(maxlenL));
	 end;
%end;


%Psmooth(&Stats, &MSNPTraitAssoc_OutData, &MSNPTraitAssoc_OutDataName);

 %if (&syserr>4) %then %goto exit;
 
%global KVars;
%let KVars=;

%global SummaryDescription;
%if &NPVars = 1 %then %do;
	%if %upcase(&Stats)=MULTILOCUSREG %then %let SummaryDescription = %str(Type of Test: Multiple-Locus Regression);
	%else %if %upcase(&Stats)=PCAREG %then %let SummaryDescription = %str(Type of Test: PCA Regression);
	%else %if %upcase(&Stats)=KM %then %let SummaryDescription = %str(Type of Test: Radial Basis Machine (Kernel Method));
	%else %if %upcase(&Stats)=HT2 %then %let SummaryDescription = %str(Type of Test: Hotelling%'s T-squared Test);	
	%else %if %upcase(&Stats)=SKAT %then %let SummaryDescription = %str(Type of Test: SKAT\!NVariant Weights: &VarWts\!NKernel: &Kernel);		
%end;

%JSLPvalMap(Multiple SNP-Trait Association, &MSNPTraitAssocScript, &MSNPTraitAssocScriptName, &MSNPTraitAssoc_OutDataName);

%exit:;

%ExitSDSProcess(&OutPath, &ProcessName, &syserr, &exiterror, &clockrunning);

%exitcheck:; /*Luciano added when debugging*/

%mend;

* run the macro;
%MultipleSNPAssociation;


