/*
JMP Genomics Analytical Process

Name: Pleiotropic Association
Author: Wendy Czika

History:
Wendy Czika		12-May-10        Created


*/

/*
-----------------------------------------------------------
Copyright (C) 2010 SAS Institute, Inc. All rights reserved. 

Notice: 
The above copyright notice and this notice must appear in
any whole or partial copy of this code and any related
documentation. Permission to use, copy, and modify the
source code contained in this file is hereby granted, but
is limited to customers of SAS with a valid license for the
SAS software product with which this file was provided. 
Except as expressly set forth herein, the terms of such
license apply.
-----------------------------------------------------------
*/

*ProcessVariables;

%global StudyName InData AlleleVars ByVars WhereStmt ListMarkerVars ListTraitVars MarkerFmt GenoDelim TraitVar ClassVars 
FixedEff IntEff IntTestsOnly AnnoInData AnnoLabel AnnoMName AnnoGroup AnnoLoc AnnoMajAllele AnnoAccession AnnoWhereStmt  
QTests MANOVAStat OutputLSM Conv MTC  PValAdj Tau Bandwidth Alpha OnlySig OutPath ServerOutPath OutName MacroPath 
PleioAssoc_OutData PleioAssocScript;
 
*ProcessBody;

%macro PleiotropicAssociation;
%include "&MacroPath./GenMacros.sas" / nosource;
%let ReorderingVars = ; * set this macro to empty to avoid certain conflict when call %GenSubset;

%GenInit(PleiotropicAssociation,28);
%if &exiterror=1 %then %goto exit;
%global FullAPName;
%let FullAPName=%str(Pleiotropic Association);

%if %length(&ListMarkerVars &AlleleVars)=0 %then %do;
 %put ERROR: Marker variables must be specified in either the Marker Variables or List-Style Specification of Marker Variables parameters.;
 %let exiterror=1;
 %goto exit;
%end;

%if %length(&ListTraitVars &TraitVar)=0 %then %do;
 %put ERROR: One or more trait variables must be specified in either the Trait Variables or List-Style Specification of Trait Variables parameters.;
 %let exiterror=1;
 %goto exit;
%end;

%let AlleleVars = &AlleleVars &ListMarkerVars;

%let TraitVar = &TraitVar &ListTraitVars;

%PrepOutDS(PleioAssoc_OutData,PleioAssoc_OutDataName,&OutName._pla);

%PrepOutFile(PleioAssocScript,PleioAssocScriptName,&ProcessName Script for &OutName..jsl);

%if (%length(&QTests)=0) %then %let QTests=Genotype Trend;

%let geno=0;
%if (%index(%upcase(&QTests),GENOTYPE)>0) %then %let geno=1;
%let trend=0;
%if (%index(%upcase(&QTests),TREND)>0) %then %let trend=1;

%global PleioAssoc_PEG PleioAssoc_PET  PleioAssoc_LSM PleioAssoc_MG PleioAssoc_MT;
%let PleioAssoc_PEG=;
%let PleioAssoc_PET=;

%let PleioAssoc_LSM=;

%if %upcase(&OutputLSM)=YES %then %let lsm=1;
%else %let lsm=0;

%if &geno=1 %then %do;
	%PrepOutDS(PleioAssoc_PEG,PleioAssoc_PEGName,&OutName._peg);
	%PrepOutDS(PleioAssoc_MG,PleioAssoc_MGName,&OutName._pmg);

	%if &lsm=1 %then %PrepOutDS(PleioAssoc_LSM,PleioAssoc_LSMName,&OutName._lsm);
%end;
%else %let lsm=0;
%if &trend=1 %then %do;
	%PrepOutDS(PleioAssoc_MT,PleioAssoc_MTName,&OutName._pmt);
	%PrepOutDS(PleioAssoc_PET,PleioAssoc_PETName,&OutName._pet);
%end;

%PrepMapandBy;
run;
%if &exiterror=1 %then %goto exit;

/* Table name from all but MIXED */
%global ParmEst EffName;
%let ParmEst=ParameterEstimates;
%let EffName=Parameter;

%let Stat=F; 
%let Gtable=ModelANOVA;
%let ModOption=%quote(/ solution); 
%let ClassOpt=;	

%if %upcase(&OutputLSM)=YES and &lsm=0 %then %do;
	%put LS means cannot be calculated for this model.;
%end;

%if %symexist(Listing)=0 %then %let Listing=NO;

data msg_all;
	length Effect $32;
	delete;
run;

data mst_all;
	length Effect $32;
	delete;
run;

data ta_all;
	length Effect $32;
 delete;
run;

data ma_a;
	length Effect $32;
  delete;
run;

data pag_all;
	length &EffName Trait $32;
	delete;
run;
	
data pat_all;
	length &EffName Trait $32;
	delete;
run;

%if &lsm=1 %then %do;
	data outlsm_all;
		length Trait $32 Effect $17;
		delete;
	run;
%end;

data a1;
	set &ds(obs=1);
run;

proc transpose data=a1 out=at;
	var &TraitVar;
run;

%if &syserr>4 %then %do;
	%let exiterror=1;
	%goto exitm;
%end;

%NObsVars(at);
%global NTrait;
%let Ntrait = &nobs;
%if &NTrait=0 %then %do;
	%let exiterror=1;
	%put ERROR: No variables found matching the Trait Variables specified.;
	%goto exitm;
%end;

%do i=1 %to &Ntrait; 
	%global _tv&i;
%end; 

proc sort data=at;
	by _name_;
run;

data _null_;
	set at;
	length is $16;
	is = "_tv" || strip(_n_);
	call symput(is,strip(_name_));
run;

%let AllTraitVars = ;
%do i=1 %to &Ntrait;
	%let AllTraitVars = &AllTraitVars &&_tv&i;
%end;

%let FixedEffVars = ;
%let i = 1;
%do %while(%length(%scan(&FixedEff,&i,'*| ')));
	%let FixedEffVars = &FixedEffVars %scan(&FixedEff,&i,'*| ');
	%let i=%eval(&i+1);
%end;

%if %length(&WhereStmt) %then %do;
	data sub;
		set &ds;
		where &WhereStmt;
	run;
	
	%if &syserr>4 %then %do;
		%let exiterror=1;
		%goto exitm;
	%end;
	
	%Nobsvars(sub);
	%if &nobs=0 %then %do;
		%put ERROR: There are no observations in the input data set that meet the criteria in the Where Clause.;
		%let exiterror=1;
		%goto exitm;
	%end;	
	%let ds=sub;
%end;
%else %Nobsvars(&ds);

%let IntEffVars=;
%let i = 1;
%do %while(%length(%scan(&IntEff,&i,'* ')));
	%let IntEffVars = &IntEffVars %scan(&IntEff,&i,'* ');
	%let i=%eval(&i+1);
%end;

%let IntEffBuild=;
%let i = 1;
%do %while(%length(%scan(&IntEff,&i,' ')));
	%let IntEffBuild = &IntEffBuild Genotype*%scan(&IntEff,&i,' ');
	%let i=%eval(&i+1);
%end;
%let NumInt = &i;

%if &syserr>4 %then %do;
	%let exiterror=1;
	%goto exitm;
%end;

%let AllVars= &ByVarsOnly &TraitVar &ClassVars &FixedEffVars &IntEffVars ;

%let two=minor;
%if %symexist(RecodeAllele) %then %do;
	%if %index(&RecodeAllele,Major)>0 %then %let two=major;
%end;

%let n2snps=%eval(2*&nummark);

%let BlockSize = %sysfunc(min(100000,%sysevalf(35000000/&nobs,floor)));
%if &nummark < &BlockSize %then %let BlockSize=&nummark;

/* Merge covariate/traits into stacked data set */
data traits(keep=&AllVars _IndivID_) traits1(keep=&AllVars _IndivID_) ;
	set &ds;
	_IndivID_=_n_;
run;

options nonotes;
proc datasets nolist;
	%do i=2 %to &BlockSize;
		append base=traits data=traits1;
	%end;
quit;
options notes;	

%let first = 1;
%let last=&BlockSize;

data _null_;
	set &ds(obs=1);
	array getname{*} &AlleleVars;
	%let Block=1;
	%do %until (&first > &nummark);	
		%if %length(&GenoCol) %then %do;
			_f = vname(getname(&first));
			_l = vname(getname(&last));
		%end;
		%else %do;
			_f = vname(getname(%eval(2*&first-1)));
			_l = vname(getname(%eval(2*&last)));
		%end;

		call symput("firstvar&Block",_f);
		call symput("lastvar&Block",_l);

		%global BlockStart&Block BlockEnd&Block;
		%let BlockStart&Block = &first;
		%let BlockEnd&Block = &last;

		%let first = %eval(&last + 1);
		%let last = %eval(&first + &BlockSize - 1);
		%let Block = %eval(&Block + 1);
		%if &last > &nummark %then %let last = &nummark;
	%end;
run;	
%let NBlocks = %eval(&Block - 1);

%let first = 1;
%let last=&BlockSize;
%let Block = 1;

data %do b=1 %to &NBlocks; avars&b(keep=&&firstvar&b--&&lastvar&b) %end;;
	set &ds(keep=&AlleleVars);
run;

options DKROCOND=nowarn;

%let Genotype=Genotype;

%do %until (&first > &nummark);
	%CreateStackedNumericDS(avars&block,_2=&two);
	%if &exiterr=1 %then %do;
		%let exiterror=1;
		%goto exitm;
	%end;	
	proc datasets nolist;
		delete avars&block;
	quit;
	
	%let ie=0;
	%let Interaction=;
	
	%if %upcase(&IntTestsOnly)=YES %then %do;
		%let ie=1;
		%let Interaction=%scan(&IntEffBuild,&ie,' ');
	%end;	
	
	%do %until(%length(%scan(&IntEffBuild,&ie,' '))=0);

		%if %upcase(&Listing)=NO %then %do;
			ods listing close;
		%end;			

		%if (&geno=1) %then %do;
			data ma; 				/* kemicl - added to ensure data is not repeated if PROC does not produce estimates */
				delete;
			run;
			
			data msg;
				delete;
			run;

			title2 "Genotype Test &Interaction";
			ods output overallanova=oa multstat=msg &Gtable=ma &ParmEst=pag %if &lsm=1 and &ie=0 %then lsmeans=lsm;;
			proc glm data=genotypes;
				class Genotype &ClassVars  &ClassOpt;
				model &AllTraitVars = &FixedEff Genotype &Interaction &ModOption;
				%if &ie=0 %then %do;
					manova h=genotype;
				%end;
				%else %do;
					manova h=&Interaction;
				%end;
				%if &lsm=1 and &ie=0 %then %do;
					lsmeans genotype / pdiff=control;
				%end;

				by _order_ &ByVars;
			quit;
			
			/*
			%if (&syserr>4) %then %do;
				%let exiterror=1;
				%goto exitm;
			%end;		
			*/
			%NObsVars(msg);
			%if ^&nobs %then %do;
				%put ERROR:  PROC GLM generated no estimates for the Genotype Test for traits &TraitVar..  There may be excessive missing data or the contrast may be nonestimable. Please see the log for more details.;
				%goto skip1;
			%end;

			%put Test = Genotype, Trait = &TraitVar, Effects = Genotype &Interaction;       			

			/* Need to merge in oa to get denominator DF for F test */
			data ma;
				length FValue NumDF DenDF ProbF 8;
				merge ma(where=(hypothesistype=3) rename=(df=NumDF source=Effect)) oa(where=(Source="Error") keep=_order_ &ByVars dependent df source rename=(df=DenDF));
				by _order_ &ByVars Dependent;
			run;
			
			data msg_all;
				set msg_all(in=orig) msg(/*where=(substr(Statistic,1,1)="%substr(&MANOVAStat,1,1)")*/ drop=hypothesis error pvalue);
				%if &ie>0 %then %do;
					if orig=0 then Effect = "&Interaction";
					if orig=1 or index(Effect,"Genotype*")>0 or index(Effect,"*Genotype")>0; 
					/* To make sure interactions in same order for Genotype and Trend tests */
					if index(Effect,"Genotype*")>1 then do;
						Effect=cats("Genotype*",tranwrd(Effect,"Genotype*",""));
					end;
					else if index(Effect,"Genotype")>1 then do;
						Effect=cats("Genotype*",tranwrd(Effect,"*Genotype",""));
					end;					
				%end;
				%else %do;
					if orig=0 then Effect = "Genotype";
					if index(Effect,"&Genotype")>0; 
				%end;					
			run;		
				
			data ma_a; 
				length Trait $32;
				set ma_a(in=orig) ma msg(where=(substr(Statistic,1,1)="%substr(&MANOVAStat,1,1)") drop=hypothesis error pvalue in=manova);
				if manova then do;
					%if &ie>0 %then %do;
						Effect = "&Interaction";
					%end;
					%else %do;
						Effect = "Genotype";
					%end;
				end;
				%if &ie>0 %then %do;
					if orig=1 or index(Effect,"Genotype*")>0 or index(Effect,"*Genotype")>0; 
					/* To make sure interactions in same order for Genotype and Trend tests */
					if index(Effect,"Genotype*")>1 then do;
						Effect=cats("Genotype*",tranwrd(Effect,"Genotype*",""));
					end;
					else if index(Effect,"Genotype")>1 then do;
						Effect=cats("Genotype*",tranwrd(Effect,"*Genotype",""));
					end;
				%end;
				%else %do;
					if index(Effect,"&Genotype")>0; 
				%end;
				if not orig and not manova then Trait = Dependent;
				if manova then Trait = "All Traits (MANOVA)";
				label Prob&Stat=' ';
				drop dependent hypothesistype statistic value source ss ms;
			run;

			data pag_all;	
				%if %length(&IntEffBuild) %then %do;
					length Interaction $32;
				%end;
				set pag_all(in=orig) pag;
				if orig=0 then do;
					Trait=dependent;
					%if %length(&IntEffBuild) %then %do;
						Interaction="&Interaction";
					%end;
				end;
				drop dependent;
			run;

			%if &lsm=1 and &ie=0 %then %do;
				data outlsm_all;
					set outlsm_all(in=orig) lsm;
					if orig=0 then do;
						Trait=Dependent;
					end;
					drop Dependent;
				run;		
			%end;				

		%end;

		%skip1:

		%if &trend=1 %then %do;
			data ta;       			/* kemicl - added to ensure data is not repeated if PROC does not produce estimates */
				delete;
			run;
			
			data mst;
				delete;
			run;			

			title2 "Trend Test &Interaction";

			options nonotes;
			ods output overallanova=oa multstat=mst &Gtable=ta &ParmEst = pat;
			proc glm data=genotypes ;
				class  &ClassVars  &ClassOpt;
				model &AllTraitVars = &FixedEff Genotype &Interaction &ModOption;
				%if &ie=0 %then %do;
					manova h=genotype;
				%end;
				%else %do;
					manova h=&Interaction;
				%end;				
				by _order_ &ByVars;
			quit;

			/* kemicl - added to catch error when all effects are not estimable */			
			 %NObsVars(mst);
			 %if ^&nobs %then %do;
				%put ERROR:  PROC GLM generated no estimates for the Trend Test for traits &TraitVar..  There may be excessive missing data or the contrast may be nonestimable. Please see the log for more details.;
				%goto skip2;
			 %end;

			 %put Test = Trend, Trait = &TraitVar, Effect = Genotype &Interaction;   		
			 
			/* Need to merge in oa to get denominator DF for F test */
			data ta;
				length FValue NumDF DenDF ProbF 8;
				merge ta(where=(hypothesistype=3) rename=(df=NumDF source=Effect)) oa(where=(Source="Error") keep=_order_ &ByVars dependent df source rename=(df=DenDF));
				by _order_ &ByVars Dependent;
			run;
									
			data mst_all;
				set mst_all(in=orig) mst(/*where=(substr(Statistic,1,1)="%substr(&MANOVAStat,1,1)")*/ drop=hypothesis error pvalue);
				%if &ie>0 %then %do;
					if orig=0 then Effect = "&Interaction";
					if orig=1 or index(Effect,"Genotype*")>0 or index(Effect,"*Genotype")>0; 
					/* To make sure interactions in same order for Genotype and Trend tests */
					if index(Effect,"Genotype*")>1 then do;
						Effect=cats("Genotype*",tranwrd(Effect,"Genotype*",""));
					end;
					else if index(Effect,"Genotype")>1 then do;
						Effect=cats("Genotype*",tranwrd(Effect,"*Genotype",""));
					end;					
				%end;
				%else %do;
					if orig=0 then Effect = "Genotype";
					if index(Effect,"&Genotype")>0; 
				%end;	
			run;
			
			data ta_all;  
				length Trait $32  ;
				set ta_all(in=orig) ta mst(where=(substr(Statistic,1,1)="%substr(&MANOVAStat,1,1)") drop=hypothesis error pvalue in=manova);
				if manova then do;
					%if &ie>0 %then %do;
						Effect = "&Interaction";
					%end;
					%else %do;
						Effect = "Genotype";
					%end;
				end;			
				%if &ie>0 %then %do;
					if orig=1 or index(Effect,"Genotype*")>0 or index(Effect,"*Genotype")>0; 
					/* To make sure interactions in same order for Genotype and Trend tests */
					if index(Effect,"Genotype*")>1 then do;
						Effect=cats("Genotype*",tranwrd(Effect,"Genotype*",""));
					end;
					else if index(Effect,"Genotype")>1 then do;
						Effect=cats("Genotype*",tranwrd(Effect,"*Genotype",""));
					end;					
				%end;
				%else %do;
					if index(Effect,"&Genotype")>0; 
				%end;				
				if not orig and not manova then Trait = Dependent;
				if manova then Trait = "All Traits (MANOVA)";
				label Prob&Stat=' ';
				drop dependent hypothesistype statistic value source ms ss;				
			 run;

			data pat_all;	
				%if %length(&IntEffBuild) %then %do;
					length Interaction $32;
				%end;
				set pat_all(in=orig) pat;
				if orig=0 then do;
					Trait=Dependent;
					%if %length(&IntEffBuild) %then %do;
						Interaction="&Interaction";
					%end;
				end;
				drop dependent;
			run;			

		%end; /* end if trend */

		%skip2:;

		options notes;

		title2;

		%let ie=%eval(&ie+1);   
		%let Interaction=%scan(&IntEffBuild,&ie,' ');

	%end; /* end loop over interactions */
	
	%let first = %eval(&last + 1);
	%let last = %eval(&first + &BlockSize - 1);
	%if &last > &nummark %then %let last = &nummark;
	%let Block = %eval(&Block + 1);		
%end; /* end block loop */

%if &geno=1 %then %do;
	%Nobsvars(ma_a);
	%if &nobs=0 %then %do;
		%put ERROR: Genotype tests did not run successfully.  See the SAS log for more details.;
		%let exiterror = 1;
		%goto exitm;
	%end;
%end;
%if &trend=1 %then %do;
	%Nobsvars(ta_all);
	%if &nobs=0 %then %do;
		%put ERROR: Trend tests did not run successfully.  See the SAS log for more details.;
		%let exiterror=1;
		%goto exitm;
	%end;
%end;

%if %length(&IntEff)>0 %then %do;
	%let NonTraitByVars=&NonTraitByVars Effect;
	%let JNonTraitByVars = &JNonTraitByVars :Effect;
	%let OrigNbyval = %eval(&OrigNbyval*&NumInt);
%end;

%let FillByVars= &ByVars;    
%if %length(&ByVars)>0 %then %do;
 %let ByVars = Trait &ByVars;
 %let ByVarsOnly = Trait &ByVarsOnly;
 %let JBy = %str(Trait,&JBy);
%end;
%else %do;
 %let ByVars = Trait;
 %let ByVarsOnly = Trait;
 %let JBy=Trait;
%end;
%let NTrait = %eval(&NTrait + 1);
%let Nbyval = %eval(&Nbyval*&Ntrait);

%if &trend=1 %then %do;
	proc sort data=Ta_all out=effect_map(keep=Effect Trait &FillByVars) nodupkey;
		by Effect &ByVars;
	run;
%end;
%else %if (&geno=1) %then %do;
	proc sort data=Ma_a out=effect_map(keep=Effect Trait &FillByVars) nodupkey;
		by Effect &ByVars;
	run;
%end;

data effect_map;
	set effect_map;
	do e=1 to &nummark;
		output;
	end;
	rename e=_order_;
run;

proc sort data=effect_map;
	by _order_;
run;

data mapdata2;                     /* kemicl - Use mapdata2 instead of mapdata to merge in all tables to make final output table */
	merge mapdata effect_map;
	by _order_;
run;

proc sort data=mapdata2;
	by _order_ &ByVars Effect ;
run;

%if (&geno=1) %then %do;

	proc sort data=ma_a;
		by _order_ &ByVars Effect ;
	run;

	data ma_a;
		merge mapdata2 ma_a ;
		by _order_ &ByVars Effect;
		rename NumDF=NumDF_Genotype DenDF=DenDF_Genotype FValue=FValue_Genotype Prob&Stat=ProbGenotype ; 
		if DenDF=0 then Rsquare_Genotype = .;
		else Rsquare_Genotype = (NumDF*FValue/DenDF)/(1+NumDF*FValue/DenDF);
	run;
	
	proc sort data=ma_a;
		by &AnnoGroup &AnnoLoc &ByVars Effect ;
	run;
%end;

%if (&trend=1) %then %do;
	proc sort data=ta_all;
		by _order_ &ByVars Effect;
	run;

	data ta_all;
		merge mapdata2 ta_all;
		by _order_ &ByVars Effect;
		rename NumDF=NumDF_Trend DenDF=DenDF_Trend FValue=FValue_Trend Prob&Stat=ProbTrend;
		if DenDF=0 then Rsquare_Trend = .;
		else Rsquare_Trend = (NumDF*FValue/DenDF)/(1+NumDF*FValue/DenDF);
	run; 
	
	proc sort data=ta_all;
  		by &AnnoGroup &AnnoLoc &ByVars Effect;
 	run;
%end;

%if &geno=1 %then %do;

	proc sort data=msg_all;
		by _order_;
	run;
	
	data &PleioAssoc_MG;
		merge mapdata msg_all;
		by _order_;
		drop _order_;
	run;
	
	%PackageInsertDataset(Genotype MANOVA results data from &ProcessName, OutLib, &PleioAssoc_MGName, _type=other);	

	%if &lsm=1 %then %do;
	
		proc sort data=outlsm_all;
			by _order_ &ByVars genotype;
		run;
		
		data &PleioAssoc_LSM;
			merge mapdata outlsm_all;
			by _order_;
			drop _order_ _genotype;
			rename Estimate = LSMeans_Estimate;
		run;
		
		%PackageInsertDataset(Genotype LS means from &ProcessName, OutLib, &PleioAssoc_LSMName, _type=other);

	%end;
	
	proc sort data=pag_all;
		by _order_;
	run;

	data &PleioAssoc_PEG;
		merge mapdata pag_all;
		by _order_;
		drop _order_;
	run;
	
	%PackageInsertDataset(Genotype parameter estimate data from &ProcessName, OutLib, &PleioAssoc_PEGName, _type=other);
	
%end;

%if &trend=1 %then %do;

	proc sort data=mst_all;
		by _order_;
	run;
	
	data &PleioAssoc_MT;
		merge mapdata mst_all;
		by _order_;
		drop _order_;
	run;
	
	%PackageInsertDataset(Trend MANOVA results data from &ProcessName, OutLib, &PleioAssoc_MTName, _type=other);
	
	proc sort data=pat_all;
		by _order_;
	run;
		
	data &PleioAssoc_PET;
		merge mapdata pat_all;
		by _order_;
		drop _order_;
	run;	
	
	%PackageInsertDataset(Trend parameter estimate data from &ProcessName, OutLib, &PleioAssoc_PETName, _type=other);
	
%end;

%let mergeds=;
/* %if &geno=1 %then %let mergeds = %str(ma_a(where=(Effect="Genotype"))); */         /* kemicl - commented this out */
%if &geno=1 %then %let mergeds=ma_a;
%if &trend=1 %then %let mergeds = &mergeds ta_all;
  	
data ma_all;
	merge &mergeds;
  	by &AnnoGroup &AnnoLoc &ByVars Effect;
run; 
 /*	
%if %length(&IntEff)>0 %then %do;                           kemicl - commented this out 
	data ma_int;
		set ma_a(where=(Effect ne "Genotype"));
	run;
	
	data ma_all;
		set ma_all ma_int;
	run;
	
%end;
*/

title;
title2;
title3;

%global PlotVar1 PlotVar2 PlotVar3 PlotVar4 QTests;

%let PlotVar1=%scan(&QTests,1);
%if %length(%scan(&QTests,2)) %then %let PlotVar2=%scan(&QTests,2);
%let QTests = &PlotVar1 &PlotVar2;

%if %length(&IntEff)>0 %then %do;
	%if %length(&ByVars)>0 %then %do;	
		%let ByVars=&ByVars Effect;
		%let JBy = %str(&JBy,Effect);
	%end;
	%else %do;
	 %let ByVars = Effect;
	 %let JBy=Effect;
	
	%end;
	%let Nbyval = %eval(&Nbyval*&NumInt);
	%let lastby = Effect;
%end;

%exitm:;

%if &exiterror=1 %then %goto exit;

proc sort data=ma_all out=&PleioAssoc_OutData;
 by &ByVars &AnnoGroup &AnnoLoc;
run;

%Psmooth(&QTests, &PleioAssoc_OutData, &PleioAssoc_OutDataName);

 %if (&syserr>4) %then %goto exit;

%JSLPvalMap(Pleiotropic Association, &PleioAssocScript, &PleioAssocScriptName, &PleioAssoc_OutDataName);

%exit:;
%ExitSDSProcess(&OutPath, &ProcessName, &syserr, &exiterror, &clockrunning);

%mend;

* run the macro;
%PleiotropicAssociation;

