/*
JMP Genomics Analytical Process

Name: Q-K Mixed Model
Author: Wendy Czika

-----------------------------------------------------------
Copyright (C) 2009 SAS Institute, Inc. All rights reserved. 

Notice: 
The above copyright notice and this notice must appear in
any whole or partial copy of this code and any related
documentation. Permission to use, copy, and modify the
source code contained in this file is hereby granted, but
is limited to customers of SAS with a valid license for the
SAS software product with which this file was provided. 
Except as expressly set forth herein, the terms of such
license apply.
-----------------------------------------------------------
*/

*ProcessVariables;

%global StudyName InData AlleleVars WhereStmt ListMarkerVars ListTraitVars MarkerFmt GenoDelim TraitVar TraitType ClassVars 
FixedEff IntEff IntTestsOnly RandEff RandOpts ParmsStmt AnnoInData AnnoLabel AnnoMName AnnoGroup AnnoLoc AnnoAccession AnnoMajAllele AnnoWhereStmt  
QTests Odds DDFM UseLowerB Listing OutputLSM OutputResid OutputPred OutputFitStat Conv MTC  PValAdj Tau Bandwidth Alpha OnlySig OutPath ServerOutPath OutName MacroPath GMethod EstMethod
QVars ListQVars QSumto1 PedID KVars ListKVars CompressedK FixCovParms SpeedK 
MultiMarkerModel StandGeno Impute Priors SelectMethod Adaptive StopCriterion SLEntry SLStay CVPMethod CVKFold Seed MaxStep 
GLMSelectOptions GLMSelectModelOptions WeightVar lockin FitSelModelOnUnrotatedData SLStaySelModel SelModelProfile
QKMM_OutData QKMMScript;

%let lockin = 0;
%let Priors = Proportional;
 
*ProcessBody;

%macro QKMixedModel;

%include "&MacroPath./GenMacros.sas" / nosource;
%include "&MacroPath./SNPTraitAssocMacro.sas" / nosource;
%include "&MacroPath./MultipleTestingAdjustment.sas" / nosource;

%GenInit(QKMixedModel,28);
%if &exiterror=1 %then %goto exit;
%global FullAPName;
%let FullAPName=%str(Q-K Mixed Model);
	
%PrepOutDS(QKMM_OutData,QKMM_OutDataName,&OutName._qkm);

%PrepOutFile(QKMMScript,QKMMScriptName,&ProcessName Script for &OutName..jsl);

%global ByVars CensorVar;
%let ByVars=;
%let CensorVar=;
%let EventTrait=;
%let RefTrait=;
%let StrataVars=;

%if %length(&ListMarkerVars &AlleleVars)=0 %then %do;
 %put ERROR: Marker variables must be specified in either the Marker Variables or List-Style Specification of Marker Variables parameters.;
 %let exiterror=1;
 %goto exit;
%end;

%if %length(&ListTraitVars &TraitVar)=0 %then %do;
 %put ERROR: One or more trait variables must be specified in either the Trait Variables or List-Style Specification of Trait Variables parameters.;
 %let exiterror=1;
 %goto exit;
%end;

%if %upcase(&FixCovParms)=YES and (%upcase(&DDFM)=SATTERTHWAITE or %upcase(&DDFM)=KENWARDROGER) %then %do;
	%put ERROR: You cannot select SATTERTHWAITE or KENWARDROGER for the Denominator Degrees of Freedom Method when the Fix Covariance Parameters checkbox is checked.;
	%let exiterror=1;
	%goto exit;
%end;

*begin: S1139177 Luciano efficiency improvement*;
/*When there missing values among trait or model variables, then using the repeated statement will return wrong results.
Therefore, making SpeedK option unavailable until finding someway to circumvent the missingness issue.*/
%let SpeedK = No; 	
%if %upcase(&SpeedK)=YES %then %do;
	%if %upcase(&TraitType) ^= CONTINUOUS %then %do;
		%put ERROR: Fast Genetic Background Variance Estimation is available only for Continuous traits.;
		%let exiterror=1;
		%goto exit;	
	%end;
	%if %length(&PedID)>0 %then %do;
		%*let SpeedK=NO;
		%put ERROR: Fast Genetic Background Variance Estimation is not available when Pedigred ID Variables is specified.;
		%let exiterror=1;
		%goto exit;
	%end;
	%if %upcase(&DDFM)=BETWITHIN %then %do;
		%put ERROR: Selecting BETWITHIN for the Denominator Degrees of Freedom Method when the Fast Background Genetic Variance Estimation checkbox is checked may lead to zero degrees of freedom for some fixed effects. Please, return to the Model Variables tab and under the Advanced Model Options choose other Denominator Degrees of Freedom Method option.;
		%let exiterror=1;
		%goto exit;
	%end;
	
	%let UseLowerB=YES;
%end;
*end: S1139177 Luciano efficiency improvement*;

%global OldAlleleVars;
%let OldAlleleVars = &AlleleVars;
%let AlleleVars = &AlleleVars &ListMarkerVars;

%CheckGenotypeFmt;
%if &exiterror %then %goto exit;

%let TraitVar = &TraitVar &ListTraitVars;

%if (%length(&QTests)=0) %then %let QTests=Genotype Trend;
%if %upcase(&TraitType)=SURVIVAL %then %do;
	%let QTests=Trend;
	%let IntEff=;
%end;
%let geno=0;
%if (%index(%upcase(&QTests),GENOTYPE)>0) %then %let geno=1;
%let trend=0;
%if (%index(%upcase(&QTests),TREND)>0) %then %let trend=1;

%global SNPTraitAssoc_PEG SNPTraitAssoc_PET SNPTraitAssoc_CPG 
	SNPTraitAssoc_CPT SNPTraitAssoc_ORG SNPTraitAssoc_ORT 
	SNPTraitAssoc_OPG SNPTraitAssoc_OPT SNPTraitAssoc_LSM;
%let SNPTraitAssoc_PEG=;
%let SNPTraitAssoc_PET=;
%let SNPTraitAssoc_CPG=;
%let SNPTraitAssoc_CPT=;
%let SNPTraitAssoc_ORG=;
%let SNPTraitAssoc_ORT=;
%let SNPTraitAssoc_LSM=;

%if %upcase(&OutputLSM)=YES %then %let lsmi=1;
%else %let lsmi=0;

%if &geno=1 %then %do;
	%PrepOutDS(SNPTraitAssoc_PEG,SNPTraitAssoc_PEGName,&OutName._peg);
	%PrepOutDS(SNPTraitAssoc_CPG,SNPTraitAssoc_CPGName,&OutName._cpg);
	%if %upcase(&OutputResid)=YES %then %PrepOutDS(SNPTraitAssoc_ORG,SNPTraitAssoc_ORGName,&OutName._org);
	%if %upcase(&OutputPred)=YES %then %PrepOutDS(SNPTraitAssoc_OPG,SNPTraitAssoc_OPGName,&OutName._opg);
	%if &lsmi=1 %then %PrepOutDS(SNPTraitAssoc_LSM,SNPTraitAssoc_LSMName,&OutName._lsm);
%end;
%if &trend=1 %then %do;
	%PrepOutDS(SNPTraitAssoc_PET,SNPTraitAssoc_PETName,&OutName._pet);
	%PrepOutDS(SNPTraitAssoc_CPT,SNPTraitAssoc_CPTName,&OutName._cpt);
	%if %upcase(&OutputResid)=YES %then %PrepOutDS(SNPTraitAssoc_ORT,SNPTraitAssoc_ORTName,&OutName._ort);
	%if %upcase(&OutputPred)=YES %then %PrepOutDS(SNPTraitAssoc_OPT,SNPTraitAssoc_OPTName,&OutName._opt);
%end;

%PrepMapandBy;
run;
%if &exiterror=1 %then %goto exit;

%put AnnoLabel = &AnnoLabel;
%put AnnoGroup = &AnnoGroup;
%put AnnoLoc = &AnnoLoc;
%put AnnoMName = &AnnoMName;

%if %length(&ListQVars) %then %do;
	data qvars;
		set &ds(keep=&ListQVars obs=1);
	run;
	
   %let dsid=%sysfunc(open(qvars,i));
   %do i=1 %to %sysfunc(attrn(&dsid,nvars));
		%let QVars=&QVars %sysfunc(varname(&dsid,&i));
	%end;
	%let rc = %sysfunc(close(&dsid));
%end;	

%if %upcase(&QSumto1)=YES %then %do;
/* Remove last Q var since sum to 1 */
	%let OrigQVars = &QVars;
	%let QVars = ;
	%let w=1;
	%do %while(%length(%scan(&OrigQVars,%eval(&w+1))));
		%let QVars = &QVars %scan(&OrigQVars,&w);
		%let w=%eval(&w+1);
	%end;
	%if %length(&QVars)=0 %then %do;
		%let QVars = &OrigQVars;
		%put WARNING: There is only one Q Matrix variable selected so the Columns of Q Matrix Sum to 1 checkbox is ignored.;
	%end;
%end;

*S1360480: check if there are interaction fixed effects, which, for now, is not allowed in the multi-marker model*;
%if %upcase(&MultiMarkerModel)= YES and (%sysfunc(find(&FixedEff,*)) or %sysfunc(find(&FixedEff,|))) %then %do;
	%put ERROR: Multi-Marker Model Selection does not allow the specification of interaction effects in the Additional Fixed Effects field from the Model Selection tab. Please, remove any interaction effects present in the Additional Fixed Effects field.; 
	%let exiterror=1;
	%goto exit;
%end;

%let FixedEff = &QVars &FixedEff; /*ListQVars has already been included into QVars*/
%if %length(&ListKVars) %then %do;
	data Kvars;
		set &ds(keep=&ListKVars obs=1);
	run;
	
   %let dsid=%sysfunc(open(kvars,i));
   %do i=1 %to %sysfunc(attrn(&dsid,nvars));
	%let KVars= &KVars %sysfunc(varname(&dsid,&i));
   %end;
   %let rc = %sysfunc(close(&dsid));
%end;	

/* Table name from all but MIXED */
%global ParmEst EffName;
%let ParmEst=ParameterEstimates;
%let EffName=Effect;

%let WhichProc=MIXED;
%let TraitOpt= ;
%let Stat=F; 
%let Gtable=Tests3;
%let TStmt=contrast;
%let Ttable=Contrasts;
%let usestrata=0;
%let ModOption=%quote(/ htype=3 ddfm=&DDFM s); 
%if %upcase(&OutputResid)=YES or %upcase(&OutputPred)=YES %then %let ModOption = %quote(&ModOption residual outp=outres);
%let ClassOpt=;
%let teststmt=%str('genotypes' Genotype 1);
%let ProcOption= ;
%let ParmEst = SolutionF;
%let EffName = Effect;
%if %length(&PedID) %then %let ClassVars = &ClassVars &PedID;

/* Need a separate check for OR when using GLIMMIX since not creating OR data set */
%let AnyOdds=0;
%if %upcase(&TraitType)=COUNT %then %do;
	%let WhichProc=GLIMMIX; 
	%let ModOption=%quote(/ link=log dist=p htype=3 ddfm=&DDFM s);
	%if %upcase(&OutputResid)=YES and %upcase(&OutputPred)=YES %then %let ModOption = %quote(&ModOption) %str(; output out=outres resid=Resid predicted=Pred);
	%else %if %upcase(&OutputResid)=YES and %upcase(&OutputPred)^=YES %then %let ModOption = %quote(&ModOption) %str(; output out=outres resid=Resid);
	%else %if %upcase(&OutputResid)^=YES and %upcase(&OutputPred)=YES %then %let ModOption = %quote(&ModOption) %str(; output out=outres predicted=Pred);
	%let ParmEst = ParameterEstimates;
%end;
%else %if %upcase(&TraitType)=BINARY or %upcase(&TraitType)=NOMINAL or %upcase(&TraitType)=ORDINAL %then %do;
	%if %length(&RandEff)>0 or %length(&KVars)>0 %then %do;
		%let WhichProc=GLIMMIX;
		%let ProcOption=%str(Method=&GMethod);		
		%if %upcase(&TraitType)=NOMINAL %then %let TraitOpt=(Ref=FIRST);
 		%else %let TraitOpt=(Event=LAST);		
		%if %upcase(&EstMethod)=YES %then %let ProcOption=%quote(&ProcOption EMPIRICAL);
		%let Stat=F;     
		%let Gtable=Tests3;	
		%let TStmt=estimate;				
		%let Ttable=Estimates;		
		%let ParmEst = ParameterEstimates;
		%if %upcase(&DDFM)=CONTAIN or %upcase(&DDFM)=BETWITHIN %then %let DDFM=RESIDUAL;
		%if %upcase(&TraitType)=BINARY %then %let ModOption=%quote(/ link=logit dist=binary htype=3 ddfm=&DDFM s);
		%else %if %upcase(&TraitType)=NOMINAL %then %do;
			%*let ModOption=%quote(/ link=glogit dist=mult htype=3 ddfm=betwithin s);
			%let ModOption=%quote(/ link=glogit dist=mult htype=3 ddfm=&DDFM s);
			%let ClassVars = &ClassVars &TraitVar;
		%end;
		%else %if %upcase(&TraitType)=ORDINAL %then %let ModOption=%quote(/ link=clogit dist=mult htype=3 ddfm=&DDFM s);
		%if %upcase(&OutputResid)=YES and %upcase(&OutputPred)=YES %then %let ModOption = %quote(&ModOption) %str(; output out=outres resid=Resid predicted=Pred);
		%else %if %upcase(&OutputResid)=YES and %upcase(&OutputPred)^=YES %then %let ModOption = %quote(&ModOption) %str(; output out=outres resid=Resid);
		%else %if %upcase(&OutputResid)^=YES and %upcase(&OutputPred)=YES %then %let ModOption = %quote(&ModOption) %str(; output out=outres predicted=Pred);
		%let teststmt=%str('genotypes' Genotype 1);		
		%if %upcase(&TraitType)=NOMINAL or %upcase(&TraitType)=ORDINAL %then %let lsmi=0;
	%end;
	%else %do;
		%let WhichProc=LOGISTIC;
		%let TraitOpt=(Event=LAST);		
		%let Stat=ChiSq;
		%*let Gtable=Type3;
		%let Gtable=ModelANOVA; /*S1146207: The logistic output formerly named Type3 is now named ModelANOVA*/
		%let TStmt=contrast;
		%let Ttable=ContrastTest;
		%let Otable=OddsRatiosWald;
		%let usestrata=1;
		%if %upcase(&TraitType)=NOMINAL %then %let ModOption=%str(/ link=glogit);
		%else %if %length(StrataVars)>0 %then %let ModOption=/ GCONV=1E-6; 
		%else %let ModOption =;
		%if %upcase(&OutputResid)=YES and %upcase(&OutputPred)=YES %then %let ModOption = %quote(&ModOption) %str(; output out=outres resdev=Resid predicted=pred);
		%else %if %upcase(&OutputResid)=YES and %upcase(&OutputPred)^=YES %then %let ModOption = %quote(&ModOption) %str(; output out=outres resdev=Resid);
		%else %if %upcase(&OutputResid)^=YES and %upcase(&OutputPred)=YES %then %let ModOption = %quote(&ModOption) %str(; output out=outres predicted=pred);
		%let ClassOpt = %str(/ cprefix=8);
		%let teststmt=%str('genotypes' Genotype 1);
		%let ParmEst = ParameterEstimates;
		%let EffName = Variable;
		%let lsmi=0;
	%end;
%end; 

%if %upcase(&OutputLSM)=YES and &lsmi=0 %then %do;
	%put LS means cannot be calculated for this model.;
%end;

%if &usestrata=0 %then %let StrataVars=;

/*S1089309
%if %upcase(&UseLowerB)=YES %then %do;
*/
%if (%upcase(&WhichProc)= MIXED or %upcase(&WhichProc)= GLIMMIX) %then %do;
	%if %upcase(&UseLowerB)=YES %then %do;
		%if %length(&ParmsStmt) %then %do;
			%if %index(&ParmsStmt,%str(/)) %then %do;
				%let ParmsStmt = %quote(&ParmsStmt lowerb=0);
			%end;
			%else %do;
				%let ParmsStmt = %quote(&ParmsStmt / lowerb=0);
			%end;
		%end;
		%else %let ParmsStmt = %str( / lowerb=0);
	%end;
%end;
%else %let ParmsStmt=;

%let CreateHTML= NO;
%if %upcase(&MultiMarkerModel)= YES %then %do;
	/*
	Reference: Rakitsch, B., Lippert, C., Stegle, O., and Borgwardt, K. (2013) Lasso multi-marker mixed model 
	for association mapping with population structure correction. Genetics 29(2): 206--214.
	*/
	%put PedID= &PedID;

	%let OrigListing = &Listing;
	%let Listing = YES;

	%if %upcase(&MarkerFmt) = %upcase(Alleles) %then %do;
		%put ERROR: Multi-Marker Model Selection is not available for SNP variable in the format of alleles. Please, use the Recode Genotype application process under Genetics Utilities menu to recode the SNP variables.;
		%let exiterror=1;
		%goto exit;	
	%end;
	
	%if %upcase(&TraitType) ^= CONTINUOUS %then %do;
		%put ERROR: Multi-Marker Model Selection is available only for Continuous traits.;
		%let exiterror=1;
		%goto exit;	
	%end;

	%if %length(&AlleleVars &ListMarkerVars &FixedEff) = 0 %then %do;
		**ListQVars has already been included into QVars, and QVars was included inot FixedEff*;
		%put ERROR: No variables were specified for selection. Variables must be specified in at least one of the following fields: SNP Variables, List-Stype Specification of SNP Variables, Q-Matrix Variables, List-Stype Q-Matrix Variables, Additional Fixed Effects.;
		%let exiterror=1;
		%goto exit;	
	%end;
	
	**TraitVar has already been included into QVars*;
	data traitdata_;
		set &ds(keep= &TraitVar);
		where 0;
	run;
		
	%let _dsdt_= %sysfunc(open(traitdata_));
	%let _ntr_ = %sysfunc(attrn(&_dsdt_, NVARS));
	%put _ntr_ = &_ntr_;
	%if &_ntr_ > 1 %then %do;
		%put ERROR: Multi-Marker Model Selection is available for one Continuous traits at a time. Please, return to the General Tab and specify only one trait variable.;
		%let exiterror=1;
		%goto exit;
	%end;
	%let _dsdt_= %sysfunc(close(&_dsdt_));

	/*Call SNPTraitAssocMacro.sas macro to collect genetic and residual variance for all traits (fixcp_all data set). 
	Also, collecting numeric genotypic (NumGenoData data set), phenotypic and covariates variables (traits data set)*/
	%let OrigFixCovParms = &FixCovParms;
	%let OrigSpeedK = &SpeedK;
	%let OrigParmsStmt = &ParmsStmt;
	%let FixCovParms = YES;
	%let SpeedK = NO;
	
	%global GetCovParms GetNumGeno;
	%let GetNumGeno = 1; /*1 means numeric genotypes will be collected when running SNPTraitAssocMacro.sas*/
	%put GetCovParms = &GetCovParms;
	%put GetNumGeno = &GetNumGeno;
		
	%if %length(&KVars) %then %do;
		%let GetCovParms = 1; /*1 means covariance parameters will be collected for the null model when running SNPTraitAssocMacro.sas*/
		data fixcp_all;
			set _null_;
			length _traitvar_ $32;
		run;
		
		%SNPTraitAssocMacro(kmodel=1);
		%if &exiterror=1 %then %goto exit;
		
		%let CovParamData = fixcp_all;
		/*data outlib.CovParamData; set &CovParamData; run;*/
			
	%end;
	%else %do;
		%let GetCovParms = 0; /*1 means covariance parameters will be collected for the null model when running SNPTraitAssocMacro.sas*/
		%SNPTraitAssocMacro(kmodel=0);
		%if &exiterror=1 %then %goto exit;
	%end;
	
	**GetCovParms and GetNumGeno must be removed at this point to reset GenMacros.sas and SNPTraitAssocMacro.sas back to their original status*;
	%symdel GetCovParms GetNumGeno;
	
	%let TransAlleleNames = trans;
	%let NumGenoData = NumGenoData;
	%let PhenoCovData = traits;
	/*data outlib.TransAlleleNames; set &TransAlleleNames; run;*/
	/*data outlib.NumGenoData; set &NumGenoData; run;*/
	/*data outlib.PhenoCovData; set &PhenoCovData; run;*/

	data &NumGenoData;
		set &NumGenoData;
		by _order_;
		if first._order_ then _indIDtmp_= 1;
		else _indIDtmp_ = _indIDtmp_ + 1;
		retain _indIDtmp_;
	run;
	
	proc sort data=&NumGenoData overwrite;
		by _indIDtmp_ _order_;
	run;
	
	%let mrkprefix= _mrk_;
	proc transpose data= &NumGenoData out=NumGenoNoMiss(drop=_name_ _indIDtmp_) prefix= &mrkprefix;
		var genotype;
		by _indIDtmp_;
		id _order_;
	run;
	/*data outlib.NumGenoNoMiss0; set NumGenoNoMiss; run;*/
	
	%if %upcase(&FitSelModelOnUnrotatedData) ^= YES %then %do;
		proc datasets nolist;
			delete &NumGenoData;
		quit;
	%end;	
	
	**Collect snp variable names **;
	data snpnames;
		set &ds(keep=&AlleleVars);
		where 0;
	run;
	/*data outlib.snpnames0; set snpnames; run;*/
	
	%NObsVars(snpnames);
	%let NumbSNPin= &nvars;
	
	proc transpose data= snpnames out=snpnames;
		var &AlleleVars;
	run;
	/*data outlib.snpnames; set snpnames; run;*/
	
	data tmpnames;
		set NumGenoNoMiss(keep= &mrkprefix:);
		where 0;
	run;
	
	proc transpose data= tmpnames out=tmpnames;
		var &mrkprefix:;
	run;
	/*data outlib.tmpnames; set tmpnames; run;*/
	
	data snptmpnames;
		merge snpnames(rename=(_name_=_orgsnp)) tmpnames(rename=(_name_=_tmpsnp));
	run;
	/*data outlib.snptmpnames; set snptmpnames; run;*/
	
	**Standardizing Genotypes*;
	%if %length(&Impute) = 0 %then 
		%let Impute = NO;
	%if %upcase(&StandGeno) = YES %then %do;
	
		proc transpose data=NumGenoNoMiss out=g1t;
		run;

		data g1t;
			set g1t;
			array av{*} Col:;
			_xbar_=mean(of Col:);
			_std_=sqrt(_xbar_*(1-_xbar_/2));
			do i=1 to dim(av);
				if _std_>0 then av{i}=(av{i}-_xbar_)/_std_;
				else av{i}=.;
				%if %upcase(&Impute) = YES %then %do;
					if missing(av{i}) then av[i] = 0;
				%end;
			end;			
		run;

		proc transpose data=g1t out=NumGenoNoMiss(drop=_name_);
			var Col:;
		run;
		/*data outlib.NumGenoNoMiss1; set NumGenoNoMiss; run;*/
		
		proc datasets nolist;
			delete g1t;
		quit;

	%end;
	
	**identifying observations with at least one missing value across markers*;
	%put StandGeno Impute = &StandGeno &Impute;
	%if %upcase(&StandGeno) ^= YES or (%upcase(&StandGeno) = YES and %upcase(&Impute) ^= YES) %then %do;
		data NumGenoNoMiss; 
			set NumGenoNoMiss;
			_NumGeno_obs = _n_;
			_missgeno = 0;
			array _mrk{*} _numeric_;
			do _i= 1 to dim(_mrk);
				if missing(_mrk[_i]) then do;
					_missgeno = 1;
					_mrk[_i] = 0; /*replace missing value with zero in order to avoid error when rotating data in PROC IML. After rotating data, missing values will be put back*/
				end;
			end;
			drop _i;
		run;
		/*data outlib.NumGenoNoMiss2; set NumGenoNoMiss; run;*/	
	%end;
	%else %do;
		data NumGenoNoMiss; 
			set NumGenoNoMiss;
			_NumGeno_obs = _n_;
			_missgeno = 0;
		run;		
	%end;
	
	**collecting names of numeric genotype variables*;
	data _NumGenoNames_;
		set NumGenoNoMiss(drop= _NumGeno_obs _missgeno);
		where 0;
	run;
	
	proc transpose data= _NumGenoNames_ out= _NumGenoNames_t;
	run;
	
	data _null_;
		set _NumGenoNames_t end= _eof_;
		call symputx("_gv"||trim(left(_n_)), _name_);
		if _eof_ then call symputx("_totgv", trim(left(_n_)));
	run;	
	%put _totgv = &_totgv;
	/*data outlib._NumGenoNames_t; set _NumGenoNames_t; run;*/
	
	**identifying observations with missing values for each trait*;		
	data TraitsNoMiss; 
		set &PhenoCovData;
		_traits_obs = _n_;
		_misstrait = 0;
		if missing(&TraitVar) then do;
			_misstrait = 1;
			&TraitVar = 0; /*replace missing value with zero in order to avoid error when rotating data in PROC IML. After rotating data, missing values will be put back*/
		end;
		drop _i &FixedEff &KVars;
	run;	
	/*data outlib.TraitsNoMiss0; set TraitsNoMiss; run;*/
	
	/*identifying observations with missing values for ClassVars and FixedEff variables.
	Every variable in ClassVars must be in the FixedEff list to be fitted into the model. 
	Therefore, it is enough to check whether FixedEff list has any variable*/
	%let clvsplit = 0; /*1 means that class variables were split, 0 otherwise*/
	%if %length(&FixedEff) %then %do;
			
		/*Using PROC GLMSELECT to build the design matrix of fixed effect variables model without the intercept. 
		The intercept variable will be create within the proc iml code that fallows.*/
		ods results off;
		ods listing close;
		proc glmselect data=&PhenoCovData outdesign(fullmodel)=FixedXNoMiss;
			*LAR, LASSO, and ElasticNet, class vars are split, so use class statement to create design matrix with split variables*;
			%*if %length(&ClassVars) and ((&SelectMethod=LAR) or (&SelectMethod=LASSO) or (&SelectMethod=ElasticNet)) %then %do;
				class &ClassVars;
			%*end;
			model &TraitVar= &FixedEff / selection=lasso(stop=none) steps=1 noint;
		run;	
		/*data outlib.FixedXNoMiss0; set FixedXNoMiss; run;*/
		
		data FixedXNoMiss;
			set FixedXNoMiss(drop=&TraitVar);
			_FixedXMat_obs = _n_;
			_missFixedXMat = 0;
			array _fix{*} _numeric_;
			do _i= 1 to dim(_fix);
				if missing(_fix[_i]) then do;
					_missFixedXMat = 1;
					_fix[_i] = 0; /*replace missing value with zero in order to avoid error when rotating data in PROC IML. After rotating data, missing values will be put back*/
				end;
			end;
			drop _i;				
		run;
		/*data outlib.FixedXNoMiss1; set FixedXNoMiss; run;*/
		
		**checking if FixedEff variables are ClassVars, if so set clvsplit to 1, indicating that fixed effect variables were split*;
		%*if %length(&ClassVars) and ((&SelectMethod=LAR) or (&SelectMethod=LASSO) or (&SelectMethod=ElasticNet)) %then %do;
		%if %length(&ClassVars) %then %do;
			%let j = 1;
			%let clv = %scan(&ClassVars,&j);
			%do %while(%length(&clv) and &clvsplit = 0);
				%let i = %index(&FixedEff,&clv);
				%if &i %then %do;
					%let clvsplit = 1;	
				%end;
				%let j = %eval(&j+1);
				%let clv = %scan(&ClassVars,&j);
			%end;
		%end;	
	%end;
	%put clvsplit = &clvsplit;
			
	**collecting names of coded FixedEff variables*;
	%if %length(&FixedEff) %then %do;
		data _fixedXnames_;
			set FixedXNoMiss(drop= _FixedXMat_obs _missFixedXMat);
			where 0;
		run;
		
		proc transpose data= _fixedXnames_ out= _fixedXnames_t;
		run;
				
		data _null_;
			set _fixedXnames_t end= _eof_;
			call symputx("_fv"||trim(left(_n_)), _name_);
			if _eof_ then call symputx("_totfv", trim(left(_n_)));
		run;
		/*data outlib._fixedXnames_t; set _fixedXnames_t; run;*/
	%end;
	
	%if %length(&KVars) %then %do;
		**Because the expected input KVars are in fact square root variables, we need to reconstruct the original K matrix for the repeated statement in proc mixed.*;
		data sqrtKmat;
			set &PhenoCovData(keep=&KVars);
		run;
	
		proc iml;
			use sqrtKmat;
			read all var _num_ into sqrt;
			Kmt= sqrt*sqrt`;
			create Kmat from Kmt;
			append from Kmt; 
		quit;
	%end;
	
	%*goto exit;
		
	**YesIntercept equals to 1 means that a model with intercept is fitted, 0 otherwise*;
	%let YesIntercept = 1;
	
	**In the presence of KVars, the orginal data set must be rotated*;
	%if %length(&KVars) %then %do;
		**Collecting background genetic variance and residual variance from the null model*;
		%let varG= ;
		%let varE= ;
		%let numcp= 2;
		data _null_;
			set &CovParamData end=_l;
			where _traitvar_= "&TraitVar"; 
			if upcase(trim(left(CovParm)))= 'VARIANCE' then call symput("varG",strip(Estimate));
			if upcase(trim(left(CovParm)))= 'RESIDUAL' then call symput("varE",strip(Estimate));
			if _l then call symputx("numcp",_n_);
		run;
		%put varG= &varG; 
		%put varE= &varE;
		%put numcp= &numcp;
		
		%if &varG = 0 %then %do;
			%put ERROR: Trait &TraitVar has estimate background genetic variance equal to zero. Therefore, the Multi-Marker Model Selection procedure cannot be applied to this trait.;
			%let exiterror= 1;
			%goto exit;
		%end;
	
		**rotating Trait variable, FixedEff variables, SNP variables*;
		proc iml; 
			reset fuzz=0.00001; 
			use Kmat;
			read all var _num_ into K;
			call svd(U,D,V,K); 
			Di = diag(D); 
			*print 'K';
			*print K;
			*print 'U';
			*print U;
			*print 'Di';
			*print Di;
			*print 'V';
			*print V; 
			*check = U*Di*V`; 
			*print 'check';
			*print check; 
			*P = U*Di; 
			*UPrU=U`*U; /*identity*/
			*VVPr=V*V`; /*identity*/
			*print 'P';
			*print P;
			*print 'UPrU';
			*print UPrU;
			*print 'VVPr';
			*print VVPr; 
	
			nc=ncol(K);
			*print nc;
			delta= &varE/&VarG; 
			*print(delta);
			DiPlusDeltaI= Di + delta*I(nc);
			RotMat= inv(sqrt(DiPlusDeltaI))*U`;
			/*
			create Umat from U;
			append from U;
			create Dmat from Di;
			append from Di;
			create RotMat from RotMat;
			append from RotMat;
			*/
			
			**rotating trait variable*;
			use TraitsNoMiss;
			read all var {&TraitVar} into Y;
			YRot= RotMat*Y;
			/*
			VarYRot= var(YRot)*(nc-1)/nc;
			VarY= var(Y)*(nc-1)/nc;
			print VarYRot VarY;
			*/
			create TraitNoMiss_Rot from YRot[colname="&TraitVar"] ;
			append from YRot;
			
			**rotating numeric genotype variables*;
			use _NumGenoNames_t;
			read all var _char_ into _genonames_t;
			_genonames_= _genonames_t`;
	
			use NumGenoNoMiss;
			read all var _genonames_ into S;
			SRot= RotMat*S;
			create NumGenoNoMiss_Rot from SRot[colname=_genonames_];
			append from SRot;
			
			**rotating fixed effect variables*;
			%if %length(&FixedEff) %then %do;
				use _fixedXnames_t;
				read all var _char_ into _fixedXnames_t;
				_fixedXnames_= _fixedXnames_t`;					
			
				use FixedXNoMiss;
				read all var _fixedXnames_ into S;
				SRot= RotMat*S;
				create FixedXNoMiss_Rot from SRot[colname=_fixedXnames_];
				append from SRot;
			%end;
	
			**rotating intercept*;
			%if &YesIntercept = 1 %then %do;
				IntRot= RotMat*J(nc,1,1);
				create InterceptNoMiss_Rot from IntRot[colname='Intercept'];
				append from IntRot;
			%end;
		quit;
	
		**merging rotated numerical trait, numerical FixedEff, and numerical marker variables into a single data set*;
		data Data_Rot;
			merge %if &YesIntercept = 1 %then InterceptNoMiss_Rot;
				TraitNoMiss_Rot
				TraitsNoMiss(keep= _misstrait)
				%if %length(&FixedEff) %then %do;
					FixedXNoMiss_Rot
					FixedXNoMiss(keep= _missFixedXMat)
				%end;
				NumGenoNoMiss_Rot
				NumGenoNoMiss(keep= _missgeno)
				;
				/*setting any missing values on the unrotated data set to missing in the rotated data set*/
				if _misstrait or _missgeno %if %length(&FixedEff) %then or _missFixedXMat; then do;
					&TraitVar = . ;
				end;	
		run;
		%let Data_GLMSEL = Data_Rot;
		/*data outlib.Data_GLMSEL; set &Data_GLMSEL; run;*/
		
		proc datasets nolist;
			delete %if &YesIntercept = 1 %then InterceptNoMiss_Rot;
				TraitNoMiss_Rot
				TraitsNoMiss
				%if %length(&FixedEff) %then %do;
					FixedXNoMiss_Rot
					FixedXNoMiss
				%end;
				NumGenoNoMiss_Rot
				NumGenoNoMiss
			;
		quit;
		
	%end;
	%else %do;
		**In the absense of KVars, the orginal data do not need to be rotated*;
		**merging original numerical trait, numerical FixedEff, and numerical marker variables into a single data set*;
		data Data_Orig;
			merge TraitsNoMiss
				%if %length(&FixedEff) %then %do;
					FixedXNoMiss
				%end;
				NumGenoNoMiss
				;
			**set any missing values on the unrotated data set to missing in the rotated data set*;
			if _misstrait or _missgeno %if %length(&FixedEff) %then or _missFixedXMat; then do;
				&TraitVar = . ;
			end;
			%if &YesIntercept = 1 %then Intercept = 1;;
		run;
		%let Data_GLMSEL = Data_Orig;
		/*data outlib.Data_GLMSEL; set &Data_GLMSEL; run;*/
		
		proc datasets nolist;
			delete TraitsNoMiss
				%if %length(&FixedEff) %then %do;
					FixedXNoMiss
				%end;
				NumGenoNoMiss
				;
		quit;
	%end;
		
	**model selection via PROC GLMSELECT*;
	%if %length(&KVars) %then %do;
		title2 "Variable Selection on the Rotated Data";
	%end;
	%else %do;
		title2 "Variable Selection on the Unrotated Data";
	%end;
	ods results on;
	ods listing;
	
	%if %upcase(&CreateHTML)=YES %then %do;
		* setup for html output;
		%global QKMixedModel_body QKMixedModel_frame QKMixedModel_contents;
		%PrepHTML(QKMixedModel_body, QKMixedModel_frame, QKMixedModel_contents, &OutName._&ProcessName._body.html, &OutName._&ProcessName._frame.html, &OutName._&ProcessName._contents.html);
		ods html 
		body = "&QKMixedModel_Body" (url="&bodyfile")
		frame = "&QKMixedModel_Frame"  (url="&framefile")
		contents = "&QKMixedModel_Contents" (url="&contentsfile")
		;
		ods graphics;
	%end;
	
	proc glmselect data=&Data_GLMSEL
		%if %length(&Seed) %then %do;
			seed=&Seed
		%end;
		&GLMSelectOptions namelen=65
		%if %upcase(&CreateHTML)=YES %then %do;
			%if &SelectMethod ^= None %then %do;
				plots=all
			%end;
		%end;
		;
		model &TraitVar = 
			%if &YesIntercept = 1 %then Intercept;
			%do i=1 %to &_totgv;
				&&_gv&i
			%end;					
			%if %length(&FixedEff) %then %do;
				%do i=1 %to &_totfv;
					&&_fv&i
				%end;
			%end;
			/ noint
			selection=&SelectMethod 
				%if &SelectMethod ^= None %then %do;
					(
						%if &StopCriterion = None %then %do;
							%if (&SelectMethod^=LAR) and (&SelectMethod^=LASSO) and (&SelectMethod^=ElasticNet) %then %do;
								select=SL 
								%if (&SelectMethod=Forward) or (&SelectMethod=Stepwise) %then %do;
									slentry=0.99
								%end;
								%if (&SelectMethod=Backward) or (&SelectMethod=Stepwise) %then %do;
									slstay=0.99
								%end;
							%end;
							stop=&MaxStep
						%end;
						%else %do;
							stop=&StopCriterion maxstep=&MaxStep 
							%if (&SelectMethod^=LAR) and (&SelectMethod^=LASSO) and (&SelectMethod^=ElasticNet) %then %do;
								select=&StopCriterion
							%end;
							%else %do;
								%if (&Adaptive = Yes) %then %do;
									ADAPTIVE
								%end;
								%if (&StopCriterion=PRESS) %then %do;
									LSCOEFFS
								%end;
							%end;
							%if (&StopCriterion=SL) %then %do;
								%if (&SelectMethod=Forward) or (&SelectMethod=Stepwise) %then %do;
									slentry=&SLEntry
								%end;
								%if (&SelectMethod=Backward) or (&SelectMethod=Stepwise) %then %do;
									slstay=&SLStay
								%end;
							%end;
							%if (&SelectMethod=Forward) or (&SelectMethod=Backward) or (&SelectMethod=Stepwise) %then %do;
								include=&lockin
							%end;
						%end;
					) 
				%end;
				%if (&StopCriterion=CV) and ^%index(%upcase(&GLMSelectModelOptions),CVMETHOD) %then %do;
					cvmethod=&CVPMethod ( &CVKFold )
				%end;
				&GLMSelectModelOptions 
		;
		
		%if %length(&WeightVar) %then %do;
			weight &WeightVar;
		%end;
		%else %if (&Priors ^= Proportional) %then %do;
			weight _Prior;
		%end;
		*score data=&Data_GLMSEL out=ts2o pred;
		ods output %if &SelectMethod ^= None %then selectedeffects=se_sel; parameterestimates=pe_sel;
	run;
	%if %upcase(&CreateHTML)=YES %then %do;
		ods html close;
	%end;
	/*data outlib.glm_se_sel; set se_sel; run;*/
	/*data outlib.glm_pe_sel; set pe_sel; run;*/
			
	**SelVars is used withing UpdateSelVars and UnsplitSelVars macros, which are defined within PredMacros.sas*;
	%global SelVars;
	%let SelVars = ;
	
	**Macro functions UpdateSelVars and UnsplitSelVars are defined within PredMacros.sas*;
	%include "&MacroPath./PredMacros.sas" / nosource;
		
	**create ts2vars data set holding selected variables*;
	%if &SelectMethod ^= None %then %do;		
		*ts2vars data set will be used by the UpdateSelVars macro*;
		%let InterceptSelected = 0;
		
		data ts2vars;
			length name $ 32;
			set se_sel;
			i = 1;
			do while(i);
				name = scan(effects,i,' ');
				if lengthn(name)=0 then i = 0;
				else do;
					if name ne "Intercept" then output;
					else call symputx("InterceptSelected", 1);
					i = i + 1;
				end;
			end;
			drop i label effects;
		run;
		
		*create new list of select variables*;
		%UpdateSelVars;
		%put SelVars = &SelVars;
	
		* for LAR, LASSO, and ElasticNet, class vars are split, so adjust names here;
		%let SelVarsUnSplit = &SelVars;
		%if (&clvsplit) %then %do;
			data ts2;
				merge &Data_GLMSEL(keep= %do i=1 %to &_totgv; &&_gv&i %end;) &PhenoCovData;
			run;
			%let ts2 = ts2;
		
			%UnsplitSelVars;
		%end;	
		
		/*
		%if %length(&SelVars)=0 %then %do;
			**zero variables selected*;
			%put WARNING: No variables selected for trait &TraitVar. An intercept-only model is assumed. You may want to try relaxing the selection criteria or using a different model selection method.;
			%let SelVars = None;
		%end;
		*/
	%end;
	%else %do;
		**SelVars will be a list of all input variables, since not variable selection was carried out.*;
		%let SelVars = ;
		%do i=1 %to &_totgv;
			%let SelVars = &SelVars "&&_gv&i";
		%end;
		%let SelVars = &SelVars &FixedEff; /*QVars are already included into FixedEff*/
		
		**ListQVars has already been included into QVars, and QVars are already included into FixedEff*;
		data QVarsFixVars_Names;
			set &ds(keep= &FixedEff);
			where 0;
		run;
		
		proc transpose data=QVarsFixVars_Names out= QVarsFixVars_Namest;
		run;
		/*data outlib.QVarsFixVars_Namest; set QVarsFixVars_Namest; run;*/
		
		data ts2vars;
			length name $ 32;
			%do i=1 %to &_totgv;
				name= "&&_gv&i";
				output;
			%end;
		run;
		
		data ts2vars;
			set ts2vars QVarsFixVars_Namest(rename=(_name_=name));
		run;
	%end;
	/*data outlib.ts2vars; set ts2vars; run;*/
	
	proc datasets nolist;
		delete &Data_GLMSEL PhenoCovData;
	quit;
	
	%if %length(&SelVars)=0 %then %do;
		%put WARNING: No variables selected for trait &TraitVar.. You may want to try relaxing the selection criteria or using a different model selection method.;
		%goto skip_0;
	%end;
	%else %do;
	
		**creating global variables needed in the SNPTraitAssocMacro.sas when FitSelModelOnUnrotatedData = YES*;
		%global selMarkerVars_g selmarkervarsorder_g selFixedEff_g
			selMarkerVars_t selmarkervarsorder_t selFixedEff_t;
			
		**SelOrigAlleleVars holds a subset of selected AlleleVars*;
		%let SelOrigAlleleVars = ; 
		%let SelOrigAlleleVars_g = ;
		%let SelOrigAlleleVars_t = ;
		%let SelMarkerVars = ;
		%let selMarkerVars_g = ;
		%let selMarkerVars_t = ;
		%let selMarkerVarsOrder = ;
		%let selMarkerVarsOrder_g = ;
		%let selMarkerVarsOrder_t = ;
		%let SelOrigFixedVars = ;
		%let selFixedEff_g = ;
		%let selFixedEff_t = ;
			
		proc sort data=ts2vars nodupkey;
			by name;
		run;

		data ts2vars;
			set ts2vars end=_eof_;
			_idmrk_= find(trim(left(name)),"&mrkprefix");
			_ixmrk_= index(trim(left(name)),"&mrkprefix");
			if _n_ = 1 then do;
				totselmrk = 0;
				totselfix = 0;
			end;
			if _idmrk_ = 1 and _ixmrk_ = 1 then do;
				_order_s = input(substr(trim(left(name)),length("&mrkprefix") + 1), best12.);
				totselmrk = totselmrk + 1;
				retain totselmrk;
			end;
			else do;
				totselfix = totselfix + 1;
				retain totselfix;
				call symputx("_selfix"||trim(left(totselfix)), trim(left(name)));
			end;
			if _eof_ then do;
				call symputx("totselmrk", trim(left(totselmrk)));
				call symputx("totselfix", trim(left(totselfix)));
			end;
		run;
		/*data outlib.ts2vars1; set ts2vars; run;*/
		
		**SelOrigFixedVars holds a subset of selected QVars and FixedEff variables*;
		%let SelOrigFixedVars = ;
		%do i=1 %to &totselfix;
			%let SelOrigFixedVars = &SelOrigFixedVars &&_selfix&i;
		%end;	
		
		%if &totselmrk > 0 %then %do;
			**collect list of selected marker variables names as specified in the input data set*;

			data OrigAllelesNames;
				set &TransAlleleNames(keep=_name_);
				if _n_ = 1 then _order_ = 0;
				%if %qupcase(&MarkerFmt)=%str(ALLELES) %then %do;
					if mod(_n_ / 2, 1) ^= 0 then do;
						_order_ = _order_ + 1;
						_allele_ = 1;
						retain _order_;
					end;
					else do;
						_allele_ = 2;
					end;
				%end;
				%else %do;
					_order_ = _n_;
				%end;
			run;
			/*data outlib.OrigAllelesNames; set OrigAllelesNames; run;*/			
			
			proc datasets nolist;
				delete TransAlleleNames;
			quit;

			proc sql UNDO_POLICY= NONE;
				create table selOrigAlleles as
				select a.*, b.*
				from ts2vars as a, OrigAllelesNames as b
				where a._order_s = b._order_
				order by _order_ %if %qupcase(&MarkerFmt)=%str(ALLELES) %then %do; , _allele_ %end;;
			quit;
			/*data outlib.selOrigAlleles; set selOrigAlleles; run;*/

			proc sql UNDO_POLICY= NONE noprint;
				select _name_ into :SelOrigAlleleVars separated by ' '
				from selOrigAlleles;
			quit;

			**must use nodupkey when MarkerFmt is Allele*;
			proc sort data= selOrigAlleles out=selMarkerVarsOrder nodupkey;
				by _order_;
			run;

			proc sql UNDO_POLICY= NONE noprint;
				select _order_ into :selMarkerVarsOrder separated by ' '
				from selMarkerVarsOrder;
				select name into :SelMarkerVars separated by ' '
				from selMarkerVarsOrder;
			quit;
			
			%if &geno=1 %then %do;
				%let SelOrigAlleleVars_g = &SelOrigAlleleVars;					
				%let selMarkerVarsOrder_g = &selMarkerVarsOrder;
				%let selMarkerVars_g = &selmarkervars;		
			%end;

			%if &trend=1 %then %do;
				%let SelOrigAlleleVars_t = &SelOrigAlleleVars;					
				%let selMarkerVarsOrder_t = &selMarkerVarsOrder;
				%let selMarkerVars_t = &selmarkervars;		
			%end;
		%end;
		%else %do;
			**zero marker variables selected*;
			%put WARNING: No marker variables selected for trait &TraitVar. You may want to try relaxing the selection criteria or using a different model selection method.;			
		%end;
		
		%put SelOrigAlleleVars = &SelOrigAlleleVars; 
		%put SelOrigAlleleVars_g = &SelOrigAlleleVars_g;
		%put SelOrigAlleleVars_t = &SelOrigAlleleVars_t;
		%put SelMarkerVars = &SelMarkerVars;
		%put selMarkerVars_g = &selMarkerVars_g;
		%put selMarkerVars_t = &selMarkerVars_t;
		%put selMarkerVarsOrder = &selMarkerVarsOrder;
		%put selMarkerVarsOrder_g = &selMarkerVarsOrder_g;
		%put selMarkerVarsOrder_t = &selMarkerVarsOrder_t;			

		**Fitting selected model using unrotated data*;
		%if %upcase(&FitSelModelOnUnrotatedData) = YES %then %do;
			%put SLStaySelModel= &SLStaySelModel;
			%let Alpha= &SLStaySelModel;

			%let GenoTesterror = 0;
			%let TrendTestError = 0;

			**creating new ds data set to be used in the SNPTraitAssocMacro.sas*;
			proc transpose data= &NumGenoData out=NumGenoSelMarkers(keep= &selMarkerVars _indIDtmp_) prefix= &mrkprefix;
				var genotype;
				by _indIDtmp_;
				id _order_;
			run;
			
			%if %sysfunc(exist(&NumGenoData)) %then %do; 
				proc datasets nolist;
					delete &NumGenoData;
				quit;
			%end;
			/*data outlib.NumGenoSelMarkers; set NumGenoSelMarkers; run;*/

			**creating new ds by merging selected numeric markers to ds. Needed because input marker format might not be numeric*;
			data newds;
				merge &ds NumGenoSelMarkers(drop= _indIDtmp_);
				_IndivID_=_n_;
			run;
			%let ds = newds;

			/*if FitSelFixedEff = Yes, then selected fixed effect will be used in the genotype and/or trend. Otherwise, all input fixed effects will be used.*/
			%let FitSelFixedEff = YES; 
			%let OrigFixedEff = &FixedEff;
			%if %upcase(&FitSelFixedEff) = YES %then %do;
				%let FixedEff = &SelOrigFixedVars;
			%end;
			%else %do;
				%let FixedEff = &OrigFixedEff;
			%end;
			%put FixedEff = &FixedEff;

			**creating selFixedEff_g and selFixedEff_t for the genotype and trend tests, also emptyting FixedEff*;
			%let selFixedEff_g = &FixedEff;
			%let selFixedEff_t = &FixedEff;
			%let FixedEff = ;

			/*S1158198: removing non-significant fixed effects*/
			/*
			**if FitSameSelFixedEff = Yes, then the same selected fixed effect will be used in both the genotype and/or trend. Otherwise, different subsets of selected fixed effects will be used.*;
			%if %upcase(&FitSelFixedEff) = YES %then %do;
				%let FitSameSelFixedEff = NO;	
			%end;
			%else %do;
				%let FitSameSelFixedEff = YES; 
			%end;
			*/

			%if &geno=1 %then %do;
				**running genotype test on unrotated data and filtering out signifincant selected variables*;

				**begin: backward loop to filter out significant variables*;
				%let _continueback_ = 1;
				%let _countback_ = 0;
				%do %while(&_continueback_);	
				
					data ma;
						length effect $32;
						delete;
					run;

					%if &_countback_ = 0 %then %do;
						title2 "Selected Variables Fitted on the Unrotated Data - Genotype Test";
					%end;
					%else %do;
						title2 "Round &_countback_ of Backward Elimination of Selected Variables Fitted on the Unrotated Data - Genotype Test";
					%end;

					ods output &Gtable=ma
						%if &WhichProc ne SURVEYREG %then %do;
							nobs = mnobs %if &WhichProc ne PHREG %then (where=(Label ? "Observations Used"));
						%end;
						&ParmEst=pag
						%if (%length(&KVars) or %length(&RandEff)) %then CovParms=cp; 
						/*
						%if %upcase(&TraitType)=BINARY %then responseprofile=rp; 							
						%if %upcase(&OutputRSquare)=YES and %index(%upcase(&WhichProc),LOGISTIC) %then
							rsquare= grsq;	
						*/
						%if %upcase(&OutputFitStat)=YES %then 
							FitStatistics = fsg;
					;

					proc &WhichProc data=&ds &ProcOption;
						class &ClassVars &selMarkerVars_g &ClassOpt;
						model &TraitVar &TraitOpt = &FixedEff &selFixedEff_g &selMarkerVars_g &ModOption;

						%if %length(&KVars)>0 %then %do;
							*begin: S1139177 Luciano efficiency improvement*;
							%if %upcase(&SpeedK)=YES %then %do;
								repeated _IndivID_ / type=lin(1) ldata=Kmats subject=_one_ local;
							%end;
							%else %do;
								random &KVars / type=toep(1) %if %length(&PedID) %then subject=&PedID;;
							%end;
							*end: S1139177 Luciano efficiency improvement*;
						%end;

						%if %length(&RandEff)>0 %then %do;
							random &RandEff %if %upcase(&TraitType)=NOMINAL %then group=&TraitVar;;
						%end;

						%if %length(&ParmsStmt) %then %do;
							/*
							%if %upcase(&OrigFixCovParms)=YES and (%length(&KVars) %then 
								parms &ParmsStmt ;
							%else*/
								parms &OrigParmsStmt ;
						%end;

						%if %length(&StrataVars)>0 %then %do;
							strata &StrataVars;
						%end;

						%if (%upcase(&OutputResid)=YES or %upcase(&OutputPred)=YES) and %length(&RandEff) %then %do;
							id _IndivID_ ;
						%end;

						/*by _order_ %if &OutputMajAllele %then MajorAllele; &ByVars;*/

						/*
						%if ((%upcase(&OutputResid)=YES or %upcase(&OutputPred)=YES) and %upcase(&TraitType)=SURVIVAL) %then %do;
							output out=outres
							%if %upcase(&OutputResid)=YES and %upcase(&TraitType)=SURVIVAL %then resdev=Resid;
							%if %upcase(&OutputPred)=YES and %upcase(&TraitType)=SURVIVAL %then predicted=pred;
						   ;
						%end;
						*/

						*begin: S1139177 Luciano efficiency improvement*;
						%if %length(&KVars) and %upcase(&SpeedK)=YES %then 
							ods exclude classlevels;; 
						*end: S1139177 Luciano efficiency improvement*;
					quit;

					%if &_countback_ = 0 %then %do;
						%NObsVars(ma);
						%if ^&nobs %then %do;
							%put ERROR: PROC &WhichProc generated no estimates for the Genotype Test for trait &TraitVar..  There may be excessive missing data or the contrast may be nonestimable. Please see the log for more details.;
							%let GenoTestError = 1;
							%let _continueback_ = 0;
							%goto skip_1;
						%end;	
	
						proc datasets noprint;
							delete ma_sel_o pag_sel_o %if &WhichProc ne SURVEYREG %then mnobs_sel_o;
								%if (%length(&KVars) or %length(&RandEff)) %then gcp_sel_o;
								%if %index(&ModOption, outp=outres) %then goutres_sel_o;
								%if %upcase(&OutputFitStat)=YES %then fsg_sel_o;
							;
							change ma= ma_sel_o pag= pag_sel_o %if &WhichProc ne SURVEYREG %then mnobs= mnobs_sel_o; 
								%if (%length(&KVars) or %length(&RandEff)) %then cp= gcp_sel_o;
								%if %index(&ModOption, outp=outres) %then outres= goutres_sel_o;
								%if %upcase(&OutputFitStat)=YES %then fsg= fsg_sel_o;
							;
						quit;
						/*data outlib.fsg; set fsg; run;*/
						/*data outlib.ma_sel_o; set ma_sel_o; run;*/
						/*data outlib.pag_sel_o; set pag_sel_o; run;*/
						/*
						%if (%length(&KVars) or %length(&RandEff)) %then %do;
							data outlib.gcp_sel_o; set gcp_sel_o; run;
						%end;
						*/

						%let madat = ma_sel_o;

					%end;
					%else %do;
						%let madat = ma;
					%end

					**filtering out significant selected variables*;
					%NObsVars(&madat);
					%let _nvi_ = &nobs;
					
					%if &_nvi_ > 0 %then %do;

						data ma_nomiss ma_miss;
							set &madat;
							_varorder_ = _n_;
							if missing(ProbF) then output ma_miss;
							else output ma_nomiss;
						run;

						%NObsVars(ma_nomiss);
						%if &nobs %then %do;
							proc sort data= ma_nomiss overwrite;
								by descending ProbF;
							run;
						%end;
						
						data ma_sig;
							set ma_miss %if &nobs %then ma_nomiss;;
						run;
						
						/*data outlib.ma_sig; set ma_sig; run;*/

						data ma_sig;
							set ma_sig;
							if (_n_ = 1 and ProbF > &SLStaySelModel) or (_n_ = 1 and missing(ProbF)) then delete; 
						run;

						%NObsVars(ma_sig);
						%let _nvo_ = &nobs;

						%if &_nvo_ = &_nvi_ %then  
							%let _continueback_ = 0;

						proc sort data=ma_sig overwrite;
							by _varorder_;
						run;
						/*data outlib.ma_sig; set ma_sig; run;*/

						%let totselmrk_filter = 0;
						%let totselfix_filter = 0;				
						data _null_;
							set ma_sig end= _eof_;
							_idmrk_= find(trim(left(Effect)),"&mrkprefix");
							_ixmrk_= index(trim(left(Effect)),"&mrkprefix");
							if _n_ = 1 then do;
								totselmrk_filter = 0;
								totselfix_filter = 0;
							end;
							if _idmrk_ = 1 and _ixmrk_ = 1 then do;
								totselmrk_filter = totselmrk_filter + 1;
								retain totselmrk_filter;
								call symputx("_selmrk_filter"||trim(left(totselmrk_filter)), trim(left(Effect)));
							end;
							else if trim(left(upcase(Effect))) ^= "INTERCEPT" then do;
								totselfix_filter = totselfix_filter + 1;
								retain totselfix_filter;
								call symputx("_selfix_filter"||trim(left(totselfix_filter)), trim(left(Effect)));
							end;
							if _eof_ then do;
								call symputx("totselmrk_filter", trim(left(totselmrk_filter)));
								call symputx("totselfix_filter", trim(left(totselfix_filter)));
							end;
						run;

						**updating list of selected marker variables*;
						%let selMarkerVars_g_sig = ;
						%let selMarkerVarsOrder_g_sig = ;
						%let SelOrigAlleleVars_g_sig = ;
						%do _i= 1 %to &totselmrk_filter;
							%let _j = 1;
							%let _k = 1;
							%let _selmrk = %scan(&selMarkerVars_g, &_j);
							%do %while(%length(&_selmrk));
								%if %upcase(&&_selmrk_filter&_i) = %upcase(&_selmrk) %then %do;
									%let selMarkerVars_g_sig = &selMarkerVars_g_sig &_selmrk;
									%let selMarkerVarsOrder_g_sig = &selMarkerVarsOrder_g_sig %scan(&selMarkerVarsOrder_g, &_j);
									%if %qupcase(&MarkerFmt)=%str(ALLELES) %then %do;
										%let SelOrigAlleleVars_g_sig = &SelOrigAlleleVars_g_sig %scan(&SelOrigAlleleVars_g, &_k) %scan(&SelOrigAlleleVars_g, &_k + 1);
									%end;
									%else %do;
										%let SelOrigAlleleVars_g_sig = &SelOrigAlleleVars_g_sig %scan(&SelOrigAlleleVars_g, &_k);
									%end;
									%let _selmrk = ;
								%end;
								%else %do;
									%let _j = %sysevalf(&_j + 1);
									%if %qupcase(&MarkerFmt)=%str(ALLELES) %then %do;
										%let _k = %sysevalf(&_k + 2);
									%end;
									%else %do;
										%let _k = %sysevalf(&_k + 1);
									%end;
									%let _selmrk = %scan(&selMarkerVars_g, &_j);
								%end;
							%end;
						%end;
						%let selMarkerVars_g = &selMarkerVars_g_sig;
						%let selMarkerVarsOrder_g = &selMarkerVarsOrder_g_sig;
						%let SelOrigAlleleVars_g = &SelOrigAlleleVars_g_sig;

						**updating list of selected fixed effect variables*;
						%let SelOrigFixedVars_g_sig = ;
						%do _i= 1 %to &totselfix_filter;
							%let _j = 1;
							%let _sel = %scan(&selFixedEff_g, &_j);
							%do %while(%length(&_sel));
								%if %upcase(&&_selfix_filter&_i) = %upcase(&_sel) %then %do;
									%let SelOrigFixedVars_g_sig = &SelOrigFixedVars_g_sig &_sel;
									%let _sel = ;
								%end;
								%else %do;
									%let _j = %sysevalf(&_j + 1);
									%let _sel = %scan(&selFixedEff_g, &_j);
								%end;
							%end;
						%end;
						%put SelOrigFixedVars_g_sig = &SelOrigFixedVars_g_sig;

						/*S1158198: removing non-significant fixed effects*/
						%*if %upcase(&FitSameSelFixedEff) ^= YES %then %do;
							%let selFixedEff_g_sig = &SelOrigFixedVars_g_sig;
						%*end;
						%*else %do;
							%*let selFixedEff_g_sig = &selFixedEff_g;
						%*end;
						%let selFixedEff_g = &selFixedEff_g_sig;
					%end;
					%else %do;
						%let selMarkerVars_g = ;
						%let selMarkerVarsOrder_g = ;
						%let SelOrigAlleleVars_g = ;	
						%let selFixedEff_g = ;
						
						%let _continueback_ = 0;
					%end;
					%put selMarkerVars_g = &selMarkerVars_g;
					%put selMarkerVarsOrder_g = &selMarkerVarsOrder_g;
					%put SelOrigAlleleVars_g = &SelOrigAlleleVars_g;
					%put selFixedEff_g = &selFixedEff_g;
					
					%let _countback_ = %sysevalf(&_countback_ + 1);
				%end; 
				**end: backward loop to filter out significant variables*;

				%if ^%length(&selMarkerVars_g) %then %do;
					%put WARNING: No marker variables selected for trait &TraitVar after filtering out selected markers using the Genotype test. You may want to try relaxing the SL for Keeping Variables in the Selected Model.;
				%end;

				%if %length(&selMarkerVars_g &selFixedEff_g) %then %do;
					**begin: running genotype test after filtering out significant selected variables on the unrotated data*;					
					
					data ma;
						length effect $32;
						delete;
					run;
					
					proc datasets noprint;
						delete ma_sel_os pag_sel_os %if &WhichProc ne SURVEYREG %then mnobs_sel_os;
							%if (%length(&KVars) or %length(&RandEff)) %then gcp_sel_os;
							%if %index(&ModOption, outp=outres) %then goutres_sel_os;
							%if %upcase(&OutputFitStat)=YES %then fsg_sel_os;
						;
					quit;

					title2 "Filtered Selected Variables Fitted on the Unrotated Data - Genotype Test";
					ods output &Gtable=ma
						%if &WhichProc ne SURVEYREG %then %do;
							nobs = mnobs %if &WhichProc ne PHREG %then (where=(Label ? "Observations Used"));
						%end;
						&ParmEst=pag
						%if (%length(&KVars) or %length(&RandEff)) %then CovParms=cp; 
						%if (&lsmi=1 and %length(&selMarkerVars_g)) %then lsmeans=lsm diffs=lsd;
						/*
						%if %upcase(&TraitType)=BINARY %then responseprofile=rp; 							
						%if %upcase(&OutputRSquare)=YES and %index(%upcase(&WhichProc),LOGISTIC) %then
							rsquare= grsq;	
						*/
						%if %upcase(&OutputFitStat)=YES %then 
							FitStatistics= fsg;
					;

					proc &WhichProc data=&ds &ProcOption;
						class &ClassVars &selMarkerVars_g &ClassOpt;
						model &TraitVar &TraitOpt = &FixedEff &selFixedEff_g &selMarkerVars_g &ModOption;

						%if %length(&KVars)>0 %then %do;
							*begin: S1139177 Luciano efficiency improvement*;
							%if %upcase(&SpeedK)=YES %then %do;
								repeated _IndivID_ / type=lin(1) ldata=Kmats subject=_one_ local;
							%end;
							%else %do;
								random &KVars / type=toep(1) %if %length(&PedID) %then subject=&PedID;;
							%end;
							*end: S1139177 Luciano efficiency improvement*;
						%end;

						%if %length(&RandEff)>0 %then %do;
							random &RandEff %if %upcase(&TraitType)=NOMINAL %then group=&TraitVar;;
						%end;

						%if %length(&ParmsStmt) %then %do;
							/*
							%if %upcase(&OrigFixCovParms)=YES and (%length(&KVars) %then 
								parms &ParmsStmt ;
							%else*/
								parms &OrigParmsStmt ;
						%end;

						%if %length(&StrataVars)>0 %then %do;
							strata &StrataVars;
						%end;

						%if &lsmi=1 %then %do;
							%let _k = 1;
							%do %while(%length(%scan(&selMarkerVars_g,&_k)));
								lsmeans %scan(&selMarkerVars_g,&_k) / diff=control('0');
								%let _k = %sysevalf(&_k + 1);
							%end;			
						%end;

						%if (%upcase(&OutputResid)=YES or %upcase(&OutputPred)=YES) and %length(&RandEff) %then %do;
							id _IndivID_ ;
						%end;

						/*by _order_ %if &OutputMajAllele %then MajorAllele; &ByVars;*/

						/*
						%if ((%upcase(&OutputResid)=YES or %upcase(&OutputPred)=YES) and %upcase(&TraitType)=SURVIVAL) %then %do;
							output out=outres
							%if %upcase(&OutputResid)=YES and %upcase(&TraitType)=SURVIVAL %then resdev=Resid;
							%if %upcase(&OutputPred)=YES and %upcase(&TraitType)=SURVIVAL %then predicted=pred;						
							;
						%end;
						*/

						*begin: S1139177 Luciano efficiency improvement*;
						%if %length(&KVars) and %upcase(&SpeedK)=YES %then 
							ods exclude classlevels;; 
						*end: S1139177 Luciano efficiency improvement*;
					run;
					
					proc datasets noprint;
						change ma= ma_sel_os pag= pag_sel_os %if &WhichProc ne SURVEYREG %then mnobs= mnobs_sel_os; 
							%if (%length(&KVars) or %length(&RandEff)) %then cp= gcp_sel_os;
							%if %index(&ModOption, outp=outres) %then outres= goutres_sel_os;
							%if %upcase(&OutputFitStat)=YES %then fsg= fsg_sel_os;
						;
					quit;

					%if %length(&selMarkerVars_g) %then %do;
						/*data outlib.ma_sel_os; set ma_sel_os; run;*/
						/*
						%if &WhichProc ne SURVEYREG %then %do;
							data outlib.mnobs_sel_os; set mnobs_sel_os; run;
						%end;
						*/
						/*data outlib.g_pag0; set pag_sel_os; run;*/
						/*
						%if (%length(&KVars) or %length(&RandEff)) %then %do;
							data outlib.gcp_sel_os; set gcp_sel_os; run;
						%end;
						*/
						/*
						%if &lsmi=1 %then %do;
							data outlib.g_lsm0; set lsm; run;
							data outlib.g_lsd0; set lsd; run;
						%end;
						*/
						/*
						%if %upcase(&OutputFitStat)=YES %then %do;
							data outlib.fsg_sel_os;
								set fsg_sel_os;
							run;
						%end;
						*/

						data ma_a;
							length Effect Trait $32;
							delete;
						run;

						data pag_all;
							length Effect Trait $32;
							delete;
						run;

						%if %upcase(&OutputResid)=YES or %upcase(&OutputPred)=YES %then %do;
							data outresg_all;
								length _Trait_ $32;
								delete;
							run;
						%end;

						%if &lsmi=1 %then %do;
							data outlsm_all;
								length Trait $32 Effect $32;
								delete;
							run;
						%end;
						
						%if %upcase(&OutputFitStat)=YES %then %do;
							data fsg_all;
								length Effect Trait $32;
								delete;
							run;						
						%end;

						%if (%length(&KVars) or %length(&RandEff)) %then %do;
							data cpg_all;
								length Trait $32;
								delete;
							run;
						%end;

						%let _k = 1;
						%do %while(%length(%scan(&selMarkerVars_g,&_k)));

							data ma_mrk;
								set ma_sel_os;
								where (trim(left(Effect)) = trim(left("%scan(&selMarkerVars_g,&_k)")));
								Effect = "Genotype";
								_order_ = %scan(&selMarkerVarsOrder_g,&_k);
							run;

							data ma_mrk;
								merge mnobs_sel_os(keep=&ByVars nobsused) ma_mrk;
								rename nobsused=SampleSize;
							run;			

							data ma_a; 
								length Trait $32;
								set ma_a(in=orig) ma_mrk;
								where index(Effect,"Genotype")>0; 
								if not orig then Trait = "&TraitVar";
								label Prob&Stat=' ';
							run;

							data pag_all;
								length Effect Trait $32;
								set pag_all(in=orig) pag_sel_os;
								if not orig then do;
									Trait = "&TraitVar";
									_order_ = %scan(&selMarkerVarsOrder_g,&_k);
								end;
							run;

							%if %upcase(&OutputResid)=YES or %upcase(&OutputPred)=YES %then %do;
								%if %sysfunc(exist(goutres_sel_os)) %then %do;
									data outresg_all;
										set outresg_all(in=orig) goutres_sel_os;
										if orig=0 then do;
											_Trait_ = "&TraitVar";
											_order_ = %scan(&selMarkerVarsOrder_g,&_k);
										end;
									run;		
								%end;
							%end;

							%if &lsmi=1 %then %do;
								data outlsm_all;
									set outlsm_all(in=orig) lsm lsd(in=idiff);
									if orig=0 then do;
										diff=0;
										Trait = "&TraitVar";
										_order_ = %scan(&selMarkerVarsOrder_g,&_k);
										if idiff then do;
											diff=1;
											Effect = "Diff with 0";
										end;
									end;
								run;	
							%end;
							
							%if %upcase(&OutputFitStat)=YES %then %do;
								data OutLib.fsg_sel_os&_k; set fsg_sel_os; run;
							
									proc transpose data=fsg_sel_os(
										%if %index(%upcase(&WhichProc),LOGISTIC) %then %do;
											rename=(
												criterion= descr 
												interceptandcovariates= value
											)
										%end;
										%else %if %index(%upcase(&WhichProc),PHREG) %then %do;
											rename=(
												criterion= descr  
												withcovariates= value
											)
										%end;
										)
										out=fsg_sel_ost(drop= _name_); 
										var value;
										id descr;
										/*by _order_ %if &OutputMajAllele %then MajorAllele; &ByVars;*/
									run;
								
								data fsg_all;
									length Effect Trait $32;
									set fsg_all(in=orig) fsg_sel_ost;
									if not orig then do;
										Effect= "Genotype";
										Trait = "&TraitVar";
										_order_ = %scan(&selMarkerVarsOrder_g,&_k);
									end;
								run;				
							%end;

							%put KVars= &KVars;
							%put RandEff= &RandEff;
							%if (%length(&KVars) or %length(&RandEff)) %then %do;
								data cpg_all;				
									set cpg_all(in=orig) gcp_sel_os;
									if orig=0 then do;
										Trait = "&TraitVar";
										_order_ = %scan(&selMarkerVarsOrder_g,&_k);
									end;
								run;
							%end;

							%let _k= %sysevalf(&_k + 1);		
						%end;

						/*data outlib.g_ma_a; set ma_a; run;*/
						/*data outlib.g_pag_all; set pag_all; run;*/
						/*
						%if %upcase(&OutputResid)=YES or %upcase(&OutputPred)=YES %then %do;
							data outlib.g_outresg_all; set outresg_all; run;
						%end;
						*/
						/*
						%if &lsmi=1 %then %do;
							data outlib.g_outlsm_all; set outlsm_all; run;
						%end;
						*/
						/*
						%if (%length(&KVars) or %length(&RandEff)) %then %do;
							data outlib.g_cpg_all; set cpg_all; run;
						%end;
						*/
						/*
						%if %upcase(&OutputFitStat)=YES %then %do;
							data outlib.g_fsg_all; set fsg_all; run; 
						%end;
						*/

						%Nobsvars(ma_a);
						%if &nobs=0 %then %do;
							%put ERROR: Genotype tests did not run successfully.  See the SAS log for more details.;
							%let exiterror = 1;
							%goto exit;
						%end;

						proc sort data=Ma_a out=effect_map(keep=Effect Trait _order_) nodupkey;
							by _order_;
						run;

						data mapdata2_g; 
							merge mapdata %if %length(&selmarkervarsorder_g) %then (where=(_order_ in (%sysfunc(tranwrd(&selmarkervarsorder_g, %str( ), %str(,)))))); effect_map;
							by _order_;
						run;

						%if &lsmi=1 %then %do;

							proc sort data=outlsm_all;
								by _order_ &ByVars diff;
							run;

							data Assoc_LSM;
								merge mapdata %if %length(&selmarkervarsorder_g) %then (where=(_order_ in (%sysfunc(tranwrd(&selmarkervarsorder_g, %str( ), %str(,)))))); outlsm_all;
								by _order_;
								/*drop _order_ diff */
								drop diff
									%let _k = 1;
									%do %while(%length(%scan(&selMarkerVars_g,&_k)));
										_%scan(&selMarkerVars_g,&_k)
										%let _k= %sysevalf(&_k + 1);
									%end;
								;
								rename Estimate = LSMeans_Estimate;
							run;
							/*data outlib.Assoc_LSM; set Assoc_LSM; run;*/ 

							data outlsm_all_s;
								set outlsm_all(where=(diff and not missing (%scan(&selMarkerVars_g, 1)) and _order_ = (%scan(&selMarkerVarsOrder_g, 1))));
								Genotype= %scan(&selMarkerVars_g, 1);
							run;
							/*data outlib.g_outlsm_all_s; set outlsm_all_s; run;*/

							%let _k = 2;
							%do %while(%length(%scan(&selMarkerVars_g,&_k)));
								data outlsm_all_s;
									set outlsm_all_s outlsm_all(in= _b_ where=(diff and not missing (%scan(&selMarkerVars_g, &_k)) and _order_ = (%scan(&selMarkerVarsOrder_g, &_k))));
									if _b_ then Genotype= %scan(&selMarkerVars_g, &_k);
								run;
								%let _k= %sysevalf(&_k + 1);
							%end;
							%let transds = outlsm_all_s;
							/*data outlib.g_transds; set &transds; run;*/

							%if %upcase(&SelModelProfile) ^= YES %then %do;
							
								%if %length(&MTC) %then %do;
									%if %length(&ByVars) %then %do;
										proc sort data=outlsm_all_s;
											by &ByVars _order_;
										run;
									%end;

									%AdjPValueName(method=%scan(&mtc,1));
									proc multtest pdata(probt)=outlsm_all_s %scan(&mtc,1) out=adjdiffs;
										%if %length(&ByVars) %then %do;
											by &ByVars;
										%end;
									run;
									%let transds = adjdiffs;
								%end;
								%else %do;			
									%let mtpvar = Probt;
									%if %length(&ByVars) %then %do;
										proc sort data=outlsm_all_s;
											by &ByVars _order_;
										run;
									%end;
								%end;

								proc transpose data=&transds out=trans1(drop=_name_) prefix=LSDiff_ suffix=_0;
									var estimate;
									by  &ByVars _order_;
									id Genotype;
								run;

								proc transpose data=&transds out=trans2(keep=ProbLS:) prefix=ProbLSDiff_ suffix=_0;
									var &mtpvar;
									by  &ByVars _order_;
									id Genotype;
								run;

								data lsmtrans;
									set trans1; set trans2;
									Effect = "Genotype";
									%if %length(&Conv) %then %do;
										array problsa{*} problsdiff_:;
										do _p=1 to dim(problsa);
											%if %upcase(&Conv)=NEGLOG %then %do;
												problsa{_p} = -log(problsa{_p});
											%end;
											%else %do;
												problsa{_p} = -log10(problsa{_p});
											%end;			
										end;
										drop _p;
									%end;					
								run;

								proc sort data=lsmtrans out=outlsm_all_s;
									by _order_ &ByVars Effect ;
								run;
								/*data outlib.g_outlsm_all_s_f; set outlsm_all_s; run;*/
							%end;
						%end;				

						data ma_a;
							merge mapdata2_g ma_a 
								%if &lsmi and %upcase(&SelModelProfile) ^= YES %then outlsm_all_s; 
								%if %upcase(&OutputFitStat)=YES %then %do;
									fsg_all(
									%if %index(%upcase(&WhichProc),MIXED) %then %do;
										rename= (
											_2_Res_Log_Likelihood= _2_Res_Log_Likelihood_Genotype
											AIC__Smaller_is_Better_= AIC_Smaller_is_Better_Genotype
											AICC__Smaller_is_Better_= AICC_Smaller_is_Better_Genotype
											BIC__Smaller_is_Better_= BIC_Smaller_is_Better_Genotype
										)
									%end;	
									)
								%end;
							;
							by _order_ &ByVars Effect;
							%if %Upcase(&Stat) ne CHISQ %then %do;
								rename NumDF=NumDF_Genotype DenDF=DenDF_Genotype FValue=FValue_Genotype Prob&Stat=ProbGenotype ; 
								if DenDF=0 then Rsquare_Genotype = .;
								else Rsquare_Genotype = (NumDF*FValue/DenDF)/(1+NumDF*FValue/DenDF);
							%end;
							%else %do;
								rename DF=DF_Genotype WaldChiSq=WaldChiSq_Genotype Prob&Stat=ProbGenotype;
							%end;
						run;
						/*data outlib.g_ma_a_f; set ma_a; run;*/						
						
						proc datasets; 	
							delete Assoc_ma;
							change ma_a = Assoc_ma;
						quit;
						
						proc sort data=Assoc_ma;
							by &AnnoGroup &AnnoLoc &ByVars Effect ;
						run;
						/*data outlib.Assoc_Ma; set Assoc_Ma; run;*/

						%if %upcase(&OutputResid)=YES or %upcase(&OutputPred)=YES %then %do;

							%Nobsvars(outresg_all);
							%let rds = %sysfunc(open(outresg_all));
							%if %upcase(&OutputResid)=YES %then %let residnum = %sysfunc(varnum(&rds,RESID));
							%if %upcase(&OutputPred)=YES %then %let prednum = %sysfunc(varnum(&rds,PRED));
							%let rc=%sysfunc(close(&rds));
							
							%if %upcase(&OutputResid)=YES and %upcase(&OutputPred)=YES %then %do;
								%if &residnum=0 %then %let _outpnum= &residnum;
								%else %if &prednum=0 %then %let _outpnum= &prednum;
								%else %let _outpnum= &residnum;
							%end;
							%else %if %upcase(&OutputResid)=YES and %upcase(&OutputPred)^=YES  %then %do;
								%let _outpnum= &residnum;
							%end;
							%else %if %upcase(&OutputResid)^=YES and %upcase(&OutputPred)=YES  %then %do;
								%let _outpnum= &prednum;
							%end;

							%if &nobs=0 or &_outpnum=0 %then %do;
								%put ERROR: The genotype residuals could not be computed.;
								%let OutputResid=NO;
								%let OutputPred=NO;
							%end;
							%else %do;

								data outresg_all;
									merge mapdata outresg_all;
									by _order_;
									%if %length(&selmarkervarsorder_g) %then %do;  
										where _order_ in (%sysfunc(tranwrd(&selmarkervarsorder_g, %str( ), %str(,))));
									%end;
								run;

								proc sort data=outresg_all;
									by _individ_ _order_ _Trait_ ;
								run;

								%if %upcase(&OutputResid)=YES %then %do;
								proc transpose data=outresg_all out=outresgt(drop=_name_) let
									prefix=Resid_;
									var resid;
									id &AnnoLabel;
									by _IndivID_;
								run;

								data Assoc_ORG;
									 merge othervars outresgt;
									 by _IndivID_;
								run;
								/*data outlib.Assoc_ORG; set Assoc_ORG; run;*/
								%end;
								
								%if %upcase(&OutputPred)=YES %then %do;
									proc transpose data=outresg_all out=outpredgt(drop=_name_) let
										prefix=Pred_;
										var pred;
										id &AnnoLabel;
										by _IndivID_;
									run;

									data Assoc_OPG;
										 merge othervars outpredgt;
										 by _IndivID_;
									run;
									/*data outlib.Assoc_OPG; set Assoc_OPG; run;*/
								%end;
							%end;
						%end;

						data Assoc_PEG;
							merge mapdata pag_all;
							by _order_;
							%if %length(&selmarkervarsorder_g) %then %do;  
								where _order_ in (%sysfunc(tranwrd(&selmarkervarsorder_g, %str( ), %str(,))));
							%end;
							*drop _order_ _ESTTYPE_;
							drop _ESTTYPE_;
						run;
						/*data outlib.Assoc_PEG; set Assoc_PEG; run;*/

						%if (%length(&KVars) or %length(&RandEff)) %then %do;
							data Assoc_CPG;
								merge mapdata cpg_all;
								by _order_;
								%if %length(&selmarkervarsorder_g) %then %do;  
									where _order_ in (%sysfunc(tranwrd(&selmarkervarsorder_g, %str( ), %str(,))));
								%end;
								*drop _order_;
							run;
							/*data outlib.Assoc_CPG; set Assoc_CPG; run;*/
						%end;	

					%end;
				%end;
				**end: running genotype test after filtering out significant selected variables on the unrotated data*;			
			%end; /*end if geno=1*/

			%*goto exit;

			%skip_1:;

			%if &trend=1 %then %do;
				**running genotype trend on unrotated data and filtering out signifincant selected variables*;

				**begin: backward loop to filter out significant variables*;
				%let _continueback_ = 1;
				%let _countback_ = 0;
				%do %while(&_continueback_);

					data ta;
						delete;
					run;						

					%if &_countback_ = 0 %then %do;
						title2 "Selected Variables Fittetd on the Unrotated Data - Trend Test";
					%end;
					%else %do;
						title2 "Round &_countback_ of Backward Elimination of Selected Variables Fittetd on the Unrotated Data - Trend Test";
					%end;

					ods output &Gtable=ta  
						&ParmEst = pat
						%if &WhichProc ne SURVEYREG %then %do;
							nobs = tnobs %if &WhichProc ne PHREG %then (where=(Label ? "Observations Used"));
						%end;
						%if (%length(&KVars) or %length(&RandEff)) %then CovParms=cp;
						/*
						%if %upcase(&TraitType)=BINARY %then responseprofile=rp; 
						%if %upcase(&OutputRSquare)=YES and %index(%upcase(&WhichProc),LOGISTIC) %then
							rsquare= trsq;	
						*/
						%if %upcase(&OutputFitStat)=YES %then 
							FitStatistics = fst;
					;

					proc &WhichProc data=&ds &ProcOption;

						class &ClassVars &ClassOpt;

						model &TraitVar &TraitOpt = &FixedEff &selFixedEff_t &selMarkerVars_t &ModOption;

						%if %length(&KVars)>0 %then %do;
							*begin: S1139177 Luciano efficiency improvement*;
							%if %upcase(&SpeedK)=YES %then %do;
								repeated _IndivID_ / type=lin(1) ldata=Kmats subject=_one_ local;
							%end;
							%else %do;
								random &KVars / type=toep(1) %if %length(&PedID) %then subject=&PedID;;
							%end;
							*end: S1139177 Luciano efficiency improvement*;
						%end;

						%if %length(&RandEff) %then %do;
							random &RandEff %if %upcase(&TraitType)=NOMINAL %then group=&TraitVar;;
						%end;

						%if %length(&ParmsStmt) %then %do;	
							/*
							%if %upcase(&OrigFixCovParms)=YES and (%length(&KVars) %then 
								parms &ParmsStmt ;
							%else */
								parms &OrigParmsStmt ;	
						%end;					

						%if %length(&StrataVars)>0 %then %do;
							strata &StrataVars;
						%end;

						%if (%upcase(&OutputResid)=YES or %upcase(&OutputPred)=YES) and %length(&RandEff) %then %do;
							id _IndivID_;
						%end;					

						/*by _order_ %if &OutputMajAllele %then MajorAllele; &ByVars;*/

						/*
						%if ((%upcase(&OutputResid)=YES or %upcase(&OutputPred)=YES) and %upcase(&TraitType)=SURVIVAL) %then %do;
							output out=outres
							%if %upcase(&OutputResid)=YES and %upcase(&TraitType)=SURVIVAL %then resdev=Resid;
							%if %upcase(&OutputPred)=YES and %upcase(&TraitType)=SURVIVAL %then predicted=pred;							
							;
						%end;
						*/

						*begin: S1139177 Luciano efficiency improvement*;
						%if %length(&KVars) and %upcase(&SpeedK)=YES %then 
							ods exclude classlevels;;
						*end: S1139177 Luciano efficiency improvement*;
					quit;
					
					%if &_countback_ = 0 %then %do;

						%NObsVars(ta);
						%if ^&nobs %then %do;
							%put ERROR: PROC &WhichProc generated no estimates for the Trend Test for trait &TraitVar..  There may be excessive missing data or the contrast may be nonestimable. Please see the log for more details.;
							%let TrendTestError = 1;
							%goto skip_2;
						%end;

						proc datasets noprint;
							delete ta_sel_o pat_sel_o %if &WhichProc ne SURVEYREG %then tnobs_sel_o;
								%if (%length(&KVars) or %length(&RandEff)) %then tcp_sel_o;
								%if %index(&ModOption, outp=outres) %then toutres_sel_o;
								%if %upcase(&OutputFitStat)=YES %then fst_sel_o;
							;
							change ta= ta_sel_o pat= pat_sel_o %if &WhichProc ne SURVEYREG %then tnobs= tnobs_sel_o; 
								%if (%length(&KVars) or %length(&RandEff)) %then cp= tcp_sel_o;
								%if %index(&ModOption, outp=outres) %then outres= toutres_sel_o;
								%if %upcase(&OutputFitStat)=YES %then fst= fst_sel_o;
							;
						quit;
						/*data outlib.ta_sel_o; set ta_sel_o; run;*/
						/*
						%if (%length(&KVars) or %length(&RandEff)) %then %do;
							data outlib.tcp_sel_o; set tcp_sel_o; run;
						%end;
						*/
						%let tadat = ta_sel_o;
					%end;
					%else %do;
						%let tadat = ta;
					%end

					**filtering out significant selected variables*;
					%NObsVars(&tadat);
					%let _nvi_ = &nobs;

					%if &_nvi_ > 0 %then %do; 
					
						data ta_nomiss ta_miss;
							set &tadat;
							_varorder_ = _n_;
							if missing(ProbF) then output ta_miss;
							else output ta_nomiss;
						run;

						%NObsVars(ta_nomiss);
						%if &nobs %then %do;
							proc sort data= ta_nomiss overwrite;
								by descending ProbF;
							run;
						%end;
						
						data ta_sig;
							set ta_miss %if &nobs %then ta_nomiss;;
						run;
						
						/*data outlib.ta_sig; set ta_sig; run;*/

						data ta_sig;
							set ta_sig;
							if (_n_ = 1 and ProbF > &SLStaySelModel) or (_n_ = 1 and missing(ProbF)) then delete; 
						run;

						%NObsVars(ta_sig);
						%let _nvo_ = &nobs;

						%if &_nvo_ = &_nvi_ %then  
							%let _continueback_ = 0;				

						proc sort data=ta_sig overwrite;
							by _varorder_;
						run;				
						/*data outlib.ta_sig; set ta_sig; run;*/

						%let totselmrk_filter = 0;
						%let totselfix_filter = 0;
						data _null_;
							set ta_sig end = _eof_;
							_idmrk_= find(trim(left(Effect)),"&mrkprefix");
							_ixmrk_= index(trim(left(Effect)),"&mrkprefix");
							if _n_ = 1 then do;
								totselmrk_filter = 0;
								totselfix_filter = 0;
							end;
							if _idmrk_ = 1 and _ixmrk_ = 1 then do;
								totselmrk_filter = totselmrk_filter + 1;
								retain totselmrk_filter;
								call symputx("_selmrk_filter"||trim(left(totselmrk_filter)), trim(left(Effect)));
							end;
							else if trim(left(upcase(Effect))) ^= "INTERCEPT" then do;
								totselfix_filter = totselfix_filter + 1;
								retain totselfix_filter;
								call symputx("_selfix_filter"||trim(left(totselfix_filter)), trim(left(Effect)));
							end;
							if _eof_ then do;
								call symputx("totselmrk_filter", trim(left(totselmrk_filter)));
								call symputx("totselfix_filter", trim(left(totselfix_filter)));
							end;
						run;

						**updating list of selected marker variables*;
						%let selMarkerVars_t_sig = ;
						%let selMarkerVarsOrder_t_sig = ;
						%let SelOrigAlleleVars_t_sig = ;
						%do _i= 1 %to &totselmrk_filter;
							%let _j = 1;
							%let _k = 1;
							%let _selmrk = %scan(&selMarkerVars_t, &_j);
							%do %while(%length(&_selmrk));
								%if %upcase(&&_selmrk_filter&_i) = %upcase(&_selmrk) %then %do;
									%let selMarkerVars_t_sig = &selMarkerVars_t_sig &_selmrk;
									%let selMarkerVarsOrder_t_sig = &selMarkerVarsOrder_t_sig %scan(&selMarkerVarsOrder_t, &_j);
									%if %qupcase(&MarkerFmt)=%str(ALLELES) %then %do;
										%let SelOrigAlleleVars_t_sig = &SelOrigAlleleVars_t_sig %scan(&SelOrigAlleleVars_t, &_k) %scan(&SelOrigAlleleVars_t, &_k + 1);
									%end;
									%else %do;
										%let SelOrigAlleleVars_t_sig = &SelOrigAlleleVars_t_sig %scan(&SelOrigAlleleVars_t, &_k);
									%end;							
									%let _selmrk = ;
								%end;
								%else %do;
									%let _j = %sysevalf(&_j + 1);
									%let _selmrk = %scan(&selMarkerVars_t, &_j);
									%if %qupcase(&MarkerFmt)=%str(ALLELES) %then %do;
										%let _k = %sysevalf(&_k + 2);
									%end;
									%else %do;
										%let _k = %sysevalf(&_k + 1);
									%end;							
								%end;
							%end;
						%end;
						%let selMarkerVars_t = &selMarkerVars_t_sig;
						%let selMarkerVarsOrder_t = &selMarkerVarsOrder_t_sig;
						%let SelOrigAlleleVars_t = &SelOrigAlleleVars_t_sig;

						**updating list of selected fixed effect variables*;
						%let SelOrigFixedVars_t_sig = ;
						%do _i= 1 %to &totselfix_filter;
							%let _j = 1;
							%let _sel = %scan(&selFixedEff_t, &_j);
							%do %while(%length(&_sel));
								%if %upcase(&&_selfix_filter&_i) = %upcase(&_sel) %then %do;
									%let SelOrigFixedVars_t_sig = &SelOrigFixedVars_t_sig &_sel;
									%let _sel = ;
								%end;
								%else %do;
									%let _j = %sysevalf(&_j + 1);
									%let _sel = %scan(&selFixedEff_t, &_j);
								%end;
							%end;
						%end;
						%put SelOrigFixedVars_t_sig = &SelOrigFixedVars_t_sig;

						/*S1158198: removing non-significant fixed effects*/
						%*if %upcase(&FitSameSelFixedEff) ^= YES %then %do;
							%let selFixedEff_t_sig = &SelOrigFixedVars_t_sig;
						%*end;
						%*else %do;
							%*let selFixedEff_t_sig = &selFixedEff_t;
						%*end;
						%let selFixedEff_t = &selFixedEff_t_sig;
					%end;
					%else %do;
						%let selMarkerVars_t = ;
						%let selMarkerVarsOrder_t = ;
						%let SelOrigAlleleVars_t = ;
						%let selFixedEff_t = ;
					
						%let _continueback_ = 0;
					%end;
					%put selMarkerVars_t = &selMarkerVars_t;
					%put selMarkerVarsOrder_t = &selMarkerVarsOrder_t;
					%put SelOrigAlleleVars_t = &SelOrigAlleleVars_t;					
					%put selFixedEff_t = &selFixedEff_t;

					%let _countback_ = %sysevalf(&_countback_ + 1);
				%end; 
				**end: backward loop to filter out significant variables*;
				
				%if ^%length(&selMarkerVars_t) %then %do;
					%put WARNING: No marker variables selected for trait &TraitVar when filtering out selected markers using the Trend test. You may want to try relaxing the SL for Keeping Variables in the Selected Model.;
				%end;

				%if %length(&selMarkerVars_t &selFixedEff_t) %then %do;
					**begin: running genotype test after filtering out significant selected variables on the unrotated data*;					
					
					data ta;
						delete;
					run;
					
					proc datasets noprint;
						delete ta_sel_os pat_sel_os %if &WhichProc ne SURVEYREG %then tnobs_sel_os;
							%if (%length(&KVars) or %length(&RandEff)) %then tcp_sel_os;
							%if %index(&ModOption, outp=outres) %then toutres_sel_os;
							%if %upcase(&OutputFitStat)=YES %then fst_sel_os;
						;
					quit;

					title2 "Filtered Selected Variables Fitted on the Unrotated Data - Trend Test";
					ods output &Gtable=ta  
						&ParmEst = pat
						%if &WhichProc ne SURVEYREG %then %do;
							nobs = tnobs %if &WhichProc ne PHREG %then (where=(Label ? "Observations Used"));
						%end;

						%if (%length(&KVars) or %length(&RandEff)) %then CovParms=cp;

						/*
						%if %upcase(&TraitType)=BINARY %then responseprofile=rp; 
						%if %upcase(&OutputRSquare)=YES and %index(%upcase(&WhichProc),LOGISTIC) %then
							rsquare= trsq;	
						*/
						%if %upcase(&OutputFitStat)=YES %then 
							FitStatistics = fst;
					;

					proc &WhichProc data=&ds &ProcOption %if %Upcase(&Odds)=YES and ^%length(&RandEff &KVars) %then alpha=&Alpha;;

						class &ClassVars &ClassOpt;

						model &TraitVar &TraitOpt = &FixedEff &selFixedEff_t &selMarkerVars_t &ModOption;

						%if %length(&KVars)>0 %then %do;
							*begin: S1139177 Luciano efficiency improvement*;
							%if %upcase(&SpeedK)=YES %then %do;
								repeated _IndivID_ / type=lin(1) ldata=Kmats subject=_one_ local;
							%end;
							%else %do;
								random &KVars / type=toep(1) %if %length(&PedID) %then subject=&PedID;;
							%end;
							*end: S1139177 Luciano efficiency improvement*;
						%end;

						%if %length(&RandEff) %then %do;
							random &RandEff %if %upcase(&TraitType)=NOMINAL %then group=&TraitVar;;
						%end;

						%if %length(&ParmsStmt) %then %do;
							/*
							%if %upcase(&OrigFixCovParms)=YES and (%length(&KVars) %then 
								parms &ParmsStmt ;
							%else */
								parms &OrigParmsStmt ;
						%end;					

						%if %length(&StrataVars)>0 %then %do;
							strata &StrataVars;
						%end;

						%if (%upcase(&OutputResid)=YES or %upcase(&OutputPred)=YES) and %length(&RandEff) %then %do;
							id _IndivID_;
						%end;					

						/*by _order_ %if &OutputMajAllele %then MajorAllele; &ByVars;*/

						/*
						%if ((%upcase(&OutputResid)=YES or %upcase(&OutputPred)=YES) and %upcase(&TraitType)=SURVIVAL) %then %do;
							output out=outres
							%if %upcase(&OutputResid)=YES and %upcase(&TraitType)=SURVIVAL %then resdev=Resid;
							%if %upcase(&OutputPred)=YES and %upcase(&TraitType)=SURVIVAL %then predicted=pred;
							;
						%end;
						*/

						*begin: S1139177 Luciano efficiency improvement*;
						%if %length(&KVars) and %upcase(&SpeedK)=YES %then 
							ods exclude classlevels;;
						*end: S1139177 Luciano efficiency improvement*;				
					run;
					
					proc datasets noprint;
						change ta= ta_sel_os pat= pat_sel_os %if &WhichProc ne SURVEYREG %then tnobs= tnobs_sel_os; 
							%if (%length(&KVars) or %length(&RandEff)) %then cp= tcp_sel_os;
							%if %index(&ModOption, outp=outres) %then outres= toutres_sel_os;
							%if %upcase(&OutputFitStat)=YES %then fst= fst_sel_os;
						;
					quit;

					%if %length(&selMarkerVars_t) %then %do; 
						/*data outlib.ta_sel_os; set ta_sel_os; run;*/
						/*
						%if &WhichProc ne SURVEYREG %then %do;
							data outlib.tnobs_sel_os; set tnobs_sel_os; run;
						%end;
						*/
						/*data outlib.g_pat0; set pat_sel_os; run;*/
						/*
						%if (%length(&KVars) or %length(&RandEff)) %then %do;
							data outlib.tcp_sel_os; set tcp_sel_os; run;
						%end;	
						*/
						/*
						%if %upcase(&OutputFitStat)=YES %then %do;
							data outlib.fst_sel_os;
								set fst_sel_os;
							run;
						%end;
						*/
						
						data ta_all;
							length Effect $32;
							delete;
						run;	

						data pat_all;
							length &EffName Trait $32;
							delete;
						run;

						%if %upcase(&OutputResid)=YES or %upcase(&OutputPred)=YES %then %do;
							data outrest_all;
								length _Trait_ $32;
								delete;
							run;
						%end;
						
						%if %upcase(&OutputFitStat)=YES %then %do;
							data fst_all;
								length Effect Trait $32;
								delete;
							run;						
						%end;

						%if (%length(&KVars) or %length(&RandEff)) %then %do;
							data cpt_all;
								length Trait $32;
								delete;
							run;
						%end;

						%*goto exit;

						%let _k = 1;
						%do %while(%length(%scan(&selMarkerVars_t,&_k)));				

							data ta_mrk;
								set ta_sel_os;
								where (trim(left(Effect)) = trim(left("%scan(&selMarkerVars_t,&_k)")));
								Effect = "Genotype";
								_order_ = %scan(&selMarkerVarsOrder_t,&_k);
							run;				

							data ta_mrk;
								merge tnobs_sel_os(keep=&ByVars nobsused) ta_mrk;
								rename nobsused=SampleSize;
							run;

							data pat_mrk;
								set pat_sel_os;
								where (trim(left(Effect)) = trim(left("%scan(&selMarkerVars_t,&_k)")));
								Effect = "Genotype";
							run;

							data ta_mrk;
								merge pat_mrk(keep=&EffName Estimate Stderr) ta_mrk(where=(effect ? "Genotype"));		
								%if %quote(&EffName)=%str(Variable) %then drop &EffName;;
								rename Estimate = Estimate_Trend StdErr= StdErr_Trend;
							run;

							data ta_all;  
								length Trait $32 ;
								set ta_all(in=orig) ta_mrk;
								where index(Effect,"Genotype")>0; 				
								if not orig then Trait = "&TraitVar";
								label Prob&Stat=' '; 
							run;

							%if %upcase(&OutputResid)=YES or %upcase(&OutputPred)=YES %then %do;
								%if %sysfunc(exist(toutres_sel_os)) %then %do;
									data outrest_all;
										set outrest_all(in=orig) toutres_sel_os;
										if orig=0 then do;
											_Trait_="&TraitVar";
											_order_ = %scan(&selMarkerVarsOrder_t,&_k);
										end;
									run;		
								%end;
							%end;

							data pat_all;	
								set pat_all(in=orig) pat_sel_os;
								if orig=0 then do;
									Trait = "&TraitVar";
									_order_ = %scan(&selMarkerVarsOrder_t,&_k);
								end;
							run;	

							%if (%length(&KVars) or %length(&RandEff)) %then %do;
								data cpt_all;	
									set cpt_all(in=orig) tcp_sel_os;
									if orig=0 then do;
										Trait = "&TraitVar";
										_order_ = %scan(&selMarkerVarsOrder_t,&_k);
									end;
								run;
							%end;	
							
							%if %upcase(&OutputFitStat)=YES %then %do;
								data OutLib.fst_sel_os&_k; set fst_sel_os; run;
								
									proc transpose data=fst_sel_os(
										%if %index(%upcase(&WhichProc),LOGISTIC) %then %do;
											rename=(
												criterion= descr 
												interceptandcovariates= value
											)
										%end;
										%else %if %index(%upcase(&WhichProc),PHREG) %then %do;
											rename=(
												criterion= descr  
												withcovariates= value
											)
										%end;
										)
										out=fst_sel_ost(drop= _name_); 
										var value;
										id descr;
										/*by _order_ %if &OutputMajAllele %then MajorAllele; &ByVars;*/
									run;
								
								data fst_all;
									length Effect Trait $32;
									set fst_all(in=orig) fst_sel_ost;
									if not orig then do;
										Effect= "Genotype";
										Trait = "&TraitVar";
										_order_ = %scan(&selMarkerVarsOrder_t,&_k);
									end;
								run;				
							%end;

							%let _k = %sysevalf(&_k + 1);	 
						%end;

						/*data outlib.t_ta_all; set ta_all; run;*/
						/*data outlib.t_pat_all; set pat_all; run;*/
						/*
						%if %upcase(&OutputResid)=YES or %upcase(&OutputPred)=YES %then %do;
							data outlib.t_outrest_all; set outrest_all; run;
						%end;
						*/
						/*
						%if (%length(&KVars) or %length(&RandEff)) %then %do;
							data outlib.t_cpt_all; set cpt_all; run;
						%end;
						*/
						/*
						%if %upcase(&OutputFitStat)=YES %then %do;
							data outlib.t_fst_all; set fst_all; run; 
						%end;
						*/

						%Nobsvars(ta_all);
						%if &nobs=0 %then %do;
							%put ERROR: Trend tests did not run successfully.  See the SAS log for more details.;
							%let exiterror=1;
							%goto exit;
						%end;

						proc sort data=Ta_all out=effect_map(keep=Effect Trait _order_) nodupkey;
							by _order_;
						run;

						data mapdata2_t; 
							merge mapdata %if %length(&selmarkervarsorder_t) %then (where=(_order_ in (%sysfunc(tranwrd(&selmarkervarsorder_t, %str( ), %str(,)))))); effect_map;
							by _order_;
						run;	

						data ta_all;
							merge mapdata2_t ta_all 
								%if %upcase(&OutputFitStat)=YES %then %do;
									fst_all(
										%if %index(%upcase(&WhichProc),MIXED) %then %do;
											rename= (
											_2_Res_Log_Likelihood= _2_Res_Log_Likelihood_Trend
											AIC__Smaller_is_Better_= AIC_Smaller_is_Better_Trend
											AICC__Smaller_is_Better_= AICC_Smaller_is_Better_Trend
											BIC__Smaller_is_Better_= BIC_Smaller_is_Better_Trend
											)
										%end;
									)
								%end;
							;
							by _order_ &ByVars Effect;
							%if %Upcase(&Stat) ne CHISQ %then %do;
								rename NumDF=NumDF_Trend DenDF=DenDF_Trend FValue=FValue_Trend Prob&Stat=ProbTrend;
								if DenDF=0 then Rsquare_Trend = .;
								else Rsquare_Trend = (NumDF*FValue/DenDF)/(1+NumDF*FValue/DenDF);
							%end;
							%else %do;
								rename DF=DF_Trend WaldChiSq=WaldChiSq_Trend Prob&Stat=ProbTrend;
							%end;
						run; 

						proc sort data=ta_all;
							by &AnnoGroup &AnnoLoc &ByVars Effect;
						run;				
						/*data outlib.t_tal_all_f0; set ta_all; run;*/

						%if %upcase(&OutputResid)=YES or %upcase(&OutputPred)=YES %then %do;

							%Nobsvars(outrest_all);
							%let rds = %sysfunc(open(outrest_all));
							%if %upcase(&OutputResid)=YES %then %let residnum = %sysfunc(varnum(&rds,RESID));
							%if %upcase(&OutputPred)=YES %then %let prednum = %sysfunc(varnum(&rds,PRED));
							%let rc=%sysfunc(close(&rds));

							%if %upcase(&OutputResid)=YES and %upcase(&OutputPred)=YES %then %do;
								%if &residnum=0 %then %let _outpnum= &residnum;
								%else %if &prednum=0 %then %let _outpnum= &prednum;
								%else %let _outpnum= &residnum;
							%end;
							%else %if %upcase(&OutputResid)=YES and %upcase(&OutputPred)^=YES  %then %do;
								%let _outpnum= &residnum;
							%end;
							%else %if %upcase(&OutputResid)^=YES and %upcase(&OutputPred)=YES  %then %do;
								%let _outpnum= &prednum;
							%end;

							%if &nobs=0 or &_outpnum=0 %then %do;
								%put ERROR: The trend residuals could not be computed.;
								%let OutputResid=NO;
								%let OutputPred=NO;
							%end;
							%else %do;

								data outrest_all;
									merge mapdata outrest_all;
									by _order_;
									%if %length(&selmarkervarsorder_t) %then %do;  
										where _order_ in (%sysfunc(tranwrd(&selmarkervarsorder_t, %str( ), %str(,))));
									%end;
								run;

								proc sort data=outrest_all;
									by _individ_ _order_ _Trait_ ;
								run;
								/*data outlib.outrest_all; set outrest_all; run;*/

								%if %upcase(&OutputResid)=YES %then %do; 
									proc transpose data=outrest_all out=outrestt(drop= _name_) let
										prefix=Resid_;
										var resid;
										id &AnnoLabel;
										by _IndivID_;
									run;

									/*data outlib.othervars; set othervars; run;*/
									data Assoc_ORT;
										 merge othervars outrestt;
										 by _IndivID_;
									run;
									/*data outlib.Assoc_ORT; set Assoc_ORT; run;*/
								%end;
								
								%if %upcase(&OutputPred)=YES %then %do; 
									proc transpose data=outrest_all out=outpredtt(drop= _name_) let
										prefix=Pred_;
										var pred;
										id &AnnoLabel;
										by _IndivID_;
									run;

									/*data outlib.othervars; set othervars; run;*/
									data Assoc_OPT;
										 merge othervars outpredtt;
										 by _IndivID_;
									run;
									/*data outlib.Assoc_OPT; set Assoc_OPT; run;*/
								%end;	
							%end;
						%end;	

						data Assoc_PET;
							merge mapdata pat_all;
							by _order_;
							%if %length(&selmarkervarsorder_t) %then %do;  
								where _order_ in (%sysfunc(tranwrd(&selmarkervarsorder_t, %str( ), %str(,))));
							%end;
							*drop _order_ _ESTTYPE_;
							drop _ESTTYPE_;
						run;	
						/*data outlib.Assoc_PET; set Assoc_PET; run;*/

						%if (%length(&KVars) or %length(&RandEff)) %then %do;
							data Assoc_CPT;
								merge mapdata cpt_all;
								by _order_;
								%if %length(&selmarkervarsorder_t) %then %do;  
									where _order_ in (%sysfunc(tranwrd(&selmarkervarsorder_t, %str( ), %str(,))));
								%end;
								*drop _order_;
							run;
							/*data outlib.Assoc_CPT; set Assoc_CPT; run;*/
						%end;		

						proc datasets; 	
							delete Assoc_ta;
							change ta_all = Assoc_ta;
						quit;		
						/*data outlib.Assoc_ta; set Assoc_ta; run;*/
					%end;
				%end;
				**end: running trend test after filtering out significant selected variables on the unrotated data*;			
			%end; /*end if trend=1*/
			
			%skip_2:;

			%if &GenoTestError and &TrendTestError %then %goto exit;

		%end; /*end if FitSelModelOnUnrotatedData = YES */
	%end; /*end if SelVars is not empty */
	
	**reset original macro variables*;
	%let SpeedK = &OrigSpeedK ;
	%let FixCovParms = &OrigFixCovParms;
	%let ParmsStmt = &OrigParmsStmt; 
	%let Listing = &OrigListing;
	
	/*
	%if %upcase(&OrigFixCovParms)=YES and (%length(&KVars) %then %do;
		%let ParmsStmt = &ParmsStmt ;
		%let FixCovParms = NO;
	%end;
	%else 
		%let ParmsStmt = &OrigParmsStmt; 
	*/
	%*goto exit;
%end;

%skip_0:;

%*goto exit;

%global QKSelProf qkdepdata;
%let QKSelProf = 0; /*1 means Yes. This macro variable will be used within SNPTraitAssocMacro.sas and GenMacros.sas*/			
%let qkdepdata = ; /*This macro variable will be used within GenMacros.sas. S1168164: Luciano added qkdepdata to fix issue of file not found on the server.*/

%let InsertQKOutput = YES;
%let Run_Psmooth = YES;
%let Run_JSLPvalMap = YES;
%if %upcase(&MultiMarkerModel) ^= YES %then %do;
	%if %length(&KVars) %then %SNPTraitAssocMacro(kmodel=1,lsm=&lsmi);
	%else %SNPTraitAssocMacro(lsm=&lsmi);
	%if &exiterror=1 %then %goto exit;
	
	/*data outlib.ma_all; set ma_all; run;*/
%end;
%else %if (%upcase(&MultiMarkerModel)= YES and %upcase(&FitSelModelOnUnrotatedData)= YES) %then %do;
		
	%if %length(&SelVars) %then %do;
	
		%let exist_mal = 0;
		%let exist_peg = 0;
		%let exist_cpg = 0;
		%let exist_lsm = 0;
		%let exist_pet = 0;
		%let exist_cpt = 0;
		
		%if %length(&selMarkerVars_g) %then %do;
			data _selsnpg;
				set snptmpnames;
				where _tmpsnp in ( 
					"%scan(&selMarkerVars_g, 1)"
					%let k_= 2;
					%do %while(%length(%scan(&selMarkerVars_g, &k_)));
						%str(,) "%scan(&selMarkerVars_g, &k_)"
						%let k_= %sysevalf(&k_ + 1);
					%end;
					)
				;
			run;

			data _null_;
				set _selsnpg end= _eof_;
				call symputx("_oldfeffg"||trim(left(_n_)), trim(left(_tmpsnp)));
				call symputx("_newfeffg"||trim(left(_n_)), trim(left(_orgsnp)));
				if _eof_ then call symputx("_nsnpg", trim(left(_n_)));
			run;
		%end;
		
		%let numbSelMarkerVars_g = 0;
		%if %length(&selMarkerVars_g) %then %do;
			%let numbSelMarkerVars_g = 1;
			%do %while(%length(%scan(&selMarkerVars_g, &numbSelMarkerVars_g)));
				%let numbSelMarkerVars_g = %sysevalf(&numbSelMarkerVars_g + 1);
			%end;
			%let numbSelMarkerVars_g = %sysevalf(&numbSelMarkerVars_g - 1);
		%end;
		
		%let numbSelMarkerVars_t = 0;
		%if %length(&selMarkerVars_t) %then %do;
			%let numbSelMarkerVars_t = 1;
			%do %while(%length(%scan(&selMarkerVars_t, &numbSelMarkerVars_t)));
				%let numbSelMarkerVars_t = %sysevalf(&numbSelMarkerVars_t + 1);
			%end;
			%let numbSelMarkerVars_t = %sysevalf(&numbSelMarkerVars_t - 1);
		%end;
		
		%put NumbSNPin= &NumbSNPin;
		%put numbSelMarkerVars_g= &numbSelMarkerVars_g;
		%put numbSelMarkerVars_t= &numbSelMarkerVars_t;
		
		%if (&NumbSNPin = &numbSelMarkerVars_g) and (&NumbSNPin = &numbSelMarkerVars_t) %then 
			%let SelModelProfile = NO;
		%put SelModelProfile= &SelModelProfile;
		
		%if (%upcase(&SelModelProfile) = YES) and (%length(&selMarkerVars_g) or %length(&selMarkerVars_t)) %then %do;
			**test unselected markers conditional on selected model even if selected model has no marker*;
			
			%let geno_orginal= &geno;
			%if &geno=1 and (&NumbSNPin = &numbSelMarkerVars_g) %then %do;
				%let geno= 0;
				
			%end;
			%let trend_orginal= &trend;
			%if &trend=1 and (&NumbSNPin = &numbSelMarkerVars_t) %then %do;
				%let trend= 0;
			%end;
			%put geno= &geno;
			%put trend= &trend;

			%let QKSelProf = 1; 		
			
			%if %length(&KVars) %then %SNPTraitAssocMacro(kmodel=1,lsm=&lsmi);
			%else %SNPTraitAssocMacro(lsm=&lsmi);
			%if &exiterror=1 %then %goto exit;
			
			%let exist_mal = 1;
			
			%let geno= &geno_orginal;
			%let trend= &trend_orginal;
			
			/*data outlib.ma_all_; set ma_all; run;*/

			%if &geno=1 and %length(&selMarkerVars_g) and (&NumbSNPin ^= &numbSelMarkerVars_g) %then %do;
						
				%if &lsmi=1 %then %do;
					/*data outlib.outlsm_all_s; set outlsm_all_s; run;*/
					/*data outlib.outlsm_all; set outlsm_all; run;*/
			
					data outlsm_all;
						set 
						outlsm_all_s(keep= &ByVars Trait Effect _order_ Genotype Estimate StdErr DF tValue Probt) 
						outlsm_all(keep= &ByVars Trait Effect _order_ Genotype Estimate StdErr DF tValue Probt);
					run;
					
					proc sort data=outlsm_all overwrite;
						by &ByVars _order_ genotype;
					run;
					/*data outlib.outlsm_all_f0; set outlsm_all; run;*/
									
					%if %length(&MTC) %then %do;
						%if %length(&ByVars) %then %do;
							proc sort data=outlsm_all;
								by &ByVars _order_;
							run;
						%end;

						%AdjPValueName(method=%scan(&mtc,1));
						proc multtest pdata(probt)=outlsm_all %scan(&mtc,1) out=adjdiffs;
							%if %length(&ByVars) %then %do;
								by &ByVars;
							%end;
						run;
						%let transds = adjdiffs;
					%end;
					%else %do;				
						%let mtpvar = Probt;
						%if %length(&ByVars) %then %do;
							proc sort data=outlsm_all;
								by &ByVars _order_;
							run;
						%end;
					%end;
					
					proc transpose data=&transds out=trans1(drop=_name_) prefix=LSDiff_ suffix=_0;
						var estimate;
						by  &ByVars _order_;
						id Genotype;
					run;

					proc transpose data=&transds out=trans2(keep=ProbLS:) prefix=ProbLSDiff_ suffix=_0;
						var &mtpvar;
						by  &ByVars _order_;
						id Genotype;
					run;

					data lsmtrans;
						set trans1; set trans2;
						Effect = "Genotype";
						%if %length(&Conv) %then %do;
							array problsa{*} problsdiff_:;
							do _p=1 to dim(problsa);
								%if %upcase(&Conv)=NEGLOG %then %do;
									problsa{_p} = -log(problsa{_p});
								%end;
								%else %do;
									problsa{_p} = -log10(problsa{_p});
								%end;			
							end;
							drop _p;
						%end;					
					run;

					proc sort data=lsmtrans out=outlsm_all;
						by _order_ &ByVars Effect ;
					run;
					/*data outlib.g_outlsm_all_f; set outlsm_all; run;*/
					
					data outlsm_all;
						merge mapdata2 outlsm_all;
						by _order_ &ByVars Effect;
					run;
					
					proc sort data=outlsm_all;
						by &AnnoGroup &AnnoLoc &ByVars Effect ;
					run;
					/*data outlib.outlsm_all_ff; set outlsm_all; run;*/
				%end;							

				data ma_all;
					merge ma_all Assoc_ma(in=_a_) %if &lsmi=1 %then outlsm_all;;
					by &AnnoGroup &AnnoLoc &ByVars Effect;
					if _a_ then Selected_Geno = 1;
					else Selected_Geno = .;
				run;

				data Assoc_PEG;
					set Assoc_PEG;
					Genotype = .;
				run;
				/*data outlib.Assoc_PEG_; set Assoc_PEG; run;*/

				data &SNPTraitAssoc_PEG;
					set Assoc_PEG &SNPTraitAssoc_PEG;
				run;
				%let exist_peg = 1;

				%if &lsmi=1 %then %do;
					data Assoc_LSM;
						set Assoc_LSM;
						Genotype = .;
					run;

					data &SNPTraitAssoc_LSM;
						 set Assoc_LSM &SNPTraitAssoc_LSM;
					run;

					%let exist_lsm = 1;
				%end;

				%if %upcase(&OutputResid)=YES %then %do;
					data &SNPTraitAssoc_ORG;
						merge Assoc_ORG &SNPTraitAssoc_ORG(keep= _IndivID_ Res:);
						by _IndivID_;
					run;
				%end;
				
				%if %upcase(&OutputPred)=YES %then %do;
					data &SNPTraitAssoc_OPG;
						merge Assoc_OPG &SNPTraitAssoc_OPG(keep= _IndivID_ Pred:);
						by _IndivID_;
					run;
				%end;

				%if (%length(&KVars) or %length(&RandEff)) %then %do;
					data &SNPTraitAssoc_CPG;
						set Assoc_CPG &SNPTraitAssoc_CPG;
					run;
					%let exist_cpg = 1;
				%end;
			%end;
			%else %if &geno=1 and %length(&selMarkerVars_g) and (&NumbSNPin = &numbSelMarkerVars_g) %then %do;

				%if &lsmi=1 %then %do;
					/*data outlib.outlsm_all_s; set outlsm_all_s; run;*/
			
					data outlsm_all;
						set outlsm_all_s(keep= &ByVars Trait Effect _order_ Genotype Estimate StdErr DF tValue Probt);
					run;
					
					proc sort data=outlsm_all overwrite;
						by &ByVars _order_ genotype;
					run;
					/*data outlib.outlsm_all_f0; set outlsm_all; run;*/
									
					%if %length(&MTC) %then %do;
						%if %length(&ByVars) %then %do;
							proc sort data=outlsm_all;
								by &ByVars _order_;
							run;
						%end;

						%AdjPValueName(method=%scan(&mtc,1));
						proc multtest pdata(probt)=outlsm_all %scan(&mtc,1) out=adjdiffs;
							%if %length(&ByVars) %then %do;
								by &ByVars;
							%end;
						run;
						%let transds = adjdiffs;
					%end;
					%else %do;				
						%let mtpvar = Probt;
						%if %length(&ByVars) %then %do;
							proc sort data=outlsm_all;
								by &ByVars _order_;
							run;
						%end;
					%end;
					
					proc transpose data=&transds out=trans1(drop=_name_) prefix=LSDiff_ suffix=_0;
						var estimate;
						by  &ByVars _order_;
						id Genotype;
					run;

					proc transpose data=&transds out=trans2(keep=ProbLS:) prefix=ProbLSDiff_ suffix=_0;
						var &mtpvar;
						by  &ByVars _order_;
						id Genotype;
					run;

					data lsmtrans;
						set trans1; set trans2;
						Effect = "Genotype";
						%if %length(&Conv) %then %do;
							array problsa{*} problsdiff_:;
							do _p=1 to dim(problsa);
								%if %upcase(&Conv)=NEGLOG %then %do;
									problsa{_p} = -log(problsa{_p});
								%end;
								%else %do;
									problsa{_p} = -log10(problsa{_p});
								%end;			
							end;
							drop _p;
						%end;					
					run;

					proc sort data=lsmtrans out=outlsm_all;
						by _order_ &ByVars Effect ;
					run;
					/*data outlib.g_outlsm_all_f; set outlsm_all; run;*/
					
					data outlsm_all;
						merge mapdata2 outlsm_all;
						by _order_ &ByVars Effect;
					run;
					
					proc sort data=outlsm_all;
						by &AnnoGroup &AnnoLoc &ByVars Effect ;
					run;
					/*data outlib.outlsm_all_ff; set outlsm_all; run;*/
				%end;	

				data ma_all;
					merge ma_all Assoc_ma(in=_a_) %if &lsmi=1 %then outlsm_all;;
					by &AnnoGroup &AnnoLoc &ByVars Effect;
					if _a_ then Selected_Geno = 1;
					else Selected_Geno = .;
				run;

				data &SNPTraitAssoc_PEG;
					set Assoc_PEG;
				run;
				%let exist_peg = 1;
			
				%if &lsmi=1 %then %do;
					data &SNPTraitAssoc_LSM;
						set Assoc_LSM;
					run;
					%let exist_lsm = 1;
				%end;
			
				%if %upcase(&OutputResid)=YES %then %do;
					data &SNPTraitAssoc_ORG;
						set Assoc_ORG; 
					run;
				%end;
				
				%if %upcase(&OutputPred)=YES %then %do;
					data &SNPTraitAssoc_OPG;
						set Assoc_OPG; 
					run;
				%end;
			
				%if (%length(&KVars) or %length(&RandEff)) %then %do;
					data &SNPTraitAssoc_CPG;
						set Assoc_CPG;
					run;
					%let exist_cpg = 1;
				%end;
			
			%end;

			%if &trend=1 and %length(&selMarkerVars_t) and (&NumbSNPin ^= &numbSelMarkerVars_t) %then %do;

				data ma_all;
					merge ma_all Assoc_ta(in=_a_);
					by &AnnoGroup &AnnoLoc &ByVars Effect;
					if _a_ then Selected_Trend = 1;
					else Selected_Trend = .;
				run;
				/*data outlib.ma_all_end; set ma_all; run;*/
				/*data outlib.Assoc_PET_; set &SNPTraitAssoc_PET; run;*/

				data &SNPTraitAssoc_PET;
					set Assoc_PET &SNPTraitAssoc_PET;
				run;
				%let exist_pet = 1;

				%if %upcase(&OutputResid)=YES %then %do;
					/*data outlib.Assoc_ORT_; set &SNPTraitAssoc_ORT; run;*/

					data &SNPTraitAssoc_ORT;
						merge Assoc_ORT &SNPTraitAssoc_ORT(keep= _IndivID_ Res:);
						by _IndivID_;
					run;
				%end;
				
				%if %upcase(&OutputPred)=YES %then %do;
					/*data outlib.Assoc_OPT_; set &SNPTraitAssoc_OPT; run;*/

					data &SNPTraitAssoc_OPT;
						merge Assoc_OPT &SNPTraitAssoc_OPT(keep= _IndivID_ Pred:);
						by _IndivID_;
					run;
				%end;

				%if (%length(&KVars) or %length(&RandEff)) %then %do;
					/*data outlib.Assoc_CPT_; set &SNPTraitAssoc_CPT; run;*/

					data &SNPTraitAssoc_CPT;
						set Assoc_CPT &SNPTraitAssoc_CPT;
					run;
					%let exist_cpt = 1;
				%end;
			%end;
			%else %if &trend=1 and %length(&selMarkerVars_t) and (&NumbSNPin = &numbSelMarkerVars_t) %then %do;

				data ma_all;
					merge ma_all Assoc_ta(in=_a_);
					by &AnnoGroup &AnnoLoc &ByVars Effect;
					if _a_ then Selected_Trend = 1;
					else Selected_Trend = .;
				run;

				data &SNPTraitAssoc_PET;
					set Assoc_PET;
				run;
				%let exist_pet = 1;

				%if %upcase(&OutputResid)=YES %then %do;
					data &SNPTraitAssoc_ORT;
						set Assoc_ORT;
					run;
				%end;
				
				%if %upcase(&OutputPred)=YES %then %do;
					data &SNPTraitAssoc_OPT;
						set Assoc_OPT;
					run;
				%end;

				%if (%length(&KVars) or %length(&RandEff)) %then %do;
					data &SNPTraitAssoc_CPT;
						set Assoc_CPT; 
					run;
					%let exist_cpt = 1;
				%end;

			%end;			
		%end;
		%else %if (%upcase(&SelModelProfile) ^= YES) and (%length(&selMarkerVars_g) or %length(&selMarkerVars_t)) %then %do;

			%*let Run_Psmooth = NO;
			%let Run_JSLPvalMap = NO;

			%if &geno=1 and %length(&selMarkerVars_g) %then %do;

				data &SNPTraitAssoc_PEG;
					set Assoc_PEG;
				run;
				%let exist_peg = 1;

				%if &lsmi=1 %then %do;
					data &SNPTraitAssoc_LSM;
						set Assoc_LSM;
					run;
					%let exist_lsm = 1;
				%end;

				%if %upcase(&OutputResid)=YES %then %do;
					data &SNPTraitAssoc_ORG;
						set Assoc_ORG; 
					run;
				%end;
				
				%if %upcase(&OutputPred)=YES %then %do;
					data &SNPTraitAssoc_OPG;
						set Assoc_OPG; 
					run;
				%end;

				%if (%length(&KVars) or %length(&RandEff)) %then %do;
					data &SNPTraitAssoc_CPG;
						set Assoc_CPG;
					run;
					%let exist_cpg = 1;
				%end;

			%end;

			%if &trend=1 and %length(&selMarkerVars_t) %then %do;

				data &SNPTraitAssoc_PET;
					set Assoc_PET;
				run;
				%let exist_pet = 1;

				%if %upcase(&OutputResid)=YES %then %do;
					data &SNPTraitAssoc_ORT;
						set Assoc_ORT;
					run;
				%end;
				
				%if %upcase(&OutputPred)=YES %then %do;
					data &SNPTraitAssoc_OPT;
						set Assoc_OPT;
					run;
				%end;

				%if (%length(&KVars) or %length(&RandEff)) %then %do;
					data &SNPTraitAssoc_CPT;
						set Assoc_CPT; 
					run;
					%let exist_cpt = 1;
				%end;

			%end;

			%if (&geno=1 and %length(&selMarkerVars_g)) and (&trend=1 and %length(&selMarkerVars_t)) %then %do;
				data ma_all;
					merge Assoc_ma Assoc_ta;
					by &AnnoGroup &AnnoLoc &ByVars Effect;
				run;
			%end;
			%else %if &geno=1 and %length(&selMarkerVars_g) %then %do;
				data ma_all;
					set Assoc_ma;
				run;
			%end;
			%else %if &trend=1 and %length(&selMarkerVars_t) %then %do;
				data ma_all;
					set Assoc_ta;
				run;
			%end;
			%let exist_mal = 1;
		%end;
		%else %do;
			%let InsertQKOutput = NO;
			%let Run_Psmooth = NO;
			%let Run_JSLPvalMap = NO;
			%put WARNING: No marker variables selected after filtering out selected markers. You may want to try relaxing the SL for Keeping Variables in the Selected Model.;
		%end;
	
		/*
		%if &exist_mal %then %do;
			data ma_all;
				length _MarkerTmp_ $32;
				set ma_all;
				_MarkerTmp_ = "&mrkprefix"||trim(left(_order_));
			run;		
		%end;
		*/

		%if &exist_peg %then %do;				
			/*
			data &SNPTraitAssoc_PEG;
				length _MarkerTmp_ $32;
				set &SNPTraitAssoc_PEG;
				_MarkerTmp_ = "&mrkprefix"||trim(left(_order_));
			run;
			*/
			
			data &SNPTraitAssoc_PEG;
				set &SNPTraitAssoc_PEG;
				%if %length(&selMarkerVars_g) %then %do;
					rename 
					%do i=1 %to &_nsnpg;
						&&_oldfeffg&i = &&_newfeffg&i
					%end;
					;
				%end;
			run;

			proc sort data=&SNPTraitAssoc_PEG overwrite;
				by _order_;
			run;		
		%end;

		%if &exist_cpg %then %do;
			/*
			data &SNPTraitAssoc_CPG;
				length _MarkerTmp_ $32;
				set &SNPTraitAssoc_CPG;
				_MarkerTmp_ = "&mrkprefix"||trim(left(_order_));
			run;
			*/
			proc sort data=&SNPTraitAssoc_CPG overwrite;
				by _order_;
			run;		
		%end;

		%if &exist_lsm %then %do;
			/*
			data &SNPTraitAssoc_LSM;
				length _MarkerTmp_ $32;
				set &SNPTraitAssoc_LSM;
				_MarkerTmp_ = "&mrkprefix"||trim(left(_order_));
			run;
			*/
			
			data &SNPTraitAssoc_LSM;
				set &SNPTraitAssoc_LSM;
				%if %length(&selMarkerVars_g) %then %do;
					rename 
					%do i=1 %to &_nsnpg;
						&&_oldfeffg&i = &&_newfeffg&i
					%end;
					;
				%end;
			run;
			
			proc sort data=&SNPTraitAssoc_LSM overwrite;
				by _order_;
			run;		
		%end;	

		%if &exist_pet %then %do;
			/*
			data &SNPTraitAssoc_PET;
				length _MarkerTmp_ $32;
				set &SNPTraitAssoc_PET;
				_MarkerTmp_ = "&mrkprefix"||trim(left(_order_));
			run;
			*/
			
			proc sort data=&SNPTraitAssoc_PET overwrite;
				by _order_;
			run;		
		%end;

		%if &exist_cpt %then %do;
			/*
			data &SNPTraitAssoc_CPT;
				length _MarkerTmp_ $32;
				set &SNPTraitAssoc_CPT;
				_MarkerTmp_ = "&mrkprefix"||trim(left(_order_));
			run;
			*/
			proc sort data=&SNPTraitAssoc_CPT overwrite;
				by _order_;
			run;		
		%end;
	%end;
	%else %do;
		%let InsertQKOutput = NO;	
		%let Run_Psmooth = NO;
		%let Run_JSLPvalMap = NO;
		%put WARNING: No variables selected for trait &TraitVar.. You may want to try relaxing the selection criteria or using a different model selection method.;
	%end;
%end;
%else %if (%upcase(&MultiMarkerModel)= YES and %upcase(&FitSelModelOnUnrotatedData ^= YES)) %then %do;
	%let InsertQKOutput = NO;
	%let Run_Psmooth = NO;
	%let Run_JSLPvalMap = NO;
	%put WARNING: Results shown are those from the analyses of the rotated data.;
%end;

%let existjslfile = 0;
%if %upcase(&MultiMarkerModel)= YES %then %do;
	
	%let existjslfile = 1;
	
	%if %length(&KVars) %then %do;
		%global QKMM_CP0_OutData;
		%let QKMM_CP0_OutData = ;
		%PrepOutDS(QKMM_CP0_OutData, QKMM_CP0_OutDataName,&OutName._cp0);
		data &QKMM_CP0_OutData;
			set &CovParamData;
			drop _traitvar_;
			Estimate = round(Estimate, .00001);
		run;
		%PackageInsertDataset(Estimated Covariance from Using Rotated Data from &ProcessName, outlib, &QKMM_CP0_OutDataName, No, No, No);					
	%end;

	%global QKMM_SV0_OutData;
	%let QKMM_SV0_OutData = ;
	%PrepOutDS(QKMM_SV0_OutData, QKMM_SV0_OutDataName,&OutName._sv0);
	
	data pe_sel;
		set pe_sel;
		_tmpord_ = _n_;
	run;
	
	proc sql UNDO_POLICY= NONE;
		create table pe_sel(drop=_tmpord_ _tmpsnp) as
		select *
		from pe_sel as a1 left join snptmpnames as a2
		on a1.Effect = a2._tmpsnp
		order by _tmpord_
		;
	quit;
	
	data &QKMM_SV0_OutData;
		length Effect $32;
		set pe_sel;
		Estimate = round(Estimate, .00001);
		StandardizedEst = round(StandardizedEst, .00001);
		StdErr = round(StdErr, .00001);
		if not missing(_orgsnp) then Effect= _orgsnp;
		drop Parameter _orgsnp;
	run;
	%PackageInsertDataset(Selected Variables Using Rotated Data from &ProcessName, outlib, &QKMM_SV0_OutDataName, No, No, No);
	
	%*put FitSelModel= &FitSelModelOnUnrotatedData &SelVars;
	%if %upcase(&FitSelModelOnUnrotatedData)= YES and %length(&SelVars) %then %do;
		%if &geno = 1 %then %do;
			%if %sysfunc(exist(ma_sel_o)) %then %do;
				%global QKMM_GV1_OutData;
				%let QKMM_GV1_OutData = ;
				%PrepOutDS(QKMM_GV1_OutData, QKMM_GV1_OutDataName,&OutName._gv1);
				
				data ma_sel_o;
					set ma_sel_o;
					_tmpord_ = _n_;
				run;

				proc sql UNDO_POLICY= NONE;
					create table ma_sel_o(drop=_tmpord_ _tmpsnp) as
					select *
					from ma_sel_o as a1 left join snptmpnames as a2
					on a1.Effect = a2._tmpsnp
					order by _tmpord_
					;
				quit;
		
				data &QKMM_GV1_OutData;
					length Effect $32;
					set ma_sel_o;
					if not missing(_orgsnp) then Effect= _orgsnp;
					drop _orgsnp;
				run;
				%PackageInsertDataset(Genotype Test of Selected Variables for the Unrotated Data from &ProcessName, outlib, &QKMM_GV1_OutDataName, No, No, No);					

				%global QKMM_PG1_OutData;
				%let QKMM_PG1_OutData = ;
				%PrepOutDS(QKMM_PG1_OutData, QKMM_PG1_OutDataName,&OutName._pg1);
				
				data pag_sel_o;
					set pag_sel_o;
					_tmpord_ = _n_;
				run;

				proc sql UNDO_POLICY= NONE;
					create table pag_sel_o(drop=_tmpord_ _tmpsnp) as
					select *
					from pag_sel_o as a1 left join snptmpnames as a2
					on a1.Effect = a2._tmpsnp
					order by _tmpord_
					;
				quit;
				
				%let _nsnp1= 0;
				data pag_sel_o;
					length Effect $32;
					set pag_sel_o end=_eof_;
					if _n_=1 then _count= 0;
					if not missing(_orgsnp) then do;
						_count= _count + 1;
						call symputx("_oldeff"||trim(left(_count)), trim(left(Effect)));
						call symputx("_neweff"||trim(left(_count)), trim(left(_orgsnp)));
						Effect= _orgsnp;
					end;
					if _eof_ then call symputx("_nsnp1", trim(left(_count)));
					retain _count;
					drop _orgsnp _count;
				run;
		
				data &QKMM_PG1_OutData;
					set pag_sel_o;
					Estimate = round(Estimate, .00001);
					StdErr = round(StdErr, .00001);	
					%if &_nsnp1 > 0 %then %do;
						rename
						%do i=1 %to &_nsnp1;
							&&_oldeff&i= &&_neweff&i
						%end;
						;
					%end;
				run;
				%PackageInsertDataset(Estimated Effects from Genotype Test of Selected Variables for the Unrotated Data from &ProcessName, outlib, &QKMM_PG1_OutDataName, No, No, No);					
			
				%if %length(&KVars) %then %do;
					%global QKMM_CG1_OutData;
					%let QKMM_CG1_OutData = ;
					%PrepOutDS(QKMM_CG1_OutData, QKMM_CG1_OutDataName,&OutName._cg1);
					data &QKMM_CG1_OutData;
						set gcp_sel_o;
						drop _traitvar_;
						Estimate = round(Estimate, .00001);
					run;
					%PackageInsertDataset(Estimated Covariance from Genotype Test of Selected Variables for the Unrotated Data from &ProcessName, outlib, &QKMM_CG1_OutDataName, No, No, No);					
				%end;			
			
			%end;
			%if %sysfunc(exist(ma_sel_os)) %then %do;
				%global QKMM_GV2_OutData;
				%let QKMM_GV2_OutData = ;
				%PrepOutDS(QKMM_GV2_OutData, QKMM_GV2_OutDataName,&OutName._gv2);
				
				data ma_sel_os;
					set ma_sel_os;
					_tmpord_ = _n_;
				run;

				proc sql UNDO_POLICY= NONE;
					create table ma_sel_os(drop=_tmpord_ _tmpsnp) as
					select *
					from ma_sel_os as a1 left join snptmpnames as a2
					on a1.Effect = a2._tmpsnp
					order by _tmpord_
					;
				quit;
		
				data &QKMM_GV2_OutData;
					length Effect $32;
					set ma_sel_os;
					if not missing(_orgsnp) then Effect= _orgsnp;
					drop _orgsnp;
				run;
				%PackageInsertDataset(Genotype Test of Filtered Selected Variables for the Unrotated Data from &ProcessName, outlib, &QKMM_GV2_OutDataName, No, No, No);					

				%global QKMM_PG2_OutData;
				%let QKMM_PG2_OutData = ;
				%PrepOutDS(QKMM_PG2_OutData, QKMM_PG2_OutDataName,&OutName._pg2);
		
				data pag_sel_os;
					set pag_sel_os;
					_tmpord_ = _n_;
				run;

				proc sql UNDO_POLICY= NONE;
					create table pag_sel_os(drop=_tmpord_ _tmpsnp) as
					select *
					from pag_sel_os as a1 left join snptmpnames as a2
					on a1.Effect = a2._tmpsnp
					order by _tmpord_
					;
				quit;	
				
				%let _nsnp2= 0;
				data pag_sel_os;
					length Effect $32;
					set pag_sel_os;
					if _n_ = 1 then _count= 0;
					if not missing(_orgsnp) then do;
						_count= _count + 1;
						call symputx("_oldeff"||trim(left(_count)), trim(left(Effect)));
						call symputx("_neweff"||trim(left(_count)), trim(left(_orgsnp)));
						Effect= _orgsnp;
					end;
					if _eof_ then call symputx("_nsnp2", trim(left(_count)));
					retain _count;
					drop _orgsnp _count;
				run;
		
				data &QKMM_PG2_OutData;
					set pag_sel_os;
					Estimate = round(Estimate, .00001);
					StdErr = round(StdErr, .00001);	
					%if &_nsnp2 > 0 %then %do;
						rename
						%do i=1 %to &_nsnp2;
							&&_oldeff&i= &&_neweff&i
						%end;
						;
					%end;
				run;
				%PackageInsertDataset(Estimated Effects from Genotype Test of Filtered Selected Variables for the Unrotated Data from &ProcessName, outlib, &QKMM_PG2_OutDataName, No, No, No);					
			
				%if %length(&KVars) %then %do;
					%global QKMM_CG2_OutData;
					%let QKMM_CG2_OutData = ;
					%PrepOutDS(QKMM_CG2_OutData, QKMM_CG2_OutDataName,&OutName._cg2);
					data &QKMM_CG2_OutData;
						set gcp_sel_os;
						drop _traitvar_;
						Estimate = round(Estimate, .00001);
					run;
					%PackageInsertDataset(Estimated Covariance from Genotype Test of Filtered Selected Variables for the Unrotated Data from &ProcessName, outlib, &QKMM_CG2_OutDataName, No, No, No);					
				%end;			
			%end;
		%end;
		
		%if &trend = 1 %then %do; 
			%if %sysfunc(exist(ta_sel_o)) %then %do;
				%global QKMM_TV1_OutData;
				%let QKMM_TV1_OutData = ;
				%PrepOutDS(QKMM_TV1_OutData, QKMM_TV1_OutDataName,&OutName._tv1);
		
				data ta_sel_o;
					set ta_sel_o;
					_tmpord_ = _n_;
				run;
				
				proc sql UNDO_POLICY= NONE;
					create table ta_sel_o(drop=_tmpord_ _tmpsnp) as
					select *
					from ta_sel_o as a1 left join snptmpnames as a2
					on a1.Effect = a2._tmpsnp
					order by _tmpord_
					;
				quit;
				
				data &QKMM_TV1_OutData;
					length Effect $32;
					set ta_sel_o;
					if not missing(_orgsnp) then Effect= _orgsnp;
					drop _orgsnp;
				run;
				%PackageInsertDataset(Trend Test of Selected Variables for the Unrotated Data from &ProcessName, outlib, &QKMM_TV1_OutDataName, No, No, No);					

				%global QKMM_PT1_OutData;
				%let QKMM_PT1_OutData = ;
				%PrepOutDS(QKMM_PT1_OutData, QKMM_PT1_OutDataName,&OutName._pt1);
		
				data pat_sel_o;
					set pat_sel_o;
					_tmpord_ = _n_;
				run;
				
				proc sql UNDO_POLICY= NONE;
					create table pat_sel_o(drop=_tmpord_ _tmpsnp) as
					select *
					from pat_sel_o as a1 left join snptmpnames as a2
					on a1.Effect = a2._tmpsnp
					order by _tmpord_
					;
				quit;
				
				data &QKMM_PT1_OutData;
					length Effect $32;
					set pat_sel_o;
					Estimate = round(Estimate, .00001);
					StdErr = round(StdErr, .00001);	
					if not missing(_orgsnp) then Effect= _orgsnp;
					drop _orgsnp;
				run;
				%PackageInsertDataset(Trend Test of Selected Variables for the Unrotated Data from &ProcessName, outlib, &QKMM_PT1_OutDataName, No, No, No);					

				%if %length(&KVars) %then %do;
					%global QKMM_CT1_OutData;
					%let QKMM_CT1_OutData = ;
					%PrepOutDS(QKMM_CT1_OutData, QKMM_CT1_OutDataName,&OutName._ct1);
					data &QKMM_CT1_OutData;
						set tcp_sel_o;
						drop _traitvar_;
						Estimate = round(Estimate, .00001);
					run;
					%PackageInsertDataset(Estimated Covariance from Trend Test of Selected Variables for the Unrotated Data from &ProcessName, outlib, &QKMM_CT1_OutDataName, No, No, No);					
				%end;

			%end;
			%if %sysfunc(exist(ta_sel_os)) %then %do;
				%global QKMM_TV2_OutData;
				%let QKMM_TV2_OutData = ;
				%PrepOutDS(QKMM_TV2_OutData, QKMM_TV2_OutDataName,&OutName._tv2);
		
				data ta_sel_os;
					set ta_sel_os;
					_tmpord_ = _n_;
				run;
				
				proc sql UNDO_POLICY= NONE;
					create table ta_sel_os(drop=_tmpord_ _tmpsnp) as
					select *
					from ta_sel_os as a1 left join snptmpnames as a2
					on a1.Effect = a2._tmpsnp
					order by _tmpord_
					;
				quit;
				
				data &QKMM_TV2_OutData;
					length Effect $32;
					set ta_sel_os;
					if not missing(_orgsnp) then Effect= _orgsnp;
					drop _orgsnp;
				run;
				%PackageInsertDataset(Trend Test of Filtered Selected Variables for the Unrotated Data from &ProcessName, outlib, &QKMM_TV2_OutDataName, No, No, No);					
			
				%global QKMM_PT2_OutData;
				%let QKMM_PT2_OutData = ;
				%PrepOutDS(QKMM_PT2_OutData, QKMM_PT2_OutDataName,&OutName._pt2);
		
				data pat_sel_os;
					set pat_sel_os;
					_tmpord_ = _n_;
				run;
				
				proc sql UNDO_POLICY= NONE;
					create table pat_sel_os(drop=_tmpord_ _tmpsnp) as
					select *
					from pat_sel_os as a1 left join snptmpnames as a2
					on a1.Effect = a2._tmpsnp
					order by _tmpord_
					;
				quit;
				
				data &QKMM_PT2_OutData;
					length Effect $32;
					set pat_sel_os;
					Estimate = round(Estimate, .00001);
					StdErr = round(StdErr, .00001);
					if not missing(_orgsnp) then Effect= _orgsnp;
					drop _orgsnp;
				run;
				%PackageInsertDataset(Trend Test of Selected Variables for the Unrotated Data from &ProcessName, outlib, &QKMM_PT2_OutDataName, No, No, No);					

				%if %length(&KVars) %then %do;
					%global QKMM_CT2_OutData;
					%let QKMM_CT2_OutData = ;
					%PrepOutDS(QKMM_CT2_OutData, QKMM_CT2_OutDataName,&OutName._ct2);
					data &QKMM_CT2_OutData;
						set tcp_sel_os;
						drop _traitvar_;
						Estimate = round(Estimate, .00001);
					run;
					%PackageInsertDataset(Estimated Covariance from Trend Test of Selected Variables for the Unrotated Data from &ProcessName, outlib, &QKMM_CT2_OutDataName, No, No, No);					
				%end;			
			%end;
		%end;	
	%end;
	
	%global _mainjslfile_;
	%let _mainjslfile_ = %quote(&QKMMScript);
	
	data _null_;
		file "&QKMMScript" lrecl=32767;
		put '//!';
		put "Names Default to Here(1);";
		put "jg:drill_down = 0;";
		put "if(host is(Windows),";
		put "	include( ConvertFilePath(""$LS_HOME/Generator/DialogHelperFunctions.jsl"", absolute, windows) ),";
		put "	include( ConvertFilePath(""$LS_HOME/Generator/DialogHelperFunctions.jsl"", absolute, POSIX) ));";
		put "	";		
		put "Here:DataList = {};";
		put "Here:TabButtonList = {};";
		put "Here:TabCodeList = {};";
		put "Here:TabAfterCodeList = {};";
		put "Here:TabOpenList = {};";
		put "Here:ModelSelSumm = Function({},{default local},";
		%if %length(&KVars) %then %do;
			put "	If(IsEmpty(Here:QKMM_CP0_Data),";
			put "		Here:QKMM_CP0_Data=open(""&ClientOutpath.&&QKMM_CP0_OutDataName..sas7bdat"", invisible);";			
			put "	);";
		%end;
		put "	If(IsEmpty(Here:QKMM_SV0_Data),";
		put "		Here:QKMM_SV0_Data=open(""&ClientOutpath.&&QKMM_SV0_OutDataName..sas7bdat"", invisible);";			
		put "	);";
		put "	HListBox(, ";
		put "		VListBox(, ";
		put "			VListBox(, ";
		%if %length(&KVars) %then %do;
			put "			Here:o= OutLineBox(""Step 0: Genetic and Residual Variance Estimates from the Null Model Fitted on the Unrotated Data"",";
			put "						Data Table Box(Here:QKMM_CP0_Data); ";
			put "				),";
			put "			Here:o= OutLineBox(""Step 1: Selected Variables from the Rotated Data"",";
			put "				VListBox(, ";
			put "					Here:olbrot= OutLineBox(""Selected Variables"",";
		%end;
		%else %do;
			put "			Here:o= OutLineBox(""Step 1: Selected Variables from the Unrotated Data"",";
			put "				VListBox(, ";
			put "					Here:olbrot= OutLineBox(""Selected Variables"",";
		%end;	
		put "							Vlistbox( ";
		put "								Data Table Box(Here:QKMM_SV0_Data); ";
		put "							),";
		%if &SelectMethod ^= None and %length(&SelVars)=0 %then %do;
			%if &InterceptSelected = 1 %then %do;
				put "					TextBox(""No variables selected for trait &TraitVar.. Intercept only model. You may want to try relaxing the selection criteria or using a different model selection method."",<<FontColor(3),<< Set Wrap(800)),";
			%end;
			%else %do;
				put "					TextBox(""No variables selected for trait &TraitVar.. You may want to try relaxing the selection criteria or using a different model selection method."",<<FontColor(3),<< Set Wrap(800)),";
			%end;			
		%end;
		put "						),";
		put "					),";
		put "				),";
		put "			),";
		%if %upcase(&FitSelModelOnUnrotatedData)= YES and %length(&SelVars) %then %do;
			%if &geno = 1 and %sysfunc(exist(ma_sel_o)) or &trend = 1 and %sysfunc(exist(ta_sel_o)) %then %do;
				put "			VListBox(, ";
				put "			Here:o= OutLineBox(""Step 2: Selected Variables Fitted on the Unrotated Data"",";
				%if &geno = 1 and %sysfunc(exist(ma_sel_o)) %then %do;
					put "			If(IsEmpty(Here:QKMM_GV1_Data),";
					put "				Here:QKMM_GV1_Data=open(""&ClientOutpath.&&QKMM_GV1_OutDataName..sas7bdat"", invisible);";
					put "			);";
					put "			If(IsEmpty(Here:QKMM_PG1_Data),";
					put "				Here:QKMM_PG1_Data=open(""&ClientOutpath.&&QKMM_PG1_OutDataName..sas7bdat"", invisible);";
					put "			);";
					%if %length(&KVars) %then %do;
						put "			If(IsEmpty(Here:QKMM_CG1_Data),";
						put "				Here:QKMM_CG1_Data=open(""&ClientOutpath.&&QKMM_CG1_OutDataName..sas7bdat"", invisible);";
						put "			);";								
					%end;
					put "			VListBox(OutLineBox(""Genotype"", ";
					put "			HListBox(, ";
					%if %length(&KVars) %then %do;
						put "				Here:olb= OutLineBox(""Genetic and Residual Variance Estimates"",";
						put "					Vlistbox( ";
						put "						Data Table Box(Here:QKMM_CG1_Data); ";
						put "					)";
						put "				),";
					%end;
					put "				Here:olb= OutLineBox(""Genotype Test"",";
					put "					Vlistbox( ";
					put "						Data Table Box(Here:QKMM_GV1_Data); ";
					put "					)";
					put "				),";
					put "				Here:olb= OutLineBox(""Genotype Estimates"",";
					put "					Vlistbox( ";
					put "						Data Table Box(Here:QKMM_PG1_Data); ";
					put "					)";
					put "				,<< Close(1)),";
					put "			),";
					put "			),";
					put "			),";
				%end;
				%if &trend = 1 and %sysfunc(exist(ta_sel_o)) %then %do;
					put "			If(IsEmpty(Here:QKMM_TV1_Data),";
					put "				Here:QKMM_TV1_Data=open(""&ClientOutpath.&&QKMM_TV1_OutDataName..sas7bdat"", invisible);";
					put "			);";
					put "			If(IsEmpty(Here:QKMM_PT1_Data),";
					put "				Here:QKMM_PT1_Data=open(""&ClientOutpath.&&QKMM_PT1_OutDataName..sas7bdat"", invisible);";
					put "			);";
					%if %length(&KVars) %then %do;
						put "			If(IsEmpty(Here:QKMM_CT1_Data),";
						put "				Here:QKMM_CT1_Data=open(""&ClientOutpath.&&QKMM_CT1_OutDataName..sas7bdat"", invisible);";
						put "			);";								
					%end;	
					put "			VListBox(OutLineBox(""Trend"", ";
					put "			HListBox(, ";
					%if %length(&KVars) %then %do;
						put "				Here:olb= OutLineBox(""Genetic and Residual Variance Estimates"",";
						put "					Vlistbox( ";
						put "						Data Table Box(Here:QKMM_CT1_Data); ";
						put "					)";
						put "				),";
					%end;
					put "				Here:olb= OutLineBox(""Trend Test"",";
					put "					Vlistbox( ";
					put "						Data Table Box(Here:QKMM_TV1_Data); ";
					put "					)";
					put "				),";
					put "				Here:olb= OutLineBox(""Trend Estimates"",";
					put "					Vlistbox( ";
					put "						Data Table Box(Here:QKMM_PT1_Data); ";
					put "					)";
					put "				,<< Close(1)),";
					put "			),";
					put "			),";
					put "			),";
				%end;
				put "			),";
				put "			),";
			%end;
			%if (&geno = 1) or (&trend = 1) %then %do;
				put "			VListBox(, ";
				put "			Here:o= OutLineBox(""Step 3: Filtered Selected Variables Fitted on the Unrotated Data"",";
				%if &geno = 1 and %sysfunc(exist(ma_sel_os)) %then %do;
					put "			If(IsEmpty(Here:QKMM_GV2_Data),";
					put "				Here:QKMM_GV2_Data=open(""&ClientOutpath.&&QKMM_GV2_OutDataName..sas7bdat"", invisible);";
					put "			);";
					put "			If(IsEmpty(Here:QKMM_PG2_Data),";
					put "				Here:QKMM_PG2_Data=open(""&ClientOutpath.&&QKMM_PG2_OutDataName..sas7bdat"", invisible);";
					put "			);";
					%if %length(&KVars) %then %do;
						put "			If(IsEmpty(Here:QKMM_CG2_Data),";
						put "				Here:QKMM_CG2_Data=open(""&ClientOutpath.&&QKMM_CG2_OutDataName..sas7bdat"", invisible);";
						put "			);";								
					%end;		
					put "			VListBox(OutLineBox(""Genotype"", ";
					put "			HListBox(, ";
					%if %length(&KVars) %then %do;
						put "				Here:olb= OutLineBox(""Genetic and Residual Variance Estimates"",";
						put "					Vlistbox( ";
						put "						Data Table Box(Here:QKMM_CG2_Data); ";
						put "					)";
						put "				),";
					%end;
					put "				Here:olb= OutLineBox(""Genotype Test"",";
					put "					Vlistbox( ";
					put "						Data Table Box(Here:QKMM_GV2_Data); ";
					put "					)";
					put "				),";
					put "				Here:olb= OutLineBox(""Genotype Estimates"",";
					put "					Vlistbox( ";
					put "						Data Table Box(Here:QKMM_PG2_Data); ";
					put "					)";
					put "				,<< Close(1)),";
					put "			),";
					put "			),";
					put "			),";
				%end;
				%else %if &geno = 1 and ^%sysfunc(exist(ma_sel_os)) %then %do;
					put "			VListBox(, ";
					put "				Here:olb= OutLineBox(""Genotype Test"",";
					put "					TextBox(""No marker variables selected for trait &TraitVar after filtering out selected markers using the Genotype test. You may want to try relaxing the SL for Keeping Variables in the Selected Model."",<<FontColor(3),<< Set Wrap(800)),";
					put "				)";
					put "			),";				
				%end;
				%if &trend = 1 and %sysfunc(exist(ta_sel_os)) %then %do;
					put "			If(IsEmpty(Here:QKMM_TV2_Data),";
					put "				Here:QKMM_TV2_Data=open(""&ClientOutpath.&&QKMM_TV2_OutDataName..sas7bdat"", invisible);";
					put "			);";
					put "			If(IsEmpty(Here:QKMM_PT2_Data),";
					put "				Here:QKMM_PT2_Data=open(""&ClientOutpath.&&QKMM_PT2_OutDataName..sas7bdat"", invisible);";
					put "			);";
					%if %length(&KVars) %then %do;
						put "			If(IsEmpty(Here:QKMM_CT2_Data),";
						put "				Here:QKMM_CT2_Data=open(""&ClientOutpath.&&QKMM_CT2_OutDataName..sas7bdat"", invisible);";
						put "			);";								
					%end;	
					put "			VListBox(OutLineBox(""Trend"", ";
					put "			HListBox(, ";
					%if %length(&KVars) %then %do;
						put "				Here:olb= OutLineBox(""Genetic and Residual Variance Estimates"",";
						put "					Vlistbox( ";
						put "						Data Table Box(Here:QKMM_CT2_Data); ";
						put "					)";
						put "				),";
					%end;
					put "				Here:olb= OutLineBox(""Trend Test"",";
					put "					Vlistbox( ";
					put "						Data Table Box(Here:QKMM_TV2_Data); ";
					put "					)";
					put "				),";
					put "				Here:olb= OutLineBox(""Trend Estimates"",";
					put "					Vlistbox( ";
					put "						Data Table Box(Here:QKMM_PT2_Data); ";
					put "					)";
					put "				,<< Close(1)),";
					put "			),";
					put "			),";
					put "			),";
				%end;
				%else %if &trend = 1 and ^%sysfunc(exist(ta_sel_os)) %then %do;
					put "			VListBox(, ";
					put "				Here:olb= OutLineBox(""Trend Test"",";
					put "					TextBox(""No marker variables selected for trait &TraitVar after filtering out selected markers using the Trend test. You may want to try relaxing the SL for Keeping Variables in the Selected Model."",<<FontColor(3),<< Set Wrap(800)),";				
					put "				)";
					put "			),";				
				%end;
				put "			),";
				put "			),";	
			%end;
		%end;
		put "		), ";
		put "	), ";
		put ");";
		put "InsertInto(DataList,{{"" ""}});";
		put "InsertInto(TabButtonList, ""Model Selection Summary"");";
		put "InsertInto(TabCodelist,expr(Here:ModelSelSumm()));";
		put "InsertInto(TabAfterCodeList, "" "");";
		put "InsertInto(TabOpenList,1);";			
		
		/*
		put "	ActionButtonMessage=NewWindow("" Warning Message"", showmenu(0), showtoolbars(0),";
		put "		VListBox(";
		put "			tb=TextBox(""Warning message: Hello Luciano."",";
		put "				<<Font Color(5),<<Set Font Size(10),<<Set Wrap(500)";
		put "			),";
		put "			TextBox("" ""),";
		put "			ButtonBox(""OK"",ActionButtonMessage<<CloseWindow);";
		put "		);";
		put "	);";
		*/		
	run;	
	
	%if %upcase(&CreateHTML)=YES %then %do;
		%PackageInsertHtml(html from &ProcessName, &OutPath, &bodyfile, &framefile, &contentsfile);
	%end;
%end;

%if %upcase(&InsertQKOutput) = YES %then %do;

	%if &geno=1 %then %do;
	
		%PackageInsertDataset(Genotype parameter estimate data from &ProcessName, OutLib, &SNPTraitAssoc_PEGName, _type=other);

		%if &lsmi=1 %then
			%PackageInsertDataset(Genotype LS means and diffs from &ProcessName, OutLib, &SNPTraitAssoc_LSMName, _type=other);;

		%if %upcase(&OutputResid)=YES %then 
			%PackageInsertDataset(Genotype residual data from &ProcessName, OutLib, &SNPTraitAssoc_ORGName);;

		%if %upcase(&OutputPred)=YES %then 
			%PackageInsertDataset(Genotype predicted data from &ProcessName, OutLib, &SNPTraitAssoc_OPGName);;

		%if (%length(&KVars) or %length(&RandEff)) %then 
			%PackageInsertDataset(Genotype covariance parameter data from &ProcessName, OutLib, &SNPTraitAssoc_CPGName, _type=other);;

	%end;

	%if &trend=1 %then %do;

		%PackageInsertDataset(Trend parameter estimate data from &ProcessName, OutLib, &SNPTraitAssoc_PETName, _type=other);;

		%if %upcase(&OutputResid)=YES %then 
			%PackageInsertDataset(Trend residual data from &ProcessName, OutLib, &SNPTraitAssoc_ORTName);;

		%if %upcase(&OutputPred)=YES %then 
			%PackageInsertDataset(Trend predicted data from &ProcessName, OutLib, &SNPTraitAssoc_OPTName);;

		%if (%length(&KVars) or %length(&RandEff)) %then
			%PackageInsertDataset(Trend covariance parameter data from &ProcessName, OutLib, &SNPTraitAssoc_CPTName, _type=other);;

	%end;
	
	%put ByVars = &ByVars;
	%put AnnoGroup = &AnnoGroup;
	%put AnnoLoc = &AnnoLoc;
	proc sort data=ma_all out=&QKMM_OutData;
	 by &ByVars &AnnoGroup &AnnoLoc;
	run;
	
	/*
	%if %length(&PlotVar2)>4 %then %let QTests=&PlotVar1 &PlotVar2;
	%else %let QTests = &PlotVar1;
	*/

	%*put Run_PsmoothLuc= &Run_Psmooth;
	%if %upcase(&Run_Psmooth) = YES %then %do;
		/*data OutLib.QKMM_OutDataName; set &QKMM_OutData; run;*/
	
		%Psmooth(&QTests, &QKMM_OutData, &QKMM_OutDataName, _lsp = &lsmi);
		%if (&syserr>4) %then %goto exit;
	%end;
	
	%PackageInsertDataset(P-value data from &ProcessName, outlib, &OutName._qkm, yes, yes, _type=anno);

	%if %upcase(&Run_JSLPvalMap) = YES %then %do;
		%if &NByVal > 1 %then %let ssr=0;
		%else %let ssr=1;
		%if &MarkerFmt = %str(Numeric Genotypes) %then %let _pi = 1;
		%else %let _pi = 0;
		%put NumGpsL = &NumGps;
		%put volcanoplot = &volcanoplot;
		%JSLPvalMap(Q-K Mixed Model, &QKMMScript, &QKMMScriptName, &QKMM_OutDataName, mod= &existjslfile, _volplot=&volcanoplot, _subsetdd=&ssr, _lsvp=&lsmi, _plotinput=&_pi);
	%end;
	
%end;

**S1168164: Luciano added qkdepdata to fix issue of file not found on the server**;
%if %length(&qkdepdata) = 0 %then %let qkdepdata= &QKMM_OutDataName; 
%put qkdepdata= &qkdepdata;

%PackageInsertFile(jsl code from &ProcessName, &OutPath, &QKMMScriptName, "&qkdepdata", text/plain, yes);

%exit:;

%ExitSDSProcess(&OutPath, &ProcessName, &syserr, &exiterror, &clockrunning);
%mend;

* run the macro;
%QKMixedModel;

