/*
JMP Genomics Analytical Process

Name: Q-K Model Fitness
Author: Luciano Silva

-----------------------------------------------------------
Copyright (C) 2014 SAS Institute, Inc. All rights reserved. 

Notice: 
The above copyright notice and this notice must appear in
any whole or partial copy of this code and any related
documentation. Permission to use, copy, and modify the
source code contained in this file is hereby granted, but
is limited to customers of SAS with a valid license for the
SAS software product with which this file was provided. 
Except as expressly set forth herein, the terms of such
license apply.
-----------------------------------------------------------
*/

*ProcessVariables;

%global StudyName InData ListTraitVars TraitVar AnnoInData AnnoLabel AnnoMName AnnoGroup AnnoLoc 
AnnoAccession AnnoMajAllele AnnoWhereStmt  
DDFM UseLowerB Listing OutPath ServerOutPath MacroPath GMethod MMethod EstMethod
QSumto1Model1 QSumto1Model2 QSumto1Model3 PedIDKModel1 PedIDKModel2 KVarsModel1 KVarsModel2
ListKVarsModel1 ListKVarsModel2 CompressedKModel1 CompressedKModel2
QVarsModel1 ListQVarsModel1 QVarsModel2 ListQVarsModel2 QVarsModel3 ListQVarsModel3 
TraitType ClassVars 
FixedEff RandEff RandOpts  
SelectQKModel SelectQKModelCriterion OutName
QKFE_OutData QKFE_Script;
 
*ProcessBody;

%macro QKModelFitness;

	%global ParmsStmt;
	%let ParmsStmt= ;

	%let OrigClassVars= &ClassVars;
	%let OrigRandEff= &RandEff;
	%let OrigParmsStmt= &ParmsStmt;

	ods noresults;
	ods html close;
	ods listing close;
	options nonotes;
	options label;

	%include "&MacroPath./GenMacros.sas" / nosource;
	
	/*begin: Needed to avoid error in the %GenInit()*/
	%global MarkerFmt;
	%let MarkerFmt= ;
	/*end*/	
	%GenInit(QKModelFitness,28);
	%if &exiterror=1 %then %goto exit;
	
	%let InDataPath= &TmpPath;
	%put InDataPath= &InDataPath;
	
	%global FullAPName;
	%let FullAPName=%str(Q-K Model Fitness);
	libname OutLib "&OutPath";
	
	%if %sysevalf(%length(&OutName) >= 17) %then
		%let OutName= %substr(&OutName, 1, 17);
	%put OutNameLuciano=&OutName;
	
	%PrepOutFile(QKFE_Script,QKFE_ScriptName,&ProcessName Script for &OutName..jsl);
	
	%global ByVars CensorVar;
	%let ByVars=;
	%let CensorVar=;
	%let StrataVars=;

	%if %length(&ListTraitVars &TraitVar)=0 %then %do;
		%put ERROR: One or more trait variables must be specified in either the Trait Variables or List-Style Specification of Trait Variables parameters.;
		%let exiterror=1;
		%goto exit;
	%end;
	
	%let TraitVar = &TraitVar &ListTraitVars;

	/*
	%if %upcase(&FixCovParms)=YES and (%upcase(&DDFM)=SATTERTHWAITE or %upcase(&DDFM)=KENWARDROGER) %then %do;
		%put ERROR: You cannot select SATTERTHWAITE or KENWARDROGER for the Denominator Degrees of Freedom Method when the Fix Covariance Parameters checkbox is checked.;
		%let exiterror=1;
		%goto exit;
	%end;
	*/

 	*begin: Luciano because PrepMapandBy requires AlleVars, which does not exist when running QKModelFitness*;		
	%let MaporBy= No;
	%if %upcase(&MaporBy) = YES %then %do;
		%PrepMapandBy;
		run;
		%if &exiterror=1 %then %goto exit;
	%end;
	%else %do;
		%put InData=&InData;
		%global ds;
		%let ds= ds;
		data &ds;
			set InLib.&InData;
		run;
	%end;
	*end*;
	
	%if %length(&ListQVarsModel1) %then %do;
		data Qvarsm1;
			set &ds(keep=&ListQVarsModel1 obs=1);
		run;

	   	%let dsid=%sysfunc(open(Qvarsm1));
	   	%let nvars= %sysfunc(attrn(&dsid,NVARS));
	   	%do i=1 %to &nvars;
			%let QVarsModel1=&QVarsModel1 %sysfunc(varname(&dsid,&i));
		%end;
		%let rc = %sysfunc(close(&dsid));
	%end;	

	%if %upcase(&QSumto1Model1)=YES and %length(&QVarsModel1)>0 %then %do;
		* Remove last Q var since sum to 1 *;
		%let Orig_QVarsModel1 = &QVarsModel1;
		%let QVarsModel1 = ;
		%let w=1;
		%do %while(%length(%scan(&Orig_QVarsModel1,%sysevalf(&w+1))));
			%let QVarsModel1 = &QVarsModel1 %scan(&Orig_QVarsModel1,&w);
			%let w=%sysevalf(&w+1);
		%end;
		%if %length(&QVarsModel1)=0 %then %do;
			%let QVarsModel1 = &Orig_QVarsModel1;
			%put WARNING: There is only one Q Matrix variable selected for Q1 Model so the Columns of Q Matrix Sum to 1 checkbox is ignored for Q1 Model.;
		%end;
	%end;
	%else %if %upcase(&QSumto1Model1)=YES and %length(&QVarsModel1)=0 %then %do;
		%put WARNING: Q Matrix variable were not selected for Q1 Model so the Columns of Q Matrix Sum to 1 checkbox is ignored for Q1 Model.;	
	%end;
	%put QVarsModel1= &QVarsModel1;
	
	%if %length(&ListQVarsModel2) %then %do;
		data Qvarsm2;
			set &ds(keep=&ListQVarsModel2 obs=1);
		run;

	   	%let dsid=%sysfunc(open(Qvarsm2));
	   	%let nvars= %sysfunc(attrn(&dsid,NVARS));
	   	%do i=1 %to &nvars;
			%let QVarsModel2=&QVarsModel2 %sysfunc(varname(&dsid,&i));
		%end;
		%let rc = %sysfunc(close(&dsid));
	%end;
	%put QVarsModel2= &QVarsModel2;
	
	%if %upcase(&QSumto1Model2)=YES and %length(&QVarsModel2)>0 %then %do;
		* Remove last Q var since sum to 1 *;
		%let Orig_QVarsModel2 = &QVarsModel2;
		%let QVarsModel2 = ;
		%let w=1;
		%do %while(%length(%scan(&Orig_QVarsModel2,%sysevalf(&w+1))));
			%let QVarsModel2 = &QVarsModel2 %scan(&Orig_QVarsModel2,&w);
			%let w=%sysevalf(&w+1);
		%end;
		%if %length(&QVarsModel2)=0 %then %do;
			%let QVarsModel2 = &Orig_QVarsModel2;
			%put WARNING: There is only one Q Matrix variable selected for Q2 Model so the Columns of Q Matrix Sum to 1 checkbox is ignored for Q2 Model.;
		%end;
	%end;
	%else %if %upcase(&QSumto1Model2)=YES and %length(&QVarsModel2)=0 %then %do;
		%put WARNING: Q Matrix variable were not selected for Q2 Model so the Columns of Q Matrix Sum to 1 checkbox is ignored for Q2 Model.;	
	%end;
	
	%if %length(&ListQVarsModel3) %then %do;
		data Qvarsm3;
			set &ds(keep=&ListQVarsModel3 obs=1);
		run;

	   	%let dsid=%sysfunc(open(Qvarsm3));
	   	%let nvars= %sysfunc(attrn(&dsid,NVARS));
	   	%do i=1 %to &nvars;
			%let QVarsModel3=&QVarsModel3 %sysfunc(varname(&dsid,&i));
		%end;
		%let rc = %sysfunc(close(&dsid));
	%end;
	%put QVarsModel3= &QVarsModel3;
	
	%if %upcase(&QSumto1Model3)=YES and %length(&QVarsModel3)>0%then %do;
		* Remove last Q var since sum to 1 *;
		%let Orig_QVarsModel3 = &QVarsModel3;
		%let QVarsModel3 = ;
		%let w=1;
		%do %while(%length(%scan(&Orig_QVarsModel3,%sysevalf(&w+1))));
			%let QVarsModel3 = &QVarsModel3 %scan(&Orig_QVarsModel3,&w);
			%let w=%sysevalf(&w+1);
		%end;
		%if %length(&QVarsModel3)=0 %then %do;
			%let QVarsModel3 = &Orig_QVarsModel3;
			%put WARNING: There is only one Q Matrix variable selected for Q3 Model so the Columns of Q Matrix Sum to 1 checkbox is ignored for Q3 Model.;
		%end;
	%end;
	%else %if %upcase(&QSumto1Model3)=YES and %length(&QVarsModel3)=0 %then %do;
		%put WARNING: Q Matrix variable were not selected for Q3 Model so the Columns of Q Matrix Sum to 1 checkbox is ignored for Q3 Model.;	
	%end;

	%if %length(&ListKVarsModel1) %then %do;
		data KVarsM1;
			set &ds(keep=&ListKVarsModel1 obs=1);
		run;

		%let dsid=%sysfunc(open(KVarsM1));
	   	%let nvars= %sysfunc(attrn(&dsid,NVARS));
	   	%do i=1 %to &nvars;
			%let KVarsModel1=&KVarsModel1 %sysfunc(varname(&dsid,&i));
		%end;
		%let rc = %sysfunc(close(&dsid));
	%end;	
	%put KVarsModel1= &KVarsModel1;
	
	%Nobsvars(&ds);
	%let nobsds= &nobs;
	
	%if %length(&KVarsModel1) and %length(&PedIDKModel1)=0 and %upcase(&CompressedKModel1)=NO %then %do;
		data _null_;
			set &ds(obs=1);
			array ks{*} &KVarsModel1;
			call symputx("NumKVars",dim(ks));
		run;
		%put nobsds= &nobsds;
		%put NumKVars= &NumKVars;
	
		%if &nobsds ne &NumKVars %then %do;
			%put ERROR: The number of columns in the K Matrix selected for K1 Model must be the same as the number of observations in the input data set.;
			%let exiterror=1;
			%goto exit;
		%end;
	%end;
	
	%if %length(&ListKVarsModel2) %then %do;
		data KVarsM2;
			set &ds(keep=&ListKVarsModel2 obs=1);
		run;

		%let dsid=%sysfunc(open(KVarsM2));
		%do i=1 %to %sysfunc(attrn(&dsid,nvars));
			%let KVarsModel2=&KVarsModel2 %sysfunc(varname(&dsid,&i));
		%end;
		%let rc = %sysfunc(close(&dsid));
	%end;
	%put KVarsModel2= &KVarsModel2;
	
	%if %length(&KVarsModel2) and %length(&PedIDKModel2)=0 and %upcase(&CompressedKModel2)=NO %then %do;
		data _null_;
			set &ds(obs=1);
			array ks{*} &KVarsModel2;
			call symputx("NumKVars",dim(ks));
		run;
		%put nobsds= &nobsds;
		%put NumKVars= &NumKVars;

		%if &nobsds ne &NumKVars %then %do;
			%put ERROR: The number of columns in the K Matrix selected for K2 Model must be the same as the number of observations in the input data set.;
			%let exiterror=1;
			%goto exit;
		%end;
	%end;

	* Table name from all but MIXED *;
	%global ParmEst EffName;
	%let ParmEst=ParameterEstimates;
	%let EffName=Effect;

	%let WhichProc=MIXED;
	%let TraitOpt= ;
	%let Stat=F; 
	%let Gtable=Tests3;
	%let TStmt=contrast;
	%let Ttable=Contrasts;
	%let usestrata=0;
	%let ModOption=%quote(/ htype=3 ddfm=&DDFM s); 
	/*%if %upcase(&OutputResid)=YES %then %let ModOption = %quote(&ModOption residual outp=outres);*/
	%let ClassOpt=;
	/*%let teststmt=%str('genotypes' Genotype 1);*/
	%let ProcOption= %str(Method=&MMethod);
	%let ParmEst = SolutionF;
	%let EffName = Effect;
	%*if %length(&PedID) %then %let ClassVars = &ClassVars &PedID;

	* Need a separate check for OR when using GLIMMIX since not creating OR data set *;
	%let AnyOdds=0;
	%if %upcase(&TraitType)=COUNT %then %do;
		%let WhichProc=GLIMMIX; 
		%let ModOption=%quote(/ link=log dist=p htype=3 ddfm=&DDFM s);
		/*%if %upcase(&OutputResid)=YES %then %let ModOption = %quote(&ModOption) %str(; output out=outres resid=Resid);*/
		%let ParmEst = ParameterEstimates;
	%end;
	%else %if %upcase(&TraitType)=BINARY or %upcase(&TraitType)=NOMINAL or %upcase(&TraitType)=ORDINAL %then %do;
		%if %length(&RandEff)>0 or %length(&KVarsModel1)>0 or %length(&KVarsModel2)>0 %then %do;
			%let WhichProc=GLIMMIX;
			/*%let ProcOption=%str(IC= PQ Method=&GMethod);*/
			%let ProcOption=%str(Method=&GMethod);
			%if %upcase(&TraitType)=NOMINAL %then 
				%let TraitOpt=(Ref=FIRST);
			%else 
				%let TraitOpt=(Event=LAST);		
			%if %upcase(&EstMethod)=YES %then 
				%let ProcOption=%quote(&ProcOption EMPIRICAL);
			%let Stat=F;     
			%let Gtable=Tests3;	
			%let TStmt=estimate;				
			%let Ttable=Estimates;		
			%let ParmEst = ParameterEstimates;
			%if %upcase(&DDFM)=CONTAIN or %upcase(&DDFM)=BETWITHIN %then 
				%let DDFM=RESIDUAL;
			%if %upcase(&TraitType)=BINARY %then 
				%let ModOption=%quote(/ link=logit dist=binary htype=3 ddfm=&DDFM s);
			%else %if %upcase(&TraitType)=NOMINAL %then %do;
				%let ModOption=%quote(/ link=glogit dist=mult htype=3 ddfm=betwithin s);
				%let ClassVars = &ClassVars &TraitVar;
			%end;
			%else %if %upcase(&TraitType)=ORDINAL %then 
				%let ModOption=%quote(/ link=clogit dist=mult htype=3 ddfm=&DDFM s);
			/*%if %upcase(&OutputResid)=YES %then %let ModOption = %quote(&ModOption) %str(; output out=outres resid=Resid);*/
			/*%let teststmt=%str('genotypes' Genotype 1);*/		
			/*%if %upcase(&TraitType)=NOMINAL or %upcase(&TraitType)=ORDINAL %then %let lsmi=0;*/
		%end;
		%else %do;
			%let WhichProc=LOGISTIC;
			%let ProcOption=;
			%let TraitOpt=(Event=LAST);		
			%let Stat=ChiSq;
			%*let Gtable=Type3;
			%let Gtable=ModelANOVA; /*S1146207: The logistic output formerly named Type3 is now named ModelANOVA*/
			%let TStmt=contrast;
			%let Ttable=ContrastTest;
			%let Otable=OddsRatiosWald;
			%let usestrata=1;
			%if %upcase(&TraitType)=NOMINAL %then 
				%let ModOption=%str(/ link=glogit);
			%else %if %length(StrataVars)>0 %then 
				%let ModOption=/ GCONV=1E-6; 
			%else %let ModOption =;
			/*%if %upcase(&OutputResid)=YES %then %let ModOption = %quote(&ModOption) %str(; output out=outres resdev=Resid);*/
			%let ClassOpt = %str(/ cprefix=8);
			/*%let teststmt=%str('genotypes' Genotype 1);*/
			%let ParmEst = ParameterEstimates;
			%let EffName = Variable;
			/*%let lsmi=0;*/
		%end;
	%end; 
	
	%put WhichProc= &WhichProc;

	/*
	%if %upcase(&OutputLSM)=YES and &lsmi=0 %then %do;
		%put LS means cannot be calculated for this model.;
	%end;
	*/

	%if &usestrata=0 %then %let StrataVars=;

	/*S1089309
	%if %upcase(&UseLowerB)=YES %then %do;
	*/
	%if (%upcase(&WhichProc)= MIXED or %upcase(&WhichProc)= GLIMMIX) %then %do;
		%if %upcase(&UseLowerB)=YES %then %do;
			%if %length(&ParmsStmt) %then %do;
				%if %index(&ParmsStmt,%str(/)) %then %do;
					%let ParmsStmt = %quote(&ParmsStmt lowerb=0);
				%end;
				%else %do;
					%let ParmsStmt = %quote(&ParmsStmt / lowerb=0);
				%end;
			%end;
			%else %let ParmsStmt = %str( / lowerb=0);
		%end;
	%end;
	%else %let ParmsStmt=;
	
	*begin: get trait variables*;
	data a1;
		set &ds(obs=1);
	run;

	%put TraitVarLuciano1= &TraitVar;

	proc transpose data=a1 out=at;
		var &TraitVar;
	run;

	%if &syserr>4 %then %do;
		%let exiterror=1;
		%goto exit;
	%end;

	%NObsVars(at);
	%global NTrait;
	%let Ntrait = &nobs;
	%put NTrait= &NTrait;
	%if &NTrait=0 %then %do;
		%let exiterror=1;
		%put ERROR: No variables found matching the Trait Variables specified.;
		%goto exit;
	%end;
	%if &NTrait > 1 %then %let OutputSurvFunc = NO;

	%do i=1 %to &Ntrait; 
		%global _tv&i;
	%end; 

	proc sort data=at;
		by _name_;
	run;

	data _null_;
		set at;
		length is $16;
		is = "_tv" || strip(_n_);
		call symput(is,strip(_name_));
	run;
	*end: get trait variables*;
	
	*begin: get models to be compared*;
	%let wQModels= 0;
	%let nQModels= 1;
	%if %length(&QVarsModel1) %then %do;
		%let wQModels= &wQModels 1; 
		%let nQModels= %sysevalf(&nQModels + 1);
	%end;
	%if %length(&QVarsModel2) %then %do;
		%let wQModels= &wQModels 2; 
		%let nQModels= %sysevalf(&nQModels + 1);
	%end;
	%if %length(&QVarsModel3) %then %do;
		%let wQModels= &wQModels 3;
		%let nQModels= %sysevalf(&nQModels + 1);
	%end;
	%put wQModels= &wQModels;
	
	%let wKModels= 0;
	%let nKModels= 1;
	%if %length(&KVarsModel1) %then %do;
		%let wKModels= &wKModels 1; 
		%let nKModels= %sysevalf(&nKModels + 1);
	%end;
	%if %length(&KVarsModel2) %then %do;
		%let wKModels= &wKModels 2; 
		%let nKModels= %sysevalf(&nKModels + 1);
	%end;
	%put wKModels= &wKModels;
	
	%let i=1;
	%let wQM= %scan(&wQModels, &i);	
	%do %while(%length(&wQM));
		%let j=1;
		%let wKM= %scan(&wKModels, &j);	
		%do %while(%length(&wKM));
			%do t=1 %to &NTrait;
				%global QKFE_FS&wQM.&wKM._T&t._OutData QKFE_RE&wQM.&wKM._T&t._OutData QKFE_FE&wQM.&wKM._T&t._OutData 
				QKFE_CP&wQM.&wKM._T&t._OutData;
				%PrepOutDS(QKFE_FS&wQM.&wKM._T&t._OutData,QKFE_FS&wQM.&wKM._T&t._OutDataName,&OutName._FSQ&wQM.K&wKM._T&t);	
				%PrepOutDS(QKFE_RE&wQM.&wKM._T&t._OutData,QKFE_RE&wQM.&wKM._T&t._OutDataName,&OutName._REQ&wQM.K&wKM._T&t);
				%PrepOutDS(QKFE_FE&wQM.&wKM._T&t._OutData,QKFE_FE&wQM.&wKM._T&t._OutDataName,&OutName._FEQ&wQM.K&wKM._T&t);
				%PrepOutDS(QKFE_CP&wQM.&wKM._T&t._OutData,QKFE_CP&wQM.&wKM._T&t._OutDataName,&OutName._CPQ&wQM.K&wKM._T&t);
			%end;	
			%let j= %sysevalf(&j + 1);
			%let wKM= %scan(&wKModels, &j);
		%end;
		%let i= %sysevalf(&i + 1);
		%let wQM= %scan(&wQModels, &i);
	%end;
	
	%if %upcase(&WhichProc)= MIXED or %upcase(&WhichProc)= LOGISTIC %then %do;
		%global QKFE_FS_ALL_OutData;
		%PrepOutDS(QKFE_FS_ALL_OutData,QKFE_FS_ALL_OutDataName,&OutName._FS_ALL);
	%end;
	%if %upcase(&WhichProc) = GLIMMIX %then %do;
		%global QKFE_FS_ALLWRE_OutData QKFE_FS_ALLWORE_OutData;
		%PrepOutDS(QKFE_FS_ALLWRE_OutData,QKFE_FS_ALLWRE_OutDataName,&OutName._FS_WRE);
		
		%if %length(&RandEff)>0 or %length(&KVarsModel1)>0 or %length(&KVarsModel2)>0 %then %do;
			%PrepOutDS(QKFE_FS_ALLWORE_OutData,QKFE_FS_ALLWORE_OutDataName,&OutName._FS_WORE);
		%end;
	%end;
	
	%let QVarsModel0=;
	%let KVarsModel0=;
	%let PedIDKModel0=;
	*end: get models to be compared*;
	
	/*begin S1206799: Luciano added this check to verify that when class variables are specified, they are specified at least
	in one of the effects model: fixed, random, or interaction.*/
	%let ClassVarsTmp= &ClassVars;
	%if %symexist(TraitVar) %then %do;
		%if %length(&TraitVar) %then %do;
			%let ClassVarsTmp= %sysfunc(tranwrd(%upcase(&ClassVarsTmp), %upcase(&TraitVar), %str()));
		%end;
	%end;

	%if %symexist(PedID) %then %do;
		%if %length(&PedID) %then %do;
			%let ClassVarsTmp= %sysfunc(tranwrd(%upcase(&ClassVarsTmp), %upcase(&PedID), %str()));
		%end;
	%end;
	
	%put ClassVarsTmp= &ClassVarsTmp;
	
	%let NeedConstant=0;
	%if %length(&ClassVarsTmp) = 0 %then %do;
		/* Need to do this for Proc Logistic/Surveylogistic/PHREG so it will produce Type3 
		table for Trend test when no Class var is present*/
		%if (%index(&WhichProc,%str(LOGISTIC))>0 or %upcase(&WhichPROC)=%str(PHREG)) %then %do;
			%let NeedConstant=1;
		%end;
		%else 
			%let NeedConstant=0;
	%end;
	%else %if %length(&ClassVarsTmp) %then %do;
	
		%let FixedEffVars = ;
		%let i = 1;
		%do %while(%length(%scan(&FixedEff,&i,'*| ')));
			%let FixedEffVars = &FixedEffVars %scan(&FixedEff,&i,'*| ');
			%let i=%eval(&i+1);
		%end;
	
		%let RandEffVars = ;
		%let i = 1;
		%do %while(%length(%scan(&RandEff,&i,'*| ')));
			%let RandEffVars = &RandEffVars %scan(&RandEff,&i,'*| ');
			%let i=%eval(&i+1);
		%end;	
	
		%let IntEffVars= ;
	
		/*Checking if ClassVars have been included into any of the variables that goes into the model statement*/
		%let FixRandEffVars= &FixedEffVars &RandEffVars &IntEffVars;
		%put FixRandEffVars= &FixRandEffVars;		
		
		%let j=1;
		%do %while(%length(%scan(&ClassVarsTmp, &j, %str( ))));
			%let j= %sysevalf(&j + 1);
			%let currclass= %scan(&ClassVarsTmp, &j, %str( ));
		%end;
		%let ncin= %sysevalf(&j - 1);
		
		%macro CheckClassVars();
			%global ncout ClassVarsOut FoundAtLeastOne_flag;
			%let ncout= 0;
			%let ClassVarsOut= ;
			%let FoundAtLeastOne_flag= 0;
			
			%let FixRandEffVarsOrig= &FixRandEffVars;
			
			%if %length(&FixRandEffVars) = 0 %then %do;
				%let ClassVarsOut= &ClassVarsTmp;
				%let FoundAtLeastOne_flag= 0;
			%end;		
			%else %do;
				%let j=1;
				%let currclass= %scan(&ClassVarsTmp, &j, %str( ));
				%do %while(%length(&currclass));
					%put currclass= &currclass;
					%let k=1;
					%let currvar= %scan(&FixRandEffVars, &k, %str( ));
					%if %length(&currvar) %then %do;
						%let CandidateClassVarOut= ;
						%let found_flag= 0;
						%do %while(%length(&currvar));
							%put &currclass &currvar;
							%if %upcase(&currclass) = %upcase(&currvar) %then %do;
								%*put YES;
								%let found_flag= 1;
								%let FoundAtLeastOne_flag= 1;
								%let FixRandEffVarsTmp=;
								%let l=1;
								%let currvartmp= %scan(&FixRandEffVars, &l, %str( ));
								%do %while(%length(&currvartmp));
									%if &l ^= &k %then %do;
										%let FixRandEffVarsTmp= &FixRandEffVarsTmp &currvartmp;
									%end;
									%let l= %sysevalf(&l + 1);
									%let currvartmp= %scan(&FixRandEffVars, &l, %str( ));
								%end;
								%let FixRandEffVars= &FixRandEffVarsTmp;
								%goto nextclassvar;
							%end;
							%else %do;
								%*put NO;
								%let CandidateClassVarOut= &currclass;
							%end;

							%let k= %sysevalf(&k + 1);
							%let currvar= %scan(&FixRandEffVars, &k, %str( ));
						%end;

						%if &found_flag = 0 %then %do;
							%let ClassVarsOut= &ClassVarsOut &CandidateClassVarOut;
						%end;
					%end;
					%else %do;
						%let ClassVarsOut= &ClassVarsOut &currclass;
					%end;

					%nextclassvar:;
					%let j= %sysevalf(&j + 1);
					%let currclass= %scan(&ClassVarsTmp, &j, %str( ));
				%end;
				%let FixRandEffVars= &FixRandEffVarsOrig;
				%put ClassVarsOut= &ClassVarsOut;
				%put FixRandEffVars= &FixRandEffVars;
			%end;

			%if %length(&ClassVarsOut) > 0 %then %do;
				%let ncout=1;
				%do %while(%length(%scan(&ClassVarsOut, &ncout, %str( ))));
					%let ncout= %sysevalf(&ncout + 1);
				%end;
				%let ncout= %sysevalf(&ncout - 1);
			%end;
		%mend CheckClassVars;
		%CheckClassVars();

		%put ncout= &ncout;
		%if &ncout %then %do;
			%if &ncout = 1 %then %do;
				%let ErrString0= Class Variable &ClassVarsOut has neither been specified as fixed, random nor interaction effect. Please, review the Model Variables Tab.;
			%end;
			%else %if &ncout > 1 %then %do;
				%let ErrString0= Class Variables &ClassVarsOut have neither been specified as fixed, random nor interaction effects. Please, review the Model Variables Tab.;
			%end;
			
			%if %length(&ProcessName) > 22 %then %do;
				%let ProcessNameShort= %substr(&ProcessName, 1, 22);
			%end;
			%else %do;
				%let ProcessNameShort= &ProcessName;
			%end;

			%let ScErr0= &ProcessNameShort._ScErr;
			%*put ScriptErr= &ScErr0;
			%global &ScErr0;
			%PrepOutFile(&ScErr0,&ScErr0.Name,saserror.jsl);
			%*put ScriptErr= &&&ScErr0;
			%*put ScriptErrName= &&&ScErr0.Name;

			data _null_;
				file "&&&ScErr0" lrecl=32767 ;
				put '//!' ;
				put "Names Default to Here(1);" ;
				put "new Window(""ERROR running &FullAPName.""," ;
				put "vlb=VListBox(" ; 
				put "textBox(""The following SAS error ocurred in process &FullAPName:""),";
				put "errtxt = TextEditBox(""\!N""||""";
				put "&ErrString0";
				put """||""\!N"",WhiteBoxStyle(0)),";
				put "errtxt << Font Color(3)," ; 
				put "errtxt << Set Wrap(1000)," ;
				/*put "TextBox(""Please refer to the SAS log displayed in the JMP log window for further details.\!N\!N""),";*/
				put ")" ;
				put ");";
			run;
			%*PackageInsertFile(jsl code from &ProcessName, &OutPath, &&&ScErr0.Name, no, no, yes);

			%put ERROR: &ErrString0;
			%let exiterror=1;
			%goto exit;
		%end;
		
		%if (%index(&WhichProc,%str(LOGISTIC))>0 or %upcase(&WhichPROC)=%str(PHREG)) %then %do;
			/* Need to do this for Proc Logistic/Surveylogistic/PHREG so it will produce Type3 
			table for Trend test when no Class var is neither included as fixed nor random effect*/
			%let FixRandEffVars= &FixedEffVars &RandEffVars;
			%put FixRandEffVars0= &FixRandEffVars;
			%CheckClassVars();
			%put FoundAtLeastOne_flag= &FoundAtLeastOne_flag;
			%if &FoundAtLeastOne_flag = 0 %then %do;
				%let NeedConstant=1;
			%end;
			%else %do;
				%let NeedConstant=0;
			%end;
		%end;
	%end;
	%put NeedConstant= &NeedConstant;
	/*end S1206799: Luciano added this check to verify that when class variables are specified, they are specified at least
	in one of the effects model: fixed, random, or interaction.*/	
	
	
	%global Flag_NonConvergence;
	%macro QKEvalMacro(Trait=, PedID=, QVars=, KVars=);
		%let Flag_NonConvergence= 0;
	
		%let tmpClassVars= &ClassVars;
		%let tmpRandEff= &RandEff;
		%let tmpds= &ds;
	
		%if %length(&PedID) %then
			%let ClassVars= &ClassVars &PedID;
	
		%put RandEffIn= &RandEff;
		%let RandEffVars = ;
		%let ii = 1;
		%do %while(%length(%scan(&RandEff,&ii,'*| ')));
			%let RandEffVars = &RandEffVars %scan(&RandEff,&ii,'*| ');
			%let ii=%eval(&ii+1);
		%end;
		
		%*put ii=&ii;
		%if &ii>1 %then %do;
			/* If just one random effect variable, use alternative random statement */
			%if &ii=2 and %length(&RandOpts)=0 and %length(&ParmsStmt)=0 and %index(&RandEffVars,%str(-))=0 %then %do;

				proc sort data=&ds out=dssort;
					by &ByVars &RandEffVars;
				run;
				%if %index(%upcase(&ClassVars),%upcase(&RandEffVars))>0 %then %do;
					%let NumRandEff = %trim(%substr(&RandEffVars,1,28)_num);
					data numrandeff;
						set dssort;
						by &ByVars &RandEffVars;
						if first.&RandEffVars then &NumRandEff+1;
					run;

					%let ClassVars = %sysfunc(tranwrd(%upcase(&ClassVars),%upcase(&RandEffVars),%str()));
					%let RandEffVars = &NumRandEff;
					%let ds = numrandeff;
				%end;
				%else 
					%let ds=dssort;	
				/*
				%if &imputed=1 %then %do;
					* Need ds to be genotypes since not running %CreateStackedNumeric below *;
					proc datasets nolist;
						delete genotypes;
						change &ds=genotypes;
					quit;
					%let ds=genotypes;
				%end;
				*/
				%let RandEff = %str(intercept / subject=&RandEffVars);
			%end;
			%else %do;
				%let foundsub=0;
				%let RandEff = %str(&RandEff / &RandOpts);
				/* Need to see if there's a SUBJECT= variable that needs to be kept */
				%let subind=%index(&RandOpts,sub=)+4;
				%if &subind=4 %then %do;
					%let subind=%index(&RandOpts,subject=)+8;
					%if &subind>8 %then
						%let foundsub=1;
				%end;
				%else 
					%let foundsub=1;
				%if &foundsub=1 %then 
					%let RandEffVars = &RandEffVars %scan(%substr(&RandOpts,&subind),1);
			%end;
		%end;
		%put RandEffOut= &RandEff;	

		data fs;
			delete;
		run;

		ods output 
			FitStatistics = fs
			%if (%length(&RandEff)>0 or %length(&KVars)>0) %then %do;
				ConvergenceStatus=convstatus(where=(status=1))
				SolutionR= par	
				CovParms= cp
			%end;
			&ParmEst = pat
			%if &WhichProc ne SURVEYREG %then %do;
				nobs = tnobs %if &WhichProc ne PHREG %then (where=(Label ? "Observations Used"));
			%end;
			%if %upcase(&TraitType)=BINARY %then responseprofile=rp; 
		;
									
		proc &WhichProc data=&ds &ProcOption;
			class &ClassVars &ClassOpt;
			model &Trait &TraitOpt = &FixedEff 
						    %if %length(&QVars)>0 %then &QVars; 
						    &ModOption;

			%if %length(&KVars)>0 %then %do; 
				random &KVars / 
					type=toep(1) 
					%if %length(&PedID) %then subject=&PedID;
					 s 
				;
			%end;

			%if %length(&RandEff)>0 %then %do; 
				random &RandEff 
					%if %upcase(&TraitType)=NOMINAL %then group=&Trait; 
					%if &ii= 1 %then / s;
					%else s;
				;
			%end;

			%if %length(&ParmsStmt) %then %do;
				parms &ParmsStmt ;
			%end;

			%if %length(&StrataVars)>0 %then %do;
				strata &StrataVars;
			%end;
			
			/*
			%if %upcase(&OutputResid)=YES and %length(&RandEff) %then %do;
				id _IndivID_ ;
			%end;
			*/
			%if %length(&ByVars)>0 %then %do;
				by &ByVars;
			%end;
		
		quit;
		
		%if &syserr>4 %then %do;
			%let exiterror=1;
			%goto exitEV;
		%end;
		
		%if (%length(&RandEff)>0 or %length(&KVars)>0) %then %do;
		
			%NObsVars(ConvStatus);
			%if &NObs %then %do;												
				%let Flag_NonConvergence = 1;					

				data fs;
					length Descr $50 Value 8;
					Descr= '';
					Value= .;
				run;

				options label;
				data fs;
					merge fs ConvStatus(keep=Reason);	
					label Reason= "Convergence Status";					
				run;
				*options nolabel;
			%end;
		%end;
	
		%let ClassVars= &tmpClassVars;
		%let RandEff= &tmpRandEff;
		%let ds= &tmpds;
		
		%exitEV:;
		
	%mend QKEvalMacro;
		
	%macro search1D(Trait=, KorQVars=, searchVars=, varsInModel=, PedID=, nvarstosearch=);
		/*nvarstosearch is used just for testing purpose*/
			
		%if %length(&nvarstosearch) %then %do;
			%let nSVars= %length(&searchVars);
			%put nSVars= &nSVars;
			%let subSearchVars= ;
			%do i=1 %to %sysfunc(min(&nvarstosearch, &nSVars));
				%let subSearchVars= &subSearchVars %scan(&searchVars, &i, ' ');
			%end;
		%end;
		%else %do;
			%let subSearchVars= &searchVars;
		%end;	
		%let SearchVars= &subsearchVars;
		%put searchVarsS1D= &searchVars;
		
		%if &KorQVars= KV %then %do;
			
			data fs_s1d;
				length _Trait_ _KVar_ Descr $50 Value 8;
				delete;
			run;
					
			%let kv= 1; 
			%let KVars_test= %scan(&searchVars, &kv, ' ');
			%do %while(%length(&KVars_test));
				%let KVars= &VarsInModel &KVars_test;

				%QKEvalMacro(Trait=&Trait, PedID=&PedID, QVars=, KVars=&KVars);
				%if &exiterror=1 %then %goto exitS1D;

				data fs_s1d;
					set fs_s1d(in=orig) fs;
					if orig=0 then do;
						_Trait_= "&Trait";
						_KVar_ = "&KVars_test";
					end;
				run;

				%let kv= %sysevalf(&kv + 1);
				%let KVars_test= %scan(&searchVars, &kv, ' ');
			%end;
			
			proc sort data=fs_s1d out=fs_s1d;
				by _Trait_ _KVar_;
			run;
			
			proc transpose data=fs_s1d out=fs_s1d(drop=_name_);
				by _Trait_ _KVar_;
				id Descr;
			run;
			
		%end;
		%else %if &KorQVars= QV %then %do;
			%*let KVars = ; *Needed to be set empty to run a model without KVars*;
			
			data fs_s1d;
				length _Trait_ _QVar_ Descr $50 Value 8;
				delete;
			run;
					
			%let qv= 1; 
			%let QVars_test= %scan(&searchVars, &qv, ' ');
			%do %while(%length(&QVars_test));				
				%let QVars= &VarsInModel &QVars_test;

				%QKEvalMacro(Trait=&Trait, PedID=, QVars=&QVars, KVars=);
				%if &exiterror=1 %then %goto exitS1D;

				data fs_s1d;
					set fs_s1d(in=orig) fs;
					if orig=0 then do;
						_Trait_= "&Trait";
						_QVar_ = "&QVars_test";
					end;
				run;

				%let qv= %sysevalf(&qv + 1);
				%let QVars_test= %scan(&searchVars, &qv, ' ');
			%end;
			
			proc sort data=fs_s1d out=fs_s1d;
				by _Trait_ _QVar_;
			run;
			
			proc transpose data=fs_s1d out=fs_s1d(drop=_name_);
				by _Trait_ _QVar_;
				id Descr;
			run;
			
		%end;		
		%exitS1D:;		
	%mend search1D;
	
	/*
	%search1D(Trait=Qtrt1, KorQVars=KV, searchVars=&origKVars, varsInModel=, PedID=, nvarstosearch=3);
	%if &exiterror=1 %then %goto exit;
	%goto exit;
	*/
	
	%macro forward1D(Trait=, Criterion=, KorQVars=, searchVars=, VarsInModel=, PedID=, TrackAICC=NO, TrackBIC=NO, nvarstosearch=);
		/*nvarstosearch is used just for testing purpose*/
	
		%if %upcase(&criterion) = AIC %then
			%let criterionvar= AIC__Smaller_is_Better_;
		%else %if %upcase(&criterion) = AICC %then	
			%let criterionvar= AICC__Smaller_is_Better_;
		%else %if %upcase(&criterion) = BIC %then	
			%let criterionvar= BIC__Smaller_is_Better_;
		
		%put criterion= &criterion;
		%put criterionvar= &criterionvar;
	
		%if &KorQVars= KV %then %do;
			%let varname= _KVar_;
		%end;
		%else %if &KorQVars= QV %then %do;
			%let varname= _QVar_;
		%end;
					
		%if %length(&nvarstosearch) %then %do;
			%let nSVars= %length(&searchVars);
			%put nSVars= &nSVars;
			%let newSearchVars= ;
			%do i=1 %to %sysfunc(min(&nvarstosearch, &nSVars));
				%let newSearchVars= &newSearchVars %scan(&searchVars, &i, ' ');
			%end;
		%end;
		%else %do;
			%let newSearchVars= &searchVars;
		%end;
		%put newSearchVars= &newSearchVars;
		
		data fs_f1d;
			length _Trait_ &varname $50 
				_order_
				AIC__Smaller_is_Better_ 
				AICC__Smaller_is_Better_ 
				BIC__Smaller_is_Better_ 8;
			delete;
		run;
		
		%if %upcase(&TrackAICC)= YES %then %do;
			data fs_f1d_trackaicc;
				length _Trait_ &varname $50 
					_order_
					AIC__Smaller_is_Better_ 
					AICC__Smaller_is_Better_ 
					BIC__Smaller_is_Better_ 8;
				delete;
			run;	
		%end;
		
		%if %upcase(&TrackBIC)= YES %then %do;
			data fs_f1d_trackbic;
				length _Trait_ &varname $50 
					_order_
					AIC__Smaller_is_Better_ 
					AICC__Smaller_is_Better_ 
					BIC__Smaller_is_Better_ 8;
				delete;
			run;	
		%end;
	
		%let newVarsInModel= &VarsInModel;
		%let fs_allmiss= 0;
		%let bestValIn= 1000000;
		%let bestValOut= 0;
		%let diffIn= %sysfunc(abs(%sysevalf(&bestValIn - &bestValOut)));
		%let diffOut= 0;
		%let kk=1;
		%do %while(%length(&newSearchVars) and &kk <= 100000 and &fs_allmiss= 0 
			and %sysevalf(%sysfunc(abs(&diffIn - &diffOut)) > 0.01 * &diffIn));
		
			%search1D(Trait=&Trait, KorQVars= &KorQVars, searchVars= &newSearchVars, varsInModel= &newVarsInModel, PedID= &PedID, nvarstosearch=);
			%if &exiterror=1 %then %goto exitF1D;
		
			proc sort data=fs_s1d out=fs_s1ds;
				by _Trait_ &criterionvar;
			run;
			
			data fs_s1ds;
				set fs_s1ds;
				where not missing(&criterionvar);
			run;
			
			%NobsVars(fs_s1ds);
			%if &nobs > 0 %then %do;
				
				%let diffIn= %sysfunc(abs(%sysevalf(&bestValIn - &bestValOut)));
				%let bestValIn = &bestValOut;
				data _null_;
					set fs_s1ds(obs=1);
					call symputx("bestVar", trim(left(&varname)));
					call symputx("bestValOut", trim(left(&criterionvar)));
				run;
				%let diffOut= %sysfunc(abs(%sysevalf(&bestValIn - &bestValOut)));
				
				%put diffIn= &diffIn;
				%put diffOut= &diffOut;
				
				%put tolLuciano1= %sysevalf(%sysfunc(abs(&diffIn - &diffOut)));
				%put tolLuciano2= %sysevalf(0.01 * &diffIn);
								
				data fs_f1d;
					set fs_f1d(in=orig) fs_s1ds(obs=1);
					if orig=0 then _order_= _n_;
				run;

				%put bestVar=&bestVar;
				%put newVarsInModel= &newVarsInModel;
				%put newSearchVars= &newSearchVars;

				%let newVarsInModel= &newVarsInModel &bestVar;
				%let newSearchVars= %sysfunc(translate(&newSearchVars $, ~ , %str( )));
				%put newSearchVars= &newSearchVars;
				%let newSearchVars= %sysfunc(transtrn(&newSearchVars, &bestVar.~, ));
				%put newSearchVars= &newSearchVars;
				%let newSearchVars= %sysfunc(translate(&newSearchVars, %str( ), ~ ));
				%put newSearchVars= &newSearchVars;
				%let newSearchVars= %sysfunc(translate(&newSearchVars, %str( ), $));
				%put newSearchVars= &newSearchVars;
				%put newVarsInModel= &newVarsInModel;
				%let kk=%sysevalf(&kk + 1);
			%end;
			%else %do;
				%let fs_allmiss= 1;
			%end;
			
			%put whileLuciano;
			%put &newSearchVars; 
			%put &kk;
			%put &fs_allmiss;
			%put &bestValIn;
			%put &bestValOut;
			
			%if %upcase(&TrackAICC)= YES %then %do;
			
				proc sort data=fs_s1d out=fs_s1d_trackaiccs;
					by _Trait_ AICC__Smaller_is_Better_;
				run;

				data fs_s1d_trackaiccs;
					set fs_s1d_trackaiccs;
					where not missing(AICC__Smaller_is_Better_);
				run;

				%NobsVars(fs_s1d_trackaiccs);
				%if &nobs > 0 %then %do;
					data fs_f1d_trackaicc;
						set fs_f1d_trackaicc(in=orig) fs_s1d_trackaiccs(obs=1);
						if orig=0 then _order_= _n_;
					run;
				%end;
			%end;
			%if %upcase(&TrackBIC)= YES %then %do;
			
				proc sort data=fs_s1d out=fs_s1d_trackbics;
					by _Trait_ BIC__Smaller_is_Better_;
				run;

				data fs_s1d_trackbics;
					set fs_s1d_trackbics;
					where not missing(BIC__Smaller_is_Better_);
				run;

				%NobsVars(fs_s1d_trackbics);
				%if &nobs > 0 %then %do;
					data fs_f1d_trackbic;
						set fs_f1d_trackbic(in=orig) fs_s1d_trackbics(obs=1);
						if orig=0 then _order_= _n_;
					run;
				%end;
			%end;
		%end;
		%exitF1D:;
	%mend forward1D;
	
	%macro TraitAndCriteriaLoop(KorQVars=, searchVars=, PedID=);
		
		%if &KorQVars= KV %then %do;
			%let varname= _KVar_;
		%end;
		%else %if &KorQVars= QV %then %do;
			%let varname= _QVar_;
		%end;
	
		data fs_f1d_aic_&varname;
			length _Trait_ $50
				_Order_Of_&varname.By_AIC 8
				&varname.Ordered_By_AIC $50 
				AIC_&varname.Ordered_By_AIC 
				AICC_&varname.Ordered_By_AIC
				BIC_&varname.Ordered_By_AIC
				_2_ResLogL_&varname.Ordered_By_AIC
				8;
			delete;
		run;
		
		data fs_f1d_aicc_&varname;
			length _Trait_ $50
				_Order_Of_&varname.By_AICc 8
				&varname.Ordered_By_AICc $50 
				AIC_&varname.Ordered_By_AICc
				AICC_&varname.Ordered_By_AICc
				BIC_&varname.Ordered_By_AICc
				_2_ResLogL_&varname.Ordered_By_AICc
				8;
			delete;
		run;
		
		data fs_f1d_bic_&varname;
			length _Trait_ $50
				_Order_Of_&varname.By_BIC 8
				&varname.Ordered_By_BIC $50 
				AIC_&varname.Ordered_By_BIC
				AICC_&varname.Ordered_By_BIC
				BIC_&varname.Ordered_By_BIC
				_2_ResLogL_&varname.Ordered_By_BIC
				8;
			delete;
		run;
	
		%do t=1 %to &NTrait;
		
			%forward1D(Trait=&&_tv&t, Criterion=AIC, KorQVars=&KorQVars, searchVars=&searchVars, VarsInModel=, PedID= &PedID, TrackAICC=YES, TrackBIC=YES, nvarstosearch=);
			%if &exiterror=1 %then %goto exitTC;
			
			data fs_f1d_aic_&varname;
				set fs_f1d_aic_&varname fs_f1d(rename=(
					_order_=_Order_Of_&varname.By_AIC
					&varname=&varname.Ordered_By_AIC
					AIC__Smaller_is_Better_= AIC_&varname.Ordered_By_AIC
					AICC__Smaller_is_Better_= AICC_&varname.Ordered_By_AIC
					BIC__Smaller_is_Better_= BIC_&varname.Ordered_By_AIC
					_2_Res_Log_Likelihood= _2_ResLogL_&varname.Ordered_By_AIC
				));
			run;
			
			*begin: using the tracking information of AICC that was stored when doing the forward search by AIC*;
			%let SubSearchVarsAICC=;
			%if %sysfunc(exist(fs_f1d_trackaicc)) %then %do;
				proc transpose data=fs_f1d_trackaicc out=fs_f1d_trackedaiccvars;
					id &varname;
				run;

				data trackedaiccvarnames;
					set fs_f1d_trackedaiccvars(drop=_name_);
					where 0;
				run;

				%let TrackedVarsAICC=;
				%let dsid=%sysfunc(open(trackedaiccvarnames));
				%let nvarsaicc= %sysfunc(attrn(&dsid,nvars));
				%do i=1 %to &nvarsaicc;
					%let TrackedVarsAICC=&TrackedVarsAICC %sysfunc(varname(&dsid,&i));
				%end;
				%let rc = %sysfunc(close(&dsid));
				%put TrackedVarsAICC= &TrackedVarsAICC;

				%let SubSearchVarsAICC= %sysfunc(translate(&SearchVars $, ~ , %str( )));
				%put SubSearchVarsAICC= &SubSearchVarsAICC;
				%do i=1 %to &nvarsaicc;
					%let currvar= %scan(&TrackedVarsAICC, &i);
					%let SubSearchVarsAICC= %sysfunc(transtrn(&SubSearchVarsAICC, &currvar.~, ));
				%end;
				%put SubSearchVarsAICC= &SubSearchVarsAICC;
				%let SubSearchVarsAICC= %sysfunc(translate(&SubSearchVarsAICC, %str( ), ~ ));
				%put SubSearchVarsAICC= &SubSearchVarsAICC;
				%let SubSearchVarsAICC= %sysfunc(translate(&SubSearchVarsAICC, %str( ), $));
				%put SubSearchVarsAICC= &SubSearchVarsAICC;						
			%end;
			
			%if %length(&SubSearchVarsAICC) %then %do;
				%forward1D(Trait=&&_tv&t, Criterion=AICC, KorQVars=&KorQVars, searchVars=&SubSearchVarsAICC, VarsInModel= &TrackedVarsAICC, PedID= &PedID, TrackAICC=NO, TrackBIC=NO, nvarstosearch=);
				%if &exiterror=1 %then %goto exitTC;
				
				data fs_f1d_aicc_&varname;
					set fs_f1d_aicc_&varname 
						fs_f1d_trackaicc(rename=(
							_order_=_Order_Of_&varname.By_AICc
							&varname=&varname.Ordered_By_AICc
							AIC__Smaller_is_Better_= AIC_&varname.Ordered_By_AICc
							AICC__Smaller_is_Better_= AICC_&varname.Ordered_By_AICc
							BIC__Smaller_is_Better_= BIC_&varname.Ordered_By_AICc
							_2_Res_Log_Likelihood= _2_ResLogL_&varname.Ordered_By_AICc
						));
				run;

				data fs_f1d_aicc_&varname;
					set fs_f1d_aicc_&varname fs_f1d(rename=(
						_order_=_Order_Of_&varname.By_AICc
						&varname=&varname.Ordered_By_AICc
						AIC__Smaller_is_Better_= AIC_&varname.Ordered_By_AICc
						AICC__Smaller_is_Better_= AICC_&varname.Ordered_By_AICc
						BIC__Smaller_is_Better_= BIC_&varname.Ordered_By_AICc
						_2_Res_Log_Likelihood= _2_ResLogL_&varname.Ordered_By_AICc
						) in=orig);
					if orig = 1 then _Order_Of_&varname.By_AICc= _n_;
				run;
				
			*end: using the tracking information of AICC that was stored when doing the forward search by AIC*;
			%end; 	
			%else %do;
				%forward1D(Trait=&&_tv&t, Criterion=AICC, KorQVars=&KorQVars, searchVars=&SearchVars, PedID= &PedID, VarsInModel=, TrackAICC=NO, TrackBIC=NO, nvarstosearch=);
				%if &exiterror=1 %then %goto exitTC;
				data fs_f1d_aicc_&varname;
					set fs_f1d_aicc_&varname 
						fs_f1d(rename=(
							_order_=_Order_Of_&varname.By_AICc
							&varname=&varname.Ordered_By_AICc
							AIC__Smaller_is_Better_= AIC_&varname.Ordered_By_AICc
							AICC__Smaller_is_Better_= AICC_&varname.Ordered_By_AICc
							BIC__Smaller_is_Better_= BIC_&varname.Ordered_By_AICc
							_2_Res_Log_Likelihood= _2_ResLogL_&varname.Ordered_By_AICc
						));
				run;
			%end;

			*begin: using the tracking information of BIC that was stored when doing the forward search by AIC*;
			%let SubSearchVarsBIC=;
			%if %sysfunc(exist(fs_f1d_trackbic)) %then %do;
				proc transpose data=fs_f1d_trackbic out=fs_f1d_trackedbicvars;
					id &varname;
				run;

				data trackedbicvarnames;
					set fs_f1d_trackedbicvars(drop=_name_);
					where 0;
				run;

				%let TrackedVarsbic=;
				%let dsid=%sysfunc(open(trackedbicvarnames));
				%let nvarsbic= %sysfunc(attrn(&dsid,nvars));
				%do i=1 %to &nvarsbic;
					%let TrackedVarsBIC=&TrackedVarsBIC %sysfunc(varname(&dsid,&i));
				%end;
				%let rc = %sysfunc(close(&dsid));
				%put TrackedVarsBIC= &TrackedVarsBIC;

				%let SubSearchVarsBIC= %sysfunc(translate(&SearchVars $, ~ , %str( )));
				%put SubSearchVarsBIC= &SubSearchVarsBIC;
				%do i=1 %to &nvarsbic;
					%let currvar= %scan(&TrackedVarsBIC, &i);
					%let SubSearchVarsBIC= %sysfunc(transtrn(&SubSearchVarsBIC, &currvar.~, ));
				%end;
				%put SubSearchVarsBIC= &SubSearchVarsBIC;
				%let SubSearchVarsBIC= %sysfunc(translate(&SubSearchVarsBIC, %str( ), ~ ));
				%put SubSearchVarsBIC= &SubSearchVarsBIC;
				%let SubSearchVarsBIC= %sysfunc(translate(&SubSearchVarsBIC, %str( ), $));
				%put SubSearchVarsBIC= &SubSearchVarsBIC;						
			%end;
			
			%if %length(&SubSearchVarsBIC) %then %do;
				%forward1D(Trait=&&tv_&t, Criterion=BIC, KorQVars=&KorQVars, searchVars=&SubSearchVarsBIC, PedID= &PedID, VarsInModel= &TrackedVarsBIC, TrackAICC=NO, TrackBIC=NO, nvarstosearch=);
				%if &exiterror=1 %then %goto exitTC;
				
				data fs_f1d_bic_&varname;
					set fs_f1d_bic_&varname 
						fs_f1d_trackbic(rename=(
							_order_=_Order_Of_&varname.By_BIC
							&varname=&varname.Ordered_By_BIC
							AIC__Smaller_is_Better_= AIC_&varname.Ordered_By_BIC
							AICC__Smaller_is_Better_= AICC_&varname.Ordered_By_BIC
							BIC__Smaller_is_Better_= BIC_&varname.Ordered_By_BIC
							_2_Res_Log_Likelihood= _2_ResLogL_&varname.Ordered_By_BIC
						));
				run;

				data fs_f1d_bic_&varname;
					set fs_f1d_bic_&varname fs_f1d(rename=(
						_order_=_Order_Of_&varname.By_BIC
						&varname=&varname.Ordered_By_BIC
						AIC__Smaller_is_Better_= AIC_&varname.Ordered_By_BIC
						AICC__Smaller_is_Better_= AICC_&varname.Ordered_By_BIC
						BIC__Smaller_is_Better_= BIC_&varname.Ordered_By_BIC
						_2_Res_Log_Likelihood= _2_ResLogL_&varname.Ordered_By_BIC
						) in= orig);
					if orig=1 then _Order_Of_&varname.By_BIC = _n_;	
				run;
			*end: using the tracking information of BIC that was stored when doing the forward search by AIC*;
			%end;
			%else %do;
				%forward1D(Trait=&&tv_&t, Criterion=BIC, KorQVars=&KorQVars, searchVars=&SearchVars, PedID= &PedID, VarsInModel=, TrackAICC=NO, TrackBIC=NO, nvarstosearch=);
				%if &exiterror=1 %then %goto exitTC;
				
				data fs_f1d_bic_&varname;
					set fs_f1d_bic_&varname 
						fs_f1d(rename=(
							_order_=_Order_Of_&varname.By_BIC
							&varname=&varname.Ordered_By_BIC
							AIC__Smaller_is_Better_= AIC_&varname.Ordered_By_BIC
							AICC__Smaller_is_Better_= AICC_&varname.Ordered_By_BIC
							BIC__Smaller_is_Better_= BIC_&varname.Ordered_By_BIC
							_2_Res_Log_Likelihood= _2_ResLogL_&varname.Ordered_By_BIC
						));
				run;
			%end;
		%end;
		
		data fs_f1d_&varname;
			merge fs_f1d_aic_&varname fs_f1d_aicc_&varname fs_f1d_bic_&varname ;
			by _Trait_;
			if first._Trait_ then _numb&varname=1;
			else _numb&varname + 1;
		run;

		data fs_f1d_&varname;
			set fs_f1d_&varname;
			by _Trait_ _Order_Of_&varname.By_AIC;
			if ^first._Order_Of_&varname.By_AIC then do;
				_Order_Of_&varname.By_AIC=.;
				&varname.Ordered_By_AIC="";
				AIC_&varname.Ordered_By_AIC=.;
			end;
		run;

		data fs_f1d_&varname;
			set fs_f1d_&varname;
			by _Trait_ _Order_Of_&varname.By_AICC;
			if ^first._Order_Of_&varname.By_AICC then do;
				_Order_Of_&varname.By_AICc=.;
				&varname.Ordered_By_AICc="";
				AICC_&varname.Ordered_By_AICc=.;
			end;
		run;

		data fs_f1d_&varname;
			set fs_f1d_&varname;
			by _Trait_ _Order_Of_&varname.By_BIC;
			if ^first._Order_Of_&varname.By_BIC then do;
				_Order_Of_&varname.By_BIC=.;
				&varname.Ordered_By_BIC="";
				BIC_&varname.Ordered_By_BIC=.;
			end;
		run;
		
		%exitTC:;
	%mend TraitAndCriteriaLoop;
	
	/*
	%put QVarsModel3= &QVarsModel3;
	%forward1D(Trait=Qtrt1, Criterion=AIC, KorQVars=QV, searchVars=&QVarsModel3, VarsInModel=, PedID=, TrackAICC=NO, TrackBIC=NO, nvarstosearch=);
	%goto exitcheck;
	*/
			
	*begin: running forward search for the Q model*;
	%let SelectQVars= No;
	%let SelectQVarsCriterion= ;
	%put SelectQVars= &SelectQVars;
	%if %upcase(&SelectQVars) = YES %then %do;
	
		%let i=1;
		%let wQM= %scan(&wQModels, &i);
		%do %while(%length(&wQM));	
			%let QVars= &&QVarsModel&wQM;
		
			%do t=1 %to &NTrait;
		
				%forward1D(Trait=&&_tv&t, Criterion=&SelectQVarsCriterion, KorQVars=QV, searchVars=&QVars, VarsInModel=, PedID=, TrackAICC=NO, TrackBIC=NO, nvarstosearch=);
				%if &exiterror=1 %then %goto exit;
			
				proc transpose data=fs_f1d out=fs_f1d_trans(drop=_name_);
					id _QVar_;	
				run;
				
				data selectedQVars&wQM._T&t;
					set fs_f1d_trans;
					where 0;
				run;
				
				%let did= %sysfunc(open(selectedQVars&wQM._T&t));
				%let nselQVar= %sysfunc(attrn(&did, NVARS));
				%let selQVar&wQM._T&t=;
				%do k=1 %to &nselQVar;
					%let selQVar&wQM._T&t= &&selQVar&wQM._T&t %sysfunc(varname(&did, &k));	
				%end;
				%let rc= %sysfunc(close(&did));
			
			%end;
	
			%let i= %sysevalf(&i + 1);
			%let wQM= %scan(&wQModels, &i);
		%end;				
	%end;
	*end: running forward search for the Q model*;
	
	*begin: running forward search for the K model*;
	%let SelectKVars= No;
	%let SelectKVarsCriterion= &SelectQVarsCriterion;
	%if %upcase(&SelectKVars) = YES %then %do;
	
		%let j=1;
		%let wKM= %scan(&wKModels, &j);
		%do %while(%length(&wKM));
			%let KVars= &&KVarsModel&wKM;
			%let PedID= &&PedIDKModel&wKM;
			
				%do t=1 %to &NTrait;
		
					%forward1D(Trait=&&_tv&t, Criterion=&SelectQVarsCriterion, KorQVars=KV, searchVars=&KVars, VarsInModel=, PedID= &PedID, TrackAICC=NO, TrackBIC=NO, nvarstosearch=);
					%if &exiterror=1 %then %goto exit;
					
					proc transpose data=fs_f1d out=fs_f1d_trans(drop=_name_);
						id _KVar_;	
					run;

					data selectedKVars&wKM._T&t;
						set fs_f1d_trans;
						where 0;
					run;

					%let did= %sysfunc(open(selectedKVars&wKM._T&t));
					%let nselKVar= %sysfunc(attrn(&did, NVARS));
					%let selKVar&wKM._T&t=;
					%do k=1 %to &nselKVar;
						%let selKVar&wKM._T&t= &&selKVar&wKM._T&t %sysfunc(varname(&did, &k));	
					%end;
					%let rc= %sysfunc(close(&did));
					
				%end;
		
			%let j= %sysevalf(&j + 1);
			%let wKM= %scan(&wKModels, &j);
		%end;			
	%end;
	*end: running forward search for the K model*;
		
	*begin: running specific Q-K models*;
	options label;
	%if %upcase(&WhichProc)= MIXED %then %do;
		data OutLib.&QKFE_FS_ALL_OutDataName;
			length _Model_ _Trait_ Descr $50 Value 8;
			label Descr= "Description";
			delete;
		run;
	%end;
	%else %if %upcase(&WhichProc)= GLIMMIX %then %do;
	
		%if %length(&RandEff) = 0 %then %do;
			data OutLib.&QKFE_FS_ALLWORE_OutDataName;
				length _Model_ _Trait_ Descr $50 Value 8;
				delete;
				label Descr= "Description";
			run;
		%end;
			
		%if (%length(&RandEff)>0 or %length(&KVarsModel1)>0 or %length(&KVarsModel2)>0) %then %do;
			data OutLib.&QKFE_FS_ALLWRE_OutDataName;
				length _Model_ _Trait_ Descr $50 Value 8;
				delete;
				label Descr= "Description";				
			run;		
		%end;
	%end;
	%else %if %upcase(&WhichProc)= LOGISTIC %then %do;
		data OutLib.&QKFE_FS_ALL_OutDataName;
			length _Model_ _Trait_ Criterion $50 InterceptOnly InterceptAndCovariates 8;
			label Criterion= "Criterion";
			delete;
		run;
	%end;
					
	%let i=1;
	%let wQM= %scan(&wQModels, &i);
	%do %while(%length(&wQM));	
		%let QVars= &&QVarsModel&wQM;
		
		proc datasets nolist;
			delete pat par;
		quit;
		
		%let j=1;
		%let wKM= %scan(&wKModels, &j);
		%do %while(%length(&wKM));
			%let KVars= &&KVarsModel&wKM;
			%let PedID= &&PedIDKModel&wKM;

			%do t=1 %to &NTrait;
			
				%let exist_FE&wQM.&wKM._T&t= 0;
				%let exist_RE&wQM.&wKM._T&t= 0;
				%let exist_CP&wQM.&wKM._T&t= 0;
				
				%QKEvalMacro(Trait=&&_tv&t, PedID=&PedID, QVars=&QVars, KVars=&KVars);
				%if &exiterror=1 %then %goto exit;
	
				data OutLib.&&QKFE_FS&wQM.&wKM._T&t._OutDataName;
					set fs;
				run;
				
				%PackageInsertDataset(Fitness Information from &ProcessName,outlib,&&QKFE_FS&wQM.&wKM._T&t._OutDataName,No, No, No);

				%if %length(&ParmEst)>0 and %sysfunc(exist(pat)) %then %do;
					data OutLib.&&QKFE_FE&wQM.&wKM._T&t._OutDataName;
						set pat;
					run;
					%PackageInsertDataset(Fixed Effects from &ProcessName,outlib,&&QKFE_FE&wQM.&wKM._T&t._OutDataName,No, No, No);
					%let exist_FE&wQM.&wKM._T&t= 1;
				%end;
	
				%if (%length(&RandEff)>0 or %length(&KVars)>0) and %sysfunc(exist(par)) %then %do;					
					data OutLib.&&QKFE_RE&wQM.&wKM._T&t._OutDataName;
						set par;
					run;
					%PackageInsertDataset(Random Effects from &ProcessName,outlib,&&QKFE_RE&wQM.&wKM._T&t._OutDataName,No, No, No);
					%let exist_RE&wQM.&wKM._T&t= 1;
				%end;
				
				%if (%length(&RandEff)>0 or %length(&KVars)>0) and %sysfunc(exist(cp)) %then %do;					
					data OutLib.&&QKFE_CP&wQM.&wKM._T&t._OutDataName;
						set cp;
					run;
					%PackageInsertDataset(Covariance Parameters from &ProcessName,outlib,&&QKFE_CP&wQM.&wKM._T&t._OutDataName,No, No, No);
					%let exist_CP&wQM.&wKM._T&t= 1;
				%end;
				
				%if %upcase(&WhichProc)= MIXED %then %do;
					data OutLib.&QKFE_FS_ALL_OutDataName;
						set OutLib.&QKFE_FS_ALL_OutDataName fs(in=orig);
						if orig = 1 then do;
							_Model_= "Q&wQM & K&wKM Model";
							_Trait_= "&&_tv&t";
						end;
					run;
				%end;
				%else %if %upcase(&WhichProc)= GLIMMIX %then %do;						
					%if (%length(&RandEff)>0 or %length(&KVars)>0) %then %do;
						data OutLib.&QKFE_FS_ALLWRE_OutDataName;
							set OutLib.&QKFE_FS_ALLWRE_OutDataName fs(in=orig);
							if orig = 1 then do;
								_Model_= "Q&wQM & K&wKM Model";
								_Trait_= "&&_tv&t";
							end;
						run;
					%end;	
					%else %do;
						data OutLib.&QKFE_FS_ALLWORE_OutDataName;
							set OutLib.&QKFE_FS_ALLWORE_OutDataName fs(in=orig);
							if orig = 1 then do;
								_Model_= "Q&wQM & K&wKM Model";
								_Trait_= "&&_tv&t";
							end;
						run;					
					%end;
				%end;
				%else %if %upcase(&WhichProc)= LOGISTIC %then %do;
					data OutLib.&QKFE_FS_ALL_OutDataName;
						set OutLib.&QKFE_FS_ALL_OutDataName fs(in=orig);
						if orig = 1 then do;
							_Model_= "Q&wQM & K&wKM Model";
							_Trait_= "&&_tv&t";
						end;
					run;					
				%end;
			%end;
			%let j= %sysevalf(&j + 1);
			%let wKM= %scan(&wKModels, &j);
		%end;
		%let i= %sysevalf(&i + 1);
		%let wQM= %scan(&wQModels, &i);
	%end;
	
	%if %upcase(&WhichProc)= MIXED or %upcase(&WhichProc)= LOGISTIC %then %do;
		%let tabdata= &QKFE_FS_ALL_OutDataName..sas7bdat;
		data fsout;
			set OutLib.&QKFE_FS_ALL_OutDataName;
			%if %upcase(&WhichProc)= LOGISTIC %then 
				rename InterceptAndCovariates= Value;
				;
		run;
	%end;
	%else %if %upcase(&WhichProc)= GLIMMIX %then %do;
		data fsout;
			delete;
		run;
	
		%let tabdata= ;
		%if %length(&RandEff) = 0 %then %do;
			%let tabdata= &QKFE_FS_ALLWORE_OutDataName..sas7bdat;
			data fsout;
				set OutLib.&QKFE_FS_ALLWORE_OutDataName;
			run;
		%end;
		
		%if (%length(&RandEff)>0 or %length(&KVarsModel1)>0 or %length(&KVarsModel2)>0) %then %do; 
			%let tabdata= &tabdata, &QKFE_FS_ALLWRE_OutDataName..sas7bdat;
		
			data fsout;
				set fsout OutLib.&QKFE_FS_ALLWRE_OutDataName;
			run;
		
		%end;
	%end;
	/*
	proc sort data=fsout out=fs_all_sorted;
		by _Trait_;
	run;

	proc means data=fs_all_sorted min max noprint;
		var value;
		by _Trait_;
		output out=minmaxall min= minvalue max= maxvalue;
	run;
	
	%do t=1 %to &NTrait;
		data _null_;
			set minmaxall;
			if trim(left(_Trait_)) = trim(left("&&_tv&t")) then do;
				call symputx("minT_&t", trim(left(minvalue)));
				call symputx("maxT_&t", trim(left(maxvalue)));
			end;
		run;
		%put minT_&t= &&minT_&t;
		%put maxT_&t= &&maxT_&t;
	%end;
	*/
	*end: running specific Q-K models*;
	
	%put ClientOutPath= &ClientOutPath;
	%let wrapvars= 110;
			
	%global _mainjslfile_;
	%let _mainjslfile_ = %quote(&QKFE_Script);

	data _null_;
		file "&QKFE_Script" lrecl=32767;
		put '//!';
		put 'wait(0.5);';
		put "Names Default to Here(1);";
		put "if(host is(Windows),";
		put "	include( ConvertFilePath(""$LS_HOME/Generator/DialogHelperFunctions.jsl"", absolute, windows) ),";
		put "	include( ConvertFilePath(""$LS_HOME/Generator/DialogHelperFunctions.jsl"", absolute, POSIX) ));";
		put "	";
		put "Here:DataList = {};";
		put "Here:TabButtonList = {};";
		put "Here:TabCodeList = {};";
		put "Here:TabAfterCodeList = {};";
		put "Here:TabOpenList = {};";	
		*Model Fitting Information Tab for Continuous Traits*;
		%if %upcase(&WhichProc) = MIXED or %upcase(&WhichProc) = LOGISTIC %then %do;
			put "Here:fsmodchart=Function({},{default local},";
			put "	if(isEmpty(Here:fsmoddata),";
			put "		Here:fsmoddata=open(""&ClientOutpath.&QKFE_FS_ALL_OutDataName..sas7bdat"", invisible);";
			put "		,//else";
			put "		CurrentDataTable(Here:fsmoddata);";
			put "	);";
			put "	VListBox(";
			put "	Here:olb= OutLineBox(""Q and K Variables Summary"",";
			put "		TextBox("" ";
			put "Q0 model variables= without Q variables";
			%let i=2;
			%let wQM= %scan(&wQModels, &i);	
			%do %while(%length(&wQM));
				%if %length(&&QVarsModel&wQM) > &wrapvars %then %do;
					%let shortlist= ;
					%do m=1 %to 5; 
						%if %length(&shortlist) < &wrapvars %then
							%let shortlist= &shortlist %scan(&&QVarsModel&wQM, &m);
						%else 
							%let m=6;

					%end;				
					put "Q&wQM model variables= &shortlist %str(... (see Q&wQM Model tab for a complete list))";
				%end;
				%else %do;
					put "Q&wQM model variables= &&QVarsModel&wQM";
				%end;
				%let i= %sysevalf(&i + 1);
				%let wQM= %scan(&wQModels, &i);
			%end;
			put "K0 model variables= without K variables";
			%let j=2;
			%let wKM= %scan(&wKModels, &j);	
			%do %while(%length(&wKM));
				%put lkvar= %length(&&KVarsModel&wKM);
				%if %length(&&KVarsModel&wKM) > &wrapvars %then %do;
					%let shortlist= ;
					%do m=1 %to 5; 
						%if %length(&shortlist) < &wrapvars %then
							%let shortlist= &shortlist %scan(&&KVarsModel&wKM, &m);
						%else 
							%let m=6;

					%end;
					put "K&wKM model variables= &shortlist %str(... (see K&wKM Model tab for a complete list))";
				%end;
				%else %do;
					put "K&wKM model variables= &&KVarsModel&wKM";	
				%end;
				%let j= %sysevalf(&j + 1);
				%let wKM= %scan(&wKModels, &j);
			%end;
			put "			"", ";
			put "			<< Justify Text()";
			put "			,SetWrap(800)),";
			put "		),";
			put "		Here:gb= Graph Builder( ";
			put "			Size( 570, 537),";
			put "			Show Control Panel( 0 ),";
			put "			Variables(";
			%if %upcase(&WhichProc)= LOGISTIC %then %do;
				put "			X( :_Model_),";
				put "			X( :Criterion, Position( 1 )),";
				put "			Y( :Model With Intercept and Covariates),";
				put "			Color( :Criterion ),";
			%end;
			%else %do;
				put "			X( :_Model_),";
				put "			X( :Description, Position( 1 )),";
				put "			Y( :Value ),";
				put "			Color( :Description ),";
			%end;
			put "			Group Y( :_Trait_ ),";
			put "			),";
			put "			Elements(";
			put "				Bar(";
			put "					X( 1 ),";
			put "					X( 2 ),";
			put "					Y,";
			put "					Legend( 20 ),";
			put "					Bar Style( ""Side by side"" ),";
			put "				)";
			put "			),";
			put "			SendToReport(";
			put "				Dispatch( {}, ""_Model_"", ScaleBox, {Rotated Labels( ""Vertical"" )} ),";
			put "				Dispatch( {}, ""Graph Builder"",OutlineBox,{Set Title( ""Model Fitting Information Plot"" )}),";
			put "				Dispatch( {}, ""graph title"", TextEditBox, {Set Text( """" )} ),";
			put "				),";
			put "			Local Data Filter(""on right"",";
			put "				Location( {0, 15} ),";
			put "				Mode( Show( 1 ), Include( 1 ) ),";
			put "				Add Filter(";
			%if %upcase(&WhichProc)= LOGISTIC %then %do;
				%if &NTrait > 1 %then %do;
					put "					columns( :_Model_, :Criterion, :_Trait_ ),";
				%end;
				%else %do;
					put "					columns( :_Model_, :Criterion),";
				%end;
				put "					Display( :_Model_, Size( 204, 174 ), List Display ),";
				put "					Display( :Criterion, Size( 204, 72 ), List Display )";
				
			%end;
			%else %do;
				%if &NTrait > 1 %then %do;
					put "					columns( :_Model_, :Description, :_Trait_ ),";
				%end;
				%else %do;
					put "					columns( :_Model_, :Description),";
				%end;
				put "					Display( :_Model_, Size( 204, 174 ), List Display ),";
				put "					Display( :Description, Size( 204, 72 ), List Display )";
			%end;
			put "				),";
			put "				Animation( Animate Column( :_Model_ ) )";
			put "			),";
			put "		),";
			put "	);";
			put ");";
			put "InsertInto(DataList,{{""Here:fsmoddata""}});";
			put "InsertInto(TabButtonList, ""Fitness Information Plot"");";
			put "InsertInto(TabCodelist,expr(Here:fsmodchart()));";
			put "InsertInto(TabAfterCodeList, "" "");";
			put "InsertInto(TabOpenList,1);";
		%end;
		*Model Fitting Information Tab for Binary and Ordinal Traits with random effects*;
		%if %upcase(&WhichProc)= GLIMMIX %then %do;
			put"	Here:fsmod1fn=Function({},{default local},";
			put"		if(isEmpty(Here:fsmod1),";
			put"			Here:fsmod1=open(""&ClientOutpath.&QKFE_FS_ALLWRE_OutDataName..sas7bdat"", invisible);";
			put"			,//else	";
			put"			CurrentDataTable(Here:fsmod1);";
			put"		); ";
			put"		Here:olbmdc= OutLineBox(""Q and K Variables Summary"",";
			put"			TextBox("" ";
			put "Q0 model variables= without Q variables";
			%let i=2;
			%let wQM= %scan(&wQModels, &i);	
			%do %while(%length(&wQM));
				%if %length(&&QVarsModel&wQM) > &wrapvars %then %do;
					%let shortlist= ;
					%do m=1 %to 5; 
						%if %length(&shortlist) < &wrapvars %then
							%let shortlist= &shortlist %scan(&&QVarsModel&wQM, &m);
						%else 
							%let m=6;

					%end;				
					put "Q&wQM model variables= &shortlist %str(... (see Q&wQM Model tab for a complete list))";
				%end;
				%else %do;
					put "Q&wQM model variables= &&QVarsModel&wQM";
				%end;
				%let i= %sysevalf(&i + 1);
				%let wQM= %scan(&wQModels, &i);
			%end;
			put "K0 model variables= without K variables";
			%let j=2;
			%let wKM= %scan(&wKModels, &j);	
			%do %while(%length(&wKM));
				%put lkvar= %length(&&KVarsModel&wKM);
				%if %length(&&KVarsModel&wKM) > &wrapvars %then %do;
					%let shortlist= ;
					%do m=1 %to 5; 
						%if %length(&shortlist) < &wrapvars %then
							%let shortlist= &shortlist %scan(&&KVarsModel&wKM, &m);
						%else 
							%let m=6;

					%end;
					put "K&wKM model variables= &shortlist %str(... (see K&wKM Model tab for a complete list))";
				%end;
				%else %do;
					put "K&wKM model variables= &&KVarsModel&wKM";	
				%end;
				%let j= %sysevalf(&j + 1);
				%let wKM= %scan(&wKModels, &j);
			%end;
			put"				"", ";
			put"				<< Justify Text()";
			put"				,SetWrap(800)),";
			put"			Here:hlbox= HListBox(";
			put"				o=OutlineBox(""Models with random effects"",";
			put"					Vlistbox(";
			put "						Here:gb= Graph Builder( ";
			put "							Size( 570, 537),";
			put "							Show Control Panel( 0 ),";
			put "							Variables(";
			put "							X( :_Model_),";
			put "							X( :Description, Position( 1 )),";
			put "							Y( :Value ),";
			put "							Color( :Description ),";
			put "							Group Y( :_Trait_ ),";
			put "						),";
			put "						Elements(";
			put "							Bar(";
			put "								X( 1 ),";
			put "								X( 2 ),";
			put "								Y,";
			put "								Legend( 20 ),";
			put "								Bar Style( ""Side by side"" ),";
			put "							)";
			put "						),";
			put "						SendToReport(";
			put "							Dispatch( {}, ""_Model_"", ScaleBox, {Rotated Labels( ""Vertical"" )} ),";
			put "							Dispatch( {}, ""Graph Builder"",OutlineBox,{Set Title( ""Models with random effects plot"" )}),";
			put "							Dispatch( {}, ""graph title"", TextEditBox, {Set Text( """" )} ),";
			put "						),";
			put "						Local Data Filter(""on right"",";
			put "							Location( {0, 15} ),";
			put "							Mode( Show( 1 ), Include( 1 ) ),";
			put "							Add Filter(";
			%if &NTrait > 1 %then %do;
				put "							columns( :_Model_, :Description, :_Trait_ ),";
			%end;
			%else %do;
				put "							columns( :_Model_, :Description),";
			%end;
			put "							Display( :_Model_, Size( 204, 174 ), List Display ),";
			put "							Display( :Description, Size( 204, 72 ), List Display )";
			put "						),";
			put "						Animation( Animate Column( :_Model_ ) )";
			put "						),";
			put "					)";
			put"					),";		
			put"				),";
			put"			),";
			put"		);";
			put"	);";
			put"	InsertInto(TabButtonList, ""Fitness Information Plot""); ";
			put"	InsertInto(TabCodelist,expr(Here:fsmod1fn()));";
			%if %length(&RandEff) = 0 %then %do;
				put"	Here:appendfsmod2fn = Function({},{default local},";
				put"		if(isEmpty(Here:fsmod2),";
				put"			Here:fsmod2=open(""&ClientOutpath.&QKFE_FS_ALLWORE_OutDataName..sas7bdat"", invisible);";
				put"			,//else	";
				put"			CurrentDataTable(Here:fsmod2);";
				put"		);";
				put"		o2=OutlineBox(""Models without random effects"",";
				put"			Vlistbox(";
				put "				Here:gb= Graph Builder( ";
				put "					Size( 570, 537),";
				put "					Show Control Panel( 0 ),";
				put "					Variables(";
				put "					X( :_Model_),";
				put "					X( :Description, Position( 1 )),";
				put "					Y( :Value ),";
				put "					Color( :Description ),";
				put "					Group Y( :_Trait_ ),";
				put "				),";
				put "				Elements(";
				put "					Bar(";
				put "						X( 1 ),";
				put "						X( 2 ),";
				put "						Y,";
				put "						Legend( 20 ),";
				put "						Bar Style( ""Side by side"" ),";
				put "					)";
				put "				),";
				put "				SendToReport(";
				put "					Dispatch( {}, ""_Model_"", ScaleBox, {Rotated Labels( ""Vertical"" )} ),";
				put "					Dispatch( {}, ""Graph Builder"",OutlineBox,{Set Title( ""Models without random effects plot"" )}),";		
				put "					Dispatch( {}, ""graph title"", TextEditBox, {Set Text( """" )} ),";
				put "				),";
				put "				Local Data Filter(""on right"",";
				put "					Location( {0, 15} ),";
				put "					Mode( Show( 1 ), Include( 1 ) ),";
				put "					Add Filter(";
				%if &NTrait > 1 %then %do;
					put "					columns( :_Model_, :Description, :_Trait_ ),";
				%end;
				%else %do;
					put "					columns( :_Model_, :Description),";
				%end;
				put "					Display( :_Model_, Size( 204, 174 ), List Display ),";
				put "					Display( :Description, Size( 204, 72 ), List Display )";
				put "				),";
				put "				Animation( Animate Column( :_Model_ ) )";
				put "				),";
				put "			)";
				put"			),";
				put"		);";	
				put"		Here:hlbox << Append(o2); ";
				put"	); ";
				put"	InsertInto(DataList,{{""Here:fsmod1"", ""Here:fsmod2""}});";
				put"	InsertInto(TabAfterCodeList, expr(Here:appendfsmod2fn()));";
			%end;
			%else %do;
				put "	InsertInto(DataList,{{""Here:fsmod1""}});";
				put "	InsertInto(TabAfterCodeList, "" "");";
			%end;
			put"	InsertInto(TabOpenList,1); ";
		%end;
		put "	Here:TabsSectionFlag = 1;";
		%macro modeltabs(wQ=, wK=, count=);
			put "// Model Q&wQ & K&wK;";
			put "InsertInto(DataList,{{""""}});";
			put "InsertInto(TabButtonList, ""Q&wQ & K&wK Model"");";
			put "InsertInto(TabOpenList, 0, Loc(TabButtonList,""Fitness Information Plot"")+1+&count);";
			put "Here:Q&wQ.K&wK._T1_fn = Function({},{default local},";
			put "	Here:FS&wQ.&wK._T1_data=open(""&ClientOutpath.&&QKFE_FS&wQ.&wK._T1_OutDataName..sas7bdat"", invisible);";
			%if %length(&ParmEst)>0 %then %do;
				put "	Here:FE&wQ.&wK._T1_data=open(""&ClientOutpath.&&QKFE_FE&wQ.&wK._T1_OutDataName..sas7bdat"", invisible);";	
			%end;
			%if (%length(&RandEff)>0 or %length(&&KVarsModel&wK)>0) %then %do;
				%if &&exist_RE&wQ.&wK._T1 = 1 %then %do;
					put "	Here:RE&wQ.&wK._T1_data=open(""&ClientOutpath.&&QKFE_RE&wQ.&wK._T1_OutDataName..sas7bdat"", invisible);";
				%end;
				%if &&exist_CP&wQ.&wK._T1 = 1 %then %do;
					put "	Here:CP&wQ.&wK._T1_data=open(""&ClientOutpath.&&QKFE_CP&wQ.&wK._T1_OutDataName..sas7bdat"", invisible);";
				%end;
			%end;
			put "	Here:vlbQ&wQ.K&wK._T1 = VListBox(";
			put "		Here:ob=OutlineBox(""Trait= &&_tv1"",";
			put "			Here:hlbq = vlistbox( ";
			put "				o=OutlineBox( ""Model Fitting Information Table"", ";
			put "					Vlistbox( ";
			put "						Data Table Box(Here:FS&wQ.&wK._T1_data); ";
			put "					) ";
			put "				), ";
			%if %length(&ParmEst)>0 and &&exist_FE&wQ.&wK._T1 = 1 %then %do;
				put "				o=OutlineBox( ""Fixed Effects Table"", ";
				put "					Vlistbox("; 
				put "						Data Table Box(Here:FE&wQ.&wK._T1_data); ";
				put "					)";
				put "				),";
			%end;
			%if (%length(&RandEff)>0 or %length(&&KVarsModel&wK)>0) %then %do;
				%if &&exist_RE&wQ.&wK._T1 = 1 %then %do;
					put "				o=OutlineBox( ""Random Effects Table"",";
					put "					Vlistbox( ";
					put "						Data Table Box(Here:RE&wQ.&wK._T1_data); ";
					put "					),<<Close(1)";
					put "				),";
				%end;
				%if &&exist_CP&wQ.&wK._T1 = 1 %then %do;
					put "				o=OutlineBox( ""Covariance Parameters Table"",";
					put "					Vlistbox( ";
					put "						Data Table Box(Here:CP&wQ.&wK._T1_data); ";
					put "					)";
					put "				),";
				%end;
			%end;
			put "			);";		
			put "		); ";
			put "	);";
			put ");";
			%if &NTrait > 1 %then %do;
				put "Here:AppendTraitsToQ&wQ.K&wK._Fn = Function({},{default local},	";
				%do t=2 %to &NTrait;
					put "	Here:FS&wQ.&wK._T&t._data=open(""&ClientOutpath.&&QKFE_FS&wQ.&wK._T&t._OutDataName..sas7bdat"", invisible);";
					%if %length(&ParmEst)>0 %then %do;
						put "	Here:FE&wQ.&wK._T&t._data=open(""&ClientOutpath.&&QKFE_FE&wQ.&wK._T&t._OutDataName..sas7bdat"", invisible);";	
					%end;
					%if (%length(&RandEff)>0 or %length(&&KVarsModel&wK)>0) %then %do;
						%if &&exist_RE&wQ.&wK._T&t = 1 %then %do;
							put "	Here:RE&wQ.&wK._T&t._data=open(""&ClientOutpath.&&QKFE_RE&wQ.&wK._T&t._OutDataName..sas7bdat"", invisible);";
						%end;
						%if &&exist_CP&wQ.&wK._T&t = 1 %then %do;
							put "	Here:CP&wQ.&wK._T&t._data=open(""&ClientOutpath.&&QKFE_CP&wQ.&wK._T&t._OutDataName..sas7bdat"", invisible);";	
						%end;
					%end;
					put "	Here:obQ&wQ.K&wK._T&t=OutlineBox(""Trait= &&_tv&t"", ";
					put "		Here:hlbq = vlistbox(";
					put "			o=OutlineBox( ""Model Fitting Information Table"",";
					put "				Vlistbox( ";
					put "					Data Table Box(Here:FS&wQ.&wK._T&t._data); ";
					put "				)";
					put "			),";
					%if %length(&ParmEst)>0 and &&exist_FE&wQ.&wK._T&t = 1 %then %do;
						put "			o=OutlineBox( ""Fixed Effects Table"",";
						put "				Vlistbox( ";
						put "					Data Table Box(Here:FE&wQ.&wK._T&t._data); ";
						put "				)";
						put "			),";
					%end;
					%if (%length(&RandEff)>0 or %length(&&KVarsModel&wK)>0) %then %do;
						%if &&exist_RE&wQ.&wK._T&t = 1 %then %do;
							put "			o=OutlineBox( ""Random Effects Table"",";
							put "				Vlistbox( ";
							put "					Data Table Box(Here:RE&wQ.&wK._T&t._data); ";
							put "				),<<Close(1)";
							put "			),";
						%end;
						%if &&exist_RE&wQ.&wK._T&t = 1 %then %do;
							put "			o=OutlineBox( ""Covariance Parameters Table"",";
							put "				Vlistbox( ";
							put "					Data Table Box(Here:CP&wQ.&wK._T&t._data); ";
							put "				)";
							put "			),";
						%end;
					%end;
					put "		);";
					put "	);";
					put "	Here:vlbQ&wQ.K&wK._T1 << append(obQ&wQ.K&wK._T&t);";
				%end;
				put ");	";
				put "InsertInto(TabAfterCodeList, expr(Here:AppendTraitsToQ&wQ.K&wK._Fn()), Loc(TabButtonList,""Fitness Information Plot"")+1+&count);";				
			%end;
			%else %do;
				put "InsertInto(TabAfterCodeList, "" "", Loc(TabButtonList,""Fitness Information Plot"")+1+&count);";							
			%end;
			put "InsertInto(TabCodeList,expr(Here:Q&wQ.K&wK._T1_fn()), Loc(TabButtonList,""Fitness Information Plot"")+1+&count);";
		%mend modeltabs;
		%let i=1;
		%let k=1;
		%let wQM= %scan(&wQModels, &i);	
		%do %while(%length(&wQM));
			%let j=1;
			%let wKM= %scan(&wKModels, &j);	
			%do %while(%length(&wKM));

				%modeltabs(wQ=&wQM, wK=&wKM, count=&k);	

				%let k= %sysevalf(&k + 1);
				%let j= %sysevalf(&j + 1);
				%let wKM= %scan(&wKModels, &j);
			%end;
			%let i= %sysevalf(&i + 1);
			%let wQM= %scan(&wQModels, &i);
		%end;
		%if %upcase(&server)= LOCAL %then %do;
			put "if(host is(Windows),";
			put "	include( ConvertFilePath(""$LS_HOME/Generator/DialogHelperFunctions.jsl"", absolute, windows) ),";
			put "	include( ConvertFilePath(""$LS_HOME/Generator/DialogHelperFunctions.jsl"", absolute, POSIX) ));";
			put "	";

			put "lobuttonwindow=Function({},{default local},";
			put "		VListBox(";
			put "			TextBox(""Click the button below to launch the Q-K Mixed Model process.""),";
			put "			ButtonBox(""Q-K Mixed Model"",";
			put "				if(host is(Windows),";
			put "					Include(ConvertFilePath(""$HOME/JMPG/ProcessLibrary/QKMixedModel.jsl"", absolute, windows)),";
			put "					Include( ConvertFilePath(""$HOME/JMPG/ProcessLibrary/QKMixedModel.jsl"", absolute, POSIX) ));";
			put "	";
			* Set input data set and Output Folder *;
			put "				jg:setDefaultParmValues(2, ""QKMixedModel"");";				
			put "				jg:QKMixedModel_updateDeps;";
			put "				jg:setCurrentParmValue(2, ""QKMixedModel"", jg:getParmNum(""QKMixedModel"",""InData""),""%sysfunc(pathname(Inlib))\&InData..sas7bdat"");";		
			put "				jg:setCurrentParmValue(2, ""QKMixedModel"", jg:getParmNum(""QKMixedModel"",""OutPath""),""&ClientOutPath"");";										
			put "				jg:QKMixedModel_updateDeps;";				
			put "				jg:setCurrentParmValue(2, ""QKMixedModel"", jg:getParmNum(""QKMixedModel"",""TraitVar""),""&TraitVar"");";	
			/*
			*Q variables*;
			%if %length(&OrigQVarsBestModel) le 32767 %then %do;
				put "				jg:setCurrentParmValue(2, ""QKMixedModel"", jg:getParmNum(""QKMixedModel"",""QVars""),""&OrigQVarsBestModel"");";
			%end;
			put "				jg:setCurrentParmValue(2, ""QKMixedModel"", jg:getParmNum(""QKMixedModel"",""ListQVars""),""&OrigListQVarsBestModel"");";
			put "				jg:setCurrentParmValue(2, ""QKMixedModel"", jg:getParmNum(""QKMixedModel"",""QSumto1""),""&QSumto1BestModel"");";
			put "				jg:QKMixedModel_updateDeps;";
			*K variables*;
			put "				jg:setCurrentParmValue(2, ""QKMixedModel"", jg:getParmNum(""QKMixedModel"",""PedID""),""&PedIDBestModel"");";		
			%if %length(&OrigKVarsBestModel) le 32767 %then %do;
				put "				jg:setCurrentParmValue(2, ""QKMixedModel"", jg:getParmNum(""QKMixedModel"",""KVars""),""&OrigKVarsBestModel"");";
			%end;
			put "				jg:setCurrentParmValue(2, ""QKMixedModel"", jg:getParmNum(""QKMixedModel"",""ListKVars""),""&OrigListKVarsBestModel"");";
			put "				jg:setCurrentParmValue(2, ""QKMixedModel"", jg:getParmNum(""QKMixedModel"",""CompressedK""),""&CompressedKBestModel"");";
			put "				jg:QKMixedModel_updateDeps;";
			*/
			*model variables*;
			put "				jg:setCurrentParmValue(2, ""QKMixedModel"", jg:getParmNum(""QKMixedModel"",""TraitType""),""&TraitType"");";
			put "				jg:setCurrentParmValue(2, ""QKMixedModel"", jg:getParmNum(""QKMixedModel"",""ClassVars""),""&OrigClassVars"");";
			put "				jg:setCurrentParmValue(2, ""QKMixedModel"", jg:getParmNum(""QKMixedModel"",""FixedEff""),""&FixedEff"");";
			put "				jg:setCurrentParmValue(2, ""QKMixedModel"", jg:getParmNum(""QKMixedModel"",""RandEff""),""&OrigRandEff"");";
			put "				jg:setCurrentParmValue(2, ""QKMixedModel"", jg:getParmNum(""QKMixedModel"",""RandOpts""),""&RandOpts"");";
			put "				jg:setCurrentParmValue(2, ""QKMixedModel"", jg:getParmNum(""QKMixedModel"",""ParmsStmt""),""&OrigParmsStmt"");";
			put "				jg:setCurrentParmValue(2, ""QKMixedModel"", jg:getParmNum(""QKMixedModel"",""DDFM""),""&DDFM"");";
			put "				jg:setCurrentParmValue(2, ""QKMixedModel"", jg:getParmNum(""QKMixedModel"",""UseLowerB""),""&UseLowerB"");";
			put "				jg:setCurrentParmValue(2, ""QKMixedModel"", jg:getParmNum(""QKMixedModel"",""GMethod""),""&GMethod"");";
			put "				jg:QKMixedModel_updateDeps;";
			/*
			put "				jg:setCurrentParmValue(2, ""QKMixedModel"", jg:getParmNum(""QKMixedModel"",""MarkerFmt""),""Numeric Genotypes"");";
			put "				jg:QKMixedModel_updateDeps;";
			*/
			put "			)";
			put "		)";
			put ");";
			put "Here:LaunchCode = lobuttonwindow();";
		%end;
	run;

	%PackageInsertFile(jsl code from &ProcessName, &OutPath, &QKFE_ScriptName, "&tabdata", text/plain, yes);
			
	%*goto exitcheck;	

	%exit:;
	%ExitSDSProcess(&OutPath, &ProcessName, &syserr, &exiterror, &clockrunning);
	
	%exitcheck:;
%mend;

*run the macro*;
%QKModelFitness;

